/**
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *  GLite Data Agents Common: SDCache class
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>, Paolo Tedesco <paolo.tedesco@cern.ch>
 *  Version info: $Id: ActiveTxCache.h,v 1.7 2008/11/13 10:43:55 ptedesco Exp $
 *  Release: $Name:  $
 *
 */
#ifndef GLITE_DATA_TRANSFER_AGENTS_ACTIONS_CHANNEL_ACTIVETXCACHE_H_
#define GLITE_DATA_TRANSFER_AGENTS_ACTIONS_CHANNEL_ACTIVETXCACHE_H_

#include "glite/data/transfer/agent/model/Job.h"
#include "glite/data/transfer/agent/model/File.h"
#include "glite/data/transfer/agent/model/Transfer.h"

#include <vector>
#include <string>
#include <boost/scoped_ptr.hpp>
#include <boost/shared_ptr.hpp>
#include <boost/utility.hpp>

namespace glite   {
namespace data    {
namespace transfer{
namespace agent   {
namespace action  {
namespace channel {

/**
 * Forward declaration of SDCache Implementation
 */
class ActiveTxCacheImpl;
    
/**
 * ActiveTxCache
 */
class ActiveTxCache : boost::noncopyable {
public:

    typedef boost::shared_ptr<model::Job>      JobPtr; 
    typedef boost::shared_ptr<model::File>     FilePtr; 
    typedef boost::shared_ptr<model::Transfer> TransferPtr; 

    /**
     * TransferEntry Type
     * A <transfer,file,job> triple
     */
    struct TransferEntry{
        
        /**
         * The embedded object
         */
        TransferPtr tx;
        FilePtr     file;
        JobPtr      job;

        /*
         * Constructor
         */        
        TransferEntry(TransferPtr t, FilePtr f, JobPtr j):
            tx(t),file(f),job(j){}
    };
    
    /**
     * Cache Entry Vector
     */
    typedef std::vector<TransferEntry> TransferEntryArray;
    
    /**
     * RequestEntry Type
     * Collects all the TransferEntry objects (triplets <transfer,file,job>) belonging 
     * to the same request. A request is identified by a request id (in the format 
     * <channel-name>__<date>_<random-string> - e.g. CERN-INFN__2007-10-16-1054_crcCFX) 
     * for urlcopy and srmcopy channels). Multiple files can be associated through a 
     * transfer to the same request id (for example in srmcopy channels).
     */
    struct RequestEntry{
        /**
         * the request id
         */
        std::string id;
        /**
         * related TransferEntry objects
         */
        TransferEntryArray transfers;
        /**
         * Constructor
         */
        RequestEntry(const std::string& i):id(i){};
    };
    
    /**
     * Request Entry Vector
     */
    typedef std::vector<RequestEntry> RequestEntryArray;
    
    /**
     * Constructor
     */
    ActiveTxCache();
        
    /**
     * Destructor
     */
    ~ActiveTxCache();

    /**
     * Set Flag
     */
    bool isSet() const {return m_isSet;}
    
    /**
     * Insert a Transfer into the Cache
     * @param tx   [IN] the transfer
     * @param file [IN] the file
     * @param job  [IN] the job
     */
    void insert(
        TransferPtr tx, 
        FilePtr     file, 
        JobPtr      job) /*throw (LogicError)*/;

    /**
     * Get a transfer cache entry <tx,file,job>
     * @param file_id [IN] the id of the file
     * @return TransferEntry if found, throw RuntimeError otherwise 
     */
    TransferEntry get(
        const std::string& file_id) const /*throw (RuntimeError)*/;
    
    /**
     * Get a Job Ptr (if exists)
     * @param job_id [IN] the id of the job
     * @return JobPtr if found, throw RuntimeError otherwise
     */
    JobPtr getJob(
        const std::string& job_id) const /*throw (RuntimeError)*/;

    /**
     * Get a File Ptr (if exists)
     * @param file_id [IN] the id of the file
     * @return FilePtr if found, throw RuntimeError otherwise 
     */
    FilePtr getFile(
        const std::string& file_id) const /*throw (RuntimeError)*/;

    /**
     * Get the Transfer cache entry that refers to a request
     */
    RequestEntry getRequest(
        const std::string& req_id) const /*throw (RuntimeError)*/;

    /**
     * Get all the Transfer cache entries
     */
    TransferEntryArray getTransferEntries() const /*throw ()*/;

    /**
     * Get the request entries
     */
    RequestEntryArray getRequests() const /*throw ()*/;

    /**
     * Remove a Transfer from the cache
     */
    void remove(const std::string& file_id) /*throw ()*/;

    /**
     * Reset the Cache
     */
    void reset() /*throw ()*/;

    /**
     * Count the number of entries in the cache
     * @return number of entries in the cache
     */
    unsigned int count() const /*throw ()*/;
    
    /**
     * Count the number of entries in the cache for a given VO
     * @param vo_name [IN] the name of the VO
     * @return number of entries in the cache for VO
     */
    unsigned int count(const std::string& vo_name) const /*throw ()*/;

    /**
     * Count the number of requests in the cache
     */
    unsigned int countRequests() const /*throw ()*/;
    /**
     * Count the number of requests in the cache for a given VO
     */
    unsigned int countRequests(const std::string& vo_name) const /*throw ()*/;
    
    /**
     * Count the number of transfers in the cache for the given source SE
     */
    unsigned int countPerSourceSE(const std::string& src_se) const /*throw ()*/;

    /**
     * Count the number of transfers in the cache for the given destination SE
     */
    unsigned int countPerDestSE(const std::string& dst_se) const /*throw ()*/;
    
    /**
     * get all the transfer entries where the transfer is in one of the given 
     * states
     * @param states [IN] combination of Transfer::State values
     * @return array of TransferEntry objects where the transfer is in one of 
     *         the given states
     */
    TransferEntryArray getTransfersByState(
        model::Transfer::State states) const /*throw()*/;
    
    /**
     * get the number of transfer entries where the transfer is in one of the 
     * given states
     * @param states [IN] combination of Transfer::State values
     * @return number of TransferEntry objects where the transfer is in one of 
     *         the given states
     */
    unsigned int countTransfersByState(
        model::Transfer::State states) const /*throw()*/;
    
private:
    /**
     * Pimpl
     */
    boost::scoped_ptr<class ActiveTxCacheImpl> m_impl;
    /**
     * Set Flag
     */
    bool m_isSet;
};

} // end namespace channel
} // end namespace action
} // end namespace agent
} // end namespace transfer
} // end namespace data
} // end namespace glite

#endif //GLITE_DATA_TRANSFER_AGENTS_ACTIONS_CHANNEL_ACTIVETXCACHE_H_
