/*
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *  GLite Data Transfer Agents: VO BaseAction Class
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>, Paolo Tedesco <paolo.tedesco@cern.ch>
 *  Version info: $Id: VOAction.h,v 1.11 2008/11/13 10:43:55 ptedesco Exp $
 *  Release: $Name:  $
 *
 */
#ifndef GLITE_DATA_TRANSFER_AGENTS_ACTION_VO_VOACTION_H_
#define GLITE_DATA_TRANSFER_AGENTS_ACTION_VO_VOACTION_H_

#include "glite/data/transfer/agent/action/Action.h"
#include "glite/data/transfer/agent/action/ExecuteException.h"
#include "glite/data/transfer/agent/dao/vo/JobDAO.h"
#include "glite/data/transfer/agent/dao/vo/FileDAO.h"
#include "glite/data/transfer/agent/dao/vo/TransferDAO.h"
#include "glite/data/transfer/agent/dao/vo/StageReqDAO.h"
#include "glite/data/transfer/agent/dao/vo/ChannelDAO.h"
#include "glite/data/agents/dao/DAOContext.h"

#include <glite/data/config/service/Logger.hxx>
#include <string>
#include <boost/scoped_ptr.hpp>
#include <set>

namespace glite   {
namespace data    {
namespace transfer{
namespace agent   {
namespace action  {
namespace vo      {
        
/**
 * Base Class For all the VO Actions
 */
class VOAction : public Action {
public:    
    
    /**    
     * set DAOContext
     */
    virtual void context(const std::string& vo_name, agents::dao::DAOContext& ctx);
    
    /**    
     * Get DAOContext
     */
    virtual agents::dao::DAOContext& context() /*throw (LogicError)*/;
    
    /**
     * Destructor
     */
    virtual ~VOAction();

    /**
     * Execute The Action
     */
    virtual void execute(const std::string& params = "") /*throw (LogicError,ExecuteException)*/;

protected:

    /**
     * Constructor
     */
    VOAction(const std::string& name);

    /**
     * Call the virtual method to execute the action (Template Pattern)
     */
    virtual void exec(const std::string& params) /*throw (LogicError, DAOException, ExecuteException)*/ = 0;

    /**
     * Update Jobs' State
     */
    void updateJobs(const std::set<std::string>& ids) /*throw (DAOLogicError,DAOException)*/;

    /**
     * Update Jobs' State
     */
    void updateJobs(const agents::dao::StringArray& ids) /*throw (DAOLogicError, DAOException)*/;

    /**
     * Send a Fail Event for a file
     */
    void failFile(
        const std::string&              file, 
        const model::Error::Category&   category, 
        const model::Error::Scope&      scope, 
        const model::Error::Phase&      phase, 
        const std::string&              reason) /*throw ()*/;

    /**
     * Send a Fail Event for a file
     */
    void failFile(
        model::File&                    file, 
        const model::Error::Category&   category, 
        const model::Error::Scope&      scope, 
        const model::Error::Phase&      phase, 
        const std::string&              reason) /*throw ()*/;

    /**
     * Send a Fail Event for all the files
     */
    void failAllFiles(
        const std::vector<std::string>& files, 
        const model::Error::Category&   category, 
        const model::Error::Scope&      scope, 
        const model::Error::Phase&      phase, 
        const std::string&              reason) /*throw ()*/;

    /**
     * Get The Proxy Certficate for a Job
     */
    void getProxyForJob(model::Job& job, std::string& proxy_file) /* throw(AgentException)*/;

    /**
     * Normalize a Surl accordingly with the VO policy
     */
    std::string normalizeSurl(const std::string& surl, const std::string& srm_endpoint) /* throw()*/;
    
    /**
     * Get the endpoint of the Catalog on the same site of the given se
     */
    std::pair<std::string,std::string> getCatalogEndpoint(const std::string& source, const std::string& vo_name) /* throw(LogicError)*/;

protected:
    
    typedef dao::vo::JobDAO       JobDAO;
    typedef dao::vo::FileDAO      FileDAO;
    typedef dao::vo::TransferDAO  TransferDAO;
    typedef dao::vo::StageReqDAO  StageReqDAO;
    typedef dao::vo::ChannelDAO   ChannelDAO;

    /**    
     * Get a JobDAO Instance
     */
    JobDAO& jobDAO() /*throw (LogicError)*/;
    
    /**    
     * Get a FileDAO Instance
     */
    FileDAO& fileDAO() /*throw (LogicError)*/;

    /**    
     * Get a TransferDAO Instance
     */
    TransferDAO& transferDAO() /*throw (LogicError)*/;

    /**    
     * Get a StageReqDAO Instance
     */
    StageReqDAO& stageReqDAO() /*throw (LogicError)*/;

    /**    
     * Get a ChannelDAO Instance
     */
    ChannelDAO& channelDAO() /*throw (LogicError)*/;

    /**
     * Get The VO Name
     */
    const std::string& voName() { return m_voName;}
    
    /**
     * The Logger
     */
    glite::config::Logger m_logger;

private:
    
    /**
     * The DAO Context
     */
    agents::dao::DAOContext* m_context;

    /**
     * The Vo Name
     */
    std::string m_voName;

    /**
     * The Job DAO
     */
    boost::scoped_ptr<JobDAO> m_jobDAO;

    /**
     * The File DAO
     */
    boost::scoped_ptr<FileDAO> m_fileDAO;
    
    /**
     * The Transfer DAO
     */
    boost::scoped_ptr<TransferDAO> m_transferDAO;
    
    /**
     * The StageReq DAO
     */
    boost::scoped_ptr<StageReqDAO> m_stageReqDAO;
    
    /**
     * The Channel DAO
     */
    boost::scoped_ptr<ChannelDAO> m_channelDAO;
};

} // End namespace vo
} // End namespace action
} // End namespace agent
} // End namespace transfer
} // End namespace data
} // End namespace glite

#endif //GLITE_DATA_TRANSFER_AGENTS_ACTION_VO_BASEACTION_H_
