/**
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *  GLite Data Transfer Agents: Job Class
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *           Paolo Tedesco <paolo.tedesco@cern.ch>
 *  Version info: $Id: Job.h,v 1.18 2009/07/02 15:21:17 molnarzs Exp $
 *  Release: $Name:  $
 *
 */
#ifndef GLITE_DATA_TRANSFER_AGENTS_MODEL_JOB_H_
#define GLITE_DATA_TRANSFER_AGENTS_MODEL_JOB_H_

#include <string>
#include "glite/data/transfer/agent/model/Property.h"

namespace glite   {
namespace data    {
namespace transfer{
namespace agent   {
namespace model   {

#warning "TODO: make a 'model' library and move these to constants of the class"
#define JOB_DEFAULTPRIORITY 3
#define JOB_DEFAULTCOPYPINLIFETIME -1
#define JOB_DEFAULTLANCONNECTION false
#define JOB_DEFAULTFAILNEARLINE false


/**
 * Job Class
 * @stereotype TransferObject
 */
struct Job {

    typedef enum{ // Define values so they could combined in OR
        S_UNDEF            = 0x0000,
        S_SUBMITTED        = 0x0001,
        S_PENDING          = S_SUBMITTED        << 1,  //0x0002
        S_READY            = S_PENDING          << 1,  //0x0004
        S_ACTIVE           = S_READY            << 1,  //0x0008
        S_DONE             = S_ACTIVE           << 1,  //0x0010
        S_DONE_WITH_ERRORS = S_DONE             << 1,  //0x0020
        S_HOLD             = S_DONE_WITH_ERRORS << 1,  //0x0040
        S_CANCELED         = S_HOLD             << 1,  //0x0080
        S_FAILED           = S_CANCELED         << 1,  //0x0100
        S_FINISHING        = S_FAILED           << 1,  //0x0200
        S_FINISHED         = S_FINISHING        << 1,  //0x0400
        S_FINISHED_DIRTY   = S_FINISHED         << 1   //0x0800
    } State;

    /**
     * Default Constructor
     */
    Job():priority(JOB_DEFAULTPRIORITY),submitTime((time_t)-1),
          maxTimeInQueue(0),cancelJob(false),finishTime((time_t)-1),
          overwriteFlag(false),
          copyPinLifetime(JOB_DEFAULTCOPYPINLIFETIME),
          lanConnection(JOB_DEFAULTLANCONNECTION),
          failNearline(JOB_DEFAULTFAILNEARLINE),
          checksum_method(false)
          {}

    /**
     * Constructor
     */
    Job(const std::string& i,               // job_id
              State        s,               // state
        const std::string& vname,           // vo_name
        const std::string& ch       = "",   // channel_name
        unsigned int pr = JOB_DEFAULTPRIORITY,// priority
              bool         cj       = false,// cancel_job
        const std::string& src      = "",   // source
        const std::string& dst      = "",   // dest
        const std::string& u_dn     = "",   // user_dn
        const std::string& p        = "",   // job_params
        const std::string& r        = "",   // reason
              time_t       t = (time_t)-1,  // submit_time
        const std::string& a_dn     = "",   // agent_dn
        const std::string& sh       = "",   // submit_host
              unsigned int mtiq     = 0,    // max_time_in_queue
              time_t       ft = (time_t)-1, // finish_time
        const std::string& mp_srv   = "",   // myproxy_server
        const std::string& sp_tkn   = "",   // space_token
        const std::string& st_class = "",   // storage_class
        const std::string& src_c    = "",   // src_catalog
        const std::string& src_c_t  = "",   // src_catalog_type
        const std::string& dst_c    = "",   // dest_catalog
        const std::string& dst_c_t  = "",   // dest_catalog_type
        const std::string& u_crd    = "",   // user_cred
        const std::string& c_id     = "",   // cred_id
        const std::string& v_crd    = "",   // voms_cred
        const std::string& i_p      = "",   // internal_params
        const std::string& s_se     = "",   // source_se
        const std::string& d_se     = "",   // dest_se
        bool         o_f      = false,// overwrite_flag
              // new in 3.2.0
        const std::string& sst      = "", // source space token
        const std::string& sstd     = "", // source space token description
        // new in 3.3.0
        int cpl = JOB_DEFAULTCOPYPINLIFETIME, // copyPinLifetime
        bool lc = JOB_DEFAULTLANCONNECTION, // lanConnection
        bool fn = JOB_DEFAULTFAILNEARLINE, // failNearline
        bool cm = false // checksum_method
       ):
         id(i),state(s),voName(vname),channelName(ch),priority(pr),
         source(src),dest(dst),userDn(u_dn),credId(c_id),userCred(u_crd),
         vomsCred(v_crd),params(p),reason(r),submitTime(t),
         submitHost(sh),maxTimeInQueue(mtiq),cancelJob(cj),finishTime(ft),
         myproxyServer(mp_srv),spaceToken(sp_tkn),storageClass(st_class),
         srcCatalog(src_c),srcCatalogType(src_c_t),
         destCatalog(dst_c),destCatalogType(dst_c_t),
         agentDn(a_dn),internalParams(i_p),
         sourceSE(s_se),destSE(d_se),overwriteFlag(o_f),
         sourceSpaceToken(sst),sourceTokenDescription(sstd),
         copyPinLifetime(cpl),lanConnection(lc),failNearline(fn),
         checksum_method(cm)
         {}

    /**
     * Id Property
     */
    Property<std::string> const id;

    /**
     * State Property
     */
    Property<State> state;

    /**
     * VO Name Property
     */
    Property<std::string> const voName;

    /**
     * Channel Property
     */
    Property<std::string> channelName;

    /**
     * Priority Property
     */
    Property<unsigned int> const priority;

    /**
     * Source Property
     */
    Property<std::string> source;

    /**
     * Dest Property
     */
    Property<std::string> dest;

    /**
     * User Dn Property
     */
    Property<std::string> const userDn;

    /**
     * CredId Property
     */
    Property<std::string> const credId;

    /**
     * User Cred Property
     */
    Property<std::string> const userCred;

    /**
     * VOMS Cred Property
     */
    Property<std::string> const vomsCred;

    /**
     * Job Params Property
     */
    Property<std::string> const params;

    /**
     * Reason Property
     */
    Property<std::string> reason;

    /**
     * Submit Time Property
     */
    Property<time_t> const submitTime;

    /**
     * Submit Host Property
     */
    Property<std::string> const submitHost;

    /**
     * MaxTimeInQueue Property
     */
    Property<unsigned int> const maxTimeInQueue;

    /**
     * CancelJob Flag
     */
    Property<bool> cancelJob;

    /**
     * Finish Time Property
     */
    Property<time_t> finishTime;

    /**
     * MyProxy Server Property
     */
    Property<std::string> myproxyServer;

    /**
     * Space Token Property
     */
    Property<std::string> const spaceToken;

    /**
     * Storage Class Property
     */
    Property<std::string> const storageClass;

    /**
     * Source Catalog Endpoint Property
     */
    Property<std::string> srcCatalog;

    /**
     * Source Catalog Type Property
     */
    Property<std::string> srcCatalogType;

    /**
     * Dest Catalog Endpoint Property
     */
    Property<std::string> destCatalog;

    /**
     * Dest Catalog Type Property
     */
    Property<std::string> destCatalogType;

    /**
     * Agent DN Property
     */
    Property<std::string> agentDn;

    /**
     * Job Internal Params Property
     */
    Property<std::string> internalParams;

    /**
     * Source SE Property
     */
    Property<std::string> sourceSE;

    /**
     * Dest SE Property
     */
    Property<std::string> destSE;

    /**
     * Overwrite Flag Property
     */
    Property<bool> const overwriteFlag;

    //--------------------------------------------------------------------------
    // added with schema version 3.2.0
    //--------------------------------------------------------------------------

    /**
     * Source space token property
     */
    Property<std::string> const sourceSpaceToken;

    /**
     * Source space token description property
     */
    Property<std::string> const sourceTokenDescription;

    //--------------------------------------------------------------------------
    // added with schema version 3.3.0
    //--------------------------------------------------------------------------
    /**
     * pin lifetime of the copy of the file
     */
    Property<int> const copyPinLifetime;

    /**
     * use "LAN" as ConnectionType (FTS by default uses WAN). Default value is false.
     */
    Property<bool> const lanConnection;

    /**
     * fail the transfer immediately if the file location is NEARLINE. Default value is false.
     */
    Property<bool> const failNearline;

    /**
     * The checksum comparison method: false: no checksum comparison, true:
     * force checksum comparison.
     */
    Property<bool> const checksum_method;

public:
    /**
     * Reset All R/W Properties
     */
    void reset(){
        state.reset();
        channelName.reset();
        source.reset();
        dest.reset();
        reason.reset();
        cancelJob.reset();
        finishTime.reset();
        myproxyServer.reset();
        srcCatalog.reset();
        srcCatalogType.reset();
        destCatalog.reset();
        destCatalogType.reset();
        agentDn.reset();
        internalParams.reset();
        sourceSE.reset();
        destSE.reset();
    }
};

} // End namespace model
} // End namespace agent
} // End namespace transfer
} // End namespace data
} // End namespace glite

#endif //GLITE_DATA_TRANSFER_AGENTS_MODEL_JOB_H_
