/*
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *  GLite Data Transfer Agents: ActionFactory Class
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *  Version info: $Id: ActionFactory.cpp,v 1.2 2006/08/14 08:38:09 badino Exp $
 *  Release: $Name:  $
 *
 */
#include "glite/data/transfer/agent/action/ActionFactory.h"
#include "glite/data/transfer/agent/action/ActionFactoryMethod.h"
#include "glite/data/agents/AgentExceptions.h"

using namespace glite::data::transfer::agent::action;
using glite::data::agents::LogicError;

/*
 * instance
 *
 * Return singleton instance
 */
ActionFactory& ActionFactory::instance(){
    static ActionFactory singleton;
    return singleton;
}

/*
 * ActionFactory
 *
 * Constructor
 */
ActionFactory::ActionFactory(){
}

/*
 * ~ActionFactory
 *
 * Destructor
 */
ActionFactory::~ActionFactory(){
    FactoryMap::iterator it;
    for(it = m_actionFactoryMap.begin(); it != m_actionFactoryMap.end(); ++it){
        // Erase FactoryMethod
        if(0 != (*it).second){
            delete (*it).second;  
            (*it).second = 0;
        }            
    }
    m_actionFactoryMap.clear();
}

/*
 * create
 *
 * Create an instance of an action of the given type
 */
Action * ActionFactory::create(const std::string& type) /*throw (LogicError)*/ {
    FactoryMap::const_iterator it = m_actionFactoryMap.find(type);
    if(it == m_actionFactoryMap.end()){
        throw LogicError("unknown action type");
    }
    return (*it).second->create();
}

/*
 * get
 *
 * Get the action factory method for a gven type
 */
ActionFactoryMethod& ActionFactory::get(const std::string& type) /*throw (LogicError)*/ {
    FactoryMap::const_iterator it = m_actionFactoryMap.find(type);
    if(it == m_actionFactoryMap.end()){
        throw LogicError("unknown action type");
    }
    return *((*it).second);
}

/*
 * registerMethod
 *
 * Register a Factory Method and Acquire its ownership
 */
void ActionFactory::registerMethod(const std::string& type, ActionFactoryMethod * factory) /*throw (LogicError)*/ {
    if(0 == factory){
        throw LogicError("null factory pointer");
    }
    std::pair<FactoryMap::iterator, bool> result = 
        m_actionFactoryMap.insert(FactoryMap::value_type(type,factory));
    if(false == result.second){
        throw LogicError("requested action type already registered");
    }
}
