/**
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *  GLite Data Transfer Agents: BaseActionsConfig Class
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *           Paolo Tedesco <paolo.tedesco@cern.ch>
 *  Version info: $Id: BaseActionsConfig.cpp,v 1.18 2009/07/28 16:10:24 riojac3 Exp $
 *  Release: $Name:  $
 *
 */
#include "BaseActionsConfig.h"
#include <glite/data/agents/AgentExceptions.h>
#include <glite/data/agents/surl-utility.h>
#include <glite/data/agents/url-utility.h>
#include <glite/data/agents/dao/DAOContext.h>
#include <glite/data/agents/cred/cred-utility.h>
#include <glite/data/agents/cred/CredServiceException.h>
#include <glite/data/agents/sd/SDConfig.h>
#include <glite/data/agents/sd/sd-utility.h>
#include <glite/data/transfer/agent/catalog/CatalogServiceFactory.h>
#include <glite/data/transfer/agent/model/Job.h>

#include <glite/data/config/service/ParamReader.hxx>
#include <boost/scoped_ptr.hpp>

using namespace glite::data::transfer::agent;
using namespace glite::data::transfer::agent::action;
using namespace glite::data::agents::sd;
using namespace glite::config;
using glite::data::agents::ConfigParamException;
using glite::data::agents::ConfigurationException;
using glite::data::agents::LogicError;
using glite::data::agents::RuntimeError;
using glite::data::agents::AgentException;
using glite::data::agents::parse_url;
using glite::data::agents::dao::DAOContext;

namespace {    
const char * const SURLNORMALIZATION_PROPERTY_NAME = "SurlNormalization";
const char * const DELEGSERVICETYPE_PROPERTY_NAME  = "DelegServiceType";
const char * const CREDSERVICETYPE_PROPERTY_NAME   = "CredServiceType";
const char * const SRMVERSION_PROPERTY_NAME        = "SrmVersion";
const char * const SRMVERSIONPOLICY_PROPERTY_NAME  = "SrmVersionPolicy";

const char * const DEFAULT_DELEGSERVICETYPE  = "deleg";   
const char * const DEFAULT_CREDSERVICETYPE   = "myproxy";

const char * const SRM_PROTOCOL              = "srm";
const char * const SRMVERSION_1_1            = "1.1";
const char * const SRMVERSION_2_2            = "2.2";
const char * const DEFAULT_SRMVERSION        = SRMVERSION_2_2;

// Undocumented features used for the some unit tests
const char * const ENABLEDELEGATION_PROPERTY_NAME  = "EnableDelegation";
const bool   DEFAULT_ENABLEDELEGATION        = true;

const char * const SURLNORMALIZATION_DISABLED          = "disabled";
const char * const SURLNORMALIZATION_COMPACT           = "compact";
const char * const SURLNORMALIZATION_COMPACT_WITH_PORT = "compact-with-port";
const char * const SURLNORMALIZATION_FULLY_QUALIFIED   = "fully-qualified";

const char * const SRMVERSIONPOLICY_DEFAULT            = "default";
const char * const SRMVERSIONPOLICY_FORCE              = "force";
const char * const SRMVERSIONPOLICY_WITH_SPACE_TOKEN   = "with-space-token";

const BaseActionsConfig::SurlNormalization DEFAULT_SURLNORMALIZATION = BaseActionsConfig::N_COMPACT;
const BaseActionsConfig::SrmVersionPolicy  DEFAULT_SRMVERSIONPOLICY  = BaseActionsConfig::P_DEFAULT;
}

/*
 * ActionsConfig
 *
 * Constructor
 */
BaseActionsConfig::BaseActionsConfig(const char * name):ComponentConfiguration(name),
    m_logger(name),
    m_enableDelegation(DEFAULT_ENABLEDELEGATION),
    m_surlNormalization(DEFAULT_SURLNORMALIZATION),
    m_delegServiceType(DEFAULT_DELEGSERVICETYPE),
    m_credServiceType(DEFAULT_CREDSERVICETYPE),
    m_srmVersion(DEFAULT_SRMVERSION),
    m_srmVersionPolicy(DEFAULT_SRMVERSIONPOLICY){
}
                                                                                                                          
/*
 * ~BaseActionsConfig
 *
 * Destructor
 */
BaseActionsConfig::~BaseActionsConfig(){
}

namespace glite{
namespace config{

/**
 * SurlNormalization string conversion
 */
template<>
struct StringConverter<BaseActionsConfig::SurlNormalization> : public EnumConverter<BaseActionsConfig::SurlNormalization>{
    StringConverter() : EnumConverter<BaseActionsConfig::SurlNormalization>(BaseActionsConfig::N_DISABLED,SURLNORMALIZATION_DISABLED){
        add(BaseActionsConfig::N_COMPACT,           SURLNORMALIZATION_COMPACT);
        add(BaseActionsConfig::N_COMPACT_WITH_PORT, SURLNORMALIZATION_COMPACT_WITH_PORT);
        add(BaseActionsConfig::N_FULLY_QUALIFIED,   SURLNORMALIZATION_FULLY_QUALIFIED);
        add(BaseActionsConfig::N_DISABLED,          SURLNORMALIZATION_DISABLED);
    }
protected:    
    virtual std::string get_default_string(){
        throw LogicError("Invalid SurlNormalization Value");
    }
};

/**
 * SrmVersionPolicy string conversion
 */
template<>
struct StringConverter<BaseActionsConfig::SrmVersionPolicy> : public EnumConverter<BaseActionsConfig::SrmVersionPolicy>{
    StringConverter() : EnumConverter<BaseActionsConfig::SrmVersionPolicy>(BaseActionsConfig::P_DEFAULT,SRMVERSIONPOLICY_DEFAULT){
        add(BaseActionsConfig::P_FORCE,             SRMVERSIONPOLICY_FORCE);
        add(BaseActionsConfig::P_WITH_SPACE_TOKEN,  SRMVERSIONPOLICY_WITH_SPACE_TOKEN);
        add(BaseActionsConfig::P_DEFAULT,           SRMVERSIONPOLICY_DEFAULT);
    }
protected:    
    virtual std::string get_default_string(){
        throw LogicError("Invalid SrmVersionPolicy Value");
    }
};

} // config
} // glite

/**
 * init
 * Initialize the Component
 */
int BaseActionsConfig::init(const Params& params){
    
    try{
        ParamReader reader(getName(),params);
    
        // Get Enable Delegation Flag
        reader.getParameter(ENABLEDELEGATION_PROPERTY_NAME,m_enableDelegation);
        
        // Get Surl Normalization 
        reader.getParameter(SURLNORMALIZATION_PROPERTY_NAME,m_surlNormalization);
    
        // Get DelegService Type
        reader.getParameter(DELEGSERVICETYPE_PROPERTY_NAME,m_delegServiceType);
        
        // Get CredService Type
        reader.getParameter(CREDSERVICETYPE_PROPERTY_NAME,m_credServiceType);
        
        // Get SrmVersion Type
        reader.getParameter(SRMVERSION_PROPERTY_NAME,m_srmVersion);
        
        // Get SrmVersion Policy
        reader.getParameter(SRMVERSIONPOLICY_PROPERTY_NAME,m_srmVersionPolicy);
        
        // Log Configuration Values
        m_log_info(getName() << " Initialized. Initailization Parameters are");
        m_log_info("Credential Delegation : " << str_enabled(m_enableDelegation));
        m_log_info("SURL Normalization    : " << StringConverter<SurlNormalization>::to_string(m_surlNormalization));
        m_log_info("Deleg Service Type    : " << m_delegServiceType);
        m_log_info("Cred  Service Type    : " << m_credServiceType);
        m_log_info("Default SRM Version   : " << m_srmVersion);
        m_log_info("SRM Version Policy    : " << StringConverter<SrmVersionPolicy>::to_string(m_srmVersionPolicy));
        
    } catch(const LogicError& exc) {
        m_log_error(getName() << " configuration Logic Error: " << exc.what());
        return -1;
    } catch(const std::exception& exc) {
        m_log_error(getName() << " configuration Unexpected Exception: " << exc.what());
        return -1;
    } catch(...) {
        m_log_error(getName() << " configuration Unhandled Exception");
        return -1;
    }
    return 0;  
}

/*
 * normalizeSurl
 *
 * Normalize a Surl accordingly with the VO policy
 */
std::string BaseActionsConfig::normalizeSurl(const std::string& surl, const std::string& srm_endpoint) /* throw()*/{
    
    std::string normalized = surl;
    try {
        switch(m_surlNormalization){
            case N_COMPACT:
                if(false == srm_endpoint.empty()){
                    normalized = glite::data::agents::compact_surl(surl,srm_endpoint,false);
                }
            break;
            case N_COMPACT_WITH_PORT:
                if(false == srm_endpoint.empty()){
                    normalized = glite::data::agents::compact_surl(surl,srm_endpoint,true);
                }
            break;
            case N_FULLY_QUALIFIED:
                if(false == srm_endpoint.empty()){
                    normalized = glite::data::agents::fully_qualify_surl(surl,srm_endpoint);
                }
            break;
            default:
                // Don't do anything
            break;
        } // End switch
    } catch(const std::exception& exc){
        m_log_warn("Failed to normalize SURL <" << surl << ">. Reason is: " << exc.what());
        normalized = surl;
    }
    m_log_debug("SURL <" << surl << "> normalized to <" << normalized << ">");
    return normalized;
}

/**
 * getProxyForJob
 * Get Proxy for a job
 */
void BaseActionsConfig::getProxyForJob(
    model::Job&                 job, 
    agents::dao::DAOContext&    ctx, 
    std::string&                proxy_file) /* throw (AgentException)*/ {

    try{
        std::string cred_service_endpoint;
        // Check if the cred_id property is present
        if(false == job.credId().empty()){
            // Get ProxyCert using the delegation plugin
            m_log_trace("getting proxy certificate (delegation plugin)");
            proxy_file = agents::cred::get_proxy_cert(
                job.userDn(),                                   // user_dn
                job.credId(),                                   // user_cred
                job.voName(),                                   // vo_name
                cred_service_endpoint,                          // cred_service_endpoint
                "",                                             // assoc_service
                "",                                             // assoc_service_type
                (true == this->delegationEnabled())?false:true, // disable_delegation
                m_delegServiceType);                            // cred_service_type
        } else {
            // Get ProxyCert using the myproxy plugin
            m_log_trace("getting proxy certificate (myproxy plugin)");
            cred_service_endpoint = job.myproxyServer();
            proxy_file = agents::cred::get_proxy_cert(
                job.userDn(),                                       // user_dn
                job.userCred(),                                     // user_cred
                job.voName(),                                       // vo_name
                cred_service_endpoint,                              // cred_service_endpoint
                job.submitHost(),                                   // assoc_service
                agents::sd::SDConfig::instance().fileTransfer(),    // assoc_service_type
                (true == this->delegationEnabled())?false:true,     // disable_delegation
                m_credServiceType);                                 // cred_service_type
            
            if(job.myproxyServer().empty()){
                // Save the credservice endpoint
                job.myproxyServer = cred_service_endpoint;
            }
        }
    } catch(const agents::AgentException& exc){
        m_log_warn("Failed to get Proxy Certificate for job " << job.id() << ". Reason is " << exc.what());
        // ORA-03113 WORKAROUND
        // Recheck the DAO Context, since it seems the Myproxy 
        // interation causes the Oracle connection to be dropped
        try{                
            ctx.check();
        } catch(...){}
        throw;
    }
}

/**
 * getSrmEndpoint
 * Get the SRM endpoint for a SURL
 * @return pair <endpoint,version>
 */
std::pair<std::string,std::string> BaseActionsConfig::getSrmEndpoint(
    const std::string&  surl, 
    std::string&        host, 
    const std::string&  spc_tkn, 
    const std::string&  vo_name) {
    
    std::pair<std::string,std::string> srm_info;
    
    std::string protocol,path,query_str;
    unsigned int port = 0;
        
    // Get Source and destination host from the SURLs
    try{
        agents::parse_url(surl, host, port, protocol, path, query_str);
        if(true == query_str.empty()){
            // no query string => the path is the SFN
            path.clear();
        }
    } catch(const LogicError& exc){
        m_log_warn("Invalid SURL [" << surl << "]: " << exc.what());
        return srm_info;
    }
    
// #warning "this should probably be modified to apply only if there is ambiguity!!!"
    
    // Choose the selector based on the SRM Version Policy
    boost::scoped_ptr<SelectPred> pred;
    switch(m_srmVersionPolicy){
        case P_FORCE:
            pred.reset(new SelectByVersion(m_srmVersion));
        break;
        case P_WITH_SPACE_TOKEN:
            if(false == spc_tkn.empty()){
                pred.reset(new SelectByVersion(SRMVERSION_2_2));
                break;                    
            } else {
                // fallback to the default case
            }
        case P_DEFAULT:
        default:
            pred.reset(new SelectByPortAndVersion(port,path,m_srmVersion));
        break;
    }
        
    // Get SRM Endpoints
    if(protocol == SRM_PROTOCOL){
        boost::scoped_ptr<Service> se(
            get_service(
                host,
                agents::sd::SDConfig::instance().srm(), 
                vo_name,
                *pred));
        
        if(0 != se.get()){
            srm_info.first   = se->endpoint;
            srm_info.second  = se->version;
        }
        if(srm_info.first.empty()){
            #warning "probably should not start the transfer, here"
            m_log_warn("Failed to find Source SRM endpoint for Surl [" << surl << "]");
        } else {
            if(srm_info.second.empty()){
                // Assign default SRM version
                m_log_debug("No SRM version returned. Assign the default one");
                srm_info.second = m_srmVersion;
            }
            m_log_debug("SRM for [" << surl << "] is [" << srm_info.first << "]. Version is [" << srm_info.second << "]");
        }
    } else {
        // Not a SURL: don't need the srm endpoint
        m_log_debug("File [" << surl << "] is not a SURL");
    }
    return srm_info;
}
