/*
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *  GLite Data Transfer Agents: ActionsConfig Class
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *  Version info: $Id: BaseActionsConfig.h,v 1.7 2008/11/13 10:43:55 ptedesco Exp $
 *  Release: $Name:  $
 *
 */
#ifndef GLITE_DATA_TRANSFER_AGENTS_ACTION_BASEACTIONSCONFIG_H_
#define GLITE_DATA_TRANSFER_AGENTS_ACTION_BASEACTIONSCONFIG_H_

#include "glite/data/config/service/ComponentConfiguration.hxx"
#include <string>
#include <boost/utility.hpp>

#include <glite/data/config/service/Logger.hxx>

namespace glite   {
namespace data    {
    
/*
 * Forward Declaration
 */
namespace agents   {
namespace dao   {
class DAOContext;
}
}
    
namespace transfer{
namespace agent   {
    
/*
 * Forward Declaration
 */
namespace model{
class Job;
}    

namespace action  {

/**
 * The base class for ActionsConfig (used to retrieve the configuration 
 * parameters from the ServiceConfigurator)
 */    
class BaseActionsConfig : boost::noncopyable, 
    public glite::config::ComponentConfiguration{
public:
    /**
     * Configure the Component
     * @param params [IN] the initialization parameters
     * @return 0 if ok
     */
    virtual int init(const Params& params);
    
    /**
     * Configure the Component
     */
    virtual int config(const Params& params) {return 0;}
                                                                                                                           
    /**
     * Start the Component
     */
    virtual int start(){return 0;}
                                                                                                                           
    /**
     * Stop the Component
     */
    virtual int stop(){return 0;}
                                                                                                                           
    /**
     * Finailize the Component
     */
    virtual int fini(){return 0;}

    // =========================================================================
    // Configuration Properties
    // =========================================================================
               
    /**
     * Returns true is Client Credentials Delegation is enabled
     * @return true is Client Credentials Delegation is enabled
     */
    bool delegationEnabled() const { return m_enableDelegation;}

    // =========================================================================
    // Helper Function
    // =========================================================================
    
    typedef enum {
        N_DISABLED = 0,
        N_COMPACT,
        N_COMPACT_WITH_PORT,
        N_FULLY_QUALIFIED
    } SurlNormalization;

    typedef enum {
        P_DEFAULT = 0,      // if the user provided a fully qualified SURL, use the endpoint from it, otherwise use the default
        P_WITH_SPACE_TOKEN, // use 2.2 if a space token is provided, otherwise use the default policy
        P_FORCE,            // force to use the version specified as default (if exists)
    } SrmVersionPolicy;
    
    /**
     * The Delegation CredService plugin type
     */
    const std::string& delegServiceType() const { return m_delegServiceType;}

    /**
     * The CredService plugin type
     */
    const std::string& credServiceType() const { return m_credServiceType;}
    
    /**
     * The Default SRM Version to use
     */
    const std::string& srmVersion() const { return m_srmVersion;}

    /**
     * Normalize a Surl accordingly with the VO policy
     * @param surl         [IN]
     * @param srm_endpoint [IN]
     * @return the surl normalized according to VO policy
     */
    std::string normalizeSurl(
        const std::string& surl, 
        const std::string& srm_endpoint) /* throw()*/;
    
    /**
     * Get the Proxy certificate for a Job
     * @param job        [IN]
     * @param ctx        [IN]
     * @param proxy_file [OUT]
     */
    void getProxyForJob(
        class model::Job&               job, 
        class agents::dao::DAOContext&  ctx, 
        std::string&                    proxy_file) /* throw (AgentException)*/;
    
    /**
     * Get the SRM endpoint for a SURL
     * @param surl      [IN] the surl
     * @param host      [IN/OUT] the host (gets converted to lowercase).
     * @param spc_tkn   [IN] the target space token, used for version selection 
     * in case of ambiguity
     * @param vo_name   [IN] the name of the VO
     * @return pair <endpoint,version>
     */
    std::pair<std::string,std::string> getSrmEndpoint(
        const std::string&  surl, 
        std::string&        host, 
        const std::string&  spc_tkn, 
        const std::string&  vo_name);

protected:
    /**
     * Constructor
     */
    explicit BaseActionsConfig(const char * name);
                                                                                                                           
    /**
     * Destructor
     */
    virtual ~BaseActionsConfig();
        
private:
    /**
     * Default Constructor. Not Implemented
     */
    BaseActionsConfig();

protected:
    glite::config::Logger m_logger;
    /**
     * Enable Delegation Flag
     */
    bool m_enableDelegation;
    
    /**
     * The SURL normalization that should be performed by VO Actions
     */
    SurlNormalization m_surlNormalization;
    
    /**
     * The Delegation CredService plugin type
     */
    std::string m_delegServiceType;

    /**
     * The myproxy CredService plugin type
     */
    std::string m_credServiceType;
    
    /**
     * The Default SRM Version to use
     */
    std::string m_srmVersion;
    
    /**
     * The Policy for choosing which SRM Version to use
     */
    SrmVersionPolicy m_srmVersionPolicy;
};

} // end namespace action
} // end namespace agent
} // end namespace transfer
} // end namespace data
} // end namespace glite

#endif //GLITE_DATA_TRANSFER_AGENTS_ACTION_BASEACTIONSCONFIG_H_
