/**
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *  GLite Data Agents Common : ActiveTxCache Implementation Class
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>, Paolo Tedesco <paolo.tedesco@cern.ch>
 *  Version info: $Id: ActiveTxCacheImpl.cpp,v 1.10 2008/11/13 10:43:55 ptedesco Exp $
 *  Release: $Name:  $
 *
 */
#include "ActiveTxCacheImpl.h"
#include <glite/data/agents/AgentExceptions.h>

#include <boost/tuple/tuple.hpp>

using namespace glite::data::transfer::agent;
using namespace glite::data::transfer::agent::action;
using namespace glite::data::transfer::agent::action::channel;
using namespace glite::data::agents;
using namespace boost::multi_index;

namespace {
const char * const ACTIVE_TX_CACHE_MODULE = "channel-action-tx-cache";

struct ObjectNotInCacheException : public RuntimeError{
    ObjectNotInCacheException(const std::string& objectType,const std::string& objectId) : 
        RuntimeError(objectType + std::string(" <") + objectId + std::string("> not in cache.")){
    }
};

}

/**
 * ActiveTxCacheImpl
 * Constructor
 */
ActiveTxCacheImpl::ActiveTxCacheImpl():
    m_logger(ACTIVE_TX_CACHE_MODULE){
}
    
/**
 * ~ActiveTxCacheImpl
 * Destructor
 */
ActiveTxCacheImpl::~ActiveTxCacheImpl(){
}

/**
 * Insert a Transfer into the Cache
 * @param tx   [IN] the transfer
 * @param file [IN] the file
 * @param job  [IN] the job
 */
void ActiveTxCacheImpl::insert(
    TransferPtr tx, 
    FilePtr     file, 
    JobPtr      job) /*throw (LogicError)*/ {
    
    // Check Preconditions
    if((0 == tx.get()) || (0 == file.get()) || (0 == job.get())){
        throw NullPointerException("Invalid transfer instance - null pointer");
    }
    
    // Get index (entries indexed by file_id)
    TxSetByFileId& id_index = m_txTable.get<file_id>();
    // Check if Transfer exists
    TxSetByFileId::iterator tx_it = id_index.find(tx->fileId());
    if(tx_it == id_index.end()) {
        // Insert Transfer in the list
        m_txTable.insert(TransferEntryImpl(tx,file,job));
        m_log_debug("Transfer <" << tx->requestId() << ":" << tx->transferId() << "> for file <" << tx->fileId() << ":" << tx->jobId() << "> inserted in cache");
    } else {
        if((*tx_it).request_id != tx->requestId()){
            m_log_warn("File <" << tx->fileId() << "> has already an active transfer <" << tx->requestId() << ":" << tx->transferId() << ">");
            throw LogicError("two active transfers for the same file");
        } else {
            m_log_debug("Transfer <" << tx->requestId() << ":" << tx->transferId() << "> for file <" << tx->fileId() << "> already in cache");
        }
    }
}

/**
 * Get a transfer cache entry <tx,file,job>
 * @param file_id [IN] the id of the file
 * @return TransferEntry if found, throw RuntimeError otherwise 
 */
ActiveTxCache::TransferEntry ActiveTxCacheImpl::get(
    const std::string& id) const /*throw (RuntimeError)*/ {
    
    // Get Index
    const TxSetByFileId& id_index = m_txTable.get<file_id>();
    // Check if Transfer exists
    TxSetByFileId::const_iterator tx_it = id_index.find(id);
    if(tx_it == id_index.end()) {
        // No Transfer found
        m_log_warn("TransferEntry for file <" << id << "> not in cache");
        throw ObjectNotInCacheException("TransferEntry for file",id);
    } 

    ActiveTxCache::TransferEntry entry((*tx_it).tx,(*tx_it).file,(*tx_it).job);
    return entry;
}

/**
 * getJob
 * Get a Job Ptr (if exists)
 */
ActiveTxCache::JobPtr ActiveTxCacheImpl::getJob(
    const std::string& id) const /*throw (RuntimeError)*/{
    // Get Index
    const TxSetByJobId& id_index = m_txTable.get<job_id>();
    // Check if Transfer exists
    TxSetByJobId::const_iterator it0,it1;
    boost::tuples::tie(it0,it1) = id_index.equal_range(id);
    if(it0 == it1) {
        // No Job found
        m_log_debug("Job [" << id << "] not in cache");
        throw ObjectNotInCacheException("Job",id);
    } 
    return (*it0).job;
}

/**
 * getFile
 * Get a File Ptr (if exists)
 */
ActiveTxCache::FilePtr ActiveTxCacheImpl::getFile(const std::string& id) const /*throw (RuntimeError)*/{
    // Get Index
    const TxSetByFileId& id_index = m_txTable.get<file_id>();
    // Check if Transfer exists
    TxSetByFileId::const_iterator it0,it1;
    boost::tuples::tie(it0,it1) = id_index.equal_range(id);
    if(it0 == it1) {
        // No File found
        m_log_debug("File <" << id << "> not in cache");
        throw ObjectNotInCacheException("File",id);
    } 
    return (*it0).file;
}

/**
 * getRequest
 * Get the Transfer cache entry that refers to a request
 */
ActiveTxCache::RequestEntry ActiveTxCacheImpl::getRequest(const std::string& id) const /*throw (RuntimeError)*/{
    ActiveTxCache::RequestEntry entry(id);
    // Get Index
    const TxSetByReqId& req_id_index = m_txTable.get<req_id>();
    TxSetByReqId::const_iterator it0,it1,it;
    boost::tuples::tie(it0,it1) = req_id_index.equal_range(id);
    if(it0 == it1){
        // No Request found
        m_log_debug("No file for Request <" << id << "> found in cache");
        throw ObjectNotInCacheException("File for request",id);
    }
    for(it = it0; it != it1; ++it ){
        entry.transfers.push_back(*it);
    }
    return entry;
}

/**
 * getTransferEntries
 * Get all the Transfer cache entries
 */
ActiveTxCache::TransferEntryArray ActiveTxCacheImpl::getTransferEntries() const{
    ActiveTxCache::TransferEntryArray entries;
    // Get Index
    const TxSetByFileId& id_index = m_txTable.get<file_id>();
    TxSetByFileId::const_iterator tx_it;
    for(tx_it = id_index.begin();tx_it != id_index.end(); ++tx_it){
        entries.push_back(ActiveTxCache::TransferEntry((*tx_it).tx,(*tx_it).file,(*tx_it).job));
    }
    return entries;
}

/**
 * getRequest
 * Get the Transfer cache entry that refers to a request
 */
ActiveTxCache::RequestEntryArray ActiveTxCacheImpl::getRequests() const /*throw (RuntimeError)*/{
    ActiveTxCache::RequestEntryArray entries;
    // Get Index
    const TxSetByReqId& req_id_index = m_txTable.get<req_id>();
    TxSetByReqId::const_iterator it,it_next;
    for(it = req_id_index.begin(); it != req_id_index.end(); ){
        ActiveTxCache::RequestEntry entry((*it).request_id);
        // Get upper bound
        it_next = req_id_index.upper_bound((*it).request_id);
        for(; it != it_next; ++it){
            entry.transfers.push_back(*it);    
        }
        entries.push_back(entry);            
    }
    return entries;
}


/**
 * remove
 * Remove an entry from the Cache
 */
void ActiveTxCacheImpl::remove(const std::string& id) {
    // Get Index
    TxSetByFileId& id_index = m_txTable.get<file_id>();
    // Check if Transfer Exists
    TxSetByFileId::iterator tx_it = id_index.find(id);
    if(tx_it == id_index.end()) {
        m_log_debug("No Transfers for file <" << id << "> found in cache");
    } else {
        // Remove Transfer from the list
        id_index.erase(tx_it);
        m_log_debug("Transfer for file <" << id << "> deleted from cache");
    }
}

/**
 * reset
 * Reset the Cache
 */
void ActiveTxCacheImpl::reset(){
    // Cleanup the table
    m_txTable.clear();
    m_log_debug("Cache cleaned");
}

/**
 * count
 * Count the number of entries in the cache
 */
unsigned int ActiveTxCacheImpl::count() const /*throw ()*/{
    // Get Index
    const TxSetByFileId& id_index = m_txTable.get<file_id>();
    return id_index.size();
}

/**
 * count
 * Count the number of entries in the cache for a given VO
 */
unsigned int ActiveTxCacheImpl::count(const std::string& name) const /*throw ()*/{
    // Get Index
    const TxSetByVoName& vo_name_index = m_txTable.get<vo_name>();
    return vo_name_index.count(name);
}

/**
 * countRequests
 * Count the number of requests in the cache
 */
unsigned int ActiveTxCacheImpl::countRequests() const /*throw ()*/{
    unsigned int count = 0;
    // Get Index
    const TxSetByReqId& req_id_index = m_txTable.get<req_id>();
    TxSetByReqId::const_iterator it;
    for(it = req_id_index.begin(); it != req_id_index.end(); ){
        count++;
        // jump to next
        it=req_id_index.upper_bound((*it).request_id);
    }
    return count;
}

/**
 * countRequests
 * Count the number of requests in the cache for a given VO
 */
unsigned int ActiveTxCacheImpl::countRequests(const std::string& name) const /*throw ()*/{
    std::set<std::string> request_id_set;
    // Get Index
    const TxSetByVoName& vo_name_index = m_txTable.get<vo_name>();
    TxSetByVoName::const_iterator it,it0,it1;
    boost::tuples::tie(it0,it1) = vo_name_index.equal_range(name);
    for(it = it0; it != it1; ++it){
        request_id_set.insert((*it).request_id);
    }
    return request_id_set.size();
}

/**
 * countPerSourceSE
 * Count the number of transfers in the cache for the given source SE
 */
unsigned int ActiveTxCacheImpl::countPerSourceSE(const std::string& src_se) const /*throw ()*/{
    // Get Index
    const TxSetBySrcSe& src_se_index = m_txTable.get<src_se_id>();
    return src_se_index.count(src_se);
}

/**
 * countPerDestSE
 * Count the number of requests in the cache for the given destination SE
 */
unsigned int ActiveTxCacheImpl::countPerDestSE(const std::string& dst_se) const /*throw ()*/{
    // Get Index
    const TxSetByDstSe& dst_se_index = m_txTable.get<dst_se_id>();
    return dst_se_index.count(dst_se);
}

/**
 * get all the transfer entries where the transfer is in one of the given 
 * states
 */
ActiveTxCache::TransferEntryArray ActiveTxCacheImpl::getTransfersByState(
    model::Transfer::State states) /*throw()*/ const{
    
    ActiveTxCache::TransferEntryArray entries;
    // Get Index
    const TxSetByFileId& id_index = m_txTable.get<file_id>();
    TxSetByFileId::const_iterator tx_it;
    for(tx_it = id_index.begin();tx_it != id_index.end(); ++tx_it){
        const ActiveTxCache::TransferPtr& transfer = tx_it->tx;
        if (states & transfer->state()){
            entries.push_back(ActiveTxCache::TransferEntry((*tx_it).tx,(*tx_it).file,(*tx_it).job));
        }
    }
    return entries;
}

/**
 * get the number of transfer entries where the transfer is in one of the 
 * given states
 */
unsigned int ActiveTxCacheImpl::countTransfersByState(
    model::Transfer::State states) /*throw()*/ const{
    
    unsigned int num_entries = 0;
    // Get Index
    const TxSetByFileId& id_index = m_txTable.get<file_id>();
    TxSetByFileId::const_iterator tx_it;
    for(tx_it = id_index.begin();tx_it != id_index.end(); ++tx_it){
        const ActiveTxCache::TransferPtr& transfer = tx_it->tx;
        if (states & transfer->state()){
            ++num_entries;
        }
    }
    return num_entries;
}

