/*
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *  GLite Data Agents Common : SDCache Implementation Class
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>, Paolo Tedesco <paolo.tedesco@cern.ch>
 *  Version info: $Id: ActiveTxCacheImpl.h,v 1.7 2008/11/13 10:43:55 ptedesco Exp $
 *  Release: $Name:  $
 *
 */
#ifndef GLITE_DATA_TRANSFER_AGENT_ACTIONS_CHANNEL_ACTIVETXCACHEIMPL_H_
#define GLITE_DATA_TRANSFER_AGENT_ACTIONS_CHANNEL_ACTIVETXCACHEIMPL_H_

#include "glite/data/transfer/agent/action/channel/ActiveTxCache.h"
#include <vector>
#include <string>
#include <boost/multi_index_container.hpp>
#include <boost/multi_index/member.hpp>
#include <boost/multi_index/composite_key.hpp>
#include <boost/multi_index/ordered_index.hpp>
#include <boost/utility.hpp>

#include <glite/data/config/service/Logger.hxx>

namespace glite   {
namespace data    {
namespace transfer{
namespace agent   {
namespace action  {
namespace channel {

/**
 * SDCache Implementation (Pimpl Idiom)
 */
class ActiveTxCacheImpl : boost::noncopyable{
public:
    
    typedef boost::shared_ptr<model::Job>      JobPtr; 
    typedef boost::shared_ptr<model::File>     FilePtr; 
    typedef boost::shared_ptr<model::Transfer> TransferPtr; 

    /**
     * Constructor
     */
    ActiveTxCacheImpl();
        
    /**
     * Destructor
     */
    ~ActiveTxCacheImpl();
   
    /**
     * Insert a Transfer into the Cache
     */
    void insert(TransferPtr tx, FilePtr file, JobPtr job) /*throw (LogicError)*/ ;

    /**
     * Get a Transfer cache entry
     */
    ActiveTxCache::TransferEntry get(const std::string& file_id) const /*throw (RuntimeError)*/ ;

    /**
     * Get a Job Ptr (if exists)
     */
    ActiveTxCache::JobPtr getJob(const std::string& job_id) const /*throw (RuntimeError)*/;

    /**
     * Get a File Ptr (if exists)
     */
    ActiveTxCache::FilePtr getFile(const std::string& file_id) const /*throw (RuntimeError)*/;

    /**
     * Get the Transfer cache entry that refers to a request
     */
    ActiveTxCache::RequestEntry getRequest(const std::string& req_id) const /*throw (RuntimeError)*/;

    /**
     * Get all the Transfer cache entries
     */
    ActiveTxCache::TransferEntryArray getTransferEntries() const  /*throw ()*/;

    /**
     * Get the request entries
     */
    ActiveTxCache::RequestEntryArray getRequests() const /*throw ()*/;
    
    /**
     * Remove a Transfer from the cache
     */
    void remove(const std::string& file_id);

    /**
     * Reset the Cache
     */
    void reset();

    /**
     * Count the number of entries in the cache
     */
    unsigned int count() const /*throw ()*/;
    /**
     * Count the number of entries in the cache for a given VO
     */
    unsigned int count(const std::string& vo_name) const /*throw ()*/;

    /**
     * Count the number of requests in the cache
     */
    unsigned int countRequests() const /*throw ()*/;
    /**
     * Count the number of requests in the cache for a given VO
     */
    unsigned int countRequests(const std::string& vo_name) const /*throw ()*/;

    /**
     * Count the number of transfers in the cache for the given source SE
     */
    unsigned int countPerSourceSE(const std::string& src_se) const /*throw ()*/;

    /**
     * Count the number of transfers in the cache for the given destination SE
     */
    unsigned int countPerDestSE(const std::string& dst_se) const /*throw ()*/;

    /**
     * get all the transfer entries where the transfer is in one of the given 
     * states
     * @param states [IN] combination of Transfer::State values
     * @return array of TransferEntry objects where the transfer is in one of 
     *         the given states
     */
    ActiveTxCache::TransferEntryArray getTransfersByState(
        model::Transfer::State states) const /*throw()*/;

    /**
     * get the number of transfer entries where the transfer is in one of the 
     * given states
     * @param states [IN] combination of Transfer::State values
     * @return number of TransferEntry objects where the transfer is in one of 
     *         the given states
     */
    unsigned int countTransfersByState(
        model::Transfer::State states) const /*throw()*/;
            
private:
    
    /**
     * Cache Entry Type
     */
    struct TransferEntryImpl : public ActiveTxCache::TransferEntry {
        
        /*
         * The keys for the indexes
         */
        std::string  const request_id;
        std::string  const file_id;
        std::string  const job_id;
        std::string  const vo_name;
        std::string  const source_se;
        std::string  const dest_se;
              
        /*
         * Constructor
         */        
        TransferEntryImpl(TransferPtr t, FilePtr f, JobPtr j):
            ActiveTxCache::TransferEntry(t,f,j),
            request_id(t->requestId.data()),
            file_id(t->fileId.data()),
            job_id(t->jobId.data()),
            vo_name(j->voName.data()),
            source_se(j->sourceSE.data()),
            dest_se(j->destSE.data()){}
    private:
        TransferEntryImpl();
    };

    /*
     * tags for accessing several indices defined below
     */
    struct req_id{};
    struct vo_name{};
    struct job_id{};
    struct file_id{};
    struct src_se_id{};
    struct dst_se_id{};
            
    /*
     * Define The Table with
     *  - a unique index File id
     *  - a non-unique index sorted by request Id
     *  - a non-unique index sorted by VO Name
     *  - a non-unique index sorted by Job ID
     *  - a non-unique index sorted by Source SE
     *  - a non-unique index sorted by Destination SE
     */
    typedef boost::multi_index::multi_index_container<
      TransferEntryImpl,
      boost::multi_index::indexed_by<
        boost::multi_index::ordered_unique<boost::multi_index::tag<file_id>,
          BOOST_MULTI_INDEX_MEMBER(TransferEntryImpl,std::string const,file_id)
        >,
        boost::multi_index::ordered_non_unique<boost::multi_index::tag<req_id>,
          BOOST_MULTI_INDEX_MEMBER(TransferEntryImpl,std::string const,request_id)
        >,
        boost::multi_index::ordered_non_unique<boost::multi_index::tag<vo_name>,
          BOOST_MULTI_INDEX_MEMBER(TransferEntryImpl,std::string const,vo_name)
        >,
        boost::multi_index::ordered_non_unique<boost::multi_index::tag<job_id>,
          BOOST_MULTI_INDEX_MEMBER(TransferEntryImpl,std::string const,job_id)
        >,
        boost::multi_index::ordered_non_unique<boost::multi_index::tag<src_se_id>,
          BOOST_MULTI_INDEX_MEMBER(TransferEntryImpl,std::string const,source_se)
        >,
        boost::multi_index::ordered_non_unique<boost::multi_index::tag<dst_se_id>,
          BOOST_MULTI_INDEX_MEMBER(TransferEntryImpl,std::string const,dest_se)
        >
      >
    > TransferTable;

    // indexes type definitions
    typedef TransferTable::index<file_id>::type     TxSetByFileId;
    typedef TransferTable::index<job_id>::type      TxSetByJobId;
    typedef TransferTable::index<req_id>::type      TxSetByReqId;
    typedef TransferTable::index<vo_name>::type     TxSetByVoName;
    typedef TransferTable::index<src_se_id>::type   TxSetBySrcSe;
    typedef TransferTable::index<dst_se_id>::type   TxSetByDstSe;
                         
    /**
     * The Transfer Table
     */
    TransferTable m_txTable;
    
    glite::config::Logger m_logger;
};

} // end namespace channel
} // end namespace action
} // end namespace agent
} // end namespace transfer
} // end namespace data
} // end namespace glite

#endif //GLITE_DATA_TRANSFER_AGENT_ACTIONS_CHANNEL_ACTIVETXCACHEIMPL_H_
