/*
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *  GLite Data Transfer Agents: CancelTransfers Class
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>, Paolo Tedesco <paolo.tedesco@cern.ch>
 *  Version info: $Id: Cancel.cpp,v 1.18 2008/11/13 10:43:55 ptedesco Exp $
 *  Release: $Name:  $
 *
 */
#include "Cancel.h"
#include "ActionsConfig.h"
#include "FileFSM.h"
#include <glite/data/transfer/agent/ts/TransferException.h>
#include <glite/data/agents/dao/dao-types.h>
#include <glite/data/agents/cred/cred-utility.h>
#include <glite/data/agents/cred/CredServiceException.h>
#include <glite/data/agents/sd/SDConfig.h>
#include <glite/data/agents/sd/sd-utility.h>
#include <glite/data/agents/UserProxyEnv.h>

#include <boost/scoped_ptr.hpp>
#include <set>

using namespace glite::data::transfer::agent;
using namespace glite::data::transfer::agent::fsm;
using namespace glite::data::transfer::agent::action;
using namespace glite::data::transfer::agent::action::channel;
using namespace glite::data::transfer::agent::dao;

using glite::data::transfer::agent::ts::TransferException;
using glite::data::transfer::agent::model::Error;
using glite::data::agents::dao::DAOContext;
using glite::data::agents::dao::DAOException;
using glite::data::agents::dao::DAOLogicError;
using glite::data::agents::UserProxyEnv;
using glite::data::agents::LogicError;
using glite::data::agents::RuntimeError;
using glite::data::agents::AgentException;
using boost::scoped_ptr;
using boost::shared_ptr;

namespace {
const unsigned int MAX_CHUNK_SIZE = 100;
}

/*
 * Register ActionFactory Method 
 */
REGISTER_FACTORY_METHOD( Cancel );
      
/*
 * Cancel
 *
 * Constructor
 */
Cancel::Cancel():ChannelAction("CancelActive"){
}

/*
 * ~Cancel
 *
 * Destructor
 */
Cancel::~Cancel(){
}

/*
 * execute
 *
 * Execute the Action
 */
void Cancel::exec(const std::string& params) /*throw (DAOException, ExecuteException, LogicError)*/{

    // First, cancel active requests 
    cancelActiveTx();
    
    // Cancel requests waiting to be processed in the channel queue
    cancelTxInQueue();
}

/**
 * clean active transfers cache before execution
 */
void Cancel::beforeExec(){
    m_log_debug("Cleaning active transfers cache.");
    ActiveTxCache& cache = ActionsConfig::instance().activeTxCache();
    cache.reset();
}

/**
 * cancelActiveTx
 * Revoke Active Transfers
 */
void Cancel::cancelActiveTx() /*throw (DAOException, ExecuteException, LogicError)*/{
    
    m_log_debug("Canceling active transfers");
    
    std::set<std::string> jobIds;
    
    // Get Active Transfers cache
    ActiveTxCache& tx_cache = ActionsConfig::instance().activeTxCache();
    
    // Get Transfer Service
    TransferService& trService = transferService();

    std::string proxy_file;
    try{
        // Start Transaction
        context().start();
        
        // Get the list of Active Tx requests
        ActiveTxCache::RequestEntryArray reqs = tx_cache.getRequests();
        ActiveTxCache::RequestEntryArray::iterator r_it;
        for(r_it = reqs.begin(); r_it != reqs.end(); ++r_it){
            // Get Request
            ActiveTxCache::RequestEntry& req = (*r_it);
            if(true == req.transfers.empty()){
                // LogicError: This should never happen, since the cache stores 
                // TransferEntries !
                try{ 
                    trService.abort(req.id);
                    m_log_info("Request " << req.id << " aborted");
                } catch(...){}
                throw LogicError("LogicError in ActiveTxCache: No transfer returned");
            }
            // Get the Job from the first TransferEntry (All transfers within a 
            // request belong to the same job)
            ActiveTxCache::JobPtr job = (*(req.transfers.begin())).job;
            
            // Check if the request should be canceled
            if(job->cancelJob() == true){
                m_log_debug("Job [" << job->id() << "] should be cancelled");
             
                // Get User Proxy File
                try{
                    // Get proxy certificate to be used for this request
                    getProxyForJob(*job, proxy_file);
                } catch(const agents::cred::CredServiceException& exc){
                    m_log_warn("Security Error. Can't get user proxy certificate: " << exc.what());
                    m_log_info("Continue with next request. Request " << req.id << " will be failed during next CheckState iteration");
                    continue;
                }
                
                m_log_debug("Revoking Request " << req.id);
                try{
                    trService.revoke(req.id,proxy_file);
                } catch(...){
                    m_log_warn("Revoke Failed. Abort the Request");
                    // The revoke method should never throw, but ...
                    try{ 
                        trService.abort(req.id);
                        m_log_info("Request " << req.id << " aborted");
                    } catch(...){}
                }
                
                // Request has been revoked: Update Transfer & File Instances
                updateRequest(req,jobIds);
                
                m_log_info("Request " << req.id << " Revoked");
            }
        }
        if(false == jobIds.empty()){
            // Update Job States
            this->updateJobs(jobIds);

            // Commit Transaction
            context().commit();

            // Start Transaction
            context().start();

            // Update cache: Remove Completed, Failed and Aborted transfers 
            updateTransfersCache(jobIds);
        }        
        // Commit Transaction
        context().commit();
    } catch(...){
        m_log_error("Error in canceling active transfers");
        // RollBack Transaction.
        context().rollback();
        throw;
    }        
}

/*
 * updateRequest
 *
 * update the status of the transfer and files belonging to an Active Transfer Request
 */
void Cancel::updateRequest(
    ActiveTxCache::RequestEntry&    req, 
    std::set<std::string>&          jobIds) /* throw(DAOException,LogicError)*/ {
    
    // Get current time
    time_t current;
    time(&current);
    
    ActiveTxCache::TransferEntryArray::iterator tx_it;
    for(tx_it = req.transfers.begin(); tx_it != req.transfers.end(); ++tx_it){
        // Get TransferEntry
        ActiveTxCache::TransferEntry& entry = (*tx_it);
        
        // Cancel the Transfer 
        entry.tx->state       = model::Transfer::S_ABORTED;
        entry.tx->reason      = "Canceled";
        entry.tx->reasonClass = model::Error::ABORTED;
        entry.tx->errorScope  = model::Error::S_AGENT;
        entry.tx->errorPhase  = model::Error::P_CANCELING;
        
        if(((time_t)-1 != entry.tx->transferTime()) && 
           (entry.tx->transferTime() < current)){
           entry.tx->duration  = current - entry.tx->transferTime();
        }
        // Update TransferEntry
        updateTransfer(entry,jobIds);
    }
}

/*
 * cancelActiveTx
 *
 * Revoke Active Transfers
 */
void Cancel::cancelTxInQueue() /*throw (DAOException, ExecuteException, LogicError)*/{

    m_log_debug("Canceling transfers in queue");
    std::set<std::string> jobIds;
  
    // Get Transfer DAO
    FileDAO& fileDao = fileDAO();
    
    try{    
        // Start Transaction
        context().start();
        
        // Get all the ids of the Files sitting in the Channel Queue which Job 
        // State is Canceling and State is Ready, Done or Hold
        StringArrayTable ids;
        fileDao.getFilesToCancel(ids,MAX_CHUNK_SIZE);
    
        m_log_debug("Got " << ids.size() << " jobs to cancel with files in channel queue");
                
        // For Each Transfer
        std::set<std::string> jobIds;
        StringArrayTable::iterator it;
        for(it = ids.begin(); it != ids.end(); ++it){
            // Get Job Id
            const std::string& job_id = (*it).first;
            // Get File Ids
            const agents::dao::StringArray& files = (*it).second;
            
            // For each file
            agents::dao::StringArray::const_iterator f_it;
            for(f_it = files.begin(); f_it != files.end(); ++f_it){
                // Cancel File
                cancelFile((*f_it),job_id);
            }
            
            // Insert Job Id
            jobIds.insert(job_id);
            
        } // End for
    
        if(false == jobIds.empty()){
            // Update Job States
            this->updateJobs(jobIds);

            // Commit Transaction
            context().commit();

            // Start Transaction
            context().start();

            // Update cache: Remove Completed, Failed and Aborted transfers 
            updateTransfersCache(jobIds);
        }        
        
        // Commit Transaction
        context().commit();
    } catch(...){
        // RollBack Transaction.
        context().rollback();
        throw;
    } 
}

/**
 * cancelFile
 * Cancel the Pending File
 */
void Cancel::cancelFile(
    const std::string& file_id, 
    const std::string& job_id) /*throw (DAOException, LogicError)*/ {

    // Get File DAO
    FileDAO& fileDao = fileDAO();
    // Get Job DAO
    JobDAO& jobDao   = jobDAO();
    
    // Get the File Object
    scoped_ptr<model::File> f(fileDao.get(file_id));
    // Get the Job Object
    scoped_ptr<model::Job> job(jobDao.get(job_id));
    
    m_log_debug("Canceling File [" << f->id() << "] - " << f->jobId());

    // #warning "bug #33712 - S_DONE should not go S_CANCELED?"
    // Check File State
    if(f->state() & (model::File::S_READY | 
        model::File::S_DONE  | 
        model::File::S_HOLD)){
        
        m_log_debug("Trigger Event CancelRequest for File [" << f->id() << "]");
    
        FileFSM f_fsm(*f,*job);
        try{
            // Notify Cancel Request
            f_fsm.onEventCanceled();
        }catch(const FSMException& exc){
            m_log_error("Error canceling File [" << file_id << "] : " << exc.what());
            try{
                f_fsm.onEventFailed(Error::CONSISTENCY,Error::S_AGENT,Error::P_CANCELING,exc.what());
            } catch(...){}
        }
        // Update File
        fileDao.update(*f);
            
        m_log_info("File <" << f->id() << ":" << f->jobId() << "> Canceled");
    }        
}
