/**
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *  GLite Data Transfer Agents: Channel BaseAction Class
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>, Paolo Tedesco <paolo.tedesco@cern.ch>
 *  Version info: $Id: ChannelAction.cpp,v 1.24 2008/11/17 10:17:39 ptedesco Exp $
 *  Release: $Name:  $
 */
#include <glite/data/transfer/agent/action/channel/ChannelAction.h>
#include "FileFSM.h"
#include "ActionsConfig.h"
#include <glite/data/transfer/agent/dao/channel/DAOFactory.h>
#include <glite/data/transfer/agent/ts/TransferServiceFactory.h>
#include <glite/data/agents/cred/cred-utility.h>
#include <glite/data/agents/cred/CredServiceException.h>
#include <glite/data/agents/sd/SDConfig.h>
#include <glite/data/agents/sd/sd-utility.h>
#include <glite/data/transfer/agent/catalog/CatalogServiceFactory.h>
#include "dao-utility.h"
#include <boost/scoped_ptr.hpp>
#include <boost/shared_ptr.hpp>

using namespace glite::data::transfer::agent;
using namespace glite::data::transfer::agent::fsm;
using namespace glite::data::transfer::agent::action;
using namespace glite::data::transfer::agent::action::channel;
using namespace glite::data::transfer::agent::ts;

using glite::data::transfer::agent::dao::channel::DAOFactory;
using glite::data::transfer::agent::dao::translateFileState;
using glite::data::transfer::agent::model::Error;
using glite::data::agents::dao::DAOContext;
using glite::data::agents::dao::DAOException;
using glite::data::agents::dao::DAOLogicError;
using glite::data::agents::LogicError;
using glite::data::agents::RuntimeError;
using glite::data::agents::AgentException;

using boost::scoped_ptr;
using boost::shared_ptr;

/**
 * ChannelAction
 * Constructor
 */
ChannelAction::ChannelAction(const std::string& name):
        m_logger((std::string)"channel-action-" + name),
        m_context(0){
    m_log_trace("ChannelAction " << name << " created");
}

/**
 * ~TransferAction
 * Destructor
 */
ChannelAction::~ChannelAction(){
    m_log_trace("ChannelAction destroyed");
}

/**
 * context
 * set DAOContext
 */
void ChannelAction::context(const std::string& channel_name, DAOContext& ctx){
    m_channelName = channel_name;
    m_context = &(ctx);
}

/**
 * context
 * Get DAOContext
 */
DAOContext& ChannelAction::context() /*throw (LogicError)*/{
    if(0 == m_context){
        throw LogicError("null context");
    }        
    return *m_context;
}

/**
 * jobDAO
 * Get a JobDAO Instance
 */
ChannelAction::JobDAO& ChannelAction::jobDAO() /*throw (LogicError)*/{
    if(0 == m_jobDAO.get()){
        // Get DAO Instance
        m_jobDAO.reset(
            DAOFactory::instance().createJobDAO(m_channelName,context()));
    }
    return *m_jobDAO;
}

/**    
 * Get a FileDAO Instance
 */
ChannelAction::FileDAO& ChannelAction::fileDAO() /*throw (ExecuteException)*/{
    if(0 == m_fileDAO.get()){
        // Get DAO Instances
        m_fileDAO.reset(
            DAOFactory::instance().createFileDAO(m_channelName,context()));
    }
    return *m_fileDAO;
}

/**    
 * Get a TransferDAO Instance
 */
ChannelAction::TransferDAO& ChannelAction::transferDAO() /*throw (ExecuteException)*/{
    if(0 == m_transferDAO.get()){
        // Get DAO Instances
        m_transferDAO.reset(
            DAOFactory::instance().createTransferDAO(m_channelName,context()));
    }
    
    return *m_transferDAO;
}
/**    
 * Get a ChannelDAO Instance
 */
ChannelAction::ChannelDAO& ChannelAction::channelDAO() /*throw (ExecuteException)*/{
    if(0 == m_channelDAO.get()){
        // Get DAO Instances
        m_channelDAO.reset(
            DAOFactory::instance().createChannelDAO(m_channelName,context()));
    }
    
    return *m_channelDAO;
}

/**    
 * Get a ChannelDAO Instance
 */
ChannelAction::TransferService& ChannelAction::transferService() /*throw (ExecuteException)*/{
    if(0 == m_transferService.get()){
        // Get TransferService Instances
        m_transferService.reset(
            TransferServiceFactory::instance().create(m_channelName));
    }
    
    return *m_transferService;
}

/**
 * execute
 * Execute the Action
 */
void ChannelAction::execute(const std::string& params) /*throw (ExecuteException)*/{

    beforeExec();

    ActiveTxCache& cache = ActionsConfig::instance().activeTxCache();
    try{
        // Check Cache Status
        if(false == cache.isSet()){
            // Fill Transfer Cache
            fillTransfersCache();
        }
        
        // Call the template method
        this->exec(params);
    } catch(const DAOException& exc){
        m_log_error("DAO Error executing Action. Reason is " << exc.what());
        // Reset Cache
        cache.reset();
        throw ExecuteException("Some problem found interacting with DataSource");
    } catch(const ExecuteException& exc){
        m_log_error("Failed To Execute Action. Reason is " << exc.what());
        // Reset Cache
        cache.reset();
        throw;
    } catch(const LogicError& exc){
        m_log_error("LogicError executing action: " << exc.what());
        // Reset Cache
        cache.reset();
        throw;
    } catch(const AgentException& exc){
        m_log_error("Failed to execute Action. Reason is " << exc.what());
        // Reset Cache
        cache.reset();
        throw;
    } catch(const std::exception& exc){
        m_log_error("Failed to execute Action. Reason is " << exc.what());
        // Reset Cache
        cache.reset();
        throw;
    } catch(...){
        m_log_error("Cannot execute Action. Unknown Reason");
        // Reset Cache
        cache.reset();
        throw;
    }
          
    m_log_debug("Action Executed Successfully");
}

/**
 * updateJobs
 * Update Jobs States
 */
void ChannelAction::updateJobs(const std::set<std::string>& ids) /*throw (DAOLogicError, DAOException)*/{

    // Convert set into an array
    agents::dao::StringArray  jobIds;
    jobIds.insert(jobIds.begin(),ids.begin(),ids.end());

    this->updateJobs(jobIds);
}

/**
 * updateJobs
 * Update Jobs States
 */
void ChannelAction::updateJobs(const agents::dao::StringArray& ids) /*throw (DAOLogicError, DAOException)*/{

    if(!ids.empty()){
        // Get Job DAO
        JobDAO& jobDao = jobDAO();
            
        m_log_debug("Updating Jobs' State");
        try{
            // Update job States
            jobDao.updateStates(ids);
        } catch(const DAOException& exc){
            m_log_error("Failed updating Jobs states: " << exc.what());
            throw;
        } catch(const DAOLogicError& exc){
            m_log_error("LogicError updating Jobs states: " << exc.what());
            throw;
        }
        m_log_debug("Jobs' State updated");
    }
}

/**
 * failAllFiles
 * Send a Fail Event for all the files
 */
void ChannelAction::failAllFiles(
    const agents::dao::StringArray& files, 
    const Error::Category&   category, 
    const Error::Scope&      scope, 
    const Error::Phase&      phase, 
    const std::string&              reason) /*throw ()*/{
 
    std::set<std::string> ids;
    return failAllFiles(files,category, scope,phase,reason,ids);
}

/**
 * failAllFiles
 * Send a Fail Event for all the files
 */
void ChannelAction::failAllFiles(
    const agents::dao::StringArray& files, 
    const Error::Category&   category, 
    const Error::Scope&      scope, 
    const Error::Phase&      phase, 
    const std::string&              reason,
    std::set<std::string>&          ids) /*throw ()*/{
 
    // Get File DAO
    FileDAO& fileDao = fileDAO();
    // Get Job DAO
    JobDAO& jobDao   = jobDAO();

    std::string job_id;
    scoped_ptr<model::Job> job;

    // For Each File
    std::vector<std::string>::const_iterator it;
    for(it = files.begin(); it != files.end(); ++it){
        try {
            // Get the File
            scoped_ptr<model::File> f(fileDao.get((*it)));
            // Get the Job
            if((0 == job.get()) || (job_id != f->jobId())){
                job_id = f->jobId();
                job.reset(jobDao.get(job_id));
            }
            
            m_log_error("Failing File [" << f->id() << "]. This File had " << f->failures() << " failures");
            
            // Change Transfer State
            FileFSM f_fsm(*f,*job);
            f_fsm.onEventFailed(category,scope,phase,reason);

            // Store JobId
            ids.insert(f->jobId());
            
            // Update File Object
            fileDao.update(*f);
            
            // Log File State Transition
            const char * state_str = "";
            try{
                state_str = translateFileState(f->state());
            }catch(...){}
            m_log_error("File [" << f->id() << "]: State is " << f->state() << " (" << state_str << "). Failures: " << f->failures());
        } catch(const DAOException& exc){
            m_log_error("Cannot get or update File instance: " << exc.what());
            // Continue with the next one
        } catch(const DAOLogicError& exc){
            m_log_error("Cannot get or update File instance (LogicError): " << exc.what());
            // Continue with the next one
        }
    } // End For
}

/**
 * getProxyForJob
 * Get Proxy for a job
 */
void ChannelAction::getProxyForJob(model::Job& job, std::string& proxy_file) /* throw (AgentException)*/ {
    return ActionsConfig::instance().getProxyForJob(job,context(),proxy_file);
}

/**
 * getProxyForJob
 * Get Proxy for a job
 */
void ChannelAction::getProxyForJob(const std::string& job_id, std::string& proxy_file) /* throw(AgentException)*/ {
    try{
        // Get The Job 
        scoped_ptr<model::Job> job(jobDAO().get(job_id));
        // Get Proxy cet for that job
        getProxyForJob(*job,proxy_file);
    } catch (const DAOException& exc){
        m_log_error("Failed to get Job [" << job_id << "]. Reson is " << exc.what());
        throw;
    }
}   

/**
 * normalizeSurl
 * Normalize a Surl accordingly with the VO policy
 */
std::string ChannelAction::normalizeSurl(const std::string& surl, const std::string& srm_endpoint) /* throw()*/{
    return ActionsConfig::instance().normalizeSurl(surl,srm_endpoint);
}

/**
 * fillTransfersCache
 * Fill Active Transfers Cache
 */
void ChannelAction::fillTransfersCache() /* throw(DAOException, LogicError)*/ {

    // Check Cache Status
    ActiveTxCache& cache = ActionsConfig::instance().activeTxCache();

    m_log_debug("Filling Active Transfer Requests Cache");
    
    // Get Active Transfers
    try{    
        // Get Transfer DAO
        TransferDAO& transferDao = transferDAO();

        // Get The Active Transfer On the Channel, ordered by Request ID
        TransferDAO::StringPairArray activeTxs;
        try {
            transferDao.getActiveTransfers(activeTxs);
        } catch(const DAOException& exc){
            m_log_error("Failed to get Active transfers. Reason is " << exc.what());
            throw;
        }
        m_log_debug("Got " << activeTxs.size() << " Active Transfer Requests");
        
        // For Each Transfer
        TransferDAO::StringPairArray::iterator it;
        for(it = activeTxs.begin(); it != activeTxs.end(); ++it){
            // Get File Id
            const std::string& file_id = (*it).first;
            // Get Request Id
            const std::string& request_id = (*it).second;
            
            // Get Transfer Instance
            shared_ptr<model::Transfer> tx(transferDao.get(file_id,request_id));
            // Get Job Id
            const std::string& job_id = tx->jobId();
            // Check if the cache already have a job instance
            shared_ptr<model::Job> job;
            try{
                // Check if the job is already in the cache
                job = cache.getJob(job_id);
            } catch(const RuntimeError& exc){
                m_log_debug(exc.what() << ". Get it from DAO");
                // Get job from DAO
                job.reset(jobDAO().get(job_id));
            }
            // Check if the file is already in the cache
            shared_ptr<model::File> file;
            try{
                // Check if the job is already in the cache
                file = cache.getFile(file_id);
            } catch(const RuntimeError& exc){
                m_log_debug(exc.what() << ". Get it from DAO");
                // Get File from DAO
                file.reset(fileDAO().get(file_id));
            }
            // Store Transfer in the Cache
            try{
                cache.insert(tx,file,job);
            } catch(const LogicError& exc){
                // bug 31161: on some circumstances (possibly changing channel 
                // type without draining the channel) there are two active 
                // transfers for the same file. This should never happen, but 
                // it causes the whole channel agent to stop because actions get 
                // disabled. Changed behavior so that if this happens an alert 
                // is logged, the transfers aborted and the file failed.
                m_log_alert("LogicError inserting transfer in cache: " << exc.what() << ".");
                std::set<std::string> job_ids;
                try{
                    // abort previous request and remove it from the cache
                    m_log_info("Aborting previous request for file " << tx->fileId());
                    ActiveTxCache::TransferEntry transferEntry = cache.get(tx->fileId());
                    ActiveTxCache::RequestEntry request = cache.getRequest(transferEntry.tx->requestId());
                    failRequest(request,Error::CONSISTENCY,Error::S_AGENT,Error::P_TX_SERVICE,exc.what(),"",true,job_ids);
                    cache.remove(tx->fileId());
                } catch (const RuntimeError& rexc){
                    m_log_warn("Failed aborting previous request for file " << tx->fileId() << ": " << rexc.what());
                }
                // add the new request to the cache and fail it
                m_log_info("Aborting new request for file " << tx->fileId());
                cache.insert(tx,file,job);
                ActiveTxCache::RequestEntry request = cache.getRequest(tx->requestId());
                failRequest(request,Error::CONSISTENCY,Error::S_AGENT,Error::P_TX_SERVICE,exc.what(),"",true,job_ids);
                cache.remove(tx->fileId());
                m_log_info("Requests for file " << tx->fileId() << " aborted successfully");
            }
        } // End for Each
        
        m_log_debug("Cache has now " << cache.count() << " entries.");
    } catch(const RuntimeError& exc){
        m_log_error("Failed to fill Active transfers cache. Reason is " << exc.what());
        throw;
    } catch(const LogicError& exc){
        m_log_error("Logic Error filling Active transfers cache. Reason is " << exc.what());
        throw;
    }
}

/**
 * updateTransfersCache
 * Update Active Transfers Cache
 */
void ChannelAction::updateTransfersCache(std::set<std::string>& jobIds) /* throw(DAOException,LogicError)*/ {

    // Check Cache Status
    ActiveTxCache& cache = ActionsConfig::instance().activeTxCache();
    
    // Get Active Transfers
    ActiveTxCache::TransferEntryArray activeTxs = cache.getTransferEntries();

    // For Each Transfer
    ActiveTxCache::TransferEntryArray::iterator it;
    for(it = activeTxs.begin(); it != activeTxs.end(); ++it){
        // Check if Transfer has been modified
        ActiveTxCache::TransferPtr& tx = (*it).tx;
        tx->reset();
        // Check if File has been modified
        ActiveTxCache::FilePtr& file = (*it).file;
        file->reset();
        // Chek if transfer is completed
        if(tx->state() & (model::Transfer::S_COMPLETED |
                          model::Transfer::S_FAILED    |
                          model::Transfer::S_ABORTED)){
            // Check if File is Completed
            if(!(file->state() & (model::File::S_DONE |
                                  model::File::S_FAILED |
                                  model::File::S_CANCELED |
                                  model::File::S_WAITING))){
                // Inconsistent File State: Transfer is finished but File is not
                m_log_warn("Inconsistent File [" << tx->fileId() << "] State: transfer <" << tx->requestId() << ":" << tx->transferId() << "> is finished while File is still Active");
                // Get the Job Ptr
                ActiveTxCache::JobPtr& job = (*it).job;
        
                // Fail File
                FileFSM f_fsm(*file,*job);
                f_fsm.onEventFailed(Error::CONSISTENCY,Error::S_AGENT,Error::P_FINALIZATION,
                    "File is active while the related transfer is completed");
                try{                                          
                    // Update File Object
                    fileDAO().update(*file);
                    // update Job's state
                    agents::dao::StringArray ids;
                    ids.push_back(tx->jobId());
                    jobDAO().updateStates(ids);
                                          
                    jobIds.insert(tx->jobId());
                } catch(const DAOException& exc){
                    m_log_error("Failed to update File [" << file->id() << "]. Reason is " << exc.what());
                    throw;
                }
            }
            // Remove the File From the cache
            cache.remove(tx->fileId());
            m_log_debug("File [" << tx->fileId() << "] removed form active transfers cache");
        }
    } // End for each transfer
    
    // For Each Job, update Job's instance
    std::set<std::string>::const_iterator j_it;
    for(j_it = jobIds.begin(); j_it != jobIds.end(); ++j_it){
        const std::string& job_id = (*j_it);
        // Check if the Job is still in cache
        try{
            // Get Job
            ActiveTxCache::JobPtr job = cache.getJob(job_id);
            // Get the new job from the DAO Source
            boost::scoped_ptr<model::Job> job_updated(jobDAO().get(job_id));
            // Update Job's State & finishTime
            job->state = job_updated->state;
            job->finishTime = job_updated->finishTime;
            // Reset Job
            job->reset();

            m_log_debug("Job [" << job_id << "] updated in the active transfer cache");
        } catch(const DAOException& exc){
            m_log_error("Failed to retrive data for Job [" << job_id << "]. Can't update the cache. Reason: " << exc.what());
            throw;
        } catch(const RuntimeError& exc){
            // Job Removed from cache: Nothing to do
            m_log_debug("Job [" << job_id << "] removed from active transfer cache");
        }
    }
    m_log_debug("Cache updated. Cache has now " << cache.count() << " entries.");
}

/**
 * updateTransfer
 * Update Transfer and File Objects
 */
void ChannelAction::updateTransfer(
    ActiveTxCache::TransferEntry&   entry, 
    std::set<std::string>&          jobIds) /* throw(DAOException,LogicError)*/ {

    // Get the file
    m_log_debug("Updating State for File [" << entry.file->id() << "]");
    m_log_trace("File [" << entry.file->id() << "]: source=[" << entry.file->sourceSurl() << "] dest=["<< entry.file->destSurl() << "]");
        
    FileFSM f_fsm(*(entry.file),*(entry.job));
    // Check the State of the Transfer
    switch(entry.tx->state()){
        case model::Transfer::S_COMPLETED:
            // Store new State
            f_fsm.onEventDone(*(entry.tx));
            m_log_info("File [" << entry.file->id() << "] Done");
        break;
        case model::Transfer::S_ABORTED:
            // Store new State
            f_fsm.onEventCanceled();
            m_log_info("File [" << entry.file->id() << "] Canceled");
        break;
        case model::Transfer::S_FAILED:
            // Store new State
            f_fsm.onEventFailed(*(entry.tx));
            m_log_info("File [" << entry.file->id() << "] Failed. Error is " << entry.file->reason());
        break;
        case model::Transfer::S_RECEIVED:
        case model::Transfer::S_PREPARING:
        case model::Transfer::S_READY:
        case model::Transfer::S_SUBMITTED:
        case model::Transfer::S_TRANSFERRED:
        case model::Transfer::S_PROCESSING:
            // Set Monitoring Information
            f_fsm.onEventRunning(*(entry.tx));
            m_log_debug("Still Processing File [" << entry.file->id() << "]");
        break;
        default:
            m_log_error("Invalid state (" << entry.tx->state() << ") returned by TransferService");
            throw LogicError("Invalid TransferService state returned");
    } // End switch

    // Update File & Transfer
    transferDAO().update(*(entry.tx));
    m_log_debug("Transfer <" << entry.tx->requestId() << ":" << entry.tx->transferId() << "> State updated");
    fileDAO().update(*(entry.file));
    m_log_debug("File [" << entry.file->id() << "] State updated");

    // Store the Job Id
    jobIds.insert(entry.tx->jobId());
}

/**
 * failRequest
 * Fail all the files in a Request
 * @param request       [IN/OUT] the request that must be failed
 * @param category      [IN] error category
 * @param scope         [IN] error scope
 * @param phase         [IN] error phase
 * @param reason        [IN] the failure reason
 * @param proxy_file    [IN] proxy file for the job
 * @param abort_request [IN] if set to true, try to call abort on the 
 * transfer service for the request (only if the request needs revoking,
 * i.e. if the transfer is active)
 * @param jobIds        [OUT] set of job ids to which the job id of the
 * failed request will be appended
 */
void ChannelAction::failRequest(
    ActiveTxCache::RequestEntry&    req, 
    const Error::Category&          category, 
    const Error::Scope&             scope, 
    const Error::Phase&             phase, 
    const std::string&              reason, 
    const std::string&              proxy_file, 
    const bool&                     abort_request, 
    std::set<std::string>&          jobIds) /* throw(DAOException,LogicError)*/ {
        
    // Get DAOs
    FileDAO& fileDao         = fileDAO();
    TransferDAO& transferDao = transferDAO();

    bool need_revoking = false;
    bool job_id_stored = false;
    // For Each File
    ActiveTxCache::TransferEntryArray& txs = req.transfers;
    ActiveTxCache::TransferEntryArray::iterator it;
    for(it = txs.begin(); it != txs.end(); ++it){
        try {
            // Get the Transfer
            ActiveTxCache::TransferPtr& tx = (*it).tx;
            // Check if the Transfer is already completed/failed
            if(!(tx->state() & (model::Transfer::S_COMPLETED |
                                model::Transfer::S_FAILED    |
                                model::Transfer::S_ABORTED))){
                // The transfer state is not completed/failed/aborted : abort it
                tx->state = model::Transfer::S_ABORTED;
                tx->reason      = reason;
                tx->reasonClass = category;
                tx->errorScope  = scope;
                tx->errorPhase  = phase;
                need_revoking = true;
                // Update Transfer Object
                transferDao.update(*tx);
            }
            // Get the File
            ActiveTxCache::FilePtr& file   = (*it).file;
            ActiveTxCache::JobPtr&  job    = (*it).job;
            FileFSM f_fsm(*file,*job);
            f_fsm.onEventFailed(category,scope,phase,reason);
            // Update File Object
            fileDao.update(*file);
            
            // Log File State Transition
            const char * state_str = "";
            try{
                state_str = translateFileState(file->state());
            }catch(...){}
            m_log_error("File [" << file->id() << "]: State is " << file->state() << " (" << state_str << "). Failures: " << file->failures());

            // Store Job Ids
            if(false == job_id_stored){
                jobIds.insert(file->jobId());
                job_id_stored = true;
            }
        } catch(const DAOException& exc){
            m_log_error("Cannot get or update File instance: " << exc.what());
            throw;
        } catch(const DAOLogicError& exc){
            m_log_error("Cannot get or update File instance (LogicError): " << exc.what());
            throw;
        }
    } // End For
    
    if(true == need_revoking){
        if(true == abort_request){
            // Abort Request (No user credential available)
            try{
                transferService().abort(req.id);
            } catch(...){}
        } else {
            // Revoke request
            try{
                transferService().revoke(req.id,proxy_file);
            } catch(const RuntimeError& exc){
                m_log_error("Failed to revoke transfer: " << exc.what());
                m_log_warn("The transfer will be aborted");
                // Abort Request
                try{
                    transferService().abort(req.id);
                } catch(...){}
            }
        }
    }
}
