/**
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *  GLite Data Transfer Agents: CheckTransferStates Class
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *           Paolo Tedesco <paolo.tedesco@cern.ch>
 *  Version info: $Id: CheckState.cpp,v 1.21 2008/11/13 10:43:55 ptedesco Exp $
 *  Release: $Name:  $
 *
 */
#include "CheckState.h"
#include "ActionsConfig.h"
#include "FileFSM.h"
#include "JobFSM.h"
#include <glite/data/transfer/agent/ts/TransferException.h>
#include <glite/data/agents/dao/dao-types.h>
#include <glite/data/agents/cred/cred-utility.h>
#include <glite/data/agents/cred/CredServiceException.h>
#include <glite/data/agents/sd/SDConfig.h>
#include <glite/data/agents/sd/sd-utility.h>
#include <glite/data/transfer/agent/model/Transfer.h>

#include <boost/scoped_ptr.hpp>
#include <boost/shared_ptr.hpp>

#include <algorithm>

using namespace glite::data::transfer::agent;
using namespace glite::data::transfer::agent::fsm;
using namespace glite::data::transfer::agent::action;
using namespace glite::data::transfer::agent::action::channel;
using namespace glite::data::transfer::agent::model;

using glite::data::transfer::agent::model::Error;
using glite::data::agents::dao::DAOContext;
using glite::data::agents::dao::DAOException;
using glite::data::agents::dao::DAOLogicError;
using glite::data::agents::LogicError;
using glite::data::agents::RuntimeError;
using glite::data::agents::AgentException;
using boost::scoped_ptr;
using boost::shared_ptr;

namespace {
const unsigned long MAX_CHUNK_SIZE = 100;

/**
 * order_by_tx_time
 * sort TransferEntry objects by transfer time
 * @param first [IN] first object to compare
 * @param second [IN] second object to compare 
 * @return first < second
 */
bool order_by_tx_time (
    const ActiveTxCache::TransferEntry& first,
    const ActiveTxCache::TransferEntry& second){
    return first.tx->transferTime() < second.tx->transferTime();
}

/**
 * order_by_job_id
 * sort TransferEntry objects by job id
 * @param first [IN] first object to compare
 * @param second [IN] second object to compare 
 * @return first < second
 */
bool order_by_job_id (
    const ActiveTxCache::TransferEntry& first,
    const ActiveTxCache::TransferEntry& second){
    return first.job->id() < second.job->id();
}

}

/**
 * Register ActionFactory Method 
 */
REGISTER_FACTORY_METHOD( CheckState );
  
/**
 * Constructor
 */
CheckState::CheckState():ChannelAction("CheckState"){
}

/**
 * Destructor
 */
CheckState::~CheckState(){
}

/**
 * exec
 * Execute the Action
 */
void CheckState::exec(const std::string& params) /*throw (LogicError, DAOException, ExecuteException)*/{

    std::set<std::string> jobIds;
    
    ActiveTxCache& cache = ActionsConfig::instance().activeTxCache();
    // Transfer Infos
    try{
        // Start Transaction
        context().start();

        // Update the state of active transfers
        //----------------------------------------------------------------------
        // Return the list of requests        
        ActiveTxCache::RequestEntryArray activeTxs = cache.getRequests();
        m_log_debug("CheckState: got " << activeTxs.size() << " requests");
        // For Each Request
        ActiveTxCache::RequestEntryArray::iterator it;
        std::string job_id,proxy_file;
        for(it = activeTxs.begin(); it != activeTxs.end(); ++it){
            // Get Request
            ActiveTxCache::RequestEntry& req = (*it);
            // Get first Transfer
            ActiveTxCache::TransferPtr& tx = (*(req.transfers.begin())).tx;
            if(job_id != tx->jobId()){
                // Get Job
                ActiveTxCache::JobPtr& job = (*(req.transfers.begin())).job;
                try{
                    // Get proxy certificate to be used for this request
                    getProxyForJob(*job, proxy_file);
                    job_id = tx->jobId();
                } catch(const agents::cred::CredServiceException& exc){
                    failRequest(req,Error::SECURITY,Error::S_AGENT,Error::P_TX_SERVICE,exc.what(),"",true,jobIds);
                    m_log_warn("Security Error: " << exc.what());
                    m_log_info("Continue with the next job");
                    continue;
                }
            }
            // Check Request State
            checkRequestState(req,proxy_file,jobIds);
        } // End for each request

        // Update Job States
        this->updateJobs(jobIds);
        
        // Commit Transaction
        context().commit();
        
        // Update cache: Remove Completed,Failed and Aborted transfers from the 
        // cache
        //----------------------------------------------------------------------
        
        // Start Transaction
        context().start();

        updateTransfersCache(jobIds);
        
        // Commit Transaction
        context().commit();

        // Ensure that the appropriate number of transfers is running (if the
        // transfer service is configured to stop before transfers)
        //----------------------------------------------------------------------
        ensureRunningTransfers();
                
        // Check if there are jobs that need to be updated
        //----------------------------------------------------------------------
        jobIds.clear();
        
        // Start Transaction
        context().start();

        checkIntegrity(jobIds);
        
        // Update Job States
        this->updateJobs(jobIds);

        // Commit Transaction
        context().commit();
    } catch(...){
        m_log_error("Error in executing CheckState action");
        // RollBack Transaction.
        context().rollback();
        throw;
    }         
}
    
/**
 * checkRequestState
 * check Request's State
 */
void CheckState::checkRequestState(
    ActiveTxCache::RequestEntry&    req, 
    const std::string&              proxy_file, 
    std::set<std::string>&          jobIds) /* throw(DAOException)*/ {

    // Get Transfer Service
    TransferService& trService = transferService();
    
    m_log_debug("Request " << req.id << " has " << req.transfers.size() << " Transfers");
            
    TransferService::TransferInfoArray transfer_info;
    ActiveTxCache::TransferEntryArray& txs = req.transfers;
    try{
        // Create TransferInfoArray
        ActiveTxCache::TransferEntryArray::iterator tx_it;
        for(tx_it = txs.begin(); tx_it != txs.end(); ++tx_it){
            // Get Transfer Object
            ActiveTxCache::TransferPtr& t = (*tx_it).tx;
            transfer_info.push_back(t.get());
        } // end for each file

        m_log_debug("Updating Request " << req.id << " State");

        try{
            // Get State from TransferService
            trService.state(req.id, transfer_info,proxy_file);
            m_log_debug("Request " << req.id << " State retrieved");
        
            // Update State
            for(tx_it = txs.begin(); tx_it != txs.end(); ++tx_it){
                ActiveTxCache::TransferEntry& t = (*tx_it);
                // Update Transfer & File properties
                updateTransfer(t,jobIds);
            } // end for each transfer
        } catch(const ts::TransferException& exc){
            m_log_error("Failed to check state for request " << req.id << ". Reason is " << exc.what());
            // Fail all Transfers in the Request
            failRequest(req,exc.category(),exc.scope(),exc.phase(),exc.what(),proxy_file,false,jobIds);
        }
    } catch(const DAOException& exc){
        m_log_error("Failed to get/update Transfers on Data Source for Request " << req.id << ". Reason is " << exc.what());
        throw;
    } catch(const RuntimeError& exc){
        m_log_error("Error checking request " << req.id << " state. Reason is " << exc.what());
        // Fail all Transfers
        failRequest(req,Error::INTERNAL_ERROR,Error::S_AGENT,Error::P_TX_SERVICE,exc.what(),proxy_file,false,jobIds);
    } catch(const LogicError& exc){
        m_log_error("LogicError checking request " << req.id << " state. Reason is " << exc.what());
        // Fail all Transfers
        failRequest(req,Error::CONSISTENCY,Error::S_AGENT,Error::P_TX_SERVICE,exc.what(),proxy_file,false,jobIds);
    }
}

/**
 * checkIntegrity
 * Check if there are active jobs with files done or in terminal state. This may 
 * happen since it's the VO agent that is responsible for the file transition 
 * from Waiting and Pending (including Prestating): in case of retries, the job's 
 * state is Active, so the VO Agent can't update the job state
 */
void CheckState::checkIntegrity(std::set<std::string>& jobIds) /* throw(DAOException)*/ {
    try{
        m_log_debug("Check if there are jobs whose state needs to be fixed");
    
        // Get the list of the ids of the Job that needs to be fixed
        agents::dao::StringArray ids;
        jobDAO().getJobsToFix(ids,MAX_CHUNK_SIZE);
        
        m_log_debug("Fix needed for " << ids.size() << " jobs");

        // For each Job
        agents::dao::StringArray::iterator it;
        for(it = ids.begin(); it != ids.end(); ++it){
            m_log_info("State of Job [" << (*it) << "] needs to be updated");
            jobIds.insert((*it));
            
            // Check if the Job has to be canceled
            scoped_ptr<model::Job> job(jobDAO().get((*it)));
            if(true == job->cancelJob()){
                m_log_info("Job [" << (*it) << "] needs to be canceled");

                // Job has also to be canceled, be sure to cancel all the related
                // files (that mat be in hold state)
                agents::dao::StringArray file_ids;
                fileDAO().getByJobId((*it),file_ids);
                // For Each File, cancel it (this may be not the most efficient 
                // way to do it, but I expect the number of jobs to fix is most 
                // of the time 0)
                agents::dao::StringArray::iterator f_it;
                for(f_it = file_ids.begin(); f_it != file_ids.end(); ++f_it){
                    // Get File
                    scoped_ptr<model::File> file(fileDAO().get((*f_it)));
                    model::File::State old_state = file->state();
                    FileFSM f_fsm(*file,*job);
                    f_fsm.onEventCanceled();
                    // Update File instance
                    fileDAO().update(*file);
                    if(old_state != file->state()){
                        m_log_info("File <" << (*f_it) << ":" << (*it) << "> canceled");
                    }
                }
            }
        }
    } catch(const DAOException& exc){
        m_log_error("Failed getting list of Jobs in inconsistent state. Reason is " << exc.what());
        throw;
    }
}

/**
 * Ensure that the appropriate number of transfers is running (if the
 * transfer service is configured to stop before transfers)
 */
void CheckState::ensureRunningTransfers(){
    TransferService& trService = transferService();
    if (!trService.stopAfterPreparation()){
        m_log_debug("Transfer service not configured to split transfer phases.");
        return;
    }
    m_log_debug("Checking transfers that need to start the transfer phase.");
    ActiveTxCache& cache = ActionsConfig::instance().activeTxCache();
    // Get Channel
    scoped_ptr<model::Channel> channel(channelDAO().get(channelName()));
    // Compute maximum number of concurrent transfer
    unsigned long channel_files = channel->files();
    // Get the number of maximum files to be transfered (== running transfers)
    unsigned long max_transfers_for_ts = trService.max() * trService.maxRequestSize();
    if((0 == channel_files) || (channel_files > max_transfers_for_ts)){
        channel_files = max_transfers_for_ts;
    }
    
    // get the number of processing transfers
    Transfer::State states_active = trService.getActiveStates();
    unsigned long num_processing = cache.countTransfersByState(states_active);
    m_log_debug("Channel has " << channel_files << " files and " << num_processing << " transfers in transfer phase");
    if (channel_files > num_processing){
        Transfer::State states_to_start = trService.getStatesToStart();
        ActiveTxCache::TransferEntryArray submitted = cache.getTransfersByState(states_to_start);
        unsigned long num_submitted = submitted.size();
        unsigned long to_start = channel_files - num_processing;
        if (to_start > num_submitted){
            to_start = num_submitted;
        }
        m_log_debug("Starting transfer phase for " << to_start << "/" << num_submitted << " submitted transfers");
        // select the transfers with the lowest transfer time
        std::sort(submitted.begin(),submitted.end(),order_by_tx_time);
        // remove other transfers
        if (to_start < num_submitted){
            ActiveTxCache::TransferEntryArray::iterator erase_start = submitted.begin() + to_start;
            submitted.erase(erase_start,submitted.end());
        }

        ActiveTxCache::TransferEntryArray::iterator it;
        for (it = submitted.begin();it != submitted.end();++it){
            ActiveTxCache::TransferEntry& entry = (*it);
            Transfer& tx = *(entry.tx.get());
            try{
                trService.startTransferPhase(tx);
            } catch (const std::exception& exc){
                m_log_error("Error starting transfer phase: " << exc.what());
            }
        }
    }

}


