/**
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *  GLite Data Transfer Agents: FetchTransfers Class
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *           Paolo Tedesco <paolo.tedesco@cern.ch>
 *           Zsolt Molnar <zsolt.molnar@cern.ch - http://www.zsoltmolnar.hu>
 *  Version info: $Id: Fetch.cpp,v 1.53 2009/07/06 17:15:20 molnarzs Exp $
 *  Release: $Name:  $
 *
 */
#include "Fetch.h"
#include "JobFlags.h"
#include "ChannelFlags.h"
#include "ActionsConfig.h"
#include "FileFSM.h"
#include "JobFSM.h"
#include <glite/data/transfer/agent/ts/TransferException.h>
#include <glite/data/transfer/agent/dao/StringArrayTable.h>
#include <glite/data/agents/dao/dao-types.h>
#include <glite/data/agents/cred/cred-utility.h>
#include <glite/data/agents/cred/CredServiceException.h>
#include <glite/data/agents/sd/SDConfig.h>
#include <glite/data/agents/sd/sd-utility.h>
#include <glite/data/agents/UserProxyEnv.h>
#include <glite/data/agents/url-utility.h>
#include <glite/data/agents/surl-utility.h>
#include <glite/data/agents/extra/utility.h>
#include <glite/data/transfer/urlcopy/details/SurlPair.h>
#include <boost/scoped_ptr.hpp>
#include <boost/tokenizer.hpp>

using namespace glite::data::transfer::agent;
using namespace glite::data::transfer::agent::fsm;
using namespace glite::data::transfer::agent::action;
using namespace glite::data::transfer::agent::action::channel;
using namespace glite::data::agents::sd;

using glite::data::transfer::agent::ts::TransferException;
using glite::data::transfer::agent::model::Error;
using ::glite::data::transfer::urlcopy::details::SurlPair;
using glite::data::agents::dao::DAOContext;
using glite::data::agents::dao::DAOException;
using glite::data::agents::dao::DAOLogicError;
using glite::data::agents::UserProxyEnv;
using glite::data::agents::LogicError;
using glite::data::agents::RuntimeError;
using glite::data::agents::AgentException;
using boost::scoped_ptr;
using boost::shared_ptr;

namespace{
const char * const SRM_PROTOCOL  = "srm";
const unsigned int MAX_SE_NUMBER = 100;
}

/**
 * Register ActionFactory Method
 */
REGISTER_FACTORY_METHOD( Fetch );

/**
 * Constructor
 */
Fetch::Fetch():ChannelAction("Fetch"){
}

/**
 * Destructor
 */
Fetch::~Fetch(){
}

/**
 * exec
 * Execute the Action
 */
void Fetch::exec(const std::string& params) /*throw (LogicError, DAOException, ExecuteException)*/{

    // Get Channel DAO
    ChannelDAO& channelDao = channelDAO();

    // Get Transfer Service
    TransferService& trService = transferService();

    // Get Channel
    scoped_ptr<model::Channel> channel(channelDao.get(channelName()));

    m_log_debug("Fetching files for Channel " << channel->name());

    // Check that channel state is active or draining (i.e. that channel allows starting new transfers)
    if((channel->state() != model::Channel::S_ACTIVE) &&
       (channel->state() != model::Channel::S_DRAIN)){
        // Channel is not Active.
        m_log_debug("Channel is not in Active or Drain state. Skip fetching");
        return;
    }

    // Check Channel Configuration Option
    ShareType share_type = getShareType(params);

    // Compute maximum number of concurrent transfer
    unsigned long channel_files = channel->files();

    // Get the number of maximum files to be transfered
    unsigned long max_transfers_for_ts = trService.max() * trService.maxRequestSize();
    if((0 == channel_files) || (channel_files > max_transfers_for_ts)){
        channel_files = max_transfers_for_ts;
    }

    // get from transfer service how many transfers can be active on channel in any state
    unsigned long max_transfers_any_state = trService.getMaxChannelTransfers(channel_files,channel->preparingFilesRatio());

    m_log_debug("Channel files = " << channel_files << " : max transfers (any state) = " << max_transfers_any_state);

    // Get Associated VOs
    agents::dao::StringArray vo_names;
    try{
        channelDao.getVONames(vo_names);
    } catch(const DAOException& exc){
        m_log_warn("Failed to get associated VO names for channel. Reason: " << exc.what());
        throw;
    }
    m_log_debug("Got " << vo_names.size() << " VOs");

    // Loop on VO Names
    while(false == vo_names.empty()){

        // Choose VO randomly
        int size = vo_names.size();
        int pos = rand() % size;
        const std::string& vo_name = vo_names.at(pos);

        m_log_debug("Fetching files for VO " << vo_name);

        try {
            // Fetch Transfer for that VO
            fetchVOTransfers(*channel,vo_name,max_transfers_any_state,share_type);
            m_log_debug("Files for VO " << vo_name << " fetched");
        } catch(const DAOException& exc){
            m_log_warn("A DAO Error occurred fetching files for VO " << vo_name << ". Reason is " << exc.what());
            m_log_error("Failed to connect to DAO Source for VO " << vo_name << ". Continue with the next vo");
        } catch(const ExecuteException& exc){
            m_log_warn("An error occurred fetching files for VO " << vo_name << ". Reason is " << exc.what());
            m_log_error("Failed to process VO " << vo_name << ". Continue with the next one");
        }
        // Remove VO name from the list
        vo_names.erase(vo_names.begin() + pos);
    }

    // Set Channel Last Active
    time_t current_time;
    time(&current_time);
    channel->lastActive = current_time;

    // Start Transaction
    context().start();

    try{
        // Update Channel
        channelDao.updateLastActive(*channel);
        // Commit Transaction
        context().commit();
    } catch(...){
        m_log_warn("Error udpating last active timestamp");
        // RollBack Transaction. Not a Critical Error. Ignore it
        context().rollback();
    }

    // update transfer type (if needed)
    if (channel->transferType() != trService.transferType()){
        m_log_info("Updating transfer type for channel '" << channel->name() << "' from '" << channel->transferType() << "' to '" << trService.transferType() << "'");
        channel->transferType = trService.transferType();
        context().start();
        try {
            channelDao.updateTransferType(*channel);
            context().commit();
        } catch (...) {
            m_log_warn("Error udpating transfer type");
            // RollBack Transaction. Not a Critical Error. Ignore it
            context().rollback();
        }
    }

}

/**
 * fetchVOTransfers
 * fetch files per VO
 * @param channel [IN] then Channel object
 * @param vo_name [IN] the name of the VO
 * @param max_transfers_any_state [IN] the maximum number of total available transfers for the channel
 * @param share_type [IN] the ShareType for the channel
 */
void Fetch::fetchVOTransfers(
    const model::Channel&   channel,
    const std::string&      vo_name,
    unsigned long           max_transfers_any_state,
    ShareType               share_type) /*throw (DAOException,LogicError)*/{

    // Get Channel DAO
    ChannelDAO& channelDao = channelDAO();

    // Get VO Share
    double vo_share = 0;
    unsigned long vo_max_transfers = 0;
    try {
        // Check Share Type
        switch(share_type){
            case ST_NORMALIZED:
                vo_share = channelDao.getShare(vo_name,true,false);
                m_log_debug("VO <" << vo_name << "> has a normalized share of <" << vo_share << ">");
            break;
            case ST_NORMALIZED_ON_ACTIVE:
                vo_share = channelDao.getShare(vo_name,true,true);
                m_log_debug("VO <" << vo_name << "> has a normalized-on-active share of <" << vo_share << ">");
            break;
            case ST_ABSOLUTE:
            default:
                vo_share = channelDao.getShare(vo_name,false,false);
                m_log_debug("VO <" << vo_name << "> has an absolute share of <" << vo_share << ">");
            break;
        }

        // Get VO limits.
        // Note These values should already be taken into account by the
        // query that computes the active share. However, the share may use a
        // different strategy, so I prefer to verify them explicitly.
        std::pair<int,model::Channel::VOState> limit = channelDao.getLimit(vo_name);
        if(model::Channel::VS_PAUSE == limit.second){
            // This channel should be paused for that VO
            m_log_debug("Channel is paused for VO <" << vo_name << ">: no new transfer will be initiated");
            vo_share = 0;
        }
        if(limit.first > 0){
            // This channel should be limited for that VO
            m_log_debug("Channel is limited for VO <" << vo_name << "> to " << limit.first << " transfers");
            if((unsigned long)limit.first < max_transfers_any_state){
                vo_max_transfers = limit.first;
            }
        }
    } catch(const DAOException& exc){
        m_log_warn("Failed to get share for VO " << vo_name);
        throw;
    }

    unsigned long vo_max_transfers_any_state = transferService().getMaxVOTransfers(vo_max_transfers,channel.preparingFilesRatio());
    if(vo_share > 0.0){
        // Get Num Of Transfers
        unsigned long num_of_transfers = getNumOfTransfers(
            vo_name,
            vo_share,
            max_transfers_any_state,
            vo_max_transfers_any_state);
        if(num_of_transfers > 0 ){
            // Fetch Job
            m_log_debug(num_of_transfers << " slot(s) available for VO " << vo_name);
            if((channel.type() == model::Channel::T_NON_DEDICATED) && (channel.seLimit() > 0)){
                // Use SE Limit
                fetchJobsWithSELimit(channel, vo_name,num_of_transfers);
            } else {
                fetchJobs(channel, vo_name,num_of_transfers);
            }
        } else {
            m_log_debug("No slots available for VO " << vo_name);
        }
    }
}

/**
 * fetchJobs
 * Fetch files per job
 */
void Fetch::fetchJobs(
    const model::Channel&   channel,
    const std::string&      vo_name,
    unsigned long           num_of_transfers) /*throw (DAOException,LogicError)*/{

    // Get Active Transfers cache
    ActiveTxCache& tx_cache = ActionsConfig::instance().activeTxCache();
    TransferService& trService = transferService();

    std::string job_id;
    agents::dao::StringArray files;
    do{
        // Clean list
        job_id.clear();
        files.clear();

        try{
            // Start Transaction
            context().start();

            try {
                // Get up to num_of_transfers Files to transfer
                m_log_debug("Fetching up to " << num_of_transfers << " transfer(s) for VO " << vo_name);
                /* this does not modify num_of_transfers */
                fileDAO().getNextJobToTransfer(vo_name,job_id,files,num_of_transfers);
		setenv(job_id.c_str(),vo_name.c_str(),1); 
                m_log_debug("Got " << files.size() << " file(s) for VO " << vo_name);
            } catch(const DAOException& exc){
                m_log_error("Failed to fetch Files to transfer from DataSource. Reason is " << exc.what());
                throw;
            }
            if((false == job_id.empty()) && (files.size() > 0)){
                unsigned long submitted_files = fetchJob(channel, vo_name, job_id, files);
                // Decrease the num of transfers
                num_of_transfers -= submitted_files;
            } else {
                m_log_debug("No more Files to transfer");
            }
            // Commit Transaction
            context().commit();

            if(false == job_id.empty()){
                // Start Transaction
                context().start();

                // Update cache: Remove Completed,Failed and Aborted transfers from the
                // cache
                std::set<std::string> jobIds;
                jobIds.insert(job_id);
                updateTransfersCache(jobIds);

                // Commit Transaction
                context().commit();
            }
        } catch(...){
            // RollBack Transaction
            context().rollback();
            throw;
        }
    } while(false == files.empty() &&
            num_of_transfers > 0 &&
            tx_cache.countRequests() < trService.max());
}

/**
 * fetchJobsWithSELimit
 * Fetch files per job
 */
void Fetch::fetchJobsWithSELimit(
    const model::Channel&   channel,
    const std::string&      vo_name,
    unsigned long           num_of_transfers) /*throw (DAOException,LogicError)*/{

    // Get Active Transfers cache
    ActiveTxCache& tx_cache = ActionsConfig::instance().activeTxCache();

    bool is_source_a_site = isSite(channel.sourceSite());
    bool is_dest_a_site   = isSite(channel.destSite());

    int limit_files = (channel.seLimit() < num_of_transfers)?channel.seLimit():num_of_transfers;

    bool job_fetched = false;
    dao::StringArrayTable ids;
    std::set<std::string> jobIds;
    do{
        // Clean list
        ids.clear();
        jobIds.clear();
        job_fetched = false;

        try{
            // Start Transaction
            context().start();

            try {
                // Get up to num_of_transfers Files to transfer
                m_log_debug("Fetching up to " << num_of_transfers << " transfer(s) for VO " << vo_name);
                fileDAO().getJobsToTransferPerSE(vo_name,ids,limit_files,MAX_SE_NUMBER);
            } catch(const DAOException& exc){
                m_log_error("Failed to fetch Files to transfer from DataSource. Reason is " << exc.what());
                throw;
            }
            // For each element in the table
            while((false == ids.empty()) && (num_of_transfers > 0)){
                // Pick a job randomly
                int pos = rand() % ids.size();
                std::string& job_id = ids.at(pos).first;
                agents::dao::StringArray& files = ids.at(pos).second;
		 setenv(job_id.c_str(),vo_name.c_str(),1); 

                if((false == job_id.empty()) && (files.size() > 0)){
                    // Check if the Job is already in cache
                    shared_ptr<model::Job> job;
                    try{
                        // Check if the job is already in the cache
                        job = tx_cache.getJob(job_id);
                    } catch(const RuntimeError& exc){
                        m_log_debug(exc.what() << ". Get it from DAO");
                        // Get job from DAO
                        job.reset(jobDAO().get(job_id));
                    }
                    // Count limit for SE
                    unsigned int num_per_se = files.size();

                    // Check source SE
                    if(false == is_source_a_site){
                        // Check how many transfer are actually active for that source SE
                        int source_count = tx_cache.countPerSourceSE(job->sourceSE());
                        if((unsigned int)source_count < channel.seLimit()){
                            num_per_se = channel.seLimit() - source_count;
                        } else {
                            num_per_se = 0;
                            m_log_debug("Limit " << channel.seLimit() << " for Source SE reached");
                        }
                    }
                    // Check dest SE
                    if(false == is_dest_a_site){
                        // Check how many transfer are actually active for that dest SE
                        int dest_count = tx_cache.countPerDestSE(job->destSE());
                        if((unsigned int)dest_count < channel.seLimit()){
                            num_per_se = ((channel.seLimit() - dest_count) < num_per_se)?(channel.seLimit() - dest_count):num_per_se;
                        } else {
                            num_per_se = 0;
                            m_log_debug("Limit " << channel.seLimit() << " for Destination SE reached");
                        }
                    }
                    if(num_per_se < files.size()){
                        // resize files list in order not to exceed se limit
                        files.erase(files.begin() + num_per_se, files.end());
                    }
                    if(num_of_transfers < files.size()){
                        // resize files list in order not to exceed num_of_transfers
                        files.erase(files.begin() + num_of_transfers, files.end());
                    }
                    if(files.size()){
                        m_log_debug("Got " << files.size() << " file(s) for VO " << vo_name);
                        unsigned long submitted_files =
                            fetchJob(channel, vo_name, job_id, files);

                        // Decrease the num of transfers
                        num_of_transfers -= submitted_files;
                        // Save Job Id
                        jobIds.insert(job_id);
                        job_fetched = true;
                    }
                } else {
                    m_log_debug("No more Files to transfer");
                }
                // Remove job from the list
                ids.erase(ids.begin() + pos);
            } // end while

            // Commit Transaction
            context().commit();

            if(false == jobIds.empty()){
                // Start Transaction
                context().start();

                // Update cache: Remove Completed,Failed and Aborted transfers from the
                // cache
                updateTransfersCache(jobIds);

                // Commit Transaction
                context().commit();
            }
        } catch(...){
            // RollBack Transaction
            context().rollback();
            throw;
        }
    } while(num_of_transfers > 0 && (true == job_fetched));
}

/**
 * fetchJob
 * Fetch files for a job
 */
unsigned long Fetch::fetchJob(
    const model::Channel&       channel,
    const std::string&          vo_name,
    const std::string&          job_id,
    agents::dao::StringArray&   files) /*throw (DAOException,LogicError)*/{

     // Get Active Transfers cache
    ActiveTxCache& tx_cache = ActionsConfig::instance().activeTxCache();

    unsigned long submitted_files = 0;

    m_log_info("Preparing to submit " << files.size() << " files(s) of Job [" << job_id << "] belonging to VO <" << vo_name << ">");

    std::string proxy_file;
    try{
        // Check if the Job is already in cache
        shared_ptr<model::Job> job;
        try{
            // Check if the job is already in the cache
            job = tx_cache.getJob(job_id);
        } catch(const RuntimeError& exc){
            m_log_debug(exc.what() << ". Get it from DAO");
            // Get job from DAO
            job.reset(jobDAO().get(job_id));
        }

        // Get proxy certificate to be used for this request
        getProxyForJob(*job, proxy_file);

        // For each File, submit new Transfer requests to the
        // configured TransferService
        submitted_files = fetchFiles(channel,files,job,proxy_file);
        m_log_debug(submitted_files << " new transfer(s) submitted or VO " << vo_name);
    } catch (const DAOException& exc){
        m_log_warn("Cannot connect to DAO Source. Continue with the next job");
        throw;
    } catch(const agents::cred::CredServiceException& exc){
        m_log_warn("Security Error: Cannot retrieve Proxy Certificate for Job [" << job_id << "]: " << exc.what());
        // Fail all the Files with a Security Error
        failAllFiles(files,Error::SECURITY,Error::S_AGENT,Error::P_TX_SERVICE,exc.what());
    }

    setenv(job_id.c_str(),vo_name.c_str(),1); 
    
    // Update Job State
    agents::dao::StringArray jobIds;
    jobIds.push_back(job_id);
    this->updateJobs(jobIds);

    return submitted_files;
}


/**
 * fetchFiles
 * fetch files per Job
 */
unsigned long Fetch::fetchFiles(
    const model::Channel&       channel,
    agents::dao::StringArray&   files,
    shared_ptr<model::Job>&     job,
    const std::string&          proxy_file)/*throw (DAOException,LogicError)*/{

    unsigned long submitted_files = 0;

    // Get Active Transfers cache
    ActiveTxCache& tx_cache = ActionsConfig::instance().activeTxCache();

    // Get Transfer Service
    TransferService& trService = transferService();

    // Get File DAO
    FileDAO& fileDao = fileDAO();

    // Get Transfer DAO
    TransferDAO& transferDao = transferDAO();

    // Get the number of max Transfers that can be processed in the same request
    unsigned long max_request_size = trService.maxRequestSize();
    m_log_debug("max_request_size = " << max_request_size);

    // Get SRM endpoints
    std::pair<std::string,std::string> src_srm;
    std::pair<std::string,std::string> dest_srm;
    getSrmEndpoints(files,*job,proxy_file,src_srm,dest_srm);

    unsigned long current_requests = tx_cache.countRequests();
    m_log_debug("Got " << current_requests << " Active Transfer Requests on channel");

    // Compute how many requests can be submitted
    unsigned long max_requests = trService.max() - current_requests;
    m_log_debug("Available Transfer Requests are " << max_requests);

    // For Each File
    agents::dao::StringArray::iterator it = files.begin();
    unsigned int request_num = 0;

    // Get number of streams for the channel
    unsigned long num_streams = channel.streams();
    m_log_debug("Transfers on this channel should use " << num_streams << " streams");

    // Get the TCP Buffer size for the channel
    unsigned long tcpbs = 0;
    try{
        tcpbs = glite::data::agents::str_to_size(channel.tcpBS());
    } catch(const LogicError& exc){
        m_log_debug("Invalid size (" << channel.tcpBS() << ") for channel TCPBS property. Reason: " << exc.what());
    }
    m_log_debug("Transfers on this channel should use a TCP BS of " << tcpbs << " (" << channel.tcpBS() << ")");

    // Get the block size for the channel
    unsigned long blocksize = 0;
    try{
        blocksize = glite::data::agents::str_to_size(channel.blocksize());
    } catch(const LogicError& exc){
        m_log_debug("Invalild size (" << channel.blocksize() << ") for channel Block Size property. Reason: " << exc.what());
    }

    while((it != files.end() && (request_num < max_requests))){
        // Increase request number
        ++request_num;

        // Get Request Size
        unsigned long request_size = std::distance(it,files.end());
        request_size = (request_size < max_request_size)?request_size:max_request_size; // min (request_size,max_request_size)

        // Array to store File instances (avoid to read it twice from the DB)
        typedef std::vector<shared_ptr<model::File> > FileArray;
        FileArray file_array;
        file_array.reserve(request_size);

        // Fill Transfer Request
        struct TransferService::TransferRequest request;
        request.parameters       = job->params();
        request.streams          = num_streams;
        request.tcpbs            = tcpbs;
        request.src_srm          = src_srm.first;
        request.src_srm_version  = src_srm.second;
        request.dest_srm         = dest_srm.first;
        request.dest_srm_version = dest_srm.second;
        request.overwrite_flag   = job->overwriteFlag();
        request.description      = job->id();
        if(false == job->spaceToken().empty()){
            request.dest_space_token = job->spaceToken();
        }
        if(false == job->storageClass().empty()){
            request.dest_token_descr = job->storageClass();
        }
        request.blocksize                   = blocksize;
        // new in 3.1.0
        // here copy timeouts from channel object; it will be the transfer
        // service not to use them if they're set to -1
        request.http_timeout                = channel.httpTimeout();
        request.log_level                   = channel.txLoglevel();
        request.urlcopy_put_timeout         = channel.urlcopyPutTimeout();
        request.urlcopy_putdone_timeout     = channel.urlcopyPutdoneTimeout();
        request.urlcopy_get_timeout         = channel.urlcopyGetTimeout();
        request.urlcopy_getdone_timeout     = channel.urlcopyGetdoneTimeout();
        request.urlcopy_tx_timeout          = channel.urlcopyTxTimeout();
        request.urlcopy_tx_markers_timeout  = channel.urlcopyTxmarksTimeout();
        request.srmcopy_direction           = channel.srmcopyDirection();
        request.srmcopy_timeout             = channel.srmcopyTimeout();
        request.srmcopy_refresh_timeout     = channel.srmcopyRefreshTimeout();
        // new in 3.2.0
        if (false == job->sourceSpaceToken().empty()){
            request.source_space_token = job->sourceSpaceToken();
        }
        if (false == job->sourceTokenDescription().empty()){
            request.source_token_descr = job->sourceTokenDescription();
        }
        request.enable_dir_check                = channel.targetDirCheck();
        request.urlcopy_first_tx_marker_timeout = channel.urlcopyFirstTxmarkTimeout();
        // new in 3.3.0
        request.tx_timeout_per_mb       = channel.txTimeoutPerMb();
        request.no_tx_activity_timeout  = channel.noTransferActivityTimeout();
        request.copy_pin_lifetime       = job->copyPinLifetime();
        request.lan_connection          = job->lanConnection();
        request.fail_nearline           = job->failNearline();
        // Array to store the File id to be clean in case of error
        agents::dao::StringArray request_file_ids;

        bool overwrite_files  = true;
        bool checksum_error = false;

        try{
            agents::dao::StringArray::iterator f_it = it;
            for(unsigned long i = 0; i < request_size; ++i,++f_it){
                // Get File Instance
                shared_ptr<model::File> f(fileDao.get(*f_it));
                // Check if file has the overwrite option
                if(std::string::npos == f->internalParams().find(FILE_PARAM_OVERWRITE)){
                    overwrite_files = false;
                }
                // Store file ptr into the array
                file_array.push_back(f);
                // Store SURLs Pair
                SurlPair sp;
                sp.source = this->normalizeSurl(f->sourceSurl(), src_srm.first);
                sp.destination = this->normalizeSurl(f->destSurl(),dest_srm.first);
                // Get the checksum info
            	boost::char_separator<char> sep(_ChecksumFieldSeparator());
            	typedef boost::tokenizer< boost::char_separator<char> > Tokenizer;
            	Tokenizer tokenizer(f->checksum(), sep);
            	std::vector<std::string> tokens;
            	std::copy(tokenizer.begin(), tokenizer.end(), std::back_inserter(tokens));

            	if (tokens.size() == 2) {
            		sp.checksum_algorithm = tokens[0];
					sp.checksum_value = tokens[1];
            	} else if (!tokens.empty()) {
            		std::string msg = "Wrongly formatted t_file:checksum field: " + f->checksum() + " (file id: " + f->id() + ")";
                    m_log_error(msg);
                    checksum_error = true;
            	}
                // new in 3.4.1
                sp.compare_checksum = job->checksum_method();
                request.surls.push_back(sp);
                // Save the file ids send during the request
                request_file_ids.push_back(*f_it);
            }
        } catch(const DAOException& exc){
            // Fail to get file instances
            m_log_error("DAO Error in getting files for Job " << job->id() << ". Reason is " << exc.what());
            throw;
        }
        if((false == request.overwrite_flag) & (true == overwrite_files)){
            // all the files have the overwrite flag set
            request.overwrite_flag = true;
            m_log_info("File(s) will be submitted with overwrite_flag set to true");
        }

        if (checksum_error) {
        	failAllFiles(request_file_ids, Error::USER, Error::S_AGENT, Error::P_TX_SERVICE, "Checksum format error");
            it += request_file_ids.size();
            continue;
        }

        // Submit Transfer Request
        struct TransferService::TransferResponse response;
        try{
            trService.submit(request,response,proxy_file);
        } catch(const TransferException& exc){
            m_log_error("Failed to submit transfer request per Job [" << job->id() << "]. Reason is " << exc.what());
            // Fail the submitted files with a Agent Error
            failAllFiles(request_file_ids,exc.category(),exc.scope(),exc.phase(),exc.what());
            // Increase iterator
            it += request_file_ids.size();
            // Continue loop
            continue;
        } catch(const LogicError& exc){
            m_log_error("LogicError in submitting transfer request per Job [" << job->id() << "]. Reason is " << exc.what());
            // Fail the submitted files with a Agent Error
            failAllFiles(request_file_ids,Error::INTERNAL_ERROR,Error::S_AGENT,Error::P_TX_SERVICE,exc.what());
            // Increase iterator
            it += request_file_ids.size();
            // Continue loop
            continue;
        }

        const std::string& request_id = response.request_id;
        m_log_info("Request " << request_id << " (Job id: " << job->id() << ") Submitted");

        // Get Current Time
        time_t current_time;
        time(&current_time);

        try{
            // Create Element of the Request
            for(unsigned long i = 0; i < request_size; ++i,++it){
                // Create a Transfer Instance
                shared_ptr<model::Transfer> transfer(new model::Transfer(
                       request_id,                   // request_id
                       (*it),                        // file_id
                       response.transfer_ids.at(i),  // transfer_id
                       job->id(),                    // job_id
                       model::Transfer::S_RECEIVED,  // state
                       trService.type(),             // transfer_type
                       "",                           // source_turl
                       "",                           // dest_turl
                       current_time));               // transfer_time

                // Get File Instance from the array
                shared_ptr<model::File> f = file_array.at(i);

                // Notify the Start Event
                FileFSM f_fsm(*f,*job);
                f_fsm.onEventStart(*transfer);

                // Save The Transfer Instance
                transferDao.create(*transfer);

                // Update The File Instance
                fileDao.update(*f);

                // Save the Instance in the ActiveTransfer Cache
                tx_cache.insert(transfer,f,job);

                m_log_info("File Transfer " << (*it) << " [" << transfer->requestId() << ":" << transfer->transferId() << "] Started");

                // Increase Submitted Files count
                ++submitted_files;
            } // End store each transfer
        } catch(const DAOException& exc){
            // Fail to save Transfer instances
            m_log_error("DAO Error saving transfers for Job [" << job->id() << "]. Reason is " << exc.what());
            try{
                // Revoke Request
                trService.revoke(request_id,proxy_file);
                m_log_info("Transfer Request " << request_id << " Revoked");
            } catch(...){}
            throw;
        } catch(const LogicError& exc){
            m_log_error("Logic Error in transferring files of Job [" << job->id() << "]. Reason is " << exc.what());
            try{
                // Revoke Request
                trService.revoke(request_id,proxy_file);
                m_log_info("Transfer Request " << request_id << " Revoked");
            } catch(...){}
            // Fail all the Files with a Consistency Error
            failAllFiles(files,Error::CONSISTENCY,Error::S_AGENT,Error::P_TX_SERVICE,exc.what());
        }

    } // End while loop

    return submitted_files;
}

/**
 * getNumOfTransfers
 * Get the number of transfers that a VO can submit
 * @param vo_name [IN] the name of the VO
 * @param vo_share [IN] the VO share
 * @param max_transfers [IN] the maximum number of transfers on the channel
 * @param vo_max_transfers [IN] the maximum number of transfers for the VO on the channel
 * @return number of transfers to be started for the VO
 */
unsigned long Fetch::getNumOfTransfers(
    const std::string&  vo_name,
    double              vo_share,
    unsigned long       max_transfers,
    unsigned long       vo_max_transfers) /*throw ()*/{

    unsigned long num_of_transfers = 0;

    // Get Active Transfers cache
    ActiveTxCache& tx_cache = ActionsConfig::instance().activeTxCache();

    // Get Transfer Service
    TransferService& trService = transferService();

    // Get the number of active transfer requests
    unsigned long current_requests = tx_cache.countRequests();
    m_log_debug("Got " << current_requests << " of active requests in the cache");

    // Get the maximum number of files to be transfered
    unsigned long max_transfers_for_ts = (trService.max() - current_requests);
    m_log_debug("Transfer Service can process up to " << max_transfers_for_ts << " Transfers on channel");

    // Get the number of active transfers for the channel
    unsigned long current_transfers = tx_cache.count();
    m_log_debug("Got " << current_transfers << " Active Transfers on channel");

    // Get the number of transfer owned by the VO
    unsigned long vo_transfers = tx_cache.count(vo_name);
    m_log_debug(vo_transfers << " Transfers already submitted by VO " << vo_name);

    // Compute available slot for the Channel
    unsigned long available = (max_transfers > current_transfers)?(max_transfers - current_transfers):0;
    if(available > max_transfers_for_ts){
        available = max_transfers_for_ts;
    }
    if(available > 0){
        // Check Quota
        unsigned long max_allowed_per_vo = (unsigned long) ((double) (max_transfers) * vo_share);
        if((vo_max_transfers > 0) && (max_allowed_per_vo > vo_max_transfers)){
            m_log_debug("VO " << vo_name << " is limited to " << vo_max_transfers << " transfers");
            max_allowed_per_vo = vo_max_transfers;
        }
        if((0 == max_allowed_per_vo) && (0 == vo_transfers) && (vo_share > 0.0)){
            max_allowed_per_vo = 1;
        }
        m_log_debug("Mamimum number of transfers for VO " << vo_name << " is " << max_allowed_per_vo);
        unsigned long allowed_per_vo = (max_allowed_per_vo > vo_transfers)?(max_allowed_per_vo - vo_transfers):0;
        m_log_debug("Allowed transfers for VO " << vo_name << " are " << allowed_per_vo);
        if(allowed_per_vo > 0){
            if(allowed_per_vo > available){
                num_of_transfers = available;
            } else {
                num_of_transfers = allowed_per_vo;
            }
            m_log_debug("Available transfers for VO " << vo_name << " are " << num_of_transfers);
        } else {
            m_log_debug("Quota Reached (" << vo_transfers << "/" << max_allowed_per_vo << ") for VO " << vo_name);
            num_of_transfers = 0;
        }
    } else {
        m_log_debug("Channel Queue Full (" << current_transfers << "/" << max_transfers << ")");
        num_of_transfers = 0;
    }

    return num_of_transfers;
}

/**
 * getShareType
 * Get ShareType Channel Configuration Option from action params
 */
Fetch::ShareType Fetch::getShareType(const std::string& params) /*throw () */{

    ShareType share_type = ST_ABSOLUTE;

    if(params.find(CHANNEL_VO_SHARE_TYPE_NORMALIZED_ON_ACTIVE) != std::string::npos){
        share_type = ST_NORMALIZED_ON_ACTIVE;
    } else if(params.find(CHANNEL_VO_SHARE_TYPE_NORMALIZED) != std::string::npos){
        share_type = ST_NORMALIZED;
    } else if(params.find(CHANNEL_VO_SHARE_TYPE_ABSOLUTE) != std::string::npos){
        share_type = ST_ABSOLUTE;
    }

    return share_type;
}

/**
 * getSrmEndpoints
 * get the SRM endpoints for the Job
 */
void Fetch::getSrmEndpoints(
    const agents::dao::StringArray&     files,
    const model::Job&                   job,
    const std::string&                  proxy_file,
    std::pair<std::string,std::string>& src_srm,
    std::pair<std::string,std::string>& dest_srm)/*throw ()*/{

    // Set the User Delegated Credential Object (could be needed by ServiceDiscovery)
    UserProxyEnv cert(proxy_file);

    // Get SRM endpoints
    std::string source      = job.source();
    std::string destination = job.dest();
    m_log_debug("Getting SRM Endpoints for Source <" << source << "> and Destination <" << destination << ">");
    try{
        if(false == files.empty()){
            // Assume all the files in the job are transferred to/form the same SEs
            scoped_ptr<model::File> f(fileDAO().get(*(files.begin())));

            // Get Source Endpoint
            try{
                src_srm = ActionsConfig::instance().getSrmEndpoint(f->sourceSurl(), source, "", job.voName());
            } catch(const std::exception& exc){
                m_log_warn("Failed to compute source SE endpoint for Job [" << job.id() << "]. Reason is " << exc.what());
            }

            // Get Destination Endpoint
            try{
                std::string tkn;
                if(false == job.spaceToken().empty()){
                    tkn = job.spaceToken();
                } else if(false == job.storageClass().empty()){
                    tkn = job.storageClass();
                }
                dest_srm = ActionsConfig::instance().getSrmEndpoint(f->destSurl(),destination, tkn, job.voName());
            } catch(const std::exception& exc){
                m_log_warn("Failed to compute destination SE endpoint for Job [" << job.id() << "]. Reason is " << exc.what());
            }
        }
    } catch(const DAOException& exc){
        m_log_warn("Cannot Get files for Job [" << job.id() << "] for retieving source and destination SRM endpoints: " << exc.what());
    } catch(const std::exception& exc){
        m_log_warn("Failed to compute source and destination SE for Job [" << job.id() << "]. Reason is " << exc.what());
    }
}

/**
 * isSite
 * Check if name corresponds to a site
 */
bool Fetch::isSite(const std::string& name) /*throw (DAOException,LogicError)*/{

    // Check the following naming convention:
    // *           : catch-all rule
    // [group_name]: the group_name should be always surrounded by []
    // site_name   : a site name otherwsie
    bool is_site = true;
    if((name == "*") ||
       ((*(name.begin()) == '[') && (*(name.rbegin()) == ']'))){
        is_site = false;
    }
    return is_site;
}
