/*
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *  GLite Data Transfer Agents: Fetch Channel Action Class
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>, Paolo Tedesco <paolo.tedesco@cern.ch>
 *  Version info: $Id: Fetch.h,v 1.14 2009/07/02 15:21:17 molnarzs Exp $
 *  Release: $Name:  $
 *
 */
#ifndef GLITE_DATA_TRANSFER_AGENTS_ACTION_CHANNEL_FETCH_H_
#define GLITE_DATA_TRANSFER_AGENTS_ACTION_CHANNEL_FETCH_H_

#include "glite/data/transfer/agent/action/channel/ChannelAction.h"
#include "glite/data/transfer/agent/action/ActionFactoryMethod.h"

#include <boost/shared_ptr.hpp>

namespace glite   {
namespace data    {
namespace transfer{
namespace agent   {
namespace action  {
namespace channel {

/**
 * The purpose of that Action is to submit Transfer to the TransferService
 */
class Fetch : public ChannelAction {
    /*
     * Declare Action Factory Method
     */
    DECLARE_FACTORY_METHOD( Fetch , "glite:Fetch");

public:
    /**
     * Constructor
     */
    Fetch();

    /**
     * Destructor
     */
    virtual ~Fetch();

protected:

    /**
     * Execute The Action
     */
    virtual void exec(const std::string& params) /*throw (LogicError, DAOException, ExecuteException)*/;

private:
    typedef enum{
        ST_ABSOLUTE = 0,
        ST_NORMALIZED,
        ST_NORMALIZED_ON_ACTIVE
    } ShareType;

    /**
     * In the t_file.checksum field of the db schema, the checksum algorithm
     * and checksum value is separated by this separator (:).
     */
    static const char* _ChecksumFieldSeparator() { return ":"; }

    /**
     * Fetch transfers for a VO
     * @param channel [IN] then Channel object
     * @param vo_name [IN] the name of the VO
     * @param max_transfers [IN] the maximum number of total available transfers for the channel
     * @param share_type [IN] the ShareType for the channel
     */
    void fetchVOTransfers(
        const model::Channel&   channel,
        const std::string&      vo_name,
        unsigned long           max_transfers,
        ShareType               share_type) /*throw (DAOException,LogicError)*/;

    /**
     * Fetch Jobs for a VO
     */
    void fetchJobs(
        const model::Channel&   channel,
        const std::string&      vo_name,
        unsigned long           num_of_transfers) /*throw (DAOException,LogicError)*/;

    /**
     * Fetch files per job
     */
    void fetchJobsWithSELimit(
        const model::Channel&   channel,
        const std::string&      vo_name,
        unsigned long           num_of_transfers) /*throw (DAOException,LogicError)*/;

    /**
     * Fetch a job
     */
    unsigned long fetchJob(
        const model::Channel&       channel,
        const std::string&          vo_name,
        const std::string&          job_id,
        agents::dao::StringArray&   files) /*throw (DAOException,LogicError)*/;

    /**
     * Fetch Files For a Job
     */
    unsigned long fetchFiles(
        const model::Channel&           channel,
        agents::dao::StringArray&       files,
        boost::shared_ptr<model::Job>&  job,
        const std::string&              proxy_file)/*throw (DAOException,LogicError)*/;

    /**
     * Get the number of transfers that a VO can submit
     * @param vo_name [IN] the name of the VO
     * @param vo_share [IN] the VO share
     * @param max_transfers [IN] the maximum number of transfers on the channel
     * @param vo_max_transfers [IN] the maximum number of transfers for the VO on the channel
     * @return number of transfers to be started for the VO
     */
    unsigned long getNumOfTransfers(
        const std::string&  vo_name,
        double              vo_share,
        unsigned long       max_transfers,
        unsigned long       vo_max_transfers) /*throw ()*/;

    /**
     * Get ShareType Channel Configuration Option from action params
     */
    ShareType getShareType(const std::string& params) /*throw()*/;

    /**
     * Get the Srm endpoints for the given job. If this operation fails, empty endpoints are returned
     */
    void getSrmEndpoints(
        const agents::dao::StringArray&     files,
        const model::Job&                   job,
        const std::string&                  proxy_file,
        std::pair<std::string,std::string>& src_srm,
        std::pair<std::string,std::string>& dest_srm)/*throw ()*/;

    /**
     * Check that the name correspond to a site
     */
    bool isSite(const std::string& name) /*throw (DAOException,LogicError)*/;
};

} // End namespace vo
} // End namespace action
} // End namespace agent
} // End namespace transfer
} // End namespace data
} // End namespace glite

#endif //GLITE_DATA_TRANSFER_AGENTS_ACTION_CHANNEL_FETCH_H_
