/*
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *  GLite Data Transfer Agents: Python Catalog Retry Strategy Class
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *           Paolo Tedesco <paolo.tedesco@cern.ch>
 *  Version info: $Id: PythonCatalogRetryStrategy.cpp,v 1.7 2008/11/13 10:43:55 ptedesco Exp $
 *  Release: $Name:  $
 *
 */
#include "PythonCatalogRetryStrategy.h"
#include "RetryConfig.h"
#include <glite/data/agents/python/PyHelper.h>
#include <glite/data/agents/python/PythonException.h>

using namespace glite::data::transfer::agent;
using namespace glite::data::transfer::agent::model;
using namespace glite::data::transfer::agent::action;
using namespace glite::data::transfer::agent::action::vo;
using namespace glite::data::agents::python;

namespace py = boost::python;
using boost::shared_ptr;

namespace {
const char * const PYTHON_RETRY_NAME = "tranfser-agent-vo-python-retry";
}
/*
 * PythonCatalogRetryStrategy
 *
 * Constructor
 */
PythonCatalogRetryStrategy::PythonCatalogRetryStrategy():CatalogRetryStrategy(),
    m_logger(PYTHON_RETRY_NAME){
}

/*
 * ~PythonCatalogRetryStrategy
 *
 * Destructor
 */
PythonCatalogRetryStrategy::~PythonCatalogRetryStrategy(){
}

/*
 * Apply
 *
 * Apply the retry strategy to a file
 */
CatalogRetryStrategy::CatalogRetryResult PythonCatalogRetryStrategy::apply(const Job& job,const FileArray& files) /*throw (LogicError,ExecuteException)*/{

    CatalogRetryResult result = CatalogRetryStrategy::WAIT;
    
    m_log_debug("Check if Job [" << job.id() << "] should be retried after catalog failure: it has " << files.size() << " file(s)");

    PyHelper& py_helper = PyHelper::instance();
    try{
        // Get the function
        if(false == RetryConfig::instance().hasCatalogRetry()){
            m_log_info("Catalog Retry not configured");
            result = CatalogRetryStrategy::FAIL;
        } else {            
            py::object func = RetryConfig::instance().catalogRetryFunction();
            m_log_debug("Got Catalog Retry Function");
                    
            // Based on the version, Use the righ signature
            if(RetryConfig::instance().catalogRetryVersion() == RetryConfig::CATALOG_RETRY_V_1_0){
                m_log_debug("Using Retry Version " << RetryConfig::instance().catalogRetryVersion());
                // Call the Retry Version
                result = py::call<CatalogRetryResult>(func.ptr(),job,files);
                m_log_debug("Function " << RetryConfig::CATALOG_RETRY_V_1_0_RETRY_FUNCTION_NAME << " called");
            } else  {
                m_log_error("Unsupported Catalog Retry Version (" << RetryConfig::instance().catalogRetryVersion() << ")");
            }
        }
    } catch(const PythonException& exc){
        m_log_error("Catalog Retry failed. Error is " << exc.what());
        result = CatalogRetryStrategy::FAIL;
    } catch(const std::exception& exc){
        m_log_error("Catalog Retry failed. Error is " << exc.what());
        result = CatalogRetryStrategy::FAIL;
    } catch(py::error_already_set){
        std::pair<std::string,std::string> err = py_helper.lastError();
        m_log_error("Catalog Retry failed. Error is " << err.first << " - " << err.second);
        result = CatalogRetryStrategy::FAIL;
    } catch(...){
        m_log_error("Catalog Retry validation failed. Unknown reason");
        throw;
    }
    m_log_debug("Catalog Retry Result for Job [" << job.id() << "] is " << result);
    
    return result;
}
