/*
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *  GLite Data Transfer Agents: BasicRetry Strategy Class
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *  Version info: $Id: PythonRetryStrategy.cpp,v 1.14 2008/11/13 10:43:55 ptedesco Exp $
 *  Release: $Name:  $
 *
 */
#include "PythonRetryStrategy.h"
#include "RetryConfig.h"
#include <glite/data/agents/python/PyHelper.h>
#include <glite/data/agents/python/PythonException.h>

using namespace glite::data::transfer::agent;
using namespace glite::data::transfer::agent::model;
using namespace glite::data::transfer::agent::action;
using namespace glite::data::transfer::agent::action::vo;
using namespace glite::data::agents::python;

namespace py = boost::python;
using boost::shared_ptr;

namespace {
const char * const PYTHON_RETRY_NAME = "tranfser-agent-vo-python-retry";
}
/*
 * PythonRetryStrategy
 *
 * Constructor
 */
PythonRetryStrategy::PythonRetryStrategy():RetryStrategy(),
    m_logger(PYTHON_RETRY_NAME){
}

/*
 * ~PythonRetryStrategy
 *
 * Destructor
 */
PythonRetryStrategy::~PythonRetryStrategy(){
}

/*
 * Apply
 *
 * Apply the retry strategy to a file
 */
RetryStrategy::RetryResult PythonRetryStrategy::apply(const Job& job,const File& file, const TransferArray& transfers) /*throw (ExecuteException)*/{

    RetryResult result = RetryStrategy::WAIT;
    
    m_log_debug("Check if File [" << file.id() << "] - " << file.jobId() << " should be started again: it has " << transfers.size() << " transfer(s)");

    PyHelper& py_helper = PyHelper::instance();
    try{
        // Translate TransferArray as list 
        py::list txs;
        TransferArray::const_iterator rit;
        for(rit = transfers.begin(); rit != transfers.end(); ++rit){
            txs.append(*(*rit));
        }
        // Get the function
        py::object func = RetryConfig::instance().retryFunction();
        m_log_debug("Got Retry Function");
                
        // Based on the version, Use the righ signature
        if(RetryConfig::instance().retryVersion() == RetryConfig::RETRY_V_1_0){
            m_log_debug("Using Retry Version " << RetryConfig::instance().retryVersion());
            // Call the Retry Version
            result = py::call<RetryResult>(func.ptr(),job,file,txs);
            m_log_debug("Function " << RetryConfig::RETRY_V_1_0_RETRY_FUNCTION_NAME << " called");
        } else  {
            m_log_error("Unsupported Retry Version (" << RetryConfig::instance().retryVersion() << ")");
        }
    } catch(const PythonException& exc){
        m_log_error("Retry failed. Error is " << exc.what());
        result = RetryStrategy::FAIL;
    } catch(const std::exception& exc){
        m_log_error("Retry failed. Error is " << exc.what());
        result = RetryStrategy::FAIL;
    } catch(py::error_already_set){
        std::pair<std::string,std::string> err = py_helper.lastError();
        m_log_error("Retry failed. Error is " << err.first << " - " << err.second);
        result = RetryStrategy::FAIL;
    } catch(...){
        m_log_error("Retry validation failed. Unknown reason");
        throw;
    }
    m_log_debug("Retry Result for File [" << file.id() << "] is " << result);
    
    return result;
}
