/*
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *  GLite Data Transfer Agents: RetryConfig Class
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *  Version info: $Id: RetryConfig.cpp,v 1.6 2008/11/13 10:43:55 ptedesco Exp $ 
 *  Release: $Name:  $
 *
 */
#include "RetryConfig.h"
#include <glite/data/agents/python/PyHelper.h>
#include <glite/data/agents/python/PythonException.h>

using namespace glite::data::transfer::agent::action::vo;
using namespace glite::data::agents::python;

namespace py = boost::python;

/*
 * Initialize Constants
 */
const char * const RetryConfig::RETRY_VERSION_FUNCTION_NAME               = "RetryVersion";
const char * const RetryConfig::RETRY_V_1_0_RETRY_FUNCTION_NAME           = "Retry";
const char * const RetryConfig::RETRY_V_1_0_INIT_FUNCTION_NAME            = "InitRetry";
const char * const RetryConfig::RETRY_V_1_0                               = "1.0";
const char * const RetryConfig::CATALOG_RETRY_VERSION_FUNCTION_NAME       = "CatalogRetryVersion";
const char * const RetryConfig::CATALOG_RETRY_V_1_0_RETRY_FUNCTION_NAME   = "CatalogRetry";
const char * const RetryConfig::CATALOG_RETRY_V_1_0_INIT_FUNCTION_NAME    = "InitCatalogRetry";
const char * const RetryConfig::CATALOG_RETRY_V_1_0                       = "1.0";

/*
 * instance
 *
 * Return the singleton Instance
 */
RetryConfig& RetryConfig::instance() {
    static RetryConfig the_instance;
    return the_instance;
}

/*
 * validateRetry
 *
 * Verify that the Retry module is fine and get the versions
 */
bool RetryConfig::validate() /*throw()*/ {
    
    bool result = false;

    PyHelper& py_helper = PyHelper::instance();
    try{
        // Load the Module
        m_module = py_helper.import(m_retryModule);
        m_log_info("Retry Module " << m_retryModule << " Loaded ");
                
        // Get the API Versison
        py::object get_version = py_helper.getFunction(m_module, RETRY_VERSION_FUNCTION_NAME);

        // Call Get Version
        m_retryVersion = py::call<std::string>(get_version.ptr());
        m_log_info("Retry Version is " << m_retryVersion);
        
        // Based on the version, check that the required functions are 
        // implemented
        if(m_retryVersion == RETRY_V_1_0){
            // Get the function
            m_function = py_helper.getFunction(m_module,RETRY_V_1_0_RETRY_FUNCTION_NAME);
            m_log_info("Got Retry Function <" << RETRY_V_1_0_RETRY_FUNCTION_NAME << ">");
            // Module is OK
            result = true;
        }
    } catch(const PythonException& exc){
        // Module Validation Failed
        m_log_info("Retry validation failed. Error is " << exc.what());
    } catch(const std::exception& exc){
        // Module Validation Failed
        m_log_info("Retry validation failed. Error is " << exc.what());
    } catch(py::error_already_set){
        // Get error message
        std::pair<std::string,std::string> err = py_helper.lastError();
        m_log_error("Retry validation failed. Error is " << err.first << " - " << err.second);
    } catch(...){
        m_log_error("Retry validation failed. Unknown reason");
    }
    if(true == result){
        m_log_info("Retry validation completed successfully");
    } else {
        return result;
    }
    if(true == this->hasCatalogRetry()){
         // Validate Catalog Retry Module
         try{
            // Load the Module
            m_catalogModule = py_helper.import(m_catalogRetryModule);
            m_log_info("Catalog Retry Module " << m_catalogRetryModule << " Loaded ");
                    
            // Get the API Versison
            py::object get_version = py_helper.getFunction(m_catalogModule, CATALOG_RETRY_VERSION_FUNCTION_NAME);
    
            // Call Get Version
            m_catalogRetryVersion = py::call<std::string>(get_version.ptr());
            m_log_info("Catalog Retry Version is " << m_catalogRetryVersion);
            
            // Based on the version, check that the required functions are 
            // implemented
            if(m_catalogRetryVersion == CATALOG_RETRY_V_1_0){
                // Get the function
                m_catalogFunction = py_helper.getFunction(m_catalogModule,CATALOG_RETRY_V_1_0_RETRY_FUNCTION_NAME);
                m_log_info("Got Catalog Retry Function <" << CATALOG_RETRY_V_1_0_RETRY_FUNCTION_NAME << ">");
                // Module is OK
                result = true;
            }
        } catch(const PythonException& exc){
            // Module Validation Failed
            m_log_info("Catalog Retry validation failed. Error is " << exc.what());
        } catch(const std::exception& exc){
            // Module Validation Failed
            m_log_info("Catalog Retry validation failed. Error is " << exc.what());
        } catch(py::error_already_set){
            // Get error message
            std::pair<std::string,std::string> err = py_helper.lastError();
            m_log_error("Catalog Retry validation failed. Error is " << err.first << " - " << err.second);
        } catch(...){
            m_log_error("Catalog Retry validation failed. Unknown reason");
        }
        
        if(true == result){
           m_log_info("Catalog Retry validation completed successfully");
        }
    }
    
    return result;
}


/*
 * init
 *
 * Initialize the Retry module
 */
bool RetryConfig::init(const std::string& params, const std::string& catalog_params) /*throw()*/ {
    bool result = false;
    
    PyHelper& py_helper = PyHelper::instance();
    
    // Based on the version, call the right method
    if(m_retryVersion == RETRY_V_1_0){
        try{
            // Get the Init function
            py::object init_function = py_helper.getFunction(m_module,RETRY_V_1_0_INIT_FUNCTION_NAME);
            m_log_info("Got Init Function <" << RETRY_V_1_0_INIT_FUNCTION_NAME << ">");
            
            // Call Get Init Function
            result = py::call<bool>(init_function.ptr(),params);
            m_log_debug(RETRY_V_1_0_INIT_FUNCTION_NAME << " returned " << str_bool(result));
        } catch(const PythonException& exc){
            // Module Initialization Failed
            m_log_warn("No InitRetry Method (" << exc.what() << "). Continue without initialization");
            result = true;
        } catch(const std::exception& exc){
            // Module Initialization Failed
            m_log_error("Retry initialization failed. Error is " << exc.what());
        } catch(py::error_already_set){
            // Get error message
            std::pair<std::string,std::string> err = py_helper.lastError();
            m_log_error("Retry initialization failed. Error is " << err.first << " - " << err.second);
        } catch(...){
            m_log_error("Retry initialization failed. Unknown reason");
        }
    }
    if(true == result){
        m_log_info("Retry Version is " << m_retryVersion);
    } else {
        return result;
    }
    
    if(true == this->hasCatalogRetry()){
        result = false;
        // Based on the version, call the right method
        if(m_catalogRetryVersion == CATALOG_RETRY_V_1_0){
            try{
                // Get the Init function
                py::object init_function = py_helper.getFunction(m_catalogModule,CATALOG_RETRY_V_1_0_INIT_FUNCTION_NAME);
                m_log_info("Got Catalog Init Function <" << CATALOG_RETRY_V_1_0_INIT_FUNCTION_NAME << ">");
                
                // Call Get Init Function
                result = py::call<bool>(init_function.ptr(),catalog_params);
                m_log_debug(CATALOG_RETRY_V_1_0_INIT_FUNCTION_NAME << " returned " << str_bool(result));
            } catch(const PythonException& exc){
                // Module Initialization Failed
                m_log_warn("No InitCatalogRetry Method (" << exc.what() << "). Continue without initialization");
                result = true;
            } catch(const std::exception& exc){
                // Module Initialization Failed
                m_log_error("Catalog Retry initialization failed. Error is " << exc.what());
            } catch(py::error_already_set){
                // Get error message
                std::pair<std::string,std::string> err = py_helper.lastError();
                m_log_error("Catalog Retry initialization failed. Error is " << err.first << " - " << err.second);
            } catch(...){
                m_log_error("Catalog Retry initialization failed. Unknown reason");
            }
        }
        if(true == result){
            m_log_info("Catalog Retry Version is " << m_catalogRetryVersion);
        } 
    }
    
    return result;
}
