/*
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *  GLite Data Transfer Agents: RetryConfig Class
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *           Paolo Tedesco <paolo.tedesco@cern.ch>
 *  Version info: $Id: RetryConfig.h,v 1.6 2008/11/13 10:43:55 ptedesco Exp $
 *  Release: $Name:  $
 *
 */
#ifndef GLITE_DATA_TRANSFER_AGENTS_VO_PYTHON_RETRYCONFIG_H_
#define GLITE_DATA_TRANSFER_AGENTS_VO_PYTHON_RETRYCONFIG_H_

#include <string>
#include <boost/python.hpp>
#include <glite/data/config/service/Logger.hxx>

namespace glite    {
namespace data     {
namespace transfer {
namespace agent    {
namespace action   {
namespace vo       {

/**
 * RetryCofnig  Class
 */
class RetryConfig {
public:

    /**
     * Return the VOActionsData singleton instance
     * @return the VOActionsData singleton instance
     */
    static RetryConfig& instance() /*throw ()*/; 

    // =========================================================================
    // Retry Static Constants
    // =========================================================================
    static const char * const RETRY_VERSION_FUNCTION_NAME;
    static const char * const RETRY_V_1_0_RETRY_FUNCTION_NAME;
    static const char * const RETRY_V_1_0_INIT_FUNCTION_NAME;
    static const char * const RETRY_V_1_0;

    // =========================================================================
    // Catalog Retry Static Constants
    // =========================================================================
    static const char * const CATALOG_RETRY_VERSION_FUNCTION_NAME;
    static const char * const CATALOG_RETRY_V_1_0_RETRY_FUNCTION_NAME;
    static const char * const CATALOG_RETRY_V_1_0_INIT_FUNCTION_NAME;
    static const char * const CATALOG_RETRY_V_1_0;

    // =========================================================================
    // VOActionsData Properties
    // =========================================================================

    /**
     * Get Retry Module Name 
     * @return the Retry Module Name
     */
    const std::string& retryModule() const {return m_retryModule;} 

    /**
     * Get Retry Module Interface Version
     * @return the Retry Module Interface Version
     */
    const std::string& retryVersion() const {return m_retryVersion;}

    /**
     * Get the module Retry Function Object
     * @return the Retry Function Object
     */
    boost::python::object retryFunction() { return m_function;}

    /**
     * Return true if the Catalog Retry Module is configured
     * @return true if the Catalog Retry Module is configured
     */
    bool hasCatalogRetry() const {return (m_catalogRetryModule.empty()?false:true);}
    
    /**
     * Get Catalog Retry Module Name 
     * @return the Catalog Retry Module Name
     */
    const std::string& catalogRetryModule() const {return m_catalogRetryModule;} 

    /**
     * Get Catalog Retry Module Interface Version
     * @return the Catalog Retry Module Interface Version
     */
    const std::string& catalogRetryVersion() const {return m_catalogRetryVersion;}

    /**
     * Get the module Catalog Retry Function Object
     * @return the Catalog Retry Function Object
     */
    boost::python::object catalogRetryFunction() { return m_catalogFunction;}

protected:
        
    /**
     * Set RetryModule 
     */
    void retryModule(const std::string& m) {m_retryModule = m;} 

    /**
     * Set CatalogRetryModule 
     */
    void catalogRetryModule(const std::string& m) {m_catalogRetryModule = m;} 

    // =========================================================================
    // VOActionsData Methods
    // =========================================================================
    
    /**
     * Verify that the module is  correct and get the module's version  
     */
    bool validate() /*throw()*/;

    /**
     * Initialize the module
     */
    bool init(const std::string& params, const std::string& catalog_params) /*throw()*/;
    
    glite::config::Logger m_logger;
    
private:

    /*
     * Declare factory methods as friends
     */
    friend class VOActionsConfig;

    /**
     * Constructor
     */
    RetryConfig () : m_logger("glite-transfer-vo-actions-python"){
    }
        
    /**
     * Destructor
     */
    ~RetryConfig (){}

private:
    /**
     * The Retry Python Module Name
     */
    std::string m_retryModule;

    /**
     * The Retry Python Module Interface Version
     */
    std::string m_retryVersion;

    /**
     * The Retry module Object
     */
    boost::python::object m_module;

    /**
     * The Retry Function Object
     */
    boost::python::object m_function;

    /**
     * The Catalog Retry Python Module Name
     */
    std::string m_catalogRetryModule;

    /**
     * The Catalog Retry Python Module Interface Version
     */
    std::string m_catalogRetryVersion;

    /**
     * The Catalog Retry module Object
     */
    boost::python::object m_catalogModule;

    /**
     * The Catalog Retry Function Object
     */
    boost::python::object m_catalogFunction;
};

} // end namespace vo
} // end namespace action
} // end namespace agent
} // end namespace transfer
} // end namespace data
} // end namespace glite

#endif //GLITE_DATA_TRANSFER_AGENTS_VO_PYTHON_RETRYCONFIG_H_
