/*
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *  GLite Data Transfer Agents: VOActionsConfig Class
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *           Paolo Tedesco <paolo.tedesco@cern.ch>
 *  Version info: $Id: VOActionsConfig.cpp,v 1.12 2009/10/06 15:40:53 molnarzs Exp $
 *  Release: $Name:  $
 *
 */
#include "VOActionsConfig.h"
#include "RetryConfig.h"
#include <glite/data/agents/AgentExceptions.h>
#include <glite/data/config/service/ParamReader.hxx>

using namespace glite::data::transfer::agent;
using namespace glite::data::transfer::agent::action;
using namespace glite::data::transfer::agent::action::vo;
using namespace glite::config;
using glite::data::agents::ConfigParamException;
using glite::data::agents::ConfigurationException;
using glite::data::agents::LogicError;
                                                                                                                          
namespace {    
const char * const VO_ACTION_NAME                   = "glite-transfer-vo-actions";
const char * const RETRYMODULE_PROPERTY_NAME        = "RetryModule";
const char * const RETRYPARAMS_PROPERTY_NAME        = "RetryParams";
const char * const CATALOGRETRYMODULE_PROPERTY_NAME = "CatalogRetryModule";
const char * const CATALOGRETRYPARAMS_PROPERTY_NAME = "CatalogRetryParams";
}

extern "C" {
/*
 * create_glite_component
 *
 * Create Component instance
 */
ComponentConfiguration * create_glite_component(){
    return new VOActionsConfig();
}
                                                                                
/*
 * destroy_glite_component
 *
 * Destroy Component instance
 */
void destroy_glite_component(ComponentConfiguration * component){
    VOActionsConfig* a = dynamic_cast<VOActionsConfig*>(component);
    if(0 != a){
        delete a;
    }
}
                                                                                
} // End extern "C"

/*
 * VOActionsConfig
 *
 * Constructor
 */
VOActionsConfig::VOActionsConfig():ComponentConfiguration(VO_ACTION_NAME),
    m_logger(VO_ACTION_NAME){
}
                                                                                                                          
/*
 * ~VOActionsConfig
 *
 * Destructor
 */
VOActionsConfig::~VOActionsConfig(){
}

/*
 * init
 *
 * Initialize the Component
 */
int VOActionsConfig::init(const Params& params){
          
    try{
        ParamReader reader(getName(),params);

        // Get Retry Module
        RetryConfig::instance().retryModule(reader.getValue<std::string>(RETRYMODULE_PROPERTY_NAME));
        // Get Retry Parameters
        std::string retry_params = _getStringParamValue(RETRYPARAMS_PROPERTY_NAME, params);
        // Get Catalog Retry Module
        if (reader.isSet(CATALOGRETRYMODULE_PROPERTY_NAME)){
            RetryConfig::instance().catalogRetryModule(_getStringParamValue(CATALOGRETRYMODULE_PROPERTY_NAME, params));
        }
        // Get Catalog Retry Parameters
        std::string catalog_retry_params = _getStringParamValue(CATALOGRETRYPARAMS_PROPERTY_NAME, params);
        // Validate Modules
        bool is_valid = RetryConfig::instance().validate();
        if(false == is_valid){
            m_log_error("Initialization failed. Retry module validation failed.");
            return -1;
        }
        
        // Initialize Retry Module
        is_valid = RetryConfig::instance().init(retry_params, catalog_retry_params);
        if(false == is_valid){
            m_log_error("Initialization failed. Retry module initialization failed.");
            return -1;
        }
        // Log Configuration Values
        m_log_info(getName() << " Initialized. Parameters are");
        m_log_info("RetryModule         : " << RetryConfig::instance().retryModule());
        m_log_debug("RetryParams         : " << retry_params);
        if (RetryConfig::instance().hasCatalogRetry()){
            m_log_info("CatalogRetryModule  : " << RetryConfig::instance().catalogRetryModule());
        } else {
            m_log_info("CatalogRetryModule  : (none)");
        }
        m_log_debug("CatalogRetryParams  : " << catalog_retry_params);
    } catch(const LogicError& exc) {
        m_log_error(getName() << " initialization Logic Error: " << exc.what());
        return -1;
    } catch(const std::exception& exc) {
        m_log_error(getName() << " initialization Unexpected Exception: " << exc.what());
        return -1;
    } catch(...) {
        m_log_error(getName() << " initialization Unhandled Exception");
        return -1;
    }
    return 0;
}

/*
 * config
 *
 * Configure the Component
 */
int VOActionsConfig::config(const Params& params){
          
    // Nothing to do
    m_log_info(getName() << " Configured");
    
    return 0;
}

std::string VOActionsConfig::_getStringParamValue(
    const std::string& paramName,
    const Params& params)
{
    Params::const_iterator it = params.find(paramName);

    if (params.end() != it) {
        ParamValue * pv = dynamic_cast<ParamValue *>(it->second);

        if (0 == pv) {
            throw InvalidParamValueException(getName(), paramName);
        }

        return pv->getValue();
    } else {
        return std::string();
    }
}
