/*
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *  GLite Data Transfer Agents: ActionsConfig Class
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *           Paolo Tedesco <paolo.tedesco@cern.ch>
 *  Version info: $Id: ActionsConfig.cpp,v 1.16 2008/12/17 15:56:45 ptedesco Exp $
 *  Release: $Name:  $
 *
 */
#include "ActionsConfig.h"
#include <glite/data/agents/AgentExceptions.h>
#include <glite/data/config/service/ParamReader.hxx>

using namespace glite::data::transfer::agent;
using namespace glite::data::transfer::agent::action;
using namespace glite::data::transfer::agent::action::vo;
using namespace glite::config;
using glite::data::agents::ConfigParamException;
using glite::data::agents::ConfigurationException;
using glite::data::agents::LogicError;

namespace {    
const char * const VO_ACTION_NAME                  = "glite-transfer-vo-actions";
const char * const MAXFAILURES_PROPERTY_NAME       = "MaxFailures";
const char * const RESUBMITDELAY_PROPERTY_NAME     = "ResubmitDelay";
const char * const CATALOGRETRYDELAY_PROPERTY_NAME = "CatalogRetryDelay";
const char * const UNKNOWNSOURCE_PROPERTY_NAME     = "EnableUnknownSource";
const char * const UNKNOWNDEST_PROPERTY_NAME       = "EnableUnknownDest";
const char * const CATALOGENABLED_PROPERTY_NAME    = "CatalogEnabled";
const char * const CHANNELCACHETTL_PROPERTY_NAME           = "ChannelCache_TTL";
const char * const CHANNELCACHEPURGEINTERVAL_PROPERTY_NAME = "ChannelCache_PurgeInterval";

const unsigned int DEFAULT_MAXFAILURES       = 3;
const unsigned int DEFAULT_RESUBMITDELAY     = 600;
const unsigned int DEFAULT_CATALOGRETRYDELAY = 600;
const bool         DEFAULT_UNKNOWNSOURCE     = false;
const bool         DEFAULT_UNKNOWNDEST       = false;
const bool         DEFAULT_CATALOGENABLED    = true;

}

extern "C" {
/*
 * create_glite_component
 *
 * Create Component instance
 */
ComponentConfiguration * create_glite_component(){
    return &(ActionsConfig::instance());
}
                                                                                
/*
 * destroy_glite_component
 *
 * Destroy Component instance
 */
void destroy_glite_component(ComponentConfiguration * component){
}
                                                                                
} // End extern "C"
    

/*
 * instance
 *
 * Return Singleton Instance
 */
ActionsConfig& ActionsConfig::instance(){
    static ActionsConfig singleton;
    return singleton;
}

/*
 * ActionsConfig
 *
 * Constructor
 */
ActionsConfig::ActionsConfig():BaseActionsConfig(VO_ACTION_NAME),
    m_maxFailures(DEFAULT_MAXFAILURES),
    m_resubmitDelay(DEFAULT_RESUBMITDELAY),
    m_catalogRetryDelay(DEFAULT_CATALOGRETRYDELAY),
    m_unknownSrcEnabled(DEFAULT_UNKNOWNSOURCE),
    m_unknownDstEnabled(DEFAULT_UNKNOWNDEST),
    m_catalogEnabled(DEFAULT_CATALOGENABLED){
}

/*
 * config
 *
 * Configure the Component
 */
int ActionsConfig::config(const Params& params){
          
    try{
        ParamReader reader(getName(),params);

        // Get Max Failures
        reader.getParameter(MAXFAILURES_PROPERTY_NAME,m_maxFailures)
            .ge(1);
        
        // Get resubmit Delay
        reader.getParameter(RESUBMITDELAY_PROPERTY_NAME,m_resubmitDelay)
            .ge(0);
        
        // Get catalog Delay
        reader.getParameter(CATALOGRETRYDELAY_PROPERTY_NAME,m_catalogRetryDelay)
            .ge(0);
        
        // Get Unknown Source
        reader.getParameter(UNKNOWNSOURCE_PROPERTY_NAME,m_unknownSrcEnabled);
        
        // Get Unknown Dest
        reader.getParameter(UNKNOWNDEST_PROPERTY_NAME,m_unknownDstEnabled);
        
        // Catalog Enabled
        reader.getParameter(CATALOGENABLED_PROPERTY_NAME,m_catalogEnabled);

        // ChannelCache TTL
        long ttl = 0;
        if (reader.isSet(CHANNELCACHETTL_PROPERTY_NAME)){
            reader.getParameter(CHANNELCACHETTL_PROPERTY_NAME,ttl)
                .ge(1);
            m_channelCache.ttl(ttl);
        }
        // ChannelCache Purge Interval
        long purge_delay = 0;
        if (reader.isSet(CHANNELCACHEPURGEINTERVAL_PROPERTY_NAME)){
            reader.getParameter(CHANNELCACHEPURGEINTERVAL_PROPERTY_NAME,purge_delay)
                .ge(1);
            m_channelCache.purgeDelay(purge_delay);
        }
        
        // Log Configuration Values
        m_log_info(getName() << " Configured. Configuration Parameters are");
        m_log_info("MaxFailures         : " << m_maxFailures);
        m_log_info("Resubmit Delay      : " << m_resubmitDelay);
        m_log_info("Catalog Retry Delay : " << m_catalogRetryDelay);
        m_log_info("Unknown Source      : " << str_enabled(m_unknownSrcEnabled));
        m_log_info("Unknown Destination : " << str_enabled(m_unknownDstEnabled));
        m_log_info("Catalog Interaction : " << str_enabled(m_catalogEnabled));
        if(0 < ttl){
            m_log_info("ChannelCache TTL    : " << ttl);
        }
        if(0 < purge_delay){
            m_log_info("Ch.Cache Purge Itrvl: " << purge_delay);
        }
    } catch(const LogicError& exc) {
        m_log_error(getName() << " configuration Logic Error: " << exc.what());
        return -1;
    } catch(const std::exception& exc) {
        m_log_error(getName() << " configuration Unexpected Exception: " << exc.what());
        throw;
    } catch(...) {
        m_log_error(getName() << " configuration Unhandled Exception");
        throw;
    }
    return 0;
}

/*
 * stop
 *
 * Stop the Component
 */
int ActionsConfig::stop(){
         
    m_channelCache.clear();
    m_log_info(getName() << " Stopped");
    return 0;
}
