/*
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *  GLite Data Transfer Agents: Allocate Action Class
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>, Paolo Tedesco <paolo.tedesco@cern.ch>
 *  Version info: $Id: Allocate.cpp,v 1.42 2008/11/13 10:43:55 ptedesco Exp $
 *  Release: $Name:  $
 *
 */
#include "Allocate.h"
#include "JobFlags.h"
#include "ActionsConfig.h"
#include "FileFSM.h"
#include "JobFSM.h"
#include <glite/data/agents/url-utility.h>
#include <glite/data/agents/UserProxyEnv.h>
#include <glite/data/agents/extra/utility.h>
#include <glite/data/agents/sd/SDConfig.h>
#include <glite/data/agents/sd/sd-utility.h>
#include <glite/data/agents/AgentExceptions.h>
#include <glite/data/agents/cred/CredServiceException.h>
#include <glite/data/transfer/agent/catalog/CatalogService.h>
#include <glite/data/transfer/agent/catalog/CatalogServiceFactory.h>
#include <glite/data/transfer/agent/catalog/CatalogServiceException.h>
#include <glite/data/transfer/agent/namegen/NameGeneration.h>
#include <glite/data/transfer/agent/namegen/NameGenerationFactory.h>
#include <glite/data/transfer/agent/namegen/NameGenerationException.h>

#include <boost/scoped_ptr.hpp>
#include <boost/shared_ptr.hpp>
#include <unistd.h>
#include <set>

using namespace glite::data::transfer::agent;
using namespace glite::data::transfer::agent::model;
using namespace glite::data::transfer::agent::fsm;
using namespace glite::data::transfer::agent::action;
using namespace glite::data::transfer::agent::catalog;
using namespace glite::data::transfer::agent::namegen;
using namespace glite::data::transfer::agent::action::vo;

using glite::data::agents::UserProxyEnv;
using glite::data::agents::LogicError;
using glite::data::agents::RuntimeError;
using glite::data::agents::dao::DAOException;
using glite::data::agents::cred::CredServiceException;
using boost::scoped_ptr;
using boost::shared_ptr;

namespace {
const unsigned int MAX_JOB_CHUNKS        = 20;
const char * const SRM_PROTOCOL          = "srm";
const char * const SFN_PROTOCOL          = "sfn";
const char * const GSIFTP_PROTOCOL       = "gsiftp";
const char * const UNKNOWN_SITE          = "UNKNOWN";
const char * const WILDCHAR_SITE         = "*";
}

/*
 * Register ActionFactory Method 
 */
REGISTER_FACTORY_METHOD( Allocate );
      
/*
 * Allocate
 *
 * Constructor
 */
Allocate::Allocate():VOAction("Allocate"){
}

/*
 * ~Allocate
 *
 * Destructor
 */
Allocate::~Allocate(){
}

/*
 * exec
 *
 * Execute the Action
 */
void Allocate::exec(const std::string& params) /*throw (DAOException, LogicError)*/{

    // Trigger Channel Cache Purge
    ActionsConfig::instance().channelCache().purge();

    // Get Job DAO
    JobDAO& jobDao = jobDAO();
            
    // Check if the Disable delegation Flag is set on the action parameters
    bool disable_deleg = 
        (params.find(JOB_PARAM_DISABLE_DELEGATION) != std::string::npos)?true:false;
    bool allocate_after_resolution = 
        (params.find(JOB_PARAM_ALLOCATE_IN_DIFFERENT_CYCLE) != std::string::npos)?false:true;
    
    // Get All the jobs to allocate
    agents::dao::StringArray  jobs;
    jobDao.getSubmittedJobs(jobs,MAX_JOB_CHUNKS);

    m_log_debug("Got " << jobs.size() << " jobs in Submitted State");
            
    // For Each Job
    StringArray::iterator j_it;
    for(j_it = jobs.begin(); j_it != jobs.end(); ++j_it){
        // Allocake Job To channel
        allocateJob((*j_it),disable_deleg,allocate_after_resolution);
    } //  End for each job

    // Update Job States
    this->updateJobs(jobs);
}


/*
 * allocateJob
 *
 * Allocate a Job to a Channel
 * @param job_id [IN]
 * @param disable_delegation [IN]
 * @param allocate_after_resolution [IN]
 */
void Allocate::allocateJob(
    const std::string&  job_id, 
    bool                disable_delegation, 
    bool                allocate_after_resolution) /*throw (DAOException, LogicError)*/{
    
    // Get Job DAO
    JobDAO& jobDao = jobDAO();
            
    m_log_debug("Perform allocation for Job [" << job_id << "]");
    
    StringArray files;
    try{
        // Get Job. Lock is already acquired
        scoped_ptr<model::Job> job(jobDao.get(job_id));

        // Get File DAO
        FileDAO& fileDao = fileDAO();

        // Get all the File for the given Job
        fileDao.getByJobId(job->id(),files);
        m_log_debug("Got " << files.size() << " Files");

        // Get Proxy Certificate
        std::string proxy_file;
        try{
            getProxyForJob(*job,proxy_file);
        } catch(const CredServiceException& exc){
            m_log_warn("Failed to get Proxy Certificate for Job [" << job_id << "]: Reason is " << exc.what());
            failAllFiles(files,Error::SECURITY,Error::S_AGENT,Error::P_ALLOCATION,exc.what());
            return;
        }
        
        bool redo_job_allocation = false;
        unsigned int count = 0;
        do{
            // Iterate Max once
            ++count;
            // Check Precondition
            if(model::Job::S_SUBMITTED != job->state()){
                // Job not in submitted state: ignore it
                break;
            }
            // Check if Job should be canceled
            if(true == job->cancelJob()){
                // Cancel Pending Job
                cancelJob(*job,files,proxy_file);
            } else {
                // Allocate job to a Channel
                bool do_resolution = allocateToChannel(*job,files,proxy_file);
                if(true == do_resolution){ 
                    // Resolve the whole Job. Job will then be allocated 
                    // during next iteration
                    resolveJob(*job,files,proxy_file);
                    // Redo job allocation, unless the --allocate-in-different-cycle 
                    // parameter is set 
                    // Note: this parameter should be used only for  tests
                    redo_job_allocation = allocate_after_resolution; 
                }
            } // end if cancelJob

            // Update Job
            jobDao.update(*job);
        } while ((redo_job_allocation == true) && (count <= 1));
        
        if(true == disable_delegation){
            // Set the disable delegation flag on the job internal params
            std::string internal_params = job->internalParams();
            if(internal_params.find(JOB_PARAM_DISABLE_DELEGATION) 
                                    == std::string::npos){
                job->internalParams = internal_params + " " + JOB_PARAM_DISABLE_DELEGATION;
            }
        }
        // Update Job
        jobDao.update(*job);
    } catch(const LogicError& exc) {
        m_log_error("LogicError in allocating Job [" << job_id << "]: " << exc.what());
        if(false == files.empty()){
            failAllFiles(files,Error::CONSISTENCY,Error::S_AGENT,Error::P_ALLOCATION,exc.what());
        }
    } catch(const DAOException& exc){
        m_log_error("Failed to Get Channel Name (DAOException): " << exc.what());
        throw;
    }        
}


/*
 * Allocate a Job to a Channel
 *
 * Allocate a Job to a channel. Return true if the Job need resolution
 */
bool Allocate::allocateToChannel(
    Job&                job, 
    const StringArray&  files, 
    const std::string&  proxy_file) /*throw (LogicError,DAOException)*/{
        
    bool do_resolution = false;
                    
    // Get FileDAO
    FileDAO& fileDao = fileDAO();
                                     
    // Set Proxy Env                                     
    UserProxyEnv cert(proxy_file);
                
    // For each file
    StringArray::const_iterator f_it;
    bool allocation_failed = false;
    for(f_it = files.begin(); f_it != files.end() && (!allocation_failed); ++f_it){
        // Get the File
        scoped_ptr<model::File> file(fileDao.get((*f_it)));
        m_log_debug("Allocating File [" << file->id() << "]");
        // Check File Status
        if(file->state() != model::File::S_SUBMITTED){
            // File is not submitted: don't do anything
            m_log_debug("File [" << file->id() << "] is not Submitted");
            continue;
        }
        // Get File SURLs
        const std::string& source_surl = file->sourceSurl();
        const std::string& dest_surl   = file->destSurl();
            
        // If Not Set
        if(source_surl.empty() || dest_surl.empty()){
            // Still need to Resolve file names from Logical table
            m_log_debug("Logical File Names not yet resolved");
            do_resolution = true;
            break;
        } 

        // Select the channel matching the Source and Destination
        bool channel_found = false;
        try{
            channel_found = selectChannel(job,*file);
        } catch(const ExecuteException& exc){
            m_log_error("Allocation Failed for File [" << file->id() << "] - Job [" << job.id() << "]");
            // Signal Error
            failFile(*file,Error::CONFIGURATION,Error::S_AGENT,Error::P_ALLOCATION,exc.what());
            continue;
        }
        if(true == channel_found){
            m_log_debug("Found channel " << job.channelName());
            try {
                // Update file status
                FileFSM f_fsm(*file,job);
                f_fsm.onEventAllocate();
                // Update File
                fileDao.update(*file);
            } catch(const LogicError& exc){
                m_log_error("LogicError allocating File [" << (*f_it) << "] as ready. Reason: " << exc.what());
                failFile(*file,Error::CONSISTENCY,Error::S_AGENT,Error::P_ALLOCATION,exc.what());
            }
        } else {
            allocation_failed = true;
            // File should be already failed by selectChannel
            if(file->state() !=  model::File::S_FAILED){
                // Signal Error (this code should never be executed!)
                failFile(*file,Error::INTERNAL_ERROR,Error::S_AGENT,Error::P_ALLOCATION,
                    "Channel Allocation failed but no reason is available");
            }
        }
    } //  End for each file
    
    if (allocation_failed) {
        for(f_it = files.begin(); f_it != files.end(); ++f_it) {
            scoped_ptr<model::File> file(fileDao.get((*f_it)));
            if(file->state() !=  model::File::S_FAILED) {
                failFile(*file,Error::CONSISTENCY, Error::S_AGENT,Error::P_ALLOCATION,"Allocation failed for another file in the same job.");
            }
        }
    }
    
    return do_resolution;
}

/*
 * selectChannel
 *
 * Get the Channel that correspond to the given source and destination
 */
bool Allocate::selectChannel(model::Job& job, model::File& file) /*throw (DAOException, LogicError, ExecuteException)*/{
    
    bool channel_found = false;
    
    // Get File SURLs
    const std::string& src_surl  = file.sourceSurl();
    const std::string& dest_surl = file.destSurl();
    
    // Get Channel DAO
    ChannelDAO& channelDao = channelDAO();
        
    try{
        // Extract Domain From SURLs
        std::pair<std::string,std::string> src_se = 
            getSiteFromSURL(src_surl ,ActionsConfig::instance().unknownSourceEnabled());
        std::pair<std::string,std::string> dst_se = 
            getSiteFromSURL(dest_surl,ActionsConfig::instance().unknownDestEnabled());
        
        // Check if SEs are blacklisted
        std::pair<bool,std::string> blacklist_check = 
            channelDao.isBlacklisted(src_se.second,src_se.first);
        if(true == blacklist_check.first){
            std::string error_reason = (std::string)"Channel allocation failed "
                "since Source SE is blacklisted (Reason: " 
                + blacklist_check.second + ")";
            m_log_error(error_reason);
            throw ExecuteException(error_reason.c_str());
            
        }
        blacklist_check = 
            channelDao.isBlacklisted(dst_se.second,dst_se.first);
        if(true == blacklist_check.first){
            std::string error_reason = (std::string)"Channel allocation failed "
                "since Destination SE is blacklisted (Reason:" 
                + blacklist_check.second + ")";
            m_log_error(error_reason);
            throw ExecuteException(error_reason.c_str());
        }
        // The Channel Instance
        shared_ptr<model::Channel> channel;
        
        // Check if the channel is already recorded in cache
        ChannelCache& cache = ActionsConfig::instance().channelCache();
        try{
            channel = cache.getFromSites(src_se.first,dst_se.first);    
            m_log_debug("Channel <" << channel->name() << "> found in cache");
        } catch(const RuntimeError& exc){
            // No channel Found in cache
            m_log_debug("No channel found in cache");
            // Check if Channel is missing
            bool is_missing = cache.isChannelMissing(src_se.first,dst_se.first);
            if(true == is_missing){
                std::string error_reason = (std::string)"No Channel found, "
                    "Channel closed for your VO or "
                    "VO not authorized for transferring between " + src_se.first 
                    + " and " + dst_se.first;
                m_log_error(error_reason);
                throw ExecuteException(error_reason.c_str());
            }
        }
        if(0 == channel.get()){
            
            // Get the channel that matches the requested domains
            std::string channel_name = this->getBySites(src_se.first,dst_se.first);
            if(true == channel_name.empty()){
                std::string error_reason = (std::string)"No Channel found, "
                    "Channel closed for your VO or "
                    "VO not authorized for transferring between " + src_se.first 
                    + " and " + dst_se.first;
                m_log_error(error_reason);
                // Record missing channel
                cache.rememberMissingChannel(src_se.first,dst_se.first);
                throw ExecuteException(error_reason.c_str());
            }
            try {
                // Get Channel Instance
                channel.reset(channelDao.get(channel_name));
                
                // Check Channel State
                if(!((channel->state == model::Channel::S_ACTIVE) ||
                     (channel->state == model::Channel::S_INACTIVE) ||
                     (channel->state == model::Channel::S_HALTED))){
                    m_log_error("Cannot Allocate to Channel <" << channel_name << ">. State not Active, Inactive or Halted");
                    std::string error_reason = (std::string)"Channel " + channel_name + " is Stopped";
                    throw ExecuteException(error_reason.c_str());
                }
                m_log_debug("Channel <" << channel_name << "> State is " << channel->state());
                // Insert Channel in cache
                cache.insert(src_se.first,dst_se.first,channel);
            } catch(const DAOException& exc){
                m_log_error("Failed to get Channel Instance");
                std::stringstream error_reason;
                error_reason << "No Channel found for sites " << src_se.first
                    << " - " << dst_se.first;
                throw ExecuteException(error_reason.str());
            }
        }                
        // Channel Found
        channel_found = true;
        m_log_debug("Channel found for sites [" << src_se.first << "] - [" << dst_se.first << "]: " << channel->name());
        
        const std::string& job_channel   = job.channelName();
        const std::string& job_source    = job.source();
        const std::string& job_source_se = job.sourceSE();
        const std::string& job_dest      = job.dest();
        const std::string& job_dest_se   = job.destSE();
        try{
            // Check the job's channel name
            if(job_channel.empty()){
                // Set the channel name on the job
                job.channelName = channel->name();
                m_log_info("Job [" << job.id() << "] Allocated to channel [" << channel->name() << "]");
            } else if(0 != channel->name().compare(job_channel)){
                std::stringstream reason;
                reason << "Job Consistency Error: File [" << file.id() 
                    << "] of job [" << job.id()
                    << "] should be assigned a channel (" << channel->name() 
                    << ") that differs from the job's one (" << job_channel
                    << ")";
                throw ExecuteException(reason.str());
            }
            // Check the source site name on the job
            if(job_source.empty()){
                job.source = src_se.first;
            } else if(0 != src_se.first.compare(job_source)){
                if(0 == src_se.second.compare(job_source)){
                    // the job's source is indeed the job source SE!
                    job.sourceSE = job_source;
                    job.source   = src_se.first;
                } else {
                    std::stringstream reason;
                    reason << "Job Consistency Error: File [" << file.id() 
                        << "] of job [" << job.id()
                        << "] is from a site (" << src_se.first
                        << ") that differs from the job's one (" << job_source
                        << ")";
                    throw ExecuteException(reason.str());
                }
            }
            // Check the source SE name on the job
            if(job_source_se.empty()){
                // Store the source SE name on the job
                job.sourceSE = src_se.second;
            } else if(0 != src_se.second.compare(job_source_se)){
                // Invalid Channel
                std::stringstream reason;
                reason << "Job Consistency Error: File [" << file.id() 
                    << "] of job [" << job.id()
                    << "] should be transferred from an SE (" << src_se.second
                    << ") that differs from the job's one (" << job_source_se
                    << ")";
                throw ExecuteException(reason.str());
            }
            if(job_dest.empty()){
                // Store the destination site name on the job
                job.dest = dst_se.first;
            } else if(0 != dst_se.first.compare(job_dest)){
                if(0 == dst_se.second.compare(job_dest)){
                    // the job's destination is indeed the job destination SE!
                    job.destSE = job_dest;
                    job.dest   = dst_se.first;
                } else {
                    std::stringstream reason;
                    reason << "Job Consistency Error: File [" << file.id() 
                        << "] of job [" << job.id()
                        << "] should be transferred to a site (" << dst_se.first
                        << ") that differs from the job's one (" << job_dest
                        << ")";
                    throw ExecuteException(reason.str());
                }
            }
            if(job.destSE().empty()){
                // Store the destination SE name on the job
                job.destSE = dst_se.second;
            } else if(0 != dst_se.second.compare(job_dest_se)){
                // Invalid Channel
                std::stringstream reason;
                reason << "Job Consistency Error: File [" << file.id() 
                    << "] of job [" << job.id()
                    << "] should be transferred to an SE (" << dst_se.second
                    << ") that differs from the job's one (" << job_dest_se
                    << ")";
                throw ExecuteException(reason.str());
            }
        } catch(const ExecuteException& exc){
            // Invalid Channel
            channel_found = false;
            m_log_error(exc.what());
            // Signal Error
            failFile(file,Error::CONSISTENCY, Error::S_AGENT,Error::P_ALLOCATION,exc.what());
        }
    } catch(const DAOException& exc){
        m_log_error("Failed to Get Channel Name (DAOException): " << exc.what());
        throw;
    } catch(const ExecuteException& exc){
        m_log_error("Failed to Get Channel Name: " << exc.what());
        throw;
    }
    
    return channel_found;
}

/*
 * getBySites
 *
 * return the channel name that served two sites, iterate through all the possibilities 
 */
std::string Allocate::getBySites(const std::string& source_site, const std::string& dest_site) /*throw (LogicError,ExecuteException)*/{

    std::string channel_name;
    std::vector<std::string> src_groups, dst_groups;
    std::vector<std::string>::iterator it, sit, dit;
    
    // Get Channel DAO
    ChannelDAO& channelDao = channelDAO();
    
    // First of all, check the dedicated channel
    channel_name = channelDao.getBySites(source_site,dest_site);
    if(true == channel_name.empty()){
        m_log_debug("No dedicated channel found between <" << source_site << "> and <" << dest_site << ">");
    } else {
        return channel_name;
    }
    // No dedicated channel found, get the source site's groups 
    this->getSiteGroups(source_site,src_groups);
    if(false == src_groups.empty()){
        // For each source group
        for(it = src_groups.begin(); it != src_groups.end(); ++it){
            channel_name = channelDao.getBySites((*it),dest_site);
            if(true == channel_name.empty()){
                m_log_debug("No cloud channel found between <" << (*it) << "> and <" << dest_site << ">");
            } else {
                return channel_name;
            }
        }
    } else {
        m_log_debug("Source site <" << source_site << "> doesn't belong to any group");
    }
    // No channel found yet, get the destination site's groups
    this->getSiteGroups(dest_site,dst_groups);
    if(false == dst_groups.empty()){
        // For each destination group
        for(it = dst_groups.begin(); it != dst_groups.end(); ++it){
            channel_name = channelDao.getBySites(source_site,(*it));
            if(true == channel_name.empty()){
                m_log_debug("No cloud channel found between <" << source_site << "> and <" << (*it) << ">");
            } else {
                return channel_name;
            }
        }
    } else {
        m_log_debug("Destination site <" << dest_site << "> doesn't belong to any group");
    }
    // For each source-dest groups combination
    for(dit = dst_groups.begin(); dit != dst_groups.end(); ++dit){
        for(sit = src_groups.begin(); sit != src_groups.end(); ++sit){
            channel_name = channelDao.getBySites((*sit),(*dit));
            if(true == channel_name.empty()){
                m_log_debug("No cloud channel found between <" << (*sit) << "> and <" << (*dit) << ">");
            } else {
                return channel_name;
            }
        }    
    }    
    // Check the catch-all channels, first *-destination
    channel_name = channelDao.getBySites(WILDCHAR_SITE,dest_site);
    if(true == channel_name.empty()){
        m_log_debug("No catch-all channel found between <" << WILDCHAR_SITE << "> and <" << dest_site << ">");
    } else {
        return channel_name;
    }
    // Check the catch-all channels, then sourec-*
    channel_name = channelDao.getBySites(source_site,WILDCHAR_SITE);
    if(true == channel_name.empty()){
        m_log_debug("No catch-all channel found between <" << source_site << "> and <" << WILDCHAR_SITE << ">");
    } else {
        return channel_name;
    }
    // For each destination group
    for(it = dst_groups.begin(); it != dst_groups.end(); ++it){
        channel_name = channelDao.getBySites(WILDCHAR_SITE,(*it));
        if(true == channel_name.empty()){
            m_log_debug("No catch-all channel found between <" << WILDCHAR_SITE << "> and <" << (*it) << ">");
        } else {
            return channel_name;
        }
    }
    // For each source group
    for(it = src_groups.begin(); it != src_groups.end(); ++it){
        channel_name = channelDao.getBySites((*it),WILDCHAR_SITE);
        if(true == channel_name.empty()){
            m_log_debug("No catch-all  channel found between <" << (*it) << "> and <" << WILDCHAR_SITE << ">");
        } else {
            return channel_name;
        }
    }
    // last chance, the *-* channel
    channel_name = channelDao.getBySites(WILDCHAR_SITE,WILDCHAR_SITE);
    if(true == channel_name.empty()){
        m_log_debug("No catch-all channel found between <" << WILDCHAR_SITE << "> and <" << WILDCHAR_SITE << ">");
    }
    return channel_name;
}

/*
 * getSiteGroups
 *
 * return the list of groups which a site belongs to
 */
void Allocate::getSiteGroups(const std::string& site, std::vector<std::string>& groups) /*throw (LogicError,ExecuteException)*/{
    // Get Channel Cache
    ChannelCache& cache = ActionsConfig::instance().channelCache();

    try{
        // Check if the grous are already recorded in cache
        cache.getSiteGroups(site,groups);
        m_log_debug(groups.size() << " Group(s) for Site <" << site << "> found in cache");
    } catch(const RuntimeError& exc){
        // No site Found in cache

        // Get Channel DAO
        ChannelDAO& channelDao = channelDAO();
        // Not groups in cache, get them from dao
        channelDao.getSiteGroups(site,groups);
        // Remeber site groups in cache
        cache.rememberSiteGroups(site,groups);
    }    
    m_log_debug("Site <" << site << "> belongs to " << groups.size() << " group(s)");
}

    
/*
 * getSiteFromSURL
 *
 * get the name of the site and the SE where the SURL is located
 */
std::pair<std::string,std::string> Allocate::getSiteFromSURL(const std::string& surl, bool enable_unknown) /*throw (LogicError,ExecuteException)*/{

    std::pair<std::string,std::string> result;
    
    std::string  hostname;
    std::string  protocol;
    unsigned int port = 0;
    // Extract Hostname
    agents::parse_url(surl,hostname,port,protocol);
    m_log_debug("Hostname for SURL " << surl << " is <" << protocol << ">://<" << hostname << ">:<" << port << ">");
    
    // Get ServiceDiscovery 
    agents::sd::SDConfig& sd_config = agents::sd::SDConfig::instance();

    // Find the Site of the SE that owns the SURL
    std::string site;
    std::string service_type;
    
    if((protocol == SRM_PROTOCOL) || (protocol == SFN_PROTOCOL)){ 
        service_type = sd_config.srm();
    } else if(protocol == GSIFTP_PROTOCOL){ 
        service_type = sd_config.gridftp();
    } else {
        // Unsupported Protocol Type
        m_log_error("Unsupported Protocol Type (" << protocol << ") for SURL " << surl);
        std::string error_reason = (std::string)"Unknwon SURL/TURL protocol: " + protocol;
        throw ExecuteException(error_reason.c_str());
    }
    
    // Find Service
    scoped_ptr<agents::sd::Service> s(agents::sd::get_service(hostname,service_type,voName()));
    if(0 != s.get()){
        // Get service site
        site = s->site;
    } else {
        if(protocol == GSIFTP_PROTOCOL){ 
            // Check For Legacy GRIDFTP Service Type
            s.reset(agents::sd::get_service(hostname,sd_config.legacyGridftp(),voName()));
        }
        if(0 != s.get()){
            // Get service site
            site = s->site;
        } else {
            if(true == enable_unknown){
                m_log_debug("Cannot Find SRM for host " << hostname);
                m_log_info("Associating Unknown Site (" << UNKNOWN_SITE << ")");
                site = UNKNOWN_SITE;
            } else {
                m_log_error("Cannot Find Site for host " << hostname);
                std::string error_reason = (std::string)"No site found for host " + hostname;
                throw ExecuteException(error_reason.c_str());
            }
        }
    }
    m_log_debug("Site for SURL " << surl << " is " << site);
      
    result.first  = site;
    result.second = hostname;
    
    return result;    
}

/*
 * cancel Job
 *
 * Cancel the Submitted Job
 */
void Allocate::cancelJob(const model::Job& job, 
                         const StringArray& files, 
                         const std::string& proxy_file) /*throw (DAOException, LogicError)*/{
    
    // Get Channel DAO
    FileDAO& fileDao = fileDAO();
    
    // Fo each file, cancel it
    StringArray::const_iterator f_it;
    for(f_it = files.begin(); f_it != files.end(); ++f_it){
        m_log_debug("Cancelling File [" << (*f_it) << "]");
        // Get the File. 
        scoped_ptr<model::File> f(fileDao.get((*f_it)));
        // Issue a Canceled Event
        FileFSM f_fsm(*f,job);
        f_fsm.onEventCanceled();
        // Update File
        fileDao.update(*f);
        m_log_info("File [" << f->id() << "] has been canceled");
    }
}
                          
/*
 * resolve Job
 *
 * Do Catalog Resolution for the Job
 */
void Allocate::resolveJob(      model::Job& job, 
                          const StringArray& files, 
                          const std::string& proxy_file) /*throw (DAOException, LogicError)*/{
    
    bool catalog_enabled = ActionsConfig::instance().catalogEnabled();
    if(true == catalog_enabled){
        try{
            // Check if the Catalog Service Factory is registered
            CatalogServiceFactory::instance();
        } catch(const LogicError&){
            m_log_warn("Job [" << job.id() << "] requires catalog resolution but no CatalogService plugin is registered for the agent");
            catalog_enabled = false;
        }
    }
    if(false == catalog_enabled){
        // Catalog Resolution is Disabled
        failAllFiles(files,Error::CONFIGURATION,Error::S_AGENT,Error::P_CATALOG_RESOLUTION,"Resolution disabled for this VO");
        return;
    }    
    if(job.internalParams().find(JOB_PARAM_SKIP_RESOLUTION) != std::string::npos){
        // Resolution is Disabled
        failAllFiles(files,Error::CONFIGURATION,Error::S_AGENT,Error::P_CATALOG_RESOLUTION,"Resolution disabled for this Job");
        return;
    }
       
    std::pair<std::string,std::string> source_catalog_data, dest_catalog_data;

    // Set the Delegated Credentail Object
    {
    UserProxyEnv cert(proxy_file);
        
    if((true == job.srcCatalog().empty()) || 
       (true == job.srcCatalogType().empty())){
        // Get Source Catalog endpoint
        m_log_debug("Getting Source Catalog Endpoint for Job (Source: " << job.source() << ")");
        source_catalog_data = getCatalogEndpoint(job.source(),voName());
        if(source_catalog_data.first.empty()){
            m_log_warn("Source catalog for Job [" << job.id() << "] (Source: " << job.source() << ") not found");
            failAllFiles(files,Error::CONFIGURATION,Error::S_SOURCE,Error::P_CATALOG_RESOLUTION,"Source Catalog not found");
            return;
        }
        m_log_debug("Source Catalog Endpoint is " << source_catalog_data.first);
        // Store Catalog Endpoint as Job Properties
        job.srcCatalog     = source_catalog_data.first;
        job.srcCatalogType = source_catalog_data.second;
    } else {
        // Get Catalog Endpoint from the Job Properties
        source_catalog_data.first  = job.srcCatalog();
        source_catalog_data.second = job.srcCatalogType();
    }
    if((true == job.destCatalog().empty()) || 
       (true == job.destCatalogType().empty())){
        // Get Catalog endpoint
        m_log_debug("Getting Destination Catalog Endpoint for Job (Dest: " << job.dest() << ")");
        dest_catalog_data = getCatalogEndpoint(job.dest(),voName());
        if(dest_catalog_data.first.empty()){
            m_log_warn("Source catalog for Job [" << job.id() << "] (Source: " << job.source() << ") not found");
            failAllFiles(files,Error::CONFIGURATION,Error::S_DESTINATION,Error::P_CATALOG_RESOLUTION,"Destination Catalog not found");
            return;
        }
        m_log_debug("Destination Catalog Endpoint is " << dest_catalog_data.first);
        // Store Catalog Endpoint as Job Properties
        job.destCatalog     = dest_catalog_data.first;
        job.destCatalogType = dest_catalog_data.second;
    } else {
        // Get Catalog Endpoint from the Job Properties
        dest_catalog_data.first  = job.destCatalog();
        dest_catalog_data.second = job.destCatalogType();
    }
    } // End UserProxy Env
        
    // Get File DAO
    FileDAO& fileDao = fileDAO();

    // For each file, check if the source SURL is set
    StringArray source_names;
    StringArray dest_names;
    StringArray need_source_resolution;
    StringArray need_dest_resolution;
    StringArray::const_iterator f_it;
    for(f_it = files.begin(); f_it != files.end(); ++f_it){
        // Get the File. 
        scoped_ptr<model::File> f(fileDao.get((*f_it)));
        // Check File Status
        if(f->state() != model::File::S_SUBMITTED){
            // File not submitted: don't do anything
            m_log_debug("File [" << f->id() << "] not in Submitted state");
            continue;
        }
        // Check if SURLs are filled
        if(true == f->logical().empty()){
            // Check Precondition
            if(f->sourceSurl().empty() || f->destSurl().empty()){
                m_log_warn("Cannot Resolve File [" << f->id() << "]. Logical Name missing");
                // Signal Error
                failFile(f->id(),Error::CONSISTENCY,Error::S_AGENT,Error::P_ALLOCATION,"Logical Name missing");
            } 
        } else {
            // Check if source file resolution is needed
            if(f->sourceSurl().empty()){
                m_log_debug("File [" << f->id() << "] need source name resolution");
                need_source_resolution.push_back(f->id());
                source_names.push_back(f->logical());
            }
            // Check if dest file resolution is needed
            if(f->destSurl().empty()){
                m_log_debug("File [" << f->id() << "] need dest name resolution");
                need_dest_resolution.push_back(f->id());
                dest_names.push_back(f->logical());
            }                
        }
    } // End For each file

    // Do the Name Resolution
    try{
        // Check Permission on Source Names
        checkPermissions(need_source_resolution,source_names,source_catalog_data.first,source_catalog_data.second,proxy_file);
        m_log_debug("Permission Checked");
        
        // Resolve Source Names
        resolveSources(need_source_resolution,source_names,job.source(),source_catalog_data.first,source_catalog_data.second,proxy_file);
        m_log_debug("Source Names resolved");
            
        // Create Destination SURL    
        StringArray dest_surls;
        StringArray dest_id_resolved;
        StringArray dest_resolved;
        setDestinations(need_dest_resolution,job.dest(), job.storageClass(), dest_id_resolved, dest_resolved, dest_surls);
        m_log_debug("Dest Names resolved");
        
        // Check that Destinations don't already exist
        checkDestinations(dest_id_resolved, dest_resolved, dest_surls, source_catalog_data.first,source_catalog_data.second,proxy_file);
        m_log_debug("Dest Names checked");

        m_log_info("Names for Job [" << job.id() << "] resolved");
    } catch(const RuntimeError& exc){
        m_log_error("An Error Occurred in Resolving File Names for Job [" << job.id() << "]: " << exc.what());
        throw;
    } catch(const LogicError& exc){
        m_log_error("A LogicError Occurred in Resolving File Names for Job [" << job.id() << "]: " << exc.what());
        throw;
    }          
}


/*
 * checkPermissions
 *
 * Check if the users is authorized to replicate the files
 */
void Allocate::checkPermissions(      StringArray& file_ids, 
                                      StringArray& logical_names, 
                                const std::string& catalog_endpoint, 
                                const std::string& catalog_type, 
                                const std::string& proxy_file) /*throw ()*/{
    // Check Preconditions
    if(true == logical_names.empty()){
        return;
    }
    
    try{
        // Create a Catalog Service Instance
        scoped_ptr<CatalogService> catalogService(
            CatalogServiceFactory::instance().create(catalog_endpoint,catalog_type,proxy_file));
    
        // Check Permission to replicate the requested LFNs
        CatalogService::CatalogResult result;
        catalogService->checkPermissions(logical_names,result);
        if(CatalogService::CatalogResult::R_SUCCESS == result.status){
            // Permission Granted
            m_log_debug("Permission granted");
        } else if(CatalogService::CatalogResult::R_FAILED == result.status){
            // Permission denied
            m_log_error("Permission denied. Reason is " << result.reason);
            failAllFiles(file_ids,Error::SECURITY,Error::S_SOURCE,Error::P_CATALOG_RESOLUTION,result.reason);
            // Remove all the files from the vectors
            file_ids.clear();
            logical_names.clear();
        } else if(CatalogService::CatalogResult::R_SOME_FAILURES == result.status){
            if (!result.reason.empty()){
                m_log_error("Permission denied on some files. Reason is " << result.reason);
            } else{
                m_log_error("Permission denied on some files. Reason is Permission denied for some files");
            }
            // Some file Failed: For each file, handle failures on just some files. 
            CatalogService::StringPairArray::const_iterator sp_it;
            StringArray::iterator i_it,l_it;
            for(sp_it = result.failures.begin(); sp_it != result.failures.end(); ++sp_it){
                const std::string& logic_name = (*sp_it).first;
                const std::string& reason     = (*sp_it).second;
                // Find the related logical name
                l_it = std::find(logical_names.begin(),logical_names.end(),logic_name);
                m_log_error("Permission denied for file <" << logic_name << ">: " << reason);
                if(l_it != logical_names.end()){
                    unsigned long dist = std::distance(logical_names.begin(),l_it);
                    if(dist > file_ids.size()){
                        throw LogicError("Inconsistent <logical_names>-<file_ids> vector sizes");
                    }
                    // Get File Id
                    i_it = (file_ids.begin()) + dist;
                    // Fail File
                    failFile((*i_it),Error::SECURITY,Error::S_SOURCE,Error::P_CATALOG_RESOLUTION,reason);
                    // Remote Failed file from the lists
                    logical_names.erase(l_it);
                    file_ids.erase(i_it);
                } else {
                    m_log_warn("Logical Name <" << logic_name << "> is not one of input files");
                }
            }                
        }
    } catch(const CatalogLogicError& exc){
        // Logic Error
        m_log_warn("Catalog LogicError in Checking Permission. Reason is " << exc.what());
        failAllFiles(file_ids,Error::CATALOG_INTERNAL_ERROR,Error::S_SOURCE,Error::P_CATALOG_RESOLUTION,exc.what());
        // Remove all the files from the vectors
        file_ids.clear();
        logical_names.clear();
    } catch(const CatalogRemoteError& exc){
        // Remote Error
        m_log_warn("RemoteError in Checking Permission. Reason is " << exc.what());
        failAllFiles(file_ids,Error::CONNECTION,Error::S_SOURCE,Error::P_CATALOG_RESOLUTION,exc.what());
        // Remove all the files from the vectors
        file_ids.clear();
        logical_names.clear();
    } catch(const LogicError& exc){
        // Logic Error
        m_log_warn("LogicError in Checking Permission. Reason is " << exc.what());
        failAllFiles(file_ids,Error::CONSISTENCY,Error::S_AGENT,Error::P_CATALOG_RESOLUTION,exc.what());
        // Remove all the files from the vectors
        file_ids.clear();
        logical_names.clear();
    } catch(const RuntimeError& exc){
        // Logic Error
        m_log_warn("RuntimeError in Checking Permission. Reason is " << exc.what());
        failAllFiles(file_ids,Error::INTERNAL_ERROR,Error::S_AGENT,Error::P_CATALOG_RESOLUTION,exc.what());
        // Remove all the files from the vectors
        file_ids.clear();
        logical_names.clear();
    }
}

/*
 * resolveSources
 *
 * Resolve the source Logical Name into physical ones
 */
void Allocate::resolveSources(const StringArray& files,const StringArray& logical_names, const std::string& source, const std::string& catalog_endpoint, const std::string& catalog_type, const std::string& proxy_file) /*throw (DAOException,LogicError)*/{
    // Check Preconditions
    if(files.size() != logical_names.size()){
        m_log_warn("Logical Names - File Id list size mismatch");
        failAllFiles(files,Error::CONSISTENCY,Error::S_AGENT,Error::P_CATALOG_RESOLUTION,"Logical Names - File Id list size mismatch");
        return;
    }
    if(true == files.empty()){
        return;
    }
    
    // Get File DAO
    FileDAO& fileDao = fileDAO();

    try{
        // Create a Catalog Service Instance
        scoped_ptr<CatalogService> catalogService(
            CatalogServiceFactory::instance().create(catalog_endpoint,catalog_type,proxy_file));
        
        // get the Replica for the requested LFNs
        std::vector<CatalogService::StringArray> surls;
        CatalogService::CatalogResult result;
        catalogService->listSurls(logical_names,source, surls,result);
        if(CatalogService::CatalogResult::R_FAILED == result.status){
            // All Files Failed
            std::string reason = (std::string)"Cannot Retrieve Surls from "\
                "Catalog. Reason is " + result.reason;
            m_log_warn(reason);
            failAllFiles(files,Error::CATALOG,Error::S_AGENT,Error::P_CATALOG_RESOLUTION,reason.c_str());
        } else {
            // For each file
            StringArray::const_iterator it;
            StringArray::const_iterator l_it = logical_names.begin();
            std::vector<CatalogService::StringArray>::iterator s_it = surls.begin();
            for(it = files.begin(); it != files.end(); ++it,++l_it,++s_it){
                // Get the File
                scoped_ptr<model::File> f(fileDao.get((*it)));
                if(*l_it != f->logical()){
                    m_log_warn("File [" << f->id() << "] : id - logical name mismatch");
                    failFile(f->id(),Error::CONSISTENCY,Error::S_AGENT,Error::P_CATALOG_RESOLUTION,"id - logical name mismatch");
                    continue;
                } 
                if(CatalogService::CatalogResult::R_SOME_FAILURES == result.status){
                    // Check if the logical name is failed
                    CatalogService::StringPairArray::iterator failed_it;
                    failed_it = std::find_if(result.failures.begin(), result.failures.end(),
                        agents::compare_first(f->logical()));
                    if(failed_it != result.failures.end()){
                        m_log_warn("Name Resolution for File [" << f->id() << "] : id failed: " << (*failed_it).second);
                        failFile(f->id(),Error::CATALOG,Error::S_SOURCE,Error::P_CATALOG_RESOLUTION,(*failed_it).second.c_str());
                        result.failures.erase(failed_it);
                        continue;
                    }
                } 
                // Get Surl
                StringArray::const_iterator found = findSurl(source,(*s_it));
                if(found == (*s_it).end()){
                    // No SURL found on the given SE/Site
                    m_log_warn("No SURL found on the given SE/Site for File [" << f->id() << "]");
                    failFile(f->id(),Error::INVALID_PATH,Error::S_SOURCE,Error::P_CATALOG_RESOLUTION,"No Source Replica available");
                    continue;
                } 
                // Store the Source SURL on the File instance
                f->sourceSurl = *found;

                m_log_info("Source SURL for File <" << f->logical() << "> is <" << f->sourceSurl() << ">");
                // Update File
                fileDao.update(*f);
            }
        }
    } catch(const CatalogLogicError& exc){
        // Catalog Error
        std::string reason = (std::string)"Cannot Retrieve Surls from Catalog (Logic Error). Reason is " + exc.what();
        m_log_error(reason);
        failAllFiles(files,Error::CATALOG_INTERNAL_ERROR,Error::S_SOURCE,Error::P_CATALOG_RESOLUTION,reason);
    } catch(const CatalogRemoteError& exc){
        std::string reason = (std::string)"Cannot Contact Remote Catalog. Reason is " + exc.what();
        m_log_warn(reason);
        failAllFiles(files,Error::CONNECTION,Error::S_SOURCE,Error::P_CATALOG_RESOLUTION,reason);
    } catch(const LogicError& exc){
        m_log_warn("LogicError in retrieving SURLs. Reason is " << exc.what());
        failAllFiles(files,Error::CONSISTENCY,Error::S_SOURCE,Error::P_CATALOG_RESOLUTION,exc.what());
    }
}

/*
 * setDestName
 *
 * Generate the Destination Physical Name
 */
void Allocate::setDestinations(const StringArray& files, 
                               const std::string& destination, 
                               const std::string& storage_class,
                                     StringArray& dest_id_resolved, 
                                     StringArray& dest_resolved, 
                                     StringArray& dest_surls) /*throw (DAOException,LogicError)*/{
  
    // Check if Source SURL is already filled                
    if(true == files.empty()){
        return;
    }
    // Get File DAO
    FileDAO& fileDao = fileDAO();

    try{
        const std::string& srm_service_type = agents::sd::SDConfig::instance().srm();
        // Create a Catalog Service Instance
        scoped_ptr<agents::sd::Service> service(agents::sd::get_service(destination,srm_service_type,voName()));
        if(0 == service.get()){
            // No SE on the given Site
            std::stringstream reason; 
            reason << "No SE registered on the given source (" << destination << ")";
            m_log_error(reason.str());
            failAllFiles(files,Error::CONFIGURATION,Error::S_DESTINATION,Error::P_CATALOG_RESOLUTION,reason.str());
            return;
        }
        // Get SE Path
        std::string se_path = agents::sd::get_se_path(*service,voName(),storage_class);
        if(true == se_path.empty()){
            // No SE MountPoint for the given service
            std::stringstream reason; 
            reason << "No SE Path for service (" << service->name << ") - VO" << voName();
            m_log_error(reason.str());
            failAllFiles(files,Error::CONFIGURATION,Error::S_DESTINATION,Error::P_CATALOG_RESOLUTION,reason.str());
            return;
        }
    
        // Use the NameGeneration interface
        scoped_ptr<NameGeneration> name_gen(NameGenerationFactory::instance().create());
        
        // For each file
        std::string surl;
        StringArray::const_iterator it;
        for(it = files.begin(); it != files.end(); ++it){
            // Get the File. Lock already acquired
            scoped_ptr<model::File> f(fileDao.get((*it)));
            try{
                // Generate the destination SURL
                f->destSurl = name_gen->generate(f->logical(),
                                            service->endpoint,se_path,storage_class);
                // Store Properties
                dest_id_resolved.push_back(f->id());
                dest_resolved.push_back(f->logical());
                dest_surls.push_back(f->destSurl());
                m_log_info("Destination SURL for File <" << f->logical() << "> is <" << f->destSurl() << ">");
                // Update File
                fileDao.update(*f);
            } catch(const NameGenerationException& exc){
                m_log_warn("Failed to Generate Destination SURL for File [" << f->logical() << "]: " << exc.what());
                failFile(f->id(),Error::INTERNAL_ERROR,Error::S_DESTINATION,Error::P_CATALOG_RESOLUTION,exc.what());
            }
        }
    } catch(const LogicError& exc){
        std::string reason = (std::string)"LogicError in generation destination SURLs: " + exc.what();
        m_log_error(reason);
        failAllFiles(files,Error::CONSISTENCY,Error::S_DESTINATION,Error::P_CATALOG_RESOLUTION,reason);
        dest_id_resolved.clear();
        dest_resolved.clear();
        dest_surls.clear();
    }
}

/*
 * checkDestinations
 *
 * Generate the Destination physical Names
 */
void Allocate::checkDestinations(const StringArray& files,
                                 const StringArray& logical_names,
                                 const StringArray& dest_surls, 
                                 const std::string& catalog_endpoint, 
                                 const std::string& catalog_type, 
                                 const std::string& proxy_file) /*throw (DAOException,LogicError)*/{
   
    // Check Preconditions
    if(files.size() != logical_names.size()){
        failAllFiles(files,Error::CONSISTENCY,Error::S_AGENT,Error::P_CATALOG_RESOLUTION,"File identifiers - logical names mismatch");
        return;
    }
    if(files.size() != dest_surls.size()){
        failAllFiles(files,Error::CONSISTENCY,Error::S_AGENT,Error::P_CATALOG_RESOLUTION,"File identifiers - dest surls mismatch");
        return;
    }
    if(files.empty()){
        return;
    }
       
    // List Surls For the requested logical_names
    // Create a Catalog Service Instance
    scoped_ptr<CatalogService> catalogService(
        CatalogServiceFactory::instance().create(catalog_endpoint,catalog_type,proxy_file));
    try{
        // get the Replica for the requested LFNs
        CatalogService::StringPairArray dest_names;
        StringArray::const_iterator l_it = logical_names.begin();
        StringArray::const_iterator d_it = dest_surls.begin();
        for(;l_it != logical_names.end(); ++l_it,++d_it){
            dest_names.push_back(CatalogService::StringPair(*l_it,*d_it));
        }
        // Check Destinations
        CatalogService::CatalogResult result;
        catalogService->checkSurls(dest_names,result);
        if(CatalogService::CatalogResult::R_FAILED == result.status){
            // All the file are Failed
            m_log_warn("Check Destination failed. Reason is " << result.reason);
            std::string reason = (std::string)"Destination Replica already exists: " + result.reason;
            failAllFiles(files,Error::CATALOG,Error::S_DESTINATION,Error::P_CATALOG_RESOLUTION,reason);
        } else if(CatalogService::CatalogResult::R_SOME_FAILURES == result.status){
            // Some file failed: loop for each logical name and check if it failed
            CatalogService::StringArray::const_iterator l_it = logical_names.begin();
            CatalogService::StringArray::const_iterator i_it = files.begin();
            for(; (l_it != logical_names.end()) && 
                  (false == result.failures.empty()); ++l_it, ++i_it){
                // Check if the file is failed
                CatalogService::StringPairArray::iterator failed_it;
                failed_it = std::find_if(result.failures.begin(), result.failures.end(),
                      agents::compare_first(*l_it));
                if(failed_it != result.failures.end()){
                    m_log_warn("Name Resolution for File [" << (*i_it) << "] : id failed: " << (*failed_it).second);
                    failFile((*i_it),Error::INTERNAL_ERROR,Error::S_DESTINATION,Error::P_CATALOG_RESOLUTION,(*failed_it).second.c_str());
                    result.failures.erase(failed_it);
                    continue;
                }
            }
        } else {
            m_log_debug("Check Destination Succeeded");
        }
    } catch(const CatalogLogicError& exc){
        std::string reason = (std::string)"Catalog LogicError in checking if destination Replica already exists: " + exc.what();
        m_log_warn(reason);
        failAllFiles(files,Error::CATALOG_INTERNAL_ERROR,Error::S_DESTINATION,Error::P_CATALOG_RESOLUTION,reason);
    } catch(const CatalogRemoteError& exc){
        // All the file are Failed
        std::string reason = (std::string)"Cannot contact the Catalog for checking destination Replica already exists. Reason is %s: " + exc.what();
        m_log_warn(reason);
        failAllFiles(files,Error::CONNECTION,Error::S_DESTINATION,Error::P_CATALOG_RESOLUTION,reason);
    } catch(const LogicError& exc){
        std::string reason = (std::string)"LogicError in checking if destination Replica already exists: " + exc.what();
        m_log_warn(reason);
        failAllFiles(files,Error::CONSISTENCY,Error::S_DESTINATION,Error::P_CATALOG_RESOLUTION,reason);
    }
}

/*
 * findSurl
 *
 * Find the proper Surl
 */
Allocate::StringArray::const_iterator Allocate::findSurl(const std::string& source, const StringArray& surls) /*throw ()*/{
    
    StringArray::const_iterator result = surls.end();
    try{
        // Get the list of SE endpoints for the given source
        const std::string& srm_service_type = agents::sd::SDConfig::instance().srm();
        scoped_ptr<agents::sd::Service> se(agents::sd::get_service(source,srm_service_type,voName()));
        if(0 == se.get()){
            // No SE found for the given source
            m_log_error("No Site/SE found for the source " << source);
            return surls.end();
        }
        
        m_log_debug("Source SE is " << se->endpoint);
       
        // For each SE endpoint, Find the files that belongs to that SE
        StringArray::const_iterator s_it;
        for(s_it = surls.begin(); s_it != surls.end();++s_it){
            const std::string& surl = (*s_it);
        
            // Get the hostname of the SURL
            std::string  hostname;
            unsigned int port = 0;
            std::string  protocol;
            try {
                agents::parse_url(surl,hostname,port,protocol);
            } catch(const LogicError& exc){
                m_log_warn("Invalid SURL (" << se->endpoint << "). Reason is " << exc.what());
                continue;
            }
            
            // Check if the hostname is the same
            if((hostname == se->hostname) && 
               ((port == 0) || (se->port == 0) || (port == se->port))){
                // SURL found
                m_log_debug("Found SURL " << surl);
                result = s_it;
                break;
            }
        }
    } catch(const LogicError& exc){
        m_log_warn("Logic Error in finding SURL for Site/SE " << source << ". Reason is " << exc.what());
    } catch(const RuntimeError& exc){
        m_log_warn("Runtime Error in finding SURL for Site/SE " << source << ". Reason is " << exc.what());
    }
    return result;
}
