/*
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *  GLite Data Transfer Agents: Allocate Action Class
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *           Paolo Tedesco <paolo.tedesco@cern.ch>
 *  Version info: $Id: Allocate.h,v 1.12 2008/11/13 10:43:55 ptedesco Exp $
 *  Release: $Name:  $
 *
 */
#ifndef GLITE_DATA_TRANSFER_AGENTS_ACTION_VO_ALLOCATE_H_
#define GLITE_DATA_TRANSFER_AGENTS_ACTION_VO_ALLOCATE_H_

#include "glite/data/transfer/agent/action/vo/VOAction.h"
#include "glite/data/transfer/agent/action/ActionFactoryMethod.h"

#include <vector>
#include <string>

namespace glite   {
namespace data    {
namespace transfer{
namespace agent   {
namespace action  {
namespace vo      {

/**
 * The purpose of that Action is to Allocate a Job to Channels
 */
class Allocate : public VOAction {
    /*
     * Declare Action Factory Method
     */
    DECLARE_FACTORY_METHOD( Allocate , "glite:Allocate");

public:
    /**
     * Constructor
     */
    Allocate();

    /**
     * Destructor
     */
    virtual ~Allocate();

protected:

    /**
     * Execute The Action
     */
    virtual void exec(const std::string& params) /*throw (DAOException, LogicError)*/;
    
private:

    typedef std::vector<std::string> StringArray;

    /**
     * Allocate a Job to a Channel
     */
    void allocateJob(const std::string& job_id, bool disable_delegation, 
        bool allocate_after_resolution) /*throw (DAOException, LogicError)*/;

    /**
     * Allocate a Job to a channel. Return true if the Job need resolution
     */
    bool allocateToChannel(model::Job& job, const StringArray& files, 
        const std::string& proxy_file) /*throw (DAOException, LogicError)*/;

    /**
     * Get the Channel that correspond to the given source and destination
     */
    bool selectChannel(model::Job& job,model::File& file) /*throw (DAOException, LogicError, ExecuteException)*/;

    /**
     * Return the channel name that served two sites, iterate through all the possibilities 
     */
    std::string getBySites(const std::string& source_site, const std::string& dest_site) /*throw (LogicError,ExecuteException)*/;

    /**
     * Return the list of groups which a site belongs to
     */
    void getSiteGroups(const std::string& site, std::vector<std::string>& groups) /*throw (LogicError,ExecuteException)*/;

    /**
     * Get the name of the site and the SE where the SURL is located
     */
    std::pair<std::string,std::string> getSiteFromSURL(const std::string& surl, bool enable_unknown) /*throw (LogicError,ExecuteException)*/;

    /**
     * Cancel a Submitted Job
     */
    void cancelJob(const model::Job& job, const StringArray& files, const std::string& proxy_file) /*throw (DAOException,LogicError)*/;

    /**
     * Resolve the Source and Destination SURLs of teh files belonging to the Job
     */
    void resolveJob(model::Job& job, const StringArray& files, const std::string& proxy_file) /*throw (DAOException,LogicError)*/;

    /**
     * Check if the User is authorized to Replicate the file
     */
    void checkPermissions(StringArray& file_ids, StringArray& logical_names, 
        const std::string& catalog_endpoint, const std::string& catalog_type, 
        const std::string& proxy_file) /*throw ()*/;
        
    /**
     * Resolve Source SURLs
     */
    void resolveSources(const StringArray& files,const StringArray& logical_names, 
        const std::string& source, const std::string& catalog_endpoint, 
        const std::string& catalog_type, const std::string& proxy_file) /*throw (DAOException,LogicError)*/;

    /**
     * Generate Destination SURLs
     */
    void setDestinations(const StringArray& files,const std::string& destination, 
        const std::string& storage_class,StringArray& dest_id_resolved, 
        StringArray& dest_resolved, StringArray& dest_surls) /*throw (DAOException,LogicError)*/;
  
    /**
     * Check if the Generated Destination SURLs already exists
     */
    void checkDestinations(const StringArray& files,const StringArray& logical_names,
        const StringArray& dest_surls, const std::string& catalog_endpoint, 
        const std::string& catalog_type, const std::string& proxy_file) /*throw (DAOException,LogicError)*/;

    /**
     * Find the SURL in the given Site/SE
     */
    StringArray::const_iterator findSurl(const std::string& source, 
        const StringArray& surls) /*throw ()*/;

};

} // End namespace vo
} // End namespace action
} // End namespace agent
} // End namespace transfer
} // End namespace data
} // End namespace glite

#endif //GLITE_DATA_TRANSFER_AGENTS_ACTION_VO_ALLOCATE_H_
