/*
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *  GLite Data Transfer Agents: Basic CatalogRetry Strategy Class
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>, Paolo Tedesco <paolo.tedesco@cern.ch>
 *  Version info: $Id: BasicCatalogRetryStrategy.cpp,v 1.8 2008/11/13 10:43:55 ptedesco Exp $
 *  Release: $Name:  $
 *
 */
#include "BasicCatalogRetryStrategy.h"
#include "ActionsConfig.h"

using namespace glite::data::transfer::agent;
using namespace glite::data::transfer::agent::model;
using namespace glite::data::transfer::agent::action;
using namespace glite::data::transfer::agent::action::vo;

using boost::shared_ptr;

namespace {
const char * const BASIC_RETRY_NAME = "tranfser-agent-vo-retry";
}
/*
 * BasicCatalogRetryStrategy
 *
 * Constructor
 */
BasicCatalogRetryStrategy::BasicCatalogRetryStrategy():CatalogRetryStrategy(),
    m_logger(BASIC_RETRY_NAME){
}

/*
 * ~BasicCatalogRetryStrategy
 *
 * Destructor
 */
BasicCatalogRetryStrategy::~BasicCatalogRetryStrategy(){
}

/*
 * processFile
 *
 * Trigger the rescheduling on a file
 */
CatalogRetryStrategy::CatalogRetryResult BasicCatalogRetryStrategy::apply(const model::Job& job,const FileArray& files) /*throw (LogicError,ExecuteException)*/{

    CatalogRetryResult result = CatalogRetryStrategy::WAIT;
    
    m_log_debug("Check if Job [" << job.id() << "] can be retried");
    
    // Get the minimum current catalog failures and the last active time 
    // for the files belonging to the Job
    unsigned long catalog_failures = 0;
    time_t last_active_time = (time_t)-1;
    bool has_remote_error = false;
    FileArray::const_iterator it;
    for(it = files.begin(); it != files.end();++it){
        const model::File& file = *(*it);
        if(0 == catalog_failures){
            catalog_failures = file.catalogFailures();
        } else if(catalog_failures > file.catalogFailures()){
            catalog_failures = file.catalogFailures();
        }
        if((time_t)-1 != last_active_time){
            last_active_time = file.finishTime();
        } else if((time_t)-1 != file.finishTime()){
            // Take the file finish time
            if(file.finishTime() > last_active_time){
                last_active_time = file.finishTime();
            }
        }
        if ((Error::CONNECTION  == file.reasonClass()) ||
            (Error::REMOTE      == file.reasonClass())){
            has_remote_error = true;
        }
    }  
    if(false == has_remote_error){
        // No Remote Failures= > Put it in FAILED State
        m_log_info("Job [" << job.id() << "] has Non-Remote Catalog Failures and will be failed");
        result = CatalogRetryStrategy::FAIL;
    } else if(catalog_failures >= ActionsConfig::instance().maxFailures()){
        // Too Many Failures = > Put it in FAILED State
        m_log_info("Too many catalog failures (" << catalog_failures << ") for Job [" << job.id() << "]: Move it to Failed");
        result = CatalogRetryStrategy::FAIL;
    } else {
        unsigned long delay = ActionsConfig::instance().catalogRetryDelay();
        // Get Current Time
        time_t current;
        time(&current);
        
        // Check if it's time to retry the File
        if((time_t)-1 == last_active_time){
            // Take the Job Submission Time
            m_log_debug("Catalog Failures: " << catalog_failures << " - Job Submission Time: " << job.submitTime());
            if(catalog_failures > 0){
                last_active_time = job.submitTime() + (catalog_failures - 1) * delay;
            } else {
                last_active_time = job.submitTime();
            }
        }
        m_log_debug("Last Active Reference Time: " << last_active_time << " - CurrentTime: " << current);
        if((current > last_active_time) && (((unsigned long)(current - last_active_time)) >= delay)){
            // Reschedule the File
            m_log_info("Reschedule Job [" << job.id() << "]. Number of tries already performed: " << catalog_failures);

            // Resubmit Job
            result = CatalogRetryStrategy::RETRY;
        } else {
            // Elapsed time not sufficient
            m_log_debug("Job [" << job.id() << "] still in Waiting");
        }
    }

    // Retry strategy returns
    m_log_debug("Retry Result for Job [" << job.id() << "] is " << result);
    
    return result;
}
