/*
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *  GLite Data Transfer Agents: BasicRetry Strategy Class
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *           Paolo Tedesco <paolo.tedesco@cern.ch>
 *  Version info: $Id: BasicRetryStrategy.cpp,v 1.9 2008/11/13 10:43:55 ptedesco Exp $
 *  Release: $Name:  $
 *
 */
#include "BasicRetryStrategy.h"
#include "ActionsConfig.h"

using namespace glite::data::transfer::agent;
using namespace glite::data::transfer::agent::model;
using namespace glite::data::transfer::agent::action;
using namespace glite::data::transfer::agent::action::vo;

using boost::shared_ptr;

namespace {
const char * const BASIC_RETRY_NAME = "tranfser-agent-vo-retry";
}
/*
 * BasicRetryStrategy
 *
 * Constructor
 */
BasicRetryStrategy::BasicRetryStrategy():RetryStrategy(),
    m_logger(BASIC_RETRY_NAME){
}

/*
 * ~BasicRetryStrategy
 *
 * Destructor
 */
BasicRetryStrategy::~BasicRetryStrategy(){
}

/*
 * processFile
 *
 * Trigger the rescheduling on a file
 */
RetryStrategy::RetryResult BasicRetryStrategy::apply(const Job& job,const File& file, const TransferArray& transfers) /*throw (ExecuteException)*/{

    RetryResult result = RetryStrategy::WAIT;
    
    m_log_debug("Check if File [" << file.id() << "] - " << file.jobId() << " can be retried");
    
    // Check File Failures
    if(file.currentFailures() >= ActionsConfig::instance().maxFailures()){
        // Too Many Failures = > Put it in HOLD State
        m_log_info("Too many failures (" << file.currentFailures() << ") for File [" << file.id() << "]: move it to HOLD");

        // File State is Hold
        result = RetryStrategy::HOLD;
    } else {
        // Count the number of attemps
        m_log_debug("Got " << transfers.size() << " tries");
        
        unsigned long delay = ActionsConfig::instance().resubmitDelay();
        // Get Current Time
        time_t current;
        time_t transfer_time;
        time(&current);
        
        // Check if it's time to resubmit the File
        if((time_t)-1 != file.finishTime()){
            // Take the file finish time
            transfer_time = file.finishTime();
        } else {
            // No error time set. Take as reference the start time of the last 
            // transfer start time
            if(false == transfers.empty()){
                // Take the last transfer Time
                transfer_time = transfers.front()->transferTime();
            } else {
                // File is failed with no transfer attempts. Take the Job 
                // Submission Time
                m_log_debug("Num Failures: " << file.failures() << " - Current " << file.currentFailures() << " - Job Submission Time: " << job.submitTime());
                if(file.failures() > 0){
                    transfer_time = job.submitTime() + (file.failures() - 1) * delay;
                } else {
                    transfer_time = job.submitTime();
                }
            }
        }
        m_log_debug("Last Transfer Reference Time: " << transfer_time << " - CurrentTime: " << current);
        if((current > transfer_time) && 
            (((unsigned long)(current - transfer_time)) >= delay)){
            // Reschedule the File
            m_log_info("Reschedule File [" << file.id() << "]. Number of tries already performed: " << transfers.size());

            // Resubmit File
            result = RetryStrategy::RETRY;
        } else {
            // Elapsed time not sufficient
            m_log_debug("File [" << file.id() << "] still in Waiting");
        }
    }

    // Retry strategy returns
    m_log_debug("Retry Result for File [" << file.id() << "] is " << result);
    
    return result;
}
