/*
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *  GLite Data Transfer Agents : ChannelCache Implementation Class
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *  Version info: $Id: ChannelCacheImpl.cpp,v 1.9 2008/11/13 10:43:55 ptedesco Exp $
 *  Release: $Name:  $
 *
 */
#include "ChannelCacheImpl.h"
#include "glite/data/agents/AgentExceptions.h"

#include <boost/tuple/tuple.hpp>

using namespace glite::data::transfer::agent;
using namespace glite::data::transfer::agent::action;
using namespace glite::data::transfer::agent::action::vo;
using namespace glite::data::agents;
using namespace boost::multi_index;

namespace {
const char * const CHANNEL_CACHE_MODULE = "vo-action-channel-cache";
const unsigned int DEFAULT_TTL          =  300; // 5 minutes
const unsigned int DEFAULT_PURGE_DELAY  =  180; // 1 minutes
    
/*
 * update_channel predicate
 */
struct update_channel {
    /**
     * Constructor
     */
    update_channel(ChannelCacheImpl::ChannelPtr& p, unsigned int ttl):
        m_channel(p),m_ttl(ttl){}

    /**
     * Call Operator
     */
    void operator()(ChannelCacheImpl::ChannelEntry& e) {
        // update channel (name and sites should be the same)
        e.channel = m_channel;
        // Refresh time and validity
        time(&(e.creationTime));
        e.validity = m_ttl;
        
    }
private:
    ChannelCacheImpl::ChannelPtr& m_channel;
    unsigned int m_ttl;
};

/*
 * update_missing_channel predicate
 */
struct update_missing_channel {
    /**
     * Constructor
     */
    update_missing_channel(unsigned int ttl):m_ttl(ttl){}

    /**
     * Call Operator
     */
    void operator()(ChannelCacheImpl::MissingChannelEntry& e) {
        // Refresh time and validity
        time(&(e.creationTime));
        e.validity = m_ttl;
        
    }
private:
    unsigned int m_ttl;
};

/*
 * Predicate to check if an entry is obsolete
 */
struct is_obsolete : public std::unary_function<const ChannelCacheImpl::CacheEntry&,bool>{
    /**
     * Constructor
     */
    is_obsolete(time_t t):m_time(t){}

    /**
     * Call Operator
     */
    bool operator()(const ChannelCacheImpl::CacheEntry& e) const {
        if(m_time > (e.creationTime + e.validity)){
            return true;
        }
        return false;
    }
private:
    time_t m_time;
};
}

/*
 * ChannelCacheImpl
 *
 * Constructor
 */
ChannelCacheImpl::ChannelCacheImpl():
    m_logger(CHANNEL_CACHE_MODULE),
    m_ttl(DEFAULT_TTL),
    m_purgeDelay(DEFAULT_PURGE_DELAY),
    m_lastPurge((time_t)-1){
}
    
/*
 * ~ChannelCacheImpl
 *
 * Destructor
 */
ChannelCacheImpl::~ChannelCacheImpl(){
    clear();
}

/*
 * insert
 *
 * Insert a Channel in the Cache
 */
void ChannelCacheImpl::insert(const std::string& ssite, const std::string& dsite, ChannelPtr c) /*throw (LogicError)*/ {
    
    // Check Preconditions
    if(0 == c.get()){
        throw NullPointerException("Invalid Channel instance - null pointer");
    }
    
    typedef ChannelTable::index<src_dst_id>::type  ch_set_by_sites;
    
    // Get Index
    ch_set_by_sites& sites_index = m_chTable.get<src_dst_id>();
    // Check if Channel exists
    ch_set_by_sites::iterator ch_it = sites_index.find(boost::make_tuple(ssite,dsite));
    if(ch_it == sites_index.end()) {
        // Insert Channel in the list
        ChannelEntry entry(c,ssite,dsite);
        entry.validity = m_ttl;
        m_chTable.insert(entry);
        m_log_debug("Channel <" << c->name() << ">, between <" << ssite << "> and <" << dsite << ">, inserted in cache");
    } else {
        // Refresh entry
        m_chTable.modify(ch_it,update_channel(c,m_ttl));
        m_log_debug("Channel " << c->name() << " between <" << ssite << "> and <" << dsite << "> was already in cache and it has been refreshed");
    }
}

/*
 * get
 *
 * Get a Channel cache entry
 */
ChannelCacheImpl::ChannelPtr ChannelCacheImpl::get(const std::string& channel_name) const /*throw (RuntimeError)*/ {
    
    typedef const ChannelTable::index<name>::type  ch_set_by_name;
    
    // Get Index
    ch_set_by_name& name_index = m_chTable.get<name>();
    // Check if Channel exists
    ch_set_by_name::const_iterator ch_it = name_index.find(channel_name);
    if(ch_it == name_index.end()) {
        // No Transfer found
        m_log_debug("Channel <" << channel_name << "> not in cache");
        throw RuntimeError("No Channel Found in cache");
    } else {
        // Check the validity of the entry
        time_t current;
        time(&current);
        if(current > ((*ch_it).creationTime + (*ch_it).validity)){
            // Do not remove entry since this method is const. Entry should be
            // removed by the purge method
            m_log_debug("Channel <" << channel_name << "> is in cache but obsolte");
            throw RuntimeError("No Channel Found in cache");
        }
    }

    return (*ch_it).channel;
}

/*
 * getFromSites
 *
 * Get a Channel cache entry (if exists)
 */
ChannelCacheImpl::ChannelPtr ChannelCacheImpl::getFromSites(const std::string& ssite, const std::string& dsite) const /*throw (RuntimeError)*/{
    
    typedef const ChannelTable::index<src_dst_id>::type  ch_set_by_sites;
    
    // Get Index
    ch_set_by_sites& sites_index = m_chTable.get<src_dst_id>();
    // Check if Channel exists
    ch_set_by_sites::const_iterator ch_it = sites_index.find(boost::make_tuple(ssite,dsite));
    if(ch_it == sites_index.end()) {
        // No Transfer found
        m_log_debug("Channel between <" << ssite << "> and <" << dsite << "> not in cache");
        throw RuntimeError("No Channel Found in cache");
    } else {
        // Check the validity of the entry
        time_t current;
        time(&current);
        if(current > ((*ch_it).creationTime + (*ch_it).validity)){
            // Do not remove entry since this method is const. Entry should be
            // removed by the purge method
            m_log_debug("Channel <" << (*ch_it).channel_name << "> is in cache but obsolete");
            throw RuntimeError("No Channel Found in cache");
        }
    } 

    return (*ch_it).channel;
}

/*
 * remove
 *
 * Removbe an entry from the Cache
 */
void ChannelCacheImpl::remove(const std::string& ssite, const std::string& dsite) {
        
    typedef ChannelTable::index<src_dst_id>::type  ch_set_by_sites;
    
    // Get Index
    ch_set_by_sites& sites_index = m_chTable.get<src_dst_id>();
    // Check if Channel exists
    ch_set_by_sites::iterator ch_it = sites_index.find(boost::make_tuple(ssite,dsite));
    if(ch_it == sites_index.end()) {
        m_log_debug("No Channel between <" << ssite << "> and <" << dsite << "> found in cache");
    } else {
        // Remove Channel from the list
        sites_index.erase(ch_it);
        m_log_debug("Channel between <" << ssite << "> and <" << dsite << "> deleted from cache");
    }
}

/*
 * count
 *
 * Count the number of entries in the cache
 */
unsigned int ChannelCacheImpl::count() const /*throw ()*/{
   
    typedef const ChannelTable::index<src_dst_id>::type  ch_set_by_sites;
    
    // Get Index
    ch_set_by_sites& sites_index = m_chTable.get<src_dst_id>();
    return sites_index.size();
}

/*
 * rememberMissingChannel
 *
 * Remember missing Channel
 */
void ChannelCacheImpl::rememberMissingChannel(const std::string& ssite, const std::string& dsite) /*throw ()*/{
    
    // Check if service is missing
    typedef MissingChannelTable::index<src_dst_id>::type  set_by_sites;
    // Check if entry exists
    set_by_sites& index = m_chMissing.get<src_dst_id>();

    // Check first if it's missing for any VO
    set_by_sites::iterator ch_it = index.find(boost::make_tuple(ssite,dsite));
    if(ch_it == index.end()) {
        // Insert MissingChannel in the list
        MissingChannelEntry entry(ssite,dsite);
        entry.validity = m_ttl;
        m_chMissing.insert(entry);
        m_log_debug("Missing Channel between <" << ssite << "> and <" << dsite << "> recorded in cache");
    } else {
        // Refresh entry
        m_chMissing.modify(ch_it,update_missing_channel(m_ttl));
        m_log_debug("Missing Channel between <" << ssite << "> and <" << dsite << "> was already recorded in cache and it has been refreshed");
    }
}

/*
 * isChannelMissing
 *
 * Is Channel missing
 */
bool ChannelCacheImpl::isChannelMissing(const std::string& ssite, const std::string& dsite) const /*throw ()*/{
    bool is_missing = false;    
    
    // Check if service is missing
    typedef const MissingChannelTable::index<src_dst_id>::type set_by_sites;
    // Check if entry exists
    set_by_sites& index = m_chMissing.get<src_dst_id>();

    // Check first if it's missing for any VO
    set_by_sites::const_iterator ch_it = index.find(boost::make_tuple(ssite,dsite));
    if(ch_it == index.end()) {
        is_missing = false;
    } else {
        // Check the validity of the entry
        time_t current;
        time(&current);
        if(current > ((*ch_it).creationTime + (*ch_it).validity)){
            // Do not remove entry since this method is const. Entry should be
            // removed by the purge method
            is_missing = false;
        } else {
            is_missing = true;
        }
    }
    if(true == is_missing){
        m_log_debug("Channel between <" << ssite << "> and <" << dsite << "> is not defined");
    }
            
    return is_missing;
}

/*
 * removeMissingChannel
 *
 * remove missing Channel
 */
void ChannelCacheImpl::removeMissingChannel(const std::string& ssite, const std::string& dsite) /*throw ()*/{
    // Check if service is missing
    typedef MissingChannelTable::index<src_dst_id>::type  set_by_sites;
    // Check if entry exists
    set_by_sites& index = m_chMissing.get<src_dst_id>();

    // Check first if it's missing for any VO
    set_by_sites::const_iterator ch_it = index.find(boost::make_tuple(ssite,dsite));
    if(ch_it == index.end()) {
        m_log_debug("Channel between <" << ssite << "> and <" << dsite << "> not found in missing cache");
    } else {
        // Remove Missing Channel from the list
        index.erase(ch_it);
        m_log_debug("Channel between <" << ssite << "> and <" << dsite << "> deleted from missing cache");
    }        
}


/*
 * rememberSiteGroups
 *
 * Remember site's groups
 */
void ChannelCacheImpl::rememberSiteGroups(const std::string& site, const std::vector<std::string>& groups) /*throw ()*/{
    // Check if site is missing
    std::vector<std::string>::const_iterator g_it;
    SiteGroupTable::iterator s_it = m_siteGroups.find(site);
    if(s_it == m_siteGroups.end()) {
        // Insert site in the list
        SiteEntry entry(site);
        entry.validity = m_ttl;
        for(g_it = groups.begin(); g_it != groups.end(); ++g_it){
            GroupEntry e((*g_it));
            e.validity = m_ttl;
            entry.groups.push_back(e);
        }
        m_siteGroups.insert(SiteGroupTable::value_type(site,entry));
        m_log_debug("Site <" << site << "> has (" << groups.size() << ") groups recorded in cache");
    } else {
        int count = 0;
        SiteEntry& entry = (*s_it).second;
        std::vector<GroupEntry>::iterator e_it;
        for(g_it = groups.begin(); g_it != groups.end(); ++g_it){
            for(e_it = entry.groups.begin(); e_it != entry.groups.end(); ++e_it){
                GroupEntry& e = (*e_it);
                if(e.group == (*g_it)){
                    // Refresh creation time & validity
                    time(&(e.creationTime));
                    e.validity = m_ttl;
                    break;                    
                }
            }
            if(e_it == entry.groups.end()){
                // Record new group-site association
                GroupEntry e((*g_it));
                e.validity = m_ttl;
                entry.groups.push_back(e);
                ++count;
            }
        }
        time(&(entry.creationTime));
        entry.validity = m_ttl;
        m_log_debug(count << " more group(s) recorder in cache for Site <" << site << ">");
    }
}

/*
 * rememberSiteGroup
 *
 * Remember site's group
 */
void ChannelCacheImpl::rememberSiteGroup(const std::string& site, const std::string& group) /*throw ()*/{
    std::vector<std::string> dummy;
    dummy.push_back(group);
    return rememberSiteGroups(site,dummy);
}

/*
 * getSiteGroups
 *
 * Is Channel missing
 */
void ChannelCacheImpl::getSiteGroups(const std::string& site, std::vector<std::string>& groups) const /*throw (RuntimeError)*/{
    SiteGroupTable::const_iterator s_it = m_siteGroups.find(site);
    if(s_it == m_siteGroups.end()) {
        m_log_debug("No Groups for site <" << site << "> recorded in cache");
        throw RuntimeError("No site's groups found in cache");
    } else {
        const SiteEntry& entry = (*s_it).second;
        std::vector<GroupEntry>::const_iterator e_it;
        for(e_it = entry.groups.begin(); e_it != entry.groups.end(); ++e_it){
            const GroupEntry& e = (*e_it);
            groups.push_back(e.group);
        }
        m_log_debug(groups.size() << " group(s) found in cache for Site <" << site << ">");
    }
}

/*
 * removeSiteGroups
 *
 * Remove Site Groups
 */
void ChannelCacheImpl::removeSiteGroups(const std::string& site, const std::vector<std::string>& groups) /*throw ()*/{
    // Check if site is missing
    std::vector<std::string>::const_iterator g_it;
    SiteGroupTable::iterator s_it = m_siteGroups.find(site);
    if(s_it == m_siteGroups.end()) {
        m_log_debug("Groups for site <" << site << "> not recorded in cache");
    } else {
        int count = 0;
        SiteEntry& entry = (*s_it).second;
        std::vector<GroupEntry>::iterator e_it;
        for(g_it = groups.begin(); g_it != groups.end(); ++g_it){
            for(e_it = entry.groups.begin(); e_it != entry.groups.end(); ++e_it){
                const GroupEntry& e = (*e_it);
                if((*g_it) == e.group){
                    // Remove group association
                    entry.groups.erase(e_it);
                    ++count;
                    break;                    
                }
            }
            if(e_it == entry.groups.end()){
                // group-site association not recorded in cache. Ignore it
            }
        }
        m_log_debug(count << " group(s) removed from cache for Site <" << site << ">");
        if(entry.groups.empty() == true){
            m_siteGroups.erase(s_it);
            m_log_debug("Site <" << site << "> removed from cache");
        }
    }
}

/*
 * removeSiteGroup
 *
 * Remove Site Group
 */
void ChannelCacheImpl::removeSiteGroup(const std::string& site, const std::string& group) /*throw ()*/{
    std::vector<std::string> dummy;
    dummy.push_back(group);
    return removeSiteGroups(site,dummy);
}

/*
 * clear
 *
 * Clear the Cache
 */
void ChannelCacheImpl::clear(){
    // Cleanup the tables
    m_chTable.clear();
    m_chMissing.clear();
    m_siteGroups.clear();
    m_log_debug("Cache cleaned");
}

/*
 * purge
 *
 * Remove obsolete entries from the Cache
 */
void ChannelCacheImpl::purge(bool use_delay){
    
    // Get Current Time
    time_t current;
    time(&current);

    // Check last Purge operation Time
    if((true == use_delay) && 
       ((time_t)-1 != m_lastPurge) && 
       (current < (m_lastPurge + m_purgeDelay))){
        // Skip purge;
        m_log_debug("Skip purge since " << m_purgeDelay << " seconds are not elapsed");
        return;
    }
    
    is_obsolete pred(current);

    unsigned int count = 0;

    // For each element in Cache
    ChannelTable::iterator it;
    do{
        // Check if it's obsolete
        it = std::find_if(m_chTable.begin(),m_chTable.end(),pred);
        if(it != m_chTable.end()){
            // Remove it from cache
            m_chTable.erase(it);
            ++count;
        } else {
            break;
        }
    } while(1);

    m_log_debug(count << " entries removed from Cache");

    count = 0;
    // For each element in Cache
    MissingChannelTable::iterator mit;
    do{
        // Check if it's obsolete
        mit = std::find_if(m_chMissing.begin(),m_chMissing.end(),pred);
        if(mit != m_chMissing.end()){
            // Remove it from cache
            m_chMissing.erase(mit);
            ++count;
        } else {
            break;
        }
    } while(1);

    m_log_debug(count << " entries removed from Missing Cache");

    count = 0;
    // For each element in Cache
    SiteGroupTable::iterator sit;
    std::vector<std::string> site_to_remove;
    for(sit = m_siteGroups.begin(); sit != m_siteGroups.end();++sit){
        std::vector<GroupEntry>::iterator git;
        SiteEntry& entry = (*sit).second;
        if(pred(entry)){
            count += entry.groups.size();
            site_to_remove.push_back((*sit).first);
        } else {
            bool had_groups = (entry.groups.empty())?false:true;
            do{
                // Check if it's obsolete
                git = std::find_if(entry.groups.begin(),entry.groups.end(),pred);
                if(git != entry.groups.end()){
                    // Remove it from cache
                    entry.groups.erase(git);
                    ++count;
                } else {
                    break;
                }
            } while(1);
            // Check if site should be removed
            if((true == had_groups) && (true == entry.groups.empty())){
                site_to_remove.push_back((*sit).first);
            }
        }
    }
    std::vector<std::string>::iterator nit;
    for(nit = site_to_remove.begin();nit != site_to_remove.end();++nit){
        m_siteGroups.erase((*nit));
    }
            
    m_log_debug(count << " group and " << site_to_remove.size() << " site entries removed from SiteGroups Cache");
    
    m_lastPurge = current;
}
