/*
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *  GLite Data Transfer Agents: ChannelCache Implementation Class
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *           Paolo Tedesco <paolo.tedesco@cern.ch>
 *  Version info: $Id: ChannelCacheImpl.h,v 1.4 2008/11/13 10:43:55 ptedesco Exp $
 *  Release: $Name:  $
 *
 */
#ifndef GLITE_DATA_TRANSFER_AGENT_ACTIONS_VO_CHANNELCACHEIMPL_H_
#define GLITE_DATA_TRANSFER_AGENT_ACTIONS_VO_CHANNELCACHEIMPL_H_

#include "glite/data/transfer/agent/action/vo/ChannelCache.h"
#include <vector>
#include <string>
#include <map>
#include <boost/multi_index_container.hpp>
#include <boost/multi_index/member.hpp>
#include <boost/multi_index/composite_key.hpp>
#include <boost/multi_index/ordered_index.hpp>
#include <boost/utility.hpp>

#include <glite/data/config/service/Logger.hxx>

namespace glite   {
namespace data    {
namespace transfer{
namespace agent   {
namespace action  {
namespace vo      {

/**
 * Channel Implementation (Pimpl Idiom)
 */
class ChannelCacheImpl : boost::noncopyable{
public:
    
    typedef boost::shared_ptr<model::Channel> ChannelPtr; 

    /**
     * Constructor
     */
    ChannelCacheImpl();
        
    /**
     * Destructor
     */
    ~ChannelCacheImpl();
   
    /**
     * Set the Validity of an entry in the Cache
     */
    void ttl(unsigned int v){m_ttl = v;}

    /**
     * Set the min delay beween two purge
     */
    void purgeDelay(unsigned int d){m_purgeDelay = d;}

    /**
     * Insert a Channel into the Cache
     */
    void insert(const std::string& ssite, const std::string& dsite, ChannelPtr c) /*throw (LogicError)*/ ;

    /**
     * Get a Channel cache entry
     */
    ChannelPtr get(const std::string& channel_name) const /*throw (RuntimeError)*/ ;

    /**
     * Get a Channel cache entry (if exists)
     */
    ChannelPtr getFromSites(const std::string& ssite, const std::string& dsite) const /*throw (RuntimeError)*/;

    /**
     * Remove a Channel from the cache
     */
    void remove(const std::string& ssite, const std::string& dsite);

    /**
     * Count the number of entries in the cache
     */
    unsigned int count() const /*throw ()*/;

    /**
     * Rememner missing Channel
     */
    void rememberMissingChannel(const std::string& ssite, const std::string& dsite) /*throw ()*/;

    /**
     * Is Channel missing
     */
    bool isChannelMissing(const std::string& ssite, const std::string& dsite) const /*throw ()*/;

    /**
     * remove missing Channel
     */
    void removeMissingChannel(const std::string& ssite, const std::string& dsite) /*throw ()*/;

    /**
     * Remember site's groups
     */
    void rememberSiteGroups(const std::string& site, const std::vector<std::string>& groups) /*throw ()*/;
    /**
     * Remember site's group
     */
    void rememberSiteGroup(const std::string& site, const std::string& group) /*throw ()*/;
    /**
     * Get site groups. If no site is recorded, throws a RuntimeError. If site 
     * has no groups, the groups array will be empty
     */
    void getSiteGroups(const std::string& site, std::vector<std::string>& groups) const /*throw (RuntimeError)*/;
    /**
     * Remove Site Groups
     */
    void removeSiteGroups(const std::string& site, const std::vector<std::string>& groups) /*throw ()*/;
    /**
     * Remove Site Group
     */
    void removeSiteGroup(const std::string& site, const std::string& group) /*throw ()*/;

    /**
     * Clear the Cache
     */
    void clear();

    /**
     * Purge invalid entries.
     * @param delay [IN] flag to specify if the purge operation should be skipped 
     *    if the previous purge was issued before the interval specified by 
     *    the &quot;purgeDelay&quot; property. 
     */
    void purge(bool use_delay = true);

public:

    /**
     * Cache Entry Type
     */
    struct CacheEntry {
        /*
         * timestamps & validity
         */
        mutable time_t       creationTime;
        mutable unsigned int validity;
        
        /*
         * Constructor
         */
        CacheEntry(): validity(0){
            time(&creationTime);
        }
    };

    /**
     * Cache Entry Type
     */
    struct ChannelEntry : public CacheEntry {

        /*
         * The Channel
         */        
        ChannelPtr channel;

        /*
         * The keys for the indexes
         */
        std::string  const channel_name;
        std::string  const source_site;
        std::string  const dest_site;
              
        /*
         * Constructor
         */        
        ChannelEntry(ChannelPtr c, const std::string& ssite, const std::string& dsite):
            channel(c),
            channel_name(c->name.data()),
            source_site(ssite),
            dest_site(dsite){}
    private:
        ChannelEntry();
    };

    /*
     * Entry used to define channel not existing in the DB
     */
    struct MissingChannelEntry : public CacheEntry {
        /*
         * The keys for the indexes
         */
        std::string const source_site;
        std::string const dest_site;
        
        /*
         * Constructor
         */
        MissingChannelEntry(const std::string& s,const std::string& d):
            source_site(s),dest_site(d){}
    private:
        MissingChannelEntry();
    };

    /*
     * Group Entry
     */
    struct GroupEntry : public CacheEntry {
        std::string group;
        /*
         * Constructor
         */
        GroupEntry(const std::string& g):group(g){}
    private:
        GroupEntry();
    };

    /*
     * Site Entry
     */
    struct SiteEntry : public CacheEntry {
        std::string site;
        std::vector<GroupEntry> groups;
        
        /*
         * Constructor
         */
        SiteEntry(const std::string& s):site(s){}
    private:
        SiteEntry();
    };
    
private:    
    glite::config::Logger m_logger;
    /*
     * tags for accessing several indices defined below
     */
    struct name{};
    struct src_dst_id{};
    struct site_group_id{};
            
    /*
     * Define The Table with
     *  - a unique index sorted by [source_site,dest_site]
     *  - a non-unique index channel name
     */
    typedef boost::multi_index::multi_index_container<
      ChannelEntry,
      boost::multi_index::indexed_by<
        boost::multi_index::ordered_unique<boost::multi_index::tag<src_dst_id>, 
          boost::multi_index::composite_key<
            ChannelEntry,
            BOOST_MULTI_INDEX_MEMBER(ChannelEntry,std::string const,source_site),
            BOOST_MULTI_INDEX_MEMBER(ChannelEntry,std::string const,dest_site)
          >
        >,
        boost::multi_index::ordered_non_unique<boost::multi_index::tag<name>,
          BOOST_MULTI_INDEX_MEMBER(ChannelEntry,std::string const,channel_name)
        >
      >
    > ChannelTable;

    /*
     * Define The Table with
     *  - a unique index sorted by [source_site,dest_site]
     */
    typedef boost::multi_index::multi_index_container<
      MissingChannelEntry,
      boost::multi_index::indexed_by<
        boost::multi_index::ordered_unique<boost::multi_index::tag<src_dst_id>, 
          boost::multi_index::composite_key<
            MissingChannelEntry,
            BOOST_MULTI_INDEX_MEMBER(MissingChannelEntry,std::string const,source_site),
            BOOST_MULTI_INDEX_MEMBER(MissingChannelEntry,std::string const,dest_site)
          >
        >
      >
    > MissingChannelTable;

    /*
     * Define the Table with a map in order to guarantee the ordering
     */
    typedef std::map<std::string, SiteEntry> SiteGroupTable;
    
    /**
     * The Validity of an entry in the Cache
     */
    unsigned int m_ttl;

    /**
     * The delay between two purge operations
     */
    unsigned int m_purgeDelay;

    /**
     * The timestamp of the last purge operation
     */
    time_t m_lastPurge;
    
    /**
     * The Channel Table
     */
    ChannelTable m_chTable;

    /**
     * The MissingChannel Table
     */
    MissingChannelTable m_chMissing;
    
    /**
     * The SiteGroup Table
     */
    SiteGroupTable m_siteGroups;
};

} // end namespace vo
} // end namespace action
} // end namespace agent
} // end namespace transfer
} // end namespace data
} // end namespace glite

#endif //GLITE_DATA_TRANSFER_AGENT_ACTIONS_VO_CHANNELCACHEIMPL_H_
