/*
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *  GLite Data Transfer Agents: CheckReadiness Action Class
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>, Paolo Tedesco <paolo.tedesco@cern.ch>
 *  Version info: $Id: CheckReadiness.cpp,v 1.12 2008/11/13 10:43:55 ptedesco Exp $
 *  Release: $Name:  $
 *
 */
#include "CheckReadiness.h"
#include "ActionsConfig.h"
#include "FileFSM.h"
#include "JobFSM.h"
#include <glite/data/agents/url-utility.h>
#include <glite/data/agents/UserProxyEnv.h>
#include <glite/data/agents/extra/utility.h>
#include <glite/data/agents/sd/SDConfig.h>
#include <glite/data/agents/sd/sd-utility.h>
#include <glite/data/agents/AgentExceptions.h>
#include <glite/data/agents/cred/CredServiceException.h>

#include <boost/scoped_ptr.hpp>
#include <set>

using namespace glite::data::transfer::agent;
using namespace glite::data::transfer::agent::fsm;
using namespace glite::data::transfer::agent::action;
using namespace glite::data::transfer::agent::action::vo;

using glite::data::transfer::agent::model::Error;
using glite::data::agents::UserProxyEnv;
using glite::data::agents::LogicError;
using glite::data::agents::RuntimeError;
using glite::data::agents::dao::DAOException;
using glite::data::agents::cred::CredServiceException;
using boost::scoped_ptr; 

namespace {
const unsigned int MAX_JOB_CHUNKS = 20;
}

/*
 * Register ActionFactory Method 
 */
REGISTER_FACTORY_METHOD( CheckReadiness );
      
/*
 * Cancel
 *
 * Constructor
 */
CheckReadiness::CheckReadiness():VOAction("CheckReadiness"){
}

/*
 * ~CheckReadiness
 *
 * Destructor
 */
CheckReadiness::~CheckReadiness(){
}

/*
 * execute
 *
 * Execute the Action
 */
void CheckReadiness::exec(const std::string& params) /*throw (DAOException, LogicError)*/{

    // Get Job DAO
    JobDAO& jobDao = jobDAO();
         
    // Get All the jobs in pending state that have Pending files or shoudl be 
    // canceled and have Hold Files
    agents::dao::StringArray  jobs;
    jobDao.getPendingJobs(jobs,MAX_JOB_CHUNKS);

    m_log_debug("Got " << jobs.size() << " jobs in (Pending,Active) State, with Pending files");
            
    // For Each Job
    StringArray jobIds;
    StringArray::iterator j_it;
    for(j_it = jobs.begin(); j_it != jobs.end(); ++j_it){
        // Check if the Job is Ready
        checkReadiness((*j_it),jobIds);
    } //  End for each job

    // Update Job States
    this->updateJobs(jobIds);
    
    m_log_debug("Jobs' state updated");
}


/*
 * checkReadiness
 *
 * Check Job Readiness
 */
void CheckReadiness::checkReadiness(const std::string& job_id, StringArray& jobIds) /*throw (DAOException, LogicError)*/{
    
    // Get Job DAO
    JobDAO& jobDao = jobDAO();
            
    m_log_debug("Checking if Job [" << job_id << "] is ready");
    
    StringArray files;
    try{
        // Get Job. Lock is already acquired
        scoped_ptr<model::Job> job(jobDao.get(job_id));
    
        // Get File DAO
        FileDAO& fileDao = fileDAO();

        // Get all the File for the given Job
        fileDao.getByJobId(job->id(),files);
        m_log_debug("Got " << files.size() << " Files");
    
        // Get Proxy Certificate
        std::string proxy_file;
        try{
            getProxyForJob(*job,proxy_file);
        } catch(const CredServiceException& exc){
            m_log_warn("Failed to get Proxy Certificate for Job [" << job_id << "]. Reason is " << exc.what());
            failAllFiles(files,Error::SECURITY,Error::S_AGENT,Error::P_CHECK_READINESS,exc.what());
            return;
        }
            
        // Check Precondition
        if(model::Job::S_PENDING == job->state()){
            jobIds.push_back(job_id);
        } else if(model::Job::S_ACTIVE == job->state()){
            // Do not store the job id: this job should be updated by the 
            // Agent (Status is Active)
            m_log_debug("State of Job [" << job_id << "] will not be updated during this iteration (it's responsibility of the channel agent to do it)");
        } else {
            // Job not in submitted state: ignore it
            return;
        }
    
#warning "TODO: check if the file is online (on disk) otherwise move it to awaiting prestage."
            
        // For each file
        StringArray::const_iterator f_it;
        for(f_it = files.begin(); f_it != files.end(); ++f_it){
            // Get the File
            scoped_ptr<model::File> file(fileDao.get((*f_it)));
            m_log_debug("Checking File [" << file->id() << "]");
    
            try{
                // Checking File State
                if(model::File::S_PENDING == file->state()){
                    // Set file as Ready
                    FileFSM f_fsm(*file,*job);
                    if(true == job->cancelJob()){
                        f_fsm.onEventCanceled();
                        
                    } else {
                        // For the moment, prestaging is not suported, so move 
                        // the pending file to ready
// #warning "add here (or in the FSM) a check; if the error occurs inside the pre-transfer hook, set the appropriate error phase"
                        f_fsm.onEventFileReady();
                    }
                    // Update File
                    fileDao.update(*file);
                } else if((model::File::S_HOLD   == file->state()) && 
                          (model::Job::S_PENDING == job->state())){
                    // Set file as Ready
                    FileFSM f_fsm(*file,*job);
                    if(true == job->cancelJob()){
                        f_fsm.onEventCanceled();
                    } 
                    // Update File
                    fileDao.update(*file);
                }
            }catch(const LogicError& exc){
                m_log_error("LogicError setting File [" << (*f_it) << "] as ready: " << exc.what());
                failFile(file->id(),Error::CONSISTENCY,Error::S_AGENT,Error::P_CHECK_READINESS,exc.what());
            }
        } // End for each file             
    } catch(const LogicError& exc) {
        m_log_error("LogicError checking if Job [" << job_id << "] is Ready: " << exc.what());
        if(false == files.empty()){
            failAllFiles(files,Error::CONSISTENCY,Error::S_AGENT,Error::P_CHECK_READINESS,exc.what());
        }
        throw;
    } catch(const DAOException& exc){
        m_log_error("Failed to check Job [" << job_id << "] readiness (DAOException): " << exc.what());
        throw;
    }       
}
