/*
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *  GLite Data Transfer Agents: Finalize Class
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *  Version info: $Id: Finalize.cpp,v 1.12 2008/11/13 10:43:55 ptedesco Exp $
 *  Release: $Name:  $
 *
 */
#include "Finalize.h"
#include "ActionsConfig.h"
#include <glite/data/transfer/agent/catalog/CatalogService.h>
#include <glite/data/transfer/agent/catalog/CatalogServiceFactory.h>
#include <glite/data/transfer/agent/catalog/CatalogServiceException.h>
#include <glite/data/transfer/agent/model/File.h>
#include <glite/data/transfer/agent/model/Job.h>
#include <glite/data/transfer/agent/model/Error.h>
#include "FileFSM.h"
#include "JobFSM.h"
#include "JobFlags.h"
#include <glite/data/agents/url-utility.h>
#include <glite/data/agents/UserProxyEnv.h>
#include <glite/data/agents/extra/utility.h>
#include <glite/data/agents/sd/SDConfig.h>
#include <glite/data/agents/sd/sd-utility.h>
#include <glite/data/agents/AgentExceptions.h>
#include <glite/data/agents/cred/CredServiceException.h>
#include <glite/data/transfer/agent/catalog/CatalogService.h>
#include <glite/data/transfer/agent/catalog/CatalogServiceFactory.h>
#include <glite/data/transfer/agent/catalog/CatalogServiceException.h>

#include <boost/scoped_ptr.hpp>

using namespace glite::data::transfer::agent;
using namespace glite::data::transfer::agent::fsm;
using namespace glite::data::transfer::agent::action;
using namespace glite::data::transfer::agent::catalog;
using namespace glite::data::transfer::agent::action::vo;
using namespace glite::data::transfer::agent::model;

using glite::data::agents::UserProxyEnv;
using glite::data::agents::LogicError;
using glite::data::agents::RuntimeError;
using glite::data::agents::dao::DAOException;
using glite::data::agents::cred::CredServiceException;
using boost::scoped_ptr;

namespace {
const unsigned int MAX_JOB_CHUNKS       = 20;
   
/**
 * Compare the the logical name of a FileStatArray
 */
struct compare_filestat_logical: std::unary_function<const CatalogService::FileStatPtr&, bool> {
    /**
     * Constructor
     */
    compare_filestat_logical(const std::string& s):m_str(s){}
    /**
     * Call Operator
     */
    bool operator()(const CatalogService::FileStatPtr& p) const {
        return ( p->logical == m_str )?true:false;
    }
private:
    const std::string& m_str;
};
/**
 * Compare the the logical name of a ReplicaStatArray
 */
struct compare_replicastat_logical: std::unary_function<const CatalogService::ReplicaStat&, bool> {
    /**
     * Constructor
     */
    compare_replicastat_logical(const std::string& s):m_str(s){}
    /**
     * Call Operator
     */
    bool operator()(const CatalogService::ReplicaStat& p) const {
        return ( p.logical == m_str )?true:false;
    }
private:
    const std::string& m_str;
};
    
}

/*
 * Register ActionFactory Method 
 */
REGISTER_FACTORY_METHOD( Finalize );
  
/*
 * Finalize
 *
 * Constructor
 */
Finalize::Finalize():VOAction("Finalize"){
}

/*
 * ~Finalize
 *
 * Destructor
 */
Finalize::~Finalize(){
}

/*
 * execute
 *
 * Execute the Action
 */
void Finalize::execute(const std::string& params) /*throw (LogicError, ExecuteException)*/{

    m_log_debug("Executing Finalize Action");
           
    try{
        // Call the template method
        this->exec(params);
    } catch(const DAOException& exc){
        m_log_error("Failed To Execute Finalize Action (DAOException). Reason is " << exc.what());
        throw ExecuteException("Some problem found in interacting with DataSource");
    } catch(const ExecuteException& exc){
        m_log_error("Failed To Execute Finalize Action. Reason is " << exc.what());
        throw;
    } catch(const LogicError& exc){
        m_log_error("LogicError in Executing Finalize Action. Reason is " << exc.what());
        throw ;
    } catch(const std::exception& exc){
        m_log_error("Failed to execute Finalize action. Reason is " << exc.what());
        throw;
    } catch(...){
        m_log_error("Cannot execute Finalize action. Unknown reason");
        throw;
    }      
    m_log_debug("Action Executed successfully");
}

/*
 * exec
 *
 * Execute the Action
 */
void Finalize::exec(const std::string& params) /*throw (DAOException, LogicError, ExecuteException)*/{

    // Check if the Disable delegation Flag is set on the action parameters
    bool disable_deleg = 
        (params.find(JOB_PARAM_DISABLE_DELEGATION) != std::string::npos)?true:false;

    // Set Job as Finishing
    prepareToFinish(disable_deleg);
    // Set Job as Finished
    finish(disable_deleg);
}

/*
 * execFinalizing
 *
 * Execute the Finalizing Action
 */
void Finalize::prepareToFinish(bool disable_delegation) /*throw (DAOException, LogicError, ExecuteException)*/{
    // Get Job DAO
    JobDAO& jobDao   = jobDAO();
    // Get All the jobs to finish
    StringArray jobs;
    jobDao.getDoneJobs(jobs, MAX_JOB_CHUNKS);
    
    m_log_debug("Got " << jobs.size() << " Done jobs");

    try{
        // Start Transaction
        context().start();
            
        // For Each Job
        StringArray jobs_to_finish;
        StringArray::iterator j_it;
        for(j_it = jobs.begin(); j_it != jobs.end(); ++j_it){
            // Mark Job as Finishing
            prepareJobToFinishing((*j_it),disable_delegation);
        } //  End for each job
    
        // Update Job States
        this->updateJobs(jobs);
        
        // Commit Transaction
        context().commit();
    } catch(...){
        // Rollback Transaction
        context().rollback();
        throw;
    }
}

/*
 * execFinishing
 *
 * Execute the Action
 */
void Finalize::finish(bool disable_delegation) /*throw (DAOException, LogicError, ExecuteException)*/{
    // Get Job DAO
    JobDAO& jobDao   = jobDAO();
    // Get All the jobs to finish
    StringArray jobs;
    jobDao.getFinishingJobs(jobs, MAX_JOB_CHUNKS);
    
    m_log_debug("Got " << jobs.size() << " Finishing jobs");

    // For Each Job
    StringArray::iterator j_it;
    for(j_it = jobs.begin(); j_it != jobs.end(); ++j_it){
        try{
            // Start Transaction
            context().start();
            
            // Finalize Job
            finishJob((*j_it),disable_delegation);
            
            // Commit Transaction
            context().commit();
        } catch(...){
            m_log_error("Failed to finalize Job [" << (*j_it) << "]");
            // Rollback Transaction
            context().rollback();
            throw;
        }
    } //  End for each job

    try{
        // Start Transaction
        context().start();
            
        // Update Job States
        this->updateJobs(jobs);
        
        // Commit Transaction
        context().commit();
    } catch(...){
        // Rollback Transaction
        context().rollback();
        throw;
    }
}

/*
 * prepareJobToFinishing
 *
 * Set a Job as Finishing
 */
void Finalize::prepareJobToFinishing(const std::string& job_id, bool disable_delegation) /*throw (DAOException,LogicError)*/{
    // Get Job DAO
    JobDAO& jobDao = jobDAO();
    
    m_log_debug("Prepare to Finishing Job [" << job_id << "]");
    
    StringArray files;
    try{
        // Get Job. Lock is already acquired
        scoped_ptr<model::Job> job(jobDao.get(job_id));
        
        // Check Precondition
        if((model::Job::S_DONE != job->state()) &&
           (model::Job::S_DONE_WITH_ERRORS != job->state())){
            // Job not in Done state: ignore it
            m_log_debug("Not in Done State (" << job->state() << ")");
            return;
        }
                
        // Get File DAO
        FileDAO& fileDao = fileDAO();

        // Get all the File for the given Job
        fileDao.getByJobId(job->id(),files);
        m_log_debug("Got " << files.size() << " Files");

        // For each file
        StringArray::const_iterator f_it;
        for(f_it = files.begin(); f_it != files.end(); ++f_it){
            // Get the File
            scoped_ptr<model::File> file(fileDao.get((*f_it)));
            // Create File FSM
            FileFSM f_fsm(*file,*job);
            // Check if Job should be canceled
            if(true == job->cancelJob()){
                // Cancel all the files
                f_fsm.onEventCanceled();
                m_log_info("File [" << file->id() << "] Canceled");
            } else if(file->state() == model::File::S_DONE){
                // Cancel all the files
                f_fsm.onEventFinishing();
                m_log_info("Finishing File [" << file->id() << "]");
            }
            // Update File
            fileDao.update(*file);
        } // End for each file
    } catch(const LogicError& exc) {
        m_log_error("LogicError in finishing Job [" << job_id << "]: " << exc.what());
        if(false == files.empty()){
            failAllFiles(files,Error::CONSISTENCY,Error::S_AGENT,Error::P_FINALIZATION,exc.what());
        }
    } catch(const DAOException& exc){
        m_log_error("Failed finishing Job [" << job_id << "] (DAOException): " << exc.what());
        throw;
    }   
}
   
/*
 * finishJob
 *
 * Finalize a Job
 */
void Finalize::finishJob(const std::string& job_id, bool disable_delegation) /*throw (DAOException,LogicError)*/{
    // Get Job DAO
    JobDAO& jobDao = jobDAO();
            
    m_log_debug("Perform finalization for Job [" << job_id << "]");
    
    StringArray files;
    try{
        // Get Job
        scoped_ptr<model::Job> job(jobDao.get(job_id));

        if(false == disable_delegation){
            disable_delegation = (job->internalParams().find(
                JOB_PARAM_DISABLE_DELEGATION) != std::string::npos)?true:false;
        }
    
        // Get File DAO
        FileDAO& fileDao = fileDAO();

        // Get all the File for the given Job
        fileDao.getByJobId(job->id(),files);
        m_log_debug("Got " << files.size() << " Files");

        // Get Proxy Certificate
        std::string proxy_file;
        try{
            getProxyForJob(*job,proxy_file);
        } catch(const CredServiceException& exc){
            m_log_warn("Failed to get Proxy Certificate for Job [" << job_id << "]. Reason is " << exc.what());
            failAllFiles(files,Error::SECURITY,Error::S_AGENT,Error::P_FINALIZATION,exc.what());
            return;
        }
        
        // Check Precondition
        if(model::Job::S_FINISHING != job->state()){
            // Job not in submitted state: ignore it
            return;
        }
        // At this point, Don't check if Job should be canceled

        // Register New Replicas
        registerFiles(*job,files,proxy_file);
    } catch(const LogicError& exc) {
        m_log_error("LogicError in finalizing Job [" << job_id << "]: " << exc.what());
        if(false == files.empty()){
            failAllFiles(files,Error::CONSISTENCY,Error::S_AGENT,Error::P_FINALIZATION,exc.what());
        }
    } catch(const DAOException& exc){
        m_log_error("Failed to Finalize Job [" << job_id << "] (DAOException): " << exc.what());
        throw;
    }     
}
    
/*
 * registerFiles
 *
 * Register New Replicas
 */
void Finalize::registerFiles(const model::Job& job, const StringArray& files, const std::string& proxy_file) /*throw (DAOException,LogicError)*/{

    // Check Precondition
    if(true == files.empty()){
        // Nothing to Do
        return;
    }

    // Get File DAO
    FileDAO& fileDao = fileDAO();
    
    bool need_registration = false;
    {
        // Check if the first file has an LFN specified
        scoped_ptr<model::File> f(fileDao.get(*(files.begin())));
        need_registration = (true == f->logical().empty())?false:true;
    }
    
    // Check if the Registration SHould be Performed    
    bool catalog_enabled = ActionsConfig::instance().catalogEnabled();
    if((true == need_registration) && (true == catalog_enabled)){
        try{
            // Check if the Catalog Service Factory is registered
            CatalogServiceFactory::instance();
        } catch(const LogicError&){
            m_log_warn("Support Catalog Registration is on but no CatalogService plugin is registered");
            catalog_enabled = false;
        }
    }
    if((false == need_registration) || 
       (false == catalog_enabled)   || 
       (job.internalParams().find(JOB_PARAM_SKIP_REGISTRATION) != std::string::npos)){
        // Resolution is Disabled: Finish Files
        // For Each File, finish it
        StringArray::const_iterator it;
        for(it = files.begin(); it != files.end(); ++it){
            finishFile(*it,job);
        } // End For
        return;
    }
    // Do the Catalog Registration
    std::pair<std::string,std::string> src_catalog_data, dst_catalog_data;
    std::pair<std::string,std::string> src_srm, dst_srm;
    {
    // Set the Delegated Credentail Object
    UserProxyEnv cert(proxy_file);

    std::string source = job.source();
    if((true == job.srcCatalog().empty()) || 
       (true == job.srcCatalogType().empty())){
        // Get Source Catalog endpoint
        m_log_debug("Getting Catalog Endpoint for Source " << source);
        src_catalog_data = getCatalogEndpoint(source,voName());
        if(src_catalog_data.first.empty()){
            failAllFiles(files,Error::CONFIGURATION,Error::S_SOURCE,Error::P_CATALOG_REGISTRATION,"Source Catalog not found");
            return;
        } 
    } else {
        // Get Catalog Endpoint from the Job Properties
        src_catalog_data.first  = job.srcCatalog();
        src_catalog_data.second = job.srcCatalogType();
    }
    std::string destination = job.dest();
    if((true == job.destCatalog().empty()) || 
       (true == job.destCatalogType().empty())){
        // Get Destination Catalog endpoint
        m_log_debug("Getting Catalog Endpoint for Destination " << destination);
        dst_catalog_data = getCatalogEndpoint(destination,voName());
        if(dst_catalog_data.first.empty()){
            failAllFiles(files,Error::CONFIGURATION,Error::S_DESTINATION,Error::P_CATALOG_REGISTRATION,"Destination Catalog not found");
            return;
        }
    } else {
        // Get Catalog Endpoint from the Job Properties
        dst_catalog_data.first  = job.destCatalog();
        dst_catalog_data.second = job.destCatalogType();
    }
    m_log_debug("Source Catalog Endpoint is " << src_catalog_data.first);
    m_log_debug("Destination Catalog Endpoint is " << dst_catalog_data.first);
    try{
        // Assume all the files in the job are transferred from one SE to the other SE
        scoped_ptr<model::File> f(fileDao.get(*(files.begin())));
        
        // Get Source Endpoint
        try{
            src_srm = ActionsConfig::instance().getSrmEndpoint(f->sourceSurl(), source, "", job.voName());
        } catch(const std::exception& exc){
            m_log_warn("Failed to compute source SE endpoint for Job [" << job.id() << "]. Reason is " << exc.what());
        }
        
        // Get Destination Endpoint
        try{
            std::string tkn;
            if(false == job.spaceToken().empty()){
                tkn = job.spaceToken();
            } else if(false == job.storageClass().empty()){
                tkn = job.storageClass();
            }
            dst_srm = ActionsConfig::instance().getSrmEndpoint(f->destSurl(),destination, tkn, job.voName());
        } catch(const std::exception& exc){
            m_log_warn("Failed to compute destination SE endpoint for Job [" << job.id() << "]. Reason is " << exc.what());
        }
    } catch(const LogicError& exc){
        m_log_warn("Failed to compute destination SE for Job [" << job.id() << "]");
    }

    } // End UserProxyEnv

    // Prepare Bulk Request
    StringArray files_to_register;
    CatalogService::StringPairArray src_names;
    CatalogService::ReplicaStatArray replicas;
    std::map<std::string,std::string> logical_id_mapping;
    StringArray::const_iterator it;
    for(it = files.begin(); it != files.end(); ++it){
        // Get the File. Lock already acquired
        scoped_ptr<model::File> f(fileDao.get((*it)));
        if(f->state() == model::File::S_FINISHING){
            // Store Id of the File To Register
            files_to_register.push_back((*it));
            // Store LFN - SURL pairs. The SURL is normalized accordingly to the VO Policy
            src_names.push_back(CatalogService::StringPair(
                f->logical(),this->normalizeSurl(f->sourceSurl(),src_srm.first)));
            replicas.push_back(CatalogService::ReplicaStat(
                f->logical(),this->normalizeSurl(f->destSurl(),dst_srm.first)));
            // insert mapping
            logical_id_mapping.insert(
                std::map<std::string,std::string>::value_type(f->logical(),f->id()));
        }
    }
    if(files_to_register.empty()){
        m_log_info("No Files to register for Job [" << job.id() << "]");
        return;         
    }
    
    try{
        scoped_ptr<CatalogService> srcCatalogService;
        if(src_catalog_data.first != dst_catalog_data.first){
            m_log_debug("Differnet Catalog endpoints: Synchronize File Stats");
            // Get Source stats
            getSourceStats(files_to_register,logical_id_mapping,src_names,replicas,src_catalog_data.first,src_catalog_data.second,proxy_file);
        } else {
            m_log_debug("Same Catalog endpoint: Don't synchronize File Stats");
        }
        if(replicas.empty()){
            m_log_info("No Files to register for Job [" << job.id() << "]");
            return;         
        }
        m_log_debug("Registering " << replicas.size() << " files");
        
        // Create a Catalog Service Instance
        scoped_ptr<CatalogService> dstCatalogService(
            CatalogServiceFactory::instance().create(dst_catalog_data.first,dst_catalog_data.second,proxy_file));
        // Register the Files
        CatalogService::CatalogResult dstResult;
        try{
            dstCatalogService->registerSurls(replicas,dstResult);
        } catch(const CatalogLogicError& exc) {
            std::string reason = (std::string)"Logic Error in registering new "\
                " replicas. Reason is " + exc.what();
            m_log_warn(reason);
            failAllFiles(files_to_register,Error::CATALOG_INTERNAL_ERROR,Error::S_DESTINATION,Error::P_CATALOG_REGISTRATION,reason);
            return;
        } catch(const CatalogRemoteError& exc) {
            std::string reason = (std::string)"Remote Error in registering new "\
                " replicas to " + dst_catalog_data.first + ". Reason is " + exc.what();
            m_log_warn(reason);
            failAllFiles(files_to_register,Error::CONNECTION,Error::S_DESTINATION,Error::P_CATALOG_REGISTRATION,reason);
            return;
        }
        // Check Return Value
        if(CatalogService::CatalogResult::R_FAILED == dstResult.status){
            // All files failed
            std::string error_reason = (std::string)"Failed to Register Replica into " + 
                dst_catalog_data.first + " . Reason is " + dstResult.reason;
            m_log_warn(error_reason);
            failAllFiles(files_to_register,Error::CATALOG,Error::S_DESTINATION,Error::P_CATALOG_REGISTRATION,error_reason);
        } else if(CatalogService::CatalogResult::R_SOME_FAILURES == dstResult.status){
            // Some Files Failed
            CatalogService::StringPairArray::const_iterator failed_it;
            // For Each Failed File
            for(failed_it = dstResult.failures.begin(); failed_it != dstResult.failures.end(); ++failed_it){
                // Get the Id of the File 
                std::map<std::string,std::string>::iterator found = 
                                    logical_id_mapping.find((*failed_it).first);
                if(found != logical_id_mapping.end()){
                    // Faile the File
                    std::string error_reason = (std::string)"Failed to Register"\
                        " Replica into " + dst_catalog_data.first + 
                        " . Reason is " + (*failed_it).second;
                    failFile((*found).second,Error::CATALOG,Error::S_DESTINATION,Error::P_CATALOG_REGISTRATION,error_reason);
                    // Remove entry form the map
                    logical_id_mapping.erase(found);
                } else {
                    m_log_warn("Invalid LFN returned: " << (*failed_it).first);
                }
            }
            // For all the Other Files, complete them
            std::map<std::string,std::string>::iterator m_it;
            for(m_it = logical_id_mapping.begin(); m_it != logical_id_mapping.end(); ++m_it){
                m_log_info("Replica registered for File [" << (*m_it).first << "]");
                // Complete File
                finishFile((*m_it).second,job);
            }
        } else {
            // Registration succeeded
            m_log_debug("Finishing " << files.size() << " files");
            
            // For Each File, Log the registration of the new replica
            CatalogService::ReplicaStatArray::iterator rp_it;
            for(rp_it = replicas.begin(); rp_it != replicas.end(); ++rp_it){
                m_log_info("Replica " << (*rp_it).surl << " registered for File [" << (*rp_it).logical << "]");
            }
            
            // For Each File, signal the Registration Event
            for(it = files.begin(); it != files.end(); ++it){
                // Complete File
                finishFile((*it),job);
            }
        }
    } catch(const CatalogLogicError& exc){
        std::string reason = (std::string)"Catalog LogicError registering "\
            "new Replicas. Reason is " + exc.what();
        m_log_warn(reason);
        failAllFiles(files,Error::CATALOG_INTERNAL_ERROR,Error::S_DESTINATION,Error::P_CATALOG_REGISTRATION,reason);
    } catch(const CatalogRemoteError& exc){
        std::string reason = (std::string)"Remote Error in cantacting catalogs "\
            "for Replica registration. Reason is %s: " + exc.what();
        m_log_warn(reason);
        failAllFiles(files,Error::CONNECTION,Error::S_DESTINATION,Error::P_CATALOG_REGISTRATION,reason);
    } catch(const LogicError& exc){
        std::string reason = (std::string)"LogicError Replica registration: " 
            + exc.what();
        m_log_warn(reason);
        failAllFiles(files,Error::CONSISTENCY,Error::S_AGENT,Error::P_CATALOG_REGISTRATION, reason);
    }    
}

/*
 * finishFile
 *
 * Finalize a file
 */
void Finalize::finishFile(const std::string& file_id, const model::Job& job) /*throw(DAOException,LogicError)*/ {

    // Get File DAO
    FileDAO& fileDao = fileDAO();

    // Get the File. Lock already acquired
    scoped_ptr<model::File> f(fileDao.get(file_id));
    
    m_log_debug("Finishing File [" << file_id << "]");
                
    // Update file status
    try{
        fsm::FileFSM f_fsm(*f,job);
        
        // File Should be either Finishing, WaitingCatalogRegistration,
        // Finished, Canceled, Failed
        if(f->state() == model::File::S_FINISHING){
            // Change File State
            f_fsm.onEventFinished();
            m_log_info("File [" << f->id() << "] Finished");
        } else if(f->state() == model::File::S_WAITING_CATALOG_REGISTRATION){
            // Nothing To Do
        } else if((f->state() == model::File::S_FINISHED) ||
                  (f->state() == model::File::S_CANCELED) ||
                  (f->state() == model::File::S_FAILED)){
            // Nothing to Do
        } else {
            f_fsm.onEventFailed(Error::CONSISTENCY,Error::S_AGENT,Error::P_FINALIZATION,"File Not in Finishing state (Job is Finishing)");
        }
    } catch(const fsm::FSMException& exc){
        m_log_info("Failed to set File State. Reason is " << exc.what());
        failFile(file_id,Error::CONSISTENCY,Error::S_AGENT,Error::P_FINALIZATION,"Failed to set final state");
        return;
    }
    
    // Update File Transfer
    fileDao.update(*f);
}

/*
 * getSourceStats
 *
 * Add the Source Stat to the Replicas Array
 */
void Finalize::getSourceStats(const StringArray& files,
                                    std::map<std::string,std::string>& logical_id_mapping,
                                    CatalogService::StringPairArray& src_names,
                                    CatalogService::ReplicaStatArray& replicas,
                              const std::string& src_catalog_endpoint,
                              const std::string& src_catalog_type,
                              const std::string& proxy_file) /*throw (DAOException,LogicError)*/ {
    // Check pre conditions
    if((true == replicas.empty()) || (replicas.size() != src_names.size())){
        m_log_warn("Logical Names - Replicas mismatch");
        replicas.clear();
        failAllFiles(files,Error::CONSISTENCY, Error::S_AGENT,Error::P_CATALOG_REGISTRATION,"Consistency problem: Invalid number of replicas-logical names");
        return;
    }

    scoped_ptr<CatalogService> srcCatalogService;
    // Source and destination catalog endpoint are different:
    // Get Stats from Source File
    m_log_debug("Get Source Stats for " << src_names.size() << " files");
    srcCatalogService.reset(CatalogServiceFactory::instance().create(
        src_catalog_endpoint,src_catalog_type,proxy_file));
    // Get Stats
    CatalogService::FileStatArray stats;
    CatalogService::CatalogResult srcResult;
    try{
        srcCatalogService->getStats(src_names, stats, srcResult);
    } catch(const CatalogLogicError& exc) {
        std::string reason = (std::string)"Logic Error in getting Stat "\
            " info for source replicas. Reason is " + exc.what();
        m_log_warn(reason);
        replicas.clear();
        failAllFiles(files,Error::CATALOG_INTERNAL_ERROR,Error::S_SOURCE,Error::P_CATALOG_REGISTRATION, reason);
        return ;
    } catch(const CatalogRemoteError& exc) {
        std::string reason = (std::string)"Remote Error in getting Stat "\
            " info for source replicas. Reason is " + exc.what();
        m_log_warn(reason);
        replicas.clear();
        failAllFiles(files, Error::CONNECTION,Error::S_SOURCE,Error::P_CATALOG_REGISTRATION,  reason);
        return;
    }
    // Check teh Return Value
    if(CatalogService::CatalogResult::R_FAILED == srcResult.status){
        // All files failed
        m_log_warn("Failed to get Source File Stats. Reason is " << srcResult.reason);
        replicas.clear();
        failAllFiles(files,Error::CATALOG,Error::S_SOURCE,Error::P_CATALOG_REGISTRATION,srcResult.reason);
        return;
    }
    // Check post conditions
    if((true == stats.empty()) || (stats.size() != src_names.size())){
        m_log_warn("Invalid Number of Stats returned");
        replicas.clear();
        failAllFiles(files,Error::CATALOG,Error::S_SOURCE,Error::P_CATALOG_REGISTRATION,"Consistency problem: Invalid number of stats returned");
        return;
    }    
    if(CatalogService::CatalogResult::R_SOME_FAILURES == srcResult.status){
        // Some Files Failed
        m_log_warn("Failed to get some of the Source File Stats");
        // Some file failed: loop for each logical name and check if it failed
        CatalogService::StringPairArray::const_iterator f_it;
        for(f_it = srcResult.failures.begin(); 
            f_it != srcResult.failures.end(); ++f_it){
            // Get the Id of the File 
            std::map<std::string,std::string>::iterator found = 
                                logical_id_mapping.find((*f_it).first);
            if(found != logical_id_mapping.end()){
                const std::string& logical_name = (*f_it).first;
                // Fail the File
                std::string error_reason = (std::string)"Failed to Get"\
                    " FileStats from " + src_catalog_endpoint + 
                    " . Reason is " + (*f_it).second;
                failFile((*found).second,Error::CATALOG_INTERNAL_ERROR,Error::S_SOURCE,Error::P_CATALOG_REGISTRATION,error_reason);
                // Remove the logical name from the names, stats and 
                // replicas vectors
                CatalogService::StringPairArray::iterator n_it    = src_names.begin();
                CatalogService::FileStatArray::iterator stat_it   = stats.begin();
                CatalogService::ReplicaStatArray::iterator rep_it = replicas.begin();
                for(;n_it != src_names.end(); ++n_it,++stat_it,++rep_it){
                    if((*n_it).first == logical_name){
                        m_log_warn("Removing File [" << logical_name << "] from logicals");
                        src_names.erase(n_it);
                        m_log_warn("Removing File [" << logical_name << "] from stats");
                        stats.erase(stat_it);
                        m_log_warn("Removing File [" << logical_name << "] from replicas");
                        replicas.erase(rep_it);
                        break;
                    }
                }                    
                // Remove entry form the map
                logical_id_mapping.erase(found);
            } else {
                m_log_warn("Invalid LFN returned: " << (*f_it).first);
            }
        }
    }
    
    // Fill the ReplicaStats Array
    CatalogService::FileStatArray::const_iterator fs_it;
    CatalogService::ReplicaStatArray::iterator rp_it = replicas.begin();
    for(fs_it = stats.begin();fs_it != stats.end(); ++fs_it, ++rp_it){
        (*rp_it).stat = (*fs_it);
    }
} // end get source stats
