/*
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *  GLite Data Transfer Agents: Retry Class
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>, Paolo Tedesco <paolo.tedesco@cern.ch>
 *  Version info: $Id: Retry.cpp,v 1.15 2008/11/13 10:43:55 ptedesco Exp $
 *  Release: $Name:  $
 *
 */
#include "glite/data/transfer/agent/action/vo/Retry.h"
#include "ActionsConfig.h"
#include "FileFSM.h"
#include "JobFSM.h"
#include <glite/data/transfer/agent/dao/StringArrayTable.h>
#include <glite/data/agents/UserProxyEnv.h>
#include <glite/data/agents/AgentExceptions.h>
#include <glite/data/agents/cred/CredServiceException.h>

#include <boost/scoped_ptr.hpp>

using namespace glite::data::transfer::agent;
using namespace glite::data::transfer::agent::fsm;
using namespace glite::data::transfer::agent::action;
using namespace glite::data::transfer::agent::action::vo;

using glite::data::transfer::agent::dao::StringArrayTable;
using glite::data::agents::UserProxyEnv;
using glite::data::agents::LogicError;
using glite::data::agents::RuntimeError;
using glite::data::agents::dao::DAOException;
using glite::data::agents::cred::CredServiceException;
using boost::scoped_ptr;
using boost::shared_ptr;
using glite::data::transfer::agent::model::Error;

namespace {
const unsigned int CHUNK_SIZE = 1000;    
}
  
/*
 * Retry
 *
 * Constructor
 */
Retry::Retry(const std::string& name, RetryStrategy * s, CatalogRetryStrategy * cs):
    VOAction(name),m_strategy(s),m_catalogStrategy(cs){
}

/*
 * ~Retry
 *
 * Destructor
 */
Retry::~Retry(){
}

/*
 * execute
 *
 * Execute the Action
 */
void Retry::execute(const std::string& params) /*throw (LogicError,ExecuteException)*/{

    m_log_debug("Executing Retry Action");
           
    try{
        // Call the template method
        this->exec(params);
    } catch(const DAOException& exc){
        m_log_error("Failed to Execute Retry Action (DAOException). Reason is " << exc.what());
        throw ExecuteException("Some problem found in interacting with DataSource");
    } catch(const ExecuteException& exc){
        m_log_error("Failed To Execute Action. Reason is " << exc.what());
        throw;
    } catch(const LogicError& exc){
        m_log_error("LogicError in Executing Action. Reason is " << exc.what());
        throw ;
    } catch(const std::exception& exc){
        m_log_error("Failed to execute action. Reason is " << exc.what());
        throw;
    } catch(...){
        m_log_error("Cannot execute action. Unknown reason");
        throw;
    }      
    m_log_debug("Action Executed successfully");
}

/*
 * exec
 *
 * Execute the Action
 */
void Retry::exec(const std::string& params) /*throw (DAOException,LogicError,ExecuteException)*/{
    
    // Retry Transfer that failed
    retryTransferFailed();
    // Retry Catalog Failures
    retryCatalogFailed();
    // Retry Prestage Failures
    retryPrestageFailed();
}

/*
 * retryTransferFailed
 *
 * Execute Retry Strategy for Transfer that failed
 */
void Retry::retryTransferFailed() /*throw (DAOException,LogicError,ExecuteException)*/{

    // Get File DAO
    FileDAO& fileDao = fileDAO();
    
    // Get All the Files In Waiting State. 
    StringArray files;
    fileDao.getWaitingFiles(files,CHUNK_SIZE);
    
    m_log_debug("Got " << files.size() << " Waiting Files");
        
    try{
        // Start Transaction
        context().start();
        
        // For Each Transfer
        StringArray::iterator it;
        for(it = files.begin(); it != files.end(); ++it){
            // Process the file
            processFile((*it));
        } // End for
    
        // Commit Transaction
        context().commit();
    } catch(...){
        // Rollback Transaction
        context().rollback();
        throw;
    }
    // Don't Update Jobs: Job State should remain Active in order to avoid race 
    // conditions between VO and Channel Agents
}

/*
 * processFile
 *
 * Process a File which transfer failed
 */
void Retry::processFile(const std::string& file_id) /*throw (DAOException,LogicError)*/ {
    
    // Get Job DAO
    JobDAO& jobDao = jobDAO();

    // Get File DAO
    FileDAO& fileDao = fileDAO();
    
    // Get Transfer DAO
    TransferDAO& transferDao = transferDAO();
    
    try{
        // Get the file
        scoped_ptr<model::File> f(fileDao.get(file_id));
        // Check Precondition
        if(model::File::S_WAITING != f->state()){
            // File not in waiting state: ignore it
            return;
        }

        m_log_debug("File [" << f->id() << "] has " << f->currentFailures() << " failures in current cycle");

        // Get The Job
        scoped_ptr<model::Job> job(jobDao.get(f->jobId()));
        
        // Check if job has to be canceled
        if(true == job->cancelJob()){
            FileFSM f_fsm(*f,*job);
            f_fsm.onEventCanceled();
            m_log_debug("File [" << f->id() << "] has been canceled");
        } else {
            // Get the Transfers
            StringArray reqIds;
            transferDao.getByFileId(file_id,reqIds,f->currentFailures());
            RetryStrategy::TransferArray transfers;
            for(StringArray::iterator t_it = reqIds.begin(); t_it != reqIds.end(); ++t_it){
                // Get Transfer
                shared_ptr<model::Transfer> t(transferDao.get(file_id,(*t_it)));
                transfers.push_back(t);    
            }
            
            m_log_debug("Check File [" << f->id() << "] - " << f->jobId() << " for resubmission");
    
            RetryStrategy::RetryResult result;
            std::string error_reason;
            Error::Category error_class = Error::UNDEF;
            try{
                // Get Proxy Certificate
                std::string proxy_file;
                getProxyForJob(*job,proxy_file);
                // Set the Delegated Credentail Object
                UserProxyEnv cert(proxy_file);
                // Apply the Retry Strategy
                result = m_strategy->apply(*job,*f,transfers);
            } catch(const CredServiceException& exc){
                m_log_warn("Failed to get Proxy Certificate for Job [" << f->id() << "]. Reason is " << exc.what());
                result = RetryStrategy::FAIL; 
                error_class = Error::SECURITY;
                error_reason = exc.what();
            } catch(const ExecuteException& exc){
                m_log_error("Retry Strategy failed with Error " << exc.what());
                result = RetryStrategy::FAIL; 
                error_class = Error::INTERNAL_ERROR;
                error_reason = exc.what();
            }
            FileFSM f_fsm(*f,*job);
            switch(result){
                case RetryStrategy::WAIT:
                    // Don't do anything
                    m_log_debug("Leave File [" << f->id() << "] in Waiting state");
                break;
                case RetryStrategy::RETRY:
                case RetryStrategy::RETRY_WITH_OVERWRITE:{
                    // Reschedule the File
                    m_log_info("Reschedule File [" << f->id() << "]. Number of tries already performed: " << transfers.size());
                    bool overwrite = false;
                    if(result == RetryStrategy::RETRY_WITH_OVERWRITE){
                       m_log_info("Overwrite file [" << f->id() << "] enabled");
                        overwrite = true;
                    }
                    f_fsm.onEventReschedule(overwrite);
                } break;
                case RetryStrategy::HOLD:{
                    // Too Many Failures = > Put it in HOLD State
                    m_log_info("Too many failures (" << f->failures() << ") for File [" << f->id() << "]: Move it to HOLD");
                    f_fsm.onEventHold();
                } break;
                case RetryStrategy::FAIL:
                default:{
                    // Error Processing File = > Put it in FAILED State
                    m_log_info("File [" << f->id() << "] failed after " << f->failures() << " attempts");
                    if (Error::UNDEF != error_class){
                        f_fsm.onEventFailed(error_class,Error::S_AGENT,Error::P_WAITING,error_reason);
                    } else {
                        // set failed state leaving current category/scope/phase
                        f_fsm.onEventFailed(f->reasonClass(),f->errorScope(),f->errorPhase(),"File will not be retried.");
                    }
                } break;
            }
        } // End if(job->cancelJob())
        // Update File
        fileDao.update(*f);
    } catch(const LogicError& exc){
        m_log_warn("LogicError in Evaluating Retry Logic for for File [" << file_id << "]. Reason is " << exc.what());
        failFile(file_id,Error::CONSISTENCY,Error::S_AGENT,Error::P_WAITING,exc.what());
    } catch(const DAOException& exc){
        m_log_error("DAOError in Evaluating Retry Logic for File [" << file_id << "]. Reason is " << exc.what());
        throw;
    }
}

/*
 * retryCatalogFailed
 *
 * Execute Retry Strategy for File that failed during Catalog Interaction
 */
void Retry::retryCatalogFailed() /*throw (DAOExcepion,LogicError,ExecuteException)*/{

    // Get File DAO
    FileDAO& fileDao = fileDAO();
    
    // Get All the Files In Waiting State. 
    StringArrayTable files;
    fileDao.getCatalogWaitingFiles(files,CHUNK_SIZE);
    
    m_log_debug("Got " << files.size() << " Catalog Waiting Files");
        
    StringArray jobs;
    try{
        // Start Transaction
        context().start();
        
        // For Each Transfer
        StringArrayTable::iterator it;
        for(it = files.begin(); it != files.end(); ++it){
            const std::string& job_id = (*it).first;
            const StringArray& files = (*it).second;
            
            jobs.push_back(job_id);
            // Process the Job
            processCatalogFailedJob(job_id,files);
        } // End for
        
        // Update Job States
        this->updateJobs(jobs);
        
        // Commit Transaction
        context().commit();
    } catch(...){
        // Rollback Transaction
        context().rollback();
        throw;
    }
}


/*
 * processCatalogFailedJob
 *
 * process a job with CatalogFailed files
 */
void Retry::processCatalogFailedJob(const std::string& job_id, const StringArray& file_ids) /*throw (DAOException,LogicError)*/ {
    
    // Get Job DAO
    JobDAO& jobDao = jobDAO();

    // Get File DAO
    FileDAO& fileDao = fileDAO();
    
    try{
        // Get Job Instance
        scoped_ptr<model::Job> job(jobDao.get(job_id));
        
        // Get the Files
        CatalogRetryStrategy::FileArray files_c;
        std::vector<shared_ptr<model::File> > files;
        for(StringArray::const_iterator f_it = file_ids.begin(); f_it != file_ids.end(); ++f_it){
            // Get Transfer
            shared_ptr<model::File> f(fileDao.get(*f_it));
            // Check Precondition
            if((model::File::S_WAITING_CATALOG_RESOLUTION   == f->state()) ||
               (model::File::S_WAITING_CATALOG_REGISTRATION == f->state())){
                   
                // Check if job has to be canceled
                if(true == job->cancelJob()){
                    FileFSM f_fsm(*f,*job);
                    f_fsm.onEventCanceled();
                    // Update File
                    fileDao.update(*f);
                    m_log_debug("File [" << f->id() << "] has been canceled");
                }  else {
                    // Take only files in Catalog waiting state
                    files_c.push_back(f);
                    files.push_back(f);
                }
            }
        }
        if(true == files.empty()){
            m_log_debug("Job [" << job->id() << "] has no Files to Retry");
            return;
        }
        
        CatalogRetryStrategy::CatalogRetryResult result;
        std::string error_reason;
        Error::Category error_class = Error::UNDEF;
        try{
            // Get Proxy Certificate
            std::string proxy_file;
            getProxyForJob(*job,proxy_file);
            // Set the Delegated Credentail Object
            UserProxyEnv cert(proxy_file);
            // Apply the Retry Strategy
            result = m_catalogStrategy->apply(*job,files_c);
        } catch(const CredServiceException& exc){
            m_log_warn("Failed to get Proxy Certificate for Job [" << job->id() << "]. Reason is " << exc.what());
            result = CatalogRetryStrategy::FAIL; 
            error_class = Error::SECURITY;
            error_reason = exc.what();
        } catch(const ExecuteException& exc){
            m_log_error("Retry Strategy failed with Error " << exc.what());
            result = CatalogRetryStrategy::FAIL; 
            error_class = Error::INTERNAL_ERROR;
            error_reason = exc.what();
        }
        switch(result){
            case CatalogRetryStrategy::WAIT:
                // Don't do anything
                m_log_debug("Leave Job [" << job->id() << "] in Catalog Waiting state");
            break;
            case CatalogRetryStrategy::RETRY:{
                // Reschedule the File
                m_log_info("Reschedule " << files.size() << " files of Job [" << job->id() << "] for Catalog interaction");
                // For Each File      
                std::vector<shared_ptr<model::File> >::iterator it;
                for(it = files.begin(); it != files.end(); ++it){
                    FileFSM f_fsm(*(*it),*job);
                    f_fsm.onEventReschedule();
                    // Update File
                    fileDao.update(*(*it));
                }
            } break;
            case CatalogRetryStrategy::FAIL:
            default:{
                // Error Processing Job = > Put all files in FAILED State
                m_log_info("Job [" << job->id() << "] should not retry Catalog interaction");
                failAllFiles(file_ids,error_class,Error::S_AGENT,Error::P_WAITING_CATALOG,error_reason);
            } break;
        }
    } catch(const LogicError& exc){
        m_log_warn("LogicError in Evaluating Catalog Retry Logic for Job [" << job_id << "]. Reason is " << exc.what());
        failAllFiles(file_ids,Error::CONSISTENCY,Error::S_AGENT,Error::P_WAITING_CATALOG,exc.what());
    } catch(const DAOException& exc){
        m_log_error("DAOError in Evaluating Catalog Retry Logic for Job [" << job_id << "]. Reason is " << exc.what());
        throw;
    }
}

/*
 * retryPrestageFailed
 *
 * Execute Retry Strategy for File that failed during Prestaging
 */
void Retry::retryPrestageFailed() /*throw (DAOException,LogicError,ExecuteException)*/{
#warning "TODO : implement retry logic for failed prestage."
}
