/*
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *  GLite Data Transfer Agents: VO VOAction Class
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>, Paolo Tedesco <paolo.tedesco@cern.ch>
 *  Version info: $Id: VOAction.cpp,v 1.13 2008/11/13 10:43:55 ptedesco Exp $
 *  Release: $Name:  $
 *
 */
#include <glite/data/transfer/agent/action/vo/VOAction.h>
#include "FileFSM.h"
#include "ActionsConfig.h"
#include <glite/data/transfer/agent/dao/vo/DAOFactory.h>
#include <glite/data/agents/cred/cred-utility.h>
#include <glite/data/agents/cred/CredServiceException.h>
#include <glite/data/agents/sd/SDConfig.h>
#include <glite/data/agents/sd/sd-utility.h>
#include <glite/data/transfer/agent/catalog/CatalogServiceFactory.h>
#include "dao-utility.h"
#include <boost/scoped_ptr.hpp>

using namespace glite::data::transfer::agent;
using namespace glite::data::transfer::agent::fsm;
using namespace glite::data::transfer::agent::action;
using namespace glite::data::transfer::agent::action::vo;

using glite::data::transfer::agent::dao::vo::DAOFactory;
using glite::data::transfer::agent::dao::translateFileState;
using glite::data::agents::dao::DAOContext;
using glite::data::agents::dao::DAOException;
using glite::data::agents::dao::DAOLogicError;
using glite::data::agents::LogicError;
using glite::data::agents::RuntimeError;
using glite::data::agents::AgentException;
using boost::scoped_ptr;

/*
 * VOAction
 *
 * Constructor
 */
VOAction::VOAction(const std::string& name):
        m_logger((std::string)"vo-action-" + name),
        m_context(0){
    m_log_trace("VOAction " << name << " created");
}

/*
 * ~VOAction
 *
 * Destructor
 */
VOAction::~VOAction(){
    m_log_trace("VOAction destroyed");
}

/*
 * context
 *    
 * set DAOContext
 */
void VOAction::context(const std::string& vo_name, DAOContext& ctx){
    m_voName = vo_name;
    m_context = &(ctx);
}

/*
 * context
 *    
 * Get DAOContext
 */
DAOContext& VOAction::context() /*throw (LogicError)*/{
    if(0 == m_context){
        throw LogicError("null context");
    }        
    return *m_context;
}

/*
 * jobDAO
 *    
 * Get a JobDAO Instance
 */
VOAction::JobDAO& VOAction::jobDAO() /*throw (LogicError)*/{
    if(0 == m_jobDAO.get()){
        // Get DAO Instances
        m_jobDAO.reset(DAOFactory::instance().createJobDAO(m_voName,context()));
    }
    
    return *m_jobDAO;
}

/*
 * fileDAO
 *    
 * Get a FileDAO Instance
 */
VOAction::FileDAO& VOAction::fileDAO() /*throw (LogicError)*/{
    if(0 == m_fileDAO.get()){
        // Get DAO Instances
        m_fileDAO.reset(DAOFactory::instance().createFileDAO(m_voName,context()));
    }
    
    return *m_fileDAO;
}

/*
 * transferDAO
 *    
 * Get a TransferDAO Instance
 */
VOAction::TransferDAO& VOAction::transferDAO() /*throw (LogicError)*/{
    if(0 == m_transferDAO.get()){
        // Get DAO Instances
        m_transferDAO.reset(DAOFactory::instance().createTransferDAO(m_voName,context()));
    }
    
    return *m_transferDAO;
}
/*
 * stageReqDAO
 *
 * Get a StageReqDAO Instance
 */
VOAction::StageReqDAO& VOAction::stageReqDAO() /*throw (LogicError)*/{
    if(0 == m_stageReqDAO.get()){
        // Get DAO Instances
        m_stageReqDAO.reset(DAOFactory::instance().createStageReqDAO(m_voName,context()));
    }
    
    return *m_stageReqDAO;
}
/*
 * channelDAO
 *
 * Get a ChannelDAO Instance
 */
VOAction::ChannelDAO& VOAction::channelDAO() /*throw (LogicError)*/{
    if(0 == m_channelDAO.get()){
        // Get DAO Instances
        m_channelDAO.reset(DAOFactory::instance().createChannelDAO(m_voName,context()));
    }
    
    return *m_channelDAO;
}

/*
 * execute
 *
 * Execute the Action
 */
void VOAction::execute(const std::string& params) /*throw (LogicError, ExecuteException)*/{

    m_log_debug("Executing Action");
            
    try{
        // Start Transaction
        context().start();
        try{
            // Call the template method
            this->exec(params);
            // Commit Transaction
            context().commit();
        } catch(...){
            // RollBack Transaction
            context().rollback();
            throw;
        }
    } catch(const DAOException& exc){
        m_log_error("DAO Error in Executing Action. Reason is " << exc.what());
        throw ExecuteException("Some problem found in interacting with DataSource");
    } catch(const ExecuteException& exc){
        m_log_error("Failed To Execute Action. Reason is " << exc.what());
        throw;
    } catch(const RuntimeError& exc){
        m_log_error("Failed To Execute Action (RuntimeError). Reason is " << exc.what());
        throw ExecuteException(exc.what());
    } catch(const LogicError& exc){
        m_log_error("LogicError in Executing Action. Reason is " << exc.what());
        throw;
    } catch(const AgentException& exc){
        m_log_error("Unrecognized Exception in Executing Action. Reason is " << exc.what());
        throw;
    } catch(const std::exception& exc){
        m_log_error("Unknown Error in executing action. Reason is " << exc.what());
        throw;
    } catch(...){
        m_log_error("Cannot execute action. Unknown reason");
        throw;
    }
    m_log_debug("Action Executed successfully");
}

/*
 * updateJobs
 *
 * Update Jobs States
 */
void VOAction::updateJobs(const std::set<std::string>& ids) /*throw (DAOLogicError, DAOException)*/{

    // Convert set into an array
    agents::dao::StringArray  jobIds;
    jobIds.insert(jobIds.begin(),ids.begin(),ids.end());

    this->updateJobs(jobIds);
}

/*
 * updateJobs
 *
 * Update Jobs States
 */
void VOAction::updateJobs(const agents::dao::StringArray& ids) /*throw (DAOLogicError, DAOException)*/{

    if(!ids.empty()){
        // Get Job DAO
        JobDAO& jobDao = jobDAO();
            
        m_log_debug("Updating Jobs' State");
        try{
            // Update job States
            jobDao.updateStates(ids);
        } catch(const DAOException& exc){
            m_log_error("Failed To Update Jobs states: " << exc.what());
            throw;
        } catch(const DAOLogicError& exc){
            m_log_error("LogicError in updating Jobs states: " << exc.what());
            throw;
        }
        m_log_debug("Jobs' State Updated");
    }
}

/*
 * failFile
 *
 * Send a Fail Event a File
 */
void VOAction::failFile(
    const std::string&              file, 
    const model::Error::Category&   category, 
    const model::Error::Scope&      scope, 
    const model::Error::Phase&      phase, 
    const std::string&              reason) /*throw ()*/{
        
    // Get File DAO
    FileDAO& fileDao = fileDAO();
    // Get Job DAO
    JobDAO& jobDao   = jobDAO();
        
    try {
        // Get the File
        scoped_ptr<model::File> f(fileDao.get(file));
        
        // Get The Job
        scoped_ptr<model::Job> job(jobDao.get(f->jobId()));
        
        m_log_error("Failing File [" << f->id() << "]. This File had " << f->failures() << " failures");
        
        // Change Transfer State
        FileFSM f_fsm(*f,*job);
        f_fsm.onEventFailed(category,scope,phase,reason);

        // Update File Object
        fileDao.update(*f);

        // Log File State Transition        
        const char * state_str = "";
        try{
            state_str = translateFileState(f->state());
        }catch(...){}
        m_log_error("File [" << f->id() << "]: State is " << f->state() << " (" << state_str << "). Failures: " << f->failures());
    } catch(const DAOException& exc){
        m_log_error("Cannot get or update File instance: " << exc.what());
    } catch(const DAOLogicError& exc){
        m_log_error("Cannot get or update File instance (LogicError): " << exc.what());
    }
}

/**
 * Send a Fail Event for a file
 */
void VOAction::failFile(
    model::File&                    file, 
    const model::Error::Category&   category, 
    const model::Error::Scope&      scope, 
    const model::Error::Phase&      phase, 
    const std::string&              reason) /*throw ()*/{

    // Get File DAO
    FileDAO& fileDao = fileDAO();
    // Get Job DAO
    JobDAO& jobDao   = jobDAO();
    try {
        // Get The Job
        scoped_ptr<model::Job> job(jobDao.get(file.jobId()));
        m_log_error("Failing File [" << file.id() << "]. This File had " << file.failures() << " failures");
        
        // Change Transfer State
        FileFSM f_fsm(file,*job);
        f_fsm.onEventFailed(category,scope,phase,reason);

        // Update File Object
        fileDao.update(file);

        // Log File State Transition        
        const char * state_str = "";
        try{
            state_str = translateFileState(file.state());
        }catch(...){}
        m_log_error("File [" << file.id() << "]: State is " << file.state() << " (" << state_str << "). Failures: " << file.failures());
    } catch(const DAOException& exc){
        m_log_error("Cannot get or update File instance: " << exc.what());
    } catch(const DAOLogicError& exc){
        m_log_error("Cannot get or update File instance (LogicError): " << exc.what());
    }

        
}


/*
 * failAllFiles
 *
 * Send a Fail Event for all the files
 */
void VOAction::failAllFiles(
    const std::vector<std::string>& files, 
    const model::Error::Category&   category, 
    const model::Error::Scope&      scope, 
    const model::Error::Phase&      phase, 
    const std::string&              reason) /*throw ()*/{
        
    // Get File DAO
    FileDAO& fileDao = fileDAO();
    // Get Job DAO
    JobDAO& jobDao   = jobDAO();

    std::string job_id;
    scoped_ptr<model::Job> job;
    
    // For Each File
    std::vector<std::string>::const_iterator it;
    for(it = files.begin(); it != files.end(); ++it){
        try {
            // Get the File
            scoped_ptr<model::File> f(fileDao.get((*it)));
            // Get the Job
            if((0 == job.get()) || (job_id != f->jobId())){
                job_id = f->jobId();
                job.reset(jobDao.get(job_id));
            }
            m_log_error("Failing File [" << f->id() << "]. This File had " << f->failures() << " failures");
            
            // Change Transfer State
            FileFSM f_fsm(*f,*job);
            f_fsm.onEventFailed(category,scope,phase,reason);

            // Update File Object
            fileDao.update(*f);
            
            // Log File State Transition        
            const char * state_str = "";
            try{
                state_str = translateFileState(f->state());
            }catch(...){}
            m_log_error("File [" << f->id() << "]: State is " << f->state() << " (" << state_str << "). Failures: " << f->failures());
        } catch(const DAOException& exc){
            m_log_error("Cannot get or update File instance: " << exc.what());
            // Continue with the next one
        } catch(const DAOLogicError& exc){
            m_log_error("Cannot get or update File instance (LogicError): " << exc.what());
            // Continue with the next one
        }
    } // End For
}

/*
 * getProxyForJob
 *
 * Get Proxy for a job
 */
void VOAction::getProxyForJob(model::Job& job, std::string& proxy_file) /* throw (AgentException)*/ {
    return ActionsConfig::instance().getProxyForJob(job,context(),proxy_file);
}

/*
 * normalizeSurl
 *
 * Normalize a Surl accordingly with the VO policy
 */
std::string VOAction::normalizeSurl(const std::string& surl, const std::string& srm_endpoint) /* throw()*/{
    return ActionsConfig::instance().normalizeSurl(surl,srm_endpoint);
}


/*
 * get_catalog_endpoint
 *
 * Get the endpoint of the Catalog on the same site of the given se
 */
std::pair<std::string,std::string> VOAction::getCatalogEndpoint(const std::string& source, const std::string& vo_name) /* throw(LogicError)*/{
    
    std::pair<std::string,std::string> catalog_data;
    
    const std::string& srm_service_type = agents::sd::SDConfig::instance().srm();
    // Get Site Name
    std::string site = agents::sd::get_site_name(source,srm_service_type,vo_name);

    try{
        // Get Catalog Endpoint
        catalog_data = 
            catalog::CatalogServiceFactory::instance().getCatalogEndpoint(site,vo_name);
    } catch(const agents::LogicError& exc){
        m_log_error("No catalog endpoint returned: " << exc.what());
        throw;
    }
    return catalog_data;
}
