/*
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *  GLite Data Transfer Agents: CatalogServiceFactory Class
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *  Version info: $Id: CatalogServiceFactory.cpp,v 1.2 2006/08/14 08:38:28 badino Exp $
 *  Release: $Name:  $
 *
 */
#include "glite/data/transfer/agent/catalog/CatalogServiceFactory.h"
#include "glite/data/transfer/agent/catalog/CatalogServiceException.h"
#include "glite/data/agents/AgentExceptions.h"
#include "glite/data/agents/sd/SDConfig.h"
#include "glite/data/agents/sd/sd-utility.h"

#include <boost/scoped_ptr.hpp>

using namespace glite::data::transfer::agent::catalog;

/*
 * Initialize Static Attirbutes
 */
CatalogServiceFactory *  CatalogServiceFactory::s_instance = 0;


/*
 * getInstance
 *
 * Return the registered Instance
 */
CatalogServiceFactory& CatalogServiceFactory::instance() /*throw (LogicError)*/ {
    if(0 == s_instance){
        throw glite::data::agents::LogicError("no CatalogServiceFactory configured");
    }
    return *s_instance;
}

/*
 * CatalogServiceFactory
 *
 * Constructor
 */
CatalogServiceFactory::CatalogServiceFactory(){
    // Register the instance
    if(0 == s_instance){
        s_instance = this;
    }
}
                                                                                                                          
/*
 * ~CatalogServiceFactory
 *
 * Destructor
 */
CatalogServiceFactory::~CatalogServiceFactory(){
    if(this != s_instance){
        s_instance = 0;
    }
}

/*
 * getCatalogEndpoint
 *
 * Return the endpoint and the type of the CatalogService to use. The 
 * default implementation uses in the information system and apply the 
 * following rules: first looks up in a Catalog of the type returned by 
 * localCatalogServiceType exists on the given site; if not retunr the first
 * instance of the type returned by globalCatalogServiceType
 */
std::pair<std::string,std::string> CatalogServiceFactory::getCatalogEndpoint(const std::string& site, const std::string& vo_name) /* throw(LogicError)*/{

    std::pair<std::string,std::string> catalog_data;
    
    if(false == site.empty()){
        // Get Local Catalog Service Type
        std::string localCatalogType = this->localCatalogServiceType();
        if(false == localCatalogType.empty()){
            // Get Local Catalog Endpoint
            boost::scoped_ptr<agents::sd::Service> service(
                agents::sd::get_service(site,localCatalogType,vo_name));
            if(0 != service.get()){
                catalog_data.first  = service->endpoint;
                catalog_data.second = localCatalogType;
            }
        }
    }
    if(true == catalog_data.first.empty()){
        // Get Global Catalog Service Type
        std::string globalCatalogType = this->globalCatalogServiceType();
        if(false == globalCatalogType.empty()){
            // Get Global Catalog Endpoint
            boost::scoped_ptr<agents::sd::Service> service(
                agents::sd::get_service_by_type(globalCatalogType,vo_name));
            if(0 != service.get()){
                catalog_data.first  = service->endpoint;
                catalog_data.second = globalCatalogType;
            }
        }
    }
    if(true == catalog_data.first.empty()){
        // No Catalog at all. Cannot Register that file
        throw agents::LogicError("No catalog available in the information system");
    }
    return catalog_data;    
}
