/**
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *  GLite Data Transfer Agents: CatalogConfig Class
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *           Paolo Tedesco <paolo.tedesco@cern.ch>
 *  Version info: $Id: CatalogConfig.cpp,v 1.6 2008/11/13 10:43:55 ptedesco Exp $ 
 *  Release: $Name:  $
 *
 */
#include "CatalogConfig.h"
#include "glite/data/agents/python/PyHelper.h"
#include "glite/data/agents/python/PythonException.h"

using namespace glite::data::transfer::agent::catalog;
using namespace glite::data::agents::python;

namespace py = boost::python;

/*
 * Initialize Constants
 */
const char * const CatalogConfig::CATALOG_VERSION_FUNCTION_NAME                   = "CatalogPluginVersion";
const char * const CatalogConfig::CATALOG_V_1_0_GLOBAL_TYPE_FUNCTION_NAME         = "GlobalCatalogType";
const char * const CatalogConfig::CATALOG_V_1_0_LOCAL_TYPE_FUNCTION_NAME          = "LocalCatalogType";
const char * const CatalogConfig::CATALOG_V_1_0_CHECK_PERMISSIONS_FUNCTION_NAME   = "CheckPermissions";
const char * const CatalogConfig::CATALOG_V_1_0_LIST_SURLS_FUNCTION_NAME          = "ListSurls";
const char * const CatalogConfig::CATALOG_V_1_0_CHECK_SURLS_FUNCTION_NAME         = "CheckSurls";
const char * const CatalogConfig::CATALOG_V_1_0_GET_STATS_FUNCTION_NAME           = "GetStats";
const char * const CatalogConfig::CATALOG_V_1_0_REGISTER_SURLS_FUNCTION_NAME      = "RegisterSurls";
const char * const CatalogConfig::CATALOG_V_1_0_GET_ENDPOINT_FUNCTION_NAME        = "GetEndpoint";
const char * const CatalogConfig::CATALOG_V_1_0_INIT_FUNCTION_NAME                = "InitCatalogPlugin";
const char * const CatalogConfig::CATALOG_V_1_0                                   = "1.0";


/*
 * instance
 *
 * Return the singleton Instance
 */
CatalogConfig& CatalogConfig::instance() {
    static CatalogConfig the_instance;
    return the_instance;
}

/*
 * validateRetry
 *
 * Verify that the Retry module is fine and get the versions
 */
bool CatalogConfig::validate() /*throw()*/ {
    
    bool result = false;
    PyHelper& py_helper = PyHelper::instance();
    
    try{
        // Load the Module
        m_module = py_helper.import(m_catalogModule);
        m_log_info("Catalog Module " << m_catalogModule << " Loaded");
                
        // Get the API Versison
        py::object get_version = py_helper.getFunction(m_module, CATALOG_VERSION_FUNCTION_NAME);

        // Call Get Version
        m_catalogVersion = py::call<std::string>(get_version.ptr());
        m_log_info("Catalog Version is " << m_catalogVersion);
        
        // Based on the version, check that the required functions are 
        // implemented
        if(m_catalogVersion == CATALOG_V_1_0){
            // Get the function
            m_checkPermissions = py_helper.getFunction(m_module,CATALOG_V_1_0_CHECK_PERMISSIONS_FUNCTION_NAME);
            m_log_info("Got Catalog Function <" << CATALOG_V_1_0_CHECK_PERMISSIONS_FUNCTION_NAME << ">");
            m_listSurls = py_helper.getFunction(m_module,CATALOG_V_1_0_LIST_SURLS_FUNCTION_NAME);
            m_log_info("Got Catalog Function <" << CATALOG_V_1_0_LIST_SURLS_FUNCTION_NAME << ">");
            m_checkSurls = py_helper.getFunction(m_module,CATALOG_V_1_0_CHECK_SURLS_FUNCTION_NAME);
            m_log_info("Got Catalog Function <" << CATALOG_V_1_0_CHECK_SURLS_FUNCTION_NAME << ">");
            m_getStats = py_helper.getFunction(m_module,CATALOG_V_1_0_GET_STATS_FUNCTION_NAME);
            m_log_info("Got Catalog Function <" << CATALOG_V_1_0_GET_STATS_FUNCTION_NAME << ">");
            m_registerSurls = py_helper.getFunction(m_module,CATALOG_V_1_0_REGISTER_SURLS_FUNCTION_NAME);
            m_log_info("Got Catalog Function <" << CATALOG_V_1_0_REGISTER_SURLS_FUNCTION_NAME << ">");
            try{
                m_getEndpoint = py_helper.getFunction(m_module,CATALOG_V_1_0_GET_ENDPOINT_FUNCTION_NAME);
                m_hasGetEndpoint = true;
                m_log_info("Got Catalog Function <" << CATALOG_V_1_0_GET_ENDPOINT_FUNCTION_NAME << ">");
            } catch(...){
                m_hasGetEndpoint = false;
                m_log_info("No Catalog Function <" << CATALOG_V_1_0_GET_ENDPOINT_FUNCTION_NAME << "> found: use the default");
            }
            // Module is OK
            result = true;
        }
    } catch(const PythonException& exc){
        // Module Validation Failed
        m_log_info("Catalog Plugin validation failed. Error is " << exc.what());
    } catch(const std::exception& exc){
        // Module Validation Failed
        m_log_info("Catalog Plugin validation failed. Error is " << exc.what());
    } catch(py::error_already_set){
        std::pair<std::string,std::string> err = py_helper.lastError();
        m_log_error("Catalog Plugin validation failed. Error is " << err.first << " - " << err.second);
    } catch(...){
        m_log_error("Catalog Plugin validation failed. Unknown reason");
    }
    if(true == result){
        m_log_info("Catalog Plugin validation completed successfully");
    }
    
    return result;
}


/*
 * init
 *
 * Initialize the Retry module
 */
bool CatalogConfig::init(const std::string& params) /*throw()*/ {
    bool result = false;
    
    PyHelper& py_helper = PyHelper::instance();
    
    // Based on the version, call the right method
    if(m_catalogVersion == CATALOG_V_1_0){
        try{
            // Get the Init function
            py::object init_function = py_helper.getFunction(m_module,CATALOG_V_1_0_INIT_FUNCTION_NAME);
            m_log_info("Got Init Function <" << CATALOG_V_1_0_INIT_FUNCTION_NAME << ">");
            
            // Call Get Init Function
            result = py::call<bool>(init_function.ptr(),params);
            m_log_debug(CATALOG_V_1_0_INIT_FUNCTION_NAME << " returned " << str_bool(result));
        } catch(const PythonException& exc){
            // Module Initialization Failed
            m_log_warn("No InitRetry Method (" << exc.what() << "). Continue without initialization");
            result = true;
        } catch(const std::exception& exc){
            // Module Initialization Failed
            m_log_error("Catalog Plugin initialization failed. Error is " << exc.what());
        } catch(py::error_already_set){
            std::pair<std::string,std::string> err = py_helper.lastError();
            m_log_error("Catalog Plugin initialization failed. Error is " << err.first << " - " << err.second);
        } catch(...){
            m_log_error("Catalog Plugin initialization failed. Unknown reason");
        }
    }
    if(true == result){
        m_log_info("Catalog Plugin Version is " << m_catalogVersion);
        
        try{
            // Get the Global Catalog Type function
            py::object type_function = py_helper.getFunction(m_module,CATALOG_V_1_0_GLOBAL_TYPE_FUNCTION_NAME);
            m_log_info("Got GetGlobalType Function <" << CATALOG_V_1_0_GLOBAL_TYPE_FUNCTION_NAME << ">");
            
            // Call Get Global Catalog Type Function
            m_globalCatalogType = py::call<std::string>(type_function.ptr());
            m_log_info("Global Catalog Type is <" << CATALOG_V_1_0_GLOBAL_TYPE_FUNCTION_NAME << ">");
        } catch(const PythonException& exc){
            // Module Initialization Failed
            m_log_warn("No Get Global Catalog Type defined (" << exc.what() << ")");
        } catch(const std::exception& exc){
            // Module Initialization Failed
            m_log_warn("No Get Global Catalog Type defined (" << exc.what() << ")");
        } catch(py::error_already_set){
            std::pair<std::string,std::string> err = py_helper.lastError();
            m_log_error("No Get Global Catalog Type defined. Error is " << err.first << " - " << err.second);
        } catch(...){
            m_log_error("No Get Global Catalog Type defined. Unknown reason");
        }
        
        try{
            // Get the Local Catalog Type function
            py::object type_function = py_helper.getFunction(m_module,CATALOG_V_1_0_LOCAL_TYPE_FUNCTION_NAME);
            m_log_info("Got GetLocalType Function <" << CATALOG_V_1_0_LOCAL_TYPE_FUNCTION_NAME << ">");
            
            // Call Get Global Catalog Type Function
            m_localCatalogType = py::call<std::string>(type_function.ptr());
            m_log_info("Local Catalog Type is <" << CATALOG_V_1_0_GLOBAL_TYPE_FUNCTION_NAME << ">");
        } catch(const PythonException& exc){
            // Module Initialization Failed
            m_log_warn("No Get Global Catalog Type defined (" << exc.what() << ")");
        } catch(const std::exception& exc){
            // Module Initialization Failed
            m_log_warn("No Get Local Catalog Type defined (" << exc.what() << ")");
        } catch(py::error_already_set){
            std::pair<std::string,std::string> err = py_helper.lastError();
            m_log_error("No Get Local Catalog Type defined. Error is " << err.first << " - " << err.second);
        } catch(...){
            m_log_error("No Get Local Catalog Type defined. Unknown reason");
        }
        
        if(m_globalCatalogType.empty() || m_localCatalogType.empty()){
            m_log_error("No Global or Local Catalog Type specified");
            result = false;
        }
    }
        
    return result;
}
