/*
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *  GLite Data Transfer Agents: CatalogConfig Class
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *           Paolo Tedesco <paolo.tedesco@cern.ch>
 *  Version info: $Id: CatalogConfig.h,v 1.6 2008/11/13 10:43:55 ptedesco Exp $
 *  Release: $Name:  $
 *
 */
#ifndef GLITE_DATA_TRANSFER_AGENTS_CATALOG_PYTHON_CATALOGCONFIG_H_
#define GLITE_DATA_TRANSFER_AGENTS_CATALOG_PYTHON_CATALOGCONFIG_H_

#include <glite/data/transfer/agent/catalog/CatalogServiceException.h>
#include <glite/data/config/service/Logger.hxx>
#include <string>
#include <boost/python.hpp>

namespace glite    {
namespace data     {
namespace transfer {
namespace agent    {
namespace catalog  {

/**
 * RetryCofnig  Class
 */
class CatalogConfig {
public:

    /**
     * Return the CatalogConfig singleton instance
     * @return the CatalogConfig singleton instance
     */
    static CatalogConfig& instance() /*throw ()*/; 

    // =========================================================================
    // Catalog Static Constants
    // =========================================================================
    static const char * const CATALOG_VERSION_FUNCTION_NAME;
    static const char * const CATALOG_V_1_0_GLOBAL_TYPE_FUNCTION_NAME;
    static const char * const CATALOG_V_1_0_LOCAL_TYPE_FUNCTION_NAME;
    static const char * const CATALOG_V_1_0_CHECK_PERMISSIONS_FUNCTION_NAME;
    static const char * const CATALOG_V_1_0_LIST_SURLS_FUNCTION_NAME;
    static const char * const CATALOG_V_1_0_CHECK_SURLS_FUNCTION_NAME;
    static const char * const CATALOG_V_1_0_GET_STATS_FUNCTION_NAME;
    static const char * const CATALOG_V_1_0_REGISTER_SURLS_FUNCTION_NAME;
    static const char * const CATALOG_V_1_0_GET_ENDPOINT_FUNCTION_NAME;
    static const char * const CATALOG_V_1_0_INIT_FUNCTION_NAME;
    static const char * const CATALOG_V_1_0;

    // =========================================================================
    // PythonCatalog Properties
    // =========================================================================

    /**
     * Get Catalog Module Name 
     * @return the Catalog Module Name
     */
    const std::string& catalogModule() const {return m_catalogModule;} 

    /**
     * Get Catalog Module Interface Version
     * @return the Catalog Module Interface Version
     */
    const std::string& catalogVersion() const {return m_catalogVersion;}

    /**
     * Returns the Global Catalog Service Type
     * @return the Global Catalog Service Type
     */
    const std::string&  globalCatalogType() const {return m_globalCatalogType;}
    
    /**
     * Returns The Local Catalog Service Type
     * @return The Local Catalog Service Type
     */
    const std::string localCatalogType() const {return m_localCatalogType;}
    
    /**
     * Get the module Catalog CheckPermissions Function Object
     * @return the CheckPermissions Function Object
     */
    boost::python::object checkPermissionsFunction() { return m_checkPermissions;}

    /**
     * Get the module Catalog ListSurls Function Object
     * @return the ListSurls Function Object
     */
    boost::python::object listSurlsFunction() { return m_listSurls;}

    /**
     * Get the module Catalog CheckSurls Function Object
     * @return the CheckSurls Function Object
     */
    boost::python::object checkSurlsFunction() { return m_checkSurls;}

    /**
     * Get the module Catalog GetStats Function Object
     * @return the GetStats Function Object
     */
    boost::python::object getStatsFunction() { return m_getStats;}

    /**
     * Get the module Catalog RegisterSurls Function Object
     * @return the RegisterSurls Function Object
     */
    boost::python::object registerSurlsFunction() { return m_registerSurls;}

    /**
     * Get the module Catalog GetEndpoint Function Object
     * @return the GetEndpoint Function Object
     */
    boost::python::object getEndpointFunction() /*throw(LogicError)*/{ 
        if(false == m_hasGetEndpoint){
            throw CatalogLogicError("Not configured");
        }
        return m_getEndpoint;
    }

protected:
        
    /**
     * Set CatalogModule 
     */
    void catalogModule(const std::string& m) {m_catalogModule = m;} 

    // =========================================================================
    // VOActionsData Methods
    // =========================================================================
    
    /**
     * Verify that the module is  correct and get the module's version  
     */
    bool validate() /*throw()*/;

    /**
     * Initialize the module
     */
    bool init(const std::string& params) /*throw()*/;
    
private:

    /*
     * Declare factory methods as friends
     */
    friend class PythonCatalogFactory;

    /**
     * Constructor
     */
    CatalogConfig () :
        m_logger("glite-transfer-catalog-python"),
        m_hasGetEndpoint(false){}
        
    /**
     * Destructor
     */
    ~CatalogConfig (){}

private:
    
    glite::config::Logger m_logger;
    
    /**
     * The Catalog Python Module Name
     */
    std::string m_catalogModule;

    /**
     * The Catalog Python Module Interface Version
     */
    std::string m_catalogVersion;

    /**
     * The Global Catalog Service Type
     */
    std::string m_globalCatalogType;
    
    /**
     * The Local Catalog Service Type
     */
    std::string m_localCatalogType;

    /**
     * The module Object
     */
    boost::python::object m_module;

    /**
     * The CheckPermissions Function Object
     */
    boost::python::object m_checkPermissions;

    /**
     * The listSurls Function Object
     */
    boost::python::object m_listSurls;

    /**
     * The CheckSurls Function Object
     */
    boost::python::object m_checkSurls;

    /**
     * The GetStats Function Object
     */
    boost::python::object m_getStats;

    /**
     * The RegisterSurls Function Object
     */
    boost::python::object m_registerSurls;

    /**
     * Check if the module has the GetEndpoint Method
     */
    bool m_hasGetEndpoint;

    /**
     * The GetEndpoint Function Object
     */
    boost::python::object m_getEndpoint;
};

} // end namespace catalog
} // end namespace agent
} // end namespace transfer
} // end namespace data
} // end namespace glite

#endif //GLITE_DATA_TRANSFER_AGENTS_CATALOG_PYTHON_CATALOGCONFIG_H_
