/**
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *  GLite Data Transfer Agents: Define the Catalog Module
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *           Paolo Tedesco <paolo.tedesco@cern.ch>
 *  Version info: $Id: CatalogModule.cpp,v 1.5 2008/11/13 10:43:55 ptedesco Exp $
 *  Release: $Name:  $
 *
 */
#include "CatalogModule.h"

#include <boost/shared_ptr.hpp>
#include <vector>
#include <string>

#include <boost/python/suite/indexing/vector_indexing_suite.hpp>

using namespace glite::data::transfer::agent::catalog;

/*
 * Python Module Name
 */    
const char * const glite::data::transfer::agent::catalog::CATALOG_MODULE_NAME = "_catalog";
const char * const glite::data::transfer::agent::catalog::REMOTE_CATALOG_EXCEPTION = "glite.fts.catalog.RemoteError";

namespace glite   {
namespace data    {
namespace transfer{
namespace agent   {
namespace catalog {

/**
 * Create and return the wrapper for the StringPair class
 */
void StringPair_class() {
                                                    
    typedef CatalogService::StringPair StringPair;
    
    // Create the Python type object
    static boost::python::object s_StringPair_class(
      boost::python::class_<StringPair>("StringPair")
       .def(boost::python::init<const StringPair&>())
       .def(boost::python::init<const std::string&, const std::string&>())
       .def_readwrite("First", &StringPair::first)
       .def_readwrite("Second", &StringPair::second)
    );
}


/**
 * Create and return the wrapper for the StringPairArray class
 */
void StringPairArray_class() {
                                                    
    typedef CatalogService::StringPairArray StringPairArray;
    
    // Create the Python type object
    static boost::python::object s_StringPairArray_class(
      boost::python::class_<StringPairArray>("StringPairArray")
       .def(boost::python::vector_indexing_suite<StringPairArray, true >())
    );
}

/**
 * Create and return the wrapper for the StringArray class
 */
void StringArray_class() {
                                                    
    typedef CatalogService::StringArray StringArray;
    
    // Create the Python type object
    static boost::python::object s_StringArray_class(
      boost::python::class_<StringArray>("StringArray")
       .def(boost::python::vector_indexing_suite<StringArray, true >())
    );
}

/**
 * Create and return the wrapper for the StringArray2 (array or StringArray) class
 */
void StringArray2_class() {
                                                    
    typedef std::vector<CatalogService::StringArray> StringArray2;
    
    // Create the Python type object
    static boost::python::object s_StringArray2_class(
      boost::python::class_<StringArray2>("StringArray2")
       .def(boost::python::vector_indexing_suite<StringArray2, true >())
    );
}


/**
 * Create and return the wrapper for the CatalogResult class
 */
void CatalogResult_class() {

    typedef CatalogService::CatalogResult CatalogResult;
    
    // Create the Enum
    static boost::python::object s_CatalogResult_ResultStatus_enum(
      boost::python::enum_<CatalogResult::ResultStatus>("ResultStatus")
        .value("Success"      , CatalogResult::R_SUCCESS)
        .value("Failed"       , CatalogResult::R_FAILED)
        .value("SomeFailures" , CatalogResult::R_SOME_FAILURES)
        .export_values()
    );
    
    // Create the Python type object
    static boost::python::object s_CatalogResult_class(
      boost::python::class_<CatalogResult>("CatalogResult")
       .def(boost::python::init<const CatalogResult&>())
       .def_readwrite("Status", &CatalogResult::status)
       .def_readwrite("Reason", &CatalogResult::reason)
       .def_readwrite("Failures", &CatalogResult::failures)
    );
}

/**
 * Create and return the wrapper for the ListSurlsResult class
 */
void ListSurlsResult_class() {
       
    typedef CatalogService::CatalogResult CatalogResult;
        
    // Create the Python type object for ListSurlsResult Class
    static boost::python::object s_ListSurlsResult_class(
      boost::python::class_<ListSurlsResult, boost::python::bases<CatalogResult> >("ListSurlsResult")
       .def(boost::python::init<const ListSurlsResult&>())
       .def_readwrite("Surls", &ListSurlsResult::surls)
    );
}

/**
 * Create and return the wrapper for the FileStat class
 */
void FileStat_class() {
                                                    
    typedef CatalogService::FileStat FileStat;
    
    // Create the Python type object
    static boost::python::object s_FileStat_class(
      boost::python::class_<FileStat,  boost::shared_ptr<FileStat>, boost::noncopyable> ("FileStat")
       .def_readwrite("Logical", &FileStat::logical)
    );
}
    
/**
 * Create and return the wrapper for the ReplicaStat class
 */
void ReplicaStat_class() {
                                                    
    typedef ReplicaStatWrapper ReplicaStat;
    
    // Create the Python type object
    static boost::python::object s_ReplicaStat_class(
      boost::python::class_<ReplicaStat, boost::shared_ptr<ReplicaStat> >("ReplicaStat")
       .def(boost::python::init<const ReplicaStat&>())
       .def(boost::python::init<const std::string&          // logical
                ,const std::string&                         // surl
                ,boost::python::optional<boost::python::object> >()) // stat
       .def_readonly("Logical", &ReplicaStat::logical)
       .def_readonly("Surl"   , &ReplicaStat::surl)
       .def_readonly("Stat"   , &ReplicaStat::stat)
    );
}

/**
 * Create and return the wrapper for the ReplicaStatArray class
 */
void ReplicaStatArray_class() {
                                                    
    typedef std::vector<ReplicaStatWrapper> ReplicaStatArray;
    
    // Create the Python type object
    static boost::python::object s_ReplicaStatArray_class(
      boost::python::class_<ReplicaStatArray>("ReplicaStatArray")
       .def(boost::python::vector_indexing_suite<ReplicaStatArray, true >())
    );
}

/**
 * Create and return the wrapper for the GetStatsResult class
 */
void GetStatsResult_class() {
                        
    typedef CatalogService::CatalogResult CatalogResult;
       
    // Create the Python type object
    static boost::python::object s_GetStatsResult_class(
      boost::python::class_<GetStatsResult, boost::python::bases<CatalogResult> >("GetStatsResult")
       .def(boost::python::init<const GetStatsResult&>())
       .def_readwrite("Stats", &GetStatsResult::stats)
    );
}

/**
 * Throw a CatalogRemote Exception
 */
void throwCatalogRemoteError(const char * msg = "Can't contact remote Catalog") {
    static boost::python::handle<> exception(
        PyErr_NewException(const_cast<char*>(REMOTE_CATALOG_EXCEPTION),PyExc_IOError,0));

    // Throw Exception    
    PyErr_SetString(exception.get(),msg);
    boost::python::throw_error_already_set();
}

BOOST_PYTHON_FUNCTION_OVERLOADS(throwCatalogRemoteError_overloads,throwCatalogRemoteError,0,1);

} // end namespace catalog
} // end namespace agent
} // end namespace transfer
} // end namespace data
} // end namespace glite


/*
 * Init Python Module
 */
BOOST_PYTHON_MODULE(_catalog){

    StringPair_class();
    StringPairArray_class();
    StringArray_class();
    StringArray2_class();
    CatalogResult_class();
    ListSurlsResult_class();
    FileStat_class();
    ReplicaStat_class();
    ReplicaStatArray_class();
    GetStatsResult_class();
    
    boost::python::def("RaiseRemoteError" ,throwCatalogRemoteError,throwCatalogRemoteError_overloads());
} // end module catalog
