/*
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *  GLite Data Transfer Agents: PythonCatalog Class
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *  Version info: $Id: PythonCatalog.cpp,v 1.8 2008/11/13 10:43:55 ptedesco Exp $
 *  Release: $Name:  $
 *
 */
#include "PythonCatalog.h"
#include "CatalogConfig.h"
#include "CatalogModule.h"
#include <glite/data/agents/python/PyHelper.h>
#include <glite/data/agents/python/PythonException.h>
#include <glite/data/agents/UserProxyEnv.h>

#include <algorithm>
#include <cctype>
#include <boost/scoped_ptr.hpp>

using namespace glite::data::transfer::agent;
using namespace glite::data::transfer::agent::catalog;
using namespace glite::data::agents;
using namespace glite::data::agents::python;
using boost::scoped_ptr;

namespace py = boost::python;

/*
 * Constants
 */
namespace {
const char * const PYTHONCATALOG_CLIENT_NAME      = "transfer-agent-catalog-python";
}


/*
 * PythonCatalog
 *
 * Constructor
 */
PythonCatalog::PythonCatalog(const std::string& e, const std::string& t, const std::string& cred):
    m_logger(PYTHONCATALOG_CLIENT_NAME),
    m_endpoint(e),
    m_type(t),
    m_cred(cred){
}
 
/*
 * ~PythonCatalog
 *
 * Destructor
 */
PythonCatalog::~PythonCatalog(){
}
    
/*
 * checkPermissions
 *
 * Check that the user permission is allowed to read and replicate the 
 * requested files
 */
void PythonCatalog::checkPermissions(const StringArray& logicals, CatalogResult& result) /*throw (CatalogLogicError, CatalogRemoteError)*/{
    if(logicals.empty()){
        // No Files to check
        throw CatalogLogicError("Empty file list");
    }
    // Set Delegated Credentials
    UserProxyEnv cert(m_cred);

    PyHelper& py_helper = PyHelper::instance();
    try{
        // Get the function
        py::object func = CatalogConfig::instance().checkPermissionsFunction();
        m_log_debug("Got CheckPermissions Function");
                
        // Based on the version, Use the right signature
        if(CatalogConfig::instance().catalogVersion() == CatalogConfig::CATALOG_V_1_0){
            m_log_debug("Using Catalog Version " << CatalogConfig::instance().catalogVersion());
            // Call the CheckPermissions
            py::object obj = py::call<py::object>(func.ptr(),m_endpoint,m_type,logicals);
            result = py::extract<CatalogResult&>(obj);
            m_log_debug("Function " << CatalogConfig::CATALOG_V_1_0_CHECK_PERMISSIONS_FUNCTION_NAME << " returned " << str_bool(CatalogResult::R_SUCCESS == result.status));
            if(CatalogResult::R_SUCCESS != result.status){
                if (result.failures.empty()){
                    m_log_error("CheckPermissions failed for all files");
                } else {
                    m_log_error("CheckPermissions failed for some files");
                }
            } else {
                m_log_debug("CheckPermissions SURLs succeeded");
            }
        } else  {
            m_log_error("Unsupported CatalogPlugin Version (" << CatalogConfig::instance().catalogVersion() << ")");
            throw CatalogLogicError("Unsupported CatalogPlugin Version");
        }
    } catch(const PythonException& exc){
        m_log_error("CheckPermissions failed. Error is " << exc.what());
        throw CatalogLogicError(exc.what());
    } catch(const std::exception& exc){
        result.status = CatalogResult::R_FAILED;
        result.reason = exc.what();
        m_log_error("CheckPermissions failed. Error is " << exc.what());
    } catch(py::error_already_set){
        std::pair<std::string,std::string> err = py_helper.lastError();
        //  Check if it's a Remote Error
        if(err.first == REMOTE_CATALOG_EXCEPTION){
            m_log_error("Remote Error in CheckPermissions: " << err.second);
            throw CatalogRemoteError(err.second);
        } 
        result.status = CatalogResult::R_FAILED;
        result.reason = (std::string)err.first.c_str() + " : " + err.second.c_str();
        m_log_error("CheckPermissions failed. Error is " << result.reason);
    } catch(...){
        m_log_error("CheckPermissions failed. Unknown reason");
        throw;
    }
}

/*
 * listSurl
 *
 * List the names of all the replicas associated to the given Logical Name
 */
void PythonCatalog::listSurls(const StringArray& logicals, const std::string& site, std::vector<StringArray>& surls, CatalogResult& result) /*throw (CatalogLogicError, CatalogRemoteError)*/{
    if(logicals.empty()){
        // No Files to check
        throw CatalogLogicError("Empty file list");
    }
    // Set Delegated Credentials
    UserProxyEnv cert(m_cred);

    PyHelper& py_helper = PyHelper::instance();
    try{
        // Get the function
        py::object func = CatalogConfig::instance().listSurlsFunction();
        m_log_debug("Got ListSurls Function");
                
        // Based on the version, Use the right signature
        if(CatalogConfig::instance().catalogVersion() == CatalogConfig::CATALOG_V_1_0){
            m_log_debug("Using Catalog Version " << CatalogConfig::instance().catalogVersion());
            // Call the CheckPermissions
            py::object obj = py::call<py::object>(func.ptr(),m_endpoint,m_type,logicals,site);
            ListSurlsResult& res_obj = py::extract<ListSurlsResult&>(obj);
            m_log_debug("Function " << CatalogConfig::CATALOG_V_1_0_LIST_SURLS_FUNCTION_NAME << " returned " << str_bool(CatalogResult::R_SUCCESS == res_obj.status));
            // Copy Surls List
            result = res_obj;
            if(false == res_obj.surls.empty()){
                surls = res_obj.surls;
            }
            if(CatalogResult::R_SUCCESS != result.status){
                if (result.failures.empty()){
                    m_log_error("ListSurls failed for all files");
                } else {
                    m_log_error("ListSurls failed for some files");
                }
            } else {
                m_log_debug("List SURLs succeeded");
            }
        } else  {
            m_log_error("Unsupported CatalogPlugin Version (" << CatalogConfig::instance().catalogVersion() << ")");
            throw CatalogLogicError("Unsupported CatalogPlugin Version");
        }
    } catch(const PythonException& exc){
        std::string reason = (std::string)"List Surls Failed. Reason is " + exc.what();
        m_log_error(reason);
        throw CatalogLogicError(exc.what());
    } catch(const std::exception& exc){
        result.status = CatalogResult::R_FAILED;
        result.reason = exc.what();
        m_log_error("List Surls Failed. Reason is " << exc.what());
    } catch(py::error_already_set){
        std::pair<std::string,std::string> err = py_helper.lastError();
        //  Check if it's a Remote Error
        if(err.first == REMOTE_CATALOG_EXCEPTION){
            m_log_error("Remote Error in List Surls: " << err.second);
            throw CatalogRemoteError(err.second);  
        } 
        result.status = CatalogResult::R_FAILED;
        result.reason = (std::string)err.first + " : " + err.second;
        m_log_error("ListSurls failed. Error is " << result.reason);
    } catch(...){
        m_log_error("ListSurls failed. Unknown reason");
        throw;
    }
}

/*
 * checkSurls
 *
 * Check that the given SURLs don't already exist 
 */
void PythonCatalog::checkSurls(const StringPairArray& names, CatalogResult& result) /*throw (CatalogLogicError, CatalogRemoteError)*/{
    if(names.empty()){
        // No Files to check
        throw CatalogLogicError("Empty file list");
    }
    
    // Set Delegated Credentials
    UserProxyEnv cert(m_cred);

    PyHelper& py_helper = PyHelper::instance();
    try{
        // Get the function
        py::object func = CatalogConfig::instance().checkSurlsFunction();
        m_log_debug("Got CheckSurls Function");
                
        // Based on the version, Use the right signature
        if(CatalogConfig::instance().catalogVersion() == CatalogConfig::CATALOG_V_1_0){
            m_log_debug("Using Catalog Version " << CatalogConfig::instance().catalogVersion());
            // Call the checkSurls
            py::object obj = py::call<py::object>(func.ptr(),m_endpoint,m_type,names);
            result = py::extract<CatalogResult&>(obj);
            m_log_debug("Function " << CatalogConfig::CATALOG_V_1_0_CHECK_SURLS_FUNCTION_NAME << " returned " << str_bool(CatalogResult::R_SUCCESS == result.status));
            if(CatalogResult::R_SUCCESS != result.status){
                if (result.failures.empty()){
                    m_log_error("CheckSurls failed for all files");
                } else {
                    m_log_error("CheckSurls failed for some files");
                }
            } else {
                m_log_debug("CheckSurls succeeded");
            }
        } else  {
            m_log_error("Unsupported CatalogPlugin Version (" << CatalogConfig::instance().catalogVersion() << ")");
            throw CatalogLogicError("Unsupported CatalogPlugin Version");
        }
    } catch(const PythonException& exc){
        m_log_error("CheckSurls failed. Error is " << exc.what());
        throw CatalogLogicError(exc.what());
    } catch(const std::exception& exc){
        m_log_error("CheckSurls failed. Error is " << exc.what());
        result.status = CatalogResult::R_FAILED;
        result.reason = exc.what();
    } catch(py::error_already_set){
        std::pair<std::string,std::string> err = py_helper.lastError();
        //  Check if it's a Remote Error
        if(err.first == REMOTE_CATALOG_EXCEPTION){
            m_log_error("Remote Error in CheckSurls: " << err.second);
            throw CatalogRemoteError(err.second);  
        } 
        result.status = CatalogResult::R_FAILED;
        result.reason = (std::string)err.first + " : " + err.second;
        m_log_error("CheckSurls failed. Error is " << result.reason);
    } catch(...){
        m_log_error("CheckSurls failed. Unknown reason");
        throw;
    }
}

/*
 * getStats
 *
 * Get the Stats info of the given files.
 */
void PythonCatalog::getStats(const StringPairArray& names, FileStatArray& stats, CatalogResult& result) /*throw (CatalogLogicError, CatalogRemoteError)*/{
    if(names.empty()){
        // No Files to check
        throw CatalogLogicError("Empty file list");
    }
    
    // Set Delegated Credentials
    UserProxyEnv cert(m_cred);

    PyHelper& py_helper = PyHelper::instance();
    try{
        // Get the function
        py::object func = CatalogConfig::instance().getStatsFunction();
        m_log_debug("Got GetStats Function");
                
        // Based on the version, Use the right signature
        if(CatalogConfig::instance().catalogVersion() == CatalogConfig::CATALOG_V_1_0){
            m_log_debug("Using Catalog Version " << CatalogConfig::instance().catalogVersion());
            // Call the GetStats
            py::object obj = py::call<py::object>(func.ptr(),m_endpoint,m_type,names);
            GetStatsResult& res_obj = py::extract<GetStatsResult&>(obj);
            m_log_debug("Function " << CatalogConfig::CATALOG_V_1_0_GET_STATS_FUNCTION_NAME << " returned " << str_bool(CatalogResult::R_SUCCESS == res_obj.status));
            // Copy Result
            result = res_obj;
            if(CatalogResult::R_SUCCESS != result.status){
                if (result.failures.empty()){
                    m_log_error("GetStats failed for all files");
                } else {
                    m_log_error("GetStats failed for some files");
                }
            } else {
                m_log_debug("GetStats succeeded");
            }
            // Extract Stats from Result: Loop through the list
            int len = py::extract<int>(res_obj.stats.attr("__len__")());
            for(int i = 0; i < len; ++i){
                py::object o = res_obj.stats[i];
                FileStatPtr s = py::extract<FileStatPtr>(o);
                boost::shared_ptr<PythonFileStat> p;
                if(0 != s.get()){
                    p.reset(new PythonFileStat(s->logical,o));
                }
                stats.push_back(p);
            }
        } else  {
            m_log_error("Unsupported CatalogPlugin Version (" << CatalogConfig::instance().catalogVersion() << ")");
            throw CatalogLogicError("Unsupported CatalogPlugin Version");
        }
    } catch(const PythonException& exc){
        m_log_error("GetStats failed. Error is " << exc.what());
        throw CatalogLogicError(exc.what());
    } catch(const std::exception& exc){
        m_log_error("GetStats failed. Error is " << exc.what());
        result.status = CatalogResult::R_FAILED;
        result.reason = exc.what();
    } catch(py::error_already_set){
        std::pair<std::string,std::string> err = py_helper.lastError();
        //  Check if it's a Remote Error
        if(err.first == REMOTE_CATALOG_EXCEPTION){
            m_log_error("Remote Error in GetStats: " << err.second);
            throw CatalogRemoteError(err.second);  
        } 
        result.status = CatalogResult::R_FAILED;
        result.reason = (std::string)err.first + " : " + err.second;
        m_log_error("GetStats failed. Error is " << result.reason);
    } catch(...){
        m_log_error("GetStats failed. Unknown reason");
        throw;
    }
}

/*
 * registerSurl
 *
 * Register the name of a new replica 
 */
void PythonCatalog::registerSurls(const ReplicaStatArray& replicas, CatalogResult& result) /*throw (CatalogLogicError, CatalogRemoteError)*/{
    if(replicas.empty()){
        // No Files to check
        throw CatalogLogicError("Empty file list");
    }
   
    // Set Delegated Credentials
    UserProxyEnv cert(m_cred);

    PyHelper& py_helper = PyHelper::instance();
    try{
        // Get the function
        py::object func = CatalogConfig::instance().registerSurlsFunction();
        m_log_debug("Got RegisterSurls Function");
            
        // Translate a ReplicaStatArray into a StringPair Array and a python list
        std::vector<ReplicaStatWrapper> stats_wrapper;
        ReplicaStatArray::const_iterator rp_it;
        for(rp_it = replicas.begin(); rp_it != replicas.end(); ++rp_it){
            stats_wrapper.push_back(ReplicaStatWrapper((*rp_it)));
        }
        
        // Based on the version, Use the right signature
        if(CatalogConfig::instance().catalogVersion() == CatalogConfig::CATALOG_V_1_0){
            m_log_debug("Using Catalog Version " << CatalogConfig::instance().catalogVersion());
                        
            // Call the CheckPermissions
            py::object obj = py::call<py::object>(func.ptr(),m_endpoint,m_type,stats_wrapper);
            result = py::extract<CatalogResult&>(obj);
            m_log_debug("Function " << CatalogConfig::CATALOG_V_1_0_REGISTER_SURLS_FUNCTION_NAME << " returned " << str_bool(CatalogResult::R_SUCCESS == result.status));
            if(CatalogResult::R_SUCCESS != result.status){
                if (result.failures.empty()){
                    m_log_error("RegisterSurls failed for all files");
                } else {
                    m_log_error("RegisterSurls failed for some files");
                }
            } else {
                m_log_info("SURLs Registration succeeded");
            }
        } else  {
            m_log_error("Unsupported CatalogPlugin Version (" << CatalogConfig::instance().catalogVersion() << ")");
            throw CatalogLogicError("Unsupported CatalogPlugin Version");
        }
    } catch(const PythonException& exc){
         m_log_error("Surls Registration Failed. Error is " << exc.what());
        throw CatalogLogicError(exc.what());
    } catch(const std::exception& exc){
        m_log_error("Surls Registration failed. Error is " << exc.what());
        result.status = CatalogResult::R_FAILED;
        result.reason = exc.what();
    } catch(py::error_already_set){
        std::pair<std::string,std::string> err = py_helper.lastError();
        //  Check if it's a Remote Error
        if(err.first == REMOTE_CATALOG_EXCEPTION){
            m_log_error("Remote Error in Surls Registration: " << err.second);
            throw CatalogRemoteError(err.second);  
        } 
        result.status = CatalogResult::R_FAILED;
        result.reason = (std::string)err.first + " : " + err.second;
        m_log_error("Surls Registration failed. Error is " << result.reason);
    } catch(...){
        m_log_error("RegisterSurls failed. Unknown reason");
        throw;
    }
}
