/**
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *  GLite Data Transfer Agents: PythonCatalogFactory Class
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *           Paolo Tedesco <paolo.tedesco@cern.ch>
 *  Version info: $Id: PythonCatalogFactory.cpp,v 1.8 2009/01/08 13:50:39 ptedesco Exp $
 *  Release: $Name:  $
 *
 */
#include "PythonCatalogFactory.h"
#include "PythonCatalog.h"
#include "CatalogConfig.h"
#include <glite/data/agents/AgentExceptions.h>
#include <glite/data/agents/python/PyHelper.h>
#include <glite/data/agents/python/PythonException.h>

#include <glite/data/config/service/ParamReader.hxx>

// Workaround for Globus module deactivation problem
#ifdef __cplusplus
extern "C"{
#endif //__cplusplus
// Fix Warning
#ifdef IOV_MAX
#undef IOV_MAX
#endif //IOV_MAX
#include "globus_gss_assist.h"
#include "globus_openssl.h"
#ifdef __cplusplus
}
#endif //__cplusplus

using namespace glite::data::transfer::agent;
using namespace glite::data::transfer::agent::catalog;
using namespace glite::data::agents::python;
using namespace glite::config;
using glite::data::agents::LogicError;
using glite::data::agents::ConfigParamException;
using glite::data::agents::ConfigurationException;

namespace py = boost::python;

/*
 * Constants
 */
namespace {
const char * const PYTHONCATALOG_CLIENT_NAME      = "transfer-agent-catalog-python";
const char * const CATALOGMODULE_PROPERTY_NAME    = "CatalogModule";
const char * const CATALOGPARAMS_PROPERTY_NAME    = "CatalogParams";
}

extern "C" {
/*
 * create_glite_component
 *
 * Create Component instance
 */
ComponentConfiguration * create_glite_component(){
    return new PythonCatalogFactory();
}
                                                                                
/*
 * destroy_glite_component
 *
 * Destroy Component instance
 */
void destroy_glite_component(ComponentConfiguration * component){
    PythonCatalogFactory * factory = dynamic_cast<PythonCatalogFactory*>(component);
    if(0 != factory){
        delete factory;
    }
}
                                                                                
} // End extern "C"

/*
 * PythonCatalogFactory
 *
 * Constructor
 */
PythonCatalogFactory::PythonCatalogFactory():CatalogServiceFactory(),
    ComponentConfiguration(PYTHONCATALOG_CLIENT_NAME), 
    m_logger(PYTHONCATALOG_CLIENT_NAME){
}
                                                                                                                          
/*
 * ~PythonCatalogFactory
 *
 * Destructor
 */
PythonCatalogFactory::~PythonCatalogFactory(){
}

/*
 * init
 *
 * Initialize the Component
 */
int PythonCatalogFactory::init(const Params& params){
    try{
        ParamReader reader(getName(),params);
        // Get Catalog Module
        std::string catalog_module;
        reader.getParameter(CATALOGMODULE_PROPERTY_NAME,catalog_module).
            required();
        CatalogConfig::instance().catalogModule(catalog_module);

        // Get Catalog Parameters
        std::string catalog_params;
        reader.getParameter(CATALOGPARAMS_PROPERTY_NAME,catalog_params);
        if (!CatalogConfig::instance().validate()){
           m_log_error("Catalog Module Validation failed");
           return -1;
        }
        
        // Initialize catalog module
        if (!CatalogConfig::instance().init(catalog_params)){
           m_log_error("Catalog Module Initialization failed");
           return -1;
        }
        // Log Configuration Values
        m_log_info(getName() << " Initialized. Parameters are");
        m_log_info("CatalogModule       : " << CatalogConfig::instance().catalogModule());
        m_log_info("CatalogParams       : " << catalog_params);
    } catch(const LogicError& exc) {
        m_log_error(getName() << " configuration Logic Error: " << exc.what());
        return -1;
    } catch(const std::exception& exc) {
        m_log_error(getName() << " configuration Unexpected Exception: " << exc.what());
        return -1;
    } catch(...) {
        m_log_error(getName() << " configuration Unhandled Exception");
        return -1;
    }

    // Workaround against globus problem in module deactivation
    (void) globus_module_activate(GLOBUS_OPENSSL_MODULE);
    (void) globus_module_activate(GLOBUS_GSI_GSS_ASSIST_MODULE);
    (void) globus_module_activate(GLOBUS_GSI_GSSAPI_MODULE);

    return 0;
}

/*
 * config
 *
 * Configure the Component
 */
int PythonCatalogFactory::config(const Params& params){
    // Nothing to do
    m_log_info("PythonCatalogFactory Configured");
    return 0;
}

/*
 * fini
 *
 * Finalize the Component
 */
int PythonCatalogFactory::fini(){
    // Nothing to do
    return 0;
}

/*
 * create
 *
 * Create a CatalogService instance
 */
CatalogService * PythonCatalogFactory::create(const std::string& endpoint,const std::string& type, const std::string& cred) /*throw (CatalogLogicError)*/{
    m_log_trace("Creating PythonCatalogFactory instance(e:" << endpoint << ",t:" << type << ",c:" << cred << ")");
    // Return new Instance (ignore type)
    return new PythonCatalog(endpoint,type,cred);
}

/*
 * globalCatalogServiceType
 *
 * Return the Global Catalog Service Type
 */
std::string PythonCatalogFactory::globalCatalogServiceType() /*throw ()*/{
    return CatalogConfig::instance().globalCatalogType();
}    

/*
 * localCatalogServiceType
 *
 * Return the Local Catalog Service Type
 */
std::string PythonCatalogFactory::localCatalogServiceType() /*throw ()*/{
    return CatalogConfig::instance().localCatalogType();
}

/**
 * Return the endpoint and the type of the CatalogService to use. The 
 * default implementation uses in the information system and apply the 
 * following rules: first looks up in a Catalog of the type returned by 
 * localCatalogServiceType exists on the given site; if not retunr the first
 * instance of the type returned by globalCatalogServiceType
 */
std::pair<std::string,std::string> PythonCatalogFactory::getCatalogEndpoint(const std::string& site, const std::string& vo_name) /* throw(LogicError)*/{

    std::pair<std::string,std::string> catalog_data;
    
    PyHelper& py_helper = PyHelper::instance();
    try{
        // Get the function
        py::object func = CatalogConfig::instance().getEndpointFunction();
        m_log_debug("Got GetEndpoint Function");
                
        // Based on the version, Use the right signature
        if(CatalogConfig::instance().catalogVersion() == CatalogConfig::CATALOG_V_1_0){
            m_log_debug("Using Catalog Version " << CatalogConfig::instance().catalogVersion());
            // Call the GetEndpoint
            py::object obj = py::call<py::object>(func.ptr(),site,vo_name);
            catalog_data = py::extract<CatalogService::StringPair>(obj);
        }   else  {
            m_log_error("Unsupported CatalogPlugin Version (" << CatalogConfig::instance().catalogVersion() << ")");
            throw CatalogLogicError("Unsupported CatalogPlugin Version");
        }
        // Log Result
        m_log_debug("Got Endpoint <" << catalog_data.first << ":" << catalog_data.second << ">");
    } catch(const agents::LogicError& exc){
        // No GetCatalog Function registered: Use the default
        catalog_data = CatalogServiceFactory::getCatalogEndpoint(site,vo_name);    
    } catch(const PythonException& exc){
        m_log_error("GetEndpoint failed. Error is " << exc.what());
        throw CatalogLogicError(exc.what());
    } catch(py::error_already_set){
        std::pair<std::string,std::string> err = py_helper.lastError();
        std::string reason = (std::string)"GetEndpoint failed. Error is [" + 
            err.first.c_str() + ":" + err.second.c_str() + "]";
        m_log_error(reason);
        throw CatalogLogicError(reason);
    } catch(...){
        m_log_error("GetEndpoint failed. Unknown reason");
        throw;
    }
    return catalog_data;
}
