/**
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *  GLite Data Transfer Agents: CatalogCatalogFactory Class
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *           Paolo Tedesco <paolo.tedesco@cern.ch> 
 *  Version info: $Id: PythonCatalogFactory.h,v 1.4 2008/11/13 10:43:55 ptedesco Exp $
 *  Release: $Name:  $
 *
 */
#ifndef GLITE_DATA_TRANSFER_AGENTS_CATALOG_CATALOG_PYTHONCATALOGFACTORY_H_
#define GLITE_DATA_TRANSFER_AGENTS_CATALOG_CATALOG_PYTHONCATALOGFACTORY_H_

#include <glite/data/config/service/Logger.hxx>
#include "glite/data/transfer/agent/catalog/CatalogServiceFactory.h"
#include "glite/data/config/service/ComponentConfiguration.hxx"

namespace glite   {
namespace data    {
namespace transfer{
namespace agent   {
namespace catalog {

/**
 * PythonCatalogFactory class. Implement the factory for the Python 
 * CatalogService Client 
 */
class PythonCatalogFactory : public CatalogServiceFactory, 
                             public glite::config::ComponentConfiguration{
public:

    /**
     * Constructor
     */ 
    PythonCatalogFactory();

    /**
     * Destructor
     */ 
    virtual ~PythonCatalogFactory();

    // =========================================================================
    // CatalogServiceFactory Methods
    // =========================================================================

    /**
     * Create a CatalogService instance
     * @param endpoint [IN] the endpoint of the service to contact 
     * @param type [IN] the type of the service to create
     * @param cred [IN] the name of the file containing the credential to be
     * used to contact the service. If this value is empty, the default will 
     * apply
     * @return a new instance of a CatalogService
     * @throws CatalogLogicError in case of error
     */
    virtual CatalogService * create(const std::string& endpoint, const std::string& type, const std::string& cred = "") /*throw (CatalogLogicError)*/; 

    /**
     * Return the Global Catalog Service Type (if any) to be used to query 
     * ServiceDiscovery. If the Catalog Deployment model doesn't support the
     * global scenario, an emty string should be returned
     * @return the Global Catalog Service Type to be used to query 
     */
    virtual std::string globalCatalogServiceType() /*throw ()*/; 

    /**
     * Return the Local Catalog Service Type (if any) to be used to query 
     * ServiceDiscovery. If the Catalog Deployment model doesn't support the
     * local scenario, an emty string should be returned
     * @return the local Catalog Service Type to be used to query 
     */
    virtual std::string localCatalogServiceType() /*throw ()*/; 

    /**
     * Return the endpoint and the type of the CatalogService to use. The 
     * default implementation uses in the information system and apply the 
     * following rules: first looks up in a Catalog of the type returned by 
     * localCatalogServiceType exists on the given site; if not retunr the first
     * instance of the type returned by globalCatalogServiceType
     */
    virtual std::pair<std::string,std::string> getCatalogEndpoint(const std::string& site, const std::string& vo_name) /* throw(LogicError)*/;
 
    // =========================================================================
    // ComponentConfiguration Methods
    // =========================================================================

    /**
     * Initialize the Component
     */
    virtual int init(const Params& params);
                                                                                                                           
    /**
     * Configure the Component
     */
    virtual int config(const Params& params);
                                                                                                                           
    /**
     * Start the Component
     */
    virtual int start(){return 0;}
                                                                                                                           
    /**
     * Stop the Component
     */
    virtual int stop(){return 0;}
                                                                                                                           
    /**
     * Finalize the Component
     */
    virtual int fini();

private:
    
    glite::config::Logger m_logger;
    
    /** @link dependency 
     * @label creates*/
    /*# PythonCatalogFactory lnkPythonCatalog; */
};

} // End namespace catalog
} // End namespace agent
} // End namespace transfer
} // End namespace data
} // End namespace glite

#endif //GLITE_DATA_TRANSFER_AGENTS_CATALOG_CATALOG_PYTHONCATALOGFACTORY_H_
