/*
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *  GLite Data Transfer Agents: ChannelAgent Class
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *  Version info: $Id: ChannelAgent.cpp,v 1.23 2009/01/08 13:50:39 ptedesco Exp $
 *  Release: $Name:  $
 *
 */
#include "ChannelAgent.h"
#include "ChannelFlags.h"
#include <glite/data/transfer/agent/scheduler/ActionScheduler.h>
#include <glite/data/transfer/agent/scheduler/SchedulerEntry.h>
#include <glite/data/transfer/agent/AgentState.h>
#include "Heartbeat.h"
#include "CleanSDCache.h"

#include <glite/data/transfer/agent/action/channel/ChannelAction.h>
#include <glite/data/transfer/agent/dao/channel/DAOFactory.h>
#include <glite/data/agents/dao/DAOContextFactory.h>
#include <glite/data/transfer/agent/dao/TSSDAOContext.h>
#include <glite/data/transfer/agent/dao/channel/CredChannelFactory.h>

#include <glite/data/agents/AgentExceptions.h>
#include <glite/data/agents/cred/cred-utility.h>

#include <glite/data/config/service/ParamReader.hxx>

#include <string>
#include <algorithm>
#include <cctype>
#include <unistd.h>

using namespace glite::data::transfer::agent;
using namespace glite::config;
using glite::data::agents::ConfigParamException;
using glite::data::agents::ConfigurationException;
using glite::data::agents::LogicError;


/*
 * Constants
 */
namespace {
const char * const CHANNEL_AGENT_NAME              = "transfer-channel-agent";
                                                                                                                          
// Properties Names
const char * const NAME_PROPERTY_NAME              = "Name";
const char * const CONTACT_PROPERTY_NAME           = "Contact";
const char * const ACTIONINTERVAL_PROPERTY_NAME    = "DefaultInterval";
const char * const SHARETYPE_PROPERTY_NAME         = "VOShareType";

const char * const FETCH_TYPE_PROPERTY_NAME        = "Fetch_Type";
const char * const CHECK_TYPE_PROPERTY_NAME        = "Check_Type";
const char * const CANCEL_TYPE_PROPERTY_NAME       = "Cancel_Type";
    
const char * const FETCH_INTERVAL_PROPERTY_NAME     = "Fetch_Interval";
const char * const CHECK_INTERVAL_PROPERTY_NAME     = "Check_Interval";
const char * const CANCEL_INTERVAL_PROPERTY_NAME    = "Cancel_Interval";
const char * const HEARTBEAT_INTERVAL_PROPERTY_NAME = "Heartbeat_Interval";
const char * const CLEANSDCACHE_INTERVAL_PROPERTY_NAME = "CleanSDCache_Interval";

// Properties Default  Values
const unsigned int DEFAULT_ACTION_INTERVAL       = 3;
const unsigned int DEFAULT_CANCEL_INTERVAL       = 60;
const unsigned int DEFAULT_HEARTBEAT_INTERVAL    = 60;
const unsigned int DEFAULT_CLEANSDCACHE_INTERVAL = 300;

//const ChannelAgent::ShareType DEFAULT_VO_SHARE_TYPE = ChannelAgent::SHARE_ABSOLUTE;
const ChannelAgent::ShareType DEFAULT_VO_SHARE_TYPE = ChannelAgent::SHARE_NORMALIZED_ON_ACTIVE;

const char * const CHANNEL_AGENT_TYPE              = "channel";

const char * const DEFAULT_FETCH_TYPE              = "glite:Fetch";
const char * const DEFAULT_CHECK_TYPE              = "glite:CheckState";
const char * const DEFAULT_CANCEL_TYPE             = "glite:CancelActive";

// VO Share Types
const char * const CHANNEL_AGENT_VO_SHARE_TYPE_ABSOLUTE   = "absolute";
const char * const CHANNEL_AGENT_VO_SHARE_TYPE_NORMALIZED = "normalized";
const char * const CHANNEL_AGENT_VO_SHARE_TYPE_NORMALIZED_ON_ACTIVE = "normalized-on-active";
}

extern "C" {
/*
 * create_glite_component
 *
 * Create Component instance
 */
ComponentConfiguration * create_glite_component(){
    return new ChannelAgent();
}
                                                                                
/*
 * destroy_glite_component
 *
 * Destroy Component instance
 */
void destroy_glite_component(ComponentConfiguration * component){
    ChannelAgent * agent = dynamic_cast<ChannelAgent *>(component);
    if(0 != agent){
        delete agent;
    }
}
                                                                                
} // End extern "C"

/*
 * ChannelAgent
 *
 * Constructor
 */
ChannelAgent::ChannelAgent():ComponentConfiguration(CHANNEL_AGENT_NAME), 
    m_logger(CHANNEL_AGENT_NAME),
    m_shareType(DEFAULT_VO_SHARE_TYPE),
    m_fetchType(DEFAULT_FETCH_TYPE),
    m_checkType(DEFAULT_CHECK_TYPE),
    m_cancelType(DEFAULT_CANCEL_TYPE),
    m_defaultInterval(DEFAULT_ACTION_INTERVAL),
    m_fetchInterval(DEFAULT_ACTION_INTERVAL),
    m_checkInterval(DEFAULT_ACTION_INTERVAL),
    m_cancelInterval(DEFAULT_CANCEL_INTERVAL),
    m_heartbeatInterval(DEFAULT_HEARTBEAT_INTERVAL),
    m_cleansdcacheInterval(DEFAULT_CLEANSDCACHE_INTERVAL){
}
                                                                                                                          
/*
 * ~ChannelAgent
 *
 * Destructor
 */
ChannelAgent::~ChannelAgent(){
}

namespace glite{
namespace config{

/**
 * SurlNormalization string conversion
 */
template<>
struct StringConverter<ChannelAgent::ShareType> : public EnumConverter<ChannelAgent::ShareType>{
    StringConverter(){
        add(ChannelAgent::SHARE_ABSOLUTE,               CHANNEL_AGENT_VO_SHARE_TYPE_ABSOLUTE);
        add(ChannelAgent::SHARE_NORMALIZED,             CHANNEL_AGENT_VO_SHARE_TYPE_NORMALIZED);
        add(ChannelAgent::SHARE_NORMALIZED_ON_ACTIVE,   CHANNEL_AGENT_VO_SHARE_TYPE_NORMALIZED_ON_ACTIVE);
    }
protected:    
    virtual std::string get_default_string(){
        throw LogicError("Invalid ShareType string");
    }
    virtual ChannelAgent::ShareType get_default_value(){
        throw LogicError("Invalid ShareType value");
    }
};

} // config
} // glite

/*
 * init
 *
 * Initialize the Component
 */
int ChannelAgent::init(const Params& params){
          
    try{
        ParamReader reader(getName(),params);
        
        // Get channel name
        reader.getParameter(NAME_PROPERTY_NAME,m_channelName)
            .required();

        // Get Contact
        reader.getParameter(CONTACT_PROPERTY_NAME,m_contact);
        
        // Get Share Type
        reader.getParameter(SHARETYPE_PROPERTY_NAME,m_shareType);

        // Get Fetch Action Type
        reader.getParameter(FETCH_TYPE_PROPERTY_NAME,m_fetchType);

        // Get Check Action Type
        reader.getParameter(CHECK_TYPE_PROPERTY_NAME,m_checkType);

        // Get Cancel Action Type
        reader.getParameter(CANCEL_TYPE_PROPERTY_NAME,m_cancelType);
        
        // Log Configuration Values
        m_log_info(getName() << " Initialized. Initialization Parameters are");
        m_log_info("Name               : " << m_channelName);
        m_log_info("Contact            : " << m_contact);
        m_log_info("VO Share Type      : " << StringConverter<ShareType>::to_string(m_shareType));
        m_log_info("Fetch  Action Type : " << m_fetchType);
        m_log_info("Check  Action Type : " << m_checkType);
        m_log_info("Cancel Action Type : " << m_cancelType);
        
    } catch(const LogicError& exc) {
        m_log_error(getName() << " configuration Error: " << exc.what());
        return -1;
    } catch(const std::exception& exc) {
        m_log_error(getName() << " configuration Unexpected Exception: " << exc.what());
        return -1;
    } catch(...) {
        m_log_error(getName() << " configuration Unhandled Exception");
        return -1;
    }
    return 0;
}

                                                                                                                          
/*
 * config
 *
 * Configure the Component
 */
int ChannelAgent::config(const Params& params){
    
    try{
        Params::const_iterator it;
        // Get Default Action Interval
        if((it = params.find(ACTIONINTERVAL_PROPERTY_NAME)) != params.end()){
            ParamValue * pv = dynamic_cast<ParamValue *>((*it).second);
            if(0 == pv){
                throw ConfigParamException(getName(),ACTIONINTERVAL_PROPERTY_NAME,ConfigParamException::INVALID);
            }
            m_defaultInterval = atoi(pv->getValue().c_str());
            if(m_defaultInterval < 1) {
                throw ConfigParamException(getName(),ACTIONINTERVAL_PROPERTY_NAME, ConfigParamException::INVALID_VALUE);
            }
        }
        // Get Allocate Action Interval
        if((it = params.find(FETCH_INTERVAL_PROPERTY_NAME)) != params.end()){
            ParamValue * pv = dynamic_cast<ParamValue *>((*it).second);
            if(0 == pv){
                throw ConfigParamException(getName(),FETCH_INTERVAL_PROPERTY_NAME,ConfigParamException::INVALID);
            }
            if(false == pv->getValue().empty()){
                m_fetchInterval = atoi(pv->getValue().c_str());
                if(m_fetchInterval < 0) {
                    throw ConfigParamException(getName(),FETCH_INTERVAL_PROPERTY_NAME, ConfigParamException::INVALID_VALUE);
                }
            } else {
                m_fetchInterval = m_defaultInterval;
            }
        } else {
            m_fetchInterval = m_defaultInterval;
        }
        // Get Check Action Interval
        if((it = params.find(CHECK_INTERVAL_PROPERTY_NAME)) != params.end()){
            ParamValue * pv = dynamic_cast<ParamValue *>((*it).second);
            if(0 == pv){
                throw ConfigParamException(getName(),CHECK_INTERVAL_PROPERTY_NAME,ConfigParamException::INVALID);
            }
            if(false == pv->getValue().empty()){
                m_checkInterval = atoi(pv->getValue().c_str());
                if(m_checkInterval < 0) {
                    throw ConfigParamException(getName(),CHECK_INTERVAL_PROPERTY_NAME, ConfigParamException::INVALID_VALUE);
                }
            } else {
                m_checkInterval = m_defaultInterval;
            }
        } else {
            m_checkInterval = m_defaultInterval;
        }
        // Get Cancel Action Interval
        if((it = params.find(CANCEL_INTERVAL_PROPERTY_NAME)) != params.end()){
            ParamValue * pv = dynamic_cast<ParamValue *>((*it).second);
            if(0 == pv){
                throw ConfigParamException(getName(),CANCEL_INTERVAL_PROPERTY_NAME,ConfigParamException::INVALID);
            }
            if(false == pv->getValue().empty()){
                m_cancelInterval = atoi(pv->getValue().c_str());
                if(m_cancelInterval < 0) {
                    throw ConfigParamException(getName(),CANCEL_INTERVAL_PROPERTY_NAME, ConfigParamException::INVALID_VALUE);
                }
            }
        }
        // Get Heartbeat Action Interval
        if((it = params.find(HEARTBEAT_INTERVAL_PROPERTY_NAME)) != params.end()){
            ParamValue * pv = dynamic_cast<ParamValue *>((*it).second);
            if(0 == pv){
                throw ConfigParamException(getName(),HEARTBEAT_INTERVAL_PROPERTY_NAME,ConfigParamException::INVALID);
            }
            if(false == pv->getValue().empty()){
                m_heartbeatInterval = atoi(pv->getValue().c_str());
                if(m_heartbeatInterval < 0) {
                    throw ConfigParamException(getName(),HEARTBEAT_INTERVAL_PROPERTY_NAME, ConfigParamException::INVALID_VALUE);
                }
            }
        }
        // Get CleanSDCache Action Interval
        if((it = params.find(CLEANSDCACHE_INTERVAL_PROPERTY_NAME)) != params.end()){
            ParamValue * pv = dynamic_cast<ParamValue *>((*it).second);
            if(0 == pv){
                throw ConfigParamException(getName(),CLEANSDCACHE_INTERVAL_PROPERTY_NAME,ConfigParamException::INVALID);
            }
            if(false == pv->getValue().empty()){
                m_cleansdcacheInterval = atoi(pv->getValue().c_str());
                if(m_cleansdcacheInterval < 0) {
                    throw ConfigParamException(getName(),CLEANSDCACHE_INTERVAL_PROPERTY_NAME, ConfigParamException::INVALID_VALUE);
                }
            }
        }
        
        // Create the DAO Context
        createContext();
        
        // Check the schema version
        dao::channel::DAOFactory::instance().checkSchema(*m_daoCtx);
        
        // Check if Channel exists
        checkChannel();
        
        // Create the Scheduler
        initScheduler();

        // Log Configuration Values
        m_log_info(getName() << " Configured. Configuration Parameters are");
        m_log_info("Default       Action Interval    : " << m_defaultInterval << " secs");
        m_log_info("Fetch         Action Interval    : " << m_fetchInterval << " secs");
        m_log_info("Check         Action Interval    : " << m_checkInterval << " secs");
        m_log_info("Cancel        Action Interval    : " << m_cancelInterval << " secs");
        m_log_info("Heartbeat     Action Interval    : " << m_heartbeatInterval << " secs");
        m_log_info("Clean SDCache Action Interval    : " << m_cleansdcacheInterval << " secs");
    } catch(const LogicError& exc) {
        m_log_error(getName() << " configuration Error: " << exc.what());
        return -1;
    } catch(const std::exception& exc) {
        m_log_error(getName() << " configuration Unexpected Exception: " << exc.what());
        return -1;
    } catch(...) {
        m_log_error(getName() << " configuration Unhandled Exception");
        return -1;
    }

    try{
        std::string dn = agents::cred::get_proxy_dn();
        m_log_info("Agent for Channel " << m_channelName << " is using DN " << dn);
    } catch(const std::exception& exc){
        m_log_warn("Cannot Get Agent DN. Reason is " << exc.what());
    }
    m_log_info(getName() << " Configured");
    

    return 0;
}

/*
 * start
 *
 * Start the Component
 */
int ChannelAgent::start(){
    // Start the Scheduler
    if(0 != m_scheduler.get()){
        bool result = m_scheduler->start();
        if(result == false){
            return -1;
        }
    } else {
        return -1;
    }
    
    m_log_info("Agent for Channel " << m_channelName << " Started");
    
    return 0;
}

/*
 * stop
 *
 * Stop the Component
 */
int ChannelAgent::stop(){
    
    // Stop the Scheduler
    if(0 != m_scheduler.get()){
        m_scheduler->stop();
    }
    m_log_info("Agent for Channel " << m_channelName << " Stopped");
    
    return 0;
}


/*
 * fini
 *
 * Finalize the Component
 */
int ChannelAgent::fini(){
    
    // Finalize the Scheduler
    finiScheduler();
    
    // Finalize context
    m_daoCtx.reset();
    
    return 0;
}

/*
 * createContext
 *
 * Create DAO Context
 */
void ChannelAgent::createContext() /*throw(ConfigurationException)*/{
    try{
        // Create DAO Context
        if(0 == m_daoCtx.get()){
            m_daoCtx.reset(agents::dao::DAOContextFactory::instance().create());
        }
    } catch(const agents::dao::DAOException& exc){
        m_log_error("Can't Create DAO Context. Reason is " << exc.what());
        throw ConfigurationException(exc.what());
    }
}
    
/*
 * checkChannel
 *
 * Check if a channel is configured
 */
void ChannelAgent::checkChannel() /*throw(ConfigurationException)*/{
    
    try{
        // Get Channel DAO        
        boost::scoped_ptr<dao::channel::ChannelDAO> channelDao(
            dao::channel::DAOFactory::instance().createChannelDAO(m_channelName,*m_daoCtx));

        // Get Channel instance
        boost::scoped_ptr<model::Channel> channel(channelDao->get(m_channelName));
        if(0 != channel.get()){
            m_log_info("Channel " << m_channelName << " exists");
        } else {
            throw ConfigurationException("No such Channel");
        }
    } catch(const agents::dao::DAOException& exc){
        m_log_error("Failed to get Channel Instance. Reason is " << exc.what());
        m_log_warn("Channel " << m_channelName << " doesn't exist");
        throw ConfigurationException("No such Channel");
    }
}

/*
 * initScheduler
 *
 * Initialize the Scheduler
 */
void ChannelAgent::initScheduler() /*throw(ConfigurationException)*/{

    // Create CredFactory fro the ChannelFactory
    if(0 == m_credFactory.get()){
        m_credFactory.reset(new dao::channel::CredChannelFactory(m_channelName));
    }
    // Create Action Scheduler
    if(0 != m_scheduler.get()){
        finiScheduler();
    }
    m_scheduler.reset(new ActionScheduler(CHANNEL_AGENT_NAME,*m_daoCtx,this));

    // Register Heartbeat
    scheduleAction(*(action::channel::Heartbeat::factory()),m_heartbeatInterval);
    
    // Register CleanSDCache
    scheduleAction(*(action::channel::CleanSDCache::factory()),m_cleansdcacheInterval);
    
    // Register Scheduler Entries
    std::string fetch_option;
    switch(m_shareType){
        case SHARE_ABSOLUTE:
            fetch_option = CHANNEL_VO_SHARE_TYPE_ABSOLUTE;
        break;
        case SHARE_NORMALIZED:
            fetch_option = CHANNEL_VO_SHARE_TYPE_NORMALIZED;
        break;
        case SHARE_NORMALIZED_ON_ACTIVE:
            fetch_option = CHANNEL_VO_SHARE_TYPE_NORMALIZED_ON_ACTIVE;
        break;
    }
    scheduleAction(m_checkType  ,m_checkInterval);
    scheduleAction(m_cancelType ,m_cancelInterval);
    scheduleAction(m_fetchType  ,m_fetchInterval ,fetch_option);
}

/*
 * scheduleEntry
 *
 * Add a schedulerEntry for the given Action Type
 */
void ChannelAgent::scheduleAction(const std::string& type, unsigned int interval,const std::string& params) /*throw(ConfigurationException)*/{
    try{
        // Get Factory Method
        action::ActionFactoryMethod& method = action::ActionFactory::instance().get(type);

        return scheduleAction(method,interval,params);
    } catch(const action::ExecuteException& exc){
        m_log_error("Invalid Action type " << type << ". Reason is " << exc.what());
        throw ConfigurationException("Unknown Action Type");
    }
}

/*
 * scheduleEntry
 *
 * Add a schedulerEntry for the given Action Factory Method
 */
void ChannelAgent::scheduleAction(action::ActionFactoryMethod& method, unsigned int interval, const std::string& params) /*throw(ConfigurationException)*/{
    if(interval > 0){
        // Try to create an Action and check that a VO Action
        boost::scoped_ptr<action::Action> a(method.create()); 
        if(0 == dynamic_cast<action::channel::ChannelAction*>(a.get())){
            m_log_error("Action type " << method.type() << " is not a Channel Action");
            throw ConfigurationException("Invalid Action Type. Not a Channel Action");
        }
        // Add Sceduler Entry
        m_scheduler->add(new SchedulerEntry(method, m_channelName, interval, params));
        m_log_info("Action type " << method.type() << " scheduled");
    } else {
        m_log_info("Action type " << method.type() << " disabled");
    }
}

/*
 * finiScheduler
 *
 * Finalize the Scheduler
 */
void ChannelAgent::finiScheduler(){
    
    // Clear Scheduler Entries
    if(0 != m_scheduler.get()){
        m_scheduler->clear();
    }
    // Delete Scheduler
    m_scheduler.reset();
}


/* 
 * onEventStart
 *
 * Handle Event Start
 */
bool ChannelAgent::onEventStart() /*throw()*/{
    if(0 != m_daoCtx.get()){
        try{
            // Set Agent State
            AgentState<dao::channel::DAOFactory> as(*m_daoCtx,
                m_channelName,
                CHANNEL_AGENT_TYPE,
                m_contact,
                m_heartbeatInterval);
            // Check if there is an agent already running
            bool agent_active = as.check();
            if(true == agent_active){
                m_log_error("Failed to start: An active instance for Channel Agent <" << m_channelName << "> detected");
                return false;
            }
            // Set the Agent Active
            as.set(model::Agent::S_STARTED);
            m_log_info("Agent State set to Started");

            // Register the DAO Context on the TSS 
            dao::g_tss_dao_context.set(m_daoCtx.get());
            m_log_debug("DAOContext registered in thread-specific storage");
        } catch(const agents::AgentException& exc){
            m_log_error("Failed to Set Agent State to Started: " << exc.what());
            return false;
        }
    } else {
        m_log_error("Invalid Context DAO");
        return false;
    }
    
    return true;
}

/*
 * onEventStop
 *
 * Handle Event Stop
 */
bool ChannelAgent::onEventStop() /*throw()*/{
    if(0 != m_daoCtx.get()){
        try{
            // Set Agent State
            AgentState<dao::channel::DAOFactory> as(*m_daoCtx,
                m_channelName,
                CHANNEL_AGENT_TYPE,
                m_contact,
                m_heartbeatInterval);
            // Set the Agent Stopped
            as.update(model::Agent::S_STOPPED);
            m_log_info("Agent State set to Stopped");
            
            // Deregister the DAO Context from the TSS 
            dao::g_tss_dao_context.set(0);
            m_log_debug("DAOContext deregistered from thread-specific storage");
        } catch(const agents::AgentException& exc){
            m_log_error("Failed to Set Agent State to Stopped: " << exc.what());
        }
    } else {
        m_log_error("Invalid Context DAO");
    }
    
    return true;
}
