/*
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *  GLite Data Transfer Agents: VOConfig Class
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *           Paolo Tedesco <paolo.tedesco@cern.ch>
 *  Version info: $Id: ChannelAgent.h,v 1.11 2008/11/13 10:43:55 ptedesco Exp $
 *  Release: $Name:  $
 *
 */
#ifndef GLITE_DATA_TRANSFER_AGENTS_CHANNELAGENT_H_
#define GLITE_DATA_TRANSFER_AGENTS_CHANNELAGENT_H_

#include "glite/data/config/service/ComponentConfiguration.hxx"
#include "glite/data/transfer/agent/scheduler/ActionScheduler.h"
#include <string>
#include <boost/scoped_ptr.hpp>
#include <glite/data/config/service/Logger.hxx>

namespace glite   {
namespace data    {
namespace transfer{
namespace agent   {
    
/*
 * Forward Declaration
 */    
namespace action{
class ActionFactoryMethod;
}
namespace dao{
class CredFactory;
}

/**
 * The ChannelAgent Class
 */    
class ChannelAgent :  public glite::config::ComponentConfiguration,
                      public ActionScheduler::Listener{
public:
    
    typedef enum{
        SHARE_ABSOLUTE = 0,
        SHARE_NORMALIZED,
        SHARE_NORMALIZED_ON_ACTIVE
    } ShareType;
    
    /**
     * Initialize the Component
     */
    virtual int init(const Params& params);
                                                                                                                           
    /**
     * Configure the Component
     */
    virtual int config(const Params& params);
                                                                                                                           
    /**
     * Start the Component
     */
    virtual int start();
                                                                                                                           
    /**
     * Stop the Component
     */
    virtual int stop();
                                                                                                                           
    /**
     * Finailize the Component
     */
    virtual int fini();
                                                                                                                           
    // ActionScheduler Listener Methods
    
    /**
     * Handle Event Start. This method would be called on the Scheduler
     * Thread when the thread is started
     */
    virtual bool onEventStart() /*throw()*/;
    /**
     * Handle Event Stop. This method would be called when the Scheduler
     * Thread when the thread is going to stop
     */
    virtual bool onEventStop() /*throw()*/;
    
protected:
    
    friend ComponentConfiguration * ::create_glite_component();
    friend void ::destroy_glite_component(ComponentConfiguration * component);

    /**
     * Constructor
     */
    ChannelAgent();
                                                                                                                           
    /**
     * Destructor
     */
    virtual ~ChannelAgent();
                        
private:

    /**
     * Create DAO Context
     */
    void createContext() /*throw(ConfigurationException)*/;

    /**
     * Check Channel
     */
    void checkChannel() /*throw(ConfigurationException)*/;

    /**
     * Initialize the Scheduler
     */
    void initScheduler() /*throw(ConfigurationException)*/;

    /**
     * Add a schedulerEntry for the given Action Type
     */
    void scheduleAction(const std::string& type, unsigned int interval, const std::string& params = "") /*throw(ConfigurationException)*/;

    /**
     * Add a schedulerEntry for the given Action Factory Method
     */
    void scheduleAction(class action::ActionFactoryMethod& method, unsigned int interval, const std::string& params = "") /*throw(ConfigurationException)*/;

    /**
     * Finalize the Scheduler
     */
    void finiScheduler();

private:
    glite::config::Logger m_logger;            
    /**
     * The Name of the Channel
     */
    std::string  m_channelName;

    /**
     * The Name of the Contact for that Agent
     */
    std::string  m_contact;

    /**
     * VO Share Type
     */
    ShareType  m_shareType;
    
    /**
     * Fetch Action Type
     */
    std::string m_fetchType;
    /**
     * Check Action Type
     */
    std::string m_checkType;
    /**
     * Cancel Action Type
     */
    std::string m_cancelType;
    /**
     * Default Action Interval
     */
    unsigned int m_defaultInterval;
    /**
     * Fetch Action Interval
     */
    unsigned int m_fetchInterval;
    /**
     * Check Action Interval
     */
    unsigned int m_checkInterval;
    /**
     * Cancel Action Interval
     */
    unsigned int m_cancelInterval;
    /**
     * Heartbeat Action Interval
     */
    unsigned int m_heartbeatInterval;
    /**
     * CleanSDCache Action Interval
     */
    unsigned int m_cleansdcacheInterval;
    /**
     * Action Scheduler
     */
    boost::scoped_ptr<class ActionScheduler> m_scheduler;
        
    /**
     * The DAO Context
     */
    boost::scoped_ptr<class agents::dao::DAOContext> m_daoCtx;
        
    /**
     * The Cred DAO Factory
     */
    boost::scoped_ptr<class dao::CredFactory> m_credFactory;
};

} // end namespace agent
} // end namespace transfer
} // end namespace data
} // end namespace glite

#endif //GLITE_DATA_TRANSFER_AGENTS_CHANNELAGENT_H_
