/*
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *  GLite Data Transfer Agents: DelegCred Class
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *  Version info: $Id: DelegCred.cpp,v 1.6 2008/11/13 10:43:55 ptedesco Exp $
 *  Release: $Name:  $
 *
 */
#include "DelegCred.h"
#include "DelegCredFactory.h"
#include "DelegCredException.h"
#include <glite/data/transfer/agent/model/Cred.h>
#include <glite/data/transfer/agent/dao/cred/CredDAO.h>
#include <glite/data/transfer/agent/dao/cred/DAOFactory.h>
#include <glite/data/transfer/agent/dao/TSSDAOContext.h>
#include <glite/data/transfer/agent/dao/TSSCredFactory.h>
#include <glite/data/agents/AgentExceptions.h>
#include <glite/data/agents/extra/utility.h>

#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <fstream>
#include <boost/scoped_ptr.hpp>

using namespace glite::data::agents;
using namespace glite::data::agents::cred;
using namespace glite::data::transfer::agent::cred::deleg;
using boost::scoped_ptr;

namespace {
const char * const PROXY_NAME_PREFIX         = "x509up_h";
}

/*
 * DelegCred
 *
 * Constructor
 */
DelegCred::DelegCred(const DelegCredFactory& f):
    CredService(f.logger()),
    m_factory(f){
}
    
/*
 * ~DelegCred
 *
 * Destructor
 */
DelegCred::~DelegCred(){
}

/*
 * getNewCertificate
 *
 * Get The proxy Certificate for the requested user
 */
void DelegCred::getNewCertificate(const std::string& userDn, const std::string& cred_id, const std::string& filename) /*throw (LogicError, DelegCredException)*/{
    
    // Get the DAO Context From the Thread Specific Storage
    agents::dao::DAOContext& ctx = dao::g_tss_dao_context.get();
    
    scoped_ptr<dao::CredDAO> credDAO;
    try{
        // Get Cred Factory from the TSS
        dao::CredFactory& factory = dao::g_tss_cred_factory.get();
        credDAO.reset(factory.create(ctx));        
    } catch(const LogicError&){
       // Use the Cred View
       credDAO.reset(dao::cred::DAOFactory::instance().createCredDAO(ctx));
    }    
    try{
        // Get the Cred Id
        scoped_ptr<model::Cred> c(credDAO->get(cred_id,userDn));
        
        // write the content of the certificate property into the file
        std::ofstream ofs(filename.c_str(), std::ios_base::binary);
        if(ofs.bad()){
            m_log_error("Failed open file " << filename << " for writing");
            throw DelegCredException("Cannot open file for storing the new proxy certificate");
        }
        // write the Content of the certificate
        ofs << c->certificate().c_str();
        // Close the file
        ofs.close();
    } catch(const agents::dao::DAOException& exc){
        // Invalid Cred Id 
        m_log_error("Failed to get certificate for user <" << userDn << ">. Reason is: " << exc.what());
        throw DelegCredException("Cannot get certificate from Data Source");
    } catch(const LogicError& exc){
        m_log_error("Logic Error in reading certificate for user <" << userDn << ">. Reason is: " << exc.what());
        throw;
    }
}

/*
 * getFileName
 *
 * Generate a Name for the Certificate Proxy File
 */
std::string DelegCred::getFileName(const std::string& userDn, const std::string& id) /*throw(LogicError)*/ {

    std::string filename;
    
    // Hash the DN:id
    unsigned long h = hash_string(userDn + id);
    std::stringstream ss;
    ss << h;
    std::string h_str = ss.str();

    // Encode the DN
    std::string encoded_dn = encodeName(userDn);

    // Compute Max length
    unsigned long filename_max = pathconf(m_factory.repository().c_str(), _PC_NAME_MAX);
    long max_length = (filename_max - 7 - strlen(PROXY_NAME_PREFIX));
    if(max_length <= 0){
        m_log_error("Failed to generate the proxy file name: prefix name (" << PROXY_NAME_PREFIX << ") too long for " << m_factory.repository());
        throw LogicError("Cannot generate proxy file name: prefix too long");
    }
    if(h_str.length() > (std::string::size_type)max_length){
        m_log_error("Failed to generate the proxy file name: hash (" << h_str << ") too long for " << m_factory.repository());
        throw LogicError("Cannot generate proxy file name: has too long");
    }
    // Generate the filename using the has
    filename = m_factory.repository() + PROXY_NAME_PREFIX + h_str;
    if(h_str.length() < max_length){
        filename.append(encoded_dn.substr(0,(max_length - h_str.length())));
    }
    return filename;
}

/*
 * minValidityTime
 *
 * Returns the validity time that the cached copy of the certificate should 
 * have. 
 */
unsigned long DelegCred::minValidityTime() /* throw() */{
    return m_factory.minValidityTime();
}

/*
 * encodeName
 *
 * Encode a string, is not url encoded since the hash is alweays prefixed
 */
std::string DelegCred::encodeName(const std::string& str) /*throw()*/{

    std::string encoded;
    encoded.reserve(str.length());
    
    // for each character
    std::string::const_iterator s_it;
    for(s_it = str.begin(); s_it != str.end(); ++s_it){
        if (isalnum((*s_it))){
            encoded.push_back(tolower((*s_it)));
        } else {
            encoded.push_back('X');
        }        
    }
    return encoded;
}
