/*
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *  GLite Data Transfer Agents: DAO Utility methods
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>, Paolo Tedesco <paolo.tedesco@cern.ch>
 *  Version info: $Id: dao-utility.cpp,v 1.11 2008/12/11 13:24:44 ptedesco Exp $
 *  Release: $Name:  $
 *
 */
#include "dao-utility.h"
#include "JobFSM.h"
#include "FSMException.h"
#include <glite/data/agents/dao/DAOContextFactory.h>
#include <glite/data/agents/dao/DAOException.h>

#include <string>
#include <map>
#include <sstream>

using namespace glite::data::transfer::agent::model;
using glite::data::agents::dao::DAOLogicError;
using glite::data::agents::dao::DAOException;

namespace glite   {
namespace data    {
namespace transfer{
namespace agent   {
namespace dao     {

/**
 * utility for enum to string conversions
 */    
template <typename Value> class Converter{
public:
    /**
     * constructor
     */
    Converter(const char* type_name) : m_type_name(type_name){}
    /**
     * destructor
     */
    virtual ~Converter(){}
    /**
     * conversion from string
     */
    Value fromstring(const std::string& strval){
        typename StringToEnum::const_iterator it = m_StringToEnum.find(strval);
        if (m_StringToEnum.end() != it){
            return it->second;
        }
        std::stringstream err;
        err << "Invalid " << m_type_name << " string: " << strval;
        throw DAOLogicError(err.str());
    }
    /**
     * conversion to string
     */
    const std::string& tostring(const Value& value){
        typename EnumToString::const_iterator it = m_EnumToString.find(value);
        if (m_EnumToString.end() != it){
            return it->second;
        }
        std::stringstream err;
        err << "Invalid " << m_type_name << " value: " << value;
        throw DAOLogicError(err.str());
    }
    
protected:
    /**
     * add a value-string couple to internal maps
     */
    void add (const Value& v,const std::string& s){
        m_EnumToString[v] = s;
        m_StringToEnum[s] = v;
    }
    // maps
    typedef std::map<std::string,Value> StringToEnum;
    typedef std::map<Value,std::string> EnumToString;
    EnumToString m_EnumToString;
    StringToEnum m_StringToEnum;
    // type name (for error messages)
    std::string m_type_name;
};

//
// Enum converter class definition macro
// name = class name
// type = enum type
// type_name = type name for error messages
// 
#define CONVERTER(name,type,type_name) \
class name : public Converter<type> { \
public: \
    static name instance(){ \
        static name s_instance; \
        return s_instance; \
    } \
protected: \
    name():Converter<type>(type_name){

#define END_CONVERTER \
    } \
};

    
// -----------------------------------------------------------------------------
// -----------------------------------------------------------------------------
// Job
// -----------------------------------------------------------------------------
// -----------------------------------------------------------------------------    

// Constraints
const unsigned int MAX_T_JOB_REASON_LEN = 1024;

/**
 * JobStateConverter
 */
CONVERTER (JobStateConverter,Job::State,"Job State")
    add(Job::S_SUBMITTED        , "Submitted");
    add(Job::S_PENDING          , "Pending");
    add(Job::S_READY            , "Ready");
    add(Job::S_ACTIVE           , "Active");
    add(Job::S_DONE             , "Done");
    add(Job::S_DONE_WITH_ERRORS , "DoneWithErrors");
    add(Job::S_HOLD             , "Hold");
    add(Job::S_CANCELED         , "Canceled");
    add(Job::S_FAILED           , "Failed");
    add(Job::S_FINISHING        , "Finishing");
    add(Job::S_FINISHED         , "Finished");
    add(Job::S_FINISHED_DIRTY   , "FinishedDirty");
// note: S_UNDEF is not allowed
END_CONVERTER

/*
 * translateJobState
 *
 * Translate Job State into a String
 */
const char * translateJobState(Job::State state) /*throw (DAOLogicError)*/{
    return JobStateConverter::instance().tostring(state).c_str();
}

/*
 * translateJobState
 *
 * Translate a String into a Job State
 */
Job::State translateJobState(const char * str_state) /*throw (DAOLogicError)*/{
    return JobStateConverter::instance().fromstring(str_state);
}
    
// -----------------------------------------------------------------------------
// -----------------------------------------------------------------------------
// File
// -----------------------------------------------------------------------------
// -----------------------------------------------------------------------------    

// Constraints
const unsigned int MAX_T_FILE_REASON_LEN = 1024;

/**
 * FileStateConverter
 */
CONVERTER (FileStateConverter,File::State,"File State")
    add(File::S_SUBMITTED                       , "Submitted");
    add(File::S_PENDING                         , "Pending");
    add(File::S_READY                           , "Ready");
    add(File::S_ACTIVE                          , "Active");
    add(File::S_DONE                            , "Done");
    add(File::S_WAITING                         , "Waiting");
    add(File::S_HOLD                            , "Hold");
    add(File::S_FAILED                          , "Failed");
    add(File::S_FINISHING                       , "Finishing");
    add(File::S_FINISHED                        , "Finished");
    add(File::S_AWAITING_PRESTAGE               , "AwaitingPrestage");
    add(File::S_PRESTAGING                      , "Prestaging");
    add(File::S_WAITING_PRESTAGE                , "WaitingPrestage");
    add(File::S_WAITING_CATALOG_RESOLUTION      , "WaitingCatalogResolution");
    add(File::S_WAITING_CATALOG_REGISTRATION    , "WaitingCatalogRegistration");
    add(File::S_CANCELED                        , "Canceled");
    // note: S_UNDEF is not allowed
END_CONVERTER


/*
 * translateState
 *
 * Translate File State into a String
 */
const char * translateFileState(File::State state) /*throw (DAOLogicError)*/{
    return FileStateConverter::instance().tostring(state).c_str();
}

/*
 * translateState
 *
 * Translate a String into a File State
 */
File::State translateFileState(const char * str_state) /*throw (DAOLogicError)*/{
    return FileStateConverter::instance().fromstring(str_state);
}

// -----------------------------------------------------------------------------
// -----------------------------------------------------------------------------
// Error
// -----------------------------------------------------------------------------
// -----------------------------------------------------------------------------    

/**
 * ErrorCategoryConverter
 */
CONVERTER (ErrorCategoryConverter,Error::Category,"Error Category")
    add(Error::UNDEF                    , "");
    add(Error::TRANSFER                 , "TRANSFER_ERROR");
    add(Error::SECURITY                 , "SECURITY_ERROR");
    add(Error::NETWORK                  , "NETWORK_ERROR");
    add(Error::CONFIGURATION            , "CONFIGURATION_ERROR");
    add(Error::CONSISTENCY              , "CONSISTENCY_ERROR");
    add(Error::CATALOG                  , "CATALOG_ERROR");
    add(Error::USER                     , "USER_ERROR");
    add(Error::FILE_EXISTS              , "FILE_EXISTS");
    add(Error::INVALID_PATH             , "INVALID_PATH");
    add(Error::NO_SPACE_LEFT            , "NO_SPACE_LEFT");
    add(Error::INVALID_SIZE             , "INVALID_SIZE");
    add(Error::REQUEST_FAILURE          , "REQUEST_FAILURE");
    add(Error::CONNECTION               , "CONNECTION_ERROR");
    add(Error::HTTP_TIMEOUT             , "HTTP_TIMEOUT");
    add(Error::REQUEST_TIMEOUT          , "REQUEST_TIMEOUT");
    add(Error::TRANSFER_TIMEOUT         , "TRANSFER_TIMEOUT");
    add(Error::INTERNAL_ERROR           , "INTERNAL_ERROR");
    add(Error::ABORTED                  , "ABORTED");
    add(Error::GENERAL_FAILURE          , "GENERAL_FAILURE");
    add(Error::LOST                     , "LOST");
    add(Error::STORAGE_INTERNAL_ERROR   , "STORAGE_INTERNAL_ERROR");
    add(Error::GRIDFTP                  , "GRIDFTP_ERROR");
    add(Error::MARKERS_TIMEOUT          , "TRANSFER_MARKERS_TIMEOUT");
    add(Error::SRM_COPY                 , "SRM_COPY");
    add(Error::CATALOG_INTERNAL_ERROR   , "CATALOG_INTERNAL_ERROR");
    add(Error::LOCALITY                 , "LOCALITY");
    add(Error::FIRST_MARKER_TIMEOUT     , "FIRST_MARKER_TIMEOUT");
    add(Error::NO_PROGRESS              , "NO_PROGRESS");
    // deprecated
    add(Error::AGENT                    , "AGENT_ERROR");
    add(Error::SOURCE                   , "SOURCE_SRM_ERROR");
    add(Error::SOURCE_PREP              , "SOURCE_PREP_SRM_ERROR");
    add(Error::SOURCE_FINAL             , "SOURCE_FINAL_SRM_ERROR");
    add(Error::DESTINATION              , "DEST_SRM_ERROR");
    add(Error::DESTINATION_PREP         , "DEST_PREP_SRM_ERROR");
    add(Error::DESTINATION_FINAL        , "DEST_FINAL_SRM_ERROR");
    add(Error::REMOTE                   , "REMOTE_ERROR");
    add(Error::CATALOG_SRC              , "SOURCE_CATALOG_ERROR");
    add(Error::CATALOG_SRC_RES          , "SOURCE_CATALOG_RESOLUTION_ERROR");
    add(Error::CATALOG_DST              , "DEST_CATALOG_ERROR");
    add(Error::CATALOG_DST_RES          , "DEST_CATALOG_RESOLUTION_ERROR");
    add(Error::CATALOG_DST_REG          , "DEST_CATALOG_REGISTRATION_ERROR");
END_CONVERTER

/**
 * translateCategory
 */
const char * translateErrorCategory(Error::Category c) /*throw (DAOLogicError)*/{
    return ErrorCategoryConverter::instance().tostring(c).c_str();
}

/**
 * translateCategory
 */
Error::Category translateErrorCategory(const char * str_category) /*throw (DAOLogicError)*/{
    return ErrorCategoryConverter::instance().fromstring(str_category);
}

/**
 * ErrorScopeConverter
 */
CONVERTER (ErrorScopeConverter,Error::Scope,"Error Scope")
    add(Error::S_UNDEF          , "");
    add(Error::S_SOURCE         , "SOURCE");
    add(Error::S_DESTINATION    , "DESTINATION");
    add(Error::S_TRANSFER       , "TRANSFER");
    add(Error::S_AGENT          , "AGENT");
END_CONVERTER

/**
 * Translate Error Scope into a String
 */
const char * translateErrorScope(Error::Scope scope) /*throw (DAOLogicError)*/{
    return ErrorScopeConverter::instance().tostring(scope).c_str();
}

/**
 * Translate a String into an Error Scope
 */
Error::Scope translateErrorScope(const char * str_scope) /*throw (DAOLogicError)*/{
    return ErrorScopeConverter::instance().fromstring(str_scope);
}

/**
 * ErrorPhaseConverter
 */
CONVERTER (ErrorPhaseConverter,Error::Phase,"Error Phase")
    add(Error::P_UNDEF                  , "");
    add(Error::P_TX_PREPARATION         , "TRANSFER_PREPARATION");
    add(Error::P_TX_FINALIZATION        , "TRANSFER_FINALIZATION");
    add(Error::P_TRANSFER               , "TRANSFER");
    add(Error::P_ALLOCATION             , "ALLOCATION");
    add(Error::P_CATALOG_RESOLUTION     , "CATALOG_RESOLUTION");
    add(Error::P_CATALOG_REGISTRATION   , "CATALOG_REGISTRATION");
    add(Error::P_PRESTAGING             , "PRESTAGING");
    add(Error::P_FINALIZATION           , "FINALIZATION");
    add(Error::P_WAITING                , "WAITING");
    add(Error::P_WAITING_CATALOG        , "WAITING_CATALOG");
    add(Error::P_CANCELING              , "CANCELING");
    add(Error::P_TX_SERVICE             , "TRANSFER_SERVICE");
    add(Error::P_PRE_TRANSFER           , "PRE_TRANSFER");
    add(Error::P_POST_TRANSFER          , "POST_TRANSFER");
    add(Error::P_CHECK_READINESS        , "CHECK_READINESS");
END_CONVERTER

/**
 * Translate Error Phase into a String
 */
const char * translateErrorPhase(Error::Phase phase) /*throw (DAOLogicError)*/{
    return ErrorPhaseConverter::instance().tostring(phase).c_str();
}

/**
 * Translate a String into an Error Phase
 */
Error::Phase translateErrorPhase(const char * str_phase) /*throw (DAOLogicError)*/{
    return ErrorPhaseConverter::instance().fromstring(str_phase);
}

// -----------------------------------------------------------------------------
// -----------------------------------------------------------------------------
// Transfer
// -----------------------------------------------------------------------------
// -----------------------------------------------------------------------------    

const unsigned int MAX_T_TRANSFER_REASON_LEN = 1024;

CONVERTER (TransferStateConverter,Transfer::State,"Transfer State")
    add(Transfer::S_RECEIVED    , "Received");
    add(Transfer::S_PREPARING   , "Preparing");
    add(Transfer::S_READY       , "Ready");
    add(Transfer::S_SUBMITTED   , "Submitted");
    add(Transfer::S_PROCESSING  , "Processing");
    add(Transfer::S_TRANSFERRED , "Transferred");
    add(Transfer::S_COMPLETED   , "Completed");
    add(Transfer::S_FAILED      , "Failed");
    add(Transfer::S_ABORTED     , "Aborted");
END_CONVERTER

/*
 * Translate Transfer State into a String
 */
const char * translateTransferState(Transfer::State state) /*throw (DAOLogicError)*/{
    return TransferStateConverter::instance().tostring(state).c_str();
}

/*
 * translateState
 *
 * Translate a String into a Transfer State
 */
Transfer::State translateTransferState(const char * str_state) /*throw (DAOLogicError)*/{
    return TransferStateConverter::instance().fromstring(str_state);
}


// -----------------------------------------------------------------------------
// -----------------------------------------------------------------------------
// StageReq
// -----------------------------------------------------------------------------
// -----------------------------------------------------------------------------    

const unsigned int MAX_T_STAGEREQ_REASON_LEN = 1024;

CONVERTER (StageReqStateConverter,StageReq::State,"StageReq State")
    add(StageReq::S_SUBMITTED   , "Submitted");
    add(StageReq::S_PROCESSING  , "Processing");
    add(StageReq::S_STAGED      , "Staged");
    add(StageReq::S_FINALIZING  , "Finalizing");
    add(StageReq::S_COMPLETED   , "Completed");
    add(StageReq::S_FAILED      , "Failed");
    add(StageReq::S_ABORTED     , "Aborted");
END_CONVERTER

/*
 * Translate StageReq State into a String
 */
const char * translateStageReqState(StageReq::State state) /*throw (DAOLogicError)*/{
    return StageReqStateConverter::instance().tostring(state).c_str();
}

/*
 * translateState
 *
 * Translate a String into a StageReq State
 */
StageReq::State translateStageReqState(const char * str_state) /*throw (DAOLogicError)*/{
    return StageReqStateConverter::instance().fromstring(str_state);
}

// -----------------------------------------------------------------------------
// -----------------------------------------------------------------------------
// Agent
// -----------------------------------------------------------------------------    
// -----------------------------------------------------------------------------

const char * const VO_PREFIX                     = "vo:";
const char * const CHANNEL_PREFIX                = "channel:";
const char * const CRED_PREFIX                   = "cred:";

/**
 * ChannelTypeConverter
 */
CONVERTER(AgentStateConverter,Agent::State,"Agent State")
    add(Agent::S_STARTED    ,"Started");
    add(Agent::S_STOPPED    ,"Stopped");
END_CONVERTER

/*
 * translateState
 *
 * Translate Agent State into a String
 */
const char * translateAgentState(Agent::State state) /*throw (DAOLogicError)*/{
    return AgentStateConverter::instance().tostring(state).c_str();
}

/*
 * translateState
 *
 * Translate a String into a Agent State
 */
Agent::State translateAgentState(const char * str_state) /*throw (DAOLogicError)*/{
    return AgentStateConverter::instance().fromstring(str_state);
}


// -----------------------------------------------------------------------------
// -----------------------------------------------------------------------------
// Schema Version
// -----------------------------------------------------------------------------    
// -----------------------------------------------------------------------------
const char * const T_SCHEMA_VERS              = "t_schema_vers";
const char * const T_CREDENTIAL_VERS          = "t_credential_vers";

const unsigned int SCHEMA_VERS_MAJOR    = QUEUE_SCHEMA_VERSION_MAJOR;
const unsigned int SCHEMA_VERS_MINOR    = QUEUE_SCHEMA_VERSION_MINOR;
const unsigned int SCHEMA_VERS_PATCH    = QUEUE_SCHEMA_VERSION_PATCH;

const unsigned int CREDENTIAL_VERS_MAJOR = QUEUE_CRED_VERSION_MAJOR;
const unsigned int CREDENTIAL_VERS_MINOR = QUEUE_CRED_VERSION_MINOR;
const unsigned int CREDENTIAL_VERS_PATCH = QUEUE_CRED_VERSION_PATCH;

/*
 * checkSchema
 *
 * Validate the DB schema
 */
void check_schema(agents::dao::DAOContext& ctx) /*throw (DAOLogicError,DAOException)*/{
    agents::dao::DAOContextFactory::instance().checkSchema(ctx,
        T_SCHEMA_VERS, SCHEMA_VERS_MAJOR,SCHEMA_VERS_MINOR,SCHEMA_VERS_PATCH);
}

/*
 * checkCredSchema
 *
 * Validate the DB schema
 */
void check_cred_schema(agents::dao::DAOContext& ctx) /*throw (DAOLogicError,DAOException)*/{
    agents::dao::DAOContextFactory::instance().checkSchema(ctx,
        T_CREDENTIAL_VERS, CREDENTIAL_VERS_MAJOR,CREDENTIAL_VERS_MINOR,CREDENTIAL_VERS_PATCH);
}

// -----------------------------------------------------------------------------
// -----------------------------------------------------------------------------
// Channel
// -----------------------------------------------------------------------------    
// -----------------------------------------------------------------------------

/**
 * ChannelStateConverter
 */
CONVERTER (ChannelStateConverter,Channel::State,"Channel State")
    add(Channel::S_ACTIVE   , "Active");
    add(Channel::S_DRAIN    , "Drain");
    add(Channel::S_INACTIVE , "Inactive");
    add(Channel::S_STOPPED  , "Stopped");
    add(Channel::S_HALTED   , "Halted");
    add(Channel::S_ARCHIVED , "Archived");
END_CONVERTER

/*
 * translateState
 *
 * Translate Channel State into a String
 */
const char * translateChannelState(Channel::State state) /*throw (DAOLogicError)*/{
    return ChannelStateConverter::instance().tostring(state).c_str();
}

/*
 * translateState
 *
 * Translate a String into a Channel State
 */
Channel::State translateChannelState(const char * str_state) /*throw (DAOLogicError)*/{
    return ChannelStateConverter::instance().fromstring(str_state);
}

/**
 * ChannelVOStateConverter
 */
CONVERTER (ChannelVOStateConverter,Channel::VOState,"Channel VOState")
    add(Channel::VS_UNDEF   , "");
    add(Channel::VS_ON      , "ON");
    add(Channel::VS_OFF     , "OFF");
    add(Channel::VS_PAUSE   , "PAUSE");
END_CONVERTER


/*
 * translateVOState
 *
 * Translate Channel VOState into a String
 */
const char * translateChannelVOState(Channel::VOState state) /*throw (DAOLogicError)*/{
    return ChannelVOStateConverter::instance().tostring(state).c_str();
}

/*
 * translateVOState
 *
 * Translate a String into a Channel VOState
 */
Channel::VOState translateChannelVOState(const char * str_state) /*throw (DAOLogicError)*/{
    return ChannelVOStateConverter::instance().fromstring(str_state);
}

/**
 * ChannelTypeConverter
 */
CONVERTER(ChannelTypeConverter,Channel::Type,"Channel Type")
    add(Channel::T_UNDEF        ,"");
    add(Channel::T_DEDICATED    ,"DEDICATED");
    add(Channel::T_NON_DEDICATED,"NONDEDICATED");
END_CONVERTER

/*
 * translateType
 *
 * Translate Channel Type into a String
 */
const char * translateChannelType(Channel::Type type) /*throw (DAOLogicError)*/{
    return ChannelTypeConverter::instance().tostring(type).c_str();
}

/*
 * translateType
 *
 * Translate a String into a Channel Type
 */
Channel::Type translateChannelType(const char * str_type) /*throw (DAOLogicError)*/{
    return ChannelTypeConverter::instance().fromstring(str_type);
}

/**
 * ChannelLogLevelConverter
 */
CONVERTER(ChannelLogLevelConverter,Channel::LogLevel,"Channel Log Level")
    add(Channel::LL_UNDEF,  "");
    add(Channel::LL_DEBUG,  "DEBUG");
    add(Channel::LL_INFO,   "INFO");
    add(Channel::LL_WARN,   "WARN");
    add(Channel::LL_ERROR,  "ERROR");
END_CONVERTER

/**
 * Translate Channel LogLevel to a string
 */
const char * translateChannelLogLevel(Channel::LogLevel level) /*throw (DAOLogicError)*/{
    return ChannelLogLevelConverter::instance().tostring(level).c_str();
}

/**
 * Translate a string to a Channel LogLevel 
 */
Channel::LogLevel translateChannelLogLevel(const char * str_loglevel) /*throw (DAOLogicError)*/{
    return ChannelLogLevelConverter::instance().fromstring(str_loglevel);
}

/**
 * ChannelLogLevelConverter
 */
CONVERTER(ChannelSrmCopyDirectionConverter,Channel::SrmCopyDirection,"Channel SRM Copy Direction")
    add(Channel::SCD_UNDEF, "");
    add(Channel::SCD_PULL,  "PULL");
    add(Channel::SCD_PUSH,  "PUSH");
END_CONVERTER

/**
 * Translate Channel SrmCopyDirection to a string
 */
const char * translateChannelSrmCopyDirection(Channel::SrmCopyDirection direction) /*throw (DAOLogicError)*/{
    return ChannelSrmCopyDirectionConverter::instance().tostring(direction).c_str();
}

/**
 * Translate a string to a Channel SrmCopyDirection
 */
Channel::SrmCopyDirection translateChannelSrmCopyDirection(const char * str_direction) /*throw (DAOLogicError)*/{
    return ChannelSrmCopyDirectionConverter::instance().fromstring(str_direction);
}

// -----------------------------------------------------------------------------
// -----------------------------------------------------------------------------
// Utility Methods
// -----------------------------------------------------------------------------
// -----------------------------------------------------------------------------    

/*
 * update_job_state
 *
 * Update the Status of a job
 */
bool update_job_state(Job& job, const agents::dao::StringArray& file_states) /*throw (DAOLogicError)*/{

    bool result = false;
    
    // Merge all The File States into a signle variable
    File::State file_state = File::S_UNDEF;
    agents::dao::StringArray::const_iterator it;
    for(it = file_states.begin(); it != file_states.end(); ++it){
        file_state = File::State(file_state | translateFileState((*it).c_str()));
    }
    
    // Update Job State
    fsm::JobFSM j_fsm(job);
    try {
        Job::State old_state = job.state();
        j_fsm.updateState(file_state);
    
        // Check if the job state is changed        
        result = (old_state == job.state()) ? false:true;
    } catch(const fsm::FSMException& exc){
        throw DAOLogicError(exc.what());
    }
    return result;
}

} // End namespace dao
} // End namespace agent
} // End namespace transfer
} // End namespace data
} // End namespace glite


