/*
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *  GLite Data Transfer Agents: OracleAgentStatements Class
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *           Paolo Tedesco <paolo.tedesco@cern.ch>
 *  Version info: $Id: AgentStatements.cpp,v 1.7 2008/11/13 10:43:55 ptedesco Exp $
 *  Release: $Name:  $
 *
 */
#include "AgentStatements.h"
#include "dao-utility.h"
#include <glite/data/agents/dao/oracle/StatementPtr.h>

#include <sstream>

using namespace glite::data::transfer::agent;
using namespace glite::data::transfer::agent::dao;
using namespace glite::data::transfer::agent::dao::oracle;
using glite::data::agents::dao::DAOLogicError;
using glite::data::agents::dao::DAOException;
using glite::data::agents::dao::oracle::OracleDAOContext;
using glite::data::agents::dao::oracle::StatementPtr;
using namespace ::oracle::occi;

namespace {
const char * const ORACLE_AGENT_STATEMENTS_NAME = "transfer-agent-dao-oracle-agent-stmt";

const char * const GET_STATEMENT_TAG                   = "t_agent_get";
const char * const LOCK_STATEMENT_TAG                  = "t_agent_lock";
}

/*
 * instance
 *
 * Return Singleton instance
 */
AgentStatements& AgentStatements::instance() /*throw ()*/{
    static AgentStatements singleton;
    return singleton;
}

/*
 * AgentStatements
 *
 * Constructor
 */
AgentStatements::AgentStatements():
    m_logger(ORACLE_AGENT_STATEMENTS_NAME){
}

/*
 * dispose
 *
 * dispose Statements For OracleTransferDAO
 */
void AgentStatements::dispose(OracleDAOContext& ctx) {
    
    // terminate Statements
/*    terminateStatement(GET_STATEMENT_TAG);
    terminateStatement(LOCK_STATEMENT_TAG);
    */
}

/*
 * createGet
 *
 * Prepare Statement For AgentDAO::get
 *
 * Bind Parameters:
 *  :1 - Agent Name
 */
Statement * AgentStatements::createGet(OracleDAOContext& ctx, bool lock) /*throw (DAOLogicError,DAOException)*/{

    // Get Tag
    const char * const tag = (lock)?LOCK_STATEMENT_TAG:GET_STATEMENT_TAG;

    Statement * stmt = 0;
    try {
        // Check if the statement is already cached
        if(false == ctx.connection().isCached("",tag)){
            // Prepare Statement
            std::stringstream query_stmt;
            query_stmt << "SELECT agent_type"
                       ", agent_state"
                       ", agent_version"
                       ", agent_host"
                       ", contact"
                       ", last_active"
                       " FROM t_agent"
                       " WHERE agent_name = :1";
            if(lock) {
                query_stmt << " FOR UPDATE WAIT 60";
            }
            m_log_trace("Preparing Statement: " << query_stmt.str());
            // Create Statements
            stmt = ctx.connection().createStatement(query_stmt.str(),tag);
        } else {
            // Create Statements
            stmt = ctx.connection().createStatement("",tag);
        }
    } catch(const SQLException& exc){
        m_log_error("Got Exception in creating Get Statement: [0x" << exc.getErrorCode() << "] " << exc.getMessage());
        throw DAOException("Failed to prepare Statement");
    }
    // Check Postcondition
    if(0 == stmt){
        throw DAOLogicError("Failed to prepare Statement - Nullpointer");
    }
    return stmt;
}

/*
 * createUpdate
 *
 * Prepare Statement For AgentDAO::update
 */
Statement * AgentStatements::createUpdate(OracleDAOContext& ctx, const model::Agent& agent, const std::string& agent_name) /*throw (DAOLogicError,DAOException)*/{

    Statement * stmt = 0;        
    try {
        // Update Agent Properties
        // -------------------------
        // | Column        | RO/RW |
        // -------------------------
        // | agent_name    |  RO   |
        // | agent_type    |  RO   |
        // | agent_state   |  RW   |
        // | agent_host    |  RO   |
        // | agent_version |  RO   |
        // | last_active   |  RW   |
        // | agent_dn      |  RO   |
        // | contact       |  RO   |
        // -------------------------
                    
        // Prepare Update Statement
        unsigned int count = 1;
        std::stringstream query_stmt;
        query_stmt << "UPDATE t_agent SET";
        // State
        if(agent.state.isSet()){
            query_stmt << ((1 == count)?"":",") << " t_agent.agent_state = :" << count;
            ++count;
        }
        if(agent.lastActive.isSet()){
            query_stmt << ((1 == count)?"":",") << " t_agent.last_active = :" << count;
            ++count;
        } else {
            if(agent.lastActive() == (time_t)-1){
                query_stmt << ((1 == count)?"":",") << " t_agent.last_active = SYSTIMESTAMP";
            }
        }
        query_stmt <<  " WHERE t_agent.agent_name = :" << count;
        if(count > 1){
            m_log_trace("Preparing Statement: " << query_stmt.str());
            // Create Statement
            stmt = ctx.connection().createStatement(query_stmt.str());

            // RAII
            StatementPtr stmt_ptr(ctx.connection(),stmt);
            
            // Set the value of the properties
            count = 1;            
            // State
            if(agent.state.isSet()){
                stmt->setString(count++,translateAgentState(agent.state()));
            }  
            // last_active
            if(agent.lastActive.isSet()){
                if((time_t)-1 != agent.lastActive()){
                    stmt->setTimestamp(count++,ctx.toTimestamp(agent.lastActive()));
                } else {
                    stmt->setNull(count++ ,OCCITIMESTAMP);
                }
            }
            // Set agent Name
            stmt->setString(count,agent_name);
            
            // Release the Pointer
            stmt_ptr.release();
        } else {
            m_log_trace("Nothing to update");
        }
    } catch(const SQLException& exc){
        m_log_error("Got Exception in creating Update Statement: [0x" << exc.getErrorCode() << "] " << exc.getMessage());
        throw DAOException("Failed to prepare Statement");
    }
    // Don't Check Postcondition (Null is allowed)   
    return stmt;
}

/*
 * createCreate
 *
 * Return the statement for the AgentDAO::create
 */
Statement* AgentStatements::createCreate(OracleDAOContext& ctx, const model::Agent& agent, const std::string& agent_name) /*throw (DAOLogicError,DAOException)*/{
    
    Statement * stmt = 0;        
    try {
        // Prepare Update Statement
        const char * const query_stmt =
           "MERGE INTO t_agent"
           " USING ( SELECT COUNT(*) AS count_name "
                     "FROM DUAL WHERE EXISTS ("
                        " SELECT NULL FROM t_agent"
                        " WHERE agent_name = :1 )"
           ") ON ( t_agent.agent_name = :2 AND count_name = 1)"
           " WHEN MATCHED THEN UPDATE SET "
           " agent_type = :3"
           " , agent_state = :4"
           " , agent_host = :5"
           " , agent_version = :6"
           " , contact = :7"
           " , last_active= NVL(:8,SYSTIMESTAMP)"
           " , agent_dn = :9"
           " WHEN NOT MATCHED THEN INSERT ("
           " agent_name"
           " , agent_type"
           " , agent_state"
           " , agent_host"
           " , agent_version"
           " , contact"
           " , last_active"
           " , agent_dn"
           " ) VALUES ("
           "   :10"
           " , :11"
           " , :12"
           " , :13"
           " , :14"
           " , :15"
           " , NVL(:16,SYSTIMESTAMP)"
           " , :17"
           " )";
        m_log_trace("Preparing Statement: " << query_stmt);
        // Create Statement
        stmt = ctx.connection().createStatement(query_stmt);
        
         // RAII
        StatementPtr stmt_ptr(ctx.connection(),stmt);
        
        // Set the value of the properties
        stmt->setString(1 ,agent_name);
        stmt->setString(2 ,agent_name);
        stmt->setString(10,agent_name);
        stmt->setString(3 ,agent.type());
        stmt->setString(11,agent.type());
        stmt->setString(4 ,translateAgentState(agent.state()));
        stmt->setString(12,translateAgentState(agent.state()));
        stmt->setString(5 ,agent.hostName());
        stmt->setString(13,agent.hostName());
        stmt->setString(6 ,agent.version());
        stmt->setString(14,agent.version());
        stmt->setString(7 ,agent.contact());
        stmt->setString(15,agent.contact());
        if((time_t)-1 != agent.lastActive()){
            stmt->setTimestamp(8 ,ctx.toTimestamp(agent.lastActive()));
            stmt->setTimestamp(16,ctx.toTimestamp(agent.lastActive()));
        } else {
            stmt->setNull(8 ,OCCITIMESTAMP);
            stmt->setNull(16,OCCITIMESTAMP);
        }
        stmt->setString(9,ctx.agentDn());
        stmt->setString(17,ctx.agentDn());
        
        // Release the Pointer
        stmt_ptr.release();
    } catch(const SQLException& exc){
        m_log_error("Got Exception in creating Insert Statement: [0x" << exc.getErrorCode() << "] " << exc.getMessage());
        throw DAOException("Failed to prepare Statement");
    }
    // Check Postcondition
    if(0 == stmt){
        throw DAOLogicError("Failed to prepare Statement - Nullpointer");
    }
    return stmt;
}
