/*
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *  GLite Data Transfer Agents: ChannelStatements Class
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *           Paolo Tedesco <paolo.tedesco@cern.ch>
 *  Version info: $Id: ChannelStatements.cpp,v 1.28 2008/12/04 08:53:30 ptedesco Exp $
 *  Release: $Name:  $
 *
 */
#include "ChannelStatements.h"
#include "dao-utility.h"
#include <glite/data/agents/dao/oracle/StatementPtr.h>

using namespace glite::data::transfer::agent;
using namespace glite::data::transfer::agent::dao;
using namespace glite::data::transfer::agent::dao::oracle;
using glite::data::agents::dao::DAOLogicError;
using glite::data::agents::dao::DAOException;
using glite::data::agents::dao::oracle::OracleDAOContext;
using glite::data::agents::dao::oracle::StatementPtr;
using namespace ::oracle::occi;


namespace {
const char * const ORACLE_CHANNEL_STATEMENTS_NAME = "transfer-agent-dao-oracle-ch-stmt";

const char * const GET_STATEMENT_VO_TAG          = "t_channel_get_vo";
const char * const GET_STATEMENT_CH_TAG          = "t_channel_get_ch";
const char * const UPDATE_LAST_ACTIVE_STATEMENT_CH_TAG = "t_channel_update_la_ch";
const char * const GET_BY_SITES_STATEMENT_VO_TAG = "t_channel_get_by_sites_vo";
const char * const HAS_SHARE_STATEMENT_VO_TAG    = "t_channel_has_share_vo";
const char * const GET_VO_NAMES_STATEMENT_CH_TAG = "t_channel_get_vo_names_ch";
const char * const GET_SHARE_STATEMENT_CH_TAG    = "t_channel_get_share_ch";
const char * const GET_LIMIT_STATEMENT_CH_TAG    = "t_channel_get_limit_ch";
const char * const IS_BLACKLISTED_STATEMENT_VO_TAG = "t_channel_is_blacklisted_vo";
const char * const GET_SITE_GROUPS_STATEMENT_TAG = "t_channel_get_site_names";
const char * const UPDATE_TRANSFER_TYPE_STATEMENT_CH_TAG = "t_channel_update_transfer_type_ch";
}

/*
 * instance
 *
 * Return Singleton instance
 */
ChannelStatements& ChannelStatements::instance() /*throw ()*/{
    static ChannelStatements singleton;
    return singleton;
}

/*
 * ChannelStatements
 *
 * Constructor
 */
ChannelStatements::ChannelStatements():
    m_logger(ORACLE_CHANNEL_STATEMENTS_NAME){
}
/*
 * dispose
 *
 * dispose Statements For OracleChannelDAO
 */
void ChannelStatements::dispose(OracleDAOContext& ctx) {
#warning "TODO : terminate Statements"
}

/*
 * createGet_VO
 *
 * Prepare Statement For ChannelDAO::get, VO View.
 *
 * Bind Parameters:
 *  :1 - Channel Name
 *  :2 - VO Name
 */
Statement * ChannelStatements::createGet_VO(OracleDAOContext& ctx) /*throw (DAOLogicError,DAOException)*/{

    // Get tag
    const char * const tag = GET_STATEMENT_VO_TAG;

    Statement * stmt = 0;
    try {
        // Check if the statement is already cached
        if(false == ctx.connection().isCached("",tag)){
            // Prepare Statement String
            const char * const query_stmt = 
                "SELECT t_channel.channel_name"
                ", t_channel.channel_state"
                ", t_channel.source_site"
                ", t_channel.dest_site"
                ", t_channel.nofiles"
                ", t_channel.nostreams"
                ", t_channel.tcp_buffer_size"
                ", t_channel.bandwidth"
                ", t_channel.nominal_throughput"
                ", t_channel.contact"
                ", t_channel.message"
                ", t_channel.admin_dn"
                ", t_channel.last_active"
                ", t_channel.last_modification"
                ", t_channel.se_limit"
                ", t_channel.channel_type"
                //--------------------------------------------------------------
                // Added with schema version 3.1.0
                ", t_channel.blocksize"
                ", t_channel.http_to"
                ", t_channel.tx_loglevel"
                ", t_channel.urlcopy_put_to"
                ", t_channel.urlcopy_putdone_to"
                ", t_channel.urlcopy_get_to"
                ", t_channel.urlcopy_getdone_to"
                ", t_channel.urlcopy_tx_to"
                ", t_channel.urlcopy_txmarks_to"
                ", t_channel.srmcopy_direction"
                ", t_channel.srmcopy_to"
                ", t_channel.srmcopy_refresh_to"
                //--------------------------------------------------------------
                // Added with schema version 3.2.0
                ", t_channel.target_dir_check"
                ", t_channel.url_copy_first_txmark_to"
                //--------------------------------------------------------------
                // Added with schema version 3.3.0
                ", t_channel.tx_to_per_mb"
                ", t_channel.no_tx_activity_to"
                ", t_channel.preparing_files_ratio"
                ", t_channel.transfer_type"
                //--------------------------------------------------------------
                " FROM t_channel, t_channel_vo_share"
                " WHERE t_channel.channel_name = t_channel_vo_share.channel_name"
                " AND t_channel.channel_name = :1"
                " AND t_channel_vo_share.vo_name = :2";
            
            m_log_trace("Preparing statement: " << query_stmt);
            
            // Create Statement
            stmt = ctx.connection().createStatement(query_stmt,tag);
        } else {
            // Create Statement
            stmt  = ctx.connection().createStatement("",tag);
        }
    } catch(const SQLException& exc){
        m_log_error("Got Exception in creating Get Statement: [0x" << exc.getErrorCode() << "] " << exc.getMessage());
        throw DAOException("Failed to prepare Statement");
    }
    // Check Postcondition
    if(0 == stmt){
        throw DAOLogicError("Failed to prepare Statement - Nullpointer");
    }
    return stmt;
}

/*
 * createGet_Channel
 *
 * Prepare Statement For ChannelDAO::get, Channel View
 *
 * Bind Parameters:
 *  :1 - Channel Name
 */
Statement * ChannelStatements::createGet_Channel(OracleDAOContext& ctx) /*throw (DAOLogicError,DAOException)*/{

    // Get tag
    const char * const tag = GET_STATEMENT_CH_TAG;

    Statement * stmt = 0;
    try {
        // Check if the statement is already cached
        if(false == ctx.connection().isCached("",tag)){
            // Prepare Statement String
            const char * const query_stmt =
                "SELECT t_channel.channel_name"
                ", t_channel.channel_state"
                ", t_channel.source_site"
                ", t_channel.dest_site"
                ", t_channel.nofiles"
                ", t_channel.nostreams"
                ", t_channel.tcp_buffer_size"
                ", t_channel.bandwidth"
                ", t_channel.nominal_throughput"
                ", t_channel.contact"
                ", t_channel.message"
                ", t_channel.admin_dn"
                ", t_channel.last_active"
                ", t_channel.last_modification"
                ", t_channel.se_limit"
                ", t_channel.channel_type"
                //--------------------------------------------------------------
                // Added with schema version 3.1.0
                ", t_channel.blocksize"
                ", t_channel.http_to"
                ", t_channel.tx_loglevel"
                ", t_channel.urlcopy_put_to"
                ", t_channel.urlcopy_putdone_to"
                ", t_channel.urlcopy_get_to"
                ", t_channel.urlcopy_getdone_to"
                ", t_channel.urlcopy_tx_to"
                ", t_channel.urlcopy_txmarks_to"
                ", t_channel.srmcopy_direction"
                ", t_channel.srmcopy_to"
                ", t_channel.srmcopy_refresh_to"
                //--------------------------------------------------------------
                // Added with schema version 3.2.0
                ", t_channel.target_dir_check"
                ", t_channel.url_copy_first_txmark_to"
                //--------------------------------------------------------------
                // Added with schema version 3.3.0
                ", t_channel.tx_to_per_mb"
                ", t_channel.no_tx_activity_to"
                ", t_channel.preparing_files_ratio"
                ", t_channel.transfer_type"
                //--------------------------------------------------------------
                " FROM t_channel"
                " WHERE t_channel.channel_name = :1"
                " AND t_channel.channel_name = :2";
            
            m_log_trace("Preparing statement: " << query_stmt);
            
            // Create Statement
            stmt = ctx.connection().createStatement(query_stmt,tag);
        } else {
            // Create StatementChannel
            stmt  = ctx.connection().createStatement("",tag);
        }
    } catch(const SQLException& exc){
        m_log_error("Got Exception in creating Get Statement: [0x" << exc.getErrorCode() << "] " << exc.getMessage());
        throw DAOException("Failed to prepare Statement");
    }
    // Check Postcondition
    if(0 == stmt){
        throw DAOLogicError("Failed to prepare Statement - Nullpointer");
    }
    return stmt;
}
    
/*
 * createUpdateState_Channel
 *
 * Prepare Statement For ChannelDAO::update, Channel View
 *
 * Bind Parameters:
 *  :1 - Last Active
 *  :2 - Channel Name
 *  :3 - Channel Name
 */
Statement * ChannelStatements::createUpdateLastActive_Channel(OracleDAOContext& ctx) /*throw (DAOLogicError,DAOException)*/{

    // Get tag
    const char * const tag = UPDATE_LAST_ACTIVE_STATEMENT_CH_TAG;

    Statement * stmt = 0;
    try {
        // Check if the statement is already cached
        if(false == ctx.connection().isCached("",tag)){
            
            // Update Channel LastActive Property
            // -----------------------------------
            
            // Prepare Statement String
            const char * const query_stmt = 
                "UPDATE t_channel SET last_active = :1" 
                " WHERE t_channel.channel_name = :2";
            
            m_log_trace("Preparing statement: " << query_stmt);
            // Create Statements
            stmt = ctx.connection().createStatement(query_stmt,tag);
        } else {
            // Create Statements
            stmt = ctx.connection().createStatement("",tag);
        }
    } catch(const SQLException& exc){
        m_log_error("Got Exception in creating Update Statement: [0x" << exc.getErrorCode() << "] " << exc.getMessage());
        throw DAOException("Failed to prepare Statement");
    }
    // Check Postcondition
    if(0 == stmt){
        throw DAOLogicError("Failed to prepare Statement - Nullpointer");
    }
    return stmt;
}

Statement * ChannelStatements::createUpdateTransferType_Channel(OracleDAOContext& ctx) /*throw (DAOLogicError,DAOException)*/{

    // Get tag
    const char * const tag = UPDATE_TRANSFER_TYPE_STATEMENT_CH_TAG;

    Statement * stmt = 0;
    try {
        // Check if the statement is already cached
        if(false == ctx.connection().isCached("",tag)){
            
            // Update Channel TransferType Property
            // -----------------------------------
            
            // Prepare Statement String
            const char * const query_stmt = 
                "UPDATE t_channel SET transfer_type = :1" 
                " WHERE t_channel.channel_name = :2";
            
            m_log_trace("Preparing statement: " << query_stmt);
            // Create Statements
            stmt = ctx.connection().createStatement(query_stmt,tag);
        } else {
            // Create Statements
            stmt = ctx.connection().createStatement("",tag);
        }
    } catch(const SQLException& exc){
        m_log_error("Got Exception in creating Update Statement: [0x" << exc.getErrorCode() << "] " << exc.getMessage());
        throw DAOException("Failed to prepare Statement");
    }
    // Check Postcondition
    if(0 == stmt){
        throw DAOLogicError("Failed to prepare Statement - Nullpointer");
    }
    return stmt;
}

/*
 * createGetBySites_VO
 *
 * Prepare Statement For ChannelDAO::getBySites, VO View
 *
 * Bind Parameters:
 *  :1 - Source Site
 *  :2 - Dest Site
 *  :3 - VO Name
 */
Statement * ChannelStatements::createGetBySites_VO(OracleDAOContext& ctx) /*throw (DAOLogicError,DAOException)*/{

    // Get tag
    const char * const tag = GET_BY_SITES_STATEMENT_VO_TAG;

    Statement * stmt = 0;
    try {
        // Check if the statement is already cached
        if(false == ctx.connection().isCached("",tag)){
            // Prepare Statement String
            const char * const query_stmt =
                "SELECT a.name FROM ("
                    "SELECT name FROM ("
                        " SELECT channel_name as name"
                        " FROM t_channel"
                        " WHERE source_site = :2 AND"
                              " dest_site = :3"
                        " AND channel_state IN ('Active','Inactive','Halted') "
                        " ORDER BY source_site DESC, dest_site DESC )"
                    " WHERE ROWNUM<=1 ) a"
                    " LEFT OUTER JOIN t_channel_vo_share"
                    " ON a.name = t_channel_vo_share.channel_name"
                    " AND t_channel_vo_share.vo_name = :3"
                    " AND (t_channel_vo_share.channel_vostate IS NULL OR "
                    "      t_channel_vo_share.channel_vostate <> 'OFF')"
                  " WHERE t_channel_vo_share.channel_share IS NOT NULL";
            
            m_log_trace("Preparing statement: " << query_stmt);
            // Create Statement
            stmt = ctx.connection().createStatement(query_stmt,tag);
        } else {
            // Create Statement
            stmt  = ctx.connection().createStatement("",tag);
        }
    } catch(const SQLException& exc){
        m_log_error("Got Exception in creating GetBySites Statement: [0x" << exc.getErrorCode() << "] " << exc.getMessage());
        throw DAOException("Failed to prepare Statement");
    }
    // Check Postcondition
    if(0 == stmt){
        throw DAOLogicError("Failed to prepare Statement - Nullpointer");
    }
    return stmt;
}

/*
 * createHasShare_VO
 *
 * Prepare Statement For ChannelDAO::hasShare, VO View
 *
 * Bind Parameters:
 *  :1 - Channel Name
 *  :2 - VO Name
 */
Statement * ChannelStatements::createHasShare_VO(OracleDAOContext& ctx) /*throw (DAOLogicError,DAOException)*/{

    // Get tag
    const char * const tag = HAS_SHARE_STATEMENT_VO_TAG;

    Statement * stmt = 0;
    try {
        // Check if the statement is already cached
        if(false == ctx.connection().isCached("",tag)){
            // Prepare Statement String
            const char * const query_stmt =
                "SELECT t_channel.channel_name"
                " FROM t_channel, t_channel_vo_share"
                " WHERE t_channel.channel_name = t_channel_vo_share.channel_name"
                " AND t_channel.channel_name = :1"
                " AND t_channel_vo_share.vo_name = :2"
                " AND t_channel_vo_share.channel_share IS NOT NULL"
                " AND (t_channel_vo_share.channel_vostate IS NULL OR "
                "      t_channel_vo_share.channel_vostate <> 'OFF')"
                " AND ROWNUM <=1";
            
            m_log_trace("Preparing statement: " << query_stmt);
            // Create Statement
            stmt = ctx.connection().createStatement(query_stmt,tag);
        } else {
            // Create Statement
            stmt  = ctx.connection().createStatement("",tag);
        }
    } catch(const SQLException& exc){
        m_log_error("Got Exception in creating HasShare Statement: [0x" << exc.getErrorCode() << "] " << exc.getMessage());
        throw DAOException("Failed to prepare Statement");
    }
    // Check Postcondition
    if(0 == stmt){
        throw DAOLogicError("Failed to prepare Statement - Nullpointer");
    }
    return stmt;
}

/*
 * createGetVONames_Channel
 *
 * Prepare Statement For ChannelDAO::getVONames, Channel View
 *
 * Bind Parameters:
 *  :1 - Channel Name
 *  :2 - limit
 *  :3 - limit
 *  :4 - offset
 *  :5 - offset
 */
Statement* ChannelStatements::createGetVONames_Channel(OracleDAOContext& ctx) /*throw (DAOLogicError,DAOException)*/{
    
    // Get tag
    const char * const tag = GET_VO_NAMES_STATEMENT_CH_TAG;

    Statement * stmt = 0;
    try {
        // Check if the statement is already cached
        if(false == ctx.connection().isCached("",tag)){
            // Prepare Statement String
            const char * const query_stmt =
                "SELECT vo_name FROM ("
                    "SELECT vo_name, ROWNUM as rn FROM ("
                        "SELECT DISTINCT t_job.vo_name as vo_name"
                        " , MAX( SYS_EXTRACT_UTC(t_job.submit_time) ) as submit_time"
                        " FROM t_channel , t_channel_vo_share , t_job"
                        " WHERE t_channel.channel_name  = t_channel_vo_share.channel_name"
                        " AND t_channel.channel_name = t_job.channel_name"
                        " AND t_job.vo_name = t_channel_vo_share.vo_name"
                        " AND t_job.job_state IN ('Active','Ready')"
                        " AND t_channel.channel_name = :1"
                        " GROUP BY t_job.vo_name"
                        " ORDER BY submit_time DESC , vo_name )"
                    " WHERE :2=0 OR ROWNUM<=:3 )"
               " WHERE :4=0 OR rn>:5";
            
            m_log_trace("Preparing statement: " << query_stmt);
            // Create Statement
            stmt = ctx.connection().createStatement(query_stmt,tag);
        } else {
            // Create Statement
            stmt  = ctx.connection().createStatement("",tag);
        }
    } catch(const SQLException& exc){
        m_log_error("Got Exception in creating GetVONames Statement: [0x" << exc.getErrorCode() << "] " << exc.getMessage());
        throw DAOException("Failed to prepare Statement");
    }
    // Check Postcondition
    if(0 == stmt){
        throw DAOLogicError("Failed to prepare Statement - Nullpointer");
    }
    return stmt;
}

/*
 * createGetShare_Channel
 *
 * Return the statement for the ChannelDAO::getShare, Channel View
 *
 * Bind Parameters:
 *  :1 - Channel Name
 *  :2 - VO Name
 */
Statement* ChannelStatements::createGetShare_Channel(OracleDAOContext& ctx) /*throw (DAOLogicError,DAOException)*/{
    
    // Get tag
    const char * const tag = GET_SHARE_STATEMENT_CH_TAG;

    Statement * stmt = 0;
    try {
        // Check if the statement is already cached
        if(false == ctx.connection().isCached("",tag)){
            // Prepare Statement String
            // Thanks to Balys Sulmanas for refactoring this query
            const char * const query_stmt =
                "SELECT channel_share,SHARE_NORM,SHARE_ACTIVE,SHARE_ACTIVE_NORM from ( "
                "SELECT "
                    // "channel_name, "
                    "vo_name, "
                    "channel_share, "
                    "DECODE(channel_share,0,0,channel_share/SUM(channel_share) OVER (PARTITION BY channel_name)) SHARE_NORM, "
                    "channel_share*cn SHARE_ACTIVE, "
                    "DECODE(channel_share*cn,0,0,channel_share*cn/SUM(channel_share*cn) OVER (PARTITION BY channel_name)) SHARE_ACTIVE_NORM "
                    "FROM ( "
                        "SELECT "
                        "S.channel_name, "
                        "S.vo_name, "
                        "DECODE(S.channel_vostate,'PAUSE',0,S.channel_share) channel_share, "
                        "(SELECT COUNT(*) FROM DUAL WHERE EXISTS ("
                            "SELECT NULL FROM t_job, t_file "
                            "WHERE t_job.job_id = t_file.job_id "
                                "AND t_file.file_state = 'Ready' "
                                "AND t_job.vo_name = S.vo_name "
                                "AND t_job.channel_name = S.channel_name)"
                        ") cn "
                        "FROM t_channel_vo_share S, t_channel C "
                        "WHERE S.channel_name = C.channel_name "
                    ")"
                    "WHERE channel_name = :1 ) WHERE vo_name = :2";
            m_log_trace("Preparing statement: " << query_stmt);
            // Create Statement
            stmt = ctx.connection().createStatement(query_stmt,tag);
        } else {
            // Create Statement
            stmt  = ctx.connection().createStatement("",tag);
        }
    } catch(const SQLException& exc){
        m_log_error("Got Exception in creating GetShare Statement: [0x" << exc.getErrorCode() << "] " << exc.getMessage());
        throw DAOException("Failed to prepare Statement");
    }
    // Check Postcondition
    if(0 == stmt){
        throw DAOLogicError("Failed to prepare Statement - Nullpointer");
    }
    return stmt;
}

/*
 * createGetLimit_Channel
 *
 * Return the statement for the ChannelDAO::getLimit, Channel View
 *
 * Bind Parameters:
 *  :1 - Channel Name
 *  :2 - VO Name
 */
Statement* ChannelStatements::createGetLimit_Channel(OracleDAOContext& ctx) /*throw (DAOLogicError,DAOException)*/{
    
    // Get tag
    const char * const tag = GET_LIMIT_STATEMENT_CH_TAG;

    Statement * stmt = 0;
    try {
        // Check if the statement is already cached
        if(false == ctx.connection().isCached("",tag)){
            // Prepare Statement String
            const char * const query_stmt =
                "SELECT t_channel_vo_share.channel_limit"
                " ,t_channel_vo_share.channel_vostate"
                " FROM t_channel, t_channel_vo_share"
                " WHERE t_channel.channel_name = t_channel_vo_share.channel_name"
                " AND t_channel.channel_name = :1"
                " AND t_channel_vo_share.vo_name = :2";
            
            m_log_trace("Preparing statement: " << query_stmt);
            // Create Statement
            stmt = ctx.connection().createStatement(query_stmt,tag);
        } else {
            // Create Statement
            stmt  = ctx.connection().createStatement("",tag);
        }
    } catch(const SQLException& exc){
        m_log_error("Got Exception in creating GetLimit Statement: [0x" << exc.getErrorCode() << "] " << exc.getMessage());
        throw DAOException("Failed to prepare Statement");
    }
    // Check Postcondition
    if(0 == stmt){
        throw DAOLogicError("Failed to prepare Statement - Nullpointer");
    }
    return stmt;
}


/*
 * createIsBlacklisted_VO
 *
 * Return the statement for the ChannelDAO::isBlacklisted, VO View
 *
 * Bind Parameters:
 *  :1 - Host name
 *  :2 - Site Name
 */
Statement* ChannelStatements::createIsBlacklisted_VO(agents::dao::oracle::OracleDAOContext& ctx) /*throw (DAOLogicError,DAOException)*/{
    
    // Get tag
    const char * const tag = IS_BLACKLISTED_STATEMENT_VO_TAG;

    Statement * stmt = 0;
    try {
        // Check if the statement is already cached
        if(false == ctx.connection().isCached("",tag)){
            // Prepare Statement String
            const char * const query_stmt =
                "SELECT t_bad_ses.se_hostname, t_bad_ses.message"
                " FROM t_bad_ses"
                " WHERE t_bad_ses.se_hostname IN (:1,:2)";
            
            m_log_trace("Preparing statement: " << query_stmt);
            // Create Statement
            stmt = ctx.connection().createStatement(query_stmt,tag);
        } else {
            // Create Statement
            stmt  = ctx.connection().createStatement("",tag);
        }
    } catch(const SQLException& exc){
        m_log_error("Got Exception in creating IsBacklisted Statement: [0x" << exc.getErrorCode() << "] " << exc.getMessage());
        throw DAOException("Failed to prepare Statement");
    }
    // Check Postcondition
    if(0 == stmt){
        throw DAOLogicError("Failed to prepare Statement - Nullpointer");
    }
    return stmt;
}

/*
 * Return the statement for the ChannelDAO::getSiteGroups
 *
 * Bind Parameters:
 *  :1 - Site Name
 */
Statement* ChannelStatements::createGetSiteGroups(agents::dao::oracle::OracleDAOContext& ctx) /*throw (DAOLogicError,DAOException)*/{
    
    // Get tag
    const char * const tag = GET_SITE_GROUPS_STATEMENT_TAG;

    Statement * stmt = 0;
    try {
        // Check if the statement is already cached
        if(false == ctx.connection().isCached("",tag)){
            // Prepare Statement String
            const char * const query_stmt =
                "SELECT t_site_group.group_name"
                " FROM t_site_group"
                " WHERE t_site_group.site_name = :1"
                " ORDER BY t_site_group.priority DESC, t_site_group.group_name";
            
            m_log_trace("Preparing statement: " << query_stmt);
            // Create Statement
            stmt = ctx.connection().createStatement(query_stmt,tag);
        } else {
            // Create Statement
            stmt  = ctx.connection().createStatement("",tag);
        }
    } catch(const SQLException& exc){
        m_log_error("Got Exception in creating GetSiteGroups Statement: [0x" << exc.getErrorCode() << "] " << exc.getMessage());
        throw DAOException("Failed to prepare Statement");
    }
    // Check Postcondition
    if(0 == stmt){
        throw DAOLogicError("Failed to prepare GetSiteGroups - Nullpointer");
    }
    return stmt;
}
