/*
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *  GLite Data Transfer Agents: ChannelStatements Class
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *  Version info: $Id: ChannelStatements.h,v 1.12 2008/12/04 08:53:30 ptedesco Exp $
 *  Release: $Name:  $
 *
 */
#ifndef GLITE_DATA_TRANSFER_AGENTS_DAO_ORACLE_CHANNELSTATEMENTS_H_
#define GLITE_DATA_TRANSFER_AGENTS_DAO_ORACLE_CHANNELSTATEMENTS_H_

#include "glite/data/transfer/agent/model/Channel.h"
#include "glite/data/agents/dao/DAOException.h"
#include "glite/data/agents/dao/oracle/OracleDAOContext.h"

#include <string>
#include <vector>
#include <boost/utility.hpp>

#include <glite/data/config/service/Logger.hxx>

namespace glite   {
namespace data    {
namespace transfer{
namespace agent   {
namespace dao     {
namespace oracle  {
 
/**
 * Oracle Data Access Object For the Channel class. 
 */
class ChannelStatements : boost::noncopyable{
public:

    /**
     * Return Singleton instance 
     * @return Singleton instance 
     */
    static ChannelStatements& instance() /*throw ()*/ ;

    // -------------------------------------------------------------------------
    // Methods
    // -------------------------------------------------------------------------

    /*
     * dispose
     *
     * dispose Statements for OracleDAOContext
     */
    void dispose(agents::dao::oracle::OracleDAOContext& ctx) /*throw ()*/;
      
    /**
     * Return the statement for the ChannelDAO::get, VO View
     *
     * Bind Parameters:
     *  :1 - Channel Name
     *  :2 - VO Name
     */
    ::oracle::occi::Statement* createGet_VO(agents::dao::oracle::OracleDAOContext& ctx) /*throw (DAOLogicError,DAOException)*/;

    /**
     * Return the statement for the ChannelDAO::get, Channel View
     *
     * Bind Parameters:
     *  :1 - Instance Channel Name
     *  :2 - Channel Name
     */
    ::oracle::occi::Statement* createGet_Channel(agents::dao::oracle::OracleDAOContext& ctx) /*throw (DAOLogicError,DAOException)*/;

    /**
     * Return the statement for the ChannelDAO::updateLastActive, Channel View
     *
     * Bind Parameters:
     *  :1 - Last Active
     *  :2 - Instance Channel Name
     */
    ::oracle::occi::Statement* createUpdateLastActive_Channel(agents::dao::oracle::OracleDAOContext& ctx) /*throw (DAOLogicError,DAOException)*/;

    /**
     * Return the statement for the ChannelDAO::updateTransferType, Channel View
     *
     * Bind Parameters:
     *  :1 - Last Active
     *  :2 - Instance Channel Name
     */
    ::oracle::occi::Statement* createUpdateTransferType_Channel(agents::dao::oracle::OracleDAOContext& ctx) /*throw (DAOLogicError,DAOException)*/;
    
    /**
     * Return the statement for the ChannelDAO::getBySites, VO View
     *
     * Bind Parameters:
     *  :1 - Source Site
     *  :2 - Dest Site
     *  :3 - VO Name
     */
    ::oracle::occi::Statement* createGetBySites_VO(agents::dao::oracle::OracleDAOContext& ctx) /*throw (DAOLogicError,DAOException)*/;
    
    /**
     * Return the statement for the ChannelDAO::hasShare, VO View
     *
     * Bind Parameters:
     *  :1 - Channel Name
     *  :2 - VO Name
     */
    ::oracle::occi::Statement* createHasShare_VO(agents::dao::oracle::OracleDAOContext& ctx) /*throw (DAOLogicError,DAOException)*/;
    
    /**
     * Return the statement for the ChannelDAO::getVONames, Channel View
     *
     * Bind Parameters:
     *  :1 - Channel Name
     *  :2 - limit
     *  :3 - limit
     *  :4 - offset
     *  :5 - offset
     */
    ::oracle::occi::Statement* createGetVONames_Channel(agents::dao::oracle::OracleDAOContext& ctx) /*throw (DAOLogicError,DAOException)*/;

    /**
     * Return the statement for the ChannelDAO::getShare, Channel View
     *
     * Bind Parameters:
     *  :1 - Channel Name
     *  :2 - VO Name
     */
    ::oracle::occi::Statement* createGetShare_Channel(agents::dao::oracle::OracleDAOContext& ctx) /*throw (DAOLogicError,DAOException)*/;

    /**
     * Return the statement for the ChannelDAO::getLimit, Channel View
     *
     * Bind Parameters:
     *  :1 - Channel Name
     *  :2 - VO Name
     */
    ::oracle::occi::Statement* createGetLimit_Channel(agents::dao::oracle::OracleDAOContext& ctx) /*throw (DAOLogicError,DAOException)*/;

    /**
     * Return the statement for the ChannelDAO::isBlacklisted, VO View
     *
     * Bind Parameters:
     *  :1 - Host name
     *  :2 - Site Name
     */
    ::oracle::occi::Statement* createIsBlacklisted_VO(agents::dao::oracle::OracleDAOContext& ctx) /*throw (DAOLogicError,DAOException)*/;
    
    /**
     * Return the statement for the ChannelDAO::getSiteGroups
     *
     * Bind Parameters:
     *  :1 - Site Name
     */
    ::oracle::occi::Statement* createGetSiteGroups(agents::dao::oracle::OracleDAOContext& ctx) /*throw (DAOLogicError,DAOException)*/;
        
private:
    glite::config::Logger m_logger;
    /**
     * Default Constructor
     */
    ChannelStatements();

    /**
     * Destructor
     */
    ~ChannelStatements(){}

};

} // End namespace oracle
} // End namespace dao
} // End namespace agent
} // End namespace transfer
} // End namespace data
} // End namespace glite

#endif //GLITE_DATA_TRANSFER_AGENTS_DAO_ORACLE_CHANNELSTATEMENTS_H_
