/**
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *  GLite Data Transfer Agents: Channel_ChannelDAO Class
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>, Paolo Tedesco <paolo.tedesco@cern.ch>
 *  Version info: $Id: Channel_ChannelDAO.cpp,v 1.20 2008/12/04 08:53:30 ptedesco Exp $
 *  Release: $Name:  $
 *
 */
#include "Channel_ChannelDAO.h"
#include "OracleChannelDAO.h"
#include "ChannelStatements.h"
#include "dao-utility.h"
#include <glite/data/agents/dao/oracle/StatementPtr.h>
#include <glite/data/agents/dao/oracle/ResultSetPtr.h>

using namespace glite::data::transfer::agent;
using namespace glite::data::transfer::agent::dao;
using namespace glite::data::transfer::agent::dao::oracle;
using glite::data::agents::dao::DAOLogicError;
using glite::data::agents::dao::DAOException;
using glite::data::agents::dao::oracle::OracleDAOContext;
using glite::data::agents::dao::oracle::StatementPtr;
using glite::data::agents::dao::oracle::ResultSetPtr;
using namespace ::oracle::occi;

/*
 * Channel_ChannelDAO
 *
 * Constructor
 */
Channel_ChannelDAO::Channel_ChannelDAO(OracleDAOContext& ctx, const std::string& channel_name, const std::string& category):
        m_logger(category),
        m_channelDaoImpl(new OracleChannelDAO(ctx,category)),
        m_channelName(channel_name){
}

/*
 * get
 *
 * Return the Channel with the given ChannelId
 */
model::Channel * Channel_ChannelDAO::get(const std::string& name) /*throw (DAOLogicError,DAOException)*/{
    
    m_log_trace("Calling ChannelDAO::get");
    
    model::Channel * channel = 0;

    // Get Context
    OracleDAOContext& ctx = m_channelDaoImpl->ctx();
    
    // Get Statement
    StatementPtr stmt(ctx.connection(),ChannelStatements::instance().createGet_Channel(ctx));
    try{
        // Set Parameters
        stmt->setString(1,name);
        stmt->setString(2,m_channelName);
        
        // Get Data
        channel = m_channelDaoImpl->get(*stmt);
    } catch(const SQLException& exc){
        m_log_error("Got SQLException in Executing Get: [0x" << exc.getErrorCode() << "] " << exc.getMessage());
        m_log_debug("Query was: " << stmt->getSQL());
        throw DAOException("Failed to get Channel - SQL Exception");
    } catch(const DAOException& exc){
        m_log_error("Got DAOException in Executing Get: " << exc.what());
        m_log_debug("Query was: " << stmt->getSQL());
        throw;
    } 
    
    m_log_trace("ChannelDAO::get Called");
    
    return channel;
}

/**
 * updateLastActive
 * Update last_active property of a Channel object
 */
void Channel_ChannelDAO::updateLastActive(const model::Channel& obj) /*throw (DAOLogicError,DAOException)*/{

    m_log_trace("Calling ChannelDAO::updateLastActive");
    
    if (obj.name() != m_channelName) {
        m_log_error("Consistency error: channel name mismatch. ChannelDAO name='" << m_channelName << "' Channel name='" << obj.name() << "'");
        throw DAOLogicError("Consistency error: channel name mismatch.");
    }
    
    // Get Context
    OracleDAOContext& ctx = m_channelDaoImpl->ctx();
    
    // Get Statement
    StatementPtr stmt(ctx.connection(),ChannelStatements::instance().createUpdateLastActive_Channel(ctx));
    try {
        // Set Parameters
        stmt->setTimestamp(1 ,m_channelDaoImpl->toTimestamp(obj.lastActive()));
        stmt->setString(2 ,obj.name());
                         
        // Update Data
        m_channelDaoImpl->update(*stmt);
    } catch(const SQLException& exc){
        m_log_error("Got SQLException in Executing Update: [0x" << exc.getErrorCode() << "] " << exc.getMessage());
        m_log_debug("Query was: " << stmt->getSQL());
        throw DAOException("Failed to update Channel - SQL Exception");
    } catch(const DAOException& exc){
        m_log_error("Got DAOException in Executing Update: " << exc.what());
        m_log_debug("Query was: " << stmt->getSQL());
        throw;
    } 
    m_log_trace("Channel Last Active Time updated");
}

/**
 * updateTransferType
 * Update transfer_type property of a Channel object
 */
void Channel_ChannelDAO::updateTransferType(const model::Channel& obj) /*throw (DAOLogicError,DAOException)*/{

    m_log_trace("Calling ChannelDAO::updateTransferType");
    
    if (obj.name() != m_channelName){
        m_log_error("Consistency error: channel name mismatch. ChannelDAO name='" << m_channelName << "' Channel name='" << obj.name() << "'");
        throw DAOLogicError("Consistency error: channel name mismatch.");
    }
    
    // Get Context
    OracleDAOContext& ctx = m_channelDaoImpl->ctx();
    
    // Get Statement
    StatementPtr stmt(ctx.connection(),ChannelStatements::instance().createUpdateTransferType_Channel(ctx));
    try {
        // Set Parameters
        stmt->setString(1 ,obj.transferType());
        stmt->setString(2 ,obj.name());
                         
        // Update Data
        m_channelDaoImpl->update(*stmt);
    } catch(const SQLException& exc){
        m_log_error("Got SQLException in Executing Update: [0x" << exc.getErrorCode() << "] " << exc.getMessage());
        m_log_debug("Query was: " << stmt->getSQL());
        throw DAOException("Failed to update Channel - SQL Exception");
    } catch(const DAOException& exc){
        m_log_error("Got DAOException in Executing Update: " << exc.what());
        m_log_debug("Query was: " << stmt->getSQL());
        throw;
    } 
    m_log_trace("Channel Transfer Type updated");
}


/**
 * Return the list of VO that are allowed to submit request to the Channel
 */
void Channel_ChannelDAO::getVONames(agents::dao::StringArray& vo_names, unsigned int limit, unsigned int offset) /*throw (DAOLogicError,DAOException)*/{
    
    m_log_trace("Entering Channel_ChannelDAO::getVONames");

    // Get Context
    OracleDAOContext& ctx = m_channelDaoImpl->ctx();
    
    // Get Statement
    StatementPtr stmt(ctx.connection(),ChannelStatements::instance().createGetVONames_Channel(ctx));
    try{
        // Set Parameters
        stmt->setString(1,m_channelName);
        stmt->setInt(2,limit);
        stmt->setInt(3,limit);
        stmt->setInt(4,offset);
        stmt->setInt(5,offset);
        
        // Execute Query
        ResultSetPtr rs(*stmt,stmt->executeQuery());
        while(0 != rs->next()){
            // Get Share
            vo_names.push_back(rs->getString(1));
        }
    } catch(const SQLException& exc){
        m_log_error("Got SQLException in Setting Parameters: [0x" << exc.getErrorCode() << "] " << exc.getMessage());
        m_log_debug("Query was: " << stmt->getSQL());
        throw DAOException("Failed to get VO Names - SQL Exception");
    } catch(...){
        m_log_warn("Failed to get VO Names");
        throw;
    }

    m_log_trace("Channel_ChannelDAO::getVONames returned " << vo_names.size() << " names");
}

/**
 * getShare
 * Return the configured share for the given VO on the Channel
 */
double Channel_ChannelDAO::getShare(
    const std::string&  vo_name, 
    bool                normalized, 
    bool                only_vo_with_active_jobs) /*throw (DAOLogicError,DAOException)*/{

    double share_val   = 0;
    double share_norm  = 0;
    double active_val  = 0;
    double active_norm = 0;
    
    m_log_trace("Entering Channel_ChannelDAO::getShare");

    // Get Context
    OracleDAOContext& ctx = m_channelDaoImpl->ctx();
    
    // Get Statement
    StatementPtr stmt(ctx.connection(),ChannelStatements::instance().createGetShare_Channel(ctx));
    try{
        // Set Parameters
        stmt->setString(1,m_channelName);
        stmt->setString(2,vo_name);
        
        // Execute Query
        ResultSetPtr rs(*stmt,stmt->executeQuery());
        if(0 != rs->next()){
            // Get Share
            share_val = rs->getDouble(1);
            // Get Share Normalized
            share_norm = rs->getDouble(2);
            // Get Active Share
            active_val = rs->getDouble(3);
            // Get Active Share Normalized
            active_norm = rs->getDouble(4);
            m_log_trace("Got shares. VO=" << vo_name << " channel=" << m_channelName << \
                " share_val=" << share_val << " share_norm=" << share_norm << \
                " active_val=" << active_val << " active_norm=" << active_norm);
        } else {
            m_log_debug("No VO found for that channel");
            m_log_trace("Query was: " << stmt->getSQL().c_str());
            throw DAOException("Requested VO is not register for the Channel");
        }
    } catch(const SQLException& exc){
        m_log_error("Got SQLException in Setting Parameters: [0x" << exc.getErrorCode() << "] " << exc.getMessage());
        m_log_debug("Query was: " << stmt->getSQL());
        throw DAOException("Failed to get Channel VO Share - SQL Exception");
    } catch(...){
        m_log_warn("Failed to get Channel VO Share");
        throw;
    }

    m_log_trace("Channel_ChannelDAO::getShare returned " << share_val << " - " << share_norm << " - " << active_val << " - " << active_norm);
    m_log_trace("normalized " << (normalized ? "true" : "false") << " - only active " << (only_vo_with_active_jobs ? "true" : "false"));

    double share = 0.0;
    if(true == normalized){
        if(true == only_vo_with_active_jobs){
            share = active_norm;
        } else {
            share = share_norm;
        }
    } else {
        if(true == only_vo_with_active_jobs){
            share = ((double)active_val) / 100;
        } else {
            share = ((double)share_val) / 100;
        }
    }
    if(share > 1.0){
        share = 1.0;
    }
    return share;   
}

/*
 * getLimit
 *
 * Return the hard limit (as absolute value) for the given VO on 
 * the channel.
 */
std::pair<int,model::Channel::VOState> Channel_ChannelDAO::getLimit(const std::string& vo_name) /*throw (DAOLogicError,DAOException)*/{
    std::pair<int,model::Channel::VOState> result;
    result.first  = 0;
    result.second = model::Channel::VS_UNDEF;
    
    std::string vo_state;
    m_log_trace("Entering Channel_ChannelDAO::getLimit");

    // Get Context
    OracleDAOContext& ctx = m_channelDaoImpl->ctx();
    
    // Get Statement
    StatementPtr stmt(ctx.connection(),ChannelStatements::instance().createGetLimit_Channel(ctx));
    try{
        // Set Parameters
        stmt->setString(1,m_channelName);
        stmt->setString(2,vo_name);
        
        // Execute Query
        ResultSetPtr rs(*stmt,stmt->executeQuery());
        if(0 != rs->next()){
            // Get Limit
            result.first = rs->getInt(1);
            // Get VOState
            vo_state = rs->getString(2);
            result.second = translateChannelVOState(vo_state.c_str());
        } else {
            m_log_debug("No VO Limit found for that channel");
            m_log_trace("Query was: " << stmt->getSQL());
            throw DAOException("Requested VO is not register for the Channel");
        }
    } catch(const SQLException& exc){
        m_log_error("Got SQLException in Setting Parameters: [0x" << exc.getErrorCode() << "] " << exc.getMessage());
        m_log_debug("Query was: " << stmt->getSQL());
        throw DAOException("Failed to get Channel VO Limit - SQL Exception");
    } catch(...){
        m_log_warn("Failed to get Channel VO Limit");
        throw;
    }

    m_log_trace("Channel_ChannelDAO::getLimit returned [" << result.first << "] - " << vo_state << " [" << result.second << "]");
    
    return result;
}

/*
 * getSiteGroups
 * 
 * Return the list of groups which a site belong to
 */
void Channel_ChannelDAO::getSiteGroups(const std::string& site, agents::dao::StringArray& groups) /*throw (DAOLogicError,DAOException)*/{
    return m_channelDaoImpl->getSiteGroups(site,groups);
}
