/*
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *  GLite Data Transfer Agents: Oracle Channel DAO Factory Class
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *           Paolo Tedesco <paolo.tedesco@cern.ch>
 *  Version info: $Id: Channel_DAOFactory.cpp,v 1.7 2008/11/13 10:43:55 ptedesco Exp $
 *  Release: $Name:  $
 *
 */
#include "Channel_DAOFactory.h"
#include "dao-utility.h"
#include "Channel_FileDAO.h"
#include "Channel_JobDAO.h"
#include "Channel_TransferDAO.h"
#include "Channel_ChannelDAO.h"
#include "Channel_AgentDAO.h"
#include "Channel_CredDAO.h"

using namespace glite::data::transfer::agent;
using namespace glite::data::transfer::agent::dao;
using namespace glite::data::transfer::agent::dao::oracle;
using glite::data::agents::dao::DAOLogicError;
using glite::data::agents::dao::DAOContext;
using glite::data::agents::dao::oracle::OracleDAOContext;

/*
 * Constants
 */
namespace {
const char * const ORACLE_CHANNEL_DAO_NAME = "transfer-agent-dao-oracle-channel";
}

/*
 * VO_DAOFactory
 *
 * Constructor
 */
Channel_DAOFactory::Channel_DAOFactory():channel::DAOFactory(),
    m_logger(ORACLE_CHANNEL_DAO_NAME){
}

/*
 * checkSchema
 *
 * Validate the DB schema
 */
void Channel_DAOFactory::checkSchema(DAOContext& ctx) /*throw (DAOLogicError,DAOException)*/{
    try{    
        check_schema(ctx);
    } catch(const agents::dao::DAOException& exc){
        m_log_error("Failed To Check Schema Version. Reason is : " << exc.what());
        throw;
    }
    try{    
        check_cred_schema(ctx);
    } catch(const agents::dao::DAOException& exc){
        m_log_error("Failed To Check Credential Schema Version. Reason is : " << exc.what());
        throw;
    }
}

/*
 * createFileDAO
 *
 * Returns DataAccessObject for model:File instances
 */
channel::FileDAO * Channel_DAOFactory::createFileDAO(const std::string& channel_name, DAOContext& ctx) /*throw (DAOLogicError)*/{
    try{
        OracleDAOContext& oracle_ctx = dynamic_cast<OracleDAOContext&>(ctx);
            
        // Create File DAO
        return new Channel_FileDAO(oracle_ctx,channel_name,ORACLE_CHANNEL_DAO_NAME);
    } catch(const std::exception& exc){
        m_log_error("Invalid DAOContext: " << exc.what());
        throw DAOLogicError("invalid DAOContext type");
    }
    throw DAOLogicError("invalid DAOFactory");
}

/*
 * createJobDAO
 *
 * Returns DataAccessObject for model:Job instances
 */
channel::JobDAO * Channel_DAOFactory::createJobDAO(const std::string& channel_name, DAOContext& ctx) /*throw (DAOLogicError)*/{
    try{
        OracleDAOContext& oracle_ctx = dynamic_cast<OracleDAOContext&>(ctx);
        
        // Create Job DAO
        return new Channel_JobDAO(oracle_ctx,channel_name,ORACLE_CHANNEL_DAO_NAME);
    } catch(const std::exception& exc){
        m_log_error("Invalid DAOContext: " << exc.what());
        throw DAOLogicError("invalid DAOContext type");
    }
    throw DAOLogicError("invalid DAOFactory");
}

/*
 * createTransferDAO
 *
 * Returns DataAccessObject for model:Transfer instances
 */
channel::TransferDAO * Channel_DAOFactory::createTransferDAO(const std::string& channel_name, DAOContext& ctx) /*throw (DAOLogicError)*/{
    try{
        OracleDAOContext& oracle_ctx = dynamic_cast<OracleDAOContext&>(ctx);
        
        // Create Transfer DAO
        return new Channel_TransferDAO(oracle_ctx,channel_name,ORACLE_CHANNEL_DAO_NAME);
    } catch(const std::exception& exc){
        m_log_error("Invalid DAOContext: " << exc.what());
        throw DAOLogicError("invalid DAOContext type");
    }
    throw DAOLogicError("invalid DAOFactory");
}

/*
 * createChannelDAO
 *
 * Returns DataAccessObject for model:Channel instances
 */
channel::ChannelDAO * Channel_DAOFactory::createChannelDAO(const std::string& channel_name, DAOContext& ctx) /*throw (DAOLogicError)*/{
    try{
        OracleDAOContext& oracle_ctx = dynamic_cast<OracleDAOContext&>(ctx);
        
        // Create Channel DAO
        return new Channel_ChannelDAO(oracle_ctx,channel_name,ORACLE_CHANNEL_DAO_NAME);
    } catch(const std::exception& exc){
        m_log_error("Invalid DAOContext: " << exc.what());
        throw DAOLogicError("invalid DAOContext type");
    }
    throw DAOLogicError("invalid DAOFactory");
}


/*
 * createAgentDAO
 *
 * Returns DataAccessObject for model:Agent instances
 */
channel::AgentDAO * Channel_DAOFactory::createAgentDAO(const std::string& channel_name, DAOContext& ctx) /*throw (DAOLogicError)*/{
    try{
        OracleDAOContext& oracle_ctx = dynamic_cast<OracleDAOContext&>(ctx);
        
        // Create Agent DAO
        return new Channel_AgentDAO(oracle_ctx,channel_name,ORACLE_CHANNEL_DAO_NAME);
    } catch(const std::exception& exc){
        m_log_error("Invalid DAOContext: " << exc.what());
        throw DAOLogicError("invalid DAOContext type");
    }
    throw DAOLogicError("invalid DAOFactory");
}

/*
 * createCredDAO
 *
 * Returns DataAccessObject for model:Cred instances
 */
channel::CredDAO * Channel_DAOFactory::createCredDAO(const std::string& channel_name, DAOContext& ctx) /*throw (DAOLogicError)*/{
    try{
        OracleDAOContext& oracle_ctx = dynamic_cast<OracleDAOContext&>(ctx);
        
        // Create Cred DAO
        return new Channel_CredDAO(oracle_ctx,channel_name,ORACLE_CHANNEL_DAO_NAME);
    } catch(const std::exception& exc){
        m_log_error("Invalid DAOContext: " << exc.what());
        throw DAOLogicError("invalid DAOContext type");
    }
    throw DAOLogicError("invalid DAOFactory");
}
