/*
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *  GLite Data Transfer Agents: Channel_FileDAO Class
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *           Paolo Tedesco <paolo.tedesco@cern.ch>
 *  Version info: $Id: Channel_FileDAO.cpp,v 1.19 2008/11/13 10:43:56 ptedesco Exp $
 *  Release: $Name:  $
 *
 */
#include "Channel_FileDAO.h"
#include "OracleFileDAO.h"
#include "FileStatements.h"
#include <glite/data/agents/dao/oracle/StatementPtr.h>

#include "dao-utility.h"

using namespace glite::data::transfer::agent;
using namespace glite::data::transfer::agent::dao;
using namespace glite::data::transfer::agent::dao::oracle;
using glite::data::agents::dao::DAOLogicError;
using glite::data::agents::dao::DAOException;
using glite::data::agents::dao::oracle::OracleDAOContext;
using glite::data::agents::dao::oracle::StatementPtr;
using namespace ::oracle::occi;

/*
 * Channel_FileDAO
 *
 * Constructor
 */
Channel_FileDAO::Channel_FileDAO(OracleDAOContext& ctx, const std::string& channel_name,const std::string& category):
        m_logger(category),
        m_fileDaoImpl(new OracleFileDAO(ctx,category)),
        m_channelName(channel_name){
}

/*
 * get
 *
 * Return the File  with the given FileId
 */
model::File * Channel_FileDAO::get(const std::string& id) /*throw (DAOLogicError,DAOException)*/{
    
    m_log_trace("Calling FileDAO::get");
    
    model::File * file = 0;

    // Get Context
    OracleDAOContext& ctx = m_fileDaoImpl->ctx();
    
    // Get Statement
    StatementPtr stmt(ctx.connection(),FileStatements::instance().createGet_Channel(ctx));
    try{
        // Set Parameters
        stmt->setString(1,id);
        stmt->setString(2,m_channelName);
        
        // Get Data
        file = m_fileDaoImpl->get(*stmt);
    } catch(const SQLException& exc){
        m_log_error("Got SQLException in Executing Get: [0x" << exc.getErrorCode() << "] " << exc.getMessage());
        m_log_debug("Query was: " << stmt->getSQL());
        throw DAOException("Failed to Get File - SQL Exception");
    } catch(const DAOException& exc){
        m_log_error("Got DAOException in Executing Get for id " << id << ": " << exc.what());
        m_log_debug("Query was: " << stmt->getSQL());
        throw;
    } 
    
    m_log_trace("FileDAO::get Called");
    
    return file;
}

/**
 * Return the File with the given Id
 */
model::File * Channel_FileDAO::getFullSearch(const std::string& id) /*throw (DAOLogicError, DAOException)*/{
    m_log_trace("Calling FileDAO::getFullSearch");
    
    model::File * file = 0;

    // Get Context
    OracleDAOContext& ctx = m_fileDaoImpl->ctx();
    
    // Get Statement
    StatementPtr stmt(ctx.connection(),FileStatements::instance().createGet_Channel(ctx,true));
    try{
        // Set Parameters
        stmt->setString(1,id);
        stmt->setString(2,m_channelName);
        
        // Get Data
        file = m_fileDaoImpl->get(*stmt);
    } catch(const SQLException& exc){
        m_log_error("Got SQLException in Executing Get: [0x" << exc.getErrorCode() << "] " << exc.getMessage());
        m_log_debug("Query was: " << stmt->getSQL());
        throw DAOException("Failed to Get File - SQL Exception");
    } catch(const DAOException& exc){
        m_log_error("Got DAOException in Executing Get for id " << id << ": " << exc.what());
        m_log_debug("Query was: " << stmt->getSQL());
        throw;
    } 
    
    m_log_trace("FileDAO::get Called");
    
    return file;
}


/*
 * update
 *
 * Update a File instance
 */
void Channel_FileDAO::update(const model::File& file) /*throw (DAOLogicError,DAOException)*/{
    
    m_log_trace("Calling FileDAO::update");
    m_log_trace("File State is " << translateFileState(file.state()));
    
    // Get Context
    OracleDAOContext& ctx = m_fileDaoImpl->ctx();
    
    // Get Statement
    StatementPtr stmt(ctx.connection(),FileStatements::instance().createUpdate_Channel(ctx,file,m_channelName));
    try {
        // No need to set Parameters (already set)
        
        // Update Data
        if(0 != stmt.get()){
            m_fileDaoImpl->update(*stmt);
        }
    } catch(const SQLException& exc){
        m_log_error("Got SQLException in Executing Update: [0x" << exc.getErrorCode() << "] " << exc.getMessage());
        m_log_debug("Query was: " << stmt->getSQL());
        throw DAOException("Failed To Update File - SQL Exception");
    } catch(const DAOException& exc){
        m_log_error("Got DAOException in Executing Update: " << exc.what());
        m_log_debug("Query was: " << stmt->getSQL());
        throw;
    } 
    m_log_trace("File Updated");
}

/*
 * getByJobId
 *
 * Return the list of File instances for the given job Id
 */
void Channel_FileDAO::getByJobId(const std::string& job_id, agents::dao::StringArray& files) /*throw (DAOLogicError,DAOException)*/{
    
    m_log_trace("Calling FileDAO::getByJobId");

    // Get Context
    OracleDAOContext& ctx = m_fileDaoImpl->ctx();
    
    // Get Statement
    StatementPtr stmt(ctx.connection(),FileStatements::instance().createGetByJobId_Channel(ctx));
    try{
        // Set Parameters
        stmt->setString(1,job_id);
        stmt->setString(2,m_channelName);
        
        // Get Data
        m_fileDaoImpl->getFileIds(*stmt,files);
        if(files.empty()){
            m_log_debug("No such JobId " << job_id);
            throw DAOException("Invalid Job Id");
        }
    } catch(const SQLException& exc){
        m_log_error("Got SQLException in Executing GetByJobId: [0x" << exc.getErrorCode() << "] " << exc.getMessage());
        m_log_debug("Query was: " << stmt->getSQL());
        throw DAOException("Failed to Get File - SQL Exception");
    } catch(const DAOException& exc){
        m_log_error("Got DAOException in Executing GetByJobId: " << exc.what());
        m_log_debug("Query was: " << stmt->getSQL());
        throw;
    } 
    
    m_log_trace("FileDAO::GetByJobId Called");
    
}

/*
 * getNextJobToTransfer
 *
 * Return the list of next files to transfer for the given channel following 
 * the rule: files in ready state, which job state is ready or active
 * order by job_priority, job submission time, job_id for the given VO. 
 * Note: Probably this shoudn't be a DAO method, but the implementation is 
 * more performat if coupled directly to the Data Sources
 */
void Channel_FileDAO::getNextJobToTransfer(
    const std::string&          vo_name, 
    std::string&                job_id, 
    agents::dao::StringArray&   file_ids, 
    unsigned int                limit) /*throw (DAOException,DAOLogicError)*/{
    
    m_log_trace("Calling FileDAO::getNextJobToTransfer");

    // Get Context
    OracleDAOContext& ctx = m_fileDaoImpl->ctx();
    
    // Get Statement
    StatementPtr stmt(ctx.connection(),FileStatements::instance().createGetNextJobToTransfer_Channel(ctx));
    try{
        // Set Parameters
        stmt->setString(1,m_channelName);
        stmt->setString(2,vo_name);
        stmt->setInt(3,limit);
        stmt->setInt(4,limit);
        
        // Get Data
        m_fileDaoImpl->getFilesAndJobIds(*stmt,job_id,file_ids);
    } catch(const SQLException& exc){
        m_log_error("Got SQLException in Executing GetNextToTransfer: [0x" << exc.getErrorCode() << "] " << exc.getMessage());
        m_log_debug("Query was: " << stmt->getSQL());
        throw DAOException("Failed to Get File - SQL Exception");
    } catch(const DAOException& exc){
        m_log_error("Got DAOException in Executing GetNextToTransfer: " << exc.what());
        m_log_debug("Query was: " << stmt->getSQL());
        throw;
    } 
    
    m_log_trace("FileDAO::getNextJobToTransfer Called");
}

/*
 * getJobsToTransferPerSE
 *
 * Return the next jobs' and files' ids, groups by SEs' pair, to transfer 
 * for the given 
 * channel following the rule: files in pending or waiting state, which job 
 * state is pending, order by job_priority, job submission time, job_id for 
 * the given VO
 */
void Channel_FileDAO::getJobsToTransferPerSE(
    const std::string& vo_name, 
    dao::StringArrayTable& ids, 
    unsigned int se_limit, 
    unsigned int limit) /*throw (DAOLogicError, DAOException)*/{
    
    m_log_trace("Calling FileDAO::getJobsToTransferPerSE");

    // Get Context
    OracleDAOContext& ctx = m_fileDaoImpl->ctx();
    
    // Get Statement
    StatementPtr stmt(ctx.connection(),FileStatements::instance().createGetJobsToTransferPerSE_Channel(ctx));
    try{
        // Set Parameters
        stmt->setString(1,m_channelName);
        stmt->setString(2,vo_name);
        stmt->setInt(3,limit);
        stmt->setInt(4,limit);
        stmt->setInt(5,se_limit);
        stmt->setInt(6,se_limit);

        // Get Data
        m_fileDaoImpl->getFilesAndJobIds(*stmt,ids);
    } catch(const SQLException& exc){
        m_log_error("Got SQLException in Executing GetJobsToTransferPerSE: [0x" << exc.getErrorCode() << "] " << exc.getMessage());
        m_log_debug("Query was: " << stmt->getSQL());
        throw DAOException("Failed to Get File - SQL Exception");
    } catch(const DAOException& exc){
        m_log_error("Got DAOException in Executing GetJobsToTransferPerSE: " << exc.what());
        m_log_debug("Query was: " << stmt->getSQL());
        throw;
    } 
    
    m_log_trace("FileDAO::getJobsToTransferPerSE Called");
}

/*
 * getJobsToCancel
 *
 * Return the next job (and file Ids) files to cancel for the given 
 * channel following the rule: files in ready state, which job 
 * state is ready or active, with canceljob flag set to true, order by 
 * job_priority, job submission time, job id 
 */
void Channel_FileDAO::getFilesToCancel(StringArrayTable& ids, unsigned int limit) /*throw (DAOException,DAOLogicError)*/{
        
    m_log_trace("Calling FileDAO::getFilesToCancel");

    // Get Context
    OracleDAOContext& ctx = m_fileDaoImpl->ctx();
    
    // Get Statement
    StatementPtr stmt(ctx.connection(),FileStatements::instance().createGetFilesToCancel_Channel(ctx));
    try{
        // Set Parameters
        stmt->setString(1,m_channelName);
        stmt->setInt(2,limit);
        stmt->setInt(3,limit);
        
        // Get Data
        m_fileDaoImpl->getFilesAndJobIds(*stmt,ids);
    } catch(const SQLException& exc){
        m_log_error("Got SQLException in Executing GetFilesToCancel: [0x" << exc.getErrorCode() << "] " << exc.getMessage());
        m_log_debug("Query was: " << stmt->getSQL());
        throw DAOException("Failed to Get File - SQL Exception");
    } catch(const DAOException& exc){
        m_log_error("Got DAOException in Executing GetFilesToCancel: " << exc.what());
        m_log_debug("Query was: " << stmt->getSQL());
        throw;
    } 
    
    m_log_trace("FileDAO::getFilesToCancel Called");
}
