/*
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *  GLite Data Transfer Agents: Channel_JobDAO Class
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>, Paolo Tedesco <paolo.tedesco@cern.ch>
 *  Version info: $Id: Channel_JobDAO.cpp,v 1.16 2008/11/13 10:43:56 ptedesco Exp $
 *  Release: $Name:  $
 *
 */
#include "Channel_JobDAO.h"
#include "OracleJobDAO.h"
#include "JobStatements.h"
#include <glite/data/agents/dao/oracle/StatementPtr.h>

using namespace glite::data::transfer::agent;
using namespace glite::data::transfer::agent::dao;
using namespace glite::data::transfer::agent::dao::oracle;
using glite::data::agents::dao::DAOLogicError;
using glite::data::agents::dao::DAOException;
using glite::data::agents::dao::oracle::OracleDAOContext;
using glite::data::agents::dao::oracle::StatementPtr;

using namespace ::oracle::occi;

/*
 * Channel_JobDAO
 *
 * Constructor
 */
Channel_JobDAO::Channel_JobDAO(OracleDAOContext& ctx, const std::string& channel_name, const std::string& category):
        m_logger(category),
        m_jobDaoImpl(new OracleJobDAO(ctx,category)),
        m_channelName(channel_name){
}

/*
 * get
 *
 * Return the Job with the given JobId
 */
model::Job * Channel_JobDAO::get(const std::string& id) /*throw (DAOLogicError,DAOException)*/{

    model::Job * job = 0;
    
    m_log_trace("Calling JobDAO::get");
    
    // Get Context
    OracleDAOContext& ctx = m_jobDaoImpl->ctx();
    
    // Get Statement
    StatementPtr stmt(ctx.connection(),JobStatements::instance().createGet_Channel(ctx));
    try{
        // Set Parameters
        stmt->setString(1,id);
        stmt->setString(2,m_channelName);
        
        // Get Data
        job = m_jobDaoImpl->get(*stmt);
    } catch(const SQLException& exc){
        m_log_error("Got SQLException in Executing Get: [0x" << exc.getErrorCode() << "] " << exc.getMessage());
        m_log_debug("Query was: " << stmt->getSQL());
        throw DAOException("Failed to get Job - SQL Exception");
    } catch(const DAOException& exc){
        m_log_error("Got DAOException in Executing Get: " << exc.what());
        m_log_debug("Query was: " << stmt->getSQL());
        throw;
    } 
    
    m_log_trace("JobDAO::get Called");
    
    return job;
}

/**
 * Return the Job with the given id searching all jobs
 */
model::Job * Channel_JobDAO::getFullSearch(const std::string& id) /*throw (DAOLogicError,DAOException)*/{
    model::Job * job = 0;
    
    m_log_trace("Calling JobDAO::getFullSearch");
    
    // Get Context
    OracleDAOContext& ctx = m_jobDaoImpl->ctx();
    
    // Get Statement
    StatementPtr stmt(ctx.connection(),JobStatements::instance().createGet_Channel(ctx,true));
    try{
        // Set Parameters
        stmt->setString(1,id);
        stmt->setString(2,m_channelName);
        
        // Get Data
        job = m_jobDaoImpl->get(*stmt);
    } catch(const SQLException& exc){
        m_log_error("Got SQLException in Executing Get: [0x" << exc.getErrorCode() << "] " << exc.getMessage());
        m_log_debug("Query was: " << stmt->getSQL());
        throw DAOException("Failed to get Job - SQL Exception");
    } catch(const DAOException& exc){
        m_log_error("Got DAOException in Executing Get: " << exc.what());
        m_log_debug("Query was: " << stmt->getSQL());
        throw;
    } 
    
    m_log_trace("JobDAO::get Called");
    
    return job;
}

/*
 * update
 *
 * Update a Job object
 */
void Channel_JobDAO::update(const model::Job& job) /*throw (DAOLogicError,DAOException)*/{

    m_log_trace("Calling JobDAO::update");
    
    // Get Context
    OracleDAOContext& ctx = m_jobDaoImpl->ctx();
    
    // Get Statement
    StatementPtr stmt(ctx.connection(),JobStatements::instance().createUpdate_Channel(ctx,job,m_channelName));
    try {
        // No need to set Parameters (already set)
        
        // Update Data
        if(0 != stmt.get()){
            m_jobDaoImpl->update(*stmt);
        }
    } catch(const SQLException& exc){
        m_log_error("Got SQLException in Executing Update: [0x" << exc.getErrorCode() << "] " << exc.getMessage());
        m_log_debug("Query was: " << stmt->getSQL());
        throw DAOException("Failed to update Job - SQL Exception");
    } catch(const DAOException& exc){
        m_log_error("Got DAOException in Executing Update: " << exc.what());
        m_log_debug("Query was: " << stmt->getSQL());
        throw;
    } 
    m_log_trace("Job updated");
}

/*
 * updateStates
 *
 * Update the state of all the transfer jobs in the given list
 */
void Channel_JobDAO::updateStates(const agents::dao::StringArray& ids) /*throw (DAOLogicError,DAOException)*/{

    // Check Precondition
    if(ids.empty()){
        throw DAOLogicError("Empty Job Id list");
    }
    
    m_log_trace("Entering Channel_JobDAO::updateStates");
    // Get Context
    OracleDAOContext& ctx = m_jobDaoImpl->ctx();
    
    // Get Statement
    StatementPtr stmt(ctx.connection(),JobStatements::instance().createGetFileStates_Channel(ctx));
    try{
        // Set Parameters
        stmt->setString(2,m_channelName);
        
        // Deletage to the Helper    
        m_jobDaoImpl->updateStates(ids,*stmt,*this);
    } catch(const SQLException& exc){
        m_log_error("Got SQLException in Setting Parameters: [0x" << exc.getErrorCode() << "] " << exc.getMessage());
        m_log_debug("Query was: " << stmt->getSQL());
        throw DAOException("Failed to update Jobs' State - SQL Exception");
    } catch(...){
        m_log_warn("Failed to update Jobs' State");
        throw;
    }

    m_log_debug("Jobs' State Updated");
}

/*
 * getJobsToFix
 *
 * Return the list of job Ids that need to be updated since the related files
 * were modified by the VO Agent: Job state should be active and the related 
 * file states can only be Done, Canceled, Hold or Failed 
 */
void Channel_JobDAO::getJobsToFix(agents::dao::StringArray& job_ids, unsigned int limit) /*throw (DAOException,DAOLogicError)*/{
    
    m_log_trace("Entering Channel_JobDAO::getJobsToFix");
    // Get Context
    OracleDAOContext& ctx = m_jobDaoImpl->ctx();
    
    // Get Statement
    StatementPtr stmt(ctx.connection(),JobStatements::instance().createGetJobsToFix_Channel(ctx));
    try{
        // Set Parameters
        stmt->setString(1,m_channelName);
        stmt->setInt(2,limit);
        stmt->setInt(3,limit);
        
        // Get Data
        m_jobDaoImpl->getJobIds(*stmt,job_ids);
    } catch(const SQLException& exc){
        m_log_error("Got SQLException in getJobsToFix : [0x" << exc.getErrorCode() << "] " << exc.getMessage());
        m_log_debug("Query was: " << stmt->getSQL());
        throw DAOException("Failed to Get Jobs to fix- SQL Exception");
    } catch(...){
        m_log_warn("Failed to update Jobs' State");
        throw;
    }
    
    m_log_trace("JobDAO::getJobsToFix Called");
}
