/**
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *  GLite Data Transfer Agents: Channel_TransferDAO Class
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *           Paolo Tedesco <paolo.tedesco@cern.ch>
 *  Version info: $Id: Channel_TransferDAO.cpp,v 1.17 2008/11/13 10:43:56 ptedesco Exp $
 *  Release: $Name:  $
 *
 */
#include "Channel_TransferDAO.h"
#include "OracleTransferDAO.h"
#include "TransferStatements.h"
#include "FileStatements.h"
#include <glite/data/agents/dao/oracle/StatementPtr.h>
#include <glite/data/agents/dao/oracle/ResultSetPtr.h>

using namespace glite::data::transfer::agent;
using namespace glite::data::transfer::agent::dao;
using namespace glite::data::transfer::agent::dao::oracle;
using glite::data::agents::dao::DAOLogicError;
using glite::data::agents::dao::DAOException;
using glite::data::agents::dao::oracle::OracleDAOContext;
using glite::data::agents::dao::oracle::StatementPtr;
using glite::data::agents::dao::oracle::ResultSetPtr;
using glite::data::agents::dao::StringArray;
using namespace ::oracle::occi;

/*
 * Channel_TransferDAO
 *
 * Constructor
 */
Channel_TransferDAO::Channel_TransferDAO(OracleDAOContext& ctx, const std::string& channel_name, const std::string& category):
        m_logger(category),
        m_transferDaoImpl(new OracleTransferDAO(ctx,category)),
        m_channelName(channel_name){
}

/*
 * get
 *
 * Return the Transfer with the given TransferId
 */
model::Transfer * Channel_TransferDAO::get(const std::string& file_id, const std::string& req_id) /*throw (DAOLogicError,DAOException)*/{
  
    m_log_trace("Calling TransferDAO::get");
    
    model::Transfer * transfer = 0;

    // Get Context
    OracleDAOContext& ctx = m_transferDaoImpl->ctx();
    
    // Get Statement
    StatementPtr stmt(ctx.connection(),TransferStatements::instance().createGet_Channel(ctx));
    try{
        // Set Parameters
        stmt->setString(1,file_id);
        stmt->setString(2,req_id);
        stmt->setString(3,m_channelName);
        
        // Get Data
        transfer = m_transferDaoImpl->get(*stmt);
    } catch(const SQLException& exc){
        m_log_error("Got SQLException in Executing Get: [0x" << exc.getErrorCode() << "] " << exc.getMessage());
        m_log_debug("Query was: " << stmt->getSQL());
        throw DAOException("Failed to get Transfer - SQL Exception");
    } catch(const DAOException& exc){
        m_log_error("Channel " << m_channelName << " - DAOException executing Get for <" << file_id << ":" << req_id << ">: " << exc.what());
        m_log_debug("Query was: " << stmt->getSQL());
        throw;
    } 
    
    m_log_trace("TransferDAO::get Called");
    
    return transfer;
}

/*
 * get
 *
 * Return the Transfer with the given TransferId
 */
model::Transfer * Channel_TransferDAO::getFullSearch(const std::string& file_id, const std::string& req_id) /*throw (DAOLogicError,DAOException)*/{
  
    m_log_trace("Calling TransferDAO::getFullSearch");
    
    model::Transfer * transfer = 0;

    // Get Context
    OracleDAOContext& ctx = m_transferDaoImpl->ctx();
    
    // Get Statement
    StatementPtr stmt(ctx.connection(),TransferStatements::instance().createGet_Channel(ctx,true));
    try{
        // Set Parameters
        stmt->setString(1,file_id);
        stmt->setString(2,req_id);
        stmt->setString(3,m_channelName);
        
        // Get Data
        transfer = m_transferDaoImpl->get(*stmt);
    } catch(const SQLException& exc){
        m_log_error("Got SQLException in Executing Get: [0x" << exc.getErrorCode() << "] " << exc.getMessage());
        m_log_debug("Query was: " << stmt->getSQL());
        throw DAOException("Failed to get Transfer - SQL Exception");
    } catch(const DAOException& exc){
        m_log_error("Got DAOException in Executing Get for <" << file_id << ":" << req_id << ">: " << exc.what());
        m_log_debug("Query was: " << stmt->getSQL());
        throw;
    } 
    
    m_log_trace("TransferDAO::get Called");
    
    return transfer;
}


/*
 * update
 *
 * Update a Transfer object
 */
void Channel_TransferDAO::update(const model::Transfer& obj) /*throw (DAOLogicError,DAOException)*/{
    
    m_log_trace("Calling TransferDAO::update");
    
    // Get Context
    OracleDAOContext& ctx = m_transferDaoImpl->ctx();
    
    // Get Statement
    StatementPtr stmt(ctx.connection(),TransferStatements::instance().createUpdate_Channel(ctx,obj,m_channelName));
    try {
        // Update Data
        if(0 != stmt.get()){
            m_transferDaoImpl->update(*stmt);
        }
    } catch(const SQLException& exc){
        m_log_error("Got SQLException in Executing Update: [0x" << exc.getErrorCode() << "] " << exc.getMessage());
        m_log_debug("Query was: " << stmt->getSQL());
        throw DAOException("Failed to update Transfer - SQL Exception");
    } catch(const DAOException& exc){
        m_log_error("Got DAOException in Executing Update: " << exc.what());
        m_log_debug("Query was: " << stmt->getSQL());
        throw;
    } 
    m_log_trace("Transfer updated");
}

/*
 * getByFileId
 *
 * Return the list of Transfer instances for the given File Id
 */
void Channel_TransferDAO::getByFileId(const std::string& file_id, StringArray& ids, unsigned long limit) /*throw (DAOLogicError,DAOException)*/{
    
    m_log_trace("Entring Channel_TransferDAO::getByFileId");
    
    // Get Context
    OracleDAOContext& ctx = m_transferDaoImpl->ctx();
    
    // Get Statement
    StatementPtr stmt(ctx.connection(),TransferStatements::instance().createGetByFileId_Channel(ctx));
    try{
        // Set Parameters
        stmt->setString(1,file_id);
        stmt->setString(2,m_channelName);
        stmt->setInt(3,limit);
        stmt->setInt(4,limit);
        
        // Get Data
        m_transferDaoImpl->getIdList(*stmt,ids);
        if(true == ids.empty()){
            if(false == isValidFile(file_id)){
                m_log_debug("No such FileId " << file_id);
                throw DAOException("invalid File Id");
            }
        }
    } catch(const SQLException& exc){
        m_log_error("Got SQLException in Executing getByFileId: [0x" << exc.getErrorCode() << "] " << exc.getMessage());
        m_log_debug("Query was: " << stmt->getSQL());
        throw DAOException("Failed to get Transfer - SQL Exception");
    } catch(const DAOException& exc){
        m_log_error("Got DAOException in Executing getByFileId: " << exc.what());
        m_log_debug("Query was: " << stmt->getSQL());
        throw;
    } 
    
    m_log_trace("TransferDAO::getByFileId Called");
}

/*
 * getByRequestId
 *
 * Return the list of Transfer instances for the given Request Id
 */
void Channel_TransferDAO::getByRequestId(const std::string& request_id, StringArray& ids) /*throw (DAOLogicError,DAOException)*/{
    
    m_log_trace("Entring Channel_TransferDAO::getByRequestId");
    
    // Get Context
    OracleDAOContext& ctx = m_transferDaoImpl->ctx();
    
    // Get Statement
    StatementPtr stmt(ctx.connection(),TransferStatements::instance().createGetByRequestId_Channel(ctx));
    try{
        // Set Parameters
        stmt->setString(1,request_id);
        stmt->setString(2,m_channelName);
        
        // Get Data
        m_transferDaoImpl->getIdList(*stmt,ids);
        if(true == ids.empty()){
            m_log_debug("No Files for requestId " << request_id);
            throw DAOException("invalid Request Id");
        }
    } catch(const SQLException& exc){
        m_log_error("Got SQLException in Executing getByRequestId: [0x" << exc.getErrorCode() << "] " << exc.getMessage());
        m_log_debug("Query was: " << stmt->getSQL());
        throw DAOException("Failed to get Transfer - SQL Exception");
    } catch(const DAOException& exc){
        m_log_error("Got DAOException in Executing getByRequestId: " << exc.what());
        m_log_debug("Query was: " << stmt->getSQL());
        throw;
    } 
    
    m_log_trace("TransferDAO::getByRequestId Called");
}

/*
 * create
 *
 * Create a Transfer instance
 */
void Channel_TransferDAO::create(const model::Transfer& tx) /*throw (DAOLogicError,DAOException)*/{
        
    m_log_trace("Calling TransferDAO::create");
    
    // Get Context
    OracleDAOContext& ctx = m_transferDaoImpl->ctx();
    
    // Get Statement
    StatementPtr stmt(ctx.connection(),TransferStatements::instance().createCreate_Channel(ctx,tx));
    try {
        // Update Data
        m_transferDaoImpl->create(*stmt);
    } catch(const SQLException& exc){
        m_log_error("Got SQLException in Executing Insert: [0x" << exc.getErrorCode() << "] " << exc.getMessage());
        m_log_debug("Query was: " << stmt->getSQL());
        throw DAOException("Failed to insert Transfer - SQL Exception");
    } catch(const DAOException& exc){
        m_log_error("Got DAOException in Executing Insert: " << exc.what());
        m_log_debug("Query was: " << stmt->getSQL());
        throw;
    } 
    m_log_trace("Transfer created");
}
    
/*
 * countActiveTransfers
 *
 * Count the active (Received & Processing) Transfer instances
 */
unsigned long Channel_TransferDAO::countActiveTransfers() /*throw (DAOLogicError,DAOException)*/{
    
    m_log_trace("Entring Channel_TransferDAO::countActiveTransfers()");

    unsigned long count = 0;    
    
    // Get Context
    OracleDAOContext& ctx = m_transferDaoImpl->ctx();
    
    // Get Statement
    StatementPtr stmt(ctx.connection(),TransferStatements::instance().createCountActiveTransfers_Channel(ctx,false));
    try{
        // Set Parameters
        stmt->setString(1,m_channelName);

        // Execute Query
        ResultSetPtr rs(*stmt,stmt->executeQuery());
        if(0 != rs->next()){
            // Get count
            count = rs->getInt(1);
        }
    } catch(const SQLException& exc){
        m_log_error("Got SQLException in Executing countActiveTransfers(): [0x" << exc.getErrorCode() << "] " << exc.getMessage());
        m_log_debug("Query was: " << stmt->getSQL());
        throw DAOException("Failed to count Transfer - SQL Exception");
    }
    
    m_log_trace("countActiveTransfers() Returned " << count);
    
    return count;
}

/*
 * countActiveTransfers
 *
 * Count the active (Received & Processing) Transfer instances the have been
 * submitted by a VO
 */
unsigned long Channel_TransferDAO::countActiveTransfers(const std::string& vo_name) /*throw (DAOLogicError,DAOException)*/{
    
    // Check Precondition
    if(vo_name.empty()){
        throw DAOLogicError("Invalid VO Name");
    }
    m_log_trace("Entring Channel_TransferDAO::countActiveTransfers");

    unsigned long count = 0;    
    
    // Get Context
    OracleDAOContext& ctx = m_transferDaoImpl->ctx();
    
    // Get Statement
    StatementPtr stmt(ctx.connection(),TransferStatements::instance().createCountActiveTransfers_Channel(ctx,true));
    try{
        // Set Parameters
        stmt->setString(1,m_channelName);
        stmt->setString(2,vo_name);
        
        // Execute Query
        ResultSetPtr rs(*stmt,stmt->executeQuery());
        if(0 != rs->next()){
            // Get count
            count = rs->getInt(1);
        }
    } catch(const SQLException& exc){
        m_log_error("Got SQLException in Executing countActiveTransfers: [0x" << exc.getErrorCode() << "] " << exc.getMessage());
        m_log_debug("Query was: " << stmt->getSQL());
        throw DAOException("Failed to count Transfer - SQL Exception");
    }
    
    m_log_trace("countActiveTransfers Returned " << count);
    
    return count;
}
    
/*
 * getActiveTransfers
 *
 * Return the list of next active transfers for the given channel following 
 * the rule: transfer in received or processing state.
 */
void Channel_TransferDAO::getActiveTransfers(StringPairArray& ids) /*throw (DAOLOgiceError,DAOException)*/{

    m_log_trace("Entring Channel_TransferDAO::getActiveTransfers");

    // Get Context
    OracleDAOContext& ctx = m_transferDaoImpl->ctx();
    
    // Get Statement
    StatementPtr stmt(ctx.connection(),TransferStatements::instance().createGetActiveTransfers_Channel(ctx));
    try{
        // Set Parameters
        stmt->setString(1,m_channelName);
        
        // Get Data
        m_transferDaoImpl->getIdPairArray(*stmt,ids);
    } catch(const SQLException& exc){
        m_log_error("Got SQLException in Executing getActiveTransfers: [0x" << exc.getErrorCode() << "] " << exc.getMessage());
        m_log_debug("Query was: " << stmt->getSQL());
        throw DAOException("Failed to get Transfer - SQL Exception");
    } catch(const DAOException& exc){
        m_log_error("Got DAOException in Executing getActiveTransfers: " << exc.what());
        m_log_debug("Query was: " << stmt->getSQL());
        throw;
    } 
    
    m_log_trace("getActiveTransfers Returned " << ids.size() << " elements");
}

/*
 * isValidFile
 *
 * Return true if the file id is valid
 */
bool Channel_TransferDAO::isValidFile(const std::string& file_id) /*throw (DAOException)*/{
    
    bool result = false;
    
    // Get Context
    OracleDAOContext& ctx = m_transferDaoImpl->ctx();
    
    // Get Statement
    StatementPtr stmt(ctx.connection(),FileStatements::instance().createGet_Channel(ctx));
    try{
        // Set Parameters
        stmt->setString(1,file_id);
        stmt->setString(2,m_channelName);
        
        // Execute Query
        ResultSetPtr rs(*stmt,stmt->executeQuery());
        // Check if there is an entry
        if(rs->next()){
            result = true;
        }
    } catch(const SQLException& exc){
        m_log_error("Got SQLException in Executing Get: [0x" << exc.getErrorCode() << "] " << exc.getMessage());
        m_log_debug("Query was: " << stmt->getSQL());
        throw DAOException("Failed to check File validity - SQL Exception");
    } catch(const DAOException& exc){
        m_log_debug("No such FileId " << file_id);
    } 
    
    return result;
}
