/*
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *  GLite Data Transfer Agents: OracleCredStatements Class
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *           Paolo Tedesco <paolo.tedesco@cern.ch>
 *  Version info: $Id: CredStatements.cpp,v 1.7 2008/11/13 10:43:56 ptedesco Exp $
 *  Release: $Name:  $
 *
 */
#include "CredStatements.h"
#include "dao-utility.h"
#include <glite/data/agents/dao/oracle/StatementPtr.h>

#include <sstream>

using namespace glite::data::transfer::agent;
using namespace glite::data::transfer::agent::dao;
using namespace glite::data::transfer::agent::dao::oracle;
using glite::data::agents::dao::DAOLogicError;
using glite::data::agents::dao::DAOException;
using glite::data::agents::dao::oracle::OracleDAOContext;
using glite::data::agents::dao::oracle::StatementPtr;
using namespace ::oracle::occi;

namespace {
const char * const ORACLE_CRED_STATEMENTS_NAME = "transfer-agent-dao-oracle-cred-stmt";

const char * const GET_STATEMENT_TAG                   = "t_credential_get";
const char * const GET_STATEMENT_VO_TAG                = "t_credential_get_vo";
const char * const GET_EXPIRING_STATEMENT_TAG          = "t_credential_get_expiring";
}

/*
 * instance
 *
 * Return Singleton instance
 */
CredStatements& CredStatements::instance() /*throw ()*/{
    static CredStatements singleton;
    return singleton;
}

/*
 * CredStatements
 *
 * Constructor
 */
CredStatements::CredStatements():
    m_logger(ORACLE_CRED_STATEMENTS_NAME){
}

/*
 * dispose
 *
 * dispose Statements For OracleCredDAO
 */
void CredStatements::dispose(OracleDAOContext& ctx) /*throw ()*/{
#warning "TODO : terminate Statements"
}

/*
 * createGet
 *
 * Prepare Statement For CredDAO::get, Cred View
 *
 * Bind Parameters:
 *  :1 - Cred ID
 *  :2 - User DN
 */
Statement * CredStatements::createGet(OracleDAOContext& ctx) /*throw (DAOLogicError, DAOException)*/{

    // Get Tag
    const char * const tag = GET_STATEMENT_TAG;

    Statement * stmt = 0;
    try {
        // Check if the statement is already cached
        if(false == ctx.connection().isCached("",tag)){
            // Prepare Statement
            const char * const query_stmt =
                   "SELECT t_credential.dlg_id"
                   ", t_credential.dn"
                   ", t_credential.proxy"
                   ", t_credential.termination_time"
                   ", t_credential.voms_attrs"
                   " FROM t_credential"
                   " WHERE t_credential.dlg_id = :1"
                   " AND t_credential.dn = :2";
            m_log_trace("Preparing Statement: " << query_stmt);
            // Create Statements
            stmt = ctx.connection().createStatement(query_stmt,tag);
        } else {
            // Create Statements
            stmt = ctx.connection().createStatement("",tag);
        }
    } catch(const SQLException& exc){
        m_log_error("Got Exception in creating Get Statement: [0x" << exc.getErrorCode() << "] " << exc.getMessage());
        throw DAOException("Failed to prepare Statement");
    }
    // Check Postcondition
    if(0 == stmt){
        throw DAOLogicError("Failed to prepare Statement - Nullpointer");
    }
    return stmt;
}

/*
 * createGet
 *
 * Prepare Statement For CredDAO::get, Cred View
 *
 * Bind Parameters:
 *  :1 - Cred ID
 *  :2 - User DN
 *  :3 - VO Name
 */
Statement * CredStatements::createGet_VO(OracleDAOContext& ctx) /*throw (DAOLogicError, DAOException)*/{

    // Get Tag
    const char * const tag = GET_STATEMENT_VO_TAG;

    Statement * stmt = 0;
    try {
        // Check if the statement is already cached
        if(false == ctx.connection().isCached("",tag)){
            // Prepare Statement
            const char * const query_stmt =
                   "SELECT t_credential.dlg_id"
                   ", t_credential.dn"
                   ", t_credential.proxy"
                   ", t_credential.termination_time"
                   ", t_credential.voms_attrs"
                   " FROM t_credential"
                   " WHERE t_credential.dlg_id = :1"
                   " AND t_credential.dn = :2"
                   " AND EXISTS ("
                    " SELECT NULL FROM t_job"
                    " WHERE t_job.vo_name = :3"
                    " AND t_job.user_dn = t_credential.dn"
                    " AND t_job.cred_id = t_credential.dlg_id"
                   ")";
            m_log_trace("Preparing Statement: " << query_stmt);
            // Create Statements
            stmt = ctx.connection().createStatement(query_stmt,tag);
        } else {
            // Create Statements
            stmt = ctx.connection().createStatement("",tag);
        }
    } catch(const SQLException& exc){
        m_log_error("Got Exception in creating Get Statement: [0x" << exc.getErrorCode() << "] " << exc.getMessage());
        throw DAOException("Failed to prepare Statement");
    }
    // Check Postcondition
    if(0 == stmt){
        throw DAOLogicError("Failed to prepare Statement - Nullpointer");
    }
    return stmt;
}


/*
 * createUpdate
 *
 * Prepare Statement For CredDAO::update, Cred View. Returns 0 if cred doesn't need to be updated
 */
Statement * CredStatements::createUpdate(OracleDAOContext& ctx, const model::Cred& c) /*throw (DAOLogicError,DAOException)*/ {

    Statement * stmt = 0;        
    std::stringstream query_stmt;
    try {
        
        // Update Cred Properties
        // -----------------------------
        // | Column            | RO/RW |
        // -----------------------------
        // | dlg_id            |  RO   |
        // | dn                |  RO   |
        // | proxy             |  RW   |
        // | termination_time  |  RW   |
        // | voms_attrs        |  RO   |
        // -----------------------------
        
        // Prepare Statement String
        unsigned int count = 1;
        query_stmt << "UPDATE t_credential SET ";
        // certificate
        if(c.certificate.isSet()){
            query_stmt << ((1 == count)?"":",") << " t_credential.proxy = :" << count;
            ++count;
        }
        // Termination Time
        if(c.expirationTime.isSet()){
            query_stmt << ((1 == count)?"":",") << " t_credential.termination_time = :" << count;
            ++count;
        }
        query_stmt << " WHERE t_credential.dlg_id = :" << count;
            ++count;
        query_stmt << " AND   t_credential.dn     = :" << count;
            ++count;
        if(count > 3){
            m_log_trace("Preparing Statement: " << query_stmt.str());
            // Create Statements
            stmt = ctx.connection().createStatement(query_stmt.str());
            
            // RAII
            StatementPtr stmt_ptr(ctx.connection(),stmt);
            
            // Set the value of the properties
            count = 1;
            // proxy
            if(c.certificate.isSet()){
                stmt->setString(count++,c.certificate());
            }    
            // termination_time
            if(c.expirationTime.isSet()){
                if((time_t)-1 != c.expirationTime()){
                    stmt->setTimestamp(count++,ctx.toTimestamp(c.expirationTime()));
                } else {
                    stmt->setNull(count++ ,OCCITIMESTAMP);
                }
            }          
            stmt->setString(count++,c.id());
            stmt->setString(count++,c.userDn());
            // Release the Pointer
            stmt_ptr.release();
        } else {
            m_log_trace("Nothing to update");
        }
    } catch(const SQLException& exc){
        m_log_error("Got Exception in creating Update Statement: [0x" << exc.getErrorCode() << "] " << exc.getMessage());
        m_log_debug("Query was: " << query_stmt.str());
        throw DAOException("Failed to prepare Statement");
    }
    // Don't Check Postcondition (Null is allowed)   
    return stmt;
}

/*
 * createGetExpiringCred
 *
 * Return the statement for the CredDAO::getExpiringCred
 *
 * Bind Parameters:
 *  :1 - Interval
 *  :2 - Limit
 *  :3 - Limit
 */
Statement* CredStatements::createGetExpiringCreds(OracleDAOContext& ctx) /*throw (DAOLogicError,DAOException)*/ {
    
    // Get Tag
    const char * const tag = GET_EXPIRING_STATEMENT_TAG;

    Statement * stmt = 0; 
    try {
        // Check if the statement is already cached
        if(false == ctx.connection().isCached("",tag)){
            // Prepare Statement String
            const char * const query_stmt =
                "SELECT cid, dn FROM("
                    "SELECT t_credential.dlg_id AS cid"
                         ", t_credential.dn AS dn"
                    " FROM t_credential"
                    " WHERE t_credential.termination_time"
                        " BETWEEN SYSTIMESTAMP"
                        " AND (SYSTIMESTAMP + NUMTODSINTERVAL(:1, 'SECOND'))"
                    " AND EXISTS ("
                        "SELECT NULL FROM T_JOB"
                        " WHERE T_JOB.CRED_ID = t_credential.dlg_id"
                        " AND   T_JOB.USER_DN = t_credential.dn"
                        " AND   T_JOB.JOB_STATE NOT IN "
                            "('Canceled','Failed','Finished','FinishedDirty')"
                    ") ORDER BY t_credential.termination_time ASC "
                 ") WHERE :2=0 OR ROWNUM<=:3";
            m_log_trace("Preparing Statement: " << query_stmt);
            // Create Statements
            stmt = ctx.connection().createStatement(query_stmt,tag);
        } else {
            // Create Statements
            stmt = ctx.connection().createStatement("",tag);
        }
    } catch(const SQLException& exc){
        m_log_error("Got Exception in Creating GetWaitingFiles Statement: [0x" << exc.getErrorCode() << "] " << exc.getMessage());
        throw DAOException("Failed to prepare Statement");
    }
    // Check Postcondition
    if(0 == stmt){
        throw DAOException("Failed to prepare Statement - Nullpointer");
    }
    return stmt;
}
