/*
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *  GLite Data Transfer Agents: Oracle Cred DAO Factory Class
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *  Version info: $Id: Cred_DAOFactory.cpp,v 1.5 2008/11/13 10:43:56 ptedesco Exp $
 *  Release: $Name:  $
 *
 */
#include "Cred_DAOFactory.h"
#include "dao-utility.h"
#include "Cred_CredDAO.h"
#include "Cred_JobDAO.h"
#include "Cred_AgentDAO.h"

using namespace glite::data::transfer::agent;
using namespace glite::data::transfer::agent::dao;
using namespace glite::data::transfer::agent::dao::oracle;
using glite::data::agents::dao::DAOLogicError;
using glite::data::agents::dao::DAOContext;
using glite::data::agents::dao::oracle::OracleDAOContext;

/*
 * Constants
 */
namespace {
const char * const ORACLE_CRED_DAO_NAME = "transfer-agent-dao-oracle-cred";
}

/*
 * Cred_DAOFactory
 *
 * Constructor
 */
Cred_DAOFactory::Cred_DAOFactory():cred::DAOFactory(),
    m_logger(ORACLE_CRED_DAO_NAME){
}

/*
 * checkSchema
 *
 * Validate the DB schema
 */
void Cred_DAOFactory::checkSchema(DAOContext& ctx) /*throw (DAOLogicError,DAOException)*/{
    try{    
        check_schema(ctx);
    } catch(const agents::dao::DAOException& exc){
        m_log_error("Failed To Check Schema Version. Reason is : " << exc.what());
        throw;
    }
    try{    
        check_cred_schema(ctx);
    } catch(const agents::dao::DAOException& exc){
        m_log_error("Failed To Check Credential Schema Version. Reason is : " << exc.what());
        throw;
    }
}

/*
 * createCredDAO
 *
 * Returns DataAccessObject for model::Cred instances
 */
cred::CredDAO * Cred_DAOFactory::createCredDAO(DAOContext& ctx) /*throw (DAOLogicError)*/{
    try{
        // Cast Context
        OracleDAOContext& oracle_ctx = dynamic_cast<OracleDAOContext&>(ctx);
    
        // Create Cred DAO
        return new Cred_CredDAO(oracle_ctx,ORACLE_CRED_DAO_NAME);
    } catch(const std::exception& exc){
        m_log_error("Invalid DAOContext: " << exc.what());
        throw DAOLogicError("invalid DAOContext type");
    }
    throw DAOLogicError("invalid DAOFactory");
}


/*
 * createJobDAO
 *
 * Returns DataAccessObject for model::Job instances
 */
cred::JobDAO * Cred_DAOFactory::createJobDAO(DAOContext& ctx) /*throw (DAOLogicError)*/{
    try{
        // Cast Context
        OracleDAOContext& oracle_ctx = dynamic_cast<OracleDAOContext&>(ctx);
    
        // Create Cred DAO
        return new Cred_JobDAO(oracle_ctx,ORACLE_CRED_DAO_NAME);
    } catch(const std::exception& exc){
        m_log_error("Invalid DAOContext: " << exc.what());
        throw DAOLogicError("invalid DAOContext type");
    }
    throw DAOLogicError("invalid DAOFactory");
}

/*
 * createAgentDAO
 *
 * Returns DataAccessObject for model::Agent instances
 */
cred::AgentDAO * Cred_DAOFactory::createAgentDAO(const std::string& name, DAOContext& ctx) /*throw (DAOLogicError)*/{
    try{
        // Cast Context
        OracleDAOContext& oracle_ctx = dynamic_cast<OracleDAOContext&>(ctx);
    
        // Create Agent DAO
        return new Cred_AgentDAO(oracle_ctx, name, ORACLE_CRED_DAO_NAME);
    } catch(const std::exception& exc){
        m_log_error("Invalid DAOContext: " << exc.what());
        throw DAOLogicError("invalid DAOContext type");
    }
    throw DAOLogicError("invalid DAOFactory");
}
