/**
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *  GLite Data Transfer Agents: OracleAgentStatements Class
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>, Paolo Tedesco <paolo.tedesco@cern.ch>
 *  Version info: $Id: DumpStatements.cpp,v 1.14 2008/12/04 08:53:30 ptedesco Exp $
 *  Release: $Name:  $
 *
 */
#include "DumpStatements.h"
#include <glite/data/agents/dao/oracle/OracleDAOContext.h>
#include <glite/data/agents/dao/DAOContextFactory.h>
#include "AgentStatements.h"
#include "ChannelStatements.h"
#include "FileStatements.h"
#include "JobStatements.h"
#include "TransferStatements.h"
#include "CredStatements.h"
#include <glite/data/agents/dao/oracle/StatementPtr.h>
#include <glite/data/config/service/ParamReader.hxx>

#include <fstream>
#include <memory>
#include <boost/scoped_ptr.hpp>

using namespace glite::data::transfer::agent;
using namespace glite::data::transfer::agent::dao;
using namespace glite::data::transfer::agent::dao::oracle;
using namespace glite::config;
using namespace glite::data::agents::dao::oracle;
using namespace glite::data::agents::dao;
using namespace glite::data::agents;

/*
 * Constants
 */
namespace {
const char * const ORACLE_DAO_DUMPER_NAME         = "transfer-agent-dao-oracle-dump";
                                                                                                                          
// Properties Names
const char * const PATH_PROPERTY_NAME             = "Path";
const char * const TYPE_PROPERTY_NAME             = "Type";

const char * const DEFAULT_PATH                   = "/tmp";
        
}

extern "C" {
/*
 * create_glite_component
 *
 * Create Component instance
 */
ComponentConfiguration * create_glite_component(){
    return new StatementsDumper();
}
                                                                                
/*
 * destroy_glite_component
 *
 * Destroy Component instance
 */
void destroy_glite_component(ComponentConfiguration * component){
    StatementsDumper * f = dynamic_cast<StatementsDumper *>(component); 
    if(0 != f){
        delete f;
    }
}
                                                                                
} // End extern "C"

namespace {
    
/**
 * Dump agent statements
 */
void dump_agent_statements(OracleDAOContext* ctx, std::ostream& out) /* throw () */{
 
    // Get VO Statements 
    if(0 != ctx){
        // Get Agent Statements
        AgentStatements& agentStmts = AgentStatements::instance();

        out << std::endl << "-------- Agents Statements --------" << std::endl;
        
        // Get the statement for the AgentDAO::get
        {
            StatementPtr stmt(ctx->connection(),agentStmts.createGet(*ctx));
            out << std::endl << "-------- AgentDAO::get --------" << std::endl
                << stmt->getSQL() << std::endl;
        }
    }
}

/**
 * Dump channel statements
 */
void dump_channel_statements(OracleDAOContext* ctx, std::ostream& out) /* throw () */{
    
    // Get VO Statements 
    if(0 != ctx){
        // Get File Statements
        ChannelStatements& chStmts = ChannelStatements::instance();
        
        out << std::endl << "-------- VO Statements --------" << std::endl;
    
        // Get the statement for the ChannelDAO::get, VO View
        {
            StatementPtr stmt(ctx->connection(),chStmts.createGet_VO(*ctx));
            out << std::endl << "-------- ChannelDAO::get --------" << std::endl
                << stmt->getSQL() << std::endl;
        }
        
        // Get the statement for the ChannelDAO::getBySites, VO View
        {
            StatementPtr stmt(ctx->connection(),chStmts.createGetBySites_VO(*ctx));
            out << std::endl << "-------- ChannelDAO::getBySites --------" << std::endl
                << stmt->getSQL() << std::endl;
        }
        
        // Get the statement for the ChannelDAO::hasShare, VO View
        {
            StatementPtr stmt(ctx->connection(),chStmts.createHasShare_VO(*ctx));
            out << std::endl << "-------- ChannelDAO::hasShare --------" << std::endl
                << stmt->getSQL() << std::endl;
        }
        // Get the statement for the ChannelDAO::getSiteGroups
        {
            StatementPtr stmt(ctx->connection(),chStmts.createGetSiteGroups(*ctx));
            out << std::endl << "-------- ChannelDAO::getSiteGroups --------" << std::endl
                << stmt->getSQL() << std::endl;
        }

        out << std::endl << "-------- Channel Statements --------" << std::endl;
        
        // Get the statement for the ChannelDAO::get, Channel View
        {
            StatementPtr stmt(ctx->connection(),chStmts.createGet_Channel(*ctx));
            out << std::endl << "-------- ChannelDAO::get --------" << std::endl
                << stmt->getSQL() << std::endl;
        }
        // Get the statement for the ChannelDAO::updateLastActive, Channel View
        {
            StatementPtr stmt(ctx->connection(),chStmts.createUpdateLastActive_Channel(*ctx));
            out << std::endl << "-------- ChannelDAO::updateLastActive --------" << std::endl
                << stmt->getSQL() << std::endl;
        }
        // Get the statement for the ChannelDAO::updateTransferType, Channel View
        {
            StatementPtr stmt(ctx->connection(),chStmts.createUpdateTransferType_Channel(*ctx));
            out << std::endl << "-------- ChannelDAO::updateTransferType --------" << std::endl
                << stmt->getSQL() << std::endl;
        }
        // Get the statement for the ChannelDAO::getVONames, Channel View
        {
            StatementPtr stmt(ctx->connection(),chStmts.createGetVONames_Channel(*ctx));
            out << std::endl << "-------- ChannelDAO::getVONames --------" << std::endl
                << stmt->getSQL() << std::endl;
        }
        // Get the statement for the ChannelDAO::getShare, Channel View
        {
            StatementPtr stmt(ctx->connection(),chStmts.createGetShare_Channel(*ctx));
            out << std::endl << "-------- ChannelDAO::getShare --------" << std::endl
                << stmt->getSQL() << std::endl;
        }
        // Get the statement for the ChannelDAO::getLimit, Channel View
        {
            StatementPtr stmt(ctx->connection(),chStmts.createGetLimit_Channel(*ctx));
            out << std::endl << "-------- ChannelDAO::getLimit --------" << std::endl
                << stmt->getSQL() << std::endl;
        }
    }
}

/**
 * Dump file statements
 */
void dump_file_statements(OracleDAOContext* ctx, std::ostream& out) /* throw () */{
    
    // Get VO Statements 
    if(0 != ctx){
        // Get File Statements
        FileStatements& fileStmts = FileStatements::instance();
    
        out << std::endl << "-------- VO Statements --------" << std::endl;
        // Get the statement for the FileDAO::get, VO View
        {
            StatementPtr stmt(ctx->connection(),fileStmts.createGet_VO(*ctx));
            out << std::endl << "-------- FileDAO::get --------" << std::endl
                << stmt->getSQL() << std::endl;
        }
        // Get the statement for the FileDAO::getByJobId, VO View
        {
            StatementPtr stmt(ctx->connection(),fileStmts.createGetByJobId_VO(*ctx));
            out << std::endl << "-------- FileDAO::getByJobId --------" << std::endl
                << stmt->getSQL() << std::endl;
        }
        // Get the statement for the FileDAO::getWaitingFiles, VO View
        {
            StatementPtr stmt(ctx->connection(),fileStmts.createGetWaitingFiles_VO(*ctx));
            out << std::endl << "-------- FileDAO::getWaitingFiles --------" << std::endl
                << stmt->getSQL() << std::endl;
        }
        // Get the statement for the FileDAO::getFilesToCancel, VO View
        {
            StatementPtr stmt(ctx->connection(),fileStmts.createGetCatalogWaitingFiles_VO(*ctx));
            out << std::endl << "-------- FileDAO::getCatalogWaitingFiles --------" << std::endl
                << stmt->getSQL() << std::endl;
        }
        out << std::endl << "-------- Channel Statements --------" << std::endl;
        
        // Get the statement for the FileDAO::get, Channel View
        {
            StatementPtr stmt(ctx->connection(),fileStmts.createGet_Channel(*ctx));
            out << std::endl << "-------- FileDAO::get --------" << std::endl
                << stmt->getSQL() << std::endl;
        }

        // Get the statement for the FileDAO::getByJobId, Channel View
        {
            StatementPtr stmt(ctx->connection(),fileStmts.createGetByJobId_Channel(*ctx));
            out << std::endl << "-------- FileDAO::getByJobId --------" << std::endl
                << stmt->getSQL() << std::endl;
        }
        // Get the statement for the FileDAO::getNextJobToTransfer, Channel View
        {
            StatementPtr stmt(ctx->connection(),fileStmts.createGetNextJobToTransfer_Channel(*ctx));
            out << std::endl << "-------- FileDAO::getNextJobToTransfer --------" << std::endl
                << stmt->getSQL() << std::endl;
        }
        // Get the statement for the FileDAO::createGetJobsToTransferPerSE_Channel, Channel View
        {
            StatementPtr stmt(ctx->connection(),fileStmts.createGetJobsToTransferPerSE_Channel(*ctx));
            out << std::endl << "-------- FileDAO::getJobsToTransferPerSE  --------" << std::endl
                << stmt->getSQL() << std::endl;
        }
        // Get the statement for the FileDAO::createGetFilesToCancel_Channel, Channel View
        {
            StatementPtr stmt(ctx->connection(),fileStmts.createGetFilesToCancel_Channel(*ctx));
            out << std::endl << "-------- FileDAO::createGetFilesToCancel --------" << std::endl
                << stmt->getSQL() << std::endl;
        }
    }
}

/**
 * Dump job statements
 */
void dump_job_statements(OracleDAOContext* ctx, std::ostream& out) /* throw () */{
    
    // Get VO Statements 
    if(0 != ctx){
        // Get File Statements
        JobStatements& jobStmts = JobStatements::instance();

        out << std::endl << "-------- VO Statements --------" << std::endl;
        
        // Get the statement for the JobDAO::get, VO View
        {
            StatementPtr stmt(ctx->connection(),jobStmts.createGet_VO(*ctx));
            out << std::endl << "-------- JobDAO::get --------" << std::endl
                << stmt->getSQL() << std::endl;
        }
        // Get the statement for the JobDAO::getSubmittedJobs, VO View
        {
            StatementPtr stmt(ctx->connection(),jobStmts.createGetSubmittedJobs_VO(*ctx));
            out << std::endl << "-------- JobDAO::getSubmittedJobs --------" << std::endl
                << stmt->getSQL() << std::endl;
        }
        // Get the statement for the JobDAO::getPendingJobs, VO View
        {
            StatementPtr stmt(ctx->connection(),jobStmts.createGetPendingJobs_VO(*ctx));
            out << std::endl << "-------- JobDAO::getPendingJobs --------" << std::endl
                << stmt->getSQL() << std::endl;
        }
        // Get the statement for the JobDAO::getDoneJobs, VO View
        {
            StatementPtr stmt(ctx->connection(),jobStmts.createGetDoneJobs_VO(*ctx));
            out << std::endl << "-------- JobDAO::getDoneJobs --------" << std::endl
                << stmt->getSQL() << std::endl;
        }
        // Get the statement for the JobDAO::getFinishingJobs, VO View
        {
            StatementPtr stmt(ctx->connection(),jobStmts.createGetFinishingJobs_VO(*ctx));
            out << std::endl << "-------- JobDAO::getFinishingJobs --------" << std::endl
                << stmt->getSQL() << std::endl;
        }
        // Get the state of all the files belonging to a given job, VO View
        {
            StatementPtr stmt(ctx->connection(),jobStmts.createGetFileStates_VO(*ctx));
            out << std::endl << "-------- JobDAO::getFileStates --------" << std::endl
                << stmt->getSQL() << std::endl;
        }
        out << std::endl << "-------- Channel Statements --------" << std::endl;
        
        // Get the statement for the JobDAO::get, Channel View
        {
            StatementPtr stmt(ctx->connection(),jobStmts.createGet_Channel(*ctx));
            out << std::endl << "-------- JobDAO::get --------" << std::endl
                << stmt->getSQL() << std::endl;
        }
        // Get the state of all the files belonging to a given job, Channel View
        {
            StatementPtr stmt(ctx->connection(),jobStmts.createGetFileStates_Channel(*ctx));
            out << std::endl << "-------- JobDAO::getFileStates --------" << std::endl
                << stmt->getSQL() << std::endl;
        }
        // Get the job that need to be fixed, Channel View
        {
            StatementPtr stmt(ctx->connection(),jobStmts.createGetJobsToFix_Channel(*ctx));
            out << std::endl << "-------- JobDAO::getJobsToFix --------" << std::endl
                << stmt->getSQL() << std::endl;
        }
        out << std::endl << "-------- Cred Statements --------" << std::endl;
        
        // Get the statement for the JobDAO::get, Channel View
        {
            StatementPtr stmt(ctx->connection(),jobStmts.createGetLast_Cred(*ctx));
            out << std::endl << "-------- JobDAO::getLast --------" << std::endl
                << stmt->getSQL() << std::endl;
        }
    }
}
            
/**
 * Dump transfer statements
 */
void dump_transfer_statements(OracleDAOContext* ctx, std::ostream& out) /* throw () */{
    
    // Get VO Statements 
    if(0 != ctx){
        // Get File Statements
        TransferStatements& trStmts = TransferStatements::instance();
        
        out << std::endl << "-------- VO Statements --------" << std::endl;
        // Get the statement for the TransferDAO::get, VO View
        {
            StatementPtr stmt(ctx->connection(),trStmts.createGet_VO(*ctx));
            out << std::endl << "-------- TransferDAO::get --------" << std::endl
                << stmt->getSQL() << std::endl;
        }
        // Get the statement for the TransferDAO::getByFileId, VO View
        {
            StatementPtr stmt(ctx->connection(),trStmts.createGetByFileId_VO(*ctx));
            out << std::endl << "-------- TransferDAO::getByFileId --------" << std::endl
                << stmt->getSQL() << std::endl;
        }
        // Get the statement for the TransferDAO::getByRequestId, VO View
        {
            StatementPtr stmt(ctx->connection(),trStmts.createGetByRequestId_VO(*ctx));
            out << std::endl << "-------- TransferDAO::getByRequestId --------" << std::endl
                << stmt->getSQL() << std::endl;
        }
        out << std::endl << "-------- Channel Statements --------" << std::endl;
        
        // Get the statement for the TransferDAO::get, Channel View
        {
            StatementPtr stmt(ctx->connection(),trStmts.createGet_Channel(*ctx));
            out << std::endl << "-------- TransferDAO::get --------" << std::endl
                << stmt->getSQL() << std::endl;
        } 
        // Get the statement for the TransferDAO::getByFileId, Channel View
        {
            StatementPtr stmt(ctx->connection(),trStmts.createGetByFileId_Channel(*ctx));
            out << std::endl << "-------- TransferDAO::getByFileId --------" << std::endl
                << stmt->getSQL() << std::endl;
        }
        // Get the statement for the TransferDAO::getByRequestId, Channel View
        {
            StatementPtr stmt(ctx->connection(),trStmts.createGetByRequestId_Channel(*ctx));
            out << std::endl << "-------- TransferDAO::getByRequestId --------" << std::endl
                << stmt->getSQL() << std::endl;
        }
        // Get the statement for the TransferDAO::getActiveTransfers, Channel View
        {
            StatementPtr stmt(ctx->connection(),trStmts.createGetActiveTransfers_Channel(*ctx));
            out << std::endl << "-------- TransferDAO::getActiveTransfers --------" << std::endl
                << stmt->getSQL() << std::endl;
        }
    }
} 
    
/**
 * Dump cred statements
 */
void dump_cred_statements(OracleDAOContext* ctx, std::ostream& out) /* throw () */{
 
    // Get Cred Statements 
    if(0 != ctx){
        // Get Agent Statements
        CredStatements& credStmts = CredStatements::instance();

        out << std::endl << "-------- Cred Statements --------" << std::endl;
        
        // Get the statement for the CredDAO::get
        {
            StatementPtr stmt(ctx->connection(),credStmts.createGet(*ctx));
            out << std::endl << "-------- CredDAO::get --------" << std::endl
                << stmt->getSQL() << std::endl;
        }
        
        out << std::endl << "-------- VO Statements --------" << std::endl;
        
        // Get the statement for the CredDAO::get
        {
            StatementPtr stmt(ctx->connection(),credStmts.createGet_VO(*ctx));
            out << std::endl << "-------- CredDAO::get --------" << std::endl
                << stmt->getSQL() << std::endl;
        }
        
        out << std::endl << "-------- Cred Statements --------" << std::endl;
        
        // Get the statement for the CredDAO::get
        {
            StatementPtr stmt(ctx->connection(),credStmts.createGetExpiringCreds(*ctx));
            out << std::endl << "-------- CredDAO::createGetExpiringCreds --------" << std::endl
                << stmt->getSQL() << std::endl;
        }
    }
}


} // End global namespace

/*
 * StatementsDumper
 *
 * Constructor
 */
StatementsDumper::StatementsDumper():
    ComponentConfiguration(ORACLE_DAO_DUMPER_NAME),
    m_path(DEFAULT_PATH),
    m_logger(ORACLE_DAO_DUMPER_NAME){
}

/*
 * ~StatementsDumper
 *
 * Destructor
 */
StatementsDumper::~StatementsDumper(){
}

/*
 * init
 *
 * Initialize the Component
 */
int StatementsDumper::config(const Params& params){
                                                                                                                          
    try{
        ParamReader reader(getName(),params);

        // Get Path
        reader.getParameter(PATH_PROPERTY_NAME,m_path);
        
        // Get Type
        reader.getParameter(TYPE_PROPERTY_NAME,m_type);
        
        // Dump The Statements
        this->dump();
    } catch(const LogicError& exc) {
        m_log_error(getName() << " configuration Error: " << exc.what());
        return -1;
    } catch(const std::exception& exc) {
        m_log_error(getName() << " configuration Unexpected Exception: " << exc.what());
        throw;
    } catch(...) {
        m_log_error(getName() << " configuration Unhandled Exception");
        throw;
    }
    return 0;
}
      
/**
 * Initialize all the statements aand dump the related sql to files into the 
 * specified path
 * @param path [IN] the directory where to store the statements
 * @param type [IN] dump only statements of the given type. Can be one fo the 
 *   following values: 'agent';'channel';'file';'job';'transfer'
 *   An empty string will dump all the statements
 */
void StatementsDumper::dump() /* throw () */{
    
    try {
        boost::scoped_ptr<OracleDAOContext> ctx;
        
        // Create a Context for the VO View
        try{
            std::auto_ptr<DAOContext> c(DAOContextFactory::instance().create());
            if(dynamic_cast<OracleDAOContext*>(c.get()) != 0){
                ctx.reset(dynamic_cast<OracleDAOContext*>(c.release()));
            }
        }catch(const DAOException& exc){
        }
        
        if((m_type == "agent") || (true == m_type.empty())){
            std::string f_name = m_path + "/agent.stmt.sql";
            std::ofstream fout;
            fout.open(f_name.c_str(), std::ofstream::out | std::ofstream::trunc);
            dump_agent_statements(ctx.get(),fout);
            fout.close();
        }
        if((m_type == "channel") || (true == m_type.empty())){
            std::string f_name = m_path + "/channel.stmt.sql";
            std::ofstream fout;
            fout.open(f_name.c_str(), std::ofstream::out | std::ofstream::trunc);
            dump_channel_statements(ctx.get(),fout);
            fout.close();
        }
        if((m_type == "file") || (true == m_type.empty())){
            std::string f_name = m_path + "/file.stmt.sql";
            std::ofstream fout;
            fout.open(f_name.c_str(), std::ofstream::out | std::ofstream::trunc);
            dump_file_statements(ctx.get(),fout);
            fout.close();
        }
        if((m_type == "job") || (true == m_type.empty())){
            std::string f_name = m_path + "/job.stmt.sql";
            std::ofstream fout;
            fout.open(f_name.c_str(), std::ofstream::out | std::ofstream::trunc);
            dump_job_statements(ctx.get(),fout);
            fout.close();
        }
        if((m_type == "transfer") || (true == m_type.empty())){
            std::string f_name = m_path + "/transfer.stmt.sql";
            std::ofstream fout;
            fout.open(f_name.c_str(), std::ofstream::out | std::ofstream::trunc);
            dump_transfer_statements(ctx.get(),fout);
            fout.close();
        }
        if((m_type == "cred") || (true == m_type.empty())){
            std::string f_name = m_path + "/cred.stmt.sql";
            std::ofstream fout;
            fout.open(f_name.c_str(), std::ofstream::out | std::ofstream::trunc);
            dump_cred_statements(ctx.get(),fout);
            fout.close();
        }
    } catch(const std::exception& exc) {
        m_log_error("Failed to dump statements. Reason is " << exc.what());
    } catch(...) {
        m_log_error("Failed to dump statements. Unknown Reason");
    }
}
