/*
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *  GLite Data Transfer Agents: OracleFileStatements Class
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *           Paolo Tedesco <paolo.tedesco@cern.ch>
 *  Version info: $Id: FileStatements.cpp,v 1.31 2008/11/13 10:43:56 ptedesco Exp $
 *  Release: $Name:  $
 *
 */
#include "FileStatements.h"
#include "dao-utility.h"
#include <glite/data/agents/dao/oracle/StatementPtr.h>

#include <sstream>

using namespace glite::data::transfer::agent;
using namespace glite::data::transfer::agent::dao;
using namespace glite::data::transfer::agent::dao::oracle;
using glite::data::agents::dao::DAOLogicError;
using glite::data::agents::dao::DAOException;
using glite::data::agents::dao::oracle::OracleDAOContext;
using glite::data::agents::dao::oracle::StatementPtr;
using namespace ::oracle::occi;

namespace {
const char * const ORACLE_FILE_STATEMENTS_NAME = "transfer-agent-dao-oracle-file-stmt";

const char * const GET_STATEMENT_VO_TAG                   = "t_file_get_vo";
const char * const GET_FULL_SEARCH_STATEMENT_VO_TAG       = "t_file_get_full_search_vo";
const char * const GET_STATEMENT_CH_TAG                   = "t_file_get_ch";
const char * const GET_FULL_SEARCH_STATEMENT_CH_TAG       = "t_file_get_full_search_ch";
const char * const UPDATE_STATEMENT_VO_TAG                = "t_file_update_vo";
const char * const UPDATE_STATEMENT_CH_TAG                = "t_file_update_ch";
const char * const GET_BY_JOBID_STATEMENT_VO_TAG          = "t_file_get_by_jobid_vo";
const char * const GET_BY_JOBID_STATEMENT_CH_TAG          = "t_file_get_by_jobid_ch";
const char * const GET_WAITING_STATEMENT_VO_TAG           = "t_file_get_waiting_vo";
const char * const GET_CATALOG_WAITING_STATEMENT_VO_TAG   = "t_file_get_cat_waiting_vo";
const char * const GET_FILES_TO_CANCEL_STATEMENT_CH_TAG   = "t_file_get_files_to_cancel_ch";
const char * const GET_JOB_TO_TX_STATEMENT_CH_TAG         = "t_file_get_job_to_transfer_ch";
const char * const GET_JOBS_TO_TX_PER_SE_STATEMENT_CH_TAG = "t_file_get_jobs_to_tx_per_se_ch";
}

/*
 * instance
 *
 * Return Singleton instance
 */
FileStatements& FileStatements::instance() /*throw ()*/{
    static FileStatements singleton;
    return singleton;
}

/*
 * FileStatements
 *
 * Constructor
 */
FileStatements::FileStatements():
    Statements(ORACLE_FILE_STATEMENTS_NAME){
}

/*
 * dispose
 *
 * dispose Statements For OracleFileDAO
 */
void FileStatements::dispose(OracleDAOContext& ctx) /*throw ()*/{
#warning "TODO : terminate Statements"
}

/*
 * createGet_VO
 *
 * Prepare Statement For FileDAO::get, VO View
 *
 * Bind Parameters:
 *  :1 - File ID
 *  :2 - VO Name
 */
namespace{
const char * create_get_vo(){
    return "SELECT t_file.file_id"
        ", t_file.job_id"
        ", t_file.file_state"
        ", t_file.logical_name"
        ", t_file.source_surl"
        ", t_file.dest_surl"
        ", t_file.reason_class"
        ", t_file.reason"
        ", t_file.num_failures"
        ", t_file.current_failures"
        ", t_file.catalog_failures"
        ", t_file.prestage_failures"
        ", t_file.filesize"
        ", t_file.checksum"
        ", t_file.finish_time"
        ", t_file.agent_dn"
        ", t_file.internal_file_params"
        ", t_file.error_scope"
        ", t_file.error_phase"
        " FROM t_file, t_job"
        " WHERE t_file.file_id = :1"
        " AND t_file.job_id = t_job.job_id"
        " AND t_job.vo_name = :2"
        " AND t_file.job_finished is NULL"
        " AND t_job.job_finished is NULL";
}

const char * create_get_full_search_vo(){
    return "SELECT t_file.file_id"
        ", t_file.job_id"
        ", t_file.file_state"
        ", t_file.logical_name"
        ", t_file.source_surl"
        ", t_file.dest_surl"
        ", t_file.reason_class"
        ", t_file.reason"
        ", t_file.num_failures"
        ", t_file.current_failures"
        ", t_file.catalog_failures"
        ", t_file.prestage_failures"
        ", t_file.filesize"
        ", t_file.checksum"
        ", t_file.finish_time"
        ", t_file.agent_dn"
        ", t_file.internal_file_params"
        ", t_file.error_scope"
        ", t_file.error_phase"
        " FROM t_file, t_job"
        " WHERE t_file.file_id = :1"
        " AND t_file.job_id = t_job.job_id"
        " AND t_job.vo_name = :2";
}
}
Statement * FileStatements::createGet_VO(
    OracleDAOContext&   ctx,
    const bool&         finished) /*throw (DAOLogicError, DAOException)*/{
    
    if (finished){
        return create_statement(ctx,GET_FULL_SEARCH_STATEMENT_VO_TAG,create_get_full_search_vo,"Get Full Search (VO)");
    } else {
        return create_statement(ctx,GET_STATEMENT_VO_TAG,create_get_vo,"Get (VO)");
    }
}

/*
 * createGet_Channel
 *
 * Prepare Statement For FileDAO::get, Channel View
 *
 * Bind Parameters:
 *  :1 - File ID
 *  :2 - Channel Name
 */
namespace{
const char * create_get_channel(){
    return "SELECT t_file.file_id"
        ", t_file.job_id"
        ", t_file.file_state"
        ", t_file.logical_name"
        ", t_file.source_surl"
        ", t_file.dest_surl"
        ", t_file.reason_class"
        ", t_file.reason"
        ", t_file.num_failures"
        ", t_file.current_failures"
        ", t_file.catalog_failures"
        ", t_file.prestage_failures"
        ", t_file.filesize"
        ", t_file.checksum"
        ", t_file.finish_time"
        ", t_file.agent_dn"
        ", t_file.internal_file_params"
        ", t_file.error_scope"
        ", t_file.error_phase"
        " FROM t_file, t_job"
        " WHERE t_file.file_id = :1"
        " AND t_file.job_id = t_job.job_id"
        " AND t_job.channel_name = :2"
        " AND t_file.job_finished is NULL"
        " AND t_job.job_finished is NULL";
}
const char * create_get_full_search_channel(){
    return "SELECT t_file.file_id"
        ", t_file.job_id"
        ", t_file.file_state"
        ", t_file.logical_name"
        ", t_file.source_surl"
        ", t_file.dest_surl"
        ", t_file.reason_class"
        ", t_file.reason"
        ", t_file.num_failures"
        ", t_file.current_failures"
        ", t_file.catalog_failures"
        ", t_file.prestage_failures"
        ", t_file.filesize"
        ", t_file.checksum"
        ", t_file.finish_time"
        ", t_file.agent_dn"
        ", t_file.internal_file_params"
        ", t_file.error_scope"
        ", t_file.error_phase"
        " FROM t_file, t_job"
        " WHERE t_file.file_id = :1"
        " AND t_file.job_id = t_job.job_id"
        " AND t_job.channel_name = :2";
}
}
Statement * FileStatements::createGet_Channel(
    OracleDAOContext&   ctx,
    const bool&         finished) /*throw (DAOLogicError,DAOException)*/{
        
    if (finished){
        return create_statement(ctx,GET_FULL_SEARCH_STATEMENT_CH_TAG,create_get_full_search_channel,"Get Full Search (Channel)");    
    } else {
        return create_statement(ctx,GET_STATEMENT_CH_TAG,create_get_channel,"Get (Channel)");
    }
}

/*
 * createUpdate_VO
 *
 * Prepare Statement For FileDAO::update, VO View. Returns 0 if job doesn't need to be updated
 */
Statement * FileStatements::createUpdate_VO(OracleDAOContext& ctx, const model::File& file, const std::string& vo_name) /*throw (DAOLogicError,DAOException)*/ {

    Statement * stmt = 0;        
    try {
        
        // Update File Properties
        // --------------------------------
        // | Column               | RO/RW |
        // --------------------------------
        // | file_id              |  RO   |
        // | job_id               |  RO   |
        // | file_state           |  RW   |
        // | logical_name         |  RO   |
        // | source_surl          |  RW   |
        // | dest_surl            |  RW   |
        // | reason               |  RW   |
        // | reasonClass          |  RW   |
        // | failures             |  RW   |
        // | current_failures     |  RW   |
        // | catalog_failures     |  RW   |
        // | prestage_failures    |  RW   |
        // | filesize             |  RW   |
        // | checksum             |  RW   |
        // | agent_dn             |  RW   |
        // | finish_time          |  RW   |
        // | internal_file_params |  RW   |
        // | error_scope          |  RW   |
        // | error_phase          |  RW   |
        // --------------------------------
        
        // Prepare Statement String
        unsigned int count = 3;
        std::stringstream query_stmt;
        query_stmt << "UPDATE ( "
                      "SELECT t_file.* from t_file, t_job WHERE"
                      " t_file.file_id = :1"
                      " AND t_file.job_id = t_job.job_id"
                      " AND t_job.vo_name = :2"
                      " AND t_file.job_finished is NULL"
                      " AND t_job.job_finished is NULL"
                      " ) t_file SET ";
        // State
        if(file.state.isSet()){
            query_stmt << ((3 == count)?"":",") << " t_file.file_state = :" << count;
            ++count;
        }
        // Source SURL
        if(file.sourceSurl.isSet()){
            query_stmt << ((3 == count)?"":",") << " t_file.source_surl = :" << count;
            ++count;
        }
        // Dest SURL
        if(file.destSurl.isSet()){
            query_stmt << ((3 == count)?"":",") << " t_file.dest_surl = :" << count;
            ++count;
        }
        // reason
        if(file.reason.isSet()){
            query_stmt << ((3 == count)?"":",") << " t_file.reason = :" << count;
            ++count;
        }
        // reason class
        if(file.reasonClass.isSet()){
            query_stmt << ((3 == count)?"":",") << " t_file.reason_class = :" << count;
            ++count;
        }
        // num_failures
        if(file.failures.isSet()){
            query_stmt << ((3 == count)?"":",") << " t_file.num_failures = :" << count;
            ++count;
        }
        // current_failures
        if(file.currentFailures.isSet()){
            query_stmt << ((3 == count)?"":",") << " t_file.current_failures = :" << count;
            ++count;
        }
        // catalog_failures
        if(file.catalogFailures.isSet()){
            query_stmt << ((3 == count)?"":",") << " t_file.catalog_failures = :" << count;
            ++count;
        }
        // prestage_failures
        if(file.prestageFailures.isSet()){
            query_stmt << ((3 == count)?"":",") << " t_file.prestage_failures = :" << count;
            ++count;
        }
        // filesize
        if(file.fileSize.isSet()){
            query_stmt << ((3 == count)?"":",") << " t_file.filesize = :" << count;
            ++count;
        }
        // checksum
        if(file.checksum.isSet()){
            query_stmt << ((3 == count)?"":",") << " t_file.checksum = :" << count;
            ++count;
        }
        // agent_dn
        if(file.agentDn.isSet()){
            query_stmt << ((3 == count)?"":",") << " t_file.agent_dn = :" << count;
            ++count;
        }
        // finish_time
        if(file.finishTime.isSet()){
            query_stmt << ((3 == count)?"":",") << " t_file.finish_time = :" << count;
            ++count;
        }
        // internal_file_params
        if(file.internalParams.isSet()){
            query_stmt << ((3 == count)?"":",") << " t_file.internal_file_params = :" << count;
            ++count;
        }
        // error_scope
        if(file.errorScope.isSet()){
            query_stmt << ((3 == count)?"":",") << " t_file.error_scope = :" << count;
            ++count;
        }
        // error_phase
        if(file.errorPhase.isSet()){
            query_stmt << ((3 == count)?"":",") << " t_file.error_phase = :" << count;
            ++count;
        }
        
        if(count > 3){
            m_log_trace("Preparing Statement: " << query_stmt.str());
            // Create Statements
            stmt = ctx.connection().createStatement(query_stmt.str());
            
            // RAII
            StatementPtr stmt_ptr(ctx.connection(),stmt);
            
            // Set the value of the properties
            stmt->setString(1,file.id());
            stmt->setString(2,vo_name);
            count = 3;
            // State
            if(file.state.isSet()){
                stmt->setString(count++,translateFileState(file.state()));
            }    
            // Source Surl
            if(file.sourceSurl.isSet()){
                stmt->setString(count++,file.sourceSurl());
            }
            // Dest Surl
            if(file.destSurl.isSet()){
                stmt->setString(count++,file.destSurl());
            }   
            // reason
            if(file.reason.isSet()){
                if(file.reason().length() < MAX_T_FILE_REASON_LEN){
                    stmt->setString(count++,file.reason());
                } else {
                    stmt->setString(count++,file.reason().substr(0,MAX_T_FILE_REASON_LEN - 1));
                }
            }
            // reason class
            if(file.reasonClass.isSet()){
                stmt->setString(count++,translateErrorCategory(file.reasonClass()));
            }
            // num_failures
            if(file.failures.isSet()){
                stmt->setInt(count++,file.failures());
            }
            // current_failures
            if(file.currentFailures.isSet()){
                stmt->setInt(count++,file.currentFailures());
            }
            // catalog_failures
            if(file.catalogFailures.isSet()){
                stmt->setInt(count++,file.catalogFailures());
            }
            // prestage_failures
            if(file.prestageFailures.isSet()){
                stmt->setInt(count++,file.prestageFailures());
            }
            // filesize
            if(file.fileSize.isSet()){
                stmt->setNumber(count++,ctx.toNumber(file.fileSize()));
            }
            // checksum
            if(file.checksum.isSet()){
                stmt->setString(count++,file.checksum());
            }
            // agent_dn
            if(file.agentDn.isSet()){
                stmt->setString(count++,file.agentDn());
            }
            // finish_time
            if(file.finishTime.isSet()){
                if((time_t)-1 != file.finishTime()){
                    stmt->setTimestamp(count++,ctx.toTimestamp(file.finishTime()));
                } else {
                    stmt->setNull(count++ ,OCCITIMESTAMP);
                }
            }                
            // internal_file_params
            if(file.internalParams.isSet()){
                stmt->setString(count++,file.internalParams());
            }
            // error_scope
            if(file.errorScope.isSet()){
                stmt->setString(count++,translateErrorScope(file.errorScope()));
            }
            // error_phase
            if(file.errorPhase.isSet()){
                stmt->setString(count++,translateErrorPhase(file.errorPhase()));
            }
            
            // Release the Pointer
            stmt_ptr.release();
        } else {
            m_log_trace("Nothing to update");
        }
    } catch(const SQLException& exc){
        m_log_error("Got Exception in creating Update Statement: [0x" << exc.getErrorCode() << "] " << exc.getMessage());
        throw DAOException("Failed to prepare Statement");
    }
    // Don't Check Postcondition (Null is allowed)
    return stmt;
}

/*
 * createUpdate_Channel
 *
 * Prepare Statement For FileDAO::update, Channel View. Returns 0 if job doesn't need to be updated
 */
Statement * FileStatements::createUpdate_Channel(OracleDAOContext& ctx, const model::File& file, const std::string& ch_name) /*throw (DAOLogicError,DAOException)*/ {

    Statement * stmt = 0;        
    try {
        // Update File Properties
        // --------------------------------
        // | Column               | RO/RW |
        // --------------------------------
        // | file_id              |  RO   |
        // | job_id               |  RO   |
        // | file_state           |  RW   |
        // | logical_name         |  RO   |
        // | source_surl          |  RO   |
        // | dest_surl            |  RO   |
        // | reason               |  RW   |
        // | reasonClass          |  RW   |
        // | failures             |  RW   |
        // | current failures     |  RW   |
        // | catalog failures     |  RO   |
        // | prestage failures    |  RO   |
        // | filesize             |  RW   |
        // | checksum             |  RW   |
        // | agent_dn             |  RW   |
        // | finish_time          |  RW   |
        // | internal_file_params |  RW   |
        // | error_scope          |  RW   |
        // | error_phase          |  RW   |
        // --------------------------------
        // Prepare Statement String
        unsigned int count = 3;
        std::stringstream query_stmt;
        query_stmt << "UPDATE ( "
                      "SELECT t_file.* from t_file, t_job WHERE"
                      " t_file.file_id = :1"
                      " AND t_file.job_id = t_job.job_id"
                      " AND t_job.channel_name = :2 ) t_file SET ";
        // State
        if(file.state.isSet()){
            query_stmt << ((3 == count)?"":",") << " t_file.file_state = :" << count;
            ++count;
        }
        // reason
        if(file.reason.isSet()){
            query_stmt << ((3 == count)?"":",") << " t_file.reason = :" << count;
            ++count;
        }
        // reason class
        if(file.reasonClass.isSet()){
            query_stmt << ((3 == count)?"":",") << " t_file.reason_class = :" << count;
            ++count;
        }
        // num_failures
        if(file.failures.isSet()){
            query_stmt << ((3 == count)?"":",") << " t_file.num_failures = :" << count;
            ++count;
        }
        // current_failures
        if(file.currentFailures.isSet()){
            query_stmt << ((3 == count)?"":",") << " t_file.current_failures = :" << count;
            ++count;
        }
        // filesize
        if(file.fileSize.isSet()){
            query_stmt << ((3 == count)?"":",") << " t_file.filesize = :" << count;
            ++count;
        }
        // checksum
        if(file.checksum.isSet()){
            query_stmt << ((3 == count)?"":",") << " t_file.checksum = :" << count;
            ++count;
        }
        // agent_dn
        if(file.agentDn.isSet()){
            query_stmt << ((3 == count)?"":",") << " t_file.agent_dn = :" << count;
            ++count;
        }
        // finish_time
        if(file.finishTime.isSet()){
            query_stmt << ((3 == count)?"":",") << " t_file.finish_time = :" << count;
            ++count;
        }
        // internal_file_params
        if(file.internalParams.isSet()){
            query_stmt << ((3 == count)?"":",") << " t_file.internal_file_params = :" << count;
            ++count;
        }
        // error_scope
        if(file.errorScope.isSet()){
            query_stmt << ((3 == count)?"":",") << " t_file.error_scope = :" << count;
            ++count;
        }
        // error_phase
        if(file.errorPhase.isSet()){
            query_stmt << ((3 == count)?"":",") << " t_file.error_phase = :" << count;
            ++count;
        }
        
        if(count > 3){
            m_log_trace("Preparing Statement: " << query_stmt.str());
            // Create Statements
            stmt = ctx.connection().createStatement(query_stmt.str());
            
            // RAII
            StatementPtr stmt_ptr(ctx.connection(),stmt);
            
            // Set the value of the properties
            stmt->setString(1,file.id());
            stmt->setString(2,ch_name);
            count = 3;
            // State
            if(file.state.isSet()){
                stmt->setString(count++,translateFileState(file.state()));
            }    
            // reason
            if(file.reason.isSet()){
                if(file.reason().length() < MAX_T_FILE_REASON_LEN){
                    stmt->setString(count++,file.reason());
                } else {
                    stmt->setString(count++,file.reason().substr(0,MAX_T_FILE_REASON_LEN - 1));
                }
            }
            // reason class
            if(file.reasonClass.isSet()){
                stmt->setString(count++,translateErrorCategory(file.reasonClass()));
            }
            // num_failures
            if(file.failures.isSet()){
                stmt->setInt(count++,file.failures());
            }
            // current_failures
            if(file.currentFailures.isSet()){
                stmt->setInt(count++,file.currentFailures());
            }
            // filesize
            if(file.fileSize.isSet()){
                stmt->setNumber(count++,ctx.toNumber(file.fileSize()));
            }
            // checksum
            if(file.checksum.isSet()){
                stmt->setString(count++,file.checksum());
            }
            // agent_dn
            if(file.agentDn.isSet()){
                stmt->setString(count++,file.agentDn());
            }
            // finish_time
            if(file.finishTime.isSet()){
                if((time_t)-1 != file.finishTime()){
                    stmt->setTimestamp(count++,ctx.toTimestamp(file.finishTime()));
                } else {
                    stmt->setNull(count++ ,OCCITIMESTAMP);
                }
            }                
            // internal_file_params
            if(file.internalParams.isSet()){
                stmt->setString(count++,file.internalParams());
            }
            // error_scope
            if(file.errorScope.isSet()){
                stmt->setString(count++,translateErrorScope(file.errorScope()));
            }
            // error_phase
            if(file.errorPhase.isSet()){
                stmt->setString(count++,translateErrorPhase(file.errorPhase()));
            }
            
            // Release the Pointer
            stmt_ptr.release();
        } else {
            m_log_trace("Nothing to update");
        }
    } catch(const SQLException& exc){
        m_log_error("Got Exception in creating Update Statement: [0x" << exc.getErrorCode() << "] " << exc.getMessage());
        throw DAOException("Failed to prepare Statement");
    }
    // Don't Check Postcondition (Null is allowed)   
    return stmt;
}

/*
 * createGetByJobId_VO
 *
 * Prepare GetByJobId Statement, VO View
 *
 * Bind Parameters:
 *  :1 - Job ID
 *  :2 - VO Name
 */
Statement * FileStatements::createGetByJobId_VO(OracleDAOContext& ctx) /*throw (DAOLogicError,DAOException)*/ {
    
    // Get Tag
    const char * const tag = GET_BY_JOBID_STATEMENT_VO_TAG;

    Statement * stmt = 0; 
    try {
        // Check if the statement is already cached
        if(false == ctx.connection().isCached("",tag)){
            // Prepare Statement String
            const char * const query_stmt =
                "SELECT t_file.file_id"
                " FROM t_file, t_job"
                " WHERE t_file.job_id = :1"
                " AND t_file.job_id = t_job.job_id"
                " AND t_job.vo_name = :2"
                " AND t_file.job_finished is NULL"
                " AND t_job.job_finished is NULL"
                " ORDER BY t_file.file_id";
            m_log_trace("Preparing Statement: " << query_stmt);
            // Create Statements
            stmt = ctx.connection().createStatement(query_stmt,tag);
        } else {
            // Create Statements
            stmt = ctx.connection().createStatement("",tag);
        }
    } catch(const SQLException& exc){
        m_log_error("Got Exception in Creating GetByJobId Statement: [0x" << exc.getErrorCode() << "] " << exc.getMessage());
        throw DAOException("Failed to prepare Statement");
    }
    // Check Postcondition
    if(0 == stmt){
        throw DAOLogicError("Failed to prepare Statement - Nullpointer");
    }
    return stmt;
}

/*
 * createGetByJobId_Channel
 *
 * Prepare GetByJobId Statement, Channel View
 *
 * Bind Parameters:
 *  :1 - Job ID
 *  :2 - Channel Name
 */
Statement * FileStatements::createGetByJobId_Channel(OracleDAOContext& ctx) /*throw (DAOLogicError,DAOException)*/ {
    
    // Get Tag
    const char * const tag = GET_BY_JOBID_STATEMENT_CH_TAG;

    Statement * stmt = 0; 
    try {
        // Check if the statement is already cached
        if(false == ctx.connection().isCached("",tag)){
            // Prepare Statement String
            const char * const query_stmt =
                "SELECT t_file.file_id"
                " FROM t_file, t_job"
                " WHERE t_file.job_id = :1"
                " AND t_file.job_id = t_job.job_id"
                " AND t_job.channel_name = :2"
                " AND t_file.job_finished is NULL"
                " AND t_job.job_finished is NULL"
                " ORDER BY t_file.file_id";
            m_log_trace("Preparing Statement: " << query_stmt);
            // Create Statements
            stmt = ctx.connection().createStatement(query_stmt,tag);
        } else {
            // Create Statements
            stmt = ctx.connection().createStatement("",tag);
        }
    } catch(const SQLException& exc){
        m_log_error("Got Exception in Creating GetByJobId Statement: [0x" << exc.getErrorCode() << "] " << exc.getMessage());
        throw DAOException("Failed to prepare Statement");
    }
    // Check Postcondition
    if(0 == stmt){
        throw DAOLogicError("Failed to prepare Statement - Nullpointer");
    }
    return stmt;
}

/*
 * createGetWaitingFiles_VO
 *
 * Return the statement for the FileDAO::getWaitingFiles
 *
 * Bind Parameters:
 *  :1 - VO Name
 *  :2 - Limit
 *  :3 - Limit
 */
Statement* FileStatements::createGetWaitingFiles_VO(OracleDAOContext& ctx) /*throw (DAOLogicError,DAOException)*/ {
    
    // Get Tag
    const char * const tag = GET_WAITING_STATEMENT_VO_TAG;

    Statement * stmt = 0; 
    try {
        // Check if the statement is already cached
        if(false == ctx.connection().isCached("",tag)){
            // Prepare Statement String
            const char * const query_stmt =
                "SELECT id FROM("
                    "SELECT t_file.file_id AS id "
                    " FROM t_file, t_job"
                    " WHERE t_file.job_id = t_job.job_id"
                    " AND t_job.vo_name = :1"
                    " AND t_file.file_state = 'Waiting'"
                    " AND t_job.job_state = 'Active'"
                    " AND t_file.job_finished is NULL"
                    " AND t_job.job_finished is NULL"
                    " ORDER BY t_job.priority DESC"
                    " , SYS_EXTRACT_UTC(t_job.submit_time)"
                    " , t_file.job_id"
                    " , t_file.file_id"
                 ") WHERE :2=0 OR ROWNUM<=:3";
            m_log_trace("Preparing Statement: " << query_stmt);
            // Create Statements
            stmt = ctx.connection().createStatement(query_stmt,tag);
        } else {
            // Create Statements
            stmt = ctx.connection().createStatement("",tag);
        }
    } catch(const SQLException& exc){
        m_log_error("Got Exception in Creating GetWaitingFiles Statement: [0x" << exc.getErrorCode() << "] " << exc.getMessage());
        throw DAOException("Failed to prepare Statement");
    }
    // Check Postcondition
    if(0 == stmt){
        throw DAOException("Failed to prepare Statement - Nullpointer");
    }
    return stmt;
}


/*
 * createGetCatalogWaitingFiles_VO
 *
 * Return the statement for the FileDAO::getCatalogWaitingFiles
 *
 * Bind Parameters:
 *  :1 - VO Name
 *  :2 - Limit
 *  :3 - Limit
 */
Statement* FileStatements::createGetCatalogWaitingFiles_VO(OracleDAOContext& ctx) /*throw (DAOLogicError,DAOException)*/ {
    
    // Get Tag
    const char * const tag = GET_CATALOG_WAITING_STATEMENT_VO_TAG;

    Statement * stmt = 0; 
    try {
        // Check if the statement is already cached
        if(false == ctx.connection().isCached("",tag)){
            // Prepare Statement String
            const char * const query_stmt =
                "SELECT fid,jid FROM("
                    "SELECT t_file.file_id AS fid"
                    " , t_file.job_id AS jid"
                    " , DENSE_RANK() OVER (ORDER BY t_job.job_id) As JobCount"
                    " FROM t_file, t_job"
                    " WHERE t_file.job_id = t_job.job_id"
                    " AND t_job.vo_name = :1"
                    " AND t_file.job_finished is NULL"
                    " AND t_job.job_finished is NULL"
                    " AND ((t_file.file_state = 'WaitingCatalogResolution'"
                           " AND t_job.job_state = 'Submitted')"
                      " OR (t_file.file_state = 'WaitingCatalogRegistration'"
                           " AND t_job.job_state = 'Finishing'))"
                    " ORDER BY t_job.priority DESC "
                    " , SYS_EXTRACT_UTC(t_job.submit_time)"
                    " , t_file.job_id"
                    " , t_file.file_id"
                 ") WHERE :2=0 OR JobCount<=:3";
            m_log_trace("Preparing Statement: " << query_stmt);
            // Create Statements
            stmt = ctx.connection().createStatement(query_stmt,tag);
        } else {
            // Create Statements
            stmt = ctx.connection().createStatement("",tag);
        }
    } catch(const SQLException& exc){
        m_log_error("Got Exception in Creating GetCatalogWaitingFiles Statement: [0x" << exc.getErrorCode() << "] " << exc.getMessage());
        throw DAOException("Failed to prepare Statement");
    }
    // Check Postcondition
    if(0 == stmt){
        throw DAOException("Failed to prepare Statement - Nullpointer");
    }
    return stmt;
}

/*
 * createGetNextToTransfer_Channel
 *
 * Return the statement for the FileDAO::getNextToTransfer
 *
 * Bind Parameters:
 *  :1 - Channel Name
 *  :2 - VO Name
 *  :3 - Limit
 *  :4 - Limit
 */
Statement* FileStatements::createGetNextJobToTransfer_Channel(OracleDAOContext& ctx) /*throw (DAOLogicError,DAOException)*/ {
    
    // Get Tag
    const char * const tag = GET_JOB_TO_TX_STATEMENT_CH_TAG;

    Statement * stmt = 0; 
    try {
        // Check if the statement is already cached
        if(false == ctx.connection().isCached("",tag)){
            // Prepare Statement String
            const char * const query_stmt =
                "SELECT f_id, j_id FROM ("
                    "SELECT t_file.file_id AS f_id, t_file.job_id AS j_id"
                    ", ROW_NUMBER() OVER (ORDER BY t_file.file_id ) FileCount"
                    " FROM t_file"
                    " WHERE t_file.job_id = ( "
                        "SELECT j_id FROM ( "
                            "SELECT t_job.job_id AS j_id"
                            ", ROW_NUMBER() OVER (ORDER BY t_job.priority DESC"
                                " , SYS_EXTRACT_UTC(t_job.submit_time)"
                                " , t_job.job_id) JobCount"
                            " FROM t_job"
                            " WHERE t_job.channel_name = ("
                                " SELECT t_channel.channel_name"
                                " FROM t_channel"
                                " WHERE t_channel.channel_name = :1"
                                " AND t_channel.channel_state IN ('Active','Drain')"
                                " ) "
                            " AND t_job.vo_name = :2"
                            " AND t_job.job_state IN ('Ready','Active')"
                            " AND ((t_job.cancel_job IS NULL)"
                              " OR (t_job.cancel_job = 'N'))"
                            " AND t_job.job_finished is NULL"
                            " AND EXISTS ("
                                " SELECT NULL FROM t_file"
                                " WHERE t_file.job_id = t_job.job_id"
                                " AND t_file.file_state = 'Ready'"
                            ")"
                        ") WHERE JobCount = 1"
                    ") AND t_file.file_state = 'Ready'"
                    " AND t_file.job_finished is NULL"
                ") WHERE :3=0 OR FileCount<=:4";
            m_log_trace("Preparing Statement: " << query_stmt);
            // Create Statements
            stmt = ctx.connection().createStatement(query_stmt,tag);
        } else {
            // Create Statements
            stmt = ctx.connection().createStatement("",tag);
        }
    } catch(const SQLException& exc){
        m_log_error("Got Exception in Creating GetNextToTransfer Statement: [0x" << exc.getErrorCode() << "] " << exc.getMessage());
        throw DAOException("Failed to prepare Statement");
    }
    // Check Postcondition
    if(0 == stmt){
        throw DAOException("Failed to prepare Statement - Nullpointer");
    }
    return stmt;    
}

/*
 * GetJobsToTransferPerSE
 *
 * Return the statement for the FileDAO::getJobsToTransferPerSE
 *
 * Bind Parameters:
 *  :1 - Channel Name
 *  :2 - VO Name
 *  :3 - Limit
 *  :4 - Limit
 *  :5 - SE Limit
 *  :6 - SE Limit
 */
Statement* FileStatements::createGetJobsToTransferPerSE_Channel(OracleDAOContext& ctx) /*throw (DAOLogicError,DAOException)*/ {
    
    // Get Tag
    const char * const tag = GET_JOBS_TO_TX_PER_SE_STATEMENT_CH_TAG;
#warning "TODO : add job_finished conditions"
    Statement * stmt = 0; 
    try {
        // Check if the statement is already cached
        if(false == ctx.connection().isCached("",tag)){
            // Prepare Statement String
            const char * const query_stmt =
            "SELECT f_id,j_id FROM (" 
             " SELECT t_file.file_id AS f_id, t_file.job_id AS j_id"
             " , ROW_NUMBER() OVER (PARTITION BY t_file.job_id ORDER BY t_file.file_id ) FileCount"
             " FROM t_file" 
             " WHERE t_file.job_id IN ("
              " SELECT j_id FROM ("
               " SELECT j_id" 
                " , JobCount"
                " , ROW_NUMBER() OVER (ORDER BY j_p DESC, j_st) NumJobs"
               " FROM ("
                " SELECT t_job.job_id AS j_id"
                 " , t_job.source_se AS src_se"
                 " , t_job.dest_se AS dst_se"
                 " , t_job.priority AS j_p"
                 " , t_job.submit_time AS j_st"
                 " , ROW_NUMBER() OVER (PARTITION BY t_job.source_se"
                    " ,  t_job.dest_se ORDER BY t_job.priority DESC"
                    " , SYS_EXTRACT_UTC(t_job.submit_time)"
                    " , t_job.job_id) JobCount"
                 " FROM t_job"
                 " WHERE t_job.channel_name = ("
                  " SELECT t_channel.channel_name"
                  " FROM t_channel"
                  " WHERE t_channel.channel_name = :1"
                  " AND t_channel.channel_state IN ('Active','Drain')"
                 ")"
                 " AND t_job.vo_name = :2"
                 " AND t_job.job_state IN ('Ready','Active')"
                 " AND ((t_job.cancel_job IS NULL)"
                  " OR (t_job.cancel_job = 'N'))"
                 " AND EXISTS ("
                  " SELECT NULL FROM t_file"
                  " WHERE t_file.job_id = t_job.job_id"
                  " AND t_file.file_state = 'Ready'"
                 ")"
                ") WHERE JobCount = 1"
               ") WHERE (:3=0 OR NumJobs<=:4)"
              ") AND t_file.file_state = 'Ready'"
             ") WHERE (:5=0 OR FileCount<=:6)";
            m_log_trace("Preparing Statement: " << query_stmt);
            // Create Statements
            stmt = ctx.connection().createStatement(query_stmt,tag);
        } else {
            // Create Statements
            stmt = ctx.connection().createStatement("",tag);
        }
    } catch(const SQLException& exc){
        m_log_error("Got Exception in Creating GetJobsToTransferPerSE Statement: [0x" << exc.getErrorCode() << "] " << exc.getMessage());
        throw DAOException("Failed to prepare Statement");
    }
    // Check Postcondition
    if(0 == stmt){
        throw DAOException("Failed to prepare Statement - Nullpointer");
    }
    return stmt;    
}

/*
 * createGetFilesToCancel_Channel
 *
 * Return the statement for the FileDAO::getFilesToCancel
 *
 * Bind Parameters:
 *  :1 - VO Name
 *  :2 - Limit
 *  :3 - Limit
 */
Statement* FileStatements::createGetFilesToCancel_Channel(OracleDAOContext& ctx) /*throw (DAOLogicError,DAOException)*/ {
    
    // Get Tag
    const char * const tag = GET_FILES_TO_CANCEL_STATEMENT_CH_TAG;

    Statement * stmt = 0; 
    try {
        // Check if the statement is already cached
        if(false == ctx.connection().isCached("",tag)){
            // Prepare Statement String
            const char * const query_stmt =
                "SELECT fid, jid FROM ("
                    "SELECT t_file.file_id AS fid, t_job.job_id AS jid"
                    " , ROW_NUMBER() over (ORDER BY t_job.priority DESC"
                        " , SYS_EXTRACT_UTC(t_job.submit_time)"
                        " , t_job.job_id"
                        " , t_file.file_id ) AS FileCount"
                    " FROM t_file , t_job"
                    " WHERE t_file.job_id = t_job.job_id"
                    " AND t_job.channel_name = :1"
                    " AND t_job.job_state IN ('Ready','Active')"
                    " AND t_file.file_state IN ('Ready','Done','Hold')"
                    " AND t_job.cancel_job = 'Y'"
                    " AND t_file.job_finished is NULL"
                    " AND t_job.job_finished is NULL"
                ") WHERE :2=0 OR FileCount<=:3";
            m_log_trace("Preparing Statement: " << query_stmt);
            // Create Statements
            stmt = ctx.connection().createStatement(query_stmt,tag);
        } else {
            // Create Statements
            stmt = ctx.connection().createStatement("",tag);
        }
    } catch(const SQLException& exc){
        m_log_error("Exception creating GetFilesToCancel Statement: [0x" << exc.getErrorCode() << "] " << exc.getMessage());
        throw DAOException("Failed to prepare Statement");
    }
    // Check Postcondition
    if(0 == stmt){
        throw DAOException("Failed to prepare Statement - Nullpointer");
    }
    return stmt;
}
