/*
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *  GLite Data Transfer Agents: OracleJobStatements Class
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *  Version info: $Id: JobStatements.h,v 1.10 2007/06/22 09:26:47 ptedesco Exp $
 *  Release: $Name:  $
 *
 */
#ifndef GLITE_DATA_TRANSFER_AGENTS_DAO_ORACLE_JOBSTATEMENTS_H_
#define GLITE_DATA_TRANSFER_AGENTS_DAO_ORACLE_JOBSTATEMENTS_H_

#include "glite/data/transfer/agent/model/Job.h"
#include "glite/data/agents/dao/DAOException.h"
#include "glite/data/agents/dao/oracle/OracleDAOContext.h"
#include "Statements.h"

#include <string>
#include <vector>
#include <boost/utility.hpp>

namespace glite   {
namespace data    {
namespace transfer{
namespace agent   {
namespace dao     {
namespace oracle  {

/**
 * Oracle Data Access Object For the model::Job class
 */
class JobStatements : boost::noncopyable, public Statements {
public:

    /**
     * Return Singleton instance 
     * @return Singleton instance 
     */
    static JobStatements& instance() /*throw ()*/ ;

    // -------------------------------------------------------------------------
    // Methods
    // -------------------------------------------------------------------------
    
    /*
     * dispose
     *
     * dispose Statements for OracleDAOContext
     */
    void dispose(agents::dao::oracle::OracleDAOContext& ctx) /*throw ()*/;
        
    /**
     * Return the statement for the JobDAO::get, VO View
     *
     * Bind Parameters:
     *  :1 - Job ID
     *  :2 - VO Name
     */
    ::oracle::occi::Statement* createGet_VO(
        agents::dao::oracle::OracleDAOContext&  ctx,
        const bool&                             finished = false) /*throw (DAOLogicError,DAOException)*/;
    
    /**
     * Return the statement for the JobDAO::get, Channel View
     *
     * Bind Parameters:
     *  :1 - Job ID
     *  :2 - Channel Name
     */
    ::oracle::occi::Statement* createGet_Channel(
        agents::dao::oracle::OracleDAOContext&  ctx,
        const bool&                             finished = false) /*throw (DAOLogicError,DAOException)*/;

    /**
     * Return the statement for the JobDAO::getLast, Cred View
     *
     * Bind Parameters:
     *  :1 - Cred ID
     *  :2 - User DN
     */
    ::oracle::occi::Statement* createGetLast_Cred(agents::dao::oracle::OracleDAOContext& ctx) /*throw (DAOLogicError,DAOException)*/;

    /**
     * Return the statement for the JobDAO::update, VO View
     */
    ::oracle::occi::Statement* createUpdate_VO(agents::dao::oracle::OracleDAOContext& ctx, const model::Job& job, const std::string& vo_name) /*throw (DAOLogicError,DAOException)*/;

    /**
     * Return the statement for the JobDAO::update, Channel View
     */
    ::oracle::occi::Statement* createUpdate_Channel(agents::dao::oracle::OracleDAOContext& ctx, const model::Job& job, const std::string& ch_name) /*throw (DAOLogicError,DAOException)*/;

    /**
     * Return the statement for the JobDAO::getSubmittedJobs, VO View
     *
     * Bind Parameters:
     *  :1 - VO Name
     *  :2 - Limit
     *  :3 - Limit
     */
    ::oracle::occi::Statement* createGetSubmittedJobs_VO(agents::dao::oracle::OracleDAOContext& ctx) /*throw (DAOLogicError,DAOException)*/;

    /**
     * Return the statement for the JobDAO::getPendingJobs, VO View
     *
     * Bind Parameters:
     *  :1 - VO Name
     *  :2 - Limit
     *  :3 - Limit
     */
    ::oracle::occi::Statement* createGetPendingJobs_VO(agents::dao::oracle::OracleDAOContext& ctx) /*throw (DAOLogicError,DAOException)*/;

    /**
     * Return the statement for the JobDAO::getDoneJobs, VO View
     *
     * Bind Parameters:
     *  :1 - VO Name
     *  :2 - Limit
     *  :3 - Limit
     */
    ::oracle::occi::Statement* createGetDoneJobs_VO(agents::dao::oracle::OracleDAOContext& ctx) /*throw (DAOLogicError,DAOException)*/;

    /**
     * Return the statement for the JobDAO::getFinishingJobs, VO View
     *
     * Bind Parameters:
     *  :1 - VO Name
     *  :2 - VO Name
     *  :3 - Limit
     *  :4 - Limit
     */
    ::oracle::occi::Statement* createGetFinishingJobs_VO(agents::dao::oracle::OracleDAOContext& ctx) /*throw (DAOLogicError,DAOException)*/;

    /**
     * Return the state of all the files belonging to a given job, VO View
     *
     * Bind Parameters:
     *  :1 - Job ID
     *  :2 - VO Name
     */
    ::oracle::occi::Statement* createGetFileStates_VO(agents::dao::oracle::OracleDAOContext& ctx) /*throw (DAOLogicError,DAOException)*/;

    /**
     * Return the state of all the files belonging to a given job, Channel View
     *
     * Bind Parameters:
     *  :1 - Job ID
     *  :2 - Channel Name
     */
    ::oracle::occi::Statement* createGetFileStates_Channel(agents::dao::oracle::OracleDAOContext& ctx) /*throw (DAOLogicError,DAOException)*/;

    /**
     * Return the statement for the getJobsToFix, Channel View
     *
     * Bind Parameters:
     *  :1 - Channel Name
     *  :2 - Limit
     *  :3 - Limit
     */
    ::oracle::occi::Statement* createGetJobsToFix_Channel(agents::dao::oracle::OracleDAOContext& ctx) /*throw (DAOLogicError,DAOException)*/;

private:

    // -------------------------------------------------------------------------
    // Constructor & Destructor
    // -------------------------------------------------------------------------

    /**
     * Constructor
     */
    JobStatements();

    /**
     * Destructor
     */
    ~JobStatements(){}

};


} // End namespace oracle
} // End namespace dao
} // End namespace agent
} // End namespace transfer
} // End namespace data
} // End namespace glite

#endif //GLITE_DATA_TRANSFER_AGENTS_DAO_ORACLE_JOBSTATEMENTS_H_
