/**
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *  GLite Data Transfer Agents: OracleAgentDAO Class
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *           Paolo Tedesco <paolo.tedesco@cern.ch>
 *  Version info: $Id: OracleAgentDAO.cpp,v 1.8 2008/11/13 10:43:56 ptedesco Exp $
 *  Release: $Name:  $
 *
 */
#include "OracleAgentDAO.h"
#include "AgentStatements.h"
#include "dao-utility.h"
#include <glite/data/agents/dao/oracle/StatementPtr.h>
#include <glite/data/agents/dao/oracle/ResultSetPtr.h>

#include <memory>

using namespace glite::data::transfer::agent;
using namespace glite::data::transfer::agent::dao;
using namespace glite::data::transfer::agent::dao::oracle;
using namespace ::oracle::occi;
using glite::data::agents::dao::oracle::StatementPtr;
using glite::data::agents::dao::oracle::ResultSetPtr;
using glite::data::agents::dao::DAOException;
using glite::data::agents::dao::DAOLogicError;

/*
 * OracleAgentDAO
 *
 * Constructor
 */
OracleAgentDAO::OracleAgentDAO(OracleDAOContext& ctx, const std::string& agent_name, const std::string& category):
        OracleDAO(ctx,category),
        m_agentName(agent_name){
}

/*
 * get
 *
 * Return the Agent with the given Name
 */
model::Agent * OracleAgentDAO::get(bool lock) /*throw (DAOLogicError, DAOException)*/{
    
    m_log_trace("Calling OracleAgentDAO::get");

    std::auto_ptr<model::Agent> agent;
    
    // Get Statement
    StatementPtr stmt(ctx().connection(),AgentStatements::instance().createGet(ctx(),lock));
    try{
        // Set Parameters
        stmt->setString(1,m_agentName);
        
        // Execute Query
        ResultSetPtr rs(*stmt,stmt->executeQuery());
        if(0 != rs->next()){
            model::Agent::State ag_state = translateAgentState(rs->getString(2).c_str());
            // Create Transfer instance
            agent.reset(new model::Agent(
                        rs->getString(1),                // agent_type
                        ag_state,                        // agent_state
                        rs->getString(3),                // agent_version
                        rs->getString(4),                // agent_host
                        rs->getString(5),                // contact
                        toTimeT(rs->getTimestamp(6))));  // last_active
        } else {
            throw DAOException("No Agent found for the given Name");
        }
    } catch(const SQLException& exc){
        m_log_error("Got SQLException in Executing Get: [0x" << exc.getErrorCode() << "] " << exc.getMessage());
        m_log_debug("Query was: " << stmt->getSQL());
        throw DAOException("Failed to get Agent - SQL Exception");
    }
    
    m_log_trace("OracleAgentDAO::get Called");
    
    return agent.release();
}

/*
 * update
 *
 * Update a Agent object
 */
void OracleAgentDAO::update(const model::Agent& obj) /*throw (DAOLogicError,DAOException)*/{
        
    m_log_trace("Calling OracleAgentDAO::update");

    // Get Statement
    StatementPtr stmt(ctx().connection(),AgentStatements::instance().createUpdate(ctx(),obj,m_agentName));
    try {
        // Don't need to set Parameters
        if(stmt.get() != 0){
            // Execute Update
            unsigned int res = stmt->executeUpdate();
            m_log_trace(res << " row(s) affected");
            m_log_trace("Query was: " << stmt->getSQL());
            m_log_trace("State is : " << translateAgentState(obj.state()));
            m_log_trace("Name is : " << m_agentName);
            if(res <= 0){
                throw DAOException("Agent instance hasn't been updated");
            }
        }
    } catch(const SQLException& exc){
        m_log_error("Got SQLException in Executing Update: [0x" << exc.getErrorCode() << "] " << exc.getMessage());
        m_log_debug("Query was: " << stmt->getSQL());
        throw DAOException("Failed to Update Agent - SQL Exception");
    }
    m_log_trace("Agent instance updated");
}

/*
 * create
 *
 * Insert a new Agent Entry
 */
void OracleAgentDAO::create(const model::Agent& obj) /*throw (DAOException)*/{

    m_log_trace("Calling OracleAgentDAO::create");

    // Get Statement
    StatementPtr stmt(ctx().connection(),AgentStatements::instance().createCreate(ctx(),obj,m_agentName));
    try {
        // Execute Update
        unsigned int res = stmt->executeUpdate();
        m_log_trace(res << " row(s) affected");
        if(res <= 0){
            throw DAOException("Agent instance hasn't been created");
        }
    } catch(const SQLException& exc){
        m_log_error("Got SQLException in Executing Insert: [0x" << exc.getErrorCode() << "] " << exc.getMessage());
        m_log_debug("Query was: " << stmt->getSQL());
        throw DAOException("Failed to Create Agent - SQL Exception");
    }
    m_log_trace("Agent instance created");
}
