/**
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *  GLite Data Transfer Agents: OracleChannelDAO Class
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *           Paolo Tedesco <paolo.tedesco@cern.ch>
 *  Version info: $Id: OracleChannelDAO.cpp,v 1.14 2008/12/04 08:53:30 ptedesco Exp $
 *  Release: $Name:  $
 *
 */
#include "OracleChannelDAO.h"
#include "ChannelStatements.h"
#include "dao-utility.h"
#include <glite/data/agents/dao/oracle/StatementPtr.h>
#include <glite/data/agents/dao/oracle/ResultSetPtr.h>
#include <glite/data/agents/dao/DAOContextFactory.h>

#include <memory>

using namespace glite::data::transfer::agent;
using namespace glite::data::transfer::agent::dao;
using namespace glite::data::transfer::agent::dao::oracle;
using namespace ::oracle::occi;
using glite::data::agents::dao::oracle::ResultSetPtr;
using glite::data::agents::dao::oracle::StatementPtr;
using glite::data::agents::dao::DAOException;
using glite::data::agents::dao::DAOLogicError;

namespace {
    
    /**
     * get a channel timeout value
     * @param rs [IN]
     * @param index [IN]
     * @return timeout value
     */
    int get_timeout(const ResultSetPtr& rs,unsigned int index){
        int ret = rs->getInt(index);
        // note : the doc I've found is rather confusing, it looks like you have 
        // to call a getXXX before calling isNull on that column, so I'm calling
        // isNull after getInt.
        if (rs->isNull(index) || ret < 0){
            ret = -1;
        }
        return ret;
    }
    
    /**
     * get a double value
     * @param rs [IN]
     * @param index [IN]
     * @param default_value [IN]
     * @return value if non null, default_value otherwise
     */
    double get_double(const ResultSetPtr& rs,unsigned int index,double default_value){
        double ret = rs->getDouble(index);
        if (rs->isNull(index)){
            ret = default_value;
        }
        return ret;
    }
    
} // namespace

/*
 * OracleChannelDAO
 *
 * Constructor
 */
OracleChannelDAO::OracleChannelDAO(OracleDAOContext& ctx, const std::string& category):
        OracleDAO(ctx,category){
}

/*
 * get
 *
 * Return the Channel with the given Channel 
 */
model::Channel * OracleChannelDAO::get(Statement& stmt) /*throw (DAOLogicError,DAOException)*/{

    m_log_trace("Calling OracleChannelDAO::get");

    std::auto_ptr<model::Channel> channel;
    
    try{
        // Execute Query
        ResultSetPtr rs(stmt,stmt.executeQuery());
        if(0 != rs->next()){
            
            model::Channel::State ch_state = translateChannelState(rs->getString(2).c_str());
            model::Channel::Type  ch_type  = translateChannelType(rs->getString(16).c_str());
            model::Channel::LogLevel ch_loglevel = translateChannelLogLevel(rs->getString(19).c_str());
            model::Channel::SrmCopyDirection ch_scp_direction = translateChannelSrmCopyDirection(rs->getString(26).c_str());
            
            // Create Channel instance
            channel.reset(new model::Channel(
                rs->getString(1),               // channel_name
                ch_state,                       // channel_state
                rs->getString(3),               // source_site
                rs->getString(4),               // dest_site
                rs->getInt(5),                  // num_of_files
                rs->getInt(6),                  // num_of_streams
                rs->getString(7),               // tcp_bs
                rs->getDouble(8),               // bandwidth
                rs->getDouble(9),               // nominal_throughput
                rs->getString(10),              // contact
                rs->getString(11),              // message
                rs->getString(12),              // admin
                toTimeT(rs->getTimestamp(13)),  // last_active
                toTimeT(rs->getTimestamp(14)),  // last_modification
                rs->getInt(15),                 // se_limit
                ch_type,                        // channel_type
                // new in 3.1.0
                rs->getString(17),              // blocksize
                get_timeout(rs,18),             // httpTimeout
                ch_loglevel,                    // txLoglevel
                get_timeout(rs,20),             // urlcopyPutTimeout
                get_timeout(rs,21),             // urlcopyPutdoneTimeout
                get_timeout(rs,22),             // urlcopyGetTimeout
                get_timeout(rs,23),             // urlcopyGetdoneTimeout
                get_timeout(rs,24),             // urlcopyTxTimeout
                get_timeout(rs,25),             // urlcopyTxmarksTimeout
                ch_scp_direction,               // srmcopyDirection
                get_timeout(rs,27),             // srmcopyTimeout
                get_timeout(rs,28),             // srmcopyRefreshTimeout
                // new in 3.2.0
                toBoolean(rs->getString(29),true),  // targetDirCheck - fix bug #39880
                get_timeout(rs,30),             // urlcopyFirstTxmarkTimeout
                // new in 3.3.0
                rs->getDouble(31),              // txTimeoutPerMb
                get_timeout(rs,32),             // noTransferActivityTimeout
#warning "define the default"                
                get_double(rs,33,2),            // preparingFilesRatio
                rs->getString(34)               // transferType
            ));
        } else {
            throw DAOException("No Channel found for the given Id");
        }
    } catch(const SQLException& exc){
        m_log_error("Got SQLException in Executing Get: [0x" << exc.getErrorCode() << "] " << exc.getMessage());
        throw DAOException("Failed to get Channel - SQL Exception");
    } catch(const DAOException& exc){
        m_log_error("Got DAOException in Executing Get: " << exc.what());
        throw;
    } 

    m_log_trace("OracleChannelDAO::get Called");
    
    return channel.release();
}

/*
 * update
 *
 * Update a TransferChannel object
 */
void OracleChannelDAO::update(Statement& stmt) /*throw (DAOLogicError,DAOException)*/{
    
    m_log_trace("Calling OracleChannelDAO::update");
    
    try {
        // Execute Update
        unsigned int res = stmt.executeUpdate();
        m_log_trace(res << " row(s) affected");
        if(res <= 0){
            throw DAOException("Channel instance hasn't been updated");
        }
    } catch(const SQLException& exc){
        m_log_error("Got SQLException in Executing Update: [0x" << exc.getErrorCode() << "] " << exc.getMessage());
        throw DAOException("Failed To Update Channel - SQL Exception");
    }
    m_log_trace("Channel instance updated");
}

/*
 * getSiteGroups
 * 
 * Return the list of groups which a site belong to
 */
void OracleChannelDAO::getSiteGroups(const std::string& site, agents::dao::StringArray& groups) /*throw (DAOLogicError,DAOException)*/{
    
    m_log_trace("Calling OracleChannelDAO::getSiteGroups");
    
    // Get Context
    OracleDAOContext& ctx = this->ctx();
    
    // Get Statement
    StatementPtr stmt(ctx.connection(),ChannelStatements::instance().createGetSiteGroups(ctx));
    try{
        // Set Parameters
        stmt->setString(1,site);
        
        // Execute Query
        ResultSetPtr rs(*stmt,stmt->executeQuery());
        while(rs->next()){
            // Get group name
            groups.push_back(rs->getString(1));
        }
    } catch(const SQLException& exc){
        m_log_error("Got SQLException in Executing getSiteGroups: [0x" << exc.getErrorCode() << "] " << exc.getMessage());
        throw DAOException("Failed to Site's Groups - SQL Exception");
    }
    
    m_log_trace("Channel_ChannelDAO::getSiteGroups returned [" << groups.size() << "] entries");
    
}
