/**
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *  GLite Data Transfer Agents: OracleDAOConfig Class
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *           Paolo Tedesco <paolo.tedesco@cern.ch>
 *  Version info: $Id: OracleDAOConfig.cpp,v 1.15 2009/01/08 13:50:39 ptedesco Exp $
 *  Release: $Name:  $
 *
 */
#include "OracleDAOConfig.h"
#include "VO_DAOFactory.h"
#include "Channel_DAOFactory.h"
#include "Cred_DAOFactory.h"
#include <glite/data/config/service/ParamReader.hxx>
#include <glite/data/agents/AgentExceptions.h>

#include <boost/scoped_ptr.hpp>

using namespace glite::data::transfer::agent;
using namespace glite::data::transfer::agent::dao::oracle;
using namespace glite::config;
using glite::data::agents::LogicError;
using glite::data::agents::ConfigurationException;
using glite::data::agents::ConfigParamException;
using boost::scoped_ptr;

/*
 * Constants
 */
namespace {
const char * const ORACLE_DAO_NAME                = "transfer-agent-dao-oracle";
                                                                                                                          
// Properties Names
const char * const VOVIEW_PROPERTY_NAME           = "VOView";
const char * const CHANNELVIEW_PROPERTY_NAME      = "ChannelView";
const char * const CREDVIEW_PROPERTY_NAME         = "CredentialsView";

// Properties Default  Values
bool         DEFAULT_VOVIEW                 = true;
bool         DEFAULT_CHANNELVIEW            = true;
bool         DEFAULT_CREDVIEW               = true;

}

extern "C" {
/*
 * create_glite_component
 *
 * Create Component instance
 */
ComponentConfiguration * create_glite_component(){
    return new OracleDAOConfig();
}
                                                                                
/*
 * destroy_glite_component
 *
 * Destroy Component instance
 */
void destroy_glite_component(ComponentConfiguration * component){
    OracleDAOConfig * f = dynamic_cast<OracleDAOConfig *>(component); 
    if(0 != f){
        delete f;
    }
}
                                                                                
} // End extern "C"

/*
 * OracleDAOConfig
 *
 * Constructor
 */
OracleDAOConfig::OracleDAOConfig():
    ComponentConfiguration(ORACLE_DAO_NAME), 
    m_logger(ORACLE_DAO_NAME){
}

/*
 * ~OracleDAOConfig
 *
 * Destructor
 */
OracleDAOConfig::~OracleDAOConfig(){
}

/*
 * init
 *
 * Initialize the Component
 */
int OracleDAOConfig::init(const Params& params){
                                                                                                                          
    try{
        bool vo_view_enabled   = DEFAULT_VOVIEW;
        bool ch_view_enabled   = DEFAULT_CHANNELVIEW;
        bool cred_view_enabled = DEFAULT_CREDVIEW;
        
        ParamReader reader(getName(),params);
        // Get Vo View
        reader.getParameter(VOVIEW_PROPERTY_NAME,vo_view_enabled);
        // Get Channel View
        reader.getParameter(CHANNELVIEW_PROPERTY_NAME,ch_view_enabled);
        // Get Cred View
        reader.getParameter(CREDVIEW_PROPERTY_NAME,cred_view_enabled);

        if(true == vo_view_enabled){
            // Create VO View DAO Factory
            m_voDAOFactory.reset(new VO_DAOFactory());
        }
        if(true == ch_view_enabled){
            // Create Channel View DAO Factory
            m_channelDAOFactory.reset(new Channel_DAOFactory());
        }
        if(true == cred_view_enabled){
            // Create Cred View DAO Factory
            m_credDAOFactory.reset(new Cred_DAOFactory());
        }
    
        // Log Configuration Values
        m_log_info(getName() << " Initialized. Initialization Parameters are");
        m_log_info("VO View             : " << str_enabled(vo_view_enabled));
        m_log_info("Channel View        : " << str_enabled(ch_view_enabled));
        m_log_info("Credentials View    : " << str_enabled(cred_view_enabled));
    } catch(const LogicError& exc) {
        m_log_error(getName() << " configuration Logic Error: " << exc.what());
        return -1;
    } catch(const std::exception& exc) {
        m_log_error(getName() << " configuration Unexpected Exception: " << exc.what());
        return -1;
    } catch(...) {
        m_log_error(getName() << " configuration Unhandled Exception");
        return -1;
    }
    return 0;
}

/*
 * config
 *
 * Configure the Component
 */
int OracleDAOConfig::config(const Params& params){
    
    m_log_info(getName() << " Configured");
    return 0;
}

/*
 * fini
 *
 * Finalize the Component
 */
int OracleDAOConfig::fini(){
    
    // Delete VO View DAO Factory
    m_voDAOFactory.reset();
    // Delete Channel View DAO Factory
    m_channelDAOFactory.reset();
    // Delete Cred View DAO Factory
    m_credDAOFactory.reset();
    return 0;
}
