/*
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *  GLite Data Transfer Agents: OracleFileDAO Class
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>, Paolo Tedesco <paolo.tedesco@cern.ch>
 *  Version info: $Id: OracleFileDAO.cpp,v 1.10 2008/11/13 10:43:56 ptedesco Exp $
 *  Release: $Name:  $
 *
 */
#include "OracleFileDAO.h"
#include "dao-utility.h"
#include <glite/data/agents/dao/oracle/ResultSetPtr.h>

#include <memory>

using namespace glite::data::transfer::agent;
using namespace glite::data::transfer::agent::dao;
using namespace glite::data::transfer::agent::dao::oracle;
using namespace ::oracle::occi;
using glite::data::agents::dao::oracle::ResultSetPtr;
using glite::data::agents::dao::DAOException;
using glite::data::agents::dao::DAOLogicError;
/*
 * OracleFileDAO
 *
 * Constructor
 */
OracleFileDAO::OracleFileDAO(OracleDAOContext& ctx, const std::string& category):
        OracleDAO(ctx,category){
}

/*
 * get
 *
 * Return the File instance with the given FileId
 */
model::File * OracleFileDAO::get(Statement& stmt) /*throw (DAOLogicError,DAOException)*/{
    
    std::auto_ptr<model::File> file;

    m_log_trace("Calling OracleFileDAO::get");
    try{
        // Execute Query
        ResultSetPtr rs(stmt,stmt.executeQuery());
        if(0 != rs->next()){
            // Do not do it inside the constructor to avoid possible leaks
            model::File::State file_state = translateFileState(rs->getString(3).c_str());
            model::Error::Category err_class = translateErrorCategory(rs->getString(7).c_str());
            model::Error::Scope err_scope = translateErrorScope(rs->getString(18).c_str());
            model::Error::Phase err_phase = translateErrorPhase(rs->getString(19).c_str());
            // Create File instance
            file.reset(new model::File(rs->getString(1),    // id
                     rs->getString(2),                      // job_id
                     file_state,                            // state
                     rs->getString(4),                      // logical
                     rs->getString(5),                      // source_surl
                     rs->getString(6),                      // dest_surl
                     err_class,                             // reason_class
                     rs->getString(8),                      // reason
                     rs->getInt(9),                         // num_failures
                     rs->getInt(10),                        // current_failures
                     rs->getInt(11),                        // catalog_failures
                     rs->getInt(12),                        // prestage_failures
                     toLongLong(rs->getNumber(13)),         // file_size
                     rs->getString(14),                     // checksum
                     toTimeT(rs->getTimestamp(15)),         // finish_time
                     rs->getString(16),                     // agent_dn
                     rs->getString(17),                     // internal_file_params
                     // 3.1.0
                     err_scope,                             // error scope
                     err_phase                              // error phase
                     ));
        } else {
            throw DAOException("No File found for the given Id");
        }
    } catch(const SQLException& exc){
        m_log_error("Got SQLException in Retrieving File: [0x" << exc.getErrorCode() << "] " << exc.getMessage());
        throw DAOException("Failed To Get File - SQL Exception");
    }
    m_log_trace("OracleFileDAO::get Called");
    
    return file.release();
}

/*
 * update
 *
 * Update a File object
 */
void OracleFileDAO::update(Statement& stmt) /*throw (DAOLogicError,DAOException)*/{
    
    m_log_trace("Calling OracleFileDAO::update");
    
    try {
        // Execute Update
        unsigned int res = stmt.executeUpdate();
        m_log_trace(res << " row(s) affected");
        if(res <= 0){
            throw DAOException("File instance hasn't been updated");
        }
    } catch(const SQLException& exc){
        m_log_error("Got SQLException in Executing Update: [0x" << exc.getErrorCode() << "] " << exc.getMessage());
        throw DAOException("Failed To Update File - SQL Exception");
    }
    m_log_trace("File instance updated");
}

/*
 * getFileIds
 *
 * Return the File Id List that correspond to the given statement
 */
void OracleFileDAO::getFileIds(Statement& stmt, agents::dao::StringArray& ids) /*throw (DAOLogicError,DAOException)*/{
    
    m_log_trace("Calling OracleFileDAO::getFileIds");
    
    try {
        // Execute Query
        ResultSetPtr rs(stmt,stmt.executeQuery());
        
        // For Each entry on the Result Set
        while(rs->next()){
            // Get Data
            ids.push_back(rs->getString(1));
        } // End while
    } catch(const SQLException& exc){
        m_log_error("Got SQLException in Executing getFileIds: [0x" << exc.getErrorCode() << "] " << exc.getMessage());
        throw DAOException("Failed To Get File Ids - SQL Exception");
    } catch(const DAOException& exc){
        m_log_error("Got DAOException in Executing getFileIds: " << exc.what());
        throw;
    }
    m_log_trace("OracleFileDAO::getFileIds Completed");
}

/*
 * getFilesAndJobIds
 *
 * Return the FileTransfer Per Job List that correspond to the given result set
 */
void OracleFileDAO::getFilesAndJobIds(
    Statement& stmt, 
    std::string& job_id, 
    agents::dao::StringArray& file_ids) /*throw (DAOLogicError,DAOException)*/{

    m_log_trace("Calling OracleFileDAO::getFilesAndJobIds");

    // Clean list
    job_id.clear();
    file_ids.clear();
    
    try {
        // Execute Query
        ResultSetPtr rs(stmt,stmt.executeQuery());

        // For Each entry on the Result Set
        while(rs->next()){
            // Get Job Id
            if(job_id.empty()){
                job_id = rs->getString(2);
            } else {
                if(job_id != rs->getString(2)){
                    m_log_error("File <" << rs->getString(1) << "> belongs to Job [" << rs->getString(2) << "], while <" << job_id << "> was expected.");
                    throw DAOLogicError("Query error: File belongs to a different job");
                }
            }
            
            // Get File Id
            file_ids.push_back(rs->getString(1));
        } // End while
    } catch(const SQLException& exc){
        m_log_error("Got SQLException in Executing getFilesAndJobIds: [0x" << exc.getErrorCode() << "] " << exc.getMessage());
        throw DAOException("Failed To Update Get FileTransfers - SQL Exception");
    } catch(const DAOException& exc){
        m_log_error("Got DAOException in Executing getFilesAndJobIds: " << exc.what());
        throw;
    }
    m_log_trace("OracleFileDAO::getFilesAndJobIds Completed");
}


/*
 * getFilesAndJobIds
 *
 * Return the FileTransfer Per Job List that correspond to the given result set
 */
void OracleFileDAO::getFilesAndJobIds(Statement& stmt, StringArrayTable& ids) /*throw (DAOLogicError,DAOException)*/{

    m_log_trace("Calling OracleFileDAO::getFilesAndJobIds");

    // Clean list
    ids.clear();
    
    try {
        // Execute Query
        ResultSetPtr rs(stmt,stmt.executeQuery());

        // For Each entry on the Result Set
        std::string file_id,job_id;
        while(rs->next()){
            // Get Job & File Id
            file_id = rs->getString(1);
            job_id  = rs->getString(2);
            // Insert entry in the Table
            insert_entry(ids,file_id,job_id);
        } // End while
    } catch(const SQLException& exc){
        m_log_error("Got SQLException in Executing getFilesAndJobIds: [0x" << exc.getErrorCode() << "] " << exc.getMessage());
        throw DAOException("Failed To Update Get FileTransfers - SQL Exception");
    } catch(const DAOException& exc){
        m_log_error("Got DAOException in Executing getFilesAndJobIds: " << exc.what());
        throw;
    }
    m_log_trace("OracleFileDAO::getFilesAndJobIds Completed");
}
