/*
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *  GLite Data Transfer Agents: OracleJobDAO Class
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *           Zsolt Molnar <zsolt.molnar@cern.ch - http://www.zsoltmolnar.hu>
 *  Version info: $Id: OracleJobDAO.cpp,v 1.19 2009/07/02 15:21:17 molnarzs Exp $
 *  Release: $Name:  $
 *
 */
#include "OracleJobDAO.h"
#include "dao-utility.h"
#include <glite/data/agents/dao/oracle/ResultSetPtr.h>
#include <memory>
#include <boost/scoped_ptr.hpp>

using namespace glite::data::transfer::agent;
using namespace glite::data::transfer::agent::dao;
using namespace glite::data::transfer::agent::dao::oracle;
using namespace ::oracle::occi;
using glite::data::agents::dao::oracle::ResultSetPtr;
using glite::data::agents::dao::DAOException;
using glite::data::agents::dao::DAOLogicError;

/*
 * OracleJobDAO
 *
 * Constructor
 */
OracleJobDAO::OracleJobDAO(OracleDAOContext& ctx, const std::string& category):
    OracleDAO(ctx,category){
}

/*
 * get
 *
 * Return the Job instance with the given JobId
 */
model::Job * OracleJobDAO::get(Statement& stmt) /*throw (DAOException)*/{

    m_log_trace("Calling OracleJobDAO::get");

    std::auto_ptr<model::Job> job;

    try{
        // Execute Query
        ResultSetPtr rs(stmt,stmt.executeQuery());
        if(0 != rs->next()){
            // Do not do it inside the constructor to avoid possible leaks
            model::Job::State job_state = translateJobState(rs->getString(2).c_str());
            // Create Job instance
            job.reset(new model::Job(
                rs->getString(1),               // job_id
                job_state,                      // state
                rs->getString(3),               // vo_name
                rs->getString(4),               // channel_name
                rs->getInt(5),                  // priority
                toBoolean(rs->getString(6),false),    // cancelJob
                rs->getString(7),               // source
                rs->getString(8),               // dest
                rs->getString(9),               // user_dn
                rs->getString(10),              // job_params
                rs->getString(11),              // reason
                toTimeT(rs->getTimestamp(12)),  // submit_time
                rs->getString(13),              // agent_dn
                rs->getString(14),              // submit_host
                rs->getInt(15),                 // max_time_in_queue
                toTimeT(rs->getTimestamp(16)),  // finish_time
                rs->getString(17),              // myproxy_server
                rs->getString(18),              // space_token
                rs->getString(19),              // storage_class
                rs->getString(20),              // src_catalog
                rs->getString(21),              // src_catalog_type
                rs->getString(22),              // dest_catalog
                rs->getString(23),              // dest_catalog_type
                rs->getString(24),              // user_cred
                rs->getString(25),              // cred_id
                rs->getString(26),              // voms_cred
                rs->getString(27),              // internal Params
                rs->getString(28),              // source_se
                rs->getString(29),              // dest_se
                toBoolean(rs->getString(30),false),   // overwrite_flag
                // new in 3.2.0
                rs->getString(31),              // source_space_token
                rs->getString(32),              // source_token_description
                // new in 3.3.0
                rs->getInt(33),                 // copy_pin_lifetime
                toBoolean(rs->getString(34),false), // lan_connection
                toBoolean(rs->getString(35),false), // fail_nearline
				// new in 3.4.1
				toBoolean(rs->getString(36),false) // compare_checksum
                ));
        } else {
            throw DAOException("No Job found for the given Id");
        }
    } catch(const SQLException& exc){
        m_log_error("Got SQLException in Executing Get: [0x" << exc.getErrorCode() << "] " << exc.getMessage());
        throw DAOException("Failed to get Job - SQL Exception");
    } catch(const DAOException& exc){
        m_log_error("Got DAOException in Executing Get: " << exc.what());
        throw;
    }

    m_log_trace("OracleJobDAO::get Called");

    return job.release();
}

/*
 * update
 *
 * Update a Job object
 */
void OracleJobDAO::update(Statement& stmt) /*throw (DAOException)*/{

    m_log_trace("Calling OracleJobDAO::update");

    try {
        // Execute Update
        unsigned int res = stmt.executeUpdate();
        m_log_trace(res << " row(s) affected");
        if(res <= 0){
            throw DAOException("Job instance hasn't been updated");
        }
    } catch(const SQLException& exc){
        m_log_error("Got SQLException in Executing Update: [0x" << exc.getErrorCode() << "] " << exc.getMessage());
        throw DAOException("Failed To Update Job - SQL Exception");
    }
    m_log_trace("Job instance updated");
}

/*
 * getJobIdList
 *
 * Return the Job id List that correspond to the given statement
 */
void OracleJobDAO::getJobIds(Statement& stmt, agents::dao::StringArray& jobIds) /*throw (DAOException)*/{

    m_log_trace("Calling OracleJobDAO::getJobIds");

    try {
        // Execute Query
        ResultSetPtr rs(stmt,stmt.executeQuery());

        // For Each entry on the Result Set
        while(rs->next()){
            // Get Data
            jobIds.push_back(rs->getString(1));
        } // End while
    } catch(const SQLException& exc){
        m_log_error("Got SQLException in Executing getJobIds: [0x" << exc.getErrorCode() << "] " << exc.getMessage());
        m_log_debug("Query was: " << stmt.getSQL());
        throw DAOException("Failed To Get Job Ids - SQL Exception");
    } catch(const DAOException& exc){
        m_log_error("Got DAOException in Executing getJobIds: " << exc.what());
        throw;
    }
    m_log_trace("OracleJobDAO::getJobIds Completed");
}

/*
 * updateStates
 *
 * Update the state of all the transfer jobs in the given list
 */
void OracleJobDAO::updateStates(const agents::dao::StringArray& ids, Statement& stmt, dao::JobDAO& jobDao) /*throw (DAOLogicError,DAOException)*/{

    m_log_trace("Entering OracleJobDAO::updateStates");

    agents::dao::StringArray file_states;

    // For Each Job Check The Status of Each Single job
    agents::dao::StringArray::const_iterator it;
    for(it = ids.begin(); it != ids.end(); ++it){
        // Get Job
        boost::scoped_ptr<model::Job> job(jobDao.get((*it)));

        // Set Parameters
        stmt.setString(1,job->id());
        // Execute Query
        file_states.clear();
        ResultSetPtr rs(stmt,stmt.executeQuery());
        while(rs->next()){
            // Get Data
            file_states.push_back(rs->getString(1));
        } // End while

        try{
            // Get Job State
            model::Job::State old_state = job->state();

            // Check file states
            update_job_state(*job, file_states);

            // Update Job
            jobDao.update(*job);

            // Log Job State Tranition
            try{
                if(job->state() != old_state){
                    m_log_info("Job [" << job->id() << "] set to [" << translateJobState(job->state()) << "]");
                }
            } catch(...){}
        } catch(const DAOLogicError& exc){
            m_log_error("Integrity problem found for job " << job->id());
            m_log_error("Failed to update State for job " << job->id() << " (job state is " << translateJobState(job->state()) << "). Reason is " << exc.what());
#warning "TODO: Fail Job and Files in order to solve the inconsistency"
            m_log_warn("Continue With the Next Job");
        }
    }

    m_log_trace("OracleJobDAO::updateStates Called");
}
