/*
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *  GLite Data Transfer Agents: OracleStageReqDAO Class
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *           Paolo Tedesco <paolo.tedesco@cern.ch>
 *  Version info: $Id: OracleStageReqDAO.cpp,v 1.5 2008/11/13 10:43:56 ptedesco Exp $
 *  Release: $Name:  $
 *
 */
#include "OracleStageReqDAO.h"
#include "dao-utility.h"
#include <glite/data/agents/dao/oracle/ResultSetPtr.h>

#include <memory>

using namespace glite::data::transfer::agent;
using namespace glite::data::transfer::agent::dao;
using namespace glite::data::transfer::agent::dao::oracle;
using namespace ::oracle::occi;
using glite::data::agents::dao::oracle::ResultSetPtr;
using glite::data::agents::dao::DAOException;
using glite::data::agents::dao::DAOLogicError;

/*
 * OracleStageReqDAO
 *
 * Constructor
 */
OracleStageReqDAO::OracleStageReqDAO(OracleDAOContext& ctx, const std::string& category):
        OracleDAO(ctx,category){
}

/*
 * get
 *
 * Return the StageReq from the given statement 
 */
model::StageReq * OracleStageReqDAO::get(Statement& stmt) /*throw (DAOLogicError,DAOException)*/{
    
    m_log_trace("Calling OracleStageReqDAO::get");

    std::auto_ptr<model::StageReq> transfer;
    
    try{
        // Execute Query
        ResultSetPtr rs(stmt,stmt.executeQuery());
        if(0 != rs->next()){
            // Do not do it inside the constructor to avoid possible leaks
            model::StageReq::State sr_state = translateStageReqState(rs->getString(5).c_str());
            model::Error::Category err_class = translateErrorCategory(rs->getString(10).c_str());
            model::Error::Scope err_scope = translateErrorScope(rs->getString(13).c_str());
            model::Error::Phase err_phase = translateErrorPhase(rs->getString(14).c_str());
            // Create StageReq instance
            transfer.reset(new model::StageReq(
                rs->getString(1),                // request_id
                rs->getString(2),                // file_id
                rs->getInt(3),                   // stage_id
                rs->getString(4),                // job_id
                sr_state,                        // state
                rs->getString(6),                // turl
                toTimeT(rs->getTimestamp(7)),    // request_time
                rs->getDouble(8),                // duration
                rs->getDouble(9),                // finalization_duration
                err_class,                       // reason_class
                rs->getString(11),               // reason
                toLongLong(rs->getNumber(12)),   // filesize
                err_scope,                       // error scope
                err_phase,                       // error phaes
                toTimeT(rs->getTimestamp(15))    // finish_time
            ));
        } else {
            throw DAOException("No StageReq found for the given Id");
        }
    } catch(const SQLException& exc){
        m_log_error("Got SQLException in Executing Get: [0x" << exc.getErrorCode() << "] " << exc.getMessage());
        throw DAOException("Failed to get StageReq - SQL Exception");
    } catch(const DAOException& exc){
        m_log_error("Got DAOException in Executing Get: " << exc.what());
        throw;
    } 
    
    m_log_trace("OracleStageReqDAO::get Called");
    
    return transfer.release();
}

/*
 * update
 *
 * Update a StageReq object
 */
void OracleStageReqDAO::update(Statement& stmt) /*throw (DAOLogicError,DAOException)*/{
    
    m_log_trace("Calling OracleStageReqDAO::update");
    
    try {
        // Execute Update
        unsigned int res = stmt.executeUpdate();
        m_log_trace(res << " row(s) affected");
        if(res <= 0){
            throw DAOException("StageReq instance hasn't been updated");
        }
    } catch(const SQLException& exc){
        m_log_error("Got SQLException in Executing Update: [0x" << exc.getErrorCode() << "] " << exc.getMessage());
        throw DAOException("Failed To Update StageReq - SQL Exception");
    }
    m_log_trace("StageReq instance updated");
}

/**
 * Create a StageReq object
 */
void OracleStageReqDAO::create(::oracle::occi::Statement& stmt) /*throw (DAOLogicError,DAOException)*/{
    m_log_trace("Calling OracleStageReqDAO::create");
    
    try {
        // Execute Update
        unsigned int res = stmt.executeUpdate();
        m_log_trace(res << " row(s) affected");
        if(res <= 0){
            throw DAOException("StageReq instance hasn't been created");
        }
    } catch(const SQLException& exc){
        m_log_error("Got SQLException in Executing Update: [0x" << exc.getErrorCode() << "] " << exc.getMessage());
        throw DAOException("Failed To Create StageReq - SQL Exception");
    }
    m_log_trace("StageReq instance create");
}

/*
 * getIdList
 *
 * Return the a list of id for the given Statement
 */
void OracleStageReqDAO::getIdList(::oracle::occi::Statement& stmt, agents::dao::StringArray& ids) /*throw (DAOLogicError,DAOException)*/{
          
    m_log_trace("Calling OracleStageReqDAO::getIdArray");
    
    try {
        // Execute Query
        ResultSetPtr rs(stmt,stmt.executeQuery());
        
        // For Each entry on the Result Set
        while(rs->next()){
            // Get Data
            ids.push_back(rs->getString(1));
        } // End while
    } catch(const SQLException& exc){
        m_log_error("Got SQLException in Executing getIdList: [0x" << exc.getErrorCode() << "] " << exc.getMessage());
        throw DAOException("Failed To Update Job - SQL Exception");
    } catch(const DAOException& exc){
        m_log_error("Got DAOException in Executing getIdList: " << exc.what());
        throw;
    }
    m_log_trace("OracleStageReqDAO::getIdList Completed");
}
