/**
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *  GLite Data Transfer Agents: OracleTransferDAO Class
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>, Paolo Tedesco <paolo.tedesco@cern.ch>
 *  Version info: $Id: OracleTransferDAO.cpp,v 1.8 2008/11/13 10:43:56 ptedesco Exp $
 *  Release: $Name:  $
 *
 */
#include "OracleTransferDAO.h"
#include "dao-utility.h"
#include <glite/data/agents/dao/oracle/ResultSetPtr.h>

#include <memory>

using namespace glite::data::transfer::agent;
using namespace glite::data::transfer::agent::dao;
using namespace glite::data::transfer::agent::dao::oracle;
using namespace ::oracle::occi;
using glite::data::agents::dao::oracle::ResultSetPtr;
using glite::data::agents::dao::DAOException;
using glite::data::agents::dao::DAOLogicError;

/*
 * OracleTransferDAO
 *
 * Constructor
 */
OracleTransferDAO::OracleTransferDAO(OracleDAOContext& ctx, const std::string& logger):
        OracleDAO(ctx,logger){
}

/*
 * get
 *
 * Return the Transfer from the given statement 
 */
model::Transfer * OracleTransferDAO::get(Statement& stmt) /*throw (DAOLogicError,DAOException)*/{
    
    m_log_trace("Calling OracleTransferDAO::get");

    std::auto_ptr<model::Transfer> transfer;
    
    try{
        // Execute Query
        ResultSetPtr rs(stmt,stmt.executeQuery());
        if(0 != rs->next()){
            // Do not do it inside the constructor to avoid possible leaks
            model::Transfer::State tx_state = translateTransferState(rs->getString(5).c_str());
            model::Error::Category err_class = translateErrorCategory(rs->getString(19).c_str());
            model::Error::Scope err_scope = translateErrorScope(rs->getString(26).c_str());
            model::Error::Phase err_phase = translateErrorPhase(rs->getString(27).c_str());
            // Create Transfer instance
            transfer.reset(new model::Transfer(
                        rs->getString(1),                // request_id
                        rs->getString(2),                // file_id
                        rs->getInt(3),                   // transfer_id
                        rs->getString(4),                // job_id
                        tx_state,                        // state
                        rs->getString(6),                // transfer_type
                        rs->getString(7),                // source_turl
                        rs->getString(8),                // dest_turl
                        toTimeT(rs->getTimestamp(9)),    // transfer_time
                        toTimeT(rs->getTimestamp(10)),   // prepare_time
                        rs->getDouble(11),               // duration
                        rs->getDouble(12),               // src_preparation_duration
                        rs->getDouble(13),               // dest_preparation_duration
                        rs->getDouble(14),               // transfer_duration
                        rs->getDouble(15),               // src_finalization_duration
                        rs->getDouble(16),               // dest_finalization_duration
                        toLongLong(rs->getNumber(17)),   // bytes_written
                        rs->getDouble(18),               // throughput
                        err_class,                       // reason_class
                        rs->getString(20),               // reason
                        toLongLong(rs->getNumber(21)),   // filesize
                        rs->getString(22),               // source_srm_token
                        rs->getString(23),               // dest_srm_token
                        rs->getString(24),               // source_host
                        rs->getString(25),               // dest_host
                        err_scope,                       // error scope
                        err_phase,                       // error phaes
                        toTimeT(rs->getTimestamp(28))    // finish_time
                        ));
        } else {
            throw DAOException("No Transfer found for the given Id");
        }
    } catch(const SQLException& exc){
        m_log_error("Got SQLException in Executing Get: [0x" << exc.getErrorCode() << "] " << exc.getMessage());
        throw DAOException("Failed to get Transfer - SQL Exception");
    } catch(const DAOException& exc){
        m_log_error("Got DAOException in Executing Get: " << exc.what());
        throw;
    } 
    
    m_log_trace("OracleTransferDAO::get Called");
    
    return transfer.release();
}

/*
 * update
 *
 * Update a Transfer object
 */
void OracleTransferDAO::update(Statement& stmt) /*throw (DAOLogicError,DAOException)*/{
    
    m_log_trace("Calling OracleTransferDAO::update");
    
    try {
        // Execute Update
        unsigned int res = stmt.executeUpdate();
        m_log_trace(res << " row(s) affected");
        if(res <= 0){
            throw DAOException("Transfer instance hasn't been updated");
        }
    } catch(const SQLException& exc){
        m_log_error("Got SQLException in Executing Update: [0x" << exc.getErrorCode() << "] " << exc.getMessage());
        throw DAOException("Failed To Update Transfer - SQL Exception");
    }
    m_log_trace("Transfer instance updated");
}

/**
 * Create a Transfer object
 */
void OracleTransferDAO::create(::oracle::occi::Statement& stmt) /*throw (DAOLogicError,DAOException)*/{
    m_log_trace("Calling OracleTransferDAO::create");
    
    try {
        // Execute Update
        unsigned int res = stmt.executeUpdate();
        m_log_trace(res << " row(s) affected");
        if(res <= 0){
            throw DAOException("Transfer instance hasn't been created");
        }
    } catch(const SQLException& exc){
        m_log_error("Got SQLException in Executing Update: [0x" << exc.getErrorCode() << "] " << exc.getMessage());
        throw DAOException("Failed To Create Transfer - SQL Exception");
    }
    m_log_trace("Transfer instance create");
}

/*
 * getIdList
 *
 * Return the a list of id for the given Statement
 */
void OracleTransferDAO::getIdList(::oracle::occi::Statement& stmt, agents::dao::StringArray& ids) /*throw (DAOLogicError,DAOException)*/{
          
    m_log_trace("Calling OracleTransferDAO::getIdArray");
    
    try {
        // Execute Query
        ResultSetPtr rs(stmt,stmt.executeQuery());
        
        // For Each entry on the Result Set
        while(rs->next()){
            // Get Data
            ids.push_back(rs->getString(1));
        } // End while
    } catch(const SQLException& exc){
        m_log_error("Got SQLException in Executing getIdList: [0x" << exc.getErrorCode() << "] " << exc.getMessage());
        throw DAOException("Failed To Update Job - SQL Exception");
    } catch(const DAOException& exc){
        m_log_error("Got DAOException in Executing getIdList: " << exc.what());
        throw;
    }
    m_log_trace("OracleTransferDAO::getIdList Completed");
}

/*
 * getIdPairArray
 *
 * Return the id-id list for the given statement
 */
void OracleTransferDAO::getIdPairArray(::oracle::occi::Statement& stmt, std::vector<std::pair<std::string,std::string> >& ids) /*throw (DAOLogicError,DAOException)*/{
    
    m_log_trace("Calling OracleTransferDAO::getIdPairArray");
    
    try {
        // Execute Query
        ResultSetPtr rs(stmt,stmt.executeQuery());
        
        // For Each entry on the Result Set
        while(rs->next()){
            // Get Data
            ids.push_back(std::pair<std::string,std::string>(rs->getString(1),rs->getString(2)));
        } // End while
    } catch(const SQLException& exc){
        m_log_error("Got SQLException in Executing getIdPairArray: [0x" << exc.getErrorCode() << "] " << exc.getMessage());
        throw DAOException("Failed To Update Get FileTransfers - SQL Exception");
    } catch(const DAOException& exc){
        m_log_error("Got DAOException in Executing getIdPairArray: " << exc.what());
        throw;
    }
    m_log_trace("OracleTransferDAO::getIdArrayTable Completed");
}
