/**
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *  GLite Data Transfer Agents: OracleStageReqStatements Class
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *           Paolo Tedesco <paolo.tedesco@cern.ch>
 *  Version info: $Id: StageReqStatements.cpp,v 1.8 2008/11/13 10:43:56 ptedesco Exp $
 *  Release: $Name:  $
 *
 */
#include "StageReqStatements.h"
#include "dao-utility.h"
#include <glite/data/agents/dao/oracle/StatementPtr.h>

#include <sstream>

using namespace glite::data::transfer::agent;
using namespace glite::data::transfer::agent::dao;
using namespace glite::data::transfer::agent::dao::oracle;
using namespace ::oracle::occi;
using glite::data::agents::dao::DAOLogicError;
using glite::data::agents::dao::DAOException;
using glite::data::agents::dao::oracle::OracleDAOContext;
using glite::data::agents::dao::oracle::StatementPtr;

namespace {
const char * const ORACLE_STAGEREQ_STATEMENTS_NAME = "transfer-agent-dao-oracle-sreq-stmt";

const char * const GET_STATEMENT_VO_TAG                   = "t_stagereq_get_vo";
const char * const UPDATE_STATEMENT_VO_TAG                = "t_stagereq_update_vo";
const char * const GET_BY_FILEID_STATEMENT_VO_TAG         = "t_stagereq_get_by_fileid_vo";
const char * const GET_BY_REQID_STATEMENT_VO_TAG          = "t_stagereq_get_by_reqid_vo";
const char * const CREATE_STATEMENT_VO_TAG                = "t_stagereq_create_vo";
const char * const COUNT_ACTIVE_STATEMENT_VO_TAG          = "t_stagereq_count_active_vo";
const char * const COUNT_ACTIVE_PER_CH_STATEMENT_VO_TAG   = "t_stagereq_count_active_per_ch_vo";
}

/*
 * instance
 *
 * Return Singleton instance
 */
StageReqStatements& StageReqStatements::instance() /*throw ()*/{
    static StageReqStatements singleton;
    return singleton;
}


/*
 * StageReqStatements
 *
 * Constructor
 */
StageReqStatements::StageReqStatements():
        m_logger(ORACLE_STAGEREQ_STATEMENTS_NAME){
}

/*
 * dispose
 *
 * dispose Statements For OracleStageReqDAO
 */
void StageReqStatements::dispose(OracleDAOContext& ctx) {
#warning "TODO : terminate Statements"
}

/*
 * createGet_VO
 *
 * Prepare Statement For StageReqDAO::get, VO View
 *
 * Bind Parameters:
 *  :1 - File ID
 *  :2 - Request ID
 *  :3 - VO Name
 */
Statement * StageReqStatements::createGet_VO(OracleDAOContext& ctx) /*throw (DAOLogicError, DAOException)*/{

    // Get Tag
    const char * const tag = GET_STATEMENT_VO_TAG;

    Statement * stmt = 0;
    try {
        // Check if the statement is already cached
        if(false == ctx.connection().isCached("",tag)){
            // Prepare Statement
            const char * const query_stmt = 
                "SELECT t_stage_req.request_id"
                ", t_stage_req.file_id"
                ", t_stage_req.stage_id"
                ", t_stage_req.job_id"
                ", t_stage_req.stage_state"
                ", t_stage_req.turl"
                ", t_stage_req.request_time"
                ", t_stage_req.duration"
                ", t_stage_req.final_duration"
                ", t_stage_req.reason_class"
                ", t_stage_req.reason"
                ", t_stage_req.filesize"
                //--------------------------------------------------------------
                // 3.1.0
                ", t_stage_req.error_scope"
                ", t_stage_req.error_phase"
                ", t_stage_req.finish_time"
                //--------------------------------------------------------------
                " FROM t_stage_req, t_job"
                " WHERE t_stage_req.file_id  = :1"
                " AND t_stage_req.request_id = :2"
                " AND t_stage_req.job_id = t_job.job_id"
                " AND t_job.vo_name = :3"
                //--------------------------------------------------------------
                // 3.1.0
                " AND t_stage_req.job_finished is NULL"
                " AND t_job.job_finished is NULL";
                //--------------------------------------------------------------                
            m_log_trace("Preparing Statement: " << query_stmt);
            // Create Statements
            stmt = ctx.connection().createStatement(query_stmt,tag);
        } else {
            // Create Statements
            stmt = ctx.connection().createStatement("",tag);
        }
    } catch(const SQLException& exc){
        m_log_error("Got Exception in creating Get Statement: [0x" << exc.getErrorCode() << "] " << exc.getMessage());
        throw DAOException("Failed to prepare Statement");
    }
    // Check Postcondition
    if(0 == stmt){
        throw DAOLogicError("Failed to prepare Statement - Nullpointer");
    }
    return stmt;
}

/*
 * createUpdate_VO
 *
 * Prepare Statement For StageReqDAO::update, VO View. Returns 0 if transfer doesn't need to be updated
 */
Statement * StageReqStatements::createUpdate_VO(OracleDAOContext& ctx, const model::StageReq& sr, const std::string& vo_name) /*throw (DAOLogicError,DAOException)*/{

    Statement * stmt = 0;        
    try {            
        // Update StageReq Properties
        // -------------------------------
        // | Column              | RO/RW |
        // -------------------------------
        // | request_id          |  RO   |
        // | file_id             |  RO   |
        // | stage_id            |  RO   |
        // | job_id              |  RO   |
        // | state               |  RW   |
        // | turl                |  RW   |
        // | requert_time        |  RW   |
        // | duration            |  RW   |
        // | final_duration      |  RW   |
        // | reason_class        |  RW   |
        // | reason              |  RW   |
        // | filesize            |  RW   |
        // | error_scope         |  RW   |
        // | error_phase         |  RW   |
        // | finish_time         |  RW   |
        // -------------------------------
        // Prepare Update Statement
        unsigned int count = 4;
        std::stringstream query_stmt;
        query_stmt << "UPDATE ( SELECT t_stage_req.* "
                       " FROM t_stage_req, t_job"
                       " WHERE t_stage_req.file_id  = :1"
                       " AND t_stage_req.request_id = :2"
                       " AND t_stage_req.job_id = t_job.job_id"
                       " AND t_job.vo_name = :3"
                       " AND t_stage_req.job_finished is NULL"
                       " AND t_job.job_finished is NULL"
                       " ) t_stage_req SET";
        // State
        if(sr.state.isSet()){
            query_stmt << ((4 == count)?"":",") << " t_stage_req.stage_state = :" << count;
            ++count;
        }
        // Source Turl
        if(sr.turl.isSet()){
            query_stmt << ((4 == count)?"":",") << " t_stage_req.turl = :" << count;
            ++count;
        }
        // request_time
        if(sr.requestTime.isSet()){
            query_stmt << ((4 == count)?"":",") << " t_stage_req.request_time = :" << count;
            ++count;
        }
        // duration
        if(sr.duration.isSet()){
            query_stmt << ((4 == count)?"":",") << " t_stage_req.duration = :" << count;
            ++count;
        }
        // final_duration
        if(sr.finalDuration.isSet()){
            query_stmt << ((4 == count)?"":",") << " t_stage_req.final_duration = :" << count;
            ++count;
        }
        // reason_class
        if(sr.reasonClass.isSet()){
            query_stmt << ((4 == count)?"":",") << " t_stage_req.reason_class = :" << count;
            ++count;
        }
        // reason
        if(sr.reason.isSet()){
            query_stmt << ((4 == count)?"":",") << " t_stage_req.reason = :" << count;
            ++count;
        }
        // filesize
        if(sr.fileSize.isSet()){
            query_stmt << ((4 == count)?"":",") << " t_stage_req.filesize = :" << count;
            ++count;
        }
        // error_scope
        if(sr.errorScope.isSet()){
            query_stmt << ((4 == count)?"":",") << " t_stage_req.error_scope = :" << count;
            ++count;
        } 
        // error_phase
        if(sr.errorPhase.isSet()){
            query_stmt << ((4 == count)?"":",") << " t_stage_req.error_phase = :" << count;
            ++count;
        }
        // finish_time
        if(sr.finishTime.isSet()){
            query_stmt << ((4 == count)?"":",") << " t_stage_req.finish_time = :" << count;
            ++count;
        }
       
        if(count > 4){
            m_log_trace("Preparing Statement: " << query_stmt.str());
            // Create Statements
            stmt = ctx.connection().createStatement(query_stmt.str());
            
            // RAII
            StatementPtr stmt_ptr(ctx.connection(),stmt);
            
            // Set the value of the properties
            stmt->setString(1,sr.fileId());
            stmt->setString(2,sr.requestId());
            stmt->setString(3,vo_name);
            count = 4;
            // State
            if(sr.state.isSet()){
                stmt->setString(count++,translateStageReqState(sr.state()));
            } 
            // Turl
            if(sr.turl.isSet()){
                stmt->setString(count++,sr.turl());
            }
            // request_time
            if(sr.requestTime.isSet()){
                if((time_t)-1 != sr.requestTime()){
                    stmt->setTimestamp(count++,ctx.toTimestamp(sr.requestTime()));
                } else {
                    stmt->setNull(count++ ,OCCITIMESTAMP);
                }
            }
            // duration
            if(sr.duration.isSet()){
                stmt->setDouble(count++,sr.duration());
            }
            // final_duration
            if(sr.finalDuration.isSet()){
                stmt->setDouble(count++,sr.finalDuration());
            }
            // reason_class
            if(sr.reasonClass.isSet()){
                stmt->setString(count++,translateErrorCategory(sr.reasonClass()));
            }
            // reason
            if(sr.reason.isSet()){
                if(sr.reason().length() < MAX_T_STAGEREQ_REASON_LEN){
                    stmt->setString(count++,sr.reason());
                } else {
                    stmt->setString(count++,sr.reason().substr(0,MAX_T_STAGEREQ_REASON_LEN - 1));
                }
            }
            // filesize
            if(sr.fileSize.isSet()){
                stmt->setNumber(count++,ctx.toNumber(sr.fileSize()));
            }
            
            // error_scope
            if(sr.errorScope.isSet()){
                stmt->setString(count++,translateErrorScope(sr.errorScope()));
            } 
            // error_phase
            if(sr.errorPhase.isSet()){
                stmt->setString(count++,translateErrorPhase(sr.errorPhase()));
            }
            // finish_time
            if(sr.finishTime.isSet()){
                if((time_t)-1 != sr.finishTime()){
                    stmt->setTimestamp(count++,ctx.toTimestamp(sr.finishTime()));
                } else {
                    stmt->setNull(count++ ,OCCITIMESTAMP);
                }
            }
            
            // Release the Pointer
            stmt_ptr.release();
        }
    } catch(const SQLException& exc){
        m_log_error("Got Exception in creating Update Statement: [0x" << exc.getErrorCode() << "] " << exc.getMessage());
        throw DAOException("Failed to prepare Statement");
    }
    // Don't Check Postcondition (Null is allowed)   
    return stmt;
}

/*
 * createGetByFileId_VO
 *
 * Prepare GetByFileId Statement, VO View
 *
 * Bind Parameters:
 *  :1 - File ID
 *  :2 - VO Name
 *  :3 - Limit
 *  :4 - Limit
 */
Statement * StageReqStatements::createGetByFileId_VO(OracleDAOContext& ctx) /*throw (DAOLogicError,DAOException)*/{
    
    // Get Tag
    const char * const tag = GET_BY_FILEID_STATEMENT_VO_TAG;

    // Get Tag
    Statement * stmt = 0; 
    try {
        // Check if the statement is already cached
        if(false == ctx.connection().isCached("",tag)){
            // Prepare Statement String
            const char * const query_stmt =
                "SELECT * FROM ("
                    "SELECT t_stage_req.request_id"
                    " FROM t_stage_req, t_job"
                    " WHERE t_stage_req.file_id = :1"
                    " AND t_stage_req.job_id = t_job.job_id"
                    " AND t_job.vo_name = :2"
                    " AND t_stage_req.job_finished is NULL"
                    " AND t_job.job_finished is NULL"
                    " ORDER BY t_stage_req.request_time DESC"
                            ", t_stage_req.request_id"
                ") WHERE (:3=0 OR ROWNUM<=:4)";
            m_log_trace("Preparing Statement: " << query_stmt);
            // Create Statements
            stmt = ctx.connection().createStatement(query_stmt,tag);
        } else {
            // Create Statements
            stmt = ctx.connection().createStatement("",tag);
        }
    } catch(const SQLException& exc){
        m_log_error("Got Exception in Creating GetByFileId Statement: [0x" << exc.getErrorCode() << "] " << exc.getMessage());
        throw DAOException("Failed to prepare Statement");
    }
    // Check Postcondition
    if(0 == stmt){
        throw DAOLogicError("Failed to prepare Statement - Nullpointer");
    }
    return stmt;
}

/*
 * createGetByRequestId_VO
 *
 * Prepare GetByRequestId Statement, VO View
 *
 * Bind Parameters:
 *  :1 - Request ID
 *  :2 - VO Name
 */
Statement * StageReqStatements::createGetByRequestId_VO(OracleDAOContext& ctx) /*throw (DAOLogicError,DAOException)*/{
    
    // Get Tag
    const char * const tag = GET_BY_REQID_STATEMENT_VO_TAG;

    // Get Tag
    Statement * stmt = 0; 
    try {
        // Check if the statement is already cached
        if(false == ctx.connection().isCached("",tag)){
            // Prepare Statement String
            const char * const query_stmt =
                "SELECT t_stage_req.file_id"
                " FROM t_stage_req, t_job"
                " WHERE t_stage_req.request_id = :1"
                " AND t_stage_req.job_id = t_job.job_id"
                " AND t_job.vo_name = :2"
                " AND t_stage_req.job_finished is NULL"
                " AND t_job.job_finished is NULL"
                " ORDER BY t_stage_req.file_id";
            m_log_debug("Preparing Statement: " << query_stmt);
            // Create Statements
            stmt = ctx.connection().createStatement(query_stmt,tag);
        } else {
            // Create Statements
            stmt = ctx.connection().createStatement("",tag);
        }
    } catch(const SQLException& exc){
        m_log_error("Got Exception in Creating GetByRequestId Statement: [0x" << exc.getErrorCode() << "] " << exc.getMessage());
        throw DAOException("Failed to prepare Statement");
    }
    // Check Postcondition
    if(0 == stmt){
        throw DAOLogicError("Failed to prepare Statement - Nullpointer");
    }
    return stmt;
}

/*
 * createCreate_VO
 *
 * Return the statement for the StageReqDAO::create, VO View
 */
Statement* StageReqStatements::createCreate_VO(OracleDAOContext& ctx, const model::StageReq& sr) /*throw (DAOLogicError,DAOException)*/{
    
    // Get Tag
    Statement * stmt = 0;        
    try {
        // Prepare Update Statement
        unsigned int count = 6;
        std::stringstream query_stmt;
        query_stmt << "INSERT INTO t_stage_req("
           " file_id"
           ", request_id"
           ", stage_id"
           ", job_id"
           ", stage_state";
        // Source Turl
        if(sr.turl.isSet()){
            query_stmt << ", turl";++count;
        }
        // request_time
        if(sr.requestTime.isSet()){
            query_stmt << ", request_time";++count;
        }
        // duration
        if(sr.duration.isSet()){
            query_stmt << ", duration";++count;
        }
        // final_duration
        if(sr.finalDuration.isSet()){
            query_stmt << ", final_duration";++count;
        }
        // reason_class
        if(sr.reasonClass.isSet()){
            query_stmt << ", reason_class";++count;
        }
        // reason
        if(sr.reason.isSet()){
            query_stmt << ", reason";++count;
        }
        // filesize
        if(sr.fileSize.isSet()){
            query_stmt << ", filesize";++count;
        } 
        // error_scope
        if(model::Error::S_UNDEF != sr.errorScope()){
            query_stmt << ", error_scope";++count;
        }
        // error_phase
        if(model::Error::P_UNDEF != sr.errorPhase()){
            query_stmt << ", error_phase";++count;
        }
        // finish_time
        if((time_t)-1 != sr.finishTime()){
            query_stmt << ", finish_time";++count;
        }
        query_stmt << " ) VALUES ("
            " :1"
            ", :2"
            ", :3"
            ", :4"
            ", :5";
        for(unsigned int i = 6; i < count; ++i){
            query_stmt << ", :" << i;
        }
        query_stmt << " )";
        m_log_trace("Preparing Statement: " << query_stmt.str());
        // Create Statements
        stmt = ctx.connection().createStatement(query_stmt.str());
        
        // RAII
        StatementPtr stmt_ptr(ctx.connection(),stmt);
            
        // Set the value of the properties
        stmt->setString(1,sr.fileId());
        stmt->setString(2,sr.requestId());
        stmt->setInt(3,sr.stageId());
        stmt->setString(4,sr.jobId());
        stmt->setString(5,translateStageReqState(sr.state()));
        count = 6;
        // Turl
        if(sr.turl.isSet()){
            stmt->setString(count++,sr.turl());
        }
        // request_time
        if(sr.requestTime.isSet()){
            stmt->setTimestamp(count++,ctx.toTimestamp(sr.requestTime()));
        }
        // duration
        if(sr.duration.isSet()){
            stmt->setDouble(count++,sr.duration());
        }
        // final_duration
        if(sr.finalDuration.isSet()){
            stmt->setDouble(count++,sr.finalDuration());
        }
        // reason_class
        if(sr.reasonClass.isSet()){
            stmt->setString(count++,translateErrorCategory(sr.reasonClass()));
        }
        // reason
        if(sr.reason.isSet()){
            if(sr.reason().length() < MAX_T_STAGEREQ_REASON_LEN){
                stmt->setString(count++,sr.reason());
            } else {
                stmt->setString(count++,sr.reason().substr(0,MAX_T_STAGEREQ_REASON_LEN - 1));
            }
        }
        // filesize
        if(sr.fileSize.isSet()){
            stmt->setNumber(count++,ctx.toNumber(sr.fileSize()));
        }
        // error_scope
        if(model::Error::S_UNDEF != sr.errorScope()){
            stmt->setString(count++,translateErrorScope(sr.errorScope()));
        }
        // error_phase
        if(model::Error::P_UNDEF != sr.errorPhase()){
            stmt->setString(count++,translateErrorPhase(sr.errorPhase()));
        }
        // finish_time
        if((time_t)-1 != sr.finishTime()){
            stmt->setTimestamp(count++,ctx.toTimestamp(sr.finishTime()));
        }
    
        // Release the Pointer
        stmt_ptr.release();
    } catch(const SQLException& exc){
        m_log_error("Got Exception in creating Insert Statement: [0x" << exc.getErrorCode() << "] " << exc.getMessage());
        throw DAOException("Failed to prepare Statement");
    }
    // Check Postcondition
    if(0 == stmt){
        throw DAOLogicError("Failed to prepare Statement - Nullpointer");
    }
    return stmt;
}

/**
 * Return the statement for the StageReqDAO::countActiveRequests, VO View
 *
 * Bind Parameters:
 *  :1 - VO Name
 */
Statement* StageReqStatements::createCountActiveRequests_VO(OracleDAOContext& ctx, bool per_channel) /*throw (DAOLogicError,DAOException)*/{
        
    // Get Tag
    const char * const tag = (true == per_channel)?COUNT_ACTIVE_STATEMENT_VO_TAG:COUNT_ACTIVE_PER_CH_STATEMENT_VO_TAG;

    Statement * stmt = 0;        
    try {
        // Check if the statement is already cached
        if(false == ctx.connection().isCached("",tag)){
            // Prepare Update Statement
            std::stringstream query_stmt;
            query_stmt <<
               "SELECT /*+ INDEX(t_stage_req stagereq_stage_state) */ COUNT(*)"
               " FROM t_stage_req, t_job"
               " WHERE t_stage_req.job_id = t_job.job_id"
               " AND t_job.vo_name = :1"
               " AND t_stage_req.stage_state IN ("
                    "'Submitted','Processing','Staged','Finalizing')"
               " AND t_stage_req.job_finished is NULL"
               " AND t_job.job_finished is NULL";
            if(true == per_channel){
                query_stmt << " AND t_job.channel_name = :2";
            }
            m_log_trace("Preparing Statement: " << query_stmt.str());
            // Create Statement
            stmt = ctx.connection().createStatement(query_stmt.str(),tag);
        } else {
            // Create Statement
            stmt = ctx.connection().createStatement("",tag);
        }
    } catch(const SQLException& exc){
        m_log_error("Got Exception in creating Insert Statement: [0x" << exc.getErrorCode() << "] " << exc.getMessage());
        throw DAOException("Failed to prepare Statement");
    }
    // Check Postcondition
    if(0 == stmt){
        throw DAOLogicError("Failed to prepare Statement - Nullpointer");
    }
    return stmt;
}
