/*
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *  GLite Data Transfer Agents: OracleTransferStatements Class
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>, Paolo Tedesco <paolo.tedesco@cern.ch>
 *  Version info: $Id: TransferStatements.cpp,v 1.23 2008/11/13 10:43:56 ptedesco Exp $
 *  Release: $Name:  $
 *
 */
#include "TransferStatements.h"
#include "dao-utility.h"
#include <glite/data/agents/dao/oracle/StatementPtr.h>

#include <sstream>

using namespace glite::data::transfer::agent;
using namespace glite::data::transfer::agent::dao;
using namespace glite::data::transfer::agent::dao::oracle;
using namespace ::oracle::occi;
using glite::data::agents::dao::DAOLogicError;
using glite::data::agents::dao::DAOException;
using glite::data::agents::dao::oracle::OracleDAOContext;
using glite::data::agents::dao::oracle::StatementPtr;

namespace {
const char * const ORACLE_TRANSFER_STATEMENTS_NAME = "transfer-agent-dao-oracle-tx-stmt";

const char * const GET_STATEMENT_VO_TAG                   = "t_transfer_get_vo";
const char * const GET_FULL_SEARCH_STATEMENT_VO_TAG       = "t_transfer_get_full_search_vo";
const char * const GET_STATEMENT_CH_TAG                   = "t_transfer_get_ch";
const char * const GET_FULL_SEARCH_STATEMENT_CH_TAG       = "t_transfer_get_full_search_ch";
const char * const UPDATE_STATEMENT_CH_TAG                = "t_transfer_update_ch";
const char * const GET_BY_FILEID_STATEMENT_VO_TAG         = "t_transfer_get_by_fileid_vo";
const char * const GET_BY_FILEID_STATEMENT_CH_TAG         = "t_transfer_get_by_fileid_ch";
const char * const GET_BY_REQID_STATEMENT_VO_TAG          = "t_transfer_get_by_reqid_vo";
const char * const GET_BY_REQID_STATEMENT_CH_TAG          = "t_transfer_get_by_reqid_ch";
const char * const CREATE_STATEMENT_CH_TAG                = "t_transfer_create_ch";
const char * const COUNT_ACTIVE_STATEMENT_CH_TAG          = "t_transfer_count_active_ch";
const char * const COUNT_ACTIVE_PER_VO_STATEMENT_CH_TAG   = "t_transfer_count_active_per_vo_ch";
const char * const GET_ACTIVE_STATEMENT_CH_TAG            = "t_transfer_get_active_ch";
}

/*
 * instance
 *
 * Return Singleton instance
 */
TransferStatements& TransferStatements::instance() /*throw ()*/{
    static TransferStatements singleton;
    return singleton;
}


/*
 * TransferStatements
 *
 * Constructor
 */
TransferStatements::TransferStatements():
    Statements(ORACLE_TRANSFER_STATEMENTS_NAME){
}

/*
 * dispose
 *
 * dispose Statements For OracleTransferDAO
 */
void TransferStatements::dispose(OracleDAOContext& ctx) {
#warning "TODO : terminate Statements"
}

/*
 * createGet_VO
 *
 * Prepare Statement For TransferDAO::get, VO View
 *
 * Bind Parameters:
 *  :1 - File ID
 *  :2 - Request ID
 *  :3 - VO Name
 */
namespace{
const char * create_get_vo(){
    return "SELECT t_transfer.request_id"
        ", t_transfer.file_id"
        ", t_transfer.transfer_id"
        ", t_transfer.job_id"
        ", t_transfer.transfer_state"
        ", t_transfer.transfer_type"
        ", t_transfer.source_turl"
        ", t_transfer.dest_turl"
        ", t_transfer.transfer_time"
        ", t_transfer.prepare_time"
        ", t_transfer.duration"
        ", t_transfer.src_prep_duration"
        ", t_transfer.dest_prep_duration"
        ", t_transfer.tx_duration"
        ", t_transfer.src_final_duration"
        ", t_transfer.dest_final_duration"
        ", t_transfer.bytes_written"
        ", t_transfer.throughput"
        ", t_transfer.reason_class"
        ", t_transfer.reason"
        ", t_transfer.filesize"
        ", t_transfer.source_srm_token"
        ", t_transfer.dest_srm_token"
        ", t_transfer.source_host"
        ", t_transfer.dest_host"
        ", t_transfer.error_scope"
        ", t_transfer.error_phase"
        ", t_transfer.finish_time"
        " FROM t_transfer,t_job"
        " WHERE t_transfer.file_id  = :1"
        " AND t_transfer.request_id = :2"
        " AND t_transfer.job_id = t_job.job_id"
        " AND t_job.vo_name = :3"
        " AND t_transfer.job_finished is NULL"
        " AND t_job.job_finished is NULL";
}
const char * create_get_full_search_vo(){
    return "SELECT t_transfer.request_id"
        ", t_transfer.file_id"
        ", t_transfer.transfer_id"
        ", t_transfer.job_id"
        ", t_transfer.transfer_state"
        ", t_transfer.transfer_type"
        ", t_transfer.source_turl"
        ", t_transfer.dest_turl"
        ", t_transfer.transfer_time"
        ", t_transfer.prepare_time"
        ", t_transfer.duration"
        ", t_transfer.src_prep_duration"
        ", t_transfer.dest_prep_duration"
        ", t_transfer.tx_duration"
        ", t_transfer.src_final_duration"
        ", t_transfer.dest_final_duration"
        ", t_transfer.bytes_written"
        ", t_transfer.throughput"
        ", t_transfer.reason_class"
        ", t_transfer.reason"
        ", t_transfer.filesize"
        ", t_transfer.source_srm_token"
        ", t_transfer.dest_srm_token"
        ", t_transfer.source_host"
        ", t_transfer.dest_host"
        ", t_transfer.error_scope"
        ", t_transfer.error_phase"
        ", t_transfer.finish_time"
        " FROM t_transfer,t_job"
        " WHERE t_transfer.file_id  = :1"
        " AND t_transfer.request_id = :2"
        " AND t_transfer.job_id = t_job.job_id"
        " AND t_job.vo_name = :3";
}
} // namespace
Statement * TransferStatements::createGet_VO(
    OracleDAOContext&   ctx,
    const bool&         finished) /*throw (DAOLogicError,DAOException)*/{

    if (finished){
        return create_statement(ctx,GET_FULL_SEARCH_STATEMENT_VO_TAG,create_get_full_search_vo,"Get Full Search (VO)");
    } else {
        return create_statement(ctx,GET_STATEMENT_VO_TAG,create_get_vo,"Get (VO)");
    }
}

/*
 * createGet_Channel
 *
 * Prepare Statement For TransferDAO::get, Channel View
 *
 * Bind Parameters:
 *  :1 - File ID
 *  :2 - Request ID
 *  :3 - Channel Name
 */
namespace{
const char * create_get_channel(){
    return "SELECT t_transfer.request_id"
        ", t_transfer.file_id"
        ", t_transfer.transfer_id"
        ", t_transfer.job_id"
        ", t_transfer.transfer_state"
        ", t_transfer.transfer_type"
        ", t_transfer.source_turl"
        ", t_transfer.dest_turl"
        ", t_transfer.transfer_time"
        ", t_transfer.prepare_time"
        ", t_transfer.duration"
        ", t_transfer.src_prep_duration"
        ", t_transfer.dest_prep_duration"
        ", t_transfer.tx_duration"
        ", t_transfer.src_final_duration"
        ", t_transfer.dest_final_duration"
        ", t_transfer.bytes_written"
        ", t_transfer.throughput"
        ", t_transfer.reason_class"
        ", t_transfer.reason"
        ", t_transfer.filesize"
        ", t_transfer.source_srm_token"
        ", t_transfer.dest_srm_token"
        ", t_transfer.source_host"
        ", t_transfer.dest_host"
        ", t_transfer.error_scope"
        ", t_transfer.error_phase"
        ", t_transfer.finish_time"
        " FROM t_transfer,t_job"
        " WHERE t_transfer.file_id  = :1"
        " AND t_transfer.request_id = :2"
        " AND t_transfer.job_id = t_job.job_id"
        " AND t_job.channel_name = :3"
        " AND t_transfer.job_finished is NULL"
        " AND t_job.job_finished is NULL";
}

const char * create_get_full_search_channel(){
    return "SELECT t_transfer.request_id"
        ", t_transfer.file_id"
        ", t_transfer.transfer_id"
        ", t_transfer.job_id"
        ", t_transfer.transfer_state"
        ", t_transfer.transfer_type"
        ", t_transfer.source_turl"
        ", t_transfer.dest_turl"
        ", t_transfer.transfer_time"
        ", t_transfer.prepare_time"
        ", t_transfer.duration"
        ", t_transfer.src_prep_duration"
        ", t_transfer.dest_prep_duration"
        ", t_transfer.tx_duration"
        ", t_transfer.src_final_duration"
        ", t_transfer.dest_final_duration"
        ", t_transfer.bytes_written"
        ", t_transfer.throughput"
        ", t_transfer.reason_class"
        ", t_transfer.reason"
        ", t_transfer.filesize"
        ", t_transfer.source_srm_token"
        ", t_transfer.dest_srm_token"
        ", t_transfer.source_host"
        ", t_transfer.dest_host"
        ", t_transfer.error_scope"
        ", t_transfer.error_phase"
        ", t_transfer.finish_time"
        " FROM t_transfer,t_job"
        " WHERE t_transfer.file_id  = :1"
        " AND t_transfer.request_id = :2"
        " AND t_transfer.job_id = t_job.job_id"
        " AND t_job.channel_name = :3";
}
}

Statement * TransferStatements::createGet_Channel(
    OracleDAOContext&   ctx,
    const bool&         finished) /*throw (DAOLogicError,DAOException)*/{
        
    if (finished){
        return create_statement(ctx,GET_FULL_SEARCH_STATEMENT_CH_TAG,create_get_full_search_channel,"Get Full Search (Channel)");
    } else {
        return create_statement(ctx,GET_STATEMENT_CH_TAG,create_get_channel,"Get (Channel)");
    }
}

/*
 * createUpdate_Channel
 *
 * Prepare Statement For TransferDAO::update, Channel View. Returns 0 if transfer doesn't need to be updated
 */
Statement * TransferStatements::createUpdate_Channel(
    OracleDAOContext&       ctx, 
    const model::Transfer&  tx, 
    const std::string&      ch_name) /*throw (DAOLogicError,DAOException)*/{

    Statement * stmt = 0;        
    try {            
        // Update Transfer Properties
        // -------------------------------
        // | Column              | RO/RW |
        // -------------------------------
        // | request_id          |  RO   |
        // | file_id             |  RO   |
        // | transfer_id         |  RO   |
        // | job_id              |  RO   |
        // | state               |  RW   |
        // | transfer_type       |  RO   |
        // | source_turl         |  RW   |
        // | dest_turl           |  RW   |
        // | transfer_time       |  RW   |
        // | prepare_time        |  RW   |
        // | duration            |  RW   |
        // | src_prep_duration   |  RW   |
        // | dest_prep_duration  |  RW   |
        // | transfer_duration   |  RW   |
        // | src_final_duration  |  RW   |
        // | dest_final_duration |  RW   |
        // | bytes_written       |  RW   |
        // | throughput          |  RW   |
        // | reason_class        |  RW   |
        // | reason              |  RW   |
        // | filesize            |  RW   |
        // | source_srm_token    |  RW   |
        // | dest_srm_token      |  RW   |
        // | source_host         |  RW   |
        // | dest_host           |  RW   |
        // | error_scope         |  RW   |
        // | error_phase         |  RW   |
        // | finish_time         |  RW   |
        // -------------------------------
        // Prepare Update Statement
        unsigned int count = 4;
        std::stringstream query_stmt;
        query_stmt << "UPDATE ( SELECT t_transfer.* "
                        " FROM t_transfer, t_job"
                        " WHERE t_transfer.file_id  = :1"
                        " AND t_transfer.request_id = :2"
                        " AND t_transfer.job_id = t_job.job_id"
                        " AND t_job.channel_name = :3 "
                        " AND t_transfer.job_finished is NULL"
                        " AND t_job.job_finished is NULL"
                        ") t_transfer SET";
        // State
        if(tx.state.isSet()){
            query_stmt << ((4 == count)?"":",") << " t_transfer.transfer_state = :" << count;
            ++count;
        }
        // Source Turl
        if(tx.sourceTurl.isSet()){
            query_stmt << ((4 == count)?"":",") << " t_transfer.source_turl = :" << count;
            ++count;
        }
        // Dest Turl
        if(tx.destTurl.isSet()){
            query_stmt << ((4 == count)?"":",") << " t_transfer.dest_turl = :" << count;
            ++count;
        }
        // transfer_time
        if(tx.transferTime.isSet()){
            query_stmt << ((4 == count)?"":",") << " t_transfer.transfer_time = :" << count;
            ++count;
        }
        // prepare_time
        if(tx.prepareTime.isSet()){
            query_stmt << ((4 == count)?"":",") << " t_transfer.prepare_time = :" << count;
            ++count;
        }
        // duration
        if(tx.duration.isSet()){
            query_stmt << ((4 == count)?"":",") << " t_transfer.duration = :" << count;
            ++count;
        }
        // src_prep_duration
        if(tx.srcPrepDuration.isSet()){
            query_stmt << ((4 == count)?"":",") << " t_transfer.src_prep_duration = :" << count;
            ++count;
        }
        // dest_prep_duration
        if(tx.dstPrepDuration.isSet()){
            query_stmt << ((4 == count)?"":",") << " t_transfer.dest_prep_duration = :" << count;
            ++count;
        }
        // tx_duration
        if(tx.txDuration.isSet()){
            query_stmt << ((4 == count)?"":",") << " t_transfer.tx_duration = :" << count;
            ++count;
        }
        // src_final_duration
        if(tx.srcFinalDuration.isSet()){
            query_stmt << ((4 == count)?"":",") << " t_transfer.src_final_duration = :" << count;
            ++count;
        }
        // dest_final_duration
        if(tx.dstFinalDuration.isSet()){
            query_stmt << ((4 == count)?"":",") << " t_transfer.dest_final_duration = :" << count;
            ++count;
        }
        // bytes_written
        if(tx.bytesWritten.isSet()){
            query_stmt << ((4 == count)?"":",") << " t_transfer.bytes_written = :" << count;
            ++count;
        }
        // throughput
        if(tx.throughput.isSet()){
            query_stmt << ((4 == count)?"":",") << " t_transfer.throughput = :" << count;
            ++count;
        }
        // reason_class
        if(tx.reasonClass.isSet()){
            query_stmt << ((4 == count)?"":",") << " t_transfer.reason_class = :" << count;
            ++count;
        }
        // reason
        if(tx.reason.isSet()){
            query_stmt << ((4 == count)?"":",") << " t_transfer.reason = :" << count;
            ++count;
        }
        // filesize
        if(tx.fileSize.isSet()){
            query_stmt << ((4 == count)?"":",") << " t_transfer.filesize = :" << count;
            ++count;
        } 
        // source_srm_token
        if(tx.srcSrmToken.isSet()){
            query_stmt << ((4 == count)?"":",") << " t_transfer.source_srm_token = :" << count;
            ++count;
        } 
        // dest_srm_token
        if(tx.destSrmToken.isSet()){
            query_stmt << ((4 == count)?"":",") << " t_transfer.dest_srm_token = :" << count;
            ++count;
        } 
        // source_host
        if(tx.srcHost.isSet()){
            query_stmt << ((4 == count)?"":",") << " t_transfer.source_host = :" << count;
            ++count;
        } 
        // dest_host
        if(tx.destHost.isSet()){
            query_stmt << ((4 == count)?"":",") << " t_transfer.dest_host = :" << count;
            ++count;
        } 
        // error_scope
        if(tx.errorScope.isSet()){
            query_stmt << ((4 == count)?"":",") << " t_transfer.error_scope = :" << count;
            ++count;
        } 
        // error_phase
        if(tx.errorPhase.isSet()){
            query_stmt << ((4 == count)?"":",") << " t_transfer.error_phase = :" << count;
            ++count;
        }
        // finish_time
        if(tx.finishTime.isSet()){
            query_stmt << ((4 == count)?"":",") << " t_transfer.finish_time = :" << count;
            ++count;
        }
        if(count > 4){
            m_log_trace("Preparing Statement: " << query_stmt.str());
            // Create Statements
            stmt = ctx.connection().createStatement(query_stmt.str());
            
            // RAII
            StatementPtr stmt_ptr(ctx.connection(),stmt);
            
            // Set the value of the properties
            stmt->setString(1,tx.fileId());
            stmt->setString(2,tx.requestId());
            stmt->setString(3,ch_name);
            count = 4;
            // State
            if(tx.state.isSet()){
                stmt->setString(count++,translateTransferState(tx.state()));
            } 
            // Source Turl
            if(tx.sourceTurl.isSet()){
                stmt->setString(count++,tx.sourceTurl());
            }
            // Dest Turl
            if(tx.destTurl.isSet()){
                stmt->setString(count++,tx.destTurl());
            }
            // transfer_time
            if(tx.transferTime.isSet()){
                if((time_t)-1 != tx.transferTime()){
                    stmt->setTimestamp(count++,ctx.toTimestamp(tx.transferTime()));
                } else {
                    stmt->setNull(count++ ,OCCITIMESTAMP);
                }
            }
            // prepare_time
            if(tx.prepareTime.isSet()){
                if((time_t)-1 != tx.prepareTime()){
                    stmt->setTimestamp(count++,ctx.toTimestamp(tx.prepareTime()));
                } else {
                    stmt->setNull(count++ ,OCCITIMESTAMP);
                }
            }
            // duration
            if(tx.duration.isSet()){
                stmt->setDouble(count++,tx.duration());
            }
            // src_prep_duration
            if(tx.srcPrepDuration.isSet()){
                stmt->setDouble(count++,tx.srcPrepDuration());
            }
            // dest_prep_duration
            if(tx.dstPrepDuration.isSet()){
                stmt->setDouble(count++,tx.dstPrepDuration());
            }
            // tx_duration
            if(tx.txDuration.isSet()){
                stmt->setDouble(count++,tx.txDuration());
            }
            // src_final_duration
            if(tx.srcFinalDuration.isSet()){
                stmt->setDouble(count++,tx.srcFinalDuration());
            }
            // dest_final_duration
            if(tx.dstFinalDuration.isSet()){
                stmt->setDouble(count++,tx.dstFinalDuration());
            }
            // bytes_written
            if(tx.bytesWritten.isSet()){
                stmt->setNumber(count++,ctx.toNumber(tx.bytesWritten()));
            }
            // throughput
            if(tx.throughput.isSet()){
                stmt->setDouble(count++,tx.throughput());
            }
            // reason_class
            if(tx.reasonClass.isSet()){
                stmt->setString(count++,translateErrorCategory(tx.reasonClass()));
            }
            // reason
            if(tx.reason.isSet()){
                if(tx.reason().length() < MAX_T_TRANSFER_REASON_LEN){
                    stmt->setString(count++,tx.reason());
                } else {
                    stmt->setString(count++,tx.reason().substr(0,MAX_T_TRANSFER_REASON_LEN - 1));
                }
            }
            // filesize
            if(tx.fileSize.isSet()){
                stmt->setNumber(count++,ctx.toNumber(tx.fileSize()));
            }             
            // source_srm_token
            if(tx.srcSrmToken.isSet()){
                stmt->setString(count++,tx.srcSrmToken());
            }
            // dest_srm_token
            if(tx.destSrmToken.isSet()){
                stmt->setString(count++,tx.destSrmToken());
            }
            // source_host
            if(tx.srcHost.isSet()){
                stmt->setString(count++,tx.srcHost());
            }
            // dest_host
            if(tx.destHost.isSet()){
                stmt->setString(count++,tx.destHost());
            }
            // error_scope
            if(tx.errorScope.isSet()){
                stmt->setString(count++,translateErrorScope(tx.errorScope()));
            } 
            // error_phase
            if(tx.errorPhase.isSet()){
                stmt->setString(count++,translateErrorPhase(tx.errorPhase()));
            }
            // finish_time
            if(tx.finishTime.isSet()){
                if((time_t)-1 != tx.finishTime()){
                    stmt->setTimestamp(count++,ctx.toTimestamp(tx.finishTime()));
                } else {
                    stmt->setNull(count++ ,OCCITIMESTAMP);
                }
            }
            
            // Release the Pointer
            stmt_ptr.release();
        }
    } catch(const SQLException& exc){
        m_log_error("Got Exception in creating Update Statement: [0x" << exc.getErrorCode() << "] " << exc.getMessage());
        throw DAOException("Failed to prepare Statement");
    }
    // Don't Check Postcondition (Null is allowed)   
    return stmt;
}

/*
 * createGetByFileId_VO
 *
 * Prepare GetByFileId Statement, VO View
 *
 * Bind Parameters:
 *  :1 - File ID
 *  :2 - VO Name
 *  :3 - Limit
 *  :4 - Limit
 */
Statement * TransferStatements::createGetByFileId_VO(OracleDAOContext& ctx) /*throw (DAOLogicError,DAOException)*/{
    
    // Get Tag
    const char * const tag = GET_BY_FILEID_STATEMENT_VO_TAG;

    // Get Tag
    Statement * stmt = 0; 
    try {
        // Check if the statement is already cached
        if(false == ctx.connection().isCached("",tag)){
            // Prepare Statement String
            const char * const query_stmt =
                "SELECT * FROM ("
                    "SELECT t_transfer.request_id"
                    " FROM t_transfer, t_job"
                    " WHERE t_transfer.file_id = :1"
                    " AND t_transfer.job_id = t_job.job_id"
                    " AND t_job.vo_name = :2"
                    " AND t_transfer.job_finished is NULL"
                    " AND t_job.job_finished is NULL"
                    " ORDER BY t_transfer.transfer_time DESC"
                            ", t_transfer.request_id"
                ") WHERE (:3=0 OR ROWNUM<=:4)";
            m_log_trace("Preparing Statement: " << query_stmt);
            // Create Statements
            stmt = ctx.connection().createStatement(query_stmt,tag);
        } else {
            // Create Statements
            stmt = ctx.connection().createStatement("",tag);
        }
    } catch(const SQLException& exc){
        m_log_error("Got Exception in Creating GetByFileId Statement: [0x" << exc.getErrorCode() << "] " << exc.getMessage());
        throw DAOException("Failed to prepare Statement");
    }
    // Check Postcondition
    if(0 == stmt){
        throw DAOLogicError("Failed to prepare Statement - Nullpointer");
    }
    return stmt;
}

/*
 * createGetByFileId_Channel
 *
 * Prepare GetByFileId Statement, Channel View
 *
 * Bind Parameters:
 *  :1 - File ID
 *  :2 - Channel Name
 *  :3 - Limit
 *  :4 - Limit
 */
Statement * TransferStatements::createGetByFileId_Channel(OracleDAOContext& ctx) /*throw (DAOLogicError,DAOException)*/{
    
    // Get Tag
    const char * const tag = GET_BY_FILEID_STATEMENT_CH_TAG;

    Statement * stmt = 0; 
    try {
        // Check if the statement is already cached
        if(false == ctx.connection().isCached("",tag)){
            // Prepare Statement String
            const char * const query_stmt =
                "SELECT * FROM ("
                    "SELECT t_transfer.request_id"
                    " FROM t_transfer, t_job"
                    " WHERE t_transfer.file_id = :1"
                    " AND t_transfer.job_id = t_job.job_id"
                    " AND t_job.channel_name = :2"
                    " AND t_transfer.job_finished is NULL"
                    " AND t_job.job_finished is NULL"
                    " ORDER BY t_transfer.transfer_time DESC"
                            ", t_transfer.request_id"
                ") WHERE (:3=0 OR ROWNUM<=:4)";
            m_log_trace("Preparing Statement: " << query_stmt);
            // Create Statements
            stmt = ctx.connection().createStatement(query_stmt,tag);
        } else {
            // Create Statements
            stmt = ctx.connection().createStatement("",tag);
        }
    } catch(const SQLException& exc){
        m_log_error("Got Exception in Creating GetByFileId Statement: [0x" << exc.getErrorCode() << "] " << exc.getMessage());
        throw DAOException("Failed to prepare Statement");
    }
    // Check Postcondition
    if(0 == stmt){
        throw DAOLogicError("Failed to prepare Statement - Nullpointer");
    }
    return stmt;
}

/*
 * createGetByRequestId_VO
 *
 * Prepare GetByRequestId Statement, VO View
 *
 * Bind Parameters:
 *  :1 - Request ID
 *  :2 - VO Name
 */
Statement * TransferStatements::createGetByRequestId_VO(OracleDAOContext& ctx) /*throw (DAOLogicError,DAOException)*/{
    
    // Get Tag
    const char * const tag = GET_BY_REQID_STATEMENT_VO_TAG;

    // Get Tag
    Statement * stmt = 0; 
    try {
        // Check if the statement is already cached
        if(false == ctx.connection().isCached("",tag)){
            // Prepare Statement String
            const char * const query_stmt =
                "SELECT t_transfer.file_id"
                " FROM t_transfer, t_job"
                " WHERE t_transfer.request_id = :1"
                " AND t_transfer.job_id = t_job.job_id"
                " AND t_job.vo_name = :2"
                " AND t_transfer.job_finished is NULL"
                " AND t_job.job_finished is NULL"
                " ORDER BY t_transfer.file_id";
            m_log_trace("Preparing Statement: " << query_stmt);
            // Create Statements
            stmt = ctx.connection().createStatement(query_stmt,tag);
        } else {
            // Create Statements
            stmt = ctx.connection().createStatement("",tag);
        }
    } catch(const SQLException& exc){
        m_log_error("Got Exception in Creating GetByRequestId Statement: [0x" << exc.getErrorCode() << "] " << exc.getMessage());
        throw DAOException("Failed to prepare Statement");
    }
    // Check Postcondition
    if(0 == stmt){
        throw DAOLogicError("Failed to prepare Statement - Nullpointer");
    }
    return stmt;
}

/*
 * createGetByRequestId_Channel
 *
 * Prepare GetByRequestId Statement, Channel View
 *
 * Bind Parameters:
 *  :1 - File ID
 *  :2 - Channel Name
 */
Statement * TransferStatements::createGetByRequestId_Channel(OracleDAOContext& ctx) /*throw (DAOLogicError,DAOException)*/{
    
    // Get Tag
    const char * const tag = GET_BY_REQID_STATEMENT_CH_TAG;

    Statement * stmt = 0; 
    try {
        // Check if the statement is already cached
        if(false == ctx.connection().isCached("",tag)){
            // Prepare Statement String
            const char * const query_stmt =
                "SELECT t_transfer.file_id"
                " FROM t_transfer, t_job"
                " WHERE t_transfer.request_id = :1"
                " AND t_transfer.job_id = t_job.job_id"
                " AND t_job.channel_name = :2"
                " AND t_transfer.job_finished is NULL"
                " AND t_job.job_finished is NULL"
                " ORDER BY t_transfer.file_id";
            m_log_trace("Preparing Statement: " << query_stmt);
            // Create Statements
            stmt = ctx.connection().createStatement(query_stmt,tag);
        } else {
            // Create Statements
            stmt = ctx.connection().createStatement("",tag);
        }
    } catch(const SQLException& exc){
        m_log_error("Got Exception in Creating GetByRequestId Statement: [0x" << exc.getErrorCode() << "] " << exc.getMessage());
        throw DAOException("Failed to prepare Statement");
    }
    // Check Postcondition
    if(0 == stmt){
        throw DAOLogicError("Failed to prepare Statement - Nullpointer");
    }
    return stmt;
}

/*
 * createCreate_Channel
 *
 * Return the statement for the TransferDAO::create, Channel View
 */
Statement* TransferStatements::createCreate_Channel(OracleDAOContext& ctx, const model::Transfer& tx) /*throw (DAOLogicError,DAOException)*/{
    
    // Get Tag
    Statement * stmt = 0;        
    try {
        // Prepare Update Statement
        unsigned int count = 6;
        std::stringstream query_stmt;
        query_stmt << "INSERT INTO t_transfer("
           " file_id"
           ", request_id"
           ", transfer_id"
           ", job_id"
           ", transfer_state";
        // Transfer Type
        if(false == tx.transferType().empty()){
            query_stmt << ", transfer_type";++count;
        }
        // Source Turl
        if(false == tx.sourceTurl().empty()){
            query_stmt << ", source_turl";++count;
        }
        // Dest Turl
        if(false == tx.destTurl().empty()){
            query_stmt << ", dest_turl";++count;
        }
        // transfer_time
        if((time_t)-1 != tx.transferTime()){
            query_stmt << ", transfer_time";++count;
        }
        // prepare_time
        if((time_t)-1 != tx.prepareTime()){
            query_stmt << ", prepare_time";++count;
        }
        // duration
        if(0 != tx.duration()){
            query_stmt << ", duration";++count;
        }
        // src_prep_duration
        if(0 != tx.srcPrepDuration()){
            query_stmt << ", src_prep_duration";++count;
        }
        // dest_prep_duration
        if(0 != tx.dstPrepDuration()){
            query_stmt << ", dest_prep_duration";++count;
        }
        // tx_duration
        if(0 != tx.txDuration()){
            query_stmt << ", tx_duration";++count;
        }
        // src_final_duration
        if(0 != tx.srcFinalDuration()){
            query_stmt << ", src_final_duration";++count;
        }
        // dest_final_duration
        if(0 != tx.dstFinalDuration()){
            query_stmt << ", dest_final_duration";++count;
        }
        // bytes_written
        if(-1 != tx.bytesWritten()){
            query_stmt << ", bytes_written";++count;
        }
        // throughput
        if(0 != tx.throughput()){
            query_stmt << ", throughput";++count;
        }
        // reason_class
        if(model::Error::UNDEF != tx.reasonClass()){
            query_stmt << ", reason_class";++count;
        }
        // reason
        if(false == tx.reason().empty()){
            query_stmt << ", reason";++count;
        }
        // filesize
        if(0 != tx.fileSize()){
            query_stmt << ", filesize";++count;
        }
        // error_scope
        if(model::Error::S_UNDEF != tx.errorScope()){
            query_stmt << ", error_scope";++count;
        }
        // error_phase
        if(model::Error::P_UNDEF != tx.errorPhase()){
            query_stmt << ", error_phase";++count;
        }
        // finish_time
        if((time_t)-1 != tx.finishTime()){
            query_stmt << ", finish_time";++count;
        }
        
        query_stmt << " ) VALUES ("
            " :1"
            ", :2"
            ", :3"
            ", :4"
            ", :5";
        for(unsigned int i = 6; i < count; ++i){
            query_stmt << ", :" << i;
        }
        query_stmt << " )";
        m_log_trace("Preparing Statement: " << query_stmt.str());
        // Create Statements
        stmt = ctx.connection().createStatement(query_stmt.str());
        
        // RAII
        StatementPtr stmt_ptr(ctx.connection(),stmt);
            
        // Set the value of the properties
        stmt->setString(1,tx.fileId());
        stmt->setString(2,tx.requestId());
        stmt->setInt(3,tx.transferId());
        stmt->setString(4,tx.jobId());
        stmt->setString(5,translateTransferState(tx.state()));
        count = 6;
        // Transfer Type
        if(false == tx.transferType().empty()){
            stmt->setString(count++,tx.transferType());
        }
        // Source Turl
        if(false == tx.sourceTurl().empty()){
            stmt->setString(count++,tx.sourceTurl());
        }
        // Dest Turl
        if(false == tx.destTurl().empty()){
            stmt->setString(count++,tx.destTurl());
        }
        // transfer_time
        if((time_t)-1 != tx.transferTime()){
            stmt->setTimestamp(count++,ctx.toTimestamp(tx.transferTime()));
        }
        // prepare_time
        if((time_t)-1 != tx.prepareTime()){
            stmt->setTimestamp(count++,ctx.toTimestamp(tx.prepareTime()));
        }
        // duration
        if(0 != tx.duration()){
            stmt->setDouble(count++,tx.duration());
        }
        // src_prep_duration
        if(0!= tx.srcPrepDuration()){
            stmt->setDouble(count++,tx.srcPrepDuration());
        }
        // dest_prep_duration
        if(0 != tx.dstPrepDuration()){
            stmt->setDouble(count++,tx.dstPrepDuration());
        }
        // tx_duration
        if(0 != tx.txDuration()){
            stmt->setDouble(count++,tx.txDuration());
        }
        // src_final_duration
        if(0 != tx.srcFinalDuration()){
            stmt->setDouble(count++,tx.srcFinalDuration());
        }
        // dest_final_duration
        if(0 != tx.dstFinalDuration()){
            stmt->setDouble(count++,tx.dstFinalDuration());
        }
        // bytes_written
        if(-1 != tx.bytesWritten()){
            stmt->setNumber(count++,ctx.toNumber(tx.bytesWritten()));
        }
        // throughput
        if(0 != tx.throughput()){
            stmt->setDouble(count++,tx.throughput());
        }
        // reason_class
        if(model::Error::UNDEF != tx.reasonClass()){
            stmt->setString(count++,translateErrorCategory(tx.reasonClass()));
        }
        // reason
        if(false == tx.reason().empty()){
            if(tx.reason().length() < MAX_T_TRANSFER_REASON_LEN){
                stmt->setString(count++,tx.reason());
            } else {
                stmt->setString(count++,tx.reason().substr(0,MAX_T_TRANSFER_REASON_LEN - 1));
            }
        }
        // filesize
        if(0 != tx.fileSize()){
            stmt->setNumber(count++,ctx.toNumber(tx.fileSize()));
        }    
        // error_scope
        if(model::Error::S_UNDEF != tx.errorScope()){
            stmt->setString(count++,translateErrorScope(tx.errorScope()));
        }
        // error_phase
        if(model::Error::P_UNDEF != tx.errorPhase()){
            stmt->setString(count++,translateErrorPhase(tx.errorPhase()));
        }
        // finish_time
        if((time_t)-1 != tx.finishTime()){
            stmt->setTimestamp(count++,ctx.toTimestamp(tx.finishTime()));
        }
        // Release the Pointer
        stmt_ptr.release();
    } catch(const SQLException& exc){
        m_log_error("Got Exception in creating Insert Statement: [0x" << exc.getErrorCode() << "] " << exc.getMessage());
        throw DAOException("Failed to prepare Statement");
    }
    // Check Postcondition
    if(0 == stmt){
        throw DAOLogicError("Failed to prepare Statement - Nullpointer");
    }
    return stmt;
}

/**
 * Return the statement for the TransferDAO::countActiveTransfers, Channel View
 *
 * Bind Parameters:
 *  :1 - Channel Name
 *  :2 - VO Name (only if per_vo == true)
 */
Statement* TransferStatements::createCountActiveTransfers_Channel(OracleDAOContext& ctx, bool per_vo) /*throw (DAOLogicError,DAOException)*/{
        
    // Get Tag
    const char * const tag = (true == per_vo)?COUNT_ACTIVE_PER_VO_STATEMENT_CH_TAG:COUNT_ACTIVE_STATEMENT_CH_TAG;

    Statement * stmt = 0;        
    try {
        // Check if the statement is already cached
        if(false == ctx.connection().isCached("",tag)){
            // Prepare Update Statement
            std::stringstream query_stmt;
            query_stmt <<
               "SELECT /*+ INDEX(t_transfer transfer_transfer_state) */ COUNT(*)"
               " FROM t_transfer, t_job"
               " WHERE t_transfer.job_id = t_job.job_id"
               " AND t_job.channel_name = :1"
               " AND t_transfer.transfer_state IN ("
                    "'Submitted','Processing','Transferred')"
               " AND t_job.job_finished is NULL"
               " AND t_transfer.job_finished is NULL";
            if(true == per_vo){
                query_stmt << " AND t_job.vo_name = :2";
            }
            m_log_trace("Preparing Statement: " << query_stmt.str());
            // Create Statement
            stmt = ctx.connection().createStatement(query_stmt.str(),tag);
        } else {
            // Create Statement
            stmt = ctx.connection().createStatement("",tag);
        }
    } catch(const SQLException& exc){
        m_log_error("Got Exception in creating Insert Statement: [0x" << exc.getErrorCode() << "] " << exc.getMessage());
        throw DAOException("Failed to prepare Statement");
    }
    // Check Postcondition
    if(0 == stmt){
        throw DAOLogicError("Failed to prepare Statement - Nullpointer");
    }
    return stmt;
}

/*
 * createGetActiveTransfers_Channel
 *
 * Return the statement for the TransferDAO::getTransfersToCancel, Channel View
 *
 * Bind Parameters:
 *  :1 - Channel Name
 */
Statement* TransferStatements::createGetActiveTransfers_Channel(agents::dao::oracle::OracleDAOContext& ctx) /*throw (DAOLogicError,DAOException)*/{
        
    // Get Tag
    const char * const tag = GET_ACTIVE_STATEMENT_CH_TAG;

    Statement * stmt = 0;        
    try {
        // Check if the statement is already cached
        if(false == ctx.connection().isCached("",tag)){
            
            // Prepare Update Statement
            const char * const query_stmt =
                "SELECT t_transfer.file_id, t_transfer.request_id"
                " FROM t_transfer, t_file, t_job"
                " WHERE t_transfer.file_id = t_file.file_id"
                " AND t_file.job_id = t_job.job_id"
                " AND t_job.channel_name = :1"
                " AND t_job.job_state = 'Active'"
                " AND t_file.file_state = 'Active'"
                " AND t_transfer.transfer_state IN ("
                      "'Received','Preparing','Ready','Submitted'"
                     ",'Processing','Transferred')"
                " AND t_job.job_finished is NULL"
                " AND t_file.job_finished is NULL"
                " AND t_transfer.job_finished is NULL"
                 " ORDER BY t_transfer.request_id"
                      " , t_transfer.transfer_id"
                      " , t_transfer.transfer_time";

            m_log_trace("Preparing Statement: " << query_stmt);
            // Create Statement
            stmt = ctx.connection().createStatement(query_stmt,tag);
        } else {
            // Create Statement
            stmt = ctx.connection().createStatement("",tag);
        }
    } catch(const SQLException& exc){
        m_log_error("Got Exception in creating GetActiveTransfers Statement: [0x" << exc.getErrorCode() << "] " << exc.getMessage());
        throw DAOException("Failed to prepare Statement");
    }
    // Check Postcondition
    if(0 == stmt){
        throw DAOException("Failed to prepare Statement - Nullpointer");
    }
    return stmt;
}
