/**
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *  GLite Data Transfer Agents: VO_ChannelDAO Class
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *           Paolo Tedesco <paolo.tedesco@cern.ch>
 *  Version info: $Id: VO_ChannelDAO.cpp,v 1.8 2008/11/13 10:43:56 ptedesco Exp $
 *  Release: $Name:  $
 *
 */
#include "VO_ChannelDAO.h"
#include "OracleChannelDAO.h"
#include "ChannelStatements.h"
#include <glite/data/agents/dao/oracle/StatementPtr.h>
#include <glite/data/agents/dao/oracle/ResultSetPtr.h>

using namespace glite::data::transfer::agent;
using namespace glite::data::transfer::agent::dao;
using namespace glite::data::transfer::agent::dao::oracle;
using glite::data::agents::dao::DAOLogicError;
using glite::data::agents::dao::DAOException;
using glite::data::agents::dao::oracle::OracleDAOContext;
using glite::data::agents::dao::oracle::StatementPtr;
using glite::data::agents::dao::oracle::ResultSetPtr;
using namespace ::oracle::occi;

/**
 * VO_ChannelDAO
 * Constructor
 */
VO_ChannelDAO::VO_ChannelDAO(OracleDAOContext& ctx, const std::string& vo_name,const std::string& category):
        m_logger(category),
        m_channelDaoImpl(new OracleChannelDAO(ctx,category)),
        m_voName(vo_name){
}

/**
 * get
 * Return the Channel with the given ChannelId
 */
model::Channel * VO_ChannelDAO::get(const std::string& name) /*throw (DAOLogicError,DAOException)*/{
   
    m_log_trace("Calling ChannelDAO::get");
    
    model::Channel * channel = 0;

    // Get Context
    OracleDAOContext& ctx = m_channelDaoImpl->ctx();
    
    // Get Statement
    StatementPtr stmt(ctx.connection(),ChannelStatements::instance().createGet_VO(ctx));
    try{
        // Set Parameters
        stmt->setString(1,name);
        stmt->setString(2,m_voName);
        
        // Get Data
        channel = m_channelDaoImpl->get(*stmt);
    } catch(const SQLException& exc){
        m_log_error("Got SQLException in Executing Get: [0x" << exc.getErrorCode() << "] " << exc.getMessage());
        m_log_debug("Query was: " << stmt->getSQL());
        throw DAOException("Failed to get Channel - SQL Exception");
    } catch(const DAOException& exc){
        m_log_error("Got DAOException in Executing Get: " << exc.what());
        m_log_debug("Query was: " << stmt->getSQL());
        throw;
    } 
    
    m_log_trace("ChannelDAO::get Called");
    
    return channel;
}

/*
 * getBySites
 *
 * Return the name of the Channel based on the provided Domain endpoints
 */
std::string VO_ChannelDAO::getBySites(const std::string& ssite, const std::string& dsite) /*throw (DAOLogicError,DAOException)*/{

    std::string channel_name;
    
    m_log_trace("Entering VO_ChannelDAO::getBySites");

    // Get Context
    OracleDAOContext& ctx = m_channelDaoImpl->ctx();
    
    // Get Statement
    StatementPtr stmt(ctx.connection(),ChannelStatements::instance().createGetBySites_VO(ctx));
    try{
        // Set Parameters
        stmt->setString(1,ssite);
        stmt->setString(2,dsite);
        stmt->setString(3,m_voName);
        
        // Execute Query
        ResultSetPtr rs(*stmt,stmt->executeQuery());
        if(0 != rs->next()){
            // Get Channel Name
            channel_name = rs->getString(1);
        }
    } catch(const SQLException& exc){
        m_log_error("Got SQLException in Setting Parameters: [0x" << exc.getErrorCode() << "] " << exc.getMessage());
        m_log_debug("Query was: " << stmt->getSQL());
        throw DAOException("Failed to get Channel Name - SQL Exception");
    } catch(...){
        m_log_warn("Failed to get Channel Name");
        throw;
    }

    m_log_trace("VO_ChannelDAO::getBySites returned " << channel_name);

    return channel_name;    
}


/*
 * hasShare
 *
 * Returns true if the VO has a share for the given channel, false otherwise
 */
bool VO_ChannelDAO::hasShare(const std::string& channel_name) /*throw (DAOLogicError,DAOException)*/{
    bool result = false;
    
    m_log_trace("Entering VO_ChannelDAO::hasShare");

    // Get Context
    OracleDAOContext& ctx = m_channelDaoImpl->ctx();
    
    // Get Statement
    StatementPtr stmt(ctx.connection(),ChannelStatements::instance().createHasShare_VO(ctx));
    try{
        // Set Parameters
        stmt->setString(1,channel_name);
        stmt->setString(2,m_voName);
        
        // Execute Query
        ResultSetPtr rs(*stmt,stmt->executeQuery());
        if(0 != rs->next()){
            // VO has a share
            result = true;
        }
    } catch(const SQLException& exc){
        m_log_error("Got SQLException in Setting Parameters: [0x" << exc.getErrorCode() << "] " << exc.getMessage());
        m_log_debug("Query was: " << stmt->getSQL());
        throw DAOException("Failed to check Channel Share - SQL Exception");
    } catch(...){
        m_log_warn("Failed to check Channel Share");
        throw;
    }

    m_log_trace("VO_ChannelDAO::hasShare returned " << str_bool(result));

    return result;
}

/*
 * isBlacklisted
 *
 * Return true if the SE or the site is backlisted. If true, also return the reason
 */
std::pair<bool,std::string> VO_ChannelDAO::isBlacklisted(const std::string& host, const std::string& site) /*throw (DAOLogicError,DAOException)*/{
    std::pair<bool,std::string> result;
    result.first = false;
    
    m_log_trace("Entering VO_ChannelDAO::isBlacklisted");

    // Get Context
    OracleDAOContext& ctx = m_channelDaoImpl->ctx();
    
    // Get Statement
    StatementPtr stmt(ctx.connection(),ChannelStatements::instance().createIsBlacklisted_VO(ctx));
    try{
        // Set Parameters
        stmt->setString(1,host);
        stmt->setString(2,site);
        
        // Execute Query
        ResultSetPtr rs(*stmt,stmt->executeQuery());
        if(0 != rs->next()){
            // SE Is Blacklisted
            result.first = true;
            result.second = rs->getString(2);
        }
    } catch(const SQLException& exc){
        m_log_error("Got SQLException in Setting Parameters: [0x" << exc.getErrorCode() << "] " << exc.getMessage());
        m_log_debug("Query was: " << stmt->getSQL());
        throw DAOException("Failed to check Blacklisted SE Share - SQL Exception");
    } catch(...){
        m_log_warn("Failed to check Blacklisted SEs");
        throw;
    }

    m_log_trace("VO_ChannelDAO::isBlacklisted returned [" << ((result.first)?"true":"false") << ":" << result.second << "]");

    return result;    
}

/*
 * getSiteGroups
 * 
 * Return the list of groups which a site belong to
 */
void VO_ChannelDAO::getSiteGroups(const std::string& site, agents::dao::StringArray& groups) /*throw (DAOLogicError,DAOException)*/{
    return m_channelDaoImpl->getSiteGroups(site,groups);
}
