/*
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *  GLite Data Transfer Agents: Oracle VO DAO Factory Class
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *           Paolo Tedesco <paolo.tedesco@cern.ch>
 *  Version info: $Id: VO_DAOFactory.cpp,v 1.7 2008/11/13 10:43:56 ptedesco Exp $
 *  Release: $Name:  $
 *
 */
#include "VO_DAOFactory.h"
#include "dao-utility.h"
#include "VO_FileDAO.h"
#include "VO_JobDAO.h"
#include "VO_TransferDAO.h"
#include "VO_StageReqDAO.h"
#include "VO_ChannelDAO.h"
#include "VO_AgentDAO.h"
#include "VO_CredDAO.h"

using namespace glite::data::transfer::agent;
using namespace glite::data::transfer::agent::dao;
using namespace glite::data::transfer::agent::dao::oracle;
using glite::data::agents::dao::DAOLogicError;
using glite::data::agents::dao::DAOContext;
using glite::data::agents::dao::oracle::OracleDAOContext;

/*
 * Constants
 */
namespace {
const char * const ORACLE_VO_DAO_NAME = "transfer-agent-dao-oracle-vo";
}

/*
 * VO_DAOFactory
 *
 * Constructor
 */
VO_DAOFactory::VO_DAOFactory():vo::DAOFactory(),
    m_logger(ORACLE_VO_DAO_NAME){
}

/*
 * checkSchema
 *
 * Validate the DB schema
 */
void VO_DAOFactory::checkSchema(DAOContext& ctx) /*throw (DAOLogicError,DAOException)*/{
    try{    
        check_schema(ctx);
    } catch(const agents::dao::DAOException& exc){
        m_log_error("Failed To Check Schema Version. Reason is : " << exc.what());
        throw;
    }
    try{    
        check_cred_schema(ctx);
    } catch(const agents::dao::DAOException& exc){
        m_log_error("Failed To Check Credential Schema Version. Reason is : " << exc.what());
        throw;
    }
}

/*
 * createFileDAO
 *
 * Returns DataAccessObject for model:File instances
 */
vo::FileDAO * VO_DAOFactory::createFileDAO(const std::string& vo_name, DAOContext& ctx) /*throw (DAOLogicError)*/{
    try{
        // Cast Context
        OracleDAOContext& oracle_ctx = dynamic_cast<OracleDAOContext&>(ctx);
    
        // Create File DAO
        return new VO_FileDAO(oracle_ctx,vo_name,ORACLE_VO_DAO_NAME);
    } catch(const std::exception& exc){
        m_log_error("Invalid DAOContext: " << exc.what());
        throw DAOLogicError("invalid DAOContext type");
    }
    throw DAOLogicError("invalid DAOFactory");
}

/*
 * createJobDAO
 *
 * Returns DataAccessObject for model:Job instances
 */
vo::JobDAO * VO_DAOFactory::createJobDAO(const std::string& vo_name, DAOContext& ctx) /*throw (DAOLogicError)*/{
    try{
        OracleDAOContext& oracle_ctx = dynamic_cast<OracleDAOContext&>(ctx);
        
        // Create Job DAO
        return new VO_JobDAO(oracle_ctx,vo_name,ORACLE_VO_DAO_NAME);
    } catch(const std::exception& exc){
        m_log_error("Invalid DAOContext: " << exc.what());
        throw DAOLogicError("invalid DAOContext type");
    }
    throw DAOLogicError("invalid DAOFactory");
}

/*
 * createTransferDAO
 *
 * Returns DataAccessObject for model:Transfer instances
 */
vo::TransferDAO * VO_DAOFactory::createTransferDAO(const std::string& vo_name, DAOContext& ctx) /*throw (DAOLogicError)*/{
    try{
        OracleDAOContext& oracle_ctx = dynamic_cast<OracleDAOContext&>(ctx);
        
        // Create Transfer DAO
        return new VO_TransferDAO(oracle_ctx,vo_name,ORACLE_VO_DAO_NAME);
    } catch(const std::exception& exc){
        m_log_error("Invalid DAOContext: " << exc.what());
        throw DAOLogicError("invalid DAOContext type");
    }
    throw DAOLogicError("invalid DAOFactory");
}


/*
 * createStageReqDAO
 *
 * Returns DataAccessObject for model:StageReq instances
 */
vo::StageReqDAO * VO_DAOFactory::createStageReqDAO(const std::string& vo_name, DAOContext& ctx) /*throw (DAOLogicError)*/{
    try{
        OracleDAOContext& oracle_ctx = dynamic_cast<OracleDAOContext&>(ctx);
        
        // Create StageReq DAO
        return new VO_StageReqDAO(oracle_ctx,vo_name,ORACLE_VO_DAO_NAME);
    } catch(const std::exception& exc){
        m_log_error("Invalid DAOContext: " << exc.what());
        throw DAOLogicError("invalid DAOContext type");
    }
    throw DAOLogicError("invalid DAOFactory");
}

/*
 * createChannelDAO
 *
 * Returns DataAccessObject for model:Channel instances
 */
vo::ChannelDAO * VO_DAOFactory::createChannelDAO(const std::string& vo_name, DAOContext& ctx) /*throw (DAOLogicError)*/{
    try{
        OracleDAOContext& oracle_ctx = dynamic_cast<OracleDAOContext&>(ctx);
    
        // Create Channel DAO
        return new VO_ChannelDAO(oracle_ctx,vo_name,ORACLE_VO_DAO_NAME);
    } catch(const std::exception& exc){
        m_log_error("Invalid DAOContext: " << exc.what());
        throw DAOLogicError("invalid DAOContext type");
    }
    throw DAOLogicError("invalid DAOFactory");
}


/*
 * createAgentDAO
 *
 * Returns DataAccessObject for model:Agent instances
 */
vo::AgentDAO * VO_DAOFactory::createAgentDAO(const std::string& vo_name, DAOContext& ctx) /*throw (DAOLogicError)*/{
    try{
        OracleDAOContext& oracle_ctx = dynamic_cast<OracleDAOContext&>(ctx);
    
        // Create Agent DAO
        return new VO_AgentDAO(oracle_ctx,vo_name,ORACLE_VO_DAO_NAME);
    } catch(const std::exception& exc){
        m_log_error("Invalid DAOContext: " << exc.what());
        throw DAOLogicError("invalid DAOContext type");
    }
    throw DAOLogicError("invalid DAOFactory");
}

/*
 * createCredDAO
 *
 * Returns DataAccessObject for model:Cred instances
 */
vo::CredDAO * VO_DAOFactory::createCredDAO(const std::string& vo_name, DAOContext& ctx) /*throw (DAOLogicError)*/{
    try{
        OracleDAOContext& oracle_ctx = dynamic_cast<OracleDAOContext&>(ctx);
    
        // Create Cred DAO
        return new VO_CredDAO(oracle_ctx,vo_name,ORACLE_VO_DAO_NAME);
    } catch(const std::exception& exc){
        m_log_error("Invalid DAOContext: " << exc.what());
        throw DAOLogicError("invalid DAOContext type");
    }
    throw DAOLogicError("invalid DAOFactory");
}
