/**
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *  GLite Data Transfer Agents: VO_FileDAO Class
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *           Paolo Tedesco <paolo.tedesco@cern.ch>
 *  Version info: $Id: VO_FileDAO.cpp,v 1.15 2008/11/13 10:43:56 ptedesco Exp $
 *  Release: $Name:  $
 *
 */
#include "VO_FileDAO.h"
#include "OracleFileDAO.h"
#include "FileStatements.h"
#include <glite/data/agents/dao/oracle/StatementPtr.h>

#include "dao-utility.h"

using namespace glite::data::transfer::agent;
using namespace glite::data::transfer::agent::dao;
using namespace glite::data::transfer::agent::dao::oracle;
using glite::data::agents::dao::DAOLogicError;
using glite::data::agents::dao::DAOException;
using glite::data::agents::dao::oracle::OracleDAOContext;
using glite::data::agents::dao::oracle::StatementPtr;
using namespace ::oracle::occi;

/*
 * VO_FileDAO
 *
 * Constructor
 */
VO_FileDAO::VO_FileDAO(OracleDAOContext& ctx, const std::string& vo_name, const std::string& logger):
        m_logger(logger),
        m_fileDaoImpl(new OracleFileDAO(ctx,logger)),
        m_voName(vo_name){
}

/*
 * get
 *
 * Return the File  with the given FileId
 */
model::File * VO_FileDAO::get(const std::string& id) /*throw (DAOLogicError,DAOException)*/{
    
    m_log_trace("Calling FileDAO::get");
    
    model::File * file = 0;

    // Get Context
    OracleDAOContext& ctx = m_fileDaoImpl->ctx();
    
    // Get Statement
    StatementPtr stmt(ctx.connection(),FileStatements::instance().createGet_VO(ctx));
    try{
        // Set Parameters
        stmt->setString(1,id);
        stmt->setString(2,m_voName);
        
        // Get Data
        file = m_fileDaoImpl->get(*stmt);
    } catch(const SQLException& exc){
        m_log_error("Got SQLException in Executing Get: [0x" << exc.getErrorCode() << "] " << exc.getMessage());
        m_log_debug("Query was: " << stmt->getSQL());
        throw DAOException("Failed to Get File - SQL Exception");
    } catch(const DAOException& exc){
        m_log_error("Got DAOException in Executing Get: " << exc.what());
        m_log_debug("Query was: " << stmt->getSQL());
        throw;
    } 
    
    m_log_trace("FileDAO::get Called");
    
    return file;
}

/*
 * get
 *
 * Return the File  with the given FileId
 */
model::File * VO_FileDAO::getFullSearch(const std::string& id) /*throw (DAOLogicError,DAOException)*/{
    
    m_log_trace("Calling FileDAO::getFullSearch");
    
    model::File * file = 0;

    // Get Context
    OracleDAOContext& ctx = m_fileDaoImpl->ctx();
    
    // Get Statement
    StatementPtr stmt(ctx.connection(),FileStatements::instance().createGet_VO(ctx,true));
    try{
        // Set Parameters
        stmt->setString(1,id);
        stmt->setString(2,m_voName);
        
        // Get Data
        file = m_fileDaoImpl->get(*stmt);
    } catch(const SQLException& exc){
        m_log_error("Got SQLException in Executing Get: [0x" << exc.getErrorCode() << "] " << exc.getMessage());
        m_log_debug("Query was: " << stmt->getSQL());
        throw DAOException("Failed to Get File - SQL Exception");
    } catch(const DAOException& exc){
        m_log_error("Got DAOException in Executing Get: " << exc.what());
        m_log_debug("Query was: " << stmt->getSQL());
        throw;
    } 
    
    m_log_trace("FileDAO::getFullSearch Called");
    
    return file;
}


/*
 * update
 *
 * Update a File instance
 */
void VO_FileDAO::update(const model::File& file) /*throw (DAOLogicError,DAOException)*/{
    
    m_log_trace("Calling FileDAO::update - File State is " << translateFileState(file.state()));
    
    // Get Context
    OracleDAOContext& ctx = m_fileDaoImpl->ctx();
    
    // Get Statement
    StatementPtr stmt(ctx.connection(),FileStatements::instance().createUpdate_VO(ctx,file,m_voName));
    try {
        // No need to set Parameters (already set)
        
        // Update Data
        if(0 != stmt.get()){
            m_fileDaoImpl->update(*stmt);
        }
    } catch(const SQLException& exc){
        m_log_error("Got SQLException in Executing Update: [0x" << exc.getErrorCode() << "] " << exc.getMessage());
        m_log_debug("Query was: " << stmt->getSQL());
        throw DAOException("Failed To Update File - SQL Exception");
    } catch(const DAOException& exc){
        m_log_error("Got DAOException in Executing Update: " << exc.what());
        m_log_debug("Query was: " << stmt->getSQL());
        throw;
    } 
    m_log_trace("File Updated");
}

/*
 * getByJobId
 *
 * Return the list of File instances for the given job Id
 */
void VO_FileDAO::getByJobId(const std::string& job_id, agents::dao::StringArray& files) /*throw (DAOLogicError,DAOException)*/{
    
    m_log_trace("Calling FileDAO::getByJobId");

    // Get Context
    OracleDAOContext& ctx = m_fileDaoImpl->ctx();
    
    // Get Statement
    StatementPtr stmt(ctx.connection(),FileStatements::instance().createGetByJobId_VO(ctx));
    try{
        // Set Parameters
        stmt->setString(1,job_id);
        stmt->setString(2,m_voName);
        
        // Get Data
        m_fileDaoImpl->getFileIds(*stmt,files);
        if(files.empty()){
            m_log_debug("No such JobId " << job_id);
            throw DAOException("Invalid Job Id");
        }
    } catch(const SQLException& exc){
        m_log_error("Got SQLException in Executing GetByJobId: [0x" << exc.getErrorCode() << "] " << exc.getMessage());
        m_log_debug("Query was: " << stmt->getSQL());
        throw DAOException("Failed to Get File - SQL Exception");
    } catch(const DAOException& exc){
        m_log_error("Got DAOException in Executing GetByJobId: " << exc.what());
        m_log_debug("Query was: " << stmt->getSQL());
        throw;
    } 
    
    m_log_trace("FileDAO::GetByJobId Called");
    
}

/*
 * getWaitingFiles
 *
 * Return the list of identifiers of the file which state is WAITING and 
 * the related job state is ACTIVE.
 */
void VO_FileDAO::getWaitingFiles(agents::dao::StringArray& fileIds, unsigned int limit) /*throw (DAOLogicError, DAOException)*/{
    
    m_log_trace("Calling FileDAO::getWaitingFiles");

    // Get Context
    OracleDAOContext& ctx = m_fileDaoImpl->ctx();
    
    // Get Statement
    StatementPtr stmt(ctx.connection(),FileStatements::instance().createGetWaitingFiles_VO(ctx));
    try{
        // Set Parameters
        stmt->setString(1,m_voName);
        stmt->setInt(2,limit);
        stmt->setInt(3,limit);
        
        // Get Data
        m_fileDaoImpl->getFileIds(*stmt,fileIds);
    } catch(const SQLException& exc){
        m_log_error("Got SQLException in Executing GetWaitingFiles: [0x" << exc.getErrorCode() << "] " << exc.getMessage());
        m_log_debug("Query was: " << stmt->getSQL());
        throw DAOException("Failed to Get File - SQL Exception");
    } catch(const DAOException& exc){
        m_log_error("Got DAOException in Executing GetWaitingFiles: " << exc.what());
        m_log_debug("Query was: " << stmt->getSQL());
        throw;
    } 
    
    m_log_trace("FileDAO::getWaitingFiles Called");
}


/*
 * getCatalogWaitingFiles
 *
 * Return the list of the job-files identifiers for the files which state 
 * is WAITING_CATALOG_RESOLUTION or WAITING_CATALOG_REGISTRATION 
 */
void VO_FileDAO::getCatalogWaitingFiles(FilesPerJobList& ids, unsigned int limit) /*throw (DAOLogicError, DAOException)*/{
    
    m_log_trace("Calling FileDAO::getCatalogWaitingFiles");

    // Get Context
    OracleDAOContext& ctx = m_fileDaoImpl->ctx();
    
    // Get Statement
    StatementPtr stmt(ctx.connection(),FileStatements::instance().createGetCatalogWaitingFiles_VO(ctx));
    try{
        // Set Parameters
        stmt->setString(1,m_voName);
        stmt->setInt(2,limit);
        stmt->setInt(3,limit);
        
        // Get Data
        m_fileDaoImpl->getFilesAndJobIds(*stmt,ids);
    } catch(const SQLException& exc){
        m_log_error("Got SQLException in Executing GetCatalogWaiting: [0x" << exc.getErrorCode() << "] " << exc.getMessage());
        m_log_debug("Query was: " << stmt->getSQL());
        throw DAOException("Failed to Get File - SQL Exception");
    } catch(const DAOException& exc){
        m_log_error("Got DAOException in Executing GetCatalogWaiting: " << exc.what());
        m_log_debug("Query was: " << stmt->getSQL());
        throw;
    } 
    
    m_log_trace("FileDAO::getCatalogWaitingFiles Called");
}
