/*
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *  GLite Data Transfer Agents: VO_JobDAO Class
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>, Paolo Tedesco <paolo.tedesco@cern.ch>
 *  Version info: $Id: VO_JobDAO.cpp,v 1.13 2008/11/13 10:43:56 ptedesco Exp $
 *  Release: $Name:  $
 *
 */
#include "VO_JobDAO.h"
#include "OracleJobDAO.h"
#include "JobStatements.h"
#include "glite/data/agents/dao/oracle/StatementPtr.h"

using namespace glite::data::transfer::agent;
using namespace glite::data::transfer::agent::dao;
using namespace glite::data::transfer::agent::dao::oracle;
using glite::data::agents::dao::DAOLogicError;
using glite::data::agents::dao::DAOException;
using glite::data::agents::dao::oracle::OracleDAOContext;
using glite::data::agents::dao::oracle::StatementPtr;
using namespace ::oracle::occi;

/*
 * VO_JobDAO
 *
 * Constructor
 */
VO_JobDAO::VO_JobDAO(OracleDAOContext& ctx, const std::string& vo_name,const std::string& category):
        m_logger(category),
        m_jobDaoImpl(new OracleJobDAO(ctx,category)),
        m_voName(vo_name){
}

/*
 * get
 *
 * Return the Job with the given JobId
 */
model::Job * VO_JobDAO::get(const std::string& id) /*throw (DAOLogicError, DAOException)*/{
   
    m_log_trace("Calling JobDAO::get");
    
    model::Job * job = 0;

    // Get Context
    OracleDAOContext& ctx = m_jobDaoImpl->ctx();
    
    // Get Statement
    StatementPtr stmt(ctx.connection(),JobStatements::instance().createGet_VO(ctx));
    try{
        // Set Parameters
        stmt->setString(1,id);
        stmt->setString(2,m_voName);
        
        // Get Data
        job = m_jobDaoImpl->get(*stmt);
    } catch(const SQLException& exc){
        m_log_error("Got SQLException in Executing Get: [0x" << exc.getErrorCode() << "] " << exc.getMessage());
        m_log_debug("Query was: " << stmt->getSQL());
        throw DAOException("Failed to get Job - SQL Exception");
    } catch(const DAOException& exc){
        m_log_error("Got DAOException in Executing Get: " << exc.what());
        m_log_debug("Query was: " << stmt->getSQL());
        throw;
    } 
    
    m_log_trace("JobDAO::get Called");
    
    return job;
}

/**
 * Return the TransferJob with the given JobId searching all jobs
 */
model::Job * VO_JobDAO::getFullSearch(const std::string& id) /*throw (DAOLogicError, DAOException)*/{
   
    m_log_trace("Calling VO_JobDAO::getFullSearch");

    model::Job * job = 0;

    // Get Context
    OracleDAOContext& ctx = m_jobDaoImpl->ctx();
    
    // Get Statement
    StatementPtr stmt(ctx.connection(),JobStatements::instance().createGet_VO(ctx,true));
    try{
        // Set Parameters
        stmt->setString(1,id);
        stmt->setString(2,m_voName);
        
        // Get Data
        job = m_jobDaoImpl->get(*stmt);
    } catch(const SQLException& exc){
        m_log_error("Got SQLException in Executing Get: [0x" << exc.getErrorCode() << "] " << exc.getMessage());
        m_log_debug("Query was: " << stmt->getSQL());
        throw DAOException("Failed to get Job - SQL Exception");
    } catch(const DAOException& exc){
        m_log_error("Got DAOException in Executing Get: " << exc.what());
        m_log_debug("Query was: " << stmt->getSQL());
        throw;
    } 
    
    m_log_trace("JobDAO::get Called");
    
    return job;
}

/*
 * update
 *
 * Update a Job object
 */
void VO_JobDAO::update(const model::Job& job) /*throw (DAOLogicError, DAOException)*/{

    m_log_trace("Calling JobDAO::update");
    
    // Get Context
    OracleDAOContext& ctx = m_jobDaoImpl->ctx();
    
    // Get Statement
    StatementPtr stmt(ctx.connection(),JobStatements::instance().createUpdate_VO(ctx,job,m_voName));
    try {
        // No need to set Parameters (already set)
        
        // Update Data
        if(0 != stmt.get()){
            m_jobDaoImpl->update(*stmt);
        }
    } catch(const SQLException& exc){
        m_log_error("Got SQLException in Executing Update: [0x" << exc.getErrorCode() << "] " << exc.getMessage());
        m_log_debug("Query was: " << stmt->getSQL());
        throw DAOException("Failed to update Job - SQL Exception");
    } catch(const DAOException& exc){
        m_log_error("Got DAOException in Executing Update: " << exc.what());
        m_log_debug("Query was: " << stmt->getSQL());
        throw;
    } 
    m_log_trace("Job updated");
}

/*
 * updateStates
 *
 * Update the state of all the transfer jobs in the given list
 */
void VO_JobDAO::updateStates(const agents::dao::StringArray& ids) /*throw (DAAOLogicError,DAOException)*/{
   
    m_log_trace("Entering VO_JobDAO::updateStates");

    // Get Context
    OracleDAOContext& ctx = m_jobDaoImpl->ctx();
    
    // Get Statement
    StatementPtr stmt(ctx.connection(),JobStatements::instance().createGetFileStates_VO(ctx));
    try{
        // Set Parameters
        stmt->setString(2,m_voName);
        
        // Deletage to the Helper    
        m_jobDaoImpl->updateStates(ids,*stmt,*this);
    } catch(const SQLException& exc){
        m_log_error("Got SQLException in Setting Parameters: [0x" << exc.getErrorCode() << "] " << exc.getMessage());
        m_log_debug("Query was: " << stmt->getSQL());
        throw DAOException("Failed to update Jobs' State - SQL Exception");
    } catch(...){
        m_log_warn("Failed to update Jobs' State");
        throw;
    }

    m_log_debug("Jobs' State Updated");
}

/*
 * getSubmittedJobs
 *
 * Return the list of Submitted Job instances
 */
void VO_JobDAO::getSubmittedJobs(agents::dao::StringArray& jobIds, unsigned int limit) /*throw (DAOLogicError, DAOException)*/{
    
    m_log_trace("Entering VO_JobDAO::getSubmittedJobs");
    
    // Get Context
    OracleDAOContext& ctx = m_jobDaoImpl->ctx();
    
    // Get Statement
    StatementPtr stmt(ctx.connection(),JobStatements::instance().createGetSubmittedJobs_VO(ctx));
    try{
        // Set Parameters
        stmt->setString(1,m_voName);
        stmt->setInt(2,limit);
        stmt->setInt(3,limit);
        
        // Get Data
        m_jobDaoImpl->getJobIds(*stmt,jobIds);
    } catch(const SQLException& exc){
        m_log_error("Got SQLException in Executing GetSubmittedJobs: [0x" << exc.getErrorCode() << "] " << exc.getMessage());
        m_log_debug("Query was: " << stmt->getSQL());
        throw DAOException("Failed to get submitted Jobs - SQL Exception");
    } catch(const DAOException& exc){
        m_log_error("Got DAOException in Executing GetSubmittedJobs: " << exc.what());
        m_log_debug("Query was: " << stmt->getSQL());
        throw;
    } 
}

/*
 * getPendingJobs
 *
 * Return the list of Pending Job instances
 */
void VO_JobDAO::getPendingJobs(agents::dao::StringArray& jobIds, unsigned int limit) /*throw (DAOLogicError, DAOException)*/{
    
    m_log_trace("Entering VO_JobDAO::getPendingJobs");
    
    // Get Context
    OracleDAOContext& ctx = m_jobDaoImpl->ctx();
    
    // Get Statement
    StatementPtr stmt(ctx.connection(),JobStatements::instance().createGetPendingJobs_VO(ctx));
    try{
        // Set Parameters
        stmt->setString(1,m_voName);
        stmt->setInt(2,limit);
        stmt->setInt(3,limit);
        
        // Get Data
        m_jobDaoImpl->getJobIds(*stmt,jobIds);
    } catch(const SQLException& exc){
        m_log_error("Got SQLException in Executing GetPendingJobs: [0x" << exc.getErrorCode() << "] " << exc.getMessage());
        m_log_debug("Query was: " << stmt->getSQL());
        throw DAOException("Failed to get submitted Jobs - SQL Exception");
    } catch(const DAOException& exc){
        m_log_error("Got DAOException in Executing GetPendingJobs: " << exc.what());
        m_log_debug("Query was: " << stmt->getSQL());
        throw;
    } 
}

/*
 * getDoneJobs
 *
 * Return the list of Done Job instances
 */
void VO_JobDAO::getDoneJobs(agents::dao::StringArray& jobIds, unsigned int limit) /*throw (DAOLogicError, DAOException)*/{
    
    m_log_trace("Entering VO_JobDAO::getDoneJobs");
    
    // Get Context
    OracleDAOContext& ctx = m_jobDaoImpl->ctx();
    
    // Get Statement
    StatementPtr stmt(ctx.connection(),JobStatements::instance().createGetDoneJobs_VO(ctx));
    try{
        // Set Parameters
        stmt->setString(1,m_voName);
        stmt->setInt(2,limit);
        stmt->setInt(3,limit);
        
        // Get Data
        m_jobDaoImpl->getJobIds(*stmt,jobIds);
    } catch(const SQLException& exc){
        m_log_error("Got SQLException in Executing GetDoneJobs: [0x" << exc.getErrorCode() << "] " << exc.getMessage());
        m_log_debug("Query was: " << stmt->getSQL());
        throw DAOException("Failed to get submitted Jobs - SQL Exception");
    } catch(const DAOException& exc){
        m_log_error("Got DAOException in Executing GetDoneJobs: " << exc.what());
        m_log_debug("Query was: " << stmt->getSQL());
        throw;
    } 
}

/*
 * getFinishingJobs
 *
 * Return the list of Finishing Job instances
 */
void VO_JobDAO::getFinishingJobs(agents::dao::StringArray& jobIds, unsigned int limit) /*throw (DAOLogicError, DAOException)*/{
    
    m_log_trace("Entering VO_JobDAO::getFinishingJobs");
    
    // Get Context
    OracleDAOContext& ctx = m_jobDaoImpl->ctx();
    
    // Get Statement
    StatementPtr stmt(ctx.connection(),JobStatements::instance().createGetFinishingJobs_VO(ctx));
    try{
        // Set Parameters
        stmt->setString(1,m_voName);
        stmt->setString(2,m_voName);
        stmt->setInt(3,limit);
        stmt->setInt(4,limit);
        
        // Get Data
        m_jobDaoImpl->getJobIds(*stmt,jobIds);
    } catch(const SQLException& exc){
        m_log_error("Got SQLException in Executing GetFinishingJobs: [0x" << exc.getErrorCode() << "] " << exc.getMessage());
        m_log_debug("Query was: " << stmt->getSQL());
        throw DAOException("Failed to get submitted Jobs - SQL Exception");
    } catch(const DAOException& exc){
        m_log_error("Got DAOException in Executing GetFinishingJobs: " << exc.what());
        m_log_debug("Query was: " << stmt->getSQL());
        throw;
    } 
}
