/*
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *  GLite Data Transfer Agents: VO_StageReqDAO Class
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *           Paolo Tedesco <paolo.tedesco@cern.ch>
 *  Version info: $Id: VO_StageReqDAO.cpp,v 1.4 2008/11/13 10:43:56 ptedesco Exp $
 *  Release: $Name:  $
 *
 */
#include "VO_StageReqDAO.h"
#include "OracleStageReqDAO.h"
#include "StageReqStatements.h"
#include "FileStatements.h"
#include <glite/data/agents/dao/oracle/StatementPtr.h>
#include <glite/data/agents/dao/oracle/ResultSetPtr.h>

using namespace glite::data::transfer::agent;
using namespace glite::data::transfer::agent::dao;
using namespace glite::data::transfer::agent::dao::oracle;
using glite::data::agents::dao::DAOLogicError;
using glite::data::agents::dao::DAOException;
using glite::data::agents::dao::oracle::OracleDAOContext;
using glite::data::agents::dao::oracle::StatementPtr;
using glite::data::agents::dao::oracle::ResultSetPtr;
using glite::data::agents::dao::StringArray;
using namespace ::oracle::occi;

/*
 * VO_StageReqDAO
 *
 * Constructor
 */
VO_StageReqDAO::VO_StageReqDAO(OracleDAOContext& ctx, const std::string& vo_name,const std::string& category):
        m_logger(category),
        m_stagereqDaoImpl(new OracleStageReqDAO(ctx,category)),
        m_voName(vo_name){
}

/*
 * get
 *
 * Return the StageReq with the given StageReqId
 */
model::StageReq * VO_StageReqDAO::get(const std::string& file_id, const std::string& req_id) /*throw (DAOLogicError,DAOException)*/{
   
    m_log_trace("Calling StageReqDAO::get");
    
    model::StageReq * req = 0;

    // Get Context
    OracleDAOContext& ctx = m_stagereqDaoImpl->ctx();
    
    // Get Statement
    StatementPtr stmt(ctx.connection(),StageReqStatements::instance().createGet_VO(ctx));
    try{
        // Set Parameters
        stmt->setString(1,file_id);
        stmt->setString(2,req_id);
        stmt->setString(3,m_voName);
        
        // Get Data
        req = m_stagereqDaoImpl->get(*stmt);
    } catch(const SQLException& exc){
        m_log_error("Got SQLException in Executing Get: [0x" << exc.getErrorCode() << "] " << exc.getMessage());
        m_log_debug("Query was: " << stmt->getSQL());
        throw DAOException("Failed to get StageReq - SQL Exception");
    } catch(const DAOException& exc){
        m_log_error("Got DAOException in Executing Get: " << exc.what());
        m_log_debug("Query was: " << stmt->getSQL());
        throw;
    } 
    
    m_log_trace("StageReqDAO::get Called");
    
    return req;
}

/*
 * getByFileId
 *
 * Return the list of StageReq instances for the given File Id
 */
void VO_StageReqDAO::getByFileId(const std::string& file_id, StringArray& ids, unsigned long limit) /*throw (DAOLogicError,DAOException)*/{
    
    m_log_trace("Entring VO_StageReqDAO::getByFileId");
    
    // Get Context
    OracleDAOContext& ctx = m_stagereqDaoImpl->ctx();
    
    // Get Statement
    StatementPtr stmt(ctx.connection(),StageReqStatements::instance().createGetByFileId_VO(ctx));
    try{
        // Set Parameters
        stmt->setString(1,file_id);
        stmt->setString(2,m_voName);        
        stmt->setInt(3,limit);
        stmt->setInt(4,limit);
        
        // Get Data
        m_stagereqDaoImpl->getIdList(*stmt,ids);
        if(true == ids.empty()){
            if(false == isValidFile(file_id)){
                m_log_debug("No such FileId " << file_id);
                throw DAOException("invalid File Id");
            }
        }
    } catch(const SQLException& exc){
        m_log_error("Got SQLException in Executing getByFileId: [0x" << exc.getErrorCode() << "] " << exc.getMessage());
        m_log_debug("Query was: " << stmt->getSQL());
        throw DAOException("Failed to get StageReq - SQL Exception");
    } catch(const DAOException& exc){
        m_log_error("Got DAOException in Executing getByFileId: " << exc.what());
        m_log_debug("Query was: " << stmt->getSQL());
        throw;
    } 
    
    m_log_trace("StageReqDAO::getByFileId Called");
}

/*
 * getByRequestId
 *
 * Return the list of StageReq instances for the given Request Id
 */
void VO_StageReqDAO::getByRequestId(const std::string& request_id, StringArray& ids) /*throw (DAOLogicError,DAOException)*/{
    m_log_trace("Entring VO_StageReqDAO::getByRequestId");
    
    // Get Context
    OracleDAOContext& ctx = m_stagereqDaoImpl->ctx();
    
    // Get Statement
    StatementPtr stmt(ctx.connection(),StageReqStatements::instance().createGetByRequestId_VO(ctx));
    try{
        // Set Parameters
        stmt->setString(1,request_id);
        stmt->setString(2,m_voName);
        
        // Get Data
        m_stagereqDaoImpl->getIdList(*stmt,ids);
        if(true == ids.empty()){
            m_log_debug("No File for requestId " << request_id);
            throw DAOException("invalid Request  Id");
        }
    } catch(const SQLException& exc){
        m_log_error("Got SQLException in Executing getByRequestId: [0x" << exc.getErrorCode() << "] " << exc.getMessage());
        m_log_debug("Query was: " << stmt->getSQL());
        throw DAOException("Failed to get StageReq - SQL Exception");
    } catch(const DAOException& exc){
        m_log_error("Got DAOException in Executing getByRequestId: " << exc.what());
        m_log_debug("Query was: " << stmt->getSQL());
        throw;
    } 
    
    m_log_trace("StageReqDAO::getByRequestId Called");
}


/*
 * update
 *
 * Update a StageReq object
 */
void VO_StageReqDAO::update(const model::StageReq& obj) /*throw (DAOLogicError,DAOException)*/{
    
    m_log_trace("Calling StageReqDAO::update");
    
    // Get Context
    OracleDAOContext& ctx = m_stagereqDaoImpl->ctx();
    
    // Get Statement
    StatementPtr stmt(ctx.connection(),StageReqStatements::instance().createUpdate_VO(ctx,obj,m_voName));
    try {
        // Update Data
        if(0 != stmt.get()){
            m_stagereqDaoImpl->update(*stmt);
        }
    } catch(const SQLException& exc){
        m_log_error("Got SQLException in Executing Update: [0x" << exc.getErrorCode() << "] " << exc.getMessage());
        m_log_debug("Query was: " << stmt->getSQL());
        throw DAOException("Failed to update Transfer - SQL Exception");
    } catch(const DAOException& exc){
        m_log_error("Got DAOException in Executing Update: " << exc.what());
        m_log_debug("Query was: " << stmt->getSQL());
        throw;
    } 
    m_log_trace("StageReq updated");
}


/*
 * create
 *
 * Create a StageReq instance
 */
void VO_StageReqDAO::create(const model::StageReq& sr) /*throw (DAOLogicError,DAOException)*/{
        
    m_log_trace("Calling StageReqDAO::create");
    
    // Get Context
    OracleDAOContext& ctx = m_stagereqDaoImpl->ctx();
    
    // Get Statement
    StatementPtr stmt(ctx.connection(),StageReqStatements::instance().createCreate_VO(ctx,sr));
    try {
        // Update Data
        m_stagereqDaoImpl->create(*stmt);
    } catch(const SQLException& exc){
        m_log_error("Got SQLException in Executing Insert: [0x" << exc.getErrorCode() << "] " << exc.getMessage());
        m_log_debug("Query was: " << stmt->getSQL());
        throw DAOException("Failed to insert Transfer - SQL Exception");
    } catch(const DAOException& exc){
        m_log_error("Got DAOException in Executing Insert: " << exc.what());
        m_log_debug("Query was: " << stmt->getSQL());
        throw;
    } 
    m_log_trace("StageReq created");
}
    
/*
 * countActiveRequests
 *
 * Count the active (Received & Processing) StageReq instances
 */
unsigned long VO_StageReqDAO::countActiveRequests() /*throw (DAOLogicError,DAOException)*/{
    
    m_log_trace("Entring VO_StageReqDAO::countActiveRequests()");

    unsigned long count = 0;    
    
    // Get Context
    OracleDAOContext& ctx = m_stagereqDaoImpl->ctx();
    
    // Get Statement
    StatementPtr stmt(ctx.connection(),StageReqStatements::instance().createCountActiveRequests_VO(ctx,false));
    try{
        // Set Parameters
        stmt->setString(1,m_voName);

        // Execute Query
        ResultSetPtr rs(*stmt,stmt->executeQuery());
        if(0 != rs->next()){
            // Get count
            count = rs->getInt(1);
        }
    } catch(const SQLException& exc){
        m_log_error("Got SQLException in Executing countActiveRequests(): [0x" << exc.getErrorCode() << "] " << exc.getMessage());
        m_log_debug("Query was: " << stmt->getSQL());
        throw DAOException("Failed to count StageReq - SQL Exception");
    }
    
    m_log_trace("countActiveRequests() Returned " << count);
    
    return count;
}

/*
 * countActiveRequests
 *
 * Count the active (Received & Processing) StageReq instances the have been
 * submitted for a channel
 */
unsigned long VO_StageReqDAO::countActiveRequests(const std::string& channel_name) /*throw (DAOLogicError,DAOException)*/{
    
    // Check Precondition
    if(channel_name.empty()){
        throw DAOLogicError("Invalid Channel Name");
    }
    m_log_trace("Entring VO_StageReqDAO::countActiveRequests");

    unsigned long count = 0;    
    
    // Get Context
    OracleDAOContext& ctx = m_stagereqDaoImpl->ctx();
    
    // Get Statement
    StatementPtr stmt(ctx.connection(),StageReqStatements::instance().createCountActiveRequests_VO(ctx,true));
    try{
        // Set Parameters
        stmt->setString(1,m_voName);
        stmt->setString(2,channel_name);
        
        // Execute Query
        ResultSetPtr rs(*stmt,stmt->executeQuery());
        if(0 != rs->next()){
            // Get count
            count = rs->getInt(1);
        }
    } catch(const SQLException& exc){
        m_log_error("Got SQLException in Executing countActiveRequests: [0x" << exc.getErrorCode() << "] " << exc.getMessage());
        m_log_debug("Query was: " << stmt->getSQL());
        throw DAOException("Failed to count StageReq - SQL Exception");
    }
    
    m_log_trace("countActiveRequests Returned " << count);
    
    return count;
}

/*
 * isValidFile
 *
 * Return true if the file id is valid
 */
bool VO_StageReqDAO::isValidFile(const std::string& file_id) /*throw (DAOLogicError,DAOException)*/{
    
    bool result = false;
    
    // Get Context
    OracleDAOContext& ctx = m_stagereqDaoImpl->ctx();
    
    // Get Statement
    StatementPtr stmt(ctx.connection(),FileStatements::instance().createGet_VO(ctx));
    try{
        // Set Parameters
        stmt->setString(1,file_id);
        stmt->setString(2,m_voName);
        
        // Execute Query
        ResultSetPtr rs(*stmt,stmt->executeQuery());
        // Check if there is an entry
        if(rs->next()){
            result = true;
        }
    } catch(const SQLException& exc){
        m_log_error("Got SQLException in Executing Get: [0x" << exc.getErrorCode() << "] " << exc.getMessage());
        m_log_debug("Query was: " << stmt->getSQL());
        throw DAOException("Failed to check File validity - SQL Exception");
    } catch(const DAOException& exc){
        m_log_debug("No such FileId " << file_id);
    } 
    
    return result;
}
