/**
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *  GLite Data Transfer Agents: VO_TransferDAO Class
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *           Paolo Tedesco <paolo.tedesco@cern.ch>
 *  Version info: $Id: VO_TransferDAO.cpp,v 1.9 2008/11/13 10:43:56 ptedesco Exp $
 *  Release: $Name:  $
 *
 */
#include "VO_TransferDAO.h"
#include "OracleTransferDAO.h"
#include "TransferStatements.h"
#include "FileStatements.h"
#include <glite/data/agents/dao/oracle/StatementPtr.h>
#include <glite/data/agents/dao/oracle/ResultSetPtr.h>

using namespace glite::data::transfer::agent;
using namespace glite::data::transfer::agent::dao;
using namespace glite::data::transfer::agent::dao::oracle;
using glite::data::agents::dao::DAOLogicError;
using glite::data::agents::dao::DAOException;
using glite::data::agents::dao::oracle::OracleDAOContext;
using glite::data::agents::dao::oracle::StatementPtr;
using glite::data::agents::dao::oracle::ResultSetPtr;
using glite::data::agents::dao::StringArray;
using namespace ::oracle::occi;

/*
 * VO_TransferDAO
 * Constructor
 */
VO_TransferDAO::VO_TransferDAO(OracleDAOContext& ctx, const std::string& vo_name,const std::string& category):
        m_logger(category),
        m_transferDaoImpl(new OracleTransferDAO(ctx,category)),
        m_voName(vo_name){
}

/*
 * get
 *
 * Return the Transfer with the given TransferId
 */
model::Transfer * VO_TransferDAO::get(const std::string& file_id, const std::string& req_id) /*throw (DAOLogicError,DAOException)*/{
   
    m_log_trace("Calling TransferDAO::get");
    
    model::Transfer * transfer = 0;

    // Get Context
    OracleDAOContext& ctx = m_transferDaoImpl->ctx();
    
    // Get Statement
    StatementPtr stmt(ctx.connection(),TransferStatements::instance().createGet_VO(ctx));
    try{
        // Set Parameters
        stmt->setString(1,file_id);
        stmt->setString(2,req_id);
        stmt->setString(3,m_voName);
        
        // Get Data
        transfer = m_transferDaoImpl->get(*stmt);
    } catch(const SQLException& exc){
        m_log_error("Got SQLException in Executing Get: [0x" << exc.getErrorCode() << "] " << exc.getMessage());
        m_log_debug("Query was: " << stmt->getSQL());
        throw DAOException("Failed to get Transfer - SQL Exception");
    } catch(const DAOException& exc){
        m_log_error("Got DAOException in Executing Get: " << exc.what());
        m_log_debug("Query was: " << stmt->getSQL());
        throw;
    } 
    
    m_log_trace("TransferDAO::get Called");
    
    return transfer;
}

/*
 * get
 *
 * Return the Transfer with the given TransferId
 */
model::Transfer * VO_TransferDAO::getFullSearch(const std::string& file_id, const std::string& req_id) /*throw (DAOLogicError,DAOException)*/{
   
    m_log_trace("Calling TransferDAO::getFullSearch");
    
    model::Transfer * transfer = 0;

    // Get Context
    OracleDAOContext& ctx = m_transferDaoImpl->ctx();
    
    // Get Statement
    StatementPtr stmt(ctx.connection(),TransferStatements::instance().createGet_VO(ctx,true));
    try{
        // Set Parameters
        stmt->setString(1,file_id);
        stmt->setString(2,req_id);
        stmt->setString(3,m_voName);
        
        // Get Data
        transfer = m_transferDaoImpl->get(*stmt);
    } catch(const SQLException& exc){
        m_log_error("Got SQLException in Executing Get: [0x" << exc.getErrorCode() << "] " << exc.getMessage());
        m_log_debug("Query was: " << stmt->getSQL());
        throw DAOException("Failed to get Transfer - SQL Exception");
    } catch(const DAOException& exc){
        m_log_error("Got DAOException in Executing Get: " << exc.what());
        m_log_debug("Query was: " << stmt->getSQL());
        throw;
    } 
    
    m_log_trace("TransferDAO::getFullSearch Called");
    
    return transfer;
}


/*
 * getByFileId
 *
 * Return the list of Transfer instances for the given File Id
 */
void VO_TransferDAO::getByFileId(const std::string& file_id, StringArray& ids, unsigned long limit) /*throw (DAOLogicError,DAOException)*/{
    
    m_log_trace("Entring VO_TransferDAO::getByFileId");
    
    // Get Context
    OracleDAOContext& ctx = m_transferDaoImpl->ctx();
    
    // Get Statement
    StatementPtr stmt(ctx.connection(),TransferStatements::instance().createGetByFileId_VO(ctx));
    try{
        // Set Parameters
        stmt->setString(1,file_id);
        stmt->setString(2,m_voName);        
        stmt->setInt(3,limit);
        stmt->setInt(4,limit);
        
        // Get Data
        m_transferDaoImpl->getIdList(*stmt,ids);
        if(true == ids.empty()){
            if(false == isValidFile(file_id)){
                m_log_debug("No such FileId " << file_id);
                throw DAOException("invalid File Id");
            }
        }
    } catch(const SQLException& exc){
        m_log_error("Got SQLException in Executing getByFileId: [0x" << exc.getErrorCode() << "] " << exc.getMessage());
        m_log_debug("Query was: " << stmt->getSQL());
        throw DAOException("Failed to get Transfer - SQL Exception");
    } catch(const DAOException& exc){
        m_log_error("Got DAOException in Executing getByFileId: " << exc.what());
        m_log_debug("Query was: " << stmt->getSQL());
        throw;
    } 
    
    m_log_trace("TransferDAO::getByFileId Called");
}

/*
 * getByRequestId
 *
 * Return the list of Transfer instances for the given Request Id
 */
void VO_TransferDAO::getByRequestId(const std::string& request_id, StringArray& ids) /*throw (DAOLogicError,DAOException)*/{
    m_log_trace("Entring VO_TransferDAO::getByRequestId");
    
    // Get Context
    OracleDAOContext& ctx = m_transferDaoImpl->ctx();
    
    // Get Statement
    StatementPtr stmt(ctx.connection(),TransferStatements::instance().createGetByRequestId_VO(ctx));
    try{
        // Set Parameters
        stmt->setString(1,request_id);
        stmt->setString(2,m_voName);
        
        // Get Data
        m_transferDaoImpl->getIdList(*stmt,ids);
        if(true == ids.empty()){
            m_log_debug("No File for requestId " << request_id);
            throw DAOException("invalid Request  Id");
        }
    } catch(const SQLException& exc){
        m_log_error("Got SQLException in Executing getByRequestId: [0x" << exc.getErrorCode() << "] " << exc.getMessage());
        m_log_debug("Query was: " << stmt->getSQL());
        throw DAOException("Failed to get Transfer - SQL Exception");
    } catch(const DAOException& exc){
        m_log_error("Got DAOException in Executing getByRequestId: " << exc.what());
        m_log_debug("Query was: " << stmt->getSQL());
        throw;
    } 
    
    m_log_trace("TransferDAO::getByRequestId Called");
}

/*
 * isValidFile
 *
 * Return true if the file id is valid
 */
bool VO_TransferDAO::isValidFile(const std::string& file_id) /*throw (DAOLogicError,DAOException)*/{
    
    bool result = false;
    
    // Get Context
    OracleDAOContext& ctx = m_transferDaoImpl->ctx();
    
    // Get Statement
    StatementPtr stmt(ctx.connection(),FileStatements::instance().createGet_VO(ctx));
    try{
        // Set Parameters
        stmt->setString(1,file_id);
        stmt->setString(2,m_voName);
        
        // Execute Query
        ResultSetPtr rs(*stmt,stmt->executeQuery());
        // Check if there is an entry
        if(rs->next()){
            result = true;
        }
    } catch(const SQLException& exc){
        m_log_error("Got SQLException in Executing Get: [0x" << exc.getErrorCode() << "] " << exc.getMessage());
        m_log_debug("Query was: " << stmt->getSQL());
        throw DAOException("Failed to check File validity - SQL Exception");
    } catch(const DAOException& exc){
        m_log_debug("No such FileId " << file_id);
    } 
    
    return result;
}
