/**
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *  GLite Data Transfer Agents: ErrorMatch Class
 *
 *  Authors: Paolo Tedesco <paolo.tedesco@cern.ch>
 *  Version info: $Id: ErrorMatch.cpp,v 1.2 2007/07/06 14:35:00 ptedesco Exp $
 *  Release: $Name:  $
 */

#include "ErrorMatch.h"

using namespace glite::data::transfer::agent;
using namespace glite::data::transfer::agent::fsm;
using namespace glite::data::transfer::agent::model;

/**
 * match an error based only on the phase
 */    
class PhaseMatch : public IErrorMatch{
public:
    
    PhaseMatch(
        const Error::Phase& phase) : 
        m_phase(phase){}
            
    virtual bool isMatch(
        const Error::Category&  category, 
        const Error::Scope&     scope, 
        const Error::Phase&     phase) const{
        return (m_phase == phase);
    }
private:
    Error::Phase m_phase;
};

/**
 * match an error based on phase and category
 */    
class PhaseCategoryMatch : public IErrorMatch{
public:
    
    PhaseCategoryMatch(
        const Error::Phase& phase,
        const Error::Category& category) : 
        m_phase(phase),
        m_category(category){}
            
    virtual bool isMatch(
        const Error::Category&  category, 
        const Error::Scope&     scope, 
        const Error::Phase&     phase) const{
        return ((m_phase == phase) && (m_category == category));
    }
private:
    Error::Phase m_phase;
    Error::Category m_category;
};

/**
 * match an error based only on the category
 */    
class CategoryMatch : public IErrorMatch{
public:
    CategoryMatch(
        const Error::Category& category) : 
        m_category(category){}
            
    virtual bool isMatch(
        const Error::Category&  category, 
        const Error::Scope&     scope, 
        const Error::Phase&     phase) const{
        return (m_category == category);
    }
    
private:
    Error::Category m_category;
};

/**
 * match an error based on phase, category and scope
 */    
class PhaseCategoryScopeMatch : public IErrorMatch{
public:
    PhaseCategoryScopeMatch(
        const Error::Phase&     phase,
        const Error::Category&  category,
        const Error::Scope&     scope) : 
        m_phase(phase),
        m_category(category),
        m_scope(scope){}
            
    virtual bool isMatch(
        const Error::Category&  category, 
        const Error::Scope&     scope, 
        const Error::Phase&     phase) const{
        return ((m_phase == phase) && (m_category == category) && (m_scope == scope));
    }
private:
    Error::Phase    m_phase;
    Error::Category m_category;
    Error::Scope    m_scope;
};


/**
 * add a match on the phase
 */
void ErrorMatch::add(const Error::Phase& phase){
    m_matches.push_back(new PhaseMatch (phase));
}
/**
 * add a match on phase and category
 */
void ErrorMatch::add(const Error::Phase& phase,const Error::Category& category){
    m_matches.push_back(new PhaseCategoryMatch (phase,category));
}
/**
 * add a match on the category
 */
void ErrorMatch::add(const Error::Category& category){
    m_matches.push_back(new CategoryMatch (category));
}
/**
 * add a match on phase and category
 */
void ErrorMatch::add(
    const Error::Phase&     phase,
    const Error::Category&  category,
    const Error::Scope&     scope){
    m_matches.push_back(new PhaseCategoryScopeMatch(phase,category,scope));
}

/**
 * return true if category, scope and phase match any of the internal 
 * criteria
 */
bool ErrorMatch::isMatch(
    const Error::Category&  category, 
    const Error::Scope&     scope, 
    const Error::Phase&     phase) const{
    std::vector<IErrorMatch*>::const_iterator it = m_matches.begin();
    for (;it != m_matches.end();it++){
        const IErrorMatch * m = (*it);
        if (m->isMatch(category,scope,phase)){
            return true;
        }
    }
    return false;
}
/**
 * destructor
 */
ErrorMatch::~ErrorMatch(){
    std::vector<IErrorMatch*>::iterator it = m_matches.begin();
    for (;it != m_matches.end();it++){
        IErrorMatch * m = (*it);
        delete m;
    }
}
