/*
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *  GLite Data Transfer Agents: FileEventListener interface
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *  Version info: $Id: EventDispatcher.cpp,v 1.2 2006/08/14 08:38:42 badino Exp $
 *  Release: $Name:  $
 *
 */
#include "EventDispatcher.h"
#include <algorithm>

using namespace glite::data::transfer::agent::fsm;

/*
 * instance
 *
 * Return singleton instance
 */
EventDispatcher& EventDispatcher::instance(){
    static EventDispatcher s_instance;
    return s_instance;
}
    
/*
 * EventDispatcher
 *
 * Constructor
 */
EventDispatcher::EventDispatcher(){
}

/*
 * ~EventDispatcher
 *
 * Destructor
 */
EventDispatcher::~EventDispatcher(){
}

/*
 * registerListener
 *
 * Register a File Event Listener
 */
void EventDispatcher::registerListener(FileEventListener* l) throw(){
    // Check if the listener is not yet registered
    FileEventListeners::iterator it = 
        std::find(m_fileListeners.begin(),m_fileListeners.end(),l);
    if(it == m_fileListeners.end()){
        // register the listener
        m_fileListeners.push_back(l);
    }
}

/*
 * deregisterListener
 *
 * Deregister a File Event Listener
 */
void EventDispatcher::deregisterListener(FileEventListener* l) throw(){
    // Check if the listener is registered
    FileEventListeners::iterator it = 
        std::find(m_fileListeners.begin(),m_fileListeners.end(),l);
    if(it != m_fileListeners.end()){
        // deregister the listener
        m_fileListeners.erase(it);
    }
}

/*
 * fileStateChanged
 *
 * Dispatch Event File State Changed
 */
void EventDispatcher::fileStateChanged(const model::File& file, const model::Job& job, model::File::State old_state) throw(){
    // For each listener
    FileEventListeners::iterator it;
    for(it = m_fileListeners.begin(); it != m_fileListeners.end(); ++it){
        try{
            FileEventListener * l = (*it);            
            if(0 != l){
                l->onStateChanged(file,job,old_state);
            }
        } catch(...){
            // Ignore Errors
        }
    }
}

/*
 * fileReady
 *
 * Dispatch Event File State Changed
 */
void EventDispatcher::fileReady(const model::File& file, const model::Job& job) throw(){
    // For each listener
    FileEventListeners::iterator it;
    for(it = m_fileListeners.begin(); it != m_fileListeners.end(); ++it){
        try{
            FileEventListener * l = (*it);            
            if(0 != l){
                l->onReady(file,job);
            }
        } catch(...){
            // Ignore Errors
        }
    }
}

/*
 * fileDone
 *
 * Dispatch Event File State Changed
 */
void EventDispatcher::fileDone(const model::File& file, const model::Job& job) throw(){
    // For each listener
    FileEventListeners::iterator it;
    for(it = m_fileListeners.begin(); it != m_fileListeners.end(); ++it){
        try{
            FileEventListener * l = (*it);            
            if(0 != l){
                l->onDone(file,job);
            }
        } catch(...){
            // Ignore Errors
        }
    }
}

/*
 * fileFinished
 *
 * Dispatch Event File State Changed
 */
void EventDispatcher::fileFinished(const model::File& file, const model::Job& job) throw(){
    // For each listener
    FileEventListeners::iterator it;
    for(it = m_fileListeners.begin(); it != m_fileListeners.end(); ++it){
        try{
            FileEventListener * l = (*it);            
            if(0 != l){
                l->onFinished(file,job);
            }
        } catch(...){
            // Ignore Errors
        }
    }
}

/*
 * registerListener
 *
 * Register a Job Event Listener
 */
void EventDispatcher::registerListener(JobEventListener* l) throw(){
    // Check if the listener is not yet registered
    JobEventListeners::iterator it = 
        std::find(m_jobListeners.begin(),m_jobListeners.end(),l);
    if(it == m_jobListeners.end()){
        // register the listener
        m_jobListeners.push_back(l);
    }
}

/*
 * deregisterListener
 *
 * Deregister a Job Event Listener
 */
void EventDispatcher::deregisterListener(JobEventListener* l) throw(){
    // Check if the listener is registered
    JobEventListeners::iterator it = 
        std::find(m_jobListeners.begin(),m_jobListeners.end(),l);
    if(it != m_jobListeners.end()){
        // deregister the listener
        m_jobListeners.erase(it);
    }
}

/*
 * jobStateChanged
 *
 * Dispatch Event Job State Changed
 */
void EventDispatcher::jobStateChanged(const model::Job& job, model::Job::State old_state) throw(){
    // For each listener
    JobEventListeners::iterator it;
    for(it = m_jobListeners.begin(); it != m_jobListeners.end(); ++it){
        try{
            JobEventListener * l = (*it);            
            if(0 != l){
                l->onStateChanged(job,old_state);
            }
        } catch(...){
            // Ignore Errors
        }
    }
}

/*
 * jobPending
 *
 * Dispatch Event Job Pending
 */
void EventDispatcher::jobPending(const model::Job& job) throw(){
    // For each listener
    JobEventListeners::iterator it;
    for(it = m_jobListeners.begin(); it != m_jobListeners.end(); ++it){
        try{
            JobEventListener * l = (*it);            
            if(0 != l){
                l->onPending(job);
            }
        } catch(...){
            // Ignore Errors
        }
    }
}

/*
 * jobFinished
 *
 * Dispatch Event Job Finished
 */
void EventDispatcher::jobFinished(const model::Job& job) throw(){
    // For each listener
    JobEventListeners::iterator it;
    for(it = m_jobListeners.begin(); it != m_jobListeners.end(); ++it){
        try{
            JobEventListener * l = (*it);            
            if(0 != l){
                l->onFinished(job);
            }
        } catch(...){
            // Ignore Errors
        }
    }
}
