/*
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *  GLite Data Transfer Agents: FSMConfig Class
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *           Paolo Tedesco <paolo.tedesco@cern.ch>
 *  Version info: $Id: FSMConfig.cpp,v 1.9 2009/01/08 13:50:39 ptedesco Exp $
 *  Release: $Name:  $
 *
 */
#include "FSMConfig.h"
#include <glite/data/agents/AgentExceptions.h>
#include <glite/data/config/service/ParamReader.hxx>
#include <string>
#include <algorithm>
#include <cctype>

using namespace glite::data::transfer::agent::fsm;
using namespace glite::config;
using glite::data::agents::LogicError;
using glite::data::agents::ConfigParamException;

/*
 * Constants
 */
namespace {
const char * const FSM_NAME                   = "transfer-agent-fsm";
                                                                                                                          
// Properties Names
const char * const HOLDENABLED_PROPERTY_NAME  = "EnableHold";
                                                                                                                          
// Properties Default  Values
const bool   DEFAULT_HOLDENABLED       = true;
}
                                                                                                                          
/*
 * Initialize Static Attirbutes
 */
FSMConfig *  FSMConfig::s_instance = 0;

extern "C" {
/*
 * create_glite_component
 *
 * Create Component instance
 */
ComponentConfiguration * create_glite_component(){
    return &(FSMConfig::instance());
}
                                                                                
/*
 * destroy_glite_component
 *
 * Destroy Component instance
 */
void destroy_glite_component(ComponentConfiguration * component){
    FSMConfig::finalize();
}
                                                                                
} // End extern "C"


/*
 * instance
 *
 * Return Singleton Instance
 */
FSMConfig& FSMConfig::instance(){
    if(0 == s_instance){
        s_instance = new FSMConfig();
    }
    return *s_instance;
}
                                                                                                                          
/*
 * finalize the singleton
 *
 * Finalize Singleton Instance
 */
void FSMConfig::finalize(){
    if(0 != s_instance){
        delete s_instance;
        s_instance = 0;
    }
}

/*
 * FSMConfig
 *
 * Constructor
 */
FSMConfig::FSMConfig():ComponentConfiguration(FSM_NAME), 
    m_logger(FSM_NAME),
    m_holdEnabled(DEFAULT_HOLDENABLED){
}
                                                                                                                          
/*
 * ~FSMConfig
 *
 * Destructor
 */
FSMConfig::~FSMConfig(){
}

/*
 * init
 *
 * Initialize the Component
 */
int FSMConfig::init(const Params& params){
          
    try{
        ParamReader reader(getName(),params);

        // Get hold enabled property
        reader.getParameter(HOLDENABLED_PROPERTY_NAME,m_holdEnabled);

        // Log Configuration Values
        m_log_info(getName() << " Initialized. Initialization Parameters are");
        m_log_info("EnableHold        : " << str_bool(m_holdEnabled));
    } catch(const LogicError& exc) {
        m_log_error(getName() << " configuration Error: " << exc.what());
        return -1;
    } catch(const std::exception& exc) {
        m_log_error(getName() << " configuration Unexpected Exception: " << exc.what());
        return -1;
    } catch(...) {
        m_log_error(getName() << " configuration Unhandled Exception");
        return -1;
    }
    return 0;
}

                                                                                                                          
/*
 * config
 *
 * Configure the Component
 */
int FSMConfig::config(const Params& params){
    // No Config Params
    m_log_info(getName() << " Configured");
    return 0;
}
