/*
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *  GLite Data Transfer Agents: FileFSM Class
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>, Paolo Tedesco <paolo.tedesco@cern.ch>
 *  Version info: $Id: FileFSM.cpp,v 1.18 2008/04/23 15:10:13 ptedesco Exp $
 *  Release: $Name:  $
 *
 */
#include "FileFSM.h"
#include "EventDispatcher.h"
#include "FSMConfig.h"
#include "InvalidStateException.h"
#include "JobFlags.h"
#include "ErrorMatch.h"

using namespace glite::data::transfer::agent;
using namespace glite::data::transfer::agent::model;
using namespace glite::data::transfer::agent::fsm;

/*
 * onEventAllocate
 *
 * Notify the Allocated Event
 */
void FileFSM::onEventAllocate() /*throw (FSMException)*/{
    // Save the old State
    File::State old_state = m_file.state();
    
    switch(m_file.state()){
        case File::S_SUBMITTED:
            // Change The State to PENDING
            m_file.state = File::S_PENDING;
        break;
        default:
            throw InvalidStateException("Event Allocate Received for File "\
                                        "not in [Submitted] state");
    }
    // Notify ChangeState Event (if needed)
    eventStateChanged(old_state);
}


/*
 * onEventFileReady
 *
 * Notify that the File is Ready
 */
void FileFSM::onEventFileReady() /*throw (FSMException)*/{
    // Save the old State
    File::State old_state = m_file.state();
    
    switch(m_file.state()){
        case File::S_PENDING:
        case File::S_PRESTAGING:
            // Call Pre-Transfer Hook
            EventDispatcher::instance().fileReady(m_file, m_job);
        
            // Change The State to READY
            m_file.state = File::S_READY;
        break;
        default:
            throw InvalidStateException("Event FileReady received for File "\
                                        "not in [Pending,Prestaging] states");
    }
    // Notify ChangeState Event (if needed)
    eventStateChanged(old_state);
}

/*
 * onEventFileNotReady
 *
 * Notify that the file is Not Ready
 */
void FileFSM::onEventFileNotReady() /*throw (FSMException)*/{
    // Save the old State
    File::State old_state = m_file.state();
    
    switch(m_file.state()){
        case File::S_PENDING:
            // Change The State to AWAITING_PRESTAGE
            m_file.state = File::S_AWAITING_PRESTAGE;
        break;
        default:
            throw InvalidStateException("Event FileNotReady received for File "\
                                        "not in [Pending] state");
    }
    // Notify ChangeState Event (if needed)
    eventStateChanged(old_state);
}

/*
 * onEventStartStaging
 *
 * Notify the Start Staging Event
 */
void FileFSM::onEventStartStaging(StageReq& r) /*throw (FSMException)*/{
    // Save the old State
    File::State old_state = m_file.state();
    
    switch(m_file.state()){
        case File::S_AWAITING_PRESTAGE:{
            // Change The State to Prestaging
            m_file.state = File::S_PRESTAGING;
            
            // Set the transfer Time
            if((time_t)-1 == r.requestTime()){
                time_t current;
                time(&current);
                r.requestTime = current;
            }
        } break;
        default:
            throw InvalidStateException("Event StartStaging received for File "\
                                        "not in [AwaitingPrestage] state");
    }
    // Notify ChangeState Event (if needed)
    eventStateChanged(old_state);
}

/*
 * onEventStaging
 *
 * Notify the Staging Event
 */
void FileFSM::onEventStaging(StageReq& t) /*throw (FSMException)*/{
    // Save the old State
    File::State old_state = m_file.state();
    
    switch(m_file.state()){
        case File::S_PRESTAGING:
            // The State Doesn't Change
        break;
        default:
            throw InvalidStateException("Event Staging received for File "\
                                        "not in [Prestaging] state");
    }
    // Notify ChangeState Event (if needed)
    eventStateChanged(old_state);
}

/*
 * onEventStart
 *
 * Notify the Start Event
 */
void FileFSM::onEventStart(Transfer& t) /*throw (FSMException)*/{
    // Save the old State
    File::State old_state = m_file.state();
    
    switch(m_file.state()){
        case File::S_READY:{
            // Change The State to ACTIVE
            m_file.state = File::S_ACTIVE;
            
            // Set the transfer Time
            if((time_t)-1 == t.transferTime()){
                time_t current;
                time(&current);
                t.transferTime = current;
            }
            // Reset finish time
            m_file.finishTime = -1;
        } break;
        default:
            throw InvalidStateException("Event Start received for File "\
                                        "not in [Ready] state");
    }
    // Notify ChangeState Event (if needed)
    eventStateChanged(old_state);
}


/*
 * onEventRunning
 *
 * Notify the Running Event
 */
void FileFSM::onEventRunning(Transfer& t) /*throw (FSMException)*/{
    // Save the old State
    File::State old_state = m_file.state();
    
    switch(m_file.state()){
        case File::S_ACTIVE:{
            // The State Doesn't Change
            
            // Check if the file size is set
            if((m_file.fileSize() <= 0) && (t.fileSize() > 0)){
                m_file.fileSize = t.fileSize();
            }
        } break;
        default:
            throw InvalidStateException("Event Running received for File "\
                                        "not in [Active] state");    
    }
    // Notify ChangeState Event (if needed)
    eventStateChanged(old_state);
}

/*
 * onEventDone
 *
 * Notify the TransferDone Event
 */
void FileFSM::onEventDone(Transfer& t) /*throw (FSMException)*/{
    // Save the old State
    File::State old_state = m_file.state();
    
    switch(m_file.state()){
        case File::S_ACTIVE:{
            // Check if the file size is set
            if((m_file.fileSize() <= 0) && (t.fileSize() > 0)){
                m_file.fileSize = t.fileSize();
            } else {
#warning "TODO : Check the File Size is the same as the one in the catalog"
            }
            if((time_t)-1 == m_file.finishTime()){
                time_t current;
                time(&current);
                m_file.finishTime = current;
            }
            // Change The State to DONE
            m_file.state = File::S_DONE;
        } break;
        default:
            throw InvalidStateException("Event TransferDone received for File "\
                                        "not in [Active] state");
    }
    // Notify ChangeState Event (if needed)
    eventStateChanged(old_state);
}

/*
 * onEventFailed
 *
 * Notify the Transfer Failed Event
 */
void FileFSM::onEventFailed(Transfer& t) /*throw ()*/{
    // Fails with error    
    onEventFailed(t.reasonClass(), t.errorScope(), t.errorPhase(), t.reason());
}

/*
 * onEventFailed
 *
 * Notify the Staging Failed Event
 */
void FileFSM::onEventFailed(StageReq& r) /*throw ()*/{
    // Fails with error    
    onEventFailed(r.reasonClass(), r.errorScope(), r.errorPhase(), r.reason());
}

/**
 * S_SUBMITTED -> S_WAITING_CATALOG_RESOLUTION
 */
class SubmittedToWaitingCatalogResolution : public ErrorMatch {
public:
    SubmittedToWaitingCatalogResolution(){
        // possibly transient catalog resolution errors
        add(Error::P_CATALOG_RESOLUTION, Error::CATALOG);
        add(Error::P_CATALOG_RESOLUTION, Error::CATALOG_INTERNAL_ERROR);
        add(Error::P_CATALOG_RESOLUTION, Error::CONNECTION);
        add(Error::P_CATALOG_RESOLUTION, Error::INTERNAL_ERROR);
        add(Error::P_CATALOG_RESOLUTION, Error::INVALID_PATH);
        add(Error::P_CATALOG_RESOLUTION, Error::NETWORK);
        //add(Error::P_CATALOG_RESOLUTION, Error::SECURITY);
    }
};

/**
 * S_PENDING -> S_WAITING_PRESTAGE
 */
class PendingToWaitingPrestage : public ErrorMatch {
public:
    PendingToWaitingPrestage(){
        // possibly transient prestaging errors (matched only on source)
        add(Error::P_PRESTAGING, Error::INTERNAL_ERROR);
        add(Error::P_PRESTAGING, Error::SECURITY,               Error::S_SOURCE);
        add(Error::P_PRESTAGING, Error::NETWORK,                Error::S_SOURCE);
        add(Error::P_PRESTAGING, Error::REQUEST_FAILURE,        Error::S_SOURCE);
        add(Error::P_PRESTAGING, Error::CONNECTION,             Error::S_SOURCE);
        add(Error::P_PRESTAGING, Error::HTTP_TIMEOUT,           Error::S_SOURCE);
        add(Error::P_PRESTAGING, Error::REQUEST_TIMEOUT,        Error::S_SOURCE);
        add(Error::P_PRESTAGING, Error::GENERAL_FAILURE,        Error::S_SOURCE);
        add(Error::P_PRESTAGING, Error::STORAGE_INTERNAL_ERROR, Error::S_SOURCE);
    }
};

/**
 * S_READY -> S_WAITING
 * S_ACTIVE -> S_WAITING
 */
class ReadyOrActiveToWaiting : public ErrorMatch {
public:
    ReadyOrActiveToWaiting(){
        // possibly transient transfer errors
        addCategories(Error::P_TX_PREPARATION);
        addCategories(Error::P_TRANSFER);
        addCategories(Error::P_TX_FINALIZATION);
    }
protected:
    void addCategories(const Error::Phase& phase){
        add(phase,Error::SECURITY);
        add(phase,Error::TRANSFER);
        add(phase,Error::FILE_EXISTS);
        add(phase,Error::INVALID_PATH);
        add(phase,Error::NO_SPACE_LEFT);
        add(phase,Error::GRIDFTP);
        add(phase,Error::NETWORK);
        add(phase,Error::INVALID_SIZE);
        add(phase,Error::REQUEST_FAILURE);
        add(phase,Error::CONNECTION);
        add(phase,Error::HTTP_TIMEOUT);
        add(phase,Error::REQUEST_TIMEOUT);
        add(phase,Error::TRANSFER_TIMEOUT);
        add(phase,Error::MARKERS_TIMEOUT);
        add(phase,Error::INTERNAL_ERROR);
        add(phase,Error::ABORTED);
        add(phase,Error::GENERAL_FAILURE);
        add(phase,Error::LOST);
        add(phase,Error::STORAGE_INTERNAL_ERROR);
        add(phase,Error::SRM_COPY);
    }
};

/**
 * S_FINISHING -> S_WAITING_CATALOG_REGISTRATION
 */
class FinishingToWaitingCatalogRegistration : public ErrorMatch{
public:
    FinishingToWaitingCatalogRegistration(){
        add(Error::P_CATALOG_REGISTRATION,Error::CATALOG);
        add(Error::P_CATALOG_REGISTRATION,Error::CATALOG_INTERNAL_ERROR);
        add(Error::P_CATALOG_REGISTRATION,Error::CONNECTION);
    }
};

/**
 * S_PRESTAGING -> S_WAITING_PRESTAGE
 */
class PrestagingToWaitingPrestage : public ErrorMatch{
public:
    PrestagingToWaitingPrestage(){
        add(Error::P_PRESTAGING, Error::INTERNAL_ERROR);
        add(Error::P_PRESTAGING, Error::SECURITY,               Error::S_SOURCE);
        add(Error::P_PRESTAGING, Error::NETWORK,                Error::S_SOURCE);
        add(Error::P_PRESTAGING, Error::REQUEST_FAILURE,        Error::S_SOURCE);
        add(Error::P_PRESTAGING, Error::CONNECTION,             Error::S_SOURCE);
        add(Error::P_PRESTAGING, Error::HTTP_TIMEOUT,           Error::S_SOURCE);
        add(Error::P_PRESTAGING, Error::REQUEST_TIMEOUT,        Error::S_SOURCE);
        add(Error::P_PRESTAGING, Error::GENERAL_FAILURE,        Error::S_SOURCE);
        add(Error::P_PRESTAGING, Error::STORAGE_INTERNAL_ERROR, Error::S_SOURCE);
    }
};

/*
 * onEventFailed
 *
 * Notify the Transfer Failed Event
 */
void FileFSM::onEventFailed(
    const Error::Category&  category, 
    const Error::Scope&     scope, 
    const Error::Phase&     phase, 
    const std::string&      reason){
    
    // Save the old State
    File::State old_state = m_file.state();

    // Set Finished Time (if it's not set)
    if((time_t)-1 == m_file.finishTime()){
        time_t current;
        time(&current);
        m_file.finishTime = current;
    }
    bool set_reason = true;
    // Switch on File State
    switch(m_file.state()){
        case File::S_SUBMITTED:{
            static SubmittedToWaitingCatalogResolution s_SubmittedToWaitingCatalogResolution;
            if (s_SubmittedToWaitingCatalogResolution.isMatch(category,scope,phase)){
                // Change The State to WAITING_CATALOG_RESOLUTION
                m_file.state = File::S_WAITING_CATALOG_RESOLUTION;
                // Increase Failures Count
                m_file.failures = m_file.failures() + 1;
                // Increase CatalogFailures Count
                m_file.catalogFailures = m_file.catalogFailures() + 1;
            } else {
                // Change The State to FAILED
                m_file.state = File::S_FAILED;
                // Increase Failures Count
                m_file.failures = m_file.failures() + 1;
                // Increase Current Failures Count
                m_file.currentFailures = m_file.currentFailures() + 1;
            }
        } 
        break; // End SUBMITTED
        case File::S_PENDING:{
            static PendingToWaitingPrestage s_PendingToWaitingPrestage;
            if (s_PendingToWaitingPrestage.isMatch(category,scope,phase)){
                // Change The State to WAITING_PRESTAGE
                m_file.state = File::S_WAITING_PRESTAGE;
                // Increase Failures Count
                m_file.failures = m_file.failures() + 1;
                // Increase prestageFailures Count
                m_file.prestageFailures = m_file.prestageFailures() + 1;
            } else {
                // Change The State to FAILED
                m_file.state = File::S_FAILED;
                // Increase Failures Count
                m_file.failures = m_file.failures() + 1;
                // Increase Current Failures Count
                m_file.currentFailures = m_file.currentFailures() + 1;
            }
        } 
        break; // end PENDING
        case File::S_READY:
        case File::S_ACTIVE:{
            static ReadyOrActiveToWaiting s_ReadyOrActiveToWaiting;
            if (s_ReadyOrActiveToWaiting.isMatch(category,scope,phase)){
                m_file.state = File::S_WAITING;
            } else {
                m_file.state = File::S_FAILED;
            }
            // Increase Failures Count
            m_file.failures = m_file.failures() + 1;
            // Increase Current Failures Count
            m_file.currentFailures = m_file.currentFailures() + 1;
        } 
        break; // End ACTIVE
        case File::S_FINISHING: {
            static FinishingToWaitingCatalogRegistration s_FinishingToWaitingCatalogRegistration;
            if (s_FinishingToWaitingCatalogRegistration.isMatch(category,scope,phase)){
                // Change The State to WAITING_CATALOG_REGISTRATION
                m_file.state = File::S_WAITING_CATALOG_REGISTRATION;
            } else {
                // Change The State to FAILED
                m_file.state = File::S_FAILED;
            }
            // Increase Failures Count
            m_file.failures = m_file.failures() + 1;
            // Increase Current Failures Count
            m_file.currentFailures = m_file.currentFailures() + 1;
        } 
        break; // End FINISHING
        case File::S_PRESTAGING:{
            static PrestagingToWaitingPrestage s_PrestagingToWaitingPrestage;
            if (s_PrestagingToWaitingPrestage.isMatch(category,scope,phase)){
                // Change The State to WAITING_PRESTAGE
                m_file.state = File::S_WAITING_PRESTAGE;
            } else {
                // Change The State to FAILED
                m_file.state = File::S_FAILED;
            }
            // Increase Failures Count
            m_file.failures = m_file.failures() + 1;
            // Increase prestageFailures Count
            m_file.prestageFailures = m_file.prestageFailures() + 1;
        } 
        break; // End PRESTAGING
        case File::S_FAILED:
            // Don't Do Anything
        break;
        case File::S_DONE: {
            // Change The State to FAILED
            m_file.state = File::S_FAILED;
            // Increase Failures Count
            m_file.failures = m_file.failures() + 1;
            // Increase Current Failures Count
            m_file.currentFailures = m_file.currentFailures() + 1;
        } 
        break; // End DONE
        case File::S_WAITING:
        case File::S_AWAITING_PRESTAGE:
        case File::S_WAITING_PRESTAGE:
        case File::S_WAITING_CATALOG_RESOLUTION:
        case File::S_WAITING_CATALOG_REGISTRATION:{
            // Change The State to FAILED
            m_file.state = File::S_FAILED;
            if((m_file.reasonClass() != Error::UNDEF) &&
               (false == m_file.reason().empty())){
                // Do not owerwrite the last reason
                set_reason = false;
            }
        } break;
        case File::S_CANCELED:
        case File::S_HOLD:
        case File::S_FINISHED:
        default:
            // Inconsistent State. Ignore Error since it's onEventFailed 
            // shouldn't throw exceptions
            set_reason = false;
        break;
    } // End switch state
    if(true == set_reason){
        // Set the error Reason class
        if(category != Error::UNDEF){
            m_file.reasonClass = category;
        } else if(Error::UNDEF == m_file.reasonClass()){
            m_file.reasonClass = Error::INTERNAL_ERROR;
        }
        // Set the error Reason
        if(false == reason.empty()){
            m_file.reason = reason;
        } else if(true == m_file.reason().empty()){
            m_file.reason = "Unknown Reason";
        }
        m_file.errorScope = scope;
        m_file.errorPhase = phase;
    }
    // Notify ChangeState Event (if needed)
    eventStateChanged(old_state);
}

/*
 * onEventReschedule
 *
 * Notify the Reschedule Event
 */
void FileFSM::onEventReschedule(bool overwrite_flag) /*throw (FSMException)*/ {
    // Save the old State
    File::State old_state = m_file.state();
    
    switch(m_file.state()){
        case File::S_WAITING:{
            // Change The State to Pending
            m_file.state = File::S_PENDING;
            // Set the file overwrite flag
            if((true == overwrite_flag) &&
               (std::string::npos == m_file.internalParams().find(FILE_PARAM_OVERWRITE))){
                m_file.internalParams() += FILE_PARAM_OVERWRITE;
            }
            // Reset the File Information
            m_file.finishTime = -1;
            m_file.reason().clear();
            m_file.reasonClass = Error::UNDEF;
        } break; // End WAITING
        case File::S_WAITING_CATALOG_RESOLUTION:{
            // Change The State to Submitted
            m_file.state = File::S_SUBMITTED;
            // Reset the Error
            m_file.finishTime = -1;
            m_file.reason().clear();
            m_file.reasonClass = Error::UNDEF;
        } break; // End WAITING_CATALOG_RESOLUTION
        case File::S_WAITING_CATALOG_REGISTRATION:{
            // Change The State to Finishing
            m_file.state = File::S_FINISHING;
            // Reset the Error
            m_file.finishTime = -1;
            m_file.reason().clear();
            m_file.reasonClass = Error::UNDEF;
        } break; // End WAITING_CATALOG_REGISTRATION
        case File::S_WAITING_PRESTAGE:{
            // Change The State to Pending
            m_file.state = File::S_PENDING;
            // Reset the Error
            m_file.finishTime = -1;
            m_file.reason().clear();
            m_file.reasonClass = Error::UNDEF;
        } break; // End WAITING_PRESTAGE
        default:
            throw InvalidStateException("Event Reschedule received for File "\
                                        "not in one of the Waiting states");
    }
    // Notify ChangeState Event (if needed)
    eventStateChanged(old_state);
}

/*
 * onEventHold
 *
 * Notify the Hold Event
 */
void FileFSM::onEventHold() /*throw (FSMException)*/ {
    // Save the old State
    File::State old_state = m_file.state();
    
    switch(m_file.state()){
        case File::S_WAITING:
        case File::S_WAITING_PRESTAGE:{
            if(true == FSMConfig::instance().holdEnabled()){
                m_file.state = File::S_HOLD;
            } else {
                m_file.state = File::S_FAILED;
            }
        }
        break;
        default:
            throw InvalidStateException("Event Hold received for File "\
                                        "not in [Waiting,WaitingPrestage] states");
    }
    // Notify ChangeState Event (if needed)
    eventStateChanged(old_state);
}
    
/*
 * onEventCanceled
 *
 * Notify the Canceled Event
 */
void FileFSM::onEventCanceled() /*throw ()*/{
    // Save the old State
    File::State old_state = m_file.state();
    
    std::string reason = "Job canceled";
    switch(m_file.state()){
        case File::S_CANCELED:
        case File::S_FINISHED:
        case File::S_FAILED:
            // Don't do anything
        break; 
        case File::S_DONE:
        case File::S_FINISHING:
            // Don't do anything
            reason = "Job canceled, but transfer completed successfully";
            // #warning "bug #33712 - DONE or FINISHING should not go to CANCELED (insert a break here?)            
        case File::S_SUBMITTED:
        case File::S_PENDING:
        case File::S_READY:
        case File::S_ACTIVE:
        case File::S_PRESTAGING:
        case File::S_WAITING:
        case File::S_WAITING_PRESTAGE:
        case File::S_WAITING_CATALOG_RESOLUTION:
        case File::S_WAITING_CATALOG_REGISTRATION:
        case File::S_AWAITING_PRESTAGE:
        case File::S_HOLD:
        default:
            // Change The State to CANCELED
            m_file.state = File::S_CANCELED;
            if(true == m_file.reason().empty()){
                m_file.reason = reason;
            }
            m_file.reasonClass  = Error::UNDEF;
            m_file.errorScope   = Error::S_UNDEF;
            m_file.errorPhase   = Error::P_UNDEF;
        break;
    }
    // Notify ChangeState Event (if needed)
    eventStateChanged(old_state);
}

/*
 * onEventFinishing
 *
 * Notify the Finishing Event
 */
void FileFSM::onEventFinishing() /*throw (FSMException)*/{
    // Save the old State
    File::State old_state = m_file.state();
    
    switch(m_file.state()){
        case File::S_DONE:
            // Call Post-Transfer Hook
            EventDispatcher::instance().fileDone(m_file, m_job);

            // Change The State to Finishing
            m_file.state = File::S_FINISHING;
        break;
        default:
            throw InvalidStateException("Event Finishing received for File "\
                                        "not in [Done] state");
    }
    // Notify ChangeState Event (if needed)
    eventStateChanged(old_state);
}

/*
 * onEventFinished
 *
 * Notify the Finished Event
 */
void FileFSM::onEventFinished() /*throw (FSMException)*/{
    // Save the old State
    File::State old_state = m_file.state();
    
    switch(m_file.state()){
        case File::S_FINISHING:
            // Change The State to Finished
            m_file.state = File::S_FINISHED;
        
            // Call Hook
            EventDispatcher::instance().fileFinished(m_file, m_job);
        break;
        default:
            throw InvalidStateException("Event Finished received for File "\
                                        "not in [Finishing] state");
    }
    // Notify ChangeState Event (if needed)
    eventStateChanged(old_state);
}

/*
 * eventStateChanged
 *
 * Dispatch the Job's State Changed event, if needed
 */
void FileFSM::eventStateChanged(File::State old_state) throw () {
    // Check if the File's State is changed
    if((true == m_file.state.isSet()) && (m_file.state() != old_state)){
        // Dispatch State Changed Event 
        EventDispatcher::instance().fileStateChanged(m_file, m_job, old_state);
    }
}
