/*
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *  GLite Data Transfer Agents: FileFSM Class
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *  Version info: $Id: FileFSM.h,v 1.6 2007/07/04 13:05:29 ptedesco Exp $
 *  Release: $Name:  $
 *
 */
#ifndef GLITE_DATA_TRANSFER_AGENTS_FSM_FILEFSM_H_
#define GLITE_DATA_TRANSFER_AGENTS_FSM_FILEFSM_H_

#include <string>
#include "glite/data/transfer/agent/model/File.h"
#include "glite/data/transfer/agent/model/Job.h"
#include "glite/data/transfer/agent/model/Transfer.h"
#include "glite/data/transfer/agent/model/StageReq.h"
#include "glite/data/transfer/agent/model/Error.h"
#include "FSMException.h"
#include <boost/utility.hpp>

namespace glite   {
namespace data    {
namespace transfer{
namespace agent   {
namespace fsm     {
/**
 * FileFSM. Implements the logic to handle the transitions between the 
 * different States of a model:File Instance. It operates as a Visitor on the 
 * File Object. This logic could be implemented also on the model:File
 * class, but then it would violate the ValueObject pattern.
 */
class FileFSM : boost::noncopyable{
public:

    /**
     * Constructor
     */
    FileFSM(glite::data::transfer::agent::model::File& f, 
      const glite::data::transfer::agent::model::Job& j):m_file(f),m_job(j){}

    /**
     * Notify the Allocated Event
     */
    void onEventAllocate() /*throw (FSMException)*/;

    /**
     * Notify that the File is Ready
     */
    void onEventFileReady() /*throw (FSMException)*/;

    /**
     * Notify that the file is Not Ready
     */
    void onEventFileNotReady() /*throw (FSMException)*/;

    /**
     * Notify the Start Staging Event
     */
    void onEventStartStaging(glite::data::transfer::agent::model::StageReq& r) /*throw (FSMException)*/;

    /**
     * Notify the Staging Event
     */
    void onEventStaging(glite::data::transfer::agent::model::StageReq& t) /*throw (FSMException)*/;

    /**
     * Notify the Start Event
     */
    void onEventStart(glite::data::transfer::agent::model::Transfer& t) /*throw (FSMException)*/;

    /**
     * Notify the Running Event
     */
    void onEventRunning(glite::data::transfer::agent::model::Transfer& t) /*throw (FSMException)*/;

    /**
     * Notify the Transfer Done Event
     */
    void onEventDone(glite::data::transfer::agent::model::Transfer& t) /*throw (FSMException)*/;

    /**
     * Notify the Transfer Failed Event
     */
    void onEventFailed(glite::data::transfer::agent::model::Transfer& t) /*throw ()*/;

    /**
     * Notify the Staging Failed Event
     */
    void onEventFailed(glite::data::transfer::agent::model::StageReq& t) /*throw ()*/;

    /**
     * Notify the Failed Event
     */
    void onEventFailed(
        const glite::data::transfer::agent::model::Error::Category& category, 
        const glite::data::transfer::agent::model::Error::Scope&    scope, 
        const glite::data::transfer::agent::model::Error::Phase&    phase, 
        const std::string&                                          reason);

    /**
     * Notify the Reschedule Event
     */
    void onEventReschedule(bool overwrite_flag = false) /*throw (FSMException)*/;

    /**
     * Notify the Hold Event
     */
    void onEventHold() /*throw (FSMException)*/ ;
    
    /**
     * Notify the Canceled Event
     */
    void onEventCanceled() /*throw ()*/;

    /**
     * Notify the Finishing Event
     */
    void onEventFinishing() /*throw (FSMException)*/;

    /**
     * Notify the Finished Event
     */
    void onEventFinished() /*throw (FSMException)*/;

private:
        
    /**
     * Dispatch the File's State Changed event, if needed
     */
    void eventStateChanged(glite::data::transfer::agent::model::File::State old_state) throw ();

private:
    /**
     * Default Constructor. Not Implemented
     */
    FileFSM();

private:
    /**
     * File Property
     */
    glite::data::transfer::agent::model::File& m_file;
    /**
     * Job Property
     */
    const glite::data::transfer::agent::model::Job& m_job;

    /** @link dependency 
     * @label throws*/
    /*# FSMException lnkFSMException; */
};

} // end namespace fsm
} // end namespace agent
} // end namespace transfer
} // end namespace data
} // end namespace glite

#endif //GLITE_DATA_TRANSFER_AGENTS_FSM_FILEFSM_H_
