/*
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *  GLite Data Transfer Agents: JobFSM Class
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *  Version info: $Id: JobFSM.cpp,v 1.8 2008/04/11 11:02:55 ptedesco Exp $
 *  Release: $Name:  $
 *
 */
#include "JobFSM.h"
#include "EventDispatcher.h"
#include "InvalidStateException.h"

using namespace glite::data::transfer::agent;
using namespace glite::data::transfer::agent::model;
using namespace glite::data::transfer::agent::fsm;

/*
 * updateState
 *
 * Update the Job state according to the states of all the related Files. 
 */
void JobFSM::updateState(File::State states) /*throw (FSMException)*/{
    
    Job::State old_state = m_job.state();
    
    // Check if job is canceled
    if(true == m_job.cancelJob()){
        // In order to be canceled the files should be either CANCELED, FAILED
        // or FINISHED
        File::State mask = (File::State)(File::S_CANCELED | 
                                         File::S_FAILED   | 
                                         File::S_FINISHED);
        if((states | mask) == mask){
            // Check that the Job is not already completed
            switch(m_job.state()){
                case Job::S_CANCELED:
                case Job::S_FINISHED:
                case Job::S_FINISHED_DIRTY:
                case Job::S_FAILED:
                    // Don't do anything
                break;
                default:
                    // Job is canceled
                    m_job.state = Job::S_CANCELED;
                break;
            } // End switch
            
            // Notify ChangeState Event (if needed)
            eventStateChanged(old_state);
            return;
        }
    }
    
    // Check Job State    
    switch(m_job.state()){
        case Job::S_SUBMITTED: {
            // Check Inconsistent State
            if(states & (File::S_READY                        |
                         File::S_ACTIVE                       |
                         File::S_DONE                         |
                         File::S_WAITING                      |
                         File::S_HOLD                         |
                         File::S_AWAITING_PRESTAGE            |
                         File::S_PRESTAGING                   |
                         File::S_WAITING_PRESTAGE             |
                         File::S_WAITING_CATALOG_REGISTRATION |
                         File::S_FINISHING                    |
                         File::S_FINISHED)){
                // The File State is not one of the expected ones
                throw InvalidStateException("inconsistent files' states for Job in Submitted state");
            }
            // If all files fail the allocation, move the job to failed
            if(states == File::S_FAILED){
                m_job.state = Job::S_FAILED;
            } else {
                // If some files fail the allocation, don't move the job to failed
                File::State mask = (File::State)(File::S_PENDING  | 
                                                 File::S_FAILED   | 
                                                 File::S_CANCELED);
                if((states | mask) == mask){
                    m_job.state = Job::S_PENDING;
                    
                    // Call Job Pending Hook
                    EventDispatcher::instance().jobPending(m_job);
                } else {
                    // State is still Submitted
                }
            }
        } break; // End SUBMITTED
        case Job::S_PENDING: {
            // Check Inconsistent State
            if((states & (File::S_SUBMITTED                   | 
                         File::S_ACTIVE                       |
                         File::S_FINISHING                    |
                         File::S_FINISHED                     |
                         File::S_WAITING                      |
                         File::S_WAITING_CATALOG_RESOLUTION   | 
                         File::S_WAITING_CATALOG_REGISTRATION)) ||
               (states == File::S_DONE)){
                throw InvalidStateException("inconsistent files states' for a Pending Job");
            }
            // Check if some files are still pending
            if(states & (File::S_PENDING |
                         File::S_AWAITING_PRESTAGE |
                         File::S_PRESTAGING |
                         File::S_WAITING_PRESTAGE)){
                // Job should still be Pending
            } else {
                // File states could be only READY,DONE,HOLD,FAILED,CANCELED
                if((states & File::S_READY) ||
                   (states == (File::S_DONE | File::S_CANCELED)) ||
                   (states == (File::S_DONE | File::S_FAILED | File::S_CANCELED))){
                    m_job.state = Job::S_READY;
                } else if(states & File::S_HOLD){
                    m_job.state = Job::S_HOLD;
                } else {
                    // Go to Failed since the Canceled should already be catched 
                    // with the check at the beginning
                    m_job.state = Job::S_FAILED;
                }
            }
        } break; // End PENDING
        case Job::S_READY: {
            // Check Inconsistent State
            if(states & (File::S_SUBMITTED                    |
                         File::S_PENDING                      |
                         File::S_AWAITING_PRESTAGE            |
                         File::S_PRESTAGING                   |
                         File::S_WAITING_PRESTAGE             |   
                         File::S_FINISHED                     |
                         File::S_FINISHING                    |
                         File::S_WAITING_CATALOG_RESOLUTION   | 
                         File::S_WAITING_CATALOG_REGISTRATION)){
                throw InvalidStateException("inconsistent files states' for a Job in Ready state");
            }

            if (states == File::S_READY){
                // Job State still Ready
            } else {
                // Files' states could be only READY,ACTIVE,WAITING,DONE,HOLD,FAILED
                if(states & (File::S_ACTIVE  | 
                             File::S_WAITING |
                             File::S_READY)){
                    // Job State should go in Active
                    m_job.state = Job::S_ACTIVE;
                } else if(states & File::S_HOLD){
                    m_job.state = Job::S_HOLD;
                } else if(states & File::S_DONE){
                    // Job State should go in Active
                    m_job.state = Job::S_ACTIVE;
                } else {
                    m_job.state = Job::S_FAILED;
                }
            }
        } break; // End READY
        case Job::S_ACTIVE:{
            // Check Inconsistent State
            if(states & (File::S_SUBMITTED                    | 
                         File::S_FINISHED                     |
                         File::S_FINISHING                    |
                         File::S_WAITING_CATALOG_RESOLUTION   | 
                         File::S_WAITING_CATALOG_REGISTRATION)){
                throw InvalidStateException("inconsistent files states' for a Job in Active state");
            }
            
            // Check if some files are still Pending, Ready or Active
            if(states & (File::S_PENDING |
                         File::S_READY   |
                         File::S_ACTIVE  |
                         File::S_WAITING  |
                         File::S_AWAITING_PRESTAGE |
                         File::S_PRESTAGING |
                         File::S_WAITING_PRESTAGE)){
                // Job should still be Active
            } else {
                if(!(states & File::S_CANCELED)){
                    // Files' states could be only DONE,HOLD,FAILED
                    if(states & File::S_HOLD){
                        if(false == m_job.cancelJob()){
                            m_job.state = Job::S_HOLD;
                        } else {
                            // Job stays Active: Wait the job is canceled
                        }
                    } else {
                        if(states == File::S_DONE){
                            // Job State should be Done
                            m_job.state = Job::S_DONE;
                        } else if (states & File::S_DONE){
                            m_job.state = Job::S_DONE_WITH_ERRORS;
                        } else {
                            m_job.state = Job::S_FAILED;
                        }
                    }
                } else {
                    // Files' states could be CANCELED,DONE,HOLD,FAILED
                    if((states & (File::S_DONE | File::S_HOLD))){
                        // Don't modify File Status, Done and Hold files should 
                        // be canceled before
                    } else {
                        // File State can be only CANCELED,FAILED
                        // but this conidition is already handled at the 
                        // beginning: Consider Job failed
                        m_job.state = Job::S_FAILED;
                    }
                }
            }
        } break; // End ACTIVE
        case Job::S_DONE: {
            // Check Inconsistent State
            if(states & (File::S_SUBMITTED                    | 
                         File::S_PENDING                      | 
                         File::S_READY                        | 
                         File::S_ACTIVE                       | 
                         File::S_FAILED                       | 
                         File::S_WAITING                      | 
                         File::S_HOLD                         |
                         File::S_AWAITING_PRESTAGE            |
                         File::S_PRESTAGING                   |
                         File::S_WAITING_PRESTAGE             |
                         File::S_WAITING_CATALOG_RESOLUTION   |
                         File::S_WAITING_CATALOG_REGISTRATION |
                         File::S_FINISHED)){
                throw InvalidStateException("inconsistent files states' for a Job in Active state");
            }
            // Check File States (Can Be only FINISHING and DONE)
            if(states & File::S_FINISHING){
                // Move Job state to finished
                m_job.state = Job::S_FINISHING;
            } else {
                // Job's state still Done
            }
        } break; // End DONE
        case Job::S_DONE_WITH_ERRORS: {
            // Check Inconsistent State
            if(states & (File::S_SUBMITTED                    | 
                         File::S_PENDING                      | 
                         File::S_READY                        | 
                         File::S_ACTIVE                       | 
                         File::S_WAITING                      | 
                         File::S_HOLD                         |
                         File::S_AWAITING_PRESTAGE            |
                         File::S_PRESTAGING                   |
                         File::S_WAITING_PRESTAGE             |
                         File::S_WAITING_CATALOG_RESOLUTION   |
                         File::S_WAITING_CATALOG_REGISTRATION |
                         File::S_FINISHED)){
                throw InvalidStateException("inconsistent files states' for a Job in Active state");
            }
            // Check File States (Can Be only FINISHING, DONE and FAILED)
            if(states == File::S_FAILED) {
                // Move Job state to Failed
                m_job.state = Job::S_FAILED;
            } else if(states & File::S_FINISHING) {
                // Move Job state to finishing
                m_job.state = Job::S_FINISHING;
            } else {
                // Job's state still DoneWithErrors
            }
        } break; // End DONE_WITH_ERRORS
        case Job::S_FINISHING: {
            // Check Inconsistent State
            if(states & (File::S_SUBMITTED                    | 
                         File::S_PENDING                      | 
                         File::S_READY                        | 
                         File::S_ACTIVE                       | 
                         File::S_DONE                         | 
                         File::S_WAITING                      | 
                         File::S_HOLD                         |
                         File::S_AWAITING_PRESTAGE            |
                         File::S_PRESTAGING                   |
                         File::S_WAITING_PRESTAGE             |
                         File::S_WAITING_CATALOG_RESOLUTION)){
                throw InvalidStateException("inconsistent files states' for a Job in Active state");
            }
            // Check File States (Can Be FINISHED, FINISHING, FAILED, WAITING_CATALOG_REG)
            if(states & (File::S_FINISHING | File::S_WAITING_CATALOG_REGISTRATION)){
                // Job is Still finishing
            } else if(states == File::S_FINISHED){
                // Job is finished
                m_job.state = Job::S_FINISHED;
                
                // Call Job Finished Hook
                EventDispatcher::instance().jobFinished(m_job);
            } else {
                // Job is failed
                m_job.state = Job::S_FINISHED_DIRTY;
                
                // Call Job Finished Hook
                EventDispatcher::instance().jobFinished(m_job);
            }
        } break; // End FINISHING
        case Job::S_FINISHED: {
            // Check Inconsistent State
            if(states != File::S_FINISHED){
                throw InvalidStateException("inconsistent files' states for JOb in Finished state");
            }
        } break; // End FINISHED
        case Job::S_FINISHED_DIRTY: {
            File::State fin_d_mask = (File::State)(File::S_FAILED   | 
                                                  File::S_FINISHED);
            
            // Check Inconsistent State
            if((states | fin_d_mask) != fin_d_mask){
                throw InvalidStateException("inconsistent files' states for Job in FinishedDirty state");
            }
        } break; // End FINISHED_DIRTY
        case Job::S_CANCELED: {
            if(states & (File::S_SUBMITTED                    | 
                         File::S_PENDING                      | 
                         File::S_READY                        | 
                         File::S_ACTIVE                       | 
                         File::S_DONE                         | 
                         File::S_WAITING                      | 
                         File::S_HOLD                         |
                         File::S_AWAITING_PRESTAGE            |
                         File::S_PRESTAGING                   |
                         File::S_WAITING_PRESTAGE             |
                         File::S_WAITING_CATALOG_RESOLUTION   |
                         File::S_WAITING_CATALOG_REGISTRATION |
                         File::S_FINISHING)){
                throw InvalidStateException("inconsistent files states' for a Job in Canceled state");
            }
        } break; // End FINISHED
        case Job::S_HOLD: {
            if(states & (File::S_SUBMITTED                    | 
                         File::S_PENDING                      | 
                         File::S_READY                        | 
                         File::S_ACTIVE                       | 
                         File::S_WAITING                      | 
                         File::S_AWAITING_PRESTAGE            |
                         File::S_PRESTAGING                   |
                         File::S_WAITING_PRESTAGE             |
                         File::S_WAITING_CATALOG_RESOLUTION   |
                         File::S_WAITING_CATALOG_REGISTRATION |
                         File::S_FINISHING                    | 
                         File::S_FINISHED)){
                throw InvalidStateException("inconsistent files states' for a Job in Hold state");
            } else {
                // Don't do anything
            }
        } break; // End HOLD
        case Job::S_FAILED: {
            if(states & (File::S_SUBMITTED                    | 
                         File::S_PENDING                      | 
                         File::S_READY                        | 
                         File::S_ACTIVE                       | 
                         File::S_WAITING                      | 
                         File::S_AWAITING_PRESTAGE            |
                         File::S_PRESTAGING                   |
                         File::S_HOLD                         |
                         File::S_WAITING_PRESTAGE             |
                         File::S_WAITING_CATALOG_RESOLUTION   |
                         File::S_WAITING_CATALOG_REGISTRATION |
                         File::S_FINISHING)){
                throw InvalidStateException("inconsistent files states' for a Job in Failed state");
            } else {
                // Don't do anything
            }
        } break; // End FAILED
        default:
            // The Job State is not one of the expected ones
            throw InvalidStateException("unknown Job's state");
        break;
    } // End switch
    
    // In case of failures, set the Job Reason 
    if(((m_job.state() == Job::S_FINISHED_DIRTY) || 
        (m_job.state() == Job::S_FAILED)) && 
       (true == m_job.reason().empty())){
        // Set the Job reason to a default value
        m_job.reason = "One or more files failed. Please have a look at the "\
            "details for more information";
    }
    
    // Notify ChangeState Event (if needed)
    eventStateChanged(old_state);
}

/*
 * eventStateChanged
 *
 * Dispatch the Job's State Changed event, if needed
 */
void JobFSM::eventStateChanged(Job::State old_state) throw () {
    // Check if the Job State is changed
    if((true == m_job.state.isSet()) && (m_job.state() != old_state)){
        // Dispatch State Changed Event 
        EventDispatcher::instance().jobStateChanged(m_job,old_state);
    }
}
