/*
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *  GLite Data Transfer Agents: Default NameGeneration Class
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *  Version info: $Id: DefaultNameGeneration.cpp,v 1.4 2008/01/08 09:35:26 ptedesco Exp $
 *  Release: $Name:  $
 *
 */
#include "DefaultNameGeneration.h"
#include "glite/data/agents/Guid.h"

#include <algorithm>
#include <cctype>
#include <sstream>
#include <cstring>

using namespace glite::data::transfer::agent::namegen;
using glite::data::agents::LogicError;
using glite::data::agents::Guid;
        
namespace{
    
const char * const LFN_PROTOCOL    = "lfn:";
const char * const GUID_PROTOCOL   = "guid:";
    
/*
 * eq_nocase
 *
 * Character Comparison that ignore the case
 */
inline bool eq_nocase(char c1, char c2){ 
    return tolower(c1) == tolower(c2);
}

/*
 * is_lfn
 *
 * Check if the given logical name is an LFN
 */
inline bool is_lfn(const std::string& logical){ 
    bool result = false;
    if(*(logical.begin()) == '/'){
        result = true;
    } else if(true == std::equal(LFN_PROTOCOL,LFN_PROTOCOL + strlen(LFN_PROTOCOL), logical.begin(),eq_nocase)){
        result = true;
    }
    return result;
}

/*
 * trim_lfn_protocol
 *
 * Remove the lfn protocol from the logical file name
 */
inline const char * trim_lfn_protocol(const std::string& logical){ 
    
    const char * lfn = logical.c_str();
    if(true == std::equal(LFN_PROTOCOL,LFN_PROTOCOL + strlen(LFN_PROTOCOL), logical.begin(),eq_nocase)){
        lfn += strlen(LFN_PROTOCOL);
    } 
    if(true == std::equal(lfn,lfn + 2, "//")){
        lfn += 2;
    }
    return lfn;
}

/*
 * trim_lfn_protocol
 *
 * Remove the guid protocol from the logical file name
 */
inline const char * trim_guid_protocol(const std::string& logical){ 
    
    const char * guid = logical.c_str();
    if(true == std::equal(GUID_PROTOCOL,GUID_PROTOCOL + strlen(GUID_PROTOCOL), logical.begin(),eq_nocase)){
        guid += strlen(GUID_PROTOCOL);
    } 
    if(true == std::equal(guid,guid + 2, "//")){
        guid += 2;
    }
    return guid;
}
} // end global namespace

/*
 * generate
 *
 * Generate a destination Name SURL starting from the logical name and the 
 * Se path
 */
std::string DefaultNameGeneration::generate(const std::string& logical, const std::string& endpoint, const std::string& se_path, const std::string& storage_class) /*throw (NameGenerationException)*/{

    std::string surl;

    // Get protocol position    
    std::string::size_type protocol_pos = endpoint.find(":");
    if(protocol_pos == std::string::npos){
        throw LogicError("Invalid endpoint: No protocol specified");
    }
    // compute SURL prefix
    std::string surl_prefix = (std::string)"srm" + 
        endpoint.substr(protocol_pos) + "?SFN=" + se_path;
    
    std::string name_str;
    if(is_lfn(logical)){
        name_str = trim_lfn_protocol(logical);
    } else {
        // Handle GUID
        std::string g_str = trim_guid_protocol(logical);
        Guid g(g_str);
        if(true == g.isNull()){
            name_str = (std::string)"/GUID/" + trim_guid_protocol(g_str);
        } else {
            // Get the hash of the Guid
            unsigned short h  = g.hash();
            unsigned int   cs = g.crossSumHash();
            std::stringstream ss;
            ss << "/" << cs << "/" << h << "/" << g_str;
            name_str = ss.str();
        }
    }                
    if(*se_path.rbegin() == '/'){
        if(*(name_str.begin()) == '/'){
            surl = surl_prefix + name_str.substr(1);
        } else {
            surl = surl_prefix + name_str;
        }
    } else {
        if(*(name_str.begin()) == '/'){
            surl = surl_prefix + name_str;
        } else {
            surl = surl_prefix + (std::string) "/" + name_str;
        }
    }
    return surl;
}
