/**
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *  GLite Data Transfer Agents: NameGenConfig Class
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *           Paolo Tedesco <paolo.tedesco@cern.ch>
 *  Version info: $Id: NameGenConfig.cpp,v 1.5 2008/11/13 10:43:56 ptedesco Exp $ 
 *  Release: $Name:  $
 *
 */
#include "NameGenConfig.h"
#include "glite/data/agents/python/PyHelper.h"
#include "glite/data/agents/python/PythonException.h"

using namespace glite::data::transfer::agent::namegen;
using namespace glite::data::agents::python;

namespace py = boost::python;

/*
 * Initialize Constants
 */
const char * const NameGenConfig::NAMEGEN_VERSION_FUNCTION_NAME          = "NameGenPluginVersion";
const char * const NameGenConfig::NAMEGEN_V_1_0_GENERATE_FUNCTION_NAME   = "Generate";
const char * const NameGenConfig::NAMEGEN_V_1_0_INIT_FUNCTION_NAME       = "InitNameGenPlugin";
const char * const NameGenConfig::NAMEGEN_V_1_0                          = "1.0";

/*
 * instance
 *
 * Return the singleton Instance
 */
NameGenConfig& NameGenConfig::instance() {
    static NameGenConfig the_instance;
    return the_instance;
}

/*
 * validateRetry
 *
 * Verify that the Retry module is fine and get the versions
 */
bool NameGenConfig::validate() /*throw()*/ {
    
    bool result = false;

    PyHelper& py_helper = PyHelper::instance();
    try{
        // Load the Module
        m_module = py_helper.import(m_namegenModule);
        m_log_info("NameGen Module " << m_namegenModule << " Loaded");
                
        // Get the API Versison
        py::object get_version = py_helper.getFunction(m_module, NAMEGEN_VERSION_FUNCTION_NAME);

        // Call Get Version
        m_namegenVersion = py::call<std::string>(get_version.ptr());
        m_log_info("NameGen Version is " << m_namegenVersion);
        
        // Based on the version, check that the required functions are 
        // implemented
        if(m_namegenVersion == NAMEGEN_V_1_0){
            // Get the function
            m_generate = py_helper.getFunction(m_module,NAMEGEN_V_1_0_GENERATE_FUNCTION_NAME);
            m_log_info("Got NameGen Function <" << NAMEGEN_V_1_0_GENERATE_FUNCTION_NAME << ">");
            // Module is OK
            result = true;
        }
    } catch(const PythonException& exc){
        // Module Validation Failed
        m_log_info("NameGen Plugin validation failed. Error is " << exc.what());
    } catch(const std::exception& exc){
        // Module Validation Failed
        m_log_info("NameGen Plugin validation failed. Error is " << exc.what());
    } catch(py::error_already_set){
        std::pair<std::string,std::string> err = py_helper.lastError();
        m_log_error("NameGen Plugin validation failed. Error is " << err.first << " - " << err.second);
    } catch(...){
        m_log_error("NameGen Plugin validation failed. Unknown reason");
    }
    if(true == result){
        m_log_info("NameGen Plugin validation completed successfully");
    }
    
    return result;
}


/*
 * init
 *
 * Initialize the Retry module
 */
bool NameGenConfig::init(const std::string& params) /*throw()*/ {
    bool result = false;
    
    PyHelper& py_helper = PyHelper::instance();
    
    // Based on the version, call the right method
    if(m_namegenVersion == NAMEGEN_V_1_0){
        try{
            // Get the Init function
            py::object init_function = py_helper.getFunction(m_module,NAMEGEN_V_1_0_INIT_FUNCTION_NAME);
            m_log_info("Got Init Function <" << NAMEGEN_V_1_0_INIT_FUNCTION_NAME << ">");
            
            // Call Get Init Function
            result = py::call<bool>(init_function.ptr(),params);
            m_log_debug(NAMEGEN_V_1_0_INIT_FUNCTION_NAME << " returned " << str_bool(result));
        } catch(const PythonException& exc){
            // Module Initialization Failed
            m_log_warn("No InitRetry Method (" << exc.what() << "). Continue without initialization");
            result = true;
        } catch(const std::exception& exc){
            // Module Initialization Failed
            m_log_error("NameGen Plugin initialization failed. Error is " << exc.what());
        } catch(py::error_already_set){
            std::pair<std::string,std::string> err = py_helper.lastError();
            m_log_error("NameGen Plugin initialization failed. Error is " << err.first << " - " << err.second);
        } catch(...){
            m_log_error("NameGen Plugin initialization failed. Unknown reason");
        }
    }
        
    return result;
}
