/*
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *  GLite Data Transfer Agents: PythonCatalog Class
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *  Version info: $Id: PythonNameGen.cpp,v 1.4 2008/11/13 10:43:56 ptedesco Exp $
 *  Release: $Name:  $
 *
 */
#include "PythonNameGen.h"
#include "NameGenConfig.h"
#include <glite/data/agents/python/PyHelper.h>
#include <glite/data/agents/python/PythonException.h>

#include <algorithm>
#include <cctype>
#include <boost/scoped_ptr.hpp>

using namespace glite::data::transfer::agent;
using namespace glite::data::transfer::agent::namegen;
using namespace glite::data::agents::python;
using boost::scoped_ptr;

namespace py = boost::python;

/*
 * Constants
 */
namespace {
const char * PYTHONNAMEGEN_NAME      = "transfer-agent-namegen-python";
}


/*
 * PythonNameGen
 *
 * Constructor
 */
PythonNameGen::PythonNameGen():
    m_logger(PYTHONNAMEGEN_NAME){
}
 
/*
 * ~PythonNameGen
 *
 * Destructor
 */
PythonNameGen::~PythonNameGen(){
}
    
/*
 * generate
 *
 * Generate a destination Name SURL starting from the logical name and the 
 * Se path
 */
std::string PythonNameGen::generate(const std::string& logical,const std::string& endpoint, const std::string& se_path, const std::string& storage_class) /*throw (NameGenerationException)*/{
    std::string name;
    
    PyHelper& py_helper = PyHelper::instance();
    try{
        // Get the function
        py::object func = NameGenConfig::instance().generateFunction();
        m_log_debug("Got Generate Function");
                
        // Based on the version, Use the right signature
        if(NameGenConfig::instance().version() == NameGenConfig::NAMEGEN_V_1_0){
            m_log_debug("Using NameGen Version " << NameGenConfig::instance().version());
            // Call the Generate
            name = py::call<std::string>(func.ptr(),logical,endpoint,se_path,storage_class);
            m_log_debug("Generate returned " << name);
        } else  {
            m_log_error("Unsupported NameGenPlugin Version (" << NameGenConfig::instance().version() << ")");
            throw NameGenerationException("Unsupported NameGenPlugin Version");
        }
    } catch(const PythonException& exc){
        m_log_error("Generate failed. Error is " << exc.what());
        throw NameGenerationException(exc.what());
    } catch(const std::exception& exc){
        m_log_error("Generate failed. Error is " << exc.what());
        throw NameGenerationException(exc.what());
    } catch(py::error_already_set){
        std::pair<std::string,std::string> err = py_helper.lastError();
        std::string reason = (std::string)err.first.c_str() + " : " + err.second.c_str();
        m_log_error("Generate failed. Error is " << reason);
        throw NameGenerationException(reason.c_str());
    } catch(...){
        m_log_error("Generate failed. Unknown reason");
        throw;
    }
    
    return name;
}
