/**
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *  GLite Data Transfer Agents: PythonNameGenFactory Class
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *           Paolo Tedesco <paolo.tedesco@cern.ch>
 *  Version info: $Id: PythonNameGenFactory.cpp,v 1.7 2009/01/08 13:50:39 ptedesco Exp $
 *  Release: $Name:  $
 *
 */
#include "PythonNameGenFactory.h"
#include "PythonNameGen.h"
#include "NameGenConfig.h"
#include <glite/data/config/service/ParamReader.hxx>
#include <glite/data/agents/AgentExceptions.h>

using namespace glite::data::transfer::agent;
using namespace glite::data::transfer::agent::namegen;
using glite::data::agents::LogicError;
using glite::data::agents::ConfigParamException;
using glite::data::agents::ConfigurationException;
using namespace glite::config;

/*
 * Constants
 */
namespace {
const char * const PYTHONNAMEGEN_NAME             = "transfer-agent-namegen-python";
const char * const NAMEGENMODULE_PROPERTY_NAME    = "NameGenModule";
const char * const NAMEGENPARAMS_PROPERTY_NAME    = "NameGenParams";
}

extern "C" {
/*
 * create_glite_component
 *
 * Create Component instance
 */
ComponentConfiguration * create_glite_component(){
    return new PythonNameGenFactory();
}
                                                                                
/*
 * destroy_glite_component
 *
 * Destroy Component instance
 */
void destroy_glite_component(ComponentConfiguration * component){
    PythonNameGenFactory * factory = dynamic_cast<PythonNameGenFactory*>(component);
    if(0 != factory){
        delete factory;
    }
}
                                                                                
} // End extern "C"

/*
 * PythonNameGenFactory
 *
 * Constructor
 */
PythonNameGenFactory::PythonNameGenFactory():NameGenerationFactory(),
        ComponentConfiguration(PYTHONNAMEGEN_NAME), 
        m_logger(PYTHONNAMEGEN_NAME){
}
                                                                                                                          
/*
 * ~PythonNameGenFactory
 *
 * Destructor
 */
PythonNameGenFactory::~PythonNameGenFactory(){
}

/*
 * init
 *
 * Initialize the Component
 */
int PythonNameGenFactory::init(const Params& params){
    try{
        Params::const_iterator it;
    
        // Get NameGen Module
        if((it = params.find(NAMEGENMODULE_PROPERTY_NAME)) != params.end()){
            ParamValue * pv = dynamic_cast<ParamValue *>((*it).second);
            if(0 == pv){
                throw ConfigParamException(getName(),NAMEGENMODULE_PROPERTY_NAME,ConfigParamException::INVALID);
            }
            // NameGen Module
            NameGenConfig::instance().module(pv->getValue());
        }
        if(false == NameGenConfig::instance().module().empty()){
            // Get NameGen Parameters
            std::string namegen_params;
            if((it = params.find(NAMEGENPARAMS_PROPERTY_NAME)) != params.end()){
                ParamValue * pv = dynamic_cast<ParamValue *>((*it).second);
                if(0 == pv){
                    throw ConfigParamException(getName(),NAMEGENPARAMS_PROPERTY_NAME,ConfigParamException::INVALID);
                }
                // NameGen Parameters
                namegen_params = pv->getValue();
            }
            // Validate Modules
            bool is_valid = NameGenConfig::instance().validate();
            if(false == is_valid){
               throw ConfigurationException("NameGen Module Validation failed");
            }
            
            // Initialize Retry Module
            is_valid = NameGenConfig::instance().init(namegen_params);
            if(false == is_valid){
               throw ConfigurationException("NameGen Module Initialization failed");
            }
            // Log Configuration Values
            m_log_info(getName() << " Initialized. Parameters are");
            m_log_info("NameGenModule       : " << NameGenConfig::instance().module());
            m_log_debug("NameGenParams       : " << namegen_params);
        } else {
            // Log Configuration Values
            m_log_info(getName() << " Initialized without module");
        }
    } catch(const LogicError& exc) {
        m_log_error(getName() << " configuration Logic Error: " << exc.what());
        return -1;
    } catch(const std::exception& exc) {
        m_log_error(getName() << " configuration Unexpected Exception: " << exc.what());
        return -1;
    } catch(...) {
        m_log_error(getName() << " configuration Unhandled Exception");
        return -1;
    }
    
    return 0;
}

/*
 * config
 *
 * Configure the Component
 */
int PythonNameGenFactory::config(const Params& params){
    // Nothing to do
    m_log_info("PythonNameGenFactory Configured");
    return 0;
}

/*
 * fini
 *
 * Finalize the Component
 */
int PythonNameGenFactory::fini(){
    // Nothing to do
    return 0;
}

/*
 * create
 *
 * Create a NameGeneration instance
 */
NameGeneration * PythonNameGenFactory::create() /*throw ()*/{
    if(false == NameGenConfig::instance().module().empty()) {
        return new PythonNameGen();
    } else {
        return NameGenerationFactory::create();
    }
}
