/**
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *  GLite Data Transfer Agents: Define the FTS Class Wrapper Module
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>, Paolo Tedesco <paolo.tedesco@cern.ch>
 *  Version info: $Id: FtsModule.cpp,v 1.15 2008/11/13 14:20:28 ptedesco Exp $
 *  Release: $Name:  $
 *
 */
#include "FtsModule.h"

#include "FtsModule_Error.h"
#include "FtsModule_File.h"
#include "FtsModule_Job.h"
#include "FtsModule_StageReq.h"
#include "FtsModule_Transfer.h"

#include "PropertyConverters.h"

#include "glite/data/transfer/agent/action/vo/RetryStrategy.h"
#include "glite/data/transfer/agent/action/vo/CatalogRetryStrategy.h"

#include <boost/python/suite/indexing/vector_indexing_suite.hpp>
#include <boost/shared_ptr.hpp>
#include <vector>
#include <string>

using namespace glite::data::transfer::agent::python;
namespace py = boost::python;
/*
 * Python Module Name
 */    
const char * const glite::data::transfer::agent::python::FTS_MODULE_NAME = "_fts";

namespace glite   {
namespace data    {
namespace transfer{
namespace agent   {
namespace python  {
    
/*
 * get_schema_version
 *
 * Return FTS Schema Version
 */
const char * get_schema_version(){
    return QUEUE_SCHEMA_VERSION; 
}

/**
 * Create the wrappers for the Property Converters
 */
void Property_converters() {

    // String    
    py::to_python_converter<
      model::Property<std::string>,
      PropertyToPython<std::string> >();
    py::implicitly_convertible<std::string, model::Property<std::string> >();
    
    // unisgned int
    py::to_python_converter<
      model::Property<unsigned int>,
      PropertyToPython<unsigned int> >();
    py::implicitly_convertible<unsigned int, model::Property<unsigned int> >();

    // bool
    py::to_python_converter<
      model::Property<bool>,
      PropertyToPython<bool> >();
    py::implicitly_convertible<bool , model::Property<bool> >();
    
    // time_t
    py::to_python_converter<
      model::Property<time_t>,
      PropertyToPython<time_t> >();
    py::implicitly_convertible<time_t , model::Property<time_t> >();

    // double
    py::to_python_converter<
      model::Property<double>,
      PropertyToPython<double> >();
    py::implicitly_convertible<double , model::Property<double> >();

    // longlong
    py::to_python_converter<
      model::Property<agents::longlong>,
      PropertyToPython<agents::longlong> >();
    py::implicitly_convertible<agents::longlong , model::Property<agents::longlong> >();

}


//------------------------------------------------------------------------------
// RetryResult
//------------------------------------------------------------------------------

/**
 * Create and return the wrapper for the RetryStrategy::RetryResult enum
 */
void RetryResult_enum() {
                                                                                                                                            
    static py::object s_RetryResult_enum(
      py::enum_<action::vo::RetryStrategy::RetryResult>("RetryResult")
        .value("Wait"   , action::vo::RetryStrategy::WAIT)
        .value("Retry"  , action::vo::RetryStrategy::RETRY)
        .value("RetryWithOverwrite", action::vo::RetryStrategy::RETRY_WITH_OVERWRITE)
        .value("Hold"   , action::vo::RetryStrategy::HOLD)
        .value("Fail"   , action::vo::RetryStrategy::FAIL)
        .export_values()
    );
}

//------------------------------------------------------------------------------
// CatalogRetryResult
//------------------------------------------------------------------------------

/**
 * Create and return the wrapper for the CatalogRetryStrategy::CatalogRetryResult enum
 */
void CatalogRetryResult_enum() {
                                                                                                                                            
    static py::object s_CatalogRetryResult_enum(
      py::enum_<action::vo::CatalogRetryStrategy::CatalogRetryResult>("CatalogRetryResult")
        .value("Wait"   , action::vo::CatalogRetryStrategy::WAIT)
        .value("Retry"  , action::vo::CatalogRetryStrategy::RETRY)
        .value("Fail"   , action::vo::CatalogRetryStrategy::FAIL)
        .export_values()
    );
}

} // end namespace python
} // end namespace agent
} // end namespace transfer
} // end namespace data
} // end namespace glite

/*
 * Init Python Module
 */
BOOST_PYTHON_MODULE(_fts){

    // Declare GetSchemaVersion   
    py::def("SchemaVersion",get_schema_version);
    
    // Declare the wrapped classes
    Property_converters();
    JobState_enum();
    Job_class();
    ErrorCategory_enum();
    ErrorScope_enum();
    ErrorPhase_enum();
    FileState_enum();
    File_class();
    TransferState_enum();
    Transfer_class();
    StageReqState_enum();
    StageReq_class();

    // Vectors
    TransferArray_class();
    FileArray_class();
    
    // Action Strategy wrapper
    RetryResult_enum();
    CatalogRetryResult_enum();
} // end module glite_fts
