/*
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *  GLite Data Transfer Agents: ActionScheduler Class
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *           Paolo Tedesco <paolo.tedesco@cern.ch>
 *  Version info: $Id: ActionScheduler.cpp,v 1.19 2008/11/13 10:43:56 ptedesco Exp $
 *  Release: $Name:  $
 *
 */
#include <glite/data/transfer/agent/scheduler/ActionScheduler.h>
#include <glite/data/transfer/agent/scheduler/SchedulerEntry.h>
#include "SchedulerConfig.h"

#include <unistd.h>

using namespace glite::data::transfer::agent;
using namespace glite::data::transfer::agent::action;
using glite::data::agents::LogicError;
using glite::data::agents::RuntimeError;
using glite::data::agents::ActiveObject;
using glite::data::agents::dao::DAOContext;
using glite::data::agents::dao::DAOException;

namespace {
const char * const ACTION_SCHEDULER_NAME    = "glite-transfer-scheduler-";
const unsigned int SLEEP_TIME               = 1;
const unsigned int CHECK_FAILURE_SLEEP_TIME = 10;
}   

/*
 * ActionScheduler
 *
 * Constructor
 */
ActionScheduler::ActionScheduler(const std::string& name, DAOContext& ctx, Listener * listener):
        ActiveObject((std::string)ACTION_SCHEDULER_NAME + name, SchedulerConfig::instance()->stopTimeout()),
        m_logger((std::string)ACTION_SCHEDULER_NAME + name),
        m_listener(listener),
        m_ctx(ctx){
}

/*
 * ~ActionScheduler
 *
 * Destructor
 */
ActionScheduler::~ActionScheduler(){
    // Stop The Thread
    stop();
    
    // Clear Scheduled Entries
    clear();
}

/*
 * add
 *
 * Add a new entry for the Scheduler
 */
void ActionScheduler::add(SchedulerEntry* entry){
    if(false == this->isStarted()){
        // Create an Action Object and Store it on the Actions List
        m_actions.push_back(entry);
    } else {
        // Discard Entry
        m_log_warn("Cannot add Scheduler entry (scheduler already started). Discard it");
        delete entry;
    }
}

/*
 * clear
 *
 * Clear Scheduled Entries
 */
void ActionScheduler::clear(){
    if(false == this->isStarted()){
        SchedulerEntryList::iterator lit;        
        for(lit = m_actions.begin(); lit != m_actions.end(); ++lit){
            SchedulerEntry * entry = (*lit);
            if(0 != entry){
                delete entry;
            }
        }
        m_actions.clear();
    }
}

/*
 * svc
 *
 * Service Main Method
 */
void ActionScheduler::svc() throw (){
    
    time_t start_time           = 0;
    time_t current_time         = 0;
    time_t last_execution_time  = 0;
    time_t last_check           = 0;
    time_t diff                 = 0;
    time(&start_time);
    
    unsigned long max_failures   = SchedulerConfig::instance()->maxFailures();
    unsigned long disable_time   = SchedulerConfig::instance()->disableTime();
    unsigned long check_interval = SchedulerConfig::instance()->checkInterval();
    
    // Action Scheduler Main Loop
#warning "TODO: Improve the Scheduler algorithm, maybe using priority Queues and timers."
    // The implementation of the scheduler is very inefficient but this 
    // was the fastest one to code. 
    try{    
        while(true == this->doRun()){
            
            // Sleep for a while
            sleep(SLEEP_TIME);
            
            // Get Current
            time(&current_time);
            
            m_log_debug("Perform Scheduler iteration on time " << current_time);

            // Block Signal
            CriticalLock lock(*this);
            
            // Check DAO Context
            if((current_time - last_check) > check_interval){
                try{
                    m_log_debug("Checking DAO Context");
                    m_ctx.check();
                    m_log_debug("DAO Context is valid");
                    last_check = current_time;
                } catch (const DAOException& exc) {
                    m_log_warn("DAO Context invalid. Reason is " << exc.what());
                    // Release Lock
                    lock.release();
                    // Sleep for a while
                    sleep(CHECK_FAILURE_SLEEP_TIME);
                    continue;
                }
            }
            
            // For each entry
            SchedulerEntryList::iterator lit;        
            for(lit = m_actions.begin(); (lit != m_actions.end()) && (true == this->doRun()); ++lit){
                SchedulerEntry * entry = (*lit);
                if(0 == entry){
                    m_log_warn("Invalid Action");
                    m_actions.erase(lit);
                    break;
                } 
                
                // Get Last Execution Time
                last_execution_time = entry->last();
                
                // Check if the interval is elapsed
                if(last_execution_time > 0){
                    diff = (current_time - last_execution_time);
                } else {
                    diff = (current_time - start_time);
                }
                if(diff >= entry->interval()){
                    
                    if(false == entry->isEnabled()){
                        m_log_info("Action " << entry->type() << " will not be executed since it's disabled");
                        continue;
                    }
                    
                    try{
                        m_log_debug("Executing Action " << entry->type());
                        
                        // Execute Action
                        entry->run(m_ctx);
                        
                        m_log_debug("Action " << entry->type() << " executed successfully");
                    } catch(const RuntimeError& exc){
                        m_log_warn("Failed to execute action. Reason is " << exc.what());
                        if((0 != max_failures) && (entry->failures() >= max_failures)){
                            // Entry has been disabled
                            m_log_info("Too many Failures. Action " << entry->type() << " will be disabled for " << disable_time << " seconds");
                            entry->disable(disable_time);
                        }
                    } catch(const LogicError& exc){
                        m_log_alert("LogicError executing Action. Reason is " << exc.what());
                        m_log_info("Action " << entry->type() << " will be disabled for " << disable_time << " seconds");
                        entry->disable(disable_time);
                    }
                }
            } // End for each action

        } // End While
    } catch(const std::exception& exc){
        m_log_error("Exception running Scheduler: " << exc.what());
    } catch(...){
        m_log_error("Cannot run Scheduler. Unknown reason");
    }

    m_log_debug("Scheduler stopped");
}

/*
 * onEventStart
 *
 * Notify Start Event
 */
bool ActionScheduler::onEventStart() throw () {
    bool result  = true;
    if(0 != m_listener){
        try{
            result = m_listener->onEventStart();
        } catch(const std::exception& exc){
            m_log_fatal("Got Exception (" << exc.what() << ") in notifying Start Event");
            result = false;
        } catch(...){
            m_log_fatal("Got Unknown Exception in notifying Start Event");
            result = false;
        }
    }
    
    return result;
}

/*
 * onEventStop
 *
 * Notify Stop Event
 */
bool ActionScheduler::onEventStop() throw (){
    bool result = true;
    if(0 != m_listener){
        try{
            result = m_listener->onEventStop();
        } catch(const std::exception& exc){
            m_log_fatal("Exception notifying stop event: " << exc.what());
            result = false;
        } catch(...){
            m_log_fatal("Unknown exception notifying stop event");
            result = false;
        }
    }
    
    return result;
}

/*
 * terminate
 *
 * Terminate Scheduler iteration
 */
void ActionScheduler::terminate() throw () {
    try{
        m_ctx.kill();
    } catch(...){
        m_log_fatal("Got Unknown Exception in terminate");
    }
}
