/*
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *  GLite Data Transfer Agents: ActionScheduler Class
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *           Paolo Tedesco <paolo.tedesco@cern.ch>
 *  Version info: $Id: SchedulerConfig.cpp,v 1.9 2009/01/08 13:50:39 ptedesco Exp $
 *  Release: $Name:  $
 *
 */
#include "SchedulerConfig.h"
#include <glite/data/agents/AgentExceptions.h>
#include <glite/data/config/service/ParamReader.hxx>

#include <unistd.h>

using namespace glite::data::transfer::agent;
using namespace glite::config;
using glite::data::agents::ConfigurationException;
using glite::data::agents::ConfigParamException;
using glite::data::agents::LogicError;

extern "C" {
/*
 * create_glite_component
 * 
 * Create Component instance
 */
ComponentConfiguration * create_glite_component(){
    return SchedulerConfig::instance();
}

/*
 * destroy_glite_component
 * 
 * Destroy Component instance
 */
void destroy_glite_component(ComponentConfiguration * component){
    SchedulerConfig::finalize();
}

} // End extern "C"

namespace {
const char * const ACTION_SCHEDULER_NAME       = "glite-transfer-scheduler";
const char * const MAX_FAILURES_PROPERTY_NAME  = "MaxFailures"; 
const char * const DISABLETIME_PROPERTY_NAME   = "DisableTime"; 
const char * const CHECKINTERVAL_PROPERTY_NAME = "CheckInterval";
const char * const STOPTIMEOUT_PROPERTY_NAME   = "StopTimeout";

unsigned long SLEEP_TIME             = 1;
unsigned long DEFAULT_MAX_FAILURES   = 0;
unsigned long DEFAULT_DISABLE_TIME   = 300;
unsigned long DEFAULT_CHECK_INTERVAL = 60;
unsigned long DEFAULT_STOP_TIMEOUT   = 100;
}
                                                                                                               
/*
 * Initialize Static Attributes
 */
SchedulerConfig * SchedulerConfig::s_instance = 0;

/*
 * instance
 *
 * Return Singleton Instance
 */
SchedulerConfig * SchedulerConfig::instance(){
    if(0 == s_instance){
        s_instance = new SchedulerConfig();
    }
    return s_instance;
}
                                                                                                                          
/*
 * finalize the singleton
 *
 * Finalize Singleton Instance
 */
void SchedulerConfig::finalize(){
    if(0 != s_instance){
        delete s_instance;
    }
}


/*
 * SchedulerConfig
 *
 * Constructor
 */
SchedulerConfig::SchedulerConfig():ComponentConfiguration(ACTION_SCHEDULER_NAME),
   m_logger(ACTION_SCHEDULER_NAME),
   m_maxFailures(DEFAULT_MAX_FAILURES),
   m_disableTime(DEFAULT_DISABLE_TIME),
   m_checkInterval(DEFAULT_CHECK_INTERVAL),
   m_stopTimeout(DEFAULT_STOP_TIMEOUT){
}

/*
 * ~SchedulerConfig
 *
 * Destructor
 */
SchedulerConfig::~SchedulerConfig(){
}

/*
 * init
 *
 * Initialize the Component
 */
int SchedulerConfig::init(const Params& params){
    // Nothing To Do
    return 0;
}

/*
 * config
 *
 * Configure the Component
 */
int SchedulerConfig::config(const Params& params){
    
    try{
        ParamReader reader(getName(),params);
       
        // Get Max Failures
        reader.getParameter(MAX_FAILURES_PROPERTY_NAME,m_maxFailures);
        // Get Disable Time
        reader.getParameter(DISABLETIME_PROPERTY_NAME,m_disableTime);
        // Get Connection Check Interval
        reader.getParameter(CHECKINTERVAL_PROPERTY_NAME,m_checkInterval);
        // Get Stop Timeout 
        reader.getParameter(STOPTIMEOUT_PROPERTY_NAME,m_stopTimeout);
        
        // Log Configuration Values
        m_log_info(getName() << " Configured");
        m_log_info("MaxFailures       : " << m_maxFailures);
        m_log_info("DisableTime       : " << m_disableTime);
        m_log_info("CheckInterval     : " << m_checkInterval);
        m_log_info("StopTimeout       : " << m_stopTimeout);
    } catch(const LogicError& exc) {
        m_log_error(getName() << " configuration Logic Error: " << exc.what());
        return -1;
    } catch(const std::exception& exc) {
        m_log_error(getName() << " configuration Unexpected Exception: " << exc.what());
        return -1;
    } catch(...) {
        m_log_error(getName() << " configuration Unhandled Exception");
        return -1;
    }
    
    return 0;
}

/*
 * fini
 *
 * Finalize the Component
 */
int SchedulerConfig::fini(){
    return 0;
}
