/*
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *  GLite Data Transfer Agents: ActionScheduler Class
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *  Version info: $Id: SchedulerEntry.cpp,v 1.11 2006/08/14 08:38:51 badino Exp $
 *  Release: $Name:  $
 *
 */
#include "glite/data/transfer/agent/scheduler/SchedulerEntry.h"
#include "glite/data/transfer/agent/scheduler/ActionScheduler.h"

#include <boost/scoped_ptr.hpp>

using namespace glite::data::transfer::agent;
using namespace glite::data::transfer::agent::action;

using boost::scoped_ptr;

/*
 * SchedulerEntry
 *
 * Constructor
 */        
SchedulerEntry::SchedulerEntry(ActionFactoryMethod& f,const std::string& scope,  time_t i, const std::string& params):
        m_actionFactory(f),
        m_actionScope(scope), 
        m_actionParams(params), 
        m_interval(i),
        m_last(0),
        m_failures(0),
        m_disableTimestamp(0){
}

/*
 * ~SchedulerEntry
 *
 * Destructor
 */        
SchedulerEntry::~SchedulerEntry(){
}

/**
 * Enable the Scheduler Entry
 */
void SchedulerEntry::enable() { 
    m_disableTimestamp = 0;
    m_failures         = 0;    
}

/*
 * disable
 *
 * Disable the Scheduler Entry
 */
void SchedulerEntry::disable(unsigned long duration){
    // Disable for duration seconds
    time(&m_disableTimestamp);
    m_disableTimestamp += duration; 
}

/*
 * isEnabled
 *
 * Return true if the the Scheduler Entry is enabled
 */
bool SchedulerEntry::isEnabled(){
    bool result = false;
    if(0 == m_disableTimestamp){
        result = true;
    } else {
        time_t current_time;
        time(&current_time);
        if(current_time > m_disableTimestamp){
            result             = true;
            m_disableTimestamp = 0;
            m_failures         = 0;
        }
    }
    return result;
}

/*
 * run
 *
 * Execute the related Action and update the last execution time
 */        
void SchedulerEntry::run(glite::data::agents::dao::DAOContext& ctx) /*throw (ExecuteException)*/{
    
    if(false == isEnabled()){
        throw ExecuteException("Action Disabled");
    }
    
    // Update Last Execution Time
    time(&m_last);
    
    try{
        // Create Actions
        scoped_ptr<Action> action(m_actionFactory.create());

        // Set Action Context
        action->context(m_actionScope,ctx);
        
        // Execute Action
        action->execute(m_actionParams);
        
        // Reset the Failures Flag
        m_failures = 0;
    } catch(...){
        m_failures++;
        throw;
    }   
}

/**
 * Get Action Type
 */        
const std::string& SchedulerEntry::type() const{
    return m_actionFactory.type();
}
