/**
 *  Copyright (c) Members of the EGEE Collaboration. 2004-2008.
 *  See http://www.eu-egee.org/partners/ for details on the copyright
 *  holders.
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 *  Requirements, changes, info for this class (not complete list...):
 *
 *  bug #58470: FTS: make syslog facility configurable
 *     (https://savannah.cern.ch/bugs/?58470)
 *
 *  Authors: Zsolt Molnar <zsolt.molnar@cern.ch - http://www.zsoltmolnar.hu>
 *  Version info: $Id: Properties.h,v 1.2 2009/12/11 11:27:04 molnarzs Exp $
 *  Release: $Name:  $
 *
 */
#ifndef GLITE_DATA_TRANSFER_AGENTS_TS_URLCOPY_PROPERTIES_H_
#define GLITE_DATA_TRANSFER_AGENTS_TS_URLCOPY_PROPERTIES_H_

#include <glite/data/config/service/ParamReader.hxx>

namespace glite   {
namespace data    {
namespace transfer{
namespace agent   {
namespace ts      {

namespace Property {

// Traits for property TransferType
class TransferType
{
public:
    typedef enum {TYPE_URLCOPY = 0, TYPE_SRMCOPY} Type; 
    
    TransferType()
        : m_value(Default())
    {
        // EMPTY
    }
    
    const Type& value() const
    {
        return m_value;
    }
    
    static const std::string& Name()
    {
        static std::string s("TransferType");
        return s;
    }
    
    static const Type& Default()
    {
        static Type val(TYPE_URLCOPY);
        return val;
    }
    
    void read(glite::config::ParamReader& reader)
    ;
private:

    Type m_value;
};

// Traits for property MaxTransfers
class MaxTransfers
{
public:
    typedef unsigned long Type; 
    
    MaxTransfers()
        : m_value(Default())
    {
        // EMPTY
    }
    
    const Type& value() const
    {
        return m_value;
    }
    
    static const std::string& Name()
    {
        static std::string s("MaxTransfers");
        return s;
    }
    
    static const Type& Default()
    {
        static Type val(50);
        return val;
    }
    
    void read(glite::config::ParamReader& reader)
    ;
private:

    Type m_value;
};

// Traits for property MaxBulkSize
class MaxBulkSize
{
public:
    typedef unsigned long Type; 
    
    MaxBulkSize()
        : m_value(Default())
    {
        // EMPTY
    }
    
    const Type& value() const
    {
        return m_value;
    }
    
    static const std::string& Name()
    {
        static std::string s("MaxBulkSize");
        return s;
    }
    
    static const Type& Default()
    {
        static Type val(100);
        return val;
    }
    
    void read(glite::config::ParamReader& reader)
    {
        reader.getParameter(Name(), m_value);
    }
private:

    Type m_value;
};

// Traits for property SRMGridFTPSplit
class SRMGridFTPSplit
{
public:
    typedef bool Type; 
    
    SRMGridFTPSplit()
        : m_value(Default())
    {
        // EMPTY
    }
    
    const Type& value() const
    {
        return m_value;
    }
    
    static const std::string& Name()
    {
        static std::string s("SRMGridFTPSplit");
        return s;
    }
    
    static const Type& Default()
    {
        static Type val(false);
        return val;
    }
    
    void read(glite::config::ParamReader& reader)
    {
        reader.getParameter(Name(), m_value);
    }
private:

    Type m_value;
};

// Traits for property SyslogFacility
class SyslogFacility
{
public:
    typedef enum {DAEMON, FTP, USER} Type; 
    
    SyslogFacility()
        : m_value(Default())
    {
        // EMPTY
    }
    
    const Type& value() const
    {
        return m_value;
    }
    
    static const std::string& Name()
    {
        static std::string s("SyslogFacility");
        return s;
    }
    
    static const Type& Default()
    {
        static Type val(FTP);
        return val;
    }
    
    void read(glite::config::ParamReader& reader)
    ;
private:

    Type m_value;
};

// Traits for property SyslogIdent
class SyslogIdent
{
public:
    typedef std::string Type; 
    
    SyslogIdent()
        : m_value(Default())
    {
        // EMPTY
    }
    
    const Type& value() const
    {
        return m_value;
    }
    
    static const std::string& Name()
    {
        static std::string s("SyslogIdent");
        return s;
    }
    
    static const Type& Default()
    {
        static Type val("FTS");
        return val;
    }
    
    void read(glite::config::ParamReader& reader)
    {
        reader.getParameter(Name(), m_value);
    }
private:

    Type m_value;
};

// Traits for property SyslogDisable
class SyslogDisable
{
public:
    typedef bool Type; 
    
    SyslogDisable()
        : m_value(Default())
    {
        // EMPTY
    }
    
    const Type& value() const
    {
        return m_value;
    }
    
    static const std::string& Name()
    {
        static std::string s("SyslogDisable");
        return s;
    }
    
    static const Type& Default()
    {
        static Type val(false);
        return val;
    }
    
    void read(glite::config::ParamReader& reader)
    {
        reader.getParameter(Name(), m_value);
    }
private:

    Type m_value;
};

} // namespace Property

struct Properties
{
    Property::TransferType transferType; 
    Property::MaxTransfers maxTransfers; 
    Property::MaxBulkSize maxBulkSize; 
    Property::SRMGridFTPSplit sRMGridFTPSplit; 
    Property::SyslogFacility syslogFacility; 
    Property::SyslogIdent syslogIdent; 
    Property::SyslogDisable syslogDisable; 
};

} // End namespace ts
} // End namespace agent
} // End namespace transfer
} // End namespace data
} // End namespace glite

#include "Properties.hxx"

#endif // GLITE_DATA_TRANSFER_AGENTS_TS_URLCOPY_PROPERTIES_H_
