/**
 *  Copyright (c) Members of the EGEE Collaboration. 2004-2008.
 *  See http://www.eu-egee.org/partners/ for details on the copyright
 *  holders.
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 *  Requirements, changes, info for this class (not complete list...):
 *
 *  bug #58470: FTS: make syslog facility configurable
 *     (https://savannah.cern.ch/bugs/?58470)
 *
 *  Authors: Zsolt Molnar <zsolt.molnar@cern.ch - http://www.zsoltmolnar.hu>
 *  Version info: $Id: Properties.hxx,v 1.2 2009/12/11 11:27:04 molnarzs Exp $
 *  Release: $Name:  $
 *
 */
#ifndef GLITE_DATA_TRANSFER_AGENTS_TS_URLCOPY_PROPERTYUTILS_HXX_
#define GLITE_DATA_TRANSFER_AGENTS_TS_URLCOPY_PROPERTYUTILS_HXX_

/** This file should be included by Properties.h only */
#include <glite/data/agents/AgentExceptions.h>
#include <glite/data/config/service/ParamReader.hxx>
#include <map>
#include <boost/assign.hpp>

namespace glite   {
namespace data    {
namespace transfer{
namespace agent   {
namespace ts      {
namespace Property {

class TestProperties;

/** StringConverter for SyslogFacility property */
template <class PROPERTY>
class StringConverter_Property_Base :
    public glite::config::EnumConverter<typename PROPERTY::Type>
{
    friend class TestProperties;
protected:
    StringConverter_Property_Base()
        : glite::config::EnumConverter<typename PROPERTY::Type>
          (
              PROPERTY::Default(),
              _FacilityNames().find(PROPERTY::Default())->second
          )
    {
        for (typename _FacilityNames_type::const_iterator i = _FacilityNames().begin();
             i != _FacilityNames().end();
             ++i)
        {
            add(i->first, i->second);
        }
    }

    virtual std::string get_default_string()
    {
        static const std::string msg = "Invalid " + PROPERTY::Name() + " string";
        throw glite::data::agents::LogicError(msg);
        return "";
    }

    virtual typename PROPERTY::Type get_default_value()
    {
        static const std::string msg = "Invalid " + PROPERTY::Name() + " value";
        throw glite::data::agents::LogicError(msg);
        return PROPERTY::Default();
    }

    typedef std::map<typename PROPERTY::Type, std::string> _FacilityNames_type;
    /** Maps facility ID-t to their name strings */
    static const _FacilityNames_type& _FacilityNames();
};

template<> inline
const StringConverter_Property_Base<SyslogFacility>::_FacilityNames_type&
    StringConverter_Property_Base<SyslogFacility>::_FacilityNames()
{
    static _FacilityNames_type m =
        boost::assign::map_list_of(SyslogFacility::DAEMON, "LOG_DAEMON")
                                  (SyslogFacility::FTP, "LOG_FTP")
                                  (SyslogFacility::USER, "LOG_USER");
    return m;
}

template<> inline
const StringConverter_Property_Base<TransferType>::_FacilityNames_type&
    StringConverter_Property_Base<TransferType>::_FacilityNames()
{
    static _FacilityNames_type m =
        boost::assign::map_list_of(TransferType::TYPE_URLCOPY, "urlcopy")
                                  (TransferType::TYPE_SRMCOPY, "srmcopy");

    return m;
}

} // end namespace Property
} // End namespace ts
} // End namespace agent
} // End namespace transfer
} // End namespace data

namespace config {

using namespace glite::data::transfer::agent::ts::Property;

// SyslogFacility string conversion
template<>
struct StringConverter<SyslogFacility::Type> :
    public StringConverter_Property_Base<SyslogFacility>
{
    StringConverter()
        : StringConverter_Property_Base<SyslogFacility>()
    {
        // EMPTY
    }
};

// TransferType string conversion
template<>
struct StringConverter<TransferType::Type> :
    public StringConverter_Property_Base<TransferType>
{
    StringConverter()
        : StringConverter_Property_Base<TransferType>()
    {
        // EMPTY
    }
};

} // End namespace config

namespace data    {
namespace transfer{
namespace agent   {
namespace ts      {
namespace Property {

inline void SyslogFacility::read(glite::config::ParamReader& reader)
{
    reader.getParameter(Name(), m_value);
}

inline void TransferType::read(glite::config::ParamReader& reader)
{
    reader.getParameter(Name(), m_value);
}

inline void MaxTransfers::read(glite::config::ParamReader& reader)
{
    reader.getParameter(Name(), m_value).gt(0);
}

} // end namespace Property
} // End namespace ts
} // End namespace agent
} // End namespace transfer
} // End namespace data
} // End namespace glite

#endif // GLITE_DATA_TRANSFER_AGENTS_TS_URLCOPY_PROPERTYUTILS_HXX_
