/**
 *  Copyright (c) Members of the EGEE Collaboration. 2004.
 *  See http://www.eu-egee.org/partners/ for details on the copyright
 *  holders.
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 *  GLite Data Transfer Agents: SrmCopyTransferService Class
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>, Paolo Tedesco <paolo.tedesco@cern.ch>
 *  Version info: $Id: SrmCopyTransferService.cpp,v 1.41 2009/12/11 16:14:25 molnarzs Exp $
 *  Release: $Name:  $
 */
#include "SrmCopyTransferService.h"
#include "UrlCopyException.h"
#include <glite/data/agents/UserProxyEnv.h>
#include <glite/data/agents/AgentExceptions.h>
#include <glite/data/transfer/urlcopy/TransferSrmCopy.h>

using glite::data::agents::UserProxyEnv;
using glite::data::agents::LogicError;
using glite::data::agents::RuntimeError;

#include <sstream>
#include <algorithm>
#include <assert.h>

using namespace glite::data::transfer::agent;
using namespace glite::data::transfer::agent::model;
using namespace glite::data::transfer::agent::ts;

namespace urlcopy = glite::data::transfer::urlcopy;

/*
 * The TransferService Type
 */
const char * const SrmCopyTransferService::TRANSFER_TYPE_SRMCOPY = "srmcopy";

/*
 * UrlCopyTransferService
 *
 * Constructor
 */
SrmCopyTransferService::SrmCopyTransferService(const std::string& channel,const std::string& category):
    TransferServiceBase(channel,category),
    m_maxRequestSize(1),
    m_srmCopyPull(true),
    m_tscProto(dynamic_cast<SrmCopyTransferService::TransferSrmCopyStarter_*>(new SrmCopyTransferService::TransferSrmCopyStarter_Default_()))
{
    // EMPTY
}

/*
 * ~SrmCopyTransferService
 *
 * Destructor
 */
SrmCopyTransferService::~SrmCopyTransferService(){
}

/*
 * type
 *
 * Return the TransferService type
 */
std::string SrmCopyTransferService::type() /*throw ()*/{
    return TRANSFER_TYPE_SRMCOPY;
}

/*
 * maxRequestSize
 *
 * Get the number of max Transfers that can be processed whitin the same request
 */
unsigned long SrmCopyTransferService::maxRequestSize() {
    return m_maxRequestSize;
}

/*
 * submit
 *
 * Submit a FileTransfer request
 */
void SrmCopyTransferService::submit(
    const TransferRequest&  req,
    TransferResponse&       out,
    const std::string&      credentials) /*throw (TransferException)*/{

    // Check Precondition
    if(req.surls.size() > m_maxRequestSize){
        m_log_warn("Cannot process request. Bulk size (" << req.surls.size() << ") exceeded");
        throw UrlCopyException(Error::CONFIGURATION,Error::S_AGENT,Error::P_TX_PREPARATION,"too many files");
    }

    // Get Transfer List
    const TransferRequest::SurlPairArray& pairs = req.surls;
    // Get Transfer Data
    const std::string& params       = req.parameters;
    const std::string& src_srm      = req.src_srm;
    const std::string& src_srm_ver  = req.src_srm_version;
    const std::string& dest_srm     = req.dest_srm;
    const std::string& dest_srm_ver = req.dest_srm_version;
    const std::string& description  = req.description;

    m_log_info("submit called. Paremeters are:");
    m_log_info("Source SRM       : " << src_srm);
    m_log_info("Source SRM Vers. : " << src_srm_ver);
    m_log_info("Destination SRM  : " << dest_srm);
    m_log_info("Dest. SRM  Vers. : " << dest_srm_ver);
    m_log_info("Dest. Space Token: " << req.dest_space_token);
    m_log_info("Dest. Tkn Descr. : " << req.dest_token_descr);
    m_log_info("Params           : " << params);
    m_log_info("Overwrite        : " << str_bool(req.overwrite_flag));
    m_log_info("Description      : " << description);
    TransferRequest::SurlPairArray::const_iterator list_it;
    int list_count = 0;
    for(list_it = pairs.begin(); list_it != pairs.end(); ++list_it,++list_count){
        const std::string& sourceSurl = (*list_it).source;
        const std::string& destSurl = (*list_it).destination;
        m_log_info("Source      #" << list_count << "   : " << sourceSurl);
        m_log_info("Destination #" << list_count << "   : " << destSurl);
    }

    // Set the environment variable for the delegated proxy
    // Note: this doesn't work in a multithreaded environment. it would be better
    // that this will implemented inside the glite-url-copy, after the fork
    UserProxyEnv cert_guard(credentials);
    try{

        // Read Parameters
        JobParams job_params;
        readParams(params,req,job_params);

        // Create the Transfer Request object
        urlcopy::TransferSrmCopy::Args args;
        args.src_endpoint    = src_srm;
        args.src_version     = src_srm_ver;
        args.dest_endpoint   = dest_srm;
        args.dest_version    = dest_srm_ver;
        args.user_tag        = description;
        // Configuration
        args.overwrite       = req.overwrite_flag;
        args.console_logging = false;
        args.verbose         = (Channel::LL_DEBUG == req.log_level);
        if (Channel::SCD_PUSH == req.srmcopy_direction){
            args.pull_mode = false;
        } else {
            args.pull_mode = true;
        }
        // Space Token
        args.dest_space_token = req.dest_space_token;
        args.dest_token_descr = req.dest_token_descr;

        if (req.http_timeout >= 0){
            args.http_timeout = req.http_timeout;
        }
        if (req.srmcopy_timeout >= 0){
            args.copy_timeout = req.srmcopy_timeout;
        }
        if (req.srmcopy_refresh_timeout >= 0){
            args.refresh_timeout = req.srmcopy_refresh_timeout;
        }
        // New in 3.2.0 - fix bug #39882
        // fill source token info even if it's not yet used
        args.source_space_token = req.source_space_token;
        args.source_token_descr = req.source_token_descr;
        args.enable_dir_check = req.enable_dir_check;
        // New in 3.3.0
        args.copy_pin_lifetime      = req.copy_pin_lifetime;
        args.lan_connection         = req.lan_connection;
        args.fail_nearline_files    = req.fail_nearline;
        args.tx_to_per_mb           = req.tx_timeout_per_mb;
        // Set Files
        args.files = req.surls;

        _setupSyslogArgs(args);

        // Start The Transfer
        assert(m_tscProto.get());
        urlcopy::TransferSrmCopy::Ptr tx = m_tscProto->start(args, m_channel);
        assert(tx.get());
        m_log_debug("submit returned ID: [" << tx->id() << "]");

        // Store the requestId
        out.request_id = tx->id();
        for(std::vector<std::pair<std::string,std::string> >::size_type i = 0; i < args.files.size(); ++i){
            out.transfer_ids.push_back(i);
        }
    } catch(const RuntimeError& exc) {
        m_log_error("Failed to start a new transfer: " << exc.what());
        throw UrlCopyException(Error::INTERNAL_ERROR,Error::S_AGENT,Error::P_TX_PREPARATION,exc.what());
    } catch(const LogicError& exc) {
        m_log_error("Logic Error in starting a new transfer: " << exc.what());
        throw;
    }catch(...){
        m_log_error("Unhandled Exception in start");
        throw;
    }
}

/*
 *  state
 *
 * Get the status of a Transfer request
 */
void SrmCopyTransferService::state(
    const std::string& requestId,
    TransferInfoArray& objs,
    const std::string& credentials) /*throw (TransferException)*/{

    // Don't Check Precondition
    //if(objs.size() > m_maxRequestSize){
    // m_log_warn("Can process request. Bulk size (" << objs.size() << ") exceeded");
    //    throw UrlCopyException(Error::CONFIGURATION,"too many files");
    //}

    m_log_debug("state called for request [" << requestId << "]");

    // fix bug #39582 : get object here. If the requestId is invalid, an exception is thrown,
    // this is ok since TransferError, LogicError and RuntimeError objects are caught by the
    // CheckState action that, in such cases, sets the transfer to failed.
    // Get Transfer Object
    urlcopy::TransferSrmCopy::Ptr tx = urlcopy::TransferSrmCopy::get(requestId);

    try{
        // Get Transfer Status
        urlcopy::TransferSrmCopyStat s = tx->status();
        m_log_info("Transfer request [" << requestId << "] is [" << urlcopy::UrlCopyStatus::toString(s.status) << "]");
        // For each file
        TransferInfoArray::const_iterator it;
        for(it = objs.begin(); it != objs.end(); ++it){
            // Get Transfer Object
            Transfer& transfer_object = *(*it);
            // Find the correct stat for the file: the transfer id is the
            // position
            if(transfer_object.transferId() >= s.files.size()){
                // Inconsistency for that file
                m_log_warn("Transfer id [" << transfer_object.transferId() << "] is unknown");
                transfer_object.state       = Transfer::S_FAILED;
                transfer_object.reasonClass = Error::CONSISTENCY;
                transfer_object.reason      = "unknown transfer id";
            } else {
                // Get File Stat
                urlcopy::TransferSrmCopyStat::FileRequest& f =
                    s.files.at(transfer_object.transferId());
                // Update Status
                switch(f.status){
                case urlcopy::UrlCopyStatus::IDLE:
                case urlcopy::UrlCopyStatus::PREPARING:
                    transfer_object.state = Transfer::S_PREPARING;
                break;
                case urlcopy::UrlCopyStatus::READY:
                    transfer_object.state = Transfer::S_READY;
                break;
                case urlcopy::UrlCopyStatus::RUNNING:
                    transfer_object.state = Transfer::S_PROCESSING;
                break;
                case urlcopy::UrlCopyStatus::DONE:
                    if(s.status == urlcopy::UrlCopyStatus::COMPLETED){
                        m_log_info("Transfer [" << transfer_object.transferId() << "] completed successfully");

                        transfer_object.state = Transfer::S_COMPLETED;
                        // Bytes written
                        if((transfer_object.bytesWritten() <= 0) && (f.size > 0)){
                            transfer_object.bytesWritten = f.size;
                        }
                    } else {
                        transfer_object.state = Transfer::S_TRANSFERRED;
                    }
                break;
                case urlcopy::UrlCopyStatus::FAILED:
                case urlcopy::UrlCopyStatus::ABORTED:
                    if(s.status == urlcopy::UrlCopyStatus::COMPLETED){

                        // Create error reason
                        std::stringstream ss;
                        ss << urlcopy::UrlCopyError::scopeToString(f.error.scope)
                           << " error during "
                           << urlcopy::UrlCopyError::phaseToString(f.error.phase)
                           << " phase: ["
                           << urlcopy::UrlCopyError::categoryToString(f.error.category)
                           << "] " << f.error.description;
                        std::string reason = ss.str();
                        if(f.status == urlcopy::UrlCopyStatus::FAILED){
                            transfer_object.state = Transfer::S_FAILED;
                        } else {
                            transfer_object.state = Transfer::S_ABORTED;
                        }
                        m_log_warn("Transfer [" << transfer_object.transferId() << "] failed (fileId = " << transfer_object.fileId() << "): " << reason);
                        translateUrlCopyError(f.error,transfer_object);
                        transfer_object.reason = f.error.description;
                    } else {
                        transfer_object.state = Transfer::S_PROCESSING;
                    }
                break;
                case urlcopy::UrlCopyStatus::COMPLETED:
                    // This should never happens
                    transfer_object.state = Transfer::S_COMPLETED;
                break;
                default:
                    // This should not happen
                    m_log_error("Unknown transfer status returned");
                    throw RuntimeError("Unknown transfer status returned");
                break;
                }// end switch

                // File size
                if((transfer_object.fileSize() <= 0) && (f.size > 0)){
                    transfer_object.fileSize     = f.size;
                }

                if(m_srmCopyPull){
                    // Set destination SRM request Token
                    if((true == transfer_object.destSrmToken().empty()) &&
                       (false == s.token.empty())){
                        transfer_object.destSrmToken = s.token;
                    }
                } else {
                    // Set source SRM request Token
                    if((true == transfer_object.srcSrmToken().empty()) &&
                       (false == s.token.empty())){
                        transfer_object.srcSrmToken = s.token;
                    }
                }
                // Update Final Stats
                if((transfer_object.state == Transfer::S_COMPLETED) ||
                   (transfer_object.state == Transfer::S_FAILED) ||
                   (transfer_object.state == Transfer::S_ABORTED)){
                    // Duration
                    if(transfer_object.duration() <= 0){
                        double diff = (double)f.done_time - (double)f.start_time;
                        if(diff > 0){
                            transfer_object.duration = diff;
                        } else {
                            diff = (double)s.finish_time - (double)s.start_time;
                            if(diff > 0){
                                transfer_object.duration = diff;
                            } else {
                                // Duration still unset: compute it from the transfer time
                                time_t tr_time = transfer_object.transferTime();
                                if(tr_time > 0){
                                    // Get the current time
                                    time_t curr_time;
                                    time(&curr_time);
                                    diff = (double)curr_time - (double)tr_time;
                                    if(diff > 0){
                                        transfer_object.duration = diff;
                                    }
                                }
                            }
                        }
                    }
                    // Set Phases' Duration
                    if(transfer_object.srcPrepDuration() <= 0.0){
                       double diff = (double)s.src_prep_done_time - (double)s.src_prep_start_time;
                        if(diff >= 0){
                            transfer_object.srcPrepDuration = diff;
                        }
                    }
                    if(transfer_object.dstPrepDuration() <= 0.0){
                        double diff = (double)s.dest_prep_done_time - (double)s.dest_prep_start_time;
                        if(diff >= 0){
                            transfer_object.dstPrepDuration = diff;
                        }
                    }
                    if(transfer_object.txDuration() <= 0.0){
                        double diff = (double)s.copy_done_time - (double)s.copy_start_time;
                        if(diff >= 0){
                            transfer_object.txDuration = diff;
                        }
                    }
                    if(transfer_object.srcFinalDuration() <= 0.0){
                        transfer_object.srcFinalDuration = 0;
                    }
                    if(transfer_object.dstFinalDuration() <= 0.0){
                        double diff = (double)s.final_done_time - (double)s.final_start_time;
                        if(diff >= 0){
                            transfer_object.dstFinalDuration = diff;
                        }
                    }
                }
            }
        }// end for each file
        if(s.status == urlcopy::UrlCopyStatus::COMPLETED){
            // Transfer is completed : clean it
            try{
                tx->clean();
                m_log_debug("Transfer request [" << requestId << "] cleared");
            } catch(...){}
        }
    } catch(const RuntimeError& exc) {
        m_log_error("Failed to check transfer status: " << exc.what());
        try{ // fix bug #39582
            tx->clean();
            m_log_debug("Transfer request [" << requestId << "] cleared");
        } catch(...){
            m_log_warn("Could not clear transfer request [" << requestId << "]");
        }
        throw UrlCopyException(Error::INTERNAL_ERROR,Error::S_AGENT,Error::P_TRANSFER,exc.what());
    } catch(const LogicError& exc) {
        m_log_error("Logic Error in checking transfer status: " << exc.what());
        try{ // fix bug #39582
            tx->clean();
            m_log_debug("Transfer request [" << requestId << "] cleared");
        } catch(...){
            m_log_warn("Could not clear transfer request [" << requestId << "]");
        }
        throw;
    }
}


/*
 * revoke
 *
 * Cancel a Pending Transfer request
 */
void SrmCopyTransferService::revoke(const std::string& requestId, const std::string& credentials) /*throw (TransferException)*/{

    m_log_debug("revoke called for request [" << requestId << "]");

    try{
        // Get Transfer Object
        urlcopy::TransferSrmCopy::Ptr tx = urlcopy::TransferSrmCopy::get(requestId);
        // Stop the Transfer
        tx->stop();
        // Get Transfer Status
        urlcopy::TransferSrmCopyStat s = tx->status();
        if(s.status != urlcopy::UrlCopyStatus::COMPLETED){
            // This should not happen. Anyway call abort
            m_log_warn("Transfer [" << requestId << "] didn't completed cleanly");
            throw RuntimeError("Transfer not completed after a stop");
        }
        if(s.error.category == urlcopy::UrlCopyError::ERROR_ABORTED){
            m_log_info("Transfer [" << requestId << "] successfully revoked");
        } else if(s.error.category == urlcopy::UrlCopyError::SUCCESS){
            m_log_warn("Transfer [" << requestId << "] was already completed");
        } else {
            m_log_warn("Transfer [" << requestId << "] was already failed");
        }
        // Transfer is completed : clean it
        try{
            tx->clean();
            m_log_debug("Transfer request [" << requestId << "] cleared");
        } catch(...){}
    } catch(const RuntimeError& exc) {
        m_log_error("Failed to stop the transfer: " << exc.what());
        throw UrlCopyException(Error::INTERNAL_ERROR,Error::S_AGENT,Error::P_TX_FINALIZATION,exc.what());
    } catch(const LogicError& exc) {
        m_log_error("stop failed (LogicError). Reason is: " << exc.what());
        throw UrlCopyException(Error::INTERNAL_ERROR,Error::S_AGENT,Error::P_TX_FINALIZATION,exc.what());
    }
}

/*
 * abort
 *
 * Abort a Pending Transfer request. This method is intended for performing
 * a best effort clean up when the user credentials are not available to
 * revoke the transfer
 */
void SrmCopyTransferService::abort(const std::string& requestId) /*throw (TransferException)*/{

    m_log_debug("abort called for request " << requestId);

    try{
        // Get Transfer Object
        urlcopy::TransferSrmCopy::Ptr tx = urlcopy::TransferSrmCopy::get(requestId);
        // Stop the Transfer
        try{
            tx->abort();
        } catch(const std::exception& exc){
            m_log_warn("Failed to abort the transfer: " << exc.what());
            // Ignore Error
        }
        // Call clean anyway
        try{
            tx->clean();
            m_log_debug("Transfer request [" << requestId << "] cleared");
        } catch(...){}
    } catch(const RuntimeError& exc) {
        m_log_warn("Failed to abort the transfer: " << exc.what());
        // Ignore Error
    } catch(const LogicError& exc) {
        m_log_warn("stop failed (LogicError). Reason is: " << exc.what());
        // Ignore Error
    }
    m_log_info("Transfer request [" << requestId << "] successfully aborted");
}

/**
 * Get the maximum number of allowed transfers in any state for the channel
 * @param channel_transfers [IN]
 * @return maximum number of allowed transfers in any state for the channel
 */
unsigned long SrmCopyTransferService::getMaxChannelTransfers(
    unsigned long channel_transfers,
    double /*preparingFilesRatio*/){
    return channel_transfers;
}

/**
 * Get the maximum number of allowed transfers in any state for a VO
 * @param vo_transfers [IN]
 * @return maximum number of allowed transfers in any state for a VO
 */
unsigned long SrmCopyTransferService::getMaxVOTransfers(
    unsigned long vo_transfers,
    double /*preparingFilesRatio*/){
    return vo_transfers;
}

/**
 * return true if transfers stop after the preparation phase to maintain constant
 * the number of transfers in the transfer phase
 * @return
 */
bool SrmCopyTransferService::stopAfterPreparation() /* throw() */{
    return false;
}

/**
 * start the transfer phase for given transfer
 * @param transfer [IN] the transfer for which transfer phase must be started
 */
void SrmCopyTransferService::startTransferPhase(
    const model::Transfer&  transfer) /*throw (LogicError)*/{
    throw LogicError("startTransferPhase called for SrmCopyTransferService");
}

/**
 * Get the transfer states that represent an active transfer
 * @return
 */
model::Transfer::State SrmCopyTransferService::getActiveStates() /* throw() */{
    throw LogicError("getActiveStates called for SrmCopyTransferService");
}

/**
 * Get the transfer states that represent a transfer that needs starting the
 * transfer phase
 * @return
 */
model::Transfer::State SrmCopyTransferService::getStatesToStart() /* throw() */{
    throw LogicError("getStatesToStart called for SrmCopyTransferService");
}
