/**
 *  Copyright (c) Members of the EGEE Collaboration. 2004. 
 *  See http://www.eu-egee.org/partners/ for details on the copyright
 *  holders.  
 *
 *  Licensed under the Apache License, Version 2.0 (the "License"); 
 *  you may not use this file except in compliance with the License. 
 *  You may obtain a copy of the License at 
 *
 *      http://www.apache.org/licenses/LICENSE-2.0 
 *
 *  Unless required by applicable law or agreed to in writing, software 
 *  distributed under the License is distributed on an "AS IS" BASIS, 
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
 *  See the License for the specific language governing permissions and 
 *  limitations under the License.
 *
 *  GLite Data Transfer Agents: SrmCopyTransferService Class
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>, Paolo Tedesco <paolo.tedesco@cern.ch>
 *  Version info: $Id: SrmCopyTransferService.h,v 1.17 2009/12/11 16:14:25 molnarzs Exp $ 
 *  Release: $Name:  $
 */
#ifndef GLITE_DATA_TRANSFER_AGENTS_TS_SRMCOPY_TRANSFERSERVICE_H_
#define GLITE_DATA_TRANSFER_AGENTS_TS_SRMCOPY_TRANSFERSERVICE_H_

#include "TransferServiceBase.h"
#include <glite/data/transfer/urlcopy/TransferSrmCopy.h>
#include <boost/scoped_ptr.hpp>

namespace glite   {
namespace data    {
namespace transfer{
namespace agent   {
namespace ts      {

namespace urlcopy = glite::data::transfer::urlcopy;
class TestSrmCopyTransferService;

/**
 * SRMCopyTransfer Service class 
 */
class SrmCopyTransferService : public TransferServiceBase {
public:    
    /**
     * The TransferService Type
     */
    static const char * const TRANSFER_TYPE_SRMCOPY;

    /**
     * Constructor
     */
    SrmCopyTransferService(const std::string& channel_name,const std::string& category);

    /**
     * Destructor
     */
    virtual ~SrmCopyTransferService();

    /**
     * Return the TransferService type
     * @return the TransferService type
     */
    virtual std::string type() /*throw ()*/;

    /**
     * Return the type of the transfer handled by the service, AS IT MUST BE 
     * WRITTEN IN THE DATABASE
     * @return TransferType for the t_channel table
     */
    virtual std::string transferType(){
        return "SRMCOPY";
    }
    
    /**
     * Get the number of max Transfers that can be processed in the same request
     * @return number of max Transfers that can be processed in the same request
     */
    virtual unsigned long maxRequestSize();

    /**
     * Submit Transfers
     * @param req [INOUT] the request containing the list of SURLs pair to
     * transfer.
     * @param out [OUT] The result  object containing the request identifier and
     * a list of transfer identifier
     * @param credentials [IN] the credentials to be used to perform the action
     * @throws TransferException in case of error
     */
    virtual void submit(const TransferRequest& req,TransferResponse& out, const std::string& credentials) /*throw (TransferException)*/;

    /**
     * Get the status of a Transfer request
     * @param requestId [IN] the Identifier of the pending Transfer request
     * @param objs [INOUT] the array that will containing the information for
     * each transfer involved on the request
     * @param credentials [IN] the credentials to be used to perform the action
     * @throws TransferException in case of error
     */
    virtual void state(const std::string& requestId, TransferInfoArray& objs, const std::string& credentials) /*throw (TransferException)*/;
   
    /**
     * Cancel a Pending Transfer request
     * @param requestId [IN] the Identifier of the pending Transfer request
     * @param credentials [IN] the credentials to be used to perform the action
     * @throws TransferException in case of error
     */
    virtual void revoke(const std::string& requestId, const std::string& credentials) /*throw (TransferException)*/;
    
    /**
     * Abort a Pending Transfer request. This method is intended for performing 
     * a best effort clean up when the user credentials are not available to
     * revoke the transfer
     * @param requestId [IN] the Identifier of the pending Transfer request
     */
    virtual void abort(const std::string& requestId) /*throw ()*/;
    
    /**
     * Get the maximum number of allowed transfers in any state for the channel
     * @param channel_transfers [IN]
     * @return maximum number of allowed transfers in any state for the channel
     */
    virtual unsigned long getMaxChannelTransfers(
        unsigned long channel_transfers,
        double preparingFilesRatio);
    
    /**
     * Get the maximum number of allowed transfers in any state for a VO
     * @param vo_transfers [IN]
     * @return maximum number of allowed transfers in any state for a VO
     */
    virtual unsigned long getMaxVOTransfers(
        unsigned long vo_transfers,
        double preparingFilesRatio);

    /**
     * return true if transfers stop after the preparation phase to maintain constant
     * the number of transfers in the transfer phase
     * @return 
     */
    virtual bool stopAfterPreparation() /* throw() */;
    
    /**
     * start the transfer phase for given transfer
     * @param transfer [IN] the transfer for which transfer phase must be started
     */
    virtual void startTransferPhase(
        const model::Transfer&  transfer) /*throw (LogicError)*/;
    
    /**
     * Get the transfer states that represent an active transfer
     * @return 
     */
    virtual model::Transfer::State getActiveStates() /* throw() */;

    /**
     * Get the transfer states that represent a transfer that needs starting the 
     * transfer phase
     * @return 
     */
    virtual model::Transfer::State getStatesToStart() /* throw() */;    
        
    // -------------------------------------------------------------------------
    // Properties
    // -------------------------------------------------------------------------
    
    /**
     * Set the number of max Transfers that can be processed in the same request
     */
    void maxRequestSize(unsigned long s){ m_maxRequestSize = s; }

private:
    /**
     * Default Constructor. Not Implemented
     */
    SrmCopyTransferService();

private:    
    /**
     * Max Request Size
     */
    unsigned long m_maxRequestSize;

    /**
     * srmcopy direction
     */
    bool m_srmCopyPull;

private: // Constructs for testability
    // Access point for whitebox tests
    friend class TestSrmCopyTransferService;

    typedef TransferStarter_<urlcopy::TransferSrmCopy> TransferSrmCopyStarter_;
    typedef TransferStarter_Default_<urlcopy::TransferSrmCopy> TransferSrmCopyStarter_Default_;

    /** Injection point for test TransferSrmCopy::start */
    template<class TEST_TRANSFER_TYPE> void injectTscStarterDependency_()
    {
        m_tscProto.reset(dynamic_cast<TransferSrmCopyStarter_*>(new TEST_TRANSFER_TYPE()));
    }

    // Dependency injection points
    /** Wrapper for transfer starter methods */
    boost::scoped_ptr<TransferSrmCopyStarter_> m_tscProto;
};

} // End namespace ts
} // End namespace agent
} // End namespace transfer
} // End namespace data
} // End namespace glite

#endif //GLITE_DATA_TRANSFER_AGENTS_TS_SRMCOPY_TRANSFERSERVICE_H_
