/*
 *  Copyright (c) Members of the EGEE Collaboration. 2004. 
 *  See http://www.eu-egee.org/partners/ for details on the copyright
 *  holders.  
 *
 *  Licensed under the Apache License, Version 2.0 (the "License"); 
 *  you may not use this file except in compliance with the License. 
 *  You may obtain a copy of the License at 
 *
 *      http://www.apache.org/licenses/LICENSE-2.0 
 *
 *  Unless required by applicable law or agreed to in writing, software 
 *  distributed under the License is distributed on an "AS IS" BASIS, 
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
 *  See the License for the specific language governing permissions and 
 *  limitations under the License.
 *
 *  GLite Data Transfer Agents: TransferServiceBase Class
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>, Paolo Tedesco <paolo.tedesco@cern.ch>
 *  Version info: $Id: TransferServiceBase.cpp,v 1.25 2009/12/03 17:52:07 molnarzs Exp $ 
 *  Release: $Name:  $
 */
#include "TransferServiceBase.h"
#include "UrlCopyException.h"
#include "Properties.h"

#include <sstream>
#include <syslog.h>

using namespace glite::data::transfer::agent;
using namespace glite::data::transfer::agent::model;
using namespace glite::data::transfer::agent::ts;

//namespace urlcopy = glite::data::transfer::urlcopy;

namespace {
const char * const NSTREAMS_OPTION      = "-p";
const char * const TCPBUFFERSIZE_OPTION = "-tcpbs";
const char * const BLOCKSIZE_OPTION     = "-bs";
}

/**
 * Translate a urlcopy Error into Error Category/Scope/Phase
 * @param e [IN] the UrlCopyError
 * @param category [OUT] Error category
 * @param scope [OUT] Error scope
 * @param phase [OUT] Error phase 
 */
void TransferServiceBase::translateUrlCopyError(
    const urlcopy::UrlCopyError& e,
    Error::Category&             category,
    Error::Scope&                scope,
    Error::Phase&                phase) {

    // translate the error category: unknown values mapped to INTERNAL_ERROR
    //--------------------------------------------------------------------------
    switch(e.category){
        case urlcopy::UrlCopyError::SUCCESS: // this should never happen
            category = Error::UNDEF;
            break;
        case urlcopy::UrlCopyError::ERROR_USER:
            category = Error::USER;
            break;
        case urlcopy::UrlCopyError::ERROR_PERMISSION:
            category = Error::SECURITY;
            break;
        case urlcopy::UrlCopyError::ERROR_FILE_EXISTS:
            category = Error::FILE_EXISTS;
            break;
        case urlcopy::UrlCopyError::ERROR_INVALID_PATH:
            category = Error::INVALID_PATH;
            break;
        case urlcopy::UrlCopyError::ERROR_NO_SPACE_LEFT:
            category = Error::NO_SPACE_LEFT;
            break;
        case urlcopy::UrlCopyError::ERROR_GRIDFTP:
            category = Error::GRIDFTP;
            break;
        case urlcopy::UrlCopyError::ERROR_NETWORK:
            category = Error::NETWORK;
            break;
        case urlcopy::UrlCopyError::ERROR_INVALID_SIZE:
            category = Error::INVALID_SIZE;
            break;
        case urlcopy::UrlCopyError::ERROR_REQUEST_FAILURE:
            category = Error::REQUEST_FAILURE;
            break;
        case urlcopy::UrlCopyError::ERROR_CONNECTION:
            category = Error::CONNECTION;
            break;
        case urlcopy::UrlCopyError::ERROR_HTTP_TIMEOUT:
            category = Error::HTTP_TIMEOUT;
            break;
        case urlcopy::UrlCopyError::ERROR_REQUEST_TIMEOUT:
            category = Error::REQUEST_TIMEOUT;
            break;
        case urlcopy::UrlCopyError::ERROR_TRANSFER_TIMEOUT:
            category = Error::TRANSFER_TIMEOUT;
            break;
        case urlcopy::UrlCopyError::ERROR_MARKERS_TIMEOUT:
            category = Error::MARKERS_TIMEOUT;
            break;
        case urlcopy::UrlCopyError::ERROR_INTERNAL_ERROR:
            category = Error::INTERNAL_ERROR;
            break;
        case urlcopy::UrlCopyError::ERROR_CONSISTENCY:
            category = Error::CONSISTENCY;
            break;
        case urlcopy::UrlCopyError::ERROR_ABORTED:
            category = Error::ABORTED;
            break;
        case urlcopy::UrlCopyError::ERROR_GENERAL_FAILURE:
            category = Error::GENERAL_FAILURE;
            break;
        case urlcopy::UrlCopyError::ERROR_LOST:
            category = Error::LOST;
            break;
        case urlcopy::UrlCopyError::ERROR_STORAGE_INTERNAL_ERROR:
            category = Error::STORAGE_INTERNAL_ERROR;
            break;
        case urlcopy::UrlCopyError::ERROR_SRM_COPY:
            category = Error::SRM_COPY;
            break;
        case urlcopy::UrlCopyError::ERROR_LOCALITY:
            category = Error::LOCALITY;
            break;
        case urlcopy::UrlCopyError::ERROR_FIRST_MARKER_TIMEOUT:
            category = Error::FIRST_MARKER_TIMEOUT;
            break;
        case urlcopy::UrlCopyError::ERROR_NO_PROGRESS:
            category = Error::NO_PROGRESS;
            break;
        default:
            category = Error::INTERNAL_ERROR;
            break;
    }

    // translate the error scope: undef and unknown values are mapped to S_AGENT
    //--------------------------------------------------------------------------
    switch (e.scope){
        case urlcopy::UrlCopyError::SCOPE_SOURCE:
            scope = Error::S_SOURCE;
            break;
        case urlcopy::UrlCopyError::SCOPE_DESTINATION:
            scope = Error::S_DESTINATION;
            break;
        case urlcopy::UrlCopyError::SCOPE_TRANSFER:
            scope = Error::S_TRANSFER;
            break;
        case urlcopy::UrlCopyError::SCOPE_AGENT:
        case urlcopy::UrlCopyError::SCOPE_UNDEF:
        default:
            scope = Error::S_AGENT;
            break;
    
    }
    
    // translate the error phase: undef and unknown values are mapped to P_TRANSFER
    //--------------------------------------------------------------------------
    switch (e.phase){
        case urlcopy::UrlCopyError::PHASE_PREPARATION:
            phase = Error::P_TX_PREPARATION;
            break;
        case urlcopy::UrlCopyError::PHASE_FINALIZATION:
            phase = Error::P_TX_FINALIZATION;
            break;
        case urlcopy::UrlCopyError::PHASE_TRANSFER:
		case urlcopy::UrlCopyError::PHASE_UNDEF:
        default:
            phase = Error::P_TRANSFER;
            break;
	}
}

/**
 * Translate a urlcopy Error into Error Category/Scope/Phase and set values in
 * given transfer object
 * @param e [IN] the UrlCopyError
 * @param transfer [OUT] transfer object 
 */
void TransferServiceBase::translateUrlCopyError(
    const urlcopy::UrlCopyError&    e,
    Transfer&                       transfer){
        
    Error::Category category;
    Error::Scope scope;
    Error::Phase phase;
    translateUrlCopyError(e,category,scope,phase);
    transfer.reasonClass = category;
    transfer.errorScope = scope;
    transfer.errorPhase = phase;
}
    
/*
 * TransferServiceBase
 *
 * Constructor
 */
TransferServiceBase::TransferServiceBase(const std::string& channel,const std::string& category):
    m_channel(channel),
    m_logger(category){
}

/*
 * ~TransferServiceBase
 *
 * Destructor
 */
TransferServiceBase::~TransferServiceBase(){
}

/*
 * channel
 *
 * Return the name of the Channel the transfer Service is responsible for
 */
std::string TransferServiceBase::channel() /*throw (TransferException)*/{
    if(m_channel.empty()){
        throw UrlCopyException(Error::CONFIGURATION,Error::S_AGENT,Error::P_TX_SERVICE,"Channel is not set");
    }
    return m_channel;
}

/*
 * max
 *
 * Get the number of max concurrent Transfers this service can support
 */
unsigned long TransferServiceBase::max()
{
    return m_maxTransfers.value();
}

/**
 * Initializes the JobParams struct with request values or looking at the 
 * params string
 */
void TransferServiceBase::readParams(
    const std::string&              params,
    const TransferRequest&          request,
    TransferServiceBase::JobParams& job_params) /* throw ()*/{
    
    job_params.streams      = request.streams;
    job_params.tcpbs        = request.tcpbs;
    job_params.blocksize    = request.blocksize;
        
    // Get Number of TCP Streams Parameter
    std::string::size_type pos = params.find(NSTREAMS_OPTION);
    if(std::string::npos != pos){
        long nstreams = atol(params.c_str() + pos + strlen(NSTREAMS_OPTION));
        if(nstreams > 0){
            job_params.streams = nstreams;
            // this->streams(nstreams);
        } else {
            m_log_warn("Invalid Streams Parameter Specified. Use the default one");
        }
    }
    
    // Get TCP Buffer Size Parameter
    pos = params.find(TCPBUFFERSIZE_OPTION);
    if(std::string::npos != pos){
        long tcpbs = atol(params.c_str() + pos + strlen(TCPBUFFERSIZE_OPTION));
        if(tcpbs >= 0){
            job_params.tcpbs = tcpbs;
            // this->tcpBufferSize(bs);
        } else {
            m_log_warn("Invalid TCPBufferSize Parameter Specified: the default one will be used");
        }
    }
    
    // Get Block Size Parameter
    pos = params.find(BLOCKSIZE_OPTION);
    if(std::string::npos != pos){
        long bs = atol(params.c_str() + pos + strlen(BLOCKSIZE_OPTION));
        if(bs >= 0){
            job_params.blocksize = bs;
            // this->blockSize(bs);
        } else {
            m_log_warn("Invalid BlockSize Parameter Specified: the default one will be used");
        }
    }

}

/*
 * clean
 *
 * Clean-up the information concerning a completed FileTransfer 
 */
void TransferServiceBase::clean(const std::string& transferId, const std::string& credentials) /*throw (TransferException)*/{
    m_log_warn("clean called for transfer " << transferId);
#warning "TODO : implement clean"
    throw UrlCopyException(Error::INTERNAL_ERROR,Error::S_AGENT,Error::P_TX_SERVICE,"Not Implemented");
}


/*
 * trace
 *
 * trace the transfer log messages
 */
void TransferServiceBase::trace(const std::string& transferId, std::ostream& out, const std::string& credentials) /*throw (TransferException)*/{
    m_log_warn("trace called for transfer " << transferId);
#warning "TODO : implement trace"
    throw UrlCopyException(Error::INTERNAL_ERROR,Error::S_AGENT,Error::P_TX_SERVICE,"Not Implemented");
}

/*
 * count
 *
 * Get the number of all the pending FileTransfer requests
 */
unsigned long TransferServiceBase::count() /*throw (TransferException)*/{
    m_log_warn("count called for channel " << m_channel);
#warning "TODO : implement count"
    throw UrlCopyException(Error::INTERNAL_ERROR,Error::S_AGENT,Error::P_TX_SERVICE,"Not Implemented");
}

void TransferServiceBase::setSysLogParameters(
    const Property::SyslogDisable& disable,
    const Property::SyslogFacility& facility,
    const Property::SyslogIdent& ident)
{
    m_syslogDisable = disable;
    m_syslogFacility = facility;
    m_syslogIdent = ident;
}

void TransferServiceBase::setMax(const Property::MaxTransfers& num)
{
    m_maxTransfers = num;
}

void TransferServiceBase::_setupSyslogArgs(urlcopy::details::TransferBase::Args& args) const
{
    args.syslog_disable = m_syslogDisable.value();

    switch (m_syslogFacility.value()) {
    case Property::SyslogFacility::DAEMON:
        args.syslog_facility = LOG_DAEMON;
        break;
    case Property::SyslogFacility::FTP:
        args.syslog_facility = LOG_FTP;
        break;
    case Property::SyslogFacility::USER:
        args.syslog_facility = LOG_USER;
        break;
    default:
        // Should not get here...
        assert(0);
    }

    args.syslog_ident = m_syslogIdent.value();

    args.syslog_facility_name =
        config::StringConverter<Property::SyslogFacility::Type>::to_string(
            m_syslogFacility.value()
        );
}
