/*
 *  Copyright (c) Members of the EGEE Collaboration. 2004. 
 *  See http://www.eu-egee.org/partners/ for details on the copyright
 *  holders.  
 *
 *  Licensed under the Apache License, Version 2.0 (the "License"); 
 *  you may not use this file except in compliance with the License. 
 *  You may obtain a copy of the License at 
 *
 *      http://www.apache.org/licenses/LICENSE-2.0 
 *
 *  Unless required by applicable law or agreed to in writing, software 
 *  distributed under the License is distributed on an "AS IS" BASIS, 
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
 *  See the License for the specific language governing permissions and 
 *  limitations under the License.
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *           Paolo Tedesco <paolo.tedesco@cern.ch>
 *           Zsolt Molnar <zsolt.molnar@cern.ch - http://www.zsoltmolnar.hu>
 *  Version info: $Id: TransferServiceBase.h,v 1.13 2009/12/11 16:14:25 molnarzs Exp $ 
 *  Release: $Name:  $
 */
#ifndef GLITE_DATA_TRANSFER_AGENTS_TS_URLCOPY_TRANSFERSERVICEBASE_H_
#define GLITE_DATA_TRANSFER_AGENTS_TS_URLCOPY_TRANSFERSERVICEBASE_H_

#include "Properties.h"
#include "glite/data/transfer/agent/ts/TransferService.h"
#include "glite/data/transfer/agent/model/Transfer.h"
#include "glite/data/transfer/agent/model/Error.h"
#include "glite/data/transfer/urlcopy/UrlCopyError.h"
#include <string>
#include <list>
#include <glite/data/config/service/Logger.hxx>
#include <glite/data/transfer/urlcopy/TransferUrlCopy.h>

namespace glite   {
namespace data    {
namespace transfer{
namespace agent   {
namespace ts      {

/**
 * Transfer Service abstract class
 */
class TransferServiceBase : public TransferService{
public:    

    /**
     * Destructor
     */
    virtual ~TransferServiceBase();

    /**
     * Return the name of the Channel the transfer Service is responsible for
     * @return the name of the Channel the transfer Service is responsible for
     * @throws TransferException in case the channel name is not set
     */
    virtual std::string channel() /*throw (TransferException)*/;
    
    /**
     * Get the number of max concurrent transfers requests the service can support
     * @return the number of max concurrent transfer requests the service can support
     */
    virtual unsigned long max();

    /**
     * Clean-up the information concerning a completed Transfer 
     * @param requestId [IN] the Identifier of the completed Transfer request
     * @param credentials [IN] the credentials to be used to perform the action
     * @throws TransferException in case the requested Transfer request 
     * doesn't exist or is not completed
     */
    virtual void clean(const std::string& requestId, const std::string& credentials) /*throw (TransferException)*/;
    
    /**
     * Trace a Transfer
     * @param requestId [IN] the Identifier of the completed Transfer request
     * @param credentials [IN] the credentials to be used to perform the action
     * @param out [IN] the buffer where to store the transfer request
     * @throws TransferException in case the requested Transfer request 
     * doesn't exist or is not completed
     */
    virtual void trace(
        const std::string&  requestId, 
        std::ostream&       out, 
        const std::string&  credentials) /*throw (TransferException)*/;

    /**
     * Get the number of all the pending Transfer requests
     * @return the number of all the pending Transfer requests
     * @throws TransferException in case of error
     */
    virtual unsigned long count() /*throw (TransferException)*/;
   
    /**
     * Translate a urlcopy Error into Error Category/Scope/Phase
     * @param e [IN] the UrlCopyError
     * @param category [OUT] Error category
     * @param scope [OUT] Error scope
     * @param phase [OUT] Error phase 
     */
    static void translateUrlCopyError(
        const glite::data::transfer::urlcopy::UrlCopyError&     e,
        glite::data::transfer::agent::model::Error::Category&   category,
        glite::data::transfer::agent::model::Error::Scope&      scope,
        glite::data::transfer::agent::model::Error::Phase&      phase);
	
    /**
     * Translate a urlcopy Error into Error Category/Scope/Phase and set values in
     * given transfer object
     * @param e [IN] the UrlCopyError
     * @param transfer [OUT] transfer object 
     */
	static void translateUrlCopyError(
        const glite::data::transfer::urlcopy::UrlCopyError& e,
        glite::data::transfer::agent::model::Transfer&      transfer);
	
    // -------------------------------------------------------------------------
    // Properties
    // -------------------------------------------------------------------------

    /** Set the Maximun number of Parallel Transfer. The name may be confusing,
     * because the corresponding getter (fixed in an interface) is "max" only.*/
    void setMax(const Property::MaxTransfers& num);
    
    void setSysLogParameters(
        const Property::SyslogDisable& disable,
        const Property::SyslogFacility& facility,
        const Property::SyslogIdent& ident);

protected:
    
    /**
     * Constructor
     */
    TransferServiceBase(const std::string& channel_name,const std::string& category);
    
    struct JobParams{
        unsigned long streams;
        unsigned long tcpbs;
        unsigned long blocksize;
    };
    
    /**
     * Initializes the JobParams struct with request values or looking at the 
     * params string
     */
    void readParams(
        const std::string&      params,
        const TransferRequest&  request,
        JobParams&              job_params) /* throw ()*/;

    void _setupSyslogArgs(urlcopy::details::TransferBase::Args& args) const;

private:
    /**
     * Default Constructor. Not Implemented
     */
    TransferServiceBase();

protected:    
    
    /**
     * The Channel the service is responsible for
     */
    std::string   m_channel;
    glite::config::Logger m_logger;
    
private:
    Property::MaxTransfers m_maxTransfers;
    Property::SyslogDisable m_syslogDisable;
    Property::SyslogFacility m_syslogFacility;
    Property::SyslogIdent m_syslogIdent;

protected: // Constructs for testing
    /** Structures for injecting transfer start method. This class
     * interacts with transfer implementations via that method, so we wrap it
     * into a function object. White box tests can capture this call by
     * injecting their test methods. */
    template<class TRANSFER_TYPE>
    struct TransferStarter_
    {
        virtual typename TRANSFER_TYPE::Ptr start(
            const typename TRANSFER_TYPE::Args& args,
            const std::string& prefix) = 0;

        virtual ~TransferStarter_() {}
    };

    /** The code in production (starts the start method of teh real class */
    template<class TRANSFER_TYPE>
    struct TransferStarter_Default_ : public TransferStarter_<TRANSFER_TYPE>
    {
        virtual typename TRANSFER_TYPE::Ptr start(
            const typename TRANSFER_TYPE::Args& args,
            const std::string& prefix)
        {
            return TRANSFER_TYPE::start(args, prefix);
        }

        virtual ~TransferStarter_Default_() {}
    };
};

} // End namespace ts
} // End namespace agent
} // End namespace transfer
} // End namespace data
} // End namespace glite

#endif //GLITE_DATA_TRANSFER_AGENTS_TS_URLCOPY_TRANSFERSERVICEBASE_H_
