/**
 *  Copyright (c) Members of the EGEE Collaboration. 2004. 
 *  See http://www.eu-egee.org/partners/ for details on the copyright
 *  holders.  
 *
 *  Licensed under the Apache License, Version 2.0 (the "License"); 
 *  you may not use this file except in compliance with the License. 
 *  You may obtain a copy of the License at 
 *
 *      http://www.apache.org/licenses/LICENSE-2.0 
 *
 *  Unless required by applicable law or agreed to in writing, software 
 *  distributed under the License is distributed on an "AS IS" BASIS, 
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
 *  See the License for the specific language governing permissions and 
 *  limitations under the License.
 *
 *  GLite Data Transfer Agents: UrlCopyTransferFactory Class
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>, Paolo Tedesco <paolo.tedesco@cern.ch>
 *  Version info: $Id: UrlCopyTransferFactory.cpp,v 1.28 2009/12/11 11:27:04 molnarzs Exp $ 
 *  Release: $Name:  $
 */
#include "UrlCopyTransferFactory.h"
#include "UrlCopyTransferService.h"
#include "SrmCopyTransferService.h"
#include "UrlCopyException.h"
#include <glite/data/config/service/ParamReader.hxx>
#include <glite/data/agents/AgentExceptions.h>

using namespace glite::data::transfer::agent;
using namespace glite::data::transfer::agent::ts;
using namespace glite::config;
using glite::data::agents::ConfigParamException;
using glite::data::agents::ConfigurationException;
using glite::data::agents::LogicError;
using glite::data::transfer::agent::model::Error;

/*
 * Constants
 */
namespace {
const char * const URLCOPY_TRANSFER_NAME           = "transfer-agent-urlcopy";

const unsigned long MIN_STREAMS                   = 1;
const unsigned long MAX_STREAMS                   = 100;    
}

extern "C" {
/*
 * create_glite_component
 *
 * Create Component instance
 */
ComponentConfiguration * create_glite_component(){
    return new UrlCopyTransferFactory();
}
                                                                                
/*
 * destroy_glite_component
 *
 * Destroy Component instance
 */
void destroy_glite_component(ComponentConfiguration * component){
    UrlCopyTransferFactory * f = dynamic_cast<UrlCopyTransferFactory *>(component); 
    if(0 != f){
        delete f;
    }
}
                                                                                
} // End extern "C"

/*
 * UrlCopyTransferFactory
 *
 * Constructor
 */
UrlCopyTransferFactory::UrlCopyTransferFactory()
  : TransferServiceFactory(),
    ComponentConfiguration(URLCOPY_TRANSFER_NAME), 
    m_logger(URLCOPY_TRANSFER_NAME)
{
    // EMPTY
}

/*
 * ~UrlCopyTransferFactory
 *
 * Destructor
 */
UrlCopyTransferFactory::~UrlCopyTransferFactory(){
}

/*
 * create
 *
 * Create a TransferService
 */
TransferService * UrlCopyTransferFactory::create(const std::string& channel_name) /*throw (TransferException)*/{
    
    // Create TransferService
    TransferServiceBase * service = 0;
    switch(m_properties.transferType.value()){
        case Property::TransferType::TYPE_URLCOPY: {
            UrlCopyTransferService * transfer_service = new UrlCopyTransferService(channel_name,URLCOPY_TRANSFER_NAME);
            transfer_service->setSrmGridFTPSplit(m_properties.sRMGridFTPSplit.value());
            service = transfer_service;
        }
        break;
        case Property::TransferType::TYPE_SRMCOPY: {
            SrmCopyTransferService * transfer_service = new SrmCopyTransferService(channel_name,URLCOPY_TRANSFER_NAME);
            transfer_service->maxRequestSize(m_properties.maxBulkSize.value());
            service = transfer_service;
        }
        break;
        default:
            throw UrlCopyException(
                Error::INTERNAL_ERROR,
                Error::S_AGENT,
                Error::P_TRANSFER,
                "Invalid Transfer Type");
    }

    service->setMax(m_properties.maxTransfers);

    service->setSysLogParameters(
        m_properties.syslogDisable,
        m_properties.syslogFacility,
        m_properties.syslogIdent);

    m_log_debug("TransferService Created");
    return service;
}

/*
 * init
 *
 * Initialize the Component
 */
int UrlCopyTransferFactory::init(const Params& params){
                                                                                                                          
    try{
        ParamReader reader(getName(),params);
        m_properties.transferType.read(reader);
        m_properties.maxTransfers.read(reader);
        // Get the SRM / gridFTP split enable parameter
        if (reader.isSet(m_properties.sRMGridFTPSplit.Name())) {
            if (m_properties.transferType.value() == Property::TransferType::TYPE_URLCOPY) {
                m_properties.sRMGridFTPSplit.read(reader);
            } else {
                m_log_warn(m_properties.sRMGridFTPSplit.Name() << " Property Ignored. Not an UrlCopy TransferService");
            }
        }
        // Get Max Bulk Size
        if (reader.isSet(m_properties.maxBulkSize.Name())){
            if (m_properties.transferType.value() == Property::TransferType::TYPE_SRMCOPY) {
                m_properties.maxBulkSize.read(reader);
            } else {
                m_log_warn(m_properties.maxBulkSize.Name() << " Property Ignored. Not an SrmCopy TransferService");
            }
        }
        
        _setupSysLog(reader);
        _logConfigValues();

        // Log Configuration Values
        m_log_info(getName() << " Initialized. Initialization Parameters are");
        m_log_info("Transfer Type\t\t: " << StringConverter<Property::TransferType::Type>::to_string(m_properties.transferType.value()));
        m_log_info("MaxTransfers\t\t: " << m_properties.maxTransfers.value());
        m_log_info("SysLogFacility\t\t: " << StringConverter<Property::SyslogFacility::Type>::to_string(m_properties.syslogFacility.value()));
        m_log_info("SysLogIdent\t\t: " << m_properties.syslogIdent.value());
        m_log_info("SysLogIdent\t\t: " << m_properties.syslogIdent.value());

        switch(m_properties.transferType.value()){
        case Property::TransferType::TYPE_URLCOPY:
            m_log_info("SRM / GridFTP split enabled\t: " << str_bool(m_properties.sRMGridFTPSplit.value()));
            break;
        case Property::TransferType::TYPE_SRMCOPY:
            m_log_info("Max Bulk Size\t\t: " << m_properties.maxBulkSize.value());
            break;
        default:
            break;
        } // End Switch
    } catch(const LogicError& exc) {
        m_log_error(getName() << " configuration Error: " << exc.what());
        return -1;
    } catch(const std::exception& exc) {
        m_log_error(getName() << " configuration Unexpected Exception: " << exc.what());
        return -1;
    } catch(...) {
        m_log_error(getName() << " configuration Unhandled Exception");
        return -1;
    }
    return 0;
}

/*
 * config
 *
 * Configure the Component
 */
int UrlCopyTransferFactory::config(const Params& params){
    return 0;
}

void UrlCopyTransferFactory::_setupSysLog(ParamReader& reader)
{
    if (reader.isSet(m_properties.syslogFacility.Name()))
    {
        m_properties.syslogFacility.read(reader);
    }

    if (reader.isSet(m_properties.syslogIdent.Name()))
    {
        m_properties.syslogIdent.read(reader);
    }

    if (reader.isSet(m_properties.syslogDisable.Name()))
    {
        m_properties.syslogDisable.read(reader);
    }
}

void UrlCopyTransferFactory::_logConfigValues()
{
    m_log_info(getName() << " Initialized. Initialization Parameters are");
    m_log_info("Transfer Type\t\t: " << StringConverter<Property::TransferType::Type>::to_string(m_properties.transferType.value()));
    m_log_info("MaxTransfers\t\t: " << m_properties.maxTransfers.value());
    m_log_info("SysLogFacility\t\t: " << StringConverter<Property::SyslogFacility::Type>::to_string(m_properties.syslogFacility.value()));;
    m_log_info("SysLogIdent\t\t: " << m_properties.syslogIdent.value());

    if (m_properties.syslogDisable.value())
        m_log_info("SysLog\t\t: disabled ");

    switch(m_properties.transferType.value()){
    case Property::TransferType::TYPE_URLCOPY:
        m_log_info("SRM / GridFTP split enabled\t: " << str_bool(m_properties.sRMGridFTPSplit.value()));
        break;
    case Property::TransferType::TYPE_SRMCOPY:
        m_log_info("Max Bulk Size\t\t: " << m_properties.maxBulkSize.value());
        break;
    default:
        break;
    } // End Switch
}
