/**
 *  Copyright (c) Members of the EGEE Collaboration. 2004.
 *  See http://www.eu-egee.org/partners/ for details on the copyright
 *  holders.
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 *  GLite Data Transfer Agents: UrlCopyTransferService Class
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *           Paolo Tedesco <paolo.tedesco@cern.ch>
 *           Zsolt Molnar <zsolt.molnar@cern.ch - http://www.zsoltmolnar.hu>
 *  Version info: $Id: UrlCopyTransferService.cpp,v 1.45 2009/12/11 16:14:25 molnarzs Exp $
 *  Release: $Name:  $
 */

#include "UrlCopyTransferService.h"
#include "UrlCopyException.h"
#include <glite/data/agents/UserProxyEnv.h>
#include <glite/data/agents/AgentExceptions.h>
#include <glite/data/agents/url-utility.h>
#include <glite/data/transfer/urlcopy/TransferUrlCopy.h>

#include <math.h>
#include <sstream>
#include <assert.h>
#include <boost/scoped_ptr.hpp>

using namespace glite::data::transfer::agent;
using namespace glite::data::transfer::agent::model;
using namespace glite::data::transfer::agent::ts;
using glite::data::agents::UserProxyEnv;
using glite::data::agents::LogicError;
using glite::data::agents::RuntimeError;

/*
 * The TransferService Type
 */
const char * const UrlCopyTransferService::TRANSFER_TYPE_URLCOPY = "urlcopy";

/*
 * UrlCopyTransferService
 *
 * Constructor
 */
UrlCopyTransferService::UrlCopyTransferService(const std::string& channel,const std::string& category)
  : TransferServiceBase(channel,category),
    m_srmGridFTPSplit(true),
    m_tucProto(dynamic_cast<UrlCopyTransferService::TransferUrlCopyStarter_*>(new UrlCopyTransferService::TransferUrlCopyStarter_Default_()))
{
    // EMPTY
}

/*
 * ~UrlCopyTransferService
 *
 * Destructor
 */
UrlCopyTransferService::~UrlCopyTransferService(){
}

/*
 * type
 *
 * Return the TransferService type
 */
std::string UrlCopyTransferService::type() /*throw ()*/{
    return TRANSFER_TYPE_URLCOPY;
}

/*
 * maxRequestSize
 *
 * Get the number of max Transfers that can be processed whitin the same request
 */
unsigned long UrlCopyTransferService::maxRequestSize() {
    return 1;
}

/*
 * submit
 *
 * Submit a FileTransfer request
 */
void UrlCopyTransferService::submit(
    const TransferRequest&  req,
    TransferResponse&       out,
    const std::string&      credentials) /*throw (TransferException)*/{

    // Check Precondition
    if(req.surls.size() != 1){
        m_log_warn("Cannot process request. Bulk size (" << req.surls.size() << ") exceeded");
        throw UrlCopyException(Error::CONFIGURATION,Error::S_AGENT,Error::P_TX_SERVICE,"too many files");
    }

    // Get Transfer Data
    const std::string& sourceSurl   = req.surls.at(0).source;
    const std::string& destSurl     = req.surls.at(0).destination;
    const std::string& checksum_algorithm = req.surls.at(0).checksum_algorithm;
    const std::string& checksum     = req.surls.at(0).checksum_value;
    const std::string& params       = req.parameters;
    const std::string& src_srm      = req.src_srm;
    const std::string& src_srm_ver  = req.src_srm_version;
    const std::string& dest_srm     = req.dest_srm;
    const std::string& dest_srm_ver = req.dest_srm_version;
    const std::string& description  = req.description;
    m_log_info("submit called. Parameters are:");
    m_log_info("Source SRM         : " << src_srm);
    m_log_info("Source SRM Vers.   : " << src_srm_ver);
    m_log_info("Destination SRM    : " << dest_srm);
    m_log_info("Dest. SRM  Vers.   : " << dest_srm_ver);
    m_log_info("Dest. Space Token  : " << req.dest_space_token);
    m_log_info("Dest. Tkn Descr.   : " << req.dest_token_descr);
    m_log_info("Params             : " << params);
    m_log_info("Overwrite          : " << str_bool(req.overwrite_flag));
    m_log_info("Description        : " << description);
    m_log_info("Source             : " << sourceSurl);
    m_log_info("Destination        : " << destSurl);
    m_log_info("Checksum algorithm : " << checksum_algorithm);
    m_log_info("Checksum           : " << checksum);

    // Set the environment variable for the delegated proxy
    // Note: this doesn't work in a multithreaded environment. it would be better
    // that this will implemented inside the glite-url-copy, after the fork
    UserProxyEnv cert_guard(credentials);
    try{

        // Read Parameters
        JobParams job_params;
        readParams(params,req,job_params);

        // Create the Transfer Request object
        urlcopy::TransferUrlCopy::Args args;
        args.src_endpoint    = src_srm;
        args.src_version     = src_srm_ver;
        args.src_name        = sourceSurl;
        args.dest_endpoint   = dest_srm;
        args.dest_version    = dest_srm_ver;
        args.dest_name       = destSurl;
        // Configuration
        args.overwrite       = req.overwrite_flag;
        args.user_tag        = description;
        args.console_logging = false;

        args.verbose         = (Channel::LL_DEBUG == req.log_level);
        args.nstreams        = job_params.streams;
        args.tcpbs           = job_params.tcpbs;
        args.blocksize       = job_params.blocksize;
        // Space Token
        args.dest_space_token = req.dest_space_token;
        args.dest_token_descr = req.dest_token_descr;
        // New in 3.1.0
        if (req.urlcopy_put_timeout >= 0){
            args.put_timeout = req.urlcopy_put_timeout;
        }
        if (req.urlcopy_putdone_timeout >= 0){
            args.putdone_timeout = req.urlcopy_putdone_timeout;
        }
        if (req.urlcopy_get_timeout >= 0){
            args.get_timeout = req.urlcopy_get_timeout;
        }
        if (req.urlcopy_getdone_timeout >= 0){
            args.getdone_timeout = req.urlcopy_getdone_timeout;
        }
        if (req.urlcopy_tx_timeout >= 0){
            args.tx_timeout = req.urlcopy_tx_timeout;
        }
        if (req.http_timeout >= 0){
            args.http_timeout = req.http_timeout;
        }
        if (req.urlcopy_tx_markers_timeout >= 0){
            args.tx_markers_timeout = req.urlcopy_tx_markers_timeout;
        }
        // New in 3.2.0 - fix bug #39882
        args.source_space_token = req.source_space_token;
        args.source_token_descr = req.source_token_descr;
        args.enable_dir_check = req.enable_dir_check;
        if (req.urlcopy_first_tx_marker_timeout >= 0){
            args.first_tx_marker_timeout = req.urlcopy_first_tx_marker_timeout;
        }
        // New in 3.3.0
        args.no_progress_timeout     = req.no_tx_activity_timeout;
        args.copy_pin_lifetime       = req.copy_pin_lifetime;
        args.lan_connection          = req.lan_connection;
        args.fail_nearline_files     = req.fail_nearline;
        args.tx_to_per_mb            = req.tx_timeout_per_mb;
        // New in 3.4.1
        args.compare_checksum        = req.surls.at(0).compare_checksum;
        args.checksum_user_algorithm = checksum_algorithm;
        args.checksum_user_value     = checksum;

        // Stop after preparation or not
        args.block_after_preparation = m_srmGridFTPSplit;

        _setupSyslogArgs(args);

        // Start The Transfer
        assert(m_tucProto.get());
        urlcopy::TransferUrlCopy::Ptr tx = m_tucProto->start(args, m_channel);
        assert(tx.get());
        m_log_debug("submit returned ID: [" << tx->id() << "]");

        // Store the requestId
        out.request_id = tx->id();
        out.transfer_ids.push_back(0);
    } catch(const RuntimeError& exc) {
        m_log_error("Failed to start a new transfer: " << exc.what());
        throw UrlCopyException(Error::INTERNAL_ERROR,Error::S_AGENT,Error::P_TX_SERVICE,exc.what());
    } catch(const LogicError& exc) {
        m_log_error("Logic Error in starting a new transfer: " << exc.what());
        throw;
    }catch(...){
        m_log_error("Unhandled Exception in start");
        throw;
    }
}

/*
 *  state
 *
 * Get the status of a Transfer request
 */
void UrlCopyTransferService::state(const std::string& requestId, TransferInfoArray& objs, const std::string& credentials) /*throw (TransferException)*/{

    // Check Precondition
    if(objs.size() != 1){
        m_log_warn("Cannot process request. Bulk size (" << objs.size() << ") exceeded");
        throw UrlCopyException(Error::CONFIGURATION,Error::S_AGENT,Error::P_TX_SERVICE,"too many files");
    }

    m_log_debug("state called for request [" << requestId << "]");

    // Get Transfer Object
    Transfer& transfer_object = *(*(objs.begin()));

    // fix bug #39582 : get object here. If the requestId is invalid, an exception is thrown,
    // this is ok since TransferError, LogicError and RuntimeError objects are caught by the
    // CheckState action that, in such cases, sets the transfer to failed.
    // Get Transfer Object
    urlcopy::TransferUrlCopy::Ptr tx = urlcopy::TransferUrlCopy::get(requestId);

    try{
        // Get Transfer Status
        urlcopy::TransferUrlCopyStat s = tx->status();
        // Set Turls
        if((true == transfer_object.sourceTurl().empty()) &&
           (false == s.source.tx_name.empty())){
            transfer_object.sourceTurl = s.source.tx_name;
        }
        if((true == transfer_object.destTurl().empty()) &&
           (false == s.destination.tx_name.empty())){
            transfer_object.destTurl = s.destination.tx_name;
        }
        // Set source SE host name
        if((true == transfer_object.srcHost().empty()) &&
           (false == transfer_object.sourceTurl().empty())){
            try{
                std::string hostname;
                glite::data::agents::parse_url(transfer_object.sourceTurl(),hostname);
                transfer_object.srcHost = hostname;
            } catch(const LogicError&){}
        }
        // Set destination SE host name
        if((true == transfer_object.destHost().empty()) &&
           (false == transfer_object.destTurl().empty())){
            try{
                std::string hostname;
                glite::data::agents::parse_url(transfer_object.destTurl(),hostname);
                transfer_object.destHost = hostname;
            } catch(const LogicError&){}
        }
        // Set source SRM request Token
        if((true == transfer_object.srcSrmToken().empty()) &&
           (false == s.source.token.empty())){
            transfer_object.srcSrmToken = s.source.token;
        }
        // Set destination SRM request Token
        if((true == transfer_object.destSrmToken().empty()) &&
           (false == s.destination.token.empty())){
            transfer_object.destSrmToken = s.destination.token;
        }
        // Fill Performance Data
        if(s.tx.transferred_bytes > 0){
            transfer_object.bytesWritten = s.tx.transferred_bytes;
        }
        if((transfer_object.fileSize() <= 0) && (s.tx.size > 0)){
            transfer_object.fileSize      = s.tx.size;
        }
        m_log_trace("requestId " << transfer_object.requestId() << ", file_id " << transfer_object.fileId() << ", transferId " << transfer_object.transferId() << ", state " << transfer_object.state() << " - bytes written " << transfer_object.bytesWritten() << " - size " << transfer_object.fileSize());
        m_log_info("Transfer request [" << requestId << "] is [" << urlcopy::UrlCopyStatus::toString(s.status) << "]");
        // Check Status
        switch(s.status){
        case urlcopy::UrlCopyStatus::IDLE:
        case urlcopy::UrlCopyStatus::PREPARING:
            transfer_object.state = Transfer::S_PREPARING;
        break;
        case urlcopy::UrlCopyStatus::READY:
            transfer_object.state = Transfer::S_READY;
        break;
        case urlcopy::UrlCopyStatus::RUNNING:
            transfer_object.state = Transfer::S_PROCESSING;
        break;
        case urlcopy::UrlCopyStatus::DONE:
        case urlcopy::UrlCopyStatus::FAILED:
        case urlcopy::UrlCopyStatus::ABORTED:
            transfer_object.state = Transfer::S_TRANSFERRED;
        break;
        case urlcopy::UrlCopyStatus::COMPLETED:{
            if(s.error.category == urlcopy::UrlCopyError::SUCCESS){
                // Transfer is completed successfully
                m_log_info("Transfer request [" << requestId << "] completed successfully");
                // Set Transfer Status and Reason class
                transfer_object.state       = Transfer::S_COMPLETED;
                transfer_object.reasonClass = Error::UNDEF;
            } else {
                // Create error reason
                std::stringstream ss;
                ss << urlcopy::UrlCopyError::scopeToString(s.error.scope)
                   << " error during "
                   << urlcopy::UrlCopyError::phaseToString(s.error.phase)
                   << " phase: ["
                   << urlcopy::UrlCopyError::categoryToString(s.error.category)
                   << "] " << s.error.description;
                std::string reason = ss.str();
                m_log_error("Transfer Request [" << requestId << "] failed: " << reason);
                // Check Source Error
                if((s.error.scope != urlcopy::UrlCopyError::SCOPE_SOURCE) &&
                   (s.source.error.category != urlcopy::UrlCopyError::SUCCESS)){
                    m_log_warn("Error also on " << urlcopy::UrlCopyError::scopeToString(s.source.error.scope) << " during " << urlcopy::UrlCopyError::phaseToString(s.source.error.phase) << ": [" << urlcopy::UrlCopyError::categoryToString(s.source.error.category) << "] " << s.source.error.description);
                }
                // Check Destination Error
                if((s.error.scope != urlcopy::UrlCopyError::SCOPE_DESTINATION) &&
                   (s.destination.error.category != urlcopy::UrlCopyError::SUCCESS)){
                    m_log_warn("Error also on " << urlcopy::UrlCopyError::scopeToString(s.destination.error.scope) << " during " << urlcopy::UrlCopyError::phaseToString(s.destination.error.phase) << ": [" << urlcopy::UrlCopyError::categoryToString(s.destination.error.category) << "] " << s.destination.error.description);
                }
                // Check Transfer Error
                if((s.error.scope != urlcopy::UrlCopyError::SCOPE_TRANSFER) &&
                   (s.tx.error.category != urlcopy::UrlCopyError::SUCCESS)){
                    m_log_warn("Error also on " << urlcopy::UrlCopyError::scopeToString(s.tx.error.scope) << " during " << urlcopy::UrlCopyError::phaseToString(s.tx.error.phase) << ": [" << urlcopy::UrlCopyError::categoryToString(s.tx.error.category) << "] " << s.tx.error.description);
                }

                // Set Transfer Status and Reason class
                if(s.error.category == urlcopy::UrlCopyError::ERROR_ABORTED){
                    transfer_object.state = Transfer::S_ABORTED;
                } else {
                    transfer_object.state = Transfer::S_FAILED;
                }

                translateUrlCopyError(s.error,transfer_object);
                transfer_object.reason = s.error.description;
            }
            // Set The Transfer Duration
            if(transfer_object.duration() <= 0.0){
                double diff = (double)s.finish_time - (double)s.start_time;
                if(diff >= 0){
                    transfer_object.duration = diff;
                } else {
                    // Duration still unset: compute it from the transfer time
                    time_t tr_time = transfer_object.transferTime();
                    if(tr_time > 0){
                        // Get the current time
                        time_t curr_time;
                        time(&curr_time);
                        diff = (double)curr_time - (double)tr_time;
                        if(diff > 0){
                            transfer_object.duration = diff;
                        }
                    }
                }
            }
            // Set Phase Duration
            if(transfer_object.srcPrepDuration() <= 0.0){
                double diff = (double)s.source.prep_done_time - (double)s.source.prep_start_time;
                if(diff >= 0){
                    transfer_object.srcPrepDuration = diff;
                }
            }
            if(transfer_object.dstPrepDuration() <= 0.0){
                double diff = (double)s.destination.prep_done_time - (double)s.destination.prep_start_time;
                if(diff >= 0){
                    transfer_object.dstPrepDuration = diff;
                }
            }
            if(transfer_object.txDuration() <= 0.0){
                double diff = (double)s.tx.done_time - (double)s.tx.start_time;
                if(diff >= 0){
                    transfer_object.txDuration = diff;
                }
            }
            if(transfer_object.srcFinalDuration() <= 0.0){
                double diff = (double)s.source.final_done_time - (double)s.source.final_start_time;
                if(diff >= 0){
                    transfer_object.srcFinalDuration = diff;
                }
            }
            if(transfer_object.dstFinalDuration() <= 0.0){
                double diff = (double)s.destination.final_done_time - (double)s.destination.final_start_time;
                if(diff >= 0){
                    transfer_object.dstFinalDuration = diff;
                }
            }
            // Set Average throughput as returned by the gridftp server
            if(transfer_object.throughput() <= 0){
                if(s.tx.avg_throughput >= 0){
                    transfer_object.throughput = s.tx.avg_throughput;
                } else if (s.tx.instant_throughput >= 0){
                    transfer_object.throughput = s.tx.instant_throughput;
                }
            }

            // Transfer is completed : clean it
            try{
                tx->clean();
                m_log_debug("Transfer request [" << requestId << "] cleared");
            } catch(...){}
        } break;
        default:
            // This should not happen
            m_log_error("Unknown transfer status returned");
            throw RuntimeError("Unknown transfer status returned");
        break;
        } // end switch
    } catch(const RuntimeError& exc) {
        m_log_error("Failed to check transfer status: " << exc.what());
        try{ // fix bug #39582
            tx->clean();
            m_log_debug("Transfer request [" << requestId << "] cleared");
        } catch(...){
            m_log_warn("Could not clear transfer request [" << requestId << "]");
        }
        throw UrlCopyException(Error::INTERNAL_ERROR,Error::S_AGENT,Error::P_TX_SERVICE,exc.what());
    } catch(const LogicError& exc) {
        m_log_error("Logic Error in checking transfer status: " << exc.what());
        try{ // fix bug #39582
            tx->clean();
            m_log_debug("Transfer request [" << requestId << "] cleared");
        } catch(...){
            m_log_warn("Could not clear transfer request [" << requestId << "]");
        }
        throw;
    }
}

/*
 * revoke
 *
 * Cancel a Pending Transfer request
 */
void UrlCopyTransferService::revoke(const std::string& requestId, const std::string& credentials) /*throw (TransferException)*/{

    m_log_debug("revoke called for request [" << requestId << "]");

    try{
        // Get Transfer Object
        urlcopy::TransferUrlCopy::Ptr tx = urlcopy::TransferUrlCopy::get(requestId);
        // Stop the Transfer
        tx->stop();
        // Get Transfer Status
        urlcopy::TransferUrlCopyStat s = tx->status();
        if(s.status != urlcopy::UrlCopyStatus::COMPLETED){
            // This should not happen. Anyway call abort
            m_log_warn("Transfer [" << requestId << "] didn't completed cleanly");
            throw RuntimeError("Transfer not completed after a stop");
        }
        if(s.error.category == urlcopy::UrlCopyError::ERROR_ABORTED){
            m_log_info("Transfer [" << requestId << "] successfully revoked");
        } else if(s.error.category == urlcopy::UrlCopyError::SUCCESS){
            m_log_warn("Transfer [" << requestId << "] was already completed");
        } else {
            m_log_warn("Transfer [" << requestId << "] was already failed");
        }
        // Transfer is completed : clean it
        try{
            tx->clean();
            m_log_debug("Transfer request [" << requestId << "] cleared");
        } catch(...){}
    } catch(const RuntimeError& exc) {
        m_log_error("Failed to stop the transfer: " << exc.what());
        throw UrlCopyException(Error::INTERNAL_ERROR,Error::S_AGENT,Error::P_TX_SERVICE,exc.what());
    } catch(const LogicError& exc) {
        m_log_error("stop failed (LogicError). Reason is: " << exc.what());
        throw UrlCopyException(Error::INTERNAL_ERROR,Error::S_AGENT,Error::P_TX_SERVICE,exc.what());
    }
}

/*
 * abort
 *
 * Abort a Pending Transfer request. This method is intended for performing
 * a best effort clean up when the user credentials are not available to
 * revoke the transfer
 */
void UrlCopyTransferService::abort(const std::string& requestId) /*throw ()*/{
    m_log_debug("abort called for request " << requestId);

    try{
        // Get Transfer Object
        urlcopy::TransferUrlCopy::Ptr tx = urlcopy::TransferUrlCopy::get(requestId);
        // Stop the Transfer
        try{
            tx->abort();
        } catch(const std::exception& exc){
            m_log_warn("Failed to abort the transfer: " << exc.what());
            // Ignore Error
        }
        // Call clean anyway
        try{
            tx->clean();
            m_log_debug("Transfer request [" << requestId << "] cleared");
        } catch(...){}
    } catch(const RuntimeError& exc) {
        m_log_warn("Failed to abort the transfer: " << exc.what());
        // Ignore Error
    } catch(const LogicError& exc) {
        m_log_warn("stop failed (LogicError). Reason is: " << exc.what());
        // Ignore Error
    }
    m_log_info("Transfer request [" << requestId << "] successfully aborted");
}

/**
 * Get the maximum number of allowed transfers in any state for the channel
 * @param channel_transfers [IN]
 * @return maximum number of allowed transfers in any state for the channel
 */
unsigned long UrlCopyTransferService::getMaxChannelTransfers(
    unsigned long channel_transfers,
    double preparingFilesRatio){
    if (m_srmGridFTPSplit) {
        return (unsigned long)ceil(channel_transfers * fmax(preparingFilesRatio,1.0));
    }
    return channel_transfers;
}

/**
 * Get the maximum number of allowed transfers in any state for a VO
 * @param vo_transfers [IN]
 * @return maximum number of allowed transfers in any state for a VO
 */
unsigned long UrlCopyTransferService::getMaxVOTransfers(
    unsigned long vo_transfers,
    double preparingFilesRatio){
    return (unsigned long)ceil(vo_transfers * fmax(preparingFilesRatio,1.0));
}

/**
 * return true if transfers stop after the preparation phase to maintain constant
 * the number of transfers in the transfer phase
 * @return
 */
bool UrlCopyTransferService::stopAfterPreparation() /* throw() __2008-04-21-1223_SI0dpc*/{
    return m_srmGridFTPSplit;
}

/**
 * start the transfer phase for given transfer
 * @param transfer [IN] the transfer for which transfer phase must be started
 */
void UrlCopyTransferService::startTransferPhase(
    const model::Transfer&  transfer) /*throw (LogicError)*/{

    // if the transfers are not blocked after preparation, then there is no need to restart them
    if (!m_srmGridFTPSplit) {
        throw LogicError("startTransferPhase called with a non-split URL copy channel");
    }

    const std::string& requestId = transfer.requestId();
    m_log_debug("starting transfer phase for request " << requestId);
    try{
        // Get Transfer Object
        urlcopy::TransferUrlCopy::Ptr tx = urlcopy::TransferUrlCopy::get(requestId);
        // Stop the Transfer
        try{
            pid_t pid = tx->continueTransfer();
            m_log_info("Signal sent to continue transfer request " << requestId << " (pid = " << pid << ")");
        } catch(const std::exception& exc){
            m_log_warn("Failed to start transfer phase: " << exc.what());
            // Ignore Error
        }
    } catch(const RuntimeError& exc) {
        m_log_warn("Failed to start transfer phase: " << exc.what());
        // Ignore Error
    } catch(const LogicError& exc) {
        m_log_warn("Failed to start transfer phase (LogicError): " << exc.what());
        // Ignore Error
    }
}

/**
 * Get the transfer states that represent an active transfer
 * @return
 */
model::Transfer::State UrlCopyTransferService::getActiveStates() /* throw() */{
    return (model::Transfer::S_PROCESSING);
}

/**
 * Get the transfer states that represent a transfer that needs starting the
 * transfer phase
 * @return
 */
model::Transfer::State UrlCopyTransferService::getStatesToStart() /* throw() */{
    return (model::Transfer::S_READY);
}

bool UrlCopyTransferService::getSrmGridFTPSplit() const
{
    return m_srmGridFTPSplit;
}

void UrlCopyTransferService::setSrmGridFTPSplit(bool b)
{
    m_srmGridFTPSplit = b;
}
