/*
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *  GLite Data Transfer Agents: VOAgent Class
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *           Paolo Tedesco <paolo.tedesco@cern.ch>
 *  Version info: $Id: VOAgent.cpp,v 1.24 2009/01/08 13:50:39 ptedesco Exp $
 *  Release: $Name:  $
 *
 */
#include "VOAgent.h"
#include "JobFlags.h"
#include <glite/data/transfer/agent/scheduler/SchedulerEntry.h>
#include <glite/data/transfer/agent/AgentState.h>
#include "Heartbeat.h"
#include "CleanSDCache.h"

#include <glite/data/transfer/agent/action/vo/VOAction.h>
#include <glite/data/transfer/agent/dao/vo/DAOFactory.h>
#include <glite/data/agents/dao/DAOContextFactory.h>
#include <glite/data/transfer/agent/dao/TSSDAOContext.h>
#include <glite/data/transfer/agent/dao/TSSCredFactory.h>
#include <glite/data/transfer/agent/dao/vo/CredVOFactory.h>

#include <glite/data/agents/AgentExceptions.h>
#include <glite/data/agents/cred/cred-utility.h>

#include <glite/data/config/service/ParamReader.hxx>

#include <string>
#include <sstream>
#include <algorithm>
#include <cctype>
#include <unistd.h>

using namespace glite::data::transfer::agent;
using namespace glite::config;
using glite::data::agents::ConfigParamException;
using glite::data::agents::ConfigurationException;
using glite::data::agents::LogicError;

/*
 * Constants
 */
namespace {
const char * const VO_AGENT_NAME                   = "transfer-vo-agent";
                                                                                                                          
// Properties Names
const char * const NAME_PROPERTY_NAME              = "Name";
const char * const CONTACT_PROPERTY_NAME           = "Contact";
const char * const DISABLEDELEGATION_PROPERTY_NAME = "DisableDelegationForTransfers";
const char * const ACTIONINTERVAL_PROPERTY_NAME    = "DefaultInterval";
    
const char * const ALLOCATE_TYPE_PROPERTY_NAME       = "Allocate_Type";
const char * const RETRY_TYPE_PROPERTY_NAME          = "Retry_Type";
const char * const CHECKREADINESS_TYPE_PROPERTY_NAME = "CheckReadiness_Type";
const char * const FINALIZE_TYPE_PROPERTY_NAME       = "Finalize_Type";
const char * const CANCEL_TYPE_PROPERTY_NAME         = "Cancel_Type";
    
const char * const ALLOCATE_INTERVAL_PROPERTY_NAME       = "Allocate_Interval";
const char * const RETRY_INTERVAL_PROPERTY_NAME          = "Retry_Interval";
const char * const CANCEL_INTERVAL_PROPERTY_NAME         = "Cancel_Interval";
const char * const CHECKREADINESS_INTERVAL_PROPERTY_NAME = "CheckReadiness_Interval";
const char * const FINALIZE_INTERVAL_PROPERTY_NAME       = "Finalize_Interval";
const char * const HEARTBEAT_INTERVAL_PROPERTY_NAME      = "Heartbeat_Interval";
const char * const CLEANSDCACHE_INTERVAL_PROPERTY_NAME   = "CleanSDCache_Interval";

// Properties Default  Values
const bool   DEFAULT_DISABLEDELEGATION           = false;
const unsigned int DEFAULT_ACTION_INTERVAL       = 3;
const unsigned int DEFAULT_RETRY_INTERVAL        = 60;
const unsigned int DEFAULT_CANCEL_INTERVAL       = 60;
const unsigned int DEFAULT_HEARTBEAT_INTERVAL    = 60;
const unsigned int DEFAULT_CLEANSDCACHE_INTERVAL = 300;

const char * const DEFAULT_ALLOCATE_TYPE               = "glite:Allocate";
const char * const DEFAULT_RETRY_TYPE                  = "glite:BasicRetry";
const char * const DEFAULT_CANCEL_TYPE                 = "glite:CancelPending";
const char * const DEFAULT_CHECKREADINESS_TYPE         = "glite:CheckReadiness";
const char * const DEFAULT_FINALIZE_TYPE               = "glite:Finalize";

const char * const VO_AGENT_TYPE                       = "vo";
}

extern "C" {
/*
 * create_glite_component
 *
 * Create Component instance
 */
ComponentConfiguration * create_glite_component(){
    return new VOAgent();
}
                                                                                
/*
 * destroy_glite_component
 *
 * Destroy Component instance
 */
void destroy_glite_component(ComponentConfiguration * component){
    VOAgent * agent = dynamic_cast<VOAgent *>(component);
    if(0 != agent){
        delete agent;
    }
}
                                                                                
} // End extern "C"

/*
 * VOAgent
 *
 * Constructor
 */
VOAgent::VOAgent():ComponentConfiguration(VO_AGENT_NAME), 
    m_logger(VO_AGENT_NAME),
    m_disableDelegation(DEFAULT_DISABLEDELEGATION),
    m_allocateType(DEFAULT_ALLOCATE_TYPE),
    m_retryType(DEFAULT_RETRY_TYPE),
    m_cancelType(DEFAULT_CANCEL_TYPE),
    m_checkReadinessType(DEFAULT_CHECKREADINESS_TYPE),
    m_finalizeType(DEFAULT_FINALIZE_TYPE),
    m_defaultInterval(DEFAULT_ACTION_INTERVAL),
    m_allocateInterval(DEFAULT_ACTION_INTERVAL),
    m_retryInterval(DEFAULT_RETRY_INTERVAL),
    m_cancelInterval(DEFAULT_CANCEL_INTERVAL),
    m_checkReadinessInterval(DEFAULT_ACTION_INTERVAL),
    m_finalizeInterval(DEFAULT_ACTION_INTERVAL),
    m_heartbeatInterval(DEFAULT_HEARTBEAT_INTERVAL),
    m_cleansdcacheInterval(DEFAULT_CLEANSDCACHE_INTERVAL){
}
                                                                                                                          
/*
 * ~VOAgent
 *
 * Destructor
 */
VOAgent::~VOAgent(){
}

/*
 * init
 *
 * Initialize the Component
 */
int VOAgent::init(const Params& params){
          
    try{
        ParamReader reader(getName(),params);
        // Get Name
        reader.getParameter(NAME_PROPERTY_NAME,m_voName)
            .required();       
        // Get Contact
        reader.getParameter(CONTACT_PROPERTY_NAME,m_contact);
        // Get Disable Delegation
        reader.getParameter(DISABLEDELEGATION_PROPERTY_NAME,m_disableDelegation);
        // Get Allocate Action Type
        reader.getParameter(ALLOCATE_TYPE_PROPERTY_NAME,m_allocateType);
        // Get Retry Action Type
        reader.getParameter(RETRY_TYPE_PROPERTY_NAME,m_retryType);
        // Get Cancel Action Type
        reader.getParameter(CANCEL_TYPE_PROPERTY_NAME,m_cancelType);
        // Get CheckReadiness Action Type
        reader.getParameter(CHECKREADINESS_TYPE_PROPERTY_NAME,m_checkReadinessType);
        // Get Finalize Action Type
        reader.getParameter(FINALIZE_TYPE_PROPERTY_NAME,m_finalizeType);
        
        // Log Configuration Values
        m_log_info(getName() << " Initialized. Initialization Parameters are");
        m_log_info("Name                  : " << m_voName);
        m_log_info("Contact               : " << m_contact);
        m_log_info("Cred. Delegation      : " << str_enabled(m_disableDelegation));
        m_log_info("Allocate Action Type  : " << m_allocateType);
        m_log_info("Retry    Action Type  : " << m_retryType);
        m_log_info("Cancel   Action Type  : " << m_cancelType);
        m_log_info("Check R. Action Type  : " << m_checkReadinessType);
        m_log_info("Finalize Action Type  : " << m_finalizeType);
    } catch(const LogicError& exc) {
        m_log_error(getName() << " configuration Error: " << exc.what());
        return -1;
    } catch(const std::exception& exc) {
        m_log_error(getName() << " configuration Unexpected Exception: " << exc.what());
        return -1;
    } catch(...) {
        m_log_error(getName() << " configuration Unhandled Exception");
        return -1;
    }
    return 0;
}

                                                                                                                          
/*
 * config
 *
 * Configure the Component
 */
int VOAgent::config(const Params& params){
    try{
        Params::const_iterator it;
        
        // Get Default Action Interval
        if((it = params.find(ACTIONINTERVAL_PROPERTY_NAME)) != params.end()){
            ParamValue * pv = dynamic_cast<ParamValue *>((*it).second);
            if(0 == pv){
                throw ConfigParamException(getName(),ACTIONINTERVAL_PROPERTY_NAME,ConfigParamException::INVALID);
            }
            m_defaultInterval = atoi(pv->getValue().c_str());
            if(m_defaultInterval < 1) {
                throw ConfigParamException(getName(),ACTIONINTERVAL_PROPERTY_NAME, ConfigParamException::INVALID_VALUE);
            }
        }
        // Get Allocate Action Interval
        if((it = params.find(ALLOCATE_INTERVAL_PROPERTY_NAME)) != params.end()){
            ParamValue * pv = dynamic_cast<ParamValue *>((*it).second);
            if(0 == pv){
                throw ConfigParamException(getName(),ALLOCATE_INTERVAL_PROPERTY_NAME,ConfigParamException::INVALID);
            }
            if(false == pv->getValue().empty()){
                m_allocateInterval = atoi(pv->getValue().c_str());
                if(m_allocateInterval < 0) {
                    throw ConfigParamException(getName(),ACTIONINTERVAL_PROPERTY_NAME, ConfigParamException::INVALID_VALUE);
                }
            } else {
                m_allocateInterval = m_defaultInterval;
            }
        } else {
            m_allocateInterval = m_defaultInterval;
        }
        // Get Retry Action Interval
        if((it = params.find(RETRY_INTERVAL_PROPERTY_NAME)) != params.end()){
            ParamValue * pv = dynamic_cast<ParamValue *>((*it).second);
            if(0 == pv){
                throw ConfigParamException(getName(),RETRY_INTERVAL_PROPERTY_NAME,ConfigParamException::INVALID);
            }
            if(false == pv->getValue().empty()){
                m_retryInterval = atoi(pv->getValue().c_str());
                if(m_retryInterval < 0) {
                    throw ConfigParamException(getName(),RETRY_INTERVAL_PROPERTY_NAME, ConfigParamException::INVALID_VALUE);
                }
            } else {
                m_retryInterval = m_defaultInterval;
            }
        } else {
            m_retryInterval = m_defaultInterval;
        }
        // Get Cancel Action Interval
        if((it = params.find(CANCEL_INTERVAL_PROPERTY_NAME)) != params.end()){
            ParamValue * pv = dynamic_cast<ParamValue *>((*it).second);
            if(0 == pv){
                throw ConfigParamException(getName(),CANCEL_INTERVAL_PROPERTY_NAME,ConfigParamException::INVALID);
            }
            if(false == pv->getValue().empty()){
                m_cancelInterval = atoi(pv->getValue().c_str());
                if(m_cancelInterval < 0) {
                    throw ConfigParamException(getName(),CANCEL_INTERVAL_PROPERTY_NAME, ConfigParamException::INVALID_VALUE);
                }
            }
        } 
        // Get CheckReadinessAction Interval
        if((it = params.find(CHECKREADINESS_INTERVAL_PROPERTY_NAME)) != params.end()){
            ParamValue * pv = dynamic_cast<ParamValue *>((*it).second);
            if(0 == pv){
                throw ConfigParamException(getName(),CHECKREADINESS_INTERVAL_PROPERTY_NAME,ConfigParamException::INVALID);
            }
            if(false == pv->getValue().empty()){
                m_checkReadinessInterval= atoi(pv->getValue().c_str());
                if(m_checkReadinessInterval < 0) {
                    throw ConfigParamException(getName(),CHECKREADINESS_INTERVAL_PROPERTY_NAME, ConfigParamException::INVALID_VALUE);
                }
            } else {
                m_checkReadinessInterval = m_defaultInterval;
            }
        } else {
            m_checkReadinessInterval = m_defaultInterval;
        }
        // Get Finalize Action Interval
        if((it = params.find(FINALIZE_INTERVAL_PROPERTY_NAME)) != params.end()){
            ParamValue * pv = dynamic_cast<ParamValue *>((*it).second);
            if(0 == pv){
                throw ConfigParamException(getName(),FINALIZE_INTERVAL_PROPERTY_NAME,ConfigParamException::INVALID);
            }
            if(false == pv->getValue().empty()){
                m_finalizeInterval = atoi(pv->getValue().c_str());
                if(m_finalizeInterval < 0) {
                    throw ConfigParamException(getName(),FINALIZE_INTERVAL_PROPERTY_NAME, ConfigParamException::INVALID_VALUE);
                }
            } else {
                m_finalizeInterval = m_defaultInterval;
            }
        } else {
            m_finalizeInterval = m_defaultInterval;
        }
        // Get Heartbeat Action Interval
        if((it = params.find(HEARTBEAT_INTERVAL_PROPERTY_NAME)) != params.end()){
            ParamValue * pv = dynamic_cast<ParamValue *>((*it).second);
            if(0 == pv){
                throw ConfigParamException(getName(),HEARTBEAT_INTERVAL_PROPERTY_NAME,ConfigParamException::INVALID);
            }
            if(false == pv->getValue().empty()){
                m_heartbeatInterval = atoi(pv->getValue().c_str());
                if(m_heartbeatInterval < 0) {
                    throw ConfigParamException(getName(),HEARTBEAT_INTERVAL_PROPERTY_NAME, ConfigParamException::INVALID_VALUE);
                }
            }
        }
        // Get CleanSDCache Action Interval
        if((it = params.find(CLEANSDCACHE_INTERVAL_PROPERTY_NAME)) != params.end()){
            ParamValue * pv = dynamic_cast<ParamValue *>((*it).second);
            if(0 == pv){
                throw ConfigParamException(getName(),CLEANSDCACHE_INTERVAL_PROPERTY_NAME,ConfigParamException::INVALID);
            }
            if(false == pv->getValue().empty()){
                m_cleansdcacheInterval = atoi(pv->getValue().c_str());
                if(m_cleansdcacheInterval < 0) {
                    throw ConfigParamException(getName(),CLEANSDCACHE_INTERVAL_PROPERTY_NAME, ConfigParamException::INVALID_VALUE);
                }
            }
        }
        // Create the Scheduler
        initScheduler();
        
        // Check the schema version
        dao::vo::DAOFactory::instance().checkSchema(*m_daoCtx);
        
        // Log Configuration Values
        m_log_info(getName() << " Configured. Configuration Parameters are");
        m_log_info("Default       Action Interval    : " << m_defaultInterval << " secs");
        m_log_info("Allocate      Action Interval    : " << m_allocateInterval << " secs");
        m_log_info("Retry         Action Interval    : " << m_retryInterval << " secs");
        m_log_info("Cancel        Action Interval    : " << m_cancelInterval << " secs");
        m_log_info("Check Read.   Action Interval    : " << m_checkReadinessInterval << " secs");
        m_log_info("Finalize      Action Interval    : " << m_finalizeInterval << " secs");
        m_log_info("Heartbeat     Action Interval    : " << m_heartbeatInterval << " secs");
        m_log_info("Clean SDCache Action Interval    : " << m_cleansdcacheInterval << " secs");
    } catch(const LogicError& exc) {
        m_log_error(getName() << " configuration Error: " << exc.what());
        return -1;
    } catch(const std::exception& exc) {
        m_log_error(getName() << " configuration Unexpected Exception: " << exc.what());
        throw;
    } catch(...) {
        m_log_error(getName() << " configuration Unhandled Exception");
        throw;
    }    
    try{
        std::string dn = agents::cred::get_proxy_dn();
        m_log_info("Agent for VO " << m_voName << " is using DN " << dn);
    } catch(const std::exception& exc){
        m_log_warn("Cannot Get Agent DN. Reason is " << exc.what());
    }
    
   m_log_info(getName() << " Configured");

    return 0;
}

/*
 * start
 *
 * Start the Component
 */
int VOAgent::start(){

    // Start the Scheduler
    if(0 != m_scheduler.get()){
        bool result = m_scheduler->start();
        if(false == result){
            return -1;
        }
    } else {
        return -1;
    }
        
    m_log_info("Agent for VO " << m_voName << " Started");
    return 0;
}

/*
 * stop
 *
 * Stop the Component
 */
int VOAgent::stop(){
        
    // Stop the Scheduler
    if(0 != m_scheduler.get()){
        m_scheduler->stop();
    } 
    
    m_log_info("Agent for VO " << m_voName << " Stopped");
    
    return 0;
}


/*
 * fini
 *
 * Finalize the Component
 */
int VOAgent::fini(){
    
    // Finalize the Scheduler
    finiScheduler();
    
    return 0;
}

/*
 * initScheduler
 *
 * Initialize the Scheduler
 */
void VOAgent::initScheduler() /*throw(ConfigException)*/{

    try{
        // Create DAO Context
        if(0 == m_daoCtx.get()){
            m_daoCtx.reset(agents::dao::DAOContextFactory::instance().create());
        }
    } catch(const agents::dao::DAOException& exc){
        m_log_error("Can't Create DAO Context. Reason is " << exc.what());
        throw ConfigurationException(exc.what());
    }
    
    // Create CredFactory fro the VOFactory
    if(0 == m_credFactory.get()){
        m_credFactory.reset(new dao::vo::CredVOFactory(m_voName));
    }
    // Create Action Scheduler
    if(0 != m_scheduler.get()){
        finiScheduler();
    }
    m_scheduler.reset(new ActionScheduler(VO_AGENT_NAME,*m_daoCtx,this));

    // Register Heartbeat
    scheduleAction(*(action::vo::Heartbeat::factory()),m_heartbeatInterval);
    
    // Register CleanSDCache
    scheduleAction(*(action::vo::CleanSDCache::factory()),m_cleansdcacheInterval);
    
    if(false == m_disableDelegation){
        scheduleAction(m_allocateType    ,m_allocateInterval);
    } else {
        scheduleAction(m_allocateType    ,m_allocateInterval, JOB_PARAM_DISABLE_DELEGATION);
    }
    scheduleAction(m_checkReadinessType ,m_checkReadinessInterval);
    scheduleAction(m_finalizeType       ,m_finalizeInterval);
    scheduleAction(m_retryType          ,m_retryInterval);
    //scheduleAction(m_cancelType         ,m_cancelInterval);
}

/*
 * scheduleEntry
 *
 * Add a schedulerEntry for the given Action Type
 */
void VOAgent::scheduleAction(const std::string& type, unsigned int interval,const std::string& params) /*throw(ConfigException)*/{
    try{
        // Get Factory Method
        action::ActionFactoryMethod& method = action::ActionFactory::instance().get(type);

        return scheduleAction(method,interval,params);
    } catch(const action::ExecuteException& exc){
        m_log_error("Invalid Action type " << type << ". Reason is " << exc.what());
        throw ConfigurationException("Unknown Action Type");
    }
}

/*
 * scheduleEntry
 *
 * Add a schedulerEntry for the given Action Factory Method
 */
void VOAgent::scheduleAction(action::ActionFactoryMethod& method, unsigned int interval,const std::string& params) /*throw(ConfigException)*/{
    if(interval > 0){
        // Try to create an Action and check that a VO Action
        boost::scoped_ptr<action::Action> a(method.create()); 
        if(0 == dynamic_cast<action::vo::VOAction*>(a.get())){
            m_log_error("Action type " << method.type() << " is not a VO Action");
            throw ConfigurationException("Invalid Action Type. Not a VO Action");
        }
        // Add Sceduler Entry
        m_scheduler->add(new SchedulerEntry(method, m_voName, interval, params));
        m_log_info("Action type " << method.type() << " scheduled");
    } else {
        m_log_info("Action type " << method.type() << " disabled");
    }
}

/*
 * finiScheduler
 *
 * Finalize the Scheduler
 */
void VOAgent::finiScheduler(){
    
    // Clear Scheduler Entries
    if(0 != m_scheduler.get()){
        m_scheduler->clear();
    }
    // Delete Scheduler
    m_scheduler.reset();
}

/* 
 * onEventStart
 *
 * Handle Event Start
 */
bool VOAgent::onEventStart() /*throw()*/{
    if(0 != m_daoCtx.get()){
        try{
            // Set Agent State
            AgentState<dao::vo::DAOFactory> as(*m_daoCtx,
                m_voName,
                VO_AGENT_TYPE,
                m_contact,
                m_heartbeatInterval);
            
            // Check if there is an agent already running
            bool agent_active = as.check();
            if(true == agent_active){
                m_log_error("Failed to start: An active instance for VO Agent <" << m_voName << "> detected");
                return false;
            }
            // Set the Agent Active
            as.set(model::Agent::S_STARTED);
            m_log_info("Agent State set to Started");
            
            // Register the DAO Context on the TSS 
            dao::g_tss_dao_context.set(m_daoCtx.get());
            m_log_debug("DAOContext registered in thread-specific storage");

            // Register the CredFactory on the TSS 
            dao::g_tss_cred_factory.set(m_credFactory.get());
            m_log_debug("CredFactory registered in thread-specific storage");
        } catch(const agents::AgentException& exc){
            m_log_error("Failed to Set Agent State to Started: " << exc.what());
            return false;
        }
    } else {
        m_log_error("Invalid Context DAO");
        return false;
    }
    return true;
}

/*
 * onEventStop
 *
 * Handle Event Stop
 */
bool VOAgent::onEventStop() /*throw()*/{
    if(0 != m_daoCtx.get()){
        try{
            // Set Agent State
            AgentState<dao::vo::DAOFactory> as(*m_daoCtx,
                m_voName,
                VO_AGENT_TYPE,
                m_contact,
                m_heartbeatInterval);
            // Set Agent State
            as.update(model::Agent::S_STOPPED);
            m_log_info("Agent State set to Stopped");

            // Deregister the CredFactory from the TSS 
            dao::g_tss_cred_factory.set(0);
            m_log_debug("CredFactory deregistered from thread-specific storage");

            // Deregister the DAO Context from the TSS 
            dao::g_tss_dao_context.set(0);
            m_log_debug("DAOContext deregistered from thread-specific storage");
        } catch(const agents::AgentException& exc){
            m_log_error("Failed to Set Agent State to Stopped: " << exc.what());
        }
    } else {
        m_log_error("Invalid Context DAO");
    }
    
    return true;
}
