/*
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *  GLite Data Transfer Agents: VOConfig Class
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *           Paolo Tedesco <paolo.tedesco@cern.ch>
 *  Version info: $Id: VOAgent.h,v 1.11 2008/11/13 10:43:56 ptedesco Exp $
 *  Release: $Name:  $
 *
 */
#ifndef GLITE_DATA_TRANSFER_AGENTS_VOAGENT_H_
#define GLITE_DATA_TRANSFER_AGENTS_VOAGENT_H_

#include "glite/data/config/service/ComponentConfiguration.hxx"
#include "glite/data/transfer/agent/scheduler/ActionScheduler.h"
#include <string>
#include <vector>
#include <boost/scoped_ptr.hpp>

#include <glite/data/config/service/Logger.hxx>

namespace glite   {
namespace data    {
namespace transfer{
namespace agent   {
    
/*
 * Forward Declaration
 */    
namespace action{
class ActionFactoryMethod;
}
namespace dao{
class CredFactory;
}

/**
 * The VOAgent Class
 */    
class VOAgent : public glite::config::ComponentConfiguration ,
                public ActionScheduler::Listener{
public:
    /**
     * Initialize the Component
     */
    virtual int init(const Params& params);
                                                                                                                           
    /**
     * Configure the Component
     */
    virtual int config(const Params& params);
                                                                                                                           
    /**
     * Start the Component
     */
    virtual int start();
                                                                                                                           
    /**
     * Stop the Component
     */
    virtual int stop();
                                                                                                                           
    /**
     * Finailize the Component
     */
    virtual int fini();
    
    // ActionScheduler Listener Methods
    
    /**
     * Handle Event Start. This method would be called on the Scheduler
     * Thread when the thread is started
     */
    virtual bool onEventStart() /*throw()*/;
    /**
     * Handle Event Stop. This method would be called when the Scheduler
     * Thread when the thread is going to stop
     */
    virtual bool onEventStop() /*throw()*/;
    
protected:
    
    friend ComponentConfiguration * ::create_glite_component();
    friend void ::destroy_glite_component(ComponentConfiguration * component);

    /**
     * Constructor
     */
    VOAgent();
                                                                                                                           
    /**
     * Destructor
     */
    virtual ~VOAgent();
                        
private:

    /**
     * Initialize the Scheduler
     */
    void initScheduler() /*throw(ConfigException)*/;

    /**
     * Add a schedulerEntry for the given Action Type
     */
    void scheduleAction(const std::string& type, unsigned int interval, const std::string& params = "") /*throw(ConfigException)*/;

    /**
     * Add a schedulerEntry for the given Action Factory Method
     */
    void scheduleAction(class action::ActionFactoryMethod& method, unsigned int interval, const std::string& params = "") /*throw(ConfigException)*/;

    /**
     * Finalize the Scheduler
     */
    void finiScheduler();

private:
    glite::config::Logger m_logger;
    /**
     * The Name of the VO
     */
    std::string  m_voName;

    /**
     * The Name of the Contact for that Agent
     */
    std::string  m_contact;

    /**
     * DisableDelegation Flag
     */
    bool m_disableDelegation;
    
    /**
     * Allocate Action Type
     */
    std::string m_allocateType;
    /**
     * Retry Action Type
     */
    std::string m_retryType;
    /**
     * Cancel Action Type
     */
    std::string m_cancelType;
    /**
     * CheckReadiness Action Type
     */
    std::string m_checkReadinessType;
    /**
     * Finalize Action Type
     */
    std::string m_finalizeType;
    /**
     * Default Action Interval
     */
    unsigned int m_defaultInterval;
    /**
     * Allocate Action Interval
     */
    unsigned int m_allocateInterval;
    /**
     * Retry Action Interval
     */
    unsigned int m_retryInterval;
    /**
     * Cancel Action Interval
     */
    unsigned int m_cancelInterval;
    /**
     * Check Readiness Action Interval
     */
    unsigned int m_checkReadinessInterval;
    /**
     * Finalize Action Interval
     */
    unsigned int m_finalizeInterval;
    /**
     * Heartbeat Action Interval
     */
    unsigned int m_heartbeatInterval;
    /**
     * CleanSDCache Action Interval
     */
    unsigned int m_cleansdcacheInterval;
    /**
     * Action Scheduler
     */
    boost::scoped_ptr<class ActionScheduler> m_scheduler;
        
    /**
     * The DAO Context
     */
    boost::scoped_ptr<class agents::dao::DAOContext> m_daoCtx;

    /**
     * The Cred DAO Factory
     */
    boost::scoped_ptr<class dao::CredFactory> m_credFactory;
};

} // end namespace agent
} // end namespace transfer
} // end namespace data
} // end namespace glite

#endif //GLITE_DATA_TRANSFER_AGENTS_VOAGENT_H_
