/*
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *  GLite Data Transfer Agents: AllocateTransfers Test Case
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>, Paolo Tedesco <paolo.tedesco@cern.ch>
 *  Version info: $Id: AllocateTestCase.cpp,v 1.23 2008/04/11 11:02:56 ptedesco Exp $
 *  Release: $Name:  $
 *
 */
#include "Allocate.h"
#include "actions/vo/ActionsConfig.h"
#include "JobFlags.h"
#include "DaoHelper.h"
#include "glite/data/transfer/agent/action/ActionFactory.h"
#include "glite/data/transfer/agent/model/File.h"
#include "glite/data/transfer/agent/model/Job.h"
#include "glite/data/transfer/agent/dao/vo/FileDAO.h"
#include "glite/data/transfer/agent/dao/vo/JobDAO.h"
#include "glite/data/transfer/agent/dao/vo/DAOFactory.h"
#include "glite/data/transfer/agent/catalog/CatalogService.h"
#include "glite/data/transfer/agent/catalog/CatalogServiceFactory.h"
#include "glite/data/agents/dao/DAOContextFactory.h"
#include "dao/dao-utility.h"

#include "dao_test_utils.h"

#include <memory>
#include <boost/scoped_ptr.hpp>
#include <boost/shared_ptr.hpp>
#include <cppunit/TestCase.h>
#include <cppunit/extensions/HelperMacros.h>

using namespace glite::data::transfer::agent;
using namespace glite::data::transfer::agent::action;
using namespace glite::data::transfer::agent::action::vo;
using namespace glite::data::transfer::agent::model;
using namespace glite::data::transfer::agent::catalog;

using boost::scoped_ptr;
using boost::shared_ptr;

/**
 * CPPUnit TestCase
 */
class AllocateTestCase : public CppUnit::TestCase{
public:
    /**
     * set-up test case
     */
    void setUp ();

    void setUp_forTest1 ();
    void setUp_forTest2 ();

    /**
     * tear-down test case
     */
    void tearDown ();

private:
    // Declare CPPUnit Methods
    // -------------------------------------------------------------------------
    CPPUNIT_TEST_SUITE( AllocateTestCase);
        CPPUNIT_TEST( testExecute );
        CPPUNIT_TEST( testExecute_1 );
        CPPUNIT_TEST( testExecute_2 );
        CPPUNIT_TEST( test_bug31099 );
    CPPUNIT_TEST_SUITE_END();

    // Test Methods
    // -------------------------------------------------------------------------

    /**
     * Test Execute Methods
     */
    // allocate in different cycle parameter set
    void testExecute();
    // allocate in different cycle parameter NOT set
    void testExecute_1();
    // 3.1.0 test with groups
    void testExecute_2();
    
    // test for bug 31099
    void test_bug31099(); 
private:
    /**
     * The Action Instance
     */
    scoped_ptr<Allocate> m_action;
};

/*
 * Register Suit
 */
CPPUNIT_TEST_SUITE_NAMED_REGISTRATION( AllocateTestCase, "AllocateTestCase" );

/*
 * Declare Constants. This unit test foresee at beginning
 * (1) - 1 SUBMITTED Job with:                    => PENDING
 *        job_id      = 5-...-00
 *        channel     = [NULL]                   channel_3 (Inactive)
 *        files       = 50000001 (SUBMITTED)     PENDING
 *                      50000002 (SUBMITTED)     PENDING (this file is without port in the hostname)
 *
 * (2) - 1 SUBMITTED Job with:                    => PENDING
 *        job_id      = 5-...-01                 
 *        channel     = [NULL]                   channel_3
 *        files       = 50000003 (SUBMITTED)     PENDING
 *                      50000004 (SUBMITTED)     FAILED  (different_channel)
 *
 * (3) - 1 SUBMITTED Job with:                    => FAILED
 *        job_id      = 5-...-02                 [NULL] since channel_2 is Drain
 *        channel     = [NULL]                   
 *        files       = 50000005 (SUBMITTED)     FAILED
 *                      50000006 (SUBMITTED)     FAILED
 *
 * (4) - 1 SUBMITTED Job with:                    => PENDING
 *        job_id      = 5-...-03
 *        channel     = [NULL]                   channel_7 (Halted)
 *        files       = 50000007 (SUBMITTED)     PENDING
 *                      50000008 (PENDING)       PENDING
 *
 * (5) - 1 SUBMITTED Job with:                    => FAILED
 *        job_id      = 5-...-04
 *        channel     = [NULL]                   [NULL] since channel_1 has no share
 *        files       = 50000009 (SUBMITTED)     FAILED
 *                      50000010 (SUBMITTED)     FAILED
 *
 * (6) - 1 SUBMITTED Job with:                    => SUBMITTED
 *        job_id      = 5-...-05
 *        channel     = [NULL]                   channel_3
 *        files       = 50000011 (SUBMITTED)     PENDING
 *                      50000012 (W_C_RES)       W_C_RES
 *
 * (7) - 1 SUBMITTED Job with:                    => PENDING
 *        job_id      = 5-...-06
 *        channel     = channel_3                channel_3
 *        files       = 50000013 (PENDING)       PENDING
 *                      50000014 (SUBMITTED)     FAILED (different channel)
 *
 * (8) - 1 SUBMITTED Job with:                    => SUBMITTED  => PENDING
 *        job_id      = 5-...-07
 *        channel     = [NULL]                   [NULL]     => channel_3
 *        files       = 50000015 (SUBMITTED)     SUBMITTED  => PENDING
 *                      50000016 (SUBMITTED)     SUBMITTED  => PENDING
 *
 * (9) - 1 SUBMITTED Job with:                    => SUBMITTED  => PENDING
 *        job_id      = 5-...-08
 *        channel     = [NULL]                   [NULL]     => channel_3
 *        files       = 50000017 (SUBMITTED)     SUBMITTED  => PENDING
 *                      50000018 (SUBMITTED)     SUBMITTED  => PENDING (src name already resolved)
 *
 * (10) - 1 SUBMITTED Job with:                    => SUBMITTED  
 *        job_id      = 5-...-09
 *        channel     = [NULL]                   [NULL]     
 *        files       = 50000019 (SUBMITTED)     SUBMITTED  => FAILED (no share on channel_1)
 *                      50000020 (W_C_RES)       W_C_RES
 *
 * (11) - 1 SUBMITTED Job with:                    => FAILED  
 *        job_id      = 5-...-10
 *        channel     = [NULL]                   [NULL]     
 *        files       = 50000021 (SUBMITTED)     FAILED (skip catalog Resolution)
 *                      50000022 (W_C_RES)       FAILED (skip catalog Resolution)
 *
 * (12) - 1 SUBMITTED Job with:                    => FAILED  
 *        job_id      = 5-...-11
 *        channel     = [NULL]                   [NULL]     
 *        files       = 50000023 (SUBMITTED)     FAILED (this lfn should fail)
 *
 * (13) - 1 SUBMITTED Job with:                    => FAILED  
 *        job_id      = 5-...-12
 *        channel     = [NULL]                   [NULL]     
 *        files       = 50000024 (SUBMITTED)     FAILED (this lfn doesn't exist)
 *
 * (14) - 1 SUBMITTED Job with:                    => FAILED  
 *        job_id      = 5-...-13
 *        channel     = [NULL]                   [NULL]     
 *        files       = 50000025 (SUBMITTED)     FAILED (this surl doesn't exist)
 *
 * (15) - 1 SUBMITTED Job with:                    => CANCELED (job has to be canceled)
 *        job_id      = 5-...-14
 *        channel     = [NULL]                   [NULL]     
 *        files       = 50000026 (SUBMITTED)     CANCELED
 *        files       = 50000027 (SUBMITTED)     CANCELED
 *
 * (16) - 1 SUBMITTED Job with:                    => CANCELED (job has to be canceled)
 *        job_id      = 5-...-15
 *        channel     = [NULL]                   [NULL]     
 *        files       = 50000028 (SUBMITTED)     CANCELED
 *        files       = 50000029 (SUBMITTED)     CANCELED
 *
 * (17) - 1 SUBMITTED Job with:                    => FAILED (channel is off)
 *        job_id      = 5-...-16
 *        channel     = [NULL]                   [NULL]     
 *        files       = 50000030 (SUBMITTED)     FAILED
 *        files       = 50000031 (SUBMITTED)     FAILED
 *
 * (18) - 1 SUBMITTED Job with:                    => PENDING (channel is paused)
 *        job_id      = 5-...-17
 *        channel     = [NULL]                   channel_9
 *        files       = 50000032 (SUBMITTED)     PENDING
 *        files       = 50000033 (SUBMITTED)     PENDING
 *
 * (19) - 1 SUBMITTED Job with:                    => PENDING
 *        job_id      = 5-...-18
 *        channel     = [NULL]                   channel_3
 *        files       = 50000034 (SUBMITTED)     PENDING
 *        files       = 50000035 (SUBMITTED)     FAILED (different SE)
 *
 * (20) - 1 SUBMITTED Job with:                    => FAILED
 *        job_id      = 5-...-19
 *        channel     = [NULL]                   
 *        files       = 50000036 (SUBMITTED)     FAILED (blacklisted SE)
 *        files       = 50000037 (SUBMITTED)     FAILED (blacklisted SE)
 *
 * -----------------------------------------------------------------------------
 * Tests for clouds channels
 *
 * (101) - 1 SUBMITTED Job with:                    => Pending
 *        job_id      = 5-...-101
 *        channel     = [NULL]                   channel_10 (or 12 if no groups are not supported)
 *        files       = 50000101 (SUBMITTED)     Pending
 *        files       = 50000102 (SUBMITTED)     Pending
 *
 * (102) - 1 SUBMITTED Job with:                    => Pending
 *        job_id      = 5-...-102
 *        channel     = [NULL]                   channel_11 (or 12 if no groups are not supported)
 *        files       = 50000103 (SUBMITTED)     Pending
 *        files       = 50000104 (SUBMITTED)     Pending
 *
 * (103) - 1 SUBMITTED Job with:                    => FAILED (no channel between Catania and Ral)
 *        job_id      = 5-...-103
 *        channel     = [NULL]                   
 *        files       = 50000105 (SUBMITTED)     FAILED
 *        files       = 50000106 (SUBMITTED)     FAILED
 * -----------------------------------------------------------------------------
 * Tests for bugs
 * (BUG1) - 1 SUBMITTED Job with:                   => FAILED (different destination hosts)
 *        job_id      = 5-...-100000000001
 *        channel     = [NULL]
 *        files       = 51000001 (SUBMITTED)     FAILED
 *        files       = 51000002 (SUBMITTED)     FAILED
 */
namespace {
const char * const JOB1_ID         = "50000000-0000-0000-0000-000000000000";
const char * const JOB2_ID         = "50000000-0000-0000-0000-000000000001";
const char * const JOB3_ID         = "50000000-0000-0000-0000-000000000002";
const char * const JOB4_ID         = "50000000-0000-0000-0000-000000000003";
const char * const JOB5_ID         = "50000000-0000-0000-0000-000000000004";
const char * const JOB6_ID         = "50000000-0000-0000-0000-000000000005";
const char * const JOB7_ID         = "50000000-0000-0000-0000-000000000006";
const char * const JOB8_ID         = "50000000-0000-0000-0000-000000000007";
const char * const JOB9_ID         = "50000000-0000-0000-0000-000000000008";
const char * const JOB10_ID        = "50000000-0000-0000-0000-000000000009";
const char * const JOB11_ID        = "50000000-0000-0000-0000-000000000010";
const char * const JOB12_ID        = "50000000-0000-0000-0000-000000000011";
const char * const JOB13_ID        = "50000000-0000-0000-0000-000000000012";
const char * const JOB14_ID        = "50000000-0000-0000-0000-000000000013";
const char * const JOB15_ID        = "50000000-0000-0000-0000-000000000014";
const char * const JOB16_ID        = "50000000-0000-0000-0000-000000000015";
const char * const JOB17_ID        = "50000000-0000-0000-0000-000000000016";
const char * const JOB18_ID        = "50000000-0000-0000-0000-000000000017";
const char * const JOB19_ID        = "50000000-0000-0000-0000-000000000018";
const char * const JOB20_ID        = "50000000-0000-0000-0000-000000000019";
const char * const JOB101_ID       = "50000000-0000-0000-0000-000000000100";
const char * const JOB102_ID       = "50000000-0000-0000-0000-000000000101";
const char * const JOB103_ID       = "50000000-0000-0000-0000-000000000102";
const char * const FILE1_ID        = "50000001";
const char * const FILE2_ID        = "50000002";
const char * const FILE3_ID        = "50000003";
const char * const FILE4_ID        = "50000004";
const char * const FILE5_ID        = "50000005";
const char * const FILE6_ID        = "50000006";
const char * const FILE7_ID        = "50000007";
const char * const FILE8_ID        = "50000008";
const char * const FILE9_ID        = "50000009";
const char * const FILE10_ID       = "50000010";
const char * const FILE11_ID       = "50000011";
const char * const FILE12_ID       = "50000012";
const char * const FILE13_ID       = "50000013";
const char * const FILE14_ID       = "50000014";
const char * const FILE15_ID       = "50000015";
const char * const FILE16_ID       = "50000016";
const char * const FILE17_ID       = "50000017";
const char * const FILE18_ID       = "50000018";
const char * const FILE19_ID       = "50000019";
const char * const FILE20_ID       = "50000020";
const char * const FILE21_ID       = "50000021";
const char * const FILE22_ID       = "50000022";
const char * const FILE23_ID       = "50000023";
const char * const FILE24_ID       = "50000024";
const char * const FILE25_ID       = "50000025";
const char * const FILE26_ID       = "50000026";
const char * const FILE27_ID       = "50000027";
const char * const FILE28_ID       = "50000028";
const char * const FILE29_ID       = "50000029";
const char * const FILE30_ID       = "50000030";
const char * const FILE31_ID       = "50000031";
const char * const FILE32_ID       = "50000032";
const char * const FILE33_ID       = "50000033";
const char * const FILE34_ID       = "50000034";
const char * const FILE35_ID       = "50000035";
const char * const FILE36_ID       = "50000036";
const char * const FILE37_ID       = "50000037";
const char * const FILE101_ID      = "50000101";
const char * const FILE102_ID      = "50000102";
const char * const FILE103_ID      = "50000103";
const char * const FILE104_ID      = "50000104";
const char * const FILE105_ID      = "50000105";
const char * const FILE106_ID      = "50000106";
const char * const FILE15_LFN      = "/tmp/src1.txt";
const char * const FILE16_LFN      = "/tmp/src2.txt";
const char * const FILE17_LFN      = "/tmp/src3.txt";
const char * const FILE18_LFN      = "/tmp/src4.txt";
const char * const FILE19_LFN      = "/tmp/src1.txt";
const char * const FILE20_LFN      = "/tmp/src2.txt";
const char * const FILE21_LFN      = "/tmp/src1.txt";
const char * const FILE22_LFN      = "/tmp/src2.txt";
const char * const FILE15_SRC_SURL = "srm://host.cern.ch:8443/srm/managerv1?SFN=/castor/cern.ch/SE/tmp/src1.txt";
const char * const FILE15_DST_SURL = "srm://host.infn.it:8443/srm/managerv1?SFN=/castor/infn.it/EGEE-SE/tmp/src1.txt";
const char * const FILE16_SRC_SURL = "srm://host.cern.ch:8443/srm/managerv1?SFN=/castor/cern.ch/SE/tmp/src2.txt";
const char * const FILE16_DST_SURL = "srm://host.infn.it:8443/srm/managerv1?SFN=/castor/infn.it/EGEE-SE/tmp/src2.txt";
const char * const FILE17_SRC_SURL = "srm://host.cern.ch:8443/tmp/src3.txt";
const char * const FILE17_DST_SURL = "srm://host.infn.it:8443/srm/managerv1?SFN=/castor/infn.it/EGEE-SE/tmp/src3.txt";
const char * const FILE18_SRC_SURL = "srm://host.cern.ch:8443/tmp/src4.txt";
const char * const FILE18_DST_SURL = "srm://host.infn.it:8443/tmp/dst4.txt";
const char * const CHANNEL3_NAME   = "channel_3";
const char * const CHANNEL7_NAME   = "channel_7";
const char * const CHANNEL8_NAME   = "channel_8";
const char * const CHANNEL9_NAME   = "channel_9";
const char * const CHANNEL10_NAME  = "channel_10";
const char * const CHANNEL11_NAME  = "channel_11";
const char * const CHANNEL12_NAME  = "channel_12";
const char * const HOST_CERN       = "host.cern.ch";
const char * const HOST_INFN       = "host.infn.it";
const char * const HOST_IN2P3      = "host.in2p3.fr";
const char * const HOST_DESY       = "host.desy.de";
const char * const HOST_NIKHEF     = "host.nikhef.nl";
const char * const HOST_RAL        = "host.ral.co.uk";
const char * const HOST_BNL        = "host.bnl.gov";
const char * const SITE_CERN       = "TEST.CERN.CH";
const char * const SITE_INFN       = "TEST.INFN.IT";
const char * const SITE_IN2P3      = "TEST.IN3P3.FR";
const char * const SITE_DESY       = "TEST.DESY.DE";
const char * const SITE_NIKHEF     = "TEST.NIKHEF.NL";
const char * const SITE_RAL        = "TEST.RAL.CO.UK";
const char * const SITE_BNL        = "TEST.BNL.GOV";
const char * const HOST_CERN_1     = "host1.cern.ch";
const char * const HOST_INFN_1     = "host1.infn.it";
const char * const HOST_CATANIA_1  = "host1.catania.infn.it";
const char * const SITE_CERN_1     = "SITE.CERN.CH";
const char * const SITE_INFN_1     = "SITE.INFN.IT";
const char * const SITE_CATANIA_1  = "SITE.CATANIA.IT";
const char * const SITE_RAL_1      = "SITE.RAL.AC.UK";
const char * const VO1_NAME        = "ALLOCATE_VO_1";
const char * const VO2_NAME        = "ALLOCATE_VO_2";
// BUGS
const char * const VO_BUGS_NAME    = "ALLOCATE_VO_BUGS";
const char * const JOBBUG1_ID      = "50000000-0000-0000-0000-100000000001";
const char * const FILEBUG1_ID     = "51000001";
const char * const FILEBUG2_ID     = "51000002";
}

extern std::string g_catalog_endpoint;

/*
 * setUp
 *
 * set-up test case
 */
void AllocateTestCase::setUp(){

    // Reset cache
    ActionsConfig::instance().channelCache().clear();
    // Reset Action
    m_action.reset();

}

/*
 * setUp
 *
 * set-up test case
 */
void AllocateTestCase::setUp_forTest1(){
    
    static bool replica_registered = false;

    // Create Action instance
    std::auto_ptr<Action> a(ActionFactory::instance().create("glite:Allocate"));
    CPPUNIT_ASSERT_MESSAGE("Null Action", a.get() != 0);
    CPPUNIT_ASSERT_MESSAGE("Invalid Action Class", dynamic_cast<Allocate*>(a.get()) != 0);

    // Store Action        
    m_action.reset(dynamic_cast<Allocate*>(a.release()));
    CPPUNIT_ASSERT_MESSAGE("Null Action", m_action.get() != 0);
    
    if(false == replica_registered){
        // Get Catalog Type
        std::string catalog_type = CatalogServiceFactory::instance().globalCatalogServiceType();
        if(catalog_type.empty()){
            catalog_type = CatalogServiceFactory::instance().localCatalogServiceType();
            if(catalog_type.empty()){
                CPPUNIT_FAIL("No Catalog Type Specified");
            }
        }
        // Create CatalogService Instance
        scoped_ptr<CatalogService> catalog(CatalogServiceFactory::instance().create(g_catalog_endpoint,catalog_type));
        CPPUNIT_ASSERT_MESSAGE("Null CatalogService", catalog.get() != 0);

        // Register Source Replicas for the Test
        CatalogService::ReplicaStatArray names;
        names.push_back(CatalogService::ReplicaStat(FILE15_LFN,FILE15_SRC_SURL));
        names.push_back(CatalogService::ReplicaStat(FILE16_LFN,FILE16_SRC_SURL));
        names.push_back(CatalogService::ReplicaStat(FILE17_LFN,FILE17_SRC_SURL));
        names.push_back(CatalogService::ReplicaStat(FILE18_LFN,FILE18_SRC_SURL));
        CatalogService::CatalogResult result;
        catalog->registerSurls(names,result);
        CPPUNIT_ASSERT_MESSAGE("registerSurls failed", result.status == CatalogService::CatalogResult::R_SUCCESS);
        replica_registered = true;
    }

}

/*
 * setUp
 *
 * set-up test case
 */
void AllocateTestCase::setUp_forTest2(){

    // Create Action instance
    std::auto_ptr<Action> a(ActionFactory::instance().create("glite:Allocate"));
    CPPUNIT_ASSERT_MESSAGE("Null Action", a.get() != 0);
    CPPUNIT_ASSERT_MESSAGE("Invalid Action Class", dynamic_cast<Allocate*>(a.get()) != 0);

    // Store Action        
    m_action.reset(dynamic_cast<Allocate*>(a.release()));
    CPPUNIT_ASSERT_MESSAGE("Null Action", m_action.get() != 0);

}

/*
 * tearDown
 *
 * tear-down test case
 */
void AllocateTestCase::tearDown(){
    ActionsConfig::instance().channelCache().clear();
}

// test for bug 31099
void AllocateTestCase::test_bug31099(){
    // bug description:
    // When a job with multiple files is submitted, if the SURLs do not belong 
    // to same couples of source and destination endpoints, the channel 
    // allocation correctly fails but the error message for the file is set to 
    // 'Channel Allocation failed but no reason is available'. 

    // Create Action instance
    std::auto_ptr<Action> a(ActionFactory::instance().create("glite:Allocate"));
    CPPUNIT_ASSERT_MESSAGE("Null Action", a.get() != 0);
    CPPUNIT_ASSERT_MESSAGE("Invalid Action Class", dynamic_cast<Allocate*>(a.get()) != 0);

    // Store Action        
    m_action.reset(dynamic_cast<Allocate*>(a.release()));
    CPPUNIT_ASSERT_MESSAGE("Null Action", m_action.get() != 0);
    // Set Context
    m_action->context(VO_BUGS_NAME,DaoHelper::ctx());
    // Execute Actions
    m_action->execute();
    // Create FileDAO instance
    scoped_ptr<dao::vo::FileDAO> fileDAO(dao::vo::DAOFactory::instance().createFileDAO(VO_BUGS_NAME, DaoHelper::ctx()));
    CPPUNIT_ASSERT_MESSAGE("Null FileDAO", fileDAO.get() != 0);
    // Create JobDAO instance
    scoped_ptr<dao::vo::JobDAO> jobDAO(dao::vo::DAOFactory::instance().createJobDAO(VO_BUGS_NAME, DaoHelper::ctx()));
    CPPUNIT_ASSERT_MESSAGE("Null JobDAO", jobDAO.get() != 0);
    shared_ptr<Job>     job;
    shared_ptr<File>    file;

    //----------------------------------------------------------------------
    // Job 1
    //----------------------------------------------------------------------
    
    // Get File instance (1) and Check it
    file = check_file(fileDAO,FILEBUG1_ID,"FILEBUG1",File::S_FAILED);
    check_error(file,"FILEBUG1",Error::CONSISTENCY,Error::S_AGENT,Error::P_ALLOCATION);
    // Get File instance (2) and Check it
    file = check_file(fileDAO,FILEBUG2_ID,"FILEBUG2",File::S_FAILED);
    check_error(file,"FILEBUG2",Error::CONSISTENCY,Error::S_AGENT,Error::P_ALLOCATION);
    
    // Get Job instance (1) and Check it
    job = check_job(jobDAO,JOBBUG1_ID,"JOBBUG1",Job::S_FAILED);
    // check that the channel has been allocated anyway ?
    CPPUNIT_ASSERT_MESSAGE("Invalid Channel (JOBBUG1)", job->channelName == CHANNEL3_NAME);
    CPPUNIT_ASSERT_MESSAGE("Invalid Source Site (JOBBUG1)", job->source == SITE_CERN);
    CPPUNIT_ASSERT_MESSAGE("Invalid Dest Site (JOBBUG1)", job->dest == SITE_INFN);
    CPPUNIT_ASSERT_MESSAGE("Invalid Source SE (JOBBUG1)", job->sourceSE == HOST_CERN);
    CPPUNIT_ASSERT_MESSAGE("Invalid Dest SE (JOBBUG1)", job->destSE == HOST_INFN);

} 

/** 
 * Test Execute Method
 * parameter 'allocate in different cycle' set
 */
void AllocateTestCase::testExecute(){

    setUp_forTest1();
    
    // Set Context
    m_action->context(VO1_NAME,DaoHelper::ctx());
    
    // Execute Actions
    m_action->execute(JOB_PARAM_ALLOCATE_IN_DIFFERENT_CYCLE);
    
    // Create FileDAO instance
    scoped_ptr<dao::vo::FileDAO> fileDAO(dao::vo::DAOFactory::instance().createFileDAO(VO1_NAME, DaoHelper::ctx()));
    CPPUNIT_ASSERT_MESSAGE("Null FileDAO", fileDAO.get() != 0);
    
    // Create JobDAO instance
    scoped_ptr<dao::vo::JobDAO> jobDAO(dao::vo::DAOFactory::instance().createJobDAO(VO1_NAME, DaoHelper::ctx()));
    CPPUNIT_ASSERT_MESSAGE("Null JobDAO", jobDAO.get() != 0);
    
    shared_ptr<Job>     job;
    shared_ptr<File>    file;
    
    //----------------------------------------------------------------------
    // Job 1
    //----------------------------------------------------------------------
    
    // Get File instance (1) and Check it
    file = check_file(fileDAO,FILE1_ID,"FILE1",File::S_PENDING);
    check_error(file,"FILE1");
    // Get File instance (2) and Check it
    file = check_file(fileDAO,FILE2_ID,"FILE2",File::S_PENDING);
    check_error(file,"FILE2");
    
    // Get Job instance (1) and Check it
    job = check_job(jobDAO,JOB1_ID,"JOB1",Job::S_PENDING);
    CPPUNIT_ASSERT_MESSAGE("Invalid Channel (JOB1)", job->channelName == CHANNEL3_NAME);
    CPPUNIT_ASSERT_MESSAGE("Invalid Source Site (JOB1)", job->source == SITE_CERN);
    CPPUNIT_ASSERT_MESSAGE("Invalid Dest Site (JOB1)", job->dest == SITE_INFN);
    CPPUNIT_ASSERT_MESSAGE("Invalid Source SE (JOB1)", job->sourceSE == HOST_CERN);
    CPPUNIT_ASSERT_MESSAGE("Invalid Dest SE (JOB1)", job->destSE == HOST_INFN);
    
    //----------------------------------------------------------------------
    // Job 2
    //----------------------------------------------------------------------
    
    // Get File instance (3) and Check it
    file = check_file(fileDAO,FILE3_ID,"FILE3",File::S_FAILED);
    check_error(file,"FILE3",Error::CONSISTENCY,Error::S_AGENT,Error::P_ALLOCATION);
    // Get File instance (4) and Check it
    file = check_file(fileDAO,FILE4_ID,"FILE4",File::S_FAILED);
    check_error(file,"FILE4",Error::CONSISTENCY,Error::S_AGENT,Error::P_ALLOCATION);

    // Get Job instance (2) and Check it
    job = check_job(jobDAO,JOB2_ID,"JOB2",Job::S_FAILED);
    CPPUNIT_ASSERT_MESSAGE("Invalid Channel (JOB2)", job->channelName == CHANNEL3_NAME);
    CPPUNIT_ASSERT_MESSAGE("Invalid Source Site (JOB2)", job->source == SITE_CERN);
    CPPUNIT_ASSERT_MESSAGE("Invalid Dest Site (JOB2)", job->dest == SITE_INFN);
    CPPUNIT_ASSERT_MESSAGE("Invalid Source SE (JOB2)", job->sourceSE == HOST_CERN);
    CPPUNIT_ASSERT_MESSAGE("Invalid Dest SE (JOB2)", job->destSE == HOST_INFN);

    //----------------------------------------------------------------------
    // Job 3
    //----------------------------------------------------------------------
    
    // Get File instance (5) and Check it
    file = check_file(fileDAO,FILE5_ID,"FILE5",File::S_FAILED);
    check_error(file,"FILE5",Error::CONFIGURATION,Error::S_AGENT,Error::P_ALLOCATION);
    // Get File instance (6) and Check it
    file = check_file(fileDAO,FILE6_ID,"FILE6",File::S_FAILED);
    check_error(file,"FILE6",Error::CONFIGURATION,Error::S_AGENT,Error::P_ALLOCATION);

    // Get Job instance (3) and Check it
    job = check_job(jobDAO,JOB3_ID,"JOB3",Job::S_FAILED);

    //----------------------------------------------------------------------
    // Job 4
    //----------------------------------------------------------------------
    
    // Get File instance (7) and Check it
    file = check_file(fileDAO,FILE7_ID,"FILE7",File::S_PENDING);
    check_error(file,"FILE7");
    // Get File instance (8) and Check it
    file = check_file(fileDAO,FILE8_ID,"FILE8",File::S_PENDING);
    check_error(file,"FILE8");

    // Get Job instance (4) and Check it
    job = check_job(jobDAO,JOB4_ID,"JOB4",Job::S_PENDING);
    CPPUNIT_ASSERT_MESSAGE("Invalid Channel (JOB4)", job->channelName == CHANNEL7_NAME);
    CPPUNIT_ASSERT_MESSAGE("Invalid Source Site (JOB4)", job->source == SITE_CERN);
    CPPUNIT_ASSERT_MESSAGE("Invalid Dest Site (JOB4)", job->dest == SITE_DESY);
    CPPUNIT_ASSERT_MESSAGE("Invalid Source SE (JOB4)", job->sourceSE == HOST_CERN);
    CPPUNIT_ASSERT_MESSAGE("Invalid Dest SE (JOB4)", job->destSE == HOST_DESY);

    //----------------------------------------------------------------------
    // Job 5
    //----------------------------------------------------------------------
    
    // Get File instance (9) and Check it
    file = check_file(fileDAO,FILE9_ID,"FILE9",File::S_FAILED);
    check_error(file,"FILE9",Error::CONFIGURATION,Error::S_AGENT,Error::P_ALLOCATION);
    // Get File instance (10) and Check it
    file = check_file(fileDAO,FILE10_ID,"FILE10",File::S_FAILED);
    check_error(file,"FILE10",Error::CONFIGURATION,Error::S_AGENT,Error::P_ALLOCATION);

    // Get Job instance (5) and Check it
    job = check_job(jobDAO,JOB5_ID,"JOB5",Job::S_FAILED);
    
    //----------------------------------------------------------------------
    // Job 6
    //----------------------------------------------------------------------
    
    // Get File instance (11) and Check it
    file = check_file(fileDAO,FILE11_ID,"FILE11",File::S_PENDING);
    check_error(file,"FILE11");
    // Get File instance (12) and Check it
    file = check_file(fileDAO,FILE12_ID,"FILE12",File::S_WAITING_CATALOG_RESOLUTION);
    check_error(file,"FILE12");
    
    // Get Job instance (6) and Check it
    job = check_job(jobDAO,JOB6_ID,"JOB6",Job::S_SUBMITTED);
    CPPUNIT_ASSERT_MESSAGE("Invalid Channel (JOB6)", job->channelName == CHANNEL3_NAME);

    //----------------------------------------------------------------------
    // Job 7
    //----------------------------------------------------------------------
    
    // Get File instance (13) and Check it
    file = check_file(fileDAO,FILE13_ID,"FILE13",File::S_FAILED);
    check_error(file,"FILE13",Error::CONSISTENCY,Error::S_AGENT,Error::P_ALLOCATION);
    // Get File instance (14) and Check it
    file = check_file(fileDAO,FILE14_ID,"FILE14",File::S_FAILED);
    check_error(file,"FILE4",Error::CONSISTENCY,Error::S_AGENT,Error::P_ALLOCATION);

    // Get Job instance (7) and Check it
    job = check_job(jobDAO,JOB7_ID,"JOB7",Job::S_FAILED);
    CPPUNIT_ASSERT_MESSAGE("Invalid Channel (JOB7)", job->channelName == CHANNEL3_NAME);

    //----------------------------------------------------------------------
    // Job 8
    //----------------------------------------------------------------------
    
    // Get File instance (15) and Check it
    file = check_file(fileDAO,FILE15_ID,"FILE15",File::S_SUBMITTED);
    CPPUNIT_ASSERT_MESSAGE("Invalid src surl (FILE15)", file->sourceSurl == FILE15_SRC_SURL);
    CPPUNIT_ASSERT_MESSAGE("Invalid dst surl (FILE15)", file->destSurl == FILE15_DST_SURL);
    check_error(file,"FILE15");
    // Get File instance (16) and Check it
    file = check_file(fileDAO,FILE16_ID,"FILE16",File::S_SUBMITTED);
    CPPUNIT_ASSERT_MESSAGE("Invalid src surl (FILE16)", file->sourceSurl() == FILE16_SRC_SURL);
    CPPUNIT_ASSERT_MESSAGE("Invalid dst surl (FILE16)", file->destSurl() == FILE16_DST_SURL);
    check_error(file,"FILE16");
    
    // Get Job instance (8) and Check it
    job = check_job(jobDAO,JOB8_ID,"JOB8",Job::S_SUBMITTED);
    CPPUNIT_ASSERT_MESSAGE("Invalid Channel (JOB8)", job->channelName().empty() == true);

    //----------------------------------------------------------------------
    // Job 9
    //----------------------------------------------------------------------
    
    // Get File instance (17) and Check it
    file = check_file(fileDAO,FILE17_ID,"FILE17",File::S_SUBMITTED);
    CPPUNIT_ASSERT_MESSAGE("Invalid src surl (FILE17)", file->sourceSurl == FILE17_SRC_SURL);
    CPPUNIT_ASSERT_MESSAGE("Invalid dst surl (FILE17)", file->destSurl == FILE17_DST_SURL);
    check_error(file,"FILE17");
    // Get File instance (18) and Check it
    file = check_file(fileDAO,FILE18_ID,"FILE18",File::S_SUBMITTED);
    CPPUNIT_ASSERT_MESSAGE("Invalid src surl (FILE18)", file->sourceSurl == FILE18_SRC_SURL);
    CPPUNIT_ASSERT_MESSAGE("Invalid dst surl (FILE18)", file->destSurl == FILE18_DST_SURL);
    check_error(file,"FILE18");

    // Get Job instance (9) and Check it
    job = check_job(jobDAO,JOB9_ID,"JOB9",Job::S_SUBMITTED);
    CPPUNIT_ASSERT_MESSAGE("Invalid Channel (JOB9)", job->channelName().empty() == true);

    //----------------------------------------------------------------------
    // Job 10
    //----------------------------------------------------------------------
    
    // Get File instance (19) and Check it
    file = check_file(fileDAO,FILE19_ID,"FILE19",File::S_SUBMITTED);
    CPPUNIT_ASSERT_MESSAGE("Invalid src surl (FILE19)", file->sourceSurl().empty() == false);
    CPPUNIT_ASSERT_MESSAGE("Invalid dst surl (FILE19)", file->destSurl().empty() == false);
    check_error(file,"FILE19");
    // Get File instance (20) and Check it
    file = check_file(fileDAO,FILE20_ID,"FILE20",File::S_WAITING_CATALOG_RESOLUTION);
    CPPUNIT_ASSERT_MESSAGE("Invalid src surl (FILE20)", file->sourceSurl().empty() == true);
    CPPUNIT_ASSERT_MESSAGE("Invalid dst surl (FILE20)", file->destSurl().empty() == true);
    check_error(file,"FILE20");

    // Get Job instance (10) and Check it
    job = check_job(jobDAO,JOB10_ID,"JOB10",Job::S_SUBMITTED);
    CPPUNIT_ASSERT_MESSAGE("Invalid Channel (JOB10)", job->channelName().empty() == true);

    //----------------------------------------------------------------------
    // Job 11
    //----------------------------------------------------------------------
    
    // Get File instance (21) and Check it
    file = check_file(fileDAO,FILE21_ID,"FILE21",File::S_FAILED);
    CPPUNIT_ASSERT_MESSAGE("Invalid src surl (FILE21)", file->sourceSurl().empty() == true);
    CPPUNIT_ASSERT_MESSAGE("Invalid dst surl (FILE21)", file->destSurl().empty() == true);
    check_error(file,"FILE21",Error::CONFIGURATION,Error::S_AGENT,Error::P_CATALOG_RESOLUTION);
    // Get File instance (22) and Check it
    file = check_file(fileDAO,FILE22_ID,"FILE22",File::S_FAILED);
    CPPUNIT_ASSERT_MESSAGE("Invalid src surl (FILE22)", file->sourceSurl().empty() == true);
    CPPUNIT_ASSERT_MESSAGE("Invalid dst surl (FILE22)", file->destSurl().empty() == true);
    check_error(file,"FILE22",Error::CONFIGURATION,Error::S_AGENT,Error::P_CATALOG_RESOLUTION);

    // Get Job instance (11) and Check it
    job = check_job(jobDAO,JOB11_ID,"JOB11",Job::S_FAILED);
    CPPUNIT_ASSERT_MESSAGE("Invalid Channel (JOB11)", job->channelName().empty() == true);

    //----------------------------------------------------------------------
    // Job 12
    //----------------------------------------------------------------------
    
    // Get File instance (23) and Check it
    file = check_file(fileDAO,FILE23_ID,"FILE23",File::S_FAILED);
    CPPUNIT_ASSERT_MESSAGE("Invalid src surl (FILE23)", file->sourceSurl().empty() == true);
    check_error(file,"FILE23",Error::SECURITY,Error::S_SOURCE,Error::P_CATALOG_RESOLUTION);
    
    // Get Job instance (12) and Check it
    job = check_job(jobDAO,JOB12_ID,"JOB12",Job::S_FAILED);
    CPPUNIT_ASSERT_MESSAGE("Invalid Channel (JOB12)", job->channelName().empty() == true);

    //----------------------------------------------------------------------
    // Job 13
    //----------------------------------------------------------------------
    
    // Get File instance (24) and Check it
    file = check_file(fileDAO,FILE24_ID,"FILE24",File::S_FAILED);
    CPPUNIT_ASSERT_MESSAGE("Invalid src surl (FILE24)", file->sourceSurl().empty() == true);
    check_error(file,"FILE24",Error::SECURITY,Error::S_SOURCE,Error::P_CATALOG_RESOLUTION);

    // Get Job instance (13) and Check it
    job = check_job(jobDAO,JOB13_ID,"JOB13",Job::S_FAILED);
    CPPUNIT_ASSERT_MESSAGE("Invalid Channel (JOB13)", job->channelName().empty() == true);

    //----------------------------------------------------------------------
    // Job 14
    //----------------------------------------------------------------------
    
    // Get File instance (25) and Check it
    file = check_file(fileDAO,FILE25_ID,"FILE25",File::S_FAILED);
    CPPUNIT_ASSERT_MESSAGE("Invalid src surl (FILE25)", file->sourceSurl().empty() == true);
    check_error(file,"FILE25",Error::INVALID_PATH,Error::S_SOURCE,Error::P_CATALOG_RESOLUTION);

    // Get Job instance (14) and Check it
    job = check_job(jobDAO,JOB14_ID,"JOB14",Job::S_FAILED);
    CPPUNIT_ASSERT_MESSAGE("Invalid Channel (JOB14)", job->channelName().empty() == true);

    //----------------------------------------------------------------------
    // Job 15
    //----------------------------------------------------------------------
    
    // Get File instance (26) and Check it
    file = check_file(fileDAO,FILE26_ID,"FILE26",File::S_CANCELED);
    check_error(file,"FILE26");
    // Get File instance (27) and Check it
    file = check_file(fileDAO,FILE27_ID,"FILE27",File::S_CANCELED);
    check_error(file,"FILE27");

    // Get Job instance (15) and Check it
    job = check_job(jobDAO,JOB15_ID,"JOB15",Job::S_CANCELED);
    CPPUNIT_ASSERT_MESSAGE("Invalid Channel (JOB15)", job->channelName().empty() == true);

    //----------------------------------------------------------------------
    // Job 16
    //----------------------------------------------------------------------
    
    // Get File instance (28) and Check it
    file = check_file(fileDAO,FILE28_ID,"FILE28",File::S_CANCELED);
    CPPUNIT_ASSERT_MESSAGE("Invalid src surl (FILE28)", file->sourceSurl().empty() == true);
    CPPUNIT_ASSERT_MESSAGE("Invalid dst surl (FILE28)", file->destSurl().empty() == true);
    check_error(file,"FILE28");
    // Get File instance (29) and Check it
    file = check_file(fileDAO,FILE29_ID,"FILE29",File::S_CANCELED);
    CPPUNIT_ASSERT_MESSAGE("Invalid src surl (FILE29)", file->sourceSurl().empty() == true);
    CPPUNIT_ASSERT_MESSAGE("Invalid dst surl (FILE29)", file->destSurl().empty() == true);
    check_error(file,"FILE29");

    // Get Job instance (16) and Check it
    job = check_job(jobDAO,JOB16_ID,"JOB16",Job::S_CANCELED);
    CPPUNIT_ASSERT_MESSAGE("Invalid Channel (JOB16)", job->channelName().empty() == true);

    //----------------------------------------------------------------------
    // Job 17
    //----------------------------------------------------------------------
    
    // Get File instance (30) and Check it
    file = check_file(fileDAO,FILE30_ID,"FILE30",File::S_FAILED);
    check_error(file,"FILE30",Error::CONFIGURATION,Error::S_AGENT,Error::P_ALLOCATION);
    // Get File instance (31) and Check it
    file = check_file(fileDAO,FILE31_ID,"FILE31",File::S_FAILED);
    check_error(file,"FILE31",Error::CONFIGURATION,Error::S_AGENT,Error::P_ALLOCATION);

    // Get Job instance (17) and Check it
    job = check_job(jobDAO,JOB17_ID,"JOB17",Job::S_FAILED);
    CPPUNIT_ASSERT_MESSAGE("Invalid Channel (JOB17)", job->channelName().empty() == true);

    //----------------------------------------------------------------------
    // Job 18
    //----------------------------------------------------------------------
    
    // Get File instance (32) and Check it
    file = check_file(fileDAO,FILE32_ID,"FILE32",File::S_PENDING);
    check_error(file,"FILE32");
    // Get File instance (33) and Check it
    file = check_file(fileDAO,FILE33_ID,"FILE33",File::S_PENDING);
    check_error(file,"FILE33");

    // Get Job instance (18) and Check it
    job = check_job(jobDAO,JOB18_ID,"JOB18",Job::S_PENDING);
    CPPUNIT_ASSERT_MESSAGE("Invalid Channel (JOB18)", job->channelName() == CHANNEL9_NAME);
    CPPUNIT_ASSERT_MESSAGE("Invalid Source Site (JOB18)", job->source == SITE_CERN);
    CPPUNIT_ASSERT_MESSAGE("Invalid Dest Site (JOB18)", job->dest == SITE_BNL);
    CPPUNIT_ASSERT_MESSAGE("Invalid Source SE (JOB18)", job->sourceSE == HOST_CERN);
    CPPUNIT_ASSERT_MESSAGE("Invalid Dest SE (JOB18)", job->destSE == HOST_BNL);

    //----------------------------------------------------------------------
    // Job 19
    //----------------------------------------------------------------------
    
    // Get File instance (34) and Check it
    file = check_file(fileDAO,FILE34_ID,"FILE34",File::S_FAILED);
    check_error(file,"FILE34",Error::CONSISTENCY,Error::S_AGENT,Error::P_ALLOCATION);
    // Get File instance (35) and Check it
    file = check_file(fileDAO,FILE35_ID,"FILE35",File::S_FAILED);
    check_error(file,"FILE35",Error::CONSISTENCY,Error::S_AGENT,Error::P_ALLOCATION);

    // Get Job instance (19) and Check it
    job = check_job(jobDAO,JOB19_ID,"JOB19",Job::S_FAILED);
    CPPUNIT_ASSERT_MESSAGE("Invalid Channel (JOB19)", job->channelName() == CHANNEL3_NAME);
    CPPUNIT_ASSERT_MESSAGE("Invalid Source Site (JOB19)", job->source == SITE_CERN);
    CPPUNIT_ASSERT_MESSAGE("Invalid Dest Site (JOB19)", job->dest == SITE_INFN);
    CPPUNIT_ASSERT_MESSAGE("Invalid Source SE (JOB19)", job->sourceSE == HOST_CERN);
    CPPUNIT_ASSERT_MESSAGE("Invalid Dest SE (JOB19)", job->destSE == HOST_INFN);

    //----------------------------------------------------------------------
    // Job 20
    //----------------------------------------------------------------------
    
    // Get File instance (36) and Check it
    file = check_file(fileDAO,FILE36_ID,"FILE36",File::S_FAILED);
    check_error(file,"FILE36",Error::CONFIGURATION,Error::S_AGENT,Error::P_ALLOCATION);
    // Get File instance (37) and Check it
    file = check_file(fileDAO,FILE37_ID,"FILE37",File::S_FAILED);
    check_error(file,"FILE37",Error::CONFIGURATION,Error::S_AGENT,Error::P_ALLOCATION);

    // Get Job instance (20) and Check it
    job = check_job(jobDAO,JOB20_ID,"JOB20",Job::S_FAILED);
    CPPUNIT_ASSERT_MESSAGE("Invalid Channel (JOB20)", job->channelName().empty() == true);

}

/**
 * Test Execute Method
 * parameter 'allocate in different cycle' NOT set
 */
void AllocateTestCase::testExecute_1(){

    setUp_forTest1();
    
    // Set Context
    m_action->context(VO1_NAME,DaoHelper::ctx());
    
    // Execute Actions
    m_action->execute();
    
    // Create FileDAO instance
    scoped_ptr<dao::vo::FileDAO> fileDAO(dao::vo::DAOFactory::instance().createFileDAO(VO1_NAME, DaoHelper::ctx()));
    CPPUNIT_ASSERT_MESSAGE("Null FileDAO", fileDAO.get() != 0);
    
    // Create JobDAO instance
    scoped_ptr<dao::vo::JobDAO> jobDAO(dao::vo::DAOFactory::instance().createJobDAO(VO1_NAME, DaoHelper::ctx()));
    CPPUNIT_ASSERT_MESSAGE("Null JobDAO", jobDAO.get() != 0);

    shared_ptr<Job>     job;
    shared_ptr<File>    file;
    
    //----------------------------------------------------------------------
    // Job 1
    //----------------------------------------------------------------------

    // Get Job instance (1) and Check it
    job = check_job(jobDAO,JOB1_ID,"JOB1",Job::S_PENDING);
    CPPUNIT_ASSERT_MESSAGE("Invalid Channel (JOB1)", job->channelName == CHANNEL3_NAME);
    CPPUNIT_ASSERT_MESSAGE("Invalid Source Site (JOB1)", job->source == SITE_CERN);
    CPPUNIT_ASSERT_MESSAGE("Invalid Dest Site (JOB1)", job->dest == SITE_INFN);
    CPPUNIT_ASSERT_MESSAGE("Invalid Source SE (JOB1)", job->sourceSE == HOST_CERN);
    CPPUNIT_ASSERT_MESSAGE("Invalid Dest SE (JOB1)", job->destSE == HOST_INFN);

    // Get File instance (1) and Check it
    file = check_file(fileDAO,FILE1_ID,"FILE1",File::S_PENDING);
    check_error(file,"FILE1");
    // Get File instance (2) and Check it
    file = check_file(fileDAO,FILE2_ID,"FILE2",File::S_PENDING);
    check_error(file,"FILE2");
    
    //----------------------------------------------------------------------
    // Job 2
    //----------------------------------------------------------------------
    
    // Get Job instance (2) and Check it
    job = check_job(jobDAO,JOB2_ID,"JOB2",Job::S_FAILED);
    CPPUNIT_ASSERT_MESSAGE("Invalid Channel (JOB2)", job->channelName == CHANNEL3_NAME);
    CPPUNIT_ASSERT_MESSAGE("Invalid Source Site (JOB2)", job->source == SITE_CERN);
    CPPUNIT_ASSERT_MESSAGE("Invalid Dest Site (JOB2)", job->dest == SITE_INFN);
    CPPUNIT_ASSERT_MESSAGE("Invalid Source SE (JOB2)", job->sourceSE == HOST_CERN);
    CPPUNIT_ASSERT_MESSAGE("Invalid Dest SE (JOB2)", job->destSE == HOST_INFN);

    // Get File instance (3) and Check it
    file = check_file(fileDAO,FILE3_ID,"FILE3",File::S_FAILED);
    check_error(file,"FILE3",Error::CONSISTENCY,Error::S_AGENT,Error::P_ALLOCATION);
    // Get File instance (4) and Check it
    file = check_file(fileDAO,FILE4_ID,"FILE4",File::S_FAILED);
    check_error(file,"FILE4",Error::CONSISTENCY,Error::S_AGENT,Error::P_ALLOCATION);

    //----------------------------------------------------------------------
    // Job 3
    //----------------------------------------------------------------------
    
    // Get Job instance (3) and Check it
    job = check_job(jobDAO,JOB3_ID,"JOB3",Job::S_FAILED);

    // Get File instance (5) and Check it
    file = check_file(fileDAO,FILE5_ID,"FILE5",File::S_FAILED);
    check_error(file,"FILE5",Error::CONFIGURATION,Error::S_AGENT,Error::P_ALLOCATION);
    // Get File instance (6) and Check it
    file = check_file(fileDAO,FILE6_ID,"FILE6",File::S_FAILED);
    check_error(file,"FILE6",Error::CONFIGURATION,Error::S_AGENT,Error::P_ALLOCATION);

    //----------------------------------------------------------------------
    // Job 4
    //----------------------------------------------------------------------
    
    // Get Job instance (4) and Check it
    job = check_job(jobDAO,JOB4_ID,"JOB4",Job::S_PENDING);
    CPPUNIT_ASSERT_MESSAGE("Invalid Channel (JOB4)", job->channelName == CHANNEL7_NAME);
    CPPUNIT_ASSERT_MESSAGE("Invalid Source Site (JOB4)", job->source == SITE_CERN);
    CPPUNIT_ASSERT_MESSAGE("Invalid Dest Site (JOB4)", job->dest == SITE_DESY);
    CPPUNIT_ASSERT_MESSAGE("Invalid Source SE (JOB4)", job->sourceSE == HOST_CERN);
    CPPUNIT_ASSERT_MESSAGE("Invalid Dest SE (JOB4)", job->destSE == HOST_DESY);

    // Get File instance (7) and Check it
    file = check_file(fileDAO,FILE7_ID,"FILE7",File::S_PENDING);
    check_error(file,"FILE7");
    // Get File instance (8) and Check it
    file = check_file(fileDAO,FILE8_ID,"FILE8",File::S_PENDING);
    check_error(file,"FILE8");

    //----------------------------------------------------------------------
    // Job 5
    //----------------------------------------------------------------------
    
    // Get Job instance (5) and Check it
    job = check_job(jobDAO,JOB5_ID,"JOB5",Job::S_FAILED);
    
    // Get File instance (9) and Check it
    file = check_file(fileDAO,FILE9_ID,"FILE9",File::S_FAILED);
    check_error(file,"FILE9",Error::CONFIGURATION,Error::S_AGENT,Error::P_ALLOCATION);
    // Get File instance (10) and Check it
    file = check_file(fileDAO,FILE10_ID,"FILE10",File::S_FAILED);
    check_error(file,"FILE10",Error::CONFIGURATION,Error::S_AGENT,Error::P_ALLOCATION);

    //----------------------------------------------------------------------
    // Job 6
    //----------------------------------------------------------------------
    
    // Get Job instance (6) and Check it
    job = check_job(jobDAO,JOB6_ID,"JOB6",Job::S_SUBMITTED);
    CPPUNIT_ASSERT_MESSAGE("Invalid Channel (JOB6)", job->channelName == CHANNEL3_NAME);

    // Get File instance (11) and Check it
    file = check_file(fileDAO,FILE11_ID,"FILE11",File::S_PENDING);
    check_error(file,"FILE11");
    // Get File instance (12) and Check it
    file = check_file(fileDAO,FILE12_ID,"FILE12",File::S_WAITING_CATALOG_RESOLUTION);
    check_error(file,"FILE12");

    //----------------------------------------------------------------------
    // Job 7
    //----------------------------------------------------------------------
    
    // Get Job instance (7) and Check it
    job = check_job(jobDAO,JOB7_ID,"JOB7",Job::S_FAILED);
    CPPUNIT_ASSERT_MESSAGE("Invalid Channel (JOB7)", job->channelName == CHANNEL3_NAME);

    // Get File instance (13) and Check it
    file = check_file(fileDAO,FILE13_ID,"FILE13",File::S_FAILED);
    check_error(file,"FILE13",Error::CONSISTENCY,Error::S_AGENT,Error::P_ALLOCATION);
    // Get File instance (14) and Check it
    file = check_file(fileDAO,FILE14_ID,"FILE14",File::S_FAILED);
    check_error(file,"FILE14",Error::CONSISTENCY,Error::S_AGENT,Error::P_ALLOCATION);

    //----------------------------------------------------------------------
    // Job 8
    //----------------------------------------------------------------------
    
    // Get Job instance (8) and Check it
    job = check_job(jobDAO,JOB8_ID,"JOB8",Job::S_PENDING);
    CPPUNIT_ASSERT_MESSAGE("Invalid Channel (JOB8)", job->channelName().empty() == false);

    // Get File instance (15) and Check it
    file = check_file(fileDAO,FILE15_ID,"FILE15",File::S_PENDING);
    CPPUNIT_ASSERT_MESSAGE("Invalid src surl (FILE15)", file->sourceSurl == FILE15_SRC_SURL);
    CPPUNIT_ASSERT_MESSAGE("Invalid dst surl (FILE15)", file->destSurl == FILE15_DST_SURL);
    check_error(file,"FILE15");
    // Get File instance (16) and Check it
    file = check_file(fileDAO,FILE16_ID,"FILE16",File::S_PENDING);
    CPPUNIT_ASSERT_MESSAGE("Invalid src surl (FILE16)", file->sourceSurl() == FILE16_SRC_SURL);
    CPPUNIT_ASSERT_MESSAGE("Invalid dst surl (FILE16)", file->destSurl() == FILE16_DST_SURL);
    check_error(file,"FILE16");

    //----------------------------------------------------------------------
    // Job 9
    //----------------------------------------------------------------------
    
    // Get Job instance (9) and Check it
    job = check_job(jobDAO,JOB9_ID,"JOB9",Job::S_PENDING);
    CPPUNIT_ASSERT_MESSAGE("Invalid Channel (JOB9)", job->channelName().empty() == false);

    // Get File instance (17) and Check it
    file = check_file(fileDAO,FILE17_ID,"FILE17",File::S_PENDING);
    CPPUNIT_ASSERT_MESSAGE("Invalid src surl (FILE17)", file->sourceSurl == FILE17_SRC_SURL);
    CPPUNIT_ASSERT_MESSAGE("Invalid dst surl (FILE17)", file->destSurl == FILE17_DST_SURL);
    check_error(file,"FILE17");
    // Get File instance (18) and Check it
    file = check_file(fileDAO,FILE18_ID,"FILE18",File::S_PENDING);
    CPPUNIT_ASSERT_MESSAGE("Invalid src surl (FILE18)", file->sourceSurl == FILE18_SRC_SURL);
    CPPUNIT_ASSERT_MESSAGE("Invalid dst surl (FILE18)", file->destSurl == FILE18_DST_SURL);
    check_error(file,"FILE18");

    //----------------------------------------------------------------------
    // Job 10
    //----------------------------------------------------------------------
    
    // Get Job instance (10) and Check it
    job = check_job(jobDAO,JOB10_ID,"JOB10",Job::S_SUBMITTED);
    CPPUNIT_ASSERT_MESSAGE("Invalid Channel (JOB10)", job->channelName().empty() == true);

    // Get File instance (19) and Check it
    file = check_file(fileDAO,FILE19_ID,"FILE19",File::S_FAILED);
    CPPUNIT_ASSERT_MESSAGE("Invalid src surl (FILE19)", file->sourceSurl().empty() == false);
    CPPUNIT_ASSERT_MESSAGE("Invalid dst surl (FILE19)", file->destSurl().empty() == false);
    check_error(file,"FILE19",Error::CONFIGURATION,Error::S_AGENT,Error::P_ALLOCATION);
    // Get File instance (20) and Check it
    file = check_file(fileDAO,FILE20_ID,"FILE20",File::S_WAITING_CATALOG_RESOLUTION);
    CPPUNIT_ASSERT_MESSAGE("Invalid src surl (FILE20)", file->sourceSurl().empty() == true);
    CPPUNIT_ASSERT_MESSAGE("Invalid dst surl (FILE20)", file->destSurl().empty() == true);
    check_error(file,"FILE20");

    //----------------------------------------------------------------------
    // Job 11
    //----------------------------------------------------------------------
    
    // Get Job instance (11) and Check it
    job = check_job(jobDAO,JOB11_ID,"JOB11",Job::S_FAILED);
    CPPUNIT_ASSERT_MESSAGE("Invalid Channel (JOB11)", job->channelName().empty() == true);

    // Get File instance (21) and Check it
    file = check_file(fileDAO,FILE21_ID,"FILE21",File::S_FAILED);
    CPPUNIT_ASSERT_MESSAGE("Invalid src surl (FILE21)", file->sourceSurl().empty() == true);
    CPPUNIT_ASSERT_MESSAGE("Invalid dst surl (FILE21)", file->destSurl().empty() == true);
    check_error(file,"FILE21",Error::CONFIGURATION,Error::S_AGENT,Error::P_CATALOG_RESOLUTION);
    // Get File instance (22) and Check it
    file = check_file(fileDAO,FILE22_ID,"FILE22",File::S_FAILED);
    CPPUNIT_ASSERT_MESSAGE("Invalid src surl (FILE22)", file->sourceSurl().empty() == true);
    CPPUNIT_ASSERT_MESSAGE("Invalid dst surl (FILE22)", file->destSurl().empty() == true);
    check_error(file,"FILE22",Error::CONFIGURATION,Error::S_AGENT,Error::P_CATALOG_RESOLUTION);
    
    //----------------------------------------------------------------------
    // Job 12
    //----------------------------------------------------------------------
    
    // Get Job instance (12) and Check it
    job = check_job(jobDAO,JOB12_ID,"JOB12",Job::S_FAILED);
    CPPUNIT_ASSERT_MESSAGE("Invalid Channel (JOB12)", job->channelName().empty() == true);

    // Get File instance (23) and Check it
    file = check_file(fileDAO,FILE23_ID,"FILE23",File::S_FAILED);
    CPPUNIT_ASSERT_MESSAGE("Invalid src surl (FILE23)", file->sourceSurl().empty() == true);
    check_error(file,"FILE23",Error::SECURITY,Error::S_SOURCE,Error::P_CATALOG_RESOLUTION);

    //----------------------------------------------------------------------
    // Job 13
    //----------------------------------------------------------------------
    
    // Get Job instance (13) and Check it
    job = check_job(jobDAO,JOB13_ID,"JOB13",Job::S_FAILED);
    CPPUNIT_ASSERT_MESSAGE("Invalid Channel (JOB13)", job->channelName().empty() == true);

    // Get File instance (24) and Check it
    file = check_file(fileDAO,FILE24_ID,"FILE24",File::S_FAILED);
    CPPUNIT_ASSERT_MESSAGE("Invalid src surl (FILE24)", file->sourceSurl().empty() == true);
    check_error(file,"FILE24",Error::SECURITY,Error::S_SOURCE,Error::P_CATALOG_RESOLUTION);

    //----------------------------------------------------------------------
    // Job 14
    //----------------------------------------------------------------------
    
    // Get Job instance (14) and Check it
    job = check_job(jobDAO,JOB14_ID,"JOB14",Job::S_FAILED);
    CPPUNIT_ASSERT_MESSAGE("Invalid Channel (JOB14)", job->channelName().empty() == true);

    // Get File instance (25) and Check it
    file = check_file(fileDAO,FILE25_ID,"FILE25",File::S_FAILED);
    CPPUNIT_ASSERT_MESSAGE("Invalid src surl (FILE25)", file->sourceSurl().empty() == true);
    check_error(file,"FILE25",Error::INVALID_PATH,Error::S_SOURCE,Error::P_CATALOG_RESOLUTION);

    //----------------------------------------------------------------------
    // Job 15
    //----------------------------------------------------------------------
    
    // Get Job instance (15) and Check it
    job = check_job(jobDAO,JOB15_ID,"JOB15",Job::S_CANCELED);
    CPPUNIT_ASSERT_MESSAGE("Invalid Channel (JOB15)", job->channelName().empty() == true);

    // Get File instance (26) and Check it
    file = check_file(fileDAO,FILE26_ID,"FILE26",File::S_CANCELED);
    check_error(file,"FILE26");
    // Get File instance (27) and Check it
    file = check_file(fileDAO,FILE27_ID,"FILE27",File::S_CANCELED);
    check_error(file,"FILE27");

    //----------------------------------------------------------------------
    // Job 16
    //----------------------------------------------------------------------
    
    // Get Job instance (16) and Check it
    job = check_job(jobDAO,JOB16_ID,"JOB16",Job::S_CANCELED);
    CPPUNIT_ASSERT_MESSAGE("Invalid Channel (JOB16)", job->channelName().empty() == true);

    // Get File instance (28) and Check it
    file = check_file(fileDAO,FILE28_ID,"FILE28",File::S_CANCELED);
    CPPUNIT_ASSERT_MESSAGE("Invalid src surl (FILE28)", file->sourceSurl().empty() == true);
    CPPUNIT_ASSERT_MESSAGE("Invalid dst surl (FILE28)", file->destSurl().empty() == true);
    check_error(file,"FILE28");
    // Get File instance (29) and Check it
    file = check_file(fileDAO,FILE29_ID,"FILE29",File::S_CANCELED);
    CPPUNIT_ASSERT_MESSAGE("Invalid src surl (FILE29)", file->sourceSurl().empty() == true);
    CPPUNIT_ASSERT_MESSAGE("Invalid dst surl (FILE29)", file->destSurl().empty() == true);
    check_error(file,"FILE29");

}

/*
 * Test Execute Method
 */
void AllocateTestCase::testExecute_2(){ // [3.1.0]

    setUp_forTest2();
    
    // Set Context
    m_action->context(VO2_NAME,DaoHelper::ctx());
    
    // Execute Actions
    m_action->execute(JOB_PARAM_ALLOCATE_IN_DIFFERENT_CYCLE);

    // Create FileDAO instance
    scoped_ptr<dao::vo::FileDAO> fileDAO(dao::vo::DAOFactory::instance().createFileDAO(VO2_NAME, DaoHelper::ctx()));
    CPPUNIT_ASSERT_MESSAGE("Null FileDAO", fileDAO.get() != 0);
    
    // Create JobDAO instance
    scoped_ptr<dao::vo::JobDAO> jobDAO(dao::vo::DAOFactory::instance().createJobDAO(VO2_NAME, DaoHelper::ctx()));
    CPPUNIT_ASSERT_MESSAGE("Null JobDAO", jobDAO.get() != 0);

    shared_ptr<Job>     job;
    shared_ptr<File>    file;
    
    //----------------------------------------------------------------------
    // Job 101
    //----------------------------------------------------------------------
    
    // Get Job instance (101) and Check it
    job = check_job(jobDAO,JOB101_ID,"JOB101",Job::S_PENDING);
    CPPUNIT_ASSERT_MESSAGE("Invalid Channel (JOB101)", job->channelName == CHANNEL10_NAME);
    CPPUNIT_ASSERT_MESSAGE("Invalid Source Site (JOB101)", job->source == SITE_INFN_1);
    CPPUNIT_ASSERT_MESSAGE("Invalid Dest Site (JOB101)", job->dest == SITE_CERN_1);
    CPPUNIT_ASSERT_MESSAGE("Invalid Source SE (JOB101)", job->sourceSE == HOST_INFN_1);
    CPPUNIT_ASSERT_MESSAGE("Invalid Dest SE (JOB101)", job->destSE == HOST_CERN_1);
    
    // Get File instance (101) and Check it
    file = check_file(fileDAO,FILE101_ID,"FILE101",File::S_PENDING);
    // Get File instance (102) and Check it
    file = check_file(fileDAO,FILE102_ID,"FILE102",File::S_PENDING);
    
    //----------------------------------------------------------------------
    // Job 102
    //----------------------------------------------------------------------
    
    // Get Job instance (102) and Check it
    job = check_job(jobDAO,JOB102_ID,"JOB102",Job::S_PENDING);
    CPPUNIT_ASSERT_MESSAGE("Invalid Channel (JOB101)", job->channelName == CHANNEL11_NAME);
    CPPUNIT_ASSERT_MESSAGE("Invalid Source Site (JOB102)", job->source == SITE_CATANIA_1);
    CPPUNIT_ASSERT_MESSAGE("Invalid Dest Site (JOB102)", job->dest == SITE_CERN_1);
    CPPUNIT_ASSERT_MESSAGE("Invalid Source SE (JOB102)", job->sourceSE == HOST_CATANIA_1);
    CPPUNIT_ASSERT_MESSAGE("Invalid Dest SE (JOB102)", job->destSE == HOST_CERN_1);

    // Get File instance (103) and Check it
    file = check_file(fileDAO,FILE103_ID,"FILE103",File::S_PENDING);
    // Get File instance (104) and Check it
    file = check_file(fileDAO,FILE104_ID,"FILE104",File::S_PENDING);

    //----------------------------------------------------------------------
    // Job 103
    //----------------------------------------------------------------------
    
    // Get Job instance (103) and Check it
    job = check_job(jobDAO,JOB103_ID,"JOB103",Job::S_FAILED);
    CPPUNIT_ASSERT_MESSAGE("Invalid Channel (JOB103)", job->channelName().empty() == true);
    
    // Get File instance (105) and Check it
    file = check_file(fileDAO,FILE105_ID,"FILE105",File::S_FAILED);
    check_error(file,"FILE105",Error::CONFIGURATION,Error::S_AGENT,Error::P_ALLOCATION);
    // Get File instance (106) and Check it
    file = check_file(fileDAO,FILE106_ID,"FILE106",File::S_FAILED);
    check_error(file,"FILE106",Error::CONFIGURATION,Error::S_AGENT,Error::P_ALLOCATION);

}
