/**
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *  GLite Data Transfer Agents: BasicRetry Test Case
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>, Paolo Tedesco <paolo.tedesco@cern.ch>
 *  Version info: $Id: BasicRetryTestCase.cpp,v 1.14 2008/04/11 11:02:56 ptedesco Exp $
 *  Release: $Name:  $
 *
 */
#include "BasicRetry.h"
#include "PythonRetry.h"
#include "DaoHelper.h"
#include "glite/data/transfer/agent/action/ActionFactory.h"
#include "glite/data/transfer/agent/model/File.h"
#include "glite/data/transfer/agent/model/Job.h"
#include "glite/data/transfer/agent/dao/vo/FileDAO.h"
#include "glite/data/transfer/agent/dao/vo/JobDAO.h"
#include "glite/data/transfer/agent/dao/vo/DAOFactory.h"

#include "dao_test_utils.h"

#include <boost/scoped_ptr.hpp>
#include <cppunit/TestCase.h>
#include <cppunit/extensions/HelperMacros.h>

using namespace glite::data::transfer::agent;
using namespace glite::data::transfer::agent::action;
using namespace glite::data::transfer::agent::action::vo;
using namespace glite::data::transfer::agent::model;

using boost::scoped_ptr;

/**
 * CPPUnit TestCase
 */
template<class R>
class BasicRetryTestCase : public CppUnit::TestCase{
public:
    /**
     * set-up test case
     */
    void setUp ();
private:
    // Declare CPPUnit Methods
    // -------------------------------------------------------------------------
    CPPUNIT_TEST_SUITE( BasicRetryTestCase);
    CPPUNIT_TEST( testExecute );
    CPPUNIT_TEST_SUITE_END();

    // Test Methods
    // -------------------------------------------------------------------------

    /**
     * Test Execute Method
     */
    void testExecute();
    
private:
    /**
     * The Action Instance
     */
    scoped_ptr<R> m_action;
};

/*
 * Declare Constants. This unit test foresee at beginning
 *
 * (1) - 1 ACTIVE Job with:                             => 
 *        job_id      = 6-...-01
 *        files       = 60000001 (WAITING)(req_6_1)     PENDING
 *                      60000002 (DONE)   (req_6_1)  
 *
 * (2) - 1 ACTIVE Job with:                             => 
 *        job_id      = 6-...-02
 *        files       = 60000003 (WAITING)(req_6_2)     PENDING
 *                      60000004 (WAITING)(req_6_3)     WAITING (delay not elapsed)
 *
 * (3) - 1 ACTIVE Job with:                             => 
 *        job_id      = 6-...-03
 *        files       = 60000005 (WAITING)(req_6_4)     PENDING
 *                      60000006 (WAITING)(req_6_4)     HOLD (too many failures)
 *
 * (4) - 1 ACTIVE Job with:                             =>  stays Active (Job state will be modified by the related channel agent)
 *        job_id      = 6-...-04
 *        files       = 60000007 (WAITING)(req_6_5,6_6,
 *                                         6_8,6_9,6_10)HOLD (too many retries)
 *                      60000008 (WAITING)(req_6_5)     HOLD (too many failures)
 *
 * (5) - 1 ACTIVE Job with:                             =>  stays Active (Job state will be modified by the related channel agent)
 *        job_id      = 6-...-05
 *        files       = 60000009 (DONE)   (req_6_7)    
 *                      60000010 (FAILED)   
 *                      60000011 (WAITING)(req_6_7)     HOLD (too many failures)
 *
 * (6) - 1 Submitted Job with:                          =>  
 *        job_id      = 6-...-06
 *        files       = 60000012 (PENDING)
 *                      60000013 (W_C_RES)              SUBMITTED
 *
 * (7) - 1 Submitted Job with:                          =>  PENDING
 *        job_id      = 6-...-07
 *        files       = 60000014 (PENDING)
 *                      60000015 (W_C_RES)              FAILED
 *
 * (8) - 1 Submitted Job with:                          =>  
 *        job_id      = 6-...-08
 *        files       = 60000016 (PENDING)
 *                      60000017 (W_C_RES)              (retry delay not elapsed)
 *
 * (9) - 1 Submitted Job with:                          =>  FAILED
 *        job_id      = 6-...-09
 *                      60000018 (W_C_RES)              FAILED
 *
 * (10) - 1 Finishing Job with:                          =>  
 *        job_id      = 6-...-10
 *        files       = 60000019 (FINISHED)
 *                      60000020 (W_C_REG)              FINISHING
 *
 * (11) - 1 Finishing Job with:                          =>  FINISHED_DIRTY
 *        job_id      = 6-...-11
 *        files       = 60000021 (FINISHED)
 *                      60000022 (W_C_REG)              FAILED
 *
 * (12) - 1 Finishing Job with:                          =>  
 *        job_id      = 6-...-12
 *        files       = 60000023 (FINISHED)
 *                      60000024 (W_C_REG)              (retry delay not elapsed)
 *
 * (13) - 1 Finishing Job with:                          =>  FINISHED_DIRTY
 *        job_id      = 6-...-13
 *                      60000025 (W_C_REG)              FAILED
 *
 * (14) - 1 ACTIVE Job with: (canceling)                 => 
 *        job_id      = 6-...-14
 *        files       = 60000026 (WAITING)(req_6_26)    CANCELED
 *                      60000027 (DONE)     
 *
 * (15) - 1 Finishing Job with: (canceling)              =>  CANCELED
 *        job_id      = 6-...-15
 *        files       = 60000028 (FINISHED)
 *                      60000029 (W_C_REG)              CANCELED
 *
 * (16) - 1 Submitted Job with: (canceling)              =>  PENDING
 *        job_id      = 6-...-16
 *        files       = 60000030 (PENDING)
 *                      60000031 (W_C_RES)              CANCELED
 *
 * (17) - 1 Finishing Job with: (canceling)              =>  
 *        job_id      = 6-...-17
 *        files       = 60000032 (FINISHING)
 *                      60000033 (W_C_REG)              CANCELED
 *
 * (18) - 1 Submitted Job with: (canceling)              =>  CANCELED
 *        job_id      = 6-...-18
 *        files       = 60000034 (FAILED)
 *                      60000035 (W_C_RES)              CANCELED
 *
 */
namespace {
const char * const JOB1_ID   = "60000000-0000-0000-0000-000000000001";
const char * const JOB2_ID   = "60000000-0000-0000-0000-000000000002";
const char * const JOB3_ID   = "60000000-0000-0000-0000-000000000003";
const char * const JOB4_ID   = "60000000-0000-0000-0000-000000000004";
const char * const JOB5_ID   = "60000000-0000-0000-0000-000000000005";
const char * const JOB6_ID   = "60000000-0000-0000-0000-000000000006";
const char * const JOB7_ID   = "60000000-0000-0000-0000-000000000007";
const char * const JOB8_ID   = "60000000-0000-0000-0000-000000000008";
const char * const JOB9_ID   = "60000000-0000-0000-0000-000000000009";
const char * const JOB10_ID  = "60000000-0000-0000-0000-000000000010";
const char * const JOB11_ID  = "60000000-0000-0000-0000-000000000011";
const char * const JOB12_ID  = "60000000-0000-0000-0000-000000000012";
const char * const JOB13_ID  = "60000000-0000-0000-0000-000000000013";
const char * const JOB14_ID  = "60000000-0000-0000-0000-000000000014";
const char * const JOB15_ID  = "60000000-0000-0000-0000-000000000015";
const char * const JOB16_ID  = "60000000-0000-0000-0000-000000000016";
const char * const JOB17_ID  = "60000000-0000-0000-0000-000000000017";
const char * const JOB18_ID  = "60000000-0000-0000-0000-000000000018";
const char * const FILE1_ID  = "60000001";
const char * const FILE2_ID  = "60000002";
const char * const FILE3_ID  = "60000003";
const char * const FILE4_ID  = "60000004";
const char * const FILE5_ID  = "60000005";
const char * const FILE6_ID  = "60000006";
const char * const FILE7_ID  = "60000007";
const char * const FILE8_ID  = "60000008";
const char * const FILE9_ID  = "60000009";
const char * const FILE10_ID = "60000010";
const char * const FILE11_ID = "60000011";
const char * const FILE12_ID = "60000012";
const char * const FILE13_ID = "60000013";
const char * const FILE14_ID = "60000014";
const char * const FILE15_ID = "60000015";
const char * const FILE16_ID = "60000016";
const char * const FILE17_ID = "60000017";
const char * const FILE18_ID = "60000018";
const char * const FILE19_ID = "60000019";
const char * const FILE20_ID = "60000020";
const char * const FILE21_ID = "60000021";
const char * const FILE22_ID = "60000022";
const char * const FILE23_ID = "60000023";
const char * const FILE24_ID = "60000024";
const char * const FILE25_ID = "60000025";
const char * const FILE26_ID = "60000026";
const char * const FILE27_ID = "60000027";
const char * const FILE28_ID = "60000028";
const char * const FILE29_ID = "60000029";
const char * const FILE30_ID = "60000030";
const char * const FILE31_ID = "60000031";
const char * const FILE32_ID = "60000032";
const char * const FILE33_ID = "60000033";
const char * const FILE34_ID = "60000034";
const char * const FILE35_ID = "60000035";
const char * const REQ1_ID   = "req_6_1";
const char * const REQ2_ID   = "req_6_2";
const char * const REQ3_ID   = "req_6_3";
const char * const REQ4_ID   = "req_6_4";
const char * const REQ5_ID   = "req_6_5";
const char * const REQ6_ID   = "req_6_6";
const char * const REQ7_ID   = "req_6_7";
const char * const REQ8_ID   = "req_6_8";
const char * const REQ9_ID   = "req_6_9";
const char * const REQ10_ID  = "req_6_10";
const char * const VO1_NAME  = "BASICRETRY_VO_1";
}

/*
 * setUp
 *
 * set-up test case
 */
template <class R>
void BasicRetryTestCase<R>::setUp(){
    // Create Action instance
    std::auto_ptr<Action> a(ActionFactory::instance().create(R::factory()->type()));
    CPPUNIT_ASSERT_MESSAGE("Null Action", a.get() != 0);
    CPPUNIT_ASSERT_MESSAGE("Invalid Action Class", dynamic_cast<R*>(a.get()) != 0);

    // Store Action        
    m_action.reset(dynamic_cast<R*>(a.release()));
    CPPUNIT_ASSERT_MESSAGE("Null Action", m_action.get() != 0);
}

/*
 *
 * Test Execute Method
 */
template <class R>
void BasicRetryTestCase<R>::testExecute(){
    
    // Set Context
    m_action->context(VO1_NAME,DaoHelper::ctx());
    
    // Execute Action 
    m_action->execute();
    
    // Create FileDAO instance
    scoped_ptr<dao::vo::FileDAO> fileDAO(dao::vo::DAOFactory::instance().createFileDAO(VO1_NAME, DaoHelper::ctx()));
    CPPUNIT_ASSERT_MESSAGE("Null FileDAO", fileDAO.get() != 0);
    
    // Create JobDAO instance
    scoped_ptr<dao::vo::JobDAO> jobDAO(dao::vo::DAOFactory::instance().createJobDAO(VO1_NAME, DaoHelper::ctx()));
    CPPUNIT_ASSERT_MESSAGE("Null JobDAO", jobDAO.get() != 0);
    
    // Check The Transfers using FileDAO

    //----------------------------------------------------------------------
    // Job 1
    //----------------------------------------------------------------------
    
    // Get Job instance (1) and Check it
    check_job(jobDAO,JOB1_ID,"JOB1",Job::S_ACTIVE);

    // Get File instance (1) and Check it
    check_file(fileDAO,FILE1_ID,"FILE1",File::S_PENDING);
    // Get File instance (2) and Check it
    check_file(fileDAO,FILE2_ID,"FILE2",File::S_DONE);
    
    //----------------------------------------------------------------------
    // Job 2
    //----------------------------------------------------------------------
    
    // Get Job instance (2) and Check it
    check_job(jobDAO,JOB2_ID,"JOB2",Job::S_ACTIVE);
    
    // Get File instance (3) and Check it
    check_file(fileDAO,FILE3_ID,"FILE3",File::S_PENDING);
    // Get File instance (4) and Check it
    check_file(fileDAO,FILE4_ID,"FILE4",File::S_WAITING);

    //----------------------------------------------------------------------
    // Job 3
    //----------------------------------------------------------------------

    // Get Job instance (3) and Check it
    check_job(jobDAO,JOB3_ID,"JOB3",Job::S_ACTIVE);

    // Get File instance (5) and Check it
    check_file(fileDAO,FILE5_ID,"FILE5",File::S_PENDING);
    // Get File instance (6) and Check it
    check_file(fileDAO,FILE6_ID,"FILE6",File::S_HOLD);
    
    //----------------------------------------------------------------------
    // Job 4
    //----------------------------------------------------------------------
    
    // Get Job instance (4) and Check it
    check_job(jobDAO,JOB4_ID,"JOB4",Job::S_ACTIVE);

    // Get File instance (7) and Check it
    check_file(fileDAO,FILE7_ID,"FILE7",File::S_HOLD);
    // Get File instance (8) and Check it
    check_file(fileDAO,FILE8_ID,"FILE8",File::S_HOLD);
    
    //----------------------------------------------------------------------
    // Job 5
    //----------------------------------------------------------------------
    
    // Get Job instance (5) and Check it
    check_job(jobDAO,JOB5_ID,"JOB5",Job::S_ACTIVE);
    
    // Get File instance (9) and Check it
    check_file(fileDAO,FILE9_ID,"FILE9",File::S_DONE);
    // Get File instance (10) and Check it
    check_file(fileDAO,FILE10_ID,"FILE10",File::S_FAILED);
    // Get File instance (11) and Check it
    check_file(fileDAO,FILE11_ID,"FILE11",File::S_HOLD);

    //----------------------------------------------------------------------
    // Job 6
    //----------------------------------------------------------------------
    
    // Get Job instance (6) and Check it
    check_job(jobDAO,JOB6_ID,"JOB6",Job::S_SUBMITTED);
    
    // Get File instance (12) and Check it
    check_file(fileDAO,FILE12_ID,"FILE12",File::S_PENDING);
    // Get File instance (13) and Check it
    check_file(fileDAO,FILE13_ID,"FILE13",File::S_SUBMITTED);

    //----------------------------------------------------------------------
    // Job 7
    //----------------------------------------------------------------------
    
    // Get Job instance (7) and Check it
    check_job(jobDAO,JOB7_ID,"JOB7",Job::S_PENDING);
    
    // Get File instance (14) and Check it
    check_file(fileDAO,FILE14_ID,"FILE14",File::S_PENDING);
    // Get File instance (15) and Check it
    check_file(fileDAO,FILE15_ID,"FILE15",File::S_FAILED);

    //----------------------------------------------------------------------
    // Job 8
    //----------------------------------------------------------------------

    // Get Job instance (8) and Check it
    check_job(jobDAO,JOB8_ID,"JOB8",Job::S_SUBMITTED);

    // Get File instance (16) and Check it
    check_file(fileDAO,FILE16_ID,"FILE16",File::S_PENDING);
    // Get File instance (17) and Check it
    check_file(fileDAO,FILE17_ID,"FILE17",File::S_WAITING_CATALOG_RESOLUTION);

    //----------------------------------------------------------------------
    // Job 9
    //----------------------------------------------------------------------

    // Get Job instance (9) and Check it
    check_job(jobDAO,JOB9_ID,"JOB9",Job::S_FAILED);
    
    // Get File instance (18) and Check it
    check_file(fileDAO,FILE18_ID,"FILE18",File::S_FAILED);

    //----------------------------------------------------------------------
    // Job 10
    //----------------------------------------------------------------------

    // Get Job instance (10) and Check it
    check_job(jobDAO,JOB10_ID,"JOB10",Job::S_FINISHING);

    // Get File instance (19) and Check it
    check_file(fileDAO,FILE19_ID,"FILE19",File::S_FINISHED);
    // Get File instance (20) and Check it
    check_file(fileDAO,FILE20_ID,"FILE20",File::S_FINISHING);

    //----------------------------------------------------------------------
    // Job 11
    //----------------------------------------------------------------------
    
    // Get Job instance (11) and Check it
    check_job(jobDAO,JOB11_ID,"JOB11",Job::S_FINISHED_DIRTY);
    
    // Get File instance (21) and Check it
    check_file(fileDAO,FILE21_ID,"FILE21",File::S_FINISHED);
    // Get File instance (22) and Check it
    check_file(fileDAO,FILE22_ID,"FILE22",File::S_FAILED);
    
    //----------------------------------------------------------------------
    // Job 12
    //----------------------------------------------------------------------

    // Get Job instance (12) and Check it
    check_job(jobDAO,JOB12_ID,"JOB12",Job::S_FINISHING);
    
    // Get File instance (23) and Check it
    check_file(fileDAO,FILE23_ID,"FILE23",File::S_FINISHED);
    // Get File instance (24) and Check it
    check_file(fileDAO,FILE24_ID,"FILE24",File::S_WAITING_CATALOG_REGISTRATION);

    //----------------------------------------------------------------------
    // Job 13
    //----------------------------------------------------------------------
    
    // Get Job instance (13) and Check it
    check_job(jobDAO,JOB13_ID,"JOB13",Job::S_FINISHED_DIRTY);
    
    // Get File instance (25) and Check it
    check_file(fileDAO,FILE25_ID,"FILE25",File::S_FAILED);

    //----------------------------------------------------------------------
    // Job 14
    //----------------------------------------------------------------------

    // Get Job instance (14) and Check it
    check_job(jobDAO,JOB14_ID,"JOB14",Job::S_ACTIVE);
    
    // Get File instance (26) and Check it
    check_file(fileDAO,FILE26_ID,"FILE26",File::S_CANCELED);
    // Get File instance (27) and Check it
    check_file(fileDAO,FILE27_ID,"FILE27",File::S_DONE);

    //----------------------------------------------------------------------
    // Job 15
    //----------------------------------------------------------------------
    
    // Get Job instance (15) and Check it
    check_job(jobDAO,JOB15_ID,"JOB15",Job::S_CANCELED);
    
    // Get File instance (28) and Check it
    check_file(fileDAO,FILE28_ID,"FILE28",File::S_FINISHED);
    // Get File instance (29) and Check it
    check_file(fileDAO,FILE29_ID,"FILE29",File::S_CANCELED);

    //----------------------------------------------------------------------
    // Job 16
    //----------------------------------------------------------------------
    
    // Get Job instance (16) and Check it
    check_job(jobDAO,JOB16_ID,"JOB16",Job::S_PENDING);
    
    // Get File instance (30) and Check it
    check_file(fileDAO,FILE30_ID,"FILE30",File::S_PENDING);
    // Get File instance (31) and Check it
    check_file(fileDAO,FILE31_ID,"FILE31",File::S_CANCELED);

    //----------------------------------------------------------------------
    // Job 17
    //----------------------------------------------------------------------

    // Get Job instance (17) and Check it
    check_job(jobDAO,JOB17_ID,"JOB17",Job::S_FINISHING);
    
    // Get File instance (32) and Check it
    check_file(fileDAO,FILE32_ID,"FILE32",File::S_FINISHING);
    // Get File instance (33) and Check it
    check_file(fileDAO,FILE33_ID,"FILE33",File::S_CANCELED);

    //----------------------------------------------------------------------
    // Job 18
    //----------------------------------------------------------------------

    // Get Job instance (18) and Check it
    check_job(jobDAO,JOB18_ID,"JOB18",Job::S_CANCELED);
    
    // Get File instance (34) and Check it
    check_file(fileDAO,FILE34_ID,"FILE34",File::S_FAILED);
    // Get File instance (35) and Check it
    check_file(fileDAO,FILE35_ID,"FILE35",File::S_CANCELED);
}

/*
 * Register Suit
 */
CPPUNIT_TEST_SUITE_NAMED_REGISTRATION( BasicRetryTestCase<BasicRetry>,  "BasicRetryTestCase" );
CPPUNIT_TEST_SUITE_NAMED_REGISTRATION( BasicRetryTestCase<PythonRetry>, "PythonBasicRetryTestCase" );
