/**
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *  GLite Data Transfer Agents: Cancel Active Transfers Test Case
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>, Paolo Tedesco <paolo.tedesco@cern.ch>
 *  Version info: $Id: CancelActiveTestCase.cpp,v 1.11 2009/07/02 15:21:17 molnarzs Exp $
 *  Release: $Name:  $
 *
 */
#include "actions/channel/Cancel.h"
#include "actions/channel/ActionsConfig.h"
#include "DaoHelper.h"
#include "glite/data/transfer/agent/action/ActionFactory.h"
#include "glite/data/transfer/agent/model/File.h"
#include "glite/data/transfer/agent/model/Job.h"
#include "glite/data/transfer/agent/model/Transfer.h"
#include "glite/data/transfer/agent/model/Channel.h"
#include "glite/data/transfer/agent/dao/channel/FileDAO.h"
#include "glite/data/transfer/agent/dao/channel/JobDAO.h"
#include "glite/data/transfer/agent/dao/channel/TransferDAO.h"
#include "glite/data/transfer/agent/dao/channel/DAOFactory.h"
#include "glite/data/transfer/agent/ts/TransferService.h"
#include "glite/data/transfer/agent/ts/TransferServiceFactory.h"

#include "glite/data/agents/dao/DAOException.h"

#include "dao_test_utils.h"

#include <memory>
#include <boost/scoped_ptr.hpp>
#include <cppunit/TestCase.h>
#include <cppunit/extensions/HelperMacros.h>
#include <sstream>

using namespace glite::data::transfer::agent;
using namespace glite::data::transfer::agent::action;
using namespace glite::data::transfer::agent::action::channel;
using namespace glite::data::transfer::agent::model;

using glite::data::agents::dao::DAOException;

using boost::scoped_ptr;

/**
 * CPPUnit TestCase
 */
class CancelActiveTestCase : public CppUnit::TestCase{
public:
    /**
     * set-up test case
     */
    void setUp ();

    /**
     * tear-down test case
     */
    void tearDown ();
private:
    // Declare CPPUnit Methods
    // -------------------------------------------------------------------------
    CPPUNIT_TEST_SUITE( CancelActiveTestCase);
    CPPUNIT_TEST( testExecute );
    CPPUNIT_TEST_SUITE_END();

    // Test Methods
    // -------------------------------------------------------------------------

    /**
     * Test Execute Method
     */
    void testExecute();

private:
    /**
     * The Action Instance
     */
    scoped_ptr<Cancel> m_action;

};

/*
 * Register Suit
 */
CPPUNIT_TEST_SUITE_NAMED_REGISTRATION( CancelActiveTestCase, "CancelActiveTestCase" );

/*
 * Declare Constants. This unit test foresee at beginning
 * - 1 ACTIVE Job with:                     => CANCELED   (*cancel_job = true)
 *        job_id  = 3-...-01
 *        vo      = VO_1
 *        channel = CANCELACTIVE_channel_1
 *        files   = 30000001 (DONE)             CANCELED
 *                  30000002 (ACTIVE)           CANCELED
 *
 * - 1 ACTIVE Job with:                     =>            (*cancel_job = true)
 *        job_id  = 3-...-02
 *        vo      = VO_1
 *        channel = CANCELACTIVE_channel_1
 *        files   = 30000003 (PENDING)
 *                  30000004 (ACTIVE)           CANCELED
 *                  30000005 (HOLD)             CANCELED
 *
 * - 1 ACTIVE Job with:                     => CANCELED   (*cancel_job = true)
 *        job_id  = 3-...-03
 *        vo      = VO_1
 *        channel = CANCELACTIVE_channel_1
 *        files   = 30000006 (ACTIVE)           CANCELED
 *                  30000007 (READY)            CANCELED
 *
 *
 * - 1 ACTIVE Job with:                     =>            (*cancel_job = true)
 *        job_id  = 3-...-04
 *        vo      = VO_1
 *        channel = CANCELACTIVE_channel_1
 *        files   = 30000008 (ACTIVE)           CANCELED
 *                  30000009 (WAITING)
 *
 * - 1 ACTIVE Job with:                     => CANCELED   (*cancel_job = true)
 *        job_id  = 3-...-05
 *        vo      = VO_1
 *        channel = CANCELACTIVE_channel_1
 *        files   = 30000010 (DONE)             CANCELED
 *                  30000011 (ACTIVE)           CANCELED
 *                  30000012 (HOLD)             CANCELED
 *                  30000013 (FAILED)
 *
 */
namespace {
const char * const JOB1_ID        = "30000000-0000-0000-0000-000000000001";
const char * const JOB2_ID        = "30000000-0000-0000-0000-000000000002";
const char * const JOB3_ID        = "30000000-0000-0000-0000-000000000003";
const char * const JOB4_ID        = "30000000-0000-0000-0000-000000000004";
const char * const JOB5_ID        = "30000000-0000-0000-0000-000000000005";
const char * const FILE1_ID       = "30000001";
const char * const FILE2_ID       = "30000002";
const char * const FILE3_ID       = "30000003";
const char * const FILE4_ID       = "30000004";
const char * const FILE5_ID       = "30000005";
const char * const FILE6_ID       = "30000006";
const char * const FILE7_ID       = "30000007";
const char * const FILE8_ID       = "30000008";
const char * const FILE9_ID       = "30000009";
const char * const FILE10_ID      = "30000010";
const char * const FILE11_ID      = "30000011";
const char * const FILE12_ID      = "30000012";
const char * const FILE13_ID      = "30000013";
std::string  REQ2_ID        = "(ReqId auto-gen)";
std::string  REQ4_ID        = "(ReqId auto-gen)";
std::string  REQ6_ID        = "(ReqId auto-gen)";
std::string  REQ8_ID        = "(ReqId auto-gen)";
std::string  REQ11_ID       = "(ReqId auto-gen)";
const char * const VO1_NAME       = "VO_1";
const char * const CHANNEL_NAME   = "CANCELACTIVE_channel_1";

const char * const TC_SRC_NAME      = "srm://host.cern.ch:8443/tmp/src.txt";
const char * const TC_DEST_NAME     = "srm://host.imperial.ac.uk:8443/tmp/dst.txt";

}

/*
 * setUp
 *
 * set-up test case
 */
void CancelActiveTestCase::setUp(){

    ActionsConfig::instance().activeTxCache().reset();

    // Create Action instance
    std::auto_ptr<Action> a(ActionFactory::instance().create("glite:CancelActive"));
    CPPUNIT_ASSERT_MESSAGE("Null Action", a.get() != 0);
    CPPUNIT_ASSERT_MESSAGE("Invalid Action Class", dynamic_cast<Cancel*>(a.get()) != 0);

    // Store Action
    m_action.reset(dynamic_cast<Cancel*>(a.release()));
    CPPUNIT_ASSERT_MESSAGE("Null Action", m_action.get() != 0);

    // Create TransferDAO instance
    scoped_ptr<dao::channel::TransferDAO> transferDao(dao::channel::DAOFactory::instance().createTransferDAO(CHANNEL_NAME, DaoHelper::ctx()));
    CPPUNIT_ASSERT_MESSAGE("Null TransferDAO", transferDao.get() != 0);

    // Add requests to TransferService
    scoped_ptr<ts::TransferService> tx_service(ts::TransferServiceFactory::instance().create(CHANNEL_NAME));
    CPPUNIT_ASSERT_MESSAGE("Null TransferService", tx_service.get() != 0);

    // Req 2
    ts::TransferService::TransferRequest  req;
    ts::TransferService::TransferResponse out;
    req.surls.resize(1);
    req.surls[0].source = TC_SRC_NAME;
    req.surls[0].destination = TC_DEST_NAME;
    // Submit Transfer
    tx_service->submit(req,out,"");
    scoped_ptr<Transfer> tx(new Transfer(out.request_id,FILE2_ID,out.transfer_ids.at(0),JOB1_ID,Transfer::S_PROCESSING));
    transferDao->create(*tx);
    REQ2_ID = out.request_id;

    // Req 4
    out.transfer_ids.clear();
    tx_service->submit(req,out,"");
    tx.reset(new Transfer(out.request_id,FILE4_ID,out.transfer_ids.at(0),JOB2_ID,Transfer::S_PROCESSING));
    transferDao->create(*tx);
    REQ4_ID = out.request_id;

    // Req 6
    out.transfer_ids.clear();
    tx_service->submit(req,out,"");
    tx.reset(new Transfer(out.request_id,FILE6_ID,out.transfer_ids.at(0),JOB3_ID,Transfer::S_PROCESSING));
    transferDao->create(*tx);
    REQ6_ID = out.request_id;

    // Req 8
    out.transfer_ids.clear();
    tx_service->submit(req,out,"");
    tx.reset(new Transfer(out.request_id,FILE8_ID,out.transfer_ids.at(0),JOB4_ID,Transfer::S_PROCESSING));
    transferDao->create(*tx);
    REQ8_ID = out.request_id;

    // Req 11
    out.transfer_ids.clear();
    tx_service->submit(req,out,"");
    tx.reset(new Transfer(out.request_id,FILE11_ID,out.transfer_ids.at(0),JOB5_ID,Transfer::S_PROCESSING));
    transferDao->create(*tx);
    REQ11_ID = out.request_id;
}

/*
 * tearDown
 *
 * tear-down test case
 */
void CancelActiveTestCase::tearDown(){
    ActionsConfig::instance().activeTxCache().reset();
}

/*
 *
 * Test Execute Method
 */
void CancelActiveTestCase::testExecute(){
    // Set Context
    m_action->context(CHANNEL_NAME,DaoHelper::ctx());

    // Execute Actions
    m_action->execute();

    // Create FileDAO instance
    scoped_ptr<dao::channel::FileDAO> fileDAO(dao::channel::DAOFactory::instance().createFileDAO(CHANNEL_NAME, DaoHelper::ctx()));
    CPPUNIT_ASSERT_MESSAGE("Null FileDAO", fileDAO.get() != 0);

    // Create JobDAO instance
    scoped_ptr<dao::channel::JobDAO> jobDAO(dao::channel::DAOFactory::instance().createJobDAO(CHANNEL_NAME, DaoHelper::ctx()));
    CPPUNIT_ASSERT_MESSAGE("Null JobDAO", jobDAO.get() != 0);

    // Create TransferDAO instance
    scoped_ptr<dao::channel::TransferDAO> transferDAO(dao::channel::DAOFactory::instance().createTransferDAO(CHANNEL_NAME, DaoHelper::ctx()));
    CPPUNIT_ASSERT_MESSAGE("Null TransferDAO", transferDAO.get() != 0);

    // Check job states
    // Check file states
    // Check if transfers have been aborted

    //----------------------------------------------------------------------
    // Job 1
    //----------------------------------------------------------------------

    // Get Job instance (1) and Check it
    check_job(jobDAO,JOB1_ID,"JOB1",Job::S_CANCELED);

    // Get File instance (1) and Check it
    check_file(fileDAO,FILE1_ID,"FILE1",File::S_CANCELED);
    // Get File instance (2) and Check it
    check_file(fileDAO,FILE2_ID,"FILE2",File::S_CANCELED);

    // Get Transfer instance (2) and Check it
    check_transfer(transferDAO,FILE2_ID,REQ2_ID,"FILE2,REQ2",Transfer::S_ABORTED);

    //----------------------------------------------------------------------
    // Job 2
    //----------------------------------------------------------------------

    // Get Job instance (2) and Check it
    check_job(jobDAO,JOB2_ID,"JOB2",Job::S_ACTIVE);

    // Get File instance (3) and Check it
    check_file(fileDAO,FILE3_ID,"FILE3",File::S_PENDING);
    // Get File instance (4) and Check it
    check_file(fileDAO,FILE4_ID,"FILE4",File::S_CANCELED); // the file is canceled but the job is not!
    // Get File instance (5) and Check it
    check_file(fileDAO,FILE5_ID,"FILE5",File::S_CANCELED);

    // Get Transfer instance (4) and Check it
    check_transfer(transferDAO,FILE4_ID,REQ4_ID,"FILE4,REQ4",Transfer::S_ABORTED);

    //----------------------------------------------------------------------
    // Job 3
    //----------------------------------------------------------------------

    // Get Job instance (3) and Check it
    check_job(jobDAO,JOB3_ID,"JOB3",Job::S_CANCELED);

    // Get File instance (6) and Check it
    check_file(fileDAO,FILE6_ID,"FILE6",File::S_CANCELED);
    // Get File instance (7) and Check it
    check_file(fileDAO,FILE7_ID,"FILE7",File::S_CANCELED);

    // Get Transfer instance (6) and Check it
    check_transfer(transferDAO,FILE6_ID,REQ6_ID,"FILE6,REQ6",Transfer::S_ABORTED);

    //----------------------------------------------------------------------
    // Job 4
    //----------------------------------------------------------------------

    // Get Job instance (4) and Check it
    check_job(jobDAO,JOB4_ID,"JOB4",Job::S_ACTIVE);

    // Get File instance (8) and Check it
    check_file(fileDAO,FILE8_ID,"FILE8",File::S_CANCELED);
    // Get File instance (9) and Check it
    check_file(fileDAO,FILE9_ID,"FILE9",File::S_WAITING);

    // Get Transfer instance (8) and Check it
    check_transfer(transferDAO,FILE8_ID,REQ8_ID,"FILE8,REQ8",Transfer::S_ABORTED);

    //----------------------------------------------------------------------
    // Job 5
    //----------------------------------------------------------------------

    // Get Job instance (5) and Check it
    check_job(jobDAO,JOB5_ID,"JOB5",Job::S_CANCELED);

    // Get File instance (10) and Check it
    check_file(fileDAO,FILE10_ID,"FILE10",File::S_CANCELED);
    // Get File instance (11) and Check it
    check_file(fileDAO,FILE11_ID,"FILE11",File::S_CANCELED);
    // Get File instance (12) and Check it
    check_file(fileDAO,FILE12_ID,"FILE12",File::S_CANCELED);
    // Get File instance (13) and Check it
    check_file(fileDAO,FILE13_ID,"FILE13",File::S_FAILED);

    // Get Transfer instance (11) and Check it
    check_transfer(transferDAO,FILE11_ID,REQ11_ID,"FILE11,REQ11",Transfer::S_ABORTED);

    // Check that tha Active Transfer Cache is empty
    unsigned long count = ActionsConfig::instance().activeTxCache().count();
    CPPUNIT_ASSERT_MESSAGE("There are transfer still in teh cache", count == 0);
}
