/*
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *  GLite Data Transfer Agents: ChannelCache  Test Case
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *  Version info: $Id: ChannelCacheTestCase.cpp,v 1.4 2008/11/13 10:43:56 ptedesco Exp $
 *  Release: $Name:  $
 *
 */
#include "glite/data/transfer/agent/action/vo/ChannelCache.h"
#include "DaoHelper.h"
#include "glite/data/transfer/agent/model/Channel.h"
#include "glite/data/transfer/agent/dao/vo/ChannelDAO.h"
#include "glite/data/transfer/agent/dao/vo/DAOFactory.h"
#include "glite/data/agents/AgentExceptions.h"

#include <boost/scoped_ptr.hpp>
#include <boost/shared_ptr.hpp>
#include <cppunit/TestCase.h>
#include <cppunit/extensions/HelperMacros.h>

using namespace glite::data::transfer::agent;
using namespace glite::data::transfer::agent::model;
using namespace glite::data::transfer::agent::dao;
using namespace glite::data::transfer::agent::action::vo;

using glite::data::agents::LogicError;
using glite::data::agents::RuntimeError;

using boost::scoped_ptr;
using boost::shared_ptr;

/**
 * CPPUnit TestCase
 */
class ChannelCacheTestCase : public CppUnit::TestCase{
public:
    /**
     * set-up test case
     */
    void setUp ();
private:
    // Declare CPPUnit Methods
    // -------------------------------------------------------------------------
    CPPUNIT_TEST_SUITE( ChannelCacheTestCase);
    CPPUNIT_TEST( testGet );
    CPPUNIT_TEST( testGetFromSites );
    CPPUNIT_TEST( testCount );
    CPPUNIT_TEST( testRemove );
    CPPUNIT_TEST( testIsMissing );
    CPPUNIT_TEST( testRemoveMissing );
    CPPUNIT_TEST( testGetSiteGroups );
    CPPUNIT_TEST( testRemoveSiteGroups );
    CPPUNIT_TEST( testPurge );
    CPPUNIT_TEST( testClear );
    CPPUNIT_TEST_SUITE_END();

    // Test Methods
    // -------------------------------------------------------------------------

    /**
     * Test the Get Method
     */
    void testGet();
    /**
     * Test the GetFromSites Method
     */
    void testGetFromSites();
    /**
     * Test the Count Method
     */
    void testCount();
    /**
     * Test the Remove Method
     */
    void testRemove();
    /**
     * Test the Is Missing
     */
    void testIsMissing();
    /**
     * Test the RemoveMissing for a VO
     */
    void testRemoveMissing();
    /**
     * Test the getSiteGroups
     */
    void testGetSiteGroups();
    /**
     * Test the RemoveSiteGroups for a VO
     */
    void testRemoveSiteGroups();    
    /**
     * Test the Purge Method
     */
    void testPurge();
    /**
     * Test the Clear Method
     */
    void testClear();
    
private:

    ChannelCache m_cache;
};

/*
 * Register Suit
 */
CPPUNIT_TEST_SUITE_NAMED_REGISTRATION( ChannelCacheTestCase, "ChannelCacheTestCase" );

namespace {
const char * const CHANNEL_NAME_1 = "CHANNELCACHE_channel_1";
const char * const CHANNEL_NAME_2 = "CHANNELCACHE_channel_2";
const char * const CHANNEL_NAME_3 = "CHANNELCACHE_channel_3";
const char * const CHANNEL_NAME_4 = "CHANNELCACHE_channel_4";
const char * const CHANNEL_NAME_5 = "CHANNELCACHE_channel_5";
const char * const VO_NAME        = "CHANNELCACHE_VO_1";
const char * const CERN_SITE      = "TEST_CC.CERN.CH";
const char * const RAL_SITE       = "TEST_CC.RAL.AC.UK";
const char * const NIKHEF_SITE    = "TEST_CC.NIKHEF.NL";
const char * const INFN_SITE      = "TEST_CC.INFN.IT";
const char * const IN2P3_SITE     = "TEST_CC.IN2P3.FR";
const char * const FNAL_SITE      = "TEST_CC.FNAL.GOV";
const char * const BNL_SITE       = "TEST_CC.BNL.GOV";
const char * const IMPERIAL_SITE  = "TEST_CC.IMPERIAL.AC.UK";
}

/*
 * setUp
 *
 * set-up test case
 */
void ChannelCacheTestCase::setUp(){
    

    // Get DAOs
    
    // Create FileDAO instance
    scoped_ptr<dao::vo::ChannelDAO> channelDAO(dao::vo::DAOFactory::instance().createChannelDAO(VO_NAME, DaoHelper::ctx()));
    CPPUNIT_ASSERT_MESSAGE("Null ChannelDAO", channelDAO.get() != 0);

    // Set Cache TTL
    m_cache.ttl(5);
    
    // Insert Valus into the Cache
    
    // Channel1
    shared_ptr<Channel> channel(channelDAO->get(CHANNEL_NAME_1));
    CPPUNIT_ASSERT_MESSAGE("Null Channel instance (CH1)", channel.get() != 0);
    m_cache.insert(CERN_SITE,RAL_SITE,channel);
    // Channel2
    channel.reset(channelDAO->get(CHANNEL_NAME_2));
    CPPUNIT_ASSERT_MESSAGE("Null Channel instance (CH2)", channel.get() != 0);
    m_cache.insert(CERN_SITE,NIKHEF_SITE,channel);
    // Channel3
    channel.reset(channelDAO->get(CHANNEL_NAME_3));
    CPPUNIT_ASSERT_MESSAGE("Null Channel instance (CH3)", channel.get() != 0);
    m_cache.insert(CERN_SITE,INFN_SITE,channel);
    // Channel4
    channel.reset(channelDAO->get(CHANNEL_NAME_4));
    CPPUNIT_ASSERT_MESSAGE("Null Channel instance (CH4)", channel.get() != 0);
    m_cache.insert(CERN_SITE,IN2P3_SITE,channel);
    // Channel5
    channel.reset(channelDAO->get(CHANNEL_NAME_5));
    CPPUNIT_ASSERT_MESSAGE("Null Channel instance (CH5)", channel.get() != 0);
    m_cache.insert(CERN_SITE,FNAL_SITE,channel);
    m_cache.insert(CERN_SITE,BNL_SITE,channel);
    
    // Remember missing
    m_cache.rememberMissingChannel(CERN_SITE,IMPERIAL_SITE);

    std::vector<std::string> groups;
    groups.push_back("[Tier0]");
    groups.push_back("[LCG]");
    // Remember groups
    m_cache.rememberSiteGroups(CERN_SITE,groups);
    // Remember group
    m_cache.rememberSiteGroup(CERN_SITE,"[WLCG]");
    
    groups.clear();
    // Remember groups
    m_cache.rememberSiteGroups(IMPERIAL_SITE,groups);


}

/*
 * Test testGet Method
 */
void ChannelCacheTestCase::testGet(){
    
    shared_ptr<Channel> channel;

    // Channel1
    channel = m_cache.get(CHANNEL_NAME_1);
    CPPUNIT_ASSERT_MESSAGE("Null pointer (CH1)", channel.get() != 0);
    CPPUNIT_ASSERT_MESSAGE("Invalid Channel name (CH1)", channel->name() == CHANNEL_NAME_1);
    CPPUNIT_ASSERT_MESSAGE("Invalid Channel Src  (CH1)", channel->sourceSite() == CERN_SITE);
    CPPUNIT_ASSERT_MESSAGE("Invalid Channel Dst  (CH1)", channel->destSite() == RAL_SITE);
    // Channel2
    channel = m_cache.get(CHANNEL_NAME_2);
    CPPUNIT_ASSERT_MESSAGE("Null pointer (CH2)", channel.get() != 0);
    CPPUNIT_ASSERT_MESSAGE("Invalid Channel name (CH2)", channel->name() == CHANNEL_NAME_2);
    CPPUNIT_ASSERT_MESSAGE("Invalid Channel Src  (CH2)", channel->sourceSite() == CERN_SITE);
    CPPUNIT_ASSERT_MESSAGE("Invalid Channel Dst  (CH2)", channel->destSite() == NIKHEF_SITE);
    // Channel3
    channel = m_cache.get(CHANNEL_NAME_3);
    CPPUNIT_ASSERT_MESSAGE("Null pointer (CH3)", channel.get() != 0);
    CPPUNIT_ASSERT_MESSAGE("Invalid Channel name (CH3)", channel->name() == CHANNEL_NAME_3);
    CPPUNIT_ASSERT_MESSAGE("Invalid Channel Src  (CH3)", channel->sourceSite() == CERN_SITE);
    CPPUNIT_ASSERT_MESSAGE("Invalid Channel Dst  (CH3)", channel->destSite() == INFN_SITE);
    // Channel4
    channel = m_cache.get(CHANNEL_NAME_4);
    CPPUNIT_ASSERT_MESSAGE("Null pointer (CH4)", channel.get() != 0);
    CPPUNIT_ASSERT_MESSAGE("Invalid Channel name (CH4)", channel->name() == CHANNEL_NAME_4);
    CPPUNIT_ASSERT_MESSAGE("Invalid Channel Src  (CH4)", channel->sourceSite() == CERN_SITE);
    CPPUNIT_ASSERT_MESSAGE("Invalid Channel Dst  (CH4)", channel->destSite() == IN2P3_SITE);
    // Channel5
    channel = m_cache.get(CHANNEL_NAME_5);
    CPPUNIT_ASSERT_MESSAGE("Null pointer (CH5)", channel.get() != 0);
    CPPUNIT_ASSERT_MESSAGE("Invalid Channel name (CH5)", channel->name() == CHANNEL_NAME_5);
    CPPUNIT_ASSERT_MESSAGE("Invalid Channel Src  (CH5)", channel->sourceSite() == CERN_SITE);
    CPPUNIT_ASSERT_MESSAGE("Invalid Channel Dst  (CH5)", channel->destSite() == "*");


}

/*
 * Test the GetFromSites Method
 */
void ChannelCacheTestCase::testGetFromSites(){
    shared_ptr<Channel> channel;

    // Channel1
    channel = m_cache.getFromSites(CERN_SITE,RAL_SITE);
    CPPUNIT_ASSERT_MESSAGE("Null pointer (CH1)", channel.get() != 0);
    CPPUNIT_ASSERT_MESSAGE("Invalid Channel name (CH1)", channel->name() == CHANNEL_NAME_1);
    // Channel2
    channel = m_cache.getFromSites(CERN_SITE,NIKHEF_SITE);
    CPPUNIT_ASSERT_MESSAGE("Null pointer (CH2)", channel.get() != 0);
    CPPUNIT_ASSERT_MESSAGE("Invalid Channel name (CH2)", channel->name() == CHANNEL_NAME_2);
    // Channel3
    channel = m_cache.getFromSites(CERN_SITE,INFN_SITE);
    CPPUNIT_ASSERT_MESSAGE("Null pointer (CH3)", channel.get() != 0);
    CPPUNIT_ASSERT_MESSAGE("Invalid Channel name (CH3)", channel->name() == CHANNEL_NAME_3);
    // Channel4
    channel = m_cache.getFromSites(CERN_SITE,IN2P3_SITE);
    CPPUNIT_ASSERT_MESSAGE("Null pointer (CH4)", channel.get() != 0);
    CPPUNIT_ASSERT_MESSAGE("Invalid Channel name (CH4)", channel->name() == CHANNEL_NAME_4);
    // Channel5
    channel = m_cache.getFromSites(CERN_SITE,FNAL_SITE);
    CPPUNIT_ASSERT_MESSAGE("Null pointer (CH5)", channel.get() != 0);
    CPPUNIT_ASSERT_MESSAGE("Invalid Channel name (CH5)", channel->name() == CHANNEL_NAME_5);
    // Channel5
    channel = m_cache.getFromSites(CERN_SITE,BNL_SITE);
    CPPUNIT_ASSERT_MESSAGE("Null pointer (CH5)", channel.get() != 0);
    CPPUNIT_ASSERT_MESSAGE("Invalid Channel name (CH5)", channel->name() == CHANNEL_NAME_5);

    try{
        channel = m_cache.getFromSites(CERN_SITE,IMPERIAL_SITE);
        CPPUNIT_ASSERT_MESSAGE("This channel should not exist", channel.get() == 0);
    } catch(const RuntimeError& exc){
        // OK
    }
}

/*
 * Test the Count Method
 */
void ChannelCacheTestCase::testCount(){

    unsigned int count = m_cache.count();
    CPPUNIT_ASSERT_MESSAGE("Invalid count", count == 6);

}

/*
 * Test the Remove Method
 */
void ChannelCacheTestCase::testRemove(){

    // Remove Channel2
    m_cache.remove(CERN_SITE,NIKHEF_SITE);

    
    try{
        // Channel2
        shared_ptr<Channel> channel = m_cache.getFromSites(CERN_SITE,NIKHEF_SITE);
        CPPUNIT_ASSERT_MESSAGE("This channel should not exist (CH2)", channel.get() == 0);
    } catch(const RuntimeError& exc){
        // OK
    }
}


/*
 * Test the Is Missing
 */
void ChannelCacheTestCase::testIsMissing(){

    // Check missing
    bool is_missing = m_cache.isChannelMissing(CERN_SITE,IMPERIAL_SITE);
    CPPUNIT_ASSERT_MESSAGE("(CM1) should be available", is_missing == true);
    // Check missing
    is_missing = m_cache.isChannelMissing(IMPERIAL_SITE,CERN_SITE);
    CPPUNIT_ASSERT_MESSAGE("(CM2) should not be available", is_missing == false);

}

/*
 * Test the RemoveMissing for a VO
 */
void ChannelCacheTestCase::testRemoveMissing(){

    // Remove missing
    m_cache.removeMissingChannel(CERN_SITE,IMPERIAL_SITE);
    // Check missing
    bool is_missing = m_cache.isChannelMissing(CERN_SITE,IMPERIAL_SITE);
    CPPUNIT_ASSERT_MESSAGE("(CM1) should not be available", is_missing == false);

}

/**
 * Test the getSiteGroups
 */
void ChannelCacheTestCase::testGetSiteGroups(){
    std::vector<std::string> groups;

    // Get groups
    m_cache.getSiteGroups(CERN_SITE,groups);
    CPPUNIT_ASSERT_MESSAGE("Invalid count #1", groups.size() == 3);
    CPPUNIT_ASSERT_MESSAGE("Invalid group #1.0", groups.at(0) == "[Tier0]");
    CPPUNIT_ASSERT_MESSAGE("Invalid group #1.1", groups.at(1) == "[LCG]");
    CPPUNIT_ASSERT_MESSAGE("Invalid group #1.2", groups.at(2) == "[WLCG]");
    
    groups.clear();
    // Get groups
    m_cache.getSiteGroups(IMPERIAL_SITE,groups);
    CPPUNIT_ASSERT_MESSAGE("Invalid count #2", groups.empty() == true);
    


    try{    
        groups.clear();
        // Get groups
        m_cache.getSiteGroups(NIKHEF_SITE,groups);
        CPPUNIT_FAIL("this site has no group");
    } catch(const RuntimeError& exc){
        // OK
    }

}
/**
 * Test the RemoveSiteGroups for a VO
 */
void ChannelCacheTestCase::testRemoveSiteGroups(){

    std::vector<std::string> groups;

    groups.push_back("[WLCG]");
    groups.push_back("[LCG]");
    // Remove groups
    m_cache.removeSiteGroups(CERN_SITE,groups);
    
    groups.clear();
    // Check groups
    m_cache.getSiteGroups(CERN_SITE,groups);
    CPPUNIT_ASSERT_MESSAGE("Invalid count", groups.size() == 1);
    
    // Remove group
    m_cache.removeSiteGroup(CERN_SITE,"[Tier0]");
    
    try{
        // groups
        m_cache.getSiteGroups(CERN_SITE,groups);
        CPPUNIT_FAIL("Site should not be in cache anymore");
    } catch(const RuntimeError& exc){
        // OK
    }

}

/*
 * Test the Purge Method
 */
void ChannelCacheTestCase::testPurge(){

    // Invoke purge
    m_cache.purge();
    // Check count
    unsigned int count = m_cache.count();
    CPPUNIT_ASSERT_MESSAGE("purge should not delete", count == 6);
    // purge again
    m_cache.purge();
    // Check count
    count = m_cache.count();
    CPPUNIT_ASSERT_MESSAGE("purge should not delete since the second purge should be skipped", count == 6);
    
    std::vector<std::string> groups;
    // Get groups
    m_cache.getSiteGroups(CERN_SITE,groups);
    CPPUNIT_ASSERT_MESSAGE("Invalid count #1", groups.size() == 3);
    
    groups.clear();
    // Get groups
    m_cache.getSiteGroups(IMPERIAL_SITE,groups);
    CPPUNIT_ASSERT_MESSAGE("Invalid count #2", groups.empty() == true);

    // Sleep for a while
    sleep(6);
    
    // Refresh groups
    m_cache.rememberSiteGroup(CERN_SITE,"[Tier0]");
    
    // purge again
    m_cache.purge(false);

    // Check count
    count = m_cache.count();
    CPPUNIT_ASSERT_MESSAGE("purge should now work", count == 0);
    
    groups.clear();
    // Get groups
    m_cache.getSiteGroups(CERN_SITE,groups);
    CPPUNIT_ASSERT_MESSAGE("Invalid count #1", groups.size() == 1);
    
    try{        
        groups.clear();
        // Get groups
        m_cache.getSiteGroups(IMPERIAL_SITE,groups);
        CPPUNIT_FAIL("This site should not exist (S2)");
    } catch(const RuntimeError& exc){
        // OK
    }
    

}

/*
 * Test the Purge Method
 */
void ChannelCacheTestCase::testClear(){

    // Remember missing
    m_cache.rememberMissingChannel(CERN_SITE,IMPERIAL_SITE);
    
    // Check Channel1
    shared_ptr<Channel> channel = m_cache.get(CHANNEL_NAME_1);
    CPPUNIT_ASSERT_MESSAGE("Null pointer (CH1)", channel.get() != 0);

    // Check missing
    bool is_missing = m_cache.isChannelMissing(CERN_SITE,IMPERIAL_SITE);
    CPPUNIT_ASSERT_MESSAGE("(CM1) should be available", is_missing == true);
    
    std::vector<std::string> groups;
    // Get groups
    m_cache.getSiteGroups(CERN_SITE,groups);
    CPPUNIT_ASSERT_MESSAGE("Invalid count #1", groups.size() == 3);
    
    groups.clear();
    // Get groups
    m_cache.getSiteGroups(IMPERIAL_SITE,groups);
    CPPUNIT_ASSERT_MESSAGE("Invalid count #2", groups.empty() == true);

    // Invoke clear
    m_cache.clear();
    // Check count
    unsigned int count = m_cache.count();
    CPPUNIT_ASSERT_MESSAGE("clear should delete", count == 0);

    try{
        // Channel2
        channel = m_cache.get(CHANNEL_NAME_1);
        CPPUNIT_FAIL("This channel should not exist (CH1)");
    } catch(const RuntimeError& exc){
        // OK
    }

    // Check missing
    is_missing = m_cache.isChannelMissing(CERN_SITE,IMPERIAL_SITE);
    CPPUNIT_ASSERT_MESSAGE("(CM1) should be available", is_missing == false);

    try{
        groups.clear();
        // Get groups
        m_cache.getSiteGroups(CERN_SITE,groups);
        CPPUNIT_FAIL("This site should not exist (S1)");
    } catch(const RuntimeError& exc){
        // OK
    }

    try{        
        groups.clear();
        // Get groups
        m_cache.getSiteGroups(IMPERIAL_SITE,groups);
        CPPUNIT_FAIL("This site should not exist (S2)");
    } catch(const RuntimeError& exc){
        // OK
    }

}
