/**
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *  GLite Data Transfer Agents: CheckReadiness Test Case
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>, Paolo Tedesco <paolo.tedesco@cern.ch>
 *  Version info: $Id: CheckReadinessTestCase.cpp,v 1.7 2008/04/11 11:02:56 ptedesco Exp $
 *  Release: $Name:  $
 *
 */
#include "CheckReadiness.h"
#include "JobFlags.h"
#include "DaoHelper.h"
#include "glite/data/transfer/agent/action/ActionFactory.h"
#include "glite/data/transfer/agent/model/File.h"
#include "glite/data/transfer/agent/model/Job.h"
#include "glite/data/transfer/agent/dao/vo/FileDAO.h"
#include "glite/data/transfer/agent/dao/vo/JobDAO.h"
#include "glite/data/transfer/agent/dao/vo/DAOFactory.h"

#include "dao_test_utils.h"

#include <memory>
#include <boost/scoped_ptr.hpp>
#include <cppunit/TestCase.h>
#include <cppunit/extensions/HelperMacros.h>

using namespace glite::data::transfer::agent;
using namespace glite::data::transfer::agent::action;
using namespace glite::data::transfer::agent::action::vo;
using namespace glite::data::transfer::agent::model;

using boost::scoped_ptr;

/**
 * CPPUnit TestCase
 */
class CheckReadinessTestCase : public CppUnit::TestCase{
public:
    /**
     * set-up test case
     */
    void setUp ();
private:
    // Declare CPPUnit Methods
    // -------------------------------------------------------------------------
    CPPUNIT_TEST_SUITE( CheckReadinessTestCase);
    CPPUNIT_TEST( testExecute );
    CPPUNIT_TEST_SUITE_END();

    // Test Methods
    // -------------------------------------------------------------------------

    /**
     * Test Execute Method
     */
    void testExecute();
    
private:
    /**
     * The Action Instance
     */
    scoped_ptr<CheckReadiness> m_action;
};

/*
 * Register Suit
 */
CPPUNIT_TEST_SUITE_NAMED_REGISTRATION( CheckReadinessTestCase, "CheckReadinessTestCase" );

/*
 * Declare Constants. This unit test foresee at beginning
 * (1) - 1 PENDING Job with:                      => READY
 *        job_id      = 4-...-00
 *        files       = 40000001 (PENDING)       READY
 *                      40000002 (PENDING)       READY
 *
 * (2) - 1 PENDING Job with:                      => READY
 *        job_id      = 4-...-01
 *        files       = 40000003 (FAILED)        FAILED
 *                      40000004 (PENDING)       READY
 *
 * (3) - 1 PENDING Job with:                      => 
 *        job_id      = 4-...-02
 *        files       = 40000005 (AWAITING_P)    
 *                      40000006 (PENDING)       READY
 *
 * (4) - 1 PENDING Job with:                      => 
 *        job_id      = 4-...-03
 *        files       = 40000007 (WAITING_P)    
 *                      40000008 (PRESTAGING)  
 *
 * (5) - 1 ACTIVE Job with:                      => 
 *        job_id      = 4-...-04
 *        files       = 40000009 (PENDING)       READY
 *                      40000010 (ACTIVE)        
 *
 * (6) - 1 PENDING Job with:                     => CANCELED (cancel_job = true)
 *        job_id      = 4-...-05
 *        files       = 40000011 (FAILED)       
 *                      40000012 (HOLD)         CANCELED
 *
 * (7) - 1 PENDING Job with:                     => CANCELED (cancel_job = true)
 *        job_id      = 4-...-06
 *        files       = 40000013 (PENDING)      CANCELED 
 *                      40000014 (HOLD)         CANCELED
 *
 * (8) - 1 PENDING Job with:                     => READY    (cancel_job = true)
 *        job_id      = 4-...-07
 *        files       = 40000015 (PENDING)      CANCELED 
 *                      40000016 (READY)        
 *                      40000017 (HOLD)         CANCELED
 *
 * (9) - 1 ACTIVE Job with:                      =>          (cancel_job = true)
 *        job_id      = 4-...-08
 *        files       = 40000018 (PENDING)      CANCELED 
 *                      40000019 (ACTIVE)        
 *                      40000020 (HOLD)         
 */
namespace {
const char * const JOB1_ID         = "40000000-0000-0000-0000-000000000000";
const char * const JOB2_ID         = "40000000-0000-0000-0000-000000000001";
const char * const JOB3_ID         = "40000000-0000-0000-0000-000000000002";
const char * const JOB4_ID         = "40000000-0000-0000-0000-000000000003";
const char * const JOB5_ID         = "40000000-0000-0000-0000-000000000004";
const char * const JOB6_ID         = "40000000-0000-0000-0000-000000000005";
const char * const JOB7_ID         = "40000000-0000-0000-0000-000000000006";
const char * const JOB8_ID         = "40000000-0000-0000-0000-000000000007";
const char * const JOB9_ID         = "40000000-0000-0000-0000-000000000008";
const char * const FILE1_ID        = "40000001";
const char * const FILE2_ID        = "40000002";
const char * const FILE3_ID        = "40000003";
const char * const FILE4_ID        = "40000004";
const char * const FILE5_ID        = "40000005";
const char * const FILE6_ID        = "40000006";
const char * const FILE7_ID        = "40000007";
const char * const FILE8_ID        = "40000008";
const char * const FILE9_ID        = "40000009";
const char * const FILE10_ID       = "40000010";
const char * const FILE11_ID       = "40000011";
const char * const FILE12_ID       = "40000012";
const char * const FILE13_ID       = "40000013";
const char * const FILE14_ID       = "40000014";
const char * const FILE15_ID       = "40000015";
const char * const FILE16_ID       = "40000016";
const char * const FILE17_ID       = "40000017";
const char * const FILE18_ID       = "40000018";
const char * const FILE19_ID       = "40000019";
const char * const FILE20_ID       = "40000020";
const char * const VO1_NAME        = "CHECK_R_VO_1";
}

/*
 * setUp
 *
 * set-up test case
 */
void CheckReadinessTestCase::setUp(){
    
    // Create Action instance
    std::auto_ptr<Action> a(ActionFactory::instance().create("glite:CheckReadiness"));
    CPPUNIT_ASSERT_MESSAGE("Null Action", a.get() != 0);
    CPPUNIT_ASSERT_MESSAGE("Invalid Action Class", dynamic_cast<CheckReadiness*>(a.get()) != 0);

    // Store Action        
    m_action.reset(dynamic_cast<CheckReadiness*>(a.release()));
    CPPUNIT_ASSERT_MESSAGE("Null Action", m_action.get() != 0);
}

/*
 *
 * Test Execute Method
 */
void CheckReadinessTestCase::testExecute(){
    
    // Set Context
    m_action->context(VO1_NAME,DaoHelper::ctx());
    
    // Execute Actions
    m_action->execute();
    
    // Create FileDAO instance
    scoped_ptr<dao::vo::FileDAO> fileDAO(dao::vo::DAOFactory::instance().createFileDAO(VO1_NAME, DaoHelper::ctx()));
    CPPUNIT_ASSERT_MESSAGE("Null FileDAO", fileDAO.get() != 0);
    
    // Create JobDAO instance
    scoped_ptr<dao::vo::JobDAO> jobDAO(dao::vo::DAOFactory::instance().createJobDAO(VO1_NAME, DaoHelper::ctx()));
    CPPUNIT_ASSERT_MESSAGE("Null JobDAO", jobDAO.get() != 0);

    // Check The Transfers using FileDAO
    
    //----------------------------------------------------------------------
    // Job 1
    //----------------------------------------------------------------------
    
    // Get Job instance (1) and Check it
    check_job(jobDAO,JOB1_ID,"JOB1",Job::S_READY);
    
    // Get File instance (1) and Check it
    check_file(fileDAO,FILE1_ID,"FILE1",File::S_READY);
    // Get File instance (2) and Check it
    check_file(fileDAO,FILE2_ID,"FILE2",File::S_READY);

    //----------------------------------------------------------------------
    // Job 2
    //----------------------------------------------------------------------

    // Get Job instance (2) and Check it
    check_job(jobDAO,JOB2_ID,"JOB2",Job::S_READY);

    // Get File instance (3) and Check it
    check_file(fileDAO,FILE3_ID,"FILE3",File::S_FAILED);
    // Get File instance (4) and Check it
    check_file(fileDAO,FILE4_ID,"FILE4",File::S_READY);

    //----------------------------------------------------------------------
    // Job 3
    //----------------------------------------------------------------------
    
    // Get Job instance (3) and Check it
    check_job(jobDAO,JOB3_ID,"JOB3",Job::S_PENDING);
    // Get File instance (5) and Check it
    check_file(fileDAO,FILE5_ID,"FILE5",File::S_AWAITING_PRESTAGE);
    // Get File instance (6) and Check it
    check_file(fileDAO,FILE6_ID,"FILE6",File::S_READY);

    //----------------------------------------------------------------------
    // Job 4
    //----------------------------------------------------------------------
    
    // Get Job instance (4) and Check it
    check_job(jobDAO,JOB4_ID,"JOB4",Job::S_PENDING);

    // Get File instance (7) and Check it
    check_file(fileDAO,FILE7_ID,"FILE7",File::S_WAITING_PRESTAGE);
    // Get File instance (8) and Check it
    check_file(fileDAO,FILE8_ID,"FILE8",File::S_PRESTAGING);

    //----------------------------------------------------------------------
    // Job 5
    //----------------------------------------------------------------------
    
    // Get Job instance (5) and Check it
    check_job(jobDAO,JOB5_ID,"JOB5",Job::S_ACTIVE);

    // Get File instance (9) and Check it
    check_file(fileDAO,FILE9_ID,"FILE9",File::S_READY);
    // Get File instance (10) and Check it
    check_file(fileDAO,FILE10_ID,"FILE10",File::S_ACTIVE);

    //----------------------------------------------------------------------
    // Job 6
    //----------------------------------------------------------------------
    
    // Get Job instance (6) and Check it
    check_job(jobDAO,JOB6_ID,"JOB6",Job::S_CANCELED);
    
    // Get File instance (11) and Check it
    check_file(fileDAO,FILE11_ID,"FILE11",File::S_FAILED);
    // Get File instance (12) and Check it
    check_file(fileDAO,FILE12_ID,"FILE12",File::S_CANCELED);
    
    //----------------------------------------------------------------------
    // Job 7
    //----------------------------------------------------------------------
    
    // Get Job instance (7) and Check it
    check_job(jobDAO,JOB7_ID,"JOB7",Job::S_CANCELED);
    
    // Get File instance (13) and Check it
    check_file(fileDAO,FILE13_ID,"FILE13",File::S_CANCELED);
    // Get File instance (14) and Check it
    check_file(fileDAO,FILE14_ID,"FILE14",File::S_CANCELED);
    
    //----------------------------------------------------------------------
    // Job 8
    //----------------------------------------------------------------------
    
    // Get Job instance (8) and Check it
    check_job(jobDAO,JOB8_ID,"JOB8",Job::S_READY);

    // Get File instance (15) and Check it
    check_file(fileDAO,FILE15_ID,"FILE15",File::S_CANCELED);
    // Get File instance (16) and Check it
    check_file(fileDAO,FILE16_ID,"FILE16",File::S_READY);
    // Get File instance (17) and Check it
    check_file(fileDAO,FILE17_ID,"FILE17",File::S_CANCELED);
    
    //----------------------------------------------------------------------
    // Job 9
    //----------------------------------------------------------------------
    
    // Get Job instance (9) and Check it
    check_job(jobDAO,JOB9_ID,"JOB9",Job::S_ACTIVE);
    
    // Get File instance (18) and Check it
    check_file(fileDAO,FILE18_ID,"FILE18",File::S_CANCELED);
    // Get File instance (19) and Check it
    check_file(fileDAO,FILE19_ID,"FILE19",File::S_ACTIVE);
    // Get File instance (20) and Check it
    check_file(fileDAO,FILE20_ID,"FILE20",File::S_HOLD);

}
