/**
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *  GLite Data Transfer Agents: CheckState Test Case
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>, Paolo Tedesco <paolo.tedesco@cern.ch>
 *  Version info: $Id: CheckStateTestCase.cpp,v 1.11 2009/07/02 15:21:17 molnarzs Exp $
 *  Release: $Name:  $
 *
 */
#include "CheckState.h"
#include "actions/channel/ActionsConfig.h"
#include "DaoHelper.h"
#include "glite/data/transfer/agent/action/ActionFactory.h"
#include "glite/data/transfer/agent/model/File.h"
#include "glite/data/transfer/agent/model/Job.h"
#include "glite/data/transfer/agent/model/Transfer.h"
#include "glite/data/transfer/agent/model/Channel.h"
#include "glite/data/transfer/agent/dao/channel/FileDAO.h"
#include "glite/data/transfer/agent/dao/channel/JobDAO.h"
#include "glite/data/transfer/agent/dao/channel/TransferDAO.h"
#include "glite/data/transfer/agent/dao/channel/DAOFactory.h"
#include "glite/data/agents/dao/DAOException.h"
#include "glite/data/transfer/agent/ts/TransferService.h"
#include "glite/data/transfer/agent/ts/TransferServiceFactory.h"

#include "dao_test_utils.h"

#include <memory>
#include <boost/scoped_ptr.hpp>
#include <cppunit/TestCase.h>
#include <cppunit/extensions/HelperMacros.h>

using namespace glite::data::transfer::agent;
using namespace glite::data::transfer::agent::action;
using namespace glite::data::transfer::agent::action::channel;
using namespace glite::data::transfer::agent::model;

using boost::scoped_ptr;

/**
 * CPPUnit TestCase
 */
class CheckStateTestCase : public CppUnit::TestCase{
public:
    /**
     * set-up test case
     */
    void setUp ();

    /**
     * tear-down test case
     */
    void tearDown ();

private:
    // Declare CPPUnit Methods
    // -------------------------------------------------------------------------
    CPPUNIT_TEST_SUITE( CheckStateTestCase);
    CPPUNIT_TEST( testExecute );
    CPPUNIT_TEST_SUITE_END();

    // Test Methods
    // -------------------------------------------------------------------------

    /**
     * Test Execute Method
     */
    void testExecute();

private:
    /**
     * The Action Instance
     */
    scoped_ptr<CheckState> m_action;
};

/*
 * Register Suit
 */
CPPUNIT_TEST_SUITE_NAMED_REGISTRATION( CheckStateTestCase, "CheckStateTestCase" );

/*
 * Declare Constants. This unit test foresee at beginning
 *
 * (1) - 1 ACTIVE Job with:                     =>  DONE
 *        job_id  = 2-...-01
 *        void    = VO_1
 *        channel = channel_2
 *        files   = 20000001 (ACTIVE) (req_2_1) DONE
 *                  20000002 (ACTIVE) (req_2_2) DONE
 *
 * (2) - 1 ACTIVE Job with:                    =>
 *        job_id  = 2-...-02
 *        void    = VO_1
 *        channel = channel_2
 *        files   = 20000003 (ACTIVE) (req_2_3) DONE
 *                  20000004 (WAITING)(req_2_4)
 *
 * (3) - 1 ACTIVE Job with:                    =>   FAILED
 *        job_id  = 2-...-03
 *        void    = VO_2
 *        channel = CHECKSTATE_channel_1
 *        files   = 20000005 (ACTIVE)(req_2_5)  DONE
 *                  20000006 (DONE)  (req_2_6)
 *                  20000007 (FAILED)(req_2_7)
 *
 * (4) - 1 ACTIVE Job with:                  =>
 *        job_id  = 2-...-04
 *        void    = VO_2
 *        channel = CHECKSTATE_channel_1
 *        files   = 20000008 (READY)
 *                  20000009 (ACTIVE)(req_2_9)  DONE
 *
 * (5) - 1 ACTIVE Job with:                    =>
 *        job_id  = 2-...-05
 *        void    = VO_1
 *        channel = CHECKSTATE_channel_1
 *        files   = 20000010 (ACTIVE)(req_2_10) DONE   (* tx in processing)
 *                  20000011 (ACTIVE)(req_2_11) ACTIVE (* tx in receiving)
 *
 * (6) - 1 ACTIVE Job with:                    =>
 *        job_id  = 2-...-06
 *        void    = VO_1
 *        channel = CHECKSTATE_channel_1
 *        files   = 20000012 (ACTIVE)(req_2_12) WAITING (* Failed during processing)
 *                  20000013 (ACTIVE)(req_2_13) DONE
 *
 * (7) - 1 ACTIVE Job with:                    => HOLD
 *        void    = VO_1
 *        channel = CHECKSTATE_channel_1
 *        job_id  = 2-...-07
 *        files   = 20000014 (ACTIVE)(req_2_14) DONE
 *                  20000015 (HOLD)(req_2_15,req_2_15_1)
 *
 * (8) - 1 ACTIVE Job with:                    =>
 *        job_id  = 2-...-08
 *        void    = VO_2
 *        channel = CHECKSTATE_channel_1
 *        files   = 20000016 (READY)(req_2_16)    (*rescheduled)
 *                  20000017 (READY)(req_2_17)    (*rescheduled)
 *
 * (9) - 1 ACTIVE Job with:                    => DONE_WITH_ERRORS
 *        job_id  = 2-...-09
 *        void    = VO_1
 *        channel = CHECKSTATE_channel_1
 *        files   = 20000018 (FAILED)         (*failed by Retry Logic)
 *                  20000019 (DONE)
 *
 * (10) - 1 ACTIVE Job with:                    =>
 *        job_id  = 2-...-10
 *        void    = VO_1
 *        channel = CHECKSTATE_channel_1
 *        files   = 20000020 (DONE)
 *                  20000021 (WAITING)
 *
 * (11) - 1 ACTIVE Job with:                    => HOLD
 *        job_id  = 2-...-11
 *        void    = VO_1
 *        channel = CHECKSTATE_channel_1
 *        files   = 20000022 (FAILED)
 *                  20000023 (DONE)
 *                  20000024 (HOLD)           (*failed by Retry Logic)
 *
 * (12) - 1 ACTIVE Job with:                    => CANCELED (cancel_job set to true)
 *        job_id  = 2-...-12
 *        void    = VO_2
 *        channel = CHECKSTATE_channel_1
 *        files   = 20000025 (CANCELED)
 *                  20000026 (HOLD)           CANCELED
 *                  20000027 (DONE)           CANCELED
 */
namespace {
const char * const JOB1_ID        = "20000000-0000-0000-0000-000000000001";
const char * const JOB2_ID        = "20000000-0000-0000-0000-000000000002";
const char * const JOB3_ID        = "20000000-0000-0000-0000-000000000003";
const char * const JOB4_ID        = "20000000-0000-0000-0000-000000000004";
const char * const JOB5_ID        = "20000000-0000-0000-0000-000000000005";
const char * const JOB6_ID        = "20000000-0000-0000-0000-000000000006";
const char * const JOB7_ID        = "20000000-0000-0000-0000-000000000007";
const char * const JOB8_ID        = "20000000-0000-0000-0000-000000000008";
const char * const JOB9_ID        = "20000000-0000-0000-0000-000000000009";
const char * const JOB10_ID       = "20000000-0000-0000-0000-000000000010";
const char * const JOB11_ID       = "20000000-0000-0000-0000-000000000011";
const char * const JOB12_ID       = "20000000-0000-0000-0000-000000000012";
const char * const FILE1_ID       = "20000001";
const char * const FILE2_ID       = "20000002";
const char * const FILE3_ID       = "20000003";
const char * const FILE4_ID       = "20000004";
const char * const FILE5_ID       = "20000005";
const char * const FILE6_ID       = "20000006";
const char * const FILE7_ID       = "20000007";
const char * const FILE8_ID       = "20000008";
const char * const FILE9_ID       = "20000009";
const char * const FILE10_ID      = "20000010";
const char * const FILE11_ID      = "20000011";
const char * const FILE12_ID      = "20000012";
const char * const FILE13_ID      = "20000013";
const char * const FILE14_ID      = "20000014";
const char * const FILE15_ID      = "20000015";
const char * const FILE16_ID      = "20000016";
const char * const FILE17_ID      = "20000017";
const char * const FILE18_ID      = "20000018";
const char * const FILE19_ID      = "20000019";
const char * const FILE20_ID      = "20000020";
const char * const FILE21_ID      = "20000021";
const char * const FILE22_ID      = "20000022";
const char * const FILE23_ID      = "20000023";
const char * const FILE24_ID      = "20000024";
const char * const FILE25_ID      = "20000025";
const char * const FILE26_ID      = "20000026";
const char * const FILE27_ID      = "20000027";
std::string  REQ1_ID        = "(ReqId auto-gen)";
std::string  REQ2_ID        = "(ReqId auto-gen)";
std::string  REQ3_ID        = "(ReqId auto-gen)";
std::string  REQ4_ID        = "20004";
std::string  REQ5_ID        = "(ReqId auto-gen)";
std::string  REQ6_ID        = "20006";
std::string  REQ7_ID        = "20007";
std::string  REQ8_ID        = "20008";
std::string  REQ9_ID        = "(ReqId auto-gen)";
std::string  REQ10_ID       = "(ReqId auto-gen)";
std::string  REQ11_ID       = "(ReqId auto-gen)";
std::string  REQ12_ID       = "(ReqId auto-gen)";
std::string  REQ13_ID       = "(ReqId auto-gen)";
std::string  REQ14_ID       = "(ReqId auto-gen)";
std::string  REQ15_ID       = "20015";
std::string  REQ15_1_ID     = "20015_1";
std::string  REQ16_ID       = "20016";
std::string  REQ17_ID       = "20017";
const char * const CHANNEL_NAME   = "CHECKSTATE_channel_1";

const char * const TC_SRC_NAME      = "srm://host.cern.ch:8443/tmp/src.txt";
const char * const TC_SRC_FAIL_NAME = "srm://host.cern.ch:8443/tmp/this-file-should-fail-in-processing.txt";
const char * const TC_DEST_NAME     = "srm://host.nikhef.nl:8443/tmp/dst.txt";

const char * const FILE1_SRC        = TC_SRC_NAME;
const char * const FILE2_SRC        = TC_SRC_NAME;
const char * const FILE3_SRC        = TC_SRC_NAME;
const char * const FILE4_SRC        = TC_SRC_NAME;
const char * const FILE5_SRC        = TC_SRC_NAME;
const char * const FILE6_SRC        = TC_SRC_NAME;
const char * const FILE7_SRC        = TC_SRC_NAME;
const char * const FILE8_SRC        = TC_SRC_NAME;
const char * const FILE9_SRC        = TC_SRC_NAME;
const char * const FILE10_SRC       = TC_SRC_NAME;
const char * const FILE11_SRC       = TC_SRC_NAME;
const char * const FILE12_SRC       = TC_SRC_FAIL_NAME;
const char * const FILE13_SRC       = TC_SRC_NAME;
const char * const FILE14_SRC       = TC_SRC_NAME;
const char * const FILE15_SRC       = TC_SRC_NAME;
const char * const FILE16_SRC       = TC_SRC_NAME;
const char * const FILE17_SRC       = TC_SRC_NAME;
}

/*
 * setUp
 *
 * set-up test case
 */
void CheckStateTestCase::setUp(){

    // Reset cache
    ActionsConfig::instance().activeTxCache().reset();

    // Create Action instance
    std::auto_ptr<Action> a(ActionFactory::instance().create("glite:CheckState"));
    CPPUNIT_ASSERT_MESSAGE("Null Action", a.get() != 0);
    CPPUNIT_ASSERT_MESSAGE("Invalid Action Class", dynamic_cast<CheckState*>(a.get()) != 0);

    // Store Action
    m_action.reset(dynamic_cast<CheckState*>(a.release()));
    CPPUNIT_ASSERT_MESSAGE("Null Action", m_action.get() != 0);

    // Create TransferDAO instance
    scoped_ptr<dao::channel::TransferDAO> transferDao(dao::channel::DAOFactory::instance().createTransferDAO(CHANNEL_NAME, DaoHelper::ctx()));
    CPPUNIT_ASSERT_MESSAGE("Null TransferDAO", transferDao.get() != 0);

    // Add requests to TransferService
    scoped_ptr<ts::TransferService> tx_service(ts::TransferServiceFactory::instance().create(CHANNEL_NAME));
    CPPUNIT_ASSERT_MESSAGE("Null TransferService", tx_service.get() != 0);

    // Req 1
    ts::TransferService::TransferRequest  req;
    ts::TransferService::TransferResponse out;
    req.surls.resize(1);
    req.surls[0].source  = FILE1_SRC;
    req.surls[0].destination = TC_DEST_NAME;
    // Submit Transfer
    tx_service->submit(req,out,"");
    scoped_ptr<Transfer> tx(new Transfer(out.request_id,FILE1_ID,out.transfer_ids.at(0),JOB1_ID,Transfer::S_PROCESSING));
    transferDao->create(*tx);
    REQ1_ID = out.request_id;

    // Req 2
    out.transfer_ids.clear();
    req.surls[0].source  = FILE2_SRC;
    tx_service->submit(req,out,"");
    tx.reset(new Transfer(out.request_id,FILE2_ID,out.transfer_ids.at(0),JOB1_ID,Transfer::S_PROCESSING));
    transferDao->create(*tx);
    REQ2_ID = out.request_id;

    // Req 3
    out.transfer_ids.clear();
    req.surls[0].source  = FILE3_SRC;
    tx_service->submit(req,out,"");
    tx.reset(new Transfer(out.request_id,FILE3_ID,out.transfer_ids.at(0),JOB2_ID,Transfer::S_PROCESSING));
    transferDao->create(*tx);
    REQ3_ID = out.request_id;

    // Req 5
    out.transfer_ids.clear();
    req.surls[0].source  = FILE5_SRC;
    tx_service->submit(req,out,"");
    tx.reset(new Transfer(out.request_id,FILE5_ID,out.transfer_ids.at(0),JOB3_ID,Transfer::S_PROCESSING));
    transferDao->create(*tx);
    REQ5_ID = out.request_id;

    // Req 9
    out.transfer_ids.clear();
    req.surls[0].source  = FILE9_SRC;
    tx_service->submit(req,out,"");
    tx.reset(new Transfer(out.request_id,FILE9_ID,out.transfer_ids.at(0),JOB4_ID,Transfer::S_PROCESSING));
    transferDao->create(*tx);
    REQ9_ID = out.request_id;

    // Req 10
    out.transfer_ids.clear();
    req.surls[0].source  = FILE10_SRC;
    tx_service->submit(req,out,"");
    tx.reset(new Transfer(out.request_id,FILE10_ID,out.transfer_ids.at(0),JOB5_ID,Transfer::S_PROCESSING));
    transferDao->create(*tx);
    REQ10_ID = out.request_id;

    // Req 11
    out.transfer_ids.clear();
    req.surls[0].source  = FILE11_SRC;
    tx_service->submit(req,out,"");
    tx.reset(new Transfer(out.request_id,FILE11_ID,out.transfer_ids.at(0),JOB5_ID,Transfer::S_RECEIVED));
    transferDao->create(*tx);
    REQ11_ID = out.request_id;

    // Req 12
    out.transfer_ids.clear();
    req.surls[0].source  = FILE12_SRC;
    tx_service->submit(req,out,"");
    tx.reset(new Transfer(out.request_id,FILE12_ID,out.transfer_ids.at(0),JOB6_ID,Transfer::S_PROCESSING));
    transferDao->create(*tx);
    REQ12_ID = out.request_id;

    // Req 13
    out.transfer_ids.clear();
    req.surls[0].source  = FILE13_SRC;
    tx_service->submit(req,out,"");
    tx.reset(new Transfer(out.request_id,FILE13_ID,out.transfer_ids.at(0),JOB6_ID,Transfer::S_PROCESSING));
    transferDao->create(*tx);
    REQ13_ID = out.request_id;

    // Req 14
    out.transfer_ids.clear();
    req.surls[0].source  = FILE14_SRC;
    tx_service->submit(req,out,"");
    tx.reset(new Transfer(out.request_id,FILE14_ID,out.transfer_ids.at(0),JOB7_ID,Transfer::S_PROCESSING));
    transferDao->create(*tx);
    REQ14_ID = out.request_id;

}

/*
 * tearDown
 *
 * tear-down test case
 */
void CheckStateTestCase::tearDown(){
    ActionsConfig::instance().activeTxCache().reset();
}

/*
 *
 * Test Execute Method
 */
void CheckStateTestCase::testExecute(){

    // Set Context
    m_action->context(CHANNEL_NAME,DaoHelper::ctx());

    // Execute Action
    m_action->execute();

    // Create FileDAO instance
    scoped_ptr<dao::channel::FileDAO> fileDAO(dao::channel::DAOFactory::instance().createFileDAO(CHANNEL_NAME, DaoHelper::ctx()));
    CPPUNIT_ASSERT_MESSAGE("Null FileDAO", fileDAO.get() != 0);

    // Create JobDAO instance
    scoped_ptr<dao::channel::JobDAO> jobDAO(dao::channel::DAOFactory::instance().createJobDAO(CHANNEL_NAME, DaoHelper::ctx()));
    CPPUNIT_ASSERT_MESSAGE("Null JobDAO", jobDAO.get() != 0);

    // Create TransferDAO instance
    scoped_ptr<dao::channel::TransferDAO> transferDAO(dao::channel::DAOFactory::instance().createTransferDAO(CHANNEL_NAME, DaoHelper::ctx()));
    CPPUNIT_ASSERT_MESSAGE("Null TransferDAO", transferDAO.get() != 0);

    // Check The Transfers using FileDAO

    //----------------------------------------------------------------------
    // Job 1
    //----------------------------------------------------------------------

    // Get Job instance (1) and Check it
    check_job(jobDAO,JOB1_ID,"JOB1",Job::S_DONE);

    // Get File instance (1) and Check it
    check_file(fileDAO,FILE1_ID,"FILE1",File::S_DONE);
    // Get File instance (2) and Check it
    check_file(fileDAO,FILE2_ID,"FILE2",File::S_DONE);
    // Get Transfer instance (1) and Check it
    check_transfer(transferDAO,FILE1_ID,REQ1_ID,"FILE1,REQ1",Transfer::S_COMPLETED);
    // Get Transfer instance (2) and Check it
    check_transfer(transferDAO,FILE2_ID,REQ2_ID,"FILE2,REQ2",Transfer::S_COMPLETED);

    //----------------------------------------------------------------------
    // Job 2
    //----------------------------------------------------------------------

    // Get Job instance (2) and Check it
    check_job(jobDAO,JOB2_ID,"JOB2",Job::S_ACTIVE);

    // Get File instance (3) and Check it
    check_file(fileDAO,FILE3_ID,"FILE3",File::S_DONE);
    // Get File instance (4) and Check it
    check_file(fileDAO,FILE4_ID,"FILE4",File::S_WAITING);
    // Get Transfer instance (3) and Check it
    check_transfer(transferDAO,FILE3_ID,REQ3_ID,"FILE3,REQ3",Transfer::S_COMPLETED);
    // Get Transfer instance (4) and Check it
    check_transfer(transferDAO,FILE4_ID,REQ4_ID,"FILE4,REQ4",Transfer::S_FAILED);

    //----------------------------------------------------------------------
    // Job 3
    //----------------------------------------------------------------------

    // Get Job instance (3) and Check it
    check_job(jobDAO,JOB3_ID,"JOB3",Job::S_DONE_WITH_ERRORS);

    // Get File instance (5) and Check it
    check_file(fileDAO,FILE5_ID,"FILE5",File::S_DONE);
    // Get File instance (6) and Check it
    check_file(fileDAO,FILE6_ID,"FILE6",File::S_DONE);
    // Get File instance (7) and Check it
    check_file(fileDAO,FILE7_ID,"FILE7",File::S_FAILED);
    // Get Transfer instance (5) and Check it
    check_transfer(transferDAO,FILE5_ID,REQ5_ID,"FILE5,REQ5",Transfer::S_COMPLETED);
    // Get Transfer instance (6) and Check it
    check_transfer(transferDAO,FILE6_ID,REQ6_ID,"FILE6,REQ6",Transfer::S_COMPLETED);
    // Get Transfer instance (7) and Check it
    check_transfer(transferDAO,FILE7_ID,REQ7_ID,"FILE7,REQ7",Transfer::S_FAILED);

    //----------------------------------------------------------------------
    // Job 4
    //----------------------------------------------------------------------

    // Get Job instance (4) and Check it
    check_job(jobDAO,JOB4_ID,"JOB4",Job::S_ACTIVE);

    // Get File instance (8) and Check it
    check_file(fileDAO,FILE8_ID,"FILE8",File::S_READY);
    // Get File instance (9) and Check it
    check_file(fileDAO,FILE9_ID,"FILE9",File::S_DONE);
    // Transfer instance (8) doesn't exists
    try{
        scoped_ptr<Transfer> transfer(transferDAO->get(FILE8_ID,REQ8_ID));
        CPPUNIT_FAIL("Transfer (FILE8,REQ8) should not exist");
    } catch(const glite::data::agents::dao::DAOException& exc){
        // OK
    }
    // Get Transfer instance (9) and Check it
    check_transfer(transferDAO,FILE9_ID,REQ9_ID,"FILE9,REQ9",Transfer::S_COMPLETED);

    //----------------------------------------------------------------------
    // Job 5
    //----------------------------------------------------------------------

    // Get Job instance (5) and Check it
    check_job(jobDAO,JOB5_ID,"JOB5",Job::S_ACTIVE);

    // Get File instance (10) and Check it
    check_file(fileDAO,FILE10_ID,"FILE10",File::S_DONE);
    // Get File instance (11) and Check it
    check_file(fileDAO,FILE11_ID,"FILE11",File::S_ACTIVE);
    // Get Transfer instance (10) and Check it
    check_transfer(transferDAO,FILE10_ID,REQ10_ID,"FILE10,REQ10",Transfer::S_COMPLETED);
    // Get Transfer instance (11) and Check it
    check_transfer(transferDAO,FILE11_ID,REQ11_ID,"FILE11,REQ11",Transfer::S_PROCESSING);

    //----------------------------------------------------------------------
    // Job 6
    //----------------------------------------------------------------------

    // Get Job instance (6) and Check it
    check_job(jobDAO,JOB6_ID,"JOB6",Job::S_ACTIVE);

    // Get File instance (12) and Check it
    check_file(fileDAO,FILE12_ID,"FILE12",File::S_WAITING);
    // Get File instance (13) and Check it
    check_file(fileDAO,FILE13_ID,"FILE13",File::S_DONE);
    // Get Transfer instance (12) and Check it
    check_transfer(transferDAO,FILE12_ID,REQ12_ID,"FILE12,REQ12",Transfer::S_FAILED);
    // Get Transfer instance (13) and Check it
    check_transfer(transferDAO,FILE13_ID,REQ13_ID,"FILE13,REQ13",Transfer::S_COMPLETED);

    //----------------------------------------------------------------------
    // Job 7
    //----------------------------------------------------------------------

    // Get Job instance (7) and Check it
    check_job(jobDAO,JOB7_ID,"JOB7",Job::S_HOLD);

    // Get File instance (14) and Check it
    check_file(fileDAO,FILE14_ID,"FILE14",File::S_DONE);
    // Get File instance (15) and Check it
    check_file(fileDAO,FILE15_ID,"FILE15",File::S_HOLD);
    // Get Transfer instance (14) and Check it
    check_transfer(transferDAO,FILE14_ID,REQ14_ID,"FILE14,REQ14",Transfer::S_COMPLETED);
    // Get Transfer instance (15) and Check it
    check_transfer(transferDAO,FILE15_ID,REQ15_ID,"FILE15,REQ15",Transfer::S_FAILED);
    // Get Transfer instance (15_1) and Check it
    check_transfer(transferDAO,FILE15_ID,REQ15_1_ID,"FILE15,REQ15_1",Transfer::S_FAILED);

    //----------------------------------------------------------------------
    // Job 8
    //----------------------------------------------------------------------

    // Get Job instance (8) and Check it
    check_job(jobDAO,JOB8_ID,"JOB8",Job::S_ACTIVE);

    // Get File instance (16) and Check it
    check_file(fileDAO,FILE16_ID,"FILE16",File::S_READY);
    // Get File instance (17) and Check it
    check_file(fileDAO,FILE17_ID,"FILE17",File::S_READY);
    // Get Transfer instance (16) and Check it
    check_transfer(transferDAO,FILE16_ID,REQ16_ID,"FILE16,REQ16",Transfer::S_FAILED);
    // Get Transfer instance (17) and Check it
    check_transfer(transferDAO,FILE17_ID,REQ17_ID,"FILE17,REQ17",Transfer::S_FAILED);

    //----------------------------------------------------------------------
    // Job 9
    //----------------------------------------------------------------------

    // Get Job instance (9) and Check it
    check_job(jobDAO,JOB9_ID,"JOB9",Job::S_DONE_WITH_ERRORS);

    // Get File instance (18) and Check it
    check_file(fileDAO,FILE18_ID,"FILE18",File::S_FAILED);
    // Get File instance (19) and Check it
    check_file(fileDAO,FILE19_ID,"FILE19",File::S_DONE);

    //----------------------------------------------------------------------
    // Job 10
    //----------------------------------------------------------------------

    // Get Job instance (10) and Check it
    check_job(jobDAO,JOB10_ID,"JOB10",Job::S_ACTIVE);

    // Get File instance (20) and Check it
    check_file(fileDAO,FILE20_ID,"FILE20",File::S_DONE);
    // Get File instance (21) and Check it
    check_file(fileDAO,FILE21_ID,"FILE21",File::S_WAITING);

    //----------------------------------------------------------------------
    // Job 11
    //----------------------------------------------------------------------

    // Get Job instance (11) and Check it
    check_job(jobDAO,JOB11_ID,"JOB11",Job::S_HOLD);

    // Get File instance (22) and Check it
    check_file(fileDAO,FILE22_ID,"FILE22",File::S_FAILED);
    // Get File instance (23) and Check it
    check_file(fileDAO,FILE23_ID,"FILE23",File::S_DONE);
    // Get File instance (24) and Check it
    check_file(fileDAO,FILE24_ID,"FILE24",File::S_HOLD);

    //----------------------------------------------------------------------
    // Job 12
    //----------------------------------------------------------------------

    // Get Job instance (12) and Check it
    check_job(jobDAO,JOB12_ID,"JOB12",Job::S_CANCELED);

    // Get File instance (25) and Check it
    check_file(fileDAO,FILE25_ID,"FILE25",File::S_CANCELED);
    // Get File instance (26) and Check it
    check_file(fileDAO,FILE26_ID,"FILE26",File::S_CANCELED);
    // Get File instance (27) and Check it
    check_file(fileDAO,FILE27_ID,"FILE27",File::S_CANCELED);

    // Check Cache Status
    ActiveTxCache& cache = ActionsConfig::instance().activeTxCache();
    unsigned int c = cache.count();
    CPPUNIT_ASSERT_MESSAGE("Invalid count", c == 1);

    std::vector<ActiveTxCache::TransferEntry> entries = cache.getTransferEntries();
    CPPUNIT_ASSERT_MESSAGE("Empty", entries.empty() == false);
    CPPUNIT_ASSERT_MESSAGE("Invalid size", entries.size() == 1);
    CPPUNIT_ASSERT_MESSAGE("Invalid Active Transfer Req ID", entries.at(0).tx->requestId == REQ11_ID);
    CPPUNIT_ASSERT_MESSAGE("Invalid Active Transfer File ID", entries.at(0).tx->fileId == FILE11_ID);

}
