/**
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *  GLite Data Transfer Agents: FetchTransfers Test Case
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>, Paolo Tedesco <paolo.tedesco@cern.ch>
 *  Version info: $Id: FetchTestCase.cpp,v 1.15 2009/07/02 15:21:17 molnarzs Exp $
 *  Release: $Name:  $
 *
 */
#include "Fetch.h"
#include "ChannelFlags.h"
#include "actions/channel/ActionsConfig.h"
#include "DaoHelper.h"
#include "glite/data/transfer/agent/action/ActionFactory.h"
#include "glite/data/transfer/agent/model/File.h"
#include "glite/data/transfer/agent/model/Job.h"
#include "glite/data/transfer/agent/model/Transfer.h"
#include "glite/data/transfer/agent/model/Channel.h"
#include "glite/data/transfer/agent/dao/channel/FileDAO.h"
#include "glite/data/transfer/agent/dao/channel/JobDAO.h"
#include "glite/data/transfer/agent/dao/channel/TransferDAO.h"
#include "glite/data/transfer/agent/dao/channel/DAOFactory.h"
#include "glite/data/transfer/agent/ts/TransferService.h"
#include "glite/data/transfer/agent/ts/TransferServiceFactory.h"

#include <memory>
#include <boost/scoped_ptr.hpp>
#include <boost/shared_ptr.hpp>
#include <cppunit/TestCase.h>
#include <cppunit/extensions/HelperMacros.h>

#include "dao_test_utils.h"

using namespace glite::data::transfer::agent;
using namespace glite::data::transfer::agent::action;
using namespace glite::data::transfer::agent::action::channel;
using namespace glite::data::transfer::agent::model;

using boost::scoped_ptr;
using boost::shared_ptr;

/**
 * CPPUnit TestCase
 */
class FetchTestCase : public CppUnit::TestCase{
public:
    /**
     * set-up test case
     */
    void setUp ();

    void setUp_forTest1();
    void setUp_forTest2();
    void setUp_forTest3();
    void setUp_forTest4();

    /**
     * tear-down test case
     */
    void tearDown ();

private:
    // Declare CPPUnit Methods
    // -------------------------------------------------------------------------
    CPPUNIT_TEST_SUITE( FetchTestCase );
    CPPUNIT_TEST( testExecute_1 );
    CPPUNIT_TEST( testExecute_2 );
    CPPUNIT_TEST( testExecute_3 );
    CPPUNIT_TEST( testExecute_4 );
    CPPUNIT_TEST( test_bug30271 );

    CPPUNIT_TEST_SUITE_END();

    // Test Methods
    // -------------------------------------------------------------------------

    /**
     * Test Execute Method
     */
    void testExecute_1();
    void testExecute_2();
    void testExecute_3();
    void testExecute_4();

    void test_bug30271();

private:
    /**
     * The Action Instance
     */
    scoped_ptr<Fetch> m_action;
};

/*
 * Register Suit
 */
CPPUNIT_TEST_SUITE_NAMED_REGISTRATION( FetchTestCase, "FetchTestCase" );

/*
 * Declare Constants. This unit test foresee at beginning
 * - 1 READY Job with:                          =>  ACTIVE
 *        job_id      = 1-...-01
 *        vo          = VO_1
 *        channel     = channel_1 (Active)
 *        files       = 10000001 (READY)            ACTIVE
 *                      10000002 (READY)            ACTIVE
 *
 * - 1 ACTIVE Job with:                         =>
 *        job_id      = 1-...-02
 *        vo          = VO_2
 *        channel     = channel_1(Active)
 *        files       = 10000003 (READY)            ACTIVE
 *                      10000004 (DONE)
 *                      10000005 (DONE)
 *
 * - 1 READY Job with:                          =>
 *        job_id      = 1-...-03
 *        vo          = VO_1
 *        channel     = channel_2(Active)
 *        files       = 10000006 (READY)          (*different channel)
 *
 * - 1 READY Job with:                          => ACTIVE
 *        job_id      = 1-...-04
 *        vo          = VO_1
 *        channel     = channel_1(Active)
 *        files       = 10000007 (READY)           WAITING (*failed during submission)
 *        files       = 10000008 (READY)           ACTIVE
 *
 * - 1 READY Job with:                          => ACTIVE
 *        job_id      = 1-...-05
 *        vo          = VO_1
 *        channel     = channel_1(Active)
 *        files       = 10000009 (READY)           WAITING (*failed during submission)
 *
 * - 1 READY Job with:                          => (* canceling Job)
 *        job_id      = 1-...-06
 *        vo          = VO_1
 *        channel     = channel_1(Active)
 *        files       = 10000010 (READY)
 *        files       = 10000011 (READY)
 *
 * - 1 ACTIVE Job with:                          => (* canceling Job)
 *        job_id      = 1-...-07
 *        vo          = VO_1
 *        channel     = channel_1(Active)
 *        files       = 10000012 (ACTIVE)
 *        files       = 10000013 (READY)
 *
 * - 1 READY Job with:                          => ACTIVE
 *        job_id      = 1-...-08
 *        vo          = VO_1
 *        channel     = channel_1(Active)
 *        files       = 10000014 (READY)           ACTIVE
 *        files       = 10000015 (READY)           ACTIVE
 *        files       = 10000031 (READY)           ACTIVE
 *        files       = 10000032 (READY)           ACTIVE
 *        files       = 10000033 (READY)           (*queue full)
 *        files       = 10000034 (READY)           (*queue full)
 *
 * - 1 READY Job with:                          => ACTIVE
 *        job_id      = 1-...-09
 *        vo          = VO_2
 *        channel     = channel_1(Active)
 *        files       = 10000016 (READY)           ACTIVE
 *        files       = 10000017 (READY)           ACTIVE
 *        files       = 10000018 (READY)           ACTIVE
 *        files       = 10000019 (READY)           ACTIVE
 *
 * - 1 READY Job with:                          => ACTIVE
 *        job_id      = 1-...-10
 *        vo          = VO_2
 *        channel     = channel_1(Active)
 *        files       = 10000020 (READY)           ACTIVE
 *        files       = 10000021 (READY)           (*queue full)
 *        files       = 10000022 (READY)           (*queue full)
 *        files       = 10000023 (READY)           (*queue full)
 *
 * - 1 ACTIVE Job with:                         => (*used to check the share)
 *        job_id      = 1-...-11
 *        vo          = VO_1
 *        channel     = channel_1(Active)
 *        files       = 10000024 (ACTIVE)
 *        files       = 10000025 (ACTIVE)
 *
 * - 1 ACTIVE Job with:                         => (*used to check the share)
 *        job_id      = 1-...-12
 *        vo          = VO_2
 *        channel     = channel_1(Active)
 *        files       = 10000026 (ACTIVE)
 *        files       = 10000027 (ACTIVE)
 *        files       = 10000028 (ACTIVE)
 *        files       = 10000029 (ACTIVE)
 *        files       = 10000030 (DONE)
 *
 * -----------------------------------------------------------------------------
 * Test2: is for checking normalized on active fetching
 * The New algorithm to compute the shares: uses FETCH_channel_3, all the VOs
 * has jobs in the queue
 * VO name share state limit
 * VO_1     50    ON    5   => should get 5 slots
 * VO_2     50   PAUSE  5   => should get 0 slots
 * VO_3     20    OFF   1   => should get 1 slot (* remember that OFF means turn-off allocation)
 * VO_4     10    ON    0   => should get 4 slots
 *
 * -----------------------------------------------------------------------------
 * Test3: Checks the SE limit on non-dedicated channels
 *
 * -----------------------------------------------------------------------------
 * Test4: Same as Test3, but for a cloud channel (replicate the input)
 *
 */
namespace {
const char * const JOB1_ID           = "10000000-0000-0000-0000-000000000001";
const char * const JOB2_ID           = "10000000-0000-0000-0000-000000000002";
const char * const JOB3_ID           = "10000000-0000-0000-0000-000000000003";
const char * const JOB4_ID           = "10000000-0000-0000-0000-000000000004";
const char * const JOB5_ID           = "10000000-0000-0000-0000-000000000005";
const char * const JOB6_ID           = "10000000-0000-0000-0000-000000000006";
const char * const JOB7_ID           = "10000000-0000-0000-0000-000000000007";
const char * const JOB8_ID           = "10000000-0000-0000-0000-000000000008";
const char * const JOB9_ID           = "10000000-0000-0000-0000-000000000009";
const char * const JOB10_ID          = "10000000-0000-0000-0000-000000000010";
const char * const JOB11_ID          = "10000000-0000-0000-0000-000000000011";
const char * const JOB12_ID          = "10000000-0000-0000-0000-000000000012";
const char * const JOB101_ID         = "10000000-0000-0000-0000-000000000101";
const char * const JOB102_ID         = "10000000-0000-0000-0000-000000000102";
const char * const JOB103_ID         = "10000000-0000-0000-0000-000000000103";
const char * const JOB104_ID         = "10000000-0000-0000-0000-000000000104";
const char * const JOB105_ID         = "10000000-0000-0000-0000-000000000105";
const char * const JOB106_ID         = "10000000-0000-0000-0000-000000000106";
const char * const JOB107_ID         = "10000000-0000-0000-0000-000000000107";
const char * const JOB108_ID         = "10000000-0000-0000-0000-000000000108";
const char * const JOB201_ID         = "10000000-0000-0000-0000-000000000201";
const char * const JOB202_ID         = "10000000-0000-0000-0000-000000000202";
const char * const JOB203_ID         = "10000000-0000-0000-0000-000000000203";
const char * const JOB204_ID         = "10000000-0000-0000-0000-000000000204";
const char * const JOB205_ID         = "10000000-0000-0000-0000-000000000205";
const char * const JOB206_ID         = "10000000-0000-0000-0000-000000000206";
const char * const JOB207_ID         = "10000000-0000-0000-0000-000000000207";
const char * const JOB208_ID         = "10000000-0000-0000-0000-000000000208";
const char * const JOB209_ID         = "10000000-0000-0000-0000-000000000209";
const char * const JOB210_ID         = "10000000-0000-0000-0000-000000000210";
const char * const JOB301_ID         = "10000000-0000-0000-0000-000000000301";
const char * const JOB302_ID         = "10000000-0000-0000-0000-000000000302";
const char * const JOB303_ID         = "10000000-0000-0000-0000-000000000303";
const char * const FILE1_ID          = "10000001";
const char * const FILE2_ID          = "10000002";
const char * const FILE3_ID          = "10000003";
const char * const FILE4_ID          = "10000004";
const char * const FILE5_ID          = "10000005";
const char * const FILE6_ID          = "10000006";
const char * const FILE7_ID          = "10000007";
const char * const FILE8_ID          = "10000008";
const char * const FILE9_ID          = "10000009";
const char * const FILE10_ID         = "10000010";
const char * const FILE11_ID         = "10000011";
const char * const FILE12_ID         = "10000012";
const char * const FILE13_ID         = "10000013";
const char * const FILE14_ID         = "10000014";
const char * const FILE15_ID         = "10000015";
const char * const FILE16_ID         = "10000016";
const char * const FILE17_ID         = "10000017";
const char * const FILE18_ID         = "10000018";
const char * const FILE19_ID         = "10000019";
const char * const FILE20_ID         = "10000020";
const char * const FILE21_ID         = "10000021";
const char * const FILE22_ID         = "10000022";
const char * const FILE23_ID         = "10000023";
const char * const FILE24_ID         = "10000024";
const char * const FILE25_ID         = "10000025";
const char * const FILE26_ID         = "10000026";
const char * const FILE27_ID         = "10000027";
const char * const FILE28_ID         = "10000028";
const char * const FILE29_ID         = "10000029";
const char * const FILE30_ID         = "10000030";
const char * const FILE31_ID         = "10000031";
const char * const FILE32_ID         = "10000032";
const char * const FILE33_ID         = "10000033";
const char * const FILE34_ID         = "10000034";
const char * const FILE131_ID        = "10000131";
const char * const FILE132_ID        = "10000132";
const char * const FILE133_ID        = "10000133";
const char * const FILE134_ID        = "10000134";
const char * const FILE135_ID        = "10000135";
const char * const FILE136_ID        = "10000136";
const char * const FILE137_ID        = "10000137";
const char * const FILE138_ID        = "10000138";
const char * const FILE139_ID        = "10000139";
const char * const FILE140_ID        = "10000140";
const char * const FILE141_ID        = "10000141";
const char * const FILE142_ID        = "10000142";
const char * const FILE143_ID        = "10000143";
const char * const FILE144_ID        = "10000144";
const char * const FILE145_ID        = "10000145";
const char * const FILE146_ID        = "10000146";
const char * const FILE147_ID        = "10000147";
const char * const FILE148_ID        = "10000148";
const char * const FILE149_ID        = "10000149";
const char * const FILE150_ID        = "10000150";
const char * const FILE151_ID        = "10000151";
const char * const FILE152_ID        = "10000152";
const char * const FILE153_ID        = "10000153";
const char * const FILE154_ID        = "10000154";
const char * const FILE155_ID        = "10000155";
const char * const FILE156_ID        = "10000156";
const char * const FILE157_ID        = "10000157";
const char * const FILE158_ID        = "10000158";
const char * const FILE159_ID        = "10000159";
const char * const FILE160_ID        = "10000160";
const char * const FILE161_ID        = "10000161";
const char * const FILE162_ID        = "10000162";
const char * const FILE163_ID        = "10000163";
const char * const FILE164_ID        = "10000164";
const char * const FILE165_ID        = "10000165";
const char * const FILE166_ID        = "10000166";
const char * const FILE167_ID        = "10000167";
const char * const FILE168_ID        = "10000168";
const char * const FILE169_ID        = "10000169";
const char * const FILE170_ID        = "10000170";
const char * const FILE201_ID        = "10000201";
const char * const FILE202_ID        = "10000202";
const char * const FILE203_ID        = "10000203";
const char * const FILE204_ID        = "10000204";
const char * const FILE205_ID        = "10000205";
const char * const FILE206_ID        = "10000206";
const char * const FILE207_ID        = "10000207";
const char * const FILE208_ID        = "10000208";
const char * const FILE209_ID        = "10000209";
const char * const FILE210_ID        = "10000210";
const char * const FILE211_ID        = "10000211";
const char * const FILE212_ID        = "10000212";
const char * const FILE213_ID        = "10000213";
const char * const FILE214_ID        = "10000214";
const char * const FILE215_ID        = "10000215";
const char * const FILE216_ID        = "10000216";
const char * const FILE217_ID        = "10000217";
const char * const FILE218_ID        = "10000218";
const char * const FILE219_ID        = "10000219";
const char * const FILE220_ID        = "10000220";
const char * const FILE221_ID        = "10000221";
const char * const FILE222_ID        = "10000222";
const char * const FILE223_ID        = "10000223";
const char * const FILE224_ID        = "10000224";
const char * const FILE225_ID        = "10000225";
const char * const FILE226_ID        = "10000226";
const char * const FILE227_ID        = "10000227";
const char * const FILE228_ID        = "10000228";
const char * const FILE229_ID        = "10000229";
const char * const FILE230_ID        = "10000230";
const char * const FILE231_ID        = "10000231";
const char * const FILE232_ID        = "10000232";
const char * const FILE233_ID        = "10000233";
const char * const FILE234_ID        = "10000234";
const char * const FILE235_ID        = "10000235";
const char * const FILE236_ID        = "10000236";
const char * const FILE237_ID        = "10000237";
const char * const FILE238_ID        = "10000238";
const char * const FILE239_ID        = "10000239";
const char * const FILE240_ID        = "10000240";
const char * const FILE301_ID        = "10000301";
const char * const FILE302_ID        = "10000302";
const char * const FILE303_ID        = "10000303";
const char * const FILE304_ID        = "10000304";
const char * const FILE305_ID        = "10000305";
const char * const FILE306_ID        = "10000306";
const char * const FILE307_ID        = "10000307";
const char * const FILE308_ID        = "10000308";
const char * const FILE309_ID        = "10000309";
const char * const CHANNEL1_NAME     = "FETCH_channel_1";
const char * const CHANNEL2_NAME     = "FETCH_channel_2";
const char * const CHANNEL3_NAME     = "FETCH_channel_3";
const char * const CHANNEL4_NAME     = "FETCH_channel_4";
const char * const CHANNEL5_NAME     = "FETCH_channel_5";
const char * const CHANNEL6_NAME     = "FETCH_channel_6";
const char * const VO1_NAME          = "VO_1";
const char * const VO2_NAME          = "VO_2";
const char * const VO3_NAME          = "VO_3";
const char * const VO4_NAME          = "VO_4";

const char * const TC_SRC_NAME      = "srm://host.cern.ch:8443/tmp/src.txt";
const char * const TC_SRC_FAIL_NAME = "srm://host.cern.ch:8443/tmp/this-file-should-fail-in-processing.txt";
const char * const TC_DEST_NAME     = "srm://host.ral.ac.uk:8443/tmp/dst.txt";
}

/*
 * setUp
 *
 * set-up test case
 */
void FetchTestCase::setUp(){

    // Reset cache
    ActionsConfig::instance().activeTxCache().reset();
    // Reset Action
    m_action.reset();

}

/*
 * setUp
 *
 * set-up test case
 */
void FetchTestCase::setUp_forTest1(){

    // Create Action instance
    std::auto_ptr<Action> a(ActionFactory::instance().create("glite:Fetch"));
    CPPUNIT_ASSERT_MESSAGE("Null Action", a.get() != 0);
    CPPUNIT_ASSERT_MESSAGE("Invalid Action Class", dynamic_cast<Fetch*>(a.get()) != 0);

    // Store Action
    m_action.reset(dynamic_cast<Fetch*>(a.release()));
    CPPUNIT_ASSERT_MESSAGE("Null Action", m_action.get() != 0);

    // Set Context
    m_action->context(CHANNEL1_NAME,DaoHelper::ctx());

    // Create TransferDAO instance
    scoped_ptr<dao::channel::TransferDAO> transferDao(dao::channel::DAOFactory::instance().createTransferDAO(CHANNEL1_NAME, DaoHelper::ctx()));
    CPPUNIT_ASSERT_MESSAGE("Null TransferDAO", transferDao.get() != 0);

    // Add requests to TransferService
    scoped_ptr<ts::TransferService> tx_service(ts::TransferServiceFactory::instance().create(CHANNEL1_NAME));
    CPPUNIT_ASSERT_MESSAGE("Null TransferService", tx_service.get() != 0);

    // Simulate submission of existing transfers
    // Req 12
    ts::TransferService::TransferRequest  req;
    ts::TransferService::TransferResponse out;
    req.surls.resize(1);
    req.surls[0].source  = TC_SRC_NAME;
    req.surls[0].destination = TC_DEST_NAME;
    tx_service->submit(req,out,"");
    scoped_ptr<Transfer> tx(new Transfer(out.request_id,FILE12_ID,out.transfer_ids.at(0),JOB7_ID,Transfer::S_PROCESSING));
    transferDao->create(*tx);
    // Req 24
    out.transfer_ids.clear();
    tx_service->submit(req,out,"");
    tx.reset(new Transfer(out.request_id,FILE24_ID,out.transfer_ids.at(0),JOB11_ID,Transfer::S_PROCESSING));
    transferDao->create(*tx);
    // Req 25
    out.transfer_ids.clear();
    tx_service->submit(req,out,"");
    tx.reset(new Transfer(out.request_id,FILE25_ID,out.transfer_ids.at(0),JOB11_ID,Transfer::S_PROCESSING));
    transferDao->create(*tx);
    // Req 26
    out.transfer_ids.clear();
    tx_service->submit(req,out,"");
    tx.reset(new Transfer(out.request_id,FILE26_ID,out.transfer_ids.at(0),JOB12_ID,Transfer::S_PROCESSING));
    transferDao->create(*tx);
    // Req 27
    out.transfer_ids.clear();
    tx_service->submit(req,out,"");
    tx.reset(new Transfer(out.request_id,FILE27_ID,out.transfer_ids.at(0),JOB12_ID,Transfer::S_PROCESSING));
    transferDao->create(*tx);
    // Req 28
    out.transfer_ids.clear();
    tx_service->submit(req,out,"");
    tx.reset(new Transfer(out.request_id,FILE28_ID,out.transfer_ids.at(0),JOB12_ID,Transfer::S_PROCESSING));
    transferDao->create(*tx);
    // Req 29
    out.transfer_ids.clear();
    tx_service->submit(req,out,"");
    tx.reset(new Transfer(out.request_id,FILE29_ID,out.transfer_ids.at(0),JOB12_ID,Transfer::S_PROCESSING));
    transferDao->create(*tx);

}

/*
 * setUp_forTest2
 *
 * set-up test case
 */
void FetchTestCase::setUp_forTest2(){

    // Create Action instance
    std::auto_ptr<Action> a(ActionFactory::instance().create("glite:Fetch"));
    CPPUNIT_ASSERT_MESSAGE("Null Action", a.get() != 0);
    CPPUNIT_ASSERT_MESSAGE("Invalid Action Class", dynamic_cast<Fetch*>(a.get()) != 0);

    // Store Action
    m_action.reset(dynamic_cast<Fetch*>(a.release()));
    CPPUNIT_ASSERT_MESSAGE("Null Action", m_action.get() != 0);

    // Set Context
    m_action->context(CHANNEL3_NAME,DaoHelper::ctx());

}

/*
 * setUp_forTest3
 *
 * set-up test case
 */
void FetchTestCase::setUp_forTest3(){

    // Create Action instance
    std::auto_ptr<Action> a(ActionFactory::instance().create("glite:Fetch"));
    CPPUNIT_ASSERT_MESSAGE("Null Action", a.get() != 0);
    CPPUNIT_ASSERT_MESSAGE("Invalid Action Class", dynamic_cast<Fetch*>(a.get()) != 0);

    // Store Action
    m_action.reset(dynamic_cast<Fetch*>(a.release()));
    CPPUNIT_ASSERT_MESSAGE("Null Action", m_action.get() != 0);

    // Set Context
    m_action->context(CHANNEL4_NAME,DaoHelper::ctx());

}

/*
 * setUp_forTest4
 *
 * set-up test case
 */
void FetchTestCase::setUp_forTest4(){

    // Create Action instance
    std::auto_ptr<Action> a(ActionFactory::instance().create("glite:Fetch"));
    CPPUNIT_ASSERT_MESSAGE("Null Action", a.get() != 0);
    CPPUNIT_ASSERT_MESSAGE("Invalid Action Class", dynamic_cast<Fetch*>(a.get()) != 0);

    // Store Action
    m_action.reset(dynamic_cast<Fetch*>(a.release()));
    CPPUNIT_ASSERT_MESSAGE("Null Action", m_action.get() != 0);

    // Set Context
    m_action->context(CHANNEL5_NAME,DaoHelper::ctx());

}

/*
 * tearDown
 *
 * tear-down test case
 */
void FetchTestCase::tearDown(){
    ActionsConfig::instance().activeTxCache().reset();
}

/*
 *
 * Test Execute Method
 */
void FetchTestCase::testExecute_1(){

    setUp_forTest1();

    // Execute Actions
    m_action->execute();

    // Create FileDAO instance
    scoped_ptr<dao::channel::FileDAO> fileDAO(dao::channel::DAOFactory::instance().createFileDAO(CHANNEL1_NAME, DaoHelper::ctx()));
    CPPUNIT_ASSERT_MESSAGE("Null FileDAO", fileDAO.get() != 0);

    // Create JobDAO instance
    scoped_ptr<dao::channel::JobDAO> jobDAO(dao::channel::DAOFactory::instance().createJobDAO(CHANNEL1_NAME, DaoHelper::ctx()));
    CPPUNIT_ASSERT_MESSAGE("Null JobDAO", jobDAO.get() != 0);

    // Create TransferDAO instance
    scoped_ptr<dao::channel::TransferDAO> transferDAO(dao::channel::DAOFactory::instance().createTransferDAO(CHANNEL1_NAME, DaoHelper::ctx()));
    CPPUNIT_ASSERT_MESSAGE("Null TransferDAO", transferDAO.get() != 0);

    // Check the states of the files
    check_file(fileDAO,FILE1_ID,"FILE1",File::S_ACTIVE);
    check_file(fileDAO,FILE2_ID,"FILE2",File::S_ACTIVE);
    check_file(fileDAO,FILE3_ID,"FILE3",File::S_ACTIVE);
    check_file(fileDAO,FILE4_ID,"FILE4",File::S_DONE);
    check_file(fileDAO,FILE5_ID,"FILE5",File::S_DONE);
    check_file(fileDAO,FILE7_ID,"FILE7",File::S_WAITING);
    check_file(fileDAO,FILE8_ID,"FILE8",File::S_ACTIVE);
    check_file(fileDAO,FILE9_ID,"FILE9",File::S_WAITING);
    check_file(fileDAO,FILE10_ID,"FILE10",File::S_READY);
    check_file(fileDAO,FILE11_ID,"FILE11",File::S_READY);
    check_file(fileDAO,FILE12_ID,"FILE12",File::S_ACTIVE);
    check_file(fileDAO,FILE13_ID,"FILE13",File::S_READY);
    check_file(fileDAO,FILE14_ID,"FILE14",File::S_ACTIVE);
    check_file(fileDAO,FILE15_ID,"FILE15",File::S_ACTIVE);
    check_file(fileDAO,FILE31_ID,"FILE31",File::S_ACTIVE);
    check_file(fileDAO,FILE32_ID,"FILE32",File::S_ACTIVE);
    check_file(fileDAO,FILE33_ID,"FILE33",File::S_READY);
    check_file(fileDAO,FILE34_ID,"FILE34",File::S_READY);
    check_file(fileDAO,FILE16_ID,"FILE16",File::S_ACTIVE);
    check_file(fileDAO,FILE17_ID,"FILE17",File::S_ACTIVE);
    check_file(fileDAO,FILE18_ID,"FILE18",File::S_ACTIVE);
    check_file(fileDAO,FILE19_ID,"FILE19",File::S_ACTIVE);
    check_file(fileDAO,FILE20_ID,"FILE20",File::S_ACTIVE);
    check_file(fileDAO,FILE21_ID,"FILE21",File::S_READY);
    check_file(fileDAO,FILE22_ID,"FILE22",File::S_READY);
    check_file(fileDAO,FILE23_ID,"FILE23",File::S_READY);
    check_file(fileDAO,FILE24_ID,"FILE24",File::S_ACTIVE);
    check_file(fileDAO,FILE25_ID,"FILE25",File::S_ACTIVE);
    check_file(fileDAO,FILE26_ID,"FILE26",File::S_ACTIVE);
    check_file(fileDAO,FILE27_ID,"FILE27",File::S_ACTIVE);
    check_file(fileDAO,FILE28_ID,"FILE28",File::S_ACTIVE);
    check_file(fileDAO,FILE29_ID,"FILE29",File::S_ACTIVE);
    check_file(fileDAO,FILE30_ID,"FILE30",File::S_DONE);

    // Check The Jobs using JobDAO
    check_job(jobDAO,JOB1_ID,"JOB1",Job::S_ACTIVE);
    check_job(jobDAO,JOB2_ID,"JOB2",Job::S_ACTIVE);
    check_job(jobDAO,JOB4_ID,"JOB4",Job::S_ACTIVE);
    check_job(jobDAO,JOB5_ID,"JOB5",Job::S_ACTIVE);
    check_job(jobDAO,JOB6_ID,"JOB6",Job::S_READY);
    check_job(jobDAO,JOB7_ID,"JOB7",Job::S_ACTIVE);
    check_job(jobDAO,JOB8_ID,"JOB8",Job::S_ACTIVE);
    check_job(jobDAO,JOB9_ID,"JOB9",Job::S_ACTIVE);
    check_job(jobDAO,JOB10_ID,"JOB10",Job::S_ACTIVE);
    check_job(jobDAO,JOB11_ID,"JOB11",Job::S_ACTIVE);
    check_job(jobDAO,JOB12_ID,"JOB12",Job::S_ACTIVE);

    // Check The Transfers using TransferDAO

    // Count Transfers For File (1)
    glite::data::agents::dao::StringArray req_ids;
    transferDAO->getByFileId(FILE1_ID,req_ids);
    CPPUNIT_ASSERT_MESSAGE("No Transfer (FILE1)", !req_ids.empty());
    CPPUNIT_ASSERT_MESSAGE("Too many Transfer instances (FILE1)", req_ids.size() == 1);

    // Get File (1) Transfer
    scoped_ptr<Transfer> tr(transferDAO->get(FILE1_ID,req_ids.front()));
    CPPUNIT_ASSERT_MESSAGE("Null Transfer (FILE1)", tr.get() != 0);
    CPPUNIT_ASSERT_MESSAGE("Invalid Transfer State (FILE1)", tr->state == Transfer::S_RECEIVED);

    // Count Transfers For File (2)
    req_ids.clear();
    transferDAO->getByFileId(FILE2_ID,req_ids);
    CPPUNIT_ASSERT_MESSAGE("No Transfer (FILE2)", !req_ids.empty());
    CPPUNIT_ASSERT_MESSAGE("Too many Transfer instances (FILE2)", req_ids.size() == 1);

    // Get File (2) Transfer
    tr.reset(transferDAO->get(FILE2_ID,req_ids.front()));
    CPPUNIT_ASSERT_MESSAGE("Null Transfer (FILE2)", tr.get() != 0);
    CPPUNIT_ASSERT_MESSAGE("Invalid Transfer State (FILE2)", tr->state == Transfer::S_RECEIVED);

    // Count Transfers For File (3)
    req_ids.clear();
    transferDAO->getByFileId(FILE3_ID,req_ids);
    CPPUNIT_ASSERT_MESSAGE("No Transfer (FILE3)", !req_ids.empty());
    CPPUNIT_ASSERT_MESSAGE("Too many Transfer instances (FILE3)", req_ids.size() == 2);

    // Get File (3) Transfer
    tr.reset(transferDAO->get(FILE3_ID,req_ids.front()));
    CPPUNIT_ASSERT_MESSAGE("Null Transfer (FILE3)", tr.get() != 0);
    CPPUNIT_ASSERT_MESSAGE("Invalid Transfer State (FILE3)", tr->state == Transfer::S_RECEIVED);

    // Count Transfers For File (7)
    req_ids.clear();
    transferDAO->getByFileId(FILE7_ID,req_ids);
    CPPUNIT_ASSERT_MESSAGE("Too many Transfer instances (FILE7)", req_ids.empty());

    // Count Transfers For File (8)
    req_ids.clear();
    transferDAO->getByFileId(FILE8_ID,req_ids);
    CPPUNIT_ASSERT_MESSAGE("No Transfer (FILE8)", !req_ids.empty());
    CPPUNIT_ASSERT_MESSAGE("Too many Transfer instances (FILE8)", req_ids.size() == 1);

    // Get File (8) Transfer
    tr.reset(transferDAO->get(FILE8_ID,req_ids.front()));
    CPPUNIT_ASSERT_MESSAGE("Null Transfer (FILE8)", tr.get() != 0);
    CPPUNIT_ASSERT_MESSAGE("Invalid Transfer State (FILE8)", tr->state == Transfer::S_RECEIVED);

    // Count Transfers For File (9)
    req_ids.clear();
    transferDAO->getByFileId(FILE9_ID,req_ids);
    CPPUNIT_ASSERT_MESSAGE("Too many Transfer instances (FILE9)", req_ids.empty());

    // Count Transfers For File (14)
    req_ids.clear();
    transferDAO->getByFileId(FILE14_ID,req_ids);
    CPPUNIT_ASSERT_MESSAGE("No Transfer (FILE14)", !req_ids.empty());
    CPPUNIT_ASSERT_MESSAGE("Too many Transfer instances (FILE14)", req_ids.size() == 1);

    // Get File (14) Transfer
    tr.reset(transferDAO->get(FILE14_ID,req_ids.front()));
    CPPUNIT_ASSERT_MESSAGE("Null Transfer (FILE14)", tr.get() != 0);
    CPPUNIT_ASSERT_MESSAGE("Invalid Transfer State (FILE14)", tr->state == Transfer::S_RECEIVED);

    // Count Transfers For File (15)
    req_ids.clear();
    transferDAO->getByFileId(FILE15_ID,req_ids);
    CPPUNIT_ASSERT_MESSAGE("No Transfer (FILE15)", !req_ids.empty());
    CPPUNIT_ASSERT_MESSAGE("Too many Transfer instances (FILE15)", req_ids.size() == 1);

    // Get File (15) Transfer
    tr.reset(transferDAO->get(FILE15_ID,req_ids.front()));
    CPPUNIT_ASSERT_MESSAGE("Null Transfer (FILE15)", tr.get() != 0);
    CPPUNIT_ASSERT_MESSAGE("Invalid Transfer State (FILE15)", tr->state == Transfer::S_RECEIVED);

    // Count Transfers For File (31)
    req_ids.clear();
    transferDAO->getByFileId(FILE31_ID,req_ids);
    CPPUNIT_ASSERT_MESSAGE("No Transfer (FILE31)", !req_ids.empty());
    CPPUNIT_ASSERT_MESSAGE("Too many Transfer instances (FILE31)", req_ids.size() == 1);

    // Get File (31) Transfer
    tr.reset(transferDAO->get(FILE31_ID,req_ids.front()));
    CPPUNIT_ASSERT_MESSAGE("Null Transfer (FILE31)", tr.get() != 0);
    CPPUNIT_ASSERT_MESSAGE("Invalid Transfer State (FILE31)", tr->state == Transfer::S_RECEIVED);

    // Count Transfers For File (32)
    req_ids.clear();
    transferDAO->getByFileId(FILE32_ID,req_ids);
    CPPUNIT_ASSERT_MESSAGE("No Transfer (FILE32)", !req_ids.empty());
    CPPUNIT_ASSERT_MESSAGE("Too many Transfer instances (FILE32)", req_ids.size() == 1);

    // Get File (32) Transfer
    tr.reset(transferDAO->get(FILE32_ID,req_ids.front()));
    CPPUNIT_ASSERT_MESSAGE("Null Transfer (FILE32)", tr.get() != 0);
    CPPUNIT_ASSERT_MESSAGE("Invalid Transfer State (FILE32)", tr->state == Transfer::S_RECEIVED);

    // Count Transfers For File (16)
    req_ids.clear();
    transferDAO->getByFileId(FILE16_ID,req_ids);
    CPPUNIT_ASSERT_MESSAGE("No Transfer (FILE16)", !req_ids.empty());
    CPPUNIT_ASSERT_MESSAGE("Too many Transfer instances (FILE16)", req_ids.size() == 1);

    // Get File (16) Transfer
    tr.reset(transferDAO->get(FILE16_ID,req_ids.front()));
    CPPUNIT_ASSERT_MESSAGE("Null Transfer (FILE16)", tr.get() != 0);
    CPPUNIT_ASSERT_MESSAGE("Invalid Transfer State (FILE16)", tr->state == Transfer::S_RECEIVED);

    // Count Transfers For File (17)
    req_ids.clear();
    transferDAO->getByFileId(FILE17_ID,req_ids);
    CPPUNIT_ASSERT_MESSAGE("No Transfer (FILE17)", !req_ids.empty());
    CPPUNIT_ASSERT_MESSAGE("Too many Transfer instances (FILE17)", req_ids.size() == 1);

    // Get File (17) Transfer
    tr.reset(transferDAO->get(FILE17_ID,req_ids.front()));
    CPPUNIT_ASSERT_MESSAGE("Null Transfer (FILE17)", tr.get() != 0);
    CPPUNIT_ASSERT_MESSAGE("Invalid Transfer State (FILE17)", tr->state == Transfer::S_RECEIVED);

    // Count Transfers For File (18)
    req_ids.clear();
    transferDAO->getByFileId(FILE18_ID,req_ids);
    CPPUNIT_ASSERT_MESSAGE("No Transfer (FILE18)", !req_ids.empty());
    CPPUNIT_ASSERT_MESSAGE("Too many Transfer instances (FILE18)", req_ids.size() == 1);

    // Get File (18) Transfer
    tr.reset(transferDAO->get(FILE18_ID,req_ids.front()));
    CPPUNIT_ASSERT_MESSAGE("Null Transfer (FILE18)", tr.get() != 0);
    CPPUNIT_ASSERT_MESSAGE("Invalid Transfer State (FILE18)", tr->state == Transfer::S_RECEIVED);

    // Count Transfers For File (19)
    req_ids.clear();
    transferDAO->getByFileId(FILE19_ID,req_ids);
    CPPUNIT_ASSERT_MESSAGE("No Transfer (FILE19)", !req_ids.empty());
    CPPUNIT_ASSERT_MESSAGE("Too many Transfer instances (FILE19)", req_ids.size() == 1);

    // Get File (19) Transfer
    tr.reset(transferDAO->get(FILE19_ID,req_ids.front()));
    CPPUNIT_ASSERT_MESSAGE("Null Transfer (FILE19)", tr.get() != 0);
    CPPUNIT_ASSERT_MESSAGE("Invalid Transfer State (FILE19)", tr->state == Transfer::S_RECEIVED);

    // Count Transfers For File (20)
    req_ids.clear();
    transferDAO->getByFileId(FILE20_ID,req_ids);
    CPPUNIT_ASSERT_MESSAGE("No Transfer (FILE20)", !req_ids.empty());
    CPPUNIT_ASSERT_MESSAGE("Too many Transfer instances (FILE20)", req_ids.size() == 1);

    // Get File (20) Transfer
    tr.reset(transferDAO->get(FILE20_ID,req_ids.front()));
    CPPUNIT_ASSERT_MESSAGE("Null Transfer (FILE20)", tr.get() != 0);
    CPPUNIT_ASSERT_MESSAGE("Invalid Transfer State (FILE20)", tr->state == Transfer::S_RECEIVED);

    // Check consistency in ActiveTransferCache
    ActiveTxCache& cache = ActionsConfig::instance().activeTxCache();

    // Count the number of request in the cache
    unsigned long count = cache.count();
    CPPUNIT_ASSERT_MESSAGE("Invalid number of active transfers", count == 20);
    count = cache.countRequests();
    CPPUNIT_ASSERT_MESSAGE("Invalid number of active requests", count == 20);
    count = cache.countRequests(VO1_NAME);
    CPPUNIT_ASSERT_MESSAGE("Invalid number of active transfers (VO1)", count == 10);
    count = cache.countRequests(VO2_NAME);
    CPPUNIT_ASSERT_MESSAGE("Invalid number of active transfers (VO2)", count == 10);

    // Check that files are in the cache
    // VO1
    shared_ptr<model::File> file_ptr = cache.getFile(FILE1_ID);
    CPPUNIT_ASSERT_MESSAGE("No file found in cache (FILE1)", file_ptr.get() != 0);
    file_ptr = cache.getFile(FILE2_ID);
    CPPUNIT_ASSERT_MESSAGE("No file found in cache (FILE2)", file_ptr.get() != 0);
    file_ptr = cache.getFile(FILE3_ID);
    CPPUNIT_ASSERT_MESSAGE("No file found in cache (FILE3)", file_ptr.get() != 0);
    try{
        file_ptr = cache.getFile(FILE7_ID);
        CPPUNIT_FAIL("file should not be in cache (FILE7)");
    } catch(glite::data::agents::RuntimeError&){/*OK*/}
    file_ptr = cache.getFile(FILE8_ID);
    CPPUNIT_ASSERT_MESSAGE("No file found in cache (FILE8)", file_ptr.get() != 0);
    try{
        file_ptr = cache.getFile(FILE9_ID);
        CPPUNIT_FAIL("file should not be in cache (FILE9)");
    } catch(glite::data::agents::RuntimeError&){/*OK*/}
    file_ptr = cache.getFile(FILE12_ID);
    CPPUNIT_ASSERT_MESSAGE("No file found in cache (FILE12)", file_ptr.get() != 0);
    file_ptr = cache.getFile(FILE14_ID);
    CPPUNIT_ASSERT_MESSAGE("No file found in cache (FILE14)", file_ptr.get() != 0);
    file_ptr = cache.getFile(FILE15_ID);
    CPPUNIT_ASSERT_MESSAGE("No file found in cache (FILE15)", file_ptr.get() != 0);
    file_ptr = cache.getFile(FILE31_ID);
    CPPUNIT_ASSERT_MESSAGE("No file found in cache (FILE31)", file_ptr.get() != 0);
    file_ptr = cache.getFile(FILE32_ID);
    CPPUNIT_ASSERT_MESSAGE("No file found in cache (FILE32)", file_ptr.get() != 0);
    file_ptr = cache.getFile(FILE24_ID);
    CPPUNIT_ASSERT_MESSAGE("No file found in cache (FILE24)", file_ptr.get() != 0);
    file_ptr = cache.getFile(FILE25_ID);
    CPPUNIT_ASSERT_MESSAGE("No file found in cache (FILE25)", file_ptr.get() != 0);
    // VO2
    file_ptr = cache.getFile(FILE16_ID);
    CPPUNIT_ASSERT_MESSAGE("No file found in cache (FILE16)", file_ptr.get() != 0);
    file_ptr = cache.getFile(FILE17_ID);
    CPPUNIT_ASSERT_MESSAGE("No file found in cache (FILE17)", file_ptr.get() != 0);
    file_ptr = cache.getFile(FILE18_ID);
    CPPUNIT_ASSERT_MESSAGE("No file found in cache (FILE18)", file_ptr.get() != 0);
    file_ptr = cache.getFile(FILE19_ID);
    CPPUNIT_ASSERT_MESSAGE("No file found in cache (FILE19)", file_ptr.get() != 0);
    file_ptr = cache.getFile(FILE20_ID);
    CPPUNIT_ASSERT_MESSAGE("No file found in cache (FILE20)", file_ptr.get() != 0);
    file_ptr = cache.getFile(FILE26_ID);
    CPPUNIT_ASSERT_MESSAGE("No file found in cache (FILE26)", file_ptr.get() != 0);
    file_ptr = cache.getFile(FILE27_ID);
    CPPUNIT_ASSERT_MESSAGE("No file found in cache (FILE27)", file_ptr.get() != 0);
    file_ptr = cache.getFile(FILE28_ID);
    CPPUNIT_ASSERT_MESSAGE("No file found in cache (FILE28)", file_ptr.get() != 0);
    file_ptr = cache.getFile(FILE29_ID);
    CPPUNIT_ASSERT_MESSAGE("No file found in cache (FILE29)", file_ptr.get() != 0);

    // Check that jobs are in the cache
    // VO1
    shared_ptr<model::Job> job_ptr = cache.getJob(JOB1_ID);
    CPPUNIT_ASSERT_MESSAGE("No Job found in cache (JOB1)", job_ptr.get() != 0);
    job_ptr = cache.getJob(JOB2_ID);
    CPPUNIT_ASSERT_MESSAGE("No Job found in cache (JOB2)", job_ptr.get() != 0);
    job_ptr = cache.getJob(JOB4_ID);
    CPPUNIT_ASSERT_MESSAGE("No Job found in cache (JOB4)", job_ptr.get() != 0);
    try{
        job_ptr = cache.getJob(JOB5_ID);
        CPPUNIT_FAIL("Job should not be in cache (JOB5)");
    } catch(glite::data::agents::RuntimeError&){/*OK*/}
    try{
        job_ptr = cache.getJob(JOB6_ID);
        CPPUNIT_FAIL("Job should not be in cache (JOB6)");
    } catch(glite::data::agents::RuntimeError&){/*OK*/}
    job_ptr = cache.getJob(JOB7_ID);
    CPPUNIT_ASSERT_MESSAGE("No Job found in cache (JOB7)", job_ptr.get() != 0);
    job_ptr = cache.getJob(JOB8_ID);
    CPPUNIT_ASSERT_MESSAGE("No Job found in cache (JOB8)", job_ptr.get() != 0);
    job_ptr = cache.getJob(JOB11_ID);
    CPPUNIT_ASSERT_MESSAGE("No Job found in cache (JOB11)", job_ptr.get() != 0);
    // VO2
    job_ptr = cache.getJob(JOB9_ID);
    CPPUNIT_ASSERT_MESSAGE("No Job found in cache (JOB9)", job_ptr.get() != 0);
    job_ptr = cache.getJob(JOB10_ID);
    CPPUNIT_ASSERT_MESSAGE("No Job found in cache (JOB10)", job_ptr.get() != 0);
    job_ptr = cache.getJob(JOB12_ID);
    CPPUNIT_ASSERT_MESSAGE("No Job found in cache (JOB12)", job_ptr.get() != 0);

    // Check that transfers are in the cache
    // VO1
    shared_ptr<model::Transfer> tx_ptr = cache.get(FILE1_ID).tx;
    CPPUNIT_ASSERT_MESSAGE("No transfer found in cache (FILE1)", tx_ptr.get() != 0);
    tx_ptr = cache.get(FILE2_ID).tx;
    CPPUNIT_ASSERT_MESSAGE("No transfer found in cache (FILE2)", tx_ptr.get() != 0);
    tx_ptr = cache.get(FILE3_ID).tx;
    CPPUNIT_ASSERT_MESSAGE("No transfer found in cache (FILE3)", tx_ptr.get() != 0);
    try{
        tx_ptr = cache.get(FILE7_ID).tx;
        CPPUNIT_FAIL("transfer should not be in cache (FILE7)");
    } catch(glite::data::agents::RuntimeError&){/*OK*/}
    tx_ptr = cache.get(FILE8_ID).tx;
    CPPUNIT_ASSERT_MESSAGE("No transfer found in cache (FILE8)", tx_ptr.get() != 0);
    try{
        tx_ptr = cache.get(FILE9_ID).tx;
        CPPUNIT_FAIL("transfer should not be in cache (FILE9)");
    } catch(glite::data::agents::RuntimeError&){/*OK*/}
    tx_ptr = cache.get(FILE12_ID).tx;
    CPPUNIT_ASSERT_MESSAGE("No transfer found in cache (FILE12)", tx_ptr.get() != 0);
    tx_ptr = cache.get(FILE14_ID).tx;
    CPPUNIT_ASSERT_MESSAGE("No transfer found in cache (FILE14)", tx_ptr.get() != 0);
    tx_ptr = cache.get(FILE15_ID).tx;
    CPPUNIT_ASSERT_MESSAGE("No transfer found in cache (FILE15)", tx_ptr.get() != 0);
    tx_ptr = cache.get(FILE31_ID).tx;
    CPPUNIT_ASSERT_MESSAGE("No transfer found in cache (FILE31)", tx_ptr.get() != 0);
    tx_ptr = cache.get(FILE32_ID).tx;
    CPPUNIT_ASSERT_MESSAGE("No transfer found in cache (FILE32)", tx_ptr.get() != 0);
    tx_ptr = cache.get(FILE24_ID).tx;
    CPPUNIT_ASSERT_MESSAGE("No transfer found in cache (FILE24)", tx_ptr.get() != 0);
    tx_ptr = cache.get(FILE25_ID).tx;
    CPPUNIT_ASSERT_MESSAGE("No transfer found in cache (FILE25)", tx_ptr.get() != 0);
    // VO2
    tx_ptr = cache.get(FILE16_ID).tx;
    CPPUNIT_ASSERT_MESSAGE("No transfer found in cache (FILE16)", tx_ptr.get() != 0);
    tx_ptr = cache.get(FILE17_ID).tx;
    CPPUNIT_ASSERT_MESSAGE("No transfer found in cache (FILE17)", tx_ptr.get() != 0);
    tx_ptr = cache.get(FILE18_ID).tx;
    CPPUNIT_ASSERT_MESSAGE("No transfer found in cache (FILE18)", tx_ptr.get() != 0);
    tx_ptr = cache.get(FILE19_ID).tx;
    CPPUNIT_ASSERT_MESSAGE("No transfer found in cache (FILE19)", tx_ptr.get() != 0);
    tx_ptr = cache.get(FILE20_ID).tx;
    CPPUNIT_ASSERT_MESSAGE("No transfer found in cache (FILE20)", tx_ptr.get() != 0);
    tx_ptr = cache.get(FILE26_ID).tx;
    CPPUNIT_ASSERT_MESSAGE("No transfer found in cache (FILE26)", tx_ptr.get() != 0);
    tx_ptr = cache.get(FILE27_ID).tx;
    CPPUNIT_ASSERT_MESSAGE("No transfer found in cache (FILE27)", tx_ptr.get() != 0);
    tx_ptr = cache.get(FILE28_ID).tx;
    CPPUNIT_ASSERT_MESSAGE("No transfer found in cache (FILE28)", tx_ptr.get() != 0);
    tx_ptr = cache.get(FILE29_ID).tx;
    CPPUNIT_ASSERT_MESSAGE("No transfer found in cache (FILE29)", tx_ptr.get() != 0);

    // Check that channel_2 has not been touched

    // Create FileDAO instance
    fileDAO.reset(dao::channel::DAOFactory::instance().createFileDAO(CHANNEL2_NAME, DaoHelper::ctx()));
    CPPUNIT_ASSERT_MESSAGE("Null FileDAO for channel 2", fileDAO.get() != 0);

    // Create JobDAO instance
    jobDAO.reset(dao::channel::DAOFactory::instance().createJobDAO(CHANNEL2_NAME, DaoHelper::ctx()));
    CPPUNIT_ASSERT_MESSAGE("Null JobDAO for channel 2", jobDAO.get() != 0);

    // Get File instance (6) and Check it
    check_file(fileDAO,FILE6_ID,"FILE6",File::S_READY);

    // Get Job instance (3) and Check it
    check_job(jobDAO,JOB3_ID,"JOB3",Job::S_READY);

}

/*
 * Test Execute Method
 */
void FetchTestCase::testExecute_2(){

    setUp_forTest2();

    // Execute Actions
    m_action->execute(CHANNEL_VO_SHARE_TYPE_NORMALIZED_ON_ACTIVE);

    // Create FileDAO instance
    scoped_ptr<dao::channel::FileDAO> fileDAO(dao::channel::DAOFactory::instance().createFileDAO(CHANNEL3_NAME, DaoHelper::ctx()));
    CPPUNIT_ASSERT_MESSAGE("Null FileDAO", fileDAO.get() != 0);

    // Create JobDAO instance
    scoped_ptr<dao::channel::JobDAO> jobDAO(dao::channel::DAOFactory::instance().createJobDAO(CHANNEL3_NAME, DaoHelper::ctx()));
    CPPUNIT_ASSERT_MESSAGE("Null JobDAO", jobDAO.get() != 0);

    // Create TransferDAO instance
    scoped_ptr<dao::channel::TransferDAO> transferDAO(dao::channel::DAOFactory::instance().createTransferDAO(CHANNEL3_NAME, DaoHelper::ctx()));
    CPPUNIT_ASSERT_MESSAGE("Null TransferDAO", transferDAO.get() != 0);

    // Check The Transfers using FileDAO
    check_file(fileDAO,FILE131_ID,"FILE131",File::S_ACTIVE);
    check_file(fileDAO,FILE132_ID,"FILE132",File::S_ACTIVE);
    check_file(fileDAO,FILE133_ID,"FILE133",File::S_ACTIVE);
    check_file(fileDAO,FILE134_ID,"FILE134",File::S_ACTIVE);
    check_file(fileDAO,FILE135_ID,"FILE135",File::S_ACTIVE);
    check_file(fileDAO,FILE136_ID,"FILE136",File::S_READY);
    check_file(fileDAO,FILE137_ID,"FILE137",File::S_READY);
    check_file(fileDAO,FILE138_ID,"FILE138",File::S_READY);
    check_file(fileDAO,FILE139_ID,"FILE139",File::S_READY);
    check_file(fileDAO,FILE140_ID,"FILE140",File::S_READY);
    check_file(fileDAO,FILE141_ID,"FILE141",File::S_READY);
    check_file(fileDAO,FILE142_ID,"FILE142",File::S_READY);
    check_file(fileDAO,FILE143_ID,"FILE143",File::S_READY);
    check_file(fileDAO,FILE144_ID,"FILE144",File::S_READY);
    check_file(fileDAO,FILE145_ID,"FILE145",File::S_READY);
    check_file(fileDAO,FILE146_ID,"FILE146",File::S_READY);
    check_file(fileDAO,FILE147_ID,"FILE147",File::S_READY);
    check_file(fileDAO,FILE148_ID,"FILE148",File::S_READY);
    check_file(fileDAO,FILE149_ID,"FILE149",File::S_READY);
    check_file(fileDAO,FILE150_ID,"FILE150",File::S_READY);
    check_file(fileDAO,FILE151_ID,"FILE151",File::S_ACTIVE);
    check_file(fileDAO,FILE152_ID,"FILE152",File::S_READY);
    check_file(fileDAO,FILE153_ID,"FILE153",File::S_READY);
    check_file(fileDAO,FILE154_ID,"FILE154",File::S_READY);
    check_file(fileDAO,FILE155_ID,"FILE155",File::S_READY);
    check_file(fileDAO,FILE156_ID,"FILE156",File::S_READY);
    check_file(fileDAO,FILE157_ID,"FILE157",File::S_READY);
    check_file(fileDAO,FILE158_ID,"FILE158",File::S_READY);
    check_file(fileDAO,FILE159_ID,"FILE159",File::S_READY);
    check_file(fileDAO,FILE160_ID,"FILE160",File::S_READY);
    check_file(fileDAO,FILE161_ID,"FILE161",File::S_ACTIVE);
    check_file(fileDAO,FILE162_ID,"FILE162",File::S_READY);
    check_file(fileDAO,FILE163_ID,"FILE163",File::S_READY);
    check_file(fileDAO,FILE164_ID,"FILE164",File::S_READY);
    check_file(fileDAO,FILE165_ID,"FILE165",File::S_READY);
    check_file(fileDAO,FILE166_ID,"FILE166",File::S_READY);
    check_file(fileDAO,FILE167_ID,"FILE167",File::S_READY);
    check_file(fileDAO,FILE168_ID,"FILE168",File::S_READY);
    check_file(fileDAO,FILE169_ID,"FILE169",File::S_READY);
    check_file(fileDAO,FILE170_ID,"FILE170",File::S_READY);

    // Check The Jobs using JobDAO
    check_job(jobDAO,JOB101_ID,"JOB101",Job::S_ACTIVE);
    check_job(jobDAO,JOB102_ID,"JOB102",Job::S_READY);
    check_job(jobDAO,JOB103_ID,"JOB103",Job::S_READY);
    check_job(jobDAO,JOB104_ID,"JOB104",Job::S_READY);
    check_job(jobDAO,JOB105_ID,"JOB105",Job::S_ACTIVE);
    check_job(jobDAO,JOB106_ID,"JOB106",Job::S_READY);
    check_job(jobDAO,JOB107_ID,"JOB107",Job::S_ACTIVE);
    check_job(jobDAO,JOB108_ID,"JOB108",Job::S_READY);

    // Check The Transfers using TransferDAO

    // Count Transfers For File (131)
    glite::data::agents::dao::StringArray req_ids;
    transferDAO->getByFileId(FILE131_ID,req_ids);
    CPPUNIT_ASSERT_MESSAGE("No Transfer (FILE131)", !req_ids.empty());
    CPPUNIT_ASSERT_MESSAGE("Too many Transfer instances (FILE131)", req_ids.size() == 1);

    // Get File (131) Transfer
    scoped_ptr<Transfer> tr(transferDAO->get(FILE131_ID,req_ids.front()));
    CPPUNIT_ASSERT_MESSAGE("Null Transfer (FILE131)", tr.get() != 0);
    CPPUNIT_ASSERT_MESSAGE("Invalid Transfer State (FILE131)", tr->state == Transfer::S_RECEIVED);

    // Count Transfers For File (132)
    req_ids.clear();
    transferDAO->getByFileId(FILE132_ID,req_ids);
    CPPUNIT_ASSERT_MESSAGE("No Transfer (FILE132)", !req_ids.empty());
    CPPUNIT_ASSERT_MESSAGE("Too many Transfer instances (FILE132)", req_ids.size() == 1);

    // Get File (132) Transfer
    tr.reset(transferDAO->get(FILE132_ID,req_ids.front()));
    CPPUNIT_ASSERT_MESSAGE("Null Transfer (FILE132)", tr.get() != 0);
    CPPUNIT_ASSERT_MESSAGE("Invalid Transfer State (FILE132)", tr->state == Transfer::S_RECEIVED);

    // Count Transfers For File (133)
    req_ids.clear();
    transferDAO->getByFileId(FILE133_ID,req_ids);
    CPPUNIT_ASSERT_MESSAGE("No Transfer (FILE133)", !req_ids.empty());
    CPPUNIT_ASSERT_MESSAGE("Too many Transfer instances (FILE133)", req_ids.size() == 1);

    // Get File (133) Transfer
    tr.reset(transferDAO->get(FILE133_ID,req_ids.front()));
    CPPUNIT_ASSERT_MESSAGE("Null Transfer (FILE133)", tr.get() != 0);
    CPPUNIT_ASSERT_MESSAGE("Invalid Transfer State (FILE133)", tr->state == Transfer::S_RECEIVED);

    // Count Transfers For File (134)
    req_ids.clear();
    transferDAO->getByFileId(FILE134_ID,req_ids);
    CPPUNIT_ASSERT_MESSAGE("No Transfer (FILE134)", !req_ids.empty());
    CPPUNIT_ASSERT_MESSAGE("Too many Transfer instances (FILE134)", req_ids.size() == 1);

    // Get File (134) Transfer
    tr.reset(transferDAO->get(FILE134_ID,req_ids.front()));
    CPPUNIT_ASSERT_MESSAGE("Null Transfer (FILE134)", tr.get() != 0);
    CPPUNIT_ASSERT_MESSAGE("Invalid Transfer State (FILE134)", tr->state == Transfer::S_RECEIVED);

    // Count Transfers For File (135)
    req_ids.clear();
    transferDAO->getByFileId(FILE135_ID,req_ids);
    CPPUNIT_ASSERT_MESSAGE("No Transfer (FILE135)", !req_ids.empty());
    CPPUNIT_ASSERT_MESSAGE("Too many Transfer instances (FILE135)", req_ids.size() == 1);

    // Get File (135) Transfer
    tr.reset(transferDAO->get(FILE135_ID,req_ids.front()));
    CPPUNIT_ASSERT_MESSAGE("Null Transfer (FILE135)", tr.get() != 0);
    CPPUNIT_ASSERT_MESSAGE("Invalid Transfer State (FILE135)", tr->state == Transfer::S_RECEIVED);

    // Count Transfers For File (136)
    req_ids.clear();
    transferDAO->getByFileId(FILE136_ID,req_ids);
    CPPUNIT_ASSERT_MESSAGE("Too many Transfer instances (FILE136)", req_ids.empty());

    // Count Transfers For File (137)
    req_ids.clear();
    transferDAO->getByFileId(FILE137_ID,req_ids);
    CPPUNIT_ASSERT_MESSAGE("Too many Transfer instances (FILE137)", req_ids.empty());

    // Count Transfers For File (138)
    req_ids.clear();
    transferDAO->getByFileId(FILE138_ID,req_ids);
    CPPUNIT_ASSERT_MESSAGE("Too many Transfer instances (FILE138)", req_ids.empty());

    // Count Transfers For File (139)
    req_ids.clear();
    transferDAO->getByFileId(FILE139_ID,req_ids);
    CPPUNIT_ASSERT_MESSAGE("Too many Transfer instances (FILE139)", req_ids.empty());

    // Count Transfers For File (140)
    req_ids.clear();
    transferDAO->getByFileId(FILE140_ID,req_ids);
    CPPUNIT_ASSERT_MESSAGE("Too many Transfer instances (FILE140)", req_ids.empty());

    // Count Transfers For File (141)
    req_ids.clear();
    transferDAO->getByFileId(FILE141_ID,req_ids);
    CPPUNIT_ASSERT_MESSAGE("Too many Transfer instances (FILE141)", req_ids.empty());

    // Count Transfers For File (142)
    req_ids.clear();
    transferDAO->getByFileId(FILE142_ID,req_ids);
    CPPUNIT_ASSERT_MESSAGE("Too many Transfer instances (FILE142)", req_ids.empty());

    // Count Transfers For File (143)
    req_ids.clear();
    transferDAO->getByFileId(FILE143_ID,req_ids);
    CPPUNIT_ASSERT_MESSAGE("Too many Transfer instances (FILE143)", req_ids.empty());

    // Count Transfers For File (144)
    req_ids.clear();
    transferDAO->getByFileId(FILE144_ID,req_ids);
    CPPUNIT_ASSERT_MESSAGE("Too many Transfer instances (FILE144)", req_ids.empty());

    // Count Transfers For File (145)
    req_ids.clear();
    transferDAO->getByFileId(FILE145_ID,req_ids);
    CPPUNIT_ASSERT_MESSAGE("Too many Transfer instances (FILE145)", req_ids.empty());

    // Count Transfers For File (146)
    req_ids.clear();
    transferDAO->getByFileId(FILE146_ID,req_ids);
    CPPUNIT_ASSERT_MESSAGE("Too many Transfer instances (FILE146)", req_ids.empty());

    // Count Transfers For File (147)
    req_ids.clear();
    transferDAO->getByFileId(FILE147_ID,req_ids);
    CPPUNIT_ASSERT_MESSAGE("Too many Transfer instances (FILE147)", req_ids.empty());

    // Count Transfers For File (148)
    req_ids.clear();
    transferDAO->getByFileId(FILE148_ID,req_ids);
    CPPUNIT_ASSERT_MESSAGE("Too many Transfer instances (FILE148)", req_ids.empty());

    // Count Transfers For File (149)
    req_ids.clear();
    transferDAO->getByFileId(FILE149_ID,req_ids);
    CPPUNIT_ASSERT_MESSAGE("Too many Transfer instances (FILE149)", req_ids.empty());

    // Count Transfers For File (150)
    req_ids.clear();
    transferDAO->getByFileId(FILE150_ID,req_ids);
    CPPUNIT_ASSERT_MESSAGE("Too many Transfer instances (FILE150)", req_ids.empty());

    // Count Transfers For File (151)
    req_ids.clear();
    transferDAO->getByFileId(FILE151_ID,req_ids);
    CPPUNIT_ASSERT_MESSAGE("No Transfer (FILE151)", !req_ids.empty());
    CPPUNIT_ASSERT_MESSAGE("Too many Transfer instances (FILE151)", req_ids.size() == 1);

    // Get File (151) Transfer
    tr.reset(transferDAO->get(FILE151_ID,req_ids.front()));
    CPPUNIT_ASSERT_MESSAGE("Null Transfer (FILE151)", tr.get() != 0);
    CPPUNIT_ASSERT_MESSAGE("Invalid Transfer State (FILE151)", tr->state == Transfer::S_RECEIVED);

    // Count Transfers For File (152)
    req_ids.clear();
    transferDAO->getByFileId(FILE152_ID,req_ids);
    CPPUNIT_ASSERT_MESSAGE("Too many Transfer instances (FILE152)", req_ids.empty());

    // Count Transfers For File (153)
    req_ids.clear();
    transferDAO->getByFileId(FILE153_ID,req_ids);
    CPPUNIT_ASSERT_MESSAGE("Too many Transfer instances (FILE153)", req_ids.empty());

    // Count Transfers For File (154)
    req_ids.clear();
    transferDAO->getByFileId(FILE154_ID,req_ids);
    CPPUNIT_ASSERT_MESSAGE("Too many Transfer instances (FILE154)", req_ids.empty());

    // Count Transfers For File (155)
    req_ids.clear();
    transferDAO->getByFileId(FILE155_ID,req_ids);
    CPPUNIT_ASSERT_MESSAGE("Too many Transfer instances (FILE155)", req_ids.empty());

    // Count Transfers For File (156)
    req_ids.clear();
    transferDAO->getByFileId(FILE156_ID,req_ids);
    CPPUNIT_ASSERT_MESSAGE("Too many Transfer instances (FILE156)", req_ids.empty());

    // Count Transfers For File (157)
    req_ids.clear();
    transferDAO->getByFileId(FILE157_ID,req_ids);
    CPPUNIT_ASSERT_MESSAGE("Too many Transfer instances (FILE157)", req_ids.empty());

    // Count Transfers For File (158)
    req_ids.clear();
    transferDAO->getByFileId(FILE152_ID,req_ids);
    CPPUNIT_ASSERT_MESSAGE("Too many Transfer instances (FILE158)", req_ids.empty());

    // Count Transfers For File (158)
    req_ids.clear();
    transferDAO->getByFileId(FILE159_ID,req_ids);
    CPPUNIT_ASSERT_MESSAGE("Too many Transfer instances (FILE159)", req_ids.empty());

    // Count Transfers For File (160)
    req_ids.clear();
    transferDAO->getByFileId(FILE160_ID,req_ids);
    CPPUNIT_ASSERT_MESSAGE("Too many Transfer instances (FILE160)", req_ids.empty());

    // Count Transfers For File (161)
    req_ids.clear();
    transferDAO->getByFileId(FILE161_ID,req_ids);
    CPPUNIT_ASSERT_MESSAGE("No Transfer (FILE161)", !req_ids.empty());
    CPPUNIT_ASSERT_MESSAGE("Too many Transfer instances (FILE161)", req_ids.size() == 1);

    // Get File (161) Transfer
    tr.reset(transferDAO->get(FILE161_ID,req_ids.front()));
    CPPUNIT_ASSERT_MESSAGE("Null Transfer (FILE161)", tr.get() != 0);
    CPPUNIT_ASSERT_MESSAGE("Invalid Transfer State (FILE161)", tr->state == Transfer::S_RECEIVED);

    // Count Transfers For File (162)
    req_ids.clear();
    transferDAO->getByFileId(FILE162_ID,req_ids);
    CPPUNIT_ASSERT_MESSAGE("Too many Transfer instances (FILE162)", req_ids.empty());

    // Count Transfers For File (163)
    req_ids.clear();
    transferDAO->getByFileId(FILE163_ID,req_ids);
    CPPUNIT_ASSERT_MESSAGE("Too many Transfer instances (FILE163)", req_ids.empty());

    // Count Transfers For File (164)
    req_ids.clear();
    transferDAO->getByFileId(FILE164_ID,req_ids);
    CPPUNIT_ASSERT_MESSAGE("Too many Transfer instances (FILE164)", req_ids.empty());

    // Count Transfers For File (165)
    req_ids.clear();
    transferDAO->getByFileId(FILE165_ID,req_ids);
    CPPUNIT_ASSERT_MESSAGE("Too many Transfer instances (FILE165)", req_ids.empty());

    // Count Transfers For File (166)
    req_ids.clear();
    transferDAO->getByFileId(FILE166_ID,req_ids);
    CPPUNIT_ASSERT_MESSAGE("Too many Transfer instances (FILE166)", req_ids.empty());

    // Count Transfers For File (167)
    req_ids.clear();
    transferDAO->getByFileId(FILE167_ID,req_ids);
    CPPUNIT_ASSERT_MESSAGE("Too many Transfer instances (FILE167)", req_ids.empty());

    // Count Transfers For File (168)
    req_ids.clear();
    transferDAO->getByFileId(FILE168_ID,req_ids);
    CPPUNIT_ASSERT_MESSAGE("Too many Transfer instances (FILE168)", req_ids.empty());

    // Count Transfers For File (169)
    req_ids.clear();
    transferDAO->getByFileId(FILE169_ID,req_ids);
    CPPUNIT_ASSERT_MESSAGE("Too many Transfer instances (FILE169)", req_ids.empty());

    // Count Transfers For File (170)
    req_ids.clear();
    transferDAO->getByFileId(FILE170_ID,req_ids);
    CPPUNIT_ASSERT_MESSAGE("Too many Transfer instances (FILE170)", req_ids.empty());

    // Check consistency in ActiveTransferCache
    ActiveTxCache& cache = ActionsConfig::instance().activeTxCache();

    // Count the number of request in the cache
    unsigned long count = cache.count();
    CPPUNIT_ASSERT_MESSAGE("Invalid number of active transfers", count == 7);
    count = cache.countRequests();
    CPPUNIT_ASSERT_MESSAGE("Invalid number of active requests", count == 7);
    count = cache.countRequests(VO1_NAME);
    CPPUNIT_ASSERT_MESSAGE("Invalid number of active transfers (VO1)", count == 5);
    count = cache.countRequests(VO2_NAME);
    CPPUNIT_ASSERT_MESSAGE("Invalid number of active transfers (VO2)", count == 0);
    count = cache.countRequests(VO3_NAME);
    CPPUNIT_ASSERT_MESSAGE("Invalid number of active transfers (VO3)", count == 1);
    count = cache.countRequests(VO4_NAME);
    CPPUNIT_ASSERT_MESSAGE("Invalid number of active transfers (VO4)", count == 1);

    // Check that files are in the cache
    shared_ptr<model::File> file_ptr = cache.getFile(FILE131_ID);
    CPPUNIT_ASSERT_MESSAGE("No file found in cache (FILE131)", file_ptr.get() != 0);
    file_ptr = cache.getFile(FILE132_ID);
    CPPUNIT_ASSERT_MESSAGE("No file found in cache (FILE132)", file_ptr.get() != 0);
    file_ptr = cache.getFile(FILE133_ID);
    CPPUNIT_ASSERT_MESSAGE("No file found in cache (FILE133)", file_ptr.get() != 0);
    file_ptr = cache.getFile(FILE134_ID);
    CPPUNIT_ASSERT_MESSAGE("No file found in cache (FILE134)", file_ptr.get() != 0);
    file_ptr = cache.getFile(FILE135_ID);
    CPPUNIT_ASSERT_MESSAGE("No file found in cache (FILE135)", file_ptr.get() != 0);
    try{
        file_ptr = cache.getFile(FILE136_ID);
        CPPUNIT_FAIL("file should not be in cache (FILE136)");
    } catch(glite::data::agents::RuntimeError&){/*OK*/}
    try{
        file_ptr = cache.getFile(FILE137_ID);
        CPPUNIT_FAIL("file should not be in cache (FILE137)");
    } catch(glite::data::agents::RuntimeError&){/*OK*/}
    try{
        file_ptr = cache.getFile(FILE138_ID);
        CPPUNIT_FAIL("file should not be in cache (FILE138)");
    } catch(glite::data::agents::RuntimeError&){/*OK*/}
    try{
        file_ptr = cache.getFile(FILE139_ID);
        CPPUNIT_FAIL("file should not be in cache (FILE139)");
    } catch(glite::data::agents::RuntimeError&){/*OK*/}
    try{
        file_ptr = cache.getFile(FILE140_ID);
        CPPUNIT_FAIL("file should not be in cache (FILE140)");
    } catch(glite::data::agents::RuntimeError&){/*OK*/}
    try{
        file_ptr = cache.getFile(FILE141_ID);
        CPPUNIT_FAIL("file should not be in cache (FILE141)");
    } catch(glite::data::agents::RuntimeError&){/*OK*/}
    try{
        file_ptr = cache.getFile(FILE142_ID);
        CPPUNIT_FAIL("file should not be in cache (FILE142)");
    } catch(glite::data::agents::RuntimeError&){/*OK*/}
    try{
        file_ptr = cache.getFile(FILE143_ID);
        CPPUNIT_FAIL("file should not be in cache (FILE143)");
    } catch(glite::data::agents::RuntimeError&){/*OK*/}
    try{
        file_ptr = cache.getFile(FILE144_ID);
        CPPUNIT_FAIL("file should not be in cache (FILE144)");
    } catch(glite::data::agents::RuntimeError&){/*OK*/}
    try{
        file_ptr = cache.getFile(FILE145_ID);
        CPPUNIT_FAIL("file should not be in cache (FILE145)");
    } catch(glite::data::agents::RuntimeError&){/*OK*/}
    try{
        file_ptr = cache.getFile(FILE146_ID);
        CPPUNIT_FAIL("file should not be in cache (FILE146)");
    } catch(glite::data::agents::RuntimeError&){/*OK*/}
    try{
        file_ptr = cache.getFile(FILE147_ID);
        CPPUNIT_FAIL("file should not be in cache (FILE147)");
    } catch(glite::data::agents::RuntimeError&){/*OK*/}
    try{
        file_ptr = cache.getFile(FILE148_ID);
        CPPUNIT_FAIL("file should not be in cache (FILE148)");
    } catch(glite::data::agents::RuntimeError&){/*OK*/}
    try{
        file_ptr = cache.getFile(FILE149_ID);
        CPPUNIT_FAIL("file should not be in cache (FILE149)");
    } catch(glite::data::agents::RuntimeError&){/*OK*/}
    try{
        file_ptr = cache.getFile(FILE150_ID);
        CPPUNIT_FAIL("file should not be in cache (FILE150)");
    } catch(glite::data::agents::RuntimeError&){/*OK*/}
    file_ptr = cache.getFile(FILE151_ID);
    CPPUNIT_ASSERT_MESSAGE("No file found in cache (FILE151)", file_ptr.get() != 0);
    try{
        file_ptr = cache.getFile(FILE152_ID);
        CPPUNIT_FAIL("file should not be in cache (FILE152)");
    } catch(glite::data::agents::RuntimeError&){/*OK*/}
    try{
        file_ptr = cache.getFile(FILE153_ID);
        CPPUNIT_FAIL("file should not be in cache (FILE153)");
    } catch(glite::data::agents::RuntimeError&){/*OK*/}
    try{
        file_ptr = cache.getFile(FILE154_ID);
        CPPUNIT_FAIL("file should not be in cache (FILE154)");
    } catch(glite::data::agents::RuntimeError&){/*OK*/}
    try{
        file_ptr = cache.getFile(FILE155_ID);
        CPPUNIT_FAIL("file should not be in cache (FILE155)");
    } catch(glite::data::agents::RuntimeError&){/*OK*/}
    try{
        file_ptr = cache.getFile(FILE156_ID);
        CPPUNIT_FAIL("file should not be in cache (FILE156)");
    } catch(glite::data::agents::RuntimeError&){/*OK*/}
    try{
        file_ptr = cache.getFile(FILE157_ID);
        CPPUNIT_FAIL("file should not be in cache (FILE157)");
    } catch(glite::data::agents::RuntimeError&){/*OK*/}
    try{
        file_ptr = cache.getFile(FILE158_ID);
        CPPUNIT_FAIL("file should not be in cache (FILE158)");
    } catch(glite::data::agents::RuntimeError&){/*OK*/}
    try{
        file_ptr = cache.getFile(FILE159_ID);
        CPPUNIT_FAIL("file should not be in cache (FILE159)");
    } catch(glite::data::agents::RuntimeError&){/*OK*/}
    try{
        file_ptr = cache.getFile(FILE160_ID);
        CPPUNIT_FAIL("file should not be in cache (FILE160)");
    } catch(glite::data::agents::RuntimeError&){/*OK*/}
    file_ptr = cache.getFile(FILE161_ID);
    CPPUNIT_ASSERT_MESSAGE("No file found in cache (FILE161)", file_ptr.get() != 0);
    try{
        file_ptr = cache.getFile(FILE162_ID);
        CPPUNIT_ASSERT_MESSAGE("No file found in cache (FILE162)", file_ptr.get() != 0);
    } catch(glite::data::agents::RuntimeError&){/*OK*/}
    try{
        file_ptr = cache.getFile(FILE163_ID);
        CPPUNIT_ASSERT_MESSAGE("No file found in cache (FILE163)", file_ptr.get() != 0);
    } catch(glite::data::agents::RuntimeError&){/*OK*/}
    try{
        file_ptr = cache.getFile(FILE164_ID);
        CPPUNIT_ASSERT_MESSAGE("No file found in cache (FILE164)", file_ptr.get() != 0);
    } catch(glite::data::agents::RuntimeError&){/*OK*/}
    try{
        file_ptr = cache.getFile(FILE165_ID);
        CPPUNIT_FAIL("file should not be in cache (FILE165)");
    } catch(glite::data::agents::RuntimeError&){/*OK*/}
    try{
        file_ptr = cache.getFile(FILE166_ID);
        CPPUNIT_FAIL("file should not be in cache (FILE166)");
    } catch(glite::data::agents::RuntimeError&){/*OK*/}
    try{
        file_ptr = cache.getFile(FILE167_ID);
        CPPUNIT_FAIL("file should not be in cache (FILE167)");
    } catch(glite::data::agents::RuntimeError&){/*OK*/}
    try{
        file_ptr = cache.getFile(FILE168_ID);
        CPPUNIT_FAIL("file should not be in cache (FILE168)");
    } catch(glite::data::agents::RuntimeError&){/*OK*/}
    try{
        file_ptr = cache.getFile(FILE169_ID);
        CPPUNIT_FAIL("file should not be in cache (FILE169)");
    } catch(glite::data::agents::RuntimeError&){/*OK*/}
    try{
        file_ptr = cache.getFile(FILE170_ID);
        CPPUNIT_FAIL("file should not be in cache (FILE170)");
    } catch(glite::data::agents::RuntimeError&){/*OK*/}

    // Check that jobs are in the cache
    shared_ptr<model::Job> job_ptr = cache.getJob(JOB101_ID);
    CPPUNIT_ASSERT_MESSAGE("No Job found in cache (JOB101)", job_ptr.get() != 0);
    try{
        job_ptr = cache.getJob(JOB102_ID);
        CPPUNIT_FAIL("Job should not be in cache (JOB102)");
    } catch(glite::data::agents::RuntimeError&){/*OK*/}
    try{
        job_ptr = cache.getJob(JOB103_ID);
        CPPUNIT_FAIL("Job should not be in cache (JOB103)");
    } catch(glite::data::agents::RuntimeError&){/*OK*/}
    try{
        job_ptr = cache.getJob(JOB104_ID);
        CPPUNIT_FAIL("Job should not be in cache (JOB104)");
    } catch(glite::data::agents::RuntimeError&){/*OK*/}
    job_ptr = cache.getJob(JOB105_ID);
    CPPUNIT_ASSERT_MESSAGE("No Job found in cache (JOB105)", job_ptr.get() != 0);
    try{
        job_ptr = cache.getJob(JOB106_ID);
        CPPUNIT_FAIL("Job should not be in cache (JOB106)");
    } catch(glite::data::agents::RuntimeError&){/*OK*/}
    job_ptr = cache.getJob(JOB107_ID);
    CPPUNIT_ASSERT_MESSAGE("No Job found in cache (JOB107)", job_ptr.get() != 0);
    try{
        job_ptr = cache.getJob(JOB108_ID);
        CPPUNIT_FAIL("Job should not be in cache (JOB108)");
    } catch(glite::data::agents::RuntimeError&){/*OK*/}

    // Check that transfers are in the cache
    shared_ptr<model::Transfer> tx_ptr = cache.get(FILE131_ID).tx;
    CPPUNIT_ASSERT_MESSAGE("No transfer found in cache (FILE131)", tx_ptr.get() != 0);
    tx_ptr = cache.get(FILE132_ID).tx;
    CPPUNIT_ASSERT_MESSAGE("No transfer found in cache (FILE132)", tx_ptr.get() != 0);
    tx_ptr = cache.get(FILE133_ID).tx;
    CPPUNIT_ASSERT_MESSAGE("No transfer found in cache (FILE133)", tx_ptr.get() != 0);
    tx_ptr = cache.get(FILE134_ID).tx;
    CPPUNIT_ASSERT_MESSAGE("No transfer found in cache (FILE134)", tx_ptr.get() != 0);
    tx_ptr = cache.get(FILE135_ID).tx;
    CPPUNIT_ASSERT_MESSAGE("No transfer found in cache (FILE135)", tx_ptr.get() != 0);
    try{
        tx_ptr = cache.get(FILE136_ID).tx;
        CPPUNIT_FAIL("transfer should not be in cache (FILE136)");
    } catch(glite::data::agents::RuntimeError&){/*OK*/}
    try{
        tx_ptr = cache.get(FILE137_ID).tx;
        CPPUNIT_FAIL("transfer should not be in cache (FILE137)");
    } catch(glite::data::agents::RuntimeError&){/*OK*/}
    try{
        tx_ptr = cache.get(FILE138_ID).tx;
        CPPUNIT_FAIL("transfer should not be in cache (FILE138)");
    } catch(glite::data::agents::RuntimeError&){/*OK*/}
    try{
        tx_ptr = cache.get(FILE139_ID).tx;
        CPPUNIT_FAIL("transfer should not be in cache (FILE139)");
    } catch(glite::data::agents::RuntimeError&){/*OK*/}
    try{
        tx_ptr = cache.get(FILE140_ID).tx;
        CPPUNIT_FAIL("transfer should not be in cache (FILE140)");
    } catch(glite::data::agents::RuntimeError&){/*OK*/}
    try{
        tx_ptr = cache.get(FILE141_ID).tx;
        CPPUNIT_FAIL("transfer should not be in cache (FILE141)");
    } catch(glite::data::agents::RuntimeError&){/*OK*/}
    try{
        tx_ptr = cache.get(FILE142_ID).tx;
        CPPUNIT_FAIL("transfer should not be in cache (FILE142)");
    } catch(glite::data::agents::RuntimeError&){/*OK*/}
    try{
        tx_ptr = cache.get(FILE143_ID).tx;
        CPPUNIT_FAIL("transfer should not be in cache (FILE143)");
    } catch(glite::data::agents::RuntimeError&){/*OK*/}
    try{
        tx_ptr = cache.get(FILE144_ID).tx;
        CPPUNIT_FAIL("transfer should not be in cache (FILE144)");
    } catch(glite::data::agents::RuntimeError&){/*OK*/}
    try{
        tx_ptr = cache.get(FILE145_ID).tx;
        CPPUNIT_FAIL("transfer should not be in cache (FILE145)");
    } catch(glite::data::agents::RuntimeError&){/*OK*/}
    try{
        tx_ptr = cache.get(FILE146_ID).tx;
        CPPUNIT_FAIL("transfer should not be in cache (FILE146)");
    } catch(glite::data::agents::RuntimeError&){/*OK*/}
    try{
        tx_ptr = cache.get(FILE147_ID).tx;
        CPPUNIT_FAIL("transfer should not be in cache (FILE147)");
    } catch(glite::data::agents::RuntimeError&){/*OK*/}
    try{
        tx_ptr = cache.get(FILE148_ID).tx;
        CPPUNIT_FAIL("transfer should not be in cache (FILE148)");
    } catch(glite::data::agents::RuntimeError&){/*OK*/}
    try{
        tx_ptr = cache.get(FILE149_ID).tx;
        CPPUNIT_FAIL("transfer should not be in cache (FILE149)");
    } catch(glite::data::agents::RuntimeError&){/*OK*/}
    try{
        tx_ptr = cache.get(FILE150_ID).tx;
        CPPUNIT_FAIL("transfer should not be in cache (FILE150)");
    } catch(glite::data::agents::RuntimeError&){/*OK*/}
    tx_ptr = cache.get(FILE151_ID).tx;
    CPPUNIT_ASSERT_MESSAGE("No transfer found in cache (FILE151)", tx_ptr.get() != 0);
    try{
        tx_ptr = cache.get(FILE152_ID).tx;
        CPPUNIT_FAIL("transfer should not be in cache (FILE152)");
    } catch(glite::data::agents::RuntimeError&){/*OK*/}
    try{
        tx_ptr = cache.get(FILE153_ID).tx;
        CPPUNIT_FAIL("transfer should not be in cache (FILE153)");
    } catch(glite::data::agents::RuntimeError&){/*OK*/}
    try{
        tx_ptr = cache.get(FILE154_ID).tx;
        CPPUNIT_FAIL("transfer should not be in cache (FILE154)");
    } catch(glite::data::agents::RuntimeError&){/*OK*/}
    try{
        tx_ptr = cache.get(FILE155_ID).tx;
        CPPUNIT_FAIL("transfer should not be in cache (FILE155)");
    } catch(glite::data::agents::RuntimeError&){/*OK*/}
    try{
        tx_ptr = cache.get(FILE156_ID).tx;
        CPPUNIT_FAIL("transfer should not be in cache (FILE156)");
    } catch(glite::data::agents::RuntimeError&){/*OK*/}
    try{
        tx_ptr = cache.get(FILE157_ID).tx;
        CPPUNIT_FAIL("transfer should not be in cache (FILE157)");
    } catch(glite::data::agents::RuntimeError&){/*OK*/}
    try{
        tx_ptr = cache.get(FILE158_ID).tx;
        CPPUNIT_FAIL("transfer should not be in cache (FILE158)");
    } catch(glite::data::agents::RuntimeError&){/*OK*/}
    try{
        tx_ptr = cache.get(FILE159_ID).tx;
        CPPUNIT_FAIL("transfer should not be in cache (FILE159)");
    } catch(glite::data::agents::RuntimeError&){/*OK*/}
    try{
        tx_ptr = cache.get(FILE160_ID).tx;
        CPPUNIT_FAIL("transfer should not be in cache (FILE160)");
    } catch(glite::data::agents::RuntimeError&){/*OK*/}
    tx_ptr = cache.get(FILE161_ID).tx;
    CPPUNIT_ASSERT_MESSAGE("No transfer found in cache (FILE161)", tx_ptr.get() != 0);
    try{
        tx_ptr = cache.get(FILE162_ID).tx;
        CPPUNIT_ASSERT_MESSAGE("transfer should not be in cache (FILE162)", tx_ptr.get() != 0);
    } catch(glite::data::agents::RuntimeError&){/*OK*/}
    try{
        tx_ptr = cache.get(FILE163_ID).tx;
        CPPUNIT_ASSERT_MESSAGE("transfer should not be in cache (FILE163)", tx_ptr.get() != 0);
    } catch(glite::data::agents::RuntimeError&){/*OK*/}
    try{
        tx_ptr = cache.get(FILE164_ID).tx;
        CPPUNIT_ASSERT_MESSAGE("transfer should not be in cache (FILE164)", tx_ptr.get() != 0);
    } catch(glite::data::agents::RuntimeError&){/*OK*/}
    try{
        tx_ptr = cache.get(FILE165_ID).tx;
        CPPUNIT_FAIL("transfer should not be in cache (FILE165)");
    } catch(glite::data::agents::RuntimeError&){/*OK*/}
    try{
        tx_ptr = cache.get(FILE166_ID).tx;
        CPPUNIT_FAIL("transfer should not be in cache (FILE166)");
    } catch(glite::data::agents::RuntimeError&){/*OK*/}
    try{
        tx_ptr = cache.get(FILE167_ID).tx;
        CPPUNIT_FAIL("transfer should not be in cache (FILE167)");
    } catch(glite::data::agents::RuntimeError&){/*OK*/}
    try{
        tx_ptr = cache.get(FILE168_ID).tx;
        CPPUNIT_FAIL("transfer should not be in cache (FILE168)");
    } catch(glite::data::agents::RuntimeError&){/*OK*/}
    try{
        tx_ptr = cache.get(FILE169_ID).tx;
        CPPUNIT_FAIL("transfer should not be in cache (FILE169)");
    } catch(glite::data::agents::RuntimeError&){/*OK*/}
    try{
        tx_ptr = cache.get(FILE170_ID).tx;
        CPPUNIT_FAIL("transfer should not be in cache (FILE170)");
    } catch(glite::data::agents::RuntimeError&){/*OK*/}

}

/*
 * Test Execute Method
 */
void FetchTestCase::testExecute_3(){

    setUp_forTest3();

    // Execute Actions
    m_action->execute(CHANNEL_VO_SHARE_TYPE_NORMALIZED_ON_ACTIVE);

    // Create FileDAO instance
    scoped_ptr<dao::channel::FileDAO> fileDAO(dao::channel::DAOFactory::instance().createFileDAO(CHANNEL4_NAME, DaoHelper::ctx()));
    CPPUNIT_ASSERT_MESSAGE("Null FileDAO", fileDAO.get() != 0);

    // Create JobDAO instance
    scoped_ptr<dao::channel::JobDAO> jobDAO(dao::channel::DAOFactory::instance().createJobDAO(CHANNEL4_NAME, DaoHelper::ctx()));
    CPPUNIT_ASSERT_MESSAGE("Null JobDAO", jobDAO.get() != 0);

    // Create TransferDAO instance
    scoped_ptr<dao::channel::TransferDAO> transferDAO(dao::channel::DAOFactory::instance().createTransferDAO(CHANNEL4_NAME, DaoHelper::ctx()));
    CPPUNIT_ASSERT_MESSAGE("Null TransferDAO", transferDAO.get() != 0);

    // Check The Transfers using FileDAO

    // Get File instance (201) and Check it
    check_file(fileDAO,FILE201_ID,"FILE201",File::S_ACTIVE);

    // Get File instance (202) and Check it
    check_file(fileDAO,FILE202_ID,"FILE202",File::S_READY);

    // Get File instance (203) and Check it
    check_file(fileDAO,FILE203_ID,"FILE203",File::S_READY);

    // Get File instance (204) and Check it
    check_file(fileDAO,FILE204_ID,"FILE204",File::S_READY);

    // Get File instance (205) and Check it
    check_file(fileDAO,FILE205_ID,"FILE205",File::S_ACTIVE);

    // Get File instance (206) and Check it
    check_file(fileDAO,FILE206_ID,"FILE206",File::S_ACTIVE);

    // Get File instance (207) and Check it
    check_file(fileDAO,FILE207_ID,"FILE207",File::S_READY);

    // Get File instance (208) and Check it
    check_file(fileDAO,FILE208_ID,"FILE208",File::S_READY);

    // Get File instance (209) and Check it
    check_file(fileDAO,FILE209_ID,"FILE209",File::S_ACTIVE);

    // Get File instance (210) and Check it
    check_file(fileDAO,FILE210_ID,"FILE210",File::S_ACTIVE);

    // Get File instance (211) and Check it
    check_file(fileDAO,FILE211_ID,"FILE211",File::S_READY);

    // Get File instance (212) and Check it
    check_file(fileDAO,FILE212_ID,"FILE212",File::S_READY);

    // Get File instance (213) and Check it
    check_file(fileDAO,FILE213_ID,"FILE213",File::S_READY);

    // Get File instance (214) and Check it
    check_file(fileDAO,FILE214_ID,"FILE214",File::S_READY);

    // Get File instance (215) and Check it
    check_file(fileDAO,FILE215_ID,"FILE215",File::S_READY);

    // Get File instance (216) and Check it
    check_file(fileDAO,FILE216_ID,"FILE216",File::S_READY);

    // Get File instance (217) and Check it
    check_file(fileDAO,FILE217_ID,"FILE217",File::S_READY);

    // Get File instance (218) and Check it
    check_file(fileDAO,FILE218_ID,"FILE218",File::S_READY);

    // Get File instance (219) and Check it
    check_file(fileDAO,FILE219_ID,"FILE219",File::S_READY);

    // Get File instance (220) and Check it
    check_file(fileDAO,FILE220_ID,"FILE220",File::S_READY);

    // Get File instance (221) and Check it
    check_file(fileDAO,FILE221_ID,"FILE221",File::S_READY);

    // Get File instance (222) and Check it
    check_file(fileDAO,FILE222_ID,"FILE222",File::S_READY);

    // Get File instance (223) and Check it
    check_file(fileDAO,FILE223_ID,"FILE223",File::S_READY);

    // Get File instance (224) and Check it
    check_file(fileDAO,FILE224_ID,"FILE224",File::S_READY);

    // Get File instance (225) and Check it
    check_file(fileDAO,FILE225_ID,"FILE225",File::S_ACTIVE);

    // Get File instance (226) and Check it
    check_file(fileDAO,FILE226_ID,"FILE226",File::S_ACTIVE);

    // Get File instance (227) and Check it
    check_file(fileDAO,FILE227_ID,"FILE227",File::S_ACTIVE);

    // Get File instance (228) and Check it
    check_file(fileDAO,FILE228_ID,"FILE228",File::S_READY);

    // Get File instance (229) and Check it
    check_file(fileDAO,FILE229_ID,"FILE229",File::S_READY);

    // Get File instance (230) and Check it
    check_file(fileDAO,FILE230_ID,"FILE230",File::S_READY);

    // Get File instance (231) and Check it
    check_file(fileDAO,FILE231_ID,"FILE231",File::S_READY);

    // Get File instance (232) and Check it
    check_file(fileDAO,FILE232_ID,"FILE232",File::S_READY);

    // Get File instance (233) and Check it
    check_file(fileDAO,FILE233_ID,"FILE233",File::S_ACTIVE);

    // Get File instance (234) and Check it
    check_file(fileDAO,FILE234_ID,"FILE234",File::S_READY);

    // Get File instance (235) and Check it
    check_file(fileDAO,FILE235_ID,"FILE235",File::S_READY);

    // Get File instance (236) and Check it
    check_file(fileDAO,FILE236_ID,"FILE236",File::S_READY);

    // Get File instance (237) and Check it
    check_file(fileDAO,FILE237_ID,"FILE237",File::S_ACTIVE);

    // Get File instance (238) and Check it
    check_file(fileDAO,FILE238_ID,"FILE238",File::S_ACTIVE);

    // Get File instance (239) and Check it
    check_file(fileDAO,FILE239_ID,"FILE239",File::S_ACTIVE);

    // Get File instance (240) and Check it
    check_file(fileDAO,FILE240_ID,"FILE240",File::S_ACTIVE);

    // Check The Jobs using JobDAO

    // Get Job instance (201) and Check it
    check_job(jobDAO,JOB201_ID,"JOB201",Job::S_ACTIVE);

    // Get Job instance (202) and Check it
    check_job(jobDAO,JOB202_ID,"JOB202",Job::S_ACTIVE);

    // Get Job instance (203) and Check it
    check_job(jobDAO,JOB203_ID,"JOB203",Job::S_ACTIVE);

    // Get Job instance (204) and Check it
    check_job(jobDAO,JOB204_ID,"JOB204",Job::S_READY);

    // Get Job instance (205) and Check it
    check_job(jobDAO,JOB205_ID,"JOB205",Job::S_READY);

    // Get Job instance (206) and Check it
    check_job(jobDAO,JOB206_ID,"JOB206",Job::S_READY);

    // Get Job instance (207) and Check it
    check_job(jobDAO,JOB207_ID,"JOB207",Job::S_ACTIVE);

    // Get Job instance (208) and Check it
    check_job(jobDAO,JOB208_ID,"JOB208",Job::S_READY);

    // Get Job instance (209) and Check it
    check_job(jobDAO,JOB209_ID,"JOB209",Job::S_ACTIVE);

    // Get Job instance (210) and Check it
    check_job(jobDAO,JOB210_ID,"JOB210",Job::S_ACTIVE);

    // Check The Transfers using TransferDAO

    // Count Transfers For File (201)
    glite::data::agents::dao::StringArray req_ids;
    transferDAO->getByFileId(FILE201_ID,req_ids);
    CPPUNIT_ASSERT_MESSAGE("No Transfer (FILE201)", !req_ids.empty());
    CPPUNIT_ASSERT_MESSAGE("Too many Transfer instances (FILE201)", req_ids.size() == 1);

    // Get File (201) Transfer
    scoped_ptr<Transfer> tr(transferDAO->get(FILE201_ID,req_ids.front()));
    CPPUNIT_ASSERT_MESSAGE("Null Transfer (FILE201)", tr.get() != 0);
    CPPUNIT_ASSERT_MESSAGE("Invalid Transfer State (FILE201)", tr->state == Transfer::S_RECEIVED);

    // Count Transfers For File (202)
    req_ids.clear();
    transferDAO->getByFileId(FILE202_ID,req_ids);
    CPPUNIT_ASSERT_MESSAGE("Too many Transfer instances (FILE202)", req_ids.empty());

    // Count Transfers For File (203)
    req_ids.clear();
    transferDAO->getByFileId(FILE203_ID,req_ids);
    CPPUNIT_ASSERT_MESSAGE("Too many Transfer instances (FILE203)", req_ids.empty());

    // Count Transfers For File (204)
    req_ids.clear();
    transferDAO->getByFileId(FILE204_ID,req_ids);
    CPPUNIT_ASSERT_MESSAGE("Too many Transfer instances (FILE204)", req_ids.empty());

    // Count Transfers For File (205)
    req_ids.clear();
    transferDAO->getByFileId(FILE205_ID,req_ids);
    CPPUNIT_ASSERT_MESSAGE("No Transfer (FILE205)", !req_ids.empty());
    CPPUNIT_ASSERT_MESSAGE("Too many Transfer instances (FILE205)", req_ids.size() == 1);

    // Get File (205) Transfer
    tr.reset(transferDAO->get(FILE205_ID,req_ids.front()));
    CPPUNIT_ASSERT_MESSAGE("Null Transfer (FILE205)", tr.get() != 0);
    CPPUNIT_ASSERT_MESSAGE("Invalid Transfer State (FILE205)", tr->state == Transfer::S_PROCESSING);

    // Count Transfers For File (206)
    req_ids.clear();
    transferDAO->getByFileId(FILE206_ID,req_ids);
    CPPUNIT_ASSERT_MESSAGE("No Transfer (FILE206)", !req_ids.empty());
    CPPUNIT_ASSERT_MESSAGE("Too many Transfer instances (FILE206)", req_ids.size() == 1);

    // Get File (206) Transfer
    tr.reset(transferDAO->get(FILE206_ID,req_ids.front()));
    CPPUNIT_ASSERT_MESSAGE("Null Transfer (FILE206)", tr.get() != 0);
    CPPUNIT_ASSERT_MESSAGE("Invalid Transfer State (FILE206)", tr->state == Transfer::S_PROCESSING);

    // Count Transfers For File (207)
    req_ids.clear();
    transferDAO->getByFileId(FILE207_ID,req_ids);
    CPPUNIT_ASSERT_MESSAGE("Too many Transfer instances (FILE207)", req_ids.empty());

    // Count Transfers For File (208)
    req_ids.clear();
    transferDAO->getByFileId(FILE208_ID,req_ids);
    CPPUNIT_ASSERT_MESSAGE("Too many Transfer instances (FILE208)", req_ids.empty());

    // Count Transfers For File (209)
    req_ids.clear();
    transferDAO->getByFileId(FILE209_ID,req_ids);
    CPPUNIT_ASSERT_MESSAGE("No Transfer (FILE209)", !req_ids.empty());
    CPPUNIT_ASSERT_MESSAGE("Too many Transfer instances (FILE209)", req_ids.size() == 1);

    // Get File (209) Transfer
    tr.reset(transferDAO->get(FILE209_ID,req_ids.front()));
    CPPUNIT_ASSERT_MESSAGE("Null Transfer (FILE209)", tr.get() != 0);
    CPPUNIT_ASSERT_MESSAGE("Invalid Transfer State (FILE209)", tr->state == Transfer::S_RECEIVED);

    // Count Transfers For File (210)
    req_ids.clear();
    transferDAO->getByFileId(FILE210_ID,req_ids);
    CPPUNIT_ASSERT_MESSAGE("No Transfer (FILE210)", !req_ids.empty());
    CPPUNIT_ASSERT_MESSAGE("Too many Transfer instances (FILE210)", req_ids.size() == 1);

    // Get File (210) Transfer
    tr.reset(transferDAO->get(FILE210_ID,req_ids.front()));
    CPPUNIT_ASSERT_MESSAGE("Null Transfer (FILE210)", tr.get() != 0);
    CPPUNIT_ASSERT_MESSAGE("Invalid Transfer State (FILE210)", tr->state == Transfer::S_RECEIVED);

    // Count Transfers For File (211)
    req_ids.clear();
    transferDAO->getByFileId(FILE211_ID,req_ids);
    CPPUNIT_ASSERT_MESSAGE("Too many Transfer instances (FILE211)", req_ids.empty());

    // Count Transfers For File (212)
    req_ids.clear();
    transferDAO->getByFileId(FILE212_ID,req_ids);
    CPPUNIT_ASSERT_MESSAGE("Too many Transfer instances (FILE212)", req_ids.empty());

    // Count Transfers For File (213)
    req_ids.clear();
    transferDAO->getByFileId(FILE213_ID,req_ids);
    CPPUNIT_ASSERT_MESSAGE("Too many Transfer instances (FILE213)", req_ids.empty());

    // Count Transfers For File (214)
    req_ids.clear();
    transferDAO->getByFileId(FILE214_ID,req_ids);
    CPPUNIT_ASSERT_MESSAGE("Too many Transfer instances (FILE214)", req_ids.empty());

    // Count Transfers For File (215)
    req_ids.clear();
    transferDAO->getByFileId(FILE215_ID,req_ids);
    CPPUNIT_ASSERT_MESSAGE("Too many Transfer instances (FILE215)", req_ids.empty());

    // Count Transfers For File (216)
    req_ids.clear();
    transferDAO->getByFileId(FILE216_ID,req_ids);
    CPPUNIT_ASSERT_MESSAGE("Too many Transfer instances (FILE216)", req_ids.empty());

    // Count Transfers For File (217)
    req_ids.clear();
    transferDAO->getByFileId(FILE217_ID,req_ids);
    CPPUNIT_ASSERT_MESSAGE("Too many Transfer instances (FILE217)", req_ids.empty());

    // Count Transfers For File (218)
    req_ids.clear();
    transferDAO->getByFileId(FILE218_ID,req_ids);
    CPPUNIT_ASSERT_MESSAGE("Too many Transfer instances (FILE218)", req_ids.empty());

    // Count Transfers For File (219)
    req_ids.clear();
    transferDAO->getByFileId(FILE219_ID,req_ids);
    CPPUNIT_ASSERT_MESSAGE("Too many Transfer instances (FILE219)", req_ids.empty());

    // Count Transfers For File (220)
    req_ids.clear();
    transferDAO->getByFileId(FILE220_ID,req_ids);
    CPPUNIT_ASSERT_MESSAGE("Too many Transfer instances (FILE220)", req_ids.empty());

    // Count Transfers For File (221)
    req_ids.clear();
    transferDAO->getByFileId(FILE221_ID,req_ids);
    CPPUNIT_ASSERT_MESSAGE("Too many Transfer instances (FILE221)", req_ids.empty());

    // Count Transfers For File (222)
    req_ids.clear();
    transferDAO->getByFileId(FILE222_ID,req_ids);
    CPPUNIT_ASSERT_MESSAGE("Too many Transfer instances (FILE222)", req_ids.empty());

    // Count Transfers For File (223)
    req_ids.clear();
    transferDAO->getByFileId(FILE223_ID,req_ids);
    CPPUNIT_ASSERT_MESSAGE("Too many Transfer instances (FILE223)", req_ids.empty());

    // Count Transfers For File (224)
    req_ids.clear();
    transferDAO->getByFileId(FILE224_ID,req_ids);
    CPPUNIT_ASSERT_MESSAGE("Too many Transfer instances (FILE224)", req_ids.empty());

    // Count Transfers For File (225)
    req_ids.clear();
    transferDAO->getByFileId(FILE225_ID,req_ids);
    CPPUNIT_ASSERT_MESSAGE("No Transfer (FILE225)", !req_ids.empty());
    CPPUNIT_ASSERT_MESSAGE("Too many Transfer instances (FILE225)", req_ids.size() == 1);

    // Get File (225) Transfer
    tr.reset(transferDAO->get(FILE225_ID,req_ids.front()));
    CPPUNIT_ASSERT_MESSAGE("Null Transfer (FILE225)", tr.get() != 0);
    CPPUNIT_ASSERT_MESSAGE("Invalid Transfer State (FILE225)", tr->state == Transfer::S_RECEIVED);

    // Count Transfers For File (226)
    req_ids.clear();
    transferDAO->getByFileId(FILE226_ID,req_ids);
    CPPUNIT_ASSERT_MESSAGE("No Transfer (FILE226)", !req_ids.empty());
    CPPUNIT_ASSERT_MESSAGE("Too many Transfer instances (FILE226)", req_ids.size() == 1);

    // Get File (226) Transfer
    tr.reset(transferDAO->get(FILE226_ID,req_ids.front()));
    CPPUNIT_ASSERT_MESSAGE("Null Transfer (FILE226)", tr.get() != 0);
    CPPUNIT_ASSERT_MESSAGE("Invalid Transfer State (FILE226)", tr->state == Transfer::S_RECEIVED);

    // Count Transfers For File (227)
    req_ids.clear();
    transferDAO->getByFileId(FILE227_ID,req_ids);
    CPPUNIT_ASSERT_MESSAGE("No Transfer (FILE227)", !req_ids.empty());
    CPPUNIT_ASSERT_MESSAGE("Too many Transfer instances (FILE227)", req_ids.size() == 1);

    // Get File (227) Transfer
    tr.reset(transferDAO->get(FILE227_ID,req_ids.front()));
    CPPUNIT_ASSERT_MESSAGE("Null Transfer (FILE227)", tr.get() != 0);
    CPPUNIT_ASSERT_MESSAGE("Invalid Transfer State (FILE227)", tr->state == Transfer::S_RECEIVED);

    // Count Transfers For File (228)
    req_ids.clear();
    transferDAO->getByFileId(FILE228_ID,req_ids);
    CPPUNIT_ASSERT_MESSAGE("Too many Transfer instances (FILE228)", req_ids.empty());

    // Count Transfers For File (229)
    req_ids.clear();
    transferDAO->getByFileId(FILE229_ID,req_ids);
    CPPUNIT_ASSERT_MESSAGE("Too many Transfer instances (FILE229)", req_ids.empty());

    // Count Transfers For File (230)
    req_ids.clear();
    transferDAO->getByFileId(FILE230_ID,req_ids);
    CPPUNIT_ASSERT_MESSAGE("Too many Transfer instances (FILE230)", req_ids.empty());

    // Count Transfers For File (231)
    req_ids.clear();
    transferDAO->getByFileId(FILE231_ID,req_ids);
    CPPUNIT_ASSERT_MESSAGE("Too many Transfer instances (FILE231)", req_ids.empty());

    // Count Transfers For File (232)
    req_ids.clear();
    transferDAO->getByFileId(FILE232_ID,req_ids);
    CPPUNIT_ASSERT_MESSAGE("Too many Transfer instances (FILE232)", req_ids.empty());

    // Count Transfers For File (233)
    req_ids.clear();
    transferDAO->getByFileId(FILE233_ID,req_ids);
    CPPUNIT_ASSERT_MESSAGE("No Transfer (FILE233)", !req_ids.empty());
    CPPUNIT_ASSERT_MESSAGE("Too many Transfer instances (FILE233)", req_ids.size() == 1);

    // Get File (233) Transfer
    tr.reset(transferDAO->get(FILE233_ID,req_ids.front()));
    CPPUNIT_ASSERT_MESSAGE("Null Transfer (FILE233)", tr.get() != 0);
    CPPUNIT_ASSERT_MESSAGE("Invalid Transfer State (FILE233)", tr->state == Transfer::S_PROCESSING);

    // Count Transfers For File (234)
    req_ids.clear();
    transferDAO->getByFileId(FILE234_ID,req_ids);
    CPPUNIT_ASSERT_MESSAGE("Too many Transfer instances (FILE234)", req_ids.empty());

    // Count Transfers For File (235)
    req_ids.clear();
    transferDAO->getByFileId(FILE235_ID,req_ids);
    CPPUNIT_ASSERT_MESSAGE("Too many Transfer instances (FILE235)", req_ids.empty());

    // Count Transfers For File (236)
    req_ids.clear();
    transferDAO->getByFileId(FILE236_ID,req_ids);
    CPPUNIT_ASSERT_MESSAGE("Too many Transfer instances (FILE236)", req_ids.empty());

    // Count Transfers For File (237)
    req_ids.clear();
    transferDAO->getByFileId(FILE237_ID,req_ids);
    CPPUNIT_ASSERT_MESSAGE("No Transfer (FILE237)", !req_ids.empty());
    CPPUNIT_ASSERT_MESSAGE("Too many Transfer instances (FILE237)", req_ids.size() == 1);

    // Get File (237) Transfer
    tr.reset(transferDAO->get(FILE237_ID,req_ids.front()));
    CPPUNIT_ASSERT_MESSAGE("Null Transfer (FILE237)", tr.get() != 0);
    CPPUNIT_ASSERT_MESSAGE("Invalid Transfer State (FILE237)", tr->state == Transfer::S_PROCESSING);

    // Count Transfers For File (238)
    req_ids.clear();
    transferDAO->getByFileId(FILE238_ID,req_ids);
    CPPUNIT_ASSERT_MESSAGE("No Transfer (FILE238)", !req_ids.empty());
    CPPUNIT_ASSERT_MESSAGE("Too many Transfer instances (FILE238)", req_ids.size() == 1);

    // Get File (238) Transfer
    tr.reset(transferDAO->get(FILE238_ID,req_ids.front()));
    CPPUNIT_ASSERT_MESSAGE("Null Transfer (FILE238)", tr.get() != 0);
    CPPUNIT_ASSERT_MESSAGE("Invalid Transfer State (FILE238)", tr->state == Transfer::S_PROCESSING);

    // Count Transfers For File (239)
    req_ids.clear();
    transferDAO->getByFileId(FILE239_ID,req_ids);
    CPPUNIT_ASSERT_MESSAGE("No Transfer (FILE239)", !req_ids.empty());
    CPPUNIT_ASSERT_MESSAGE("Too many Transfer instances (FILE239)", req_ids.size() == 1);

    // Get File (239) Transfer
    tr.reset(transferDAO->get(FILE239_ID,req_ids.front()));
    CPPUNIT_ASSERT_MESSAGE("Null Transfer (FILE239)", tr.get() != 0);
    CPPUNIT_ASSERT_MESSAGE("Invalid Transfer State (FILE239)", tr->state == Transfer::S_PROCESSING);

    // Count Transfers For File (240)
    req_ids.clear();
    transferDAO->getByFileId(FILE240_ID,req_ids);
    CPPUNIT_ASSERT_MESSAGE("No Transfer (FILE237)", !req_ids.empty());
    CPPUNIT_ASSERT_MESSAGE("Too many Transfer instances (FILE237)", req_ids.size() == 1);

    // Get File (240) Transfer
    tr.reset(transferDAO->get(FILE240_ID,req_ids.front()));
    CPPUNIT_ASSERT_MESSAGE("Null Transfer (FILE240)", tr.get() != 0);
    CPPUNIT_ASSERT_MESSAGE("Invalid Transfer State (FILE240)", tr->state == Transfer::S_PROCESSING);

    // Check consistency in ActiveTransferCache
    ActiveTxCache& cache = ActionsConfig::instance().activeTxCache();

    // Count the number of request in the cache
    unsigned long count = cache.count();
    CPPUNIT_ASSERT_MESSAGE("Invalid number of active transfers", count == 13);

    // Check that files are in the cache
    shared_ptr<model::File> file_ptr = cache.getFile(FILE201_ID);
    CPPUNIT_ASSERT_MESSAGE("No file found in cache (FILE201)", file_ptr.get() != 0);
    try{
        file_ptr = cache.getFile(FILE202_ID);
        CPPUNIT_FAIL("file should not be in cache (FILE202)");
    } catch(glite::data::agents::RuntimeError&){/*OK*/}
    try{
        file_ptr = cache.getFile(FILE203_ID);
        CPPUNIT_FAIL("file should not be in cache (FILE203)");
    } catch(glite::data::agents::RuntimeError&){/*OK*/}
    try{
        file_ptr = cache.getFile(FILE204_ID);
        CPPUNIT_FAIL("file should not be in cache (FILE204)");
    } catch(glite::data::agents::RuntimeError&){/*OK*/}
    file_ptr = cache.getFile(FILE205_ID);
    CPPUNIT_ASSERT_MESSAGE("No file found in cache (FILE205)", file_ptr.get() != 0);
    file_ptr = cache.getFile(FILE206_ID);
    CPPUNIT_ASSERT_MESSAGE("No file found in cache (FILE206)", file_ptr.get() != 0);
    try{
        file_ptr = cache.getFile(FILE207_ID);
        CPPUNIT_FAIL("file should not be in cache (FILE207)");
    } catch(glite::data::agents::RuntimeError&){/*OK*/}
    try{
        file_ptr = cache.getFile(FILE208_ID);
        CPPUNIT_FAIL("file should not be in cache (FILE208)");
    } catch(glite::data::agents::RuntimeError&){/*OK*/}
    file_ptr = cache.getFile(FILE209_ID);
    CPPUNIT_ASSERT_MESSAGE("No file found in cache (FILE209)", file_ptr.get() != 0);
    file_ptr = cache.getFile(FILE210_ID);
    CPPUNIT_ASSERT_MESSAGE("No file found in cache (FILE210)", file_ptr.get() != 0);
    try{
        file_ptr = cache.getFile(FILE211_ID);
        CPPUNIT_FAIL("file should not be in cache (FILE211)");
    } catch(glite::data::agents::RuntimeError&){/*OK*/}
    try{
        file_ptr = cache.getFile(FILE212_ID);
        CPPUNIT_FAIL("file should not be in cache (FILE212)");
    } catch(glite::data::agents::RuntimeError&){/*OK*/}
    try{
        file_ptr = cache.getFile(FILE213_ID);
        CPPUNIT_FAIL("file should not be in cache (FILE213)");
    } catch(glite::data::agents::RuntimeError&){/*OK*/}
    try{
        file_ptr = cache.getFile(FILE214_ID);
        CPPUNIT_FAIL("file should not be in cache (FILE214)");
    } catch(glite::data::agents::RuntimeError&){/*OK*/}
    try{
        file_ptr = cache.getFile(FILE215_ID);
        CPPUNIT_FAIL("file should not be in cache (FILE215)");
    } catch(glite::data::agents::RuntimeError&){/*OK*/}
    try{
        file_ptr = cache.getFile(FILE216_ID);
        CPPUNIT_FAIL("file should not be in cache (FILE216)");
    } catch(glite::data::agents::RuntimeError&){/*OK*/}
    try{
        file_ptr = cache.getFile(FILE217_ID);
        CPPUNIT_FAIL("file should not be in cache (FILE217)");
    } catch(glite::data::agents::RuntimeError&){/*OK*/}
    try{
        file_ptr = cache.getFile(FILE218_ID);
        CPPUNIT_FAIL("file should not be in cache (FILE218)");
    } catch(glite::data::agents::RuntimeError&){/*OK*/}
    try{
        file_ptr = cache.getFile(FILE219_ID);
        CPPUNIT_FAIL("file should not be in cache (FILE219)");
    } catch(glite::data::agents::RuntimeError&){/*OK*/}
    try{
        file_ptr = cache.getFile(FILE220_ID);
        CPPUNIT_FAIL("file should not be in cache (FILE220)");
    } catch(glite::data::agents::RuntimeError&){/*OK*/}
    try{
        file_ptr = cache.getFile(FILE221_ID);
        CPPUNIT_FAIL("file should not be in cache (FILE221)");
    } catch(glite::data::agents::RuntimeError&){/*OK*/}
    try{
        file_ptr = cache.getFile(FILE222_ID);
        CPPUNIT_FAIL("file should not be in cache (FILE222)");
    } catch(glite::data::agents::RuntimeError&){/*OK*/}
    try{
        file_ptr = cache.getFile(FILE223_ID);
        CPPUNIT_FAIL("file should not be in cache (FILE223)");
    } catch(glite::data::agents::RuntimeError&){/*OK*/}
    try{
        file_ptr = cache.getFile(FILE224_ID);
        CPPUNIT_FAIL("file should not be in cache (FILE224)");
    } catch(glite::data::agents::RuntimeError&){/*OK*/}
    file_ptr = cache.getFile(FILE225_ID);
    CPPUNIT_ASSERT_MESSAGE("No file found in cache (FILE225)", file_ptr.get() != 0);
    file_ptr = cache.getFile(FILE226_ID);
    CPPUNIT_ASSERT_MESSAGE("No file found in cache (FILE226)", file_ptr.get() != 0);
    file_ptr = cache.getFile(FILE227_ID);
    CPPUNIT_ASSERT_MESSAGE("No file found in cache (FILE227)", file_ptr.get() != 0);
    try{
        file_ptr = cache.getFile(FILE228_ID);
        CPPUNIT_FAIL("file should not be in cache (FILE228)");
    } catch(glite::data::agents::RuntimeError&){/*OK*/}
    try{
        file_ptr = cache.getFile(FILE229_ID);
        CPPUNIT_FAIL("file should not be in cache (FILE229)");
    } catch(glite::data::agents::RuntimeError&){/*OK*/}
    try{
        file_ptr = cache.getFile(FILE230_ID);
        CPPUNIT_FAIL("file should not be in cache (FILE230)");
    } catch(glite::data::agents::RuntimeError&){/*OK*/}
    try{
        file_ptr = cache.getFile(FILE231_ID);
        CPPUNIT_FAIL("file should not be in cache (FILE231)");
    } catch(glite::data::agents::RuntimeError&){/*OK*/}
    try{
        file_ptr = cache.getFile(FILE232_ID);
        CPPUNIT_FAIL("file should not be in cache (FILE232)");
    } catch(glite::data::agents::RuntimeError&){/*OK*/}
    file_ptr = cache.getFile(FILE233_ID);
    CPPUNIT_ASSERT_MESSAGE("No file found in cache (FILE233)", file_ptr.get() != 0);
    try{
        file_ptr = cache.getFile(FILE235_ID);
        CPPUNIT_FAIL("file should not be in cache (FILE235)");
    } catch(glite::data::agents::RuntimeError&){/*OK*/}
    try{
        file_ptr = cache.getFile(FILE236_ID);
        CPPUNIT_FAIL("file should not be in cache (FILE236)");
    } catch(glite::data::agents::RuntimeError&){/*OK*/}
    file_ptr = cache.getFile(FILE237_ID);
    CPPUNIT_ASSERT_MESSAGE("No file found in cache (FILE237)", file_ptr.get() != 0);
    file_ptr = cache.getFile(FILE238_ID);
    CPPUNIT_ASSERT_MESSAGE("No file found in cache (FILE238)", file_ptr.get() != 0);
    file_ptr = cache.getFile(FILE239_ID);
    CPPUNIT_ASSERT_MESSAGE("No file found in cache (FILE239)", file_ptr.get() != 0);
    file_ptr = cache.getFile(FILE240_ID);
    CPPUNIT_ASSERT_MESSAGE("No file found in cache (FILE240)", file_ptr.get() != 0);

    // Check that jobs are in the cache
    shared_ptr<model::Job> job_ptr = cache.getJob(JOB201_ID);
    CPPUNIT_ASSERT_MESSAGE("No Job found in cache (JOB201)", job_ptr.get() != 0);
    job_ptr = cache.getJob(JOB202_ID);
    CPPUNIT_ASSERT_MESSAGE("No Job found in cache (JOB202)", job_ptr.get() != 0);
    job_ptr = cache.getJob(JOB203_ID);
    CPPUNIT_ASSERT_MESSAGE("No Job found in cache (JOB203)", job_ptr.get() != 0);
    try{
        job_ptr = cache.getJob(JOB204_ID);
        CPPUNIT_FAIL("Job should not be in cache (JOB204)");
    } catch(glite::data::agents::RuntimeError&){/*OK*/}
    try{
        job_ptr = cache.getJob(JOB205_ID);
        CPPUNIT_FAIL("Job should not be in cache (JOB205)");
    } catch(glite::data::agents::RuntimeError&){/*OK*/}
    try{
        job_ptr = cache.getJob(JOB206_ID);
        CPPUNIT_FAIL("Job should not be in cache (JOB206)");
    } catch(glite::data::agents::RuntimeError&){/*OK*/}
    job_ptr = cache.getJob(JOB207_ID);
    CPPUNIT_ASSERT_MESSAGE("No Job found in cache (JOB207)", job_ptr.get() != 0);
    try{
        job_ptr = cache.getJob(JOB208_ID);
        CPPUNIT_FAIL("Job should not be in cache (JOB208)");
    } catch(glite::data::agents::RuntimeError&){/*OK*/}
    job_ptr = cache.getJob(JOB209_ID);
    CPPUNIT_ASSERT_MESSAGE("No Job found in cache (JOB209)", job_ptr.get() != 0);
    job_ptr = cache.getJob(JOB210_ID);
    CPPUNIT_ASSERT_MESSAGE("No Job found in cache (JOB210)", job_ptr.get() != 0);

    // Check that transfers are in the cache
    shared_ptr<model::Transfer> tx_ptr = cache.get(FILE201_ID).tx;
    CPPUNIT_ASSERT_MESSAGE("No transfer found in cache (FILE201)", tx_ptr.get() != 0);
    try{
        tx_ptr = cache.get(FILE202_ID).tx;
        CPPUNIT_FAIL("transfer should not be in cache (FILE202)");
    } catch(glite::data::agents::RuntimeError&){/*OK*/}
    try{
        tx_ptr = cache.get(FILE203_ID).tx;
        CPPUNIT_FAIL("transfer should not be in cache (FILE203)");
    } catch(glite::data::agents::RuntimeError&){/*OK*/}
    try{
        tx_ptr = cache.get(FILE204_ID).tx;
        CPPUNIT_FAIL("transfer should not be in cache (FILE204)");
    } catch(glite::data::agents::RuntimeError&){/*OK*/}
    tx_ptr = cache.get(FILE205_ID).tx;
    CPPUNIT_ASSERT_MESSAGE("No transfer found in cache (FILE205)", tx_ptr.get() != 0);
    tx_ptr = cache.get(FILE206_ID).tx;
    CPPUNIT_ASSERT_MESSAGE("No transfer found in cache (FILE206)", tx_ptr.get() != 0);
    try{
        tx_ptr = cache.get(FILE207_ID).tx;
        CPPUNIT_FAIL("transfer should not be in cache (FILE207)");
    } catch(glite::data::agents::RuntimeError&){/*OK*/}
    try{
        tx_ptr = cache.get(FILE208_ID).tx;
        CPPUNIT_FAIL("transfer should not be in cache (FILE208)");
    } catch(glite::data::agents::RuntimeError&){/*OK*/}
    tx_ptr = cache.get(FILE209_ID).tx;
    CPPUNIT_ASSERT_MESSAGE("No transfer found in cache (FILE209)", tx_ptr.get() != 0);
    tx_ptr = cache.get(FILE210_ID).tx;
    CPPUNIT_ASSERT_MESSAGE("No transfer found in cache (FILE210)", tx_ptr.get() != 0);
    try{
        tx_ptr = cache.get(FILE211_ID).tx;
        CPPUNIT_FAIL("transfer should not be in cache (FILE211)");
    } catch(glite::data::agents::RuntimeError&){/*OK*/}
    try{
        tx_ptr = cache.get(FILE212_ID).tx;
        CPPUNIT_FAIL("transfer should not be in cache (FILE212)");
    } catch(glite::data::agents::RuntimeError&){/*OK*/}
    try{
        tx_ptr = cache.get(FILE213_ID).tx;
        CPPUNIT_FAIL("transfer should not be in cache (FILE213)");
    } catch(glite::data::agents::RuntimeError&){/*OK*/}
    try{
        tx_ptr = cache.get(FILE214_ID).tx;
        CPPUNIT_FAIL("transfer should not be in cache (FILE214)");
    } catch(glite::data::agents::RuntimeError&){/*OK*/}
    try{
        tx_ptr = cache.get(FILE215_ID).tx;
        CPPUNIT_FAIL("transfer should not be in cache (FILE215)");
    } catch(glite::data::agents::RuntimeError&){/*OK*/}
    try{
        tx_ptr = cache.get(FILE216_ID).tx;
        CPPUNIT_FAIL("transfer should not be in cache (FILE216)");
    } catch(glite::data::agents::RuntimeError&){/*OK*/}
    try{
        tx_ptr = cache.get(FILE217_ID).tx;
        CPPUNIT_FAIL("transfer should not be in cache (FILE217)");
    } catch(glite::data::agents::RuntimeError&){/*OK*/}
    try{
        tx_ptr = cache.get(FILE218_ID).tx;
        CPPUNIT_FAIL("transfer should not be in cache (FILE218)");
    } catch(glite::data::agents::RuntimeError&){/*OK*/}
    try{
        tx_ptr = cache.get(FILE219_ID).tx;
        CPPUNIT_FAIL("transfer should not be in cache (FILE219)");
    } catch(glite::data::agents::RuntimeError&){/*OK*/}
    try{
        tx_ptr = cache.get(FILE220_ID).tx;
        CPPUNIT_FAIL("transfer should not be in cache (FILE220)");
    } catch(glite::data::agents::RuntimeError&){/*OK*/}
    try{
        tx_ptr = cache.get(FILE221_ID).tx;
        CPPUNIT_FAIL("transfer should not be in cache (FILE221)");
    } catch(glite::data::agents::RuntimeError&){/*OK*/}
    try{
        tx_ptr = cache.get(FILE222_ID).tx;
        CPPUNIT_FAIL("transfer should not be in cache (FILE222)");
    } catch(glite::data::agents::RuntimeError&){/*OK*/}
    try{
        tx_ptr = cache.get(FILE223_ID).tx;
        CPPUNIT_FAIL("transfer should not be in cache (FILE223)");
    } catch(glite::data::agents::RuntimeError&){/*OK*/}
    try{
        tx_ptr = cache.get(FILE224_ID).tx;
        CPPUNIT_FAIL("transfer should not be in cache (FILE224)");
    } catch(glite::data::agents::RuntimeError&){/*OK*/}
    tx_ptr = cache.get(FILE225_ID).tx;
    CPPUNIT_ASSERT_MESSAGE("No transfer found in cache (FILE225)", tx_ptr.get() != 0);
    tx_ptr = cache.get(FILE226_ID).tx;
    CPPUNIT_ASSERT_MESSAGE("No transfer found in cache (FILE226)", tx_ptr.get() != 0);
    tx_ptr = cache.get(FILE227_ID).tx;
    CPPUNIT_ASSERT_MESSAGE("No transfer found in cache (FILE227)", tx_ptr.get() != 0);
    try{
        tx_ptr = cache.get(FILE228_ID).tx;
        CPPUNIT_FAIL("transfer should not be in cache (FILE228)");
    } catch(glite::data::agents::RuntimeError&){/*OK*/}
    try{
        tx_ptr = cache.get(FILE229_ID).tx;
        CPPUNIT_FAIL("transfer should not be in cache (FILE229)");
    } catch(glite::data::agents::RuntimeError&){/*OK*/}
    try{
        tx_ptr = cache.get(FILE230_ID).tx;
        CPPUNIT_FAIL("transfer should not be in cache (FILE230)");
    } catch(glite::data::agents::RuntimeError&){/*OK*/}
    try{
        tx_ptr = cache.get(FILE231_ID).tx;
        CPPUNIT_FAIL("transfer should not be in cache (FILE231)");
    } catch(glite::data::agents::RuntimeError&){/*OK*/}
    try{
        tx_ptr = cache.get(FILE232_ID).tx;
        CPPUNIT_FAIL("transfer should not be in cache (FILE232)");
    } catch(glite::data::agents::RuntimeError&){/*OK*/}
    tx_ptr = cache.get(FILE233_ID).tx;
    CPPUNIT_ASSERT_MESSAGE("No transfer found in cache (FILE233)", tx_ptr.get() != 0);
    try{
        tx_ptr = cache.get(FILE234_ID).tx;
        CPPUNIT_FAIL("transfer should not be in cache (FILE234)");
    } catch(glite::data::agents::RuntimeError&){/*OK*/}
    try{
        tx_ptr = cache.get(FILE235_ID).tx;
        CPPUNIT_FAIL("transfer should not be in cache (FILE235)");
    } catch(glite::data::agents::RuntimeError&){/*OK*/}
    try{
        tx_ptr = cache.get(FILE236_ID).tx;
        CPPUNIT_FAIL("transfer should not be in cache (FILE236)");
    } catch(glite::data::agents::RuntimeError&){/*OK*/}
    tx_ptr = cache.get(FILE237_ID).tx;
    CPPUNIT_ASSERT_MESSAGE("No transfer found in cache (FILE237)", tx_ptr.get() != 0);
    tx_ptr = cache.get(FILE238_ID).tx;
    CPPUNIT_ASSERT_MESSAGE("No transfer found in cache (FILE238)", tx_ptr.get() != 0);
    tx_ptr = cache.get(FILE239_ID).tx;
    CPPUNIT_ASSERT_MESSAGE("No transfer found in cache (FILE239)", tx_ptr.get() != 0);
    tx_ptr = cache.get(FILE240_ID).tx;
    CPPUNIT_ASSERT_MESSAGE("No transfer found in cache (FILE240)", tx_ptr.get() != 0);

}

/*
 * Test Execute Method
 */
void FetchTestCase::testExecute_4(){

    setUp_forTest4();

    // Execute Actions
    m_action->execute(CHANNEL_VO_SHARE_TYPE_NORMALIZED_ON_ACTIVE);

    // Create FileDAO instance
    scoped_ptr<dao::channel::FileDAO> fileDAO(dao::channel::DAOFactory::instance().createFileDAO(CHANNEL5_NAME, DaoHelper::ctx()));
    CPPUNIT_ASSERT_MESSAGE("Null FileDAO", fileDAO.get() != 0);

    // Create JobDAO instance
    scoped_ptr<dao::channel::JobDAO> jobDAO(dao::channel::DAOFactory::instance().createJobDAO(CHANNEL5_NAME, DaoHelper::ctx()));
    CPPUNIT_ASSERT_MESSAGE("Null JobDAO", jobDAO.get() != 0);

    // Create TransferDAO instance
    scoped_ptr<dao::channel::TransferDAO> transferDAO(dao::channel::DAOFactory::instance().createTransferDAO(CHANNEL5_NAME, DaoHelper::ctx()));
    CPPUNIT_ASSERT_MESSAGE("Null TransferDAO", transferDAO.get() != 0);

    // Check The Transfers using FileDAO

    // Get File instance (301) and Check it
    check_file(fileDAO,FILE301_ID,"FILE301",File::S_ACTIVE);

    // Get File instance (302) and Check it
    check_file(fileDAO,FILE302_ID,"FILE302",File::S_ACTIVE);

    // Get File instance (303) and Check it
    check_file(fileDAO,FILE303_ID,"FILE303",File::S_READY);

    // Get File instance (304) and Check it
    check_file(fileDAO,FILE304_ID,"FILE304",File::S_ACTIVE);

    // Get File instance (305) and Check it
    check_file(fileDAO,FILE305_ID,"FILE305",File::S_ACTIVE);

    // Get File instance (306) and Check it
    check_file(fileDAO,FILE306_ID,"FILE306",File::S_READY);

    // Get File instance (307) and Check it
    check_file(fileDAO,FILE307_ID,"FILE307",File::S_ACTIVE);

    // Get File instance (308) and Check it
    check_file(fileDAO,FILE308_ID,"FILE308",File::S_ACTIVE);

    // Get File instance (309) and Check it
    check_file(fileDAO,FILE309_ID,"FILE309",File::S_READY);

    // Check The Jobs using JobDAO

    // Get Job instance (301) and Check it
    check_job(jobDAO,JOB301_ID,"JOB301",Job::S_ACTIVE);

    // Get Job instance (302) and Check it
    check_job(jobDAO,JOB302_ID,"JOB302",Job::S_ACTIVE);

    // Get Job instance (303) and Check it
    check_job(jobDAO,JOB303_ID,"JOB303",Job::S_ACTIVE);

    // Check The Transfers using TransferDAO

    // Count Transfers For File (301)
    glite::data::agents::dao::StringArray req_ids;
    transferDAO->getByFileId(FILE301_ID,req_ids);
    CPPUNIT_ASSERT_MESSAGE("No Transfer (FILE301)", !req_ids.empty());
    CPPUNIT_ASSERT_MESSAGE("Too many Transfer instances (FILE301)", req_ids.size() == 1);

    // Get File (301) Transfer
    scoped_ptr<Transfer> tr(transferDAO->get(FILE301_ID,req_ids.front()));
    CPPUNIT_ASSERT_MESSAGE("Null Transfer (FILE301)", tr.get() != 0);
    CPPUNIT_ASSERT_MESSAGE("Invalid Transfer State (FILE301)", tr->state == Transfer::S_RECEIVED);

    // Count Transfers For File (302)
    req_ids.clear();
    transferDAO->getByFileId(FILE302_ID,req_ids);
    CPPUNIT_ASSERT_MESSAGE("No Transfer (FILE302)", !req_ids.empty());
    CPPUNIT_ASSERT_MESSAGE("Too many Transfer instances (FILE302)", req_ids.size() == 1);

    // Get File (302) Transfer
    tr.reset(transferDAO->get(FILE302_ID,req_ids.front()));
    CPPUNIT_ASSERT_MESSAGE("Null Transfer (FILE302)", tr.get() != 0);
    CPPUNIT_ASSERT_MESSAGE("Invalid Transfer State (FILE302)", tr->state == Transfer::S_RECEIVED);

    // Count Transfers For File (303)
    req_ids.clear();
    transferDAO->getByFileId(FILE303_ID,req_ids);
    CPPUNIT_ASSERT_MESSAGE("Too many Transfer instances (FILE303)", req_ids.empty());

    // Count Transfers For File (304)
    req_ids.clear();
    transferDAO->getByFileId(FILE304_ID,req_ids);
    CPPUNIT_ASSERT_MESSAGE("No Transfer (FILE304)", !req_ids.empty());
    CPPUNIT_ASSERT_MESSAGE("Too many Transfer instances (FILE304)", req_ids.size() == 1);

    // Get File (304) Transfer
    tr.reset(transferDAO->get(FILE304_ID,req_ids.front()));
    CPPUNIT_ASSERT_MESSAGE("Null Transfer (FILE304)", tr.get() != 0);
    CPPUNIT_ASSERT_MESSAGE("Invalid Transfer State (FILE304)", tr->state == Transfer::S_PROCESSING);

    // Count Transfers For File (305)
    req_ids.clear();
    transferDAO->getByFileId(FILE305_ID,req_ids);
    CPPUNIT_ASSERT_MESSAGE("No Transfer (FILE305)", !req_ids.empty());
    CPPUNIT_ASSERT_MESSAGE("Too many Transfer instances (FILE305)", req_ids.size() == 1);

    // Get File (305) Transfer
    tr.reset(transferDAO->get(FILE305_ID,req_ids.front()));
    CPPUNIT_ASSERT_MESSAGE("Null Transfer (FILE305)", tr.get() != 0);
    CPPUNIT_ASSERT_MESSAGE("Invalid Transfer State (FILE305)", tr->state == Transfer::S_RECEIVED);

    // Count Transfers For File (306)
    req_ids.clear();
    transferDAO->getByFileId(FILE306_ID,req_ids);
    CPPUNIT_ASSERT_MESSAGE("Too many Transfer instances (FILE306)", req_ids.empty());

    // Count Transfers For File (307)
    req_ids.clear();
    transferDAO->getByFileId(FILE307_ID,req_ids);
    CPPUNIT_ASSERT_MESSAGE("No Transfer (FILE307)", !req_ids.empty());
    CPPUNIT_ASSERT_MESSAGE("Too many Transfer instances (FILE307)", req_ids.size() == 1);

    // Get File (307) Transfer
    tr.reset(transferDAO->get(FILE307_ID,req_ids.front()));
    CPPUNIT_ASSERT_MESSAGE("Null Transfer (FILE307)", tr.get() != 0);
    CPPUNIT_ASSERT_MESSAGE("Invalid Transfer State (FILE307)", tr->state == Transfer::S_RECEIVED);

    // Count Transfers For File (308)
    req_ids.clear();
    transferDAO->getByFileId(FILE308_ID,req_ids);
    CPPUNIT_ASSERT_MESSAGE("No Transfer (FILE308)", !req_ids.empty());
    CPPUNIT_ASSERT_MESSAGE("Too many Transfer instances (FILE308)", req_ids.size() == 1);

    // Get File (308) Transfer
    tr.reset(transferDAO->get(FILE308_ID,req_ids.front()));
    CPPUNIT_ASSERT_MESSAGE("Null Transfer (FILE308)", tr.get() != 0);
    CPPUNIT_ASSERT_MESSAGE("Invalid Transfer State (FILE308)", tr->state == Transfer::S_RECEIVED);

    // Count Transfers For File (309)
    req_ids.clear();
    transferDAO->getByFileId(FILE309_ID,req_ids);
    CPPUNIT_ASSERT_MESSAGE("Too many Transfer instances (FILE309)", req_ids.empty());

    // Check consistency in ActiveTransferCache
    ActiveTxCache& cache = ActionsConfig::instance().activeTxCache();

    // Count the number of request in the cache
    unsigned long count = cache.count();
    CPPUNIT_ASSERT_MESSAGE("Invalid number of active transfers", count == 6);

    // Check that files are in the cache
    shared_ptr<model::File> file_ptr = cache.getFile(FILE301_ID);
    CPPUNIT_ASSERT_MESSAGE("No file found in cache (FILE301)", file_ptr.get() != 0);
    file_ptr = cache.getFile(FILE302_ID);
    CPPUNIT_ASSERT_MESSAGE("No file found in cache (FILE302)", file_ptr.get() != 0);
    try{
        file_ptr = cache.getFile(FILE303_ID);
        CPPUNIT_FAIL("file should not be in cache (FILE303)");
    } catch(glite::data::agents::RuntimeError&){/*OK*/}
    file_ptr = cache.getFile(FILE304_ID);
    CPPUNIT_ASSERT_MESSAGE("No file found in cache (FILE304)", file_ptr.get() != 0);
    file_ptr = cache.getFile(FILE305_ID);
    CPPUNIT_ASSERT_MESSAGE("No file found in cache (FILE305)", file_ptr.get() != 0);
    try{
        file_ptr = cache.getFile(FILE306_ID);
        CPPUNIT_FAIL("file should not be in cache (FILE306)");
    } catch(glite::data::agents::RuntimeError&){/*OK*/}
    file_ptr = cache.getFile(FILE307_ID);
    CPPUNIT_ASSERT_MESSAGE("No file found in cache (FILE307)", file_ptr.get() != 0);
    file_ptr = cache.getFile(FILE308_ID);
    CPPUNIT_ASSERT_MESSAGE("No file found in cache (FILE308)", file_ptr.get() != 0);
    try{
        file_ptr = cache.getFile(FILE309_ID);
        CPPUNIT_FAIL("file should not be in cache (FILE309)");
    } catch(glite::data::agents::RuntimeError&){/*OK*/}

    // Check that jobs are in the cache
    shared_ptr<model::Job> job_ptr = cache.getJob(JOB301_ID);
    CPPUNIT_ASSERT_MESSAGE("No Job found in cache (JOB301)", job_ptr.get() != 0);
    job_ptr = cache.getJob(JOB302_ID);
    CPPUNIT_ASSERT_MESSAGE("No Job found in cache (JOB302)", job_ptr.get() != 0);
    job_ptr = cache.getJob(JOB303_ID);
    CPPUNIT_ASSERT_MESSAGE("No Job found in cache (JOB303)", job_ptr.get() != 0);

    // Check that transfers are in the cache
    shared_ptr<model::Transfer> tx_ptr = cache.get(FILE301_ID).tx;
    CPPUNIT_ASSERT_MESSAGE("No transfer found in cache (FILE301)", tx_ptr.get() != 0);
    tx_ptr = cache.get(FILE302_ID).tx;
    CPPUNIT_ASSERT_MESSAGE("No transfer found in cache (FILE302)", tx_ptr.get() != 0);
    try{
        tx_ptr = cache.get(FILE303_ID).tx;
        CPPUNIT_FAIL("transfer should not be in cache (FILE303)");
    } catch(glite::data::agents::RuntimeError&){/*OK*/}
    tx_ptr = cache.get(FILE304_ID).tx;
    CPPUNIT_ASSERT_MESSAGE("No transfer found in cache (FILE304)", tx_ptr.get() != 0);
    tx_ptr = cache.get(FILE305_ID).tx;
    CPPUNIT_ASSERT_MESSAGE("No transfer found in cache (FILE305)", tx_ptr.get() != 0);
    try{
        tx_ptr = cache.get(FILE306_ID).tx;
        CPPUNIT_FAIL("transfer should not be in cache (FILE306)");
    } catch(glite::data::agents::RuntimeError&){/*OK*/}
    tx_ptr = cache.get(FILE307_ID).tx;
    CPPUNIT_ASSERT_MESSAGE("No transfer found in cache (FILE307)", tx_ptr.get() != 0);
    tx_ptr = cache.get(FILE308_ID).tx;
    CPPUNIT_ASSERT_MESSAGE("No transfer found in cache (FILE308)", tx_ptr.get() != 0);
    try{
        tx_ptr = cache.get(FILE309_ID).tx;
        CPPUNIT_FAIL("transfer should not be in cache (FILE309)");
    } catch(glite::data::agents::RuntimeError&){/*OK*/}

}

void FetchTestCase::test_bug30271(){

    // Create Action instance
    std::auto_ptr<Action> a(ActionFactory::instance().create("glite:Fetch"));
    CPPUNIT_ASSERT_MESSAGE("Null Action", a.get() != 0);
    CPPUNIT_ASSERT_MESSAGE("Invalid Action Class", dynamic_cast<Fetch*>(a.get()) != 0);

    // Store Action
    m_action.reset(dynamic_cast<Fetch*>(a.release()));
    CPPUNIT_ASSERT_MESSAGE("Null Action", m_action.get() != 0);

    // Set Context
    m_action->context(CHANNEL6_NAME,DaoHelper::ctx());

    m_action->execute();

}


