/**
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *  GLite Data Transfer Agents: Finalize Test Case
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>, Paolo Tedesco <paolo.tedesco@cern.ch>
 *  Version info: $Id: FinalizeTestCase.cpp,v 1.7 2008/04/11 11:02:56 ptedesco Exp $
 *  Release: $Name:  $
 *
 */
#include "Finalize.h"
#include "DaoHelper.h"
#include "glite/data/transfer/agent/action/ActionFactory.h"
#include "glite/data/transfer/agent/model/File.h"
#include "glite/data/transfer/agent/model/Job.h"
#include "glite/data/transfer/agent/dao/vo/FileDAO.h"
#include "glite/data/transfer/agent/dao/vo/JobDAO.h"
#include "glite/data/transfer/agent/dao/vo/DAOFactory.h"
#include "glite/data/transfer/agent/catalog/CatalogService.h"
#include "glite/data/transfer/agent/catalog/CatalogServiceFactory.h"
#include "glite/data/agents/surl-utility.h"

#include "dao_test_utils.h"

#include <memory>
#include <boost/scoped_ptr.hpp>
#include <vector>
#include <algorithm>
#include <cppunit/TestCase.h>
#include <cppunit/extensions/HelperMacros.h>

using namespace glite::data::transfer::agent;
using namespace glite::data::transfer::agent::action;
using namespace glite::data::transfer::agent::action::vo;
using namespace glite::data::transfer::agent::model;
using namespace glite::data::transfer::agent::catalog;

using boost::scoped_ptr;
using glite::data::agents::compact_surl;

/**
 * CPPUnit TestCase
 */
class FinalizeTestCase : public CppUnit::TestCase{
public:
    /**
     * set-up test case
     */
    void setUp ();
private:
    // Declare CPPUnit Methods
    // -------------------------------------------------------------------------
    CPPUNIT_TEST_SUITE( FinalizeTestCase);
    CPPUNIT_TEST( testExecute );
    CPPUNIT_TEST_SUITE_END();

    // Test Methods
    // -------------------------------------------------------------------------

    /**
     * Test Execute Method
     */
    void testExecute();
    
private:
    /**
     * The Action Instance
     */
    scoped_ptr<Finalize> m_action;
};

/*
 * Register Suit
 */
CPPUNIT_TEST_SUITE_NAMED_REGISTRATION( FinalizeTestCase, "FinalizeTestCase" );

/*
 * Declare Constants. This unit test foresee at beginning
 * - 1 DONE Job with:                         =>  FINISHED
 *        job_id      = 9-...-00
 *        files       = 90000001 (DONE)           FINISHED (no lfn)
 *                      90000002 (DONE)           FINISHED (no lfn)
 *
 * - 1 DONE_WITH_ERRORS Job with:             =>  FINISHED_DIRTY
 *        job_id      = 9-...-01
 *        files       = 90000003 (DONE)           FINISHED (no lfn)
 *                      90000004 (FAILED)         FAILED  (no lfn)
 *
 * - 1 DONE Job with:                         =>  FINISHED
 *        job_id      = 9-...-02
 *        files       = 90000005 (DONE)           FINISHED (lfn)
 *                      90000006 (DONE)           FINISHED (lfn)
 *
 * - 1 DONE_WITH_ERRORS Job with:             =>  FINISHED_DIRTY
 *        job_id      = 9-...-03
 *        files       = 90000007 (DONE)           FINISHED (lfn)
 *                      90000008 (FAILED)         FAILED
 *
 * - 1 FINISHING Job with:                    =>  FINISHED
 *        job_id      = 9-...-04
 *        files       = 90000009 (FINISHING)      FINISHED (no lfn)
 *                      90000010 (FINISHING)      FINISHED (no lfn)
 *
 * - 1 FINISHING Job with:                    =>  FINISHING
 *        job_id      = 9-...-05
 *        files       = 90000011 (FINISHING)      FINISHED (lfn)
 *                      90000012 (W_C_REG)        W_C_REG  (lfn)
 *
 * - 1 FINISHING Job with:                    =>  
 *        job_id      = 9-...-06
 *        files       = 90000013 (FINISHED)       
 *                      90000014 (W_C_REG)        
 *
 * - 1 FINISHING Job with:                    =>  FINISHED
 *        job_id      = 9-...-07
 *        files       = 90000015 (FINISHED)       
 *                      90000016 (FINISHED)        
 *
 * - 1 DONE Job with:                         =>  FINISHING
 *        job_id      = 9-...-08
 *        files       = 90000017 (DONE)           FINISHED (lfn)
 *                      90000018 (DONE)           W_C_REG (lfn should fail)
 *
 * - 1 DONE Job with:                         =>  FINISHED_DIRTY
 *        job_id      = 9-...-09
 *        files       = 90000019 (DONE)           FINISHED (lfn)
 *                      90000020 (DONE)           FAILED (lfn not registered)
 *
 * - 1 DONE Job with:                         =>  CANCELED (cancelJob = Y)
 *        job_id      = 9-...-10
 *        files       = 90000021 (DONE)           CANCELED
 *                      90000022 (DONE)           CANCELED
 *
 * - 1 DONE Job with:                         =>  CANCELED (cancelJob = Y)
 *        job_id      = 9-...-11
 *        files       = 90000023 (DONE)           CANCELED (with lfn)
 *                      90000024 (DONE)           CANCELED (with lfn)
 *
 * - 1 FINISHING Job with:                    =>  CANCELED (cancelJob = Y)
 *        job_id      = 9-...-12
 *        files       = 90000025 (FINISHING)      FINISHED (no lfn)
 *                      90000026 (FINISHING)      FINISHED (no lfn)
 *
 * - 1 FINISHING Job with:                    =>  CANCELED (cancelJob = Y)
 *        job_id      = 9-...-13
 *        files       = 90000027 (FINISHING)      FINISHED (with lfn)
 *                      90000028 (FINISHING)      FINISHED (with lfn)
 *
 * - 1 DONE Job with:                         =>  FINISHED (skip-registation)
 *        job_id      = 9-...-14
 *        files       = 90000029 (DONE)           FINISHED (surl not registered)
 *                      90000030 (DONE)           FINISHED (surl not registered)
 */
namespace {
const char * const JOB1_ID           = "90000000-0000-0000-0000-000000000000";
const char * const JOB2_ID           = "90000000-0000-0000-0000-000000000001";
const char * const JOB3_ID           = "90000000-0000-0000-0000-000000000002";
const char * const JOB4_ID           = "90000000-0000-0000-0000-000000000003";
const char * const JOB5_ID           = "90000000-0000-0000-0000-000000000004";
const char * const JOB6_ID           = "90000000-0000-0000-0000-000000000005";
const char * const JOB7_ID           = "90000000-0000-0000-0000-000000000006";
const char * const JOB8_ID           = "90000000-0000-0000-0000-000000000007";
const char * const JOB9_ID           = "90000000-0000-0000-0000-000000000008";
const char * const JOB10_ID          = "90000000-0000-0000-0000-000000000009";
const char * const JOB11_ID          = "90000000-0000-0000-0000-000000000010";
const char * const JOB12_ID          = "90000000-0000-0000-0000-000000000011";
const char * const JOB13_ID          = "90000000-0000-0000-0000-000000000012";
const char * const JOB14_ID          = "90000000-0000-0000-0000-000000000013";
const char * const JOB15_ID          = "90000000-0000-0000-0000-000000000014";
const char * const FILE1_ID          = "90000001";
const char * const FILE2_ID          = "90000002";
const char * const FILE3_ID          = "90000003";
const char * const FILE4_ID          = "90000004";
const char * const FILE5_ID          = "90000005";
const char * const FILE6_ID          = "90000006";
const char * const FILE7_ID          = "90000007";
const char * const FILE8_ID          = "90000008";
const char * const FILE9_ID          = "90000009";
const char * const FILE10_ID         = "90000010";
const char * const FILE11_ID         = "90000011";
const char * const FILE12_ID         = "90000012";
const char * const FILE13_ID         = "90000013";
const char * const FILE14_ID         = "90000014";
const char * const FILE15_ID         = "90000015";
const char * const FILE16_ID         = "90000016";
const char * const FILE17_ID         = "90000017";
const char * const FILE18_ID         = "90000018";
const char * const FILE19_ID         = "90000019";
const char * const FILE20_ID         = "90000020";
const char * const FILE21_ID         = "90000021";
const char * const FILE22_ID         = "90000022";
const char * const FILE23_ID         = "90000023";
const char * const FILE24_ID         = "90000024";
const char * const FILE25_ID         = "90000025";
const char * const FILE26_ID         = "90000026";
const char * const FILE27_ID         = "90000027";
const char * const FILE28_ID         = "90000028";
const char * const FILE29_ID         = "90000029";
const char * const FILE30_ID         = "90000030";
const char * const FILE5_LFN         = "/tmp/src9-5.txt";
const char * const FILE6_LFN         = "/tmp/src9-6.txt";
const char * const FILE7_LFN         = "/tmp/src9-7.txt";
const char * const FILE8_LFN         = "/tmp/src9-8.txt";
const char * const FILE11_LFN        = "/tmp/src9-11.txt";
const char * const FILE12_LFN        = "/tmp/src9-12.txt";
const char * const FILE17_LFN        = "/tmp/src9-17.txt";
const char * const FILE18_LFN        = "/fail/src9-18.txt";
const char * const FILE19_LFN        = "/tmp/src9-19.txt";
const char * const FILE20_LFN        = "/tmp/src9-20.txt";
const char * const FILE23_LFN        = "/tmp/src9-23.txt";
const char * const FILE24_LFN        = "/tmp/src9-24.txt";
const char * const FILE27_LFN        = "/tmp/src9-27.txt";
const char * const FILE28_LFN        = "/tmp/src9-28.txt";
const char * const FILE29_LFN        = "/tmp/src9-29.txt";
const char * const FILE30_LFN        = "/tmp/src9-30.txt";
const char * const FILE5_SRC_SURL    = "srm://host.cern.ch:8443/srm/managerv1?SFN=/castor/cern.ch/SE/tmp/src9-5.txt";
const char * const FILE5_DST_SURL    = "srm://host.ral.ac.uk/pnfs/ral.ac.uk/SE/VOLATILE/tmp/src9-5.txt";
const char * const FILE6_SRC_SURL    = "srm://host.cern.ch:8443/srm/managerv1?SFN=/castor/cern.ch/SE/tmp/src9-6.txt";
const char * const FILE6_DST_SURL    = "srm://host.ral.ac.uk/pnfs/ral.ac.uk/SE/VOLATILE/tmp/src9-6.txt";
const char * const FILE7_SRC_SURL    = "srm://host.cern.ch:8443/srm/managerv1?SFN=/castor/cern.ch/SE/tmp/src9-7.txt";
const char * const FILE7_DST_SURL    = "srm://host.ral.ac.uk/pnfs/ral.ac.uk/SE/VOLATILE/tmp/src9-7.txt";
const char * const FILE8_SRC_SURL    = "srm://host.cern.ch:8443/srm/managerv1?SFN=/castor/cern.ch/SE/tmp/src9-8.txt";
const char * const FILE8_DST_SURL    = "srm://host.ral.ac.uk/pnfs/ral.ac.uk/SE/VOLATILE/tmp/src9-8.txt";
const char * const FILE11_DST_SURL   = "srm://host.ral.ac.uk/pnfs/ral.ac.uk/SE/VOLATILE/tmp/src9-11.txt";
const char * const FILE11_SRC_SURL   = "srm://host.cern.ch:8443/srm/managerv1?SFN=/castor/cern.ch/SE/tmp/src9-11.txt";
const char * const FILE12_DST_SURL   = "srm://host.ral.ac.uk/pnfs/ral.ac.uk/SE/VOLATILE/tmp/src9-12.txt";
const char * const FILE12_SRC_SURL   = "srm://host.cern.ch:8443/srm/managerv1?SFN=/castor/cern.ch/SE/tmp/src9-12.txt";
const char * const FILE17_DST_SURL   = "srm://host.ral.ac.uk/pnfs/ral.ac.uk/SE/VOLATILE/tmp/src9-17.txt";
const char * const FILE17_SRC_SURL   = "srm://host.cern.ch:8443/srm/managerv1?SFN=/castor/cern.ch/SE/tmp/src9-17.txt";
const char * const FILE18_DST_SURL   = "srm://host.ral.ac.uk/pnfs/ral.ac.uk/SE/VOLATILE/fail/src9-18.txt";
const char * const FILE18_SRC_SURL   = "srm://host.cern.ch:8443/srm/managerv1?SFN=/castor/cern.ch/SE/fail/src9-18.txt";
const char * const FILE19_DST_SURL   = "srm://host.ral.ac.uk/pnfs/ral.ac.uk/SE/VOLATILE/tmp/src9-19.txt";
const char * const FILE19_SRC_SURL   = "srm://host.cern.ch:8443/srm/managerv1?SFN=/castor/cern.ch/SE/tmp/src9-19.txt";
const char * const FILE20_DST_SURL   = "srm://host.ral.ac.uk/pnfs/ral.ac.uk/SE/VOLATILE/tmp/src9-20.txt";
const char * const FILE20_SRC_SURL   = "srm://host.cern.ch:8443/srm/managerv1?SFN=/castor/cern.ch/SE/tmp/src9-20.txt";
const char * const FILE23_DST_SURL   = "srm://host.ral.ac.uk/pnfs/ral.ac.uk/SE/VOLATILE/tmp/src9-23.txt";
const char * const FILE23_SRC_SURL   = "srm://host.cern.ch:8443/srm/managerv1?SFN=/castor/cern.ch/SE/tmp/src9-23.txt";
const char * const FILE24_DST_SURL   = "srm://host.ral.ac.uk/pnfs/ral.ac.uk/SE/VOLATILE/tmp/src9-24.txt";
const char * const FILE24_SRC_SURL   = "srm://host.cern.ch:8443/srm/managerv1?SFN=/castor/cern.ch/SE/tmp/src9-24.txt";
const char * const FILE27_DST_SURL   = "srm://host.ral.ac.uk/pnfs/ral.ac.uk/SE/VOLATILE/tmp/src9-27.txt";
const char * const FILE27_SRC_SURL   = "srm://host.cern.ch:8443/srm/managerv1?SFN=/castor/cern.ch/SE/tmp/src9-27.txt";
const char * const FILE28_DST_SURL   = "srm://host.ral.ac.uk/pnfs/ral.ac.uk/SE/VOLATILE/tmp/src9-28.txt";
const char * const FILE28_SRC_SURL   = "srm://host.cern.ch:8443/srm/managerv1?SFN=/castor/cern.ch/SE/tmp/src9-28.txt";
const char * const FILE29_DST_SURL   = "srm://host.ral.ac.uk/pnfs/ral.ac.uk/SE/VOLATILE/tmp/src9-29.txt";
const char * const FILE29_SRC_SURL   = "srm://host.cern.ch:8443/srm/managerv1?SFN=/castor/cern.ch/SE/tmp/src9-29.txt";
const char * const FILE30_DST_SURL   = "srm://host.ral.ac.uk/pnfs/ral.ac.uk/SE/VOLATILE/tmp/src9-30.txt";
const char * const FILE30_SRC_SURL   = "srm://host.cern.ch:8443/srm/managerv1?SFN=/castor/cern.ch/SE/tmp/src9-30.txt";
const char * const VO1_NAME          = "FINALIZE_VO_1";
}

extern std::string g_catalog_endpoint;

/*
 * setUp
 *
 * set-up test case
 */
void FinalizeTestCase::setUp(){

    // Create Action instance
    std::auto_ptr<Action> a(ActionFactory::instance().create("glite:Finalize"));
    CPPUNIT_ASSERT_MESSAGE("Null Action", a.get() != 0);
    CPPUNIT_ASSERT_MESSAGE("Invalid Action Class", dynamic_cast<Finalize*>(a.get()) != 0);

    // Store Action        
    m_action.reset(dynamic_cast<Finalize*>(a.release()));
    CPPUNIT_ASSERT_MESSAGE("Null Action", m_action.get() != 0);
    
    // Get Catalog Type
    std::string catalog_type = CatalogServiceFactory::instance().localCatalogServiceType();
    if(catalog_type.empty()){
        catalog_type = CatalogServiceFactory::instance().globalCatalogServiceType();
        if(catalog_type.empty()){
            CPPUNIT_FAIL("No Catalog Type Specified");
        }
    }
    // Create CatalogService Instance
    scoped_ptr<CatalogService> catalog(CatalogServiceFactory::instance().create(g_catalog_endpoint,catalog_type));
    CPPUNIT_ASSERT_MESSAGE("Null CatalogService", catalog.get() != 0);

    CatalogService::ReplicaStatArray names;
    names.push_back(CatalogService::ReplicaStat(FILE5_LFN ,compact_surl(FILE5_SRC_SURL,"httpg://host.cern.ch:8443",false)));
    names.push_back(CatalogService::ReplicaStat(FILE6_LFN ,compact_surl(FILE6_SRC_SURL,"httpg://host.cern.ch:8443",false)));
    names.push_back(CatalogService::ReplicaStat(FILE7_LFN ,compact_surl(FILE7_SRC_SURL,"httpg://host.cern.ch:8443",false)));
    names.push_back(CatalogService::ReplicaStat(FILE8_LFN ,compact_surl(FILE8_SRC_SURL,"httpg://host.cern.ch:8443",false)));
    names.push_back(CatalogService::ReplicaStat(FILE11_LFN,compact_surl(FILE11_SRC_SURL,"httpg://host.cern.ch:8443",false)));
    names.push_back(CatalogService::ReplicaStat(FILE12_LFN,compact_surl(FILE12_SRC_SURL,"httpg://host.cern.ch:8443",false)));
    names.push_back(CatalogService::ReplicaStat(FILE17_LFN,compact_surl(FILE17_SRC_SURL,"httpg://host.cern.ch:8443",false)));
    names.push_back(CatalogService::ReplicaStat(FILE19_LFN,compact_surl(FILE19_SRC_SURL,"httpg://host.cern.ch:8443",false)));
    names.push_back(CatalogService::ReplicaStat(FILE23_LFN,compact_surl(FILE23_SRC_SURL,"httpg://host.cern.ch:8443",false)));
    names.push_back(CatalogService::ReplicaStat(FILE24_LFN,compact_surl(FILE24_SRC_SURL,"httpg://host.cern.ch:8443",false)));
    names.push_back(CatalogService::ReplicaStat(FILE27_LFN,compact_surl(FILE27_SRC_SURL,"httpg://host.cern.ch:8443",false)));
    names.push_back(CatalogService::ReplicaStat(FILE28_LFN,compact_surl(FILE28_SRC_SURL,"httpg://host.cern.ch:8443",false)));
    names.push_back(CatalogService::ReplicaStat(FILE29_LFN,compact_surl(FILE29_SRC_SURL,"httpg://host.cern.ch:8443",false)));
    names.push_back(CatalogService::ReplicaStat(FILE30_LFN,compact_surl(FILE30_SRC_SURL,"httpg://host.cern.ch:8443",false)));
    CatalogService::CatalogResult result;
    catalog->registerSurls(names,result);
    CPPUNIT_ASSERT_MESSAGE("Register Surls failed", result.status == CatalogService::CatalogResult::R_SUCCESS);

}

/*
 *
 * Test Execute Method
 */
void FinalizeTestCase::testExecute(){

    // Set Context
    m_action->context(VO1_NAME,DaoHelper::ctx());
    
    // Execute Action
    m_action->execute();
    
    // Create FileDAO instance
    scoped_ptr<dao::vo::FileDAO> fileDAO(dao::vo::DAOFactory::instance().createFileDAO(VO1_NAME, DaoHelper::ctx()));
    CPPUNIT_ASSERT_MESSAGE("Null FileDAO", fileDAO.get() != 0);
   
    // Create FileDAO instance
    scoped_ptr<dao::vo::JobDAO> jobDAO(dao::vo::DAOFactory::instance().createJobDAO(VO1_NAME, DaoHelper::ctx()));
    CPPUNIT_ASSERT_MESSAGE("Null JobDAO", jobDAO.get() != 0);
    
    // Get Catalog Type
    std::string catalog_type = CatalogServiceFactory::instance().globalCatalogServiceType();
    if(catalog_type.empty()){
        catalog_type = CatalogServiceFactory::instance().localCatalogServiceType();
        if(catalog_type.empty()){
            CPPUNIT_FAIL("No Catalog Type Specified");
        }
    }
    // Create CatalogService Instance
    scoped_ptr<CatalogService> catalog(CatalogServiceFactory::instance().create(g_catalog_endpoint,catalog_type));
    CPPUNIT_ASSERT_MESSAGE("Null CatalogService", catalog.get() != 0);

    //----------------------------------------------------------------------
    // Job 1
    //----------------------------------------------------------------------

    // Get TransferJob instance (1) and Check it
    check_job(jobDAO,JOB1_ID,"JOB1",Job::S_FINISHED);

    // Get FileTransfer instance (1) and Check it
    check_file(fileDAO,FILE1_ID,"FILE1",File::S_FINISHED);
    // Get File instance (2) and Check it
    check_file(fileDAO,FILE2_ID,"FILE2",File::S_FINISHED);

    //----------------------------------------------------------------------
    // Job 2
    //----------------------------------------------------------------------
    
    // Get TransferJob instance (2) and Check it
    check_job(jobDAO,JOB2_ID,"JOB2",Job::S_FINISHED_DIRTY);

    // Get File instance (3) and Check it
    check_file(fileDAO,FILE3_ID,"FILE3",File::S_FINISHED);
    // Get File instance (4) and Check it
    check_file(fileDAO,FILE4_ID,"FILE4",File::S_FAILED);

    //----------------------------------------------------------------------
    // Job 3
    //----------------------------------------------------------------------
    
    // Get TransferJob instance (3) and Check it
    check_job(jobDAO,JOB3_ID,"JOB3",Job::S_FINISHED);

    // Get File instance (5) and Check it
    check_file(fileDAO,FILE5_ID,"FILE5",File::S_FINISHED);
    // Get File instance (6) and Check it
    check_file(fileDAO,FILE6_ID,"FILE6",File::S_FINISHED);

    //----------------------------------------------------------------------
    // Job 4
    //----------------------------------------------------------------------
    
    // Get TransferJob instance (4) and Check it
    check_job(jobDAO,JOB4_ID,"JOB4",Job::S_FINISHED_DIRTY);

    // Get File instance (7) and Check it
    check_file(fileDAO,FILE7_ID,"FILE7",File::S_FINISHED);
    // Get File instance (8) and Check it
    check_file(fileDAO,FILE8_ID,"FILE8",File::S_FAILED);

    //----------------------------------------------------------------------
    // Job 5
    //----------------------------------------------------------------------
    
    // Get TransferJob instance (5) and Check it
    check_job(jobDAO,JOB5_ID,"JOB5",Job::S_FINISHED);

    // Get File instance (9) and Check it
    check_file(fileDAO,FILE9_ID,"FILE9",File::S_FINISHED);
    // Get File instance (10) and Check it
    check_file(fileDAO,FILE10_ID,"FILE10",File::S_FINISHED);

    //----------------------------------------------------------------------
    // Job 6
    //----------------------------------------------------------------------
    
    // Get TransferJob instance (6) and Check it
    check_job(jobDAO,JOB6_ID,"JOB6",Job::S_FINISHING);

    // Get File instance (11) and Check it
    check_file(fileDAO,FILE11_ID,"FILE11",File::S_FINISHED);
    // Get File instance (12) and Check it
    check_file(fileDAO,FILE12_ID,"FILE12",File::S_WAITING_CATALOG_REGISTRATION);

    //----------------------------------------------------------------------
    // Job 7
    //----------------------------------------------------------------------
    
    // Get TransferJob instance (7) and Check it
    check_job(jobDAO,JOB7_ID,"JOB7",Job::S_FINISHING);

    // Get File instance (13) and Check it
    check_file(fileDAO,FILE13_ID,"FILE13",File::S_FINISHED);
    // Get File instance (14) and Check it
    check_file(fileDAO,FILE14_ID,"FILE14",File::S_WAITING_CATALOG_REGISTRATION);

    //----------------------------------------------------------------------
    // Job 8
    //----------------------------------------------------------------------
    
    // Get TransferJob instance (8) and Check it
    check_job(jobDAO,JOB8_ID,"JOB8",Job::S_FINISHED);

    // Get File instance (15) and Check it
    check_file(fileDAO,FILE15_ID,"FILE15",File::S_FINISHED);
    // Get File instance (16) and Check it
    check_file(fileDAO,FILE16_ID,"FILE16",File::S_FINISHED);

    //----------------------------------------------------------------------
    // Job 9
    //----------------------------------------------------------------------
    
    // Get TransferJob instance (9) and Check it
    check_job(jobDAO,JOB9_ID,"JOB9",Job::S_FINISHING);

    // Get File instance (17) and Check it
    check_file(fileDAO,FILE17_ID,"FILE17",File::S_FINISHED);
    // Get File instance (18) and Check it
    check_file(fileDAO,FILE18_ID,"FILE18",File::S_WAITING_CATALOG_REGISTRATION);

    //----------------------------------------------------------------------
    // Job 10
    //----------------------------------------------------------------------
    
    // Get TransferJob instance (10) and Check it
    check_job(jobDAO,JOB10_ID,"JOB10",Job::S_FINISHING);

    // Get File instance (19) and Check it
    check_file(fileDAO,FILE19_ID,"FILE19",File::S_FINISHED);
    // Get File instance (20) and Check it
    check_file(fileDAO,FILE20_ID,"FILE20",File::S_WAITING_CATALOG_REGISTRATION);

    //----------------------------------------------------------------------
    // Job 11
    //----------------------------------------------------------------------
    
    // Get TransferJob instance (11) and Check it
    check_job(jobDAO,JOB11_ID,"JOB11",Job::S_CANCELED);

    // Get File instance (21) and Check it
    check_file(fileDAO,FILE21_ID,"FILE21",File::S_CANCELED);
    // Get File instance (22) and Check it
    check_file(fileDAO,FILE22_ID,"FILE22",File::S_CANCELED);

    //----------------------------------------------------------------------
    // Job 12
    //----------------------------------------------------------------------
    
    // Get TransferJob instance (12) and Check it
    check_job(jobDAO,JOB12_ID,"JOB12",Job::S_CANCELED);

    // Get File instance (23) and Check it
    check_file(fileDAO,FILE23_ID,"FILE23",File::S_CANCELED);
    // Get File instance (24) and Check it
    check_file(fileDAO,FILE24_ID,"FILE24",File::S_CANCELED);

    //----------------------------------------------------------------------
    // Job 13
    //----------------------------------------------------------------------
    
    // Get TransferJob instance (13) and Check it
    check_job(jobDAO,JOB13_ID,"JOB13",Job::S_CANCELED);

    // Get File instance (25) and Check it
    check_file(fileDAO,FILE25_ID,"FILE25",File::S_FINISHED);
    // Get File instance (26) and Check it
    check_file(fileDAO,FILE26_ID,"FILE26",File::S_FINISHED);

    //----------------------------------------------------------------------
    // Job 14
    //----------------------------------------------------------------------
    
    // Get TransferJob instance (14) and Check it
    check_job(jobDAO,JOB14_ID,"JOB14",Job::S_CANCELED);

    // Get File instance (27) and Check it
    check_file(fileDAO,FILE27_ID,"FILE27",File::S_FINISHED);
    // Get File instance (28) and Check it
    check_file(fileDAO,FILE28_ID,"FILE28",File::S_FINISHED);

    //----------------------------------------------------------------------
    // Job 15
    //----------------------------------------------------------------------
    
    // Get TransferJob instance (15) and Check it
    check_job(jobDAO,JOB15_ID,"JOB15",Job::S_FINISHED);

    // Get File instance (29) and Check it
    check_file(fileDAO,FILE29_ID,"FILE29",File::S_FINISHED);
    // Get File instance (30) and Check it
    check_file(fileDAO,FILE30_ID,"FILE30",File::S_FINISHED);

    // -------------------------------------------------
    // Check Replicas using CatalogService
    // ---------------------------------------------------------------------

    // Get the list of replicas for the requested files
    CatalogService::StringArray logicals;
    logicals.push_back(FILE5_LFN);
    logicals.push_back(FILE6_LFN);
    logicals.push_back(FILE7_LFN);
    logicals.push_back(FILE8_LFN);
    logicals.push_back(FILE11_LFN);
    logicals.push_back(FILE12_LFN);
    logicals.push_back(FILE17_LFN);
    logicals.push_back(FILE19_LFN);
    logicals.push_back(FILE20_LFN);
    logicals.push_back(FILE23_LFN);
    logicals.push_back(FILE24_LFN);
    logicals.push_back(FILE27_LFN);
    logicals.push_back(FILE28_LFN);
    logicals.push_back(FILE29_LFN);
    logicals.push_back(FILE30_LFN);
    std::vector<CatalogService::StringArray> surls;
    CatalogService::CatalogResult result;
    catalog->listSurls(logicals,"",surls,result);
    CPPUNIT_ASSERT_MESSAGE("ListSurls Failed", result.status == CatalogService::CatalogResult::R_SOME_FAILURES);

    // Check if replica (5) is registered on the catalog
    bool found = std::find(surls[0].begin(), surls[0].end(),FILE5_DST_SURL) != surls[0].end();
    CPPUNIT_ASSERT_MESSAGE("No replica registered (FILE5)", found);
    
    // Check if replica (6) is registered on the catalog
    found = std::find(surls[1].begin(), surls[1].end(),FILE6_DST_SURL) != surls[1].end();
    CPPUNIT_ASSERT_MESSAGE("No replica registered (FILE6)", found);
    
    // Check if replica (7) is registered on the catalog
    found = std::find(surls[2].begin(), surls[2].end(),FILE7_DST_SURL) != surls[2].end();
    CPPUNIT_ASSERT_MESSAGE("No replica registered (FILE7)", found);

    // Check if replica (8) is not registered on the catalog
    found = std::find(surls[3].begin(), surls[3].end(),FILE8_DST_SURL) != surls[3].end();
    CPPUNIT_ASSERT_MESSAGE("No replica is registered (FILE8)", !found);

    // Check if replica (11) is not registered on the catalog
    found = std::find(surls[4].begin(), surls[4].end(),FILE11_DST_SURL) != surls[4].end();
    CPPUNIT_ASSERT_MESSAGE("No replica is registered (FILE11)", found);

    // Check if replica (12) is registered on the catalog
    found = std::find(surls[5].begin(), surls[5].end(),FILE12_DST_SURL) != surls[5].end();
    CPPUNIT_ASSERT_MESSAGE("Replica registered (FILE12)", !found);

    // Check if replica (17) is registered on the catalog
    found = std::find(surls[6].begin(), surls[6].end(),FILE17_DST_SURL) != surls[6].end();
    CPPUNIT_ASSERT_MESSAGE("No replica registered (FILE17)", found);

    // Check if replica (19) is registered on the catalog
    found = std::find(surls[7].begin(), surls[7].end(),FILE19_DST_SURL) != surls[7].end();
    CPPUNIT_ASSERT_MESSAGE("No replica registered (FILE19)", found);

    // Check if replica (20) is registered on the catalog
    found = std::find(surls[8].begin(), surls[8].end(),FILE20_DST_SURL) != surls[8].end();
    CPPUNIT_ASSERT_MESSAGE("Replica registered (FILE20)", !found);

    // Check if replica (23) is registered on the catalog
    found = std::find(surls[9].begin(), surls[9].end(),FILE23_DST_SURL) != surls[9].end();
    CPPUNIT_ASSERT_MESSAGE("Replica registered (FILE23)", !found);

    // Check if replica (24) is registered on the catalog
    found = std::find(surls[10].begin(), surls[10].end(),FILE24_DST_SURL) != surls[10].end();
    CPPUNIT_ASSERT_MESSAGE("Replica registered (FILE24)", !found);

    // Check if replica (27) is registered on the catalog
    found = std::find(surls[11].begin(), surls[11].end(),FILE27_DST_SURL) != surls[11].end();
    CPPUNIT_ASSERT_MESSAGE("Replica not registered (FILE27)", found);

    // Check if replica (28) is registered on the catalog
    found = std::find(surls[12].begin(), surls[12].end(),FILE28_DST_SURL) != surls[10].end();
    CPPUNIT_ASSERT_MESSAGE("Replica not registered (FILE28)", found);

    // Check if replica (29) is registered on the catalog
    found = std::find(surls[13].begin(), surls[13].end(),FILE29_DST_SURL) != surls[13].end();
    CPPUNIT_ASSERT_MESSAGE("Replica registered (FILE29)", !found);

    // Check if replica (30) is registered on the catalog
    found = std::find(surls[14].begin(), surls[14].end(),FILE30_DST_SURL) != surls[14].end();
    CPPUNIT_ASSERT_MESSAGE("Replica registered (FILE30)", !found);

}
