/*
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *  GLite Data Transfer Agents: main function
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *  Version info: $Id: TestActions.cpp,v 1.26 2009/07/06 17:15:20 molnarzs Exp $
 *  Release: $Name:  $
 *
 */
#include "DaoHelper.h"
#include "glite/data/config/service/glite-config.h"
#include "glite/data/config/service/glite-config-register.hxx"

#include <stdlib.h>
#include <string>
#include <iostream>
#include <unistd.h>
#include <cppunit/extensions/TestFactoryRegistry.h>
#include <cppunit/ui/text/TestRunner.h>
#include <cppunit/CompilerOutputter.h>

using std::cout;
using std::cerr;
using std::cin;
using std::endl;

namespace {
const char * const CONFIG_NAME = "glite-transfer-agent-test-actions";
const char * const PROGNAME    = "glite_data_transfer_agent_test_actions";
const char * const PROGVERSION = "0.0";
const char * const PROGAUTHOR  = "(C) EGEE";

bool  g_use_python        = false;
bool  g_use_smarter_retry = false;
}

std::string g_catalog_endpoint;

/*
 * print_usage_and_die
 *
 * Print Program Usage and ther exit
 */
void print_usage_and_die(){
    cout << endl;
    cout << "<" << PROGNAME << "> Version " << PROGVERSION << " by " << PROGAUTHOR;
    cout << endl;
    cout << "usage: " << PROGNAME << "[-l <glite_location_user>] [-s] [-P [-x]] [-e endpoint]" << endl;
    cout << " Where:" << endl;
    cout << " -l : specify the glite location. Configuration files should be located under the etc folder on that path" << endl;
    cout << " -s : Enable the Compiler Outputter" << endl;
    cout << " -c : configuration" << endl;
    cout << " -P : use Python strategies" << endl;
    cout << " -x : use Python smarter retry" << endl;
    cout << " -e : the catalog endpoints" << endl;
    cout << " Where:" << endl;
    cout << " Optional parameters:" << endl;
    cout << "  -h : print this screen" << endl;
    exit(-1);
}

/*
 * suite
 *
 * Instantiate Test Suite
 */
CppUnit::Test * suite() {
    CppUnit::TestFactoryRegistry& registry = CppUnit::TestFactoryRegistry::getRegistry();
#if 0
    CppUnit::TestFactoryRegistry& channelCacheTestCase = CppUnit::TestFactoryRegistry::getRegistry("ChannelCacheTestCase");
    CppUnit::TestFactoryRegistry& allocateTestCase     = CppUnit::TestFactoryRegistry::getRegistry("AllocateTestCase");
    CppUnit::TestFactoryRegistry& finalizeTestCase     = CppUnit::TestFactoryRegistry::getRegistry("FinalizeTestCase");
    CppUnit::TestFactoryRegistry& checkReadyTestCase   = CppUnit::TestFactoryRegistry::getRegistry("CheckReadinessTestCase");

    CppUnit::TestFactoryRegistry& basicRetryTestCase   = CppUnit::TestFactoryRegistry::getRegistry("BasicRetryTestCase");
    CppUnit::TestFactoryRegistry& pythonRetryTestCase  = CppUnit::TestFactoryRegistry::getRegistry("PythonBasicRetryTestCase");
    CppUnit::TestFactoryRegistry& activeTxCacheTestCase= CppUnit::TestFactoryRegistry::getRegistry("ActiveTxCacheTestCase");
    CppUnit::TestFactoryRegistry& fetchTestCase        = CppUnit::TestFactoryRegistry::getRegistry("FetchTestCase");
    CppUnit::TestFactoryRegistry& checkStateTestCase   = CppUnit::TestFactoryRegistry::getRegistry("CheckStateTestCase");
    CppUnit::TestFactoryRegistry& cancelActiveTestCase = CppUnit::TestFactoryRegistry::getRegistry("CancelActiveTestCase");
    CppUnit::TestFactoryRegistry& smarterRetryTestCase = CppUnit::TestFactoryRegistry::getRegistry("SmarterRetryTestCase");

    CppUnit::TestFactoryRegistry& cancelPendingTestCase= CppUnit::TestFactoryRegistry::getRegistry("CancelPendingTestCase");
#endif
    CppUnit::TestFactoryRegistry& fetchTestCase        = CppUnit::TestFactoryRegistry::getRegistry("FetchTestCase");

#if 0
    registry.registerFactory(&channelCacheTestCase);
    registry.registerFactory(&allocateTestCase);
    if(true == g_use_python){
        if(true == g_use_smarter_retry){
            registry.registerFactory(&smarterRetryTestCase);
        } else {
            registry.registerFactory(&pythonRetryTestCase);
        }
    } else {
        registry.registerFactory(&basicRetryTestCase);
    }
    registry.registerFactory(&activeTxCacheTestCase);
    registry.registerFactory(&fetchTestCase);
    registry.registerFactory(&checkStateTestCase);
    registry.registerFactory(&finalizeTestCase);
    registry.registerFactory(&checkReadyTestCase);
    registry.registerFactory(&cancelActiveTestCase);
    registry.registerFactory(&cancelPendingTestCase);
#endif
    registry.registerFactory(&fetchTestCase);

    return registry.makeTest();
}


/*
 * main
 *
 * Transfer FTS Main function
 */
int main(int argc, char* argv[]) {

    // Get Options
    const char * location = 0;
    const char * config   = CONFIG_NAME;
    bool selfTest = false;
    int flag;
    while ((flag = getopt (argc, argv, "c:l:e:Pxsh")) != -1) {
        switch (flag) {
            case 'c':
                config = optarg;
            break;
            case 'l':
                location = optarg;
            break;
            case 'P':
                g_use_python = true;
            break;
            case 'x':
                g_use_smarter_retry = true;
            break;
            case 'e':
                g_catalog_endpoint = optarg;
            break;
            case 's':
                selfTest = true;
            break;
            case 'h':
            default:
                print_usage_and_die();
            break;
        } // End switch
    } // End while

    if(0 != location){
        // Set "GLite Location User"
        setenv("GLITE_LOCATION",location,1);
    }

    // Initialize the Application
    int initres = glite_config_initialize(config);
    if(initres < 0){
        cerr << "Initialization failed " << endl;
        return -1;
    }

    CppUnit::TextUi::TestRunner runner;
    runner.addTest(suite());   // Add the suite to the test runner

    if(true == selfTest){
        // Change the default outputter to a compiler error format outputter
        // The test runner owns the new outputter.
        runner.setOutputter(
           CppUnit::CompilerOutputter::defaultOutputter(&runner.result(),cerr));
    }

    // Run the test
    bool wasSucessful = runner.run();

    DaoHelper::dispose();

    // Finalize the Application
    glite_config_finalize();

    return wasSucessful ? 0 : 1;
}
