/**
 *  Copyright (c) Members of the EGEE Collaboration. 2004.
 *  See http://www.eu-egee.org/partners/ for details on the copyright
 *  holders.
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 *  GLite Data Transfer Agents: Test case for separation of SRM interaction from gridftp transfer
 *
 *  Authors: Paolo Tedesco <paolo.tedesco@cern.ch>
 *  Version info: $Id: SrmGridFtpSplitTestCase.cpp,v 1.1 2008/04/23 15:10:14 ptedesco Exp $
 *  Release: $Name:  $
 *
 */

#include "Fetch.h"
#include "CheckState.h"
#include "ChannelFlags.h"
#include "actions/channel/ActionsConfig.h"
#include "glite/data/transfer/agent/action/ActionFactory.h"
#include "glite/data/transfer/agent/model/File.h"
#include "glite/data/transfer/agent/model/Job.h"
#include "glite/data/transfer/agent/model/Transfer.h"
#include "glite/data/transfer/agent/model/Channel.h"
#include "glite/data/transfer/agent/dao/channel/FileDAO.h"
#include "glite/data/transfer/agent/dao/channel/JobDAO.h"
#include "glite/data/transfer/agent/dao/channel/TransferDAO.h"
#include "glite/data/transfer/agent/dao/channel/DAOFactory.h"
#include "glite/data/transfer/agent/ts/TransferService.h"
#include "glite/data/transfer/agent/ts/TransferServiceFactory.h"

#include "DaoHelper.h"
#include "dao_test_utils.h"
 
#include <cppunit/TestCase.h>
#include <cppunit/extensions/HelperMacros.h>
 
#include <boost/scoped_ptr.hpp>
#include <boost/shared_ptr.hpp>

using namespace glite::data::transfer::agent;
using namespace glite::data::transfer::agent::action;
using namespace glite::data::transfer::agent::action::channel;
using namespace glite::data::transfer::agent::model;

using boost::scoped_ptr;
using boost::shared_ptr;
 
/**
 * CPPUnit TestCase
 */
class SrmGridFtpSplitTestCase : public CppUnit::TestCase{
public:
    
    /**
     * Test case setup
     */
    void setUp();
    
private:
    
    // Declare CPPUnit Methods
    // -------------------------------------------------------------------------
    CPPUNIT_TEST_SUITE( SrmGridFtpSplitTestCase );
        CPPUNIT_TEST( testSplit_01 );
        CPPUNIT_TEST( testSplit_02 );
    CPPUNIT_TEST_SUITE_END();
    
    /**
     * Test split
     */
    void testSplit_01();
    void testSplit_02();

    // Fetch action
    scoped_ptr<Fetch> m_fetch;

    // CheckState action
    scoped_ptr<CheckState> m_checkState;
        
};

CPPUNIT_TEST_SUITE_NAMED_REGISTRATION( SrmGridFtpSplitTestCase, "SrmGridFtpSplitTestCase" );

namespace{
    
    
    // Constants
    //--------------------------------------------------------------------------
    
    // maximum files on the channel (i.e. max num of transfers in the transfer
    // phase)
    const unsigned int CHANNEL1_FILES = 10;
    // maximum files for the transfer service (i.e. max num of transfers in any
    // state)
    const unsigned int TX_SERVICE_MAX_FILES = 15;
    // expected number of files fetched for job1 after 1st fetch execution
    const unsigned int FETCHED_JOB1 = 9;
    // expected number of files fetched for job2 after 1st fetch execution
    const unsigned int FETCHED_JOB2 = 6;
    
} // namespace

#define CHECK_VALUE(val,exp)\
    if (val != exp){\
        std::stringstream ss;\
        ss << "Wrong value for " << #val << ": got " << val << " expected " << exp;\
        CPPUNIT_FAIL(ss.str().c_str());\
    }
    
/**
 * Test case setup
 */
void SrmGridFtpSplitTestCase::setUp(){
    
    // Reset cache
    ActionsConfig::instance().activeTxCache().reset();
    
    // Create action instances
    m_fetch.reset(dynamic_cast<Fetch*>(
        ActionFactory::instance().create("glite:Fetch")));
    CPPUNIT_ASSERT_MESSAGE("Null Fetch action", m_fetch.get() != 0);

    m_checkState.reset(dynamic_cast<CheckState*>(
        ActionFactory::instance().create("glite:CheckState")));
    CPPUNIT_ASSERT_MESSAGE("Null CheckState action", m_checkState.get() != 0);
        
}

/**
 * testSplit_01
 * Tests: start with a channel with no current transfers, channel limit is 10.
 * Run the Fetch action; it asks the transfer service how many new transfers 
 * should be started, answer should be 20.
 * Check that 20 transfers have been started, 12 for dteam (job1) and 8 for ops
 * (job2).
 */
void SrmGridFtpSplitTestCase::testSplit_01(){

    // Constants
    //--------------------------------------------------------------------------        
    
    // Channel
    const char * const CHANNEL_NAME = "CHANNEL_01";
    
    // Jobs
    const char * const JOB1_ID = "00000001-0000-0000-0000-000000000001";
    const char * const JOB2_ID = "00000001-0000-0000-0000-000000000002";
    
    // Files
    const char * const FILE101_ID = "00100101";
    const char * const FILE102_ID = "00100102";
    const char * const FILE103_ID = "00100103";
    const char * const FILE104_ID = "00100104";
    const char * const FILE105_ID = "00100105";
    const char * const FILE106_ID = "00100106";
    const char * const FILE107_ID = "00100107";
    const char * const FILE108_ID = "00100108";
    const char * const FILE109_ID = "00100109";
    const char * const FILE110_ID = "00100110";
    const char * const FILE111_ID = "00100111";
    const char * const FILE112_ID = "00100112";
    const char * const FILE113_ID = "00100113";
    const char * const FILE114_ID = "00100114";
    const char * const FILE115_ID = "00100115";

    const char * const FILE201_ID = "00100201";
    const char * const FILE202_ID = "00100202";
    const char * const FILE203_ID = "00100203";
    const char * const FILE204_ID = "00100204";
    const char * const FILE205_ID = "00100205";
    const char * const FILE206_ID = "00100206";
    const char * const FILE207_ID = "00100207";
    const char * const FILE208_ID = "00100208";
    const char * const FILE209_ID = "00100209";
    const char * const FILE210_ID = "00100210";
    
    // Set actions context
    //--------------------------------------------------------------------------
    m_fetch->context(CHANNEL_NAME,DaoHelper::ctx());
    m_checkState->context(CHANNEL_NAME,DaoHelper::ctx());
        
    // DAO objects
    //--------------------------------------------------------------------------
    // FileDAO
    scoped_ptr<dao::channel::FileDAO> fileDAO(
        dao::channel::DAOFactory::instance().createFileDAO(CHANNEL_NAME, DaoHelper::ctx()));
    CPPUNIT_ASSERT_MESSAGE("Null FileDAO", fileDAO.get() != 0);
    
    // JobDAO
    scoped_ptr<dao::channel::JobDAO> jobDAO(
        dao::channel::DAOFactory::instance().createJobDAO(CHANNEL_NAME, DaoHelper::ctx()));
    CPPUNIT_ASSERT_MESSAGE("Null JobDAO", jobDAO.get() != 0);
    
    // TransferDAO
    scoped_ptr<dao::channel::TransferDAO> transferDAO(
        dao::channel::DAOFactory::instance().createTransferDAO(CHANNEL_NAME, DaoHelper::ctx()));
    CPPUNIT_ASSERT_MESSAGE("Null TransferDAO", transferDAO.get() != 0);
    
    // execute the Fetch action
    //--------------------------------------------------------------------------
    m_fetch->execute(CHANNEL_VO_SHARE_TYPE_NORMALIZED_ON_ACTIVE);
    
    // read all the files and check that the expected number has been fetched
    //--------------------------------------------------------------------------
    check_file(fileDAO,FILE101_ID,"FILE101",File::S_ACTIVE);
    check_file(fileDAO,FILE102_ID,"FILE102",File::S_ACTIVE);
    check_file(fileDAO,FILE103_ID,"FILE103",File::S_ACTIVE);
    check_file(fileDAO,FILE104_ID,"FILE104",File::S_ACTIVE);
    check_file(fileDAO,FILE105_ID,"FILE105",File::S_ACTIVE);
    check_file(fileDAO,FILE106_ID,"FILE106",File::S_ACTIVE);
    check_file(fileDAO,FILE107_ID,"FILE107",File::S_ACTIVE);
    check_file(fileDAO,FILE108_ID,"FILE108",File::S_ACTIVE);
    check_file(fileDAO,FILE109_ID,"FILE109",File::S_ACTIVE);
    check_file(fileDAO,FILE110_ID,"FILE110",File::S_ACTIVE);
    check_file(fileDAO,FILE111_ID,"FILE111",File::S_ACTIVE);
    check_file(fileDAO,FILE112_ID,"FILE112",File::S_ACTIVE);
    check_file(fileDAO,FILE113_ID,"FILE113",File::S_READY);
    check_file(fileDAO,FILE114_ID,"FILE114",File::S_READY);
    check_file(fileDAO,FILE115_ID,"FILE115",File::S_READY);

    check_file(fileDAO,FILE201_ID,"FILE201",File::S_ACTIVE);
    check_file(fileDAO,FILE202_ID,"FILE202",File::S_ACTIVE);
    check_file(fileDAO,FILE203_ID,"FILE203",File::S_ACTIVE);
    check_file(fileDAO,FILE204_ID,"FILE204",File::S_ACTIVE);
    check_file(fileDAO,FILE205_ID,"FILE205",File::S_ACTIVE);
    check_file(fileDAO,FILE206_ID,"FILE206",File::S_ACTIVE);
    check_file(fileDAO,FILE207_ID,"FILE207",File::S_ACTIVE);
    check_file(fileDAO,FILE208_ID,"FILE208",File::S_ACTIVE);
    check_file(fileDAO,FILE209_ID,"FILE209",File::S_READY);
    check_file(fileDAO,FILE210_ID,"FILE210",File::S_READY);
    
    // Get Active Transfers cache
    ActiveTxCache& tx_cache = ActionsConfig::instance().activeTxCache();
    CHECK_VALUE(tx_cache.countRequests(),20);

#define COUNT_TRANSFERS(a) {\
    unsigned int val = tx_cache.countTransfersByState(Transfer::a);\
    printf("Transfers in " #a " state : %ud\n",val);\
}
        
    // check that there are no transfers in the active transfer state
    unsigned int received_after_fetch = tx_cache.countTransfersByState(Transfer::S_RECEIVED);
    CHECK_VALUE (received_after_fetch,20);
    
    // execute the CheckState action
    //--------------------------------------------------------------------------
    m_checkState->execute();
    unsigned int ready_after_1st_checkstate = tx_cache.countTransfersByState(Transfer::S_READY);
    CHECK_VALUE (ready_after_1st_checkstate,20);
    
    m_checkState->execute();
    unsigned int ready_after_2nd_checkstate = tx_cache.countTransfersByState(Transfer::S_READY);
    CHECK_VALUE (ready_after_2nd_checkstate,10);
    unsigned int transferred_after_2nd_checkstate = tx_cache.countTransfersByState(Transfer::S_TRANSFERRED);
    CHECK_VALUE (transferred_after_2nd_checkstate,10);
    
    m_checkState->execute();
    unsigned int transferred_after_3rd_checkstate = tx_cache.countTransfersByState(Transfer::S_TRANSFERRED);
    CHECK_VALUE (transferred_after_3rd_checkstate,10);
    
}

/**
 * testSplit_02
 * Tests: start with a channel like this:
 *   channel limit = 10
 *   transfers in preparation phase = 3 (2 dteam, 1 ops)
 *   transfers in transfer phase = 10 (6 dteam, 4 ops)
 *   transfers in finalization phase = 2 (1 dteam, 1 ops)
 * Run the fetch action; expect that 5 transfers are fetched: 3 for dteam and 2 for ops.
 */
void SrmGridFtpSplitTestCase::testSplit_02(){
    
    // Constants
    //--------------------------------------------------------------------------        
    
    // Channel
    const char * const CHANNEL_NAME = "CHANNEL_02";
    
    // Jobs
    const char * const JOB1_ID = "00000002-0000-0000-0000-000000000001";
    const char * const JOB2_ID = "00000002-0000-0000-0000-000000000002";
    
    // Files
    const char * const FILE101_ID = "00200101";
    const char * const FILE102_ID = "00200102";
    const char * const FILE103_ID = "00200103";
    const char * const FILE104_ID = "00200104";
    const char * const FILE105_ID = "00200105";
    const char * const FILE106_ID = "00200106";
    const char * const FILE107_ID = "00200107";
    const char * const FILE108_ID = "00200108";
    const char * const FILE109_ID = "00200109";
    const char * const FILE110_ID = "00200110";
    const char * const FILE111_ID = "00200111";
    const char * const FILE112_ID = "00200112";
    const char * const FILE113_ID = "00200113";
    const char * const FILE114_ID = "00200114";
    const char * const FILE115_ID = "00200115";

    const char * const FILE201_ID = "00200201";
    const char * const FILE202_ID = "00200202";
    const char * const FILE203_ID = "00200203";
    const char * const FILE204_ID = "00200204";
    const char * const FILE205_ID = "00200205";
    const char * const FILE206_ID = "00200206";
    const char * const FILE207_ID = "00200207";
    const char * const FILE208_ID = "00200208";
    const char * const FILE209_ID = "00200209";
    const char * const FILE210_ID = "00200210";
    
    // Set actions context
    //--------------------------------------------------------------------------
    m_fetch->context(CHANNEL_NAME,DaoHelper::ctx());
    m_checkState->context(CHANNEL_NAME,DaoHelper::ctx());
        
    // DAO objects
    //--------------------------------------------------------------------------
    // FileDAO
    scoped_ptr<dao::channel::FileDAO> fileDAO(
            dao::channel::DAOFactory::instance().createFileDAO(CHANNEL_NAME, DaoHelper::ctx()));
    CPPUNIT_ASSERT_MESSAGE("Null FileDAO", fileDAO.get() != 0);
    
    // JobDAO
    scoped_ptr<dao::channel::JobDAO> jobDAO(
            dao::channel::DAOFactory::instance().createJobDAO(CHANNEL_NAME, DaoHelper::ctx()));
    CPPUNIT_ASSERT_MESSAGE("Null JobDAO", jobDAO.get() != 0);
    
    // TransferDAO
    scoped_ptr<dao::channel::TransferDAO> transferDAO(
            dao::channel::DAOFactory::instance().createTransferDAO(CHANNEL_NAME, DaoHelper::ctx()));
    CPPUNIT_ASSERT_MESSAGE("Null TransferDAO", transferDAO.get() != 0);
    
    // execute the action
    //--------------------------------------------------------------------------
    m_fetch->execute(CHANNEL_VO_SHARE_TYPE_NORMALIZED_ON_ACTIVE);
    
    // read all the files and check that the expected number has been fetched
    //--------------------------------------------------------------------------
    check_file(fileDAO,FILE101_ID,"FILE101",File::S_ACTIVE);
    check_file(fileDAO,FILE102_ID,"FILE102",File::S_ACTIVE);
    check_file(fileDAO,FILE103_ID,"FILE103",File::S_ACTIVE);
    check_file(fileDAO,FILE104_ID,"FILE104",File::S_ACTIVE);
    check_file(fileDAO,FILE105_ID,"FILE105",File::S_ACTIVE);
    check_file(fileDAO,FILE106_ID,"FILE106",File::S_ACTIVE);
    check_file(fileDAO,FILE107_ID,"FILE107",File::S_ACTIVE);
    check_file(fileDAO,FILE108_ID,"FILE108",File::S_ACTIVE);
    check_file(fileDAO,FILE109_ID,"FILE109",File::S_ACTIVE);
    check_file(fileDAO,FILE110_ID,"FILE110",File::S_ACTIVE);
    check_file(fileDAO,FILE111_ID,"FILE111",File::S_ACTIVE);
    check_file(fileDAO,FILE112_ID,"FILE112",File::S_ACTIVE);
    check_file(fileDAO,FILE113_ID,"FILE113",File::S_READY);
    check_file(fileDAO,FILE114_ID,"FILE114",File::S_READY);
    check_file(fileDAO,FILE115_ID,"FILE115",File::S_READY);

    check_file(fileDAO,FILE201_ID,"FILE201",File::S_ACTIVE);
    check_file(fileDAO,FILE202_ID,"FILE202",File::S_ACTIVE);
    check_file(fileDAO,FILE203_ID,"FILE203",File::S_ACTIVE);
    check_file(fileDAO,FILE204_ID,"FILE204",File::S_ACTIVE);
    check_file(fileDAO,FILE205_ID,"FILE205",File::S_ACTIVE);
    check_file(fileDAO,FILE206_ID,"FILE206",File::S_ACTIVE);
    check_file(fileDAO,FILE207_ID,"FILE207",File::S_ACTIVE);
    check_file(fileDAO,FILE208_ID,"FILE208",File::S_ACTIVE);
    check_file(fileDAO,FILE209_ID,"FILE209",File::S_READY);
    check_file(fileDAO,FILE210_ID,"FILE210",File::S_READY);
    
}


