/*
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *  GLite Data Transfer Agents: CatalogService Test Case
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *  Version info: $Id: CatalogServiceTestCase.cpp,v 1.10 2006/08/23 14:31:48 badino Exp $
 *  Release: $Name:  $
 *
 */
#include "glite/data/transfer/agent/catalog/CatalogServiceFactory.h"
#include "glite/data/transfer/agent/catalog/CatalogService.h"
#include "glite/data/transfer/agent/catalog/CatalogServiceException.h"

#include <boost/scoped_ptr.hpp>
#include <cppunit/TestCase.h>
#include <cppunit/extensions/HelperMacros.h>
#include <unistd.h>
#include <cstring>
 
using namespace glite::data::transfer::agent;
using namespace glite::data::transfer::agent::catalog;
using boost::scoped_ptr;

extern const char * g_endpoint;

/**
 * CPPUnit TestCase
 */
class CatalogServiceTestCase : public CppUnit::TestCase{
public:
    /**
     * set-up test case
     */
    void setUp ();
private:
    // Declare CPPUnit Methods
    // -------------------------------------------------------------------------
    CPPUNIT_TEST_SUITE( CatalogServiceTestCase);
    CPPUNIT_TEST( testGetCatalogEndpoint );
    CPPUNIT_TEST( testRegisterSurlWithLFN );
    CPPUNIT_TEST( testRegisterSurlWithGUID );
    CPPUNIT_TEST( testRegisterSurlWithLFN_Error );
    CPPUNIT_TEST( testRegisterSurlWithGUID_Error );
    CPPUNIT_TEST( testRegisterSurlWithLFN_Error_2 );
    CPPUNIT_TEST( testRegisterSurlWithGUID_Error_2 );
    CPPUNIT_TEST( testCheckPermissionsWithLFN );
    CPPUNIT_TEST( testCheckPermissionsWithGUID );
    CPPUNIT_TEST( testCheckPermissions_Error );
    CPPUNIT_TEST( testCheckPermissions_RemoteError );
    CPPUNIT_TEST( testCheckSurlsWithLFN );
    CPPUNIT_TEST( testCheckSurlsWithGUID );
    CPPUNIT_TEST( testCheckSurls_Error );
    CPPUNIT_TEST( testListSurlsWithLFN );
    CPPUNIT_TEST( testListSurlsWithGUID );
    CPPUNIT_TEST( testListSurls_Error );
    CPPUNIT_TEST( testGetStatsWithLFN );
    CPPUNIT_TEST( testGetStatsWithGUID );
    CPPUNIT_TEST( testGetStats_Error );
    CPPUNIT_TEST( testRegisterWithStats );
    CPPUNIT_TEST_SUITE_END();

    // Test Methods
    // -------------------------------------------------------------------------
    /**
     * Test the Get Endpoint Method
     */
    void testGetCatalogEndpoint();
    /**
     * Test the register SURL passing an LFN
     */
    void testRegisterSurlWithLFN();
    /**
     * Test the register SURL passing a GUID
     */
    void testRegisterSurlWithGUID();
    /**
     * Test the register SURL that should fail
     */
    void testRegisterSurlWithLFN_Error();
    /**
     * Test the register SURL that should fail
     */
    void testRegisterSurlWithGUID_Error();
    /**
     * Test the register SURL that should fail
     */
    void testRegisterSurlWithLFN_Error_2();
    /**
     * Test the register SURL that should fail
     */
    void testRegisterSurlWithGUID_Error_2();
    /**
     * Test checkPermissions passing an LFN
     */
    void testCheckPermissionsWithLFN();
    /**
     * Test checkPermissions passing a GUID
     */
    void testCheckPermissionsWithGUID();
    /**
     * Test checkPermissions that should fail
     */
    void testCheckPermissions_Error();
    /**
     * Test checkPermissions that should fail
     */
    void testCheckPermissions_RemoteError();
    /**
     * Test checkSurls passing an LFN
     */
    void testCheckSurlsWithLFN();
    /**
     * Test checkSurls passing a GUID
     */
    void testCheckSurlsWithGUID();
    /**
     * Test checkSurls that should fail
     */
    void testCheckSurls_Error();
    /**
     * Test the listSurl passing an LFN
     */
    void testListSurlsWithLFN();
    /**
     * Test the listSurl passing a GUID
     */
    void testListSurlsWithGUID();
    /**
     * Test the listSurl that should fail
     */
    void testListSurls_Error();
    /**
     * Test the GetStats passing an LFN
     */
    void testGetStatsWithLFN();
    /**
     * Test the GetStats passing a GUID
     */
    void testGetStatsWithGUID();
    /**
     * Test the GetStats that should fail
     */
    void testGetStats_Error();
    /**
     * Test registration with Stats
     */
    void testRegisterWithStats();
private:
    /**
     * CatalogService Instance
     */
    scoped_ptr<CatalogService> m_service;
};

/*
 * Register Suit
 */
CPPUNIT_TEST_SUITE_NAMED_REGISTRATION( CatalogServiceTestCase, "CatalogServiceTestCase" );

namespace {
const char * const CS_TESTCASE_SE_NAME       = "castor.cern.ch";
const char * const CS_TESTCASE_LFN_1         = "/tmp/test/lfn-to-register-00.txt";    
const char * const CS_TESTCASE_SURL_1        = "srm://castor.cern.ch:8443/srm/managerv1?SFN=/tmp/test/lfn-to-register-00.txt";    
const char * const CS_TESTCASE_LFN_2         = "lfn:///tmp/test/lfn-to-register-01.txt";    
const char * const CS_TESTCASE_SURL_2        = "srm://castor.cern.ch:8443/srm/managerv1?SFN=/tmp/test/lfn-to-register-01.txt";    
const char * const CS_TESTCASE_LFN_3         = "lfn:/tmp/test/lfn-to-register-02.txt";    
const char * const CS_TESTCASE_SURL_3        = "srm://castor.cern.ch:8443/srm/managerv1?SFN=/tmp/test/lfn-to-register-02.txt";    
const char * const CS_TESTCASE_GUID_1        = "0000e6f0-4e81-1255-af7f-000000000000";
const char * const CS_TESTCASE_SURL_G1       = "srm://castor.cern.ch:8443/srm/managerv1?SFN=/tmp/test/lfn-to-register-with-guid-00.txt";    
const char * const CS_TESTCASE_GUID_2        = "guid:0000e6f0-4e81-1255-af7f-000000000001";
const char * const CS_TESTCASE_SURL_G2       = "srm://castor.cern.ch:8443/srm/managerv1?SFN=/tmp/test/lfn-to-register-with-guid-01.txt";    
const char * const CS_TESTCASE_GUID_3        = "guid://0000e6f0-4e81-1255-af7f-000000000002";
const char * const CS_TESTCASE_SURL_G3       = "srm://castor.cern.ch:8443/srm/managerv1?SFN=/tmp/test/lfn-to-register-with-guid-02.txt";    
const char * const CS_TESTCASE_FAIL_LFN_1    = "/fail/test/this-should-fail.txt";
const char * const CS_TESTCASE_FAILREMOTE_LFN_1 = "/failremote/test/this-should-fail-with-remote-error.txt";
const char * const CS_TESTCASE_FAIL_SURL_1   = "srm://castor.cern.ch:8443/srm/managerv1?SFN=/fail/test/this-should-fail.txt";
const char * const CS_TESTCASE_SURL_1_2      = "srm://castor.cern.ch:8443/srm/managerv1?SFN=/tmp/test/lfn-to-register-00.txt.replica";    
const char * const CS_TESTCASE_SURL_1_3      = "srm://se.nowhere.com:8443/srm/managerv1?SFN=/tmp/test/lfn-to-register-00.txt.replica";    
const char * const CS_TESTCASE_SURL_1_4      = "srm://se.here.com:8443/srm/managerv1?SFN=/tmp/test/lfn-to-register-with-stat-00.txt.replica";    
const char * const CS_TESTCASE_SURL_2_3      = "srm://se.nowhere.com:8443/srm/managerv1?SFN=/tmp/test/lfn-to-register-01.txt.replica";    
const char * const CS_TESTCASE_SURL_3_3      = "srm://se.nowhere.com:8443/srm/managerv1?SFN=/tmp/test/lfn-to-register-02.txt.replica";    
const char * const CS_TESTCASE_SURL_G1_2     = "srm://castor.cern.ch:8443/srm/managerv1?SFN=/tmp/test/lfn-to-register-with-guid-00.txt.replica"; 
const char * const CS_TESTCASE_SURL_G1_3     = "srm://se.nowhere.com:8443/srm/managerv1?SFN=/tmp/test/lfn-to-register-with-guid-00.txt.replica";
const char * const CS_TESTCASE_SURL_G2_3     = "srm://se.nowhere.com:8443/srm/managerv1?SFN=/tmp/test/lfn-to-register-with-guid-01.txt.replica";
const char * const CS_TESTCASE_SURL_G3_3     = "srm://se.nowhere.com:8443/srm/managerv1?SFN=/tmp/test/lfn-to-register-with-guid-02.txt.replica";
const char * const CS_TESTCASE_FAIL_G_1      = "fail0000-4e81-1255-af7f-000000000000";
const char * const CS_TESTCASE_FAIL_SURL_G1  = "srm://castor.cern.ch:8443/srm/managerv1?SFN=/fail/test/this-should-fail-guid.txt";
}

/*
 * setUp
 *
 * set-up test case
 */
void CatalogServiceTestCase::setUp(){

    try{
        CatalogServiceFactory& factory = CatalogServiceFactory::instance();
        // Get Catalog Type
        std::string type = factory.globalCatalogServiceType();
        if(type.empty()){
            type = factory.localCatalogServiceType();
            if(type.empty()){
                CPPUNIT_FAIL("No Type Specified");
            }
        }
        // Create CatalogService Instance
        m_service.reset(factory.create(g_endpoint,type));
    } catch(const std::exception& exc){
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        CPPUNIT_FAIL("Got Unknown Error");
    }
}

/*
 * Test RegisterSurlWithLFN
 */
void CatalogServiceTestCase::testGetCatalogEndpoint(){
    
    try{
        CatalogServiceFactory& factory = CatalogServiceFactory::instance();
        // Get Endpoint (Should return local catalog type)
        std::pair<std::string,std::string> catalog_data = factory.getCatalogEndpoint("CERN","MyVO");
        CPPUNIT_ASSERT_MESSAGE("Invalid endpoint (1)", catalog_data.first.empty() == false);
        CPPUNIT_ASSERT_MESSAGE("Empty type       (1)", catalog_data.second.empty() == false);
        if(false == factory.localCatalogServiceType().empty()){
            CPPUNIT_ASSERT_MESSAGE("Invalid type     (1)", catalog_data.second == factory.localCatalogServiceType());
        } else {
            CPPUNIT_ASSERT_MESSAGE("Invalid type     (1)", catalog_data.second == factory.globalCatalogServiceType());
        }
        // Get Endpoint (Should return global catalog type)
        catalog_data = factory.getCatalogEndpoint("","MyVO");
        CPPUNIT_ASSERT_MESSAGE("Invalid endpoint (2)", catalog_data.first.empty() == false);
        CPPUNIT_ASSERT_MESSAGE("Empty type       (2)", catalog_data.second.empty() == false);
        if(false == factory.globalCatalogServiceType().empty()){
            CPPUNIT_ASSERT_MESSAGE("Invalid type     (2)", catalog_data.second == factory.globalCatalogServiceType());
        } else {
            CPPUNIT_ASSERT_MESSAGE("Invalid type     (2)", catalog_data.second == factory.localCatalogServiceType());
        }
    } catch(const std::exception& exc){
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        CPPUNIT_FAIL("Got Unknown Error");
    }    
}

/*
 * Test RegisterSurlWithLFN
 */
void CatalogServiceTestCase::testRegisterSurlWithLFN(){
    
    try{
        // Create Request
        CatalogService::ReplicaStatArray replicas;
        replicas.push_back(CatalogService::ReplicaStat(CS_TESTCASE_LFN_1,CS_TESTCASE_SURL_1));
        replicas.push_back(CatalogService::ReplicaStat(CS_TESTCASE_LFN_2,CS_TESTCASE_SURL_2));
        replicas.push_back(CatalogService::ReplicaStat(CS_TESTCASE_LFN_3,CS_TESTCASE_SURL_3));
        
        // Register SURLs
        CatalogService::CatalogResult result;
        m_service->registerSurls(replicas,result);
        CPPUNIT_ASSERT_MESSAGE("Failed on LFNs", result.status == CatalogService::CatalogResult::R_SUCCESS);
    } catch(const std::exception& exc){
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        CPPUNIT_FAIL("Got Unknown Error");
    }    
}
/*
 * Test RegisterSurlWithGUID
 */
void CatalogServiceTestCase::testRegisterSurlWithGUID(){
    
    try{
        // Create Request
        CatalogService::ReplicaStatArray replicas;
        replicas.push_back(CatalogService::ReplicaStat(CS_TESTCASE_GUID_1,CS_TESTCASE_SURL_G1));
        replicas.push_back(CatalogService::ReplicaStat(CS_TESTCASE_GUID_2,CS_TESTCASE_SURL_G2));
        replicas.push_back(CatalogService::ReplicaStat(CS_TESTCASE_GUID_3,CS_TESTCASE_SURL_G3));

        // Register SURLs
        CatalogService::CatalogResult result;
        m_service->registerSurls(replicas,result);
        CPPUNIT_ASSERT_MESSAGE("Failed on GUIDs", result.status == CatalogService::CatalogResult::R_SUCCESS);
    } catch(const std::exception& exc){
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        CPPUNIT_FAIL("Got Unknown Error");
    }    
}

namespace {
/**
 * compare_guid Function Object
 */
class compare_first : public std::unary_function<const std::pair<std::string,std::string>&,bool>{
 public:
     /**
      * Constructor
      */
     compare_first(const std::string& s):m_str(s){}
     /**
      * Call Operator
      */
     bool operator()(const std::pair<std::string,std::string>& p){
         return (m_str == p.first)?true:false;
     }
private:
    compare_first();
    std::string m_str;
};
}

/*
 * Test RegisterSurl That should Fail
 */
void CatalogServiceTestCase::testRegisterSurlWithLFN_Error(){
    
    try{
        // Create Request
        CatalogService::ReplicaStatArray replicas;
        replicas.push_back(CatalogService::ReplicaStat(CS_TESTCASE_FAIL_LFN_1,CS_TESTCASE_FAIL_SURL_1));
        
        // Register SURL
        CatalogService::CatalogResult result;
        m_service->registerSurls(replicas,result);
        CPPUNIT_ASSERT_MESSAGE("This LFN should fail", result.status != CatalogService::CatalogResult::R_SUCCESS);
        if(CatalogService::CatalogResult::R_SOME_FAILURES == result.status){
            CPPUNIT_ASSERT_MESSAGE("That LFN should fail", 
                std::find_if(result.failures.begin(),result.failures.end(),
                    compare_first(CS_TESTCASE_FAIL_LFN_1)) != result.failures.end());
        }    
    } catch(const std::exception& exc){
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        CPPUNIT_FAIL("Got Unknown Error");
    }    
}
/*
 * Test RegisterSurl That should Fail
 */
void CatalogServiceTestCase::testRegisterSurlWithGUID_Error(){
    
    try{
        // Create Request
        CatalogService::ReplicaStatArray replicas;
        replicas.push_back(CatalogService::ReplicaStat(CS_TESTCASE_FAIL_G_1,CS_TESTCASE_FAIL_SURL_G1));
        
        // Register SURL
        CatalogService::CatalogResult result;
        m_service->registerSurls(replicas,result);
        CPPUNIT_ASSERT_MESSAGE("This GUID should fail", result.status != CatalogService::CatalogResult::R_SUCCESS);
        if(CatalogService::CatalogResult::R_SOME_FAILURES == result.status){
            CPPUNIT_ASSERT_MESSAGE("That GUID should fail", 
                std::find_if(result.failures.begin(),result.failures.end(),
                    compare_first(CS_TESTCASE_FAIL_G_1)) != result.failures.end());
        }
    } catch(const std::exception& exc){
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        CPPUNIT_FAIL("Got Unknown Error");
    }    
}

/*
 * Test RegisterSurl That should Fail
 */
void CatalogServiceTestCase::testRegisterSurlWithLFN_Error_2(){
    
    try{
        // Create Request
        CatalogService::ReplicaStatArray replicas;
        replicas.push_back(CatalogService::ReplicaStat(CS_TESTCASE_LFN_1,CS_TESTCASE_SURL_1_2));
        replicas.push_back(CatalogService::ReplicaStat(CS_TESTCASE_FAIL_LFN_1,CS_TESTCASE_FAIL_SURL_1));
        
        // Register SURL
        CatalogService::CatalogResult result;
        m_service->registerSurls(replicas,result);
        CPPUNIT_ASSERT_MESSAGE("This LFN should fail", result.status != CatalogService::CatalogResult::R_SUCCESS);
        if(CatalogService::CatalogResult::R_SOME_FAILURES == result.status){
            CPPUNIT_ASSERT_MESSAGE("That LFN should fail", 
                std::find_if(result.failures.begin(),result.failures.end(),
                    compare_first(CS_TESTCASE_FAIL_LFN_1)) != result.failures.end());
            CPPUNIT_ASSERT_MESSAGE("That LFN shouldn't fail", 
                std::find_if(result.failures.begin(),result.failures.end(),
                    compare_first(CS_TESTCASE_LFN_1)) == result.failures.end());
        } else {
            try{
                // Check that the CS_TESTCASE_SURL_1 is deregistered
                CatalogService::StringArray names;
                std::vector<CatalogService::StringArray> surls;
                names.push_back(CS_TESTCASE_LFN_1);
                // List Surls
                CatalogService::CatalogResult listResult;
                m_service->listSurls(names,"",surls,listResult);
                CPPUNIT_ASSERT_MESSAGE("listSurls failed", result.status == CatalogService::CatalogResult::R_SUCCESS);
                if(surls.size() > 0){ 
                    CatalogService::StringArray& array = surls.at(0);
                    CatalogService::StringArray::iterator it =
                        std::find(array.begin(),array.end(),CS_TESTCASE_SURL_1_2);
                    CPPUNIT_ASSERT_MESSAGE("Replica Not Deleted", it == array.end());
                }
            } catch(const std::exception& exc){
                CPPUNIT_FAIL("An exception on list Replica");
            }
        }
    } catch(const std::exception& exc){
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        CPPUNIT_FAIL("Got Unknown Error");
    }    
}

/*
 * Test RegisterSurl That should Fail
 */
void CatalogServiceTestCase::testRegisterSurlWithGUID_Error_2(){
    
    try{
        // Create Request
        CatalogService::ReplicaStatArray replicas;
        replicas.push_back(CatalogService::ReplicaStat(CS_TESTCASE_GUID_1,CS_TESTCASE_SURL_G1_2));
        replicas.push_back(CatalogService::ReplicaStat(CS_TESTCASE_FAIL_G_1,CS_TESTCASE_FAIL_SURL_G1));
        
        // Register SURL
        CatalogService::CatalogResult result;
        m_service->registerSurls(replicas,result);
        CPPUNIT_ASSERT_MESSAGE("This GUID should fail", result.status != CatalogService::CatalogResult::R_SUCCESS);
        if(CatalogService::CatalogResult::R_SOME_FAILURES == result.status){
            CPPUNIT_ASSERT_MESSAGE("That GUID should fail", 
                std::find_if(result.failures.begin(),result.failures.end(),
                    compare_first(CS_TESTCASE_FAIL_G_1)) != result.failures.end());
            CPPUNIT_ASSERT_MESSAGE("That GUID shouldn't fail", 
                std::find_if(result.failures.begin(),result.failures.end(),
                    compare_first(CS_TESTCASE_GUID_1)) == result.failures.end());
        } else {
            try{
                // Check that the CS_TESTCASE_SURL_G1 is deregistered
                CatalogService::StringArray names;
                std::vector<CatalogService::StringArray> surls;
                names.push_back(CS_TESTCASE_GUID_1);
                // List Surls
                CatalogService::CatalogResult listResult;
                m_service->listSurls(names,"",surls,listResult);
                CPPUNIT_ASSERT_MESSAGE("listSurls failed", result.status == CatalogService::CatalogResult::R_SUCCESS);
                if(surls.size() > 0){ 
                    CatalogService::StringArray& array = surls.at(0);
                    CatalogService::StringArray::iterator it =
                        std::find(array.begin(),array.end(),CS_TESTCASE_SURL_G1_2);
                    CPPUNIT_ASSERT_MESSAGE("Replica Not Deleted", it == array.end());
                }
            } catch(const std::exception& exc){
                CPPUNIT_FAIL("An exception on list Replica");
            }
        }
    } catch(const std::exception& exc){
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        CPPUNIT_FAIL("Got Unknown Error");
    }    
}

/*
 * Test CheckPermissionsWithLFN
 */
void CatalogServiceTestCase::testCheckPermissionsWithLFN(){
    
    try{
        // Create Request
        CatalogService::StringArray names;
        names.push_back(CS_TESTCASE_LFN_1);
        names.push_back(CS_TESTCASE_LFN_2);
        names.push_back(CS_TESTCASE_LFN_3);
        
        // Check Permission
        CatalogService::CatalogResult result;
        m_service->checkPermissions(names,result);
        CPPUNIT_ASSERT_MESSAGE("Permission not granted on LFNs", result.status == CatalogService::CatalogResult::R_SUCCESS);
    } catch(const std::exception& exc){
        std::string reason = (std::string)"Failed on LFNs :" + exc.what();
        CPPUNIT_FAIL(reason.c_str());
    } catch(...){
        CPPUNIT_FAIL("Got Unknown Error");
    } 
}
/*
 * Test CheckPermissionsWithGUID
 */
void CatalogServiceTestCase::testCheckPermissionsWithGUID(){
    
    try{
        // Create Request
        CatalogService::StringArray names;
        names.push_back(CS_TESTCASE_GUID_1);
        names.push_back(CS_TESTCASE_GUID_2);
        names.push_back(CS_TESTCASE_GUID_3);
        
        // Check Permission
        CatalogService::CatalogResult result;
        m_service->checkPermissions(names,result);
        CPPUNIT_ASSERT_MESSAGE("Permission not granted on GUIDs", result.status == CatalogService::CatalogResult::R_SUCCESS);
    } catch(const std::exception& exc){
        std::string reason = (std::string)"Failed on GUIDs :" + exc.what();
        CPPUNIT_FAIL(reason.c_str());
    } catch(...){
        CPPUNIT_FAIL("Got Unknown Error");
    } 
}
/*
 * Test CheckPermissions that should fail
 */
void CatalogServiceTestCase::testCheckPermissions_Error(){
    
    std::string surl;
    try{
        // Create Request
        CatalogService::StringArray names;
        names.push_back(CS_TESTCASE_FAIL_LFN_1);

        // Check Permission
        CatalogService::CatalogResult result;
        m_service->checkPermissions(names,result);
        CPPUNIT_ASSERT_MESSAGE("Permission granted on LFN", result.status != CatalogService::CatalogResult::R_SUCCESS);
        if(result.status == CatalogService::CatalogResult::R_SOME_FAILURES){
            CPPUNIT_ASSERT_MESSAGE("That LFN should fail", std::find_if(result.failures.begin(),result.failures.end(),compare_first(CS_TESTCASE_FAIL_LFN_1)) != result.failures.end());
        }
    } catch(const std::exception& exc){
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        CPPUNIT_FAIL("Got Unknown Error");
    } 
}


/*
 * Test CheckPermissions that should fail
 */
void CatalogServiceTestCase::testCheckPermissions_RemoteError(){
    
    // Test it only for PythonCatalog
    if(0 == strcmp(g_endpoint,"fake-endpoint")){
        try{
            // Create Request
            CatalogService::StringArray names;
            names.push_back(CS_TESTCASE_FAILREMOTE_LFN_1);
    
            // Check Permission
            CatalogService::CatalogResult result;
            try{
                m_service->checkPermissions(names,result);
                CPPUNIT_FAIL("This method should fail");
            } catch(const CatalogRemoteError& exc){
                // OK
            }
        } catch(const std::exception& exc){
            CPPUNIT_FAIL(exc.what());
        } catch(...){
            CPPUNIT_FAIL("Got Unknown Error");
        } 
    }
}

/*
 * Test ListSurlsWithLFN
 */
void CatalogServiceTestCase::testListSurlsWithLFN(){
    
    try{
        
        CatalogService::StringArray names;
        std::vector<CatalogService::StringArray> surls;
    
        names.push_back(CS_TESTCASE_LFN_1);
        names.push_back(CS_TESTCASE_LFN_2);
        names.push_back(CS_TESTCASE_LFN_3);

        // List Surls
        CatalogService::CatalogResult result;
        m_service->listSurls(names,"",surls,result);
        CPPUNIT_ASSERT_MESSAGE("ListSurls failed on LFN", result.status == CatalogService::CatalogResult::R_SUCCESS);

        CPPUNIT_ASSERT_MESSAGE("No surls returned (empty array)", false == surls.empty());
        CPPUNIT_ASSERT_MESSAGE("Invalid surls size", names.size() == surls.size());
        CPPUNIT_ASSERT_MESSAGE("No surls returned (LFN1)", false == surls.at(0).empty());
        CPPUNIT_ASSERT_MESSAGE("No surls returned (LFN2)", false == surls.at(1).empty());
        CPPUNIT_ASSERT_MESSAGE("No surls returned (LFN3)", false == surls.at(2).empty());
        
        CPPUNIT_ASSERT_MESSAGE("Invalid surl returned (LFN1)", (*surls.at(0).begin()) == CS_TESTCASE_SURL_1);
        CPPUNIT_ASSERT_MESSAGE("Invalid surl returned (LFN2)", (*surls.at(1).begin()) == CS_TESTCASE_SURL_2);
        CPPUNIT_ASSERT_MESSAGE("Invalid surl returned (LFN3)", (*surls.at(2).begin()) == CS_TESTCASE_SURL_3);
    } catch(const std::exception& exc){
        std::string reason = (std::string)"Failed for LFNs :" + exc.what();
        CPPUNIT_FAIL(reason.c_str());
    } catch(...){
        CPPUNIT_FAIL("Got Unknown Error");
    }
}

/*
 * Test ListSurlsWithGUID
 */
void CatalogServiceTestCase::testListSurlsWithGUID(){
    
    try{
        CatalogService::StringArray names;
        std::vector<CatalogService::StringArray> surls;
    
        names.push_back(CS_TESTCASE_GUID_1);
        names.push_back(CS_TESTCASE_GUID_2);
        names.push_back(CS_TESTCASE_GUID_3);

        // List Surls
        CatalogService::CatalogResult result;
        m_service->listSurls(names,"",surls,result);
        CPPUNIT_ASSERT_MESSAGE("ListSurls failed on GUIDs", result.status == CatalogService::CatalogResult::R_SUCCESS);

        CPPUNIT_ASSERT_MESSAGE("No surls returned (empty array)", false == surls.empty());
        CPPUNIT_ASSERT_MESSAGE("Invalid surls size", names.size() == surls.size());
        CPPUNIT_ASSERT_MESSAGE("No surls returned (GUID1)", false == surls.at(0).empty());
        CPPUNIT_ASSERT_MESSAGE("No surls returned (GUID2)", false == surls.at(1).empty());
        CPPUNIT_ASSERT_MESSAGE("No surls returned (GUID3)", false == surls.at(2).empty());

        CPPUNIT_ASSERT_MESSAGE("Invalid surl returned (GUID1)", (*surls.at(0).begin()) == CS_TESTCASE_SURL_G1);
        CPPUNIT_ASSERT_MESSAGE("Invalid surl returned (GUID2)", (*surls.at(1).begin()) == CS_TESTCASE_SURL_G2);
        CPPUNIT_ASSERT_MESSAGE("Invalid surl returned (GUID3)", (*surls.at(2).begin()) == CS_TESTCASE_SURL_G3);
    } catch(const std::exception& exc){
        std::string reason = (std::string)"Failed for GUIDs :" + exc.what();
        CPPUNIT_FAIL(reason.c_str());
    } catch(...){
        CPPUNIT_FAIL("Got Unknown Error");
    }
}

/*
 * Test ListSurls that should fail
 */
void CatalogServiceTestCase::testListSurls_Error(){
    
    try{
        CatalogService::StringArray names;
        std::vector<CatalogService::StringArray> surls;
    
        names.push_back(CS_TESTCASE_FAIL_LFN_1);

        // List Surls
        CatalogService::CatalogResult result;
        m_service->listSurls(names,"",surls,result);
        CPPUNIT_ASSERT_MESSAGE("That Lfn should fail", result.status != CatalogService::CatalogResult::R_SUCCESS);
        if(result.status == CatalogService::CatalogResult::R_SOME_FAILURES){
            CPPUNIT_ASSERT_MESSAGE("That LFN should fail", std::find_if(result.failures.begin(),result.failures.end(),compare_first(CS_TESTCASE_FAIL_LFN_1)) != result.failures.end());    
        }
    } catch(const std::exception& exc){
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        CPPUNIT_FAIL("Got Unknown Error");
    }
}

/*
 * Test CheckSurlsWithLFN
 */
void CatalogServiceTestCase::testCheckSurlsWithLFN(){
    
    try{
        // Create Request
        CatalogService::StringPairArray names;
        names.push_back(CatalogService::StringPair(CS_TESTCASE_LFN_1,CS_TESTCASE_SURL_1_3));
        names.push_back(CatalogService::StringPair(CS_TESTCASE_LFN_2,CS_TESTCASE_SURL_2_3));
        names.push_back(CatalogService::StringPair(CS_TESTCASE_LFN_3,CS_TESTCASE_SURL_3_3));
        
        // Check Surls
        CatalogService::CatalogResult result;
        m_service->checkSurls(names,result);
        CPPUNIT_ASSERT_MESSAGE("Surls for LFNs already exist", result.status == CatalogService::CatalogResult::R_SUCCESS);
    } catch(const std::exception& exc){
        std::string reason = (std::string)"Failed on LFNs :" + exc.what();
        CPPUNIT_FAIL(reason.c_str());
    } catch(...){
        CPPUNIT_FAIL("Got Unknown Error");
    } 
}
/*
 * Test CheckSurlsWithGUID
 */
void CatalogServiceTestCase::testCheckSurlsWithGUID(){
    
    try{
        // Create Request
        CatalogService::StringPairArray names;
        names.push_back(CatalogService::StringPair(CS_TESTCASE_GUID_1,CS_TESTCASE_SURL_1_3));
        names.push_back(CatalogService::StringPair(CS_TESTCASE_GUID_2,CS_TESTCASE_SURL_2_3));
        names.push_back(CatalogService::StringPair(CS_TESTCASE_GUID_3,CS_TESTCASE_SURL_3_3));
        
        // Check Surls
        CatalogService::CatalogResult result;
        m_service->checkSurls(names,result);
        CPPUNIT_ASSERT_MESSAGE("Surls for GUIDs already exist", result.status == CatalogService::CatalogResult::R_SUCCESS);
    } catch(const std::exception& exc){
        std::string reason = (std::string)"Failed on GUIDs :" + exc.what();
        CPPUNIT_FAIL(reason.c_str());
    } catch(...){
        CPPUNIT_FAIL("Got Unknown Error");
    } 
}
/*
 * Test CheckSurls that should fail
 */
void CatalogServiceTestCase::testCheckSurls_Error(){
    
    std::string surl;
    try{
        // Create Request
        CatalogService::StringPairArray names;
        names.push_back(CatalogService::StringPair(CS_TESTCASE_LFN_1,CS_TESTCASE_SURL_1));
       
        // Check Surls
        CatalogService::CatalogResult result;
        m_service->checkSurls(names,result);
        CPPUNIT_ASSERT_MESSAGE("Surls for LFNs should exist", result.status != CatalogService::CatalogResult::R_SUCCESS);
        if(result.status == CatalogService::CatalogResult::R_SOME_FAILURES){
            CPPUNIT_ASSERT_MESSAGE("That LFN already had the replica", std::find_if(result.failures.begin(),result.failures.end(),compare_first(CS_TESTCASE_LFN_1)) != result.failures.end());
        }
    } catch(const std::exception& exc){
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        CPPUNIT_FAIL("Got Unknown Error");
    } 
}

/*
 * Test GetStatsWithLFN
 */
void CatalogServiceTestCase::testGetStatsWithLFN(){
    
    try{
        // Create Request
        CatalogService::StringPairArray names;
        names.push_back(CatalogService::StringPair(CS_TESTCASE_LFN_1,CS_TESTCASE_SURL_1));
        names.push_back(CatalogService::StringPair(CS_TESTCASE_LFN_2,CS_TESTCASE_SURL_2));
        names.push_back(CatalogService::StringPair(CS_TESTCASE_LFN_3,CS_TESTCASE_SURL_3));
        
        // Get Stats
        CatalogService::FileStatArray stats;
        CatalogService::CatalogResult result;
        m_service->getStats(names,stats,result);
        CPPUNIT_ASSERT_MESSAGE("GetStats Failed", result.status == CatalogService::CatalogResult::R_SUCCESS);
        CPPUNIT_ASSERT_MESSAGE("Invalid Number Of Stats Returned", stats.size() == 3);
        CatalogService::FileStatArray::iterator it;
        for(it = stats.begin(); it != stats.end(); ++it){
            const CatalogService::FileStatPtr& stat = (*it);
            CPPUNIT_ASSERT_MESSAGE("Invalid Stat", stat.get() != 0);
            CPPUNIT_ASSERT_MESSAGE("Empty Stat Logical Name", stat->logical.empty() == false);
        }
    } catch(const std::exception& exc){
        std::string reason = (std::string)"Failed on LFNs :" + exc.what();
        CPPUNIT_FAIL(reason.c_str());
    } catch(...){
        CPPUNIT_FAIL("Got Unknown Error");
    } 
}
/*
 * Test GetStatsWithGUID
 */
void CatalogServiceTestCase::testGetStatsWithGUID(){
    
    try{
        // Create Request
        CatalogService::StringPairArray names;
        names.push_back(CatalogService::StringPair(CS_TESTCASE_GUID_1,CS_TESTCASE_SURL_G1));
        names.push_back(CatalogService::StringPair(CS_TESTCASE_GUID_2,CS_TESTCASE_SURL_G2));
        names.push_back(CatalogService::StringPair(CS_TESTCASE_GUID_3,CS_TESTCASE_SURL_G3));
        
        // Get Stats
        CatalogService::FileStatArray stats;
        CatalogService::CatalogResult result;
        m_service->getStats(names,stats,result);
        CPPUNIT_ASSERT_MESSAGE("GetStats Failed", result.status == CatalogService::CatalogResult::R_SUCCESS);
        CPPUNIT_ASSERT_MESSAGE("Invalid Number Of Stats Returned", stats.size() == 3);
        CatalogService::FileStatArray::iterator it;
        for(it = stats.begin(); it != stats.end(); ++it){
            const CatalogService::FileStatPtr& stat = (*it);
            CPPUNIT_ASSERT_MESSAGE("Invalid Stat", stat.get() != 0);
            CPPUNIT_ASSERT_MESSAGE("Empty Stat Logical Name", stat->logical.empty() == false);
        }
    } catch(const std::exception& exc){
        std::string reason = (std::string)"Failed on GUIDs :" + exc.what();
        CPPUNIT_FAIL(reason.c_str());
    } catch(...){
        CPPUNIT_FAIL("Got Unknown Error");
    } 
}
/*
 * Test GetStats that should fail
 */
void CatalogServiceTestCase::testGetStats_Error(){
    
    std::string surl;
    try{
        // Create Request
        CatalogService::StringPairArray names;
        names.push_back(CatalogService::StringPair(CS_TESTCASE_FAIL_LFN_1,CS_TESTCASE_FAIL_SURL_1));
       
        // Check Surls
        CatalogService::FileStatArray stats;
        CatalogService::CatalogResult result;
        m_service->getStats(names,stats,result);
        CPPUNIT_ASSERT_MESSAGE("This getStats should fail", result.status != CatalogService::CatalogResult::R_SUCCESS);
        if(result.status == CatalogService::CatalogResult::R_SOME_FAILURES){
            CPPUNIT_ASSERT_MESSAGE("That LFN shouldn't exsit", std::find_if(result.failures.begin(),result.failures.end(),compare_first(CS_TESTCASE_FAIL_LFN_1)) != result.failures.end());
        }
    } catch(const std::exception& exc){
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        CPPUNIT_FAIL("Got Unknown Error");
    } 
}

/*
 * Test RegisterWithStats
 */
void CatalogServiceTestCase::testRegisterWithStats(){
    
    try{
        // Create Request
        CatalogService::StringPairArray names;
        names.push_back(CatalogService::StringPair(CS_TESTCASE_LFN_1,CS_TESTCASE_SURL_1));
        
        // Get Stats
        CatalogService::FileStatArray stats;
        CatalogService::CatalogResult result;
        m_service->getStats(names,stats,result);
        CPPUNIT_ASSERT_MESSAGE("GetStats Failed", result.status == CatalogService::CatalogResult::R_SUCCESS);
        CPPUNIT_ASSERT_MESSAGE("Invalid Number Of Stats Returned", stats.size() == 1);
        CatalogService::FileStatArray::iterator it;
        for(it = stats.begin(); it != stats.end(); ++it){
            const CatalogService::FileStatPtr& stat = (*it);
            CPPUNIT_ASSERT_MESSAGE("Invalid Stat", stat.get() != 0);
            CPPUNIT_ASSERT_MESSAGE("Empty Stat Logical Name", stat->logical.empty() == false);
        }
        
        // Register New replica with the stats
        CatalogService::ReplicaStatArray replicas;
        replicas.push_back(CatalogService::ReplicaStat(CS_TESTCASE_LFN_1,CS_TESTCASE_SURL_1_4,stats.at(0)));
        // Register SURL
        CatalogService::CatalogResult reg_result;
        m_service->registerSurls(replicas,reg_result);
        CPPUNIT_ASSERT_MESSAGE("Failed register with stat", reg_result.status == CatalogService::CatalogResult::R_SUCCESS);
    } catch(const std::exception& exc){
        std::string reason = (std::string)"Failed register with stats:" + exc.what();
        CPPUNIT_FAIL(reason.c_str());
    } catch(...){
        CPPUNIT_FAIL("Got Unknown Error");
    } 
}
