/*
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *  GLite Data Transfer Agents: main function
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *  Version info: $Id: OraTestSetup.cpp,v 1.3 2006/08/23 14:31:49 badino Exp $
 *  Release: $Name:  $
 *
 */
#include "occi.h"

#include <stdlib.h>
#include <cstring>
#include <iostream>
#include <unistd.h>
#include <fstream>
#include <sstream>

using std::cout;
using std::cerr;
using std::cin;
using std::endl;
using namespace ::oracle::occi;

namespace {
const char * const CONFIG_NAME = "glite-transfer-agent-setup-test-cred-ora";
const char * const PROGNAME    = "glite_data_transfer_agent_setup-test-cred-ora";
const char * const PROGVERSION = "0.0";
const char * const PROGAUTHOR  = "(C) EGEE";
}

/*
 * print_usage_and_die
 *
 * Print Program Usage and ther exit
 */
void print_usage_and_die(){
    cout << endl;
    cout << "<" << PROGNAME << "> Version " << PROGVERSION << " by " << PROGAUTHOR;
    cout << endl;
    cout << "usage: " << PROGNAME << " " << endl;
    cout << " -u : Specify the DN of the user" << endl;
    cout << " -i : Specify the delegation id" << endl;
    cout << " -p : specify the location of the proxy file" << endl;
    cout << " -U : Specify the DB user name" << endl;
    cout << " -P : Specify the DB User Password" << endl;
    cout << " -C : specify the DB Connect String" << endl;
    cout << " -v : specify the certificate validity in respect of current timestamp (may be negative)" << endl;
    cout << " Where:" << endl;
    cout << " Optional parameters:" << endl;
    cout << "  -h : print this screen" << endl;
    exit(-1);
}

/*
 * main
 *
 * Main function
 */
int main(int argc, char* argv[]) {

    // Get Options
    bool verbose = false;
    const char * user_dn     = 0;
    const char * dlg_id      = 0;
    const char * proxy_file  = 0;
    const char * db_user     = 0;
    const char * db_password = 0;
    const char * db_connect  = 0;
    int          validity    = 3600;
    int flag;
    while ((flag = getopt (argc, argv, "u:i:p:U:P:C:v:dh")) != -1) {
        switch (flag) {
            case 'u':
                user_dn = optarg;
            break;
            case 'i':
                dlg_id = optarg;
            break;
            case 'p':
                proxy_file = optarg;
            break;
            case 'U':
                db_user = optarg;
            break;
            case 'P':
                db_password = optarg;
            break;
            case 'C':
                db_connect = optarg;
            break;
            case 'v':
                validity = atoi(optarg);
            break;
            case 'd':
                verbose = true;
            break;
            case 'h':
            default:
                print_usage_and_die();
            break;
        } // End switch
    } // End while

    if((0 == user_dn)     || (0 == strlen(user_dn)) ||
       (0 == dlg_id)      || (0 == strlen(dlg_id))  ||
       (0 == proxy_file)  || (0 == strlen(proxy_file))  ||
       (0 == db_user)     || (0 == strlen(db_user))  ||
       (0 == db_password) || (0 == strlen(db_password))  ||
       (0 == db_connect)  || (0 == strlen(db_connect))){
        print_usage_and_die();
    }

    // Check if the proxy file exists
    std::ifstream ifs(proxy_file, std::ios::in | std::ios_base::binary);
    if(ifs.bad()){
        cerr << "Can't open Proxy file " << proxy_file << endl;
        return -1;
    }
    
    int result = 0;
    // Connect to the DB
    Environment * env = 0;
    Connection *conn  = 0;
    Statement * stmt  = 0;
    try{
        env = Environment::createEnvironment(Environment::DEFAULT);
        if(verbose) cout << "Environment Created" << endl;
            
        conn = env->createConnection(db_user, db_password, db_connect);
        if(verbose) cout << "Connection Created" << endl;
   
        stmt = conn->createStatement();
        if(verbose) cout << "Statement Created" << endl;

        // Prepare the Statement to execute
        stmt->setSQL("INSERT INTO t_credential("\
            "dlg_id,dn,proxy,voms_attrs,termination_time)"\
            " VALUES (:1,:2,:3,:4,:5)");
        if(verbose) cout << "Statement prepared" << endl;
        
        // Get the timestamp
        time_t current;
        time(&current);
        current += validity;
        struct tm * tmp_tm = gmtime(&current);
        Timestamp timestamp(env,                    // Environment
                            tmp_tm->tm_year + 1900, // Year
                            tmp_tm->tm_mon + 1,     // Month
                            tmp_tm->tm_mday,        // Day
                            tmp_tm->tm_hour,        // Hour
                            tmp_tm->tm_min,         // Minute
                            tmp_tm->tm_sec,         // Second
                            0,                      // Fraction of Seconds
                            0,                      // tz hour
                            0);                     // tz minute
        if(verbose) cout << "Timestamp created" << endl;
        
        // Read the Proxy File and put it into a Clob
        std::string s;
        char buffer[1024];
        while(!ifs.eof()){
            ifs.read(buffer,1024);
            int bytes_read = ifs.gcount();
            if(verbose) cout << "Read " << bytes_read  << "Bytes" << endl;
            if(bytes_read  > 0){
                s.append(buffer,bytes_read);
            }
        }
        if(verbose) cout << "Stream copied" << endl;
        //if(verbose) cout << "proxy value is [" << endl << s << endl << "]" << endl;
        
        // Bind the variables
        stmt->setString(1,dlg_id);
        stmt->setString(2,user_dn);
        stmt->setString(3,s);
        stmt->setString(4,"");
        stmt->setTimestamp(5,timestamp);
        
        if(verbose) cout << "Executing Statement..." << endl;
        stmt->executeUpdate();
        if(verbose) cout << "Done" << endl;
            
        conn->commit();
        if(verbose) cout << "Committed" << endl;
        
    } catch(const SQLException& exc){
        cerr << "Failed to fill credentials for executing the Unit Test: " 
             << exc.getMessage().c_str() << endl;
        result = -1;
    }        
    if(0 != stmt){
       conn->terminateStatement(stmt);
    }
    if(0 != conn){
       env->terminateConnection(conn);
    }
    if(0 != env){
        Environment::terminateEnvironment(env);
    }
    return result;
}
