/*
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *  GLite Data Transfer Agents: AgentDAO Test Case
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *  Version info: $Id: AgentDaoTestCase.cpp,v 1.8 2009/08/28 16:32:16 molnarzs Exp $
 *  Release: $Name:  $
 *
 */
#include "DaoHelper.h"
#include "glite/data/transfer/agent/model/Agent.h"
#include "glite/data/transfer/agent/dao/vo/AgentDAO.h"
#include "glite/data/transfer/agent/dao/channel/AgentDAO.h"
#include "glite/data/agents/dao/DAOException.h"
#include "glite/data/transfer/agent/dao/vo/DAOFactory.h"
#include "glite/data/transfer/agent/dao/channel/DAOFactory.h"
#include "glite/data/transfer/agent/dao/cred/DAOFactory.h"

#include <boost/scoped_ptr.hpp>
#include <cppunit/TestCase.h>
#include <cppunit/extensions/HelperMacros.h>

using namespace glite::data::transfer::agent;
using namespace glite::data::transfer::agent::model;
using glite::data::agents::dao::DAOException;
using glite::data::agents::longlong;
using glite::data::agents::dao::StringArray;

using boost::scoped_ptr;

/**
 * CPPUnit AgentDaoTestCase
 */
class AgentDaoTestCase : public CppUnit::TestCase{
public:
    /**
     * set-up test case
     */
    void setUp();

private:
    // Declare CPPUnit Methods
    // -------------------------------------------------------------------------
    CPPUNIT_TEST_SUITE( AgentDaoTestCase );
    // VO View
    CPPUNIT_TEST( testGet_VO_Error );
    CPPUNIT_TEST( testCreate_VO );
    CPPUNIT_TEST( testCreate_VO_2 );
    CPPUNIT_TEST( testGet_VO );
    CPPUNIT_TEST( testUpdate_VO );
    // Channel View
    CPPUNIT_TEST( testGet_Ch_Error );
    CPPUNIT_TEST( testCreate_Ch );
    CPPUNIT_TEST( testCreate_Ch_2 );
    CPPUNIT_TEST( testGet_Ch );
    CPPUNIT_TEST( testUpdate_Ch );
    // Cred View
    CPPUNIT_TEST( testGet_Cr_Error );
    CPPUNIT_TEST( testCreate_Cr );
    CPPUNIT_TEST( testCreate_Cr_2 );
    CPPUNIT_TEST( testGet_Cr );
    CPPUNIT_TEST( testUpdate_Cr );
    // Unit test for #10534
    CPPUNIT_TEST( testUpdate_VO_CH );
    CPPUNIT_TEST( testUpdate_VO_CH_CR );
    CPPUNIT_TEST_SUITE_END();

    // VO Test Methods
    // -------------------------------------------------------------------------

    /**
     * Test Get Method
     */
    void testGet_VO();
    /**
     * Test Get Method
     */
    void testGet_VO_Error();
    /**
     * Test Update Method
     */
    void testUpdate_VO();
    /**
     * Test Update Method
     */
    void testUpdate_VO_CH();
    /**
     * Test Update Method
     */
    void testUpdate_VO_CH_CR();
    /**
     * Test Create Method
     */
    void testCreate_VO();
    /**
     * Test Create Method
     */
    void testCreate_VO_2();

    // Channel Test Methods
    // -------------------------------------------------------------------------

    /**
     * Test Get Method
     */
    void testGet_Ch();
    /**
     * Test Get Method
     */
    void testGet_Ch_Error();
    /**
     * Test Update Method
     */
    void testUpdate_Ch();
    /**
     * Test Create Method
     */
    void testCreate_Ch();
    /**
     * Test Create Method
     */
    void testCreate_Ch_2();

    // Cred Test Methods
    // -------------------------------------------------------------------------

    /**
     * Test Get Method
     */
    void testGet_Cr();
    /**
     * Test Get Method
     */
    void testGet_Cr_Error();
    /**
     * Test Update Method
     */
    void testUpdate_Cr();
    /**
     * Test Create Method
     */
    void testCreate_Cr();
    /**
     * Test Create Method
     */
    void testCreate_Cr_2();

private:
    /**
     * The VO AgentDAO Instance
     */
    scoped_ptr<dao::vo::AgentDAO> m_voAgentDAO;
    /**
     * The Channel AgentDAO Instance
     */
    scoped_ptr<dao::channel::AgentDAO> m_chAgentDAO;
    /**
     * The Cred AgentDAO Instance
     */
    scoped_ptr<dao::cred::AgentDAO> m_crAgentDAO;
};

/*
 * Register Suit
 */
CPPUNIT_TEST_SUITE_NAMED_REGISTRATION( AgentDaoTestCase, "AgentDaoTestCase" );

namespace {
const char * const AGENTDAO_TESTCASE_VO1_NAME       = "AGENT_VO_1";
const char * const AGENTDAO_TESTCASE_CHANNEL1_NAME  = "AGENT_channel_1";
const char * const AGENTDAO_TESTCASE_CRED1_NAME     = "AGENT_cred_1";
}
/*
 * setUp
 *
 * set-up test case
 */
void AgentDaoTestCase::setUp(){

    try{
        // Create VO AgentDAO instance
        m_voAgentDAO.reset(dao::vo::DAOFactory::instance().createAgentDAO(AGENTDAO_TESTCASE_VO1_NAME, DaoHelper::ctx()));
        CPPUNIT_ASSERT_MESSAGE("Null VO AgentDAO", m_voAgentDAO.get() != 0);

        // Create Channel AgentDAO instance
        m_chAgentDAO.reset(dao::channel::DAOFactory::instance().createAgentDAO(AGENTDAO_TESTCASE_CHANNEL1_NAME, DaoHelper::ctx()));
        CPPUNIT_ASSERT_MESSAGE("Null Channel AgentDAO", m_chAgentDAO.get() != 0);

        // Create Cred AgentDAO instance
        m_crAgentDAO.reset(dao::cred::DAOFactory::instance().createAgentDAO(AGENTDAO_TESTCASE_CRED1_NAME, DaoHelper::ctx()));
        CPPUNIT_ASSERT_MESSAGE("Null Cred AgentDAO", m_crAgentDAO.get() != 0);
    } catch(const std::exception& exc){
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        CPPUNIT_FAIL("Got Unknown Error");
    }
}

/*
 * testGet
 *
 * Test Get Method
 */
void AgentDaoTestCase::testGet_VO(){
    try{
        // Get Agent instance
        scoped_ptr<model::Agent> a(m_voAgentDAO->get());
        CPPUNIT_ASSERT_MESSAGE("Null Agent (VO1)", a.get() != 0);
    } catch(const std::exception& exc){
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        CPPUNIT_FAIL("Got Unknown Error");
    }
}

/*
 * testGet
 *
 * Test Get Method
 */
void AgentDaoTestCase::testGet_VO_Error(){
    try{
        // Get Agent instance
        scoped_ptr<model::Agent> a(m_voAgentDAO->get());
        CPPUNIT_FAIL("This agent shouldn't be there (VO1)");
    } catch(const DAOException& exc){
        // OK
    } catch(const std::exception& exc){
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        CPPUNIT_FAIL("Got Unknown Error");
    }
}


/*
 * testUpdate
 *
 * Test Update Method
 */
void AgentDaoTestCase::testUpdate_VO(){

    try{
        // Start Transaction
        DaoHelper::ctx().start();

        // Get Agent instance
        scoped_ptr<model::Agent> a(m_voAgentDAO->get());
        CPPUNIT_ASSERT_MESSAGE("Null Agent (VO1)", a.get() != 0);

        time_t current;
        time(&current);

        a->state = model::Agent::S_STOPPED;
        a->lastActive = current;

        // Update Agent
        m_voAgentDAO->update(*a);

        // Commit Transaction
        DaoHelper::ctx().commit();

        // Get Agent instance
        a.reset(m_voAgentDAO->get());
        CPPUNIT_ASSERT_MESSAGE("Null Agent (VO1)", a.get() != 0);
        CPPUNIT_ASSERT_MESSAGE("Invalid State (VO1)", a->state == model::Agent::S_STOPPED);
        CPPUNIT_ASSERT_MESSAGE("Invalid Last Active (VO1)", a->lastActive == current);
    } catch(const std::exception& exc){
        DaoHelper::ctx().rollback();
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        DaoHelper::ctx().rollback();
        CPPUNIT_FAIL("Got Unknown Error");
    }
}



/*
 * testCreate
 *
 * Test Create Method
 */
void AgentDaoTestCase::testCreate_VO(){
    try{
        // Start Transaction
        DaoHelper::ctx().start();

        // Create an Agent Instance
        scoped_ptr<model::Agent> a(new model::Agent("fts",
                model::Agent::S_STARTED,
                "1.0.0",
                "localhost",
                "me@nowhere.com"));
        CPPUNIT_ASSERT_MESSAGE("Null Agent (VO1)", a.get() != 0);

        // Create Agent instance
        m_voAgentDAO->create(*a);

        // Commit Transaction
        DaoHelper::ctx().commit();

        // Get Agent instance
        a.reset(m_voAgentDAO->get());
        CPPUNIT_ASSERT_MESSAGE("Null Agent (VO1)", a.get() != 0);
        CPPUNIT_ASSERT_MESSAGE("Invalid Type (VO1)", a->type == "fts");
        CPPUNIT_ASSERT_MESSAGE("Invalid State (VO1)", a->state == model::Agent::S_STARTED);
        CPPUNIT_ASSERT_MESSAGE("Invalid Version (VO1)", a->version == "1.0.0");
        CPPUNIT_ASSERT_MESSAGE("Invalid Host (VO1)", a->hostName == "localhost");
        CPPUNIT_ASSERT_MESSAGE("Invalid Contact (VO1)", a->contact == "me@nowhere.com");
    } catch(const std::exception& exc){
        DaoHelper::ctx().rollback();
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        DaoHelper::ctx().rollback();
        CPPUNIT_FAIL("Got Unknown Error");
    }
}

/*
 * testCreate_2
 *
 * Test Create Method
 */
void AgentDaoTestCase::testCreate_VO_2(){
    try{
        // Test that the agent already exists
        scoped_ptr<model::Agent> a(m_voAgentDAO->get());
        CPPUNIT_ASSERT_MESSAGE("Null Agent (VO1)", a.get() != 0);

        // Start Transaction
        DaoHelper::ctx().start();

        // Create an Agent Instance
        a.reset(new model::Agent("fps",
                model::Agent::S_STOPPED,
                "1.2.3",
                "localhost",
                "you@nowhere.com"));
        CPPUNIT_ASSERT_MESSAGE("Null Agent (VO1)", a.get() != 0);

        // Create Agent instance
        m_voAgentDAO->create(*a);

        // Commit Transaction
        DaoHelper::ctx().commit();

        // Get Agent instance
        a.reset(m_voAgentDAO->get());
        CPPUNIT_ASSERT_MESSAGE("Null Agent (VO1)", a.get() != 0);
        CPPUNIT_ASSERT_MESSAGE("Invalid Type (VO1)", a->type == "fps");
        CPPUNIT_ASSERT_MESSAGE("Invalid State (VO1)", a->state == model::Agent::S_STOPPED);
        CPPUNIT_ASSERT_MESSAGE("Invalid Version (VO1)", a->version == "1.2.3");
        CPPUNIT_ASSERT_MESSAGE("Invalid Host (VO1)", a->hostName == "localhost");
        CPPUNIT_ASSERT_MESSAGE("Invalid Contact (VO1)", a->contact == "you@nowhere.com");
    } catch(const std::exception& exc){
        DaoHelper::ctx().rollback();
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        DaoHelper::ctx().rollback();
        CPPUNIT_FAIL("Got Unknown Error");
    }
}


/*
 * testGet
 *
 * Test Get Method
 */
void AgentDaoTestCase::testGet_Ch(){
    try{
        // Get Agent instance
        scoped_ptr<model::Agent> a(m_chAgentDAO->get());
        CPPUNIT_ASSERT_MESSAGE("Null Agent (CH1)", a.get() != 0);
    } catch(const std::exception& exc){
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        CPPUNIT_FAIL("Got Unknown Error");
    }
}

/*
 * testGet
 *
 * Test Get Method
 */
void AgentDaoTestCase::testGet_Ch_Error(){
    try{
        // Get Agent instance
        scoped_ptr<model::Agent> a(m_chAgentDAO->get());
        CPPUNIT_FAIL("This agent shouldn't be there (CH1)");
    } catch(const DAOException& exc){
        // OK
    } catch(const std::exception& exc){
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        CPPUNIT_FAIL("Got Unknown Error");
    }
}


/*
 * testUpdate
 *
 * Test Update Method
 */
void AgentDaoTestCase::testUpdate_Ch(){

    try{
        // Start Transaction
        DaoHelper::ctx().start();

        // Get Agent instance
        scoped_ptr<model::Agent> a(m_chAgentDAO->get());
        CPPUNIT_ASSERT_MESSAGE("Null Agent (CH1)", a.get() != 0);

        time_t current;
        time(&current);

        a->state = model::Agent::S_STOPPED;
        a->lastActive = current;

        // Update Agent
        m_chAgentDAO->update(*a);

        // Commit Transaction
        DaoHelper::ctx().commit();

        // Get Agent instance
        a.reset(m_chAgentDAO->get());
        CPPUNIT_ASSERT_MESSAGE("Null Agent (CH1)", a.get() != 0);
        CPPUNIT_ASSERT_MESSAGE("Invalid State (VO1)", a->state == model::Agent::S_STOPPED);
        CPPUNIT_ASSERT_MESSAGE("Invalid Last Active (CH1)", a->lastActive == current);
    } catch(const std::exception& exc){
        DaoHelper::ctx().rollback();
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        DaoHelper::ctx().rollback();
        CPPUNIT_FAIL("Got Unknown Error");
    }
}

/*
 * testCreate
 *
 * Test Create Method
 */
void AgentDaoTestCase::testCreate_Ch(){
    try{
        // Start Transaction
        DaoHelper::ctx().start();

        // Create an Agent Instance
        scoped_ptr<model::Agent> a(new model::Agent("channel",
                model::Agent::S_STARTED,
                "1.0.1",
                "localhost",
                "me@nowhere.com"));
        CPPUNIT_ASSERT_MESSAGE("Null Agent (CH1)", a.get() != 0);

        // Create Agent instance
        m_chAgentDAO->create(*a);

        // Commit Transaction
        DaoHelper::ctx().commit();

        // Get Agent instance
        a.reset(m_chAgentDAO->get());
        CPPUNIT_ASSERT_MESSAGE("Null Agent (CH1)", a.get() != 0);
        CPPUNIT_ASSERT_MESSAGE("Invalid Type (CH1)", a->type == "channel");
        CPPUNIT_ASSERT_MESSAGE("Invalid State (CH1)", a->state == model::Agent::S_STARTED);
        CPPUNIT_ASSERT_MESSAGE("Invalid Version (CH1)", a->version == "1.0.1");
        CPPUNIT_ASSERT_MESSAGE("Invalid Host (CH1)", a->hostName == "localhost");
        CPPUNIT_ASSERT_MESSAGE("Invalid Contact (CH1)", a->contact == "me@nowhere.com");
    } catch(const std::exception& exc){
        DaoHelper::ctx().rollback();
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        DaoHelper::ctx().rollback();
        CPPUNIT_FAIL("Got Unknown Error");
    }
}

/*
 * testCreate_2
 *
 * Test Create Method
 */
void AgentDaoTestCase::testCreate_Ch_2(){
    try{
        // Test that the agent already exists
        scoped_ptr<model::Agent> a(m_chAgentDAO->get());
        CPPUNIT_ASSERT_MESSAGE("Null Agent (CH1)", a.get() != 0);

        // Start Transaction
        DaoHelper::ctx().start();

        // Create an Agent Instance
        a.reset(new model::Agent("channel",
                model::Agent::S_STOPPED,
                "1.2.4",
                "localhost",
                "you@nowhere.com"));
        CPPUNIT_ASSERT_MESSAGE("Null Agent (CH1)", a.get() != 0);

        // Create Channel instance
        m_chAgentDAO->create(*a);

        // Commit Transaction
        DaoHelper::ctx().commit();

        // Get Transfer instance
        a.reset(m_chAgentDAO->get());
        CPPUNIT_ASSERT_MESSAGE("Null Agent (CH1)", a.get() != 0);
        CPPUNIT_ASSERT_MESSAGE("Invalid Type (CH1)", a->type == "channel");
        CPPUNIT_ASSERT_MESSAGE("Invalid State (CH1)", a->state == model::Agent::S_STOPPED);
        CPPUNIT_ASSERT_MESSAGE("Invalid Version (CH1)", a->version == "1.2.4");
        CPPUNIT_ASSERT_MESSAGE("Invalid Host (CH1)", a->hostName == "localhost");
        CPPUNIT_ASSERT_MESSAGE("Invalid Contact (CH1)", a->contact == "you@nowhere.com");
    } catch(const std::exception& exc){
        DaoHelper::ctx().rollback();
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        DaoHelper::ctx().rollback();
        CPPUNIT_FAIL("Got Unknown Error");
    }
}

/*
 * testGet
 *
 * Test Get Method
 */
void AgentDaoTestCase::testGet_Cr(){
    try{
        // Get Agent instance
        scoped_ptr<model::Agent> a(m_crAgentDAO->get());
        CPPUNIT_ASSERT_MESSAGE("Null Agent (CR)", a.get() != 0);
    } catch(const std::exception& exc){
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        CPPUNIT_FAIL("Got Unknown Error");
    }
}

/*
 * testGet
 *
 * Test Get Method
 */
void AgentDaoTestCase::testGet_Cr_Error(){
    try{
        // Get Agent instance
        scoped_ptr<model::Agent> a(m_crAgentDAO->get());
        CPPUNIT_FAIL("This agent shouldn't be there (CH1)");
    } catch(const DAOException& exc){
        // OK
    } catch(const std::exception& exc){
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        CPPUNIT_FAIL("Got Unknown Error");
    }
}


/*
 * testUpdate
 *
 * Test Update Method
 */
void AgentDaoTestCase::testUpdate_Cr(){

    try{
        // Start Transaction
        DaoHelper::ctx().start();

        // Get Agent instance
        scoped_ptr<model::Agent> a(m_crAgentDAO->get());
        CPPUNIT_ASSERT_MESSAGE("Null Agent (CR1)", a.get() != 0);

        time_t current;
        time(&current);

        a->state = model::Agent::S_STOPPED;
        a->lastActive = current;

        // Update Agent
        m_crAgentDAO->update(*a);

        // Commit Transaction
        DaoHelper::ctx().commit();

        // Get Agent instance
        a.reset(m_crAgentDAO->get());
        CPPUNIT_ASSERT_MESSAGE("Null Agent (CR1)", a.get() != 0);
        CPPUNIT_ASSERT_MESSAGE("Invalid State (CR1)", a->state == model::Agent::S_STOPPED);
        CPPUNIT_ASSERT_MESSAGE("Invalid Last Active (CR1)", a->lastActive == current);
    } catch(const std::exception& exc){
        DaoHelper::ctx().rollback();
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        DaoHelper::ctx().rollback();
        CPPUNIT_FAIL("Got Unknown Error");
    }
}

/*
 * testCreate
 *
 * Test Create Method
 */
void AgentDaoTestCase::testCreate_Cr(){
    try{
        // Start Transaction
        DaoHelper::ctx().start();

        // Create an Agent Instance
        scoped_ptr<model::Agent> a(new model::Agent("cred",
                model::Agent::S_STARTED,
                "1.0.1",
                "localhost",
                "me@nowhere.com"));
        CPPUNIT_ASSERT_MESSAGE("Null Agent (CR1)", a.get() != 0);

        // Create Agent instance
        m_crAgentDAO->create(*a);

        // Commit Transaction
        DaoHelper::ctx().commit();

        // Get Agent instance
        a.reset(m_crAgentDAO->get());
        CPPUNIT_ASSERT_MESSAGE("Null Agent (CR1)", a.get() != 0);
        CPPUNIT_ASSERT_MESSAGE("Invalid Type (CR1)", a->type == "cred");
        CPPUNIT_ASSERT_MESSAGE("Invalid State (CR1)", a->state == model::Agent::S_STARTED);
        CPPUNIT_ASSERT_MESSAGE("Invalid Version (CR1)", a->version == "1.0.1");
        CPPUNIT_ASSERT_MESSAGE("Invalid Host (CR1)", a->hostName == "localhost");
        CPPUNIT_ASSERT_MESSAGE("Invalid Contact (CR1)", a->contact == "me@nowhere.com");
    } catch(const std::exception& exc){
        DaoHelper::ctx().rollback();
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        DaoHelper::ctx().rollback();
        CPPUNIT_FAIL("Got Unknown Error");
    }
}

/*
 * testCreate_2
 *
 * Test Create Method
 */
void AgentDaoTestCase::testCreate_Cr_2(){
    try{
        // Test that the agent already exists
        scoped_ptr<model::Agent> a(m_crAgentDAO->get());
        CPPUNIT_ASSERT_MESSAGE("Null Agent (CR1)", a.get() != 0);

        // Start Transaction
        DaoHelper::ctx().start();

        // Create an Agent Instance
        a.reset(new model::Agent("cred",
                model::Agent::S_STOPPED,
                "1.2.4",
                "localhost",
                "you@nowhere.com"));
        CPPUNIT_ASSERT_MESSAGE("Null Agent (CR1)", a.get() != 0);

        // Create Channel instance
        m_crAgentDAO->create(*a);

        // Commit Transaction
        DaoHelper::ctx().commit();

        // Get Transfer instance
        a.reset(m_crAgentDAO->get());
        CPPUNIT_ASSERT_MESSAGE("Null Agent (CR1)", a.get() != 0);
        CPPUNIT_ASSERT_MESSAGE("Invalid Type (CR1)", a->type == "cred");
        CPPUNIT_ASSERT_MESSAGE("Invalid State (CR1)", a->state == model::Agent::S_STOPPED);
        CPPUNIT_ASSERT_MESSAGE("Invalid Version (CR1)", a->version == "1.2.4");
        CPPUNIT_ASSERT_MESSAGE("Invalid Host (CR1)", a->hostName == "localhost");
        CPPUNIT_ASSERT_MESSAGE("Invalid Contact (CR1)", a->contact == "you@nowhere.com");
    } catch(const std::exception& exc){
        DaoHelper::ctx().rollback();
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        DaoHelper::ctx().rollback();
        CPPUNIT_FAIL("Got Unknown Error");
    }
}

/*
 * testUpdate
 *
 * Test Update Method
 */
void AgentDaoTestCase::testUpdate_VO_CH(){

    try{
        // Start Transaction
        DaoHelper::ctx().start();

        // Get Agent instance
        scoped_ptr<model::Agent> a1(m_voAgentDAO->get());
        CPPUNIT_ASSERT_MESSAGE("Null Agent (VO1)", a1.get() != 0);
        scoped_ptr<model::Agent> a2(m_chAgentDAO->get());
        CPPUNIT_ASSERT_MESSAGE("Null Agent (CH1)", a2.get() != 0);

        a1->state = model::Agent::S_STOPPED;
        a2->state = model::Agent::S_STOPPED;

        // Update Agent
        m_voAgentDAO->update(*a1);
        m_chAgentDAO->update(*a2);

        // Commit Transaction
        DaoHelper::ctx().commit();

        // Get Agent instance
        a1.reset(m_voAgentDAO->get());
        a2.reset(m_chAgentDAO->get());
        CPPUNIT_ASSERT_MESSAGE("Null Agent (VO1)", a1.get() != 0);
        CPPUNIT_ASSERT_MESSAGE("Null Agent (CH1)", a2.get() != 0);
        CPPUNIT_ASSERT_MESSAGE("Invalid Type (VO1)", a1->type == "fps");
        CPPUNIT_ASSERT_MESSAGE("Invalid Type (CH1)", a2->type == "channel");
    } catch(const std::exception& exc){
        DaoHelper::ctx().rollback();
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        DaoHelper::ctx().rollback();
        CPPUNIT_FAIL("Got Unknown Error");
    }
}

/*
 * testUpdate
 *
 * Test Update Method
 */
void AgentDaoTestCase::testUpdate_VO_CH_CR(){

    try{
        // Start Transaction
        DaoHelper::ctx().start();

        // Get Agent instance
        scoped_ptr<model::Agent> a1(m_voAgentDAO->get());
        CPPUNIT_ASSERT_MESSAGE("Null Agent (VO1)", a1.get() != 0);
        scoped_ptr<model::Agent> a2(m_chAgentDAO->get());
        CPPUNIT_ASSERT_MESSAGE("Null Agent (CH1)", a2.get() != 0);
        scoped_ptr<model::Agent> a3(m_crAgentDAO->get());
        CPPUNIT_ASSERT_MESSAGE("Null Agent (CR1)", a3.get() != 0);

        a1->state = model::Agent::S_STOPPED;
        a2->state = model::Agent::S_STOPPED;
        a3->state = model::Agent::S_STOPPED;

        // Update Agent
        m_voAgentDAO->update(*a1);
        m_chAgentDAO->update(*a2);
        m_crAgentDAO->update(*a3);

        // Commit Transaction
        DaoHelper::ctx().commit();

        // Get Agent instance
        a1.reset(m_voAgentDAO->get());
        a2.reset(m_chAgentDAO->get());
        a3.reset(m_crAgentDAO->get());
        CPPUNIT_ASSERT_MESSAGE("Null Agent (VO1)", a1.get() != 0);
        CPPUNIT_ASSERT_MESSAGE("Null Agent (CH1)", a2.get() != 0);
        CPPUNIT_ASSERT_MESSAGE("Null Agent (CR1)", a3.get() != 0);
        CPPUNIT_ASSERT_MESSAGE("Invalid Type (VO1)", a1->type == "fps");
        CPPUNIT_ASSERT_MESSAGE("Invalid Type (CH1)", a2->type == "channel");
        CPPUNIT_ASSERT_MESSAGE("Invalid Type (CR)", a3->type == "cred");
    } catch(const std::exception& exc){
        DaoHelper::ctx().rollback();
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        DaoHelper::ctx().rollback();
        CPPUNIT_FAIL("Got Unknown Error");
    }
}

