/*
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *  GLite Data Transfer Agents: CredDAO Test Case
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *  Version info: $Id: CredDaoTestCase.cpp,v 1.5 2008/04/23 15:10:14 ptedesco Exp $
 *  Release: $Name:  $
 *
 */
#include "DaoHelper.h"
#include "glite/data/transfer/agent/model/Cred.h"
#include "glite/data/transfer/agent/dao/vo/CredDAO.h"
#include "glite/data/transfer/agent/dao/channel/CredDAO.h"
#include "glite/data/transfer/agent/dao/cred/CredDAO.h"
#include "glite/data/agents/dao/DAOException.h"
#include "glite/data/transfer/agent/dao/vo/DAOFactory.h"
#include "glite/data/transfer/agent/dao/channel/DAOFactory.h"
#include "glite/data/transfer/agent/dao/cred/DAOFactory.h"

#include <sys/time.h>
#include <boost/scoped_ptr.hpp>
#include <cppunit/TestCase.h>
#include <cppunit/extensions/HelperMacros.h>

using namespace glite::data::transfer::agent;
using namespace glite::data::transfer::agent::model;
using glite::data::agents::dao::DAOException;

using boost::scoped_ptr;

/**
 * CPPUnit CredDaoTestCase
 */
class CredDaoTestCase : public CppUnit::TestCase{
public:
    /**
     * set-up test case
     */
    void setUp ();
private:
    // Declare CPPUnit Methods
    // -------------------------------------------------------------------------
    CPPUNIT_TEST_SUITE( CredDaoTestCase);
    // VO View
    CPPUNIT_TEST( testGet_VO );
    CPPUNIT_TEST( testGet_VO_Error );
    CPPUNIT_TEST( testGet_VO_Error_2 );
    // Channel View
    CPPUNIT_TEST( testGet_Ch );
    CPPUNIT_TEST( testGet_Ch_Error );
    // Cred View
    CPPUNIT_TEST( testGet_Cred );
    CPPUNIT_TEST( testGet_Cred_Error );
    CPPUNIT_TEST( testUpdate_Cred );
    CPPUNIT_TEST( testGetExpiring_Cred );
    CPPUNIT_TEST( testGet_Cred_Stress );
    CPPUNIT_TEST_SUITE_END();

    // Test Methods VO View
    // -------------------------------------------------------------------------

    /**
     * Test Get Method
     */
    void testGet_VO();
    void testGet_VO_Error();
    void testGet_VO_Error_2();
    
    // Test Methods Channel View
    // -------------------------------------------------------------------------

    /**
     * Test Get Method
     */
    void testGet_Ch();
    void testGet_Ch_Error();
    

    // Test Methods Cred View
    // -------------------------------------------------------------------------

    /**
     * Test Get Method
     */
    void testGet_Cred();
    void testGet_Cred_Error();
    void testUpdate_Cred();
    void testGetExpiring_Cred();
    void testGet_Cred_Stress();
    
private:
    /**
     * The VO CredDAO Instance
     */
    scoped_ptr<dao::vo::CredDAO> m_voCredDAO;
    /**
     * The Channel CredDAO Instance
     */
    scoped_ptr<dao::channel::CredDAO> m_chCredDAO;
    /**
     * The Cred CredDAO Instance
     */
    scoped_ptr<dao::cred::CredDAO> m_credCredDAO;
};

/*
 * Register Suit
 */
CPPUNIT_TEST_SUITE_NAMED_REGISTRATION( CredDaoTestCase, "CredDaoTestCase" );

namespace {
const char * const TC_CRED1_NAME     = "CRED_id_1";
const char * const TC_CRED2_NAME     = "CRED_id_2";
const char * const TC_CRED3_NAME     = "CRED_id_3";
const char * const TC_CRED4_NAME     = "CRED_id_4";
const char * const TC_CRED5_NAME     = "CRED_id_5";
const char * const TC_CRED6_NAME     = "CRED_id_6";
const char * const TC_CRED7_NAME     = "CRED_id_7";
const char * const TC_CRED8_NAME     = "CRED_id_8";
const char * const TC_CRED9_NAME     = "CRED_id_9";
const char * const TC_CRED10_NAME    = "CRED_id_10";
const char * const TC_DN1            = "My DN 1";
const char * const TC_DN2            = "My DN 2";
const char * const TC_DN3            = "My DN 3";
const char * const TC_DN4            = "My DN 4";
const char * const TC_DN5            = "My DN 5";
const char * const TC_DN6            = "My DN 6";
const char * const TC_DN7            = "My DN 7";
const char * const TC_DN8            = "My DN 8";
const char * const TC_DN9            = "My DN 9";
const char * const TC_DN10           = "My DN 10";
const char * const TC_VO1            = "CRED_VO_1";
const char * const TC_CHANNEL1       = "CRED_channel_1";

const int LOOP_ITERATIONS = 1000;

} // end global namespace

/*
 * setUp
 *
 * set-up test case
 */
void CredDaoTestCase::setUp(){
    
    try{
        // Create VO CredDAO instance
        m_voCredDAO.reset(
            dao::vo::DAOFactory::instance().createCredDAO(TC_VO1,DaoHelper::ctx()));
        CPPUNIT_ASSERT_MESSAGE("Null VO CredDAO", m_voCredDAO.get() != 0);        
        
        // Create VO CredDAO instance
        m_chCredDAO.reset(
            dao::channel::DAOFactory::instance().createCredDAO(TC_CHANNEL1,DaoHelper::ctx()));
        CPPUNIT_ASSERT_MESSAGE("Null Channel CredDAO", m_chCredDAO.get() != 0);        

        // Create Cred CredDAO instance
        m_credCredDAO.reset(
            dao::cred::DAOFactory::instance().createCredDAO(DaoHelper::ctx()));
        CPPUNIT_ASSERT_MESSAGE("Null Cred CredDAO", m_credCredDAO.get() != 0);        
    } catch(const std::exception& exc){
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        CPPUNIT_FAIL("Got Unknown Error");
    }
}

/*
 * testGet
 *
 * Test Get Method
 */
void CredDaoTestCase::testGet_VO(){
    try{
        // Get Cred instance
        scoped_ptr<Cred> c(m_voCredDAO->get(TC_CRED1_NAME, TC_DN1));
        CPPUNIT_ASSERT_MESSAGE("Null Cred (CRED 1)", c.get() != 0);
        
        // Check the User DN is correct
        CPPUNIT_ASSERT_MESSAGE("Invalid User DN (CRED 1)", c->userDn == TC_DN1);
        CPPUNIT_ASSERT_MESSAGE("Invalid Cred Id (CRED 1)", c->id == TC_CRED1_NAME);
        CPPUNIT_ASSERT_MESSAGE("Invalid Certificate (CRED 1)", !c->certificate().empty());
        CPPUNIT_ASSERT_MESSAGE("Invalid VOMS attr (CRED 1)", !c->vomsAttributes().empty());
    } catch(const std::exception& exc){
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        CPPUNIT_FAIL("Got Unknown Error");
    }
}

/*
 * testGet
 *
 * Test Get Method
 */
void CredDaoTestCase::testGet_VO_Error(){
    try{
        // Get Cred instance
        scoped_ptr<Cred> c(m_voCredDAO->get(TC_CRED1_NAME, TC_DN2));
        CPPUNIT_FAIL("This Cred shouldn't exists");
    } catch(const DAOException& exc){
        // OK
    } catch(const std::exception& exc){
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        CPPUNIT_FAIL("Got Unknown Error");
    }
}

/*
 * testGet
 *
 * Test Get Method
 */
void CredDaoTestCase::testGet_VO_Error_2(){
    try{
        // Get Cred instance
        scoped_ptr<Cred> c(m_voCredDAO->get(TC_CRED2_NAME, TC_DN2));
        CPPUNIT_FAIL("This Cred shouldn't exists");
    } catch(const DAOException& exc){
        // OK
    } catch(const std::exception& exc){
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        CPPUNIT_FAIL("Got Unknown Error");
    }
}


/*
 * testGet
 *
 * Test Get Method
 */
void CredDaoTestCase::testGet_Ch(){
    try{
        // Get Cred instance
        scoped_ptr<Cred> c(m_chCredDAO->get(TC_CRED1_NAME, TC_DN1));
        CPPUNIT_ASSERT_MESSAGE("Null Cred (CRED 1)", c.get() != 0);
        
        // Check the User DN is correct
        CPPUNIT_ASSERT_MESSAGE("Invalid User DN (CRED 1)", c->userDn == TC_DN1);
        CPPUNIT_ASSERT_MESSAGE("Invalid Cred Id (CRED 1)", c->id == TC_CRED1_NAME);
        CPPUNIT_ASSERT_MESSAGE("Invalid Certificate (CRED 1)", !c->certificate().empty());
        CPPUNIT_ASSERT_MESSAGE("Invalid VOMS attr (CRED 1)", !c->vomsAttributes().empty());
    } catch(const std::exception& exc){
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        CPPUNIT_FAIL("Got Unknown Error");
    }
}

/*
 * testGet
 *
 * Test Get Method
 */
void CredDaoTestCase::testGet_Ch_Error(){
    try{
        // Get Cred instance
        scoped_ptr<Cred> c(m_chCredDAO->get(TC_CRED1_NAME, TC_DN2));
        CPPUNIT_FAIL("This Cred shouldn't exists");
    } catch(const DAOException& exc){
        // OK
    } catch(const std::exception& exc){
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        CPPUNIT_FAIL("Got Unknown Error");
    }
}

/*
 * testGet
 *
 * Test Get Method
 */
void CredDaoTestCase::testGet_Cred(){
    try{
        // Get Cred instance
        scoped_ptr<Cred> c(m_credCredDAO->get(TC_CRED1_NAME, TC_DN1));
        CPPUNIT_ASSERT_MESSAGE("Null Cred (CRED 1)", c.get() != 0);
        
        // Check the User DN is correct
        CPPUNIT_ASSERT_MESSAGE("Invalid User DN (CRED 1)", c->userDn == TC_DN1);
        CPPUNIT_ASSERT_MESSAGE("Invalid Cred Id (CRED 1)", c->id == TC_CRED1_NAME);
        CPPUNIT_ASSERT_MESSAGE("Invalid Certificate (CRED 1)", !c->certificate().empty());
        CPPUNIT_ASSERT_MESSAGE("Invalid VOMS attr (CRED 1)", !c->vomsAttributes().empty());
    } catch(const std::exception& exc){
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        CPPUNIT_FAIL("Got Unknown Error");
    }
}

/*
 * testGet
 *
 * Test Get Method
 */
void CredDaoTestCase::testGet_Cred_Error(){
    try{
        // Get Cred instance
        scoped_ptr<Cred> c(m_credCredDAO->get(TC_CRED1_NAME, TC_DN2));
        CPPUNIT_FAIL("This Cred shouldn't exists");
    } catch(const DAOException& exc){
        // OK
    } catch(const std::exception& exc){
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        CPPUNIT_FAIL("Got Unknown Error");
    }
}

/*
 * testGet_stress
 *
 * Test Get Method
 */
void CredDaoTestCase::testGet_Cred_Stress(){
    try{
        
        struct timeval start_time,stop_time;
        long sec_diff,usec_diff;
                        
        // Perform iterations
        gettimeofday(&start_time,0);
        for(int i = 0; i < LOOP_ITERATIONS; ++i){
            // Get Cred instance
            scoped_ptr<Cred> c(m_credCredDAO->get(TC_CRED2_NAME, TC_DN2));
            CPPUNIT_ASSERT_MESSAGE("Null Cred (Cred2)", c.get() != 0);
        }
        gettimeofday(&stop_time,0);
        if(start_time.tv_sec < stop_time.tv_sec){
            if(start_time.tv_usec < stop_time.tv_usec){
                sec_diff  = stop_time.tv_sec  - start_time.tv_sec;
                usec_diff = stop_time.tv_usec - start_time.tv_usec;
            } else {
                sec_diff  = stop_time.tv_sec  - start_time.tv_sec - 1;
                usec_diff = stop_time.tv_usec + 1000000 - start_time.tv_usec;
            }
        } else {
            sec_diff = 0;
            usec_diff = stop_time.tv_usec - start_time.tv_usec;
        }
        printf("Cred Get %d iterations in %d secs and %d usec\n",(int)LOOP_ITERATIONS,(int)sec_diff,(int)usec_diff);
    } catch(const std::exception& exc){
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        CPPUNIT_FAIL("Got Unknown Error");
    }
}

/*
 * testUpdate
 *
 * Test Update Method
 */
void CredDaoTestCase::testUpdate_Cred(){
    try{
        // Get Cred instance
        scoped_ptr<Cred> c(m_credCredDAO->get(TC_CRED1_NAME, TC_DN1));
        CPPUNIT_ASSERT_MESSAGE("Null Cred           (1)", c.get() != 0);
        CPPUNIT_ASSERT_MESSAGE("Invalid User DN     (1)", c->userDn == TC_DN1);
        CPPUNIT_ASSERT_MESSAGE("Invalid Cred Id     (1)", c->id == TC_CRED1_NAME);
        CPPUNIT_ASSERT_MESSAGE("Invalid Certificate (1)", !c->certificate().empty());
        CPPUNIT_ASSERT_MESSAGE("Invalid VOMS attr   (1)", !c->vomsAttributes().empty());
        
        // Store Old properties
        std::string proxy = c->certificate();
        time_t validity   = c->expirationTime();
        time_t current;
        time(&current);
        
        DaoHelper::ctx().start();

        // Update the information        
        c->certificate    = "This is a fake proxy";
        c->expirationTime = current + 10;
        m_credCredDAO->update(*c);
        
        DaoHelper::ctx().commit();

        // Get Again the instance
        c.reset(m_credCredDAO->get(TC_CRED1_NAME, TC_DN1));
        CPPUNIT_ASSERT_MESSAGE("Null Cred (2)", c.get() != 0);
        CPPUNIT_ASSERT_MESSAGE("Invalid User DN (2)", c->userDn == TC_DN1);
        CPPUNIT_ASSERT_MESSAGE("Invalid Cred Id (2)", c->id == TC_CRED1_NAME);
        CPPUNIT_ASSERT_MESSAGE("Invalid Certificate (2)", c->certificate == "This is a fake proxy");
        CPPUNIT_ASSERT_MESSAGE("Invalid VOMS attr   (2)", !c->vomsAttributes().empty());
        CPPUNIT_ASSERT_MESSAGE("Invalid Expiration  (2)", c->expirationTime == current + 10);
        
        DaoHelper::ctx().start();

        // Reset to the previous values
        c->certificate    = proxy;
        c->expirationTime = validity;
        m_credCredDAO->update(*c);
        
        DaoHelper::ctx().commit();

        // Get Again the instance
        c.reset(m_credCredDAO->get(TC_CRED1_NAME, TC_DN1));
        CPPUNIT_ASSERT_MESSAGE("Null Cred           (3)", c.get() != 0);
        CPPUNIT_ASSERT_MESSAGE("Invalid User DN     (3)", c->userDn == TC_DN1);
        CPPUNIT_ASSERT_MESSAGE("Invalid Cred Id     (3)", c->id == TC_CRED1_NAME);
        CPPUNIT_ASSERT_MESSAGE("Invalid Certificate (3)", c->certificate == proxy);
        CPPUNIT_ASSERT_MESSAGE("Invalid VOMS attr   (3)", !c->vomsAttributes().empty());
        CPPUNIT_ASSERT_MESSAGE("Invalid Expiration  (3)", c->expirationTime == validity);

    } catch(const std::exception& exc){
        DaoHelper::ctx().rollback();
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        DaoHelper::ctx().rollback();
        CPPUNIT_FAIL("Got Unknown Error");
    }
}


/*
 * testGetExpiring
 *
 * Test GetExpiring Method
 */
void CredDaoTestCase::testGetExpiring_Cred(){
    try {
        // Get all Creds that will expire within 1 hour
        unsigned int validity = 3600;
        dao::cred::CredDAO::StringPairArray ids;
        m_credCredDAO->getExpiringCreds(ids,validity);
        CPPUNIT_ASSERT_MESSAGE("Empty Result    (1)"  , !ids.empty());
        CPPUNIT_ASSERT_MESSAGE("Invalid number  (1)"  , ids.size() == 4);
        CPPUNIT_ASSERT_MESSAGE("Invalid CredID  (1-0)", ids.at(0).first  == TC_CRED4_NAME);
        CPPUNIT_ASSERT_MESSAGE("Invalid UserDN  (1-0)", ids.at(0).second == TC_DN4);
        CPPUNIT_ASSERT_MESSAGE("Invalid CredID  (1-1)", ids.at(1).first  == TC_CRED9_NAME);
        CPPUNIT_ASSERT_MESSAGE("Invalid UserDN  (1-1)", ids.at(1).second == TC_DN9);
        CPPUNIT_ASSERT_MESSAGE("Invalid CredID  (1-2)", ids.at(2).first  == TC_CRED1_NAME);
        CPPUNIT_ASSERT_MESSAGE("Invalid UserDN  (1-2)", ids.at(2).second == TC_DN1);
        CPPUNIT_ASSERT_MESSAGE("Invalid CredID  (1-3)", ids.at(3).first  == TC_CRED2_NAME);
        CPPUNIT_ASSERT_MESSAGE("Invalid UserDN  (1-3)", ids.at(3).second == TC_DN2);
        
        // Get all Creds that will expire within 25 minutes second
        validity = 1500;
        ids.clear();
        m_credCredDAO->getExpiringCreds(ids,validity);
        CPPUNIT_ASSERT_MESSAGE("Empty Result    (2)"  , !ids.empty());
        CPPUNIT_ASSERT_MESSAGE("Invalid number  (2)"  , ids.size() == 1);
        CPPUNIT_ASSERT_MESSAGE("Invalid CredID  (2-0)", ids.at(0).first  == TC_CRED4_NAME);
        CPPUNIT_ASSERT_MESSAGE("Invalid UserDN  (2-0)", ids.at(0).second == TC_DN4);
        
        // Get all Creds that will expire within 1 day
        validity = 86400;
        ids.clear();
        m_credCredDAO->getExpiringCreds(ids,validity);
        CPPUNIT_ASSERT_MESSAGE("Empty Result    (3)"  , !ids.empty());
        CPPUNIT_ASSERT_MESSAGE("Invalid number  (3)"  , ids.size() == 6);
        CPPUNIT_ASSERT_MESSAGE("Invalid CredID  (3-0)", ids.at(0).first  == TC_CRED4_NAME);
        CPPUNIT_ASSERT_MESSAGE("Invalid UserDN  (3-0)", ids.at(0).second == TC_DN4);
        CPPUNIT_ASSERT_MESSAGE("Invalid CredID  (3-1)", ids.at(1).first  == TC_CRED9_NAME);
        CPPUNIT_ASSERT_MESSAGE("Invalid UserDN  (3-1)", ids.at(1).second == TC_DN9);
        CPPUNIT_ASSERT_MESSAGE("Invalid CredID  (3-2)", ids.at(2).first  == TC_CRED1_NAME);
        CPPUNIT_ASSERT_MESSAGE("Invalid UserDN  (3-2)", ids.at(2).second == TC_DN1);
        CPPUNIT_ASSERT_MESSAGE("Invalid CredID  (3-3)", ids.at(3).first  == TC_CRED2_NAME);
        CPPUNIT_ASSERT_MESSAGE("Invalid UserDN  (3-3)", ids.at(3).second == TC_DN2);
        CPPUNIT_ASSERT_MESSAGE("Invalid CredID  (3-4)", ids.at(4).first  == TC_CRED3_NAME);
        CPPUNIT_ASSERT_MESSAGE("Invalid UserDN  (3-4)", ids.at(4).second == TC_DN3);
        CPPUNIT_ASSERT_MESSAGE("Invalid CredID  (3-5)", ids.at(5).first  == TC_CRED10_NAME);
        CPPUNIT_ASSERT_MESSAGE("Invalid UserDN  (3-5)", ids.at(5).second == TC_DN10);

        // Get all Creds that will expire within 1 day, limited search
        validity = 86400;
        ids.clear();
        m_credCredDAO->getExpiringCreds(ids,validity,3);
        CPPUNIT_ASSERT_MESSAGE("Empty Result    (4)"  , !ids.empty());
        CPPUNIT_ASSERT_MESSAGE("Invalid number  (4)"  , ids.size() == 3);
        CPPUNIT_ASSERT_MESSAGE("Invalid CredID  (4-0)", ids.at(0).first  == TC_CRED4_NAME);
        CPPUNIT_ASSERT_MESSAGE("Invalid UserDN  (4-0)", ids.at(0).second == TC_DN4);
        CPPUNIT_ASSERT_MESSAGE("Invalid CredID  (4-1)", ids.at(1).first  == TC_CRED9_NAME);
        CPPUNIT_ASSERT_MESSAGE("Invalid UserDN  (4-1)", ids.at(1).second == TC_DN9);
        CPPUNIT_ASSERT_MESSAGE("Invalid CredID  (4-2)", ids.at(2).first  == TC_CRED1_NAME);
        CPPUNIT_ASSERT_MESSAGE("Invalid UserDN  (4-2)", ids.at(2).second == TC_DN1);

    } catch(const std::exception& exc){
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        CPPUNIT_FAIL("Got Unknown Error");
    }
}
