/**
 *  Copyright (c) Members of the EGEE Collaboration. 2004-2008.
 *  See http://www.eu-egee.org/partners/ for details on the copyright
 *  holders.
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 *  GetShareTestCase
 *
 *  Authors: Paolo Tedesco <paolo.tedesco@cern.ch>
 *  Version info: $Id: GetShareTestCase.cpp,v 1.3 2008/12/11 13:24:44 ptedesco Exp $
 *  Release: $Name:  $
 *
 */
#include "DaoHelper.h"
#include <glite/data/transfer/agent/model/Channel.h>
#include <glite/data/transfer/agent/dao/vo/ChannelDAO.h>
#include <glite/data/transfer/agent/dao/channel/ChannelDAO.h>
#include <glite/data/agents/dao/DAOException.h>
#include <glite/data/transfer/agent/dao/vo/DAOFactory.h>
#include <glite/data/transfer/agent/dao/channel/DAOFactory.h>
#include <glite/data/agents/dao/DAOContextFactory.h>
#include "dao-utility.h"

#include <boost/scoped_ptr.hpp>
#include <cppunit/TestCase.h>
#include <cppunit/extensions/HelperMacros.h>

using namespace glite::data::transfer::agent;
using namespace glite::data::transfer::agent::model;
using glite::data::agents::dao::DAOContextFactory;
using glite::data::agents::dao::DAOException;
using glite::data::agents::longlong;
using glite::data::agents::dao::StringArray;

using boost::scoped_ptr;

/**
 * GetShareTestCase
 */
class GetShareTestCase : public CppUnit::TestCase{
public:
    void setUp();
private:
    // Declare CPPUnit Methods
    CPPUNIT_TEST_SUITE( GetShareTestCase );
        CPPUNIT_TEST( test_01 );
        CPPUNIT_TEST( test_02 );
        CPPUNIT_TEST( test_03 );
        CPPUNIT_TEST( test_04 );
        CPPUNIT_TEST( test_05 );
        CPPUNIT_TEST( test_06 );
        CPPUNIT_TEST( test_07 );
        CPPUNIT_TEST( test_08 );
        CPPUNIT_TEST( test_09 );
        CPPUNIT_TEST( test_10 );
        CPPUNIT_TEST( test_11 );
        CPPUNIT_TEST( test_12 );
    CPPUNIT_TEST_SUITE_END();
    
    void test_01();
    void test_02();
    void test_03();
    void test_04();
    void test_05();
    void test_06();
    void test_07();
    void test_08();
    void test_09();
    void test_10();
    void test_11();
    void test_12();
    
    scoped_ptr<dao::channel::ChannelDAO> m_channel_1;
    scoped_ptr<dao::channel::ChannelDAO> m_channel_2;
    scoped_ptr<dao::channel::ChannelDAO> m_channel_3;
    scoped_ptr<dao::channel::ChannelDAO> m_channel_4;
    scoped_ptr<dao::channel::ChannelDAO> m_channel_5;
};

CPPUNIT_TEST_SUITE_NAMED_REGISTRATION( GetShareTestCase, "GetShareTestCase" );

void GetShareTestCase::setUp(){
    // Create ChannelDAO instances
    m_channel_1.reset(dao::channel::DAOFactory::instance().createChannelDAO("GETSHARE_channel_1", DaoHelper::ctx()));
    CPPUNIT_ASSERT_MESSAGE("createChannelDAO (1) failed", m_channel_1.get() != 0);
    m_channel_2.reset(dao::channel::DAOFactory::instance().createChannelDAO("GETSHARE_channel_2", DaoHelper::ctx()));
    CPPUNIT_ASSERT_MESSAGE("createChannelDAO (2) failed", m_channel_2.get() != 0);
    m_channel_3.reset(dao::channel::DAOFactory::instance().createChannelDAO("GETSHARE_channel_3", DaoHelper::ctx()));
    CPPUNIT_ASSERT_MESSAGE("createChannelDAO (3) failed", m_channel_3.get() != 0);
    m_channel_4.reset(dao::channel::DAOFactory::instance().createChannelDAO("GETSHARE_channel_4", DaoHelper::ctx()));
    CPPUNIT_ASSERT_MESSAGE("createChannelDAO (4) failed", m_channel_4.get() != 0);
    m_channel_5.reset(dao::channel::DAOFactory::instance().createChannelDAO("GETSHARE_channel_5", DaoHelper::ctx()));
    CPPUNIT_ASSERT_MESSAGE("createChannelDAO (5) failed", m_channel_5.get() != 0);
}

namespace{
    double abs(double t){
        return (t >= 0) ? t : -t;
    }
}

#define CHECK_SHARE(val,exp)\
    if (abs(val - exp) > 0.00001){\
        std::stringstream ss;\
        ss << "Wrong value for " << #val << ": got " << val << " expected " << exp;\
        CPPUNIT_FAIL(ss.str().c_str());\
    }

void GetShareTestCase::test_01(){
    // shares are percentages. getShare is expected to return a double 
    // with the percentage (10 -> 0.1)
    // false,false -> get absolute share
    double share_1 = m_channel_1->getShare("GETSHARE_VO_1",false,false);
    CHECK_SHARE(share_1,0.1);
}

void GetShareTestCase::test_02(){
    // true,false -> get normalized share. We have two VOs with a share
    // of 10%, so each one gets 50% (0.5)
    double share_1 = m_channel_1->getShare("GETSHARE_VO_1",true,false);
    CHECK_SHARE(share_1,0.5);
    double share_2 = m_channel_1->getShare("GETSHARE_VO_2",true,false);
    CHECK_SHARE(share_2,0.5);
}

void GetShareTestCase::test_03(){
    // The getshare query does not consider VO limits
    double share_1 = m_channel_2->getShare("GETSHARE_VO_1",true,false);
    CHECK_SHARE(share_1,0.5);
    double share_2 = m_channel_2->getShare("GETSHARE_VO_2",true,false);
    CHECK_SHARE(share_2,0.5);
}

void GetShareTestCase::test_04(){
    // getshare query keeps into account the VO state (VO3 here is 'PAUSE' -> no fetch)
    // normalized shares don't take into account paused VOs
    double share_1 = m_channel_3->getShare("GETSHARE_VO_1",true,false);
    CHECK_SHARE(share_1,0.5);
    double share_2 = m_channel_3->getShare("GETSHARE_VO_2",true,false);
    CHECK_SHARE(share_2,0.5);
    double share_3 = m_channel_3->getShare("GETSHARE_VO_3",true,false);
    CHECK_SHARE(share_3,0);
}

void GetShareTestCase::test_05(){
    // getshare query keeps into account the VO state :
    // VO1 -> ON
    // VO2 -> OFF -> no allocation, but share does not change
    // VO3 -> PAUSE -> no fetch, share = 0
    double share_1 = m_channel_3->getShare("GETSHARE_VO_1",false,false);
    CHECK_SHARE(share_1,0.1);
    double share_2 = m_channel_3->getShare("GETSHARE_VO_2",false,false);
    CHECK_SHARE(share_2,0.1);
    double share_3 = m_channel_3->getShare("GETSHARE_VO_3",false,false);
    CHECK_SHARE(share_3,0);
}

void GetShareTestCase::test_06(){
    // 'Normalized on active' shares: only VOs with files in 'ready' state get
    // a share (here VO1; VO2 has an active file and needs no fetching).
    // NOTE: the NUMBER of files in ready state doesn't matter, it matters
    // only if there's at least one or not
    double share_1 = m_channel_1->getShare("GETSHARE_VO_1",true,true);
    CHECK_SHARE(share_1,1);
    double share_2 = m_channel_1->getShare("GETSHARE_VO_2",true,true);
    CHECK_SHARE(share_2,0);
}

void GetShareTestCase::test_07(){
    // only VOs with active (ready) jobs and normalized = false
    // sets to 0 the shares of VOs without ready files, returns the absolute 
    // shares for the other VOs.
    double share_1 = m_channel_1->getShare("GETSHARE_VO_1",false,true);
    CHECK_SHARE(share_1,0.1);
    double share_2 = m_channel_1->getShare("GETSHARE_VO_2",false,true);
    CHECK_SHARE(share_2,0);
}

void GetShareTestCase::test_08(){
    // fix bug 39821
    double share_1 = m_channel_2->getShare("GETSHARE_VO_1",true,true);
    CHECK_SHARE(share_1,0.5);
    double share_2 = m_channel_2->getShare("GETSHARE_VO_2",true,true);
    CHECK_SHARE(share_2,0.5);
}

void GetShareTestCase::test_09(){
    // normalized-on-active: vo limits ARE taken into account!!! (bug 39821)
    // here the limit exceeds the share, so the effect is not noticed...
    double share_1 = m_channel_3->getShare("GETSHARE_VO_1",true,true);
    CHECK_SHARE(share_1,0.5);
    double share_2 = m_channel_3->getShare("GETSHARE_VO_2",true,true);
    CHECK_SHARE(share_2,0.5);
    double share_3 = m_channel_3->getShare("GETSHARE_VO_3",true,true);
    CHECK_SHARE(share_3,0);
}

void GetShareTestCase::test_10(){
    // for the normalized on active, the number of jobs and files in ready state
    // does not matter; it matters only if there is at least one file in ready 
    // state or not
    double share_1 = m_channel_4->getShare("GETSHARE_VO_1",true,true);
    CHECK_SHARE(share_1,0);
    double share_2 = m_channel_4->getShare("GETSHARE_VO_2",true,true);
    CHECK_SHARE(share_2,0);
    double share_3 = m_channel_4->getShare("GETSHARE_VO_3",true,true);
    CHECK_SHARE(share_3,0.5);
    double share_4 = m_channel_4->getShare("GETSHARE_VO_4",true,true);
    CHECK_SHARE(share_4,0.5);
}

void GetShareTestCase::test_11(){
    // Testing the DENY VO state
    double share_1 = m_channel_5->getShare("GETSHARE_VO_1",false,false);
    CHECK_SHARE(share_1,0.25);
    double share_2 = m_channel_5->getShare("GETSHARE_VO_2",false,false);
    CHECK_SHARE(share_2,0.25);
    double share_3 = m_channel_5->getShare("GETSHARE_VO_3",false,false);
    CHECK_SHARE(share_3,0);
    double share_4 = m_channel_5->getShare("GETSHARE_VO_4",false,false);
    CHECK_SHARE(share_4,0.25);
}

void GetShareTestCase::test_12(){
    // Testing the DENY VO state
    double share_1 = m_channel_5->getShare("GETSHARE_VO_1",true,false);
    CHECK_SHARE(share_1,1./3.);
    double share_2 = m_channel_5->getShare("GETSHARE_VO_2",true,false);
    CHECK_SHARE(share_2,1./3.);
    double share_3 = m_channel_5->getShare("GETSHARE_VO_3",true,false);
    CHECK_SHARE(share_3,0);
    double share_4 = m_channel_5->getShare("GETSHARE_VO_4",true,false);
    CHECK_SHARE(share_4,1./3.);
}

