/*
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *  GLite Data Transfer Agents: main function
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *  Version info: $Id: TestDao.cpp,v 1.16 2008/08/07 07:51:47 ptedesco Exp $
 *  Release: $Name:  $
 *
 */
#include "DaoHelper.h"
#include "glite/data/config/service/glite-config.h"
#include "glite/data/config/service/glite-config-register.hxx"

#include <stdlib.h>
#include <string>
#include <iostream>
#include <unistd.h>
#include <cppunit/extensions/TestFactoryRegistry.h>
#include <cppunit/ui/text/TestRunner.h>
#include <cppunit/CompilerOutputter.h>

using std::cout;
using std::cerr;
using std::cin;
using std::endl;

namespace {
const char * const CONFIG_NAME = "glite-transfer-agent-test-dao";
const char * const PROGNAME    = "glite_data_transfer_agent_test_dao";
const char * const PROGVERSION = "0.0";
const char * const PROGAUTHOR  = "(C) EGEE";
}

/*
 * print_usage_and_die
 *
 * Print Program Usage and ther exit
 */
void print_usage_and_die(){
    cout << endl;
    cout << "<" << PROGNAME << "> Version " << PROGVERSION << " by " << PROGAUTHOR;
    cout << endl;
    cout << "usage: " << PROGNAME << "[-l <glite_location_user>] [-s] " << endl;
    cout << " Where:" << endl;
    cout << " -l : specify the glite location. Configuration files should be located under the etc folder on that path" << endl;
    cout << " -s : Enable the Compiler Outputter" << endl;
    cout << " -c : configuration" << endl;
    cout << " Where:" << endl;
    cout << " Optional parameters:" << endl;
    cout << "  -h : print this screen" << endl;
    exit(-1);
}

/*
 * suite
 *
 * Instantiate Test Suite
 */
CppUnit::Test * suite() {

    CppUnit::TestFactoryRegistry& registry = CppUnit::TestFactoryRegistry::getRegistry();
    CppUnit::TestFactoryRegistry& fileDaoTestCase     = CppUnit::TestFactoryRegistry::getRegistry("FileDaoTestCase");
    CppUnit::TestFactoryRegistry& jobDaoTestCase      = CppUnit::TestFactoryRegistry::getRegistry("JobDaoTestCase");
    CppUnit::TestFactoryRegistry& channelDaoTestCase  = CppUnit::TestFactoryRegistry::getRegistry("ChannelDaoTestCase");
    CppUnit::TestFactoryRegistry& transferDaoTestCase = CppUnit::TestFactoryRegistry::getRegistry("TransferDaoTestCase");
    CppUnit::TestFactoryRegistry& stagereqDaoTestCase = CppUnit::TestFactoryRegistry::getRegistry("StageReqDaoTestCase");
    CppUnit::TestFactoryRegistry& agentDaoTestCase    = CppUnit::TestFactoryRegistry::getRegistry("AgentDaoTestCase");
    CppUnit::TestFactoryRegistry& credDaoTestCase     = CppUnit::TestFactoryRegistry::getRegistry("CredDaoTestCase");
    CppUnit::TestFactoryRegistry& getShareTestCase    = CppUnit::TestFactoryRegistry::getRegistry("GetShareTestCase");
    CppUnit::TestFactoryRegistry& testAgentStatements = CppUnit::TestFactoryRegistry::getRegistry("TestAgentStatements");
    registry.registerFactory(&fileDaoTestCase);
    registry.registerFactory(&jobDaoTestCase);
    registry.registerFactory(&channelDaoTestCase);
    registry.registerFactory(&transferDaoTestCase);
    registry.registerFactory(&stagereqDaoTestCase);
    registry.registerFactory(&agentDaoTestCase);
    registry.registerFactory(&credDaoTestCase);
    registry.registerFactory(&getShareTestCase);
    registry.registerFactory(&testAgentStatements);
    return registry.makeTest();
}


/*
 * main
 *
 * Transfer FTS Main function
 */
int main(int argc, char* argv[]) {

    // Get Options
    const char * location = 0;
    const char * config   = CONFIG_NAME;
    bool selfTest = false;
    int flag;
    while ((flag = getopt (argc, argv, "c:l:sh")) != -1) {
        switch (flag) {
            case 'c':
                config = optarg;
            break;
            case 'l':
                location = optarg;
            break;
            case 's':
                selfTest = true;
            break;
            case 'h':
            default:
                print_usage_and_die();
            break;
        } // End switch
    } // End while

    if(0 != location){
        // Set "GLite Location User"
        setenv("GLITE_LOCATION_USER",location,1);
    }
    
    // Initialize the Application
    int initres = glite_config_initialize(config);
    if(initres < 0){
        cerr << "Initialization failed " << endl;
        return -1;
    }
    
    CppUnit::TextUi::TestRunner runner;
    runner.addTest(suite());   // Add the suite to the test runner

    if(true == selfTest){ 
        // Change the default outputter to a compiler error format outputter
        // The test runner owns the new outputter.
        runner.setOutputter( 
           CppUnit::CompilerOutputter::defaultOutputter(&runner.result(),cerr));
    }
          
    // Run the test
    bool wasSucessful = runner.run();
    if(!wasSucessful){
        cerr << "Some errors occured during the test" << endl;
    }
    
    DaoHelper::dispose();
    
    // Finalize the Application
    glite_config_finalize();
    
    return wasSucessful ? 0 : 1;
}
