/*
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *  GLite Data Transfer Agents: TransferDAO Test Case
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>, Paolo Tedesco <paolo.tedesco@cern.ch>
 *  Version info: $Id: TransferDaoTestCase.cpp,v 1.18 2008/04/23 15:10:15 ptedesco Exp $
 *  Release: $Name:  $
 *
 */
#include "DaoHelper.h"
#include "glite/data/transfer/agent/model/Transfer.h"
#include "glite/data/transfer/agent/dao/vo/TransferDAO.h"
#include "glite/data/transfer/agent/dao/channel/TransferDAO.h"
#include "glite/data/agents/dao/DAOException.h"
#include "glite/data/transfer/agent/dao/vo/DAOFactory.h"
#include "glite/data/transfer/agent/dao/channel/DAOFactory.h"

#include <boost/scoped_ptr.hpp>
#include <cppunit/TestCase.h>
#include <cppunit/extensions/HelperMacros.h>
#include <cstring>

using namespace glite::data::transfer::agent;
using namespace glite::data::transfer::agent::model;
using glite::data::agents::dao::DAOException;
using glite::data::agents::longlong;
using glite::data::agents::dao::StringArray;

using boost::scoped_ptr;

/**
 * CPPUnit TransferDaoTestCase
 */
class TransferDaoTestCase : public CppUnit::TestCase{
public:
    /**
     * set-up test case
     */
    void setUp ();
private:
    // Declare CPPUnit Methods
    // -------------------------------------------------------------------------
    CPPUNIT_TEST_SUITE( TransferDaoTestCase);
    // VO View
    CPPUNIT_TEST( testGet_VO );
    CPPUNIT_TEST( testGet_VO_Error );
    CPPUNIT_TEST( testGet_VO_3 );
    CPPUNIT_TEST( testGetByFileId_VO );
    CPPUNIT_TEST( testGetByFileId_VO_2 );
    CPPUNIT_TEST( testGetByFileId_VO_3 );
    CPPUNIT_TEST( testGetByFileId_VO_Error );
    CPPUNIT_TEST( testGetByRequestId_VO );
    CPPUNIT_TEST( testGetByRequestId_VO_Error );
    // Channel View
    CPPUNIT_TEST( testGet_Ch );
    CPPUNIT_TEST( testGet_Ch_Error );
    CPPUNIT_TEST( testGet_Ch_2 );
    CPPUNIT_TEST( testUpdate_Ch );
    CPPUNIT_TEST( testUpdate_Ch_2 );
    CPPUNIT_TEST( testUpdate_Ch_3 );
    CPPUNIT_TEST( testUpdate_Ch_4 );
    CPPUNIT_TEST( testUpdate_Ch_5 );
    CPPUNIT_TEST( testUpdate_Ch_6 );
    CPPUNIT_TEST( testUpdate_reason );
    CPPUNIT_TEST( testUpdate_Ch_Error );
    CPPUNIT_TEST( testGetByFileId_Ch );
    CPPUNIT_TEST( testGetByFileId_Ch_2 );
    CPPUNIT_TEST( testGetByFileId_Ch_3 );
    CPPUNIT_TEST( testGetByFileId_Ch_Error );
    CPPUNIT_TEST( testGetByRequestId_Ch );
    CPPUNIT_TEST( testGetByRequestId_Ch_Error );
    CPPUNIT_TEST( testCreate_Ch );
    CPPUNIT_TEST( testCountActiveTransfers_Ch );
    CPPUNIT_TEST( testCountActiveTransfers_Ch_2 );
    CPPUNIT_TEST( testGetActiveTransfers_Ch );
    // Vo View (after channel modification)
    CPPUNIT_TEST( testGet_VO_2 );
    // bug specific tests
    CPPUNIT_TEST( test_ggus_32512 );
    CPPUNIT_TEST_SUITE_END();

    // VO Test Methods
    // -------------------------------------------------------------------------

    /**
     * Test Get Method
     */
    void testGet_VO();
    void testGet_VO_2();
    void testGet_VO_Error();
    void testGet_VO_3();
    /**
     * Test GetByFileID Method
     */
    void testGetByFileId_VO();
    void testGetByFileId_VO_2();
    void testGetByFileId_VO_3();
    void testGetByFileId_VO_Error();
    /**
     * Test GetBySites Method
     */
    void testGetByRequestId_VO();    
    void testGetByRequestId_VO_Error();    

    // Channel Test Methods
    // -------------------------------------------------------------------------

    /**
     * Test Get Method
     */
    void testGet_Ch();
    void testGet_Ch_Error();
    void testGet_Ch_2();
    /**
     * Test Update Method
     */
    void testUpdate_Ch();
    void testUpdate_reason();
    void testUpdate_Ch_2();
    void testUpdate_Ch_3();
    void testUpdate_Ch_4();
    void testUpdate_Ch_5();
    void testUpdate_Ch_Error();
    void testUpdate_Ch_6();
    /**
     * Test GetByFileID Method
     */
    void testGetByFileId_Ch();
    void testGetByFileId_Ch_2();
    void testGetByFileId_Ch_3();
    void testGetByFileId_Ch_Error();
    /**
     * Test GetBySites Method
     */
    void testGetByRequestId_Ch();    
    void testGetByRequestId_Ch_Error();    
    /**
     * Test Create Method
     */
    void testCreate_Ch();
    void testCreate_Ch_2();
    /**
     * Test CountActiveTransfer Method
     */
    void testCountActiveTransfers_Ch();
    void testCountActiveTransfers_Ch_2();
    /**
     * Test GetActiveTransfers Method
     */
    void testGetActiveTransfers_Ch();
    
    /**
     * bug specific tests
     */
    void test_ggus_32512();
    
private:
    /**
     * The VO FileDAO Instance
     */
    scoped_ptr<dao::vo::TransferDAO> m_voTransferDAO;
    /**
     * The Transfer FileDAO Instance
     */
    scoped_ptr<dao::channel::TransferDAO> m_chTransferDAO;
};

/*
 * Register Suit
 */
CPPUNIT_TEST_SUITE_NAMED_REGISTRATION( TransferDaoTestCase, "TransferDaoTestCase" );

namespace {
const char * const TRANSFERDAO_TESTCASE_JOB1_ID        = "20000000-0000-0000-0000-000000000001";
const char * const TRANSFERDAO_TESTCASE_JOB2_ID        = "20000000-0000-0000-0000-000000000002";
const char * const TRANSFERDAO_TESTCASE_JOB3_ID        = "20000000-0000-0000-0000-000000000003";
const char * const TRANSFERDAO_TESTCASE_JOB4_ID        = "20000000-0000-0000-0000-000000000004";
const char * const TRANSFERDAO_TESTCASE_JOB5_ID        = "20000000-0000-0000-0000-000000000005";
const char * const TRANSFERDAO_TESTCASE_JOB6_ID        = "20000000-0000-0000-0000-000000000006";
const char * const TRANSFERDAO_TESTCASE_JOB7_ID        = "20000000-0000-0000-0000-000000000007";
const char * const TRANSFERDAO_TESTCASE_JOB8_ID        = "20000000-0000-0000-0000-000000000008";
const char * const TRANSFERDAO_TESTCASE_FILE1_ID       = "30000001";
const char * const TRANSFERDAO_TESTCASE_FILE2_ID       = "30000002";
const char * const TRANSFERDAO_TESTCASE_FILE3_ID       = "30000003";
const char * const TRANSFERDAO_TESTCASE_FILE4_ID       = "30000004";
const char * const TRANSFERDAO_TESTCASE_FILE5_ID       = "30000005";
const char * const TRANSFERDAO_TESTCASE_FILE6_ID       = "30000006";
const char * const TRANSFERDAO_TESTCASE_FILE7_ID       = "30000007";
const char * const TRANSFERDAO_TESTCASE_FILE8_ID       = "30000008";
const char * const TRANSFERDAO_TESTCASE_FILE9_ID       = "30000009";
const char * const TRANSFERDAO_TESTCASE_FILE_NEW_ID    = "30001000";

const char * const TRANSFERDAO_TESTCASE_FILE604_ID     = "60000004";
    
const char * const TRANSFERDAO_TESTCASE_REQ1_ID        = "req_3_1";
const char * const TRANSFERDAO_TESTCASE_REQ2_ID        = "req_3_2";
const char * const TRANSFERDAO_TESTCASE_REQ3_ID        = "req_3_3";
const char * const TRANSFERDAO_TESTCASE_REQ4_ID        = "req_3_4";
const char * const TRANSFERDAO_TESTCASE_REQ5_ID        = "req_3_5";
const char * const TRANSFERDAO_TESTCASE_REQ6_ID        = "req_3_6";
const char * const TRANSFERDAO_TESTCASE_REQ7_ID        = "req_3_7";
const char * const TRANSFERDAO_TESTCASE_REQ_NEW_ID     = "req_3_new";

const char * const TRANSFERDAO_TESTCASE_REQ8_ID        = "req_5_1";
const char * const TRANSFERDAO_TESTCASE_REQ9_ID        = "req_5_2";
const char * const TRANSFERDAO_TESTCASE_REQ10_ID       = "req_5_3";

const char * const TRANSFERDAO_TESTCASE_VO1_NAME       = "TRANSFER_VO_1";
const char * const TRANSFERDAO_TESTCASE_VO2_NAME       = "TRANSFER_VO_2";
const char * const TRANSFERDAO_TESTCASE_VO3_NAME       = "TRANSFER_VO_3";
const char * const TRANSFERDAO_TESTCASE_FILE1_SOURCE   = "file:///tmp/src.txt";
const char * const TRANSFERDAO_TESTCASE_FILE1_DEST     = "file:///tmp/dst.txt";
const char * const TRANSFERDAO_TESTCASE_CHANNEL1_NAME  = "TRANSFER_channel_1";
const char * const TRANSFERDAO_TESTCASE_CHANNEL2_NAME  = "TRANSFER_channel_2";
}

/*
 * setUp
 *
 * set-up test case
 */
void TransferDaoTestCase::setUp(){
    
    try{
        // Create VO TransferDAO instance
        m_voTransferDAO.reset(
            dao::vo::DAOFactory::instance().createTransferDAO(TRANSFERDAO_TESTCASE_VO1_NAME, DaoHelper::ctx()));
        CPPUNIT_ASSERT_MESSAGE("Null VO TransferDAO", m_voTransferDAO.get() != 0);

        // Create Channel TransferDAO instance
        m_chTransferDAO.reset(
            dao::channel::DAOFactory::instance().createTransferDAO(TRANSFERDAO_TESTCASE_CHANNEL1_NAME, DaoHelper::ctx()));
        CPPUNIT_ASSERT_MESSAGE("Null Channel TransferDAO", m_chTransferDAO.get() != 0);
    } catch(const std::exception& exc){
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        CPPUNIT_FAIL("Got Unknown Error");
    }
}

/*
 * testGet
 *
 * Test Get Method
 */
void TransferDaoTestCase::testGet_VO(){
    try{
        // Get Transfer instance
        scoped_ptr<model::Transfer> t(m_voTransferDAO->get(TRANSFERDAO_TESTCASE_FILE1_ID,TRANSFERDAO_TESTCASE_REQ1_ID));
        CPPUNIT_ASSERT_MESSAGE("Null Transfer (TRANSFER1)", t.get() != 0);
        
        // Check Some Precondition
        CPPUNIT_ASSERT_MESSAGE("Invalid Source TURL (TRANSFER1)", t->sourceTurl == TRANSFERDAO_TESTCASE_FILE1_SOURCE);
        CPPUNIT_ASSERT_MESSAGE("Invalid Dest TURL (TRANSFER1)", t->destTurl == TRANSFERDAO_TESTCASE_FILE1_DEST);
        CPPUNIT_ASSERT_MESSAGE("Invalid State (TRANSFER1)", t->state == Transfer::S_PROCESSING);

        // Get Transfer instance
        t.reset(m_voTransferDAO->get(TRANSFERDAO_TESTCASE_FILE2_ID,TRANSFERDAO_TESTCASE_REQ1_ID));
        CPPUNIT_ASSERT_MESSAGE("Null Transfer (TRANSFER2)", t.get() != 0);
        // Check Some Precondition
        CPPUNIT_ASSERT_MESSAGE("Invalid State (TRANSFER2)", t->state == Transfer::S_COMPLETED);
    } catch(const std::exception& exc){
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        CPPUNIT_FAIL("Got Unknown Error");
    }
}

/*
 * testGet
 *
 * Test Get Method
 */
void TransferDaoTestCase::testGet_VO_2(){
    try{
        // Get Transfer instance
        scoped_ptr<model::Transfer> t(m_voTransferDAO->get(TRANSFERDAO_TESTCASE_FILE9_ID,TRANSFERDAO_TESTCASE_REQ7_ID));
        CPPUNIT_ASSERT_MESSAGE("Null Transfer (TRANSFER7)", t.get() != 0);
        
        // Check Properties
        CPPUNIT_ASSERT_MESSAGE("Invalid Src Token (TRANSFER7)",t->srcSrmToken == "token-1");
        CPPUNIT_ASSERT_MESSAGE("Invalid Dst Token (TRANSFER7)",t->destSrmToken == "token-2");
        CPPUNIT_ASSERT_MESSAGE("Invalid Src Host  (TRANSFER7)",t->srcHost == "host.source");
        CPPUNIT_ASSERT_MESSAGE("Invalid Dt Host   (TRANSFER7)",t->destHost == "host.dest");
    } catch(const std::exception& exc){
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        CPPUNIT_FAIL("Got Unknown Error");
    }
}

/*
 * testGet
 *
 * Test Get Method
 */
void TransferDaoTestCase::testGet_VO_Error(){
    try{
        // Get Transfer instance
        scoped_ptr<model::Transfer> t(m_voTransferDAO->get(TRANSFERDAO_TESTCASE_FILE3_ID,TRANSFERDAO_TESTCASE_REQ2_ID));
        CPPUNIT_FAIL("This Transfer shouldn't be reachable (TRANSFER3)");
    } catch(const DAOException& exc){
        // OK
    } catch(const std::exception& exc){
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        CPPUNIT_FAIL("Got Unknown Error");
    }
    try{
        // Get Transfer instance
        scoped_ptr<model::Transfer> t(m_voTransferDAO->get(TRANSFERDAO_TESTCASE_FILE1_ID,TRANSFERDAO_TESTCASE_REQ3_ID));
        CPPUNIT_FAIL("This Transfer shouldn't be reachable (Invalid file_id-req_id pair)");
    } catch(const DAOException& exc){
        // OK
    } catch(const std::exception& exc){
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        CPPUNIT_FAIL("Got Unknown Error");
    }
}

void TransferDaoTestCase::testGet_VO_3(){
    try{
        // Get Transfer instance
        scoped_ptr<model::Transfer> t(m_voTransferDAO->get(
            TRANSFERDAO_TESTCASE_FILE604_ID,
            TRANSFERDAO_TESTCASE_REQ8_ID));
        CPPUNIT_ASSERT_MESSAGE("Null Transfer (TRANSFER8)", t.get() != 0);
        // Check Properties
        CPPUNIT_ASSERT_MESSAGE("Invalid Reason (TRANSFER8)",t->reason() == "failure");
        CPPUNIT_ASSERT_MESSAGE("Invalid Reason Class (TRANSFER8)",t->reasonClass() == Error::SECURITY);
        CPPUNIT_ASSERT_MESSAGE("Invalid Error Scope (TRANSFER8)",t->errorScope() == Error::S_SOURCE);
        CPPUNIT_ASSERT_MESSAGE("Invalid Error Phase (TRANSFER8)",t->errorPhase() == Error::P_TX_PREPARATION);
    } catch(const std::exception& exc){
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        CPPUNIT_FAIL("Got Unknown Error");
    }
}

/*
 * testGetByFileId
 *
 * Test GetByFileId Method
 */
void TransferDaoTestCase::testGetByFileId_VO(){
    
    StringArray reqIds;
    try{
        m_voTransferDAO->getByFileId(TRANSFERDAO_TESTCASE_FILE4_ID,reqIds);
        CPPUNIT_ASSERT_MESSAGE("Invalid Files ID", !reqIds.empty());
        
        reqIds.clear();
        m_voTransferDAO->getByFileId(TRANSFERDAO_TESTCASE_FILE4_ID,reqIds,1);
        CPPUNIT_ASSERT_MESSAGE("Too many transfers", reqIds.size() == 1);
        
    } catch(const std::exception& exc){
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        CPPUNIT_FAIL("Got Unknown Error");
    }
}

/*
 * testGetByFileId
 *
 * Test GetByFileId Method
 */
void TransferDaoTestCase::testGetByFileId_VO_2(){
    
    StringArray reqIds;
    try{
        m_voTransferDAO->getByFileId(TRANSFERDAO_TESTCASE_FILE6_ID,reqIds);
        CPPUNIT_ASSERT_MESSAGE("Pending File (FILE6)", reqIds.empty());
    } catch(const std::exception& exc){
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        CPPUNIT_FAIL("Got Unknown Error");
    }
}

/*
 * testGetByFileId
 *
 * Test GetByFileId Method
 */
void TransferDaoTestCase::testGetByFileId_VO_3(){
    
    StringArray reqIds;
    try{
        m_voTransferDAO->getByFileId(TRANSFERDAO_TESTCASE_FILE5_ID,reqIds,1);
        CPPUNIT_ASSERT_MESSAGE("Too many transfers", reqIds.size() == 1);

        reqIds.clear();
        m_voTransferDAO->getByFileId(TRANSFERDAO_TESTCASE_FILE5_ID,reqIds,2);
        CPPUNIT_ASSERT_MESSAGE("Too many transfers", reqIds.size() == 2);
        
        // Get Transfer instances
        scoped_ptr<model::Transfer> t1(m_voTransferDAO->get(TRANSFERDAO_TESTCASE_FILE5_ID,reqIds.at(0)));
        CPPUNIT_ASSERT_MESSAGE("Null pointer returned #0", t1.get() != 0);
        scoped_ptr<model::Transfer> t2(m_voTransferDAO->get(TRANSFERDAO_TESTCASE_FILE5_ID,reqIds.at(1)));
        CPPUNIT_ASSERT_MESSAGE("Null pointer returned #1", t2.get() != 0);

        // Check the transfer times
        CPPUNIT_ASSERT_MESSAGE("Invalid order", t1->transferTime() >= t2->transferTime());
    } catch(const std::exception& exc){
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        CPPUNIT_FAIL("Got Unknown Error");
    }
}

/*
 * testGetByFileId
 *
 * Test GetByFileId Method
 */
void TransferDaoTestCase::testGetByFileId_VO_Error(){
    
    StringArray reqIds;
    try{
        m_voTransferDAO->getByFileId(TRANSFERDAO_TESTCASE_FILE3_ID,reqIds);
        CPPUNIT_FAIL("VO_2 Transfers shoudn't be reachable");
    } catch(const DAOException& exc){
        // OK
    } catch(const std::exception& exc){
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        CPPUNIT_FAIL("Got Unknown Error");
    }
}

/*
 * testGetByRequestId
 *
 * Test GetByRequestId Method
 */
void TransferDaoTestCase::testGetByRequestId_VO(){
    
    StringArray reqIds;
    try{
        m_voTransferDAO->getByRequestId(TRANSFERDAO_TESTCASE_REQ1_ID,reqIds);
        CPPUNIT_ASSERT_MESSAGE("Invalid Request ID", !reqIds.empty());
    } catch(const std::exception& exc){
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        CPPUNIT_FAIL("Got Unknown Error");
    }
}

/*
 * testGetByFileId
 *
 * Test GetByFileId Method
 */
void TransferDaoTestCase::testGetByRequestId_VO_Error(){
    
    StringArray reqIds;
    try{
        m_voTransferDAO->getByRequestId(TRANSFERDAO_TESTCASE_REQ2_ID,reqIds);
        CPPUNIT_FAIL("VO_2 Transfers shoudn't be reachable");
    } catch(const DAOException& exc){
        // OK
    } catch(const std::exception& exc){
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        CPPUNIT_FAIL("Got Unknown Error");
    }
}


/*
 * testGet
 *
 * Test Get Method
 */
void TransferDaoTestCase::testGet_Ch(){
    try{
        // Get Transfer instance
        scoped_ptr<model::Transfer> t(m_chTransferDAO->get(TRANSFERDAO_TESTCASE_FILE1_ID,TRANSFERDAO_TESTCASE_REQ1_ID));
        CPPUNIT_ASSERT_MESSAGE("Null Transfer (TRANSFER1)", t.get() != 0);
        
        // Check Some Precondition
        CPPUNIT_ASSERT_MESSAGE("Invalid Source TURL (TRANSFER1)", t->sourceTurl == TRANSFERDAO_TESTCASE_FILE1_SOURCE);
        CPPUNIT_ASSERT_MESSAGE("Invalid Dest TURL (TRANSFER1)", t->destTurl == TRANSFERDAO_TESTCASE_FILE1_DEST);
        CPPUNIT_ASSERT_MESSAGE("Invalid State (TRANSFER1)", t->state == Transfer::S_PROCESSING);

        // Get Transfer instance
        t.reset(m_chTransferDAO->get(TRANSFERDAO_TESTCASE_FILE2_ID,TRANSFERDAO_TESTCASE_REQ1_ID));
        CPPUNIT_ASSERT_MESSAGE("Null Transfer (TRANSFER2)", t.get() != 0);
        // Check Some Precondition
        CPPUNIT_ASSERT_MESSAGE("Invalid State (TRANSFER2)", t->state == Transfer::S_COMPLETED);
    } catch(const std::exception& exc){
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        CPPUNIT_FAIL("Got Unknown Error");
    }
}

/*
 * testGet
 *
 * Test Get Method
 */
void TransferDaoTestCase::testGet_Ch_Error(){
    try{
        // Get Transfer instance
        scoped_ptr<model::Transfer> t(m_chTransferDAO->get(TRANSFERDAO_TESTCASE_FILE7_ID,TRANSFERDAO_TESTCASE_REQ5_ID));
        CPPUNIT_FAIL("This Transfer shouldn't be reachable (TRANSFER7)");
    } catch(const DAOException& exc){
        // OK
    } catch(const std::exception& exc){
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        CPPUNIT_FAIL("Got Unknown Error");
    }
    try{
        // Get Transfer instance
        scoped_ptr<model::Transfer> t(m_chTransferDAO->get(TRANSFERDAO_TESTCASE_FILE1_ID,TRANSFERDAO_TESTCASE_REQ3_ID));
        CPPUNIT_FAIL("This Transfer shouldn't be reachable (Invalid file_id-req_id pair)");
    } catch(const DAOException& exc){
        // OK
    } catch(const std::exception& exc){
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        CPPUNIT_FAIL("Got Unknown Error");
    }
}

void TransferDaoTestCase::testGet_Ch_2(){
    try{
        // Get Transfer instance
        scoped_ptr<model::Transfer> t(m_chTransferDAO->get(
            TRANSFERDAO_TESTCASE_FILE604_ID,
            TRANSFERDAO_TESTCASE_REQ8_ID));
        CPPUNIT_ASSERT_MESSAGE("Null Transfer (TRANSFER8)", t.get() != 0);
        // Check Properties
        CPPUNIT_ASSERT_MESSAGE("Invalid Reason (TRANSFER8)",t->reason() == "failure");
        CPPUNIT_ASSERT_MESSAGE("Invalid Reason Class (TRANSFER8)",t->reasonClass() == Error::SECURITY);
        CPPUNIT_ASSERT_MESSAGE("Invalid Error Scope (TRANSFER8)",t->errorScope() == Error::S_SOURCE);
        CPPUNIT_ASSERT_MESSAGE("Invalid Error Phase (TRANSFER8)",t->errorPhase() == Error::P_TX_PREPARATION);
    } catch(const std::exception& exc){
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        CPPUNIT_FAIL("Got Unknown Error");
    }
}

/*
 * testUpdate
 *
 * Test Update Method
 */
void TransferDaoTestCase::testUpdate_Ch(){
    
    try{
        // Start Transaction
        DaoHelper::ctx().start();
        
        // Get Transfer instance
        scoped_ptr<model::Transfer> t(m_chTransferDAO->get(TRANSFERDAO_TESTCASE_FILE1_ID,TRANSFERDAO_TESTCASE_REQ1_ID));
        CPPUNIT_ASSERT_MESSAGE("Null Transfer (TRANSFER1)",t.get() != 0);

        time_t current;
        time(&current);
        // Set Transfer Properties        
        t->sourceTurl = "file:///tmp/source.txt";
        t->destTurl = "file:///tmp/dest.txt";
        t->transferTime = current;
        t->bytesWritten = 50111000111LL;
        t->duration = 1234.56;
        // Update Transfer
        m_chTransferDAO->update(*t);
        
        // Commit Transfer
        DaoHelper::ctx().commit();
        
        // Get Transfer Instance
        t.reset(m_chTransferDAO->get(TRANSFERDAO_TESTCASE_FILE1_ID,TRANSFERDAO_TESTCASE_REQ1_ID));
        CPPUNIT_ASSERT_MESSAGE("Null Transfer (TRANSFER1)",t.get() != 0);
        CPPUNIT_ASSERT_MESSAGE("Invalid SourceTURL (TRANSFER1)",t->sourceTurl == "file:///tmp/source.txt");
        CPPUNIT_ASSERT_MESSAGE("Invalid DestTURL (TRANSFER1)",t->destTurl == "file:///tmp/dest.txt");
        CPPUNIT_ASSERT_MESSAGE("Invalid TransferTime (TRANSFER1)",t->transferTime == current);
        CPPUNIT_ASSERT_MESSAGE("Invalid BytesWritten (TRANSFER1)",t->bytesWritten == 50111000111LL);
        CPPUNIT_ASSERT_MESSAGE("Invalid Duration (TRANSFER1)",t->duration == 1234.56);
        
    } catch(const DAOException& exc){
        // OK
        DaoHelper::ctx().rollback();
    } catch(const std::exception& exc){
        DaoHelper::ctx().rollback();
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        DaoHelper::ctx().rollback();
        CPPUNIT_FAIL("Got Unknown Error");
    }
}

/*
 * testUpdate
 *
 * Test Update Method
 */
void TransferDaoTestCase::testUpdate_Ch_2(){
    
    try{
        // Start Transaction
        DaoHelper::ctx().start();
        
        // Get Transfer instance
        scoped_ptr<model::Transfer> t(m_chTransferDAO->get(TRANSFERDAO_TESTCASE_FILE1_ID,TRANSFERDAO_TESTCASE_REQ1_ID));
        CPPUNIT_ASSERT_MESSAGE("Null Transfer (TRANSFER1)",t.get() != 0);
        
        // Update Transfer
        m_chTransferDAO->update(*t);
        // Commit Transactyion
        DaoHelper::ctx().commit();

        // Get Transfer Instance
        t.reset(m_chTransferDAO->get(TRANSFERDAO_TESTCASE_FILE1_ID,TRANSFERDAO_TESTCASE_REQ1_ID));
        CPPUNIT_ASSERT_MESSAGE("Null Transfer (TRANSFER1)",t.get() != 0);
    } catch(const std::exception& exc){
        DaoHelper::ctx().rollback();
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        DaoHelper::ctx().rollback();
        CPPUNIT_FAIL("Got Unknown Error");
    }
}


/*
 * testUpdate
 *
 * Test Update Method
 */
void TransferDaoTestCase::testUpdate_Ch_3(){
    
    try{
        // Start Transaction
        DaoHelper::ctx().start();
        
        // Get Transfer instance
        scoped_ptr<model::Transfer> t(m_chTransferDAO->get(TRANSFERDAO_TESTCASE_FILE1_ID,TRANSFERDAO_TESTCASE_REQ1_ID));
        CPPUNIT_ASSERT_MESSAGE("Null Transfer (TRANSFER1)",t.get() != 0);

        time_t current;
        time(&current);
        // Set Transfer Properties        
        t->state  = model::Transfer::S_TRANSFERRED;
        t->sourceTurl = "file:///tmp/source.txt-1";
        t->destTurl = "file:///tmp/dest.txt-1";
        t->transferTime = current;
        t->prepareTime = current - 10;
        t->duration = 1.23;
        t->srcPrepDuration  = 0.1;
        t->dstPrepDuration  = 0.2;
        t->txDuration       = 0.3;
        t->srcFinalDuration = 0.4;
        t->dstFinalDuration = 0.5;
        t->bytesWritten = 50111000112LL;
        t->throughput = 123.456;
        t->reasonClass = model::Error::NETWORK;
        t->reason = "Another reason";
        t->fileSize = 50111000222LL;
        
        // Update Transfer
        m_chTransferDAO->update(*t);
        
        // Commit Transfer
        DaoHelper::ctx().commit();
        
        // Get Transfer Instance
        t.reset(m_chTransferDAO->get(TRANSFERDAO_TESTCASE_FILE1_ID,TRANSFERDAO_TESTCASE_REQ1_ID));
        CPPUNIT_ASSERT_MESSAGE("Null Transfer (TRANSFER1)",t.get() != 0);
        CPPUNIT_ASSERT_MESSAGE("Invalid State (TRANSFER1)",t->state == model::Transfer::S_TRANSFERRED);
        CPPUNIT_ASSERT_MESSAGE("Invalid SourceTURL (TRANSFER1)",t->sourceTurl == "file:///tmp/source.txt-1");
        CPPUNIT_ASSERT_MESSAGE("Invalid DestTURL (TRANSFER1)",t->destTurl == "file:///tmp/dest.txt-1");
        CPPUNIT_ASSERT_MESSAGE("Invalid TransferTime (TRANSFER1)",t->transferTime == current);
        CPPUNIT_ASSERT_MESSAGE("Invalid PrepareTime (TRANSFER1)",t->prepareTime == (current - 10));
        CPPUNIT_ASSERT_MESSAGE("Invalid Duration (TRANSFER1)",t->duration == 1.23);
        CPPUNIT_ASSERT_MESSAGE("Invalid Src P Duration (TRANSFER1)",t->srcPrepDuration == 0.1);
        CPPUNIT_ASSERT_MESSAGE("Invalid Dst P Duration (TRANSFER1)",t->dstPrepDuration == 0.2);
        CPPUNIT_ASSERT_MESSAGE("Invalid TX Duration (TRANSFER1)",t->txDuration == 0.3);
        CPPUNIT_ASSERT_MESSAGE("Invalid Src F Duration (TRANSFER1)",t->srcFinalDuration == 0.4);
        CPPUNIT_ASSERT_MESSAGE("Invalid Dst F Duration (TRANSFER1)",t->dstFinalDuration == 0.5);
        CPPUNIT_ASSERT_MESSAGE("Invalid BytesWritten (TRANSFER1)",t->bytesWritten == 50111000112LL);
        CPPUNIT_ASSERT_MESSAGE("Invalid throughput (TRANSFER1)",t->throughput == 123.456);
        CPPUNIT_ASSERT_MESSAGE("Invalid reasonClass (TRANSFER1)",t->reasonClass == model::Error::NETWORK);
        CPPUNIT_ASSERT_MESSAGE("Invalid reason (TRANSFER1)",t->reason == "Another reason");
        CPPUNIT_ASSERT_MESSAGE("Invalid filesize (TRANSFER1)",t->fileSize == 50111000222LL);
        
    } catch(const DAOException& exc){
        // OK
        DaoHelper::ctx().rollback();
    } catch(const std::exception& exc){
        DaoHelper::ctx().rollback();
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        DaoHelper::ctx().rollback();
        CPPUNIT_FAIL("Got Unknown Error");
    }
}

/*
 * testUpdate
 *
 * Test Update Method
 */
void TransferDaoTestCase::testUpdate_Ch_4(){
    
    try{
        // Start Transaction
        DaoHelper::ctx().start();
        
        // Get Transfer instance
        scoped_ptr<model::Transfer> t(m_chTransferDAO->get(TRANSFERDAO_TESTCASE_FILE9_ID,TRANSFERDAO_TESTCASE_REQ7_ID));
        CPPUNIT_ASSERT_MESSAGE("Null Transfer (TRANSFER7)",t.get() != 0);

        // Set Transfer Properties        
        t->srcSrmToken  = "token-src-1";
        t->destSrmToken = "token-dst-1";
        t->srcHost  = "host.source.site";
        t->destHost = "host.dest.site";
        
        // Update Transfer
        m_chTransferDAO->update(*t);
        
        // Commit Transfer
        DaoHelper::ctx().commit();
        
        // Get Transfer Instance
        t.reset(m_chTransferDAO->get(TRANSFERDAO_TESTCASE_FILE9_ID,TRANSFERDAO_TESTCASE_REQ7_ID));
        CPPUNIT_ASSERT_MESSAGE("Null Transfer (TRANSFER7)",t.get() != 0);
        CPPUNIT_ASSERT_MESSAGE("Invalid Src Token (TRANSFER7)",t->srcSrmToken == "token-src-1");
        CPPUNIT_ASSERT_MESSAGE("Invalid Dst Token (TRANSFER7)",t->destSrmToken == "token-dst-1");
        CPPUNIT_ASSERT_MESSAGE("Invalid Src Host  (TRANSFER7)",t->srcHost == "host.source.site");
        CPPUNIT_ASSERT_MESSAGE("Invalid Dt Host   (TRANSFER7)",t->destHost == "host.dest.site");
        
    } catch(const DAOException& exc){
        // OK
        DaoHelper::ctx().rollback();
    } catch(const std::exception& exc){
        DaoHelper::ctx().rollback();
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        DaoHelper::ctx().rollback();
        CPPUNIT_FAIL("Got Unknown Error");
    }
}


/*
 * testUpdate
 *
 * Test Update Method
 */
void TransferDaoTestCase::testUpdate_Ch_5(){
    
    try{
        // Start Transaction
        DaoHelper::ctx().start();
        
        // Get Transfer instance
        scoped_ptr<model::Transfer> t(m_chTransferDAO->get(TRANSFERDAO_TESTCASE_FILE9_ID,TRANSFERDAO_TESTCASE_REQ7_ID));
        CPPUNIT_ASSERT_MESSAGE("Null Transfer (TRANSFER7)",t.get() != 0);

        time_t current;
        time(&current);
        // Set Transfer Properties        
        t->state  = model::Transfer::S_TRANSFERRED;
        t->sourceTurl = "file:///tmp/source.txt-1";
        t->destTurl = "file:///tmp/dest.txt-1";
        t->transferTime = current;
        t->reasonClass = model::Error::NETWORK;
        t->reason = "Another reason";
        t->fileSize = 33;
        t->srcSrmToken  = "token-1";
        t->destSrmToken = "token-2";
        t->srcHost  = "host.source";
        t->destHost = "host.dest";
        
        // Update Transfer
        m_chTransferDAO->update(*t);
        // Commit Transfer
        DaoHelper::ctx().commit();
        
        // Get Transfer Instance
        t.reset(m_chTransferDAO->get(TRANSFERDAO_TESTCASE_FILE9_ID,TRANSFERDAO_TESTCASE_REQ7_ID));
        CPPUNIT_ASSERT_MESSAGE("Null Transfer (TRANSFER7)",t.get() != 0);
        CPPUNIT_ASSERT_MESSAGE("Invalid State (TRANSFER7)",t->state == model::Transfer::S_TRANSFERRED);
        CPPUNIT_ASSERT_MESSAGE("Invalid SourceTURL (TRANSFER7)",t->sourceTurl == "file:///tmp/source.txt-1");
        CPPUNIT_ASSERT_MESSAGE("Invalid DestTURL (TRANSFER7)",t->destTurl == "file:///tmp/dest.txt-1");
        CPPUNIT_ASSERT_MESSAGE("Invalid TransferTime (TRANSFER7)",t->transferTime == current);
        CPPUNIT_ASSERT_MESSAGE("Invalid reasonClass (TRANSFER7)",t->reasonClass == model::Error::NETWORK);
        CPPUNIT_ASSERT_MESSAGE("Invalid reason (TRANSFER7)",t->reason == "Another reason");
        CPPUNIT_ASSERT_MESSAGE("Invalid filesize (TRANSFER7)",t->fileSize == 33);
        CPPUNIT_ASSERT_MESSAGE("Invalid Src Token (TRANSFER7)",t->srcSrmToken == "token-1");
        CPPUNIT_ASSERT_MESSAGE("Invalid Dst Token (TRANSFER7)",t->destSrmToken == "token-2");
        CPPUNIT_ASSERT_MESSAGE("Invalid Src Host  (TRANSFER7)",t->srcHost == "host.source");
        CPPUNIT_ASSERT_MESSAGE("Invalid Dt Host   (TRANSFER7)",t->destHost == "host.dest");
        
    } catch(const DAOException& exc){
        // OK
        DaoHelper::ctx().rollback();
    } catch(const std::exception& exc){
        DaoHelper::ctx().rollback();
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        DaoHelper::ctx().rollback();
        CPPUNIT_FAIL("Got Unknown Error");
    }
}

void TransferDaoTestCase::testUpdate_Ch_6(){
    try{
        DaoHelper::ctx().start();
        // Get Transfer instance
        scoped_ptr<model::Transfer> t(m_chTransferDAO->get(
            TRANSFERDAO_TESTCASE_FILE604_ID,
            TRANSFERDAO_TESTCASE_REQ9_ID));
        CPPUNIT_ASSERT_MESSAGE("Null Transfer (TRANSFER9)", t.get() != 0);
        
        time_t current;
        time(&current);
        
        t->reason = "lorem ipsum";
        t->reasonClass = Error::INVALID_SIZE;
        t->errorScope = Error::S_AGENT;
        t->errorPhase = Error::P_PRESTAGING;
        t->finishTime = current;
        
        
        // Update Transfer
        m_chTransferDAO->update(*t);
        // Commit Transfer
        DaoHelper::ctx().commit();
        
        // Check Properties
        CPPUNIT_ASSERT_MESSAGE("Invalid Reason (TRANSFER9)",t->reason() == "lorem ipsum");
        CPPUNIT_ASSERT_MESSAGE("Invalid Reason Class (TRANSFER9)",t->reasonClass() == Error::INVALID_SIZE);
        CPPUNIT_ASSERT_MESSAGE("Invalid Error Scope (TRANSFER9)",t->errorScope() == Error::S_AGENT);
        CPPUNIT_ASSERT_MESSAGE("Invalid Error Phase (TRANSFER9)",t->errorPhase() == Error::P_PRESTAGING);
        CPPUNIT_ASSERT_MESSAGE("Invalid Finish Time (TRANSFER9)",t->finishTime() == current);
    } catch(const std::exception& exc){
        DaoHelper::ctx().rollback();
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        DaoHelper::ctx().rollback();
        CPPUNIT_FAIL("Got Unknown Error");
    }
}

/*
 * testUpdate_reason
 *
 * Test Update Method
 */
void TransferDaoTestCase::testUpdate_reason(){
    
    try{
        // Start Transaction
        DaoHelper::ctx().start();
        
        // Get Transfer instance
        scoped_ptr<model::Transfer> t(m_chTransferDAO->get(TRANSFERDAO_TESTCASE_FILE1_ID,TRANSFERDAO_TESTCASE_REQ1_ID));
        CPPUNIT_ASSERT_MESSAGE("Null Transfer (TRANSFER1)",t.get() != 0);
        
        const char * const r = "This is a long reason string, more than 1024 characters:"\
        "0123456789012345678901234567890123456789012345678901234567890123456789"\
        "0123456789012345678901234567890123456789012345678901234567890123456789"\
        "0123456789012345678901234567890123456789012345678901234567890123456789"\
        "0123456789012345678901234567890123456789012345678901234567890123456789"\
        "0123456789012345678901234567890123456789012345678901234567890123456789"\
        "0123456789012345678901234567890123456789012345678901234567890123456789"\
        "0123456789012345678901234567890123456789012345678901234567890123456789"\
        "0123456789012345678901234567890123456789012345678901234567890123456789"\
        "0123456789012345678901234567890123456789012345678901234567890123456789"\
        "0123456789012345678901234567890123456789012345678901234567890123456789"\
        "0123456789012345678901234567890123456789012345678901234567890123456789"\
        "0123456789012345678901234567890123456789012345678901234567890123456789"\
        "0123456789012345678901234567890123456789012345678901234567890123456789"\
        "0123456789012345678901234567890123456789012345678901234567890123456789"\
        "0123456789012345678901234567890123456789012345678901234567890123456789"\
        "0123456789012345678901234567890123456789012345678901234567890123456789"\
        "0123456789012345678901234567890123456789012345678901234567890123456789"\
        "0123456789012345678901234567890123456789012345678901234567890123456789"\
        "0123456789012345678901234567890123456789012345678901234567890123456789"\
        "0123456789012345678901234567890123456789012345678901234567890123456789";
        t->reason = r;
        
        // Update Transfer
        m_chTransferDAO->update(*t);
        // Commit Transactyion
        DaoHelper::ctx().commit();

        // Get Transfer Instance
        t.reset(m_chTransferDAO->get(TRANSFERDAO_TESTCASE_FILE1_ID,TRANSFERDAO_TESTCASE_REQ1_ID));
        CPPUNIT_ASSERT_MESSAGE("Null Transfer (TRANSFER1)",t.get() != 0);
        CPPUNIT_ASSERT_MESSAGE("Invalid Transfer reason length (TRANSFER1)", t->reason().length() < strlen(r));
    } catch(const std::exception& exc){
        DaoHelper::ctx().rollback();
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        DaoHelper::ctx().rollback();
        CPPUNIT_FAIL("Got Unknown Error");
    }
}



/*
 * testUpdate
 *
 * Test Update Method
 */
void TransferDaoTestCase::testUpdate_Ch_Error(){
    
    try{
        // Start Transaction
        DaoHelper::ctx().start();
        
        // Get Transfer instance 
        scoped_ptr<model::Transfer> t(new model::Transfer(
                    TRANSFERDAO_TESTCASE_REQ5_ID,
                    TRANSFERDAO_TESTCASE_FILE7_ID,
                    0,
                    TRANSFERDAO_TESTCASE_JOB5_ID,
                    model::Transfer::S_RECEIVED));
        CPPUNIT_ASSERT_MESSAGE("Null new Transfer",t.get() != 0);
        
        t->state = model::Transfer::S_COMPLETED;
        // Update Transfer
        m_chTransferDAO->update(*t);
        CPPUNIT_FAIL("This Transfer shouldn't be updatable (TRANSFER7)");
    } catch(const DAOException& exc){
        // OK
        DaoHelper::ctx().rollback();
    } catch(const std::exception& exc){
        DaoHelper::ctx().rollback();
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        DaoHelper::ctx().rollback();
        CPPUNIT_FAIL("Got Unknown Error");
    }
}

/*
 * testGetByFileId
 *
 * Test GetByFileId Method
 */
void TransferDaoTestCase::testGetByFileId_Ch(){
    
    StringArray reqIds;
    try{
        m_chTransferDAO->getByFileId(TRANSFERDAO_TESTCASE_FILE4_ID,reqIds);
        CPPUNIT_ASSERT_MESSAGE("Invalid Files ID", !reqIds.empty());
    } catch(const std::exception& exc){
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        CPPUNIT_FAIL("Got Unknown Error");
    }
}

/*
 * testGetByFileId
 *
 * Test GetByFileId Method
 */
void TransferDaoTestCase::testGetByFileId_Ch_2(){
    
    StringArray reqIds;
    try{
        m_chTransferDAO->getByFileId(TRANSFERDAO_TESTCASE_FILE6_ID,reqIds);
        CPPUNIT_ASSERT_MESSAGE("Pending File (FILE6)", reqIds.empty());
    } catch(const std::exception& exc){
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        CPPUNIT_FAIL("Got Unknown Error");
    }
}

/*
 * testGetByFileId
 *
 * Test GetByFileId Method
 */
void TransferDaoTestCase::testGetByFileId_Ch_3(){
    
    StringArray reqIds;
    try{
        m_chTransferDAO->getByFileId(TRANSFERDAO_TESTCASE_FILE5_ID,reqIds,1);
        CPPUNIT_ASSERT_MESSAGE("Too many transfers", reqIds.size() == 1);

        reqIds.clear();
        m_chTransferDAO->getByFileId(TRANSFERDAO_TESTCASE_FILE5_ID,reqIds,2);
        CPPUNIT_ASSERT_MESSAGE("Too many transfers", reqIds.size() == 2);
        
        // Get Transfer instances
        scoped_ptr<model::Transfer> t1(m_chTransferDAO->get(TRANSFERDAO_TESTCASE_FILE5_ID,reqIds.at(0)));
        CPPUNIT_ASSERT_MESSAGE("Null pointer returned #0", t1.get() != 0);
        scoped_ptr<model::Transfer> t2(m_chTransferDAO->get(TRANSFERDAO_TESTCASE_FILE5_ID,reqIds.at(1)));
        CPPUNIT_ASSERT_MESSAGE("Null pointer returned #1", t2.get() != 0);

        // Check the transfer times
        CPPUNIT_ASSERT_MESSAGE("Invalid order", t1->transferTime() >= t2->transferTime());
    } catch(const std::exception& exc){
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        CPPUNIT_FAIL("Got Unknown Error");
    }
}

/*
 * testGetByFileId
 *
 * Test GetByFileId Method
 */
void TransferDaoTestCase::testGetByFileId_Ch_Error(){
    
    StringArray reqIds;
    try{
        m_chTransferDAO->getByFileId(TRANSFERDAO_TESTCASE_FILE7_ID,reqIds);
        CPPUNIT_FAIL("Channel_2 Transfers (FILE7) shoudn't be reachable");
    } catch(const DAOException& exc){
        // OK
    } catch(const std::exception& exc){
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        CPPUNIT_FAIL("Got Unknown Error");
    }
}

/*
 * testGetByRequestId
 *
 * Test GetByRequestId Method
 */
void TransferDaoTestCase::testGetByRequestId_Ch(){
    
    StringArray reqIds;
    try{
        m_chTransferDAO->getByRequestId(TRANSFERDAO_TESTCASE_REQ1_ID,reqIds);
        CPPUNIT_ASSERT_MESSAGE("Invalid Request ID", reqIds.empty() == false);
    } catch(const std::exception& exc){
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        CPPUNIT_FAIL("Got Unknown Error");
    }
}

/*
 * testGetByRequestId
 *
 * Test GetByRequestId Method
 */
void TransferDaoTestCase::testGetByRequestId_Ch_Error(){
    
    StringArray reqIds;
    try{
        m_chTransferDAO->getByRequestId(TRANSFERDAO_TESTCASE_REQ5_ID,reqIds);
        CPPUNIT_FAIL("Channel_2 Transfers (REQ5) shoudn't be reachable");
    } catch(const DAOException& exc){
        // OK
    } catch(const std::exception& exc){
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        CPPUNIT_FAIL("Got Unknown Error");
    }
}

/*
 * testCreate_Ch
 *
 * Test Create Method
 */
void TransferDaoTestCase::testCreate_Ch(){
    try{
        // Start Transaction
        DaoHelper::ctx().start();
        
        // Get Transfer instance 
        scoped_ptr<model::Transfer> t(new model::Transfer(
                    TRANSFERDAO_TESTCASE_REQ_NEW_ID,
                    TRANSFERDAO_TESTCASE_FILE_NEW_ID,
                    0,
                    TRANSFERDAO_TESTCASE_JOB7_ID,
                    model::Transfer::S_RECEIVED,
                    "FOO"));
        CPPUNIT_ASSERT_MESSAGE("Null Transfer (TRANSFER_NEW)",t.get() != 0);
        
        // Create Transfer
        m_chTransferDAO->create(*t);
        // Commit Transactyion
        DaoHelper::ctx().commit();

        // Get Transfer Instance
        t.reset(m_chTransferDAO->get(TRANSFERDAO_TESTCASE_FILE_NEW_ID,TRANSFERDAO_TESTCASE_REQ_NEW_ID));
        CPPUNIT_ASSERT_MESSAGE("Null Transfer (TRANSFER_NEW)",t.get() != 0);
        CPPUNIT_ASSERT_MESSAGE("Invalid Transfer type",t->transferType == "FOO");
    } catch(const std::exception& exc){
        DaoHelper::ctx().rollback();
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        DaoHelper::ctx().rollback();
        CPPUNIT_FAIL("Got Unknown Error");
    }
}

/*
 * testCountActiveTransfers_Ch
 *
 * Test CountActiveTransfer Method
 */
void TransferDaoTestCase::testCountActiveTransfers_Ch(){
    try{
        unsigned long count = m_chTransferDAO->countActiveTransfers();
        CPPUNIT_ASSERT_MESSAGE("Invalid count returned",count != 0);
    } catch(const std::exception& exc){
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        CPPUNIT_FAIL("Got Unknown Error");
    }
}

/*
 * testCountActiveTransfers_Ch
 *
 * Test CountActiveTransfer Method
 */
void TransferDaoTestCase::testCountActiveTransfers_Ch_2(){
    try{
        unsigned long count = m_chTransferDAO->countActiveTransfers(TRANSFERDAO_TESTCASE_VO1_NAME);
        CPPUNIT_ASSERT_MESSAGE("Invalid count returned (VO_1)",count != 0);
        
        count = m_chTransferDAO->countActiveTransfers(TRANSFERDAO_TESTCASE_VO3_NAME);
        CPPUNIT_ASSERT_MESSAGE("Invalid count returned (VO_3)",count == 0);
    } catch(const std::exception& exc){
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        CPPUNIT_FAIL("Got Unknown Error");
    }
}

/*
 * testGetActiveTransfers_Ch
 *
 * Test GetActiveTransfers Method
 */
void TransferDaoTestCase::testGetActiveTransfers_Ch(){
    
    dao::channel::TransferDAO::StringPairArray ids;
    try{
        m_chTransferDAO->getActiveTransfers(ids);
        CPPUNIT_ASSERT_MESSAGE("no active transfers",!ids.empty());
    } catch(const std::exception& exc){
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        CPPUNIT_FAIL("Got Unknown Error");
    }
}

/**
 * test_ggus_32512
 */
void TransferDaoTestCase::test_ggus_32512(){
    // transfer_state
    // dest_turl
    // target_srm_token
    // dest_host
    
    static const char * const channel_id = "bug_channel";
    static const char * const file_id = "70000001";
    static const char * const req_id = "req_1_1";
    
    try{
        scoped_ptr<dao::channel::TransferDAO> chTransferDAO(
            dao::channel::DAOFactory::instance().createTransferDAO(
                channel_id, DaoHelper::ctx()));
        
        DaoHelper::ctx().start();
        // Get Transfer instance
        scoped_ptr<model::Transfer> t(chTransferDAO->get(file_id,req_id));
        CPPUNIT_ASSERT_MESSAGE("Null transfer", t.get() != 0);

        t->state = Transfer::S_ABORTED;
        t->destTurl = "gsiftp://bar:1234/tmp/bar.txt";
        t->destSrmToken = "1234";
        t->destHost = "bar.bar";
                
        // Update Transfer
        chTransferDAO->update(*t);
        // Commit Transfer
        DaoHelper::ctx().commit();
        
        // Check Properties
        CPPUNIT_ASSERT_MESSAGE("Invalid state",t->state() == Transfer::S_ABORTED);
        CPPUNIT_ASSERT_MESSAGE("Invalid destTurl",t->destTurl() == "gsiftp://bar:1234/tmp/bar.txt");
        CPPUNIT_ASSERT_MESSAGE("Invalid destSrmToken",t->destSrmToken() == "1234");
        CPPUNIT_ASSERT_MESSAGE("Invalid destHost",t->destHost() == "bar.bar");
    } catch(const std::exception& exc){
        DaoHelper::ctx().rollback();
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        DaoHelper::ctx().rollback();
        CPPUNIT_FAIL("Got Unknown Error");
    }

}
