/**
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *  GLite Data Transfer Agents: Actions test utilities
 *
 *  Authors: Paolo Tedesco <paolo.tedesco@cern.ch>
 *  Version info: $Id: dao_test_utils.h,v 1.8 2008/08/08 12:55:51 ptedesco Exp $
 *  Release: $Name:  $
 *
 */

#ifndef GLITE_DATA_TRANSFER_AGENTS_TEST_DAOTESTUTILS_H_
#define GLITE_DATA_TRANSFER_AGENTS_TEST_DAOTESTUTILS_H_

#include "glite/data/transfer/agent/dao/channel/FileDAO.h"
#include "glite/data/transfer/agent/dao/channel/JobDAO.h"
#include "glite/data/transfer/agent/dao/channel/TransferDAO.h"

#include "glite/data/transfer/agent/dao/vo/FileDAO.h"
#include "glite/data/transfer/agent/dao/vo/JobDAO.h"
#include "glite/data/transfer/agent/dao/vo/TransferDAO.h"

#include "glite/data/transfer/agent/model/Error.h"
#include "glite/data/transfer/agent/model/Job.h"
#include "glite/data/transfer/agent/model/File.h"
#include "glite/data/transfer/agent/model/Transfer.h"
#include "glite/data/transfer/agent/model/Channel.h"

#include "model_test_utils.h"

#include <boost/scoped_ptr.hpp>
#include <boost/shared_ptr.hpp>
#include <cppunit/extensions/HelperMacros.h>
#include <sstream>

using namespace glite::data::transfer::agent;
using glite::data::agents::dao::DAOException;

namespace{

template <typename T>
const T& get_reference(const T& t){
    return t;
}

template <typename T>
const T& get_reference(const boost::shared_ptr<T>& t){
    return *(t.get());
}

template <typename T>
const T& get_reference(const boost::scoped_ptr<T>& t){
    return *(t.get());
}

//------------------------------------------------------------------------------
// These functions retrieve the object with the specified id and check that its
// state is the expected one. 
// The return value is a pointer to the retrieved object, that can be used to
// perform additional checks.
//------------------------------------------------------------------------------

/**
 * check_state
 * check state of given object
 * @param obj [IN] the model object
 * @param obj_name [IN] object name (for error message)
 * @param expected [IN] the expected object state
 */
template <typename ModelObject,typename State>
void check_state(
    const ModelObject&  obj,
    const char* const   obj_name,
    State               expected){
    State value = get_reference(obj).state();
    if (value != expected){
        FAIL_TEST("Invalid state (" << obj_name << ") : got " << str(value) << " expected " << str(expected));
    }
}
    
/**
 * check_job
 * Get job with given id and check its state
 * @param jobDAO [IN] the JobDAO object to use to get the job
 * @param job_id [IN] the job id
 * @param job_name [IN] job name (for error message)
 * @param expected [IN] the expected job state
 * @return Job object with given id
 */
template <typename JobDAO>
boost::shared_ptr<model::Job> check_job(
    const boost::scoped_ptr<JobDAO>&  jobDAO,
    const char* const                 job_id,
    const char* const                 job_name,
    model::Job::State                 expected){
        
    try{
        boost::shared_ptr<model::Job> job(jobDAO->getFullSearch(job_id));
        if (0 == job.get()){
            FAIL_TEST("Null job (" << job_name << ")");
        }
        check_state(job,job_name,expected);
        return job;
    } catch (const DAOException& exc){
        FAIL_TEST("Got DAOException (" << job_name << ") : " << exc.what());
        return boost::shared_ptr<model::Job>(); // silence the warning
    } 
}

/**
 * check_file
 * Get file with given id and check its state
 * @param fileDAO [IN] the FileDAO object to use to get the job
 * @param file_id [IN] the file id
 * @param file_name [IN] file name (for error message)
 * @param expected [IN] the expected file state
 * @return File object with given id
 */
template <typename FileDAO>
boost::shared_ptr<model::File> check_file(
    const boost::scoped_ptr<FileDAO>&   fileDAO,
    const char* const                   file_id,
    const char* const                   file_name,
    model::File::State                  expected){
        
    try{
        boost::shared_ptr<model::File> file(fileDAO->getFullSearch(file_id));
        if (0 == file.get()){
            FAIL_TEST("Null file (" << file_name << ")"); 
        }
        check_state(file,file_name,expected);
        return file;
    } catch (const DAOException& exc){
        FAIL_TEST("Got DAOException (" << file_name << ") : " << exc.what());
        return boost::shared_ptr<model::File>(); // silence the warning
    }
}

/**
 * check_transfer
 * Get transfer with given file id/request id and check its state
 * @param transferDAO [IN] the TransferDAO object to use to get the job
 * @param file_id [IN] the file id
 * @param request_id [IN] the request id
 * @param transfer_name [IN] transfer name (for error message)
 * @param expected [IN] the expected transfer state
 * @return Transfer object with given id
 */
template <typename TransferDAO>
boost::shared_ptr<model::Transfer> check_transfer(
    const boost::scoped_ptr<TransferDAO>&   transferDAO,
    const char* const                       file_id,
    const std::string&                      request_id,
    const char* const                       transfer_name,
    model::Transfer::State                  expected){
        
    try{
        boost::shared_ptr<model::Transfer> transfer(transferDAO->getFullSearch(file_id,request_id));
        if (0 == transfer.get()){
            FAIL_TEST("Null transfer (" << transfer_name << ")");
        }
        check_state(transfer,transfer_name,expected);
        return transfer;
    } catch (const DAOException& exc){
        FAIL_TEST("Got DAOException (" << transfer_name << ") : " << exc.what());
        return boost::shared_ptr<model::Transfer>(); // silence the warning
    } 
}

//------------------------------------------------------------------------------
// category/scope/phase checks
//------------------------------------------------------------------------------

/**
 * check_error
 * @param object [IN] the model object
 * @param object_id [IN] model object id (for error reporting)
 * @param category [IN] expected error category
 * @param scope [IN] expected error scope
 * @param phase [IN] expected error phase 
 */
template <typename ObjectModel> 
void check_error(
    ObjectModel&            object,
    const char* const&      object_id,
    model::Error::Category  category,
    model::Error::Scope     scope,
    model::Error::Phase     phase){
    
    model::Error::Category  category_val    = get_reference(object).reasonClass();
    model::Error::Scope     scope_val       = get_reference(object).errorScope();
    model::Error::Phase     phase_val       = get_reference(object).errorPhase();
    if (category_val != category){
        FAIL_TEST("Invalid Error Category (" << object_id << "): got " << 
            str(category_val) << " expected " << str(category));
    }
    if (scope_val != scope){
        FAIL_TEST("Invalid Error Scope (" << object_id << "): got " << 
            str(scope_val) << " expected " << str(scope));
    }
    if (phase_val != phase){
        FAIL_TEST("Invalid Error Phase (" << object_id << "): got " << 
            str(phase_val) << " expected " << str(phase));
    }
}

/**
 * check_error
 * @param object [IN] the model object
 * @param object_id [IN] model object id (for error reporting)
 */
template <typename ObjectModel> 
void check_error(
    ObjectModel&        object,
    const char* const   object_id){
    check_error(object,object_id,model::Error::UNDEF,model::Error::S_UNDEF,model::Error::P_UNDEF);
}

//------------------------------------------------------------------------------
// property value checks
//------------------------------------------------------------------------------

/**
 * Get the id (name) of a channel (for error reporting)
 * @param ch
 * @return 
 */
std::string get_id(const model::Channel& ch){
    std::string ret = "channel ";
    ret += ch.name();
    return ret;
}

/**
 * Get the id of a job (for error reporting)
 * @param job 
 * @return 
 */
std::string get_id(const model::Job& job){
    std::string ret = "job ";
    ret += job.id();
    return ret; 
}
/**
 * Get the id of a file (for error reporting)
 * @param file 
 * @return 
 */
std::string get_id(const model::File& file){
    std::string ret = "file ";
    ret += file.id();
    return ret; 
}
/**
 * Get the id of a transfer (for error reporting)
 * @param transfer 
 * @return 
 */
std::string get_id(const model::Transfer& transfer){
    std::stringstream ret;
    ret << "transfer ";
    ret << transfer.requestId();
    ret << ":";
    ret << transfer.transferId();
    return ret.str();
}

/**
 * get the id of the object in the shared_ptr
 * @param object 
 * @return 
 */
template <class ModelObject>
std::string get_id(ModelObject& object){
    return get_id(get_reference(object));
}
        
/**
 * check the value of a property
 * @param object 
 * @param propertyName 
 * @param propertyValue 
 * @param expectedValue 
 */
template<
    class ModelObject,
    typename PropertyType,
    typename ValueType>
void check_model_object_value(
    const ModelObject& object,
    const std::string& propertyName,
    const PropertyType& propertyValue,
    const ValueType expectedValue){
    if (propertyValue != expectedValue){
        std::stringstream err;
        err << "Invalid " << propertyName << "(" << get_id(object) << "): got '" << propertyValue << "' expected '" << expectedValue << "'";
        CPPUNIT_FAIL(err.str().c_str());
    }
}

// helper macro
#define CHECK_MODEL_OBJECT_VALUE(object,property,expectedValue) check_model_object_value(object,#property,get_reference(object).property(),expectedValue)

} // namespace

#endif // GLITE_DATA_TRANSFER_AGENTS_TEST_DAOTESTUTILS_H_
