/**
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *  GLite Data Transfer Agents: FileFSM Test Case
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>, Paolo Tedesco <paolo.tedesco@cern.ch>
 *  Version info: $Id: FileFSMTestCase.cpp,v 1.9 2008/04/09 16:07:08 ptedesco Exp $
 *  Release: $Name:  $
 *
 */
#include "FileFSM.h"
#include "glite/data/transfer/agent/fsm/FileEventListener.h"
#include "model_test_utils.h"
#include <vector>
#include <algorithm>
#include <cppunit/TestCase.h>
#include <cppunit/extensions/HelperMacros.h>

using namespace glite::data::transfer::agent::fsm;

using glite::data::transfer::agent::model::File;
using glite::data::transfer::agent::model::Job;
using glite::data::transfer::agent::model::Transfer;
using glite::data::transfer::agent::model::StageReq;
using glite::data::transfer::agent::model::Error;

/**
 * Test FileEventListener
 */
class TestFileEventListener : public FileEventListener{
public:
    /**
     * Return singleton instance
     */
    static TestFileEventListener& instance(){
        static TestFileEventListener s_instance;
        return s_instance;
    }
    void reset(){
        m_stateChanged = false;
        m_ready        = false;
        m_done         = false;
        m_finished     = false;
    }
    bool stateChanged() const { return m_stateChanged; }
    bool ready() const        { return m_ready; }
    bool done() const         { return m_done; }
    bool finished() const     { return m_finished; }

    // -------------------------------------------------------------------------
    // Event Handlers
    // -------------------------------------------------------------------------
    
    /**
     * onStateChanged
     */
    virtual void onStateChanged(const File& file, const Job& job, File::State old_state) throw(){
        m_stateChanged = true;    
    };
    /**
     * onReady
     */
    virtual void onReady(const File& file, const Job& job) throw(){
        m_ready = true;    
    };
    /**
     * onDone
     */
    virtual void onDone(const File& file, const Job& job) throw(){
        m_done = true;
    };
    /**
     * onFinished
     */
    virtual void onFinished(const File& file, const Job& job) throw(){
        m_finished = true;
    };
protected:
    /**
     * Constructor
     */
    TestFileEventListener(){
        reset();    
    }
    /**
     * Destructor
     */
    virtual ~TestFileEventListener(){}
private:

    /**
     * StateChanged Flag
     */
    bool m_stateChanged;
    /**
     * Ready Flag
     */
    bool m_ready;
    /**
     * Done Flag
     */
    bool m_done;
    /**
     * Finished Flag
     */
    bool m_finished;
};

/**
 * CPPUnit TestCase
 */
class FileFSMTestCase : public CppUnit::TestCase{
public:
    /**
     * set-up test case
     */
    void setUp ();
private:
    // Declare CPPUnit Methods
    // -------------------------------------------------------------------------
    CPPUNIT_TEST_SUITE( FileFSMTestCase);
    CPPUNIT_TEST( testOnEventAllocate            );
    CPPUNIT_TEST( testOnEventAllocate_Error      );
    CPPUNIT_TEST( testOnEventFileReady           );
    CPPUNIT_TEST( testOnEventFileReady_Error     );
    CPPUNIT_TEST( testOnEventFileNotReady        );
    CPPUNIT_TEST( testOnEventFileNotReady_Error  );
    CPPUNIT_TEST( testOnEventStartStaging        );
    CPPUNIT_TEST( testOnEventStartStaging_Error  );
    CPPUNIT_TEST( testOnEventStaging             );
    CPPUNIT_TEST( testOnEventStaging_Error       );
    CPPUNIT_TEST( testOnEventStart               );
    CPPUNIT_TEST( testOnEventStart_Error         );
    CPPUNIT_TEST( testOnEventRunning             );
    CPPUNIT_TEST( testOnEventRunning_Error       );
    CPPUNIT_TEST( testOnEventDone                );
    CPPUNIT_TEST( testOnEventDone_Error          );
    CPPUNIT_TEST( testOnEventFailed              );
    CPPUNIT_TEST( testOnEventReschedule          );
    CPPUNIT_TEST( testOnEventReschedule_Error    );
    CPPUNIT_TEST( testOnEventHold                );
    CPPUNIT_TEST( testOnEventHold_Error          );
    CPPUNIT_TEST( testOnEventCanceled            );
    CPPUNIT_TEST( testOnEventFinishing           );
    CPPUNIT_TEST( testOnEventFinishing_Error     );
    CPPUNIT_TEST( testOnEventFinished            );
    CPPUNIT_TEST( testOnEventFinished_Error      );
    CPPUNIT_TEST_SUITE_END();

    // Test Methods
    // -------------------------------------------------------------------------

    /**
     * Test the Allocate Event
     */
    void testOnEventAllocate();
    void testOnEventAllocate_Error();

    /**
     * Test the FileReady Event
     */
    void testOnEventFileReady();
    void testOnEventFileReady_Error();

    /**
     * Test the FileNotReady Event
     */
    void testOnEventFileNotReady();
    void testOnEventFileNotReady_Error();

    /**
     * Test the StartStaging Event
     */
    void testOnEventStartStaging();
    void testOnEventStartStaging_Error();

    /**
     * Test the Staging Event
     */
    void testOnEventStaging();
    void testOnEventStaging_Error();

    /**
     * Test the Start Event
     */
    void testOnEventStart();
    void testOnEventStart_Error();

    /**
     * Test the Running Event
     */
    void testOnEventRunning();
    void testOnEventRunning_Error();
    
    /**
     * Test the Done Event
     */
    void testOnEventDone();
    void testOnEventDone_Error();

    /**
     * Test the Transfer Failed Event
     */
    void testOnEventFailed();

    /**
     * Test the Reschedule Event
     */
    void testOnEventReschedule();
    void testOnEventReschedule_Error();

    /**
     * Test the Hold Event
     */
    void testOnEventHold();
    void testOnEventHold_Error();

    /**
     * Test the Canceled Event
     */
    void testOnEventCanceled();
    
    /**
     * Test the Finishing Event
     */
    void testOnEventFinishing();
    void testOnEventFinishing_Error();

    /**
     * Test the Finished Event
     */
    void testOnEventFinished();
    void testOnEventFinished_Error();
};

/*
 * Register Suit
 */
CPPUNIT_TEST_SUITE_NAMED_REGISTRATION( FileFSMTestCase, "FileFSMTestCase" );

/*
 * setUp
 *
 * set-up test case
 */
void FileFSMTestCase::setUp(){
    
    // Instantiate Listener
    TestFileEventListener::instance().reset();
}

/**
 * Test the Allocate Event
 * test that the file event listener received the proper events.
 */
void FileFSMTestCase::testOnEventAllocate(){
    // Create an Object
    File file;
    Job job;
    FileFSM fsm(file,job);

    // Check From SUBMITTED
    file.state = (File::S_SUBMITTED);
    fsm.onEventAllocate();
    CPPUNIT_ASSERT_MESSAGE("Invalid State From SUBMITTED", file.state == File::S_PENDING);
    CPPUNIT_ASSERT_MESSAGE("Listener: Invalid State Changed", TestFileEventListener::instance().stateChanged() == true);
    CPPUNIT_ASSERT_MESSAGE("Listener: Invalid Ready", TestFileEventListener::instance().ready() == false);
    CPPUNIT_ASSERT_MESSAGE("Listener: Invalid Done" , TestFileEventListener::instance().done() == false);
    CPPUNIT_ASSERT_MESSAGE("Listener: Invalid Finished", TestFileEventListener::instance().finished() == false);

}

/**
 * Test the Allocate Event
 * test that the onEventAllocate event generates an exception if the file is in
 * a state from which this event should not be received.
 */
void FileFSMTestCase::testOnEventAllocate_Error(){
    // Create an Object
    File file;
    Job job;
    FileFSM fsm(file,job);
    
    // Define Bad Cases
    typedef std::vector<File::State> BadCaseSet;
    BadCaseSet           bad_cases;
    BadCaseSet::iterator b_it;
    bad_cases.push_back(File::S_UNDEF);
    bad_cases.push_back(File::S_PENDING);
    bad_cases.push_back(File::S_READY);
    bad_cases.push_back(File::S_ACTIVE);
    bad_cases.push_back(File::S_DONE);
    bad_cases.push_back(File::S_WAITING);
    bad_cases.push_back(File::S_HOLD);
    bad_cases.push_back(File::S_FAILED);
    bad_cases.push_back(File::S_AWAITING_PRESTAGE);
    bad_cases.push_back(File::S_PRESTAGING);
    bad_cases.push_back(File::S_WAITING_PRESTAGE);
    bad_cases.push_back(File::S_WAITING_CATALOG_RESOLUTION);
    bad_cases.push_back(File::S_WAITING_CATALOG_REGISTRATION);
    bad_cases.push_back(File::S_CANCELED);
    bad_cases.push_back(File::S_FINISHED);
    bad_cases.push_back(File::S_FINISHING);
    
    for(b_it = bad_cases.begin(); b_it != bad_cases.end(); ++b_it){
    
        try{
            TestFileEventListener::instance().reset();
            // Check From invalid State
            file.state = (*b_it);
            fsm.onEventAllocate();
            std::stringstream error;
            error << "State " << (*b_it) << " should give an exception";
            CPPUNIT_FAIL(error.str().c_str());
            CPPUNIT_ASSERT_MESSAGE("Listener: Invalid State Changed", TestFileEventListener::instance().stateChanged() == true);
            CPPUNIT_ASSERT_MESSAGE("Listener: Invalid Ready", TestFileEventListener::instance().ready() == false);
            CPPUNIT_ASSERT_MESSAGE("Listener: Invalid Done" , TestFileEventListener::instance().done() == false);
            CPPUNIT_ASSERT_MESSAGE("Listener: Invalid Finished", TestFileEventListener::instance().finished() == false);
        } catch(const FSMException& exc){
        }
    }            

}

/*
 * Test the FileReady Event
 */
void FileFSMTestCase::testOnEventFileReady(){
    // Create an Object
    File file;
    Job job;
    FileFSM fsm(file,job);

    // Check From PENDING
    file.state = File::S_PENDING;
    fsm.onEventFileReady();
    CPPUNIT_ASSERT_MESSAGE("Invalid State From PENDING", file.state == File::S_READY);
    CPPUNIT_ASSERT_MESSAGE("Listener: Invalid State Changed", TestFileEventListener::instance().stateChanged() == true);
    CPPUNIT_ASSERT_MESSAGE("Listener: Invalid Ready", TestFileEventListener::instance().ready() == true);
    CPPUNIT_ASSERT_MESSAGE("Listener: Invalid Done" , TestFileEventListener::instance().done() == false);
    CPPUNIT_ASSERT_MESSAGE("Listener: Invalid Finished", TestFileEventListener::instance().finished() == false);

    TestFileEventListener::instance().reset();
    // Check From PRESTAGING
    file.state = File::S_PRESTAGING;
    fsm.onEventFileReady();
    CPPUNIT_ASSERT_MESSAGE("Invalid State From PRESTAGING", file.state == File::S_READY);
    CPPUNIT_ASSERT_MESSAGE("Listener: Invalid State Changed", TestFileEventListener::instance().stateChanged() == true);
    CPPUNIT_ASSERT_MESSAGE("Listener: Invalid Ready", TestFileEventListener::instance().ready() == true);
    CPPUNIT_ASSERT_MESSAGE("Listener: Invalid Done" , TestFileEventListener::instance().done() == false);

}
/*
 * Test the FileReady Event
 */
void FileFSMTestCase::testOnEventFileReady_Error(){
    // Create an Object
    File file;
    Job job;
    FileFSM fsm(file,job);

    // Define Bad Cases
    typedef std::vector<File::State> BadCaseSet;
    BadCaseSet           bad_cases;
    BadCaseSet::iterator b_it;
    bad_cases.push_back(File::S_UNDEF);
    bad_cases.push_back(File::S_SUBMITTED);
    bad_cases.push_back(File::S_READY);
    bad_cases.push_back(File::S_ACTIVE);
    bad_cases.push_back(File::S_DONE);
    bad_cases.push_back(File::S_WAITING);
    bad_cases.push_back(File::S_HOLD);
    bad_cases.push_back(File::S_FAILED);
    bad_cases.push_back(File::S_AWAITING_PRESTAGE);
    bad_cases.push_back(File::S_WAITING_PRESTAGE);
    bad_cases.push_back(File::S_WAITING_CATALOG_RESOLUTION);
    bad_cases.push_back(File::S_WAITING_CATALOG_REGISTRATION);
    bad_cases.push_back(File::S_CANCELED);
    bad_cases.push_back(File::S_FINISHED);
    bad_cases.push_back(File::S_FINISHING);
    
    for(b_it = bad_cases.begin(); b_it != bad_cases.end(); ++b_it){
        try{
            // Check From invalid State
            file.state = (*b_it);
            fsm.onEventFileReady();
            std::stringstream error;
            error << "State " << (*b_it) << " should give an exception";
            CPPUNIT_FAIL(error.str().c_str());
        } catch(const FSMException& exc){
            CPPUNIT_ASSERT_MESSAGE("Listener: Invalid State Changed", TestFileEventListener::instance().stateChanged() == false);
            CPPUNIT_ASSERT_MESSAGE("Listener: Invalid Ready", TestFileEventListener::instance().ready() == false);
            CPPUNIT_ASSERT_MESSAGE("Listener: Invalid Done" , TestFileEventListener::instance().done() == false);
            CPPUNIT_ASSERT_MESSAGE("Listener: Invalid Finished", TestFileEventListener::instance().finished() == false);
        }
    }            

}

/*
 * Test the FileNotReady Event
 */
void FileFSMTestCase::testOnEventFileNotReady(){
    // Create an Object
    File file;
    Job job;
    FileFSM fsm(file,job);

    // Check From PENDING
    file.state = File::S_PENDING;
    fsm.onEventFileNotReady();
    CPPUNIT_ASSERT_MESSAGE("Invalid State From PENDING", file.state == File::S_AWAITING_PRESTAGE);
    CPPUNIT_ASSERT_MESSAGE("Listener: Invalid State Changed", TestFileEventListener::instance().stateChanged() == true);
    CPPUNIT_ASSERT_MESSAGE("Listener: Invalid Ready", TestFileEventListener::instance().ready() == false);
    CPPUNIT_ASSERT_MESSAGE("Listener: Invalid Done" , TestFileEventListener::instance().done() == false);
    CPPUNIT_ASSERT_MESSAGE("Listener: Invalid Finished", TestFileEventListener::instance().finished() == false);

}

/*
 * Test the FileNotReady Event
 */
void FileFSMTestCase::testOnEventFileNotReady_Error(){
    // Create an Object
    File file;
    Job job;
    FileFSM fsm(file,job);

    // Define Bad Cases
    typedef std::vector<File::State> BadCaseSet;
    BadCaseSet           bad_cases;
    BadCaseSet::iterator b_it;
    bad_cases.push_back(File::S_UNDEF);
    bad_cases.push_back(File::S_SUBMITTED);
    bad_cases.push_back(File::S_READY);
    bad_cases.push_back(File::S_ACTIVE);
    bad_cases.push_back(File::S_DONE);
    bad_cases.push_back(File::S_WAITING);
    bad_cases.push_back(File::S_HOLD);
    bad_cases.push_back(File::S_FAILED);
    bad_cases.push_back(File::S_AWAITING_PRESTAGE);
    bad_cases.push_back(File::S_WAITING_PRESTAGE);
    bad_cases.push_back(File::S_PRESTAGING);
    bad_cases.push_back(File::S_WAITING_CATALOG_RESOLUTION);
    bad_cases.push_back(File::S_WAITING_CATALOG_REGISTRATION);
    bad_cases.push_back(File::S_CANCELED);
    bad_cases.push_back(File::S_FINISHED);
    bad_cases.push_back(File::S_FINISHING);
    
    for(b_it = bad_cases.begin(); b_it != bad_cases.end(); ++b_it){
        try{
            // Check From invalid State
            file.state = (*b_it);
            fsm.onEventFileNotReady();
            std::stringstream error;
            error << "State " <<  (*b_it) << " should give an exception";
            CPPUNIT_FAIL(error.str().c_str());
        } catch(const FSMException& exc){
            CPPUNIT_ASSERT_MESSAGE("Listener: Invalid State Changed", TestFileEventListener::instance().stateChanged() == false);
            CPPUNIT_ASSERT_MESSAGE("Listener: Invalid Ready", TestFileEventListener::instance().ready() == false);
            CPPUNIT_ASSERT_MESSAGE("Listener: Invalid Done" , TestFileEventListener::instance().done() == false);
            CPPUNIT_ASSERT_MESSAGE("Listener: Invalid Finished", TestFileEventListener::instance().finished() == false);
        }
    }            

}

/*
 * Test the StartStaging Event
 */
void FileFSMTestCase::testOnEventStartStaging(){
    // Create an Object
    File file;
    Job job;
    FileFSM fsm(file,job);
    StageReq req;

    // Check From AWAITING_PRESTAGE
    file.state = File::S_AWAITING_PRESTAGE;
    fsm.onEventStartStaging(req);
    CPPUNIT_ASSERT_MESSAGE("Invalid State From AWAITING_PRESTAGE", file.state == File::S_PRESTAGING);
    CPPUNIT_ASSERT_MESSAGE("Listener: Invalid State Changed", TestFileEventListener::instance().stateChanged() == true);
    CPPUNIT_ASSERT_MESSAGE("Listener: Invalid Ready", TestFileEventListener::instance().ready() == false);
    CPPUNIT_ASSERT_MESSAGE("Listener: Invalid Done" , TestFileEventListener::instance().done() == false);
    CPPUNIT_ASSERT_MESSAGE("Listener: Invalid Finished", TestFileEventListener::instance().finished() == false);

}
    
/*
 * Test the StartStaging Event
 */
void FileFSMTestCase::testOnEventStartStaging_Error(){
    // Create an Object
    File file;
    Job job;
    FileFSM fsm(file,job);
    StageReq req;

    // Define Bad Cases
    typedef std::vector<File::State> BadCaseSet;
    BadCaseSet           bad_cases;
    BadCaseSet::iterator b_it;
    bad_cases.push_back(File::S_UNDEF);
    bad_cases.push_back(File::S_SUBMITTED);
    bad_cases.push_back(File::S_PENDING);
    bad_cases.push_back(File::S_READY);
    bad_cases.push_back(File::S_ACTIVE);
    bad_cases.push_back(File::S_DONE);
    bad_cases.push_back(File::S_WAITING);
    bad_cases.push_back(File::S_HOLD);
    bad_cases.push_back(File::S_FAILED);
    bad_cases.push_back(File::S_WAITING_PRESTAGE);
    bad_cases.push_back(File::S_PRESTAGING);
    bad_cases.push_back(File::S_WAITING_CATALOG_RESOLUTION);
    bad_cases.push_back(File::S_WAITING_CATALOG_REGISTRATION);
    bad_cases.push_back(File::S_CANCELED);
    bad_cases.push_back(File::S_FINISHED);
    bad_cases.push_back(File::S_FINISHING);
    
    for(b_it = bad_cases.begin(); b_it != bad_cases.end(); ++b_it){
        try{
            // Check From invalid State
            file.state = (*b_it);
            fsm.onEventStartStaging(req);
            std::stringstream error;
            error << "State " << (*b_it) << " should give an exception";
            CPPUNIT_FAIL(error.str().c_str());
        } catch(const FSMException& exc){
            CPPUNIT_ASSERT_MESSAGE("Listener: Invalid State Changed", TestFileEventListener::instance().stateChanged() == false);
            CPPUNIT_ASSERT_MESSAGE("Listener: Invalid Ready", TestFileEventListener::instance().ready() == false);
            CPPUNIT_ASSERT_MESSAGE("Listener: Invalid Done" , TestFileEventListener::instance().done() == false);
            CPPUNIT_ASSERT_MESSAGE("Listener: Invalid Finished", TestFileEventListener::instance().finished() == false);
        }
    }            

}

/*
 * Test the Staging Event
 */
void FileFSMTestCase::testOnEventStaging(){
    // Create an Object
    File file;
    Job job;
    FileFSM fsm(file,job);
    StageReq req;

    // Check From PRESTAGING
    file.state = File::S_PRESTAGING;
    fsm.onEventStaging(req);
    CPPUNIT_ASSERT_MESSAGE("Invalid State From PRESTAGING", file.state == File::S_PRESTAGING);
    CPPUNIT_ASSERT_MESSAGE("Listener: Invalid State Changed", TestFileEventListener::instance().stateChanged() == false);
    CPPUNIT_ASSERT_MESSAGE("Listener: Invalid Ready", TestFileEventListener::instance().ready() == false);
    CPPUNIT_ASSERT_MESSAGE("Listener: Invalid Done" , TestFileEventListener::instance().done() == false);
    CPPUNIT_ASSERT_MESSAGE("Listener: Invalid Finished", TestFileEventListener::instance().finished() == false);

}
/*
 * Test the Staging Event
 */
void FileFSMTestCase::testOnEventStaging_Error(){
    // Create an Object
    File file;
    Job job;
    FileFSM fsm(file,job);
    StageReq req;

    // Define Bad Cases
    typedef std::vector<File::State> BadCaseSet;
    BadCaseSet           bad_cases;
    BadCaseSet::iterator b_it;
    bad_cases.push_back(File::S_UNDEF);
    bad_cases.push_back(File::S_SUBMITTED);
    bad_cases.push_back(File::S_PENDING);
    bad_cases.push_back(File::S_READY);
    bad_cases.push_back(File::S_ACTIVE);
    bad_cases.push_back(File::S_DONE);
    bad_cases.push_back(File::S_WAITING);
    bad_cases.push_back(File::S_HOLD);
    bad_cases.push_back(File::S_FAILED);
    bad_cases.push_back(File::S_AWAITING_PRESTAGE);
    bad_cases.push_back(File::S_WAITING_PRESTAGE);
    bad_cases.push_back(File::S_WAITING_CATALOG_RESOLUTION);
    bad_cases.push_back(File::S_WAITING_CATALOG_REGISTRATION);
    bad_cases.push_back(File::S_CANCELED);
    bad_cases.push_back(File::S_FINISHED);
    bad_cases.push_back(File::S_FINISHING);
    
    for(b_it = bad_cases.begin(); b_it != bad_cases.end(); ++b_it){
        try{
            // Check From invalid State
            file.state = (*b_it);
            fsm.onEventStaging(req);
            std::stringstream error;
            error << "State " << (*b_it) << " should give an exception";
            CPPUNIT_FAIL(error.str().c_str());
        } catch(const FSMException& exc){
            CPPUNIT_ASSERT_MESSAGE("Listener: Invalid State Changed", TestFileEventListener::instance().stateChanged() == false);
            CPPUNIT_ASSERT_MESSAGE("Listener: Invalid Ready", TestFileEventListener::instance().ready() == false);
            CPPUNIT_ASSERT_MESSAGE("Listener: Invalid Done" , TestFileEventListener::instance().done() == false);
            CPPUNIT_ASSERT_MESSAGE("Listener: Invalid Finished", TestFileEventListener::instance().finished() == false);
        }
    }            

}

/*
 * Test OnEventStart Method
 */
void FileFSMTestCase::testOnEventStart(){
    
    // Create an Object
    File file;
    Job job;
    FileFSM fsm(file,job);
    Transfer tx;

    // Check From READY
    file.state = (File::S_READY);
    fsm.onEventStart(tx);
    CPPUNIT_ASSERT_MESSAGE("Invalid State From READY", file.state == File::S_ACTIVE);
    CPPUNIT_ASSERT_MESSAGE("Listener: Invalid State Changed", TestFileEventListener::instance().stateChanged() == true);
    CPPUNIT_ASSERT_MESSAGE("Listener: Invalid Ready", TestFileEventListener::instance().ready() == false);
    CPPUNIT_ASSERT_MESSAGE("Listener: Invalid Done" , TestFileEventListener::instance().done() == false);
    CPPUNIT_ASSERT_MESSAGE("Listener: Invalid Finished", TestFileEventListener::instance().finished() == false);

}

/*
 * Test OnEventStart Method
 */
void FileFSMTestCase::testOnEventStart_Error(){
    // Create an Object
    File file;
    Job job;
    FileFSM fsm(file,job);
    Transfer tx;

    // Define Bad Cases
    typedef std::vector<File::State> BadCaseSet;
    BadCaseSet           bad_cases;
    BadCaseSet::iterator b_it;
    bad_cases.push_back(File::S_UNDEF);
    bad_cases.push_back(File::S_SUBMITTED);
    bad_cases.push_back(File::S_PENDING);
    bad_cases.push_back(File::S_ACTIVE);
    bad_cases.push_back(File::S_DONE);
    bad_cases.push_back(File::S_WAITING);
    bad_cases.push_back(File::S_HOLD);
    bad_cases.push_back(File::S_FAILED);
    bad_cases.push_back(File::S_AWAITING_PRESTAGE);
    bad_cases.push_back(File::S_PRESTAGING);
    bad_cases.push_back(File::S_WAITING_PRESTAGE);
    bad_cases.push_back(File::S_WAITING_CATALOG_RESOLUTION);
    bad_cases.push_back(File::S_WAITING_CATALOG_REGISTRATION);
    bad_cases.push_back(File::S_CANCELED);
    bad_cases.push_back(File::S_FINISHED);
    bad_cases.push_back(File::S_FINISHING);
    
    for(b_it = bad_cases.begin(); b_it != bad_cases.end(); ++b_it){
        try{
            // Check From invalid State
            file.state = (*b_it);
            fsm.onEventStart(tx);
            std::stringstream error;
            error << "State " << (*b_it) << " should give an exception";
            CPPUNIT_FAIL(error.str().c_str());
        } catch(const FSMException& exc){
            CPPUNIT_ASSERT_MESSAGE("Listener: Invalid State Changed", TestFileEventListener::instance().stateChanged() == false);
            CPPUNIT_ASSERT_MESSAGE("Listener: Invalid Ready", TestFileEventListener::instance().ready() == false);
            CPPUNIT_ASSERT_MESSAGE("Listener: Invalid Done" , TestFileEventListener::instance().done() == false);
            CPPUNIT_ASSERT_MESSAGE("Listener: Invalid Finished", TestFileEventListener::instance().finished() == false);
        }
    }            

}

/*
 * Test OnEventRunning Method
 */
void FileFSMTestCase::testOnEventRunning(){
    
    // Create an Object
    File file;
    Job job;
    FileFSM fsm(file,job);
    Transfer tx;

    // Check From ACTIVE
    file.state = (File::S_ACTIVE);
    fsm.onEventRunning(tx);
    CPPUNIT_ASSERT_MESSAGE("Invalid State From ACTIVE", file.state == File::S_ACTIVE);
    CPPUNIT_ASSERT_MESSAGE("Listener: Invalid State Changed", TestFileEventListener::instance().stateChanged() == false);
    CPPUNIT_ASSERT_MESSAGE("Listener: Invalid Ready", TestFileEventListener::instance().ready() == false);
    CPPUNIT_ASSERT_MESSAGE("Listener: Invalid Done" , TestFileEventListener::instance().done() == false);
    CPPUNIT_ASSERT_MESSAGE("Listener: Invalid Finished", TestFileEventListener::instance().finished() == false);

}

/*
 * Test OnEventRunning Method
 */
void FileFSMTestCase::testOnEventRunning_Error(){
    // Create an Object
    File file;
    Job job;
    FileFSM fsm(file,job);
    Transfer tx;

    // Define Bad Cases
    typedef std::vector<File::State> BadCaseSet;
    BadCaseSet           bad_cases;
    BadCaseSet::iterator b_it;
    bad_cases.push_back(File::S_UNDEF);
    bad_cases.push_back(File::S_SUBMITTED);
    bad_cases.push_back(File::S_PENDING);
    bad_cases.push_back(File::S_READY);
    bad_cases.push_back(File::S_DONE);
    bad_cases.push_back(File::S_WAITING);
    bad_cases.push_back(File::S_HOLD);
    bad_cases.push_back(File::S_FAILED);
    bad_cases.push_back(File::S_AWAITING_PRESTAGE);
    bad_cases.push_back(File::S_PRESTAGING);
    bad_cases.push_back(File::S_WAITING_PRESTAGE);
    bad_cases.push_back(File::S_WAITING_CATALOG_RESOLUTION);
    bad_cases.push_back(File::S_WAITING_CATALOG_REGISTRATION);
    bad_cases.push_back(File::S_CANCELED);
    bad_cases.push_back(File::S_FINISHED);
    bad_cases.push_back(File::S_FINISHING);
    
    for(b_it = bad_cases.begin(); b_it != bad_cases.end(); ++b_it){
        try{
            // Check From invalid State
            file.state = (*b_it);
            fsm.onEventRunning(tx);
            std::stringstream error;
            error << "State " << (*b_it) << " should give an exception";
            CPPUNIT_FAIL(error.str().c_str());
        } catch(const FSMException& exc){
            CPPUNIT_ASSERT_MESSAGE("Listener: Invalid State Changed", TestFileEventListener::instance().stateChanged() == false);
            CPPUNIT_ASSERT_MESSAGE("Listener: Invalid Ready", TestFileEventListener::instance().ready() == false);
            CPPUNIT_ASSERT_MESSAGE("Listener: Invalid Done" , TestFileEventListener::instance().done() == false);
            CPPUNIT_ASSERT_MESSAGE("Listener: Invalid Finished", TestFileEventListener::instance().finished() == false);
        }
    }            

}

/*
 * Test the Done Event
 */
void FileFSMTestCase::testOnEventDone(){
    // Create an Object
    File file;
    Job job;
    FileFSM fsm(file,job);
    Transfer tx;

    // Check From ACTIVE
    file.state = (File::S_ACTIVE);
    fsm.onEventDone(tx);
    CPPUNIT_ASSERT_MESSAGE("Invalid State From ACTIVE", file.state == File::S_DONE);
    CPPUNIT_ASSERT_MESSAGE("Listener: Invalid State Changed", TestFileEventListener::instance().stateChanged() == true);
    CPPUNIT_ASSERT_MESSAGE("Listener: Invalid Ready", TestFileEventListener::instance().ready() == false);
    // Done is set on the done->finishing transition
    CPPUNIT_ASSERT_MESSAGE("Listener: Invalid Done" , TestFileEventListener::instance().done() == false);
    CPPUNIT_ASSERT_MESSAGE("Listener: Invalid Finished", TestFileEventListener::instance().finished() == false);

}

/*
 * Test the Finish Event that returns Error
 */
void FileFSMTestCase::testOnEventDone_Error(){
    // Create an Object
    File file;
    Job job;
    FileFSM fsm(file,job);
    Transfer tx;

    // Define Bad Cases
    typedef std::vector<File::State> BadCaseSet;
    BadCaseSet           bad_cases;
    BadCaseSet::iterator b_it;
    bad_cases.push_back(File::S_UNDEF);
    bad_cases.push_back(File::S_SUBMITTED);
    bad_cases.push_back(File::S_PENDING);
    bad_cases.push_back(File::S_READY);
    bad_cases.push_back(File::S_DONE);
    bad_cases.push_back(File::S_WAITING);
    bad_cases.push_back(File::S_HOLD);
    bad_cases.push_back(File::S_FAILED);
    bad_cases.push_back(File::S_AWAITING_PRESTAGE);
    bad_cases.push_back(File::S_PRESTAGING);
    bad_cases.push_back(File::S_WAITING_PRESTAGE);
    bad_cases.push_back(File::S_WAITING_CATALOG_RESOLUTION);
    bad_cases.push_back(File::S_WAITING_CATALOG_REGISTRATION);
    bad_cases.push_back(File::S_CANCELED);
    bad_cases.push_back(File::S_FINISHED);
    bad_cases.push_back(File::S_FINISHING);
    
    for(b_it = bad_cases.begin(); b_it != bad_cases.end(); ++b_it){
        try{
            // Check From invalid State
            file.state = (*b_it);
            fsm.onEventDone(tx);
            std::stringstream error;
            error << "State " << (*b_it) << " should give an exception";
            CPPUNIT_FAIL(error.str().c_str());
        } catch(const FSMException& exc){
            CPPUNIT_ASSERT_MESSAGE("Listener: Invalid State Changed", TestFileEventListener::instance().stateChanged() == false);
            CPPUNIT_ASSERT_MESSAGE("Listener: Invalid Ready", TestFileEventListener::instance().ready() == false);
            CPPUNIT_ASSERT_MESSAGE("Listener: Invalid Done" , TestFileEventListener::instance().done() == false);
            CPPUNIT_ASSERT_MESSAGE("Listener: Invalid Finished", TestFileEventListener::instance().finished() == false);
        }
    }            

}

/**
 * check_onerror
 * helper function to test the onEventFailed method
 */
void check_onerror(
    File&                   file,
    const File::State&      initial,
    FileFSM&                fsm,
    const Error::Category&  category,
    const Error::Scope&     scope,
    const Error::Phase&     phase,
    const File::State&      expected){
    file.state = initial;
    fsm.onEventFailed(category,scope,phase,"a reason");
    if (file.state() != expected){
        FAIL_TEST("onEventFailed transition from " << str(initial) << " failed: file state is " << str(file.state()) << " (Expected " << str(expected) << ")");
    }
    bool changed = (initial != expected);
    if (TestFileEventListener::instance().stateChanged() != changed){
        bool value = TestFileEventListener::instance().stateChanged();
        FAIL_TEST("onEventFailed transition " << str(initial) << " -> " << str(expected) << " : invalid stateChanged value in listener : " << value << " (expected " << changed << ")"); 
    }
    if (TestFileEventListener::instance().ready()){
        FAIL_TEST("onEventFailed transition " << str(initial) << " -> " << str(expected) << " : invalid ready value in listener"); 
    }
    if (TestFileEventListener::instance().done()){
        FAIL_TEST("onEventFailed transition " << str(initial) << " -> " << str(expected) << " : invalid done value in listener"); 
    }
    if (TestFileEventListener::instance().finished()){
        FAIL_TEST("onEventFailed transition " << str(initial) << " -> " << str(expected) << " : invalid finished value in listener"); 
    }
}

/**
 * Test the Failed event
 */
void FileFSMTestCase::testOnEventFailed(){
    // Create an Object
    File file;
    Job job;
    FileFSM fsm(file,job);
    
    check_onerror(file,File::S_SUBMITTED,fsm,Error::CATALOG,Error::S_SOURCE,Error::P_CATALOG_RESOLUTION,File::S_WAITING_CATALOG_RESOLUTION);
    check_onerror(file,File::S_SUBMITTED,fsm,Error::SECURITY,Error::S_SOURCE,Error::P_CATALOG_RESOLUTION,File::S_FAILED);

    check_onerror(file,File::S_PENDING,fsm,Error::INTERNAL_ERROR,Error::S_AGENT,Error::P_PRESTAGING,File::S_WAITING_PRESTAGE);
    check_onerror(file,File::S_PENDING,fsm,Error::SECURITY,Error::S_SOURCE,Error::P_PRESTAGING,File::S_WAITING_PRESTAGE);
    check_onerror(file,File::S_PENDING,fsm,Error::SECURITY,Error::S_DESTINATION,Error::P_PRESTAGING,File::S_FAILED);
    
    check_onerror(file,File::S_READY,fsm,Error::SECURITY,Error::S_DESTINATION,Error::P_TX_PREPARATION,File::S_WAITING);
    check_onerror(file,File::S_READY,fsm,Error::TRANSFER,Error::S_SOURCE,Error::P_TRANSFER,File::S_WAITING);
    check_onerror(file,File::S_READY,fsm,Error::GRIDFTP,Error::S_AGENT,Error::P_TX_FINALIZATION,File::S_WAITING);
    check_onerror(file,File::S_READY,fsm,Error::SECURITY,Error::S_AGENT,Error::P_TX_SERVICE,File::S_FAILED);

    check_onerror(file,File::S_ACTIVE,fsm,Error::SECURITY,Error::S_DESTINATION,Error::P_TX_PREPARATION,File::S_WAITING);
    check_onerror(file,File::S_ACTIVE,fsm,Error::TRANSFER,Error::S_SOURCE,Error::P_TRANSFER,File::S_WAITING);
    check_onerror(file,File::S_ACTIVE,fsm,Error::GRIDFTP,Error::S_AGENT,Error::P_TX_FINALIZATION,File::S_WAITING);
    check_onerror(file,File::S_ACTIVE,fsm,Error::SECURITY,Error::S_AGENT,Error::P_TX_SERVICE,File::S_FAILED);

    check_onerror(file,File::S_FINISHING,fsm,Error::CATALOG,Error::S_AGENT,Error::P_CATALOG_REGISTRATION,File::S_WAITING_CATALOG_REGISTRATION);
    check_onerror(file,File::S_FINISHING,fsm,Error::CATALOG_INTERNAL_ERROR,Error::S_AGENT,Error::P_CATALOG_REGISTRATION,File::S_WAITING_CATALOG_REGISTRATION);
    check_onerror(file,File::S_FINISHING,fsm,Error::SECURITY,Error::S_AGENT,Error::P_CATALOG_REGISTRATION,File::S_FAILED);
    
    check_onerror(file,File::S_PRESTAGING,fsm,Error::INTERNAL_ERROR,Error::S_AGENT,Error::P_PRESTAGING,File::S_WAITING_PRESTAGE);
    check_onerror(file,File::S_PRESTAGING,fsm,Error::SECURITY,Error::S_SOURCE,Error::P_PRESTAGING,File::S_WAITING_PRESTAGE);
    check_onerror(file,File::S_PRESTAGING,fsm,Error::SECURITY,Error::S_DESTINATION,Error::P_PRESTAGING,File::S_FAILED);
}

/*
 * Test the Reschedule Event
 */
void FileFSMTestCase::testOnEventReschedule(){
    // Create an Object
    File file;
    Job job;
    FileFSM fsm(file,job);

    // Check From WAITING
    file.state = File::S_WAITING;
    fsm.onEventReschedule();
    CPPUNIT_ASSERT_MESSAGE("Invalid State From WAITING", file.state == File::S_PENDING);
    CPPUNIT_ASSERT_MESSAGE("Listener: Invalid State Changed", TestFileEventListener::instance().stateChanged() == true);
    CPPUNIT_ASSERT_MESSAGE("Listener: Invalid Ready", TestFileEventListener::instance().ready() == false);
    CPPUNIT_ASSERT_MESSAGE("Listener: Invalid Done" , TestFileEventListener::instance().done() == false);
    CPPUNIT_ASSERT_MESSAGE("Listener: Invalid Finished", TestFileEventListener::instance().finished() == false);

    TestFileEventListener::instance().reset();
    // Check From WAITING_CATALOG_RESOLUTION
    file.state = File::S_WAITING_CATALOG_RESOLUTION;
    fsm.onEventReschedule();
    CPPUNIT_ASSERT_MESSAGE("Invalid State From WAITING_CATALOG_RESOLUTION", file.state == File::S_SUBMITTED);
    CPPUNIT_ASSERT_MESSAGE("Listener: Invalid State Changed", TestFileEventListener::instance().stateChanged() == true);
    CPPUNIT_ASSERT_MESSAGE("Listener: Invalid Ready", TestFileEventListener::instance().ready() == false);
    CPPUNIT_ASSERT_MESSAGE("Listener: Invalid Done" , TestFileEventListener::instance().done() == false);
    CPPUNIT_ASSERT_MESSAGE("Listener: Invalid Finished", TestFileEventListener::instance().finished() == false);

    TestFileEventListener::instance().reset();
    // Check From WAITING_CATALOG_REGISTRATION
    file.state = File::S_WAITING_CATALOG_REGISTRATION;
    fsm.onEventReschedule();
    CPPUNIT_ASSERT_MESSAGE("Invalid State From WAITING_CATALOG_REGISTRATION", file.state == File::S_FINISHING);
    CPPUNIT_ASSERT_MESSAGE("Listener: Invalid State Changed", TestFileEventListener::instance().stateChanged() == true);
    CPPUNIT_ASSERT_MESSAGE("Listener: Invalid Ready", TestFileEventListener::instance().ready() == false);
    CPPUNIT_ASSERT_MESSAGE("Listener: Invalid Done" , TestFileEventListener::instance().done() == false);
    CPPUNIT_ASSERT_MESSAGE("Listener: Invalid Finished", TestFileEventListener::instance().finished() == false);

    TestFileEventListener::instance().reset();
    // Check From WAITING_PRESTAGE
    file.state = File::S_WAITING_PRESTAGE;
    fsm.onEventReschedule();
    CPPUNIT_ASSERT_MESSAGE("Invalid State From WAITING_PRESTAGE", file.state == File::S_PENDING);
    CPPUNIT_ASSERT_MESSAGE("Listener: Invalid State Changed", TestFileEventListener::instance().stateChanged() == true);
    CPPUNIT_ASSERT_MESSAGE("Listener: Invalid Ready", TestFileEventListener::instance().ready() == false);
    CPPUNIT_ASSERT_MESSAGE("Listener: Invalid Done" , TestFileEventListener::instance().done() == false);
    CPPUNIT_ASSERT_MESSAGE("Listener: Invalid Finished", TestFileEventListener::instance().finished() == false);

}

/*
 * Test the Reschedule Event that returns Error
 */
void FileFSMTestCase::testOnEventReschedule_Error(){
    // Create an Object
    File file;
    Job job;
    FileFSM fsm(file,job);

    // Define Bad Cases
    typedef std::vector<File::State> BadCaseSet;
    BadCaseSet           bad_cases;
    BadCaseSet::iterator b_it;
    bad_cases.push_back(File::S_UNDEF);
    bad_cases.push_back(File::S_SUBMITTED);
    bad_cases.push_back(File::S_PENDING);
    bad_cases.push_back(File::S_READY);
    bad_cases.push_back(File::S_ACTIVE);
    bad_cases.push_back(File::S_DONE);
    bad_cases.push_back(File::S_HOLD);
    bad_cases.push_back(File::S_FAILED);
    bad_cases.push_back(File::S_AWAITING_PRESTAGE);
    bad_cases.push_back(File::S_PRESTAGING);
    bad_cases.push_back(File::S_CANCELED);
    bad_cases.push_back(File::S_FINISHING);
    bad_cases.push_back(File::S_FINISHED);
    
    for(b_it = bad_cases.begin(); b_it != bad_cases.end(); ++b_it){
        try{
            // Check From invalid State
            file.state = (*b_it);
            fsm.onEventReschedule();
            std::stringstream error;
            error << "State " << (*b_it) << " should give an exception";
            CPPUNIT_FAIL(error.str().c_str());
        } catch(const FSMException& exc){
            CPPUNIT_ASSERT_MESSAGE("Listener: Invalid State Changed", TestFileEventListener::instance().stateChanged() == false);
            CPPUNIT_ASSERT_MESSAGE("Listener: Invalid Ready", TestFileEventListener::instance().ready() == false);
            CPPUNIT_ASSERT_MESSAGE("Listener: Invalid Done" , TestFileEventListener::instance().done() == false);
            CPPUNIT_ASSERT_MESSAGE("Listener: Invalid Finished", TestFileEventListener::instance().finished() == false);
        }
    }            

}

/*
 * Test the Reschedule Event
 */
void FileFSMTestCase::testOnEventHold(){
    // Create an Object
    File file;
    Job job;
    FileFSM fsm(file,job);

    // Check From WAITING
    file.state = (File::S_WAITING);
    fsm.onEventHold();
    CPPUNIT_ASSERT_MESSAGE("Invalid State From WAITING", file.state  == File::S_HOLD);
    CPPUNIT_ASSERT_MESSAGE("Listener: Invalid State Changed", TestFileEventListener::instance().stateChanged() == true);
    CPPUNIT_ASSERT_MESSAGE("Listener: Invalid Ready", TestFileEventListener::instance().ready() == false);
    CPPUNIT_ASSERT_MESSAGE("Listener: Invalid Done" , TestFileEventListener::instance().done() == false);
    CPPUNIT_ASSERT_MESSAGE("Listener: Invalid Finished", TestFileEventListener::instance().finished() == false);

    TestFileEventListener::instance().reset();
    // Check From WAITING_PRESTAGE
    file.state = (File::S_WAITING_PRESTAGE);
    fsm.onEventHold();
    CPPUNIT_ASSERT_MESSAGE("Invalid State From WAITING_PRESTAGE", file.state  == File::S_HOLD);
    CPPUNIT_ASSERT_MESSAGE("Listener: Invalid State Changed", TestFileEventListener::instance().stateChanged() == true);
    CPPUNIT_ASSERT_MESSAGE("Listener: Invalid Ready", TestFileEventListener::instance().ready() == false);
    CPPUNIT_ASSERT_MESSAGE("Listener: Invalid Done" , TestFileEventListener::instance().done() == false);
    CPPUNIT_ASSERT_MESSAGE("Listener: Invalid Finished", TestFileEventListener::instance().finished() == false);

}

/*
 * Test the Hold Event that returns Error
 */
void FileFSMTestCase::testOnEventHold_Error(){
    // Create an Object
    File file;
    Job job;
    FileFSM fsm(file,job);

    // Define Bad Cases
    typedef std::vector<File::State> BadCaseSet;
    BadCaseSet           bad_cases;
    BadCaseSet::iterator b_it;
    bad_cases.push_back(File::S_UNDEF);
    bad_cases.push_back(File::S_SUBMITTED);
    bad_cases.push_back(File::S_PENDING);
    bad_cases.push_back(File::S_READY);
    bad_cases.push_back(File::S_ACTIVE);
    bad_cases.push_back(File::S_DONE);
    bad_cases.push_back(File::S_HOLD);
    bad_cases.push_back(File::S_FAILED);
    bad_cases.push_back(File::S_AWAITING_PRESTAGE);
    bad_cases.push_back(File::S_PRESTAGING);
    bad_cases.push_back(File::S_WAITING_CATALOG_RESOLUTION);
    bad_cases.push_back(File::S_WAITING_CATALOG_REGISTRATION);
    bad_cases.push_back(File::S_CANCELED);
    bad_cases.push_back(File::S_FINISHING);
    bad_cases.push_back(File::S_FINISHED);
    
    for(b_it = bad_cases.begin(); b_it != bad_cases.end(); ++b_it){
        try{
            // Check From invalid State
            file.state = (*b_it);
            fsm.onEventHold();
            std::stringstream error;
            error << "State " << (*b_it) << " should give an exception";
            CPPUNIT_FAIL(error.str().c_str());
        } catch(const FSMException& exc){
            CPPUNIT_ASSERT_MESSAGE("Listener: Invalid State Changed", TestFileEventListener::instance().stateChanged() == false);
            CPPUNIT_ASSERT_MESSAGE("Listener: Invalid Ready", TestFileEventListener::instance().ready() == false);
            CPPUNIT_ASSERT_MESSAGE("Listener: Invalid Done" , TestFileEventListener::instance().done() == false);
            CPPUNIT_ASSERT_MESSAGE("Listener: Invalid Finished", TestFileEventListener::instance().finished() == false);
        }
    }            

}

/*
 * Test the Canceled Event
 */
void FileFSMTestCase::testOnEventCanceled(){
    // Create an Object
    File file;
    Job job;
    FileFSM fsm(file,job);

    // Define Bad Cases
    typedef std::vector<File::State> CaseSet;
    CaseSet           states,canceled;
    CaseSet::iterator it;
    canceled.push_back(File::S_UNDEF);
    canceled.push_back(File::S_SUBMITTED);
    canceled.push_back(File::S_PENDING);
    canceled.push_back(File::S_READY);
    canceled.push_back(File::S_ACTIVE);
    canceled.push_back(File::S_DONE);
    canceled.push_back(File::S_WAITING);
    canceled.push_back(File::S_HOLD);
    states.push_back  (File::S_FAILED);
    canceled.push_back(File::S_AWAITING_PRESTAGE);
    canceled.push_back(File::S_PRESTAGING);
    canceled.push_back(File::S_WAITING_PRESTAGE);
    canceled.push_back(File::S_WAITING_CATALOG_RESOLUTION);
    canceled.push_back(File::S_WAITING_CATALOG_REGISTRATION);
    states.push_back  (File::S_CANCELED);
    canceled.push_back(File::S_FINISHING);
    states.push_back  (File::S_FINISHED);
    
    for(it = states.begin(); it != states.end(); ++it){
        TestFileEventListener::instance().reset();
        file.state = (*it);
        fsm.onEventCanceled();
        std::stringstream error;
        error << "State " << (*it) << " should not change";
        CPPUNIT_ASSERT_MESSAGE(error.str().c_str(), file.state  == (*it));
        if(file.state() == (*it)){
            CPPUNIT_ASSERT_MESSAGE("Listener: Invalid State Changed", TestFileEventListener::instance().stateChanged() == false);
        } else {
            CPPUNIT_ASSERT_MESSAGE("Listener: Invalid State Changed", TestFileEventListener::instance().stateChanged() == true);
        }
        CPPUNIT_ASSERT_MESSAGE("Listener: Invalid Ready", TestFileEventListener::instance().ready() == false);
        CPPUNIT_ASSERT_MESSAGE("Listener: Invalid Done" , TestFileEventListener::instance().done() == false);
        CPPUNIT_ASSERT_MESSAGE("Listener: Invalid Finished", TestFileEventListener::instance().finished() == false);
    }
    
    for(it = canceled.begin(); it != canceled.end(); ++it){
        TestFileEventListener::instance().reset();
        file.state = (*it);
        fsm.onEventCanceled();
        std::stringstream error;
        error << "State " << (*it) << " should be canceled";
        if(file.state() == (*it)){
            CPPUNIT_ASSERT_MESSAGE("Listener: Invalid State Changed", TestFileEventListener::instance().stateChanged() == false);
        } else {
            CPPUNIT_ASSERT_MESSAGE("Listener: Invalid State Changed", TestFileEventListener::instance().stateChanged() == true);
        }
        CPPUNIT_ASSERT_MESSAGE("Listener: Invalid State Changed", TestFileEventListener::instance().stateChanged() == true);
        CPPUNIT_ASSERT_MESSAGE("Listener: Invalid Ready", TestFileEventListener::instance().ready() == false);
        CPPUNIT_ASSERT_MESSAGE("Listener: Invalid Done" , TestFileEventListener::instance().done() == false);
        CPPUNIT_ASSERT_MESSAGE("Listener: Invalid Finished", TestFileEventListener::instance().finished() == false);
    }

}

/*
 * Test the Finishing Event
 */
void FileFSMTestCase::testOnEventFinishing(){
    // Create an Object
    File file;
    Job job;
    FileFSM fsm(file,job);

    // Check From DONE
    file.state = (File::S_DONE);
    fsm.onEventFinishing();
    CPPUNIT_ASSERT_MESSAGE("Invalid State From DONE", file.state  == File::S_FINISHING);
    CPPUNIT_ASSERT_MESSAGE("Listener: Invalid State Changed", TestFileEventListener::instance().stateChanged() == true);
    CPPUNIT_ASSERT_MESSAGE("Listener: Invalid Ready", TestFileEventListener::instance().ready() == false);
    CPPUNIT_ASSERT_MESSAGE("Listener: Invalid Done" , TestFileEventListener::instance().done() == true);
    CPPUNIT_ASSERT_MESSAGE("Listener: Invalid Finished", TestFileEventListener::instance().finished() == false);

}
/*
 * Test the Finishing Event
 */
void FileFSMTestCase::testOnEventFinishing_Error(){
    // Create an Object
    File file;
    Job job;
    FileFSM fsm(file,job);

    // Define Bad Cases
    typedef std::vector<File::State> BadCaseSet;
    BadCaseSet           bad_cases;
    BadCaseSet::iterator b_it;
    bad_cases.push_back(File::S_UNDEF);
    bad_cases.push_back(File::S_SUBMITTED);
    bad_cases.push_back(File::S_PENDING);
    bad_cases.push_back(File::S_READY);
    bad_cases.push_back(File::S_ACTIVE);
    bad_cases.push_back(File::S_WAITING);
    bad_cases.push_back(File::S_HOLD);
    bad_cases.push_back(File::S_FAILED);
    bad_cases.push_back(File::S_AWAITING_PRESTAGE);
    bad_cases.push_back(File::S_PRESTAGING);
    bad_cases.push_back(File::S_WAITING_PRESTAGE);
    bad_cases.push_back(File::S_WAITING_CATALOG_RESOLUTION);
    bad_cases.push_back(File::S_WAITING_CATALOG_REGISTRATION);
    bad_cases.push_back(File::S_CANCELED);
    bad_cases.push_back(File::S_FINISHING);
    bad_cases.push_back(File::S_FINISHED);
    
    for(b_it = bad_cases.begin(); b_it != bad_cases.end(); ++b_it){
        try{
            // Check From invalid State
            file.state = (*b_it);
            fsm.onEventFinishing();
            std::stringstream error;
            error << "State " << (*b_it) << " should give an exception";
            CPPUNIT_FAIL(error.str().c_str());
        } catch(const FSMException& exc){
            CPPUNIT_ASSERT_MESSAGE("Listener: Invalid State Changed", TestFileEventListener::instance().stateChanged() == false);
            CPPUNIT_ASSERT_MESSAGE("Listener: Invalid Ready", TestFileEventListener::instance().ready() == false);
            CPPUNIT_ASSERT_MESSAGE("Listener: Invalid Done" , TestFileEventListener::instance().done() == false);
            CPPUNIT_ASSERT_MESSAGE("Listener: Invalid Finished", TestFileEventListener::instance().finished() == false);
        }
    }            

}

/*
 * Test the Finished Event
 */
void FileFSMTestCase::testOnEventFinished(){
    // Create an Object
    File file;
    Job job;
    FileFSM fsm(file,job);

    // Check From FINISHING
    file.state = (File::S_FINISHING);
    fsm.onEventFinished();
    CPPUNIT_ASSERT_MESSAGE("Invalid State From FINISHING", file.state  == File::S_FINISHED);
    CPPUNIT_ASSERT_MESSAGE("Listener: Invalid State Changed", TestFileEventListener::instance().stateChanged() == true);
    CPPUNIT_ASSERT_MESSAGE("Listener: Invalid Ready", TestFileEventListener::instance().ready() == false);
    CPPUNIT_ASSERT_MESSAGE("Listener: Invalid Done" , TestFileEventListener::instance().done() == false);
    CPPUNIT_ASSERT_MESSAGE("Listener: Invalid Finished", TestFileEventListener::instance().finished() == true);

}

/*
 * Test the Completed Event that returns Error
 */
void FileFSMTestCase::testOnEventFinished_Error(){
    // Create an Object
    File file;
    Job job;
    FileFSM fsm(file,job);

    // Define Bad Cases
    typedef std::vector<File::State> BadCaseSet;
    BadCaseSet           bad_cases;
    BadCaseSet::iterator b_it;
    bad_cases.push_back(File::S_UNDEF);
    bad_cases.push_back(File::S_SUBMITTED);
    bad_cases.push_back(File::S_PENDING);
    bad_cases.push_back(File::S_READY);
    bad_cases.push_back(File::S_ACTIVE);
    bad_cases.push_back(File::S_DONE);
    bad_cases.push_back(File::S_WAITING);
    bad_cases.push_back(File::S_HOLD);
    bad_cases.push_back(File::S_FAILED);
    bad_cases.push_back(File::S_AWAITING_PRESTAGE);
    bad_cases.push_back(File::S_PRESTAGING);
    bad_cases.push_back(File::S_WAITING_PRESTAGE);
    bad_cases.push_back(File::S_WAITING_CATALOG_RESOLUTION);
    bad_cases.push_back(File::S_WAITING_CATALOG_REGISTRATION);
    bad_cases.push_back(File::S_CANCELED);
    bad_cases.push_back(File::S_FINISHED);
    
    for(b_it = bad_cases.begin(); b_it != bad_cases.end(); ++b_it){
        try{
            // Check From invalid State
            file.state = (*b_it);
            fsm.onEventFinished();
            std::stringstream error;
            error << "State " << (*b_it) << " should give an exception";
            CPPUNIT_FAIL(error.str().c_str());
        } catch(const FSMException& exc){
            CPPUNIT_ASSERT_MESSAGE("Listener: Invalid State Changed", TestFileEventListener::instance().stateChanged() == false);
            CPPUNIT_ASSERT_MESSAGE("Listener: Invalid Ready", TestFileEventListener::instance().ready() == false);
            CPPUNIT_ASSERT_MESSAGE("Listener: Invalid Done" , TestFileEventListener::instance().done() == false);
            CPPUNIT_ASSERT_MESSAGE("Listener: Invalid Finished", TestFileEventListener::instance().finished() == false);
        }
    }            

}
