/*
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *  GLite Data Transfer Agents: JobFSM Test Case
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *  Version info: $Id: JobFSMTestCase.cpp,v 1.6 2008/04/09 16:07:08 ptedesco Exp $
 *  Release: $Name:  $
 *
 */
#include "JobFSM.h"
#include "glite/data/transfer/agent/fsm/JobEventListener.h"

#include <sstream>
#include <iomanip>
#include <map>
#include <set>
#include <cppunit/TestCase.h>
#include <cppunit/extensions/HelperMacros.h>

using namespace glite::data::transfer::agent::fsm;

using glite::data::transfer::agent::model::File;
using glite::data::transfer::agent::model::Job;
using glite::data::transfer::agent::model::Error;

/**
 * Test JobEventListener
 */
class TestJobEventListener : public JobEventListener{
public:
    /**
     * Return singleton instance
     */
    static TestJobEventListener& instance(){
        static TestJobEventListener s_instance;
        return s_instance;
    }
    void reset(){
        m_stateChanged = false;
        m_pending      = false;
        m_finished     = false;
    }
    bool stateChanged() const { return m_stateChanged; }
    bool pending() const      { return m_pending; }
    bool finished() const     { return m_finished; }
    
    // -------------------------------------------------------------------------
    // Event Handlers
    // -------------------------------------------------------------------------
    
    /**
     * onStateChanged
     */
    virtual void onStateChanged(const Job& job, Job::State old_state) throw(){
        m_stateChanged = true;    
    };
    /**
     * onPending
     */
    virtual void onPending(const Job& job) throw(){
        m_pending = true;    
    };
    /**
     * onFinished
     */
    virtual void onFinished(const Job& job) throw(){
        m_finished = true;
    };
protected:
    /**
     * Constructor
     */
    TestJobEventListener(){
        reset();    
    }
    /**
     * Destructor
     */
    virtual ~TestJobEventListener(){}
private:

    /**
     * StateChanged Flag
     */
    bool m_stateChanged;
    /**
     * Pending Flag
     */
    bool m_pending;
    /**
     * Finished Flag
     */
    bool m_finished;
};

/**
 * CPPUnit TestCase
 */
class JobFSMTestCase : public CppUnit::TestCase{
public:
    /**
     * set-up test case
     */
    void setUp ();
private:
    // Declare CPPUnit Methods
    // -------------------------------------------------------------------------
    CPPUNIT_TEST_SUITE( JobFSMTestCase);
    CPPUNIT_TEST( testUpdateState_from_Submitted      );
    CPPUNIT_TEST( testUpdateState_from_Pending        );
    CPPUNIT_TEST( testUpdateState_from_Ready          );
    CPPUNIT_TEST( testUpdateState_from_Active         );
    CPPUNIT_TEST( testUpdateState_from_Done           );
    CPPUNIT_TEST( testUpdateState_from_DoneWithErrors );
    CPPUNIT_TEST( testUpdateState_from_Finishing      );
    CPPUNIT_TEST( testUpdateState_from_Finished       );
    CPPUNIT_TEST( testUpdateState_from_FinishedDirty  );
    CPPUNIT_TEST( testUpdateState_from_Hold           );
    CPPUNIT_TEST( testUpdateState_from_Failed         );
    CPPUNIT_TEST( testUpdateState_from_Canceled       );
    CPPUNIT_TEST( testUpdateState_canceling_from_Submitted      );
    CPPUNIT_TEST( testUpdateState_canceling_from_Pending        );
    CPPUNIT_TEST( testUpdateState_canceling_from_Ready          );
    CPPUNIT_TEST( testUpdateState_canceling_from_Active         );
    CPPUNIT_TEST( testUpdateState_canceling_from_Done           );
    CPPUNIT_TEST( testUpdateState_canceling_from_DoneWithErrors );
    CPPUNIT_TEST( testUpdateState_canceling_from_Finishing      );
    CPPUNIT_TEST( testUpdateState_canceling_from_Finished       );
    CPPUNIT_TEST( testUpdateState_canceling_from_FinishedDirty  );
    CPPUNIT_TEST( testUpdateState_canceling_from_Hold           );
    CPPUNIT_TEST( testUpdateState_canceling_from_Failed         );
    CPPUNIT_TEST_SUITE_END();

    // Test Methods
    // -------------------------------------------------------------------------
    
    /*
     * tests for updateState 
     */
    void testUpdateState_from_Submitted();
    void testUpdateState_from_Pending();
    void testUpdateState_from_Ready();
    void testUpdateState_from_Active();
    void testUpdateState_from_Done();
    void testUpdateState_from_DoneWithErrors();
    void testUpdateState_from_Finishing();
    void testUpdateState_from_Finished();
    void testUpdateState_from_FinishedDirty();
    void testUpdateState_from_Hold();
    void testUpdateState_from_Failed();
    void testUpdateState_from_Canceled();
    
    /*
     * tests for updateState when the job is Canceling
     */
    void testUpdateState_canceling_from_Submitted();
    void testUpdateState_canceling_from_Pending();
    void testUpdateState_canceling_from_Ready();
    void testUpdateState_canceling_from_Active();
    void testUpdateState_canceling_from_Done();
    void testUpdateState_canceling_from_DoneWithErrors();
    void testUpdateState_canceling_from_Finishing();
    void testUpdateState_canceling_from_Finished();
    void testUpdateState_canceling_from_FinishedDirty();
    void testUpdateState_canceling_from_Hold();
    void testUpdateState_canceling_from_Failed();
};

/*
 * Register Suit
 */
CPPUNIT_TEST_SUITE_NAMED_REGISTRATION( JobFSMTestCase, "JobFSMTestCase" );

/*
 * setUp
 *
 * set-up test case
 */
void JobFSMTestCase::setUp(){
    
    // Instantiate Listener
    TestJobEventListener::instance().reset();
}

/*
 * test updateState from a Pending State
 */
void JobFSMTestCase::testUpdateState_from_Submitted(){
    // Create an Object
    Job job;
    JobFSM fsm(job);
    
    // Define Good Cases
    typedef std::map<File::State,Job::State> CaseMap;
    typedef CaseMap::value_type Case;
    CaseMap           good_cases;
    CaseMap::iterator g_it;
    good_cases.insert(Case(File::S_SUBMITTED                 , Job::S_SUBMITTED));
    good_cases.insert(Case(File::S_WAITING_CATALOG_RESOLUTION, Job::S_SUBMITTED));
    good_cases.insert(Case(File::S_PENDING                   , Job::S_PENDING));
    good_cases.insert(Case(File::S_FAILED                    , Job::S_FAILED));
    
    // Combine everything together (maybe not a smart algorithm, but it should work)
    int good_cases_size = good_cases.size();
    for(int i= 0; i < good_cases_size; ++i){
        CaseMap other_cases;
        for(g_it = good_cases.begin(); g_it != good_cases.end(); ++g_it){
            CaseMap::iterator g1_it = g_it;
            ++g1_it;
            for(; g1_it != good_cases.end(); ++g1_it){
                File::State file_states = File::State((*g_it).first | (*g1_it).first);
                if((file_states & File::S_SUBMITTED) || 
                   (file_states & File::S_WAITING_CATALOG_RESOLUTION)){
                    other_cases.insert(Case(file_states,Job::S_SUBMITTED));
                } else {
                    other_cases.insert(Case(file_states,Job::S_PENDING));
                }
            }
        }
        good_cases.insert(other_cases.begin(),other_cases.end());
    }
    
    // Define Bad Cases
    typedef std::set<File::State> BadCaseSet;
    BadCaseSet           bad_cases;
    BadCaseSet::iterator b_it;

    bad_cases.insert(File::S_READY);
    bad_cases.insert(File::S_AWAITING_PRESTAGE);
    bad_cases.insert(File::S_PRESTAGING);
    bad_cases.insert(File::S_WAITING_PRESTAGE);
    bad_cases.insert(File::S_ACTIVE);
    bad_cases.insert(File::S_DONE);
    bad_cases.insert(File::S_WAITING);
    bad_cases.insert(File::S_HOLD);
    bad_cases.insert(File::S_FINISHING);
    bad_cases.insert(File::S_WAITING_CATALOG_REGISTRATION);
    bad_cases.insert(File::S_FINISHED);
    
    // Combine bad cases together (maybe not a smart algorithm, but it should work)
    int bad_cases_size = bad_cases.size();
    for(int i= 0; i < bad_cases_size; ++i){
        BadCaseSet other_bad_cases;
        for(b_it = bad_cases.begin(); b_it != bad_cases.end(); ++b_it){
            BadCaseSet::iterator b1_it = b_it;
            ++b1_it;
            for(; b1_it != bad_cases.end(); ++b1_it){
                File::State other_bad_case = (File::State)((*b_it) | (*b1_it));
                if(good_cases.find(other_bad_case) == good_cases.end()){
                    other_bad_cases.insert(other_bad_case);
                }
            }
        }
        bad_cases.insert(other_bad_cases.begin(),other_bad_cases.end());
    }
    
    // Combine with the good ones
    BadCaseSet other_bad_cases;
    for(g_it = good_cases.begin(); g_it != good_cases.end(); ++g_it){
        for(b_it = bad_cases.begin(); b_it != bad_cases.end(); ++b_it){
            File::State other_bad_case = (File::State)((*g_it).first | (*b_it));
            if(good_cases.find(other_bad_case) == good_cases.end()){
                other_bad_cases.insert(other_bad_case);
            }
        }
    }
    bad_cases.insert(other_bad_cases.begin(),other_bad_cases.end());

    File::State file_states;

    // Check Good Cases
    for(g_it = good_cases.begin(); g_it != good_cases.end(); ++g_it){
        TestJobEventListener::instance().reset();
        file_states = (*g_it).first;
        job.state = (Job::S_SUBMITTED);
        fsm.updateState(file_states);
        std::stringstream error;
        error << "Invalid Job State for File States " 
                               << std::hex << std::showbase << (*g_it).first
              << ". Expected " << std::hex << std::showbase << (*g_it).second 
              << ". Got "      << std::hex << std::showbase << job.state();
        CPPUNIT_ASSERT_MESSAGE(error.str().c_str(), job.state == (*g_it).second);
        if(job.state == Job::S_SUBMITTED){
            CPPUNIT_ASSERT_MESSAGE("Listener: Invalid State Changed", TestJobEventListener::instance().stateChanged() == false);
        } else {
            CPPUNIT_ASSERT_MESSAGE("Listener: Invalid State Changed", TestJobEventListener::instance().stateChanged() == true);
        }
        if(job.state == Job::S_PENDING){
            CPPUNIT_ASSERT_MESSAGE("Listener: Invalid Pending", TestJobEventListener::instance().pending() == true);
        } else {
            CPPUNIT_ASSERT_MESSAGE("Listener: Invalid Pending", TestJobEventListener::instance().pending() == false);
        }
        CPPUNIT_ASSERT_MESSAGE("Listener: Invalid Finished" , TestJobEventListener::instance().finished() == false);
    }
    
    TestJobEventListener::instance().reset();
    // Check Bad Cases
    for(b_it = bad_cases.begin(); b_it != bad_cases.end(); ++b_it){
        file_states = (*b_it);
        job.state = (Job::S_SUBMITTED);
        try{
            fsm.updateState(file_states);
            std::stringstream error;
            error << "Invalid Job State for File States " 
                               << std::hex << std::showbase << (*b_it)
                  << ". Got "  << std::hex << std::showbase << job.state();
            CPPUNIT_FAIL(error.str().c_str());
        } catch(const FSMException&){
            CPPUNIT_ASSERT_MESSAGE("Listener: Invalid State Changed", TestJobEventListener::instance().stateChanged() == false);
            CPPUNIT_ASSERT_MESSAGE("Listener: Invalid Pending", TestJobEventListener::instance().pending() == false);
            CPPUNIT_ASSERT_MESSAGE("Listener: Invalid Finished" , TestJobEventListener::instance().finished() == false);
        }
    }

}
/*
 * test updateState from a Pending State
 */
void JobFSMTestCase::testUpdateState_from_Pending(){
    // Create an Object
    Job job;
    JobFSM fsm(job);
    
    // Define Good Cases
    typedef std::map<File::State,Job::State> CaseMap;
    typedef CaseMap::value_type Case;
    CaseMap           good_cases;
    CaseMap::iterator g_it;
    good_cases.insert(Case(File::S_PENDING          , Job::S_PENDING));
    good_cases.insert(Case(File::S_READY            , Job::S_READY));
    good_cases.insert(Case(File::S_AWAITING_PRESTAGE, Job::S_PENDING));
    good_cases.insert(Case(File::S_PRESTAGING       , Job::S_PENDING));
    good_cases.insert(Case(File::S_WAITING_PRESTAGE , Job::S_PENDING));
    good_cases.insert(Case(File::S_FAILED           , Job::S_FAILED));
    good_cases.insert(Case(File::S_HOLD             , Job::S_HOLD));
    good_cases.insert(Case(File::S_DONE             , Job::S_DONE));
    
    File::State failed_mask = (File::State)(File::S_FAILED | File::S_DONE);
    File::State hold_mask   = (File::State)(File::S_FAILED | File::S_DONE | File::S_HOLD);
    
    // Combine everything together (maybe not a smart algorithm, but it should work)
    int good_cases_size = good_cases.size();
    for(int i= 0; i < good_cases_size; ++i){
        CaseMap other_cases;
        for(g_it = good_cases.begin(); g_it != good_cases.end(); ++g_it){
            CaseMap::iterator g1_it = g_it;
            ++g1_it;
            for(; g1_it != good_cases.end(); ++g1_it){
                File::State file_states = File::State((*g_it).first | (*g1_it).first);
                if((file_states & File::S_PENDING) || 
                   (file_states & File::S_AWAITING_PRESTAGE) ||
                   (file_states & File::S_PRESTAGING) ||
                   (file_states & File::S_WAITING_PRESTAGE)){
                    other_cases.insert(Case(file_states,Job::S_PENDING));
                } else if((file_states | failed_mask) == failed_mask){
                    other_cases.insert(Case(file_states,Job::S_FAILED));
                } else if((file_states | hold_mask) == hold_mask){
                    other_cases.insert(Case(file_states,Job::S_HOLD));
                } else {
                    other_cases.insert(Case(file_states,Job::S_READY));
                }
            }
        }
        good_cases.insert(other_cases.begin(),other_cases.end());
    }
    
    // Define Bad Cases
    typedef std::set<File::State> BadCaseSet;
    BadCaseSet           bad_cases;
    BadCaseSet::iterator b_it;
    
    bad_cases.insert(File::S_SUBMITTED);
    bad_cases.insert(File::S_WAITING_CATALOG_RESOLUTION);
    bad_cases.insert(File::S_ACTIVE);
    bad_cases.insert(File::S_WAITING);
    bad_cases.insert(File::S_DONE);
    bad_cases.insert(File::S_FINISHING);
    bad_cases.insert(File::S_WAITING_CATALOG_REGISTRATION);
    bad_cases.insert(File::S_FINISHED);
    
    // Combine bad cases together (maybe not a smart algorithm, but it should work)
    int bad_cases_size = bad_cases.size();
    for(int i= 0; i < bad_cases_size; ++i){
        BadCaseSet other_bad_cases;
        for(b_it = bad_cases.begin(); b_it != bad_cases.end(); ++b_it){
            BadCaseSet::iterator b1_it = b_it;
            ++b1_it;
            for(; b1_it != bad_cases.end(); ++b1_it){
                File::State other_bad_case = (File::State)((*b_it) | (*b1_it));
                if(good_cases.find(other_bad_case) == good_cases.end()){
                    other_bad_cases.insert(other_bad_case);
                }
            }
        }
        bad_cases.insert(other_bad_cases.begin(),other_bad_cases.end());
    }
    
    // Combine with the good ones
    BadCaseSet other_bad_cases;
    for(g_it = good_cases.begin(); g_it != good_cases.end(); ++g_it){
        for(b_it = bad_cases.begin(); b_it != bad_cases.end(); ++b_it){
            File::State other_bad_case = (File::State)((*g_it).first | (*b_it));
            if(good_cases.find(other_bad_case) == good_cases.end()){
                other_bad_cases.insert(other_bad_case);
            }
        }
    }
    bad_cases.insert(other_bad_cases.begin(),other_bad_cases.end());

    File::State file_states;

    // Check Good Cases
    for(g_it = good_cases.begin(); g_it != good_cases.end(); ++g_it){
        TestJobEventListener::instance().reset();
        file_states = (*g_it).first;
        if(file_states == File::S_DONE){
            // Skip it: it should be a bad case
            continue;
        }
        job.state = (Job::S_PENDING);
        fsm.updateState(file_states);
        std::stringstream error;
        error << "Invalid Job State for File States " 
                               << std::hex << std::showbase << (*g_it).first
              << ". Expected " << std::hex << std::showbase << (*g_it).second 
              << ". Got "      << std::hex << std::showbase << job.state();
        CPPUNIT_ASSERT_MESSAGE(error.str().c_str(), job.state == (*g_it).second);
        if(job.state == Job::S_PENDING){
            CPPUNIT_ASSERT_MESSAGE("Listener: Invalid State Changed", TestJobEventListener::instance().stateChanged() == false);
        } else {
            CPPUNIT_ASSERT_MESSAGE("Listener: Invalid State Changed", TestJobEventListener::instance().stateChanged() == true);
        }
        CPPUNIT_ASSERT_MESSAGE("Listener: Invalid Pending", TestJobEventListener::instance().pending() == false);
        CPPUNIT_ASSERT_MESSAGE("Listener: Invalid Finished" , TestJobEventListener::instance().finished() == false);        }
    
    TestJobEventListener::instance().reset();
    // Check Bad Cases
    for(b_it = bad_cases.begin(); b_it != bad_cases.end(); ++b_it){
        file_states = (*b_it);
        job.state = (Job::S_PENDING);
        try{
            fsm.updateState(file_states);
            std::stringstream error;
            error << "Invalid Job State for File States " 
                               << std::hex << std::showbase << (*b_it)
                  << ". Got "  << std::hex << std::showbase << job.state();
            CPPUNIT_FAIL(error.str().c_str());
        } catch(const FSMException&){
            CPPUNIT_ASSERT_MESSAGE("Listener: Invalid State Changed", TestJobEventListener::instance().stateChanged() == false);
            CPPUNIT_ASSERT_MESSAGE("Listener: Invalid Pending", TestJobEventListener::instance().pending() == false);
            CPPUNIT_ASSERT_MESSAGE("Listener: Invalid Finished" , TestJobEventListener::instance().finished() == false);
        }
    }

}

/*
 * test updateState from an Ready State
 */
void JobFSMTestCase::testUpdateState_from_Ready(){
    // Create an Object
    Job job;
    JobFSM fsm(job);
    
    // Define Good Cases
    typedef std::map<File::State,Job::State> CaseMap;
    typedef CaseMap::value_type Case;
    CaseMap           good_cases;
    CaseMap::iterator g_it;
    good_cases.insert(Case(File::S_READY    , Job::S_READY));
    good_cases.insert(Case(File::S_ACTIVE   , Job::S_ACTIVE));
    good_cases.insert(Case(File::S_DONE     , Job::S_ACTIVE));
    good_cases.insert(Case(File::S_WAITING  , Job::S_ACTIVE));
    good_cases.insert(Case(File::S_HOLD     , Job::S_HOLD));
    good_cases.insert(Case(File::S_FAILED   , Job::S_FAILED));
    
    // Combine everything together (maybe not a smart algorithm, but it should work)
    int good_cases_size = good_cases.size();
    for(int i= 0; i < good_cases_size; ++i){
        CaseMap other_cases;
        for(g_it = good_cases.begin(); g_it != good_cases.end(); ++g_it){
            CaseMap::iterator g1_it = g_it;
            ++g1_it;
            for(; g1_it != good_cases.end(); ++g1_it){
                File::State file_states = File::State((*g_it).first | (*g1_it).first);
                if(file_states == File::State(File::S_DONE | File::S_FAILED)){
                    other_cases.insert(Case(file_states,Job::S_ACTIVE));
                } else if(file_states == File::State(File::S_DONE | File::S_HOLD)){
                    other_cases.insert(Case(file_states,Job::S_HOLD));
                } else if(file_states == File::State(File::S_FAILED | File::S_HOLD)){
                    other_cases.insert(Case(file_states,Job::S_HOLD));
                } else if(file_states == File::State( File::S_DONE | File::S_FAILED | File::S_HOLD)){
                    other_cases.insert(Case(file_states,Job::S_HOLD));
                } else {
                    other_cases.insert(Case(file_states,Job::S_ACTIVE));
                }
            }
        }
        good_cases.insert(other_cases.begin(),other_cases.end());
    }
    
    // Define Bad Cases
    typedef std::set<File::State> BadCaseSet;
    BadCaseSet           bad_cases;
    BadCaseSet::iterator b_it;
    
    bad_cases.insert(File::S_SUBMITTED);
    bad_cases.insert(File::S_WAITING_CATALOG_RESOLUTION);
    bad_cases.insert(File::S_WAITING_CATALOG_REGISTRATION);
    bad_cases.insert(File::S_PENDING);
    bad_cases.insert(File::S_AWAITING_PRESTAGE);
    bad_cases.insert(File::S_PRESTAGING);
    bad_cases.insert(File::S_WAITING_PRESTAGE);
    bad_cases.insert(File::S_FINISHING);
    bad_cases.insert(File::S_FINISHED);
    
    // Combine bad cases together (maybe not a smart algorithm, but it should work)
    int bad_cases_size = bad_cases.size();
    for(int i= 0; i < bad_cases_size; ++i){
        BadCaseSet other_bad_cases;
        for(b_it = bad_cases.begin(); b_it != bad_cases.end(); ++b_it){
            BadCaseSet::iterator b1_it = b_it;
            ++b1_it;
            for(; b1_it != bad_cases.end(); ++b1_it){
                File::State other_bad_case = (File::State)((*b_it) | (*b1_it));
                if(good_cases.find(other_bad_case) == good_cases.end()){
                    other_bad_cases.insert(other_bad_case);
                }
            }
        }
        bad_cases.insert(other_bad_cases.begin(),other_bad_cases.end());
    }
    
    // Combine with the good ones
    BadCaseSet other_bad_cases;
    for(g_it = good_cases.begin(); g_it != good_cases.end(); ++g_it){
        for(b_it = bad_cases.begin(); b_it != bad_cases.end(); ++b_it){
            File::State other_bad_case = (File::State)((*g_it).first | (*b_it));
            if(good_cases.find(other_bad_case) == good_cases.end()){
                other_bad_cases.insert(other_bad_case);
            }
        }
    }
    bad_cases.insert(other_bad_cases.begin(), other_bad_cases.end());

    File::State file_states;

    // Check Good Cases
    for(g_it = good_cases.begin(); g_it != good_cases.end(); ++g_it){
        TestJobEventListener::instance().reset();
        file_states = (*g_it).first;
        job.state = (Job::S_READY);
        fsm.updateState(file_states);
        std::stringstream error;
        error << "Invalid Job State for File States " 
                               << std::hex << std::showbase << (*g_it).first
              << ". Expected " << std::hex << std::showbase << (*g_it).second 
              << ". Got "      << std::hex << std::showbase << job.state();
        CPPUNIT_ASSERT_MESSAGE(error.str().c_str(), job.state == (*g_it).second);
        if(job.state == Job::S_READY){
            CPPUNIT_ASSERT_MESSAGE("Listener: Invalid State Changed", TestJobEventListener::instance().stateChanged() == false);
        } else {
            CPPUNIT_ASSERT_MESSAGE("Listener: Invalid State Changed", TestJobEventListener::instance().stateChanged() == true);
        }
        CPPUNIT_ASSERT_MESSAGE("Listener: Invalid Pending", TestJobEventListener::instance().pending() == false);
        CPPUNIT_ASSERT_MESSAGE("Listener: Invalid Finished" , TestJobEventListener::instance().finished() == false);
    }
    TestJobEventListener::instance().reset();
    // Check Bad Cases
    for(b_it = bad_cases.begin(); b_it != bad_cases.end(); ++b_it){
        file_states = (*b_it);
        job.state = (Job::S_READY);
        try{
            fsm.updateState(file_states);
            std::stringstream error;
            error << "Invalid Job State for File States " 
                               << std::hex << std::showbase << (*b_it)
                  << ". Got "  << std::hex << std::showbase << job.state();
            CPPUNIT_FAIL(error.str().c_str());
        } catch(const FSMException&){
            CPPUNIT_ASSERT_MESSAGE("Listener: Invalid State Changed", TestJobEventListener::instance().stateChanged() == false);
            CPPUNIT_ASSERT_MESSAGE("Listener: Invalid Pending", TestJobEventListener::instance().pending() == false);
            CPPUNIT_ASSERT_MESSAGE("Listener: Invalid Finished" , TestJobEventListener::instance().finished() == false);
        }
    }

}

/*
 * test updateState from an Active State
 */
void JobFSMTestCase::testUpdateState_from_Active(){
    // Create an Object
    Job job;
    JobFSM fsm(job);
    
    // Define Good Cases
    typedef std::map<File::State,Job::State> CaseMap;
    typedef CaseMap::value_type Case;
    CaseMap           good_cases;
    CaseMap::iterator g_it;
    good_cases.insert(Case(File::S_PENDING          , Job::S_ACTIVE));
    good_cases.insert(Case(File::S_AWAITING_PRESTAGE, Job::S_ACTIVE));
    good_cases.insert(Case(File::S_PRESTAGING       , Job::S_ACTIVE));
    good_cases.insert(Case(File::S_WAITING_PRESTAGE , Job::S_ACTIVE));
    good_cases.insert(Case(File::S_ACTIVE           , Job::S_ACTIVE));
    good_cases.insert(Case(File::S_READY            , Job::S_ACTIVE));
    good_cases.insert(Case(File::S_DONE             , Job::S_DONE));
    good_cases.insert(Case(File::S_WAITING          , Job::S_ACTIVE));
    good_cases.insert(Case(File::S_HOLD             , Job::S_HOLD));
    good_cases.insert(Case(File::S_FAILED           , Job::S_FAILED));
    
    // Combine everything together (maybe not a smart algorithm, but it should work)
    int good_cases_size = good_cases.size();
    for(int i= 0; i < good_cases_size; ++i){
        CaseMap other_cases;
        for(g_it = good_cases.begin(); g_it != good_cases.end(); ++g_it){
            CaseMap::iterator g1_it = g_it;
            ++g1_it;
            for(; g1_it != good_cases.end(); ++g1_it){
                File::State file_states = File::State((*g_it).first | (*g1_it).first);
                if(file_states == File::State(File::S_DONE | File::S_FAILED)){
                    other_cases.insert(Case(file_states,Job::S_DONE_WITH_ERRORS));
                } else if(file_states == File::State(File::S_DONE | File::S_HOLD)){
                    other_cases.insert(Case(file_states,Job::S_HOLD));
                } else if(file_states == File::State(File::S_FAILED | File::S_HOLD)){
                    other_cases.insert(Case(file_states,Job::S_HOLD));
                } else if(file_states == File::State( File::S_DONE | File::S_FAILED | File::S_HOLD)){
                    other_cases.insert(Case(file_states,Job::S_HOLD));
                } else {
                    other_cases.insert(Case(file_states,Job::S_ACTIVE));
                }
            }
        }
        good_cases.insert(other_cases.begin(),other_cases.end());
    }
    
    // Define Bad Cases
    typedef std::set<File::State> BadCaseSet;
    BadCaseSet           bad_cases;
    BadCaseSet::iterator b_it;
    
    bad_cases.insert(File::S_SUBMITTED);
    bad_cases.insert(File::S_WAITING_CATALOG_RESOLUTION);
    bad_cases.insert(File::S_FINISHING);
    bad_cases.insert(File::S_WAITING_CATALOG_REGISTRATION);
    bad_cases.insert(File::S_FINISHED);
    
    // Combine bad cases together (maybe not a smart algorithm, but it should work)
    int bad_cases_size = bad_cases.size();
    for(int i= 0; i < bad_cases_size; ++i){
        BadCaseSet other_bad_cases;
        for(b_it = bad_cases.begin(); b_it != bad_cases.end(); ++b_it){
            BadCaseSet::iterator b1_it = b_it;
            ++b1_it;
            for(; b1_it != bad_cases.end(); ++b1_it){
                File::State other_bad_case = (File::State)((*b_it) | (*b1_it));
                if(good_cases.find(other_bad_case) == good_cases.end()){
                    other_bad_cases.insert(other_bad_case);
                }
            }
        }
        bad_cases.insert(other_bad_cases.begin(),other_bad_cases.end());
    }
    
    // Combine with the good ones
    BadCaseSet other_bad_cases;
    for(g_it = good_cases.begin(); g_it != good_cases.end(); ++g_it){
        for(b_it = bad_cases.begin(); b_it != bad_cases.end(); ++b_it){
            File::State other_bad_case = (File::State)((*g_it).first | (*b_it));
            if(good_cases.find(other_bad_case) == good_cases.end()){
                other_bad_cases.insert(other_bad_case);
            }
        }
    }
    bad_cases.insert(other_bad_cases.begin(), other_bad_cases.end());

    File::State file_states;

    // Check Good Cases
    for(g_it = good_cases.begin(); g_it != good_cases.end(); ++g_it){
        TestJobEventListener::instance().reset();
        file_states = (*g_it).first;
        job.state = (Job::S_ACTIVE);
        fsm.updateState(file_states);
        std::stringstream error;
        error << "Invalid Job State for File States " 
                               << std::hex << std::showbase << (*g_it).first
              << ". Expected " << std::hex << std::showbase << (*g_it).second 
              << ". Got "      << std::hex << std::showbase << job.state();
        CPPUNIT_ASSERT_MESSAGE(error.str().c_str(), job.state == (*g_it).second);
        if(job.state == Job::S_ACTIVE){
            CPPUNIT_ASSERT_MESSAGE("Listener: Invalid State Changed", TestJobEventListener::instance().stateChanged() == false);
        } else {
            CPPUNIT_ASSERT_MESSAGE("Listener: Invalid State Changed", TestJobEventListener::instance().stateChanged() == true);
        }
        CPPUNIT_ASSERT_MESSAGE("Listener: Invalid Pending", TestJobEventListener::instance().pending() == false);
        CPPUNIT_ASSERT_MESSAGE("Listener: Invalid Finished" , TestJobEventListener::instance().finished() == false);
    }
    
    TestJobEventListener::instance().reset();
    // Check Bad Cases
    for(b_it = bad_cases.begin(); b_it != bad_cases.end(); ++b_it){
        file_states = (*b_it);
        job.state = (Job::S_ACTIVE);
        try{
            fsm.updateState(file_states);
            std::stringstream error;
            error << "Invalid Job State for File States " 
                               << std::hex << std::showbase << (*b_it)
                  << ". Got "  << std::hex << std::showbase << job.state();
            CPPUNIT_FAIL(error.str().c_str());
        } catch(const FSMException&){
            CPPUNIT_ASSERT_MESSAGE("Listener: Invalid State Changed", TestJobEventListener::instance().stateChanged() == false);
            CPPUNIT_ASSERT_MESSAGE("Listener: Invalid Pending", TestJobEventListener::instance().pending() == false);
            CPPUNIT_ASSERT_MESSAGE("Listener: Invalid Finished" , TestJobEventListener::instance().finished() == false);
        }
    }

}

/*
 * test updateState from a Done State
 */
void JobFSMTestCase::testUpdateState_from_Done(){
    // Create an Object
    Job job;
    JobFSM fsm(job);
    
    // Define Good Cases
    typedef std::map<File::State,Job::State> CaseMap;
    typedef CaseMap::value_type Case;
    CaseMap           good_cases;
    CaseMap::iterator g_it;
    good_cases.insert(Case(File::S_DONE     , Job::S_DONE));  
    good_cases.insert(Case(File::S_FINISHING, Job::S_FINISHING));  

    // Combine everything together (maybe not a smart algorithm, but it should work)
    int good_cases_size = good_cases.size();
    for(int i= 0; i < good_cases_size; ++i){
        CaseMap other_cases;
        for(g_it = good_cases.begin(); g_it != good_cases.end(); ++g_it){
            CaseMap::iterator g1_it = g_it;
            ++g1_it;
            for(; g1_it != good_cases.end(); ++g1_it){
                File::State file_states = File::State((*g_it).first | (*g1_it).first);
                other_cases.insert(Case(file_states,Job::S_FINISHING));
            }
        }
        good_cases.insert(other_cases.begin(),other_cases.end());
    }
        
    // Define Bad Cases
    typedef std::set<File::State> BadCaseSet;
    BadCaseSet           bad_cases;
    BadCaseSet::iterator b_it;
    
    bad_cases.insert(File::S_SUBMITTED);
    bad_cases.insert(File::S_WAITING_CATALOG_RESOLUTION);
    bad_cases.insert(File::S_PENDING);
    bad_cases.insert(File::S_AWAITING_PRESTAGE);
    bad_cases.insert(File::S_PRESTAGING);
    bad_cases.insert(File::S_WAITING_PRESTAGE);
    bad_cases.insert(File::S_READY);
    bad_cases.insert(File::S_ACTIVE);
    bad_cases.insert(File::S_WAITING);
    bad_cases.insert(File::S_HOLD);
    bad_cases.insert(File::S_FAILED);
    bad_cases.insert(File::S_WAITING_CATALOG_REGISTRATION);
    bad_cases.insert(File::S_FINISHED);
    
    // Combine bad cases together (maybe not a smart algorithm, but it should work)
    int bad_cases_size = bad_cases.size();
    for(int i= 0; i < bad_cases_size; ++i){
        BadCaseSet other_bad_cases;
        for(b_it = bad_cases.begin(); b_it != bad_cases.end(); ++b_it){
            BadCaseSet::iterator b1_it = b_it;
            ++b1_it;
            for(; b1_it != bad_cases.end(); ++b1_it){
                File::State other_bad_case = (File::State)((*b_it) | (*b1_it));
                if(good_cases.find(other_bad_case) == good_cases.end()){
                    other_bad_cases.insert(other_bad_case);
                }
            }
        }
        bad_cases.insert(other_bad_cases.begin(),other_bad_cases.end());
    }
    
    // Combine with the good ones
    BadCaseSet other_bad_cases;
    for(g_it = good_cases.begin(); g_it != good_cases.end(); ++g_it){
        for(b_it = bad_cases.begin(); b_it != bad_cases.end(); ++b_it){
            File::State other_bad_case = (File::State)((*g_it).first | (*b_it));
            if(good_cases.find(other_bad_case) == good_cases.end()){
                other_bad_cases.insert(other_bad_case);
            }
        }
    }
    bad_cases.insert(other_bad_cases.begin(),other_bad_cases.end());

    File::State file_states;

    // Check Good Cases
    for(g_it = good_cases.begin(); g_it != good_cases.end(); ++g_it){
        TestJobEventListener::instance().reset();
        file_states = (*g_it).first;
        job.state = (Job::S_DONE);
        fsm.updateState(file_states);
        std::stringstream error;
        error << "Invalid Job State for File States " 
                               << std::hex << std::showbase << (*g_it).first
              << ". Expected " << std::hex << std::showbase << (*g_it).second 
              << ". Got "      << std::hex << std::showbase << job.state();
        CPPUNIT_ASSERT_MESSAGE(error.str().c_str(), job.state == (*g_it).second);
        if(job.state == Job::S_DONE){
            CPPUNIT_ASSERT_MESSAGE("Listener: Invalid State Changed", TestJobEventListener::instance().stateChanged() == false);
        } else {
            CPPUNIT_ASSERT_MESSAGE("Listener: Invalid State Changed", TestJobEventListener::instance().stateChanged() == true);
        }
        CPPUNIT_ASSERT_MESSAGE("Listener: Invalid Pending", TestJobEventListener::instance().pending() == false);
        CPPUNIT_ASSERT_MESSAGE("Listener: Invalid Finished" , TestJobEventListener::instance().finished() == false);
    }
    
    TestJobEventListener::instance().reset();
    // Check Bad Cases
    for(b_it = bad_cases.begin(); b_it != bad_cases.end(); ++b_it){
        file_states = (*b_it);
        job.state = (Job::S_DONE);
        try{
            fsm.updateState(file_states);
            std::stringstream error;
            error << "Invalid Job State for File States " 
                               << std::hex << std::showbase << (*b_it)
                  << ". Got "  << std::hex << std::showbase << job.state();
            CPPUNIT_FAIL(error.str().c_str());
        } catch(const FSMException&){
            CPPUNIT_ASSERT_MESSAGE("Listener: Invalid State Changed", TestJobEventListener::instance().stateChanged() == false);
            CPPUNIT_ASSERT_MESSAGE("Listener: Invalid Pending", TestJobEventListener::instance().pending() == false);
            CPPUNIT_ASSERT_MESSAGE("Listener: Invalid Finished" , TestJobEventListener::instance().finished() == false);
        }
    }

}

/*
 * test updateState from an DoneWithErrors State
 */
void JobFSMTestCase::testUpdateState_from_DoneWithErrors(){
    // Create an Object
    Job job;
    JobFSM fsm(job);
    
    // Define Good Cases
    typedef std::map<File::State,Job::State> CaseMap;
    typedef CaseMap::value_type Case;
    CaseMap           good_cases;
    CaseMap::iterator g_it;
    good_cases.insert(Case(File::S_DONE     , Job::S_DONE_WITH_ERRORS));  
    good_cases.insert(Case(File::S_FAILED   , Job::S_FAILED));  
    good_cases.insert(Case(File::S_FINISHING, Job::S_FINISHING));  

    // Combine everything together (maybe not a smart algorithm, but it should work)
    int good_cases_size = good_cases.size();
    for(int i= 0; i < good_cases_size; ++i){
        CaseMap other_cases;
        for(g_it = good_cases.begin(); g_it != good_cases.end(); ++g_it){
            CaseMap::iterator g1_it = g_it;
            ++g1_it;
            for(; g1_it != good_cases.end(); ++g1_it){
                File::State file_states = File::State((*g_it).first | (*g1_it).first);
                if(file_states & File::S_FINISHING){
                    other_cases.insert(Case(file_states,Job::S_FINISHING));
                } else {
                    other_cases.insert(Case(file_states,Job::S_DONE_WITH_ERRORS));
                }
            }
        }
        good_cases.insert(other_cases.begin(),other_cases.end());
    }
        
    // Define Bad Cases
    typedef std::set<File::State> BadCaseSet;
    BadCaseSet           bad_cases;
    BadCaseSet::iterator b_it;
    
    bad_cases.insert(File::S_SUBMITTED);
    bad_cases.insert(File::S_WAITING_CATALOG_RESOLUTION);
    bad_cases.insert(File::S_PENDING);
    bad_cases.insert(File::S_AWAITING_PRESTAGE);
    bad_cases.insert(File::S_PRESTAGING);
    bad_cases.insert(File::S_WAITING_PRESTAGE);
    bad_cases.insert(File::S_READY);
    bad_cases.insert(File::S_ACTIVE);
    bad_cases.insert(File::S_WAITING);
    bad_cases.insert(File::S_HOLD);
    bad_cases.insert(File::S_WAITING_CATALOG_REGISTRATION);
    bad_cases.insert(File::S_FINISHED);
    
    // Combine bad cases together (maybe not a smart algorithm, but it should work)
    int bad_cases_size = bad_cases.size();
    for(int i= 0; i < bad_cases_size; ++i){
        BadCaseSet other_bad_cases;
        for(b_it = bad_cases.begin(); b_it != bad_cases.end(); ++b_it){
            BadCaseSet::iterator b1_it = b_it;
            ++b1_it;
            for(; b1_it != bad_cases.end(); ++b1_it){
                File::State other_bad_case = (File::State)((*b_it) | (*b1_it));
                if(good_cases.find(other_bad_case) == good_cases.end()){
                    other_bad_cases.insert(other_bad_case);
                }
            }
        }
        bad_cases.insert(other_bad_cases.begin(),other_bad_cases.end());
    }
    
    // Combine with the good ones
    BadCaseSet other_bad_cases;
    for(g_it = good_cases.begin(); g_it != good_cases.end(); ++g_it){
        for(b_it = bad_cases.begin(); b_it != bad_cases.end(); ++b_it){
            File::State other_bad_case = (File::State)((*g_it).first | (*b_it));
            if(good_cases.find(other_bad_case) == good_cases.end()){
                other_bad_cases.insert(other_bad_case);
            }
        }
    }
    bad_cases.insert(other_bad_cases.begin(),other_bad_cases.end());

    File::State file_states;

    // Check Good Cases
    for(g_it = good_cases.begin(); g_it != good_cases.end(); ++g_it){
        TestJobEventListener::instance().reset();
        file_states = (*g_it).first;
        job.state = (Job::S_DONE_WITH_ERRORS);
        fsm.updateState(file_states);
        std::stringstream error;
        error << "Invalid Job State for File States " 
                               << std::hex << std::showbase << (*g_it).first
              << ". Expected " << std::hex << std::showbase << (*g_it).second 
              << ". Got "      << std::hex << std::showbase << job.state();
        CPPUNIT_ASSERT_MESSAGE(error.str().c_str(), job.state == (*g_it).second);
        if(job.state == Job::S_DONE_WITH_ERRORS){
            CPPUNIT_ASSERT_MESSAGE("Listener: Invalid State Changed", TestJobEventListener::instance().stateChanged() == false);
        } else {
            CPPUNIT_ASSERT_MESSAGE("Listener: Invalid State Changed", TestJobEventListener::instance().stateChanged() == true);
        }
        CPPUNIT_ASSERT_MESSAGE("Listener: Invalid Pending", TestJobEventListener::instance().pending() == false);
        CPPUNIT_ASSERT_MESSAGE("Listener: Invalid Finished" , TestJobEventListener::instance().finished() == false);
    }
    
    TestJobEventListener::instance().reset();
    // Check Bad Cases
    for(b_it = bad_cases.begin(); b_it != bad_cases.end(); ++b_it){
        file_states = (*b_it);
        job.state = (Job::S_DONE_WITH_ERRORS);
        try{
            fsm.updateState(file_states);
            std::stringstream error;
            error << "Invalid Job State for File States " 
                               << std::hex << std::showbase << (*b_it)
                  << ". Got "  << std::hex << std::showbase << job.state();
            CPPUNIT_FAIL(error.str().c_str());
        } catch(const FSMException&){
            CPPUNIT_ASSERT_MESSAGE("Listener: Invalid State Changed", TestJobEventListener::instance().stateChanged() == false);
            CPPUNIT_ASSERT_MESSAGE("Listener: Invalid Pending", TestJobEventListener::instance().pending() == false);
            CPPUNIT_ASSERT_MESSAGE("Listener: Invalid Finished" , TestJobEventListener::instance().finished() == false);
        }
    }

}

/*
 * test updateState from a Finishing State
 */
void JobFSMTestCase::testUpdateState_from_Finishing(){
    // Create an Object
    Job job;
    JobFSM fsm(job);
    
    // Define Good Cases
    typedef std::map<File::State,Job::State> CaseMap;
    typedef CaseMap::value_type Case;
    CaseMap           good_cases;
    CaseMap::iterator g_it;
    good_cases.insert(Case(File::S_FINISHING                   , Job::S_FINISHING));  
    good_cases.insert(Case(File::S_WAITING_CATALOG_REGISTRATION, Job::S_FINISHING));  
    good_cases.insert(Case(File::S_FAILED                      , Job::S_FINISHED_DIRTY));  
    good_cases.insert(Case(File::S_FINISHED                    , Job::S_FINISHED));  

    // Combine everything together (maybe not a smart algorithm, but it should work)
    int good_cases_size = good_cases.size();
    for(int i= 0; i < good_cases_size; ++i){
        CaseMap other_cases;
        for(g_it = good_cases.begin(); g_it != good_cases.end(); ++g_it){
            CaseMap::iterator g1_it = g_it;
            ++g1_it;
            for(; g1_it != good_cases.end(); ++g1_it){
                File::State file_states = File::State((*g_it).first | (*g1_it).first);
                if(file_states == (File::S_FINISHED | File::S_FAILED)){
                    other_cases.insert(Case(file_states,Job::S_FINISHED_DIRTY));
                } else {
                    other_cases.insert(Case(file_states,Job::S_FINISHING));
                }
            }
        }
        good_cases.insert(other_cases.begin(),other_cases.end());
    }
        
    // Define Bad Cases
    typedef std::set<File::State> BadCaseSet;
    BadCaseSet           bad_cases;
    BadCaseSet::iterator b_it;
    
    bad_cases.insert(File::S_SUBMITTED);
    bad_cases.insert(File::S_WAITING_CATALOG_RESOLUTION);
    bad_cases.insert(File::S_PENDING);
    bad_cases.insert(File::S_AWAITING_PRESTAGE);
    bad_cases.insert(File::S_PRESTAGING);
    bad_cases.insert(File::S_WAITING_PRESTAGE);
    bad_cases.insert(File::S_READY);
    bad_cases.insert(File::S_ACTIVE);
    bad_cases.insert(File::S_DONE);
    bad_cases.insert(File::S_WAITING);
    bad_cases.insert(File::S_HOLD);
    
    // Combine bad cases together (maybe not a smart algorithm, but it should work)
    int bad_cases_size = bad_cases.size();
    for(int i= 0; i < bad_cases_size; ++i){
        BadCaseSet other_bad_cases;
        for(b_it = bad_cases.begin(); b_it != bad_cases.end(); ++b_it){
            BadCaseSet::iterator b1_it = b_it;
            ++b1_it;
            for(; b1_it != bad_cases.end(); ++b1_it){
                File::State other_bad_case = (File::State)((*b_it) | (*b1_it));
                if(good_cases.find(other_bad_case) == good_cases.end()){
                    other_bad_cases.insert(other_bad_case);
                }
            }
        }
        bad_cases.insert(other_bad_cases.begin(),other_bad_cases.end());
    }
    
    // Combine with the good ones
    BadCaseSet other_bad_cases;
    for(g_it = good_cases.begin(); g_it != good_cases.end(); ++g_it){
        for(b_it = bad_cases.begin(); b_it != bad_cases.end(); ++b_it){
            File::State other_bad_case = (File::State)((*g_it).first | (*b_it));
            if(good_cases.find(other_bad_case) == good_cases.end()){
                other_bad_cases.insert(other_bad_case);
            }
        }
    }
    bad_cases.insert(other_bad_cases.begin(),other_bad_cases.end());

    File::State file_states;

    // Check Good Cases
    for(g_it = good_cases.begin(); g_it != good_cases.end(); ++g_it){
        TestJobEventListener::instance().reset();
        file_states = (*g_it).first;
        job.state = (Job::S_FINISHING);
        fsm.updateState(file_states);
        std::stringstream error;
        error << "Invalid Job State for File States " 
                               << std::hex << std::showbase << (*g_it).first
              << ". Expected " << std::hex << std::showbase << (*g_it).second 
              << ". Got "      << std::hex << std::showbase << job.state();
        CPPUNIT_ASSERT_MESSAGE(error.str().c_str(), job.state == (*g_it).second);
        if(job.state == Job::S_FINISHING){
            CPPUNIT_ASSERT_MESSAGE("Listener: Invalid State Changed", TestJobEventListener::instance().stateChanged() == false);
        } else {
            CPPUNIT_ASSERT_MESSAGE("Listener: Invalid State Changed", TestJobEventListener::instance().stateChanged() == true);
        }
        CPPUNIT_ASSERT_MESSAGE("Listener: Invalid Pending", TestJobEventListener::instance().pending() == false);
        if(job.state == Job::S_FINISHED || job.state == Job::S_FINISHED_DIRTY){
            CPPUNIT_ASSERT_MESSAGE("Listener: Invalid Finished" , TestJobEventListener::instance().finished() == true);
        } else {
            CPPUNIT_ASSERT_MESSAGE("Listener: Invalid Finished" , TestJobEventListener::instance().finished() == false);
        }
    }
    
    TestJobEventListener::instance().reset();
    // Check Bad Cases
    for(b_it = bad_cases.begin(); b_it != bad_cases.end(); ++b_it){
        file_states = (*b_it);
        job.state = (Job::S_FINISHING);
        try{
            fsm.updateState(file_states);
            std::stringstream error;
            error << "Invalid Job State for File States " 
                               << std::hex << std::showbase << (*b_it)
                  << ". Got "  << std::hex << std::showbase << job.state();
            CPPUNIT_FAIL(error.str().c_str());
        } catch(const FSMException&){
            CPPUNIT_ASSERT_MESSAGE("Listener: Invalid State Changed", TestJobEventListener::instance().stateChanged() == false);
            CPPUNIT_ASSERT_MESSAGE("Listener: Invalid Pending", TestJobEventListener::instance().pending() == false);
            CPPUNIT_ASSERT_MESSAGE("Listener: Invalid Finished" , TestJobEventListener::instance().finished() == false);
        }
    }

}

/*
 * test updateState from a Finished State
 */
void JobFSMTestCase::testUpdateState_from_Finished(){
    // Create an Object
    Job job;
    JobFSM fsm(job);
    
    // Define Good Cases
    typedef std::map<File::State,Job::State> CaseMap;
    typedef CaseMap::value_type Case;
    CaseMap           good_cases;
    CaseMap::iterator g_it;
    good_cases.insert(Case(File::S_FINISHED, Job::S_FINISHED));  
        
    // Define Bad Cases
    typedef std::set<File::State> BadCaseSet;
    BadCaseSet           bad_cases;
    BadCaseSet::iterator b_it;
    
    bad_cases.insert(File::S_SUBMITTED);
    bad_cases.insert(File::S_WAITING_CATALOG_RESOLUTION);
    bad_cases.insert(File::S_PENDING);
    bad_cases.insert(File::S_AWAITING_PRESTAGE);
    bad_cases.insert(File::S_PRESTAGING);
    bad_cases.insert(File::S_WAITING_PRESTAGE);
    bad_cases.insert(File::S_READY);
    bad_cases.insert(File::S_ACTIVE);
    bad_cases.insert(File::S_DONE);
    bad_cases.insert(File::S_WAITING);
    bad_cases.insert(File::S_HOLD);
    bad_cases.insert(File::S_FAILED);
    bad_cases.insert(File::S_FINISHING);
    bad_cases.insert(File::S_WAITING_CATALOG_REGISTRATION);
    
    // Combine bad cases together (maybe not a smart algorithm, but it should work)
    int bad_cases_size = bad_cases.size();
    for(int i= 0; i < bad_cases_size; ++i){
        BadCaseSet other_bad_cases;
        for(b_it = bad_cases.begin(); b_it != bad_cases.end(); ++b_it){
            BadCaseSet::iterator b1_it = b_it;
            ++b1_it;
            for(; b1_it != bad_cases.end(); ++b1_it){
                File::State other_bad_case = (File::State)((*b_it) | (*b1_it));
                if(good_cases.find(other_bad_case) == good_cases.end()){
                    other_bad_cases.insert(other_bad_case);
                }
            }
        }
        bad_cases.insert(other_bad_cases.begin(),other_bad_cases.end());
    }
    
    // Combine with the good ones
    BadCaseSet other_bad_cases;
    for(g_it = good_cases.begin(); g_it != good_cases.end(); ++g_it){
        for(b_it = bad_cases.begin(); b_it != bad_cases.end(); ++b_it){
            File::State other_bad_case = (File::State)((*g_it).first | (*b_it));
            if(good_cases.find(other_bad_case) == good_cases.end()){
                other_bad_cases.insert(other_bad_case);
            }
        }
    }
    bad_cases.insert(other_bad_cases.begin(),other_bad_cases.end());
        
    File::State file_states;

    // Check Good Cases
    for(g_it = good_cases.begin(); g_it != good_cases.end(); ++g_it){
        TestJobEventListener::instance().reset();
        file_states = (*g_it).first;
        job.state = (Job::S_FINISHED);
        fsm.updateState(file_states);
        std::stringstream error;
        error << "Invalid Job State for File States " 
                                << std::hex << std::showbase << (*g_it).first
                << ". Expected " << std::hex << std::showbase << (*g_it).second 
                << ". Got "      << std::hex << std::showbase << job.state();
        CPPUNIT_ASSERT_MESSAGE(error.str().c_str(), job.state == (*g_it).second);
        if(job.state == Job::S_FINISHED){
            CPPUNIT_ASSERT_MESSAGE("Listener: Invalid State Changed", TestJobEventListener::instance().stateChanged() == false);
        } else {
            CPPUNIT_ASSERT_MESSAGE("Listener: Invalid State Changed", TestJobEventListener::instance().stateChanged() == true);
        }
        CPPUNIT_ASSERT_MESSAGE("Listener: Invalid Pending", TestJobEventListener::instance().pending() == false);
        CPPUNIT_ASSERT_MESSAGE("Listener: Invalid Finished" , TestJobEventListener::instance().finished() == false);
    }
    
    TestJobEventListener::instance().reset();
    // Check Bad Cases
    for(b_it = bad_cases.begin(); b_it != bad_cases.end(); ++b_it){
        try{
            file_states = (*b_it);
            job.state = (Job::S_FINISHED);
        
            fsm.updateState(file_states);
            std::stringstream error;
            error << "Invalid Job State for File States " 
                            << std::hex << std::showbase << (*b_it)
                << ". Got "  << std::hex << std::showbase << job.state();
            CPPUNIT_FAIL(error.str().c_str());
        } catch(const FSMException&){
            CPPUNIT_ASSERT_MESSAGE("Listener: Invalid State Changed", TestJobEventListener::instance().stateChanged() == false);
            CPPUNIT_ASSERT_MESSAGE("Listener: Invalid Pending", TestJobEventListener::instance().pending() == false);
            CPPUNIT_ASSERT_MESSAGE("Listener: Invalid Finished" , TestJobEventListener::instance().finished() == false);
        }
    }

}

/*
 * test updateState from a FinishedDirty State
 */
void JobFSMTestCase::testUpdateState_from_FinishedDirty(){
    // Create an Object
    Job job;
    JobFSM fsm(job);
    
    // Define Good Cases
    typedef std::map<File::State,Job::State> CaseMap;
    typedef CaseMap::value_type Case;
    CaseMap           good_cases;
    CaseMap::iterator g_it;
    good_cases.insert(Case(File::S_FINISHED, Job::S_FINISHED_DIRTY));  
    good_cases.insert(Case(File::S_FAILED  , Job::S_FINISHED_DIRTY));  
        
    // Combine everything together (maybe not a smart algorithm, but it should work)
    int good_cases_size = good_cases.size();
    for(int i= 0; i < good_cases_size; ++i){
        CaseMap other_cases;
        for(g_it = good_cases.begin(); g_it != good_cases.end(); ++g_it){
            CaseMap::iterator g1_it = g_it;
            ++g1_it;
            for(; g1_it != good_cases.end(); ++g1_it){
                File::State file_states = File::State((*g_it).first | (*g1_it).first);
                other_cases.insert(Case(file_states,Job::S_FINISHED_DIRTY));
            }
        }
        good_cases.insert(other_cases.begin(),other_cases.end());
    }
    
    // Define Bad Cases
    typedef std::set<File::State> BadCaseSet;
    BadCaseSet           bad_cases;
    BadCaseSet::iterator b_it;
    
    bad_cases.insert(File::S_SUBMITTED);
    bad_cases.insert(File::S_WAITING_CATALOG_RESOLUTION);
    bad_cases.insert(File::S_PENDING);
    bad_cases.insert(File::S_AWAITING_PRESTAGE);
    bad_cases.insert(File::S_PRESTAGING);
    bad_cases.insert(File::S_WAITING_PRESTAGE);
    bad_cases.insert(File::S_READY);
    bad_cases.insert(File::S_ACTIVE);
    bad_cases.insert(File::S_DONE);
    bad_cases.insert(File::S_WAITING);
    bad_cases.insert(File::S_HOLD);
    bad_cases.insert(File::S_FINISHING);
    bad_cases.insert(File::S_WAITING_CATALOG_REGISTRATION);
    
    // Combine bad cases together (maybe not a smart algorithm, but it should work)
    int bad_cases_size = bad_cases.size();
    for(int i= 0; i < bad_cases_size; ++i){
        BadCaseSet other_bad_cases;
        for(b_it = bad_cases.begin(); b_it != bad_cases.end(); ++b_it){
            BadCaseSet::iterator b1_it = b_it;
            ++b1_it;
            for(; b1_it != bad_cases.end(); ++b1_it){
                File::State other_bad_case = (File::State)((*b_it) | (*b1_it));
                if(good_cases.find(other_bad_case) == good_cases.end()){
                    other_bad_cases.insert(other_bad_case);
                }
            }
        }
        bad_cases.insert(other_bad_cases.begin(),other_bad_cases.end());
    }
    
    // Combine with the good ones
    BadCaseSet other_bad_cases;
    for(g_it = good_cases.begin(); g_it != good_cases.end(); ++g_it){
        for(b_it = bad_cases.begin(); b_it != bad_cases.end(); ++b_it){
            File::State other_bad_case = (File::State)((*g_it).first | (*b_it));
            if(good_cases.find(other_bad_case) == good_cases.end()){
                other_bad_cases.insert(other_bad_case);
            }
        }
    }
    bad_cases.insert(other_bad_cases.begin(),other_bad_cases.end());

    File::State file_states;

    // Check Good Cases
    for(g_it = good_cases.begin(); g_it != good_cases.end(); ++g_it){
        TestJobEventListener::instance().reset();
        file_states = (*g_it).first;
        job.state = (Job::S_FINISHED_DIRTY);
        fsm.updateState(file_states);
        std::stringstream error;
        error << "Invalid Job State for File States " 
                               << std::hex << std::showbase << (*g_it).first
              << ". Expected " << std::hex << std::showbase << (*g_it).second 
              << ". Got "      << std::hex << std::showbase << job.state();
        CPPUNIT_ASSERT_MESSAGE(error.str().c_str(), job.state == (*g_it).second);
        if(job.state == Job::S_FINISHED_DIRTY){
            CPPUNIT_ASSERT_MESSAGE("Listener: Invalid State Changed", TestJobEventListener::instance().stateChanged() == false);
        } else {
            CPPUNIT_ASSERT_MESSAGE("Listener: Invalid State Changed", TestJobEventListener::instance().stateChanged() == true);
        }
        CPPUNIT_ASSERT_MESSAGE("Listener: Invalid Pending", TestJobEventListener::instance().pending() == false);
        CPPUNIT_ASSERT_MESSAGE("Listener: Invalid Finished" , TestJobEventListener::instance().finished() == false);
    }
    
    TestJobEventListener::instance().reset();
    // Check Bad Cases
    for(b_it = bad_cases.begin(); b_it != bad_cases.end(); ++b_it){
        file_states = (*b_it);
        job.state = (Job::S_FINISHED_DIRTY);
        try{
            fsm.updateState(file_states);
            std::stringstream error;
            error << "Invalid Job State for File States " 
                               << std::hex << std::showbase << (*b_it)
                  << ". Got "  << std::hex << std::showbase << job.state();
            CPPUNIT_FAIL(error.str().c_str());
        } catch(const FSMException&){
            CPPUNIT_ASSERT_MESSAGE("Listener: Invalid State Changed", TestJobEventListener::instance().stateChanged() == false);
            CPPUNIT_ASSERT_MESSAGE("Listener: Invalid Pending", TestJobEventListener::instance().pending() == false);
            CPPUNIT_ASSERT_MESSAGE("Listener: Invalid Finished" , TestJobEventListener::instance().finished() == false);
        }
    }

}

/*
 * test updateState from a Hold State
 */
void JobFSMTestCase::testUpdateState_from_Hold(){
    // Create an Object
    Job job;
    JobFSM fsm(job);
    
    // Define Good Cases
    typedef std::map<File::State,Job::State> CaseMap;
    typedef CaseMap::value_type Case;
    CaseMap           good_cases;
    CaseMap::iterator g_it;
    good_cases.insert(Case(File::S_HOLD    , Job::S_HOLD));  
    good_cases.insert(Case(File::S_DONE    , Job::S_HOLD));  
    good_cases.insert(Case(File::S_FAILED  , Job::S_HOLD));
        
    // Combine everything together (maybe not a smart algorithm, but it should work)
    int good_cases_size = good_cases.size();
    for(int i= 0; i < good_cases_size; ++i){
        CaseMap other_cases;
        for(g_it = good_cases.begin(); g_it != good_cases.end(); ++g_it){
            CaseMap::iterator g1_it = g_it;
            ++g1_it;
            for(; g1_it != good_cases.end(); ++g1_it){
                File::State file_states = File::State((*g_it).first | (*g1_it).first);
                other_cases.insert(Case(file_states,Job::S_HOLD));
            }
        }
        good_cases.insert(other_cases.begin(),other_cases.end());
    }
    
    // Define Bad Cases
    typedef std::set<File::State> BadCaseSet;
    BadCaseSet           bad_cases;
    BadCaseSet::iterator b_it;
    
    bad_cases.insert(File::S_SUBMITTED);
    bad_cases.insert(File::S_WAITING_CATALOG_RESOLUTION);
    bad_cases.insert(File::S_PENDING);
    bad_cases.insert(File::S_AWAITING_PRESTAGE);
    bad_cases.insert(File::S_PRESTAGING);
    bad_cases.insert(File::S_WAITING_PRESTAGE);
    bad_cases.insert(File::S_READY);
    bad_cases.insert(File::S_ACTIVE);
    bad_cases.insert(File::S_WAITING);
    bad_cases.insert(File::S_FINISHING);
    bad_cases.insert(File::S_FINISHED);
    bad_cases.insert(File::S_WAITING_CATALOG_REGISTRATION);
    
    // Combine bad cases together (maybe not a smart algorithm, but it should work)
    int bad_cases_size = bad_cases.size();
    for(int i= 0; i < bad_cases_size; ++i){
        BadCaseSet other_bad_cases;
        for(b_it = bad_cases.begin(); b_it != bad_cases.end(); ++b_it){
            BadCaseSet::iterator b1_it = b_it;
            ++b1_it;
            for(; b1_it != bad_cases.end(); ++b1_it){
                File::State other_bad_case = (File::State)((*b_it) | (*b1_it));
                if(good_cases.find(other_bad_case) == good_cases.end()){
                    other_bad_cases.insert(other_bad_case);
                }
            }
        }
        bad_cases.insert(other_bad_cases.begin(),other_bad_cases.end());
    }
    
    // Combine with the good ones
    BadCaseSet other_bad_cases;
    for(g_it = good_cases.begin(); g_it != good_cases.end(); ++g_it){
        for(b_it = bad_cases.begin(); b_it != bad_cases.end(); ++b_it){
            File::State other_bad_case = (File::State)((*g_it).first | (*b_it));
            if(good_cases.find(other_bad_case) == good_cases.end()){
                other_bad_cases.insert(other_bad_case);
            }
        }
    }
    bad_cases.insert(other_bad_cases.begin(),other_bad_cases.end());
        
    File::State file_states;

    // Check Good Cases
    for(g_it = good_cases.begin(); g_it != good_cases.end(); ++g_it){
        TestJobEventListener::instance().reset();
        file_states = (*g_it).first;
        job.state = (Job::S_HOLD);
        fsm.updateState(file_states);
        std::stringstream error;
        error << "Invalid Job State for File States " 
                                << std::hex << std::showbase << (*g_it).first
                << ". Expected " << std::hex << std::showbase << (*g_it).second 
                << ". Got "      << std::hex << std::showbase << job.state();
        CPPUNIT_ASSERT_MESSAGE(error.str().c_str(), job.state == (*g_it).second);
        if(job.state == Job::S_HOLD){
            CPPUNIT_ASSERT_MESSAGE("Listener: Invalid State Changed", TestJobEventListener::instance().stateChanged() == false);
        } else {
            CPPUNIT_ASSERT_MESSAGE("Listener: Invalid State Changed", TestJobEventListener::instance().stateChanged() == true);
        }
        CPPUNIT_ASSERT_MESSAGE("Listener: Invalid Pending", TestJobEventListener::instance().pending() == false);
        CPPUNIT_ASSERT_MESSAGE("Listener: Invalid Finished" , TestJobEventListener::instance().finished() == false);
    }
    
    TestJobEventListener::instance().reset();
    // Check Bad Cases
    for(b_it = bad_cases.begin(); b_it != bad_cases.end(); ++b_it){
        try{
            file_states = (*b_it);
            job.state = (Job::S_HOLD);
        
            fsm.updateState(file_states);
            std::stringstream error;
            error << "Invalid Job State for File States " 
                            << std::hex << std::showbase << (*b_it)
                << ". Got "  << std::hex << std::showbase << job.state();
            CPPUNIT_FAIL(error.str().c_str());
        } catch(const FSMException&){
            CPPUNIT_ASSERT_MESSAGE("Listener: Invalid State Changed", TestJobEventListener::instance().stateChanged() == false);
            CPPUNIT_ASSERT_MESSAGE("Listener: Invalid Pending", TestJobEventListener::instance().pending() == false);
            CPPUNIT_ASSERT_MESSAGE("Listener: Invalid Finished" , TestJobEventListener::instance().finished() == false);
        }
    }

}

/*
 * test updateState from a Failed State
 */
void JobFSMTestCase::testUpdateState_from_Failed(){
    // Create an Object
    Job job;
    JobFSM fsm(job);
    
    // Define Good Cases
    typedef std::map<File::State,Job::State> CaseMap;
    typedef CaseMap::value_type Case;
    CaseMap           good_cases;
    CaseMap::iterator g_it;
    good_cases.insert(Case(File::S_FAILED  , Job::S_FAILED));  
    good_cases.insert(Case(File::S_DONE    , Job::S_FAILED));  
    good_cases.insert(Case(File::S_FINISHED, Job::S_FAILED));
        
    // Combine everything together (maybe not a smart algorithm, but it should work)
    int good_cases_size = good_cases.size();
    for(int i= 0; i < good_cases_size; ++i){
        CaseMap other_cases;
        for(g_it = good_cases.begin(); g_it != good_cases.end(); ++g_it){
            CaseMap::iterator g1_it = g_it;
            ++g1_it;
            for(; g1_it != good_cases.end(); ++g1_it){
                File::State file_states = File::State((*g_it).first | (*g1_it).first);
                other_cases.insert(Case(file_states,Job::S_FAILED));
            }
        }
        good_cases.insert(other_cases.begin(),other_cases.end());
    }
    
    // Define Bad Cases
    typedef std::set<File::State> BadCaseSet;
    BadCaseSet           bad_cases;
    BadCaseSet::iterator b_it;
    
    bad_cases.insert(File::S_SUBMITTED);
    bad_cases.insert(File::S_WAITING_CATALOG_RESOLUTION);
    bad_cases.insert(File::S_PENDING);
    bad_cases.insert(File::S_AWAITING_PRESTAGE);
    bad_cases.insert(File::S_PRESTAGING);
    bad_cases.insert(File::S_WAITING_PRESTAGE);
    bad_cases.insert(File::S_READY);
    bad_cases.insert(File::S_ACTIVE);
    bad_cases.insert(File::S_WAITING);
    bad_cases.insert(File::S_HOLD);
    bad_cases.insert(File::S_FINISHING);
    bad_cases.insert(File::S_WAITING_CATALOG_REGISTRATION);
    
    // Combine bad cases together (maybe not a smart algorithm, but it should work)
    int bad_cases_size = bad_cases.size();
    for(int i= 0; i < bad_cases_size; ++i){
        BadCaseSet other_bad_cases;
        for(b_it = bad_cases.begin(); b_it != bad_cases.end(); ++b_it){
            BadCaseSet::iterator b1_it = b_it;
            ++b1_it;
            for(; b1_it != bad_cases.end(); ++b1_it){
                File::State other_bad_case = (File::State)((*b_it) | (*b1_it));
                if(good_cases.find(other_bad_case) == good_cases.end()){
                    other_bad_cases.insert(other_bad_case);
                }
            }
        }
        bad_cases.insert(other_bad_cases.begin(),other_bad_cases.end());
    }
    
    // Combine with the good ones
    BadCaseSet other_bad_cases;
    for(g_it = good_cases.begin(); g_it != good_cases.end(); ++g_it){
        for(b_it = bad_cases.begin(); b_it != bad_cases.end(); ++b_it){
            File::State other_bad_case = (File::State)((*g_it).first | (*b_it));
            if(good_cases.find(other_bad_case) == good_cases.end()){
                other_bad_cases.insert(other_bad_case);
            }
        }
    }
    bad_cases.insert(other_bad_cases.begin(),other_bad_cases.end());

    File::State file_states;

    // Check Good Cases
    for(g_it = good_cases.begin(); g_it != good_cases.end(); ++g_it){
        TestJobEventListener::instance().reset();
        file_states = (*g_it).first;
        job.state = (Job::S_FAILED);
        fsm.updateState(file_states);
        std::stringstream error;
        error << "Invalid Job State for File States " 
                               << std::hex << std::showbase << (*g_it).first
              << ". Expected " << std::hex << std::showbase << (*g_it).second 
              << ". Got "      << std::hex << std::showbase << job.state();
        CPPUNIT_ASSERT_MESSAGE(error.str().c_str(), job.state == (*g_it).second);
        if(job.state == Job::S_FAILED){
            CPPUNIT_ASSERT_MESSAGE("Listener: Invalid State Changed", TestJobEventListener::instance().stateChanged() == false);
        } else {
            CPPUNIT_ASSERT_MESSAGE("Listener: Invalid State Changed", TestJobEventListener::instance().stateChanged() == true);
        }
        CPPUNIT_ASSERT_MESSAGE("Listener: Invalid Pending", TestJobEventListener::instance().pending() == false);
        CPPUNIT_ASSERT_MESSAGE("Listener: Invalid Finished" , TestJobEventListener::instance().finished() == false);
    }
    
    TestJobEventListener::instance().reset();
    // Check Bad Cases
    for(b_it = bad_cases.begin(); b_it != bad_cases.end(); ++b_it){
        file_states = (*b_it);
        job.state = (Job::S_FAILED);
        try{
            fsm.updateState(file_states);
            std::stringstream error;
            error << "Invalid Job State for File States " 
                               << std::hex << std::showbase << (*b_it)
                  << ". Got "  << std::hex << std::showbase << job.state();
            CPPUNIT_FAIL(error.str().c_str());
        } catch(const FSMException&){
            CPPUNIT_ASSERT_MESSAGE("Listener: Invalid State Changed", TestJobEventListener::instance().stateChanged() == false);
            CPPUNIT_ASSERT_MESSAGE("Listener: Invalid Pending", TestJobEventListener::instance().pending() == false);
            CPPUNIT_ASSERT_MESSAGE("Listener: Invalid Finished" , TestJobEventListener::instance().finished() == false);
        }
    }

}

/*
 * test updateState from a Canceled State
 */
void JobFSMTestCase::testUpdateState_from_Canceled(){
    // Create an Object
    Job job;
    JobFSM fsm(job);
    
    // Define Good Cases
    typedef std::map<File::State,Job::State> CaseMap;
    typedef CaseMap::value_type Case;
    CaseMap           good_cases;
    CaseMap::iterator g_it;
    good_cases.insert(Case(File::S_FAILED  , Job::S_CANCELED));  
    good_cases.insert(Case(File::S_FINISHED, Job::S_CANCELED));
    good_cases.insert(Case(File::S_CANCELED, Job::S_CANCELED));
        
    // Combine everything together (maybe not a smart algorithm, but it should work)
    int good_cases_size = good_cases.size();
    for(int i= 0; i < good_cases_size; ++i){
        CaseMap other_cases;
        for(g_it = good_cases.begin(); g_it != good_cases.end(); ++g_it){
            CaseMap::iterator g1_it = g_it;
            ++g1_it;
            for(; g1_it != good_cases.end(); ++g1_it){
                File::State file_states = File::State((*g_it).first | (*g1_it).first);
                other_cases.insert(Case(file_states,Job::S_CANCELED));
            }
        }
        good_cases.insert(other_cases.begin(),other_cases.end());
    }
    
    // Define Bad Cases
    typedef std::set<File::State> BadCaseSet;
    BadCaseSet           bad_cases;
    BadCaseSet::iterator b_it;
    
    bad_cases.insert(File::S_SUBMITTED);
    bad_cases.insert(File::S_WAITING_CATALOG_RESOLUTION);
    bad_cases.insert(File::S_PENDING);
    bad_cases.insert(File::S_AWAITING_PRESTAGE);
    bad_cases.insert(File::S_PRESTAGING);
    bad_cases.insert(File::S_WAITING_PRESTAGE);
    bad_cases.insert(File::S_READY);
    bad_cases.insert(File::S_ACTIVE);
    bad_cases.insert(File::S_DONE);
    bad_cases.insert(File::S_WAITING);
    bad_cases.insert(File::S_HOLD);
    bad_cases.insert(File::S_FINISHING);
    bad_cases.insert(File::S_WAITING_CATALOG_REGISTRATION);
    
    // Combine bad cases together (maybe not a smart algorithm, but it should work)
    int bad_cases_size = bad_cases.size();
    for(int i= 0; i < bad_cases_size; ++i){
        BadCaseSet other_bad_cases;
        for(b_it = bad_cases.begin(); b_it != bad_cases.end(); ++b_it){
            BadCaseSet::iterator b1_it = b_it;
            ++b1_it;
            for(; b1_it != bad_cases.end(); ++b1_it){
                File::State other_bad_case = (File::State)((*b_it) | (*b1_it));
                if(good_cases.find(other_bad_case) == good_cases.end()){
                    other_bad_cases.insert(other_bad_case);
                }
            }
        }
        bad_cases.insert(other_bad_cases.begin(),other_bad_cases.end());
    }
    
    // Combine with the good ones
    BadCaseSet other_bad_cases;
    for(g_it = good_cases.begin(); g_it != good_cases.end(); ++g_it){
        for(b_it = bad_cases.begin(); b_it != bad_cases.end(); ++b_it){
            File::State other_bad_case = (File::State)((*g_it).first | (*b_it));
            if(good_cases.find(other_bad_case) == good_cases.end()){
                other_bad_cases.insert(other_bad_case);
            }
        }
    }
    bad_cases.insert(other_bad_cases.begin(),other_bad_cases.end());

    File::State file_states;

    // Check Good Cases
    for(g_it = good_cases.begin(); g_it != good_cases.end(); ++g_it){
        file_states = (*g_it).first;
        {
        TestJobEventListener::instance().reset();
        job.cancelJob = true;
        job.state = (Job::S_CANCELED);
        fsm.updateState(file_states);
        std::stringstream error;
        error << "Invalid (1) Job State for File States " 
                               << std::hex << std::showbase << (*g_it).first
              << ". Expected " << std::hex << std::showbase << (*g_it).second 
              << ". Got "      << std::hex << std::showbase << job.state();
        CPPUNIT_ASSERT_MESSAGE(error.str().c_str(), job.state == (*g_it).second);
        if(job.state == Job::S_CANCELED){
            CPPUNIT_ASSERT_MESSAGE("Listener: Invalid State Changed", TestJobEventListener::instance().stateChanged() == false);
        } else {
            CPPUNIT_ASSERT_MESSAGE("Listener: Invalid State Changed", TestJobEventListener::instance().stateChanged() == true);
        }
        CPPUNIT_ASSERT_MESSAGE("Listener: Invalid Pending", TestJobEventListener::instance().pending() == false);
        CPPUNIT_ASSERT_MESSAGE("Listener: Invalid Finished" , TestJobEventListener::instance().finished() == false);
        }
        
        {
        TestJobEventListener::instance().reset();
        job.cancelJob = false;
        job.state = (Job::S_CANCELED);
        fsm.updateState(file_states);
        std::stringstream error;
        error << "Invalid (2) Job State for File States " 
                               << std::hex << std::showbase << (*g_it).first
              << ". Expected " << std::hex << std::showbase << (*g_it).second 
              << ". Got "      << std::hex << std::showbase << job.state();
        CPPUNIT_ASSERT_MESSAGE(error.str().c_str(), job.state == (*g_it).second);
        if(job.state == Job::S_CANCELED){
            CPPUNIT_ASSERT_MESSAGE("Listener: Invalid State Changed", TestJobEventListener::instance().stateChanged() == false);
        } else {
            CPPUNIT_ASSERT_MESSAGE("Listener: Invalid State Changed", TestJobEventListener::instance().stateChanged() == true);
        }
        CPPUNIT_ASSERT_MESSAGE("Listener: Invalid Pending", TestJobEventListener::instance().pending() == false);
        CPPUNIT_ASSERT_MESSAGE("Listener: Invalid Finished" , TestJobEventListener::instance().finished() == false);
        }
    }
    
    TestJobEventListener::instance().reset();
    // Check Bad Cases
    for(b_it = bad_cases.begin(); b_it != bad_cases.end(); ++b_it){
        file_states   = (*b_it);
        {
        job.cancelJob = true;
        job.state     = (Job::S_CANCELED);
        try{
            fsm.updateState(file_states);
            std::stringstream error;
            error << "Invalid (1) Job State for File States " 
                               << std::hex << std::showbase << (*b_it)
                  << ". Got "  << std::hex << std::showbase << job.state();
            CPPUNIT_FAIL(error.str().c_str());
        } catch(const FSMException&){
            CPPUNIT_ASSERT_MESSAGE("Listener: Invalid State Changed", TestJobEventListener::instance().stateChanged() == false);
            CPPUNIT_ASSERT_MESSAGE("Listener: Invalid Pending", TestJobEventListener::instance().pending() == false);
            CPPUNIT_ASSERT_MESSAGE("Listener: Invalid Finished" , TestJobEventListener::instance().finished() == false);
        }
        }
        {            
        job.cancelJob = false;
        job.state     = (Job::S_CANCELED);
        try{
            fsm.updateState(file_states);
            std::stringstream error;
            error << "Invalid (2) Job State for File States " 
                               << std::hex << std::showbase << (*b_it)
                  << ". Got "  << std::hex << std::showbase << job.state();
            CPPUNIT_FAIL(error.str().c_str());
        } catch(const FSMException&){
            CPPUNIT_ASSERT_MESSAGE("Listener: Invalid State Changed", TestJobEventListener::instance().stateChanged() == false);
            CPPUNIT_ASSERT_MESSAGE("Listener: Invalid Pending", TestJobEventListener::instance().pending() == false);
            CPPUNIT_ASSERT_MESSAGE("Listener: Invalid Finished" , TestJobEventListener::instance().finished() == false);
        }
        }
    }

}

/*
 * test updateState canceling from a Pending State
 */
void JobFSMTestCase::testUpdateState_canceling_from_Submitted(){
    // Create an Object
    Job job;
    JobFSM fsm(job);
    job.cancelJob = true;
    
    // Define Good Cases
    typedef std::map<File::State,Job::State> CaseMap;
    typedef CaseMap::value_type Case;
    CaseMap           good_cases;
    CaseMap::iterator g_it;
    good_cases.insert(Case(File::S_SUBMITTED                 , Job::S_SUBMITTED));
    good_cases.insert(Case(File::S_WAITING_CATALOG_RESOLUTION, Job::S_SUBMITTED));
    good_cases.insert(Case(File::S_PENDING                   , Job::S_PENDING));
    good_cases.insert(Case(File::S_FAILED                    , Job::S_CANCELED));
    good_cases.insert(Case(File::S_CANCELED                  , Job::S_CANCELED));
    
    // Combine everything together (maybe not a smart algorithm, but it should work)
    int good_cases_size = good_cases.size();
    for(int i= 0; i < good_cases_size; ++i){
        CaseMap other_cases;
        for(g_it = good_cases.begin(); g_it != good_cases.end(); ++g_it){
            CaseMap::iterator g1_it = g_it;
            ++g1_it;
            for(; g1_it != good_cases.end(); ++g1_it){
                File::State file_states = File::State((*g_it).first | (*g1_it).first);
                if(file_states == (File::S_FAILED | File::S_CANCELED)){
                    other_cases.insert(Case(file_states,Job::S_CANCELED));
                } else if((file_states & File::S_SUBMITTED) || 
                   (file_states & File::S_WAITING_CATALOG_RESOLUTION)){
                    other_cases.insert(Case(file_states,Job::S_SUBMITTED));
                } else {
                    other_cases.insert(Case(file_states,Job::S_PENDING));
                }
            }
        }
        good_cases.insert(other_cases.begin(),other_cases.end());
    }

    File::State file_states;

    // Check Good Cases
    for(g_it = good_cases.begin(); g_it != good_cases.end(); ++g_it){
        TestJobEventListener::instance().reset();
        file_states = (*g_it).first;
        job.state = (Job::S_SUBMITTED);
        fsm.updateState(file_states);
        std::stringstream error;
        error << "Invalid Job State for File States " 
                               << std::hex << std::showbase << (*g_it).first
              << ". Expected " << std::hex << std::showbase << (*g_it).second 
              << ". Got "      << std::hex << std::showbase << job.state();
        CPPUNIT_ASSERT_MESSAGE(error.str().c_str(), job.state == (*g_it).second);
        if(job.state == Job::S_SUBMITTED){
            CPPUNIT_ASSERT_MESSAGE("Listener: Invalid State Changed", TestJobEventListener::instance().stateChanged() == false);
        } else {
            CPPUNIT_ASSERT_MESSAGE("Listener: Invalid State Changed", TestJobEventListener::instance().stateChanged() == true);
        }
        if(job.state == Job::S_PENDING){
            CPPUNIT_ASSERT_MESSAGE("Listener: Invalid Pending", TestJobEventListener::instance().pending() == true);
        } else {
            CPPUNIT_ASSERT_MESSAGE("Listener: Invalid Pending", TestJobEventListener::instance().pending() == false);
        }
        CPPUNIT_ASSERT_MESSAGE("Listener: Invalid Finished" , TestJobEventListener::instance().finished() == false);
    }

}

/*
 * test updateState from a Pending State
 */
void JobFSMTestCase::testUpdateState_canceling_from_Pending(){
    // Create an Object
    Job job;
    JobFSM fsm(job);
    job.cancelJob = true;
    
    // Define Good Cases
    typedef std::map<File::State,Job::State> CaseMap;
    typedef CaseMap::value_type Case;
    CaseMap           good_cases;
    CaseMap::iterator g_it;
    good_cases.insert(Case(File::S_PENDING          , Job::S_PENDING));
    good_cases.insert(Case(File::S_READY            , Job::S_READY));
    good_cases.insert(Case(File::S_AWAITING_PRESTAGE, Job::S_PENDING));
    good_cases.insert(Case(File::S_PRESTAGING       , Job::S_PENDING));
    good_cases.insert(Case(File::S_WAITING_PRESTAGE , Job::S_PENDING));
    good_cases.insert(Case(File::S_FAILED           , Job::S_CANCELED));
    good_cases.insert(Case(File::S_HOLD             , Job::S_HOLD));
    good_cases.insert(Case(File::S_DONE             , Job::S_DONE));
    good_cases.insert(Case(File::S_CANCELED         , Job::S_CANCELED));
    
    File::State canceled_mask = (File::State)(File::S_FAILED | File::S_CANCELED);
    File::State failed_mask = (File::State)(File::S_FAILED | File::S_DONE);
    File::State hold_mask   = (File::State)(File::S_FAILED | File::S_DONE | File::S_HOLD);
    File::State hold_mask_2   = (File::State)(File::S_FAILED | File::S_HOLD | File::S_CANCELED);
    
    // Combine everything together (maybe not a smart algorithm, but it should work)
    int good_cases_size = good_cases.size();
    for(int i= 0; i < good_cases_size; ++i){
        CaseMap other_cases;
        for(g_it = good_cases.begin(); g_it != good_cases.end(); ++g_it){
            CaseMap::iterator g1_it = g_it;
            ++g1_it;
            for(; g1_it != good_cases.end(); ++g1_it){
                File::State file_states = File::State((*g_it).first | (*g1_it).first);
                if((file_states & File::S_PENDING) || 
                   (file_states & File::S_AWAITING_PRESTAGE) ||
                   (file_states & File::S_PRESTAGING) ||
                   (file_states & File::S_WAITING_PRESTAGE)){
                    other_cases.insert(Case(file_states,Job::S_PENDING));
                } else if((file_states | canceled_mask) == canceled_mask){
                    other_cases.insert(Case(file_states,Job::S_CANCELED));
                } else if((file_states | failed_mask) == failed_mask){
                    other_cases.insert(Case(file_states,Job::S_FAILED));
                } else if((file_states | hold_mask) == hold_mask){
                    other_cases.insert(Case(file_states,Job::S_HOLD));
                } else if((file_states | hold_mask_2) == hold_mask_2){
                    other_cases.insert(Case(file_states,Job::S_HOLD));
                } else if(file_states == (File::S_DONE | File::S_CANCELED | File::S_HOLD)){
                    other_cases.insert(Case(file_states,Job::S_HOLD));
                } else if(file_states == (File::S_DONE | File::S_CANCELED | File::S_HOLD | File::S_FAILED)){
                    other_cases.insert(Case(file_states,Job::S_HOLD));
                } else {
                    other_cases.insert(Case(file_states,Job::S_READY));
                }
            }
        }
        good_cases.insert(other_cases.begin(),other_cases.end());
    }

    File::State file_states;

    // Check Good Cases
    for(g_it = good_cases.begin(); g_it != good_cases.end(); ++g_it){
        TestJobEventListener::instance().reset();
        file_states = (*g_it).first;
        if(file_states == File::S_DONE){
            // Skip it: it should be a bad case
            continue;
        }
        job.state = (Job::S_PENDING);
        fsm.updateState(file_states);
        std::stringstream error;
        error << "Invalid Job State for File States " 
                               << std::hex << std::showbase << (*g_it).first
              << ". Expected " << std::hex << std::showbase << (*g_it).second 
              << ". Got "      << std::hex << std::showbase << job.state();
        CPPUNIT_ASSERT_MESSAGE(error.str().c_str(), job.state == (*g_it).second);
        if(job.state == Job::S_PENDING){
            CPPUNIT_ASSERT_MESSAGE("Listener: Invalid State Changed", TestJobEventListener::instance().stateChanged() == false);
        } else {
            CPPUNIT_ASSERT_MESSAGE("Listener: Invalid State Changed", TestJobEventListener::instance().stateChanged() == true);
        }
        CPPUNIT_ASSERT_MESSAGE("Listener: Invalid Pending", TestJobEventListener::instance().pending() == false);
        CPPUNIT_ASSERT_MESSAGE("Listener: Invalid Finished" , TestJobEventListener::instance().finished() == false);
    }

}

/*
 * test updateState canceling from an Ready State
 */
void JobFSMTestCase::testUpdateState_canceling_from_Ready(){
    // Create an Object
    Job job;
    JobFSM fsm(job);
    job.cancelJob = true;
    
    // Define Good Cases
    typedef std::map<File::State,Job::State> CaseMap;
    typedef CaseMap::value_type Case;
    CaseMap           good_cases;
    CaseMap::iterator g_it;
    good_cases.insert(Case(File::S_READY    , Job::S_READY));
    good_cases.insert(Case(File::S_ACTIVE   , Job::S_ACTIVE));
    good_cases.insert(Case(File::S_DONE     , Job::S_ACTIVE));
    good_cases.insert(Case(File::S_WAITING  , Job::S_ACTIVE));
    good_cases.insert(Case(File::S_HOLD     , Job::S_HOLD));
    good_cases.insert(Case(File::S_FAILED   , Job::S_CANCELED));
    good_cases.insert(Case(File::S_CANCELED , Job::S_CANCELED));
    
    File::State canceled_mask = (File::State)(File::S_FAILED | File::S_CANCELED);
    File::State failed_mask = (File::State)(File::S_FAILED | File::S_DONE);
    File::State hold_mask   = (File::State)(File::S_FAILED | File::S_DONE | File::S_HOLD);
    File::State hold_mask_2   = (File::State)(File::S_FAILED | File::S_HOLD | File::S_CANCELED);
   
    // Combine everything together (maybe not a smart algorithm, but it should work)
    int good_cases_size = good_cases.size();
    for(int i= 0; i < good_cases_size; ++i){
        CaseMap other_cases;
        for(g_it = good_cases.begin(); g_it != good_cases.end(); ++g_it){
            CaseMap::iterator g1_it = g_it;
            ++g1_it;
            for(; g1_it != good_cases.end(); ++g1_it){
                File::State file_states = File::State((*g_it).first | (*g1_it).first);
                if((file_states | canceled_mask) == canceled_mask){
                    other_cases.insert(Case(file_states,Job::S_CANCELED));
                } else if((file_states | failed_mask) == failed_mask){
                    other_cases.insert(Case(file_states,Job::S_ACTIVE));
                } else if((file_states | hold_mask) == hold_mask){
                    other_cases.insert(Case(file_states,Job::S_HOLD));
                } else if((file_states | hold_mask_2) == hold_mask_2){
                    other_cases.insert(Case(file_states,Job::S_HOLD));
                } else if(file_states == (File::S_DONE | File::S_CANCELED | File::S_HOLD)){
                    other_cases.insert(Case(file_states,Job::S_HOLD));
                } else if(file_states == (File::S_DONE | File::S_CANCELED | File::S_HOLD | File::S_FAILED)){
                    other_cases.insert(Case(file_states,Job::S_HOLD));
                } else {
                    other_cases.insert(Case(file_states,Job::S_ACTIVE));
                }
            }
        }
        good_cases.insert(other_cases.begin(),other_cases.end());
    }

    File::State file_states;

    // Check Good Cases
    for(g_it = good_cases.begin(); g_it != good_cases.end(); ++g_it){
        TestJobEventListener::instance().reset();
        file_states = (*g_it).first;
        job.state = (Job::S_READY);
        fsm.updateState(file_states);
        std::stringstream error;
        error << "Invalid Job State for File States " 
                               << std::hex << std::showbase << (*g_it).first
              << ". Expected " << std::hex << std::showbase << (*g_it).second 
              << ". Got "      << std::hex << std::showbase << job.state();
        CPPUNIT_ASSERT_MESSAGE(error.str().c_str(), job.state == (*g_it).second);
        if(job.state == Job::S_READY){
            CPPUNIT_ASSERT_MESSAGE("Listener: Invalid State Changed", TestJobEventListener::instance().stateChanged() == false);
        } else {
            CPPUNIT_ASSERT_MESSAGE("Listener: Invalid State Changed", TestJobEventListener::instance().stateChanged() == true);
        }
        CPPUNIT_ASSERT_MESSAGE("Listener: Invalid Pending", TestJobEventListener::instance().pending() == false);
        CPPUNIT_ASSERT_MESSAGE("Listener: Invalid Finished" , TestJobEventListener::instance().finished() == false);
    }

}

/*
 * test updateState cancleing from an Active State
 */
void JobFSMTestCase::testUpdateState_canceling_from_Active(){
    // Create an Object
    Job job;
    JobFSM fsm(job);
    job.cancelJob = true;
    
    // Define Good Cases
    typedef std::map<File::State,Job::State> CaseMap;
    typedef CaseMap::value_type Case;
    CaseMap           good_cases;
    CaseMap::iterator g_it;
    good_cases.insert(Case(File::S_PENDING          , Job::S_ACTIVE));
    good_cases.insert(Case(File::S_AWAITING_PRESTAGE, Job::S_ACTIVE));
    good_cases.insert(Case(File::S_PRESTAGING       , Job::S_ACTIVE));
    good_cases.insert(Case(File::S_WAITING_PRESTAGE , Job::S_ACTIVE));
    good_cases.insert(Case(File::S_ACTIVE           , Job::S_ACTIVE));
    good_cases.insert(Case(File::S_READY            , Job::S_ACTIVE));
    good_cases.insert(Case(File::S_DONE             , Job::S_DONE));
    good_cases.insert(Case(File::S_WAITING          , Job::S_ACTIVE));
    good_cases.insert(Case(File::S_HOLD             , Job::S_ACTIVE));
    good_cases.insert(Case(File::S_FAILED           , Job::S_CANCELED));
    good_cases.insert(Case(File::S_CANCELED         , Job::S_CANCELED));
    
    File::State canceled_mask = (File::State)(File::S_FAILED | File::S_CANCELED);
    File::State failed_mask   = (File::State)(File::S_FAILED | File::S_DONE);
   
    // Combine everything together (maybe not a smart algorithm, but it should work)
    int good_cases_size = good_cases.size();
    for(int i= 0; i < good_cases_size; ++i){
        CaseMap other_cases;
        for(g_it = good_cases.begin(); g_it != good_cases.end(); ++g_it){
            CaseMap::iterator g1_it = g_it;
            ++g1_it;
            for(; g1_it != good_cases.end(); ++g1_it){
                File::State file_states = File::State((*g_it).first | (*g1_it).first);
                if((file_states | canceled_mask) == canceled_mask){
                    other_cases.insert(Case(file_states,Job::S_CANCELED));
                } else if((file_states | failed_mask) == failed_mask){
                    other_cases.insert(Case(file_states,Job::S_DONE_WITH_ERRORS));
                } else {
                    other_cases.insert(Case(file_states,Job::S_ACTIVE));
                }
            }
        }
        good_cases.insert(other_cases.begin(),other_cases.end());
    }

    File::State file_states;

    // Check Good Cases
    for(g_it = good_cases.begin(); g_it != good_cases.end(); ++g_it){
        TestJobEventListener::instance().reset();
        file_states = (*g_it).first;
        job.state = (Job::S_ACTIVE);
        fsm.updateState(file_states);
        std::stringstream error;
        error << "Invalid Job State for File States " 
                               << std::hex << std::showbase << (*g_it).first
              << ". Expected " << std::hex << std::showbase << (*g_it).second 
              << ". Got "      << std::hex << std::showbase << job.state();
        CPPUNIT_ASSERT_MESSAGE(error.str().c_str(), job.state == (*g_it).second);
        if(job.state == Job::S_ACTIVE){
            CPPUNIT_ASSERT_MESSAGE("Listener: Invalid State Changed", TestJobEventListener::instance().stateChanged() == false);
        } else {
            CPPUNIT_ASSERT_MESSAGE("Listener: Invalid State Changed", TestJobEventListener::instance().stateChanged() == true);
        }
        CPPUNIT_ASSERT_MESSAGE("Listener: Invalid Pending", TestJobEventListener::instance().pending() == false);
        CPPUNIT_ASSERT_MESSAGE("Listener: Invalid Finished" , TestJobEventListener::instance().finished() == false);
    }

}

/*
 * test updateState canceling from a Done State
 */
void JobFSMTestCase::testUpdateState_canceling_from_Done(){
    // Create an Object
    Job job;
    JobFSM fsm(job);
    job.cancelJob = true;
    
    // Define Good Cases
    typedef std::map<File::State,Job::State> CaseMap;
    typedef CaseMap::value_type Case;
    CaseMap           good_cases;
    CaseMap::iterator g_it;
    good_cases.insert(Case(File::S_DONE     , Job::S_DONE));  
    good_cases.insert(Case(File::S_FINISHING, Job::S_FINISHING));  
    good_cases.insert(Case(File::S_CANCELED , Job::S_CANCELED));  
    
    // Combine everything together (maybe not a smart algorithm, but it should work)
    int good_cases_size = good_cases.size();
    for(int i= 0; i < good_cases_size; ++i){
        CaseMap other_cases;
        for(g_it = good_cases.begin(); g_it != good_cases.end(); ++g_it){
            CaseMap::iterator g1_it = g_it;
            ++g1_it;
            for(; g1_it != good_cases.end(); ++g1_it){
                File::State file_states = File::State((*g_it).first | (*g1_it).first);
                if(file_states & File::S_FINISHING){
                    other_cases.insert(Case(file_states,Job::S_FINISHING));
                } else {
                    other_cases.insert(Case(file_states,Job::S_DONE));
                }
            }
        }
        good_cases.insert(other_cases.begin(),other_cases.end());
    }

    File::State file_states;

    // Check Good Cases
    for(g_it = good_cases.begin(); g_it != good_cases.end(); ++g_it){
        TestJobEventListener::instance().reset();
        file_states = (*g_it).first;
        job.state = (Job::S_DONE);
        fsm.updateState(file_states);
        std::stringstream error;
        error << "Invalid Job State for File States " 
                               << std::hex << std::showbase << (*g_it).first
              << ". Expected " << std::hex << std::showbase << (*g_it).second 
              << ". Got "      << std::hex << std::showbase << job.state();
        CPPUNIT_ASSERT_MESSAGE(error.str().c_str(), job.state == (*g_it).second);
        if(job.state == Job::S_DONE){
            CPPUNIT_ASSERT_MESSAGE("Listener: Invalid State Changed", TestJobEventListener::instance().stateChanged() == false);
        } else {
            CPPUNIT_ASSERT_MESSAGE("Listener: Invalid State Changed", TestJobEventListener::instance().stateChanged() == true);
        }
        CPPUNIT_ASSERT_MESSAGE("Listener: Invalid Pending", TestJobEventListener::instance().pending() == false);
        CPPUNIT_ASSERT_MESSAGE("Listener: Invalid Finished" , TestJobEventListener::instance().finished() == false);
    }

}

/*
 * test updateState canceling from a DoneWithErrors State
 */
void JobFSMTestCase::testUpdateState_canceling_from_DoneWithErrors(){
    // Create an Object
    Job job;
    JobFSM fsm(job);
    job.cancelJob = true;
    
    // Define Good Cases
    typedef std::map<File::State,Job::State> CaseMap;
    typedef CaseMap::value_type Case;
    CaseMap           good_cases;
    CaseMap::iterator g_it;
    good_cases.insert(Case(File::S_DONE     , Job::S_DONE_WITH_ERRORS));  
    good_cases.insert(Case(File::S_FAILED   , Job::S_CANCELED));  
    good_cases.insert(Case(File::S_FINISHING, Job::S_FINISHING));  
    good_cases.insert(Case(File::S_CANCELED , Job::S_CANCELED));  
    
    // Combine everything together (maybe not a smart algorithm, but it should work)
    int good_cases_size = good_cases.size();
    for(int i= 0; i < good_cases_size; ++i){
        CaseMap other_cases;
        for(g_it = good_cases.begin(); g_it != good_cases.end(); ++g_it){
            CaseMap::iterator g1_it = g_it;
            ++g1_it;
            for(; g1_it != good_cases.end(); ++g1_it){
                File::State file_states = File::State((*g_it).first | (*g1_it).first);
                if(file_states & File::S_FINISHING){
                    other_cases.insert(Case(file_states,Job::S_FINISHING));
                } else if(file_states == (File::S_FAILED | File::S_CANCELED)){
                    other_cases.insert(Case(file_states,Job::S_CANCELED));
                } else {
                    other_cases.insert(Case(file_states,Job::S_DONE_WITH_ERRORS));
                }
            }
        }
        good_cases.insert(other_cases.begin(),other_cases.end());
    }

    File::State file_states;

    // Check Good Cases
    for(g_it = good_cases.begin(); g_it != good_cases.end(); ++g_it){
        TestJobEventListener::instance().reset();
        file_states = (*g_it).first;
        job.state = (Job::S_DONE_WITH_ERRORS);
        fsm.updateState(file_states);
        std::stringstream error;
        error << "Invalid Job State for File States " 
                               << std::hex << std::showbase << (*g_it).first
              << ". Expected " << std::hex << std::showbase << (*g_it).second 
              << ". Got "      << std::hex << std::showbase << job.state();
        CPPUNIT_ASSERT_MESSAGE(error.str().c_str(), job.state == (*g_it).second);
        if(job.state == Job::S_DONE_WITH_ERRORS){
            CPPUNIT_ASSERT_MESSAGE("Listener: Invalid State Changed", TestJobEventListener::instance().stateChanged() == false);
        } else {
            CPPUNIT_ASSERT_MESSAGE("Listener: Invalid State Changed", TestJobEventListener::instance().stateChanged() == true);
        }
        CPPUNIT_ASSERT_MESSAGE("Listener: Invalid Pending", TestJobEventListener::instance().pending() == false);
        CPPUNIT_ASSERT_MESSAGE("Listener: Invalid Finished" , TestJobEventListener::instance().finished() == false);
    }

}

/*
 * test updateState from a Finishing State
 */
void JobFSMTestCase::testUpdateState_canceling_from_Finishing(){
    // Create an Object
    Job job;
    JobFSM fsm(job);
    job.cancelJob = true;

    // Define Good Cases
    typedef std::map<File::State,Job::State> CaseMap;
    typedef CaseMap::value_type Case;
    CaseMap           good_cases;
    CaseMap::iterator g_it;
    good_cases.insert(Case(File::S_FINISHING                   , Job::S_FINISHING));  
    good_cases.insert(Case(File::S_WAITING_CATALOG_REGISTRATION, Job::S_FINISHING));  
    good_cases.insert(Case(File::S_FAILED                      , Job::S_CANCELED));  
    good_cases.insert(Case(File::S_FINISHED                    , Job::S_CANCELED));  
    good_cases.insert(Case(File::S_CANCELED                    , Job::S_CANCELED));  

    // Combine everything together (maybe not a smart algorithm, but it should work)
    int good_cases_size = good_cases.size();
    for(int i= 0; i < good_cases_size; ++i){
        CaseMap other_cases;
        for(g_it = good_cases.begin(); g_it != good_cases.end(); ++g_it){
            CaseMap::iterator g1_it = g_it;
            ++g1_it;
            for(; g1_it != good_cases.end(); ++g1_it){
                File::State file_states = File::State((*g_it).first | (*g1_it).first);
                if(file_states & (File::S_FINISHING | File::S_WAITING_CATALOG_REGISTRATION)){
                    other_cases.insert(Case(file_states,Job::S_FINISHING));
                } else {
                    other_cases.insert(Case(file_states,Job::S_CANCELED));
                }
            }
        }
        good_cases.insert(other_cases.begin(),other_cases.end());
    }

    File::State file_states;

    // Check Good Cases
    for(g_it = good_cases.begin(); g_it != good_cases.end(); ++g_it){
        TestJobEventListener::instance().reset();
        file_states = (*g_it).first;
        job.state = (Job::S_FINISHING);
        fsm.updateState(file_states);
        std::stringstream error;
        error << "Invalid Job State for File States " 
                               << std::hex << std::showbase << (*g_it).first
              << ". Expected " << std::hex << std::showbase << (*g_it).second 
              << ". Got "      << std::hex << std::showbase << job.state();
        CPPUNIT_ASSERT_MESSAGE(error.str().c_str(), job.state == (*g_it).second);
        if(job.state == Job::S_FINISHING){
            CPPUNIT_ASSERT_MESSAGE("Listener: Invalid State Changed", TestJobEventListener::instance().stateChanged() == false);
        } else {
            CPPUNIT_ASSERT_MESSAGE("Listener: Invalid State Changed", TestJobEventListener::instance().stateChanged() == true);
        }
        CPPUNIT_ASSERT_MESSAGE("Listener: Invalid Pending", TestJobEventListener::instance().pending() == false);
        CPPUNIT_ASSERT_MESSAGE("Listener: Invalid Finished" , TestJobEventListener::instance().finished() == false);
    }

}

/*
 * test updateState canceling from a Finished State
 */
void JobFSMTestCase::testUpdateState_canceling_from_Finished(){
    // Create an Object
    Job job;
    JobFSM fsm(job);
    job.cancelJob = true;
 
    // Define Good Cases
    typedef std::map<File::State,Job::State> CaseMap;
    typedef CaseMap::value_type Case;
    CaseMap           good_cases;
    CaseMap::iterator g_it;
    good_cases.insert(Case(File::S_FINISHED, Job::S_FINISHED));  
    good_cases.insert(Case(File::S_CANCELED, Job::S_FINISHED));  
    good_cases.insert(Case(File::State(File::S_FINISHED | File::S_CANCELED), Job::S_FINISHED));  

    File::State file_states;

    // Check Good Cases
    for(g_it = good_cases.begin(); g_it != good_cases.end(); ++g_it){
        TestJobEventListener::instance().reset();
        file_states = (*g_it).first;
        job.state = (Job::S_FINISHED);
        fsm.updateState(file_states);
        std::stringstream error;
        error << "Invalid Job State for File States " 
                               << std::hex << std::showbase << (*g_it).first
              << ". Expected " << std::hex << std::showbase << (*g_it).second 
              << ". Got "      << std::hex << std::showbase << job.state();
        CPPUNIT_ASSERT_MESSAGE(error.str().c_str(), job.state == (*g_it).second);
        if(job.state == Job::S_FINISHED){
            CPPUNIT_ASSERT_MESSAGE("Listener: Invalid State Changed", TestJobEventListener::instance().stateChanged() == false);
        } else {
            CPPUNIT_ASSERT_MESSAGE("Listener: Invalid State Changed", TestJobEventListener::instance().stateChanged() == true);
        }
        CPPUNIT_ASSERT_MESSAGE("Listener: Invalid Pending", TestJobEventListener::instance().pending() == false);
        CPPUNIT_ASSERT_MESSAGE("Listener: Invalid Finished" , TestJobEventListener::instance().finished() == false);
    }

}

/*
 * test updateState canceling from a FinishedDirty State
 */
void JobFSMTestCase::testUpdateState_canceling_from_FinishedDirty(){
    // Create an Object
    Job job;
    JobFSM fsm(job);
    job.cancelJob = true;

    // Define Good Cases
    typedef std::map<File::State,Job::State> CaseMap;
    typedef CaseMap::value_type Case;
    CaseMap           good_cases;
    CaseMap::iterator g_it;
    good_cases.insert(Case(File::S_FINISHED, Job::S_FINISHED_DIRTY));  
    good_cases.insert(Case(File::S_FAILED  , Job::S_FINISHED_DIRTY));  
    good_cases.insert(Case(File::S_CANCELED, Job::S_FINISHED_DIRTY));  
        
    // Combine everything together (maybe not a smart algorithm, but it should work)
    int good_cases_size = good_cases.size();
    for(int i= 0; i < good_cases_size; ++i){
        CaseMap other_cases;
        for(g_it = good_cases.begin(); g_it != good_cases.end(); ++g_it){
            CaseMap::iterator g1_it = g_it;
            ++g1_it;
            for(; g1_it != good_cases.end(); ++g1_it){
                File::State file_states = File::State((*g_it).first | (*g1_it).first);
                other_cases.insert(Case(file_states,Job::S_FINISHED_DIRTY));
            }
        }
        good_cases.insert(other_cases.begin(),other_cases.end());
    }

    File::State file_states;

    // Check Good Cases
    for(g_it = good_cases.begin(); g_it != good_cases.end(); ++g_it){
        TestJobEventListener::instance().reset();
        file_states = (*g_it).first;
        job.state = (Job::S_FINISHED_DIRTY);
        fsm.updateState(file_states);
        std::stringstream error;
        error << "Invalid Job State for File States " 
                               << std::hex << std::showbase << (*g_it).first
              << ". Expected " << std::hex << std::showbase << (*g_it).second 
              << ". Got "      << std::hex << std::showbase << job.state();
        CPPUNIT_ASSERT_MESSAGE(error.str().c_str(), job.state == (*g_it).second);
        if(job.state == Job::S_FINISHED_DIRTY){
            CPPUNIT_ASSERT_MESSAGE("Listener: Invalid State Changed", TestJobEventListener::instance().stateChanged() == false);
        } else {
            CPPUNIT_ASSERT_MESSAGE("Listener: Invalid State Changed", TestJobEventListener::instance().stateChanged() == true);
        }
        CPPUNIT_ASSERT_MESSAGE("Listener: Invalid Pending", TestJobEventListener::instance().pending() == false);
        CPPUNIT_ASSERT_MESSAGE("Listener: Invalid Finished" , TestJobEventListener::instance().finished() == false);
    }

}

/*
 * test updateState canceling from a Hold State
 */
void JobFSMTestCase::testUpdateState_canceling_from_Hold(){
    // Create an Object
    Job job;
    JobFSM fsm(job);
    job.cancelJob = true;

    // Define Good Cases
    typedef std::map<File::State,Job::State> CaseMap;
    typedef CaseMap::value_type Case;
    CaseMap           good_cases;
    CaseMap::iterator g_it;
    good_cases.insert(Case(File::S_HOLD    , Job::S_HOLD));  
    good_cases.insert(Case(File::S_DONE    , Job::S_HOLD));  
    good_cases.insert(Case(File::S_FAILED  , Job::S_CANCELED));
    good_cases.insert(Case(File::S_CANCELED, Job::S_CANCELED));
        
    // Combine everything together (maybe not a smart algorithm, but it should work)
    int good_cases_size = good_cases.size();
    for(int i= 0; i < good_cases_size; ++i){
        CaseMap other_cases;
        for(g_it = good_cases.begin(); g_it != good_cases.end(); ++g_it){
            CaseMap::iterator g1_it = g_it;
            ++g1_it;
            for(; g1_it != good_cases.end(); ++g1_it){
                File::State file_states = File::State((*g_it).first | (*g1_it).first);
                if(file_states & (File::S_DONE | File::S_HOLD)){
                    other_cases.insert(Case(file_states,Job::S_HOLD));
                } else {
                    other_cases.insert(Case(file_states,Job::S_CANCELED));
                }
            }
        }
        good_cases.insert(other_cases.begin(),other_cases.end());
    }

    File::State file_states;

    // Check Good Cases
    for(g_it = good_cases.begin(); g_it != good_cases.end(); ++g_it){
        TestJobEventListener::instance().reset();
        file_states = (*g_it).first;
        job.state = (Job::S_HOLD);
        fsm.updateState(file_states);
        std::stringstream error;
        error << "Invalid Job State for File States " 
                               << std::hex << std::showbase << (*g_it).first
              << ". Expected " << std::hex << std::showbase << (*g_it).second 
              << ". Got "      << std::hex << std::showbase << job.state();
        CPPUNIT_ASSERT_MESSAGE(error.str().c_str(), job.state == (*g_it).second);
        if(job.state == Job::S_HOLD){
            CPPUNIT_ASSERT_MESSAGE("Listener: Invalid State Changed", TestJobEventListener::instance().stateChanged() == false);
        } else {
            CPPUNIT_ASSERT_MESSAGE("Listener: Invalid State Changed", TestJobEventListener::instance().stateChanged() == true);
        }
        CPPUNIT_ASSERT_MESSAGE("Listener: Invalid Pending", TestJobEventListener::instance().pending() == false);
        CPPUNIT_ASSERT_MESSAGE("Listener: Invalid Finished" , TestJobEventListener::instance().finished() == false);
    }

}

/*
 * test updateState canceling from a Failed State
 */
void JobFSMTestCase::testUpdateState_canceling_from_Failed(){
    // Create an Object
    Job job;
    JobFSM fsm(job);
    job.cancelJob = true;
    
    // Define Good Cases
    typedef std::map<File::State,Job::State> CaseMap;
    typedef CaseMap::value_type Case;
    CaseMap           good_cases;
    CaseMap::iterator g_it;
    good_cases.insert(Case(File::S_FAILED  , Job::S_FAILED));  
    good_cases.insert(Case(File::S_DONE    , Job::S_FAILED));  
    good_cases.insert(Case(File::S_FINISHED, Job::S_FAILED));
    good_cases.insert(Case(File::S_CANCELED, Job::S_FAILED));
        
    // Combine everything together (maybe not a smart algorithm, but it should work)
    int good_cases_size = good_cases.size();
    for(int i= 0; i < good_cases_size; ++i){
        CaseMap other_cases;
        for(g_it = good_cases.begin(); g_it != good_cases.end(); ++g_it){
            CaseMap::iterator g1_it = g_it;
            ++g1_it;
            for(; g1_it != good_cases.end(); ++g1_it){
                File::State file_states = File::State((*g_it).first | (*g1_it).first);
                other_cases.insert(Case(file_states,Job::S_FAILED));
            }
        }
        good_cases.insert(other_cases.begin(),other_cases.end());
    }

    File::State file_states;

    // Check Good Cases
    for(g_it = good_cases.begin(); g_it != good_cases.end(); ++g_it){
        TestJobEventListener::instance().reset();
        file_states = (*g_it).first;
        job.state = (Job::S_FAILED);
        fsm.updateState(file_states);
        std::stringstream error;
        error << "Invalid Job State for File States " 
                               << std::hex << std::showbase << (*g_it).first
              << ". Expected " << std::hex << std::showbase << (*g_it).second 
              << ". Got "      << std::hex << std::showbase << job.state();
        CPPUNIT_ASSERT_MESSAGE(error.str().c_str(), job.state == (*g_it).second);
        if(job.state == Job::S_FAILED){
            CPPUNIT_ASSERT_MESSAGE("Listener: Invalid State Changed", TestJobEventListener::instance().stateChanged() == false);
        } else {
            CPPUNIT_ASSERT_MESSAGE("Listener: Invalid State Changed", TestJobEventListener::instance().stateChanged() == true);
        }
        CPPUNIT_ASSERT_MESSAGE("Listener: Invalid Pending", TestJobEventListener::instance().pending() == false);
        CPPUNIT_ASSERT_MESSAGE("Listener: Invalid Finished" , TestJobEventListener::instance().finished() == false);
    }

}
