/*
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *  GLite Data Transfer Agents: Actions test utilities
 *
 *  Authors: Paolo Tedesco <paolo.tedesco@cern.ch>
 *  Version info: $Id: model_test_utils.h,v 1.3 2008/12/11 13:24:44 ptedesco Exp $
 *  Release: $Name:  $
 *
 */
#ifndef GLITE_DATA_TRANSFER_AGENTS_TEST_MODELTESTUTILS_H_
#define GLITE_DATA_TRANSFER_AGENTS_TEST_MODELTESTUTILS_H_

#include "glite/data/transfer/agent/model/Error.h"
#include "glite/data/transfer/agent/model/Job.h"
#include "glite/data/transfer/agent/model/File.h"
#include "glite/data/transfer/agent/model/Transfer.h"

#include <cppunit/extensions/HelperMacros.h>
#include <sstream>
#include <vector>

using namespace glite::data::transfer::agent;

namespace{

#define FAIL_TEST(msg) {std::stringstream err; err << msg; CPPUNIT_FAIL(err.str().c_str());}

//------------------------------------------------------------------------------
//                          string conversion functions
//------------------------------------------------------------------------------

#define E_STR(a) if (value == model::Error::a) return #a

/**
 * Error::Category to string
 */
char * str(const model::Error::Category& value){  
    E_STR(UNDEF);
    E_STR(TRANSFER);
    E_STR(SECURITY);
    E_STR(NETWORK);
    E_STR(CONFIGURATION);
    E_STR(CONSISTENCY);
    E_STR(CATALOG);
    E_STR(USER);
    E_STR(FILE_EXISTS);
    E_STR(INVALID_PATH);
    E_STR(NO_SPACE_LEFT);
    E_STR(INVALID_SIZE);
    E_STR(REQUEST_FAILURE);
    E_STR(CONNECTION);
    E_STR(HTTP_TIMEOUT);
    E_STR(REQUEST_TIMEOUT);
    E_STR(TRANSFER_TIMEOUT);
    E_STR(INTERNAL_ERROR);
    E_STR(ABORTED);
    E_STR(GENERAL_FAILURE);
    E_STR(LOST);
    E_STR(STORAGE_INTERNAL_ERROR);
    E_STR(GRIDFTP);
    E_STR(MARKERS_TIMEOUT);
    E_STR(SRM_COPY);
    E_STR(CATALOG_INTERNAL_ERROR);
    E_STR(LOCALITY);
    E_STR(FIRST_MARKER_TIMEOUT);
    E_STR(NO_PROGRESS);
    // deprecated
    E_STR(AGENT);
    E_STR(SOURCE);
    E_STR(SOURCE_PREP);
    E_STR(SOURCE_FINAL);
    E_STR(DESTINATION);
    E_STR(DESTINATION_PREP);
    E_STR(DESTINATION_FINAL);
    E_STR(REMOTE);
    E_STR(CATALOG_SRC);
    E_STR(CATALOG_SRC_RES);
    E_STR(CATALOG_DST);
    E_STR(CATALOG_DST_RES);
    E_STR(CATALOG_DST_REG);
    return "<Unknown Category>";
}

/**
 * Error::Scope to string
 */
char * str(const model::Error::Scope& value){
    E_STR(S_UNDEF);
    E_STR(S_SOURCE);
    E_STR(S_DESTINATION);
    E_STR(S_TRANSFER);
    E_STR(S_AGENT);
    return "<Unknown Scope>";
}

/**
 * Error::Phase to string
 */
char * str(const model::Error::Phase& value){
    E_STR(P_UNDEF);
    E_STR(P_TX_PREPARATION);
    E_STR(P_TX_FINALIZATION);
    E_STR(P_TRANSFER);
    E_STR(P_ALLOCATION);
    E_STR(P_CATALOG_RESOLUTION);
    E_STR(P_CATALOG_REGISTRATION);
    E_STR(P_PRESTAGING);
    E_STR(P_FINALIZATION);
    E_STR(P_WAITING);
    E_STR(P_WAITING_CATALOG);
    E_STR(P_CANCELING);
    E_STR(P_TX_SERVICE);
    E_STR(P_PRE_TRANSFER);
    E_STR(P_POST_TRANSFER);
    E_STR(P_CHECK_READINESS);
    return "<Unknown Phase>";
}


typedef std::vector<std::string> FlagEnumStrings;

template <typename FlagEnum>
void check_flag_value(
    FlagEnum            flag,
    FlagEnum&           value,
    const std::string&  valueString,
    FlagEnumStrings&    strings){
        
    if ((flag & value) == flag){
        value = (FlagEnum)(value - flag);
        strings.push_back(valueString);
    }
}

template <typename FlagEnum>
std::string join_flag_values(
    const FlagEnumStrings&  strings,
    FlagEnum                value,
    FlagEnum                expected,
    const std::string&      unknown){
    
    if (0 == strings.size()){
        return unknown;
    }
    std::stringstream ret;
    FlagEnumStrings::const_iterator it = strings.begin();
    ret << (*it);
    for (++ it;it != strings.end();++it){
        ret << " | " << (*it);
    }
    if (expected != value){
        ret << " | " << unknown << " (" << value << ")";
    }
    return ret.str();
}
        

#define F_STR(a) check_flag_value(model::File::a,value,#a,strings);

/**
 * File::State to string
 */
std::string str(model::File::State value){
    if (model::File::S_UNDEF == value){
        // S_UNDEF -> S_UNDEF
        return "S_UNDEF";
    } else {
        // check combined values
        FlagEnumStrings strings;
        F_STR(S_SUBMITTED);
        F_STR(S_PENDING);
        F_STR(S_READY);
        F_STR(S_ACTIVE);
        F_STR(S_DONE);
        F_STR(S_WAITING);
        F_STR(S_HOLD);
        F_STR(S_FAILED);
        F_STR(S_FINISHING);
        F_STR(S_FINISHED);
        F_STR(S_AWAITING_PRESTAGE);
        F_STR(S_PRESTAGING);
        F_STR(S_WAITING_PRESTAGE);
        F_STR(S_WAITING_CATALOG_RESOLUTION);
        F_STR(S_WAITING_CATALOG_REGISTRATION);
        F_STR(S_CANCELED);
        return join_flag_values(strings,value,model::File::S_UNDEF,"Unknown File State");
    }
}

#define J_STR(a) check_flag_value(model::Job::a,value,#a,strings);

/**
 * Job::State to string
 */
std::string str(model::Job::State value){
    if (model::Job::S_UNDEF == value){
        // S_UNDEF -> S_UNDEF
        return "S_UNDEF";
    } else {
        // check combined values
        FlagEnumStrings strings;
        J_STR(S_SUBMITTED);
        J_STR(S_PENDING);
        J_STR(S_READY);
        J_STR(S_ACTIVE);
        J_STR(S_DONE);
        J_STR(S_DONE_WITH_ERRORS);
        J_STR(S_HOLD);
        J_STR(S_CANCELED);
        J_STR(S_FAILED);
        J_STR(S_FINISHING);
        J_STR(S_FINISHED);
        J_STR(S_FINISHED_DIRTY);
        return join_flag_values(strings,value,model::Job::S_UNDEF,"Unknown Job State");
    }
}

#define T_STR(a) check_flag_value(model::Transfer::a,value,#a,strings);

/**
 * Transfer::State to string
 */
std::string str(model::Transfer::State value){
    if (model::Transfer::S_UNDEF == value){
        // S_UNDEF -> S_UNDEF
        return "S_UNDEF";
    } else {
        // check combined values
        FlagEnumStrings strings;
        T_STR(S_RECEIVED);
        T_STR(S_PREPARING);
        T_STR(S_READY);
        T_STR(S_SUBMITTED);
        T_STR(S_PROCESSING);
        T_STR(S_TRANSFERRED);
        T_STR(S_COMPLETED);
        T_STR(S_FAILED);
        T_STR(S_ABORTED);
        return join_flag_values(strings,value,model::Transfer::S_UNDEF,"Unknown Transfer State");
    }
}

} // namespace

#endif
