/*
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *  GLite Data Transfer Agents: NameGeneration Test Case
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *  Version info: $Id: NameGenerationTestCase.cpp,v 1.3 2006/08/23 14:31:51 badino Exp $
 *  Release: $Name:  $
 *
 */
#include "glite/data/transfer/agent/namegen/NameGenerationFactory.h"
#include "glite/data/transfer/agent/namegen/NameGeneration.h"
#include "glite/data/transfer/agent/namegen/NameGenerationException.h"

#include <boost/scoped_ptr.hpp>
#include <cppunit/TestCase.h>
#include <cppunit/extensions/HelperMacros.h>
#include <unistd.h>
 
using namespace glite::data::transfer::agent;
using namespace glite::data::transfer::agent::namegen;
using boost::scoped_ptr;

/**
 * CPPUnit TestCase
 */
class NameGenerationTestCase : public CppUnit::TestCase{
public:
    /**
     * set-up test case
     */
    void setUp ();
private:
    // Declare CPPUnit Methods
    // -------------------------------------------------------------------------
    CPPUNIT_TEST_SUITE( NameGenerationTestCase);
    CPPUNIT_TEST( testGenerateWithLFN );
    CPPUNIT_TEST( testGenerateWithGUID );
    CPPUNIT_TEST_SUITE_END();

    // Test Methods
    // -------------------------------------------------------------------------

    /**
     * Test the name generation passing an LFN
     */
    void testGenerateWithLFN();
    /**
     * Test the name generation passing a GUID
     */
    void testGenerateWithGUID ();
private:
    /**
     * NameGeneration Instance
     */
    scoped_ptr<NameGeneration> m_service;
};

/*
 * Register Suit
 */
CPPUNIT_TEST_SUITE_NAMED_REGISTRATION( NameGenerationTestCase, "NameGenerationTestCase" );

namespace {
const char * const NG_TESTCASE_ENDPOINT      = "httpg://castor.cern.ch:8443/srm/managerv1";
const char * const NG_TESTCASE_SE_PATH       = "/SE";
const char * const NG_TESTCASE_LFN_1         = "/tmp/test/lfn1.txt";    
const char * const NG_TESTCASE_SURL_1        = "srm://castor.cern.ch:8443/srm/managerv1?SFN=/SE/tmp/test/lfn1.txt";    
const char * const NG_TESTCASE_LFN_2         = "lfn:///tmp/test/lfn2.txt";    
const char * const NG_TESTCASE_SURL_2        = "srm://castor.cern.ch:8443/srm/managerv1?SFN=/SE/tmp/test/lfn2.txt";    
const char * const NG_TESTCASE_LFN_3         = "lfn:/tmp/test/lfn3.txt";    
const char * const NG_TESTCASE_SURL_3        = "srm://castor.cern.ch:8443/srm/managerv1?SFN=/SE/tmp/test/lfn3.txt";    
const char * const NG_TESTCASE_GUID_1        = "0000e6f0-4e81-1255-af7f-000000000000";
const char * const NG_TESTCASE_SURL_G1       = "srm://castor.cern.ch:8443/srm/managerv1?SFN=/SE/9/35128/0000e6f0-4e81-1255-af7f-000000000000";
const char * const NG_TESTCASE_GUID_2        = "guid:0000e6f0-4e81-1255-af7f-000000000001";
const char * const NG_TESTCASE_SURL_G2       = "srm://castor.cern.ch:8443/srm/managerv1?SFN=/SE/8/35127/0000e6f0-4e81-1255-af7f-000000000001";
const char * const NG_TESTCASE_GUID_3        = "guid://0000e6f0-4e81-1255-af7f-000000000002";
const char * const NG_TESTCASE_SURL_G3       = "srm://castor.cern.ch:8443/srm/managerv1?SFN=/SE/7/35126/0000e6f0-4e81-1255-af7f-000000000002";
const char * const NG_TESTCASE_GUID_4        = "guid://0000e6f0-4e81-1255-af7f-00000000003";
const char * const NG_TESTCASE_SURL_G4       = "srm://castor.cern.ch:8443/srm/managerv1?SFN=/SE/GUID/0000e6f0-4e81-1255-af7f-00000000003";
}

/*
 * setUp
 *
 * set-up test case
 */
void NameGenerationTestCase::setUp(){

    try{
        NameGenerationFactory& factory = NameGenerationFactory::instance();
        // Create NameGeneration Instance
        m_service.reset(factory.create());
    } catch(const std::exception& exc){
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        CPPUNIT_FAIL("Got Unknown Error");
    }
}

/*
 * Test GenerateWithLFN
 */
void NameGenerationTestCase::testGenerateWithLFN(){
    
    try{
        std::string surl;
        surl = m_service->generate(NG_TESTCASE_LFN_1,NG_TESTCASE_ENDPOINT,NG_TESTCASE_SE_PATH,"");
        CPPUNIT_ASSERT_MESSAGE("Failed LFN 1", surl == NG_TESTCASE_SURL_1);

        surl = m_service->generate(NG_TESTCASE_LFN_2,NG_TESTCASE_ENDPOINT,NG_TESTCASE_SE_PATH,"");
        CPPUNIT_ASSERT_MESSAGE("Failed LFN 2", surl == NG_TESTCASE_SURL_2);

        surl = m_service->generate(NG_TESTCASE_LFN_3,NG_TESTCASE_ENDPOINT,NG_TESTCASE_SE_PATH,"");
        CPPUNIT_ASSERT_MESSAGE("Failed LFN 3", surl == NG_TESTCASE_SURL_3);
    } catch(const std::exception& exc){
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        CPPUNIT_FAIL("Got Unknown Error");
    }    
}
/*
 * Test GenerateGUID
 */
void NameGenerationTestCase::testGenerateWithGUID(){
    
    try{
        std::string surl;
        surl = m_service->generate(NG_TESTCASE_GUID_1,NG_TESTCASE_ENDPOINT,NG_TESTCASE_SE_PATH,"");
        CPPUNIT_ASSERT_MESSAGE("Failed GUID 1", surl == NG_TESTCASE_SURL_G1);

        surl = m_service->generate(NG_TESTCASE_GUID_2,NG_TESTCASE_ENDPOINT,NG_TESTCASE_SE_PATH,"");
        CPPUNIT_ASSERT_MESSAGE("Failed GUID 2", surl == NG_TESTCASE_SURL_G2);

        surl = m_service->generate(NG_TESTCASE_GUID_3,NG_TESTCASE_ENDPOINT,NG_TESTCASE_SE_PATH,"");
        CPPUNIT_ASSERT_MESSAGE("Failed GUID 3", surl == NG_TESTCASE_SURL_G3);

        surl = m_service->generate(NG_TESTCASE_GUID_4,NG_TESTCASE_ENDPOINT,NG_TESTCASE_SE_PATH,"");
        CPPUNIT_ASSERT_MESSAGE("Failed GUID 4", surl == NG_TESTCASE_SURL_G4);
    } catch(const std::exception& exc){
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        CPPUNIT_FAIL("Got Unknown Error");
    }    
}
