/*
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *  GLite Data Transfer Agents: Python Test Case
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>, Paolo Tedesco <paolo.tedesco@cern.ch>
 *  Version info: $Id: PythonTestCase.cpp,v 1.11 2008/11/13 10:43:56 ptedesco Exp $
 *  Release: $Name:  $
 *
 */
#include "glite/data/agents/python/PyHelper.h"
#include "glite/data/agents/python/PythonException.h"
#include "glite/data/transfer/agent/model/Job.h"
#include "glite/data/transfer/agent/model/File.h"
#include "glite/data/transfer/agent/model/Transfer.h"
#include "glite/data/transfer/agent/model/StageReq.h"

#include <boost/python.hpp>
#include <boost/shared_ptr.hpp>
#include <cppunit/TestCase.h>
#include <cppunit/extensions/HelperMacros.h>

using namespace glite::data::agents::python;
using namespace glite::data::transfer::agent;

namespace py = boost::python;

/**
 * CPPUnit TestCase
 */
class PythonTestCase : public CppUnit::TestCase{
public:
    /**
     * set-up test case
     */
    void setUp ();
private:
    // Declare CPPUnit Methods
    // -------------------------------------------------------------------------
    CPPUNIT_TEST_SUITE( PythonTestCase);
    CPPUNIT_TEST( testJob );
    CPPUNIT_TEST( testFile );
    CPPUNIT_TEST( testTransfer );
    CPPUNIT_TEST( testStageReq );
    CPPUNIT_TEST( testFileArray );
    CPPUNIT_TEST( testTransferArray );
    CPPUNIT_TEST_SUITE_END();

    // Test Methods
    // -------------------------------------------------------------------------

    /**
     * Test Job Class
     */
    void testJob();
    /**
     * Test File class
     */
    void testFile();
    /**
     * Test Transfer class
     */
    void testTransfer();
    /**
     * Test the StageReq Class
     */
    void testStageReq();
    /**
     * Test FileArray class
     */
    void testFileArray();
    /**
     * Test TransferArray class
     */
    void testTransferArray();
};

/*
 * Register Suit
 */
CPPUNIT_TEST_SUITE_NAMED_REGISTRATION( PythonTestCase, "PythonTestCase" );


namespace {
const char * const PYTHON_TEST_MODULE        = "test_fts_python";
const char * const PYTHON_TEST_VERSION       = "TestSchemaVersion";
const char * const PYTHON_TEST_JOB           = "TestJob";
const char * const PYTHON_TEST_FILE          = "TestFile";
const char * const PYTHON_TEST_TRANSFER      = "TestTransfer";
const char * const PYTHON_TEST_STAGEREQ      = "TestStageReq";
const char * const PYTHON_TEST_FILE_ARRAY    = "TestFileArray";
const char * const PYTHON_TEST_TRANSFER_ARRAY= "TestTransferArray";
}

/*
 * setUp
 *
 * set-up test case
 */
void PythonTestCase::setUp(){
    
    // Test Schema Version
    try{
        // Import a python Module
        py::object module = PyHelper::instance().import(PYTHON_TEST_MODULE);
        CPPUNIT_ASSERT_MESSAGE("Null Module", module.ptr() != 0);
        
        // Get the function object
        py::object function = PyHelper::instance().getFunction(module, PYTHON_TEST_VERSION);
        CPPUNIT_ASSERT_MESSAGE("Null function", function.ptr() != 0);
        
        // Invoke the Function
        bool result = py::call<bool>(function.ptr(),QUEUE_SCHEMA_VERSION);
        CPPUNIT_ASSERT_MESSAGE("Invalid schema version.", result == true);
    } catch(py::error_already_set){
        std::pair<std::string,std::string> err = PyHelper::instance().lastError();
        std::string reason = (std::string)err.first.c_str() + " : " + err.second.c_str();
        CPPUNIT_FAIL(reason.c_str());
    } catch(const std::exception& exc){
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        CPPUNIT_FAIL("Got Unknown Error");
    }  
}

/*
 * Test Job Class
 */
void PythonTestCase::testJob(){
    
    try{
        // Import a python Module
        py::object module = PyHelper::instance().import(PYTHON_TEST_MODULE);
        CPPUNIT_ASSERT_MESSAGE("Null Module", module.ptr() != 0);
        
        // Get the function object
        py::object function = PyHelper::instance().getFunction(module, PYTHON_TEST_JOB);
        CPPUNIT_ASSERT_MESSAGE("Null function", function.ptr() != 0);
        
        time_t current;
        time(&current);
        
        // Create a Job
        // Note : internal_params are set but are NOT visible from python module
        model::Job job(
                     "10000000-0000-0000-0000-000000000001",// job_id
                     model::Job::S_SUBMITTED,               // state
                     "VO1",                                 // vo_name
                     "CHANNEL_1",                           // channel_name
                     4,                                     // priority
                     false,                                 // cancel_job
                     "SRC_SITE",                            // source
                     "DST_SITE",                            // dest
                     "My DN",                               // user_dn
                     "some params",                         // job_params
                     "a reason",                            // reason
                     current,                               // submit_time
                     "Agent DN",                            // agent_dn
                     "localhost",                           // submit_host
                     0,                                     // max_time_in_queue
                     (time_t)-1,                            // finish_time
                     "myproxy.cern.ch",                     // myproxy_server
                     "SPACE_TOKEN",                         // space_token
                     "volatile",                            // storage_class
                     "CRED",                                // user_cred
                     "CRED_ID",                             // cred_id
                     "VOMS_CRED",                           // voms_cred
                     "--internal");                         // internal_params

        boost::shared_ptr<model::Job> j(new model::Job(
                     "10000000-0000-0000-0000-000000000001",// job_id
                     model::Job::S_SUBMITTED,               // state
                     "VO1",                                 // vo_name
                     "CHANNEL_1",                           // channel_name
                     4,                                     // priority
                     false,                                 // cancel_job
                     "SRC_SITE",                            // source
                     "DST_SITE",                            // dest
                     "My DN",                               // user_dn
                     "some params",                         // job_params
                     "a reason",                            // reason
                     current,                               // submit_time
                     "Agent DN",                            // agent_dn
                     "localhost",                           // submit_host
                     0,                                     // max_time_in_queue
                     (time_t)-1,                            // finish_time
                     "myproxy.cern.ch",                     // myproxy_server
                     "SPACE_TOKEN",                         // space_token
                     "volatile",                            // storage_class
                     "CRED",                                // user_cred
                     "CRED_ID",                             // cred_id
                     "VOMS_CRED",                           // voms_cred
                     "--internal"));                        // internal_params

        // Invoke the Function with the object
        std::string result = py::call<std::string>(function.ptr(),job);
        std::string msg = (std::string)"Job obj:" + result;
        CPPUNIT_ASSERT_MESSAGE(msg.c_str(), result.empty() == true);
        // Invoke the Function with the pointer
        result = py::call<std::string>(function.ptr(),j);
        msg = (std::string)"Job ptr:" + result;
        CPPUNIT_ASSERT_MESSAGE(msg.c_str(), result.empty() == true);
    } catch(py::error_already_set){
        std::pair<std::string,std::string> err = PyHelper::instance().lastError();
        std::string reason = (std::string)err.first.c_str() + " : " + err.second.c_str();
        CPPUNIT_FAIL(reason.c_str());
    } catch(const std::exception& exc){
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        CPPUNIT_FAIL("Got Unknown Error");
    }
}

/*
 * Test File Class
 */
void PythonTestCase::testFile(){
    
    try{
        // Import a python Module
        py::object module = PyHelper::instance().import(PYTHON_TEST_MODULE);
        CPPUNIT_ASSERT_MESSAGE("Null Module", module.ptr() != 0);
        
        // Get the function object
        py::object function = PyHelper::instance().getFunction(module, PYTHON_TEST_FILE);
        CPPUNIT_ASSERT_MESSAGE("Null function", function.ptr() != 0);
        
        // Create a File
        // Note : internal_params are set but are NOT visible from python module
        model::File file(
                     "100",                                 // id
                     "10000000-0000-0000-0000-000000000001",// job_id
                     model::File::S_PENDING  ,              // state
                     "lfn:///tmp/test.txt",                 // logical
                     "srm://srm.test.com/tmp/test.txt",     // source
                     "srm://srm.nowhere.com/tmp/test.txt",  // dest
                     model::Error::INTERNAL_ERROR,          // reason class
                     "a reason",                            // reason
                     5,                                     // failures
                     4,                                     // current_failures
                     3,                                     // catalog_failures
                     2,                                     // prestage_failures
                     12345678,                              // filesize
                     "0x12345",                             // check_sum
                     (time_t)-1,                            // finish_time
                     "Agent DN",                            // agent_dn
                     "--internal",                          // internal_params
                     // added with schema version 3.1.0
                     model::Error::S_SOURCE,         // error_scope
                     model::Error::P_TX_PREPARATION  // error_phase
                     );

        boost::shared_ptr<model::File> f(new model::File(
                     "100",                                 // id
                     "10000000-0000-0000-0000-000000000001",// job_id
                     model::File::S_PENDING  ,              // state
                     "lfn:///tmp/test.txt",                 // logical
                     "srm://srm.test.com/tmp/test.txt",     // source
                     "srm://srm.nowhere.com/tmp/test.txt",  // dest
                     model::Error::INTERNAL_ERROR,          // reason class
                     "a reason",                            // reason
                     5,                                     // failures
                     4,                                     // current_failures
                     3,                                     // catalog_failures
                     2,                                     // prestage_failures
                     12345678,                              // filesize
                     "0x12345",                             // check_sum
                     (time_t)-1,                            // finish_time
                     "Agent DN",                            // agent_dn
                     "--internal",                          // internal_params
                     // added with schema version 3.1.0
                     model::Error::S_SOURCE,         // error_scope
                     model::Error::P_TX_PREPARATION  // error_phase
                     ));

        // Invoke the Function with the object
        std::string result = py::call<std::string>(function.ptr(),file);
        std::string msg = (std::string)"File obj:" + result;
        CPPUNIT_ASSERT_MESSAGE(msg.c_str(), result.empty() == true);
        // Invoke the Function with the pointer
        result = py::call<std::string>(function.ptr(),f);
        msg = (std::string)"File ptr:" + result;
        CPPUNIT_ASSERT_MESSAGE(msg.c_str(), result.empty() == true);
    } catch(py::error_already_set){
        std::pair<std::string,std::string> err = PyHelper::instance().lastError();
        std::string reason = (std::string)err.first.c_str() + " : " + err.second.c_str();
        CPPUNIT_FAIL(reason.c_str());
    } catch(const std::exception& exc){
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        CPPUNIT_FAIL("Got Unknown Error");
    }
}

/*
 * Test Transfer Class
 */
void PythonTestCase::testTransfer(){
    
    try{
        // Import a python Module
        py::object module = PyHelper::instance().import(PYTHON_TEST_MODULE);
        CPPUNIT_ASSERT_MESSAGE("Null Module", module.ptr() != 0);
        
        // Get the function object
        py::object function = PyHelper::instance().getFunction(module, PYTHON_TEST_TRANSFER);
        CPPUNIT_ASSERT_MESSAGE("Null function", function.ptr() != 0);
        
        time_t current;
        time(&current);
        
        // Create a Transfer
        model::Transfer transfer(
                     "req_1_1",                             // req_id
                     "100",                                 // file_id
                     0,                                     // transfer_id
                     "10000000-0000-0000-0000-000000000001",// job_id
                     model::Transfer::S_RECEIVED,           // state
                     "foo",                                 // transfer type
                     "gsiftp://srm.test.com/tmp/test.txt",  // source
                     "gsiftp://srm.nowhere.com/tmp/test.txt",// dest
                     current,                               // transfer_time
                     current - 10,                          // prepare_time
                     0.1,                                   // duration
                     0.2,                                   // src_preparation_duration
                     0.3,                                   // dest_preparation_duration
                     0.4,                                   // transfer_duration
                     0.5,                                   // src_finalization_duration
                     0.6,                                   // dest_finalization_duration
                     12345,                                 // bytes_written
                     1.1,                                   // throughput
                     model::Error::SECURITY,                // reason class
                     "a reason",                            // reason
                     12345678,                              // filesize
                     "source_srm_token",                    // source_srm_token
                     "dest_srm_token",                      // dest_srm_token
                     "source_host",                         // source_host
                     "dest_host",                           // dest_host
                     // added with schema version 3.1.0
                     model::Error::S_DESTINATION,    // error_scope
                     model::Error::P_TRANSFER,       // error_phase
                     current + 10                           // finish_time
                     );

        // Create a Job
        boost::shared_ptr<model::Transfer> tx(new model::Transfer(
                     "req_1_1",                             // req_id
                     "100",                                 // file_id
                     0,                                     // transfer_id
                     "10000000-0000-0000-0000-000000000001",// job_id
                     model::Transfer::S_RECEIVED,           // state
                     "foo",                                 // transfer type
                     "gsiftp://srm.test.com/tmp/test.txt",  // source
                     "gsiftp://srm.nowhere.com/tmp/test.txt",// dest
                     current,                               // transfer_time
                     current - 10,                          // prepare_time
                     0.1,                                   // duration
                     0.2,                                   // src_preparation_duration
                     0.3,                                   // dest_preparation_duration
                     0.4,                                   // transfer_duration
                     0.5,                                   // src_finalization_duration
                     0.6,                                   // dest_finalization_duration
                     12345,                                 // bytes_written
                     1.1,                                   // throughput
                     model::Error::SECURITY,                // reason class
                     "a reason",                            // reason
                     12345678,                              // filesize
                     "source_srm_token",                    // source_srm_token
                     "dest_srm_token",                      // dest_srm_token
                     "source_host",                         // source_host
                     "dest_host",                           // dest_host
                     // added with schema version 3.1.0
                     model::Error::S_DESTINATION,    // error_scope
                     model::Error::P_TRANSFER,       // error_phase
                     current + 10                           // finish_time
                     ));

        // Invoke the Function with the object
        std::string result = py::call<std::string>(function.ptr(),transfer);
        std::string msg = (std::string)"Tx obj:" + result;
        CPPUNIT_ASSERT_MESSAGE(msg.c_str(), result.empty() == true);
        // Invoke the Function with the pointer
        result = py::call<std::string>(function.ptr(),tx);
        msg = (std::string)"Tx ptr:" + result;
        CPPUNIT_ASSERT_MESSAGE(msg.c_str(), result.empty() == true);
    } catch(py::error_already_set){
        std::pair<std::string,std::string> err = PyHelper::instance().lastError();
        std::string reason = (std::string)err.first.c_str() + " : " + err.second.c_str();
        CPPUNIT_FAIL(reason.c_str());
    } catch(const std::exception& exc){
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        CPPUNIT_FAIL("Got Unknown Error");
    }
}

/*
 * Test StageReq Class
 */
void PythonTestCase::testStageReq(){
    
    try{
        // Import a python Module
        py::object module = PyHelper::instance().import(PYTHON_TEST_MODULE);
        CPPUNIT_ASSERT_MESSAGE("Null Module", module.ptr() != 0);
        
        // Get the function object
        py::object function = PyHelper::instance().getFunction(module, PYTHON_TEST_STAGEREQ);
        CPPUNIT_ASSERT_MESSAGE("Null function", function.ptr() != 0);
        
        time_t current;
        time(&current);
        
        // Create a StageReq
        model::StageReq stagereq(
                     "stagereq_1_1",                  // req_id
                     "100",                                 // file_id
                     0,                                     // stage_id
                     "10000000-0000-0000-0000-000000000001",// job_id
                     model::StageReq::S_SUBMITTED,          // state
                     "gsiftp://srm.test.com/tmp/test.txt",  // turl
                     current,                               // request_time
                     0.1,                                   // duration
                     0.2,                                   // finalization_duration
                     model::Error::SOURCE_PREP,             // reason class
                     "a reason",                            // reason
                     12345678,                              // filesize
                     // added with schema version 3.1.0
                     model::Error::S_AGENT,          // error_scope
                     model::Error::P_CATALOG_REGISTRATION,       // error_phase
                     current + 10                           // finish_time
                     );

        boost::shared_ptr<model::StageReq> sr(new model::StageReq(
                     "stagereq_1_1",                  // req_id
                     "100",                                 // file_id
                     0,                                     // stage_id
                     "10000000-0000-0000-0000-000000000001",// job_id
                     model::StageReq::S_SUBMITTED,          // state
                     "gsiftp://srm.test.com/tmp/test.txt",  // turl
                     current,                               // request_time
                     0.1,                                   // duration
                     0.2,                                   // finalization_duration
                     model::Error::SOURCE_PREP,             // reason class
                     "a reason",                            // reason
                     12345678,                              // filesize
                     // added with schema version 3.1.0
                     model::Error::S_AGENT,          // error_scope
                     model::Error::P_CATALOG_REGISTRATION,       // error_phase
                     current + 10                           // finish_time
                     ));

        // Invoke the Function with the object
        std::string result = py::call<std::string>(function.ptr(),stagereq);
        std::string msg = (std::string)"StageReq obj:" + result;
        CPPUNIT_ASSERT_MESSAGE(msg.c_str(), result.empty() == true);
        // Invoke the Function with the pointer
        result = py::call<std::string>(function.ptr(),sr);
        msg = (std::string)"StageReq ptr:" + result;
        CPPUNIT_ASSERT_MESSAGE(msg.c_str(), result.empty() == true);
    } catch(py::error_already_set){
        std::pair<std::string,std::string> err = PyHelper::instance().lastError();
        std::string reason = (std::string)err.first.c_str() + " : " + err.second.c_str();
        CPPUNIT_FAIL(reason.c_str());
    } catch(const std::exception& exc){
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        CPPUNIT_FAIL("Got Unknown Error");
    }
}


/*
 * Test FileArray Class
 */
void PythonTestCase::testFileArray(){
    
    try{
        // Import a python Module
        py::object module = PyHelper::instance().import(PYTHON_TEST_MODULE);
        CPPUNIT_ASSERT_MESSAGE("Null Module", module.ptr() != 0);
        
        // Get the function object
        py::object function = PyHelper::instance().getFunction(module, PYTHON_TEST_FILE_ARRAY);
        CPPUNIT_ASSERT_MESSAGE("Null function", function.ptr() != 0);

        std::vector<boost::shared_ptr<model::File> >       file_array;
        std::vector<boost::shared_ptr<const model::File> > file_const_array;
        for (int i = 0; i < 10; ++i){        
            boost::shared_ptr<model::File> f(new model::File(
                     "100",                                 // id
                     "10000000-0000-0000-0000-000000000001",// job_id
                     model::File::S_PENDING  ,              // state
                     "lfn:///tmp/test.txt",                 // logical
                     "srm://srm.test.com/tmp/test.txt",     // source
                     "srm://srm.nowhere.com/tmp/test.txt",  // dest
                     model::Error::INTERNAL_ERROR,          // reason class
                     "a reason",                            // reason
                     5,                                     // failures
                     4,                                     // current_failures
                     3,                                     // catalog_failures
                     2,                                     // prestage_failures
                     12345678,                              // filesize
                     "0x12345",                             // check_sum
                     (time_t)-1,                            // finish_time
                     "Agent DN",                            // agent_dn
                     "--internal",                          // internal_params
                     // added with schema version 3.1.0
                     model::Error::S_SOURCE,         // error_scope
                     model::Error::P_TX_PREPARATION  // error_phase
                     ));
            // Save Object
            file_array.push_back(f);
            file_const_array.push_back(f);
        }

        std::string result,msg;
        // Invoke the Function with the const array
        result = py::call<std::string>(function.ptr(),file_const_array);
        msg = (std::string)"File Const Array:" + result;
        CPPUNIT_ASSERT_MESSAGE(msg.c_str(), result.empty() == true);
        // Invoke the Function with the array
        result = py::call<std::string>(function.ptr(),file_array);
        msg = (std::string)"File Array:" + result;
        CPPUNIT_ASSERT_MESSAGE(msg.c_str(), result.empty() == true);
    } catch(py::error_already_set){
        std::pair<std::string,std::string> err = PyHelper::instance().lastError();
        std::string reason = (std::string)err.first.c_str() + " : " + err.second.c_str();
        CPPUNIT_FAIL(reason.c_str());
    } catch(const std::exception& exc){
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        CPPUNIT_FAIL("Got Unknown Error");
    }
}

/*
 * Test TransferArray Class
 */
void PythonTestCase::testTransferArray(){
    
    try{
        // Import a python Module
        py::object module = PyHelper::instance().import(PYTHON_TEST_MODULE);
        CPPUNIT_ASSERT_MESSAGE("Null Module", module.ptr() != 0);
        
        // Get the function object
        py::object function = PyHelper::instance().getFunction(module, PYTHON_TEST_TRANSFER_ARRAY);
        CPPUNIT_ASSERT_MESSAGE("Null function", function.ptr() != 0);
        
        time_t current;
        time(&current);
        
        std::vector<boost::shared_ptr<model::Transfer> > transfer_array;
        std::vector<boost::shared_ptr<const model::Transfer> > transfer_const_array;
        for(int i = 0; i < 15; ++i){
            // Create a Transfer
            boost::shared_ptr<model::Transfer> tx(new model::Transfer(
                     "req_1_1",                       // req_id
                     "100",                                 // file_id
                     0,                                     // transfer_id
                     "10000000-0000-0000-0000-000000000001",// job_id
                     model::Transfer::S_RECEIVED,           // state
                     "foo",                                 // transfer type
                     "gsiftp://srm.test.com/tmp/test.txt",  // source
                     "gsiftp://srm.nowhere.com/tmp/test.txt",// dest
                     current,                               // transfer_time
                     current - 10,                          // prepare_time
                     0.1,                                   // duration
                     0.2,                                   // src_preparation_duration
                     0.3,                                   // dest_preparation_duration
                     0.4,                                   // transfer_duration
                     0.5,                                   // src_finalization_duration
                     0.6,                                   // dest_finalization_duration
                     12345,                                 // bytes_written
                     1.1,                                   // throughput
                     model::Error::SECURITY,                // reason class
                     "a reason",                            // reason
                     12345678,                              // filesize
                     "source_srm_token",                    // source_srm_token
                     "dest_srm_token",                      // dest_srm_token
                     "source_host",                         // source_host
                     "dest_host",                           // dest_host
                     // added with schema version 3.1.0
                     model::Error::S_DESTINATION,           // error_scope
                     model::Error::P_TRANSFER,              // error_phase
                     current + 10                           // finish_time
                     ));
                     
            // Save Object
            transfer_array.push_back(tx);
            transfer_const_array.push_back(tx);
        }

        std::string result,msg;
        // Invoke the Function with the const array
        result = py::call<std::string>(function.ptr(),transfer_const_array);
        msg = (std::string)"Tx Const Array:" + result;
        CPPUNIT_ASSERT_MESSAGE(msg.c_str(), result.empty() == true);
        // Invoke the Function with the array
        result = py::call<std::string>(function.ptr(),transfer_array);
        msg = (std::string)"Tx Array:" + result;
        CPPUNIT_ASSERT_MESSAGE(msg.c_str(), result.empty() == true);
    } catch(py::error_already_set){
        std::pair<std::string,std::string> err = PyHelper::instance().lastError();
        std::string reason = (std::string)err.first.c_str() + " : " + err.second.c_str();
        CPPUNIT_FAIL(reason.c_str());
    } catch(const std::exception& exc){
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        CPPUNIT_FAIL("Got Unknown Error");
    }
}
