/*
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *  GLite Data Transfer Agents: 13301 Unit Test Case
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *  Version info: $Id: TestCase13301.cpp,v 1.5 2008/11/13 10:43:56 ptedesco Exp $
 *  Release: $Name:  $
 *
 */
#include "Fetch.h"
#include "ChannelFlags.h"
#include "actions/channel/ActionsConfig.h"
#include "glite/data/agents/dao/DAOContext.h"
#include "glite/data/agents/dao/DAOContextFactory.h"
#include "glite/data/transfer/agent/action/ActionFactory.h"
#include "glite/data/transfer/agent/model/File.h"
#include "glite/data/transfer/agent/model/Job.h"
#include "glite/data/transfer/agent/model/Transfer.h"
#include "glite/data/transfer/agent/model/Channel.h"
#include "glite/data/transfer/agent/dao/channel/FileDAO.h"
#include "glite/data/transfer/agent/dao/channel/JobDAO.h"
#include "glite/data/transfer/agent/dao/channel/TransferDAO.h"
#include "glite/data/transfer/agent/dao/channel/DAOFactory.h"
#include "glite/data/transfer/agent/ts/TransferService.h"
#include "glite/data/transfer/agent/ts/TransferServiceFactory.h"

#include <memory>
#include <boost/scoped_ptr.hpp>
#include <cppunit/TestCase.h>
#include <cppunit/extensions/HelperMacros.h>
#include <glite/data/config/service/Logger.hxx>

using namespace glite::data::transfer::agent;
using namespace glite::data::transfer::agent::action;
using namespace glite::data::transfer::agent::action::channel;
using namespace glite::data::transfer::agent::model;
using namespace glite::data::transfer::agent::dao::channel;
using namespace glite::data::transfer::agent::ts;

using boost::scoped_ptr;

/**
 * CPPUnit TestCase
 */
class TestCase13301 : public CppUnit::TestCase{
public:
    /**
     * set-up test case
     */
    void setUp ();

    /**
     * tear-down test case
     */
    void tearDown ();
private:
    // Declare CPPUnit Methods
    // -------------------------------------------------------------------------
    CPPUNIT_TEST_SUITE( TestCase13301 );
    CPPUNIT_TEST( testExecute1 );
    CPPUNIT_TEST( testExecute2 );
    CPPUNIT_TEST_SUITE_END();

    // Test Methods
    // -------------------------------------------------------------------------

    /**
     * Test Execute Method
     */
    void testExecute1();

    /**
     * Test Execute Method
     */
    void testExecute2();

private:
    /**
     * The Action Instance
     */
    scoped_ptr<Fetch> m_action;
};

/*
 * Register Suit
 */
CPPUNIT_TEST_SUITE_NAMED_REGISTRATION( TestCase13301, "TestCase13301" );

/*
 * Declare Constants. This unit test foresee at beginning
 */
namespace {
const char * const T1_CHANNEL_NAME  = "channel13301_1";
const char * const T2_CHANNEL_NAME  = "channel13301_2";
const char * const VO1_NAME         = "VO13301_1";
const char * const VO2_NAME         = "VO13301_2";
const char * const VO4_NAME         = "VO13301_4";

const unsigned long NUM_ACTIVE_TRANSFERS         = 20;
const unsigned long MAX_TRANSFERS_REQ            = 50;
const unsigned long T1_NUM_ACTIVE_TRANSFERS_VO_1 = 6;
const unsigned long T1_NUM_ACTIVE_TRANSFERS_VO_2 = 12;
const unsigned long T1_NUM_ACTIVE_TRANSFERS_VO_4 = 2;
const unsigned long T1_MAX_TRANSFERS             = 24;
const unsigned long T2_NUM_ACTIVE_TRANSFERS_VO_1 = 6;
const unsigned long T2_NUM_ACTIVE_TRANSFERS_VO_2 = 12;
const unsigned long T2_NUM_ACTIVE_TRANSFERS_VO_4 = 2;
const unsigned long T2_MAX_TRANSFERS             = 50;
}

/*
 * setUp
 *
 * set-up test case
 */
void TestCase13301::setUp(){
    
    try{
        ActionsConfig::instance().activeTxCache().reset();
        
        // Create Action instance
        std::auto_ptr<Action> a(ActionFactory::instance().create("glite:Fetch"));
        CPPUNIT_ASSERT_MESSAGE("Null Action", a.get() != 0);
        CPPUNIT_ASSERT_MESSAGE("Invalid Action Class", dynamic_cast<Fetch*>(a.get()) != 0);

        // Store Action        
        m_action.reset(dynamic_cast<Fetch*>(a.release()));
        CPPUNIT_ASSERT_MESSAGE("Null Action", m_action.get() != 0);
    } catch(const std::exception& exc){
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        CPPUNIT_FAIL("Got Unknown Error");
    }
}

/*
 * tearDown
 *
 * tear-down test case
 */
void TestCase13301::tearDown(){
    ActionsConfig::instance().activeTxCache().reset();
}
/*
 *
 * Test Execute Method
 */
void TestCase13301::testExecute1(){
    
    const char * CAT = "unit-test-13301_1";
    try{
        
        // Get Context per Channel Operations
        scoped_ptr<glite::data::agents::dao::DAOContext> chCtx(
            glite::data::agents::dao::DAOContextFactory::instance().create());
        
        // Set Context
        m_action->context(T1_CHANNEL_NAME,*chCtx);
        
        // Create the Transfer DAO Instance
        scoped_ptr<TransferDAO> txDAO(
            DAOFactory::instance().createTransferDAO(T1_CHANNEL_NAME, *chCtx));

        // Create the Channel DAO Instance
        scoped_ptr<dao::channel::ChannelDAO> chDAO(
            DAOFactory::instance().createChannelDAO(T1_CHANNEL_NAME, *chCtx));

        // Check Preconditions
        // ---------------------------------------------------------------------
          
        // Get the VO Share for the two VOs
        double share = chDAO->getShare(VO1_NAME,true,true);
        _log_debug(CAT,"VO1 Share is " << share);
        CPPUNIT_ASSERT_MESSAGE("Precondition check failed: Invalid Share (VO1)", share == (double)0.5);
        share = chDAO->getShare(VO2_NAME,true,true);
        _log_debug(CAT,"VO2 Share is " << share);
        CPPUNIT_ASSERT_MESSAGE("Precondition check failed: Invalid Share (VO1)", share == (double)0.5);
        
        // Test Max Transfers
        scoped_ptr<Channel> ch(chDAO->get(T1_CHANNEL_NAME));
        CPPUNIT_ASSERT_MESSAGE("Precondition check failed: Invalid Channel", ch.get() != 0);
        unsigned long max_transfers = ch->files();
        _log_debug(CAT,"Max Transfers for channel is " << max_transfers);
        CPPUNIT_ASSERT_MESSAGE("Precondition Failed: Invalid max file transfers", max_transfers == T1_MAX_TRANSFERS);
        
        // Execute Actions
        m_action->execute(CHANNEL_VO_SHARE_TYPE_NORMALIZED_ON_ACTIVE);
       
        // Check now the Number of transfers
        unsigned long count = ActionsConfig::instance().activeTxCache().count();
        _log_debug(CAT,"Got " << count << " active transfers");
        CPPUNIT_ASSERT_MESSAGE("Invalid number of active transfers ", count == T1_MAX_TRANSFERS);
        count = ActionsConfig::instance().activeTxCache().count(VO1_NAME);
        _log_debug(CAT,"Got " << count << " active transfers for VO1");
        CPPUNIT_ASSERT_MESSAGE("Invalid number of active transfers (VO1)", count == (T1_MAX_TRANSFERS - (T1_NUM_ACTIVE_TRANSFERS_VO_2 + T1_NUM_ACTIVE_TRANSFERS_VO_4)));
        count = ActionsConfig::instance().activeTxCache().count(VO2_NAME);
        _log_debug(CAT,"Got " << count << " active transfers for VO2");
        CPPUNIT_ASSERT_MESSAGE("Invalid number of active transfers (VO2)", count == T1_NUM_ACTIVE_TRANSFERS_VO_2);
        
    } catch(const std::exception& exc){
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        CPPUNIT_FAIL("Got Unknown Error");
    }
}

/*
 *
 * Test Execute Method
 */
void TestCase13301::testExecute2(){
    
    const char * CAT = "unit-test-13301_2";
    try{
        
        // Get Context per Channel Operations
        scoped_ptr<glite::data::agents::dao::DAOContext> chCtx(
            glite::data::agents::dao::DAOContextFactory::instance().create());
        
        // Set Context
        m_action->context(T2_CHANNEL_NAME,*chCtx);
        
        // Create the Transfer DAO Instance
        scoped_ptr<TransferDAO> txDAO(
            DAOFactory::instance().createTransferDAO(T2_CHANNEL_NAME, *chCtx));

        // Create the Channel DAO Instance
        scoped_ptr<dao::channel::ChannelDAO> chDAO(
            DAOFactory::instance().createChannelDAO(T2_CHANNEL_NAME, *chCtx));
        
        // Check Preconditions
        // ---------------------------------------------------------------------
        
        // Get the VO Share for the two VOs
        double share = chDAO->getShare(VO1_NAME,true,true);
        _log_debug(CAT,"VO1 Share is " << share);
        CPPUNIT_ASSERT_MESSAGE("Precondition check failed: Invalid Share (VO1)", share == (double)0.5);
        share = chDAO->getShare(VO2_NAME,true,true);
        _log_debug(CAT,"VO2 Share is " << share);
        CPPUNIT_ASSERT_MESSAGE("Precondition check failed: Invalid Share (VO1)", share == (double)0.5);
        
        // Test Max Transfers
        scoped_ptr<Channel> ch(chDAO->get(T2_CHANNEL_NAME));
        CPPUNIT_ASSERT_MESSAGE("Precondition check failed: Invalid Channel", ch.get() != 0);
        unsigned long max_transfers = ch->files();
        _log_debug(CAT,"Max Transfers for channel is " << max_transfers);
        CPPUNIT_ASSERT_MESSAGE("Precondition Failed: Invalid max file transfers", max_transfers == T2_MAX_TRANSFERS);
        
        // Execute Actions
        m_action->execute(CHANNEL_VO_SHARE_TYPE_NORMALIZED_ON_ACTIVE);
       
        // Check now the Number of transfers
        unsigned long count = ActionsConfig::instance().activeTxCache().count();
        _log_debug(CAT,"Got " << count << " active transfers");
        CPPUNIT_ASSERT_MESSAGE("Invalid number of active transfers ", count == T2_MAX_TRANSFERS);
        count = ActionsConfig::instance().activeTxCache().count(VO1_NAME);
        _log_debug(CAT,"Got " << count << " active transfers for VO1");
        CPPUNIT_ASSERT_MESSAGE("Invalid number of active transfers (VO1)", 
            (count <= (T2_MAX_TRANSFERS/2) && count >= (T2_MAX_TRANSFERS/2 - T2_NUM_ACTIVE_TRANSFERS_VO_4)));
        count = ActionsConfig::instance().activeTxCache().count(VO2_NAME);
        _log_debug(CAT,"Got " << count << " active transfers for VO2");
        CPPUNIT_ASSERT_MESSAGE("Invalid number of active transfers (VO2)", 
            (count <= (T2_MAX_TRANSFERS/2) && count >= (T2_MAX_TRANSFERS/2 - T2_NUM_ACTIVE_TRANSFERS_VO_4)));
        
    } catch(const std::exception& exc){
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        CPPUNIT_FAIL("Got Unknown Error");
    }
}
