/*
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *  GLite Data Transfer Agents: 13301 Unit Test Case
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *  Version info: $Id: TestCase13724.cpp,v 1.5 2008/11/13 10:43:56 ptedesco Exp $
 *  Release: $Name:  $
 *
 */
#include "Fetch.h"
#include "ChannelFlags.h"
#include "actions/channel/ActionsConfig.h"
#include "glite/data/agents/dao/DAOContext.h"
#include "glite/data/agents/dao/DAOContextFactory.h"
#include "glite/data/transfer/agent/action/ActionFactory.h"
#include "glite/data/transfer/agent/model/File.h"
#include "glite/data/transfer/agent/model/Job.h"
#include "glite/data/transfer/agent/model/Transfer.h"
#include "glite/data/transfer/agent/model/Channel.h"
#include "glite/data/transfer/agent/dao/channel/FileDAO.h"
#include "glite/data/transfer/agent/dao/channel/JobDAO.h"
#include "glite/data/transfer/agent/dao/channel/TransferDAO.h"
#include "glite/data/transfer/agent/dao/channel/DAOFactory.h"
#include "glite/data/transfer/agent/ts/TransferService.h"
#include "glite/data/transfer/agent/ts/TransferServiceFactory.h"

#include <memory>
#include <boost/scoped_ptr.hpp>
#include <cppunit/TestCase.h>
#include <cppunit/extensions/HelperMacros.h>

using namespace glite::data::transfer::agent;
using namespace glite::data::transfer::agent::action;
using namespace glite::data::transfer::agent::action::channel;
using namespace glite::data::transfer::agent::model;
using namespace glite::data::transfer::agent::dao::channel;
using namespace glite::data::transfer::agent::ts;

using boost::scoped_ptr;

/**
 * CPPUnit TestCase
 */
class TestCase13724 : public CppUnit::TestCase{
public:
    /**
     * set-up test case
     */
    void setUp ();

    /**
     * tear-down test case
     */
    void tearDown ();
private:
    // Declare CPPUnit Methods
    // -------------------------------------------------------------------------
    CPPUNIT_TEST_SUITE( TestCase13724 );
    CPPUNIT_TEST( testExecute1 );
    CPPUNIT_TEST( testExecute2 );
    CPPUNIT_TEST_SUITE_END();

    // Test Methods
    // -------------------------------------------------------------------------

    /**
     * Test Execute Method
     */
    void testExecute1();

    /**
     * Test Execute Method
     */
    void testExecute2();

private:
    /**
     * The Action Instance
     */
    scoped_ptr<Fetch> m_action;
};

/*
 * Register Suit
 */
CPPUNIT_TEST_SUITE_NAMED_REGISTRATION( TestCase13724, "TestCase13724" );

/*
 * Declare Constants. This unit test foresee at beginning
 */
namespace {
const char * const T1_CHANNEL_NAME  = "channel13724_1";
const char * const T2_CHANNEL_NAME  = "channel13724_2";
const char * const VO1_NAME         = "VO13724_1";
const char * const T1_JOB1_ID       = "10000000-0000-0000-0000-000000000001";
const char * const T2_JOB1_ID       = "20000000-0000-0000-0000-000000000001";
const char * const T2_JOB2_ID       = "20000000-0000-0000-0000-000000000002";
const char * const T1_FILE1_ID      = "10000001";
const char * const T1_FILE2_ID      = "10000002";
const char * const T2_FILE1_ID      = "20000001";
const char * const T2_FILE2_ID      = "20000002";    
const char * const T2_FILE3_ID      = "20000003";
const char * const T2_FILE4_ID      = "20000004";    
}

/*
 * setUp
 *
 * set-up test case
 */
void TestCase13724::setUp(){
    
    try{
        ActionsConfig::instance().activeTxCache().reset();
        
        // Create Action instance
        std::auto_ptr<Action> a(ActionFactory::instance().create("glite:Fetch"));
        CPPUNIT_ASSERT_MESSAGE("Null Action", a.get() != 0);
        CPPUNIT_ASSERT_MESSAGE("Invalid Action Class", dynamic_cast<Fetch*>(a.get()) != 0);

        // Store Action        
        m_action.reset(dynamic_cast<Fetch*>(a.release()));
        CPPUNIT_ASSERT_MESSAGE("Null Action", m_action.get() != 0);
    } catch(const std::exception& exc){
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        CPPUNIT_FAIL("Got Unknown Error");
    }
}

/*
 * tearDown
 *
 * tear-down test case
 */
void TestCase13724::tearDown(){
    ActionsConfig::instance().activeTxCache().reset();
}
/*
 *
 * Test Execute Method
 */
void TestCase13724::testExecute1(){
    
    try{
        // Get Context per Channel Operations
        scoped_ptr<glite::data::agents::dao::DAOContext> chCtx(
            glite::data::agents::dao::DAOContextFactory::instance().create());
        
        // Set Context
        m_action->context(T1_CHANNEL_NAME,*chCtx);

        // Create the Job DAO Instance
        scoped_ptr<JobDAO> jobDAO(
            DAOFactory::instance().createJobDAO(T1_CHANNEL_NAME, *chCtx));

        // Create the File DAO Instance
        scoped_ptr<FileDAO> fileDAO(
            DAOFactory::instance().createFileDAO(T1_CHANNEL_NAME, *chCtx));
        
        // Create the Transfer DAO Instance
        scoped_ptr<TransferDAO> txDAO(
            DAOFactory::instance().createTransferDAO(T1_CHANNEL_NAME, *chCtx));

        // Check Preconditions
        // ---------------------------------------------------------------------
        
        // Check Job's State
        scoped_ptr<Job> job(jobDAO->get(T1_JOB1_ID));
        CPPUNIT_ASSERT_MESSAGE("Precondition Failed: Invalid Job State (1)", job->state == Job::S_READY);        
        // Check Files' State
        scoped_ptr<File> file(fileDAO->get(T1_FILE1_ID));
        CPPUNIT_ASSERT_MESSAGE("Precondition Failed: Invalid File State (1)", file->state == File::S_READY);
        file.reset(fileDAO->get(T1_FILE2_ID));
        CPPUNIT_ASSERT_MESSAGE("Precondition Failed: Invalid File State (2)", file->state == File::S_READY);
                
        // Execute Actions
        try{
            m_action->execute(CHANNEL_VO_SHARE_TYPE_NORMALIZED_ON_ACTIVE);
        } catch(const ExecuteException& exc){
        }
        // Check now the Files' States
        file.reset(fileDAO->get(T1_FILE1_ID));
        CPPUNIT_ASSERT_MESSAGE("Invalid File State (1)", file->state == File::S_WAITING);
        file.reset(fileDAO->get(T1_FILE2_ID));
        CPPUNIT_ASSERT_MESSAGE("Invalid File State (2)", file->state == File::S_WAITING);
        // Check now the Job's State
        job.reset(jobDAO->get(T1_JOB1_ID));
        CPPUNIT_ASSERT_MESSAGE("Invalid Job State (1)", job->state == Job::S_ACTIVE);
    } catch(const std::exception& exc){
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        CPPUNIT_FAIL("Got Unknown Error");
    }
}

/*
 *
 * Test Execute Method
 */
void TestCase13724::testExecute2(){
    
    try{
        // Get Context per Channel Operations
        scoped_ptr<glite::data::agents::dao::DAOContext> chCtx(
            glite::data::agents::dao::DAOContextFactory::instance().create());
        
        // Set Context
        m_action->context(T2_CHANNEL_NAME,*chCtx);

        // Create the Job DAO Instance
        scoped_ptr<JobDAO> jobDAO(
            DAOFactory::instance().createJobDAO(T2_CHANNEL_NAME, *chCtx));

        // Create the File DAO Instance
        scoped_ptr<FileDAO> fileDAO(
            DAOFactory::instance().createFileDAO(T2_CHANNEL_NAME, *chCtx));
        
        // Create the Transfer DAO Instance
        scoped_ptr<TransferDAO> txDAO(
            DAOFactory::instance().createTransferDAO(T2_CHANNEL_NAME, *chCtx));
        
        // Check Preconditions
        // ---------------------------------------------------------------------
        
        // Check Job's State
        scoped_ptr<Job> job(jobDAO->get(T2_JOB1_ID));
        CPPUNIT_ASSERT_MESSAGE("Precondition Failed: Invalid Job State (1)", job->state == Job::S_READY);        
        job.reset(jobDAO->get(T2_JOB2_ID));
        CPPUNIT_ASSERT_MESSAGE("Precondition Failed: Invalid Job State (2)", job->state == Job::S_READY);
        // Check Files' State
        scoped_ptr<File> file(fileDAO->get(T2_FILE1_ID));
        CPPUNIT_ASSERT_MESSAGE("Precondition Failed: Invalid File State (1)", file->state == File::S_READY);
        file.reset(fileDAO->get(T2_FILE2_ID));
        CPPUNIT_ASSERT_MESSAGE("Precondition Failed: Invalid File State (2)", file->state == File::S_READY);
        file.reset(fileDAO->get(T2_FILE3_ID));
        CPPUNIT_ASSERT_MESSAGE("Precondition Failed: Invalid File State (3)", file->state == File::S_READY);
        file.reset(fileDAO->get(T2_FILE4_ID));
        CPPUNIT_ASSERT_MESSAGE("Precondition Failed: Invalid File State (4)", file->state == File::S_READY);
                
        // Execute Actions
        try{
            m_action->execute(CHANNEL_VO_SHARE_TYPE_NORMALIZED_ON_ACTIVE);
        } catch(const ExecuteException& exc){
        }
        // Check now the Files' States
        file.reset(fileDAO->get(T2_FILE1_ID));
        CPPUNIT_ASSERT_MESSAGE("Invalid File State (1)", file->state == File::S_WAITING);
        file.reset(fileDAO->get(T2_FILE2_ID));
        CPPUNIT_ASSERT_MESSAGE("Invalid File State (2)", file->state == File::S_WAITING);
        file.reset(fileDAO->get(T2_FILE3_ID));
        CPPUNIT_ASSERT_MESSAGE("Invalid File State (3)", file->state == File::S_ACTIVE);
        file.reset(fileDAO->get(T2_FILE4_ID));
        CPPUNIT_ASSERT_MESSAGE("Invalid File State (4)", file->state == File::S_ACTIVE);
        // Check now the Job's State
        job.reset(jobDAO->get(T2_JOB1_ID));
        CPPUNIT_ASSERT_MESSAGE("Invalid Job State (1)", job->state == Job::S_ACTIVE);
        job.reset(jobDAO->get(T2_JOB2_ID));
        CPPUNIT_ASSERT_MESSAGE("Invalid Job State (2)", job->state == Job::S_ACTIVE);
    } catch(const std::exception& exc){
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        CPPUNIT_FAIL("Got Unknown Error");
    }
}
