/*
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *  GLite Data Transfer Agents: FetchTransfers Test Case
 *
 *  Authors: Paolo Tedesco <paolo.tedesco@cern.ch>
 *  Version info: $Id: TestCase31161.cpp,v 1.2 2008/01/08 09:35:27 ptedesco Exp $
 *  Release: $Name:  $
 *
 */
#include "CheckState.h"
#include "ChannelFlags.h"
#include "DaoHelper.h"
#include "actions/channel/ActionsConfig.h"
#include "glite/data/transfer/agent/action/ActionFactory.h"
#include "glite/data/transfer/agent/model/File.h"
#include "glite/data/transfer/agent/model/Job.h"
#include "glite/data/transfer/agent/model/Transfer.h"
#include "glite/data/transfer/agent/model/Channel.h"
#include "glite/data/transfer/agent/dao/channel/FileDAO.h"
#include "glite/data/transfer/agent/dao/channel/JobDAO.h"
#include "glite/data/transfer/agent/dao/channel/TransferDAO.h"
#include "glite/data/transfer/agent/dao/channel/DAOFactory.h"
#include "glite/data/transfer/agent/ts/TransferService.h"
#include "glite/data/transfer/agent/ts/TransferServiceFactory.h"

#include "dao_test_utils.h"

#include <memory>
#include <boost/scoped_ptr.hpp>
#include <boost/shared_ptr.hpp>
#include <cppunit/TestCase.h>
#include <cppunit/extensions/HelperMacros.h>

using namespace glite::data::transfer::agent;
using namespace glite::data::transfer::agent::action;
using namespace glite::data::transfer::agent::action::channel;
using namespace glite::data::transfer::agent::model;

using boost::scoped_ptr;
using boost::shared_ptr;

/**
 * CPPUnit TestCase
 */
class TestCase31161 : public CppUnit::TestCase{

private:
    // Declare CPPUnit Methods
    // -------------------------------------------------------------------------
    CPPUNIT_TEST_SUITE( TestCase31161 );
        CPPUNIT_TEST( test_bug31161 );
    CPPUNIT_TEST_SUITE_END();

    // Test Methods
    // -------------------------------------------------------------------------
    void test_bug31161();
};

/*
 * Register Suit
 */
CPPUNIT_TEST_SUITE_NAMED_REGISTRATION( TestCase31161, "TestCase31161" );

namespace {
const char * const CHANNEL_NAME     = "channel_31161";
const char * const VO1_NAME          = "VO_1";
const char * const VO2_NAME          = "VO_2";
}

/**
 * test_bug31161
 */
void TestCase31161::test_bug31161(){
    scoped_ptr<CheckState> m_action;
    shared_ptr<Job>         job;
    shared_ptr<File>        file;   
    shared_ptr<Transfer>    transfer;
    try{
        // Reset cache
        ActionsConfig::instance().activeTxCache().reset();
        // Create Action instance
        std::auto_ptr<Action> a(ActionFactory::instance().create("glite:CheckState"));
        CPPUNIT_ASSERT_MESSAGE("Null Action", a.get() != 0);
        CPPUNIT_ASSERT_MESSAGE("Invalid Action Class", dynamic_cast<CheckState*>(a.get()) != 0);
        // Store Action        
        m_action.reset(dynamic_cast<CheckState*>(a.release()));
        CPPUNIT_ASSERT_MESSAGE("Null Action", m_action.get() != 0);
        // Set Context
        m_action->context(CHANNEL_NAME,DaoHelper::ctx());
        // execute
        m_action->execute();
        
        static const char * channel_name    = "channel_31161";
        // static const char * job_id          = "10000000-0000-0000-0000-000000000401";
        static const char * file_id         = "10000401";
        static const char * tx_1_id         = "10401";
        static const char * tx_2_id         = "10402";

        // Create FileDAO instance
        scoped_ptr<dao::channel::FileDAO> fileDAO(dao::channel::DAOFactory::instance().createFileDAO(channel_name, DaoHelper::ctx()));
        CPPUNIT_ASSERT_MESSAGE("Null FileDAO", fileDAO.get() != 0);
        // Create JobDAO instance
        scoped_ptr<dao::channel::JobDAO> jobDAO(dao::channel::DAOFactory::instance().createJobDAO(channel_name, DaoHelper::ctx()));
        CPPUNIT_ASSERT_MESSAGE("Null JobDAO", jobDAO.get() != 0);
        // Create TransferDAO instance
        scoped_ptr<dao::channel::TransferDAO> transferDAO(dao::channel::DAOFactory::instance().createTransferDAO(channel_name, DaoHelper::ctx()));
        CPPUNIT_ASSERT_MESSAGE("Null TransferDAO", transferDAO.get() != 0);
        
        transfer = check_transfer(transferDAO,file_id,tx_1_id,tx_1_id,Transfer::S_ABORTED);
        check_error(transfer,tx_1_id,Error::CONSISTENCY,Error::S_AGENT,Error::P_TX_SERVICE);

        transfer = check_transfer(transferDAO,file_id,tx_2_id,tx_2_id,Transfer::S_ABORTED);
        check_error(transfer,tx_2_id,Error::CONSISTENCY,Error::S_AGENT,Error::P_TX_SERVICE);
        
        file = check_file(fileDAO,file_id,file_id,File::S_FAILED);
        check_error(file,file_id,Error::CONSISTENCY,Error::S_AGENT,Error::P_TX_SERVICE);

        // reset cache and action
        ActionsConfig::instance().activeTxCache().reset();
        m_action.reset();
    } catch(const std::exception& exc){
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        CPPUNIT_FAIL("Got Unknown Error");
    }
}
