/*
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *  GLite Data Transfer Agents: FetchTransfers Test Case
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *  Version info: $Id: ActionSchedulerTestCase.cpp,v 1.10 2007/02/14 15:57:46 badino Exp $
 *  Release: $Name:  $
 *
 */
#include "FakeActionConfig.h"
#include "FakeAction.h"
#include "DummyAction.h"

#include "glite/data/transfer/agent/scheduler/ActionScheduler.h"
#include "glite/data/transfer/agent/scheduler/SchedulerEntry.h"
#include "glite/data/agents/dao/DAOContext.h"
#include "glite/data/agents/dao/DAOContextFactory.h"

#include <boost/scoped_ptr.hpp>
#include <cppunit/TestCase.h>
#include <cppunit/extensions/HelperMacros.h>
#include <unistd.h>

using namespace glite::data::transfer::agent;
using namespace glite::data::transfer::agent::action;
using namespace glite::data::agents;

using boost::scoped_ptr;

/**
 * CPPUnit TestCase
 */
class ActionSchedulerTestCase : public CppUnit::TestCase{
public:
    /**
     * set-up test case
     */
    void setUp ();
    
    /**
     * tearDown test case
     */
    void tearDown ();

    static void finalize() ;
    
private:
    // Declare CPPUnit Methods
    // -------------------------------------------------------------------------
    CPPUNIT_TEST_SUITE( ActionSchedulerTestCase);
    CPPUNIT_TEST( testFakeAction );
    CPPUNIT_TEST( testFakeAction_Disabled );
    CPPUNIT_TEST( testFakeAction_Reenabled );
    CPPUNIT_TEST( testStop );
    CPPUNIT_TEST_SUITE_END();

    // Test Methods
    // -------------------------------------------------------------------------

    /**
     * Test Fake Action Method
     */
    void testFakeAction();

    /**
     * Test Disabled Action Method
     */
    void testFakeAction_Disabled();

    /**
     * Test Reenabled Action Method
     */
    void testFakeAction_Reenabled();

    /**
     * Test Stop  Method
     */
    void testStop();
    
private:
    
    /**
     * Action Scheduler
     */
    static scoped_ptr<ActionScheduler> m_scheduler;
        
    /**
     * The DAO Context
     */
    static scoped_ptr<dao::DAOContext> m_daoCtx;

    /**
     * count the number of test executed
     */
    static int m_count;
};

/*
 * Register Suit
 */
CPPUNIT_TEST_SUITE_NAMED_REGISTRATION( ActionSchedulerTestCase, "ActionSchedulerTestCase" );

namespace {
const unsigned int DUMMY_INTERVAL = 1;
const unsigned int FAKE_INTERVAL  = 2;
}

scoped_ptr<ActionScheduler> ActionSchedulerTestCase::m_scheduler;
scoped_ptr<dao::DAOContext> ActionSchedulerTestCase::m_daoCtx;
int                         ActionSchedulerTestCase::m_count = 0;

/*
 * finalize
 *
 * test case finalization
 */
void ActionSchedulerTestCase::finalize() {
   m_scheduler.reset();
   m_daoCtx.reset();
}
void ActionSchedulerTestCase_finalize() {
    ActionSchedulerTestCase::finalize();
}

/*
 * setUp
 *
 * set-up test case
 */
void ActionSchedulerTestCase::setUp(){
    
    try{
        // Create Action Scheduler
        if(0 == m_scheduler.get()){
            // Create DAO Context. Use the VO
            if(0 == m_daoCtx.get()){
                m_daoCtx.reset(dao::DAOContextFactory::instance().create());
            }
    
            m_scheduler.reset(new ActionScheduler("ActionSchedulerTestCase",*m_daoCtx));
    
            // Register Scheduler Entries
            m_scheduler->add(new SchedulerEntry(*DummyAction::factory(), "Dummy", DUMMY_INTERVAL));
            m_scheduler->add(new SchedulerEntry(*FakeAction::factory() , "Fake" , FAKE_INTERVAL));
            
            // Start the Scheduler
            bool res = m_scheduler->start();
            CPPUNIT_ASSERT_MESSAGE("Scheduler Not Started", res == true);
           
            m_count = 3;
        }
    } catch(const dao::DAOException& exc){
        CPPUNIT_FAIL(exc.what());
    } catch(const std::exception& exc){
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        CPPUNIT_FAIL("Got Unknown Error");
    }
}

/*
 * tearDown
 *
 * tearDown test case
 */
void ActionSchedulerTestCase::tearDown(){
    
    try{
        --m_count;
        if(0 == m_count){
           // Stop the Scheduler
           m_scheduler->stop();
        }
    } catch(const dao::DAOException& exc){
        CPPUNIT_FAIL(exc.what());
    } catch(const std::exception& exc){
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        CPPUNIT_FAIL("Got Unknown Error");
    } 
}


/*
 * testFakeAction
 *
 * Test FakeAction
 */
void ActionSchedulerTestCase::testFakeAction(){
    
    try{
        // Sleep for a while
        sleep(5);
        
        // Check that the FakeAction count is > 0
        CPPUNIT_ASSERT_MESSAGE("Action Not Started", FakeActionConfig::instance()->count() > 0);
    } catch(const std::exception& exc){
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        CPPUNIT_FAIL("Got Unknown Error");
    }
}


/*
 * testFakeAction_Disabled
 *
 * Test FakeAction
 */
void ActionSchedulerTestCase::testFakeAction_Disabled(){
    
    try{
        // Sleep for a while
        sleep(10);
        
        // Check that the FakeAction is disabled
        CPPUNIT_ASSERT_MESSAGE("Action Is Still Enabled", FakeActionConfig::instance()->isDisabled() == true);
    } catch(const std::exception& exc){
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        CPPUNIT_FAIL("Got Unknown Error");
    }
}

/*
 * testFakeAction_Reenabled
 *
 * Test FakeAction
 */
void ActionSchedulerTestCase::testFakeAction_Reenabled(){
    
    try{
        // Sleep for a while
        sleep(10);
        
        // Check that the FakeAction is enabled
        CPPUNIT_ASSERT_MESSAGE("Action Is Still Disabled", FakeActionConfig::instance()->isDisabled() == true);
    } catch(const std::exception& exc){
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        CPPUNIT_FAIL("Got Unknown Error");
    }
}


/*
 * testStop
 *
 * Test FakeAction
 */
void ActionSchedulerTestCase::testStop(){
    
    try{
        ActionScheduler sched("ActionSchedulerTestCase2",*m_daoCtx);
    
        // Register Scheduler Entries
        sched.add(new SchedulerEntry(*SleepAction::factory(), "Sleep", 1));
            
        // Start the Scheduler
        bool res = sched.start();
        CPPUNIT_ASSERT_MESSAGE("Scheduler Not Started", res == true);

        // Sleep for a while
        sleep(3);

        // stop the Scheduler
        sched.stop();

        // Sleep for a while
        sleep(3);
    } catch(const std::exception& exc){
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        CPPUNIT_FAIL("Got Unknown Error");
    }
}
