/**
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *  GLite Data Transfer Agents: TransferService Test Case
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *           Paolo Tedesco <paolo.tedesco@cern.ch>
 *  Version info: $Id: TransferServiceTestCase.cpp,v 1.15 2009/12/11 11:27:04 molnarzs Exp $
 *  Release: $Name:  $
 *
 *  NOTE: this is a test of the STUB IMPLEMENTATION of the transfer service interface
 */
#include "TransferServiceTestCase.h"
#include "glite/data/transfer/agent/model/Transfer.h"
#include "glite/data/transfer/agent/ts/TransferServiceFactory.h"
#include "glite/data/agents/AgentExceptions.h"

#include <dao_test_utils.h>

using namespace glite::data::transfer::agent;
using namespace glite::data::transfer::agent::model;

TransferServiceTestCase::ReqIdArray TransferServiceTestCase::s_requests;
TransferServiceTestCase::TxIdArray TransferServiceTestCase::s_transfers;

namespace {
const char * const TC_SRC_NAME                    = "file:///tmp/src/this-file-should-succeed.txt";
const char * const TC_SRC_FAIL_SUBMIT_NAME        = "file:///tmp/src/this-file-should-fail-during-submission.txt";
const char * const TC_SRC_FAIL_AFTER_SUBMIT_NAME  = "file:///tmp/src/this-file-should-fail-after-submit.txt";
const char * const TC_SRC_FAIL_IN_PROCESSING_NAME = "file:///tmp/src/this-file-should-fail-in-processing.txt";
const char * const TC_DEST_NAME                   = "file:///tmp/dstfile.txt";
const char * const TC_SRC_NAME_CHECKSUM_OK        = "file:///tmp/src/this-file-should-succeed-with-checksum.txt";
}
/*
 * Register Suit
 */
CPPUNIT_TEST_SUITE_NAMED_REGISTRATION( TransferServiceTestCase, "TransferServiceTestCase" );

/*
 * setUp
 *
 * set-up test case
 */
void TransferServiceTestCase::setUp(){

    // Create TransferService instance
    m_transferService.reset(TransferServiceFactory::instance().create("stub"));
    CPPUNIT_ASSERT_MESSAGE("Null Transfer Service", m_transferService.get() != 0);

}

/*
 * Test Submit Method (single)
 */
void TransferServiceTestCase::testSubmit_single(){

    {// Req 1: OK
    // Test Submit
    TransferService::TransferRequest  req;
    TransferService::TransferResponse out;
    req.surls.resize(1);
    req.surls[0].source = TC_SRC_NAME;
    req.surls[0].destination = TC_DEST_NAME;
    // Submit Transfer
    m_transferService->submit(req,out,"");
    CPPUNIT_ASSERT_MESSAGE("Empty RequestID (1)", false == out.request_id.empty());
    CPPUNIT_ASSERT_MESSAGE("Empty TransferId (1)", false == out.transfer_ids.empty());
    CPPUNIT_ASSERT_MESSAGE("Invalid number of TransferId (1)", out.transfer_ids.size() == 1);

    s_requests.push_back(out.request_id);
    s_transfers.push_back(TxId(out.request_id,out.transfer_ids.at(0)));
    }
    {// Req 2: Fail After Submit
    // Test Submit
    TransferService::TransferRequest  req;
    TransferService::TransferResponse out;
    req.surls.resize(1);
    req.surls[0].source  = TC_SRC_FAIL_AFTER_SUBMIT_NAME;
    req.surls[0].destination = TC_DEST_NAME;
    // Submit Transfer
    m_transferService->submit(req,out,"");
    CPPUNIT_ASSERT_MESSAGE("Empty RequestID (2)", false == out.request_id.empty());
    CPPUNIT_ASSERT_MESSAGE("Empty TransferId (2)", false == out.transfer_ids.empty());
    CPPUNIT_ASSERT_MESSAGE("Invalid number of TransferId (2)", out.transfer_ids.size() == 1);

    s_requests.push_back(out.request_id);
    s_transfers.push_back(TxId(out.request_id,out.transfer_ids.at(0)));
    }
    {// Req 3: Fail In Processing
    // Test Submit
    TransferService::TransferRequest  req;
    TransferService::TransferResponse out;
    req.surls.resize(1);
    req.surls[0].source  = TC_SRC_FAIL_IN_PROCESSING_NAME;
    req.surls[0].destination = TC_DEST_NAME;
    // Submit Transfer
    m_transferService->submit(req,out,"");
    CPPUNIT_ASSERT_MESSAGE("Empty RequestID (3)", false == out.request_id.empty());
    CPPUNIT_ASSERT_MESSAGE("Empty TransferId (3)", false == out.transfer_ids.empty());
    CPPUNIT_ASSERT_MESSAGE("Invalid number of TransferId (3)", out.transfer_ids.size() == 1);

    s_requests.push_back(out.request_id);
    s_transfers.push_back(TxId(out.request_id,out.transfer_ids.at(0)));
    }

}

/*
 * Test Submit Method (single)
 */
void TransferServiceTestCase::testSubmit_single_Error(){

    try{
        // Test Submit
        TransferService::TransferRequest  req;
        TransferService::TransferResponse out;
        req.surls.resize(1);
        req.surls[0].source  = TC_SRC_FAIL_SUBMIT_NAME;
        req.surls[0].destination = TC_DEST_NAME;
        // Submit Transfer
        m_transferService->submit(req,out,"");
        CPPUNIT_FAIL("This file shouldn't be submittable");
    } catch(const TransferException& exc){
        // OK
    }
}

/*
 * Test Submit Method (bulk)
 */
void TransferServiceTestCase::testSubmit_bulk(){

    // Test Submit
    TransferService::TransferRequest  req;
    TransferService::TransferResponse out;
    req.surls.resize(3);
    req.surls[0].source  = TC_SRC_NAME;
    req.surls[0].destination = TC_DEST_NAME;
    req.surls[1].source  = TC_SRC_NAME;
    req.surls[1].destination = TC_DEST_NAME;
    req.surls[2].source  = TC_SRC_NAME;
    req.surls[2].destination = TC_DEST_NAME;
    // Submit Transfer
    m_transferService->submit(req,out,"");
    CPPUNIT_ASSERT_MESSAGE("Empty RequestID", false == out.request_id.empty());
    CPPUNIT_ASSERT_MESSAGE("Empty TransferId", false == out.transfer_ids.empty());
    CPPUNIT_ASSERT_MESSAGE("Invalid number of TransferId", out.transfer_ids.size() == 3);

    s_requests.push_back(out.request_id);
    s_transfers.push_back(TxId(out.request_id,out.transfer_ids.at(0)));
    s_transfers.push_back(TxId(out.request_id,out.transfer_ids.at(1)));
    s_transfers.push_back(TxId(out.request_id,out.transfer_ids.at(2)));

}

/*
 * Test Submit Method (bulk)
 */
void TransferServiceTestCase::testSubmit_bulk_Error(){

    try{
        // Test Submit
        TransferService::TransferRequest  req;
        TransferService::TransferResponse out;
        req.surls.resize(3);
        req.surls[0].source  = TC_SRC_NAME;
        req.surls[0].destination = TC_DEST_NAME;
        req.surls[1].source  = TC_SRC_FAIL_SUBMIT_NAME;
        req.surls[1].destination = TC_DEST_NAME;
        req.surls[2].source  = TC_SRC_NAME;
        req.surls[2].destination = TC_DEST_NAME;
        // Submit Transfer
        m_transferService->submit(req,out,"");
        CPPUNIT_FAIL("This file shouldn't be submittable");
    } catch(const TransferException& exc){
        // OK
    }
}

/**
 * Test State Method
 */
void TransferServiceTestCase::testState_single(){

    CPPUNIT_ASSERT_MESSAGE("Invalid Number of requests", s_requests.size() >= 3 );
    CPPUNIT_ASSERT_MESSAGE("Invalid Number of transfers", s_transfers.size() >= 3 );

    // Get Transfer State
    { // Req 1
    TransferService::TransferInfoArray array;
    Transfer t(s_transfers.at(0).first,"file1",s_transfers.at(0).second,"Job1",Transfer::S_RECEIVED);
    array.push_back(&t);
    m_log_debug("Querying status for request <" << s_requests.at(0) << ">");
    m_transferService->state(s_requests.at(0),array,"");
    CPPUNIT_ASSERT_MESSAGE("Invalid State (1-1)", t.state == Transfer::S_PROCESSING);
    m_log_debug("Querying status for request <" << s_requests.at(0) << ">");
    m_transferService->state(s_requests.at(0),array,"");
    CPPUNIT_ASSERT_MESSAGE("Invalid State (1-2)", t.state == Transfer::S_COMPLETED);
    try{
        m_log_debug("Querying status for request <" << s_requests.at(0) << ">");
        m_transferService->state(s_requests.at(0),array,"");
        CPPUNIT_FAIL("Request Id should have been removed (1)");
    } catch(const TransferException& exc){
        // OK
        m_log_debug("request <" << s_requests.at(0) << "> correctly removed");
    }
    }
    { // Req 2
    TransferService::TransferInfoArray array;
    Transfer t(s_transfers.at(1).first,"file2",s_transfers.at(1).second,"Job2",Transfer::S_RECEIVED);
    array.push_back(&t);
    m_log_debug("Querying status for request <" << s_requests.at(1) << ">");
    m_transferService->state(s_requests.at(1),array,"");
    CPPUNIT_ASSERT_MESSAGE("Invalid State (2-1)", t.state == Transfer::S_FAILED);
    try{
        m_transferService->state(s_requests.at(1),array,"");
        CPPUNIT_FAIL("Request Id should have been removed (2)");
    } catch(const TransferException& exc){
        // OK
        m_log_debug("request <" << s_requests.at(1) << "> correctly removed");
    }
    }
    { // Req 3
    TransferService::TransferInfoArray array;
    Transfer t(s_transfers.at(2).first,"file3",s_transfers.at(2).second,"Job3",Transfer::S_RECEIVED);
    array.push_back(&t);
    m_log_debug("Querying status for request <" << s_requests.at(2) << ">");
    m_transferService->state(s_requests.at(2),array,"");
    CPPUNIT_ASSERT_MESSAGE("Invalid State (3-1)", t.state == Transfer::S_PROCESSING);
    m_log_debug("Querying status for request <" << s_requests.at(2) << ">");
    m_transferService->state(s_requests.at(2),array,"");
    CPPUNIT_ASSERT_MESSAGE("Invalid State (3-2)", t.state == Transfer::S_FAILED);
    try{
        m_transferService->state(s_requests.at(2),array,"");
        CPPUNIT_FAIL("Request Id should have been removed (3)");
    } catch(const TransferException& exc){
        // OK
        m_log_debug("request <" << s_requests.at(2) << "> correctly removed");
    }
    }

}

/**
 * Test State Method
 */
void TransferServiceTestCase::testState_single_Error(){

    // Get Transfer State
    { // Req 1
    TransferService::TransferInfoArray array;
    Transfer t(s_transfers.at(0).first,"file1",111111,"Job1",Transfer::S_RECEIVED);
    array.push_back(&t);
    try{
        m_transferService->state(s_requests.at(1),array,"");
        CPPUNIT_FAIL("Request Id doesn't match transfer id(1)");
    } catch(const glite::data::agents::LogicError& exc){
        // OK
    }
    }
    { // Req 2
    TransferService::TransferInfoArray array;
    Transfer t("Invalid_Request_ID","file2",0,"Job2",Transfer::S_RECEIVED);
    array.push_back(&t);
    try{
        m_transferService->state("Invalid_Request_ID",array,"");
        CPPUNIT_FAIL("Invalid Request Id (2)");
    } catch(const TransferException& exc){
        // OK
    }
    }

}

/**
 * Test State Method
 */
void TransferServiceTestCase::testState_bulk(){

    try{
        CPPUNIT_ASSERT_MESSAGE("Invalid Number of requests", s_requests.size() >= 4 );
        CPPUNIT_ASSERT_MESSAGE("Invalid Number of transfers", s_transfers.size() >= 6 );

        // Get Transfer State
        { // Req 1
        TransferService::TransferInfoArray array;
        Transfer t1(s_transfers.at(3).first,"file1",s_transfers.at(3).second,"Job1",Transfer::S_RECEIVED);
        Transfer t2(s_transfers.at(4).first,"file2",s_transfers.at(4).second,"Job1",Transfer::S_RECEIVED);
        Transfer t3(s_transfers.at(5).first,"file3",s_transfers.at(5).second,"Job1",Transfer::S_RECEIVED);
        array.push_back(&t1);
        array.push_back(&t2);
        array.push_back(&t3);
        m_log_debug("Querying status for request <" << s_requests.at(3) << ">");
        m_transferService->state(s_requests.at(3),array,"");
        CPPUNIT_ASSERT_MESSAGE("Invalid State (4:1-1)", t1.state == Transfer::S_PROCESSING);
        CPPUNIT_ASSERT_MESSAGE("Invalid State (4:2-1)", t2.state == Transfer::S_PROCESSING);
        CPPUNIT_ASSERT_MESSAGE("Invalid State (4:3-1)", t3.state == Transfer::S_PROCESSING);
        m_log_debug("Querying status for request <" << s_requests.at(3) << ">");
        m_transferService->state(s_requests.at(3),array,"");
        CPPUNIT_ASSERT_MESSAGE("Invalid State (4:1-2)", t1.state == Transfer::S_COMPLETED);
        CPPUNIT_ASSERT_MESSAGE("Invalid State (4:2-2)", t2.state == Transfer::S_COMPLETED);
        CPPUNIT_ASSERT_MESSAGE("Invalid State (4:3-2)", t3.state == Transfer::S_COMPLETED);
        try{
            m_log_debug("Querying status for request <" << s_requests.at(3) << ">");
            m_transferService->state(s_requests.at(3),array,"");
            CPPUNIT_FAIL("Request Id should have been removed (4)");
        } catch(const TransferException& exc){
            // OK
            m_log_debug("request <" << s_requests.at(3) << "> correctly removed");
        }
        }
    } catch(const std::exception& exc){
        CPPUNIT_FAIL(exc.what());
    } catch(...){
        CPPUNIT_FAIL("Got Unknown Error");
    }
}

/**
 * Test State Method
 */
void TransferServiceTestCase::testState_bulk_Error(){


    CPPUNIT_ASSERT_MESSAGE("Invalid Number of requests", s_requests.size() >= 4 );
    CPPUNIT_ASSERT_MESSAGE("Invalid Number of transfers", s_transfers.size() >= 6 );

    // Get Transfer State
    { // Req 4
    m_log_debug("testState_bulk_Error:1");
    TransferService::TransferInfoArray array;
    Transfer t1(s_transfers.at(3).first,"file1",111111,"Job1",Transfer::S_RECEIVED);
    Transfer t2(s_transfers.at(4).first,"file1",222222,"Job1",Transfer::S_RECEIVED);
    Transfer t3(s_transfers.at(5).first,"file1",333333,"Job1",Transfer::S_RECEIVED);
    array.push_back(&t1);
    array.push_back(&t2);
    array.push_back(&t3);
    try{
        m_log_debug("testState_bulk_Error:1 - before state");
        m_transferService->state(s_requests.at(2),array,"");
        CPPUNIT_FAIL("Request Id doesn't match transfer id(1)");
    } catch(const glite::data::agents::LogicError& exc){
        // OK
        m_log_debug("testState_bulk_Error:1 => OK");
    }
    }
    { // Req 4
    m_log_debug("testState_bulk_Error:2");
    TransferService::TransferInfoArray array;
    Transfer t1(s_transfers.at(3).first,"file1",s_transfers.at(3).second,"Job1",Transfer::S_RECEIVED);
    Transfer t2(s_transfers.at(4).first,"file2",s_transfers.at(4).second,"Job1",Transfer::S_RECEIVED);
    array.push_back(&t1);
    array.push_back(&t2);
    try{
        m_log_debug("testState_bulk_Error:2 - before state");
        m_transferService->state(s_requests.at(3),array,"");
        CPPUNIT_FAIL("Invalid Request size (2)");
    } catch(const glite::data::agents::LogicError& exc){
        // OK
        m_log_debug("testState_bulk_Error:2 => OK");
    }
    }
    { // Req 4
    m_log_debug("testState_bulk_Error:3");
    TransferService::TransferInfoArray array;
    Transfer t1(s_transfers.at(3).first,"file1",s_transfers.at(3).second,"Job1",Transfer::S_RECEIVED);
    Transfer t2(s_transfers.at(4).first,"file2",s_transfers.at(4).second,"Job1",Transfer::S_RECEIVED);
    Transfer t3(s_transfers.at(5).first,"file3",s_transfers.at(5).second,"Job1",Transfer::S_RECEIVED);
    array.push_back(&t1);
    array.push_back(&t2);
    array.push_back(&t3);
    try{
        m_log_debug("testState_bulk_Error:3 - before state");
        m_transferService->state("Invalid_Request_ID",array,"");
        CPPUNIT_FAIL("Invalid Request id (3)");
    } catch(const TransferException& exc){
        // OK
        m_log_debug("testState_bulk_Error:3 => OK");
    }
    }

}

/**
 * Test State Method
 */
void TransferServiceTestCase::testClean(){
#warning "TODO"
}

/**
 * Test State Method
 */
void TransferServiceTestCase::testTrace(){
#warning "TODO"
}

/**
 * Test List Method
 */
void TransferServiceTestCase::testCount(){

    // List Transfers' State
    unsigned long c = m_transferService->count();
    CPPUNIT_ASSERT_MESSAGE("No Active Requests", c > 0);
    CPPUNIT_ASSERT_MESSAGE("Invalid number of active Requests", c == s_requests.size());

}

/**
 * test_name_rule
 */
void TransferServiceTestCase::test_name_rule(){
    TransferService::TransferRequest  req;
    TransferService::TransferResponse out;
    req.surls.resize(1);
    req.surls[0].source  = "srm://x.y:1234/srm/managerv2?SFN=/foo/"
        "fail-S_PREPARING"
        "-c-FILE_EXISTS"
        "-p-P_TRANSFER"
        "-s-S_DESTINATION"
        "-m-The%20target%20file%20already%20exists";
    req.surls[0].destination = "srm://z.k:1207/srm/managerv2?SFN=/bar.dat";
    // Submit Transfer
    m_transferService->submit(req,out,"");

    CPPUNIT_ASSERT_MESSAGE("Empty RequestID", false == out.request_id.empty());
    CPPUNIT_ASSERT_MESSAGE("Empty TransferId", false == out.transfer_ids.empty());
    CPPUNIT_ASSERT_MESSAGE("Invalid number of TransferId", out.transfer_ids.size() == 1);

    TransferService::TransferInfoArray array;
    Transfer t(out.request_id,"x-file",out.transfer_ids.at(0),"x-job",Transfer::S_PREPARING);
    array.push_back(&t);
    m_transferService->state(out.request_id,array,"");

    CPPUNIT_ASSERT_MESSAGE("Unexpected TransferInfoArray size",array.size() == 1);

    CPPUNIT_ASSERT_MESSAGE("Unexpected state",t.state == model::Transfer::S_FAILED);
    CPPUNIT_ASSERT_MESSAGE("Unexpected category",t.reasonClass == model::Error::FILE_EXISTS);
    CPPUNIT_ASSERT_MESSAGE("Unexpected scope",t.errorScope == model::Error::S_DESTINATION);
    CPPUNIT_ASSERT_MESSAGE("Unexpected phase",t.errorPhase == model::Error::P_TRANSFER);
    if (!(t.reason == "The target file already exists")){
        CPPUNIT_FAIL(t.reason().c_str());
    }
}

/**
 * check additional properties of t_transfer are filled
 */
void TransferServiceTestCase::test_transfer_properties(){
    TransferService::TransferRequest  req;
    TransferService::TransferResponse out;
    req.surls.resize(1);
    req.surls[0].source  = "srm://x.y:1234/srm/managerv2?SFN=/foo.dat";
    req.surls[0].destination = "srm://z.k:1207/srm/managerv2?SFN=/bar.dat";
    // Submit Transfer
    m_transferService->submit(req,out,"");

    CPPUNIT_ASSERT_MESSAGE("Empty RequestID", false == out.request_id.empty());
    CPPUNIT_ASSERT_MESSAGE("Empty TransferId", false == out.transfer_ids.empty());
    CPPUNIT_ASSERT_MESSAGE("Invalid number of TransferId", out.transfer_ids.size() == 1);

    TransferService::TransferInfoArray array;
    Transfer t(out.request_id,"x-file",out.transfer_ids.at(0),"x-job",Transfer::S_PREPARING);
    array.push_back(&t);

    // call state: S_PREPARING -> S_PROCESSING
    m_transferService->state(out.request_id,array,"");

    CPPUNIT_ASSERT_MESSAGE("Unexpected TransferInfoArray size",array.size() == 1);
    static const char * const tx_id = "x-transfer";
    check_state(t,tx_id,model::Transfer::S_PROCESSING);
    check_error(t,tx_id);

    CHECK_MODEL_OBJECT_VALUE(t,srcHost,"x.y");
    CHECK_MODEL_OBJECT_VALUE(t,destHost,"z.k");

    CHECK_MODEL_OBJECT_VALUE(t,sourceTurl,"gsiftp://x.y:1234//foo.dat");
    CHECK_MODEL_OBJECT_VALUE(t,destTurl,"gsiftp://z.k:1207//bar.dat");

    CHECK_MODEL_OBJECT_VALUE(t,fileSize,687907840);

    CPPUNIT_ASSERT_MESSAGE("Invalid transferTime",(t.transferTime() != (time_t)-1));
    CPPUNIT_ASSERT_MESSAGE("Invalid prepareTime",(t.prepareTime() != (time_t)-1));

    // call state: S_PROCESSING -> S_COMPLETED
    m_transferService->state(out.request_id,array,"");

    check_state(t,tx_id,model::Transfer::S_COMPLETED);
    check_error(t,tx_id);
    CHECK_MODEL_OBJECT_VALUE(t,bytesWritten,687907840);
    CPPUNIT_ASSERT_MESSAGE("Invalid txDuration",(t.txDuration() != 0));
    CPPUNIT_ASSERT_MESSAGE("Invalid throughput",(t.throughput() != 0));
    CPPUNIT_ASSERT_MESSAGE("Invalid srcFinalDuration",(t.srcFinalDuration() != 0));
    CPPUNIT_ASSERT_MESSAGE("Invalid dstFinalDuration",(t.dstFinalDuration() != 0));

}

void TransferServiceTestCase::testChecksum()
{
    {// Req 1: Compare checksum, ok
    // Test Submit
    TransferService::TransferRequest  req;
    TransferService::TransferResponse out;
    req.surls.resize(1);
    req.surls[0].source = TC_SRC_NAME_CHECKSUM_OK;
    req.surls[0].destination = TC_DEST_NAME;
    req.surls[0].compare_checksum = true;
    req.surls[0].checksum_value = "chksum_value";
    req.surls[0].checksum_algorithm = "chksum_algorithm";

    // Submit Transfer
    m_transferService->submit(req,out,"");
    CPPUNIT_ASSERT_MESSAGE("Empty RequestID (1)", false == out.request_id.empty());
    CPPUNIT_ASSERT_MESSAGE("Empty TransferId (1)", false == out.transfer_ids.empty());
    CPPUNIT_ASSERT_MESSAGE("Invalid number of TransferId (1)", out.transfer_ids.size() == 1);

    s_requests.push_back(out.request_id);
    s_transfers.push_back(TxId(out.request_id,out.transfer_ids.at(0)));
    }
    {// Req 2: Fail After Submit
    // Test Submit
    TransferService::TransferRequest  req;
    TransferService::TransferResponse out;
    req.surls.resize(1);
    req.surls[0].source  = TC_SRC_FAIL_AFTER_SUBMIT_NAME;
    req.surls[0].destination = TC_DEST_NAME;
    // Submit Transfer
    m_transferService->submit(req,out,"");
    CPPUNIT_ASSERT_MESSAGE("Empty RequestID (2)", false == out.request_id.empty());
    CPPUNIT_ASSERT_MESSAGE("Empty TransferId (2)", false == out.transfer_ids.empty());
    CPPUNIT_ASSERT_MESSAGE("Invalid number of TransferId (2)", out.transfer_ids.size() == 1);

    s_requests.push_back(out.request_id);
    s_transfers.push_back(TxId(out.request_id,out.transfer_ids.at(0)));
    }
    {// Req 3: Fail In Processing
    // Test Submit
    TransferService::TransferRequest  req;
    TransferService::TransferResponse out;
    req.surls.resize(1);
    req.surls[0].source  = TC_SRC_FAIL_IN_PROCESSING_NAME;
    req.surls[0].destination = TC_DEST_NAME;
    // Submit Transfer
    m_transferService->submit(req,out,"");
    CPPUNIT_ASSERT_MESSAGE("Empty RequestID (3)", false == out.request_id.empty());
    CPPUNIT_ASSERT_MESSAGE("Empty TransferId (3)", false == out.transfer_ids.empty());
    CPPUNIT_ASSERT_MESSAGE("Invalid number of TransferId (3)", out.transfer_ids.size() == 1);

    s_requests.push_back(out.request_id);
    s_transfers.push_back(TxId(out.request_id,out.transfer_ids.at(0)));
    }

}
