/**
 *  Copyright (c) Members of the EGEE Collaboration. 2004-2008.
 *  See http://www.eu-egee.org/partners/ for details on the copyright
 *  holders.
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 *  GLite Data Transfer Agents: name-based rules for stub transfer service
 *
 *  Authors: Paolo Tedesco <paolo.tedesco@cern.ch>
 *  Version info: $Id: StubRules.cpp,v 1.2 2008/06/09 13:17:06 ptedesco Exp $
 *  Release: $Name:  $
 *
 */

#include "StubRules.h"
#include <sstream>

using namespace glite::data::transfer::agent::model;
using namespace glite::data::transfer::agent::ts;

namespace {
    
/**
 * UrlDecoder
 */
class UrlDecoder {
public:
    
    /**
     * instance
     * @return singleton instance
     */
    static UrlDecoder& instance(){
        static UrlDecoder s_instance;
        return s_instance;
    }
    
    /**
     * decode
     * @param input [IN] url-encoded string
     * @return url-decoded string
     */
    std::string decode(const std::string& input) const{
        std::stringstream ret;
        boost::match_results<std::string::const_iterator> what;
        boost::match_flag_type flags = boost::match_default;
        std::string::const_iterator s = input.begin();
        std::string::const_iterator e = input.end();
        bool match_found = false;
        while (boost::regex_search(s,e,what,m_decode,flags)){
            match_found = true;
            const std::string& match = what[0];
            std::string::difference_type p = what.position();
            std::string::difference_type l = what.length();
            std::string prev(s,s+p);
            ret << prev;
            ret << decode_hex(match);
            s += p + l;
        }
        if (match_found) {
            std::string::difference_type remaining_start = s - input.begin();
            std::string remaining(input,remaining_start);
            ret << remaining;
            return ret.str();
        } else {
            return input;
        }
    }
    
private:

    /**
     * Constructor
     */
    UrlDecoder() :
        m_decode("%[0-9a-fA-F]{2}"){
        m_char2int['0'] = 0x0;
        m_char2int['1'] = 0x1;
        m_char2int['2'] = 0x2;
        m_char2int['3'] = 0x3;
        m_char2int['4'] = 0x4;
        m_char2int['5'] = 0x5;
        m_char2int['6'] = 0x6;
        m_char2int['7'] = 0x7;
        m_char2int['8'] = 0x8;
        m_char2int['9'] = 0x9;
        m_char2int['a'] = 0xA;
        m_char2int['A'] = 0xA;
        m_char2int['b'] = 0xB;
        m_char2int['B'] = 0xB;
        m_char2int['c'] = 0xC;
        m_char2int['C'] = 0xC;
        m_char2int['d'] = 0xD;
        m_char2int['D'] = 0xD;
        m_char2int['e'] = 0xE;
        m_char2int['E'] = 0xE;
        m_char2int['f'] = 0xF;
        m_char2int['F'] = 0xF;
    }
    
    /**
     * convert a "%xx" hex string to a character
     * @param hex [IN] the "%xx" hex string
     * @return corresponding char
     */
    char decode_hex(const std::string& hex) const{
        char ret_def = ' ';
        if (3 != hex.length()){
            return ret_def;
        }
        char ret = (char)0;
        char2int::const_iterator it_1 = m_char2int.find(hex[1]);
        char2int::const_iterator it_2 = m_char2int.find(hex[2]);
        if ((m_char2int.end() == it_1) ||
            (m_char2int.end() == it_2)){
            return ret_def;
        }
        ret += (char)(it_1->second);
        ret <<= 4;
        ret += (char)(it_2->second);
        return ret;
    }
    
    // match % char followed by two hex digits
    boost::regex m_decode;
    
    // hex char conversion
    typedef std::map<char,unsigned short> char2int;
    char2int m_char2int;

};

} 

/**
 * instance
 * @return singleton instance
 */
TransferRule& TransferRule::instance(){
    static TransferRule s_instance;
    return s_instance;
}

/**
 * apply rule from given surl; if rule matches naming convention, set transfer
 * object properties accordingly
 * @param surl [IN] the surl to match
 * @param states [IN] one or more transfer states to match
 * @param transfer_object [OUT] the transfer object
 * @return true if rule applied, false otherwise
 */
bool TransferRule::applyRule(
    const std::string&  surl,    
    Transfer::State     states,
    Transfer&           transfer_object){
    
    boost::match_results<std::string::const_iterator> what;
    boost::match_flag_type flags = boost::match_default;
    if (boost::regex_search(surl.begin(),surl.end(),what,m_regex,flags)){
        const std::string& str_tx_state = what[1];
        Transfer::State tx_state = Transfer::S_UNDEF;
        if (from_string(str_tx_state,tx_state) && (tx_state & states)){
            const std::string& str_err_category = what[2];
            const std::string& str_err_phase    = what[3];
            const std::string& str_err_scope    = what[4];
            const std::string& str_err_message  = what[6];
            Error::Category err_category    = Error::UNDEF;
            Error::Phase err_phase          = Error::P_UNDEF;
            Error::Scope err_scope          = Error::S_UNDEF;
            if (from_string(str_err_category,err_category) &&
                from_string(str_err_phase,err_phase) &&
                from_string(str_err_scope,err_scope)){
                
                transfer_object.state = Transfer::S_FAILED;
                transfer_object.reasonClass = err_category;
                transfer_object.errorScope = err_scope;
                transfer_object.errorPhase = err_phase;
                if (!str_err_message.empty()){
                    transfer_object.reason = UrlDecoder::instance().decode(str_err_message);
                } else {
                    transfer_object.reason = "An error occurred."; 
                }
                return true;
            }
        }
    }
    return false;
}

/**
 * constructor
 */
TransferRule::TransferRule() : 
    m_regex("fail-([\\u_]+)-c-([\\u_]+)-p-([\\u_]+)-s-([\\u_]+)(-m-(.+))?"){
    // transfer status map
    m_txStatusMap["S_RECEIVED"]     = Transfer::S_RECEIVED;
    m_txStatusMap["S_PREPARING"]    = Transfer::S_PREPARING;
    m_txStatusMap["S_READY"]        = Transfer::S_READY;
    m_txStatusMap["S_SUBMITTED"]    = Transfer::S_SUBMITTED;
    m_txStatusMap["S_PROCESSING"]   = Transfer::S_PROCESSING;
    m_txStatusMap["S_TRANSFERRED"]  = Transfer::S_TRANSFERRED;
    m_txStatusMap["S_COMPLETED"]    = Transfer::S_COMPLETED;
    m_txStatusMap["S_FAILED"]       = Transfer::S_FAILED;
    m_txStatusMap["S_ABORTED"]      = Transfer::S_ABORTED;
    // category map
    m_categoryMap["TRANSFER"]               = Error::TRANSFER;
    m_categoryMap["SECURITY"]               = Error::SECURITY;
    m_categoryMap["NETWORK"]                = Error::NETWORK;
    m_categoryMap["CONFIGURATION"]          = Error::CONFIGURATION;
    m_categoryMap["CONSISTENCY"]            = Error::CONSISTENCY;
    m_categoryMap["CATALOG"]                = Error::CATALOG;
    m_categoryMap["USER"]                   = Error::USER;
    m_categoryMap["FILE_EXISTS"]            = Error::FILE_EXISTS;
    m_categoryMap["INVALID_PATH"]           = Error::INVALID_PATH;
    m_categoryMap["NO_SPACE_LEFT"]          = Error::NO_SPACE_LEFT;
    m_categoryMap["INVALID_SIZE"]           = Error::INVALID_SIZE;
    m_categoryMap["REQUEST_FAILURE"]        = Error::REQUEST_FAILURE;
    m_categoryMap["CONNECTION"]             = Error::CONNECTION;
    m_categoryMap["HTTP_TIMEOUT"]           = Error::HTTP_TIMEOUT;
    m_categoryMap["REQUEST_TIMEOUT"]        = Error::REQUEST_TIMEOUT;
    m_categoryMap["TRANSFER_TIMEOUT"]       = Error::TRANSFER_TIMEOUT;
    m_categoryMap["INTERNAL_ERROR"]         = Error::INTERNAL_ERROR;
    m_categoryMap["ABORTED"]                = Error::ABORTED;
    m_categoryMap["GENERAL_FAILURE"]        = Error::GENERAL_FAILURE;
    m_categoryMap["LOST"]                   = Error::LOST;
    m_categoryMap["STORAGE_INTERNAL_ERROR"] = Error::STORAGE_INTERNAL_ERROR;
    m_categoryMap["GRIDFTP"]                = Error::GRIDFTP;
    m_categoryMap["MARKERS_TIMEOUT"]        = Error::MARKERS_TIMEOUT;
    m_categoryMap["SRM_COPY"]               = Error::SRM_COPY;
    m_categoryMap["CATALOG_INTERNAL_ERROR"] = Error::CATALOG_INTERNAL_ERROR;
    // scope map
    m_scopeMap["S_SOURCE"]      = Error::S_SOURCE;
    m_scopeMap["S_DESTINATION"] = Error::S_DESTINATION;
    m_scopeMap["S_TRANSFER"]    = Error::S_TRANSFER;
    m_scopeMap["S_AGENT"]       = Error::S_AGENT;
    // phase map
    m_phaseMap["P_TX_PREPARATION"]          = Error::P_TX_PREPARATION;
    m_phaseMap["P_TX_FINALIZATION"]         = Error::P_TX_FINALIZATION;
    m_phaseMap["P_TRANSFER"]                = Error::P_TRANSFER;
    m_phaseMap["P_TX_SERVICE"]              = Error::P_TX_SERVICE;
}
