/**
 *  Copyright (c) Members of the EGEE Collaboration. 2004-2008.
 *  See http://www.eu-egee.org/partners/ for details on the copyright
 *  holders.
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 *  name-based rules for stub transfer service
 *
 *  Authors: Paolo Tedesco <paolo.tedesco@cern.ch>
 *  Version info: $Id: StubRules.h,v 1.2 2008/06/09 13:17:06 ptedesco Exp $
 *  Release: $Name:  $
 *
 */
#ifndef GLITE_DATA_TRANSFER_AGENTS_TS_TEST_STUBRULES_H_
#define GLITE_DATA_TRANSFER_AGENTS_TS_TEST_STUBRULES_H_

#include <string>
#include <map>

#include <boost/regex.hpp>
#include <glite/data/transfer/agent/model/Error.h>
#include <glite/data/transfer/agent/model/Transfer.h>

namespace glite   {
namespace data    {
namespace transfer{
namespace agent   {
namespace ts      {

/**
 * Apply a regular expression to file names to determine if transfer should fail 
 * with a specific error; file name must be in the form
 * fail-<tx phase>-c-<category>-p-<phase>-s-<scope>-m-<message>
 */
class TransferRule{
public:
    
    /**
     * instance
     * @return singleton instance
     */
    static TransferRule& instance();
    
    /**
     * apply rule from given surl; if rule matches naming convention, set transfer
     * object properties accordingly
     * @param surl [IN] the surl to match
     * @param states [IN] one or more transfer states to match
     * @param transfer_object [OUT] the transfer object
     * @return true if rule applied, false otherwise
     */
    bool applyRule(
        const std::string&      surl,    
        model::Transfer::State  states,
        model::Transfer&        transfer_object);

private:
    
    /**
     * constructor
     */
    TransferRule();

    // utils to search in maps
    //--------------------------------------------------------------------------
        
    /**
     * search string in map and set value
     * @param map [IN] the map string -> type
     * @param str_value [IN] the string value
     * @param value [OUT] the typed value
     * @return true if found, false otherwise
     */
    template <typename T>
    bool from_string(
        std::map<std::string,T> map,
        const std::string& str_value,
        T& value){
        typename std::map<std::string,T>::const_iterator it = map.find(str_value);
        if (map.end() != it){
            value = it->second;
            return true;
        }
        return false;
    }
        
    /**
     * string to transfer state
     * @param str_value [IN]
     * @param value [OUT]
     * @return true if found, false otherwise
     */
    bool from_string(
        const std::string&      str_value,
        model::Transfer::State& value){
        return from_string(m_txStatusMap,str_value,value);
    }
    
    /**
     * string to error category
     * @param str_value [IN]
     * @param value [OUT]
     * @return true if found, false otherwise
     */
    bool from_string(
        const std::string&      str_value,
        model::Error::Category& value){
        return from_string(m_categoryMap,str_value,value);
    }

    /**
     * string to error scope
     * @param str_value [IN]
     * @param value [OUT]
     * @return true if found, false otherwise
     */
    bool from_string(
        const std::string&      str_value,
        model::Error::Scope&    value){
        return from_string(m_scopeMap,str_value,value);
    }

    /**
     * string to error phase
     * @param str_value [IN]
     * @param value [OUT]
     * @return true if found, false otherwise
     */
    bool from_string(
        const std::string&      str_value,
        model::Error::Phase&    value){
        return from_string(m_phaseMap,str_value,value);
    }
    
    // string -> transfer status
    typedef std::map<std::string,model::Transfer::State> TxStatusMap;
    TxStatusMap m_txStatusMap;

    // string -> error category
    typedef std::map<std::string,model::Error::Category> CategoryMap;
    CategoryMap m_categoryMap;
    
    // string -> error scope
    typedef std::map<std::string,model::Error::Scope> ScopeMap;
    ScopeMap m_scopeMap;

    // string -> error phase
    typedef std::map<std::string,model::Error::Phase> PhaseMap;
    PhaseMap m_phaseMap;
    
    // the regular expression
    boost::regex m_regex;
    
};        
    
} // namespace ts
} // namespace agent
} // namespace transfer
} // namespace data
} // namespace glite

#endif // GLITE_DATA_TRANSFER_AGENTS_TS_TEST_STUBRULES_H_


