/**
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *  GLite Data Transfer Agents: StubTransferFactory Class
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>,
 *           Paolo Tedesco <paolo.tedesco@cern.ch>
 *  Version info: $Id: StubTransferFactory.cpp,v 1.11 2009/01/08 13:50:40 ptedesco Exp $
 *  Release: $Name:  $
 *
 */
#include "StubTransferFactory.h"
#include "StubTransferService.h"
#include <glite/data/agents/AgentExceptions.h>
#include <glite/data/config/service/ParamReader.hxx>

#include <stdlib.h>

using namespace glite::data::transfer::agent::ts;
using namespace glite::data::transfer::agent::model;
using namespace glite::config;
using glite::data::agents::LogicError;
using glite::data::agents::ConfigParamException;

/*
 * Constants
 */
namespace {
const char * const STUB_TRANSFER_NAME            = "transfer-agent-ts-stub";
    
const char * const MAXTRANSFERS_PROPERTY_NAME    = "MaxTransfers";
const char * const MAXREQUESTSIZE_PROPERTY_NAME  = "MaxRequestSize";
const char * const STOPAFTERPREPARATION_PROPERTY_NAME = "StopAfterPreparation";
const char * const TRANSFERSERVICETYPE_PROPERTY_NAME  = "TransferServiceType";
}

extern "C" {
/*
 * create_glite_component
 *
 * Create Component instance
 */
ComponentConfiguration * create_glite_component(){
    return new StubTransferFactory();
}
                                                                                
/*
 * destroy_glite_component
 *
 * Destroy Component instance
 */
void destroy_glite_component(ComponentConfiguration * component){
    StubTransferFactory * f = dynamic_cast<StubTransferFactory *>(component); 
    if(0 != f){
        delete f;
    }
}
                                                                                
} // End extern "C"

/*
 * StubTransferFactory
 *
 * Constructor
 */
StubTransferFactory::StubTransferFactory():TransferServiceFactory(),
    ComponentConfiguration(STUB_TRANSFER_NAME), 
    m_logger(STUB_TRANSFER_NAME),
    m_type("stub"),
    m_maxTransfers(10),
    m_maxRequestSize(1),
    m_stopAfterPreparation(false){
}
                                                                                                                          
/*
 * ~StubTransferFactory
 *
 * Destructor
 */
StubTransferFactory::~StubTransferFactory(){
}

/*
 * create
 *
 * Create a TransferService
 */
TransferService * StubTransferFactory::create(const std::string& channel_name) /*throw (LogicError)*/{
    
    StubTransferService * service = new StubTransferService(m_type,channel_name,m_logger);         
    service->maxTransfers(m_maxTransfers);
    service->maxRequestSize(m_maxRequestSize);
    service->stopAfterPreparation(m_stopAfterPreparation);
    m_log_debug("Stub TransferService Created");
    return service;
}

/*
 * init
 *
 * Initialize the Component
 */
int StubTransferFactory::init(const Params& params){
                                                                                                                          
    try{
        ParamReader reader(getName(),params);
        
        // Get Transfer Service Type
        reader.getParameter(TRANSFERSERVICETYPE_PROPERTY_NAME,m_type);
        
        // Get Max Parallel Transfers 
        reader.getParameter(MAXTRANSFERS_PROPERTY_NAME,m_maxTransfers)
            .ne(0);
        
        // Get Max Request Size Transfers 
        reader.getParameter(MAXREQUESTSIZE_PROPERTY_NAME,m_maxRequestSize)
            .ne(0);
        
        // Get StopAfterPreparation
        reader.getParameter(STOPAFTERPREPARATION_PROPERTY_NAME,m_stopAfterPreparation);
        
        // Log Configuration Values
        m_log_info(getName() << " Initialized. Initialization Parameters are");
        m_log_info("TransfersService Type  : " << m_type);
        m_log_info("MaxTransfers           : " << m_maxTransfers);
        m_log_info("MaxRequestSize         : " << m_maxRequestSize);
        m_log_info("stopAfterPreparation   : " << str_bool(m_stopAfterPreparation));
        
    } catch(const agents::LogicError& exc) {
        m_log_error(getName() << " configuration Error: " << exc.what());
        return -1;
    } catch(const std::exception& exc) {
        m_log_error(getName() << " configuration Unexpected Exception: " << exc.what());
        return -1;
    } catch(...) {
        m_log_error(getName() << " configuration Unhandled Exception");
        return -1;
    }
    return 0;
}

/*
 * config
 *
 * Configure the Component
 */
int StubTransferFactory::config(const Params& params){
    // No Config Params
    return 0;
}
