/**
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *  GLite Data Transfer Agents: Storkproxy Class
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>, Paolo Tedesco <paolo.tedesco@cern.ch>
 *  Version info: $Id: StubTransferService.cpp,v 1.16 2009/07/02 15:21:17 molnarzs Exp $
 *  Release: $Name:  $
 *
 */
#include "StubTransferService.h"
#include "StubRules.h"

#include <glite/data/agents/AgentExceptions.h>
#include <glite/data/agents/extra/Handle.h>
#include <glite/data/agents/extra/TempFile.h>

#include <glite/data/agents/Url.h>
#include <glite/data/agents/url-utility.h>

#include <math.h>
#include <boost/regex.hpp>
#include <map>
#include <string>
#include <algorithm>

#include <model_test_utils.h>

using namespace glite::data::transfer::agent;
using namespace glite::data::transfer::agent::model;
using namespace glite::data::transfer::agent::ts;
using glite::data::agents::LogicError;
using glite::data::agents::ConfigParamException;
using glite::data::agents::Handle;
using glite::data::agents::TempFile;

namespace{
/**
 * get host name and TURL
 * @param surl [IN]
 * @param host [OUT]
 * @param turl [OUT]
 */
void set_host_and_turl(
    const std::string&      surl_string,
    Property<std::string>&  host,
    Property<std::string>&  turl){

    glite::data::agents::Url surl(surl_string);
    host = surl.host;
    std::stringstream s_turl;
    s_turl << "gsiftp://" << surl.host << ":" << surl.port;
    typedef std::vector<std::pair<std::string,std::string> > QueryString;
    QueryString queryString;
    glite::data::agents::parse_query_string(surl.query,queryString);
    for (QueryString::const_iterator it = queryString.begin();it != queryString.end();++it){
        std::string key(it->first);
        std::transform(key.begin(),key.end(),key.begin(),(int(*)(int))std::tolower);
        if (key == "sfn"){
            s_turl << "/" << it->second;
            break;
        }
    }
    turl = s_turl.str();
}

}

/*
 * Init static members
 */
StubTransferService::RequestTable StubTransferService::s_requests;

/*
 * StubTransferService
 *
 * Constructor
 */
StubTransferService::StubTransferService(
    const std::string&  type,
    const std::string&  channel,
    glite::config::Logger& logger):
        m_logger(logger),
        m_type(type),
        m_channel(channel),
        m_maxTransfers(1),
        m_maxRequestSize(1),
        m_stopAfterPreparation(false){
}

/*
 * ~StubTransferService
 *
 * Destructor
 */
StubTransferService::~StubTransferService(){
}

/*
 * type
 *
 * Return the TransferService type
 */
std::string StubTransferService::type() /*throw ()*/{
    return m_type;
}

/*
 * channel
 *
 * Return the name of the Channel the transfer Service is responsible for
 */
std::string StubTransferService::channel() /*throw (LogicError)*/{
    if(m_channel.empty()){
        throw LogicError("Channel is not set");
    }
    return m_channel;
}

/*
 * maxConcurrents
 *
 * Get the number of max concurrent Transfers this service can support
 */
unsigned long StubTransferService::max() {
    return m_maxTransfers;
}

/*
 * maxRequestSize
 *
 * Get the number of max Transfers that can be processed whitin the same request
 */
unsigned long StubTransferService::maxRequestSize() {
    return m_maxRequestSize;
}


/*
 * submit
 *
 * Submit a FileTransfer request
 */
void StubTransferService::submit(
    const TransferRequest&  req,
    TransferResponse&       out,
    const std::string&      credentials) /*throw (LogicError, TransferException)*/{

    // Check Precondition
    if(req.surls.size() > m_maxRequestSize){
        m_log_warn("Cannot process request. Bulk size (" << req.surls.size() << ") exceeded");
        throw LogicError("too many files");
    }

    // check that there are no transfers already for the destination surl
    TransferRequest::SurlPairArray::const_iterator it;
    for(it = req.surls.begin(); it != req.surls.end(); ++it){
        const std::string& sourceSurl = (*it).source;
        // const std::string& destSurl   = (*it).second;

        // check Failure
        if(std::string::npos != sourceSurl.find("fail-during-submission")){
            throw TransferException(Error::TRANSFER,Error::S_AGENT,Error::P_TRANSFER,"Failed To submit file");
        }
    }

    // Generate a request id
    std::string name_tmpl = (std::string)"ts-stub__" + m_channel;
    Handle tmph;
    TempFile f(TempFile::generate(name_tmpl,"",tmph.get()));
    // Store the requestId
    out.request_id = f.name();

    // Store request
    unsigned int count = 0;
    for(it = req.surls.begin(); it != req.surls.end(); ++it, ++count){
        const std::string& sourceSurl = (*it).source;
        const std::string& destSurl   = (*it).destination;

        // Store Request Id
        StubRequest entry(out.request_id,count);
        entry.src_surl   = sourceSurl;
        entry.dst_surl   = destSurl;
        entry.state = model::Transfer::S_RECEIVED;
        s_requests.insert(entry);

        // Store transfer Id
        out.transfer_ids.push_back(count);

        m_log_info("Transfer <" << out.request_id << ":" << count << "> received");
    }
}

/*
 * revoke
 *
 * Cancel a Pending FileTransfer request
 */
void StubTransferService::revoke(const std::string& requestId, const std::string& credentials) /*throw ()*/{

    typedef RequestTable::index<req_id>::type  set_by_req_id;
    // Get Index
    set_by_req_id& index = s_requests.get<req_id>();
    // Erase Request Id
    index.erase(requestId);
}

/*
 * abort
 *
 * Cancel a Pending FileTransfer request
 */
void StubTransferService::abort(const std::string& requestId) /*throw ()*/{
    typedef RequestTable::index<req_id>::type  set_by_req_id;
    // Get Index
    set_by_req_id& index = s_requests.get<req_id>();
    // Erase Request Id
    index.erase(requestId);
}

/*
 *  state
 *
 * Get the status of a FileTransfer request
 */
void StubTransferService::state(
    const std::string& request_id,
    TransferInfoArray& objs,
    const std::string& /*credentials*/) /*throw (LogicError,TransferException)*/{

    m_log_debug("Checking state of request " << request_id);

    typedef RequestTable::index<req_id>::type  set_by_req_id;
    // Get Index
    set_by_req_id& index = s_requests.get<req_id>();
    set_by_req_id::iterator r_it,r_it0,r_it1;
    boost::tuples::tie(r_it0,r_it1) = index.equal_range(request_id);
    if(r_it0 == r_it1){
        // No such request id
        m_log_error("Request <" << request_id << "> not found");
        throw TransferException(Error::INTERNAL_ERROR,Error::S_AGENT,Error::P_TRANSFER,"Invalid request Id: not found");
    }
    // Check Precondition
    if(std::distance(r_it0,r_it1) != objs.size()){
        m_log_error("Inconsistency in Request <" << request_id << ">");
        throw LogicError("Invalid TransferInfo array size: mismatch");
    }

    typedef RequestTable::index<id>::type  set_by_id;
    // Get Unique ID Index
    set_by_id& id_index = s_requests.get<id>();

    // Upate State of each transfer object
    std::vector<unsigned int> files_to_remove;
    for(r_it = r_it0; r_it != r_it1; ++r_it){

        // Hack: I donlt really understand why (*r_it) is const, but since this
        // is a mock component, I don't care
        StubRequest& entry = const_cast<StubRequest&>(*r_it);

        // Get Transfer Object
        Transfer * transfer_object_ptr = 0;
        TransferInfoArray::iterator o_it;
        for(o_it = objs.begin(); o_it != objs.end(); ++o_it){
            if((*o_it)->transferId() == entry.tx_id){
                transfer_object_ptr = (*o_it);
                break;
            }
        }
        if(0 == transfer_object_ptr){
            // Mismatch
            m_log_error("Inconsistency (Mismatch) in Request <" << request_id << ">");
            throw LogicError("Inconsistent TransferInfo array");
        }
        Transfer& transfer_object = *(transfer_object_ptr);

        m_log_debug("Checking state of transfer <" << transfer_object.requestId() << ":" << transfer_object.transferId() << ">");

        // Check if Transfer exists
        set_by_id::iterator tx_it = id_index.find(boost::make_tuple(transfer_object.requestId(),transfer_object.transferId()));
        if(tx_it == id_index.end()){
            // Tx Doesn't exists
            transfer_object.state = model::Transfer::S_FAILED;
            transfer_object.reasonClass = Error::INTERNAL_ERROR;
            transfer_object.errorScope = Error::S_AGENT;
            transfer_object.errorPhase = Error::P_TRANSFER;
            transfer_object.reason = "No such Active Transfer";
        } else if(entry.request_id != transfer_object.requestId()){
            // Inconsistency
            transfer_object.state = model::Transfer::S_FAILED;
            transfer_object.reasonClass = Error::INTERNAL_ERROR;
            transfer_object.errorScope = Error::S_AGENT;
            transfer_object.errorPhase = Error::P_TRANSFER;
            transfer_object.reason = "Invalid Transfer Request ID";
        } else {
            m_log_debug("before update: transfer state is " << str(transfer_object.state()) << "; entry state is " << str(entry.state));

            // Update transfer Request state
            if (transfer_object.state() != Transfer::S_UNDEF && (
                entry.state == Transfer::S_UNDEF || entry.state < transfer_object.state())){
                entry.state = transfer_object.state();
            }
            // Check transfer state
            switch(entry.state){
                case model::Transfer::S_RECEIVED:
                case model::Transfer::S_PREPARING:
                case model::Transfer::S_READY:
                    //----------------------------------------------------------
                    // set file size
                    transfer_object.fileSize = (agents::longlong) ((entry.src_surl.length() << 24) + (entry.dst_surl.length() << 10));
                    // set prepare time
                    transfer_object.prepareTime = time(NULL);
                    // Set source and destination hosts and turls
                    set_host_and_turl(entry.src_surl,transfer_object.srcHost,transfer_object.sourceTurl);
                    set_host_and_turl(entry.dst_surl,transfer_object.destHost,transfer_object.destTurl);
                    //----------------------------------------------------------
                    if (m_stopAfterPreparation){
                        //------------------------------------------------------
                        time_t current = time(NULL);
                        double duration = (double)(transfer_object.prepareTime() - current) / 2;
                        transfer_object.srcPrepDuration() = duration;
                        transfer_object.dstPrepDuration() = duration;
                        //------------------------------------------------------
                        transfer_object.state = model::Transfer::S_READY;
                        entry.state = model::Transfer::S_READY;
                        break;
                    }
                case model::Transfer::S_SUBMITTED:
                    // Check if the transfer is failed
                    if(std::string::npos != entry.src_surl.find("fail-after-submit")){
                        transfer_object.state = model::Transfer::S_FAILED;
                        transfer_object.reasonClass = Error::TRANSFER;
                        transfer_object.errorPhase = Error::P_TX_PREPARATION;
                        transfer_object.errorScope = Error::S_AGENT;
                        transfer_object.reason = "Failed during Preparation";
                    } else {
                        //------------------------------------------------------
                        // set transfer time
                        transfer_object.transferTime = time(NULL);
                        //------------------------------------------------------
                        if (!m_stopAfterPreparation){
                            model::Transfer::State states = (model::Transfer::State)(
                                    model::Transfer::S_RECEIVED |
                                    model::Transfer::S_PREPARING |
                                    model::Transfer::S_READY |
                                    model::Transfer::S_SUBMITTED);
                            if (TransferRule::instance().applyRule(entry.src_surl,states,transfer_object)){
                                // applied rule -> do nothing
                            } else {
                                transfer_object.state = model::Transfer::S_PROCESSING;
                                entry.state = model::Transfer::S_PROCESSING;
                            }
                        } else {
                            // do nothing
                        }
                    }
                    break;
                case model::Transfer::S_PROCESSING:
                    // Check if the transfer is failed
                    if(std::string::npos != entry.src_surl.find("fail-in-processing")){
                        transfer_object.state = model::Transfer::S_FAILED;
                        transfer_object.reasonClass = Error::TRANSFER;
                        transfer_object.errorPhase = Error::P_TRANSFER;
                        transfer_object.errorScope = Error::S_TRANSFER;
                        transfer_object.reason = "Failed during Transfer";
                    } else {
                        model::Transfer::State states = (model::Transfer::State)(
                                model::Transfer::S_PROCESSING |
                                model::Transfer::S_TRANSFERRED);
                        if (TransferRule::instance().applyRule(entry.src_surl,states,transfer_object)){
                            // applied rule -> do nothing
                        } else {
                            //--------------------------------------------------
                            // set transfer duration
                            double txDuration = (double)(transfer_object.transferTime() - time(NULL));
                            if (txDuration <= 0){
                                txDuration = 1;
                            }
                            transfer_object.txDuration = txDuration;
                            // set transferred bytes
                            transfer_object.bytesWritten = transfer_object.fileSize();
                            // set throughput
                            transfer_object.throughput = (double)(transfer_object.fileSize() / txDuration);
                            //--------------------------------------------------

                            if (m_stopAfterPreparation){
                                transfer_object.state = model::Transfer::S_TRANSFERRED;
                                entry.state = model::Transfer::S_TRANSFERRED;
                            } else {
                                //----------------------------------------------
                                transfer_object.srcFinalDuration = txDuration / 2;
                                transfer_object.dstFinalDuration = txDuration / 2;
                                //----------------------------------------------
                                transfer_object.state = model::Transfer::S_COMPLETED;
                                entry.state = model::Transfer::S_COMPLETED;
                            }
                        }
                    }
                    break;
                case model::Transfer::S_TRANSFERRED:
                    //----------------------------------------------------------
                    {
                    double final_duration = (double)(transfer_object.transferTime() + transfer_object.txDuration() - time(NULL)) / 2;
                    transfer_object.srcFinalDuration = final_duration;
                    transfer_object.dstFinalDuration = final_duration;
                    }
                    //----------------------------------------------------------
                    transfer_object.state   = model::Transfer::S_COMPLETED;
                    entry.state             = model::Transfer::S_COMPLETED;
                    break;
                case model::Transfer::S_COMPLETED:
                case model::Transfer::S_FAILED:
                case model::Transfer::S_ABORTED:
                    // Nothing to DO
                    break;
                default:
                    transfer_object.state = model::Transfer::S_FAILED;
                    transfer_object.reasonClass = Error::INTERNAL_ERROR;
                    transfer_object.errorPhase = Error::P_TRANSFER;
                    transfer_object.errorScope = Error::S_AGENT;
                    transfer_object.reason = "Invalid Transfer State";
                    break;
            } // End swicth
        }
        m_log_debug("after update: transfer state is " << str(transfer_object.state()) << "; entry state is " << str(entry.state));
        // If transfer is completed, set the Duration
        if(Transfer::S_COMPLETED == transfer_object.state()){
            if(transfer_object.duration() <= 0.0){
                time_t tr_time = transfer_object.transferTime();
                if(tr_time > 0){
                    // Get the current time
                    time_t curr_time;
                    time(&curr_time);
                    double diff = (double)curr_time - (double)tr_time;
                    if(diff > 0){
                        transfer_object.duration = diff;
                    }
                }
            }
        }
        if((Transfer::S_FAILED == transfer_object.state()) ||
           (Transfer::S_ABORTED == transfer_object.state())||
           (Transfer::S_COMPLETED == transfer_object.state())){
            // Remove Transfer From Request Map
            files_to_remove.push_back(entry.tx_id);
        }
    } // End for each entry

    // Remove completed Files
    std::vector<unsigned int>::iterator txid_it;
    for(txid_it = files_to_remove.begin(); txid_it != files_to_remove.end(); ++txid_it){
        set_by_id::iterator tx_it = id_index.find(boost::make_tuple(request_id,*txid_it));
        id_index.erase(tx_it);
        m_log_info("Transfer <" << request_id << ":" << *txid_it << "> terminated");
    }
}

/*
 * clean
 *
 * Clean-up the information concerning a completed FileTransfer
 */
void StubTransferService::clean(const std::string& requestId, const std::string& credentials) /*throw (LogicError)*/{
#warning "TODO"
    throw LogicError("Not Implemented");
}

/*
 * trace
 *
 * Trace a Transfer
 */
void StubTransferService::trace(const std::string& requestId, std::ostream& out, const std::string& credentials) /*throw (LogicError, TransferException)*/{
#warning "TODO"
    throw LogicError("Not Implemented");
}

/*
 * count
 *
 * Get the number of all the pending transfer requests on the channel
 */
unsigned long StubTransferService::count() /*throw (LogicError, TransferException)*/{

    unsigned int count = 0;

    typedef const RequestTable::index<req_id>::type set_by_req_id;

    // Get Index
    set_by_req_id& index = s_requests.get<req_id>();
    set_by_req_id::const_iterator it;
    for(it = index.begin(); it != index.end(); ){
        count++;
        // jump to next
        it=index.upper_bound((*it).request_id);
    }

    return count;
}

/**
 * Get the maximum number of allowed transfers in any state for the channel
 * @param channel_transfers [IN]
 * @return maximum number of allowed transfers in any state for the channel
 */
unsigned long StubTransferService::getMaxChannelTransfers(unsigned long channel_transfers,double preparingFilesRatio){
    if (m_stopAfterPreparation){
        return (unsigned long)ceil(channel_transfers * preparingFilesRatio);
    }
    return channel_transfers;
}

/**
 * Get the maximum number of allowed transfers in any state for a VO
 * @param vo_transfers [IN]
 * @return maximum number of allowed transfers in any state for a VO
 */
unsigned long StubTransferService::getMaxVOTransfers(unsigned long vo_transfers,double preparingFilesRatio){
    if (m_stopAfterPreparation){
        return (unsigned long)ceil(vo_transfers * preparingFilesRatio);
    }
    return vo_transfers;
}
/**
 * return true if transfers stop after the preparation phase to maintain constant
 * the number of transfers in the transfer phase
 * @return
 */
bool StubTransferService::stopAfterPreparation() /* throw() */{
    return m_stopAfterPreparation;
}

/**
 * start the transfer phase for given transfer
 * @param transfer [IN] the transfer for which transfer phase must be started
 */
void StubTransferService::startTransferPhase(
    const model::Transfer&  transfer) /*throw (LogicError)*/{
    if (!m_stopAfterPreparation){
        throw LogicError("startTransferPhase called for StubTransferService but stopAfterPreparation flag is not set");
    }
    const std::string& request_id = transfer.requestId();
    typedef RequestTable::index<req_id>::type  set_by_req_id;
    // Get Index
    set_by_req_id& index = s_requests.get<req_id>();
    set_by_req_id::iterator r_it,r_it0,r_it1;
    boost::tuples::tie(r_it0,r_it1) = index.equal_range(request_id);
    if(r_it0 == r_it1){
        // No such request id
        m_log_error("Request <" << request_id << "> not found");
        throw TransferException(Error::INTERNAL_ERROR,Error::S_AGENT,Error::P_TRANSFER,"Invalid request Id: not found");
    }

    typedef RequestTable::index<id>::type  set_by_id;
    // Upate State of each transfer object
    std::vector<unsigned int> files_to_remove;
    for(r_it = r_it0; r_it != r_it1; ++r_it){
        StubRequest& entry = const_cast<StubRequest&>(*r_it);
        entry.state = Transfer::S_PROCESSING;
        m_log_debug("Request " << request_id << " is now Processing");
    }
}

/**
 * Get the transfer states that represent an active transfer
 * @return
 */
model::Transfer::State StubTransferService::getActiveStates() /* throw() */{
    return (model::Transfer::S_PROCESSING);
}

/**
 * Get the transfer states that represent a transfer that needs starting the
 * transfer phase
 * @return
 */
model::Transfer::State StubTransferService::getStatesToStart() /* throw() */{
    return (model::Transfer::S_READY);
}

