/**
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *  GLite Data Transfer Agents: StubTransferService Class
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *           Paolo Tedesco <paolo.tedesco@cern.ch>
 *  Version info: $Id: StubTransferService.h,v 1.6 2008/12/04 08:53:30 ptedesco Exp $
 *  Release: $Name:  $
 *
 */
#ifndef GLITE_DATA_TRANSFER_AGENTS_TS_TEST_STUBTRANSFERSERVICE_H_
#define GLITE_DATA_TRANSFER_AGENTS_TS_TEST_STUBTRANSFERSERVICE_H_

#include "glite/data/transfer/agent/ts/TransferService.h"
#include <string>
#include <vector>
#include <boost/multi_index_container.hpp>
#include <boost/multi_index/member.hpp>
#include <boost/multi_index/composite_key.hpp>
#include <boost/multi_index/ordered_index.hpp>
#include <boost/utility.hpp>
#include <glite/data/config/service/Logger.hxx>

namespace glite   {
namespace data    {
namespace transfer{
namespace agent   {
namespace ts      {
    
/**
 * Strok Transfer Service Base class. Provides some common methods for 
 * contacting Stork (use the Template Patter)
 */
class StubTransferService : public TransferService {
public:    
    /**
     * Constructor
     */
    StubTransferService(const std::string& type, const std::string& channel, glite::config::Logger& logger);

    /**
     * Destructor
     */
    virtual ~StubTransferService();

    /**
     * Return the TransferService type
     * @return the TransferService type
     */
    virtual std::string type() /*throw ()*/;

    /**
     * Return the type of the transfer handled by the service, AS IT MUST BE 
     * WRITTEN IN THE DATABASE
     * @return TransferType for the t_channel table
     */
    virtual std::string transferType(){
        return "URLCOPY";
    }
    
    /**
     * Return the name of the Channel the transfer Service is responsible for
     * @return the name of the Channel the transfer Service is responsible for
     * @throws LogicError in case the channel name is not set
     */
    virtual std::string channel() /*throw (LogicError)*/;

    /**
     * Get the number of max concurrent Transfers the service can support
     * @return the number of Max Concurrent Transfer the service can support
     */
    virtual unsigned long max();

    /**
     * Get the number of max Transfers that can be processed in the same request
     * @return number of max Transfers that can be processed in the same request
     */
    virtual unsigned long maxRequestSize();

    /**
     * Submit Transfers
     * @param req [INOUT] the request containing the list of SURLs pair to
     * transfer.
     * @param out [OUT] The result  object containing the request identifier and
     * a list of transfer identifier
     * @param credentials [IN] the credentials to be used to perform the action
     * @throws LogicError or TransferException in case of error
     */
    virtual void submit(const TransferRequest& req, TransferResponse& out, const std::string& credentials) /*throw (LogicError, TransferException)*/;

    /**
     * Cancel a Pending Transfer request
     * @param transferId [IN] the Identifier of the pending Transfer request
     * @param credentials [IN] the credentials to be used to perform the action
     */
    virtual void revoke(const std::string& requestId, const std::string& credentials) /*throw ()*/;
    
    /**
     * Cancel a Pending Transfer request
     * @param transferId [IN] the Identifier of the pending Transfer request
     */
    virtual void abort(const std::string& requestId) /*throw ()*/;
    
    /**
     * Get the status of a Transfer request
     * @param requestId [IN] the Identifier of the pending Transfer request
     * @param objs [INOUT] the array that will containing the information for
     * each transfer involved on the request
     * @param credentials [IN] the credentials to be used to perform the action
     * @return a structure containing state of a FileTransfer request
     * @throws TransferException in case of error
     */
    virtual void state(const std::string& requestId, TransferInfoArray& objs, const std::string& credentials) /*throw (TransferException)*/;
   
    /**
     * Clean-up the information concerning a completed FileTransfer 
     * @return the number of all the pending FileTransfer requests
     * @param credentials [IN] the credentials to be used to perform the action
     * @throws LogicError in case the requested FileTransfer request 
     * doesn't exist or is not completed
     */
    virtual void clean(const std::string& requestId, const std::string& credentials) /*throw (LogicError)*/;
    
    /**
     * Trace a Transfer
     * @param requestId [IN] the Identifier of the completed Transfer request
     * @param out [IN] the buffer where to store the transfer request
     * @param credentials [IN] the credentials to be used to perform the action
     * @throws LogicError in case the requested Transfer request 
     * doesn't exist or is not completed
     * @throws TransferException otherwise
     */
    virtual void trace(const std::string& requestId, std::ostream& out, const std::string& credentials) /*throw (LogicError, TransferException)*/;

    /**
     * Get the number of all the pending transfer requests on the channel
     * @return the number of all the pending transfer requests on the channel
     * @throws LogicError or TransferException in case of error
     */
    virtual unsigned long count() /*throw (LogicError, TransferException)*/;
    
    /**
     * Get the maximum number of allowed transfers in any state for the channel
     * @param channel_transfers [IN]
     * @param preparingFilesRatio [IN] ratio between number of files in preparation 
     * phase and in the actual transfer phase configured for the channel
     * @return maximum number of allowed transfers in any state for the channel
     */
    virtual unsigned long getMaxChannelTransfers(
        unsigned long channel_transfers,
        double preparingFilesRatio);
    
    /**
     * Get the maximum number of allowed transfers in any state for a VO
     * @param vo_transfers [IN]
     * @param preparingFilesRatio [IN] ratio between number of files in preparation 
     * phase and in the actual transfer phase configured for the channel
     * @return maximum number of allowed transfers in any state for a VO
     */
    virtual unsigned long getMaxVOTransfers(
        unsigned long vo_transfers,
        double preparingFilesRatio);

    /**
     * return true if transfers stop after the preparation phase to maintain constant
     * the number of transfers in the transfer phase
     * @return 
     */
    virtual bool stopAfterPreparation() /* throw() */;
    
    /**
     * start the transfer phase for given transfer
     * @param transfer [IN] the transfer for which transfer phase must be started
     */
    virtual void startTransferPhase(
        const model::Transfer&  transfer) /*throw (LogicError)*/;

    /**
     * Get the transfer states that represent an active transfer
     * @return 
     */
    virtual model::Transfer::State getActiveStates() /* throw() */;

    /**
     * Get the transfer states that represent a transfer that needs starting the 
     * transfer phase
     * @return 
     */
    virtual model::Transfer::State getStatesToStart() /* throw() */;    
        
private:
    
    /**
     * Default Constructor. Not Implemented
     */
    StubTransferService();

protected:

    friend class StubTransferFactory;

    /**
     * The Logger
     */
    glite::config::Logger m_logger;

    /**
     * Set the number of max concurrent Transfers the service can support
     */
    void maxTransfers(unsigned long m){m_maxTransfers = m;}

    /**
     * Set the number of max Transfers that can be processed in the same request
     */
    void maxRequestSize(unsigned long m){m_maxRequestSize = m;}
    
    /**
     * set the stopAfterPreparation flag
     */ 
    void stopAfterPreparation(bool value){m_stopAfterPreparation = value;}
    
private:    
    /**
     * The Transfer Service Type
     */
    std::string m_type;

    /**
     * The Channel the service is responsible for
     */
    std::string m_channel;
    
    /**
     * Max Parallel Transfers
     */
    unsigned long m_maxTransfers;

    /**
     * Max Bulk Size
     */
    unsigned long m_maxRequestSize;

    /**
     * flag signaling if transfers should be stopped after the preparation phase
     */
    bool m_stopAfterPreparation;
    
    /**
     * StubRequest
     */
    struct StubRequest{
        std::string const      request_id;
        unsigned int const     tx_id;
        std::string            src_surl;
        std::string            dst_surl;
        model::Transfer::State state;
        StubRequest(const std::string& r,unsigned int i):
            request_id(r),tx_id(i),state(model::Transfer::S_UNDEF){}
    };
    
    /*
     * tags for accessing several indices defined below
     */
    struct id{};
    struct req_id{};
            
    /*
     * Request Table ID Composite Key
     */
    struct id_key: boost::multi_index::composite_key<
      StubRequest,
      BOOST_MULTI_INDEX_MEMBER(StubRequest,std::string const,request_id),
      BOOST_MULTI_INDEX_MEMBER(StubRequest,unsigned int const,tx_id)
    >{};
        
    /*
     * Define The Table with
     *  - a unique index <reqid-txid>
     *  - a non-unique index sorted by Request ID
     */
    typedef boost::multi_index::multi_index_container<
      StubRequest,
      boost::multi_index::indexed_by<
        boost::multi_index::ordered_unique<boost::multi_index::tag<id>,id_key>,
        boost::multi_index::ordered_non_unique<boost::multi_index::tag<req_id>,
          BOOST_MULTI_INDEX_MEMBER(StubRequest,std::string const,request_id)
        >
      >
    > RequestTable;
    
    /**
     * Active Requests
     */
    static RequestTable s_requests;
    
};

} // End namespace ts
} // End namespace agent
} // End namespace transfer
} // End namespace data
} // End namespace glite

#endif //GLITE_DATA_TRANSFER_AGENTS_TS_TEST_STUBTRANSFERSERVICE_H_
